@@ -218,7 +218,12 @@ namespace Commons.Xml.Relaxng.Derivative
 		public abstract bool Nullable { get; }
 
 		// fills QName collection
-		public abstract void GetLabels (LabelList elements, LabelList attributes);
+		public void GetLabels (LabelList elements, LabelList attributes)
+		{
+			GetLabels (elements, attributes, false);
+		}
+
+		public abstract void GetLabels (LabelList elements, LabelList attributes, bool collectNameClass);
 
 		internal void AddNameLabel (LabelList names, RdpNameClass nc)
 		{
@@ -436,7 +441,7 @@ namespace Commons.Xml.Relaxng.Derivative
 			get { return RdpContentType.Empty; }
 		}
 
-		public override void GetLabels (LabelList elements, LabelList attributes)
+		public override void GetLabels (LabelList elements, LabelList attributes, bool collectNameClass)
 		{
 			// do nothing
 		}
@@ -504,7 +509,7 @@ namespace Commons.Xml.Relaxng.Derivative
 			return false;
 		}
 
-		public override void GetLabels (LabelList elements, LabelList attributes)
+		public override void GetLabels (LabelList elements, LabelList attributes, bool collectNameClass)
 		{
 			// FIXME: Supposed to clear something here?
 		}
@@ -559,7 +564,7 @@ namespace Commons.Xml.Relaxng.Derivative
 			return true;
 		}
 
-		public override void GetLabels (LabelList elements, LabelList attributes)
+		public override void GetLabels (LabelList elements, LabelList attributes, bool collectNameClass)
 		{
 			// do nothing
 		}
@@ -642,6 +647,40 @@ namespace Commons.Xml.Relaxng.Derivative
 		{
 			return l.ContainsText () || r.ContainsText ();
 		}
+
+		// 7.3 (group/interleave attribute names) and
+		// part of 7.4 (interleave element names)
+		// FIXME: Actually it should be done against the correct
+		// simplified grammar, expanding all refs.
+		internal void CheckNameOverlap (bool checkElements)
+		{
+			// expecting that all items are interned.
+			bool checkAttributes = false;
+			Hashtable lca = new Hashtable ();
+			Hashtable lce = checkElements ? new Hashtable () : null;
+			LValue.GetLabels (lce, lca, true);
+			if ((lce == null || lce.Count == 0) && lca.Count == 0)
+				return;
+
+			Hashtable rca = new Hashtable ();
+			Hashtable rce = checkElements ? new Hashtable () : null;
+			RValue.GetLabels (rce, rca, true);
+			if ((rce == null || rce.Count == 0) && rca.Count == 0)
+				return;
+
+			foreach (RdpNameClass ncl in lca.Values)
+				foreach (RdpNameClass ncr in rca.Values)
+					if (RdpUtil.NameClassOverlap (ncl, ncr))
+						throw new RelaxngException ("Duplicate attributes inside a group or an interleave is not allowed.");
+
+			if (!checkElements)
+				return;
+
+			foreach (RdpNameClass ncl in lce.Values)
+				foreach (RdpNameClass ncr in rce.Values)
+					if (RdpUtil.NameClassOverlap (ncl, ncr))
+						throw new RelaxngException ("Duplicate attributes inside a group or an interleave is not allowed.");
+		}
 	}
 
 	// Choice
@@ -675,10 +714,10 @@ namespace Commons.Xml.Relaxng.Derivative
 			}
 		}
 
-		public override void GetLabels (LabelList elements, LabelList attributes)
+		public override void GetLabels (LabelList elements, LabelList attributes, bool collectNameClass)
 		{
-			LValue.GetLabels (elements, attributes);
-			RValue.GetLabels (elements, attributes);
+			LValue.GetLabels (elements, attributes, collectNameClass);
+			RValue.GetLabels (elements, attributes, collectNameClass);
 		}
 
 
@@ -783,10 +822,10 @@ namespace Commons.Xml.Relaxng.Derivative
 			}
 		}
 
-		public override void GetLabels (LabelList elements, LabelList attributes)
+		public override void GetLabels (LabelList elements, LabelList attributes, bool collectNameClass)
 		{
-			LValue.GetLabels (elements, attributes);
-			RValue.GetLabels (elements, attributes);
+			LValue.GetLabels (elements, attributes, collectNameClass);
+			RValue.GetLabels (elements, attributes, collectNameClass);
 		}
 
 		internal override RdpPattern ReduceEmptyAndNotAllowed (ref bool result, Hashtable visited)
@@ -870,6 +909,9 @@ namespace Commons.Xml.Relaxng.Derivative
 			LValue.CheckConstraints (attribute, oneOrMore, oneOrMoreGroup, oneOrMore, list, dataExcept);
 			RValue.CheckConstraints (attribute, oneOrMore, oneOrMoreGroup, oneOrMore, list, dataExcept);
 
+			// 7.3
+			CheckNameOverlap (true);
+
 			// 7.4
 			// TODO: (1) unique name analysis
 			// (2) text/text prohibited
@@ -896,11 +938,13 @@ namespace Commons.Xml.Relaxng.Derivative
 			}
 		}
 
-		public override void GetLabels (LabelList elements, LabelList attributes)
+		public override void GetLabels (LabelList elements, LabelList attributes, bool collectNameClass)
 		{
-			LValue.GetLabels (elements, attributes);
+			LValue.GetLabels (elements, attributes, collectNameClass);
 			if (LValue.Nullable)
-				RValue.GetLabels (elements, attributes);
+				RValue.GetLabels (elements, attributes, collectNameClass);
+			else
+				RValue.GetLabels (null, attributes, collectNameClass);
 		}
 
 		public override RdpPattern TextDeriv (string s, XmlReader reader)
@@ -956,6 +1000,9 @@ namespace Commons.Xml.Relaxng.Derivative
 
 			LValue.CheckConstraints (attribute, oneOrMore, oneOrMore, oneOrMoreInterleave, list, dataExcept);
 			RValue.CheckConstraints (attribute, oneOrMore, oneOrMore, oneOrMoreInterleave, list, dataExcept);
+
+			// 7.3
+			CheckNameOverlap (false);
 		}
 	}
 
@@ -1015,9 +1062,9 @@ namespace Commons.Xml.Relaxng.Derivative
 			get { return Child.Nullable; }
 		}
 
-		public override void GetLabels (LabelList elements, LabelList attributes)
+		public override void GetLabels (LabelList elements, LabelList attributes, bool collectNameClass)
 		{
-			Child.GetLabels (elements, attributes);
+			Child.GetLabels (elements, attributes, collectNameClass);
 		}
 
 		internal override RdpPattern ReduceEmptyAndNotAllowed (ref bool result, Hashtable visited)
@@ -1085,7 +1132,6 @@ namespace Commons.Xml.Relaxng.Derivative
 				throw new RelaxngException ("oneOrMore is not allowed under except of a data.");
 			this.Child.CheckConstraints (attribute, true, oneOrMoreGroup, oneOrMoreInterleave, list, dataExcept);
 		}
-
 	}
 
 	// List
@@ -1133,9 +1179,9 @@ namespace Commons.Xml.Relaxng.Derivative
 			get { return RdpContentType.Simple; }
 		}
 
-		public override void GetLabels (LabelList elements, LabelList attributes)
+		public override void GetLabels (LabelList elements, LabelList attributes, bool collectNameClass)
 		{
-			Child.GetLabels (elements, attributes);
+			Child.GetLabels (elements, attributes, collectNameClass);
 		}
 
 		public override RdpPattern TextDeriv (string s, XmlReader reader)
@@ -1189,7 +1235,7 @@ namespace Commons.Xml.Relaxng.Derivative
 			get { return RdpContentType.Simple; }
 		}
 
-		public override void GetLabels (LabelList elements, LabelList attributes)
+		public override void GetLabels (LabelList elements, LabelList attributes, bool collectNameClass)
 		{
 			// do nothing.
 		}
@@ -1316,7 +1362,7 @@ namespace Commons.Xml.Relaxng.Derivative
 			get { return RdpContentType.Simple; }
 		}
 
-		public override void GetLabels (LabelList elements, LabelList attributes)
+		public override void GetLabels (LabelList elements, LabelList attributes, bool collectNameClass)
 		{
 			// do nothing
 		}
@@ -1377,9 +1423,14 @@ namespace Commons.Xml.Relaxng.Derivative
 			get { return RdpContentType.Empty; }
 		}
 
-		public override void GetLabels (LabelList elements, LabelList attributes)
+		public override void GetLabels (LabelList elements, LabelList attributes, bool collectNameClass)
 		{
-			AddNameLabel (attributes, NameClass);
+			if (attributes != null) {
+				if (collectNameClass)
+					attributes [NameClass] = NameClass;
+				else
+					AddNameLabel (attributes, NameClass);
+			}
 		}
 
 		bool isExpanded;
@@ -1490,9 +1541,14 @@ namespace Commons.Xml.Relaxng.Derivative
 			}
 		}
 
-		public override void GetLabels (LabelList elements, LabelList attributes)
+		public override void GetLabels (LabelList elements, LabelList attributes, bool collectNameClass)
 		{
-			AddNameLabel (elements, NameClass);
+			if (elements != null) {
+				if (collectNameClass)
+					elements [NameClass] = NameClass;
+				else
+					AddNameLabel (elements, NameClass);
+			}
 		}
 
 
@@ -1569,9 +1625,9 @@ namespace Commons.Xml.Relaxng.Derivative
 			get { return false; }
 		}
 
-		public override void GetLabels (LabelList elements, LabelList attributes)
+		public override void GetLabels (LabelList elements, LabelList attributes, bool collectNameClass)
 		{
-			LValue.GetLabels (elements, attributes);
+			LValue.GetLabels (elements, attributes, collectNameClass);
 		}
 
 		public override RdpPattern TextDeriv (string s, XmlReader reader)
