@@ -199,12 +199,18 @@ namespace System.Resources
 					if (n != null) {
 						object v = null;
 						string val = get_value (reader, "value");
+
 						if (mt != null && tt != null) {
 							TypeConverter c = TypeDescriptor.GetConverter (tt);
 							v = c.ConvertFrom (Convert.FromBase64String (val));
 						} else if (tt != null) {
-							TypeConverter c = TypeDescriptor.GetConverter (tt);
-							v = c.ConvertFromString (val);
+							// MS seems to handle Byte[] without any mimetype :-(
+							if (t.StartsWith("System.Byte[], mscorlib")) {
+								v = Convert.FromBase64String(val);
+							} else {
+								TypeConverter c = TypeDescriptor.GetConverter (tt);
+								v = c.ConvertFromInvariantString (val);
+							}
 						} else if (mt != null) {
 							byte [] data = Convert.FromBase64String (val);
 							BinaryFormatter f = new BinaryFormatter ();
@@ -260,6 +266,7 @@ namespace System.Resources
 		void IDisposable.Dispose ()
 		{
 			Dispose(true);
+			GC.SuppressFinalize(this);
 		}
 
 		protected virtual void Dispose(bool disposing) {
