@@ -123,14 +123,14 @@ namespace System.Windows.Forms {
 			if (control == this.control)
 				return;
 
-			prop_desc = TypeDescriptor.GetProperties (control).Find (property_name, false);
-			data_type = prop_desc.PropertyType; // Getting the PropertyType is kinda slow and it should never change, so it is cached
+			prop_desc = TypeDescriptor.GetProperties (control).Find (property_name, true);			
 			
 			if (prop_desc == null)
 				throw new ArgumentException (String.Concat ("Cannot bind to property '", property_name, "' on target control."));
 			if (prop_desc.IsReadOnly)
 				throw new ArgumentException (String.Concat ("Cannot bind to property '", property_name, "' because it is read only."));
-
+				
+			data_type = prop_desc.PropertyType; // Getting the PropertyType is kinda slow and it should never change, so it is cached
 			control.Validating += new CancelEventHandler (ControlValidatingHandler);
 
 			this.control = control;
@@ -141,7 +141,7 @@ namespace System.Windows.Forms {
 			if (control == null)
 				return;
 
-			manager = control.BindingContext [data_source, property_name];
+			manager = control.BindingContext [data_source];
 			manager.AddBinding (this);
 
 			WirePropertyValueChangedEvent ();
@@ -153,6 +153,9 @@ namespace System.Windows.Forms {
 
 		internal void PushData ()
 		{
+			if (IsBinding == false)
+				return;
+
 			data = prop_desc.GetValue (control);
 			data = FormatData (data);
 			SetPropertyValue (data);
@@ -160,6 +163,9 @@ namespace System.Windows.Forms {
 
 		internal void PullData ()
 		{
+			if (IsBinding == false)
+				return;
+
 			if (is_null_desc != null) {
 				bool is_null = (bool) is_null_desc.GetValue (manager.Current);
 				if (is_null) {
