@@ -5,10 +5,10 @@
 // distribute, sublicense, and/or sell copies of the Software, and to
 // permit persons to whom the Software is furnished to do so, subject to
 // the following conditions:
-// 
+//
 // The above copyright notice and this permission notice shall be
 // included in all copies or substantial portions of the Software.
-// 
+//
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 // EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 // MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
@@ -142,34 +142,39 @@ namespace System.Windows.Forms {
 			FontChanged+=new EventHandler(RedrawEvent);
 			SizeChanged+=new EventHandler(RedrawEvent);
 
-			SetStyle(ControlStyles.UserPaint, true);
-			SetStyle(ControlStyles.AllPaintingInWmPaint, true);
-			SetStyle(ControlStyles.ResizeRedraw, true);
-			SetStyle(ControlStyles.StandardClick | ControlStyles.StandardDoubleClick, false);
+			SetStyle(ControlStyles.ResizeRedraw | 
+				ControlStyles.Opaque | 
+				ControlStyles.UserMouse | 
+				ControlStyles.SupportsTransparentBackColor | 
+				ControlStyles.CacheText |
+				ControlStyles.DoubleBuffer, true);
+			SetStyle(ControlStyles.StandardClick, false);
 		}
 		#endregion	// Public Constructors
 
 		#region Public Instance Properties
 		[Localizable(true)]
 		[DefaultValue(FlatStyle.Standard)]
+		[MWFDescription("Determines look of button"), MWFCategory("Appearance")]
 		public FlatStyle FlatStyle {
 			get {
 				return flat_style;
 			}
 
-			set { 
-				flat_style = value; 
+			set {
+				flat_style = value;
 				Redraw();
 			}
 		}
 		
 		[Localizable(true)]
+		[MWFDescription("Sets image to be displayed on button face"), MWFCategory("Appearance")]
 		public Image Image {
 			get {
 				return image;
 			}
 
-			set { 
+			set {
 				image = value;
 				Redraw();
 			}
@@ -177,6 +182,7 @@ namespace System.Windows.Forms {
 
 		[Localizable(true)]
 		[DefaultValue(ContentAlignment.MiddleCenter)]
+		[MWFDescription("Sets the alignment of the image to be displayed on button face"), MWFCategory("Appearance")]
 		public ContentAlignment ImageAlign {
 			get {
 				return image_alignment;
@@ -192,6 +198,7 @@ namespace System.Windows.Forms {
 		[DefaultValue(-1)]
 		[Editor("System.Windows.Forms.Design.ImageIndexEditor, " + Consts.AssemblySystem_Design, typeof(System.Drawing.Design.UITypeEditor))]
 		[TypeConverter(typeof(ImageIndexConverter))]
+		[MWFDescription("Index of image to display, if ImageList is used for button face images"), MWFCategory("Appearance")]
 		public int ImageIndex {
 			get {
 				if (image_list==null) {
@@ -207,16 +214,13 @@ namespace System.Windows.Forms {
 		}
 
 		[DefaultValue(null)]
+		[MWFDescription("ImageList used for ImageIndex"), MWFCategory("Appearance")]
 		public ImageList ImageList {
 			get {
 				return image_list;
 			}
 
 			set {
-				if (image_list != null) {
-					image_list.Dispose();
-				}
-
 				image_list = value;
 				if (value != null) {
 					if (image != null) {
@@ -250,6 +254,7 @@ namespace System.Windows.Forms {
 
 		[Localizable(true)]
 		[DefaultValue(ContentAlignment.MiddleCenter)]
+		[MWFDescription("Alignment for button text"), MWFCategory("Appearance")]
 		public virtual ContentAlignment TextAlign {
 			get {
 				return text_alignment;
@@ -312,7 +317,7 @@ namespace System.Windows.Forms {
 							text_format.LineAlignment=StringAlignment.Far;
 							break;
 						}
-					}	
+					}
 					Redraw();
 				}
 			}
@@ -322,7 +327,7 @@ namespace System.Windows.Forms {
 
 		#region Protected Instance Properties
 		protected override CreateParams CreateParams {
-			get { 
+			get {
 				CreateParams	cp;
 
 				cp=base.CreateParams;
@@ -421,17 +426,15 @@ namespace System.Windows.Forms {
 
 		protected override void OnMouseEnter(EventArgs e) {
 			is_entered=true;
-			if ((this.flat_style == FlatStyle.Flat) || (this.flat_style == FlatStyle.Popup)) {
+			if ( is_enabled )
 				Redraw();
-			}
 			base.OnMouseEnter(e);
 		}
 
 		protected override void OnMouseLeave(EventArgs e) {
 			is_entered=false;
-			if ((this.flat_style == FlatStyle.Flat) || (this.flat_style == FlatStyle.Popup)) {
+			if ( is_enabled )
 				Redraw();
-			}
 			base.OnMouseLeave(e);
 		}
 
