@@ -50,6 +50,7 @@ namespace System.Windows.Forms
 			check_onclick = false;
 			three_dcheckboxes = false;
 			listbox_info.item_height = FontHeight + 2;
+			SetStyle (ControlStyles.ResizeRedraw, true);
 		}
 
 		#region events
@@ -185,7 +186,7 @@ namespace System.Windows.Forms
 
 		public bool GetItemChecked (int index)
 		{
-			return (GetItemCheckState (index) == CheckState.Checked);
+			return (GetItemCheckState (index) != CheckState.Unchecked);
 		}
 		
 		public CheckState GetItemCheckState (int index)
@@ -205,12 +206,6 @@ namespace System.Windows.Forms
 		{
 			base.OnClick (e);
 			
-			if (check_onclick) {
-				if (focused_item != -1) {
-					SetItemChecked (focused_item, !GetItemChecked (focused_item));
-				}
-			}
-			
 			if (Click != null)			
 				Click (this, EventArgs.Empty);
 		}
@@ -315,42 +310,46 @@ namespace System.Windows.Forms
 		#region Private Methods
 
 		internal override void OnMouseDownLB (object sender, MouseEventArgs e)
-		{
-			base.OnMouseDownLB (sender, e);
-			
-			Rectangle hit_rect, item_rect;
+		{			
+			Rectangle hit_rect;
 			CheckState value =  CheckState.Checked;
 			bool set_value = false;
 			int index = IndexFromPointDisplayRectangle (e.X, e.Y);
 
+			if (Click != null) {
+				if (e.Button == MouseButtons.Left) {
+					Click (this, e);
+				}
+			}	
+
 			if (index == -1)
 				return;
 			
 			/* CheckBox hit */
-			hit_rect = item_rect = GetItemDisplayRectangle (index, LBoxInfo.top_item); // Full item rect
+			hit_rect = GetItemDisplayRectangle (index, LBoxInfo.top_item); // Full item rect
 			hit_rect.X += ThemeEngine.Current.CheckedListBoxCheckRectangle().X;
 			hit_rect.Y += ThemeEngine.Current.CheckedListBoxCheckRectangle().Y;
 			hit_rect.Width = ThemeEngine.Current.CheckedListBoxCheckRectangle().Width;
 			hit_rect.Height = ThemeEngine.Current.CheckedListBoxCheckRectangle().Height;
 			
-			if ((Items.GetListBoxItem (index)).State == CheckState.Checked)
-					value = CheckState.Unchecked;
-
-			if (hit_rect.Contains (e.X, e.Y) == true)  {				
+			if ((Items.GetListBoxItem (index)).State == CheckState.Checked) { //From checked to unchecked
+				value = CheckState.Unchecked;
 				set_value = true;
-
 			} else {
-				if (item_rect.Contains (e.X, e.Y) == true) {
-					if (!check_onclick) {
-						if ((Items.GetListBoxItem (index)).Selected == true)
-							set_value = true;
-					}
+				
+				if (check_onclick) {
+					set_value = true;
+				} else {
+					if ((Items.GetListBoxItem (index)).Selected == true)
+						set_value = true;
 				}
 			}
 
 			if (set_value)
 				SetItemCheckState (index, value);
 			
+			SelectedItemFromNavigation (index);
+			SetFocusedItem (index);
 		}
 
 		internal override void UpdateItemInfo (UpdateOperation operation, int first, int last)
@@ -364,9 +363,11 @@ namespace System.Windows.Forms
 
 		public class ObjectCollection : ListBox.ObjectCollection
 		{		
+			private CheckedListBox owner;
+
 			public ObjectCollection (CheckedListBox owner) : base (owner)
 			{
-				
+				this.owner = owner;				
 			}
 
 			public int Add (object item,  bool isChecked)
@@ -385,6 +386,9 @@ namespace System.Windows.Forms
 				box_item.State = check;
 				object_items.Add (item);
 				listbox_items.Add (box_item);
+				if (check == CheckState.Checked)
+					owner.OnItemCheck (new ItemCheckEventArgs (cnt, check, CheckState.Unchecked));
+				owner.UpdateItemInfo (UpdateOperation.AddItems, cnt, cnt);
 				return cnt;
 			}
 		}
