@@ -52,8 +52,6 @@ namespace System.Windows.Forms
 			form.Text = "Color";
 			
 			form.Size = new Size( 221, 332 ); // 300
-			
-			colorDialogPanel = new ColorDialogPanel( this );
 		}
 		#endregion	// Public Constructors
 		
@@ -201,6 +199,7 @@ namespace System.Windows.Forms
 		#region Protected Instance Methods
 		protected override bool RunDialog( IntPtr hwndOwner )
 		{
+			colorDialogPanel = new ColorDialogPanel (this);
 			form.Controls.Add( colorDialogPanel );
 			
 			if ( customColors != null )
@@ -265,7 +264,8 @@ namespace System.Windows.Forms
 				
 				okButton = new Button( );
 				cancelButton = new Button( );
-				helpButton = new Button( );
+				if (colorDialog.ShowHelp)
+					helpButton = new Button( );
 				defineColoursButton = new Button( );
 				addColoursButton = new Button( );
 				
@@ -383,11 +383,14 @@ namespace System.Windows.Forms
 				cancelButton.TabIndex = 1;
 				cancelButton.Text = Locale.GetText( "Cancel" );
 				// helpButton
-				helpButton.FlatStyle = FlatStyle.System;
-				helpButton.Location = new Point( 149, 271 );
-				helpButton.Size = new Size( 66, 22 );
-				helpButton.TabIndex = 5;
-				helpButton.Text = Locale.GetText( "Help" );
+				if (colorDialog.ShowHelp) {
+					helpButton.FlatStyle = FlatStyle.System;
+					helpButton.Location = new Point( 149, 271 );
+					helpButton.Size = new Size( 66, 22 );
+					helpButton.TabIndex = 5;
+					helpButton.Text = Locale.GetText ( "Help" );
+				}
+
 				// addColoursButton
 				addColoursButton.FlatStyle = FlatStyle.System;
 				addColoursButton.Location = new Point( 227, 271 );
@@ -400,7 +403,6 @@ namespace System.Windows.Forms
 				baseColorControl.Size = new Size( 212, 231 );
 				baseColorControl.TabIndex = 13;
 				// colorMatrixControl
-				//colorMatrixControl.BackColor = SystemColors.Control;
 				colorMatrixControl.Location = new Point( 227, 7 );
 				colorMatrixControl.Size = new Size( 179, 190 );
 				colorMatrixControl.TabIndex = 14;
@@ -431,7 +433,8 @@ namespace System.Windows.Forms
 				Controls.Add( defineColoursButton );
 				Controls.Add( okButton );
 				Controls.Add( cancelButton );
-				Controls.Add( helpButton );
+				if (colorDialog.ShowHelp)
+					Controls.Add( helpButton );
 				Controls.Add( addColoursButton );
 				
 				Controls.Add( baseColorControl );
@@ -464,16 +467,14 @@ namespace System.Windows.Forms
 				
 				if ( !colorDialog.AllowFullOpen )
 					defineColoursButton.Enabled = false;
-				
-				if ( !colorDialog.ShowHelp )
-					helpButton.Enabled = false;
-				
+
 				if ( colorDialog.FullOpen )
 					DoButtonDefineColours( );
 				
 				defineColoursButton.Click += new EventHandler( OnClickButtonDefineColours );
 				addColoursButton.Click += new EventHandler( OnClickButtonAddColours );
-				helpButton.Click += new EventHandler( OnClickHelpButton );
+				if (colorDialog.ShowHelp)
+					helpButton.Click += new EventHandler( OnClickHelpButton );
 				cancelButton.Click += new EventHandler( OnClickCancelButton );
 				okButton.Click += new EventHandler( OnClickOkButton );
 				
@@ -1068,7 +1069,7 @@ namespace System.Windows.Forms
 				{
 					base.OnPaint( pe );
 					
-					pe.Graphics.FillRectangle( ThemeEngine.Current.ResPool.GetSolidBrush( SystemColors.Control ), 0, 0, 26, 23 );
+					pe.Graphics.FillRectangle( ThemeEngine.Current.ResPool.GetSolidBrush( BackColor ), 0, 0, 26, 23 );
 					
 					ControlPaint.DrawBorder3D( pe.Graphics, new Rectangle( 3, 3, 20, 18 ) );
 					
@@ -1077,8 +1078,11 @@ namespace System.Windows.Forms
 					
 					if ( isSelected )
 					{
-						pe.Graphics.DrawRectangle( new Pen( ThemeEngine.Current.ResPool.GetSolidBrush( Color.Black ) ),
-									  new Rectangle( 2, 2, 20, 18 ) );
+						using ( Pen pen = new Pen( ThemeEngine.Current.ResPool.GetSolidBrush( Color.Black ) ) )
+						{
+							pe.Graphics.DrawRectangle( pen,
+										  new Rectangle( 2, 2, 20, 18 ) );
+						}
 					}
 					
 					if ( hasFocus && isSelected )
@@ -1698,14 +1702,16 @@ namespace System.Windows.Forms
 				
 				public void Draw( )
 				{
-					Pen pen = new Pen( ThemeEngine.Current.ResPool.GetSolidBrush( cursorColor ), 3 );
-					
-					Graphics graphics = Graphics.FromImage( bitmap );
-					
-					graphics.DrawLine( pen, 11, 0, 11, 7 );
-					graphics.DrawLine( pen, 11, 14, 11, 21 );
-					graphics.DrawLine( pen, 0, 11, 7, 11 );
-					graphics.DrawLine( pen, 14, 11, 21, 11 );
+					using( Pen pen = new Pen( ThemeEngine.Current.ResPool.GetSolidBrush( cursorColor ), 3 ) )
+					{
+						using( Graphics graphics = Graphics.FromImage( bitmap ) )
+						{
+							graphics.DrawLine( pen, 11, 0, 11, 7 );
+							graphics.DrawLine( pen, 11, 14, 11, 21 );
+							graphics.DrawLine( pen, 0, 11, 7, 11 );
+							graphics.DrawLine( pen, 14, 11, 21, 11 );
+						}
+					}
 				}
 				
 				public Bitmap Bitmap
@@ -2044,7 +2050,7 @@ namespace System.Windows.Forms
 			
 			private void Draw( PaintEventArgs e )
 			{
-				e.Graphics.FillRectangle( new SolidBrush( SystemColors.Control ), new Rectangle( 0, 0, 16, 203 ) );
+				e.Graphics.FillRectangle( ThemeEngine.Current.ResPool.GetSolidBrush( BackColor ), new Rectangle( 0, 0, 16, 203 ) );
 				
 				Point[] trianglePoints = new Point[ 3 ]
 				{
