@@ -62,7 +62,7 @@ namespace System.Windows.Forms
 		private ComboListBox listbox_ctrl;		
 		private TextBox textbox_ctrl;
 		private bool process_textchanged_event;
-		private bool has_focus;		
+		private bool has_focus;	
 
 		[ComVisible(true)]
 		public class ChildAccessibleObject : AccessibleObject {
@@ -94,6 +94,7 @@ namespace System.Windows.Forms
 			internal int original_height;		/* Control's height is recalculated for not Simple Styles */
 			internal Rectangle listbox_area;	/* ListBox area in Simple combox, not used in the rest */
 			internal bool droppeddown;		/* Is the associated ListBox dropped down? */
+			internal int combosimple_height;	/* Simple default height */
 
 			public ComboBoxInfo ()
 			{
@@ -102,6 +103,7 @@ namespace System.Windows.Forms
 				item_height = 0;
 				droppeddown = false;
 				original_height = -1;
+				combosimple_height = 150;
 			}
 		}
 
@@ -233,15 +235,16 @@ namespace System.Windows.Forms
 									
 				if (dropdown_style == ComboBoxStyle.Simple) {
 					if (listbox_ctrl != null) {						
-						Controls.Remove (listbox_ctrl);
+						Controls.RemoveImplicit (listbox_ctrl);
 						listbox_ctrl.Dispose ();						
 						listbox_ctrl = null;
+						Height = combobox_info.original_height;
 					}
 				}
 
 				if (dropdown_style != ComboBoxStyle.DropDownList && value == ComboBoxStyle.DropDownList) {
 					if (textbox_ctrl != null) {						
-						Controls.Remove (textbox_ctrl);
+						Controls.RemoveImplicit (textbox_ctrl);
 						textbox_ctrl.Dispose ();						
 						textbox_ctrl = null;						
 					}
@@ -250,15 +253,14 @@ namespace System.Windows.Forms
 				dropdown_style = value;					
 				
 				if (dropdown_style == ComboBoxStyle.Simple) {
-					CBoxInfo.show_button = false;
-					
-					if (combobox_info.original_height != -1)
-						Height = combobox_info.original_height;
+					CBoxInfo.show_button = false;					
+					combobox_info.original_height = Height;
+					Height = combobox_info.combosimple_height;
 					
 					CreateComboListBox ();
 
 					if (IsHandleCreated == true) {
-						Controls.Add (listbox_ctrl);
+						Controls.AddImplicit (listbox_ctrl);
 					}
 				}
 				else {
@@ -268,11 +270,12 @@ namespace System.Windows.Forms
 	
 				if (dropdown_style != ComboBoxStyle.DropDownList && textbox_ctrl == null) {
 					textbox_ctrl = new TextBox ();
+					textbox_ctrl.BorderStyle = BorderStyle.None;
 					textbox_ctrl.TextChanged += new EventHandler (OnTextChangedEdit);
 					textbox_ctrl.KeyPress += new KeyPressEventHandler(textbox_ctrl_KeyPress);
 
 					if (IsHandleCreated == true) {
-						Controls.Add (textbox_ctrl);
+						Controls.AddImplicit (textbox_ctrl);
 					}
 				}
 				
@@ -357,11 +360,11 @@ namespace System.Windows.Forms
 		}
 
 		[Localizable (true)]
-		public virtual int ItemHeight {
+		public int ItemHeight {
 			get { return combobox_info.item_height; }
 			set {
 				if (value < 0)
-					throw new ArgumentOutOfRangeException ("The item height value is less than zero");
+					throw new ArgumentException ("The item height value is less than zero");
 
 				combobox_info.item_height = value;
 				CalcTextArea ();
@@ -420,7 +423,7 @@ namespace System.Windows.Forms
 		public override int SelectedIndex {
 			get { return selected_index; }
 			set {
-				if (value < -2 || value >= Items.Count)
+				if (value <= -2 || value >= Items.Count)
 					throw new ArgumentOutOfRangeException ("Index of out range");
 
 				if (selected_index == value)
@@ -599,12 +602,12 @@ namespace System.Windows.Forms
 			if (disposing == true) {
 				if (listbox_ctrl != null) {
 					listbox_ctrl.Dispose ();
-					Controls.Remove (listbox_ctrl);
+					Controls.RemoveImplicit (listbox_ctrl);
 					listbox_ctrl = null;
 				}			
 			
 				if (textbox_ctrl != null) {
-					Controls.Remove (textbox_ctrl);
+					Controls.RemoveImplicit (textbox_ctrl);
 					textbox_ctrl.Dispose ();
 					textbox_ctrl = null;
 				}			
@@ -621,11 +624,18 @@ namespace System.Windows.Forms
 
 		public int FindString (string s)
 		{
-			return FindString (s, 0);
+			return FindString (s, -1);
 		}
 
 		public int FindString (string s, int startIndex)
 		{
+			if (Items.Count == 0) 
+				return -1; // No exception throwing if empty
+
+			if (startIndex < -1 || startIndex >= Items.Count - 1)
+				throw new  ArgumentOutOfRangeException ("Index of out range");
+
+			startIndex++;
 			for (int i = startIndex; i < Items.Count; i++) {
 				if ((GetItemText (Items[i])).StartsWith (s))
 					return i;
@@ -636,11 +646,18 @@ namespace System.Windows.Forms
 
 		public int FindStringExact (string s)
 		{
-			return FindStringExact (s, 0);
+			return FindStringExact (s, -1);
 		}
 
 		public int FindStringExact (string s, int startIndex)
 		{
+			if (Items.Count == 0) 
+				return -1; // No exception throwing if empty
+
+			if (startIndex < -1 || startIndex >= Items.Count - 1)
+				throw new ArgumentOutOfRangeException ("Index of out range");
+
+			startIndex++;
 			for (int i = startIndex; i < Items.Count; i++) {
 				if ((GetItemText (Items[i])).Equals (s))
 					return i;
@@ -650,11 +667,11 @@ namespace System.Windows.Forms
 		}
 
 		public int GetItemHeight (int index)
-		{
-			if (index < 0 || index >= Items.Count )
-				throw new ArgumentOutOfRangeException ("The item height value is less than zero");
-				
+		{	
 			if (DrawMode == DrawMode.OwnerDrawVariable && IsHandleCreated == true) {
+
+				if (index < 0 || index >= Items.Count )
+					throw new ArgumentOutOfRangeException ("The item height value is less than zero");
 				
 				if ((Items.GetComboBoxItem (index)).ItemHeight != -1) {
 					return (Items.GetComboBoxItem (index)).ItemHeight;
@@ -768,11 +785,15 @@ namespace System.Windows.Forms
 			base.OnHandleCreated (e);
 
 			if (listbox_ctrl != null) {
-				Controls.Add (listbox_ctrl);
+				Controls.AddImplicit (listbox_ctrl);
+
+				if (dropdown_style == ComboBoxStyle.Simple)
+					Height = combobox_info.combosimple_height;
+				
 			}
 			
 			if (textbox_ctrl != null) {
-				Controls.Add (textbox_ctrl);
+				Controls.AddImplicit (textbox_ctrl);
 			}
 
 			CalcTextArea ();
@@ -800,10 +821,14 @@ namespace System.Windows.Forms
 		}
 
 		protected override void OnResize (EventArgs e)
-		{
+		{			
 			base.OnResize (e);
 			AdjustHeightForDropDown ();
-			CalcTextArea ();			
+
+			if (listbox_ctrl != null)
+				listbox_ctrl.CalcListBoxArea ();
+
+			CalcTextArea ();
 		}
 
 		protected override void OnSelectedIndexChanged (EventArgs e)
@@ -863,7 +888,7 @@ namespace System.Windows.Forms
 		}		
 
 		protected override void SetBoundsCore (int x, int y, int width, int height, BoundsSpecified specified)
-		{
+		{			
 			base.SetBoundsCore (x, y, width, height, specified);
 		}
 
@@ -892,9 +917,9 @@ namespace System.Windows.Forms
 
 			case Msg.WM_PAINT: {
 				PaintEventArgs	paint_event;
-				paint_event = XplatUI.PaintEventStart (Handle);
+				paint_event = XplatUI.PaintEventStart (Handle, true);
 				OnPaintCB (paint_event);
-				XplatUI.PaintEventEnd (Handle);
+				XplatUI.PaintEventEnd (Handle, true);
 				return;
 			}
 
@@ -1002,17 +1027,18 @@ namespace System.Windows.Forms
 					state |= DrawItemState.Focus;
 				}
 				
+				state |= DrawItemState.ComboBoxEdit;				
 				OnDrawItem (new DrawItemEventArgs (dc, Font, item_rect,
 							selected_index, state, ForeColor, BackColor));
 			}						
 			
-			if (clip.IntersectsWith (combobox_info.listbox_area) == true) {
+			if (DropDownStyle == ComboBoxStyle.Simple && clip.IntersectsWith (combobox_info.listbox_area) == true) {
 				dc.FillRectangle (ThemeEngine.Current.ResPool.GetSolidBrush (Parent.BackColor), 
 						combobox_info.listbox_area);
 			}
 			
 			if (CBoxInfo.show_button) {
-				dc.FillRectangle (ThemeEngine.Current.ResPool.GetSolidBrush (ThemeEngine.Current.ColorButtonFace),
+				dc.FillRectangle (ThemeEngine.Current.ResPool.GetSolidBrush (ThemeEngine.Current.ColorControl),
 					combobox_info.button_rect);
 
 				ThemeEngine.Current.CPDrawComboButton (dc,
@@ -1420,7 +1446,7 @@ namespace System.Windows.Forms
 			}
 
 			public ComboListBox (ComboBox owner) : base ()
-			{	
+			{					
 				this.owner = owner;								
 				top_item = 0;
 				last_item = 0;
@@ -1498,7 +1524,7 @@ namespace System.Windows.Forms
 						vscrollbar_ctrl.Visible = false;						
 					}					
 				}
-				else {
+				else {					
 					/* Need vertical scrollbar */
 					if (vscrollbar_ctrl == null) {
 						vscrollbar_ctrl = new VScrollBarLB ();
@@ -1508,7 +1534,7 @@ namespace System.Windows.Forms
 						vscrollbar_ctrl.Maximum = 0;
 						vscrollbar_ctrl.ValueChanged += new EventHandler (VerticalScrollEvent);
 						
-						Controls.Add (vscrollbar_ctrl);
+						Controls.AddImplicit (vscrollbar_ctrl);
 					}
 					
 					vscrollbar_ctrl.Height = height - ThemeEngine.Current.DrawComboListBoxDecorationBottom (owner.DropDownStyle) -
@@ -1516,10 +1542,9 @@ namespace System.Windows.Forms
 							
 					vscrollbar_ctrl.Location = new Point (width - vscrollbar_ctrl.Width - ThemeEngine.Current.DrawComboListBoxDecorationRight (owner.DropDownStyle), 
 							ThemeEngine.Current.DrawComboListBoxDecorationTop (owner.DropDownStyle));
-						
-					vscrollbar_ctrl.Maximum = owner.Items.Count - owner.MaxDropDownItems;
+										
+					vscrollbar_ctrl.Maximum = owner.Items.Count - (owner.DropDownStyle == ComboBoxStyle.Simple ? page_size : owner.maxdrop_items);
 					show_scrollbar = vscrollbar_ctrl.Visible = true;
-					
 				}
 				
 				Size = new Size (width, height);
@@ -1596,7 +1621,11 @@ namespace System.Windows.Forms
 
 				item_rect.X = ThemeEngine.Current.DrawComboListBoxDecorationRight (owner.DropDownStyle);
 				item_rect.Width = textarea_drawable.Width;
-				item_rect.Y = 2 + ((height - 2) * (index - first_displayble));
+				item_rect.Y = ((height - 2) * (index - first_displayble));
+
+				if (owner.dropdown_style == ComboBoxStyle.Simple)
+					item_rect.Y += 2;
+
 				item_rect.Height = height;
 				return item_rect;
 			}
@@ -1758,7 +1787,7 @@ namespace System.Windows.Forms
 				/* Previous item */
     				if (GetHighLightedIndex () != -1) {    					
 					invalidate = GetItemDisplayRectangle (GetHighLightedIndex (), top_item);
-	    				if (ClientRectangle.Contains (invalidate))
+	    				if (ClientRectangle.IntersectsWith (invalidate))
 	    					Invalidate (invalidate);
 	    			}
 				
@@ -1767,7 +1796,7 @@ namespace System.Windows.Forms
     				if (highlighted_item != null) {
 	    				 /* Current item */
 	    				invalidate = GetItemDisplayRectangle (GetHighLightedIndex (), top_item);
-	    				if (ClientRectangle.Contains (invalidate))
+	    				if (ClientRectangle.IntersectsWith (invalidate))
 	    					Invalidate (invalidate);
 	    			}
     				
@@ -1879,7 +1908,7 @@ namespace System.Windows.Forms
 			}
 
 			private void OnPaintPUW (Object o, PaintEventArgs pevent)
-			{
+			{				
 				Draw (pevent.ClipRectangle,pevent.Graphics);
 			}
 
@@ -1919,7 +1948,7 @@ namespace System.Windows.Forms
 				UpdateLastVisibleItem ();
 				Refresh ();
 			}			
-
+			
 			#endregion Private Methods
 		}
 	}
