@@ -34,7 +34,7 @@ using System.Runtime.InteropServices;
 using System.Runtime.Serialization;
 
 namespace System.Windows.Forms {
-	[Editor("System.Drawing.Design.CursorEditor, System.Drawing.Design, Version=1.0.5000.0, Culture=neutral, PublicKeyToken=b03f5f7f11d50a3a", typeof(System.Drawing.Design.UITypeEditor))]
+	[Editor("System.Drawing.Design.CursorEditor, " + Consts.AssemblySystem_Drawing_Design, typeof(System.Drawing.Design.UITypeEditor))]
 	[Serializable]
 	[TypeConverter(typeof(CursorConverter))]
 	public sealed class Cursor : IDisposable, ISerializable {
@@ -114,6 +114,9 @@ namespace System.Windows.Forms {
 			}
 		}
 
+		private Cursor(SerializationInfo info, StreamingContext context) {
+		}
+
 		private Cursor() {
 		}
 
@@ -170,15 +173,24 @@ namespace System.Windows.Forms {
 			}
 		}
 
+		[MonoTODO("Implement setting a null cursor, and add XplatUI method to get current cursor")]
 		public static Cursor Current {
 			get {
-				return current;
+				if (current != null) {
+					return current;
+				}
+				return Cursors.Default;
 			}
 
 			set {
 				if (current != value) {
 					current = value;
-					XplatUI.OverrideCursor(current.handle);
+					if (value != null){
+						// FIXME - define and set empty cursor
+						current = null;
+						XplatUI.OverrideCursor(IntPtr.Zero);
+					} else
+						XplatUI.OverrideCursor(current.handle);
 				}
 			}
 		}
@@ -273,6 +285,8 @@ namespace System.Windows.Forms {
 				this.mask.Dispose();
 				this.mask = null;
 			}
+
+			GC.SuppressFinalize (this);
 		}
 
 		public void Draw(Graphics g, Rectangle targetRect) {
@@ -373,15 +387,10 @@ namespace System.Windows.Forms {
             
 			cursor_dir = new CursorDir ();
 			cursor_dir.idReserved = reader.ReadUInt16();
-			if (cursor_dir.idReserved != 0) {
-				throw new System.ArgumentException ("Invalid Argument", "stream");
-			}
-			
 			cursor_dir.idType = reader.ReadUInt16();
-			if (cursor_dir.idType != 2) { //must be 2
-				throw new System.ArgumentException ("Invalid Argument", "stream");
-			}
-
+			if (cursor_dir.idReserved != 0 || !(cursor_dir.idType == 2 || cursor_dir.idType == 1))
+				throw new System.ArgumentException ("Invalid Argument, format error", "stream");
+		
 			entry_count = reader.ReadUInt16();
 			cursor_dir.idCount = entry_count;
 			cursor_dir.idEntries = new CursorEntry[entry_count];
@@ -560,29 +569,33 @@ namespace System.Windows.Forms {
 					}
 					writer.Write(cih.biXPelsPerMeter);
 					writer.Write(cih.biYPelsPerMeter);
-					writer.Write(cih.biClrUsed);
-					writer.Write(cih.biClrImportant);
 
 					// write color table
 					if (xor) {
+						writer.Write(cih.biClrUsed);
+						writer.Write(cih.biClrImportant);
 						color_count = ci.cursorColors.Length;
 						for (int j = 0; j < color_count; j++) {
 							writer.Write (ci.cursorColors[j]);
 						}
 					} else {
+						// 2 used colors
+						writer.Write(2);
+						// 2 important colors
+						writer.Write(2);
+						
 						writer.Write((uint)0x00000000);
 						writer.Write((uint)0x00ffffff);
 					}
 
 					// write image bits
-					if (xor) {
+					if (xor) 
 						writer.Write(ci.cursorXOR);
-					} else {
+					else
 						writer.Write(ci.cursorAND);
-					}
+
 					writer.Flush();
 
-					// create bitmap from stream and return
 					bmp = new Bitmap(stream);
 
 					if (transparent) {
