@@ -67,8 +67,8 @@ namespace System.Windows.Forms
 			#region Private Constructors
 			internal HitTestInfo ()
 			{
-				column = 0;
-				row = 0;
+				column = -1;
+				row = -1;
 				type =  HitTestType.None;
 			}
 			#endregion
@@ -103,7 +103,7 @@ namespace System.Windows.Forms
 
 			public override string ToString ()
 			{
-				return base.ToString ();
+				return "{ " + type + "," + row + "," + column + "}";
 			}
 
 		}
@@ -128,8 +128,8 @@ namespace System.Windows.Forms
 		private bool allow_navigation;
 		private bool allow_sorting;
 		private Color alternating_backcolor;
+		private Color backColor;
 		private Color background_color;
-		internal BorderStyle border_style;
 		private Color caption_backcolor;
 		private Font caption_font;
 		private Color caption_forecolor;
@@ -150,7 +150,7 @@ namespace System.Windows.Forms
 		private Color link_hovercolor;
 		private Color parentrowsback_color;
 		private Color parentrowsfore_color;
-		internal bool parentrows_visible;
+		private bool parentrows_visible;
 		private int preferredcolumn_width;
 		private int preferredrow_height;
 		private bool _readonly;
@@ -164,8 +164,6 @@ namespace System.Windows.Forms
 		private GridTableStylesCollection styles_collection;
 		private DataGridParentRowsLabelStyle parentrowslabel_style;
 		internal DataGridCell current_cell;
-		private Color forecolor;
-		private Color backcolor;
 		private DataGridTableStyle default_style;
 		private DataGridTableStyle current_style;
 		internal HScrollBar horiz_scrollbar;
@@ -175,10 +173,14 @@ namespace System.Windows.Forms
 		internal int horz_pixeloffset;
 		internal bool is_editing; 	// Current cell is edit mode
 		internal bool is_changing;	// Indicates if current cell is been changed (in edit mode)
+		internal bool is_adding;	// Indicates when we are adding a row
 		private Hashtable selected_rows;
 		private bool ctrl_pressed;
 		private bool shift_pressed;
 		private bool begininit;
+		private CurrencyManager cached_currencymgr;
+		private CurrencyManager cached_currencymgr_events;
+		private bool accept_listmgrevents;
 		#endregion // Local Variables
 
 		#region Public Constructors
@@ -188,6 +190,7 @@ namespace System.Windows.Forms
 			allow_navigation = true;
 			allow_sorting = true;
 			begininit = false;
+			backColor = ThemeEngine.Current.DataGridBackColor;
 			alternating_backcolor = def_alternating_backcolor;
 			background_color = def_background_color;
 			border_style = BorderStyle.Fixed3D;
@@ -211,7 +214,7 @@ namespace System.Windows.Forms
 			link_hovercolor = def_link_hovercolor;
 			parentrowsback_color = def_parentrowsback_color;
 			parentrowsfore_color = def_parentrowsfore_color;
-			parentrows_visible = false; // should be true (temp)
+			parentrows_visible = true;
 			preferredcolumn_width = ThemeEngine.Current.DataGridPreferredColumnWidth;
 			_readonly = false;
 			rowheaders_visible = true;
@@ -226,13 +229,14 @@ namespace System.Windows.Forms
 			horz_pixeloffset = 0;
 			is_editing = false;
 			is_changing = false;
-			forecolor = SystemColors.WindowText;
+			is_adding = false;
 			parentrowslabel_style = DataGridParentRowsLabelStyle.Both;
-			backcolor = SystemColors.Window;
 			selected_rows = new Hashtable ();
 			ctrl_pressed = false;
 			shift_pressed = false;
 			preferredrow_height = def_preferredrow_height = FontHeight + 3;
+			cached_currencymgr_events = cached_currencymgr = null;
+			accept_listmgrevents = true;
 
 			default_style = new DataGridTableStyle (true);
 			styles_collection = new GridTableStylesCollection (this);
@@ -246,7 +250,7 @@ namespace System.Windows.Forms
 			vert_scrollbar.Scroll += new ScrollEventHandler (GridVScrolled);			
 			KeyUp += new KeyEventHandler (OnKeyUpDG);			
 
-			SetStyle (ControlStyles.UserPaint | ControlStyles.AllPaintingInWmPaint, true);
+			SetStyle (ControlStyles.UserMouse, true);
 
 		}
 
@@ -294,15 +298,12 @@ namespace System.Windows.Forms
 			}
 		}
 
-		public Color BackColor {
+		public override Color BackColor {
 			get {
-				return backcolor;
+				return backColor;
 			}
 			set {
-				if (backcolor != value) {
-					backcolor = value;
-					Refresh ();
-				}
+				backColor = value;
 			}
 		}
 
@@ -321,7 +322,7 @@ namespace System.Windows.Forms
 
 		[Browsable(false)]
 		[EditorBrowsable(EditorBrowsableState.Never)]
-		public Image BackgroundImage {
+		public override Image BackgroundImage {
 			get {
 				return base.BackgroundImage;
 			}
@@ -334,17 +335,11 @@ namespace System.Windows.Forms
 		[DispId(-504)]
 		[DefaultValue(BorderStyle.Fixed3D)]
 		public BorderStyle BorderStyle {
-			get {
-				return border_style;
-			}
-
-			set {
-				 if (border_style != value) {
-					border_style = value;
-					grid_drawing.CalcGridAreas ();
-					OnBorderStyleChanged (EventArgs.Empty);
-					Refresh ();
-				}
+			get { return InternalBorderStyle; }
+			set { 
+				InternalBorderStyle = value; 
+				CalcAreasAndInvalidate ();
+				OnBorderStyleChanged (EventArgs.Empty);
 			}
 		}
 
@@ -378,8 +373,7 @@ namespace System.Windows.Forms
 				}
 
 				caption_font = value;
-				grid_drawing.CalcGridAreas ();
-				Refresh ();
+				CalcAreasAndInvalidate ();
 			}
 		}
 
@@ -420,8 +414,7 @@ namespace System.Windows.Forms
 			set {
 				if (caption_visible != value) {
 					caption_visible = value;
-					grid_drawing.CalcGridAreas ();
-					Refresh ();
+					CalcAreasAndInvalidate ();
 					OnCaptionVisibleChanged (EventArgs.Empty);
 				}
 			}
@@ -436,8 +429,7 @@ namespace System.Windows.Forms
 			set {
 				if (columnheaders_visible != value) {
 					columnheaders_visible = value;
-					grid_drawing.CalcGridAreas ();
-					Refresh ();
+					CalcAreasAndInvalidate ();
 				}
 			}
 		}
@@ -452,8 +444,31 @@ namespace System.Windows.Forms
 			set {
 				if (!current_cell.Equals (value)) {
 					CancelEditing ();
+					
+					int old_row = current_cell.RowNumber;
+					
+					if (value.RowNumber >= RowsCount) {
+						value.RowNumber = RowsCount == 0 ? 0 : RowsCount - 1;
+					}
+					
+					if (value.ColumnNumber >= CurrentTableStyle.GridColumnStyles.Count) {
+						value.ColumnNumber = CurrentTableStyle.GridColumnStyles.Count == 0 ? 0: CurrentTableStyle.GridColumnStyles.Count - 1;
+					}
+					
 					EnsureCellVisilibility (value);
-					current_cell = value;
+					current_cell = value;					
+					
+					if (current_cell.RowNumber != old_row) {
+						grid_drawing.InvalidateRowHeader (old_row);
+					}
+					
+					accept_listmgrevents = false;
+
+					if (cached_currencymgr_events !=  null) {
+						cached_currencymgr_events.Position = current_cell.RowNumber;
+					}
+					accept_listmgrevents = true;
+					InvalidateCurrentRowHeader ();
 					OnCurrentCellChanged (EventArgs.Empty);
 				}
 			}
@@ -463,6 +478,10 @@ namespace System.Windows.Forms
 		[DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
 		public int CurrentRowIndex {
 			get {
+				if (ListManager == null) {
+					return -1;
+				}
+				
 				return current_cell.RowNumber;
 			}
 
@@ -485,15 +504,15 @@ namespace System.Windows.Forms
 		}
 
 		[DefaultValue(null)]
-		[Editor ("System.Windows.Forms.Design.DataMemberListEditor, System.Design, Version=1.0.5000.0, Culture=neutral, PublicKeyToken=b03f5f7f11d50a3a", "System.Drawing.Design.UITypeEditor, System.Drawing, Version=1.0.5000.0, Culture=neutral, PublicKeyToken=b03f5f7f11d50a3a")]
+		[Editor ("System.Windows.Forms.Design.DataMemberListEditor, " + Consts.AssemblySystem_Design, "System.Drawing.Design.UITypeEditor, " + Consts.AssemblySystem_Drawing)]
 		public string DataMember {
 			get { return datamember; }
 			set {
 				if (SetDataMember (value)) {					
-					
+					SetDataSource (datasource);
 					if (styles_collection.Contains (value) == true) {
 						CurrentTableStyle = styles_collection[value];
-						current_style.CreateColumnsForTable (true);
+						current_style.CreateColumnsForTable (false);
 					} else {
 						CurrentTableStyle = default_style;
 						current_style.GridColumnStyles.Clear ();
@@ -505,7 +524,7 @@ namespace System.Windows.Forms
 
 		[DefaultValue(null)]
 		[RefreshProperties(RefreshProperties.Repaint)]
-		[TypeConverter("System.Windows.Forms.Design.DataSourceConverter, System.Design, Version=1.0.5000.0, Culture=neutral, PublicKeyToken=b03f5f7f11d50a3a")]
+		[TypeConverter("System.Windows.Forms.Design.DataSourceConverter, " + Consts.AssemblySystem_Design)]
 		public object DataSource {
 			get {
 				return datasource;
@@ -546,17 +565,13 @@ namespace System.Windows.Forms
 			}
 		}
 
-		public Color ForeColor {
+		public override Color ForeColor {
 			get {
-				return forecolor;
+				return base.ForeColor;
 			}
 
 			set {
-				if (forecolor != value) {
-					forecolor = value;
-					OnForeColorChanged (EventArgs.Empty);
-					Refresh ();
-				}
+				base.ForeColor = value;
 			}
 		}
 
@@ -567,7 +582,7 @@ namespace System.Windows.Forms
 
 			set {
 				if (value == Color.Empty) {
-					throw new ArgumentNullException ("Color.Empty value is invalid.");
+					throw new ArgumentException ("Color.Empty value is invalid.");
 				}
 
 				if (gridline_color != value) {
@@ -598,7 +613,7 @@ namespace System.Windows.Forms
 
 			set {
 				if (value == Color.Empty) {
-					throw new ArgumentNullException ("Color.Empty value is invalid.");
+					throw new ArgumentException ("Color.Empty value is invalid.");
 				}
 
 				if (header_backcolor != value) {
@@ -616,7 +631,7 @@ namespace System.Windows.Forms
 			set {
 				if (header_font != null && !header_font.Equals (value)) {
 					header_font = value;
-					Refresh ();
+					CalcAreasAndInvalidate ();
 				}
 			}
 		}
@@ -698,7 +713,16 @@ namespace System.Windows.Forms
 					return null;
 				}
 
-				return (CurrencyManager) BindingContext [real_datasource, DataMember];
+				if (cached_currencymgr != null) {
+					return cached_currencymgr;
+				}
+
+				// If we bind real_datasource object we do not get the events from ListManger
+				// since the object is not the datasource and does not match
+				cached_currencymgr = (CurrencyManager) BindingContext [real_datasource, DataMember];
+				cached_currencymgr_events = (CurrencyManager) BindingContext [datasource, DataMember];
+				ConnectListManagerEvents ();
+				return cached_currencymgr;
 			}
 
 			set {
@@ -764,8 +788,7 @@ namespace System.Windows.Forms
 			set {
 				if (parentrows_visible != value) {
 					parentrows_visible = value;
-					grid_drawing.CalcGridAreas ();
-					Refresh ();
+					CalcAreasAndInvalidate ();
 					OnParentRowsVisibleChanged (EventArgs.Empty);
 				}
 			}
@@ -799,8 +822,7 @@ namespace System.Windows.Forms
 			set {
 				if (preferredrow_height != value) {
 					preferredrow_height = value;
-					grid_drawing.CalcGridAreas ();
-					Refresh ();
+					CalcAreasAndInvalidate ();
 				}
 			}
 		}
@@ -815,7 +837,7 @@ namespace System.Windows.Forms
 				if (_readonly != value) {
 					_readonly = value;
 					OnReadOnlyChanged (EventArgs.Empty);
-					Refresh ();
+					CalcAreasAndInvalidate ();
 				}
 			}
 		}
@@ -829,8 +851,7 @@ namespace System.Windows.Forms
 			set {
 				if (rowheaders_visible != value) {
 					rowheaders_visible = value;
-					grid_drawing.CalcGridAreas ();
-					Refresh ();
+					CalcAreasAndInvalidate ();
 				}
 			}
 		}
@@ -844,8 +865,7 @@ namespace System.Windows.Forms
 			set {
 				if (rowheaders_width != value) {
 					rowheaders_width = value;
-					grid_drawing.CalcGridAreas ();
-					Refresh ();
+					CalcAreasAndInvalidate ();
 				}
 			}
 		}
@@ -945,11 +965,11 @@ namespace System.Windows.Forms
 			get { return first_visiblerow; }
 			set { first_visiblerow = value;}
 		}
-
+		
 		internal int RowsCount {
-			get {
+			get {				
 				if (ListManager != null) {
-					return ListManager.Count;
+					return ListManager.Count;					
 				}
 
 				return 0;
@@ -958,15 +978,34 @@ namespace System.Windows.Forms
 
 		internal int RowHeight {
 			get {
-				if (preferredrow_height > Font.Height + 3 + 1 /* line */) {
-					return preferredrow_height;
+				if (CurrentTableStyle.CurrentPreferredRowHeight > Font.Height + 3 + 1 /* line */) {
+					return CurrentTableStyle.CurrentPreferredRowHeight;
 
 				} else {
 					return Font.Height + 3 + 1 /* line */;
 				}
 			}
 		}
-
+		
+		internal bool ShowEditRow {
+			get {
+				if (ListManager != null && ListManager.CanAddRows == false) {
+					return false;
+				}
+								
+				return _readonly == false;
+			}
+		}
+		
+		// It should only be shown if there are relations that
+		// we do not support right now
+		internal bool ShowParentRowsVisible {
+			get {
+				//See parentrows_visible;
+				return false;
+			}
+		}
+		
 		#endregion Private Instance Properties
 
 		#region Public Instance Methods
@@ -984,7 +1023,15 @@ namespace System.Windows.Forms
 
 		protected virtual void CancelEditing ()
 		{			
-			CurrentTableStyle.GridColumnStyles[current_cell.ColumnNumber].Abort (current_cell.RowNumber);
+			if (current_cell.ColumnNumber < CurrentTableStyle.GridColumnStyles.Count) {
+				CurrentTableStyle.GridColumnStyles[current_cell.ColumnNumber].Abort (current_cell.RowNumber);
+			}
+			
+			if (is_adding == true) {
+				ListManager.RemoveAt (RowsCount - 1);
+				is_adding = false;
+			}
+			
 			is_editing = false;
 			is_changing = false;
 			InvalidateCurrentRowHeader ();
@@ -1018,7 +1065,7 @@ namespace System.Windows.Forms
 
 		protected virtual DataGridColumnStyle CreateGridColumn (PropertyDescriptor prop, bool isDefault)
 		{
-			return CreateGridColumn (prop, isDefault);
+			throw new NotImplementedException();
 		}
 
 		protected override void Dispose (bool disposing)
@@ -1027,12 +1074,18 @@ namespace System.Windows.Forms
 		}
 
 		public virtual bool EndEdit (DataGridColumnStyle gridColumn, int rowNumber, bool shouldAbort)
-		{
-			if (is_editing == false) {
-				return false;
-			}
+		{						
+			if (is_adding == true) {				
+				if (shouldAbort) {
+					ListManager.CancelCurrentEdit ();
+				} else {
+					ListManager.EndCurrentEdit ();
+					CalcAreasAndInvalidate ();
+				}
+				is_adding = false;
+			} 
 
-			if (shouldAbort) {
+			if (shouldAbort || gridColumn.ParentReadOnly ==true) {
 				gridColumn.Abort (rowNumber);
 			} else {
 				gridColumn.Commit (ListManager, rowNumber);
@@ -1089,11 +1142,10 @@ namespace System.Windows.Forms
 			int old_first_visiblerow = first_visiblerow;
 			first_visiblerow = se.NewValue;
 			grid_drawing.UpdateVisibleRowCount ();
-
+			
 			if (first_visiblerow == old_first_visiblerow) {
 				return;
-			}
-
+			}			
 			ScrollToRow (old_first_visiblerow, first_visiblerow);
 		}
 
@@ -1290,6 +1342,37 @@ namespace System.Windows.Forms
 				OnRowHeaderClick (EventArgs.Empty);
 				break;
 			}
+
+			case HitTestType.ColumnHeader:
+			{
+				if (allow_sorting == false)
+					break;
+
+				if (ListManager.List is IBindingList == false)
+					break;
+			
+				ListSortDirection direction = ListSortDirection.Ascending;
+				PropertyDescriptor prop = CurrentTableStyle.GridColumnStyles[testinfo.Column].PropertyDescriptor;
+				IBindingList list = (IBindingList) ListManager.List;
+
+				if (list.SortProperty != null) {
+					CurrentTableStyle.GridColumnStyles[list.SortProperty].ArrowDrawingMode 
+						= DataGridColumnStyle.ArrowDrawing.No;
+				}
+
+				if (prop == list.SortProperty && list.SortDirection == ListSortDirection.Ascending) {
+					direction = ListSortDirection.Descending;
+				}
+				
+				CurrentTableStyle.GridColumnStyles[testinfo.Column].ArrowDrawingMode =
+					direction == ListSortDirection.Ascending ? 
+					DataGridColumnStyle.ArrowDrawing.Ascending : DataGridColumnStyle.ArrowDrawing.Descending;
+				
+				list.ApplySort (prop, direction);
+				Refresh ();
+				break;
+			}
+			
 			default:
 				break;
 			}
@@ -1314,14 +1397,27 @@ namespace System.Windows.Forms
 		{
 			base.OnMouseWheel (e);
 
-			if (e.Delta > 0) {
-				if (current_cell.RowNumber > 0) {
-					CurrentCell = new DataGridCell (current_cell.RowNumber - 1, current_cell.ColumnNumber);
+			if (ctrl_pressed == false) { // scroll horizontal
+				if (e.Delta > 0) {
+					if (current_cell.RowNumber > 0) {
+						CurrentCell = new DataGridCell (current_cell.RowNumber - 1, current_cell.ColumnNumber);
+					}
 				}
-			}
-			else {
-				if (current_cell.RowNumber < RowsCount - 1) {
-					CurrentCell = new DataGridCell (current_cell.RowNumber + 1, current_cell.ColumnNumber);					
+				else {
+					if (current_cell.RowNumber < RowsCount - 1) {
+						CurrentCell = new DataGridCell (current_cell.RowNumber + 1, current_cell.ColumnNumber);					
+					}
+				}
+			} else {
+				if (e.Delta > 0) {
+					if (current_cell.ColumnNumber > 0) {
+						CurrentCell = new DataGridCell (current_cell.RowNumber, current_cell.ColumnNumber - 1);
+					}
+				}
+				else {
+					if (current_cell.ColumnNumber < CurrentTableStyle.GridColumnStyles.Count - 1) {
+						CurrentCell = new DataGridCell (current_cell.RowNumber, current_cell.ColumnNumber + 1);					
+					}
 				}
 			}
 		}
@@ -1335,12 +1431,11 @@ namespace System.Windows.Forms
 
 		protected override void OnPaint (PaintEventArgs pe)
 		{
-			grid_drawing.OnPaint (pe);
+			ThemeEngine.Current.DataGridPaint (pe, this);
 		}
 
 		protected override void OnPaintBackground (PaintEventArgs ebe)
 		{
-			base.OnPaintBackground (ebe);
 		}
 
 		protected virtual void OnParentRowsLabelStyleChanged (EventArgs e)
@@ -1424,8 +1519,9 @@ namespace System.Windows.Forms
 				}
 				break;
 			}
+			case Keys.Tab:
 			case Keys.Right:
-			{
+			{				
 				if (current_cell.ColumnNumber + 1 < CurrentTableStyle.GridColumnStyles.Count) {
 					CurrentCell = new DataGridCell (current_cell.RowNumber, current_cell.ColumnNumber + 1);
 					EditCell (current_cell);
@@ -1475,8 +1571,7 @@ namespace System.Windows.Forms
 				break;
 			}
 			case Keys.Delete:
-			{
-				ICollection keys = selected_rows.Keys;
+			{				
 				foreach (int row in selected_rows.Keys) {
 					ListManager.RemoveAt (row);						
 				}
@@ -1502,8 +1597,7 @@ namespace System.Windows.Forms
 
 			return base.ProcessKeyPreview (ref m);
 		}
-
-		[MonoTODO]	
+		
 		protected bool ProcessTabKey (Keys keyData)
 		{
 			return false;
@@ -1516,7 +1610,7 @@ namespace System.Windows.Forms
 
 		public override void ResetBackColor ()
 		{
-			background_color = def_background_color;
+			backColor = ThemeEngine.Current.DataGridBackColor;
 		}
 
 		public override void ResetForeColor ()
@@ -1555,8 +1649,7 @@ namespace System.Windows.Forms
 		}
 
 		protected void ResetSelection ()
-		{
-			ICollection keys = selected_rows.Keys;
+		{			
 			foreach (int row in selected_rows.Keys) {
 				grid_drawing.InvalidateRow (row);
 				grid_drawing.InvalidateRowHeader (row);
@@ -1588,10 +1681,9 @@ namespace System.Windows.Forms
 
 		public void SetDataBinding (object dataSource, string dataMember)
 		{
-			if (SetDataSource (dataSource) == false  && SetDataMember (dataMember) == false) {
-				return;
-			}
-
+			dataMember = null;
+			SetDataSource (dataSource);
+			SetDataMember (dataMember);		
 			SetNewDataSource ();
 		}
 
@@ -1685,9 +1777,19 @@ namespace System.Windows.Forms
 			grid_drawing.CalcGridAreas ();
 			Invalidate ();
 		}
+		
+		private void ConnectListManagerEvents ()
+		{
+			cached_currencymgr_events.CurrentChanged += new EventHandler (OnListManagerCurrentChanged);			
+		}
+		
+		private void DisconnectListManagerEvents ()
+		{
+			
+		}
 
 		// EndEdit current editing operation
-		public virtual bool EndEdit (bool shouldAbort)
+		internal virtual bool EndEdit (bool shouldAbort)
 		{
 			return EndEdit (CurrentTableStyle.GridColumnStyles[current_cell.ColumnNumber],
 				current_cell.RowNumber, shouldAbort);
@@ -1695,23 +1797,12 @@ namespace System.Windows.Forms
 
 		private void EnsureCellVisilibility (DataGridCell cell)
 		{
-			if (current_cell.RowNumber != cell.RowNumber) {
-				grid_drawing.InvalidateRowHeader (current_cell.RowNumber);
-				grid_drawing.InvalidateRowHeader (cell.RowNumber);
-			}
-
-			if (cell.ColumnNumber < first_visiblecolumn ||
-				cell.ColumnNumber >= first_visiblecolumn + visiblecolumn_count) {
-
-				int col, pixel;
-
-				if (cell.ColumnNumber + 1 >= first_visiblecolumn + visiblecolumn_count) {
-					col = 1 + cell.ColumnNumber - visiblecolumn_count;
-				}else {
-					col = cell.RowNumber;
-				}
-
-				pixel = grid_drawing.GetColumnStartingPixel (col);
+			if (cell.ColumnNumber <= first_visiblecolumn ||
+				cell.ColumnNumber + 1 >= first_visiblecolumn + visiblecolumn_count) {			
+					
+				first_visiblecolumn = grid_drawing.GetFirstColumnForColumnVisilibility (first_visiblecolumn, cell.ColumnNumber);						
+				
+				int pixel = grid_drawing.GetColumnStartingPixel (first_visiblecolumn);
 				ScrollToColumnInPixels (pixel);
 			}
 
@@ -1733,6 +1824,57 @@ namespace System.Windows.Forms
 				vert_scrollbar.Value = first_visiblerow;
 			}
 		}
+		
+		internal IEnumerable GetDataSource (object source, string member)
+		{	
+			IListSource src = (IListSource) source;
+			IList list = src.GetList();
+			IListSource listsource;
+			ITypedList typedlist;
+					
+			if (source is IEnumerable) {
+				return (IEnumerable) source;
+			}
+			
+			if(src.ContainsListCollection == false)	{
+				return list;
+			}
+			
+			listsource = (IListSource) source;
+			
+			if (listsource == null) {
+				return null;
+			}
+			
+			list = src.GetList ();
+			
+			if (list == null) {
+				return null;
+			}
+			
+			typedlist = (ITypedList) list;
+				
+			if (typedlist == null) {
+				return null;
+			}			
+
+			PropertyDescriptorCollection col = typedlist.GetItemProperties (new PropertyDescriptor [0]);
+			PropertyDescriptor prop = col.Find (member, true);
+								
+			if (prop == null) {
+				if (col.Count > 0) {
+					prop = col[0];
+
+					if (prop == null) {
+						return null;
+					}
+				}
+			}
+			
+			IEnumerable result =  (IEnumerable)(prop.GetValue (list[0]));
+			return result;		
+			
+		}
 
 		internal void InvalidateCurrentRowHeader ()
 		{
@@ -1740,32 +1882,32 @@ namespace System.Windows.Forms
 		}
 
 		private bool SetDataMember (string member)
-		{
-			Console.WriteLine ("SetDataMember {0}", member);
+		{			
 			if (member == datamember) {
 				return false;
 			}
 
 			datamember = member;
-			real_datasource = DataSourceHelper.GetResolvedDataSource (datasource, member);
+			real_datasource = GetDataSource (datasource, member);
+			DisconnectListManagerEvents ();
+			cached_currencymgr = cached_currencymgr_events = null;
 			return true;
 		}
 
 		private bool SetDataSource (object source)
-		{
-			if (datasource != null && datasource.Equals (source)) {
-				return false;
-			}
+		{			
 
 			if (source != null && source as IListSource != null && source as IList != null) {
 				throw new Exception ("Wrong complex data binding source");
 			}
-
+			
+			current_cell = new DataGridCell ();
 			datasource = source;
+			DisconnectListManagerEvents ();
+			cached_currencymgr = cached_currencymgr_events = null;
 			try {
-				real_datasource = DataSourceHelper.GetResolvedDataSource (datasource, DataMember);
-			}catch (Exception e) {
-				datamember = "";
+				real_datasource = GetDataSource (datasource, DataMember);
+			}catch (Exception e) {				
 				real_datasource = source;
 			}
 
@@ -1774,13 +1916,12 @@ namespace System.Windows.Forms
 		}
 
 		private void SetNewDataSource ()
-		{
-			// Create Table Style
-			// Create columns Styles
-			// Bind data				
-			current_style.GridColumnStyles.Clear ();
+		{				
+			if (ListManager != null && TableStyles[datamember] == null) {
+				current_style.GridColumnStyles.Clear ();
+			}
 			current_style.CreateColumnsForTable (false);
-			CalcAreasAndInvalidate ();
+			CalcAreasAndInvalidate ();			
 		}
 
 		private void OnKeyUpDG (object sender, KeyEventArgs e)
@@ -1797,35 +1938,53 @@ namespace System.Windows.Forms
 			}
 		}
 		
+		private void OnListManagerCurrentChanged (object sender, EventArgs e)
+		{			
+			if (accept_listmgrevents == false) {
+				return;
+			}
+			
+			CurrentCell = new DataGridCell (cached_currencymgr_events.Position, current_cell.RowNumber);
+		}
+		
 		private void OnTableStylesCollectionChanged (object sender, CollectionChangeEventArgs e)
 		{				
-			Console.WriteLine ("Datagrid.TableStyles Collection Changed {0}, null {1}, name src {2}, name table style {3}", 
-				e.Action,
-				e.Element == null, ListManager.ListName, ((DataGridTableStyle)e.Element).MappingName);
+			if (ListManager == null)
+				return;
+			
+			switch (e.Action){
+				case CollectionChangeAction.Add: {
+					if (e.Element != null && String.Compare (ListManager.ListName, ((DataGridTableStyle)e.Element).MappingName, true) == 0) {
+						CurrentTableStyle = (DataGridTableStyle)e.Element;
+						((DataGridTableStyle) e.Element).CreateColumnsForTable (false);
+					}
+					break;
+				}
+
+				case CollectionChangeAction.Remove: {
+					if (e.Element != null && String.Compare (ListManager.ListName, ((DataGridTableStyle)e.Element).MappingName, true) == 0) {
+						CurrentTableStyle = default_style;						
+						current_style.GridColumnStyles.Clear ();
+						current_style.CreateColumnsForTable (false);
+					}
+					break;
+				}	
+
 				
-			int cnt = ((DataGridTableStyle) e.Element).GridColumnStyles.Count;
-			
-			if (String.Compare (ListManager.ListName, ((DataGridTableStyle)e.Element).MappingName, true) == 0) {			
-				CurrentTableStyle = (DataGridTableStyle)e.Element;
-				((DataGridTableStyle) e.Element).CreateColumnsForTable (false);				
-			}
+				case CollectionChangeAction.Refresh: {
+					if (e.Element != null && String.Compare (ListManager.ListName, ((DataGridTableStyle)e.Element).MappingName, true) == 0) {
+						CurrentTableStyle = (DataGridTableStyle)e.Element;
+						((DataGridTableStyle) e.Element).CreateColumnsForTable (false);
+					} else {
+						CurrentTableStyle = default_style;
+						current_style.GridColumnStyles.Clear ();
+						current_style.CreateColumnsForTable (false);
 
-			
-
-			/*switch (e.Action)  {
-				case CollectionChangeAction.Add:
-					break;
-				
-				case CollectionChangeAction.Remove:
-					break;
-					
-				case CollectionChangeAction.Refresh:
+					}
 					break;
 
-				default:
-					break;
-			}*/
-			
+				}
+			}						
 			CalcAreasAndInvalidate ();
 		}
 
@@ -1834,6 +1993,13 @@ namespace System.Windows.Forms
 			ResetSelection (); // Invalidates selected rows
 			is_editing = false;
 			is_changing = false;
+			
+			if (ShowEditRow && is_adding == false && cell.RowNumber >= RowsCount) {
+				ListManager.AddNew ();
+				is_adding = true;
+				Invalidate (); // We have just added a new row
+			}
+			
 			CurrentTableStyle.GridColumnStyles[cell.ColumnNumber].Edit (ListManager,
 				cell.RowNumber, GetCellBounds (cell.RowNumber, cell.ColumnNumber),
 				_readonly, string.Empty, true);
@@ -1841,8 +2007,7 @@ namespace System.Windows.Forms
 
 		private void ShiftSelection (int index)
 		{
-			int shorter_item = -1, dist = RowsCount + 1, cur_dist;
-			ICollection keys = selected_rows.Keys;
+			int shorter_item = -1, dist = RowsCount + 1, cur_dist;			
 
 			foreach (int row in selected_rows.Keys) {
 
@@ -1884,6 +2049,9 @@ namespace System.Windows.Forms
 
 			if (pixel > horz_pixeloffset) { // ScrollRight
 				int pixels = pixel - horz_pixeloffset;
+				
+				horz_pixeloffset = horiz_scrollbar.Value = pixel;
+				grid_drawing.UpdateVisibleColumn ();
 
 				// Columns header
 				invalidate_column.X = grid_drawing.ColumnsHeadersArea.X + grid_drawing.ColumnsHeadersArea.Width - pixels;
@@ -1897,49 +2065,70 @@ namespace System.Windows.Forms
 				invalidate.Y = grid_drawing.CellsArea.Y;
 				invalidate.Width = pixels;
 				invalidate.Height = grid_drawing.CellsArea.Height;
+				
+				
+				if (columnheaders_visible == true) {
+					invalidate.Y -= grid_drawing.ColumnsHeadersArea.Height;
+					invalidate.Height += grid_drawing.ColumnsHeadersArea.Height;
+				}
+				
 				XplatUI.ScrollWindow (Handle, grid_drawing.CellsArea, -pixels, 0, false);
+				Invalidate (invalidate_column);
+				Invalidate (invalidate);
+
 
 			} else {
 				int pixels = horz_pixeloffset - pixel;
+				Rectangle area = grid_drawing.CellsArea;
+				
+				horz_pixeloffset = horiz_scrollbar.Value = pixel;
+				grid_drawing.UpdateVisibleColumn ();
 
 				// Columns header
 				invalidate_column.X = grid_drawing.ColumnsHeadersArea.X;
 				invalidate_column.Y = grid_drawing.ColumnsHeadersArea.Y;
 				invalidate_column.Width = pixels;
 				invalidate_column.Height = grid_drawing.ColumnsHeadersArea.Height;
-				XplatUI.ScrollWindow (Handle, grid_drawing.ColumnsHeadersArea, pixels, 0, false);
+				//XplatUI.ScrollWindow (Handle, grid_drawing.ColumnsHeadersArea, pixels, 0, false);
 
 				// Cells
 				invalidate.X =  grid_drawing.CellsArea.X;
 				invalidate.Y =  grid_drawing.CellsArea.Y;
 				invalidate.Width = pixels;
 				invalidate.Height = grid_drawing.CellsArea.Height;
-				XplatUI.ScrollWindow (Handle, grid_drawing.CellsArea, pixels, 0, false);
-			}
-
-			horz_pixeloffset = horiz_scrollbar.Value = pixel;
-			grid_drawing.UpdateVisibleColumn ();
-			Invalidate (invalidate_column);
-			Invalidate (invalidate);
+				
+				if (columnheaders_visible == true) {
+					invalidate.Y -= grid_drawing.ColumnsHeadersArea.Height;
+					invalidate.Height += grid_drawing.ColumnsHeadersArea.Height;
+					area.Y -= grid_drawing.ColumnsHeadersArea.Height;
+					area.Height += grid_drawing.ColumnsHeadersArea.Height;
+				}
+				
+				XplatUI.ScrollWindow (Handle, area, pixels, 0, false);
+				Invalidate (invalidate);
+			}		
+			
 		}
 
 		private void ScrollToRow (int old_row, int new_row)
 		{
-			Rectangle invalidate = new Rectangle ();
-
+			Rectangle invalidate = new Rectangle ();			
+			
 			if (new_row > old_row) { // Scrolldown
 				int scrolled_rows = new_row - old_row;
 				int pixels = scrolled_rows * RowHeight;
-
+				Rectangle rows_area = grid_drawing.CellsArea; // Cells area - partial rows space
+				rows_area.Height = grid_drawing.CellsArea.Height - grid_drawing.CellsArea.Height % RowHeight;
+				
 				invalidate.X =  grid_drawing.CellsArea.X;
-				invalidate.Y =  grid_drawing.CellsArea.Y + grid_drawing.CellsArea.Height - pixels;
+				invalidate.Y =  grid_drawing.CellsArea.Y + rows_area.Height - pixels;
 				invalidate.Width = grid_drawing.CellsArea.Width;
 				invalidate.Height = pixels;
 
-				XplatUI.ScrollWindow (Handle, grid_drawing.CellsArea, 0, -pixels, false);
+				XplatUI.ScrollWindow (Handle, rows_area, 0, -pixels, false);
 
 			} else { // ScrollUp
-				int scrolled_rows = old_row - new_row;
+				int scrolled_rows = old_row - new_row;				
 				int pixels = scrolled_rows * RowHeight;
 
 				invalidate.X =  grid_drawing.CellsArea.X;
@@ -1950,7 +2139,7 @@ namespace System.Windows.Forms
 			}
 
 			// Right now we use ScrollWindow Invalidate, let's leave remarked it here for X11 if need it
-			//Invalidate ();
+			//Invalidate (invalidate);
 			Invalidate (grid_drawing.RowsHeadersArea);
 		}
 
