@@ -107,7 +107,14 @@ namespace System.Windows.Forms
 			}
 			#endregion Public Instance Methods
 		}
-
+		
+		internal enum ArrowDrawing
+		{
+			No		= 0,
+			Ascending 	= 1,
+			Descending  	= 2
+		}		
+		
 		#region	Local Variables
 		internal HorizontalAlignment alignment;
 		private int fontheight;
@@ -116,12 +123,14 @@ namespace System.Windows.Forms
 		private string mapping_name;
 		private string null_text;
 		private PropertyDescriptor property_descriptor;
-		private bool read_only;
+		private bool _readonly;
 		private int width;
 		internal bool is_default;
 		internal DataGrid grid;
 		private DataGridColumnHeaderAccessibleObject accesible_object;
 		private StringFormat string_format_hdr;
+		static string def_null_text = "(null)";
+		private ArrowDrawing arrow_drawing = ArrowDrawing.No;
 		#endregion	// Local Variables
 
 		#region Constructors
@@ -142,10 +151,10 @@ namespace System.Windows.Forms
 			fontheight = -1;
 			table_style = null;
 			header_text = string.Empty;
-			mapping_name  = "(null)";
-			null_text = string.Empty;
+			mapping_name  = string.Empty;
+			null_text = def_null_text;
 			accesible_object = new DataGridColumnHeaderAccessibleObject (this);
-			read_only = false;
+			_readonly = false;
 			width = -1;
 			grid = null;
 			is_default = false;
@@ -287,11 +296,11 @@ namespace System.Windows.Forms
 		[DefaultValue(false)]
 		public virtual bool ReadOnly  {
 			get {
-				return read_only;
+				return _readonly;
 			}
 			set {
-				if (value != read_only) {
-					read_only = value;
+				if (value != _readonly) {
+					_readonly = value;
 					
 					if (table_style != null && table_style.DataGrid != null) {
 						table_style.DataGrid.CalcAreasAndInvalidate ();
@@ -328,6 +337,39 @@ namespace System.Windows.Forms
 		#endregion	// Public Instance Properties
 		
 		#region Private Instance Properties
+
+		internal ArrowDrawing ArrowDrawingMode {
+			get { return arrow_drawing; }
+			set { arrow_drawing = value; }
+		}
+		
+		// The logic seems to be that: 
+		// - If DataGrid.ReadOnly is true all the tables and columns are readonly ignoring other settings
+		// - If DataGridTableStyle.ReadOnly is true all columns are readonly ignoring other settings
+		// - If DataGrid.ReadOnly and DataGridTableStyle.ReadOnly are false, the columns settings are mandatory
+		//
+		internal bool ParentReadOnly {
+			get {				
+				if (grid != null) {
+					if (grid.ReadOnly == true) {
+						return true;
+					}
+					
+					if (grid.ListManager != null && grid.ListManager.CanAddRows == false) {
+						return true;
+					}				
+				}
+				
+				if (table_style != null) {
+					if (table_style.ReadOnly == true) {
+						return true;
+					}
+				}
+				
+				return false;
+			}
+		}
+		
 		internal DataGridTableStyle TableStyle {
 			set { table_style = value; }
 		}
@@ -435,7 +477,7 @@ namespace System.Windows.Forms
 		protected internal virtual void SetColumnValueAtRow (CurrencyManager source, int rowNum,  object value)
 		{
 			CheckValidDataSource (source);
-			property_descriptor.SetValue (source.GetItem (rowNum), value);
+			property_descriptor.SetValue (source.GetItem (rowNum), value);			
 		}
 
 		protected virtual void SetDataGrid (DataGrid value)
@@ -478,28 +520,66 @@ namespace System.Windows.Forms
 		internal void PaintHeader (Graphics g, Rectangle bounds, int colNum)
 		{	
 			// Background
-			g.FillRectangle (ThemeEngine.Current.ResPool.GetSolidBrush (DataGridTableStyle.DataGrid.ParentRowsBackColor), 
+			g.FillRectangle (ThemeEngine.Current.ResPool.GetSolidBrush (DataGridTableStyle.CurrentHeaderBackColor), 
 				bounds);
+				
+			if (grid.FlatMode == false) {			
 			
-			// Paint Borders			
-			g.DrawLine (ThemeEngine.Current.ResPool.GetPen (ThemeEngine.Current.ColorButtonHilight),			
-				bounds.X, bounds.Y, bounds.X + bounds.Width, bounds.Y);
-			
-			if (colNum == 0) {	
-				g.DrawLine (ThemeEngine.Current.ResPool.GetPen (ThemeEngine.Current.ColorButtonHilight),
-					bounds.X, bounds.Y, bounds.X, bounds.Y + bounds.Height);
-			} else {
-				g.DrawLine (ThemeEngine.Current.ResPool.GetPen (ThemeEngine.Current.ColorButtonHilight),
-					bounds.X, bounds.Y + 2, bounds.X, bounds.Y + bounds.Height - 2);
+				// Paint Borders
+				g.DrawLine (ThemeEngine.Current.ResPool.GetPen (ThemeEngine.Current.ColorControlLight),			
+					bounds.X, bounds.Y, bounds.X + bounds.Width, bounds.Y);
+				
+				if (colNum == 0) {	
+					g.DrawLine (ThemeEngine.Current.ResPool.GetPen (ThemeEngine.Current.ColorControlLight),
+						bounds.X, bounds.Y, bounds.X, bounds.Y + bounds.Height);
+				} else {
+					g.DrawLine (ThemeEngine.Current.ResPool.GetPen (ThemeEngine.Current.ColorControlLight),
+						bounds.X, bounds.Y + 2, bounds.X, bounds.Y + bounds.Height - 2);
+				}
+				
+				g.DrawLine (ThemeEngine.Current.ResPool.GetPen (ThemeEngine.Current.ColorControlDark),
+					bounds.X + bounds.Width - 1, bounds.Y + 2 , bounds.X + bounds.Width - 1, bounds.Y + bounds.Height - 2);
 			}
 			
-			g.DrawLine (ThemeEngine.Current.ResPool.GetPen (ThemeEngine.Current.ColorButtonShadow),
-				bounds.X + bounds.Width - 1, bounds.Y + 2 , bounds.X + bounds.Width - 1, bounds.Y + bounds.Height - 2);
-			
-			bounds.X += 3;
-			bounds.Width -=	3;
-			g.DrawString (HeaderText, DataGridTableStyle.DataGrid.Font, ThemeEngine.Current.ResPool.GetSolidBrush (DataGridTableStyle.ForeColor), 
+			bounds.X += 2;
+			bounds.Width -=	2;
+			g.DrawString (HeaderText, DataGridTableStyle.HeaderFont, ThemeEngine.Current.ResPool.GetSolidBrush (DataGridTableStyle.CurrentHeaderForeColor), 
 				bounds, string_format_hdr);
+
+			if (arrow_drawing != ArrowDrawing.No) {
+				// Draw 6 x 6
+				Point pnt = new Point (bounds.X + bounds.Width  - 12, bounds.Y + ((bounds.Height - 6)/2));
+				
+				if (arrow_drawing == ArrowDrawing.Ascending) {
+					g.DrawLine (SystemPens.ControlLightLight, pnt.X + 6, pnt.Y + 6, pnt.X + 3, pnt.Y);
+					g.DrawLine (SystemPens.ControlDark, pnt.X, pnt.Y + 6, pnt.X + 6, pnt.Y + 6);
+					g.DrawLine (SystemPens.ControlDark, pnt.X, pnt.Y + 6, pnt.X + 3, pnt.Y);
+				} else {
+					g.DrawLine (SystemPens.ControlLightLight, pnt.X + 6, pnt.Y, pnt.X + 3, pnt.Y + 6);
+					g.DrawLine (SystemPens.ControlDark, pnt.X, pnt.Y, pnt.X + 6, pnt.Y);
+					g.DrawLine (SystemPens.ControlDark, pnt.X, pnt.Y, pnt.X + 3, pnt.Y + 6);
+				}
+				
+			}
+		}
+				
+		internal void PaintNewRow (Graphics g, Rectangle bounds, Brush backBrush, Brush foreBrush)
+		{
+			g.FillRectangle (backBrush, bounds);
+			PaintGridLine (g, bounds);
+		}
+		
+		internal void PaintGridLine (Graphics g, Rectangle bounds)
+		{
+			if (table_style.CurrentGridLineStyle != DataGridLineStyle.Solid) {
+				return;
+			}
+			
+			g.DrawLine (ThemeEngine.Current.ResPool.GetPen (table_style.CurrentGridLineColor),
+				bounds.X, bounds.Y + bounds.Height - 1, bounds.X + bounds.Width - 1, bounds.Y + bounds.Height - 1);
+			
+			g.DrawLine (ThemeEngine.Current.ResPool.GetPen (table_style.CurrentGridLineColor),
+				bounds.X + bounds.Width - 1, bounds.Y , bounds.X + bounds.Width - 1, bounds.Y + bounds.Height);
 		}
 		
 		#endregion Private Instance Methods
