@@ -28,6 +28,7 @@
 // NOT COMPLETE
 
 using System.Drawing;
+using System.Drawing.Drawing2D;
 
 namespace System.Windows.Forms
 {
@@ -38,22 +39,20 @@ namespace System.Windows.Forms
 		private DataGrid grid;
 
 		// Areas
-		private Rectangle client_area;		// ClientRectangle - BorderStyle decorations, effetive client area
-		private Rectangle caption_area;
-		private Rectangle parent_rows;
-		private Rectangle columnshdrs_area;	// Used columns header area
-		private int columnshdrs_maxwidth; 	// Total width (max width) for columns headrs
-		private Rectangle rowshdrs_area;	// Used Headers rows area
-		private int rowshdrs_maxheight; 	// Total height for rows (max height)
-		private Rectangle cells_area;
-		private bool partial_row;
-
+		internal Rectangle caption_area;
+		internal Rectangle parent_rows;
+		internal Rectangle columnshdrs_area;	// Used columns header area
+		internal int columnshdrs_maxwidth; 	// Total width (max width) for columns headrs
+		internal Rectangle rowshdrs_area;	// Used Headers rows area
+		internal int rowshdrs_maxheight; 	// Total height for rows (max height)
+		internal Rectangle cells_area;
+		internal Font font_newrow = new Font (FontFamily.GenericSansSerif, 16);
 		#endregion // Local Variables
 
 
 		public DataGridDrawing (DataGrid datagrid)
 		{
-			 grid = datagrid;
+			 grid = datagrid;			 
 		}
 
 		#region Public Instance Methods
@@ -79,6 +78,9 @@ namespace System.Windows.Forms
 
 			if (cnt == 0)
 				return 0;
+				
+			if (grid.CurrentTableStyle.CurrentRowHeadersVisible)
+				width += rowshdrs_area.X + rowshdrs_area.Width;
 
 			for (int col = 0; col < cnt; col++) {
 				width += grid.CurrentTableStyle.GridColumnStyles[col].Width;
@@ -106,19 +108,25 @@ namespace System.Windows.Forms
 
 			return 0;
 		}
-
-		public void CalcClientArea ()
+		
+		// Which column has to be the first visible column to ensure a column visibility
+		public int GetFirstColumnForColumnVisilibility (int current_first_visiblecolumn, int column)
 		{
-			client_area = grid.ClientRectangle;
-			client_area.X += BorderStyleSize;
-			client_area.Y += BorderStyleSize;
-			client_area.Width -= BorderStyleSize * 2;
-			client_area.Height -= BorderStyleSize * 2;
-
-			//Console.WriteLine ("CalcClientArea");
-
-			//Console.WriteLine ("CalcClientArea ClientRectangle {0}, ClientArea {1}, BorderStyleSize {2}",
-			//	 grid.ClientRectangle, client_area, BorderStyleSize);
+			int new_col = column;
+			int width = 0;
+			
+			if (column > current_first_visiblecolumn) { // Going forward								
+				for (new_col = column; new_col >= 0; new_col--){
+					width += grid.CurrentTableStyle.GridColumnStyles[new_col].Width;
+					
+					if (width >= cells_area.Width)
+						return new_col + 1;
+						//return new_col < grid.CurrentTableStyle.GridColumnStyles.Count ? new_col + 1 : grid.CurrentTableStyle.GridColumnStyles.Count;
+				}
+				return 0;
+			} else {				
+				return  column;
+			}			
 		}
 
 		public void CalcGridAreas ()
@@ -127,7 +135,7 @@ namespace System.Windows.Forms
 				return;
 
 			/* Order is important. E.g. row headers max. height depends on caption */
-			CalcClientArea ();
+			grid.horz_pixeloffset = 0;			
 			CalcCaption ();
 			CalcParentRows ();
 			CalcRowsHeaders ();
@@ -136,11 +144,8 @@ namespace System.Windows.Forms
 
 			UpdateVisibleRowCount (); // need it to be able to calcultate the need of horz scrollbar
 			if (SetUpVerticalScrollBar ()) { // We need a Vertical ScrollBar
-				if (grid.caption_visible) {
-					caption_area.Width -= grid.vert_scrollbar.Width;
-				}
-
-				if (grid.parentrows_visible) {
+				
+				if (grid.ShowParentRowsVisible) {
 					parent_rows.Width -= grid.vert_scrollbar.Width;
 				}
 
@@ -150,7 +155,7 @@ namespace System.Windows.Forms
 					}
 				}
 
-				if (cells_area.X + cells_area.Width > grid.vert_scrollbar.Location.X) {
+				if (cells_area.X + cells_area.Width >= grid.vert_scrollbar.Location.X) {
 					cells_area.Width -= grid.vert_scrollbar.Width;
 				}
 			}
@@ -158,9 +163,9 @@ namespace System.Windows.Forms
 			if (SetUpHorizontalScrollBar ()) { // We need a Horizontal ScrollBar
 				cells_area.Height -= grid.horiz_scrollbar.Height;
 
-				if (rowshdrs_area.Y + rowshdrs_area.Height > client_area.Y + client_area.Height) {
-					rowshdrs_area.Height -= grid.horiz_scrollbar.Width;
-					rowshdrs_maxheight -= grid.horiz_scrollbar.Width;
+				if (rowshdrs_area.Y + rowshdrs_area.Height > grid.ClientRectangle.Y + grid.ClientRectangle.Height) {
+					rowshdrs_area.Height -= grid.horiz_scrollbar.Height;
+					rowshdrs_maxheight -= grid.horiz_scrollbar.Height;
 				}
 			}
 
@@ -183,9 +188,9 @@ namespace System.Windows.Forms
 				return;
 			}
 
-			caption_area.X = client_area.X;
-			caption_area.Y = client_area.Y;
-			caption_area.Width = client_area.Width;
+			caption_area.X = grid.ClientRectangle.X;
+			caption_area.Y = grid.ClientRectangle.Y;
+			caption_area.Width = grid.ClientRectangle.Width;
 			caption_area.Height = grid.CaptionFont.Height + 6;
 
 			//Console.WriteLine ("DataGridDrawing.CalcCaption {0}", caption_area);
@@ -196,10 +201,10 @@ namespace System.Windows.Forms
 			if (grid.caption_visible) {
 				cells_area.Y = caption_area.Y + caption_area.Height;
 			} else {
-				cells_area.Y = client_area.Y;
+				cells_area.Y = grid.ClientRectangle.Y;
 			}
 
-			if (grid.parentrows_visible) {
+			if (grid.ShowParentRowsVisible) {
 				cells_area.Y += parent_rows.Height;
 			}
 
@@ -207,9 +212,9 @@ namespace System.Windows.Forms
 				cells_area.Y += columnshdrs_area.Height;
 			}
 
-			cells_area.X = client_area.X + rowshdrs_area.Width;
-			cells_area.Width = client_area.X + client_area.Width - cells_area.X;
-			cells_area.Height = client_area.Y + client_area.Height - cells_area.Y;
+			cells_area.X = grid.ClientRectangle.X + rowshdrs_area.Width;
+			cells_area.Width = grid.ClientRectangle.X + grid.ClientRectangle.Width - cells_area.X;
+			cells_area.Height = grid.ClientRectangle.Y + grid.ClientRectangle.Height - cells_area.Y;
 
 			//Console.WriteLine ("DataGridDrawing.CalcCellsArea {0}", cells_area);
 		}
@@ -226,22 +231,22 @@ namespace System.Windows.Forms
 			if (grid.caption_visible) {
 				columnshdrs_area.Y = caption_area.Y + caption_area.Height;
 			} else {
-				columnshdrs_area.Y = client_area.Y;
+				columnshdrs_area.Y = grid.ClientRectangle.Y;
 			}
 
-			if (grid.parentrows_visible) {
+			if (grid.ShowParentRowsVisible) {
 				columnshdrs_area.Y += parent_rows.Height;
 			}
 
-			columnshdrs_area.X = client_area.X;
+			columnshdrs_area.X = grid.ClientRectangle.X;
 			columnshdrs_area.Height = ColumnsHeaderHeight;
 			width_all_cols = CalcAllColumnsWidth ();
 
 			// TODO: take into account Scrollbars
-			columnshdrs_maxwidth = client_area.X + client_area.Width - columnshdrs_area.X;
+			columnshdrs_maxwidth = grid.ClientRectangle.X + grid.ClientRectangle.Width - columnshdrs_area.X;
 			max_width_cols = columnshdrs_maxwidth;
 
-			if (grid.rowheaders_visible) {
+			if (grid.CurrentTableStyle.CurrentRowHeadersVisible) {
 				max_width_cols -= grid.RowHeaderWidth;
 			}
 
@@ -250,7 +255,7 @@ namespace System.Windows.Forms
 			} else {
 				columnshdrs_area.Width = width_all_cols;
 
-				if (grid.rowheaders_visible) {
+				if (grid.CurrentTableStyle.CurrentRowHeadersVisible) {
 					columnshdrs_area.Width += grid.RowHeaderWidth;
 				}
 			}
@@ -260,7 +265,7 @@ namespace System.Windows.Forms
 
 		public void CalcParentRows ()
 		{
-			if (grid.parentrows_visible == false) {
+			if (grid.ShowParentRowsVisible == false) {
 				parent_rows = Rectangle.Empty;
 				return;
 			}
@@ -269,11 +274,11 @@ namespace System.Windows.Forms
 				parent_rows.Y = caption_area.Y + caption_area.Height;
 
 			} else {
-				parent_rows.Y = client_area.Y;
+				parent_rows.Y = grid.ClientRectangle.Y;
 			}
 
-			parent_rows.X = client_area.X;
-			parent_rows.Width = client_area.Width;
+			parent_rows.X = grid.ClientRectangle.X;
+			parent_rows.Width = grid.ClientRectangle.Width;
 			parent_rows.Height = grid.CaptionFont.Height + 3;
 
 			//Console.WriteLine ("DataGridDrawing.CalcParentRows {0}", parent_rows);
@@ -281,7 +286,7 @@ namespace System.Windows.Forms
 
 		public void CalcRowsHeaders ()
 		{
-			if (grid.rowheaders_visible == false) {
+			if (grid.CurrentTableStyle.CurrentRowHeadersVisible == false) {
 				rowshdrs_area = Rectangle.Empty;
 				return;
 			}
@@ -289,10 +294,10 @@ namespace System.Windows.Forms
 			if (grid.caption_visible) {
 				rowshdrs_area.Y = caption_area.Y + caption_area.Height;
 			} else {
-				rowshdrs_area.Y = client_area.Y;
+				rowshdrs_area.Y = grid.ClientRectangle.Y;
 			}
 
-			if (grid.parentrows_visible) {
+			if (grid.ShowParentRowsVisible) {
 				rowshdrs_area.Y += parent_rows.Height;
 			}
 
@@ -300,10 +305,10 @@ namespace System.Windows.Forms
 				rowshdrs_area.Y += ColumnsHeaderHeight;
 			}
 
-			rowshdrs_area.X = client_area.X;
+			rowshdrs_area.X = grid.ClientRectangle.X;
 			rowshdrs_area.Width = grid.RowHeaderWidth;
 			rowshdrs_area.Height = grid.visiblerow_count * grid.RowHeight;
-			rowshdrs_maxheight = client_area.Height + client_area.Y - rowshdrs_area.Y;
+			rowshdrs_maxheight = grid.ClientRectangle.Height + grid.ClientRectangle.Y - rowshdrs_area.Y;
 
 			//Console.WriteLine ("DataGridDrawing.CalcRowsHeaders {0} {1}", rowshdrs_area,
 			//	rowshdrs_maxheight);
@@ -321,45 +326,50 @@ namespace System.Windows.Forms
 			grid.first_visiblecolumn = FromPixelToColumn (grid.horz_pixeloffset);
 
 			col = FromPixelToColumn (max_pixel);
-			grid.visiblecolumn_count = (col - grid.first_visiblecolumn);
-			grid.visiblecolumn_count++;
-
-			//Console.WriteLine ("UpdateVisibleColumn col: {0}, cnt {1}",
-			//	grid.first_visiblecolumn, grid.visiblecolumn_count);
+			
+			grid.visiblecolumn_count = 1 + col - grid.first_visiblecolumn;
+			
+			if (grid.first_visiblecolumn + grid.visiblecolumn_count + 1 < grid.CurrentTableStyle.GridColumnStyles.Count) { 
+				grid.visiblecolumn_count++; // Partially visible column
+			}
 		}
 
 		public void UpdateVisibleRowCount ()
 		{
 			int max_height = cells_area.Height;
+			int total_rows = grid.RowsCount;
+			
+			if (grid.ShowEditRow) {
+				total_rows++;
+			}
 
-			int rows_height = (grid.RowsCount - grid.first_visiblerow) * grid.RowHeight;
+			int rows_height = (total_rows - grid.first_visiblerow) * grid.RowHeight;
 			int max_rows = max_height / grid.RowHeight;
 
 			//Console.WriteLine ("UpdateVisibleRowCount {0} {1}/{2} (row h) {3}",
 			//	max_rows, max_height, grid.RowHeight, cells_area.Height);
 
-			if (max_rows > grid.RowsCount) {
-				max_rows = grid.RowsCount;
+			if (max_rows > total_rows) {
+				max_rows = total_rows;
 			}
 
 			if (rows_height > cells_area.Height) {
 				grid.visiblerow_count = max_rows;
 			} else {
-				grid.visiblerow_count = grid.RowsCount;
-			}
+				grid.visiblerow_count = total_rows;
+			}	
 
-			if (grid.visiblerow_count + grid.first_visiblerow > grid.RowsCount)
-				grid.visiblerow_count = grid.RowsCount - grid.first_visiblerow;
+			CalcRowsHeaders (); // Height depends on num of visible rows		
+			
+			if (grid.visiblerow_count + grid.first_visiblerow > total_rows)
+				grid.visiblerow_count = total_rows - grid.first_visiblerow;
 
 			if (grid.visiblerow_count < max_rows) {
 				grid.visiblerow_count = max_rows;
-				grid.first_visiblerow = grid.RowsCount - max_rows;
+				grid.first_visiblerow = total_rows - max_rows;
 				grid.Invalidate ();
-			}
-
-			// TODO: Partial visible rows? May be only paint them and not take then into account for any
-			// anything else
-			//grid.visiblerow_count++;
+			}		
+			
 		}
 
 		// From Point to Cell
@@ -370,6 +380,7 @@ namespace System.Windows.Forms
 			// TODO: Add missing ColumnResize and RowResize checks
 			if (columnshdrs_area.Contains (x, y)) {
 				hit.type = DataGrid.HitTestType.ColumnHeader;
+				hit.column = FromPixelToColumn (x + grid.horz_pixeloffset);
 				return hit;
 			}
 
@@ -404,8 +415,7 @@ namespace System.Windows.Forms
 
 				if (y <= pos_y + grid.RowHeight) { // Found row
 					hit.row = row;
-					hit.type = DataGrid.HitTestType.Cell;
-					int cnt = grid.CurrentTableStyle.GridColumnStyles.Count;
+					hit.type = DataGrid.HitTestType.Cell;					
 					int col_pixel;
 					int column_cnt = grid.first_visiblecolumn + grid.visiblecolumn_count;
 					for (int column = grid.first_visiblecolumn; column < column_cnt; column++) {
@@ -447,147 +457,8 @@ namespace System.Windows.Forms
 
 			grid.Invalidate (caption_area);
 		}
-
-		public void OnPaint (PaintEventArgs pe)
-		{
-			Console.WriteLine ("OnPaint {0}", pe.ClipRectangle);
-			ThemeEngine.Current.CPDrawBorderStyle (pe.Graphics, grid.ClientRectangle, grid.border_style);
-
-			if (pe.ClipRectangle.IntersectsWith (parent_rows)) {
-				pe.Graphics.FillRectangle (ThemeEngine.Current.ResPool.GetSolidBrush (grid.ParentRowsBackColor), parent_rows);
-			}
-
-			PaintCaption (pe.Graphics, pe.ClipRectangle);
-			PaintColumnsHdrs (pe.Graphics, pe.ClipRectangle);
-			PaintRowsHeaders (pe.Graphics, pe.ClipRectangle);
-			PaintRows (pe.Graphics, cells_area, pe.ClipRectangle);
-
-			// Paint scrollBar corner
-			if (grid.vert_scrollbar.Visible && grid.horiz_scrollbar.Visible) {
-
-				Rectangle corner = new Rectangle (client_area.X + client_area.Width - grid.horiz_scrollbar.Height,
-					 client_area.Y + client_area.Height - grid.horiz_scrollbar.Height,
-					 grid.horiz_scrollbar.Height, grid.horiz_scrollbar.Height);
-
-				if (pe.ClipRectangle.IntersectsWith (corner)) {
-					pe.Graphics.FillRectangle (ThemeEngine.Current.ResPool.GetSolidBrush (grid.ParentRowsBackColor),
-						corner);
-				}
-			}
-		}
-
-		public void PaintCaption (Graphics g, Rectangle clip)
-		{
-			Region modified_area =  new Region (clip);
-			modified_area.Intersect (caption_area);
-
-			g.FillRectangle (ThemeEngine.Current.ResPool.GetSolidBrush (grid.CaptionBackColor),
-				Rectangle.Ceiling (modified_area.GetBounds (g)));
-
-			g.DrawString (grid.CaptionText, grid.CaptionFont,
-				ThemeEngine.Current.ResPool.GetSolidBrush (grid.CaptionForeColor),
-				caption_area);
-
-			modified_area.Dispose ();
-		}
-
-		public void PaintColumnsHdrs (Graphics g, Rectangle clip)
-		{
-			Rectangle columns_area = columnshdrs_area;
-
-			if (grid.rowheaders_visible) { // Paint corner shared between row and column header
-				Rectangle rect_bloc = columnshdrs_area;
-				rect_bloc.Width = grid.RowHeaderWidth;
-				rect_bloc.Height = columnshdrs_area.Height;
-				if (clip.IntersectsWith (rect_bloc)) {
-					g.FillRectangle (ThemeEngine.Current.ResPool.GetSolidBrush (grid.ParentRowsBackColor), rect_bloc);
-				}
-
-				columns_area.X += grid.RowHeaderWidth;
-				columns_area.Width -= grid.RowHeaderWidth;
-			}
-
-			// Set unused area
-			Rectangle columnshdrs_area_complete = columns_area;
-			columnshdrs_area_complete.Width = columnshdrs_maxwidth;
-			Region not_usedarea = new Region (clip);
-
-			if (grid.rowheaders_visible) {
-				columnshdrs_area_complete.Width -= grid.RowHeaderWidth;
-			}
-
-			not_usedarea.Intersect (columnshdrs_area_complete);
-
-			// Set column painting
-			Rectangle rect_columnhdr = new Rectangle ();
-			int cnt = grid.CurrentTableStyle.GridColumnStyles.Count, col_pixel;
-			Region prev_clip = g.Clip, current_clip;
-			rect_columnhdr.Y = columns_area.Y;
-			rect_columnhdr.Height = columns_area.Height;
-
-			int column_cnt = grid.first_visiblecolumn + grid.visiblecolumn_count;
-			for (int column = grid.first_visiblecolumn; column < column_cnt; column++) {
-
-				col_pixel = GetColumnStartingPixel (column);
-				rect_columnhdr.X = columns_area.X + col_pixel - grid.horz_pixeloffset;
-				rect_columnhdr.Width = grid.CurrentTableStyle.GridColumnStyles[column].Width;
-
-				if (clip.IntersectsWith (rect_columnhdr) == false)
-					continue;
-
-				not_usedarea.Exclude (rect_columnhdr);
-				current_clip = new Region (columns_area);
-				g.Clip = current_clip;
-
-				grid.CurrentTableStyle.GridColumnStyles[column].PaintHeader (g, rect_columnhdr, column);
-
-				g.Clip = prev_clip;
-				current_clip.Dispose ();
-			}
-
-			// This fills with background colourt the unused part in the row headers
-			g.FillRectangle (ThemeEngine.Current.ResPool.GetSolidBrush (grid.BackgroundColor),
-				Rectangle.Ceiling (not_usedarea.GetBounds (g)));
-
-			//Console.WriteLine ("PaintColumnsHdrs Clean {0}", Rectangle.Ceiling (not_usedarea.GetBounds (g)));
-
-			not_usedarea.Dispose ();
-		}
-
-		public void PaintRowsHeaders (Graphics g, Rectangle clip)
-		{
-			Rectangle rowshdrs_area_complete = rowshdrs_area;
-			rowshdrs_area_complete.Height = rowshdrs_maxheight;
-			Rectangle rect_row = new Rectangle ();
-			rect_row.X = rowshdrs_area.X;
-			Region not_usedarea = new Region (clip);
-			not_usedarea.Intersect (rowshdrs_area_complete);
-
-			int rowcnt = grid.FirstVisibleRow + grid.VisibleRowCount;
-
-			if (rowcnt < grid.RowsCount) { // Paint one row more for partial rows
-				rowcnt++;
-			}
-
-			for (int row = grid.FirstVisibleRow; row < rowcnt; row++) {
-
-				rect_row.Width = rowshdrs_area.Width;
-				rect_row.Height = grid.RowHeight;
-				rect_row.Y = rowshdrs_area.Y + ((row - grid.FirstVisibleRow) * grid.RowHeight);
-
-				if (clip.IntersectsWith (rect_row)) {
-					PaintRowHeader (g, rect_row, row);
-					not_usedarea.Exclude (rect_row);
-				}
-			}
-
-			// This fills with background colourt the unused part in the row headers
-			g.FillRectangle (ThemeEngine.Current.ResPool.GetSolidBrush (grid.BackgroundColor),
-				Rectangle.Ceiling (not_usedarea.GetBounds (g)));
-
-			not_usedarea.Dispose ();
-		}
-
+		
+		
 		public void InvalidateRow (int row)
 		{
 			if (row < grid.FirstVisibleRow || row > grid.FirstVisibleRow + grid.VisibleRowCount) {
@@ -611,49 +482,7 @@ namespace System.Windows.Forms
 			rect_rowhdr.Height = grid.RowHeight;
 			rect_rowhdr.Y = rowshdrs_area.Y + ((row - grid.FirstVisibleRow) * grid.RowHeight);
 			grid.Invalidate (rect_rowhdr);
-		}
-
-		public void PaintRowHeader (Graphics g, Rectangle bounds, int row)
-		{
-			// Background
-			g.FillRectangle (ThemeEngine.Current.ResPool.GetSolidBrush (grid.ParentRowsBackColor),
-				bounds);
-
-			// Paint Borders
-			g.DrawLine (ThemeEngine.Current.ResPool.GetPen (ThemeEngine.Current.ColorButtonHilight),
-				bounds.X, bounds.Y, bounds.X + bounds.Width, bounds.Y);
-
-			g.DrawLine (ThemeEngine.Current.ResPool.GetPen (ThemeEngine.Current.ColorButtonHilight),
-				bounds.X, bounds.Y + 2, bounds.X, bounds.Y + bounds.Height - 2);
-
-			g.DrawLine (ThemeEngine.Current.ResPool.GetPen (ThemeEngine.Current.ColorButtonShadow),
-				bounds.X + bounds.Width - 1, bounds.Y + 2 , bounds.X + bounds.Width - 1, bounds.Y + bounds.Height - 2);
-
-			g.DrawLine (ThemeEngine.Current.ResPool.GetPen (ThemeEngine.Current.ColorButtonShadow),
-				bounds.X, bounds.Y + bounds.Height -1, bounds.X + bounds.Width, bounds.Y  + bounds.Height -1);
-
-			// Draw arrow
-			if (row == grid.CurrentCell.RowNumber) {
-
-				if (grid.is_changing == true) {
-					g.DrawString ("...", grid.Font,
-						ThemeEngine.Current.ResPool.GetSolidBrush (Color.Black),
-						bounds);
-
-				} else {
-					int cx = 16;
-					int cy = 16;
-					Bitmap	bmp = new Bitmap (cx, cy);
-					Graphics gr = Graphics.FromImage (bmp);
-					Rectangle rect_arrow = new Rectangle (0, 0, cx, cy);
-					ControlPaint.DrawMenuGlyph (gr, rect_arrow, MenuGlyph.Arrow);
-					bmp.MakeTransparent ();
-					g.DrawImage (bmp, bounds.X + 1, bounds.Y + 1, cx, cy);
-					gr.Dispose ();
-					bmp.Dispose ();
-				}
-			}
-		}
+		}	
 
 		public void InvalidateColumn (DataGridColumnStyle column)
 		{
@@ -674,91 +503,7 @@ namespace System.Windows.Forms
 			rect_col.Height = cells_area.Height;
 			grid.Invalidate (rect_col);
 		}
-
-		public void PaintRows (Graphics g, Rectangle cells, Rectangle clip)
-		{
-			Rectangle rect_row = new Rectangle ();
-			rect_row.X = cells.X;
-			Region not_usedarea = new Region (clip);
-			not_usedarea.Intersect (cells_area);
-
-			int rowcnt = grid.FirstVisibleRow + grid.VisibleRowCount;
-
-			if (rowcnt < grid.RowsCount) { // Paint one row more for partial rows
-				rowcnt++;
-			}
-
-			for (int row = grid.FirstVisibleRow; row < rowcnt; row++) {
-				// Get cell's width for column style
-				rect_row.Width = cells.Width;
-				rect_row.Height = grid.RowHeight;
-				rect_row.Y = cells.Y + ((row - grid.FirstVisibleRow) * grid.RowHeight);
-
-				if (clip.IntersectsWith (rect_row)) {
-					PaintRow (g, row, rect_row);
-					not_usedarea.Exclude (rect_row);
-				}
-			}
-
-			g.FillRectangle (ThemeEngine.Current.ResPool.GetSolidBrush (grid.BackgroundColor),
-				Rectangle.Ceiling (not_usedarea.GetBounds (g)));
-
-			not_usedarea.Dispose ();
-		}
-
-		public void PaintRow (Graphics g, int row, Rectangle row_rect)
-		{
-			//Console.WriteLine ("PaintRow row: {0}, rect {1}", row, row_rect);
-
-			Rectangle rect_cell = new Rectangle ();
-			int cnt = grid.CurrentTableStyle.GridColumnStyles.Count;
-			int col_pixel;
-			Color backcolor, forecolor;
-			Region prev_clip = g.Clip;
-			Region current_clip;
-			Region not_usedarea = new Region (row_rect);
-
-			rect_cell.Y = row_rect.Y;
-			rect_cell.Height = row_rect.Height;
-
-			// PaintCells at row, column
-			int column_cnt = grid.first_visiblecolumn + grid.visiblecolumn_count;
-			for (int column = grid.first_visiblecolumn; column < column_cnt; column++) {
-
-				col_pixel = GetColumnStartingPixel (column);
-
-				rect_cell.X = row_rect.X + col_pixel - grid.horz_pixeloffset;
-				rect_cell.Width = grid.CurrentTableStyle.GridColumnStyles[column].Width;
-
-				current_clip = new Region (row_rect);
-				g.Clip = current_clip;
-
-				if (grid.IsSelected (row)) {
-					backcolor =  grid.SelectionBackColor;
-					forecolor =  grid.SelectionForeColor;
-				} else {
-					backcolor =  grid.BackgroundColor;
-					forecolor =  grid.ForeColor;
-				}
-
-				not_usedarea.Exclude (rect_cell);
-
-				grid.CurrentTableStyle.GridColumnStyles[column].Paint (g, rect_cell, grid.ListManager, row,
-					ThemeEngine.Current.ResPool.GetSolidBrush (backcolor),
-					ThemeEngine.Current.ResPool.GetSolidBrush (forecolor),
-					grid.RightToLeft == RightToLeft.Yes);
-
-				g.Clip = prev_clip;
-				current_clip.Dispose ();
-			}
-
-			g.FillRectangle (ThemeEngine.Current.ResPool.GetSolidBrush (grid.BackgroundColor),
-				Rectangle.Ceiling (not_usedarea.GetBounds (g)));
-
-			not_usedarea.Dispose ();
-
-		}
-
+				
 		// Return true if the scrollbar is needed
 		public bool SetUpHorizontalScrollBar ()
 		{
@@ -770,13 +515,13 @@ namespace System.Windows.Forms
 				return false;
 			}
 
-			grid.horiz_scrollbar.Location = new Point (client_area.X, client_area.Y +
-				client_area.Height - grid.horiz_scrollbar.Height);
+			grid.horiz_scrollbar.Location = new Point (grid.ClientRectangle.X, grid.ClientRectangle.Y +
+				grid.ClientRectangle.Height - grid.horiz_scrollbar.Height);
 
-			grid.horiz_scrollbar.Size = new Size (client_area.Width,
+			grid.horiz_scrollbar.Size = new Size (grid.ClientRectangle.Width,
 				grid.horiz_scrollbar.Height);
 
-			grid.horiz_scrollbar.Maximum = width_all - cells_area.Width;
+			grid.horiz_scrollbar.Maximum = width_all;// - cells_area.Width;
 			grid.horiz_scrollbar.LargeChange = cells_area.Width;
 			grid.Controls.Add (grid.horiz_scrollbar);
 			grid.horiz_scrollbar.Visible = true;
@@ -786,19 +531,39 @@ namespace System.Windows.Forms
 		// Return true if the scrollbar is needed
 		public bool SetUpVerticalScrollBar ()
 		{
-			if (grid.visiblerow_count == grid.RowsCount) {
+			int y, height;
+			int allrows = grid.RowsCount;
+
+			if (grid.ShowEditRow) {
+				allrows++;
+			}
+			
+			if (grid.visiblerow_count == allrows) {
 				grid.vert_scrollbar.Visible = false;
 				grid.Controls.Remove (grid.vert_scrollbar);
 				return false;
 			}
+			
+			if (grid.caption_visible) {
+				y = grid.ClientRectangle.Y + caption_area.Height;
+				height = grid.ClientRectangle.Height - caption_area.Height;
+			} else {
+				y = grid.ClientRectangle.Y;
+				height = grid.ClientRectangle.Height;
+			}
 
-			grid.vert_scrollbar.Location = new Point (client_area.X +
-				client_area.Width - grid.vert_scrollbar.Width, client_area.Y);
+			grid.vert_scrollbar.Location = new Point (grid.ClientRectangle.X +
+				grid.ClientRectangle.Width - grid.vert_scrollbar.Width, y);
 
 			grid.vert_scrollbar.Size = new Size (grid.vert_scrollbar.Width,
-				client_area.Height);
+				height);
 
 			grid.vert_scrollbar.Maximum = grid.RowsCount;
+			
+			if (grid.ShowEditRow) {
+				grid.vert_scrollbar.Maximum++;	
+			}
+			
 			grid.vert_scrollbar.LargeChange = VLargeChange;
 
 			grid.Controls.Add (grid.vert_scrollbar);
@@ -809,22 +574,6 @@ namespace System.Windows.Forms
 		#endregion // Public Instance Methods
 
 		#region Instance Properties
-		internal int BorderStyleSize {
-			get {
-				switch (grid.border_style) {
-					case BorderStyle.Fixed3D:
-						return 2;
-					case BorderStyle.FixedSingle:
-						return 1;
-					case BorderStyle.None:
-					default:
-						break;
-					}
-
-				return 0;
-			}
-		}
-
 		public Rectangle CellsArea {
 			get {
 				return cells_area;
@@ -836,7 +585,7 @@ namespace System.Windows.Forms
 			get {
 				Rectangle columns_area = columnshdrs_area;
 
-				if (grid.rowheaders_visible) {
+				if (grid.CurrentTableStyle.CurrentRowHeadersVisible) {
 					columns_area.X += grid.RowHeaderWidth;
 					columns_area.Width -= grid.RowHeaderWidth;
 				}
@@ -846,7 +595,7 @@ namespace System.Windows.Forms
 
 		public int ColumnsHeaderHeight {
 			get {
-				return grid.Font.Height + 6;
+				return grid.CurrentTableStyle.HeaderFont.Height + 6;
 			}
 		}
 
