@@ -17,7 +17,7 @@
 // OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 // WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 //
-// Copyright (c) 2004 Novell, Inc.
+// Copyright (c) 2004-2005 Novell, Inc.
 //
 // Authors:
 //	John BouAntoun	jba-mono@optusnet.com.au
@@ -34,6 +34,7 @@ using System;
 using System.Drawing;
 using System.Collections;
 using System.ComponentModel;
+using System.Runtime.InteropServices;
 using System.Windows.Forms;
 
 namespace System.Windows.Forms {
@@ -53,8 +54,8 @@ namespace System.Windows.Forms {
 		#region Local variables
 		
 		protected static readonly Color DefaultMonthBackColor = ThemeEngine.Current.ColorWindow;
-		protected static readonly Color DefaultTitleBackColor = ThemeEngine.Current.ColorActiveTitle;
-		protected static readonly Color DefaultTitleForeColor = ThemeEngine.Current.ColorTitleText;
+		protected static readonly Color DefaultTitleBackColor = ThemeEngine.Current.ColorActiveCaption;
+		protected static readonly Color DefaultTitleForeColor = ThemeEngine.Current.ColorActiveCaptionText;
 		protected static readonly Color DefaultTrailingForeColor = Color.Gray;
 		
 		internal MonthCalendar			month_calendar;
@@ -75,6 +76,43 @@ namespace System.Windows.Forms {
 		
 		#endregion	// Local variables
 		
+		#region DateTimePickerAccessibleObject Subclass
+		[ComVisible(true)]
+		public class DateTimePickerAccessibleObject : ControlAccessibleObject {
+			#region DateTimePickerAccessibleObject Local Variables
+			private DateTimePicker	owner;
+			#endregion	// DateTimePickerAccessibleObject Local Variables
+
+			#region DateTimePickerAccessibleObject Constructors
+			public DateTimePickerAccessibleObject(DateTimePicker owner) : base(owner) {
+				this.owner = owner;
+			}
+			#endregion	// DateTimePickerAccessibleObject Constructors
+
+			#region DateTimePickerAccessibleObject Properties
+			public override AccessibleStates State {
+				get {
+					AccessibleStates	retval;
+
+					retval = AccessibleStates.Default;
+
+					if (owner.Checked) {
+						retval |= AccessibleStates.Checked;
+					}
+
+					return retval;
+				}
+			}
+
+			public override string Value {
+				get {
+					return owner.text;
+				}
+			}
+			#endregion	// DateTimePickerAccessibleObject Properties
+		}
+		#endregion	// DateTimePickerAccessibleObject Sub-class
+
 		#region public constructors
 		
 		// only public constructor
@@ -113,7 +151,8 @@ namespace System.Windows.Forms {
 			MouseDown += new MouseEventHandler (MouseDownHandler);			
 			Paint += new PaintEventHandler (PaintHandler);
 			
-			SetStyle (ControlStyles.UserPaint | ControlStyles.AllPaintingInWmPaint, true);
+			SetStyle (ControlStyles.UserPaint | ControlStyles.StandardClick, false);
+			SetStyle (ControlStyles.FixedHeight, true);
 		}
 		
 		#endregion
@@ -123,7 +162,7 @@ namespace System.Windows.Forms {
 		// no reason why this is overridden
 		[Browsable(false)]
 		[EditorBrowsable(EditorBrowsableState.Never)]
-		public Color BackColor {
+		public override Color BackColor {
 			set {
 				base.BackColor = value;
 			}
@@ -364,7 +403,7 @@ namespace System.Windows.Forms {
 		[Browsable(false)]
 		[EditorBrowsable(EditorBrowsableState.Advanced)]
 		[DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
-		public string Text {
+		public override string Text {
 			set {
 				// TODO: if the format is a custom format we need to do a custom parse here
 				DateTime parsed_value = DateTime.Parse (value);
@@ -418,7 +457,66 @@ namespace System.Windows.Forms {
 		
 		// raised when the date Value is changed
 		public event EventHandler ValueChanged;
-		
+
+		[Browsable(false)]
+		[EditorBrowsable(EditorBrowsableState.Never)]
+		public new event EventHandler BackColorChanged {
+			add {
+				base.BackColorChanged += value;
+			}
+
+			remove {
+				base.BackColorChanged -= value;
+			}
+		}
+
+		[Browsable(false)]
+		[EditorBrowsable(EditorBrowsableState.Never)]
+		public new event EventHandler BackgroundImageChanged {
+			add {
+				base.BackgroundImageChanged += value;
+			}
+
+			remove {
+				base.BackgroundImageChanged -= value;
+			}
+		}
+
+		[Browsable(false)]
+		[EditorBrowsable(EditorBrowsableState.Never)]
+		public new event EventHandler ForeColorChanged {
+			add {
+				base.ForeColorChanged += value;
+			}
+
+			remove {
+				base.ForeColorChanged -= value;
+			}
+		}
+
+		[Browsable(false)]
+		[EditorBrowsable(EditorBrowsableState.Never)]
+		public new event PaintEventHandler Paint {
+			add {
+				base.Paint += value;
+			}
+
+			remove {
+				base.Paint -= value;
+			}
+		}
+
+		[Browsable(false)]
+		[EditorBrowsable(EditorBrowsableState.Advanced)]
+		public new event EventHandler TextChanged {
+			add {
+				base.TextChanged += value;
+			}
+
+			remove {
+				base.TextChanged -= value;
+			}
+		}
 		#endregion	// public events
 		
 		#region protected properties
@@ -481,6 +579,11 @@ namespace System.Windows.Forms {
 				this.DropDown (this, eventargs);
 			}
 		}
+
+		protected override void OnFontChanged(EventArgs e) {
+			// FIXME - do we need to update/invalidate/recalc our stuff?
+			base.OnFontChanged (e);
+		}
 		
 		// raises the format changed event
 		protected virtual void OnFormatChanged (EventArgs e) {
