@@ -17,7 +17,7 @@
 // OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 // WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 //
-// Copyright (c) 2004 Novell, Inc.
+// Copyright (c) 2004-2005 Novell, Inc.
 //
 // Authors:
 //	Peter Bartok	pbartok@novell.com
@@ -44,9 +44,10 @@ namespace System.Windows.Forms {
 		#region Local Variables
 		internal static Form		active_form;
 		internal bool			closing;
-		FormBorderStyle			formBorderStyle;
-		private static bool		autoscale;
-		private static Size		autoscale_base_size;
+		FormBorderStyle			form_border_style;
+		private bool		        autoscale;
+		private Size		        autoscale_base_size;
+		private static Icon		default_icon;
 		internal bool			is_modal;
 		internal bool			end_modal;			// This var is being monitored by the application modal loop
 		private bool			control_box;
@@ -62,7 +63,7 @@ namespace System.Windows.Forms {
 		private Form			owner;
 		private Form.ControlCollection	owned_forms;
 		private MdiClient		mdi_container;
-		private MdiChildContext		mdi_child_context;
+		private InternalWindowManager	window_manager;
 		private Form			mdi_parent;
 		private bool			key_preview;
 		private MainMenu		menu;
@@ -72,11 +73,17 @@ namespace System.Windows.Forms {
 		private SizeGripStyle		size_grip_style;
 		private Rectangle		maximized_bounds;
 		private Rectangle		default_maximized_bounds;
+		private double			opacity;
+		internal ApplicationContext	context;
 		Color				transparency_key;
 
 		#endregion	// Local Variables
 
 		#region Private & Internal Methods
+		static Form ()
+		{
+			default_icon = (Icon)Locale.GetResource("mono.ico");
+		}
 		#endregion	// Private & Internal Methods
 
 		#region Public Classes
@@ -88,13 +95,9 @@ namespace System.Windows.Forms {
 			}
 
 			public override void Add(Control value) {
-				for (int i=0; i<list.Count; i++) {
-					if (list[i]==value) {
-						// Do we need to do anything here?
-						return;
-					}
-				}
-				list.Add(value);
+				if (Contains (value))
+					return;
+				AddToList (value);
 				((Form)value).owner=(Form)owner;
 			}
 
@@ -106,16 +109,22 @@ namespace System.Windows.Forms {
 		#endregion	// Public Classes
 
 		#region Public Constructor & Destructor
-		public Form() {
+		public Form ()
+		{
+			SizeF current_scale = GetAutoScaleSize (DeviceContext, Font);
+
+			autoscale = true;
+			autoscale_base_size = new Size ((int)current_scale.Width, (int) current_scale.Height);
 			closing = false;
 			is_modal = false;
 			end_modal = false;
 			dialog_result = DialogResult.None;
 			start_position = FormStartPosition.WindowsDefaultLocation;
-			formBorderStyle = FormBorderStyle.Sizable;
+			form_border_style = FormBorderStyle.Sizable;
 			key_preview = false;
+			opacity = 1D;
 			menu = null;
-			icon = null;
+			icon = default_icon;
 			minimum_size = new Size(0, 0);
 			maximum_size = new Size(0, 0);
 			control_box = true;
@@ -186,7 +195,7 @@ namespace System.Windows.Forms {
 			set {
 			}
 		}
-			
+
 		[DefaultValue(true)]
 		public bool AutoScale {
 			get {
@@ -194,7 +203,7 @@ namespace System.Windows.Forms {
 			}
 
 			set {
-				autoscale=value;
+				autoscale = value;
 			}
 		}
 
@@ -207,7 +216,7 @@ namespace System.Windows.Forms {
 			}
 
 			set {
-				autoscale_base_size=value;
+				autoscale_base_size = value;
 			}
 		}
 
@@ -311,10 +320,23 @@ namespace System.Windows.Forms {
 		[DispId(-504)]
 		public FormBorderStyle FormBorderStyle {
 			get {
-				return formBorderStyle;
+				return form_border_style;
 			}
 			set {
-				formBorderStyle = value;
+				form_border_style = value;
+
+				if (window_manager == null) {
+					if (IsHandleCreated) {
+						XplatUI.SetBorderStyle(window.Handle, form_border_style);
+					}
+
+					if (value == FormBorderStyle.FixedToolWindow ||
+							value == FormBorderStyle.SizableToolWindow)
+						window_manager = new InternalWindowManager (this);
+				} else {
+					window_manager.UpdateBorderStyle (value);
+				}
+
 				UpdateStyles();
 			}
 		}
@@ -344,7 +366,9 @@ namespace System.Windows.Forms {
 				if (icon != value) {
 					icon = value;
 
-					XplatUI.SetIcon(Handle, icon);
+					if (IsHandleCreated) {
+						XplatUI.SetIcon(Handle, icon == null ? default_icon : icon);
+					}
 				}
 			}
 		}
@@ -374,6 +398,16 @@ namespace System.Windows.Forms {
 			}
 		}
 
+		[Browsable(false)]
+		[DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
+		public Form ActiveMdiChild {
+			get {
+				if (!IsMdiContainer)
+					return null;
+				return (Form) mdi_container.ActiveMdiChild;
+			}
+		}
+
 		[Browsable(false)]
 		[EditorBrowsable(EditorBrowsableState.Advanced)]
 		public bool IsRestrictedWindow {
@@ -462,7 +496,8 @@ namespace System.Windows.Forms {
 
 				mdi_parent = value;
 				if (mdi_parent != null) {
-					mdi_child_context = new MdiChildContext (this);
+					window_manager = new MdiWindowManager (this,
+							mdi_parent.MdiContainer);
 					mdi_parent.MdiContainer.Controls.Add (this);
 				}
 
@@ -474,31 +509,57 @@ namespace System.Windows.Forms {
 			get { return mdi_container; }
 		}
 
+		internal InternalWindowManager WindowManager {
+			get { return window_manager; }
+		}
+
 		[DefaultValue(null)]
 		public MainMenu Menu {
 			get {
 				return menu;
 			}
 
-			set {				
-				if (menu != value) {					
+			set {
+				if (menu != value) {
 					menu = value;
 
-					menu.SetForm (this);
-					MenuAPI.SetMenuBarWindow (menu.Handle, this);
-				
-					if (IsHandleCreated && menu != null) {	
-						XplatUI.SetMenu(window.Handle, menu.Handle);
-					}
+					if (menu != null && !IsMdiChild) {
+						menu.SetForm (this);
 
-					// FIXME - Do we still need this?
-					this.SetBoundsCore(0, 0, 0, 0, BoundsSpecified.None);
+						if (IsHandleCreated) {
+							XplatUI.SetMenu (window.Handle, menu);
+						}
 
-					ThemeEngine.Current.CalcMenuBarSize (DeviceContext, menu.Handle, ClientSize.Width);
+						UpdateBounds (bounds.X, bounds.Y, bounds.Width, bounds.Height, ClientSize.Width, ClientSize.Height - 
+							ThemeEngine.Current.CalcMenuBarSize (DeviceContext, menu, ClientSize.Width));
+					} else
+						UpdateBounds ();
 				}
 			}
 		}
 
+		public MainMenu MergedMenu {
+			get {
+				if (!IsMdiChild || window_manager == null)
+					return null;
+				return ((MdiWindowManager) window_manager).MergedMenu;
+			}
+		}
+
+		// This is the menu in display and being used because of merging this can
+		// be different then the menu that is actually assosciated with the form
+		internal MainMenu ActiveMenu {
+			get {
+				if (IsMdiChild)
+					return null;
+
+				Form amc = ActiveMdiChild;
+				if (amc == null || amc.Menu == null)
+					return menu;
+				return amc.MergedMenu;
+			}
+		}
+
 		[DefaultValue(true)]
 		public bool MinimizeBox {
 			get {
@@ -536,6 +597,20 @@ namespace System.Windows.Forms {
 			}
 		}
 
+		[MonoTODO("Investigate ways to implement opacity")]
+		[DefaultValue(1D)]
+		[TypeConverter(typeof(OpacityConverter))]
+		public double Opacity {
+			get {
+				return opacity;
+			}
+
+			set {
+				opacity = value;
+			}
+		}
+			
+
 		[Browsable(false)]
 		[DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
 		public Form[] OwnedForms {
@@ -700,30 +775,29 @@ namespace System.Windows.Forms {
 		[DefaultValue(FormWindowState.Normal)]
 		public FormWindowState WindowState {
 			get {
-				return XplatUI.GetWindowState(window.Handle);
+				return XplatUI.GetWindowState(Handle);
 			}
 
 			set {
-				XplatUI.SetWindowState(window.Handle, value);
+				XplatUI.SetWindowState(Handle, value);
 			}
 		}
 
 		#endregion	// Public Instance Properties
 
 		#region Protected Instance Properties
-		[MonoTODO("Need to add MDI support")]
+		[MonoTODO("Need to set start position properly")]
 		protected override CreateParams CreateParams {
 			get {
-				CreateParams cp;
+				CreateParams cp = new CreateParams ();
 
-				cp = new CreateParams();
-
-				cp.Caption = "FormWindow";
+				cp.Caption = Text;
 				cp.ClassName = XplatUI.DefaultClassName;
 				cp.ClassStyle = 0;
 				cp.ExStyle = 0;
 				cp.Param = 0;
 				cp.Parent = IntPtr.Zero;
+
 //				if (start_position == FormStartPosition.WindowsDefaultLocation) {
 					cp.X = unchecked((int)0x80000000);
 					cp.Y = unchecked((int)0x80000000);
@@ -733,10 +807,30 @@ namespace System.Windows.Forms {
 //				}
 				cp.Width = Width;
 				cp.Height = Height;
-				
-				cp.Style = (int)(WindowStyles.WS_CLIPSIBLINGS | WindowStyles.WS_CLIPCHILDREN);
 
-				switch (FormBorderStyle) {
+				cp.Style |= (int)WindowStyles.WS_CLIPCHILDREN | (int)WindowStyles.WS_CLIPSIBLINGS;
+
+				if (IsMdiChild) {
+					cp.Style |= (int)WindowStyles.WS_CHILD;
+					cp.Parent = Parent.Handle;
+
+					cp.ExStyle |= (int) (WindowStyles.WS_EX_WINDOWEDGE | WindowStyles.WS_EX_MDICHILD);
+					switch (FormBorderStyle) {
+					case FormBorderStyle.None:
+						break;
+					case FormBorderStyle.FixedToolWindow:
+					case FormBorderStyle.SizableToolWindow:
+						cp.ExStyle |= (int) WindowStyles.WS_EX_TOOLWINDOW;
+						goto default;
+					default:
+						cp.Style |= (int) WindowStyles.WS_OVERLAPPEDWINDOW;
+						break;
+					}
+					
+				} else {
+
+					switch (FormBorderStyle) {
+
 					case FormBorderStyle.Fixed3D: {
 						cp.Style |= (int)WindowStyles.WS_CAPTION;
 						cp.ExStyle |= (int)WindowStyles.WS_EX_OVERLAPPEDWINDOW;
@@ -772,6 +866,7 @@ namespace System.Windows.Forms {
 						cp.ExStyle |= (int)(WindowStyles.WS_EX_WINDOWEDGE | WindowStyles.WS_EX_TOOLWINDOW);
 						break;
 					}
+					}
 				}
 
 				if (ShowInTaskbar) {
@@ -793,6 +888,7 @@ namespace System.Windows.Forms {
 				if (HelpButton) {
 					cp.ExStyle |= (int)WindowStyles.WS_EX_CONTEXTHELP;
 				}
+
 				return cp;
 			}
 		}
@@ -807,7 +903,7 @@ namespace System.Windows.Forms {
 			get {
 				return new Size (250, 250);
 			}
-		}		
+		}
 
 		protected Rectangle MaximizedBounds {
 			get {
@@ -825,19 +921,33 @@ namespace System.Windows.Forms {
 		#endregion	// Protected Instance Properties
 
 		#region Public Static Methods
-		[MonoTODO("Figure out the math")]
 		[EditorBrowsable(EditorBrowsableState.Advanced)]
-		public static SizeF GetAutoScaleSize(Font font) {
-			SizeF	result;
-
-			result = new SizeF(250, 250);
-			result.Width *= font.SizeInPoints / 12;
-			result.Height *= font.SizeInPoints / 12;
-			return result;
+		public static SizeF GetAutoScaleSize (Font font)
+		{
+			return XplatUI.GetAutoScaleSize(font);
 		}
+
 		#endregion	// Public Static Methods
 
 		#region Public Instance Methods
+		internal SizeF GetAutoScaleSize (Graphics g, Font font)
+		{
+			//
+			// The following constants come from the dotnet mailing list
+			// discussion: http://discuss.develop.com/archives/wa.exe?A2=ind0203A&L=DOTNET&P=R3655
+			//
+			// The magic number is "Its almost the length
+			// of the string with a smattering added in
+			// for compat with earlier code".
+			//
+	
+			string magic_string = "The quick brown fox jumped over the lazy dog.";
+			double magic_number = 44.549996948242189;
+			float width = (float) (g.MeasureString (magic_string, font).Width / magic_number);
+			
+			return new SizeF (width, font.Height);
+		}
+						 
 		public void Activate() {
 			Form	active;
 
@@ -857,7 +967,10 @@ namespace System.Windows.Forms {
 			OnClosing (args);
 			if (!args.Cancel) {
 				OnClosed (EventArgs.Empty);
-				closing = true;
+				DestroyHandle();
+				if (context != null) {
+					XplatUI.PostQuitMessage(0);
+				}
 				return;
 			}
 		}
@@ -887,13 +1000,9 @@ namespace System.Windows.Forms {
 		public DialogResult ShowDialog(IWin32Window ownerWin32) {
 			Form		previous;
 
-			#if broken
-			Control		owner = null;
-
 			if (ownerWin32 != null) {
-				owner = Control.FromHandle(ownerWin32.Handle);
+				this.owner = (Form)Control.FromHandle(ownerWin32.Handle);
 			}
-			#endif
 
 			if (is_modal) {
 				return DialogResult.None;
@@ -905,7 +1014,7 @@ namespace System.Windows.Forms {
 
 			#if broken
 			// Can't do this, will screw us in the modal loop
-			form_parent_window.Parent = owner;
+			form_parent_window.Parent = this.owner;
 			#endif
 
 			previous = Form.ActiveForm;
@@ -952,8 +1061,28 @@ namespace System.Windows.Forms {
 		}
 
 		[EditorBrowsable(EditorBrowsableState.Advanced)]
-		protected void ApplyAutoScaling() {
-			// Hm, not sure what this does
+		protected void ApplyAutoScaling()
+		{
+			SizeF current_size_f = GetAutoScaleSize (DeviceContext, Font);
+			Size current_size = new Size ((int) current_size_f.Width, (int) current_size_f.Height);
+
+			if (current_size == autoscale_base_size)
+				return;
+
+			if (Environment.GetEnvironmentVariable ("MONO_MWF_SCALING") == "disable"){
+				Console.WriteLine ("Not scaling");
+				return;
+			}
+			
+			//
+			// I tried applying the Fudge height factor from:
+			// http://blogs.msdn.com/mharsh/archive/2004/01/25/62621.aspx
+			// but it makes things larger without looking better.
+			//
+			Scale (current_size_f.Width / AutoScaleBaseSize.Width,
+			       current_size_f.Height / AutoScaleBaseSize.Height);
+			
+			AutoScaleBaseSize = current_size;
 		}
 
 		protected void CenterToParent() {
@@ -1014,6 +1143,9 @@ namespace System.Windows.Forms {
 		[EditorBrowsable(EditorBrowsableState.Advanced)]
 		protected override void CreateHandle() {
 			base.CreateHandle ();
+			if (icon != null) {
+				XplatUI.SetIcon(window.Handle, icon);
+			}
 		}
 
 		[EditorBrowsable(EditorBrowsableState.Advanced)]
@@ -1063,11 +1195,26 @@ namespace System.Windows.Forms {
 
 				this.is_visible = visible;
 			}
-			
-			if (menu != null) {
-				XplatUI.SetMenu(window.Handle, menu.Handle);
+
+			if (!IsMdiChild) {
+				switch (StartPosition) {
+				case FormStartPosition.CenterScreen:
+					this.CenterToScreen();
+					break;
+				case FormStartPosition.CenterParent:
+					this.CenterToParent ();
+					break;
+				}
+			} else {
+				Left = 25 * MdiParent.MdiContainer.ChildrenCreated + 1;
+				Top = 25 * MdiParent.MdiContainer.ChildrenCreated + 1;
+				MdiParent.MdiContainer.ChildrenCreated++;
 			}
-			
+
+			if (menu != null) {
+				XplatUI.SetMenu(window.Handle, menu);
+			}
+
 			OnLoad(EventArgs.Empty);
 
 			// Send initial location
@@ -1088,6 +1235,7 @@ namespace System.Windows.Forms {
 
 		[EditorBrowsable(EditorBrowsableState.Advanced)]
 		protected override void OnHandleCreated(EventArgs e) {
+			XplatUI.SetBorderStyle(window.Handle, form_border_style);
 			base.OnHandleCreated (e);
 		}
 
@@ -1115,6 +1263,10 @@ namespace System.Windows.Forms {
 			if (Load != null) {
 				Load(this, e);
 			}
+			if (AutoScale){
+				ApplyAutoScaling ();
+				AutoScale = false;
+			}
 		}
 
 		[EditorBrowsable(EditorBrowsableState.Advanced)]
@@ -1162,8 +1314,8 @@ namespace System.Windows.Forms {
 		[EditorBrowsable(EditorBrowsableState.Advanced)]
 		protected override void OnPaint (PaintEventArgs pevent) {
 			base.OnPaint (pevent);
-		}		
-		
+		}
+
 		[EditorBrowsable(EditorBrowsableState.Advanced)]
 		protected override void OnResize(EventArgs e) {
 			base.OnResize(e);
@@ -1190,8 +1342,8 @@ namespace System.Windows.Forms {
 			}
 
 			// Give our menu a shot
-			if (menu != null) {
-				return menu.ProcessCmdKey(ref msg, keyData);
+			if (ActiveMenu != null) {
+				return ActiveMenu.ProcessCmdKey(ref msg, keyData);
 			}
 
 			return false;
@@ -1268,7 +1420,12 @@ namespace System.Windows.Forms {
 				y = maximum_size.Height;
 			}
 
-			base.SetClientSizeCore (x, y);
+			Rectangle ClientRect = new Rectangle(0, 0, x, y);
+			Rectangle WindowRect;
+			CreateParams cp = this.CreateParams;
+
+			if (XplatUI.CalculateWindowRect(Handle, ref ClientRect, cp.Style, cp.ExStyle, ActiveMenu, out WindowRect) )
+				SetBoundsCore(bounds.X, bounds.Y, WindowRect.Width, WindowRect.Height, BoundsSpecified.Size);
 		}
 
 		[EditorBrowsable(EditorBrowsableState.Advanced)]
@@ -1283,23 +1440,15 @@ namespace System.Windows.Forms {
 		[EditorBrowsable(EditorBrowsableState.Advanced)]
 		protected override void WndProc(ref Message m) {
 
-			if (IsMdiChild && mdi_child_context.HandleMessage (ref m)) {
+			if (window_manager != null && window_manager.HandleMessage (ref m)) {
 				return;
 			}
 
 			switch((Msg)m.Msg) {
 				case Msg.WM_CLOSE: {
-					CancelEventArgs args = new CancelEventArgs();
-
-					OnClosing(args);
-
-					if (!args.Cancel) {
-						OnClosed(EventArgs.Empty);
-						closing = true;
-						base.WndProc(ref m);
-						break;
-					}
-					break;
+					Close();
+					base.WndProc(ref m);
+					return;
 				}
 
 				case Msg.WM_ACTIVATE: {
@@ -1317,54 +1466,50 @@ namespace System.Windows.Forms {
 				}
 
 				case Msg.WM_SETFOCUS: {
-#if not
-					if (this.ActiveControl != null) {
+					if (ActiveControl != null && ActiveControl != this) {
 						ActiveControl.Focus();
+						return;	// FIXME - do we need to run base.WndProc, even though we just changed focus?
 					}
-#endif
 					base.WndProc(ref m);
 					return;
 				}
 
 				// Menu drawing
 				case Msg.WM_NCLBUTTONDOWN: {
-					if (this.menu != null) {
-						int x = LowOrder ((int) m.LParam.ToInt32 ()) ;
-						int y = HighOrder ((int) m.LParam.ToInt32 ());						
-						menu.OnMouseDown(this, new MouseEventArgs (FromParamToMouseButtons ((int) m.WParam.ToInt32()), mouse_clicks, x, y, 0));
+					if (ActiveMenu != null) {
+						ActiveMenu.OnMouseDown(this, new MouseEventArgs (FromParamToMouseButtons ((int) m.WParam.ToInt32()), mouse_clicks, LowOrder ((int) m.LParam.ToInt32 ()), HighOrder ((int) m.LParam.ToInt32 ()), 0));
 					}
 					base.WndProc(ref m);
 					return;
 				}
 
 				case Msg.WM_NCMOUSEMOVE: {
-					if (this.menu != null) {
-						menu.OnMouseMove(this, new MouseEventArgs (FromParamToMouseButtons ((int) m.WParam.ToInt32()), 
-							mouse_clicks, LowOrder ((int) m.LParam.ToInt32 ()), HighOrder ((int) m.LParam.ToInt32 ()), 0));
+					if (ActiveMenu != null) {
+						ActiveMenu.OnMouseMove(this, new MouseEventArgs (FromParamToMouseButtons ((int) m.WParam.ToInt32()), mouse_clicks, LowOrder ((int) m.LParam.ToInt32 ()), HighOrder ((int) m.LParam.ToInt32 ()), 0));
 					}
 					base.WndProc(ref m);
 					return;
 				}
 
 				case Msg.WM_NCPAINT: {
-					if (this.menu != null) {
+					if (ActiveMenu != null) {
 						Point pnt = XplatUI.GetMenuOrigin(window.Handle);
-						MenuAPI.DrawMenuBar (menu.Handle, new Rectangle (pnt.X, pnt.Y, ClientSize.Width, 0));
+
+						ActiveMenu.Draw (new Rectangle (pnt.X, pnt.Y, ClientSize.Width, 0));
 					}
 
 					base.WndProc(ref m);
 					return;
 				}
 
-				// This message is only received under Win32
 				case Msg.WM_NCCALCSIZE: {
 					XplatUIWin32.NCCALCSIZE_PARAMS	ncp;
 
-					if ((menu != null) && (m.WParam == (IntPtr)1)) {
+					if ((ActiveMenu != null) && (m.WParam == (IntPtr)1)) {
 						ncp = (XplatUIWin32.NCCALCSIZE_PARAMS)Marshal.PtrToStructure(m.LParam, typeof(XplatUIWin32.NCCALCSIZE_PARAMS));
 
 						// Adjust for menu
-						ncp.rgrc1.top += ThemeEngine.Current.CalcMenuBarSize (DeviceContext, menu.menu_handle, ClientSize.Width);
+						ncp.rgrc1.top += ThemeEngine.Current.CalcMenuBarSize (DeviceContext, menu, ClientSize.Width);
 						Marshal.StructureToPtr(ncp, m.LParam, true);
 					}
 					DefWndProc(ref m);
@@ -1395,6 +1540,11 @@ namespace System.Windows.Forms {
 		}
 		#endregion	// Protected Instance Methods
 
+		internal void RemoveWindowManager ()
+		{
+			window_manager = null;
+		}
+
 		#region Events
 		public event EventHandler Activated;
 		public event EventHandler Closed;
