@@ -39,32 +39,33 @@ namespace System.Windows.Forms {
 	internal class Hwnd : IDisposable {
 		#region Local Variables
 		private static Hashtable	windows	= new Hashtable(100, 0.5f);
-		private const int	menu_height = 14;			// FIXME - Read this value from somewhere
-		private const int	caption_height = 0;			// FIXME - Read this value from somewhere
-		private const int	tool_caption_height = 0;		// FIXME - Read this value from somewhere
-
+		//private const int	menu_height = 14;			// FIXME - Read this value from somewhere
+		
 		private IntPtr		handle;
 		internal IntPtr		client_window;
 		internal IntPtr		whole_window;
-		internal IntPtr		menu_handle;
+		internal Menu		menu;
 		internal TitleStyle	title_style;
-		internal BorderStyle	border_style;
-		internal Border3DStyle	edge_style;
+		internal FormBorderStyle	border_style;
 		internal int		x;
 		internal int		y;
 		internal int		width;
 		internal int		height;
+		internal bool		allow_drop;
 		internal Hwnd		parent;
 		internal bool		visible;
+		internal bool		zero_sized;
 		internal Rectangle	invalid;
 		internal bool		expose_pending;
-		internal bool		erase_pending;
 		internal bool		nc_expose_pending;
 		internal bool		configure_pending;
+		internal bool		reparented;
 		internal Graphics	client_dc;
 		internal object		user_data;
 		internal Rectangle	client_rectangle;
 		internal ArrayList	marshal_free_list;
+		internal int		caption_height;
+		internal int		tool_caption_height;
 		#endregion	// Local Variables
 
 		#region Constructors and destructors
@@ -74,8 +75,8 @@ namespace System.Windows.Forms {
 			width = 0;
 			height = 0;
 			visible = false;
-			menu_handle = IntPtr.Zero;
-			border_style = BorderStyle.None;
+			menu = null;
+			border_style = FormBorderStyle.None;
 			client_window = IntPtr.Zero;
 			whole_window = IntPtr.Zero;
 			handle = IntPtr.Zero;
@@ -83,9 +84,8 @@ namespace System.Windows.Forms {
 			invalid = Rectangle.Empty;
 			expose_pending = false;
 			nc_expose_pending = false;
-			edge_style = Border3DStyle.Raised;
+			reparented = false;
 			client_rectangle = Rectangle.Empty;
-			erase_pending = true;
 			marshal_free_list = new ArrayList(2);
 		}
 
@@ -96,7 +96,6 @@ namespace System.Windows.Forms {
 				Marshal.FreeHGlobal((IntPtr)marshal_free_list[i]);
 			}
 			marshal_free_list.Clear();
-Console.WriteLine("Disposing window {0:X} (whole: {1:X})", client_window.ToInt32(), whole_window.ToInt32());
 		}
 		#endregion
 
@@ -133,26 +132,37 @@ Console.WriteLine("Disposing window {0:X} (whole: {1:X})", client_window.ToInt32
 			}
 		}
 
-		public static Rectangle GetWindowRectangle(BorderStyle border_style, IntPtr menu_handle, TitleStyle title_style, Rectangle client_rect) {
+		public static Rectangle GetWindowRectangle(FormBorderStyle border_style,
+				Menu menu, TitleStyle title_style, int caption_height,
+				int tool_caption_height, Rectangle client_rect)
+		{
 			Rectangle	rect;
 
 			rect = new Rectangle(client_rect.Location, client_rect.Size);
 
-			if (menu_handle != IntPtr.Zero) {
+			if (menu != null) {
+				int menu_height = menu.Rect.Height;
 				rect.Y -= menu_height;
 				rect.Height += menu_height;
 			}
 
-			if (border_style == BorderStyle.Fixed3D) {
-				rect.X -= 2;
-				rect.Y -= 2;
-				rect.Width += 4;
-				rect.Height += 4;
-			} else if (border_style == BorderStyle.FixedSingle) {
+			if (border_style == FormBorderStyle.Fixed3D) {
+				Size border_3D_size = ThemeEngine.Current.Border3DSize;
+
+				rect.X -= border_3D_size.Width;
+				rect.Y -= border_3D_size.Height;
+				rect.Width += border_3D_size.Width * 2;
+				rect.Height += border_3D_size.Height * 2;
+			} else if (border_style == FormBorderStyle.FixedSingle) {
 				rect.X -= 1;
 				rect.Y -= 1;
 				rect.Width += 2;
 				rect.Height += 2;
+			} else if ((int) border_style == 0xFFFF) {
+				rect.X -= 3;
+				rect.Y -= 3;
+				rect.Width += 6;
+				rect.Height += 6;
 			}
 
 			if (title_style == TitleStyle.Normal) {
@@ -166,32 +176,40 @@ Console.WriteLine("Disposing window {0:X} (whole: {1:X})", client_window.ToInt32
 			return rect;
 		}
 
-		public static Rectangle GetClientRectangle(BorderStyle border_style, IntPtr menu_handle, TitleStyle title_style, int width, int height) {
+		public static Rectangle GetClientRectangle(FormBorderStyle border_style, Menu menu, TitleStyle title_style, int caption_height, int tool_caption_height, int width, int height) {
 			Rectangle rect;
 
 			rect = new Rectangle(0, 0, width, height);
 
-			if (menu_handle != IntPtr.Zero) {
+			if (menu != null) {
+				int menu_height = menu.Rect.Height;
 				rect.Y += menu_height;
 				rect.Height -= menu_height;
 			}
+			
+			if (border_style == FormBorderStyle.Fixed3D) {
+				Size border_3D_size = ThemeEngine.Current.Border3DSize;
 
-			if (border_style == BorderStyle.Fixed3D) {
-				rect.X += 2;
-				rect.Y += 2;
-				rect.Width -= 4;
-				rect.Height -= 4;
-			} else if (border_style == BorderStyle.FixedSingle) {
+				rect.X += border_3D_size.Width;
+				rect.Y += border_3D_size.Height;
+				rect.Width -= border_3D_size.Width * 2;
+				rect.Height -= border_3D_size.Height * 2;
+			} else if (border_style == FormBorderStyle.FixedSingle) {
 				rect.X += 1;
 				rect.Y += 1;
 				rect.Width -= 2;
 				rect.Height -= 2;
+			} else if ((int) border_style == 0xFFFF) {
+				rect.X += 3;
+				rect.Y += 3;
+				rect.Width -= 6;
+				rect.Height -= 6;
 			}
 
 			if (title_style == TitleStyle.Normal)  {
 				rect.Y += caption_height;
 				rect.Height -= caption_height;
-			} else if (title_style == TitleStyle.Normal)  {
+			} else if (title_style == TitleStyle.Tool)  {
 				rect.Y += tool_caption_height;
 				rect.Height -= tool_caption_height;
 			}
@@ -201,7 +219,7 @@ Console.WriteLine("Disposing window {0:X} (whole: {1:X})", client_window.ToInt32
 		#endregion	// Static Methods
 
 		#region Instance Properties
-		public BorderStyle BorderStyle {
+		public FormBorderStyle BorderStyle {
 			get {
 				return border_style;
 			}
@@ -251,51 +269,10 @@ Console.WriteLine("Disposing window {0:X} (whole: {1:X})", client_window.ToInt32
 
 		public Rectangle DefaultClientRect {
 			get {
-				Rectangle rect;
-
-				rect = new Rectangle(0, 0, width, height);
-
-				if (border_style == BorderStyle.Fixed3D) {
-					rect.X += 2;
-					rect.Y += 2;
-					rect.Width -= 4;
-					rect.Height -= 4;
-				} else if (border_style == BorderStyle.FixedSingle) {
-					rect.X += 1;
-					rect.Y += 1;
-					rect.Width -= 2;
-					rect.Height -= 2;
-				}
-
-				if (this.title_style == TitleStyle.Normal)  {
-					rect.Y += caption_height;
-					rect.Height -= caption_height;
-				} else if (this.title_style == TitleStyle.Normal)  {
-					rect.Y += tool_caption_height;
-					rect.Height -= tool_caption_height;
-				}
-
-				return rect;
-			}
-		}
-
-		public Border3DStyle EdgeStyle {
-			get {
-				return edge_style;
-			}
-
-			set {
-				edge_style = value;
-			}
-		}
-
-		public bool ErasePending {
-			get {
-				return erase_pending;
-			}
-
-			set {
-				erase_pending = value;
+				// We pass a Zero for the menu handle so the menu size is
+				// not computed this is done via an WM_NCCALC
+				return GetClientRectangle (border_style, null, title_style,
+						caption_height, tool_caption_height, width, height);
 			}
 		}
 
@@ -328,26 +305,37 @@ Console.WriteLine("Disposing window {0:X} (whole: {1:X})", client_window.ToInt32
 			}
 		}
 
-		public IntPtr MenuHandle {
+		public Menu Menu {
 			get {
-				return menu_handle;
+				return menu;
 			}
 
 			set {
-				menu_handle = value;
+				menu = value;
+			}
+		}
+
+		public bool Reparented {
+			get {
+				return reparented;
+			}
+
+			set {
+				reparented = value;
 			}
 		}
 
 		public Point MenuOrigin {
 			get {
 				Point	pt;
+				Size	border_3D_size = ThemeEngine.Current.Border3DSize;
 
 				pt = new Point(0, 0);
 
-				if (border_style == BorderStyle.Fixed3D) {
-					pt.X += 2;
-					pt.Y += 2;
-				} else if (border_style == BorderStyle.FixedSingle) {
+				if (border_style == FormBorderStyle.Fixed3D) {
+					pt.X += border_3D_size.Width;
+					pt.Y += border_3D_size.Height;
+				} else if (border_style == FormBorderStyle.FixedSingle) {
 					pt.X += 1;
 					pt.Y += 1;
 				}
@@ -391,6 +379,16 @@ Console.WriteLine("Disposing window {0:X} (whole: {1:X})", client_window.ToInt32
 			}
 		}
 
+		public int CaptionHeight {
+			get { return caption_height; }
+			set { caption_height = value; }
+		}
+
+		public int ToolCaptionHeight {
+			get { return tool_caption_height; }
+			set { tool_caption_height = value; }
+		}
+
 		public TitleStyle TitleStyle {
 			get {
 				return title_style;
@@ -472,7 +470,15 @@ Console.WriteLine("Disposing window {0:X} (whole: {1:X})", client_window.ToInt32
 				invalid = new Rectangle (x, y, width, height);
 				return;
 			}
-			invalid = Rectangle.Union (invalid, new Rectangle (x, y, width, height));
+
+			int right, bottom;
+			right = Math.Max (invalid.Right, x + width);
+			bottom = Math.Max (invalid.Bottom, y + height);
+			invalid.X = Math.Min (invalid.X, x);
+			invalid.Y = Math.Min (invalid.Y, y);
+
+			invalid.Width = right - invalid.X;
+			invalid.Height = bottom - invalid.Y;
 		}
 
 		public void AddInvalidArea(Rectangle rect) {
@@ -487,6 +493,11 @@ Console.WriteLine("Disposing window {0:X} (whole: {1:X})", client_window.ToInt32
 			invalid = Rectangle.Empty;
 			expose_pending = false;
 		}
+
+		public override string ToString() {
+			return String.Format("Hwnd, ClientWindow:0x{0:X}, WholeWindow:0x{1:X}, Handle:0x{2:X}", client_window.ToInt32(), whole_window.ToInt32(), handle.ToInt32());
+		}
+
 		#endregion	// Methods
 	}
 }
