@@ -88,7 +88,6 @@ namespace System.Windows.Forms
     		{
 			// Defaults in the Spec
 			autosize = false;
-			border_style = BorderStyle.None;
 			string_format = new StringFormat();
 			TextAlign = ContentAlignment.TopLeft;
 			image = null;
@@ -96,9 +95,7 @@ namespace System.Windows.Forms
 			image_list = null;
 			image_align = ContentAlignment.MiddleCenter;
 			SetUseMnemonic (UseMnemonic);
-
-			BackColor = ThemeEngine.Current.ColorButtonFace;
-			ForeColor = ThemeEngine.Current.ColorWindowText;
+			flat_style = FlatStyle.Standard;
 
 			CalcPreferredHeight ();
 			CalcPreferredWidth ();
@@ -106,9 +103,12 @@ namespace System.Windows.Forms
 			AutoSizeChanged = null;
     			TextAlignChanged = null;
 
-			SetStyle (ControlStyles.ResizeRedraw, true);
 			SetStyle (ControlStyles.Selectable, false);
-			SetStyle (ControlStyles.UserPaint | ControlStyles.AllPaintingInWmPaint, true);
+			SetStyle (ControlStyles.ResizeRedraw | 
+				ControlStyles.UserPaint | 
+				ControlStyles.AllPaintingInWmPaint |
+				ControlStyles.SupportsTransparentBackColor |
+				ControlStyles.DoubleBuffer, true);
 			
 			HandleCreated += new EventHandler (OnHandleCreatedLB);
 		}
@@ -153,19 +153,8 @@ namespace System.Windows.Forms
 		[DefaultValue(BorderStyle.None)]
 		[DispId(-504)]
     		public virtual BorderStyle BorderStyle {
-    			get {
-    				return border_style;
-    			}
-    			set {
-				if (!Enum.IsDefined (typeof (BorderStyle), value))
-					throw new InvalidEnumArgumentException (string.Format("Enum argument value '{0}' is not valid for BorderStyle", value));
-
-				if (border_style == value)
-					return;
-
-    				border_style = value;
-				Refresh ();
-    			}
+    			get { return InternalBorderStyle; }
+    			set { InternalBorderStyle = value; }
     		}
 
     		protected override CreateParams CreateParams {
@@ -458,6 +447,9 @@ namespace System.Windows.Forms
     		protected override void Dispose(bool disposing)
 		{
 			base.Dispose (disposing);
+
+			if (disposing)
+				string_format.Dispose ();
 		}
 
     		protected void DrawImage (Graphics g, Image image, Rectangle area, ContentAlignment img_align)
@@ -487,7 +479,11 @@ namespace System.Windows.Forms
     		protected override void OnFontChanged (EventArgs e)
     		{
 			base.OnFontChanged (e);
-			CalcPreferredHeight ();
+			if (autosize) {
+				CalcAutoSize();
+			} else {
+				CalcPreferredHeight ();
+			}
 			Refresh ();
     		}
 
@@ -512,8 +508,11 @@ namespace System.Windows.Forms
     		protected override void OnTextChanged (EventArgs e)
     		{
 			base.OnTextChanged (e);			
-			CalcPreferredWidth ();
-			CalcAutoSize ();
+			if (autosize) {
+				CalcAutoSize ();
+			} else {
+				CalcPreferredWidth ();
+			}
 			Refresh ();
     		}
 
@@ -542,10 +541,10 @@ namespace System.Windows.Forms
 
     		public override string ToString()
     		{
-    			return base.ToString () + "Text: " + Text;
+    			return base.ToString () + ", Text: " + Text;
     		}
 
-    		protected override void WndProc(ref Message m)
+    		protected override void WndProc (ref Message m)
     		{
 			switch ((Msg) m.Msg) {
 				case Msg.WM_DRAWITEM: {
@@ -615,5 +614,5 @@ namespace System.Windows.Forms
 
 		#endregion Private Methods
 
-    	}
-    }
+	}
+}
