@@ -93,7 +93,8 @@ namespace System.Windows.Forms
 			DisabledLinkColor = ThemeEngine.Current.ColorGrayText;
 			LinkColor = Color.FromArgb (255, 0, 0, 255);
 			VisitedLinkColor = Color.FromArgb (255, 128, 0, 128);
-			SetStyle (ControlStyles.Selectable, true);			
+			SetStyle (ControlStyles.Selectable, false);
+			SetStyle (ControlStyles.Opaque, true);
 		}
 
 		#region Public Properties
@@ -402,7 +403,7 @@ namespace System.Windows.Forms
 		{
 			ThemeEngine.Current.DrawLinkLabel (pevent.Graphics, pevent.ClipRectangle, this);
 			DrawImage (pevent.Graphics, Image, ClientRectangle, image_align);
-			base.OnPaint(pevent);
+			// Do not call base.OnPaint since it's the Label class 
 		}
 
 		protected override void OnPaintBackground (PaintEventArgs e)
