@@ -110,7 +110,6 @@ namespace System.Windows.Forms
 			AllItems
 		}
 
-		private BorderStyle border_style;
 		private int column_width;
 		private DrawMode draw_mode;
 		private int horizontal_extent;
@@ -124,7 +123,6 @@ namespace System.Windows.Forms
 		private SelectionMode selection_mode;
 		private bool sorted;
 		private bool use_tabstops;
-		private int top_index;
 		private int column_width_internal;
 		private VScrollBar vscrollbar_ctrl;
 		private HScrollBar hscrollbar_ctrl;
@@ -132,6 +130,7 @@ namespace System.Windows.Forms
 		private bool ctrl_pressed;
 		private bool shift_pressed;
 		private bool has_focus;
+		private bool use_item_height;
 		
 		internal int focused_item;		
 		internal ListBoxInfo listbox_info;
@@ -150,7 +149,6 @@ namespace System.Windows.Forms
 			focused_item = -1;
 			selection_mode = SelectionMode.One;
 			sorted = false;
-			top_index = 0;
 			use_tabstops = true;
 			BackColor = ThemeEngine.Current.ColorWindow;
 			ColumnWidth = 0;
@@ -158,6 +156,7 @@ namespace System.Windows.Forms
 			ctrl_pressed = false;
 			shift_pressed = false;
 			has_focus = false;
+			use_item_height = false;
 
 			items = new ObjectCollection (this);
 			selected_indices = new SelectedIndexCollection (this);
@@ -190,7 +189,7 @@ namespace System.Windows.Forms
 			GotFocus += new EventHandler (OnGotFocus);
 			LostFocus += new EventHandler (OnLostFocus);
 			
-			SetStyle (ControlStyles.UserPaint | ControlStyles.AllPaintingInWmPaint, true);
+			SetStyle (ControlStyles.UserPaint, false);
 		}
 
 		#region Events
@@ -248,18 +247,8 @@ namespace System.Windows.Forms
 		[DefaultValue (BorderStyle.Fixed3D)]
 		[DispId(-504)]
 		public BorderStyle BorderStyle {
-			get { return border_style; }
-
-    			set {
-				if (!Enum.IsDefined (typeof (BorderStyle), value))
-					throw new InvalidEnumArgumentException (string.Format("Enum argument value '{0}' is not valid for BorderStyle", value));
-
-				if (border_style == value)
-					return;
-
-    				border_style = value;
-				base.Refresh ();
-    			}
+			get { return InternalBorderStyle; }
+			set { InternalBorderStyle = value; }
 		}
 
 		[DefaultValue (0)]
@@ -267,7 +256,7 @@ namespace System.Windows.Forms
 		public int ColumnWidth {
 			get { return column_width; }
 			set {
-				if (column_width < 0)
+				if (value < 0)
 					throw new ArgumentException ("A value less than zero is assigned to the property.");
 
     				column_width = value;
@@ -299,7 +288,7 @@ namespace System.Windows.Forms
 					throw new InvalidEnumArgumentException (string.Format("Enum argument value '{0}' is not valid for DrawMode", value));
 					
 				if (value == DrawMode.OwnerDrawVariable && multicolumn == true)
-					throw new InvalidEnumArgumentException ("Cannot have variable height and multicolumn");
+					throw new ArgumentException ("Cannot have variable height and multicolumn");
 
 				if (draw_mode == value)
 					return;
@@ -379,6 +368,7 @@ namespace System.Windows.Forms
 					return;
 
 				listbox_info.item_height = value;
+				use_item_height = true;
 				CalcClientArea ();
 			}
 		}
@@ -425,9 +415,6 @@ namespace System.Windows.Forms
 						itemsHeight += items.GetListBoxItem (i).ItemHeight;
 				}
 				
-				itemsHeight += ThemeEngine.Current.DrawListBoxDecorationTop (BorderStyle);
-				itemsHeight += ThemeEngine.Current.DrawListBoxDecorationBottom (BorderStyle);
-				
 				return itemsHeight;
 			}
 		}
@@ -435,9 +422,6 @@ namespace System.Windows.Forms
 		public override RightToLeft RightToLeft {
 			get { return base.RightToLeft; }
 			set {
-				if (base.RightToLeft == value)
-					return;
-
     				base.RightToLeft = value;    				
 				base.Refresh ();
 			}
@@ -597,15 +581,16 @@ namespace System.Windows.Forms
 		[Browsable (false)]
 		[DesignerSerializationVisibility (DesignerSerializationVisibility.Hidden)]
 		public int TopIndex {
-			get { return top_index;}
+			get { return LBoxInfo.top_item; }
 			set {
-				if (value == top_index)
+				if (value == LBoxInfo.top_item)
 					return;
 
 				if (value < 0 || value >= Items.Count)
 					return;
 
-				value = top_index;
+				LBoxInfo.top_item = value;
+				UpdatedTopItem ();
 				base.Refresh ();
 			}
 		}
@@ -673,11 +658,18 @@ namespace System.Windows.Forms
 
 		public int FindString (String s)
 		{
-			return FindString (s, 0);
+			return FindString (s, -1);
 		}
 
 		public int FindString (string s,  int startIndex)
 		{
+			if (Items.Count == 0)
+				return -1; // No exception throwing if empty
+
+			if (startIndex < -1 || startIndex >= Items.Count - 1)
+				throw new ArgumentOutOfRangeException ("Index of out range");
+
+			startIndex++;
 			for (int i = startIndex; i < Items.Count; i++) {
 				if ((GetItemText (Items[i])).StartsWith (s))
 					return i;
@@ -688,11 +680,18 @@ namespace System.Windows.Forms
 
 		public int FindStringExact (string s)
 		{
-			return FindStringExact (s, 0);
+			return FindStringExact (s, -1);
 		}
 
 		public int FindStringExact (string s,  int startIndex)
 		{
+			if (Items.Count == 0)
+				return -1; // No exception throwing if empty
+
+			if (startIndex < -1 || startIndex >= Items.Count - 1)
+				throw new ArgumentOutOfRangeException ("Index of out range");
+
+			startIndex++;
 			for (int i = startIndex; i < Items.Count; i++) {
 				if ((GetItemText (Items[i])).Equals (s))
 					return i;
@@ -736,18 +735,24 @@ namespace System.Windows.Forms
 				
 				if (DrawMode == DrawMode.OwnerDrawVariable) {
 					rect.Y = 0;
-					for (int i = 0; i < index; i++) {
-						rect.Y += GetItemHeight (i);
-					}					
+					if (index >= listbox_info.top_item) {
+						for (int i = listbox_info.top_item; i < index; i++) {
+							rect.Y += GetItemHeight (i);
+						}
+					} else {
+						for (int i = index; i < listbox_info.top_item; i++) {
+							rect.Y -= GetItemHeight (i);
+						}
+					}
 				} else {
-					rect.Y = ItemHeight * index;	
+					rect.Y = ItemHeight * (index - listbox_info.top_item);	
 				}				
 			}
 			else {
 				int which_page;
 
 				which_page = index / listbox_info.page_size;
-				rect.Y = (index % listbox_info.page_size) * ItemHeight;
+				rect.Y = ((index - listbox_info.top_item) % listbox_info.page_size) * ItemHeight;
 				rect.X = which_page * ColumnWidthInternal;
 				rect.Height = ItemHeight;
 				rect.Width = ColumnWidthInternal;
@@ -772,7 +777,12 @@ namespace System.Windows.Forms
 		// Only returns visible points
 		public int IndexFromPoint (int x, int y)
 		{
-			for (int i = LBoxInfo.top_item; i < LBoxInfo.last_item; i++) {
+
+			if (Items.Count == 0) {
+				return -1;
+			}
+
+			for (int i = LBoxInfo.top_item; i <= LBoxInfo.last_item; i++) {
 				if (GetItemRectangle (i).Contains (x,y) == true)
 					return i;
 			}
@@ -823,11 +833,14 @@ namespace System.Windows.Forms
 		protected override void OnFontChanged (EventArgs e)
 		{
 			base.OnFontChanged (e);
-			listbox_info.item_height = FontHeight;
-
-			RellocateScrollBars ();
-			CalcClientArea ();
-			UpdateItemInfo (UpdateOperation.AllItems, 0, 0);
+			if (!use_item_height) {
+				listbox_info.item_height = FontHeight;
+				RellocateScrollBars ();
+				CalcClientArea ();
+				UpdateItemInfo (UpdateOperation.AllItems, 0, 0);
+			} else {
+				base.Refresh ();
+			}
 		}
 
 		protected override void OnHandleCreated (EventArgs e)
@@ -835,8 +848,10 @@ namespace System.Windows.Forms
 			base.OnHandleCreated (e);
 
 			UpdateInternalClientRect (ClientRectangle);
-			Controls.Add (vscrollbar_ctrl);
-			Controls.Add (hscrollbar_ctrl);
+			SuspendLayout ();
+			Controls.AddImplicit (vscrollbar_ctrl);
+			Controls.AddImplicit (hscrollbar_ctrl);
+			ResumeLayout ();
 			UpdateItemInfo (UpdateOperation.AllItems, 0, 0);
 		}
 
@@ -962,9 +977,9 @@ namespace System.Windows.Forms
 
 			case Msg.WM_PAINT: {
 				PaintEventArgs	paint_event;
-				paint_event = XplatUI.PaintEventStart (Handle);
+				paint_event = XplatUI.PaintEventStart (Handle, true);
 				OnPaintLB (paint_event);
-				XplatUI.PaintEventEnd (Handle);
+				XplatUI.PaintEventEnd (Handle, true);
 				return;
 			}
 
@@ -984,14 +999,9 @@ namespace System.Windows.Forms
 		#region Private Methods
 
 		internal void CalcClientArea ()
-		{
-			listbox_info.textdrawing_rect = listbox_info.client_rect;
-			listbox_info.textdrawing_rect.Y += ThemeEngine.Current.DrawListBoxDecorationTop (BorderStyle);
-			listbox_info.textdrawing_rect.X += ThemeEngine.Current.DrawListBoxDecorationLeft (BorderStyle);
-			//BUG: Top and Left decorations
-			listbox_info.textdrawing_rect.Height -= ThemeEngine.Current.DrawListBoxDecorationBottom (BorderStyle);
-			listbox_info.textdrawing_rect.Width -= ThemeEngine.Current.DrawListBoxDecorationRight (BorderStyle);
-
+		{		
+			listbox_info.textdrawing_rect = listbox_info.client_rect;			
+			
 			if (listbox_info.show_verticalsb)
 				listbox_info.textdrawing_rect.Width -= vscrollbar_ctrl.Width;
 
@@ -1011,24 +1021,21 @@ namespace System.Windows.Forms
 				}
 								
 			} else {			
-				listbox_info.page_size = listbox_info.textdrawing_rect.Height / listbox_info.item_height;
+				listbox_info.page_size = listbox_info.textdrawing_rect.Height / ItemHeight;
 			}
 
 			if (listbox_info.page_size == 0) {
 				listbox_info.page_size = 1;
 			}
 
-			/* Adjust size to visible the maxim number of displayable items */
+			/* Adjust size to visible the maximum number of displayable items */
 			if (IntegralHeight == true) {
 
 				// From MS Docs: The integral height is based on the height of the ListBox, rather than
 				// the client area height. As a result, when the IntegralHeight property is set true,
 				// items can still be partially shown if scroll bars are displayed.
 
-				int remaining =  (listbox_info.client_rect.Height -
-					ThemeEngine.Current.DrawListBoxDecorationBottom (BorderStyle) -
-					ThemeEngine.Current.DrawListBoxDecorationBottom (BorderStyle)) %
-					listbox_info.item_height;
+				int remaining = listbox_info.client_rect.Height % listbox_info.item_height;
 
 				if (remaining > 0) {
 					listbox_info.client_rect.Height -= remaining;
@@ -1040,19 +1047,27 @@ namespace System.Windows.Forms
 		}
 
 		internal void Draw (Rectangle clip, Graphics dc)
-		{	
-			if (LBoxInfo.textdrawing_rect.Contains (clip) == false) {
-				// IntegralHeight has effect, we also have to paint the unused area
+		{				
+			// IntegralHeight has effect, we also have to paint the unused area	
+			if (IntegralHeight) {				
 				if (ClientRectangle.Height > listbox_info.client_rect.Height) {
 					Region area = new Region (ClientRectangle);
-					area.Exclude (listbox_info.client_rect);
+					area.Exclude (LBoxInfo.textdrawing_rect);
 
-					dc.FillRectangle (ThemeEngine.Current.ResPool.GetSolidBrush (Parent.BackColor),
+					if (listbox_info.show_horizontalsb) {
+						area.Exclude (new Rectangle (hscrollbar_ctrl.Location.X, hscrollbar_ctrl.Location.Y,
+							hscrollbar_ctrl.Width, hscrollbar_ctrl.Height));
+					}						
+
+					dc.FillRectangle (ThemeEngine.Current.ResPool.GetSolidBrush (BackColor),
 						area.GetBounds (dc));
-				}
 
-				dc.FillRectangle (ThemeEngine.Current.ResPool.GetSolidBrush (BackColor), LBoxInfo.textdrawing_rect);				
-			}					
+					area.Dispose ();
+				}
+			}
+			
+			dc.FillRectangle (ThemeEngine.Current.ResPool.GetSolidBrush (BackColor),
+					LBoxInfo.textdrawing_rect);
 
 			if (Items.Count > 0) {
 				Rectangle item_rect;
@@ -1074,7 +1089,7 @@ namespace System.Windows.Forms
 					if (has_focus == true && focused_item == i)
 						state |= DrawItemState.Focus;
 						
-					if (hscrollbar_ctrl != null && hscrollbar_ctrl.Visible) {
+					if (MultiColumn == false && hscrollbar_ctrl != null && hscrollbar_ctrl.Visible) {
 						item_rect.X -= hscrollbar_ctrl.Value;
 						item_rect.Width += hscrollbar_ctrl.Value;
 					}
@@ -1082,9 +1097,8 @@ namespace System.Windows.Forms
 					OnDrawItem (new DrawItemEventArgs (dc, Font, item_rect,
 						i, state, ForeColor, BackColor));
 				}
-			}			
+			}
 			
-			ThemeEngine.Current.DrawListBoxDecorations (dc, this);
 		}
 
 		// Converts a GetItemRectangle to a one that we can display
@@ -1095,33 +1109,36 @@ namespace System.Windows.Forms
 			item_rect = GetItemRectangle (index);
 			item_rect.X -= first_item_rect.X;
 			item_rect.Y -= first_item_rect.Y;
-
-			item_rect.Y += ThemeEngine.Current.DrawListBoxDecorationTop (BorderStyle);
-			item_rect.X += ThemeEngine.Current.DrawListBoxDecorationLeft (BorderStyle);
-			item_rect.Width -= ThemeEngine.Current.DrawListBoxDecorationRight (BorderStyle);
-
+			
 			return item_rect;
 		}
 
 		// Value Changed
 		private void HorizontalScrollEvent (object sender, EventArgs e)
 		{
-			if (multicolumn) {
-				LBoxInfo.top_item = listbox_info.page_size * hscrollbar_ctrl.Value;
-				LBoxInfo.last_item = LastVisibleItem ();
+			if (!multicolumn) {
+				base.Refresh ();
+				return;
 			}
+
+			int top_item = LBoxInfo.top_item;
+			int last_item = LBoxInfo.last_item;
+
+			LBoxInfo.top_item = listbox_info.page_size * hscrollbar_ctrl.Value;
+			LBoxInfo.last_item = LastVisibleItem ();
 			
-			base.Refresh ();
+			if (top_item != LBoxInfo.top_item || last_item != LBoxInfo.last_item)
+				base.Refresh ();
 		}
 
 		// Only returns visible points. The diference of with IndexFromPoint is that the rectangle
 		// has screen coordinates
 		internal int IndexFromPointDisplayRectangle (int x, int y)
-		{				
-			if (LBoxInfo.top_item == LBoxInfo.last_item)
+		{	
+			if (Items.Count == 0)
 				return -1;
 			
-    			for (int i = LBoxInfo.top_item; i <= LBoxInfo.last_item; i++) {
+			for (int i = LBoxInfo.top_item; i <= LBoxInfo.last_item; i++) {
 				if (GetItemDisplayRectangle (i, LBoxInfo.top_item).Contains (x, y) == true)
 					return i;
 			}
@@ -1146,15 +1163,9 @@ namespace System.Windows.Forms
 					if (item_rect.X > LBoxInfo.textdrawing_rect.Width)
 						return i - 1;
 				}
-				else {
-					if (IntegralHeight) {
-						if (item_rect.Y + item_rect.Height > top_y) {
-							return i - 1;
-						}
-					}
-					else {
-						if (item_rect.Y + item_rect.Height > top_y)
-							return i - 1;
+				else {					
+					if (item_rect.Y + item_rect.Height > top_y) {
+						return i;
 					}
 				}
 			}
@@ -1163,10 +1174,14 @@ namespace System.Windows.Forms
 
 		private void UpdatedTopItem ()
 		{
-			if (multicolumn) {
-				int col = (LBoxInfo.top_item / LBoxInfo.page_size);
-				hscrollbar_ctrl.Value = col;
-			}				
+			if (multicolumn) {				
+				int col = LBoxInfo.top_item / LBoxInfo.page_size;
+				
+				if (col > hscrollbar_ctrl.Maximum)
+					hscrollbar_ctrl.Value = hscrollbar_ctrl.Maximum;
+				else
+					hscrollbar_ctrl.Value = col;
+			}
 			else {
 				if (LBoxInfo.top_item > vscrollbar_ctrl.Maximum)
 					vscrollbar_ctrl.Value = vscrollbar_ctrl.Maximum;
@@ -1243,9 +1258,19 @@ namespace System.Windows.Forms
 
 			case ItemNavigation.Next: {
 				if (focused_item + 1 < Items.Count) {	
-					if (focused_item + 1 > LBoxInfo.last_item) {
-						LBoxInfo.top_item++;
-						UpdatedTopItem ();						
+					int actualHeight = 0;
+					if (draw_mode == DrawMode.OwnerDrawVariable) {
+						for (int i = LBoxInfo.top_item; i <= focused_item + 1; i++)
+							actualHeight += GetItemHeight (i);
+					} else {
+						actualHeight = ((focused_item + 1) - LBoxInfo.top_item + 1) * ItemHeight;
+					}
+					if (actualHeight >= LBoxInfo.textdrawing_rect.Height) {
+						int bal = IntegralHeight ? 0 : (listbox_info.textdrawing_rect.Height == actualHeight ? 0 : 1);
+						if (focused_item + bal >= LBoxInfo.last_item) {
+							LBoxInfo.top_item++;
+							UpdatedTopItem ();						
+						}
 					}
 					selected_index = focused_item + 1;
 				}
@@ -1336,6 +1361,9 @@ namespace System.Windows.Forms
 		private void OnKeyDownLB (object sender, KeyEventArgs e)
 		{					
 			int new_item = -1;
+			
+			if (Items.Count == 0)
+				return;
 
 			switch (e.KeyCode) {
 				
@@ -1449,31 +1477,18 @@ namespace System.Windows.Forms
 		internal void RellocateScrollBars ()
 		{
 			if (listbox_info.show_verticalsb) {
-
-				vscrollbar_ctrl.Size = new Size (vscrollbar_ctrl.Width,
-					listbox_info.client_rect.Height - ThemeEngine.Current.DrawListBoxDecorationTop (BorderStyle) -
-					ThemeEngine.Current.DrawListBoxDecorationBottom (BorderStyle));
-
-				vscrollbar_ctrl.Location = new Point (listbox_info.client_rect.Width - vscrollbar_ctrl.Width
-					- ThemeEngine.Current.DrawListBoxDecorationRight (BorderStyle),
-					ThemeEngine.Current.DrawListBoxDecorationTop (BorderStyle));
-
+				vscrollbar_ctrl.Size = new Size (vscrollbar_ctrl.Width,	ClientRectangle.Height);
+				vscrollbar_ctrl.Location = new Point (ClientRectangle.Width - vscrollbar_ctrl.Width, 0);
 			}
 
 			if (listbox_info.show_horizontalsb) {
-
-				int width;
-
-				width = listbox_info.client_rect.Width - (ThemeEngine.Current.DrawListBoxDecorationLeft (BorderStyle) + ThemeEngine.Current.DrawListBoxDecorationRight (BorderStyle));
+				int width = listbox_info.client_rect.Width;
 
 				if (listbox_info.show_verticalsb)
 					width -= vscrollbar_ctrl.Width;
 
 				hscrollbar_ctrl.Size = new Size (width, hscrollbar_ctrl.Height);
-
-				hscrollbar_ctrl.Location = new Point (ThemeEngine.Current.DrawListBoxDecorationLeft (BorderStyle),
-					listbox_info.client_rect.Height - hscrollbar_ctrl.Height
-					- ThemeEngine.Current.DrawListBoxDecorationTop (BorderStyle));
+				hscrollbar_ctrl.Location = new Point (0, ClientRectangle.Height - hscrollbar_ctrl.Height);
 			}
 
 			CalcClientArea ();
@@ -1490,13 +1505,13 @@ namespace System.Windows.Forms
     			selected_indices.AddIndex (index);
     			selected_items.AddObject (Items[index]);
 
-    			if (ClientRectangle.Contains (invalidate))
+    			if (ClientRectangle.IntersectsWith (invalidate))
     				Invalidate (invalidate);
 
 		}		
 		
 		// An item navigation operation (mouse or keyboard) has caused to select a new item
-		private void SelectedItemFromNavigation (int index)
+		internal void SelectedItemFromNavigation (int index)
 		{
 			switch (SelectionMode) {
     				case SelectionMode.None: // Do nothing
@@ -1583,7 +1598,7 @@ namespace System.Windows.Forms
 			}
 		}
 		
-		void SetFocusedItem (int index)
+		internal void SetFocusedItem (int index)
 		{			
 			Rectangle invalidate;
 			int prev = focused_item;			
@@ -1618,7 +1633,7 @@ namespace System.Windows.Forms
 				selected_items.RemoveObject (Items[index]);
 			}
 
-			if (ClientRectangle.Contains (invalidate))
+			if (ClientRectangle.IntersectsWith (invalidate))
 				Invalidate (invalidate);
 		}
 
@@ -1787,6 +1802,8 @@ namespace System.Windows.Forms
 					vscrollbar_ctrl.Maximum = Items.Count - listbox_info.page_size;
 
 				RellocateScrollBars ();
+			} else if (vscrollbar_ctrl.Maximum > 0) {
+				vscrollbar_ctrl.Maximum = 0;
 			}
 
 			CalcClientArea ();
@@ -1795,10 +1812,14 @@ namespace System.Windows.Forms
 		// Value Changed
 		private void VerticalScrollEvent (object sender, EventArgs e)
 		{
+			int top_item = LBoxInfo.top_item;
+			int last_item = LBoxInfo.last_item;
+
 			LBoxInfo.top_item = /*listbox_info.page_size + */ vscrollbar_ctrl.Value;
 			LBoxInfo.last_item = LastVisibleItem ();
 
-			base.Refresh ();
+			if (top_item != LBoxInfo.top_item || last_item != LBoxInfo.last_item)
+				base.Refresh ();
 		}
 
 		#endregion Private Methods
