@@ -47,6 +47,7 @@ namespace System.Windows.Forms
 			value_member = new BindingMemberInfo (string.Empty);
 			display_member = string.Empty;
 			data_manager = null;
+			SetStyle (ControlStyles.StandardClick | ControlStyles.UserPaint, false);
 		}
 
 		#region Events
@@ -60,7 +61,7 @@ namespace System.Windows.Forms
 
 		[DefaultValue(null)]
 		[RefreshProperties(RefreshProperties.Repaint)]
-		[TypeConverter("System.Windows.Forms.Design.DataSourceConverter, System.Design, Version=1.0.5000.0, Culture=neutral, PublicKeyToken=b03f5f7f11d50a3a")]
+		[TypeConverter("System.Windows.Forms.Design.DataSourceConverter, " + Consts.AssemblySystem_Design)]
 		public object DataSource {
 			get { return data_source; }
 			set {
@@ -80,7 +81,7 @@ namespace System.Windows.Forms
 
 		[DefaultValue("")]
 		[Editor("System.Windows.Forms.Design.DataMemberFieldEditor, " + Consts.AssemblySystem_Design, typeof(System.Drawing.Design.UITypeEditor))]
-		[TypeConverter("System.Windows.Forms.Design.DataMemberFieldConverter, System.Design, Version=1.0.5000.0, Culture=neutral, PublicKeyToken=b03f5f7f11d50a3a")]
+		[TypeConverter("System.Windows.Forms.Design.DataMemberFieldConverter, " + Consts.AssemblySystem_Design)]
 		public string DisplayMember {
 			get { 
 				return display_member;				
@@ -299,8 +300,8 @@ namespace System.Windows.Forms
 				return;
 			}
 			
-			data_manager = (CurrencyManager) BindingContext [data_source, ValueMember];
-			data_manager.PositionChanged += new EventHandler (OnPositionChanged);			
+			data_manager = (CurrencyManager) BindingContext [data_source];
+			data_manager.PositionChanged += new EventHandler (OnPositionChanged);
 		}		
 		
 		private void OnPositionChanged (object sender, EventArgs e)
