@@ -24,16 +24,13 @@
 //	Jordi Mas i Hernandez, jordi@ximian.com
 //
 // TODO:
-//   - Keys to be handled ENTER/PAGE UP/PAGE DOWN/HOME/END/ARROWS/CTRL/SHIFT
 //   - Item text editing
 //   - Column resizing/reodering
 //   - Feedback for item activation, change in cursor types as mouse moves.
 //   - HideSelection
-//   - Focused item (broken and not drawn)
 //   - LabelEdit
 //   - Manual column resizing
 //   - Drag and drop
-//   - Clipping 
 
 
 // NOT COMPLETE
@@ -44,6 +41,7 @@ using System.ComponentModel;
 using System.ComponentModel.Design;
 using System.Drawing;
 using System.Runtime.InteropServices;
+using System.Globalization;
 
 namespace System.Windows.Forms
 {
@@ -65,8 +63,7 @@ namespace System.Windows.Forms
 		private ColumnHeaderCollection columns;
 		private bool ctrl_pressed;
 		private bool shift_pressed;
-		private bool draw_headers = true; // Used for painting. Do we need to draw column headers ?
-		private ListViewItem focused_item;
+		internal ListViewItem focused_item;
 		private bool full_row_select = false;
 		private bool grid_lines = false;
 		private ColumnHeaderStyle header_style = ColumnHeaderStyle.Clickable;
@@ -87,10 +84,14 @@ namespace System.Windows.Forms
 		private int layout_wd;    // We might draw more than our client area
 		private int layout_ht;    // therefore we need to have these two.
 		//private TextBox editor;   // Used for editing an item text
-		private ScrollBar h_scroll; // used for scrolling horizontally
-		private ScrollBar v_scroll; // used for scrolling vertically
-		private int h_marker;		// Position markers for scrolling
-		private int v_marker;
+		internal ScrollBar h_scroll; // used for scrolling horizontally
+		internal ScrollBar v_scroll; // used for scrolling vertically
+		internal int h_marker;		// Position markers for scrolling
+		internal int v_marker;
+		internal Rectangle client_area; // ClientRectangle - scrollbars
+		private int keysearch_tickcnt;
+		private string keysearch_text;
+		static private readonly int keysearch_keydelay = 1000;
 
 		// internal variables
 		internal ImageList large_image_list;
@@ -139,6 +140,7 @@ namespace System.Windows.Forms
 			h_scroll = new HScrollBar ();
 			v_scroll = new VScrollBar ();
 			h_marker = v_marker = 0;
+			keysearch_tickcnt = 0;
 
 			// scroll bars are disabled initially
 			h_scroll.Visible = false;
@@ -157,7 +159,7 @@ namespace System.Windows.Forms
 			base.Paint += new PaintEventHandler (ListView_Paint);
 			SizeChanged += new EventHandler (ListView_SizeChanged);
 
-			this.SetStyle (ControlStyles.UserPaint | ControlStyles.AllPaintingInWmPaint, true);
+			this.SetStyle (ControlStyles.UserPaint | ControlStyles.StandardClick, false);
 		}
 		#endregion	// Public Constructors
 
@@ -284,19 +286,8 @@ namespace System.Windows.Forms
 		[DefaultValue (BorderStyle.Fixed3D)]
 		[DispId (-504)]
 		public BorderStyle BorderStyle {
-			get { return border_style; }
-			set {
-				if (value != BorderStyle.Fixed3D && value != BorderStyle.FixedSingle  && 
-					value != BorderStyle.None) {
-					throw new InvalidEnumArgumentException (string.Format 
-						("Enum argument value '{0}' is not valid for BorderStyle", value));
-				}
-				
-				if (border_style != value) {
-					border_style = value;
-					this.Redraw (true);
-				}
-			}
+			get { return InternalBorderStyle; }
+			set { InternalBorderStyle = value; }
 		}
 
 		[DefaultValue (false)]
@@ -534,7 +525,7 @@ namespace System.Windows.Forms
 				// do a hit test for the scrolled position
 				else {
 					foreach (ListViewItem item in this.items) {
-						if (item.EntireRect.Contains (h_marker, v_marker))
+						if (item.EntireRect.X >= h_marker && item.EntireRect.Y >= v_marker)
 							return item;
 					}
 					return null;
@@ -553,6 +544,7 @@ namespace System.Windows.Forms
 				}
 				
 				if (view != value) {
+					h_scroll.Value = v_scroll.Value = 0;
 					view = value; 
 					Redraw (true);
 				}
@@ -561,6 +553,38 @@ namespace System.Windows.Forms
 		#endregion	// Public Instance Properties
 
 		#region Internal Methods Properties
+		
+		internal int FirstVisibleIndex {
+			get {
+				// there is no item
+				if (this.items.Count == 0)
+					return 0;
+									
+				if (h_marker == 0 && v_marker == 0)
+					return 0;					
+				
+				foreach (ListViewItem item in this.items) {
+					if (item.EntireRect.X + item.EntireRect.Width >= h_marker 
+						&& item.EntireRect.Y + item.EntireRect.Height >= v_marker)
+						return item.Index;
+				}
+				return 0;
+
+			}
+		}
+
+		
+		internal int LastItemIndex {			
+			get {							
+				for (int i = FirstVisibleIndex; i < Items.Count; i++) {						
+					if (Items[i].EntireRect.Y > v_marker + ClientRectangle.Bottom)						
+							return i -1;					
+				}
+				
+				return Items.Count - 1;
+			}
+		}
+		
 		internal int TotalWidth {
 			get { return Math.Max (this.Width, this.layout_wd); }
 		}
@@ -690,6 +714,8 @@ namespace System.Windows.Forms
 
 		private void CalculateScrollBars ()
 		{
+			client_area = ClientRectangle;
+			
 			if (!this.scrollable || this.items.Count <= 0) {
 				h_scroll.Visible = false;
 				v_scroll.Visible = false;
@@ -697,56 +723,70 @@ namespace System.Windows.Forms
 			}
 
 			// making a scroll bar visible might make
-			// other scroll bar visible
-			if (layout_wd > this.Width) {
+			// other scroll bar visible			
+			if (layout_wd > client_area.Right) {
 				h_scroll.Visible = true;
-				if ((layout_ht + h_scroll.Height) > Height)
-					v_scroll.Visible = true;
-			} else if (layout_ht > Height) {
+				if ((layout_ht + h_scroll.Height) > client_area.Bottom) {
+					v_scroll.Visible = true;					
+				}
+				else {
+					v_scroll.Visible = false;
+				}
+			} else if (layout_ht > client_area.Bottom) {				
 				v_scroll.Visible = true;
-				if ((layout_wd + v_scroll.Width) > Width)
+				if ((layout_wd + v_scroll.Width) > client_area.Right) {
 					h_scroll.Visible = true;
-			}
+				}
+				else {
+					h_scroll.Visible = false;
+				}
+			} else {
+				h_scroll.Visible = false;
+				v_scroll.Visible = false;
+			}			
 
 			if (h_scroll.Visible) {
-				h_scroll.Location = new Point (0, Height - h_scroll.Height);
+				h_scroll.Location = new Point (client_area.X, client_area.Bottom - h_scroll.Height);
 				h_scroll.Minimum = 0;
 
 				// if v_scroll is visible, adjust the maximum of the
 				// h_scroll to account for the width of v_scroll
 				if (v_scroll.Visible) {
 					h_scroll.Maximum = layout_wd + v_scroll.Width;
-					h_scroll.Width = Width - v_scroll.Width;
+					h_scroll.Width = client_area.Width - v_scroll.Width;
 				}
 				else {
 					h_scroll.Maximum = layout_wd;
-					h_scroll.Width = Width;
+					h_scroll.Width = client_area.Width;
 				}
    
-				h_scroll.LargeChange = Width;
+				h_scroll.LargeChange = client_area.Width;
 				h_scroll.SmallChange = Font.Height;
+				client_area.Height -= h_scroll.Height;
 			}
 
 			// vertical scrollbar
 			if (v_scroll.Visible) {
-				v_scroll.Location = new Point (Width - v_scroll.Width, 0);
+				v_scroll.Location = new Point (client_area.Right - v_scroll.Width, client_area.Y);
 				v_scroll.Minimum = 0;
 
 				// if h_scroll is visible, adjust the maximum of the
 				// v_scroll to account for the height of h_scroll
 				if (h_scroll.Visible) {
 					v_scroll.Maximum = layout_ht + h_scroll.Height;
-					v_scroll.Height = Height - h_scroll.Height;
+					v_scroll.Height = client_area.Height; // - h_scroll.Height already done 
 				} else {
 					v_scroll.Maximum = layout_ht;
-					v_scroll.Height = Height;
+					v_scroll.Height = client_area.Height;
 				}
 
-				v_scroll.LargeChange = Height;
+				v_scroll.LargeChange = client_area.Height;
 				v_scroll.SmallChange = Font.Height;
+				client_area.Width -= v_scroll.Width;
 			}
 		}
-
+		
+		
 		// Sets the location of every item on
 		// the ListView as per the view
 		private void CalculateListView (ListViewAlignment align)
@@ -767,11 +807,12 @@ namespace System.Windows.Forms
 			case View.Details:
 				// ColumnHeaders are not drawn if headerstyle is none
 				int ht = (this.header_style == ColumnHeaderStyle.None) ? 
-					2 : this.Font.Height + 5;
+					0 : this.Font.Height + 3;
+				
 				if (columns.Count > 0) {
 					foreach (ColumnHeader col in columns) {
 						col.X = current_pos_x;
-						col.Y = 0;
+						col.Y = current_pos_y;
 						col.CalcColumnHeader ();
 						current_pos_x += col.Wd;
 					}
@@ -809,11 +850,11 @@ namespace System.Windows.Forms
 
 					// top (default) and snaptogrid alignments are handled same way
 					if (align == ListViewAlignment.Left) {
-						max = this.Height;
+						max = client_area.Height;
 						foreach (ListViewItem item in items) {
 							item.location.X = current_pos_x +
 								horizontal_spacing;
-							item.location.Y = current_pos_y;
+							item.location.Y = 0;
 							item.CalcListViewItem ();
 							current_pos_y += item_ht;
 
@@ -828,7 +869,7 @@ namespace System.Windows.Forms
 							else {
 								// is there enough space for another row ?
 								if ((current_pos_y + vertical_spacing
-								     + item_ht) <= this.Height)
+								     + item_ht) <= client_area.Height)
 									current_pos_y += vertical_spacing;
 								else {
 									// start another column
@@ -846,7 +887,7 @@ namespace System.Windows.Forms
 						this.layout_wd = current_pos_x;
 					}
 					else { // other default/top alignment
-						max = this.Width;
+						max = client_area.Width;
 						foreach (ListViewItem item in items) {
 							item.location.X = current_pos_x +
 								horizontal_spacing;
@@ -866,7 +907,7 @@ namespace System.Windows.Forms
 							else {
 								// is there enough space for another column?
 								if ((current_pos_x + horizontal_spacing
-								     + item_wd) <= this.Width)
+								     + item_wd) <= client_area.Width)
 									continue;
 								else {
 									// start another row
@@ -893,7 +934,7 @@ namespace System.Windows.Forms
 					item_ht = items [0].EntireRect.Height;
 					item_wd = items [0].EntireRect.Width;
 
-					max = this.Height / item_ht;
+					max = client_area.Height / item_ht;
 					if (max == 0)
 						max = 1; // we draw at least one row
 
@@ -922,9 +963,37 @@ namespace System.Windows.Forms
 			}
 
                         CalculateScrollBars ();
-		}		
-		
+                        
+		}
 
+		private bool KeySearchString (KeyEventArgs ke)
+		{
+			int current_tickcnt = Environment.TickCount;
+			if (keysearch_tickcnt > 0 && current_tickcnt - keysearch_tickcnt > keysearch_keydelay) {
+				keysearch_text = string.Empty;
+			}
+			
+			keysearch_text += (char) ke.KeyData;
+			keysearch_tickcnt = current_tickcnt;
+
+			int i = FocusedItem.Index;
+			while (true) {
+				if (CultureInfo.CurrentCulture.CompareInfo.IsPrefix (Items[i].Text, keysearch_text,
+					CompareOptions.IgnoreCase)) {
+					SetFocusedItem (Items [i]);
+					items [i].Selected = true;
+					EnsureVisible (i);
+					break;
+				}
+				i = (i + 1  < Items.Count) ? i+1 : 0;
+
+				if (i == FocusedItem.Index)
+					break;
+			}
+			return true;
+		}
+
+				
 		// Event Handlers
 		private void ListView_DoubleClick (object sender, EventArgs e)
 		{
@@ -934,9 +1003,9 @@ namespace System.Windows.Forms
 		}
 
 		private void ListView_KeyDown (object sender, KeyEventArgs ke)
-		{
+		{			
 			int index = -1;
-			if (ke.Handled)
+			if (ke.Handled || Items.Count == 0)
 				return;
 
 			ke.Handled = true;
@@ -944,61 +1013,77 @@ namespace System.Windows.Forms
 			switch (ke.KeyCode) {
 
 			case Keys.ControlKey:
-				this.ctrl_pressed = true;
+				ctrl_pressed = true;
 				break;
 
-			case Keys.Down:				
-				// FIXME:TODO
+			case Keys.Down:
+				if (focused_item != null && focused_item.Index + 1 < Items.Count) {
+					index = focused_item.Index + 1;
+				}
 				break;
 
 			case Keys.End:
-				this.v_scroll.Value = this.v_scroll.Maximum;
+				index = Items.Count - 1;
 				break;
 
-			case Keys.Home:
-				this.v_scroll.Value = this.v_scroll.Minimum;
+			case Keys.Home:			
+				index = 0;
 				break;
 
 			case Keys.Left:
 				index = -1;
-				if (this.last_clicked_item != null)
-					index = this.last_clicked_item.Index;
+				if (focused_item != null)
+					index = focused_item.Index;
 				else
 					break;
 
 				if (index > 0)
 					index -= 1;
-
-				this.last_clicked_item = this.items [index];
-				this.last_clicked_item.Selected = true;
-				this.EnsureVisible (index);
+									
 				break;
 
 			case Keys.Right:
-				if (this.last_clicked_item != null)
-					index = this.last_clicked_item.Index + 1;
+				if (focused_item != null)
+					index = focused_item.Index + 1;
 				else
 					index = 1;
 
-				if (index == this.items.Count)
-					break;
+				if (index == items.Count)
+					index = -1;
 
-				this.last_clicked_item = this.items [index];
-				this.last_clicked_item.Selected = true;
-				this.EnsureVisible (index);
 				break;
 
 			case Keys.ShiftKey:
-				this.shift_pressed = true;
+				shift_pressed = true;
 				break;
 
 			case Keys.Up:				
-				// FIXME:TODO
+				if (focused_item != null)
+					index = focused_item.Index;
+				else
+					break;
+
+				if (index > 0)
+					index--;
+
+				if (index < 0) {
+					index = -1;
+				}
 				break;
 
 			default:
-				ke.Handled = false;
-				break;
+				if (KeySearchString (ke)) {
+					ke.Handled = true;
+				} else {
+					ke.Handled = false;
+				}
+				return;
+			}
+			
+			if (index != -1) {
+				items [index].Selected = true;
+				SetFocusedItem (items [index]);				
+				EnsureVisible (index);
 			}
 		}
 
@@ -1036,7 +1121,6 @@ namespace System.Windows.Forms
 
 					if (this.clicked_column != null) {
 						this.clicked_column.pressed = true;
-						this.draw_headers = true;
 						this.Redraw (false);
 						return;
 					}
@@ -1081,17 +1165,35 @@ namespace System.Windows.Forms
 					}
 				}
 			}
-			
-			// set the FocusedItem to be the current clicked_item
-			this.focused_item = this.clicked_item;
 
-			if (this.clicked_item != null) {
-				this.clicked_item.Selected = true;
-				// Raise the event
-				this.OnSelectedIndexChanged (new EventArgs ());
+			// set the FocusedItem to be the current clicked_item
+			SetFocusedItem (clicked_item);
+
+			if (clicked_item != null) {
+				bool changed = !clicked_item.Selected;
+				clicked_item.Selected = true;
+				
+				// Only Raise the event if the selected item has changed
+				if (changed)
+					OnSelectedIndexChanged (EventArgs.Empty);
+
+				// Raise double click if the item was clicked. On MS the
+				// double click is only raised if you double click an item
+				if (me.Clicks > 1 && this.clicked_item != null)
+					OnDoubleClick (EventArgs.Empty);
+				else if (me.Clicks == 1 && clicked_item != null)
+					OnClick (EventArgs.Empty);
 
 				this.Redraw (false);
-			}		
+			} else if (selected_indices.Count > 0) {
+				// NOTE: selected_indices isn't computed properly so
+				// this doesn't actually work
+				
+				// Raise the event if there was at least one item
+				// selected and the user click on a dead area (unselecting all)
+				OnSelectedIndexChanged (EventArgs.Empty);
+				Redraw (false);
+			}
 		}
 
 		private void ListView_MouseHover (object sender, EventArgs e)
@@ -1127,13 +1229,11 @@ namespace System.Windows.Forms
 				if (this.clicked_column.pressed == false &&
 				    this.clicked_column.Rect.Contains (hit)) {
 					this.clicked_column.pressed = true;
-					this.draw_headers = true;
 					this.Redraw (false);
 				}
 				else if (this.clicked_column.pressed && 
 					 ! this.clicked_column.Rect.Contains (hit)) {
 					this.clicked_column.pressed = false;
-					this.draw_headers = true;
 					this.Redraw (false);
 				}
 			}
@@ -1150,7 +1250,6 @@ namespace System.Windows.Forms
 			if (this.clicked_column != null) {
 				if (this.clicked_column.pressed) {
 					this.clicked_column.pressed = false;
-					this.draw_headers = true;
 					this.Redraw (false);
 
 					// Raise the ColumnClick event
@@ -1193,73 +1292,13 @@ namespace System.Windows.Forms
 		{
 			if (this.Width <= 0 || this.Height <=  0 ||
 			    this.Visible == false || this.updating == true)
-				return;
-
+				return;	
+				
 			CalculateScrollBars ();
 
 			ThemeEngine.Current.DrawListView (pe.Graphics,
 					pe.ClipRectangle, this);
-
-			// We paint on the screen as per the location set
-			// by the two scrollbars. In case of details view
-			// since column headers can scroll only horizontally
-			// and items can scroll in both directions, paiting is
-			// done separtely for the column header and the items.
-
-			Rectangle srcRect = this.ClientRectangle;
-			Rectangle dstRect = this.ClientRectangle;
-
-			// set the visible starting point
-			if (scrollable) {
-				srcRect.X += h_marker;
-				srcRect.Y += v_marker;
-
-				if (h_scroll.Visible) {
-					srcRect.Height -= h_scroll.Height;
-					dstRect.Height -= h_scroll.Height;
-				}
-
-				if (v_scroll.Visible) {
-					srcRect.Width -= v_scroll.Width;
-					dstRect.Width -= v_scroll.Width;
-				}
-			}
-
-			// We paint the column headers always at the top, in case
-			// of vertical scrolling. Therefore, we advance the painting
-			// by the amount equal to the column height.
-                        /*
-			if (this.view == View.Details &&
-			    this.Columns.Count > 0 &&
-			    this.header_style != ColumnHeaderStyle.None &&
-			    v_marker > 0 ) {
-
-				int col_ht = this.Columns [0].Ht;
-
-				if (this.draw_headers) {
-					this.draw_headers = false;
-					// Move the source rect by the amount of horizontal
-					// scrolling done so far.
-					Rectangle headerSrc = new Rectangle (h_marker, 0,
-									     srcRect.Width, col_ht);
-					// dest rect is always stable at 0,0
-					Rectangle headerDst = new Rectangle (0, 0, srcRect.Width, col_ht);
-					pe.Graphics.DrawImage (this.ImageBuffer, headerDst,
-							       headerSrc, GraphicsUnit.Pixel);
-				}
-
-				dstRect.Y += col_ht;
-				srcRect.Y += col_ht;
-			}
-                        */
-                        
-			// Draw the border of the list view
-			// The border is painted here separately, because
-			// our imagebuffer might be scrollable
-			ThemeEngine.Current.CPDrawBorderStyle (pe.Graphics,
-							       this.ClientRectangle,
-							       this.BorderStyle);
-
+					
 			// Raise the Paint event
 			if (Paint != null)
 				Paint (this, pe);
@@ -1267,7 +1306,18 @@ namespace System.Windows.Forms
 
 		private void ListView_SizeChanged (object sender, EventArgs e)
 		{
-			CalculateScrollBars ();
+			CalculateListView (alignment);
+		}
+		
+		private void SetFocusedItem (ListViewItem item)
+		{
+			if (focused_item != null)
+				focused_item.Focused = false;
+			
+			if (item != null)
+				item.Focused = true;
+				
+			focused_item = item;
 		}
 
 		private void HorizontalScroller (object sender, EventArgs e)
@@ -1275,10 +1325,17 @@ namespace System.Windows.Forms
 			// Avoid unnecessary flickering, when button is
 			// kept pressed at the end
 			if (h_marker != h_scroll.Value) {
+				
+				int pixels =  h_marker - h_scroll.Value;
+				Rectangle area = client_area;
+				
+				if (View == View.Details && Columns.Count > 0) {
+					area.Y += Columns[0].Ht;
+					area.Height -= Columns[0].Ht;
+				}
+				
 				h_marker = h_scroll.Value;
-				// draw the headers again
-				this.draw_headers = true;
-				this.Refresh ();
+				XplatUI.ScrollWindow (Handle, area, pixels, 0, false);
 			}
 		}
 
@@ -1287,8 +1344,16 @@ namespace System.Windows.Forms
 			// Avoid unnecessary flickering, when button is
 			// kept pressed at the end
 			if (v_marker != v_scroll.Value) {
+				int pixels =  v_marker - v_scroll.Value;
+				Rectangle area = client_area;
+				
+				if (View == View.Details && Columns.Count > 0) {
+					area.Y += Columns[0].Ht;
+					area.Height -= Columns[0].Ht;
+				}
+				
 				v_marker = v_scroll.Value;
-				this.Refresh ();
+				XplatUI.ScrollWindow (Handle, area, 0, pixels, false);
 			}
 		}
 		#endregion	// Internal Methods Properties
@@ -1300,12 +1365,36 @@ namespace System.Windows.Forms
 		}
 
 		protected override void Dispose (bool disposing)
-		{
+		{			
+			if (disposing) {			
+				h_scroll.Dispose ();
+				v_scroll.Dispose ();
+				
+				large_image_list = null;
+				small_image_list = null;
+				state_image_list = null;
+			}
+			
 			base.Dispose (disposing);
 		}
 
 		protected override bool IsInputKey (Keys keyData)
 		{
+			switch (keyData) {
+			case Keys.Up:
+			case Keys.Down:
+			case Keys.PageUp:
+			case Keys.PageDown:
+			case Keys.Right:
+			case Keys.Left:
+			case Keys.End:
+			case Keys.Home:				
+				return true;
+
+			default:
+				break;
+			}
+			
 			return base.IsInputKey (keyData);
 		}
 
@@ -1341,8 +1430,10 @@ namespace System.Windows.Forms
 		protected override void OnHandleCreated (EventArgs e)
 		{
 			base.OnHandleCreated (e);
-			this.Controls.Add (this.v_scroll);
-			this.Controls.Add (this.h_scroll);			
+			SuspendLayout ();
+			Controls.AddImplicit (this.v_scroll);
+			Controls.AddImplicit (this.h_scroll);
+			ResumeLayout ();
 		}
 
 		protected override void OnHandleDestroyed (EventArgs e)
@@ -1420,8 +1511,7 @@ namespace System.Windows.Forms
 		public void Clear ()
 		{
 			columns.Clear ();
-			items.Clear ();
-			this.Redraw (true);
+			items.Clear ();	// Redraw (true) called here			
 		}
 
 		public void EndUpdate ()
@@ -1439,11 +1529,11 @@ namespace System.Windows.Forms
 				return;
 
 			// dimensions of visible area
-			int view_wd = this.Width - (this.v_scroll.Visible ? this.v_scroll.Width : 0);
-			int view_ht = this.Height - (this.h_scroll.Visible ? this.h_scroll.Height : 0);
+			int view_wd = client_area.Width;
+			int view_ht = client_area.Height;
 			// visible area is decided by the h_marker and v_marker
 			Rectangle view_rect = new Rectangle (h_marker, v_marker, view_wd, view_ht);
-
+			
 			// an item's bounding rect
 			Rectangle rect = this.items [index].EntireRect;
 
@@ -1546,15 +1636,15 @@ namespace System.Windows.Forms
 			}
 
 			bool ICollection.IsSynchronized {
-				get { return list.IsSynchronized; }
+				get { return false; }
 			}
 
 			object ICollection.SyncRoot {
-				get { return list.SyncRoot; }
+				get { return this; }
 			}
 
 			bool IList.IsFixedSize {
-				get { return list.IsFixedSize; }
+				get { return true; }
 			}
 
 			object IList.this [int index] {
@@ -1658,11 +1748,11 @@ namespace System.Windows.Forms
 			}
 
 			object ICollection.SyncRoot {
-				get { return list.SyncRoot; }
+				get { return this; }
 			}
 
 			bool IList.IsFixedSize {
-				get { return list.IsFixedSize; }
+				get { return true; }
 			}
 
 			object IList.this [int index] {
@@ -1762,11 +1852,11 @@ namespace System.Windows.Forms
 			}
 
 			bool ICollection.IsSynchronized {
-				get { return list.IsSynchronized; }
+				get { return true; }
 			}
 
 			object ICollection.SyncRoot {
-				get { return list.SyncRoot; }
+				get { return this; }
 			}
 
 			bool IList.IsFixedSize {
@@ -1961,11 +2051,11 @@ namespace System.Windows.Forms
 			}
 
 			bool ICollection.IsSynchronized {
-				get { return list.IsSynchronized; }
+				get { return true; }
 			}
 
 			object ICollection.SyncRoot {
-				get { return list.SyncRoot; }
+				get { return this; }
 			}
 
 			bool IList.IsFixedSize {
@@ -2015,6 +2105,10 @@ namespace System.Windows.Forms
 			public void AddRange (ListViewItem [] values)
 			{
 				list.Clear ();
+				owner.SelectedItems.list.Clear ();
+				owner.SelectedIndices.list.Clear ();
+				owner.CheckedItems.list.Clear ();
+				owner.CheckedIndices.list.Clear ();
 
 				foreach (ListViewItem item in values) {
 					item.owner = this.owner;
@@ -2029,7 +2123,14 @@ namespace System.Windows.Forms
 
 			public virtual void Clear ()
 			{
+				owner.SetFocusedItem (null);
+				owner.h_scroll.Value = owner.v_scroll.Value = 0;
 				list.Clear ();
+				owner.SelectedItems.list.Clear ();
+				owner.SelectedIndices.list.Clear ();
+				owner.CheckedItems.list.Clear ();
+				owner.CheckedIndices.list.Clear ();
+				owner.Redraw (true);
 			}
 
 			public bool Contains (ListViewItem item)
@@ -2087,10 +2188,7 @@ namespace System.Windows.Forms
 
 			void IList.Remove (object item)
 			{
-				if (list.Contains (item)) {
-					list.Remove (item);
-					owner.Redraw (true);
-				}
+				Remove ((ListViewItem) item);
 			}
 
 			public int IndexOf (ListViewItem item)
@@ -2126,10 +2224,15 @@ namespace System.Windows.Forms
 
 			public virtual void Remove (ListViewItem item)
 			{
-				if (list.Contains (item)) {
-					list.Remove (item);
-					owner.Redraw (true);
-				}
+				if (!list.Contains (item))
+					return;
+	 				
+				owner.SelectedItems.list.Remove (item);
+				owner.SelectedIndices.list.Remove (item.Index);
+				owner.CheckedItems.list.Remove (item);
+				owner.CheckedIndices.list.Remove (item.Index);
+				list.Remove (item);
+				owner.Redraw (true);				
 			}
 
 			public virtual void RemoveAt (int index)
@@ -2138,6 +2241,10 @@ namespace System.Windows.Forms
 					throw new ArgumentOutOfRangeException ("Index out of range.");
 
 				list.RemoveAt (index);
+				owner.SelectedItems.list.RemoveAt (index);
+				owner.SelectedIndices.list.RemoveAt (index);
+				owner.CheckedItems.list.RemoveAt (index);
+				owner.CheckedIndices.list.RemoveAt (index);
 				owner.Redraw (false);
 			}
 			#endregion	// Public Methods
@@ -2180,11 +2287,11 @@ namespace System.Windows.Forms
 			}
 
 			object ICollection.SyncRoot {
-				get { return list.SyncRoot; }
+				get { return this; }
 			}
 
 			bool IList.IsFixedSize {
-				get { return list.IsFixedSize; }
+				get { return true; }
 			}
 
 			object IList.this [int index] {
@@ -2288,11 +2395,11 @@ namespace System.Windows.Forms
 			}
 
 			object ICollection.SyncRoot {
-				get { return list.SyncRoot; }
+				get { return this; }
 			}
 
 			bool IList.IsFixedSize {
-				get { return list.IsFixedSize; }
+				get { return true; }
 			}
 
 			object IList.this [int index] {
@@ -2309,6 +2416,9 @@ namespace System.Windows.Forms
 					((ListViewItem) list [i]).selected = false;
 
 				list.Clear ();
+				
+				if (owner != null)
+					owner.Invalidate ();
 			}
 
 			public bool Contains (ListViewItem item)
