@@ -36,16 +36,17 @@ using System.Runtime.InteropServices;
 
 namespace System.Windows.Forms
 {
-	[Designer ("Microsoft.VisualStudio.Windows.Forms.MenuDesigner, Microsoft.VisualStudio, Version=1.0.5000.0, Culture=neutral, PublicKeyToken=b03f5f7f11d50a3a", "System.ComponentModel.Design.IDesigner")]
+	[Designer ("Microsoft.VisualStudio.Windows.Forms.MenuDesigner, " + Consts.AssemblyMicrosoft_VisualStudio, "System.ComponentModel.Design.IDesigner")]
 	[ToolboxItemFilter("System.Windows.Forms", ToolboxItemFilterType.Allow)]
 	[ListBindable(false)]
 	public abstract class Menu : Component
 	{
 		internal MenuItemCollection menu_items;
 		internal IntPtr menu_handle = IntPtr.Zero;
-		internal bool is_dirty = true;
-		internal bool creating = false;
 		internal Menu parent_menu = null;
+		System.Drawing.Rectangle rect;
+		internal Control Wnd;
+		internal MenuTracker tracker;
 
 		public const int FindHandle = 0;
 		public const int FindShortcut = 1;
@@ -64,24 +65,13 @@ namespace System.Windows.Forms
 		[EditorBrowsableAttribute(EditorBrowsableState.Advanced)]
 		[DesignerSerializationVisibilityAttribute(DesignerSerializationVisibility.Hidden)]
 		public IntPtr Handle {
-			get {
-				if (IsDirty && creating == false) {					
-					Dispose (true);
-				}
-
-				if (menu_handle == IntPtr.Zero) {
-					MenuChanged ();
-				}
-
-				return menu_handle;
-			}
+			get { return menu_handle; }
 		}
 
-		internal virtual void MenuChanged ()
+		internal virtual void OnMenuChanged (EventArgs e)
 		{
-			menu_handle = CreateMenuHandle ();
-			CreateItems ();
-			IsDirty = false;
+			if (MenuChanged != null)
+				MenuChanged (this, e);
 		}
 
 		[BrowsableAttribute(false)]
@@ -114,11 +104,51 @@ namespace System.Windows.Forms
 
 		#region Private Properties
 
-		internal bool IsDirty {
-			get { return is_dirty; }
-			set { is_dirty = value; }
+		internal System.Drawing.Rectangle Rect {
+			get {
+				return rect;
+			}
 		}
 
+		internal MenuItem SelectedItem  {
+			get {
+				foreach (MenuItem item in MenuItems)
+					if ((item.Status & DrawItemState.Selected) == DrawItemState.Selected)
+						return item;
+
+				return null;
+			}
+		}
+
+		internal int Height {
+			get { return rect.Height; }
+			set { rect.Height = value; }
+		}
+
+		internal int Width {
+			get { return rect.Width; }
+			set { rect.Width = value; }
+		}
+
+		internal int X {
+			get { return rect.X; }
+			set { rect.X = value; }
+		}
+
+		internal int Y {
+			get { return rect.Y; }
+			set { rect.Y = value; }
+		}
+
+		internal MenuTracker Tracker {
+			get {
+				Menu top = this;
+				while (top.parent_menu != null)
+					top = top.parent_menu;
+
+				return top.tracker;
+			}
+		}
 		#endregion Private Properties
 
 		#region Public Methods
@@ -130,22 +160,13 @@ namespace System.Windows.Forms
 			menu_items = new MenuItemCollection (this);
 
 			for (int i = 0; i < menuSrc.MenuItems.Count ; i++)
-				menu_items.Add (menuSrc.MenuItems [i]);
-		}
-
-		protected virtual IntPtr CreateMenuHandle ()
-		{
-			IntPtr menu;
-
-			menu = MenuAPI.CreatePopupMenu (this);
-			return menu;
+				menu_items.Add (menuSrc.MenuItems [i].CloneMenu ());
 		}
 
 		protected override void Dispose (bool disposing)
 		{		
 			if (disposing) {
 				if (menu_handle != IntPtr.Zero) {
-					MenuAPI.DestroyMenu (menu_handle);
 					menu_handle = IntPtr.Zero;
 				}
 			}
@@ -218,7 +239,7 @@ namespace System.Windows.Forms
 					case MenuMerge.MergeItems:
 					{
 						int pos = FindMergePosition (menuSrc.MenuItems[i].MergeOrder - 1);						
-						MenuItems.Add (pos, menuSrc.MenuItems[i].CloneMenu ());
+						MenuItems.Insert (pos, menuSrc.MenuItems[i].CloneMenu ());
 						
 						break;
 					}
@@ -229,9 +250,11 @@ namespace System.Windows.Forms
 			}		
 		}
 
-		protected internal virtual bool ProcessCmdKey (ref Message msg, Keys keyData)
+		internal bool ProcessCmdKey (ref Message msg, Keys keyData)
 		{
-			return false;
+			if (tracker == null)
+				return false;
+			return tracker.ProcessKeys (ref msg, keyData);
 		}
 
 		public override string ToString ()
@@ -241,19 +264,7 @@ namespace System.Windows.Forms
 
 		#endregion Public Methods
 
-		#region Private Methods
-
-		internal void CreateItems ()
-		{
-			creating = true;
-
-			for (int i = 0; i < menu_items.Count; i++)
-				menu_items[i].Create ();
-
-			creating = false;
-		}
-
-		#endregion Private Methods
+		internal event EventHandler MenuChanged;
 
 		[ListBindable(false)]
 		public class MenuItemCollection : IList, ICollection, IEnumerable
@@ -288,7 +299,7 @@ namespace System.Windows.Forms
 				get { return false;}
 			}
 
-			public MenuItem this [int index] {
+			public virtual MenuItem this [int index] {
 				get {
 					if (index < 0 || index >= Count)
 						throw new ArgumentOutOfRangeException ("Index of out range");
@@ -308,11 +319,16 @@ namespace System.Windows.Forms
 
 			public virtual int Add (MenuItem mi)
 			{
-				mi.parent_menu = owner;
-				mi.Index = items.Count;
+				if (mi.Parent != null)
+					mi.Parent.MenuItems.Remove (mi);
+				
 				items.Add (mi);
-
-				owner.IsDirty = true;
+				mi.Index = items.Count - 1;
+				UpdateItem (mi);
+				
+				owner.OnMenuChanged (EventArgs.Empty);
+				if (owner.parent_menu != null)
+					owner.parent_menu.OnMenuChanged (EventArgs.Empty);
 				return items.Count - 1;
 			}
 
@@ -339,12 +355,30 @@ namespace System.Windows.Forms
 					new_items.Add (items[i]);
 
 				items = new_items;
-				UpdateItemsIndices ();
-				owner.IsDirty = true;
+				UpdateItemsIndices ();				
+				UpdateItem (mi);
 
 				return index;
 			}
 
+			private void UpdateItem (MenuItem mi)
+			{
+				mi.parent_menu = owner;
+				owner.OnMenuChanged (EventArgs.Empty);
+				if (owner.parent_menu != null)
+					owner.parent_menu.OnMenuChanged (EventArgs.Empty);
+				if (owner.Tracker != null)
+					owner.Tracker.AddShortcuts (mi);
+			}
+
+			internal void Insert (int index, MenuItem mi)
+			{
+				if (index < 0 || index > Count)
+					throw new ArgumentOutOfRangeException ("Index of out range");
+				UpdateItemsIndices ();
+				UpdateItem (mi);
+			}
+
 			public virtual MenuItem Add (string s, EventHandler e)
 			{
 				MenuItem item = new MenuItem (s, e);
@@ -372,8 +406,14 @@ namespace System.Windows.Forms
 
 			public virtual void Clear ()
 			{
+				MenuTracker tracker = owner.Tracker;
+				foreach (MenuItem item in items) {
+					if (tracker != null)
+						tracker.RemoveShortcuts (item);
+					item.parent_menu = null;
+				}
 				items.Clear ();
-				owner.IsDirty = true;
+				owner.OnMenuChanged (EventArgs.Empty);
 			}
 
 			public bool Contains (MenuItem value)
@@ -408,7 +448,7 @@ namespace System.Windows.Forms
 
 			void IList.Insert (int index, object value)
 			{
-				Add (index, (MenuItem) value);
+				Insert (index, (MenuItem) value);
 			}
 
 			void IList.Remove (object value)
@@ -431,10 +471,16 @@ namespace System.Windows.Forms
 				if (index < 0 || index >= Count)
 					throw new ArgumentOutOfRangeException ("Index of out range");
 
+				MenuItem item = (MenuItem) items [index];
+				MenuTracker tracker = owner.Tracker;
+				if (tracker != null)
+					tracker.RemoveShortcuts (item);
+				item.parent_menu = null;
+
 				items.RemoveAt (index);
 
 				UpdateItemsIndices ();
-				owner.IsDirty = true;
+				owner.OnMenuChanged (EventArgs.Empty);
 			}
 
 			#endregion Public Methods
