@@ -39,7 +39,7 @@ namespace System.Windows.Forms
 	[DesignTimeVisible(false)]
 	[ToolboxItem(false)]
 	public class MenuItem : Menu
-	{		
+	{
 		internal bool separator;
 		internal bool break_;
 		internal bool bar_break;
@@ -52,6 +52,8 @@ namespace System.Windows.Forms
 		private bool showshortcut;
 		private int index;
 		private bool mdilist;
+		private Hashtable mdilist_items;
+		private MdiClient mdicontainer;
 		private bool defaut_item;
 		private bool visible;
 		private bool ownerdraw;
@@ -62,6 +64,7 @@ namespace System.Windows.Forms
 		private bool menubar;
 		private DrawItemState status;
 		private MenuMerge mergetype;
+		internal Rectangle bounds;
 		
 		public MenuItem (): base (null)
 		{	
@@ -111,6 +114,7 @@ namespace System.Windows.Forms
 
 		private void CommonConstructor (string text)
 		{
+			defaut_item = false;
 			separator = false;
 			break_ = false;
 			bar_break = false;
@@ -178,7 +182,11 @@ namespace System.Windows.Forms
 		[Browsable(false)]
 		public int Index {
 			get { return index; }
-			set { index = value; }
+			set { 
+				if (Parent != null && Parent.MenuItems != null && (value < 0 || value >= Parent.MenuItems.Count))
+					throw new ArgumentException ("'" + value + "' is not a valid value for 'value'");
+				index = value; 
+			}
 		}
 
 		[Browsable(false)]
@@ -189,7 +197,20 @@ namespace System.Windows.Forms
 		[DefaultValue(false)]
 		public bool MdiList {
 			get { return mdilist; }
-			set { mdilist = value; }
+			set {
+				if (mdilist == value)
+					return;
+				mdilist = value;
+
+				if (mdilist || mdilist_items == null)
+					return;
+
+				foreach (MenuItem item in mdilist_items.Keys)
+					MenuItems.Remove (item);
+				mdilist_items.Clear ();
+				mdilist_items = null;
+				
+			}
 		}
 
 		protected int MenuID {
@@ -284,8 +305,8 @@ namespace System.Windows.Forms
 						mi.Visible = value;
 				}
 
-				parent_menu.IsDirty = true;
-				parent_menu.creating = false;
+				if (parent_menu != null)
+					parent_menu.OnMenuChanged (EventArgs.Empty);
 			}
 		}
 
@@ -293,6 +314,11 @@ namespace System.Windows.Forms
 
 		#region Private Properties
 
+		internal new int Height {
+			get { return bounds.Height; }
+			set { bounds.Height = value; }
+		}
+
 		internal bool IsPopup {
 			get {
 				if (menu_items.Count > 0)
@@ -332,11 +358,26 @@ namespace System.Windows.Forms
 			set { status = value; }
 		}
 		
+		internal new int Width {
+			get { return bounds.Width; }
+			set { bounds.Width = value; }
+		}
+
+		internal new int X {
+			get { return bounds.X; }
+			set { bounds.X = value; }
+		}
+
 		internal int XTab {
 			get { return xtab; }
 			set { xtab = value; }
 		}
 
+		internal new int Y {
+			get { return bounds.Y; }
+			set { bounds.Y = value; }
+		}
+
 		#endregion Private Properties
 
 		#region Public Methods
@@ -453,21 +494,32 @@ namespace System.Windows.Forms
 
 		#region Private Methods
 
-		internal void Create ()
-		{
-			IntPtr hSubMenu = IntPtr.Zero;
-
-			menuid = index = MenuAPI.InsertMenuItem (Parent.Handle, -1, true, this, ref hSubMenu);
-			IsDirty = false;
-
-			if (IsPopup) {
-				menu_handle = hSubMenu;
-				CreateItems ();
-			}
-		}
-		
 		internal void PerformDrawItem (DrawItemEventArgs e)
 		{
+			if (mdilist && mdilist_items == null) {
+				do {
+					// Add the mdilist for the first time
+					mdilist_items = new Hashtable ();
+
+					MainMenu main = GetMainMenu ();
+					if (main == null || main.GetForm () == null)
+						break;
+
+					Form form = main.GetForm ();
+					mdicontainer = form.MdiContainer;
+					if (mdicontainer == null)
+						break;
+
+					foreach (Form mdichild in mdicontainer.Controls) {
+						MenuItem item = new MenuItem (mdichild.Text);
+						item.Click += new EventHandler (MdiWindowClickHandler);
+						MenuItems.Add (item);
+						mdilist_items.Add (item, form);
+					}
+
+				} while (false);
+			}
+
 			OnDrawItem (e);
 		}
 		
@@ -601,6 +653,18 @@ namespace System.Windows.Forms
 			return "";
 		}
 
+		private void MdiWindowClickHandler (object sender, EventArgs e)
+		{
+			Form mdichild = (Form) mdilist_items [SelectedItem];
+
+			// people could add weird items to the Window menu
+			// so we can't assume its just us
+			if (mdichild == null)
+				return;
+
+			mdicontainer.ActivateChild (mdichild);
+		}
+
 		#endregion Private Methods
 
 	}
