@@ -35,7 +35,7 @@ using System.Windows.Forms.Design;
 
 namespace System.Windows.Forms.PropertyGridInternal 
 {
-	public class PropertyGridView : System.Windows.Forms.ScrollableControl, IWindowsFormsEditorService
+	internal class PropertyGridView : System.Windows.Forms.ScrollableControl, IWindowsFormsEditorService
 	{
 
 		#region Private Members
@@ -49,10 +49,11 @@ namespace System.Windows.Forms.PropertyGridInternal
 		private int splitter_location;
 		private int open_grid_item_count = -1;
 		private int skipped_grid_items;
-		private Form dropdown_form;
+		private PropertyGridDropDown dropdown_form;
+		private bool dropdown_form_showing;
 		private Form dialog_form;
-		private ListBox listBox;
 		private VScrollBar vbar;
+		private StringFormat stringFormat;
 		#endregion
 
 		#region Contructors
@@ -63,28 +64,31 @@ namespace System.Windows.Forms.PropertyGridInternal
 			property_grid.SelectedGridItemChanged+=new SelectedGridItemChangedEventHandler(HandleSelectedGridItemChanged);
 			property_grid.PropertyValueChanged+=new PropertyValueChangedEventHandler(HandlePropertyValueChanged);
 
+			stringFormat = new StringFormat();
+			stringFormat.FormatFlags = StringFormatFlags.NoWrap;
+			stringFormat.Trimming = StringTrimming.None;
+
 			this.BackColor = Color.Beige;
 			grid_textbox = new PropertyGridTextBox();
 			grid_textbox.DropDownButtonClicked +=new EventHandler(DropDownButtonClicked);
 			grid_textbox.DialogButtonClicked +=new EventHandler(DialogButtonClicked);
 
-			dropdown_form = new Form();
+			
+			dropdown_form = new PropertyGridDropDown();
 			dropdown_form.FormBorderStyle = FormBorderStyle.None;
-			dropdown_form.Deactivate +=new EventHandler(dropdown_form_Deactivate);
+			dropdown_form.ShowInTaskbar = false;
 			
 			dialog_form = new Form();
 			//dialog_form.FormBorderStyle = FormBorderStyle.None;
 
-			listBox = new ListBox();
-			listBox.Dock = DockStyle.Fill;
-			listBox.SelectedIndexChanged +=new EventHandler(listBox_SelectedIndexChanged);
-			dropdown_form.Controls.Add(listBox);
+			
 
-			grid_textbox.Visible = true;
+			grid_textbox.Visible = false;
 			grid_textbox.Font = this.Font;//new Font(this.Font,FontStyle.Bold);
 			grid_textbox.BackColor = this.BackColor;
 			// Not working at all, used to??
 			grid_textbox.Validating += new CancelEventHandler(TextBoxValidating);
+			this.Controls.Add(grid_textbox);
 
 			vbar = new VScrollBar();
 			vbar.Visible = false;
@@ -113,6 +117,17 @@ namespace System.Windows.Forms.PropertyGridInternal
 		
 		protected override void OnPaint(PaintEventArgs e)
 		{
+			if (property_grid.SelectedGridItem != null && property_grid.SelectedGridItem.GridItemType == GridItemType.Property) 
+			{
+				grid_textbox.Visible = true;
+				if (!grid_textbox.Focused)
+					grid_textbox.Focus();
+			}
+			else 
+			{
+				grid_textbox.Visible = false;
+			}
+
 			// Decide if we need a scrollbar
 			open_grid_item_count = 0;
 
@@ -128,20 +143,11 @@ namespace System.Windows.Forms.PropertyGridInternal
 			DrawGridItems(property_grid.grid_items, e, 1, ref yLoc);
 
 			DrawGrid(e, yLoc);
+			e.Graphics.DrawRectangle(SystemPens.ControlDark, 0,0,Width-1,Height-1 );
 			
 			
 
 
-			if (property_grid.SelectedGridItem != null && property_grid.SelectedGridItem.GridItemType == GridItemType.Property) 
-			{
-				this.Controls.Add(grid_textbox);
-				if (!grid_textbox.Focused)
-					grid_textbox.Focus();
-			}
-			else 
-			{
-				this.Controls.Remove(grid_textbox);
-			}
 			UpdateScrollBar();
 			
 			base.OnPaint(e);
@@ -149,41 +155,57 @@ namespace System.Windows.Forms.PropertyGridInternal
 
 		protected override void OnMouseMove (MouseEventArgs e) 
 		{
-			if (resizing_grid) {
-				splitter_location = Math.Max(e.X,V_INDENT);
+
+			if (resizing_grid) 
+			{
+				splitter_location = Math.Max(e.X,2*V_INDENT);
 				Refresh();
 			}
+			if (e.X > splitter_location - RESIZE_WIDTH && e.X < splitter_location + RESIZE_WIDTH) 
+				this.Cursor = Cursors.SizeWE;
+			else
+				this.Cursor = Cursors.Default;
 			base.OnMouseMove (e);
 		}
 
-		private GridItem GetSelectedGridItem (GridItemCollection grid_items, int y) 
+		private GridItem GetSelectedGridItem (GridItemCollection grid_items, int y, ref int current) 
 		{
-			foreach (GridItem child_grid_item in grid_items) {
-				if (y > child_grid_item.Top && y < child_grid_item.Top + ROW_HEIGHT) {
+			foreach (GridItem child_grid_item in grid_items) 
+			{
+				if (y > current && y < current + ROW_HEIGHT) 
+				{
 					return child_grid_item;
 				}
-				GridItem foundItem = GetSelectedGridItem(child_grid_item.GridItems, y);
-				if (foundItem != null)
-					return foundItem;
+				current += ROW_HEIGHT;
+				if (child_grid_item.Expanded)
+				{
+					GridItem foundItem = GetSelectedGridItem(child_grid_item.GridItems, y, ref current);
+					if (foundItem != null)
+						return foundItem;
+				}
 			}
 			return null;
 		}
 
 		protected override void OnMouseDown (MouseEventArgs e) 
 		{
-			if (e.X > splitter_location - RESIZE_WIDTH && e.X < splitter_location + RESIZE_WIDTH) {
+			if (e.X > splitter_location - RESIZE_WIDTH && e.X < splitter_location + RESIZE_WIDTH) 
+			{
 				resizing_grid = true;
 			}
-			else {
-				GridItem foundItem = GetSelectedGridItem(property_grid.grid_items, e.Y);
+			else 
+			{
+				int offset = -vbar.Value*ROW_HEIGHT;
+				GridItem foundItem = GetSelectedGridItem(property_grid.grid_items, e.Y, ref offset);
 				
 				if (foundItem != null)
 				{
 					if (foundItem.Expandable) 
 					{
-						if (foundItem.PlusMinusBounds.Contains(e.X,e.Y))
+						if (e.X >=3 && e.X <= 11 && (e.Y % ROW_HEIGHT >= 3 && e.Y % ROW_HEIGHT <= 11))
 						{
 							foundItem.Expanded = !foundItem.Expanded;
+							Invalidate();
 						}
 					}
 					this.property_grid.SelectedGridItem = foundItem;
@@ -253,46 +275,37 @@ namespace System.Windows.Forms.PropertyGridInternal
 			{
 				DrawGridItem (grid_item, pevent, depth, ref yLoc);
 				if (grid_item.Expanded)
-					DrawGridItems(grid_item.GridItems, pevent, depth, ref yLoc);
+					DrawGridItems(grid_item.GridItems, pevent, (grid_item.GridItemType == GridItemType.Category) ? depth : depth+1, ref yLoc);
 			}
 		}
 
-		private void DrawGridItem (GridItem grid_item, PaintEventArgs pevent, int depth, ref int yLoc) 
+		private void DrawGridItemLabel(GridItem grid_item, PaintEventArgs pevent, Rectangle rect)
 		{
-			// left column
-			pevent.Graphics.FillRectangle (ThemeEngine.Current.ResPool.GetSolidBrush (property_grid.LineColor), 0,yLoc,V_INDENT, ROW_HEIGHT);
+			int x = rect.X+1;
+			if (grid_item.Parent != null && grid_item.Parent.GridItemType != GridItemType.Category)
+				x += V_INDENT;
 
-			if (grid_item.Expandable) 
-			{
-				grid_item.PlusMinusBounds = DrawPlusMinus(pevent, 3, yLoc+3, grid_item.Expanded, grid_item.GridItemType == GridItemType.Category);
-			}
-			
+			Font font = this.Font;
+			Brush brush = SystemBrushes.WindowText;
 			if (grid_item.GridItemType == GridItemType.Category)
 			{
-				pevent.Graphics.FillRectangle (ThemeEngine.Current.ResPool.GetSolidBrush (property_grid.LineColor), depth*V_INDENT,yLoc,ClientRectangle.Width-(depth*V_INDENT), ROW_HEIGHT);
+				font = new Font(font, FontStyle.Bold);
+				brush = SystemBrushes.ControlDark;
 			}
-			
+
 			if (grid_item == property_grid.SelectedGridItem && grid_item.GridItemType != GridItemType.Category)
 			{
-				pevent.Graphics.FillRectangle (ThemeEngine.Current.ResPool.GetSolidBrush (Color.Blue), new Rectangle(V_INDENT,yLoc, splitter_location-V_INDENT, ROW_HEIGHT));
+				pevent.Graphics.FillRectangle (SystemBrushes.Highlight, rect);
 				// Label
-				pevent.Graphics.DrawString(grid_item.Label,this.Font,SystemBrushes.Window,new RectangleF(1+depth*V_INDENT,yLoc+2, splitter_location-(5+depth*V_INDENT), ROW_HEIGHT-4));
-				
-				grid_textbox.Location = new Point(splitter_location+1, yLoc);
-				grid_textbox.Size = new Size(ClientRectangle.Width-splitter_location,ROW_HEIGHT);
-			}
-			else
-			{
-				Font font = this.Font;
-				Brush brush = SystemBrushes.WindowText;
-				if (grid_item.GridItemType == GridItemType.Category)
-				{
-					font = new Font(font, FontStyle.Bold);
-					brush = SystemBrushes.ControlDark;
-				}
-				// Label
-				pevent.Graphics.DrawString(grid_item.Label,font,brush,new RectangleF(1+depth*V_INDENT,yLoc+2, splitter_location-(5+depth*V_INDENT), ROW_HEIGHT-4));
+				brush = SystemBrushes.HighlightText;
 			}
+
+			
+			pevent.Graphics.DrawString(grid_item.Label,font,brush,new Rectangle(x, rect.Y + 2,x-rect.X+rect.Width-2,rect.Height-2),stringFormat);
+		}
+
+		private void DrawGridItemValue(GridItem grid_item, PaintEventArgs pevent, Rectangle rect)
+		{
 			// Value
 			if (grid_item.PropertyDescriptor != null)
 			{
@@ -308,9 +321,9 @@ namespace System.Windows.Forms.PropertyGridInternal
 
 				if (grid_item == property_grid.SelectedGridItem)
 				{
+					grid_textbox.ReadOnly = false;
 					grid_textbox.DropDownButtonVisible = false;
 					grid_textbox.DialogButtonVisible = false;
-					listBox.Items.Clear();
 					if (editor != null) 
 					{
 						UITypeEditorEditStyle style = editor.GetEditStyle();
@@ -327,12 +340,25 @@ namespace System.Windows.Forms.PropertyGridInternal
 					}
 					else 
 					{
-						if (grid_item.PropertyDescriptor.Converter.GetStandardValuesSupported()) 
+						try
+						{
+							if (grid_item.PropertyDescriptor.Converter != null)
+							{
+								if (grid_item.PropertyDescriptor.Converter.GetStandardValuesSupported()) 
+								{
+							
+									grid_textbox.DropDownButtonVisible = true;
+									grid_textbox.ReadOnly = true;
+								}
+							}
+							else
+							{
+								System.Console.WriteLine("Converter not available for type {0}",grid_item.PropertyDescriptor.PropertyType);
+							}
+						
+						}
+						catch (Exception ex)
 						{
-							foreach (object obj in grid_item.PropertyDescriptor.Converter.GetStandardValues())
-								listBox.Items.Add(obj);
-							grid_textbox.DropDownButtonVisible = true;
-							grid_textbox.ReadOnly = true;
 						}
 					}
 				}
@@ -342,20 +368,70 @@ namespace System.Windows.Forms.PropertyGridInternal
 				int xLoc = splitter_location+1;
 				if (paintsValue)
 				{
-					pevent.Graphics.DrawRectangle(ThemeEngine.Current.ResPool.GetPen(Color.Black), splitter_location+2,yLoc+2, 20, ROW_HEIGHT-4);
+					pevent.Graphics.DrawRectangle(ThemeEngine.Current.ResPool.GetPen(Color.Black), splitter_location+2,rect.Y+2, 20, ROW_HEIGHT-4);
 					try
 					{
-						editor.PaintValue(grid_item.Value, pevent.Graphics, new Rectangle(splitter_location+3,yLoc+3, 19, ROW_HEIGHT-5));
+						editor.PaintValue(grid_item.Value, pevent.Graphics, new Rectangle(splitter_location+3,rect.Y+3, 19, ROW_HEIGHT-5));
 					}
 					catch (Exception ex)
 					{
+						System.Console.WriteLine(ex.Message);
+						System.Console.WriteLine("Paint Value failed for type {0}",grid_item.PropertyDescriptor.PropertyType);
 						// design time stuff is not playing nice
 					}
 					xLoc += 27;
 				}
 
-				string value = grid_item.PropertyDescriptor.Converter.ConvertToString(grid_item.Value);
-				pevent.Graphics.DrawString(value,this.Font,SystemBrushes.WindowText,new RectangleF(xLoc,yLoc+2, ClientRectangle.Width-(xLoc), ROW_HEIGHT-4));
+				Font font = this.Font;
+				try 
+				{
+					if (grid_item.PropertyDescriptor.Converter != null)
+					{
+						string value = grid_item.PropertyDescriptor.Converter.ConvertToString(grid_item.Value);
+						if (grid_item.PropertyDescriptor.CanResetValue(property_grid.SelectedObject))
+							font = new Font(font, FontStyle.Bold);
+				
+						pevent.Graphics.DrawString(value,font,SystemBrushes.WindowText,new RectangleF(xLoc,rect.Y+2, ClientRectangle.Width-(xLoc), ROW_HEIGHT),stringFormat);
+					}
+					else
+					{
+						System.Console.WriteLine("No converter for type {0}",grid_item.PropertyDescriptor.PropertyType);
+					}
+
+				}
+				catch (Exception e)
+				{
+				}
+				if (grid_item == property_grid.SelectedGridItem && grid_item.GridItemType != GridItemType.Category)
+				{
+					grid_textbox.SetBounds(xLoc, rect.Top, ClientRectangle.Width-xLoc - (vbar.Visible ? vbar.Width: 0),ROW_HEIGHT);
+				}
+			}
+		}
+
+		private void DrawGridItem (GridItem grid_item, PaintEventArgs pevent, int depth, ref int yLoc) 
+		{
+			if (yLoc > -ROW_HEIGHT && yLoc < ClientRectangle.Height)
+			{
+			
+				// left column
+				pevent.Graphics.FillRectangle (ThemeEngine.Current.ResPool.GetSolidBrush (property_grid.LineColor), 0,yLoc,V_INDENT, ROW_HEIGHT);
+
+				if (grid_item.Expandable) 
+				{
+					grid_item.PlusMinusBounds = DrawPlusMinus(pevent, 3, yLoc+3, grid_item.Expanded, grid_item.GridItemType == GridItemType.Category);
+				}
+			
+				if (grid_item.GridItemType == GridItemType.Category)
+				{
+					pevent.Graphics.FillRectangle (ThemeEngine.Current.ResPool.GetSolidBrush (property_grid.LineColor), depth*V_INDENT,yLoc,ClientRectangle.Width-(depth*V_INDENT), ROW_HEIGHT);
+				}
+
+				DrawGridItemLabel(grid_item, pevent, new Rectangle(depth*V_INDENT,yLoc, splitter_location-depth*V_INDENT, ROW_HEIGHT));
+				DrawGridItemValue(grid_item, pevent, new Rectangle(splitter_location+2,yLoc, ClientRectangle.Width-splitter_location-2, ROW_HEIGHT));
+			
+				
+				
 			}
 			grid_item.Top = yLoc;
 			yLoc += ROW_HEIGHT;
@@ -384,13 +460,21 @@ namespace System.Windows.Forms.PropertyGridInternal
 
 		private void TextBoxValidating (object sender, CancelEventArgs e) 
 		{
-			if (this.property_grid.SelectedGridItem != null) {
+			if (this.property_grid.SelectedGridItem != null) 
+			{
 				PropertyDescriptor desc = property_grid.SelectedGridItem.PropertyDescriptor;
-				if (desc != null) {
+				if (desc != null) 
+				{
 					try 
 					{
-						
-						SetPropertyValue(desc.Converter.ConvertFromString(grid_textbox.Text));
+						if (desc.Converter != null)
+						{
+							SetPropertyValue(desc.Converter.ConvertFromString(grid_textbox.Text));
+						}
+						else
+						{
+							System.Console.WriteLine("No converter for type {0}",desc.PropertyType);
+						}
 					}
 					catch (Exception ex)
 					{
@@ -407,18 +491,23 @@ namespace System.Windows.Forms.PropertyGridInternal
 
 		private void dropdown_form_Deactivate (object sender, EventArgs e) 
 		{
+			dropdown_form_showing = false;
 			dropdown_form.Hide();
+			//dropdown_form = new Form();
 		}
 
 		private void listBox_SelectedIndexChanged (object sender, EventArgs e) 
 		{
-			if (this.property_grid.SelectedGridItem != null) {
+			if (this.property_grid.SelectedGridItem != null) 
+			{
 				PropertyDescriptor desc = property_grid.SelectedGridItem.PropertyDescriptor;
-				if (desc != null) {
-					SetPropertyValue(listBox.SelectedItem);
+				if (desc != null) 
+				{
+					SetPropertyValue(((ListBox)sender).SelectedItem);
 				}
 			}
 			dropdown_form.Hide();
+			//dropdown_form = new Form();
 			Refresh();
 		}
 
@@ -436,8 +525,17 @@ namespace System.Windows.Forms.PropertyGridInternal
 
 		private void DropDownButtonClicked (object sender, EventArgs e) 
 		{
-			if (listBox.Items.Count > 0)
+			if (property_grid.SelectedGridItem.PropertyDescriptor.GetEditor(typeof(UITypeEditor)) == null)
 			{
+				//dropdown_form.FormBorderStyle = FormBorderStyle.None;
+				dropdown_form.Deactivate +=new EventHandler(dropdown_form_Deactivate);
+				ListBox listBox = new ListBox();
+				listBox.Dock = DockStyle.Fill;
+				listBox.SelectedIndexChanged +=new EventHandler(listBox_SelectedIndexChanged);
+				foreach (object obj in property_grid.SelectedGridItem.PropertyDescriptor.Converter.GetStandardValues())
+					listBox.Items.Add(obj);
+				dropdown_form.Controls.Clear();
+				dropdown_form.Controls.Add(listBox);
 				dropdown_form.Location = PointToScreen(new Point(grid_textbox.Location.X,grid_textbox.Location.Y+ROW_HEIGHT));
 				dropdown_form.Width = grid_textbox.Width;
 				dropdown_form.Show();
@@ -447,7 +545,7 @@ namespace System.Windows.Forms.PropertyGridInternal
 				UITypeEditor editor = (UITypeEditor)property_grid.SelectedGridItem.PropertyDescriptor.GetEditor(typeof(UITypeEditor));
 				System.ComponentModel.Design.ServiceContainer service_container = new System.ComponentModel.Design.ServiceContainer();
 				service_container.AddService(typeof(System.Windows.Forms.Design.IWindowsFormsEditorService), this);
-				editor.EditValue(null, service_container,property_grid.SelectedGridItem.Value);
+				SetPropertyValue(editor.EditValue(new ITypeDescriptorContextImpl(this.property_grid), service_container,property_grid.SelectedGridItem.Value));
 			}
 		}
 		
@@ -455,12 +553,12 @@ namespace System.Windows.Forms.PropertyGridInternal
 		{
 			//dialog_form.Location = PointToScreen(new Point(grid_textbox.Location.X,grid_textbox.Location.Y+ROW_HEIGHT));
 			//dropdown_form.Width = grid_textbox.Width;
-			dialog_form.ShowDialog(this);
+			dialog_form.Show();
 		}
 
 		private void HandleScroll(object sender, ScrollEventArgs e)
 		{
-			if (e.NewValue < 0)
+			if (e.NewValue <= 0)
 			{
 				e.NewValue = 0;
 				if (e.NewValue == vbar.Value)return;
@@ -475,11 +573,15 @@ namespace System.Windows.Forms.PropertyGridInternal
 			{
 				case ScrollEventType.SmallDecrement:
 					XplatUI.ScrollWindow(Handle, 0, ROW_HEIGHT, false);
-					Invalidate(new Rectangle(0,0,ClientRectangle.Width,ROW_HEIGHT));
+					grid_textbox.Top += ROW_HEIGHT;
+					Invalidate(ClientRectangle);
+					//Invalidate(new Rectangle(0,0,ClientRectangle.Width,ROW_HEIGHT));
 					break;
 				case ScrollEventType.SmallIncrement:
 					XplatUI.ScrollWindow(Handle, 0, -ROW_HEIGHT, false);
-					Invalidate(new Rectangle(0,ClientRectangle.Bottom-ROW_HEIGHT,ClientRectangle.Width,ROW_HEIGHT));
+					grid_textbox.Top -= ROW_HEIGHT;
+					Invalidate(ClientRectangle);
+					//Invalidate(new Rectangle(0,ClientRectangle.Bottom-ROW_HEIGHT,ClientRectangle.Width,ROW_HEIGHT));
 					break;
 				case ScrollEventType.LargeDecrement:
 					XplatUI.ScrollWindow(Handle, 0, ROW_HEIGHT, false);
@@ -489,13 +591,13 @@ namespace System.Windows.Forms.PropertyGridInternal
 					XplatUI.ScrollWindow(Handle, 0, -ROW_HEIGHT, false);
 					Invalidate(ClientRectangle);
 					break;
-				/*case ScrollEventType.ThumbTrack:
+				case ScrollEventType.ThumbTrack:
 					XplatUI.ScrollWindow(Handle, 0, -(vbar.Value-e.NewValue), false);
 					Invalidate(ClientRectangle);
 					break;
 				case ScrollEventType.ThumbPosition:
 					Invalidate(ClientRectangle);
-					break;*/
+					break;
 			}
 		}
 
@@ -506,10 +608,16 @@ namespace System.Windows.Forms.PropertyGridInternal
 			//Region clip = new Region();
 			//if (property_grid.SelectedGridItem != null)
 			//	clip.Union(new Rectangle(0,property_grid.SelectedGridItem.Top, ClientRectangle.Width, ROW_HEIGHT));
-				//	clip.Union(new Rectangle(0,property_grid.SelectedGridItem.Top, ClientRectangle.Width, ROW_HEIGHT));
+			//	clip.Union(new Rectangle(0,property_grid.SelectedGridItem.Top, ClientRectangle.Width, ROW_HEIGHT));
 
 			if (e.NewSelection.PropertyDescriptor != null)
+			{
 				grid_textbox.Text = e.NewSelection.PropertyDescriptor.Converter.ConvertToString(e.NewSelection.Value);
+				if (e.NewSelection.PropertyDescriptor.CanResetValue(property_grid.SelectedObject))
+					grid_textbox.Font = new Font(this.Font, FontStyle.Bold);
+				else
+					grid_textbox.Font = this.Font;
+			}
 
 			Invalidate(/*clip*/this.ClientRectangle);
 			Update();
@@ -518,25 +626,47 @@ namespace System.Windows.Forms.PropertyGridInternal
 		private void HandlePropertyValueChanged(object s, PropertyValueChangedEventArgs e)
 		{
 			if (e.ChangedItem.PropertyDescriptor != null)
+			{
 				grid_textbox.Text = e.ChangedItem.PropertyDescriptor.Converter.ConvertToString(e.ChangedItem.Value);
+				if (e.ChangedItem.PropertyDescriptor.CanResetValue(property_grid.SelectedObject))
+					grid_textbox.Font = new Font(this.Font, FontStyle.Bold);
+				else
+					grid_textbox.Font = this.Font;
+			}
 		}
 		
 		#region IWindowsFormsEditorService Members
 
 		public void CloseDropDown()
 		{
-			// TODO:  Add PropertyGrid.CloseDropDown implementation
+			dropdown_form_showing = false;
+			dropdown_form.Hide();
 		}
 
 		public void DropDownControl(Control control)
 		{
-			// TODO:  Add PropertyGrid.DropDownControl implementation
+			//dropdown_form.FormBorderStyle = FormBorderStyle.None;
+			dropdown_form.Deactivate +=new EventHandler(dropdown_form_Deactivate);
+			dropdown_form.Size = control.Size;
+			control.Dock = DockStyle.Fill;
+			dropdown_form.Controls.Clear();
+			dropdown_form.Controls.Add(control);
+			dropdown_form.Location = PointToScreen(new Point(grid_textbox.Location.X,grid_textbox.Location.Y+ROW_HEIGHT));
+			dropdown_form.Width = grid_textbox.Width;
+
+			dropdown_form_showing = true;
+			dropdown_form.Show();
+			System.Windows.Forms.MSG msg = new MSG();
+			while (XplatUI.GetMessage(ref msg, IntPtr.Zero, 0, 0) && dropdown_form_showing) 
+			{
+				XplatUI.TranslateMessage(ref msg);
+				XplatUI.DispatchMessage(ref msg);
+			}
 		}
 
 		public System.Windows.Forms.DialogResult ShowDialog(Form dialog)
 		{
-			// TODO:  Add PropertyGrid.ShowDialog implementation
-			return DialogResult.OK;
+			return dialog.ShowDialog(this);
 		}
 
 		#endregion
@@ -544,6 +674,85 @@ namespace System.Windows.Forms.PropertyGridInternal
 		#region DropDownForm Class
 		#endregion DropDownForm Class
 
+		#region Internal Classes
+		internal class ITypeDescriptorContextImpl : System.ComponentModel.ITypeDescriptorContext
+		{
+			private PropertyGrid property_grid;
+			public ITypeDescriptorContextImpl(PropertyGrid propertyGrid)
+			{
+				property_grid = propertyGrid;
+			}
+			#region ITypeDescriptorContext Members
+
+			public void OnComponentChanged()
+			{
+				// TODO:  Add SystemComp.OnComponentChanged implementation
+			}
+
+			public IContainer Container
+			{
+				get
+				{
+					return property_grid as IContainer;
+				}
+			}
+
+			public bool OnComponentChanging()
+			{
+				// TODO:  Add SystemComp.OnComponentChanging implementation
+				return false;
+			}
+
+			public object Instance
+			{
+				get
+				{
+					return property_grid.SelectedGridItem.Value;
+				}
+			}
+
+			public PropertyDescriptor PropertyDescriptor
+			{
+				get
+				{
+					return property_grid.SelectedGridItem.PropertyDescriptor;
+				}
+			}
+
+			#endregion
+
+			#region IServiceProvider Members
+
+			public object GetService(Type serviceType)
+			{
+				// TODO:  Add SystemComp.GetService implementation
+				return null;
+			}
+
+			#endregion
+
+		}
+
+
+		
+		/*
+			class ComboListBox
+		*/
+		internal class PropertyGridDropDown : Form 
+		{
+			protected override CreateParams CreateParams
+			{
+				get 
+				{
+					CreateParams cp = base.CreateParams;				
+					cp.Style = unchecked ((int)(WindowStyles.WS_POPUP | WindowStyles.WS_VISIBLE | WindowStyles.WS_CLIPSIBLINGS | WindowStyles.WS_CLIPCHILDREN));
+					cp.ExStyle |= (int)(WindowStyles.WS_EX_TOOLWINDOW | WindowStyles.WS_EX_TOPMOST);				
+					return cp;
+				}
+			}
+
+		}
+		#endregion
 
 	}
 }
