@@ -164,8 +164,7 @@ namespace System.Windows.Forms
 			base.Resize += new EventHandler (OnResizeSB);
 			base.TabStop = false;
                         
-			SetStyle (ControlStyles.UserPaint | ControlStyles.AllPaintingInWmPaint, true);
-			SetStyle (ControlStyles.ResizeRedraw | ControlStyles.Opaque, true);
+			SetStyle (ControlStyles.UserPaint | ControlStyles.StandardClick, false);
 		}
 
 		#region Internal & Private Properties
@@ -296,6 +295,7 @@ namespace System.Windows.Forms
 
 		[DefaultValue (10)]
 		[RefreshProperties(RefreshProperties.Repaint)]
+		[MWFDescription("Scroll amount when clicking in the scroll area"), MWFCategory("Behaviour")]
 		public int LargeChange {
 			get {
 				if (large_change > maximum)
@@ -321,6 +321,7 @@ namespace System.Windows.Forms
 
 		[DefaultValue (100)]
 		[RefreshProperties(RefreshProperties.Repaint)]
+		[MWFDescription("Highest value for scrollbar"), MWFCategory("Behaviour")]
 		public int Maximum {
 			get { return maximum; }
 			set {
@@ -342,6 +343,7 @@ namespace System.Windows.Forms
 
 		[DefaultValue (0)]
 		[RefreshProperties(RefreshProperties.Repaint)]
+		[MWFDescription("Smallest value for scrollbar"), MWFCategory("Behaviour")]
 		public int Minimum {
 			get { return minimum; }
 			set {
@@ -362,6 +364,7 @@ namespace System.Windows.Forms
 		}
 
 		[DefaultValue (1)]
+		[MWFDescription("Scroll amount when clicking scroll arrows"), MWFCategory("Behaviour")]
 		public int SmallChange {
 			get { return small_change; }
 			set {
@@ -393,6 +396,7 @@ namespace System.Windows.Forms
 
 		[Bindable(true)]
 		[DefaultValue (0)]
+		[MWFDescription("Current value for scrollbar"), MWFCategory("Behaviour")]
 		public int Value {
 			get { return position; }
 			set {
@@ -406,7 +410,11 @@ namespace System.Windows.Forms
 					OnValueChanged (EventArgs.Empty);
 
 					ClearDirty ();
-					UpdatePos (Value, true);
+					
+					if (IsHandleCreated) {
+						UpdateThumbPos (thumb_area.Y + (int)(((float)(position - minimum)) * pixel_per_pos), false);
+					}
+					
 					InvalidateDirty ();
 				}
 			}
@@ -433,8 +441,8 @@ namespace System.Windows.Forms
 			base.OnHandleCreated (e);		
 
 			CalcButtonSizes ();
-			CalcThumbArea ();
-			UpdatePos (Value, true);
+			CalcThumbArea ();			
+			UpdateThumbPos (thumb_area.Y + (int)(((float)(position - minimum)) * pixel_per_pos), false);			
 		}
 
 		protected virtual void OnScroll (ScrollEventArgs event_args)
@@ -478,9 +486,9 @@ namespace System.Windows.Forms
 				{
 					PaintEventArgs	paint_event;
 
-					paint_event = XplatUI.PaintEventStart (Handle);
+					paint_event = XplatUI.PaintEventStart (Handle, true);
 					OnPaintSB (paint_event);
-					XplatUI.PaintEventEnd (Handle);
+					XplatUI.PaintEventEnd (Handle, true);
 					return;
 				}
 
@@ -621,10 +629,12 @@ namespace System.Windows.Forms
 
 			case TimerType.RepeatButton:
 			{
-				if ((firstbutton_state & ButtonState.Pushed) == ButtonState.Pushed)
+				if ((firstbutton_state & ButtonState.Pushed) == ButtonState.Pushed &&
+						position != Minimum)
 					SmallDecrement();
 
-				if ((secondbutton_state & ButtonState.Pushed) == ButtonState.Pushed)
+				if ((secondbutton_state & ButtonState.Pushed) == ButtonState.Pushed &&
+					position != Maximum)
 					SmallIncrement();
 
 				break;
@@ -1091,8 +1101,8 @@ namespace System.Windows.Forms
     			else
     				if (newPos > maximum + 1 - large_change)
     					pos = maximum + 1 - large_change;
-    					else
-    						pos = newPos;
+				else
+					pos = newPos;
 
 			// pos can't be less than minimum
 			if (pos < minimum)
