@@ -198,6 +198,7 @@ namespace System.Windows.Forms {
 		#region Public Constructors
 		public ScrollableControl() {
 			SetStyle(ControlStyles.ContainerControl, true);
+			SetStyle(ControlStyles.AllPaintingInWmPaint, false);
 			auto_scroll = false;
 			auto_hscroll = false;
 			auto_vscroll = false;
@@ -237,33 +238,43 @@ namespace System.Windows.Forms {
 
 				auto_scroll = value;
 				if (!auto_scroll) {
-					Controls.Remove(hscrollbar);
+					SuspendLayout ();
+
+					Controls.RemoveImplicit (hscrollbar);
 					hscrollbar.Dispose();
 					hscrollbar = null;
+					hscroll_visible = false;
 
-					Controls.Remove(vscrollbar);
+					Controls.RemoveImplicit (vscrollbar);
 					vscrollbar.Dispose();
 					vscrollbar = null;
+					vscroll_visible = false;
 
-					Controls.Remove(sizegrip);
+					Controls.RemoveImplicit (sizegrip);
 					sizegrip.Dispose();
 					sizegrip = null;
+
+					ResumeLayout ();
 				} else {
+					SuspendLayout ();
+
 					hscrollbar = new HScrollBar();
 					hscrollbar.Visible = false;
 					hscrollbar.ValueChanged += new EventHandler(HandleScrollBar);
 					hscrollbar.Height = SystemInformation.HorizontalScrollBarHeight;
-					this.Controls.Add(hscrollbar);
+					this.Controls.AddImplicit (hscrollbar);
 
 					vscrollbar = new VScrollBar();
 					vscrollbar.Visible = false;
 					vscrollbar.ValueChanged += new EventHandler(HandleScrollBar);
 					vscrollbar.Width = SystemInformation.VerticalScrollBarWidth;
-					this.Controls.Add(vscrollbar);
+					this.Controls.AddImplicit (vscrollbar);
 
 					sizegrip = new SizeGrip();
 					sizegrip.Visible = false;
-					this.Controls.Add(sizegrip);
+					this.Controls.AddImplicit (sizegrip);
+
+					ResumeLayout ();
 				}
 			}
 		}
@@ -339,7 +350,6 @@ namespace System.Windows.Forms {
 				Rectangle rect;
 				
 				rect = base.DisplayRectangle;
-				
 				if (vscroll_visible) {
 					rect.Width -= vscrollbar.Width;
 					if (rect.Width < 0) {
@@ -376,13 +386,7 @@ namespace System.Windows.Forms {
 		#region Protected Instance Methods
 		protected override CreateParams CreateParams {
 			get {
-				CreateParams	ret;
-
-				ret = base.CreateParams;
-
-				ret.Style |= (int)(WindowStyles.WS_CLIPCHILDREN | WindowStyles.WS_CLIPSIBLINGS | WindowStyles.WS_VISIBLE);
-
-				return ret;
+				return base.CreateParams;
 			}
 		}
 
@@ -544,7 +548,9 @@ namespace System.Windows.Forms {
 
 				hscrollbar.Left = 0;
 				hscrollbar.Top = client.Height - SystemInformation.HorizontalScrollBarHeight;
-				hscrollbar.Maximum = Math.Max(0, canvas.Width - client.Width + SystemInformation.VerticalScrollBarWidth);
+				hscrollbar.LargeChange = DisplayRectangle.Width;
+				hscrollbar.SmallChange = DisplayRectangle.Width / 10;
+				hscrollbar.Maximum = canvas.Width - 1;
 
 				hscroll_visible = true;
 			} else {
@@ -558,7 +564,9 @@ namespace System.Windows.Forms {
 				vscrollbar.Top = 0;
 
 				// FIXME - Working around some scrollbar bugs here; shouldn't have to add the height again (see canvas+= above)
-				vscrollbar.Maximum = Math.Max(0, canvas.Height - client.Height + SystemInformation.HorizontalScrollBarHeight);
+				vscrollbar.LargeChange = DisplayRectangle.Height;
+				vscrollbar.SmallChange = DisplayRectangle.Height / 10;
+				vscrollbar.Maximum = canvas.Height - 1;
 				vscroll_visible = true;
 			} else {
 				vscroll_visible = false;
