@@ -60,7 +60,7 @@ namespace System.Windows.Forms {
 		public TabControl ()
 		{
 			tab_pages = new TabPageCollection (this);
-			SetStyle (ControlStyles.UserPaint, true);
+			SetStyle (ControlStyles.UserPaint, false);
 			padding = ThemeEngine.Current.TabControlDefaultPadding;
 			item_size = ThemeEngine.Current.TabControlDefaultItemSize;
 
@@ -215,9 +215,29 @@ namespace System.Windows.Forms {
 				SuspendLayout ();
 
 				Rectangle invalid = Rectangle.Empty;
+				bool refresh = false;
+				
+				if (-1 != value && show_slider && value < slider_pos) {
+					slider_pos = value;
+					refresh = true;
+				}
+
+				int le = TabPages [value].TabBounds.Right;
+				int re = ThemeEngine.Current.GetTabControlLeftScrollRect (this).Left;
+
+				if (-1 != value && show_slider && le > re) {
+					int diff = le - re;
+					int ind = value - 1;
+					while (ind > 0 && diff > 0) {
+						diff -= TabPages [ind++].Width;
+					}
+					slider_pos = ind - 1;
+					refresh = true;
+				}
 
 				if (selected_index != -1) {
-					invalid = GetTabRect (selected_index);
+					if (!refresh)
+						invalid = GetTabRect (selected_index);
 					Controls [selected_index].Visible = false;
 				}
 				selected_index = value;
@@ -229,12 +249,15 @@ namespace System.Windows.Forms {
 				if (selected_index != -1) {
 					invalid = Rectangle.Union (invalid, GetTabRect (selected_index));
 					selected.Visible = true;
-					selected.Focus ();
+					// selected.Focus ();
 				}
 
 				ResumeLayout ();
 
-				if (selected_index != -1 && selected.Row != BottomRow) {
+				if (refresh) {
+					SizeTabs ();
+					Refresh ();
+				} else if (selected_index != -1 && selected.Row != BottomRow) {
 					DropRow (TabPages [selected_index].Row);
 					// calculating what to invalidate here seems to be slower then just
 					// refreshing the whole thing
@@ -343,11 +366,9 @@ namespace System.Windows.Forms {
 		#endregion	// Internal Properties
 
 		#region Protected Instance Properties
-		[MonoTODO ("Anything special need to be done?")]
 		protected override CreateParams CreateParams {
 			get {
 				CreateParams c = base.CreateParams;
-				// Do we need to do anything here?
 				return c;
 			}
 		}
@@ -370,6 +391,17 @@ namespace System.Windows.Forms {
 			return GetTab (index);
 		}
 
+		public override string ToString ()
+		{
+			string res = String.Concat (base.ToString (),
+					", TabPages.Count: ",
+					TabCount);
+			if (TabCount > 0)
+				res = String.Concat (res, ", TabPages[0]: ",
+						TabPages [0]);
+			return res;
+		}
+
 		#endregion	// Public Instance Methods
 
 		#region Protected Instance Methods
@@ -378,17 +410,106 @@ namespace System.Windows.Forms {
 			return new TabControl.ControlCollection (this);
 		}
 
+		protected void UpdateTabSelection (bool uiselected)
+		{
+			ResizeTabPages ();
+		}
+
 		protected override void CreateHandle ()
 		{
 			base.CreateHandle ();
 			ResizeTabPages ();
 		}
 
+		protected override void OnHandleCreated (EventArgs e)
+		{
+			base.OnHandleCreated (e);
+		}
+
+		protected override void OnHandleDestroyed (EventArgs e)
+		{
+			base.OnHandleDestroyed (e);
+		}
+
+		protected virtual void OnDrawItem (DrawItemEventArgs e)
+		{
+			if (DrawItem != null && DrawMode == TabDrawMode.OwnerDrawFixed)
+				DrawItem (this, e);
+		}
+
+		internal void OnDrawItemInternal (DrawItemEventArgs e)
+		{
+			OnDrawItem (e);
+		}
+
+		protected override void OnFontChanged (EventArgs e)
+		{
+			base.OnFontChanged (e);
+			ResizeTabPages ();
+		}
+
+		protected override void OnResize (EventArgs e)
+		{
+			base.OnResize (e);
+		}
+
+		protected override void OnStyleChanged (EventArgs e)
+		{
+			base.OnStyleChanged (e);
+		}
+
+		protected override bool ProcessKeyPreview (ref Message m)
+		{
+			if (ProcessKeyEventArgs (ref m))
+				return true;
+			return base.ProcessKeyPreview (ref m);
+		}
+
+		protected override void OnKeyDown (KeyEventArgs e)
+		{
+			if (e.KeyCode == Keys.Tab && (e.KeyData & Keys.Control) != 0) {
+				if ((e.KeyData & Keys.Shift) == 0)
+					SelectedIndex = (SelectedIndex + 1) % TabCount;
+				else
+					SelectedIndex = (SelectedIndex - 1) % TabCount;
+				e.Handled = true;
+			} else if (e.KeyCode == Keys.Home) {
+				SelectedIndex = 0;
+				e.Handled = true;
+			} else if (e.KeyCode == Keys.End) {
+				SelectedIndex = TabCount - 1;
+				e.Handled = true;
+			} else if (e.KeyCode == Keys.Left && SelectedIndex > 0) {
+				SelectedIndex--;
+				e.Handled = true;
+			} else if (e.KeyCode == Keys.Right && SelectedIndex < TabCount - 1) {
+				SelectedIndex++;
+				e.Handled = true;
+			}
+
+			base.OnKeyDown (e);
+		}
+
+		protected override bool IsInputKey (Keys key)
+		{
+			switch (key & Keys.KeyCode) {
+			case Keys.Prior:
+			case Keys.Home:
+				return true;
+			}
+			return base.IsInputKey (key);
+		}
+
 		protected override void Dispose (bool disposing)
 		{
 			base.Dispose (disposing);
 		}
 
+		protected void RemoveAll ()
+		{
+			Controls.Clear ();
+		}
+
 		protected virtual object [] GetItems ()
 		{
 			TabPage [] pages = new TabPage [Controls.Count];
@@ -414,9 +535,9 @@ namespace System.Windows.Forms {
 			switch ((Msg) m.Msg) {
 			case Msg.WM_PAINT:
 				PaintEventArgs	paint_event;
-				paint_event = XplatUI.PaintEventStart (Handle);
+				paint_event = XplatUI.PaintEventStart (Handle, true);
 				PaintInternal (paint_event);
-				XplatUI.PaintEventEnd (Handle);
+				XplatUI.PaintEventEnd (Handle, true);
 				break;
 			default:
 				base.WndProc (ref m);
@@ -435,9 +556,7 @@ namespace System.Windows.Forms {
 		#region Internal & Private Methods
 		private bool CanScrollRight {
 			get {
-				if (TabPages [TabCount - 1].TabBounds.Right > ClientRectangle.Right - 40)
-					return true;
-				return false;
+				return (slider_pos < TabCount - 1);
 			}
 		}
 
@@ -455,7 +574,8 @@ namespace System.Windows.Forms {
 					if (CanScrollRight) {
 						slider_pos++;
 						SizeTabs ();
-						Refresh ();
+
+						Invalidate (new Rectangle (0, 0, Width, DisplayRectangle.Top));
 					} else {
 						Invalidate (right);
 					}
@@ -465,7 +585,8 @@ namespace System.Windows.Forms {
 					if (CanScrollLeft) {
 						slider_pos--;
 						SizeTabs ();
-						Refresh ();
+
+						Invalidate (new Rectangle (0, 0, Width, DisplayRectangle.Top));
 					} else {
 						Invalidate (left);
 					}
@@ -731,13 +852,18 @@ namespace System.Windows.Forms {
 
 			prev_row = TabPages [0].Row;
 
+			// Reset the slider position if the slider isn't needed
+			// anymore (ie window size was increased so all tabs are visible)
+			if (!show_slider)
+				slider_pos = 0;
+
 			for (int i = slider_pos; i < TabPages.Count; i++) {
 				TabPage page = TabPages [i];
 				int width;
  
 				if (SizeMode == TabSizeMode.Fixed) {
 					width = item_size.Width;
-				} else {                                        
+				} else {					
 					width = (int) DeviceContext.MeasureString (page.Text, Font).Width + (Padding.X * 2);
 				}
 
@@ -808,7 +934,7 @@ namespace System.Windows.Forms {
 				int l = page.TabBounds.Left - 4;
 				int r = page.TabBounds.Right + 4;
 				int y = page.TabBounds.Y;
-				int h = page.TabBounds.Height + 2;
+				int h = page.TabBounds.Height + 3;
 
 				if (l < left_edge)
 					l = left_edge;
@@ -837,9 +963,6 @@ namespace System.Windows.Forms {
 
 		private void PaintInternal (PaintEventArgs pe)
 		{
-			if (this.Width <= 0 || this.Height <=  0 || this.Visible == false)
-				return;
-
 			Draw (pe.Graphics, pe.ClipRectangle);
 			// On MS the Paint event never seems to be raised
 		}
@@ -909,7 +1032,7 @@ namespace System.Windows.Forms {
 		#endregion	// Events
 
 
-		#region Class TabPage.ControlCollection
+		#region Class TaControl.ControlCollection
 		public class ControlCollection : System.Windows.Forms.Control.ControlCollection {
 
 			private TabControl owner;
@@ -937,8 +1060,19 @@ namespace System.Windows.Forms {
 					owner.ResizeTabPages ();
 				}
 			}
+
+			public override void Remove (Control value)
+			{
+				TabPage page = value as TabPage;
+				if (page != null) {
+					int index = owner.IndexForTabPage (page);
+					if (index == owner.SelectedIndex)
+						owner.SelectedIndex = (index + 1) % owner.TabCount;
+				}
+				base.Remove (value);
+			}
 		}
-		#endregion	// Class TabPage.ControlCollection
+		#endregion	// Class TabControl.ControlCollection
 
 		#region Class TabPage.TabPageCollection
 		public class TabPageCollection	: IList, ICollection, IEnumerable {
@@ -1046,27 +1180,39 @@ namespace System.Windows.Forms {
 
 			int IList.Add (object value)
 			{
-				//	 return owner.Controls.Add ((TabPage) value);
-				return -1;
+				TabPage page = value as TabPage;
+				if (value == null)
+					throw new ArgumentException ("value");
+				owner.Controls.Add (page);
+                                return owner.Controls.IndexOf (page);
 			}
 
-			bool IList.Contains (object page)
+			bool IList.Contains (object value)
 			{
-				return Contains ((TabPage) page);
+				TabPage page = value as TabPage;
+				if (page == null)
+					return false;
+				return Contains (page);
 			}
 
-			int IList.IndexOf (object page)
+			int IList.IndexOf (object value)
 			{
+				TabPage page = value as TabPage;
+				if (page == null)
+					return -1;
 				return IndexOf ((TabPage) page);
 			}
 
 			void IList.Insert (int index, object value)
 			{
-				controls.Insert (index, (TabPage) value);
+				throw new NotSupportedException ();
 			}
 
 			void IList.Remove (object value)
 			{
+				TabPage page = value as TabPage;
+				if (page == null)
+					return;
 				Remove ((TabPage) value);
 			}
 		}
