@@ -33,19 +33,16 @@ using System.Drawing;
 
 namespace System.Windows.Forms {
 	public class TextBox : TextBoxBase {
-		#region Local Variables
-		internal char			password_char;
-		#endregion	// Local Variables
-
 		#region Public Constructors
 		public TextBox() {
-			accepts_return = false;
-			password_char = '\u25cf';
-			scrollbars = ScrollBars.None;
+			scrollbars = RichTextBoxScrollBars.None;
 			alignment = HorizontalAlignment.Left;
 			this.LostFocus +=new EventHandler(TextBox_LostFocus);
 			this.BackColor = ThemeEngine.Current.ColorWindow;
 			this.ForeColor = ThemeEngine.Current.ColorWindowText;
+
+			SetStyle (ControlStyles.StandardClick | ControlStyles.StandardDoubleClick, false);
+			SetStyle (ControlStyles.FixedHeight, true);
 		}
 		#endregion	// Public Constructors
 
@@ -85,7 +82,7 @@ namespace System.Windows.Forms {
 		}
 
 		[Localizable(true)]
-		[DefaultValue("")]
+		[DefaultValue('\0')]
 		public char PasswordChar {
 			get {
 				return password_char;
@@ -94,6 +91,11 @@ namespace System.Windows.Forms {
 			set {
 				if (value != password_char) {
 					password_char = value;
+					if (!multiline) {
+						document.PasswordChar = value.ToString();
+					} else {
+						document.PasswordChar = "";
+					}
 				}
 			}
 		}
@@ -102,12 +104,13 @@ namespace System.Windows.Forms {
 		[Localizable(true)]
 		public ScrollBars ScrollBars {
 			get {
-				return scrollbars;
+				return (ScrollBars)scrollbars;
 			}
 
 			set {
-				if (value != scrollbars) {
-					scrollbars = value;
+				if (value != (ScrollBars)scrollbars) {
+					scrollbars = (RichTextBoxScrollBars)value;
+					base.CalculateScrollBars();
 				}
 			}
 		}
@@ -144,6 +147,16 @@ namespace System.Windows.Forms {
 			set {
 				if (value != alignment) {
 					alignment = value;
+
+					// MS word-wraps if alignment isn't left
+					if (multiline) {
+						if (alignment != HorizontalAlignment.Left) {
+							document.Wrap = true;
+						} else {
+							document.Wrap = word_wrap;
+						}
+					}
+
 					for (int i = 1; i <= document.Lines; i++) {
 						document.GetLine(i).Alignment = value;
 					}
