@@ -25,7 +25,8 @@
 //
 
 // NOT COMPLETE
-#define Debug
+#undef Debug
+#undef DebugClick
 
 using System.ComponentModel;
 using System.ComponentModel.Design;
@@ -49,20 +50,22 @@ namespace System.Windows.Forms {
 		internal int			max_length;
 		internal bool			modified;
 		internal bool			multiline;
+		internal char			password_char;
 		internal bool			read_only;
 		internal bool			word_wrap;
 		internal Document		document;
 		internal LineTag		caret_tag;		// tag our cursor is in
 		internal int			caret_pos;		// position on the line our cursor is in (can be 0 = beginning of line)
-		internal int			viewport_x;		// left visible pixel
-		internal int			viewport_y;		// top visible pixel
-		internal HScrollBar		hscroll;
-		internal VScrollBar		vscroll;
-		internal ScrollBars		scrollbars;
-		internal bool			grabbed;
+		internal ImplicitHScrollBar	hscroll;
+		internal ImplicitVScrollBar	vscroll;
+		internal RichTextBoxScrollBars	scrollbars;
 		internal bool			richtext;
 		internal int			requested_height;
-
+		internal int			canvas_width;
+		internal int			canvas_height;
+		internal int			track_width = 20;
+		internal DateTime		click_last;
+		internal CaretSelection		click_mode;
 		#if Debug
 		internal static bool	draw_lines = false;
 		#endif
@@ -83,11 +86,18 @@ namespace System.Windows.Forms {
 			max_length = 32767;
 			modified = false;
 			multiline = false;
+			password_char = '\0';
 			read_only = false;
 			word_wrap = true;
 			richtext = false;
 			document = new Document(this);
+			document.WidthChanged += new EventHandler(document_WidthChanged);
+			document.HeightChanged += new EventHandler(document_HeightChanged);
+			//document.CaretMoved += new EventHandler(CaretMoved);
+			document.Wrap = false;
 			requested_height = -1;
+			click_last = DateTime.Now;
+			click_mode = CaretSelection.Position;
 
 			MouseDown += new MouseEventHandler(TextBoxBase_MouseDown);
 			MouseUp += new MouseEventHandler(TextBoxBase_MouseUp);
@@ -95,23 +105,34 @@ namespace System.Windows.Forms {
 			SizeChanged += new EventHandler(TextBoxBase_SizeChanged);
 			FontChanged += new EventHandler(TextBoxBase_FontOrColorChanged);
 			ForeColorChanged += new EventHandler(TextBoxBase_FontOrColorChanged);
+			MouseWheel += new MouseEventHandler(TextBoxBase_MouseWheel);
 			
-			scrollbars = ScrollBars.None;
+			scrollbars = RichTextBoxScrollBars.None;
 
-			hscroll = new HScrollBar();
-			hscroll.ValueChanged +=new EventHandler(hscroll_ValueChanged);
-			hscroll.Enabled = true;
+			hscroll = new ImplicitHScrollBar();
+			hscroll.ValueChanged += new EventHandler(hscroll_ValueChanged);
+			hscroll.control_style &= ~ControlStyles.Selectable;
+			hscroll.Enabled = false;
 			hscroll.Visible = false;
 
-			vscroll = new VScrollBar();
+			vscroll = new ImplicitVScrollBar();
+			vscroll.ValueChanged += new EventHandler(vscroll_ValueChanged);
+			vscroll.control_style &= ~ControlStyles.Selectable;
+			vscroll.Enabled = false;
 			vscroll.Visible = false;
 
-			this.Controls.Add(hscroll);
-			this.Controls.Add(vscroll);
-
+			SuspendLayout ();
+			this.Controls.AddImplicit (hscroll);
+			this.Controls.AddImplicit (vscroll);
+			ResumeLayout ();
+			
 			//SetStyle(ControlStyles.ResizeRedraw, true);
-			SetStyle(ControlStyles.AllPaintingInWmPaint, true);
-			SetStyle(ControlStyles.UserPaint, true);
+			SetStyle(ControlStyles.UserPaint | ControlStyles.StandardClick, false);
+
+			canvas_width = ClientSize.Width;
+			canvas_height = ClientSize.Height;
+
+			CalculateScrollBars();
 		}
 		#endregion	// Internal Constructor
 
@@ -155,8 +176,8 @@ namespace System.Windows.Forms {
 				if (value != auto_size) {
 					auto_size = value;
 					if (auto_size) {
-						if (PreferredHeight != Height) {
-							Height = PreferredHeight;
+						if (PreferredHeight != ClientSize.Height) {
+							ClientSize = new Size(ClientSize.Width, PreferredHeight);
 						}
 					}
 					OnAutoSizeChanged(EventArgs.Empty);
@@ -188,15 +209,10 @@ namespace System.Windows.Forms {
 		[DefaultValue(BorderStyle.Fixed3D)]
 		[DispId(-504)]
 		public BorderStyle BorderStyle {
-			get {
-				return border_style;
-			}
-
-			set {
-				if (value != border_style) {
-					border_style = value;
-					OnBorderStyleChanged(EventArgs.Empty);
-				}
+			get { return InternalBorderStyle; }
+			set { 
+				InternalBorderStyle = value; 
+				OnBorderStyleChanged(EventArgs.Empty);
 			}
 		}
 
@@ -271,7 +287,7 @@ namespace System.Windows.Forms {
 				for (i = 0; i < l; i++) {
 					document.Add(i+1, CaseAdjust(value[i]), alignment, Font, brush);
 				}
-				document.RecalculateDocument(CreateGraphics());
+				CalculateDocument();
 				OnTextChanged(EventArgs.Empty);
 			}
 		}
@@ -280,6 +296,9 @@ namespace System.Windows.Forms {
 		[Localizable(true)]
 		public virtual int MaxLength {
 			get {
+				if (max_length == 2147483646) {	// We don't distinguish between single and multi-line limits
+					return 0;
+				}
 				return max_length;
 			}
 
@@ -316,7 +335,6 @@ namespace System.Windows.Forms {
 			set {
 				if (value != multiline) {
 					multiline = value;
-
 					// Make sure we update our size; the user may have already set the size before going to multiline
 					if (multiline && requested_height != -1) {
 						Height = requested_height;
@@ -327,6 +345,19 @@ namespace System.Windows.Forms {
 				}
 
 				document.multiline = multiline;
+
+				if (multiline) {
+					document.Wrap = word_wrap;
+					document.PasswordChar = "";
+
+				} else {
+					document.Wrap = false;
+					if (this.password_char != '\0') {
+						document.PasswordChar = password_char.ToString();
+					} else {
+						document.PasswordChar = "";
+					}
+				}
 			}
 		}
 
@@ -361,8 +392,10 @@ namespace System.Windows.Forms {
 			}
 
 			set {
-				document.ReplaceSelection(CaseAdjust(value));
-				OnTextChanged(EventArgs.Empty);
+				if (!read_only) {
+					document.ReplaceSelection(CaseAdjust(value));
+					OnTextChanged(EventArgs.Empty);
+				}
 			}
 		}
 
@@ -431,14 +464,15 @@ namespace System.Windows.Forms {
 					for (i = 1; i < document.Lines; i++) {
 						sb.Append(document.GetLine(i).text.ToString() + Environment.NewLine);
 					}
-
+					sb.Append(document.GetLine(document.Lines).text.ToString());
 					return sb.ToString();
 				}
 			}
 
 			set {
-				if (value == base.Text)
+				if (value == base.Text) {
 					return;
+				}
 
 				if (value != null) {
 					Line	line;
@@ -447,6 +481,7 @@ namespace System.Windows.Forms {
 						string[]	lines;
 
 						lines = value.Split(new char[] {'\n'});
+
 						for (int i = 0; i < lines.Length; i++) {
 							if (lines[i].EndsWith("\r")) {
 								lines[i] = lines[i].Substring(0, lines[i].Length - 1);
@@ -463,7 +498,7 @@ namespace System.Windows.Forms {
 					} else {
 						document.Clear();
 						document.Add(1, CaseAdjust(value), alignment, Font, ThemeEngine.Current.ResPool.GetSolidBrush(ForeColor));
-						document.RecalculateDocument(CreateGraphics());
+						CalculateDocument();
 						line = document.GetLine(1);
 						document.SetSelectionStart(line, 0);
 						document.SetSelectionEnd(line, value.Length);
@@ -471,7 +506,8 @@ namespace System.Windows.Forms {
 					}
 				}
 				base.Text = value;
-				OnTextChanged(EventArgs.Empty);
+				// Not needed, base.Text already fires it
+				// OnTextChanged(EventArgs.Empty);
 			}
 		}
 
@@ -492,6 +528,7 @@ namespace System.Windows.Forms {
 					for (i = 1; i < document.Lines; i++) {
 						total += document.GetLine(i).text.Length + Environment.NewLine.Length;
 					}
+					total += document.GetLine(i).text.Length;
 
 					return total;
 				}
@@ -507,7 +544,10 @@ namespace System.Windows.Forms {
 
 			set {
 				if (value != word_wrap) {
-					word_wrap = value;
+					if (multiline) {
+						word_wrap = value;
+						document.Wrap = value;
+					}
 				}
 			}
 		}
@@ -529,33 +569,13 @@ namespace System.Windows.Forms {
 
 		#region Public Instance Methods
 		public void AppendText(string text) {
-
 			if (multiline) {
-				string[]	lines;
-				int		linecount;
-
-				// Break the string into separate lines
-				lines = text.Split(new char[] {'\n'});
-				linecount = lines.Length;
-				for (int i = 0; i < linecount; i++) {
-					if (lines[i].EndsWith("\r")) {
-						lines[i] = lines[i].Substring(0, lines[i].Length - 1);
-					}
-				}
-
 				// Grab the formatting for the last element
 				document.MoveCaret(CaretDirection.CtrlEnd);
+				document.Insert(document.CaretLine, document.CaretPosition, text);
 
-				// Insert the first line
-				document.InsertString(document.CaretLine, document.CaretPosition, lines[0]);
-
-				for (int i = 1; i < linecount; i++) {
-					document.Add(document.CaretLine.LineNo+i, CaseAdjust(lines[i]), alignment, document.CaretTag.font, document.CaretTag.color);
-				}
-
-				document.RecalculateDocument(CreateGraphics());
+				CalculateDocument();
 				document.MoveCaret(CaretDirection.CtrlEnd);
-				Invalidate();
 			} else {
 				document.MoveCaret(CaretDirection.CtrlEnd);
 				document.InsertStringAtCaret(text, true);
@@ -569,28 +589,36 @@ namespace System.Windows.Forms {
 		}
 
 		public void ClearUndo() {
-			// FIXME
-			throw new NotImplementedException();
+			document.undo.Clear();
 		}
 
 		public void Copy() {
-			// FIXME
-			throw new NotImplementedException();
+			DataObject	o;
+
+			o = new DataObject(DataFormats.Text, SelectedText);
+			if (this is RichTextBox) {
+				o.SetData(DataFormats.Rtf, ((RichTextBox)this).SelectedRtf);
+			}
+			Clipboard.SetDataObject(o);
 		}
 
 		public void Cut() {
-			// FIXME
-			throw new NotImplementedException();
+			DataObject	o;
+
+			o = new DataObject(DataFormats.Text, SelectedText);
+			if (this is RichTextBox) {
+				o.SetData(DataFormats.Rtf, ((RichTextBox)this).SelectedRtf);
+			}
+			Clipboard.SetDataObject(o);
+			document.ReplaceSelection("");
 		}
 
 		public void Paste() {
-			// FIXME
-			throw new NotImplementedException();
+			Paste(null, false);
 		}
 
 		public void ScrollToCaret() {
-			// FIXME
-			throw new NotImplementedException();
+			this.CaretMoved(this, EventArgs.Empty);
 		}
 
 		public void Select(int start, int length) {
@@ -608,27 +636,11 @@ namespace System.Windows.Forms {
 		}
 
 		public override string ToString() {
-			StringBuilder	sb;
-			int		i;
-			int		end;
-
-			if (document == null) {
-				return String.Empty;
-			}
-
-			sb = new StringBuilder();
-
-			end = document.Lines;
-
-			for (i = 1; i < end; i++) {
-				sb.Append(document.GetLine(i).text.ToString() + "\n");
-			}
-
-			return sb.ToString();
+			return Text;
 		}
 
 		public void Undo() {
-			return;
+			document.undo.Undo();
 		}
 		#endregion	// Public Instance Methods
 
@@ -639,15 +651,18 @@ namespace System.Windows.Forms {
 
 		protected override bool IsInputKey(Keys keyData) {
 			switch (keyData) {
+#if not
+				// We handle Enter in ProcessDialogKey
 				case Keys.Enter: {
 					if (multiline && (accepts_return || ((keyData & Keys.Control) != 0))) {
 						return true;
 					}
 					return false;
 				}
+#endif
 
 				case Keys.Tab: {
-					if (accepts_tab) {
+					if (accepts_tab && multiline) {
 						return true;
 					}
 					return false;
@@ -679,7 +694,7 @@ namespace System.Windows.Forms {
 			base.OnFontChanged (e);
 
 			if (auto_size) {
-				if (PreferredHeight != Height) {
+				if (PreferredHeight != ClientSize.Height) {
 					Height = PreferredHeight;
 				}
 			}
@@ -718,81 +733,185 @@ namespace System.Windows.Forms {
 		}
 
 		protected override bool ProcessDialogKey(Keys keyData) {
-			switch (keyData & Keys.KeyCode) {
-				case Keys.Left: {
-					document.SetSelectionToCaret(true);
+			bool control;
+			bool shift;
 
-					if ((Control.ModifierKeys & Keys.Control) != 0) {
+			control = (Control.ModifierKeys & Keys.Control) != 0;
+			shift = (Control.ModifierKeys & Keys.Shift) != 0;
+
+			switch (keyData & Keys.KeyCode) {
+				case Keys.X: {	// Cut (Ctrl-X)
+					if (control) {
+						Cut();
+						return true;
+					}
+					return false;
+				}
+
+				case Keys.C: {	// Copy (Ctrl-C)
+					if (control) {
+						Copy();
+						return true;
+					}
+					return false;
+				}
+
+				case Keys.V: {	// Paste (Ctrl-V)
+					if (control) {
+						return Paste(null, true);
+					}
+					return false;
+				}
+
+				case Keys.Z: {	// Undo (Ctrl-Z)
+					if (control) {
+						Undo();
+						return true;
+					}
+					return false;
+				}
+
+				case Keys.Left: {
+					if (control) {
 						document.MoveCaret(CaretDirection.WordBack);
 					} else {
-						document.MoveCaret(CaretDirection.CharBack);
+						if (!document.selection_visible || shift) {
+							document.MoveCaret(CaretDirection.CharBack);
+						} else {
+							document.MoveCaret(CaretDirection.SelectionStart);
+						}
 					}
+
+					if (!shift) {
+						document.SetSelectionToCaret(true);
+					} else {
+						document.SetSelectionToCaret(false);
+					}
+
+					CaretMoved(this, null);
 					return true;
 				}
 
 				case Keys.Right: {
-					document.SetSelectionToCaret(true);
-
-					if ((Control.ModifierKeys & Keys.Control) != 0) {
+					if (control) {
 						document.MoveCaret(CaretDirection.WordForward);
 					} else {
-						document.MoveCaret(CaretDirection.CharForward);
+						if (!document.selection_visible || shift) {
+							document.MoveCaret(CaretDirection.CharForward);
+						} else {
+							document.MoveCaret(CaretDirection.SelectionEnd);
+						}
 					}
+					if (!shift) {
+						document.SetSelectionToCaret(true);
+					} else {
+						document.SetSelectionToCaret(false);
+					}
+
+					CaretMoved(this, null);
 					return true;
 				}
 
 				case Keys.Up: {
-					document.SetSelectionToCaret(true);
-					document.MoveCaret(CaretDirection.LineUp);
+					if (control) {
+						if (document.CaretPosition == 0) {
+							document.MoveCaret(CaretDirection.LineUp);
+						} else {
+							document.MoveCaret(CaretDirection.Home);
+						}
+					} else {
+						document.MoveCaret(CaretDirection.LineUp);
+					}
+
+					if ((Control.ModifierKeys & Keys.Shift) == 0) {
+						document.SetSelectionToCaret(true);
+					} else {
+						document.SetSelectionToCaret(false);
+					}
+
+					CaretMoved(this, null);
 					return true;
 				}
 
 				case Keys.Down: {
-					document.SetSelectionToCaret(true);
-					document.MoveCaret(CaretDirection.LineDown);
+					if (control) {
+						if (document.CaretPosition == document.CaretLine.Text.Length) {
+							document.MoveCaret(CaretDirection.LineDown);
+						} else {
+							document.MoveCaret(CaretDirection.End);
+						}
+					} else {
+						document.MoveCaret(CaretDirection.LineDown);
+					}
+
+					if ((Control.ModifierKeys & Keys.Shift) == 0) {
+						document.SetSelectionToCaret(true);
+					} else {
+						document.SetSelectionToCaret(false);
+					}
+
+					CaretMoved(this, null);
 					return true;
 				}
 
 				case Keys.Home: {
-					document.SetSelectionToCaret(true);
-
 					if ((Control.ModifierKeys & Keys.Control) != 0) {
 						document.MoveCaret(CaretDirection.CtrlHome);
 					} else {
 						document.MoveCaret(CaretDirection.Home);
 					}
+
+					if ((Control.ModifierKeys & Keys.Shift) == 0) {
+						document.SetSelectionToCaret(true);
+					} else {
+						document.SetSelectionToCaret(false);
+					}
+
+					CaretMoved(this, null);
 					return true;
 				}
 
 				case Keys.End: {
-					document.SetSelectionToCaret(true);
-
 					if ((Control.ModifierKeys & Keys.Control) != 0) {
 						document.MoveCaret(CaretDirection.CtrlEnd);
 					} else {
 						document.MoveCaret(CaretDirection.End);
 					}
+
+					if ((Control.ModifierKeys & Keys.Shift) == 0) {
+						document.SetSelectionToCaret(true);
+					} else {
+						document.SetSelectionToCaret(false);
+					}
+
+					CaretMoved(this, null);
 					return true;
 				}
 
 				case Keys.Enter: {
-					if (multiline && (accepts_return || ((Control.ModifierKeys & Keys.Control) != 0))) {
+					// ignoring accepts_return, fixes bug #76355
+					if (!read_only && multiline && (accepts_return || (FindForm().AcceptButton == null) || ((Control.ModifierKeys & Keys.Control) != 0))) {
+						Line	line;
+
 						if (document.selection_visible) {
 							document.ReplaceSelection("");
 						}
 						document.SetSelectionToCaret(true);
 
-						document.Split(document.CaretLine, document.CaretTag, document.CaretPosition);
+						line = document.CaretLine;
+
+						document.Split(document.CaretLine, document.CaretTag, document.CaretPosition, false);
 						OnTextChanged(EventArgs.Empty);
-						document.UpdateView(document.CaretLine, 2, 0);
+						document.UpdateView(line, 2, 0);
 						document.MoveCaret(CaretDirection.CharForward);
+						CaretMoved(this, null);
 						return true;
 					}
 					break;
 				}
 
 				case Keys.Tab: {
-					if (accepts_tab) {
+					if (!read_only && accepts_tab && multiline) {
 						document.InsertChar(document.CaretLine, document.CaretPosition, '\t');
 						if (document.selection_visible) {
 							document.ReplaceSelection("");
@@ -800,6 +919,7 @@ namespace System.Windows.Forms {
 						document.SetSelectionToCaret(true);
 
 						OnTextChanged(EventArgs.Empty);
+						CaretMoved(this, null);
 						return true;
 					}
 					break;
@@ -807,11 +927,16 @@ namespace System.Windows.Forms {
 
 
 				case Keys.Back: {
+					if (read_only) {
+						break;
+					}
+
 					// delete only deletes on the line, doesn't do the combine
 					if (document.selection_visible) {
 						document.ReplaceSelection("");
 					}
 					document.SetSelectionToCaret(true);
+
 					if (document.CaretPosition == 0) {
 						if (document.CaretLine.LineNo > 1) {
 							Line	line;
@@ -823,44 +948,105 @@ namespace System.Windows.Forms {
 							document.Combine(line, document.CaretLine);
 							document.UpdateView(line, 1, 0);
 							document.PositionCaret(line, new_caret_pos);
+							//document.MoveCaret(CaretDirection.CharForward);
 							document.UpdateCaret();
 							OnTextChanged(EventArgs.Empty);
 						}
 					} else {
-						document.DeleteChar(document.CaretTag, document.CaretPosition, false);
-						document.MoveCaret(CaretDirection.CharBack);
+						if (!control || document.CaretPosition == 0) {
+							document.DeleteChar(document.CaretTag, document.CaretPosition, false);
+							document.MoveCaret(CaretDirection.CharBack);
+						} else {
+							int start_pos;
+
+							start_pos = document.CaretPosition - 1;
+
+							while ((start_pos > 0) && !Document.IsWordSeparator(document.CaretLine.Text[start_pos - 1])) {
+								start_pos--;
+							}
+							document.DeleteChars(document.CaretTag, start_pos, document.CaretPosition - start_pos);
+							document.PositionCaret(document.CaretLine, start_pos);
+						}
+						document.UpdateCaret();
 						OnTextChanged(EventArgs.Empty);
 					}
+					CaretMoved(this, null);
 					return true;
 				}
 
-				case Keys.Delete: {
-					// delete only deletes on the line, doesn't do the combine
-					if (document.CaretPosition == document.CaretLine.text.Length) {
-						if (document.CaretLine.LineNo < document.Lines) {
-							Line	line;
-
-							line = document.GetLine(document.CaretLine.LineNo + 1);
-							document.Combine(document.CaretLine, line);
-							document.UpdateView(document.CaretLine, 2, 0);
-							OnTextChanged(EventArgs.Empty);
-
-#if Debug
-							Line	check_first;
-							Line	check_second;
-
-							check_first = document.GetLine(document.CaretLine.LineNo);
-							check_second = document.GetLine(check_first.line_no + 1);
-
-							Console.WriteLine("Post-UpdateView: Y of first line: {0}, second line: {1}", check_first.Y, check_second.Y);
-#endif
-
-							// Caret doesn't move
-						}
-					} else {
-						document.DeleteChar(document.CaretTag, document.CaretPosition, true);
-						OnTextChanged(EventArgs.Empty);
+				case Keys.Insert: {
+					if (shift) {
+						Paste(null, true);
+						return true;
 					}
+
+					if (control) {
+						Copy();
+						return true;
+					}
+
+					// FIXME - need overwrite/insert toggle?
+					return false;
+				}
+
+				case Keys.Delete: {
+					if (shift) {
+						Cut();
+						return true;
+					}
+
+					if (read_only) {
+						break;
+					}
+
+					if (document.selection_visible) {
+						document.ReplaceSelection("");
+					} else {
+						// DeleteChar only deletes on the line, doesn't do the combine
+						if (document.CaretPosition == document.CaretLine.Text.Length) {
+							if (document.CaretLine.LineNo < document.Lines) {
+								Line	line;
+
+								line = document.GetLine(document.CaretLine.LineNo + 1);
+								document.Combine(document.CaretLine, line);
+								document.UpdateView(document.CaretLine, 2, 0);
+
+								#if not_Debug
+								Line	check_first;
+								Line	check_second;
+
+								check_first = document.GetLine(document.CaretLine.LineNo);
+								check_second = document.GetLine(check_first.line_no + 1);
+
+								Console.WriteLine("Post-UpdateView: Y of first line: {0}, second line: {1}", check_first.Y, check_second.Y);
+								#endif
+
+								// Caret doesn't move
+							}
+						} else {
+							if (!control) {
+								document.DeleteChar(document.CaretTag, document.CaretPosition, true);
+							} else {
+								int end_pos;
+
+								end_pos = document.CaretPosition;
+
+								while ((end_pos < document.CaretLine.Text.Length) && !Document.IsWordSeparator(document.CaretLine.Text[end_pos])) {
+									end_pos++;
+								}
+
+								if (end_pos < document.CaretLine.Text.Length) {
+									end_pos++;
+								}
+								document.DeleteChars(document.CaretTag, document.CaretPosition, end_pos - document.CaretPosition);
+							}
+						}
+					}
+
+					OnTextChanged(EventArgs.Empty);
+					document.AlignCaret();
+					document.UpdateCaret();
+					CaretMoved(this, null);
 					return true;
 				}
 			}
@@ -879,6 +1065,11 @@ namespace System.Windows.Forms {
 				}
 			}
 
+			document.ViewPortWidth = width;
+			document.ViewPortHeight = height;
+
+			CalculateDocument();
+
 			base.SetBoundsCore (x, y, width, height, specified);
 		}
 
@@ -887,10 +1078,10 @@ namespace System.Windows.Forms {
 				case Msg.WM_PAINT: {
 					PaintEventArgs	paint_event;
 
-					paint_event = XplatUI.PaintEventStart(Handle);
+					paint_event = XplatUI.PaintEventStart(Handle, true);
 					
 					PaintControl(paint_event);
-					XplatUI.PaintEventEnd(Handle);
+					XplatUI.PaintEventEnd(Handle, true);
 					DefWndProc(ref m);
 					return;
 				}
@@ -898,7 +1089,6 @@ namespace System.Windows.Forms {
 				case Msg.WM_SETFOCUS: {
 					// Set caret
 					document.CaretHasFocus();
-Console.WriteLine("Creating caret");
 					base.WndProc(ref m);
 					return;
 				}
@@ -906,7 +1096,6 @@ Console.WriteLine("Creating caret");
 				case Msg.WM_KILLFOCUS: {
 					// Kill caret
 					document.CaretLostFocus();
-Console.WriteLine("Destroying caret");
 					base.WndProc(ref m);
 					return;
 				}
@@ -924,27 +1113,47 @@ Console.WriteLine("Destroying caret");
 						return;
 					}
 
-					if (m.WParam.ToInt32() >= 32) {	// FIXME, tabs should probably go through
+					// Ctrl-Backspace generates a real char, whack it
+					if (m.WParam.ToInt32() == 127) {
+						return;
+					}
+
+					if (!read_only && (m.WParam.ToInt32() >= 32)) {	// FIXME, tabs should probably go through
 						if (document.selection_visible) {
 							document.ReplaceSelection("");
 						}
 
 						switch (character_casing) {
 							case CharacterCasing.Normal: {
-								document.InsertCharAtCaret((char)m.WParam, true);
-								OnTextChanged(EventArgs.Empty);
+								if (document.Length < max_length) {
+									document.InsertCharAtCaret((char)m.WParam, true);
+									OnTextChanged(EventArgs.Empty);
+									CaretMoved(this, null);
+								} else {
+									XplatUI.AudibleAlert();
+								}
 								return;
 							}
 
 							case CharacterCasing.Lower: {
-								document.InsertCharAtCaret(Char.ToLower((char)m.WParam), true);
-								OnTextChanged(EventArgs.Empty);
+								if (document.Length < max_length) {
+									document.InsertCharAtCaret(Char.ToLower((char)m.WParam), true);
+									OnTextChanged(EventArgs.Empty);
+									CaretMoved(this, null);
+								} else {
+									XplatUI.AudibleAlert();
+								}
 								return;
 							}
 
 							case CharacterCasing.Upper: {
-								document.InsertCharAtCaret(Char.ToUpper((char)m.WParam), true);
-								OnTextChanged(EventArgs.Empty);
+								if (document.Length < max_length) {
+									document.InsertCharAtCaret(Char.ToUpper((char)m.WParam), true);
+									OnTextChanged(EventArgs.Empty);
+									CaretMoved(this, null);
+								} else {
+									XplatUI.AudibleAlert();
+								}
 								return;
 							}
 						}
@@ -982,6 +1191,9 @@ Console.WriteLine("Destroying caret");
 		[EditorBrowsable(EditorBrowsableState.Never)]
 		public event PaintEventHandler	Paint;
 		public event EventHandler	ReadOnlyChanged;
+
+		internal event EventHandler	HScrolled;
+		internal event EventHandler	VScrolled;
 		#endregion	// Events
 
 		#region Private Methods
@@ -995,12 +1207,14 @@ Console.WriteLine("Destroying caret");
 			}
 		}
 
-static int current;
+		#if Debug
+		static int current;
+		#endif
 
 		private void PaintControl(PaintEventArgs pevent) {
 			// Fill background
 			pevent.Graphics.FillRectangle(ThemeEngine.Current.ResPool.GetSolidBrush(BackColor), pevent.ClipRectangle);
-			//pevent.Graphics.TextRenderingHint=TextRenderingHint.AntiAlias;
+			pevent.Graphics.TextRenderingHint=TextRenderingHint.AntiAlias;
 
 			// Draw the viewable document
 			document.Draw(pevent.Graphics, pevent.ClipRectangle);
@@ -1008,22 +1222,7 @@ static int current;
 			Rectangle	rect = ClientRectangle;
 			rect.Width--;
 			rect.Height--;
-			pevent.Graphics.DrawRectangle(ThemeEngine.Current.ResPool.GetPen(ThemeEngine.Current.ColorButtonShadow), rect);
-
-			// Set the scrollbar
-			switch (scrollbars) {
-				case ScrollBars.Both: {
-					break;
-				}
-				case ScrollBars.Vertical: {
-					break;
-				}
-				case ScrollBars.Horizontal: {
-					hscroll.Minimum = 0;
-					hscroll.Maximum = document.Width - this.Width;
-					break;
-				}
-			}
+			//pevent.Graphics.DrawRectangle(ThemeEngine.Current.ResPool.GetPen(ThemeEngine.Current.ColorControlDark), rect);
 
 			#if Debug
 				int		start;
@@ -1035,8 +1234,8 @@ static int current;
 				p = new Pen(Color.Red, 1);
 
 				// First, figure out from what line to what line we need to draw
-				start = document.GetLineByPixel(pevent.ClipRectangle.Top - viewport_y, false).line_no;
-				end = document.GetLineByPixel(pevent.ClipRectangle.Bottom - viewport_y, false).line_no;
+				start = document.GetLineByPixel(pevent.ClipRectangle.Top - document.ViewPortY, false).line_no;
+				end = document.GetLineByPixel(pevent.ClipRectangle.Bottom - document.ViewPortY, false).line_no;
 
 				//Console.WriteLine("Starting drawing on line '{0}'", document.GetLine(start));
 				//Console.WriteLine("Ending drawing on line '{0}'", document.GetLine(end));
@@ -1057,19 +1256,52 @@ static int current;
 		}
 
 		private void TextBoxBase_MouseDown(object sender, MouseEventArgs e) {
-			LineTag	tag;
-			Line	line;
-			int	pos;
-
 			if (e.Button == MouseButtons.Left) {
-				document.PositionCaret(e.X, e.Y);
-				document.SetSelectionToCaret(true);
-				this.grabbed = true;
+				TimeSpan interval;
+
+				interval = DateTime.Now - click_last;
+				document.PositionCaret(e.X + document.ViewPortX, e.Y + document.ViewPortY);
 				this.Capture = true;
+				
+				// Handle place caret/select word/select line behaviour
+				if (e.Clicks == 1) {
+					if (SystemInformation.DoubleClickTime < interval.TotalMilliseconds) {
+						#if DebugClick
+							Console.WriteLine("Single Click Invalidating from char {0} to char {1} ({2})", document.selection_start.pos, document.selection_end.pos, document.selection_start.line.text.ToString(document.selection_start.pos, document.selection_end.pos - document.selection_start.pos));
+						#endif
+						document.SetSelectionToCaret(true);
+						click_mode = CaretSelection.Position;
+					} else {
+						#if DebugClick
+							Console.WriteLine("Tripple Click Selecting line");
+						#endif
+						document.ExpandSelection(CaretSelection.Line, false);
+						click_mode = CaretSelection.Line;
+					}
+				} else {
+					// We select the line if the word is already selected, and vice versa
+					if (click_mode != CaretSelection.Word) {
+						if (click_mode == CaretSelection.Line) {
+							document.Invalidate(document.selection_start.line, 0, document.selection_start.line, document.selection_start.line.text.Length);
+						}
+						click_mode = CaretSelection.Word;
+						document.ExpandSelection(CaretSelection.Word, false);	// Setting initial selection
+					} else {
+						click_mode = CaretSelection.Line;
+						document.ExpandSelection(CaretSelection.Line, false);	// Setting initial selection
+					}
+				}
+
+				// Reset
+				click_last = DateTime.Now;
 				return;
 			}
 
 			#if Debug
+				LineTag	tag;
+				Line	line;
+				int	pos;
+
 				if (e.Button == MouseButtons.Right) {
 					draw_lines = !draw_lines;
 					this.Invalidate();
@@ -1082,7 +1314,7 @@ static int current;
 					return;
 				}
 
-				tag = document.FindTag(e.X, e.Y, out pos, false);
+				tag = document.FindTag(e.X + document.ViewPortX, e.Y + document.ViewPortY, out pos, false);
 
 				Console.WriteLine("Click found tag {0}, character {1}", tag, pos);
 				line = tag.line;
@@ -1109,38 +1341,156 @@ static int current;
 
 		private void TextBoxBase_MouseUp(object sender, MouseEventArgs e) {
 			this.Capture = false;
-			this.grabbed = false;
 			if (e.Button == MouseButtons.Left) {
-				document.PositionCaret(e.X + viewport_x, e.Y + viewport_y);
-				document.SetSelectionToCaret(false);
-				document.DisplayCaret();
+				document.PositionCaret(e.X + document.ViewPortX, e.Y + document.ViewPortY);
+				if (click_mode == CaretSelection.Position) {
+					document.SetSelectionToCaret(false);
+					document.DisplayCaret();
+				} else {
+					document.ExpandSelection(click_mode, true);
+				}
 				return;
 			}
 		}
-		#endregion	// Private Methods
-
 
 		private void TextBoxBase_SizeChanged(object sender, EventArgs e) {
+			canvas_width = ClientSize.Width;
+			canvas_height = ClientSize.Height;
 
-			// First, check which scrollbars we need
-			
-			hscroll.Bounds = new Rectangle (ClientRectangle.Left, ClientRectangle.Bottom - hscroll.Height, Width, hscroll.Height);
+			// We always move them, they just might not be displayed
+			hscroll.Bounds = new Rectangle (ClientRectangle.Left, ClientRectangle.Bottom - hscroll.Height, ClientSize.Width, hscroll.Height);
+			vscroll.Bounds = new Rectangle (ClientRectangle.Right - vscroll.Width, ClientRectangle.Top, vscroll.Width, ClientSize.Height);
 			
 		}
 
+		private void TextBoxBase_MouseWheel(object sender, MouseEventArgs e) {
+			Line	line;
+			int	line_no;
+			int	target;
+
+			if (!vscroll.Enabled) {
+				return;
+			}
+
+			if (e.Delta < 0) {
+				line_no = document.GetLineByPixel(document.ViewPortY, false).line_no + SystemInformation.MouseWheelScrollLines;
+				if (line_no > document.Lines) {
+					line_no = document.Lines;
+				}
+			} else {
+				line_no = document.GetLineByPixel(document.ViewPortY, false).line_no - SystemInformation.MouseWheelScrollLines;
+				if (line_no < 1) {
+					line_no = 1;
+				}
+			}
+
+			line = document.GetLine(line_no);
+			if (line.Y < vscroll.Maximum) {
+				vscroll.Value = line.Y;
+			} else {
+				vscroll.Value = vscroll.Maximum;
+			}
+		}
+
+		internal void CalculateDocument() {
+			if (!IsHandleCreated) {
+				return;
+			}
+			document.RecalculateDocument(CreateGraphics());
+			CalculateScrollBars();
+//blah Console.WriteLine("TextBox.cs(1175) Invalidate called in CalculateDocument");
+			Invalidate();	// FIXME - do we need this?
+		}
+
+		internal void CalculateScrollBars() {
+			// FIXME - need separate calculations for center and right alignment
+			// No scrollbars for a single line
+			if (document.Width >= ClientSize.Width) {
+				hscroll.Enabled = true;
+				hscroll.Minimum = 0;
+				hscroll.Maximum = document.Width - ClientSize.Width;
+			} else {
+				hscroll.Maximum = document.ViewPortWidth;
+				hscroll.Enabled = false;
+			}
+
+			if (document.Height >= ClientSize.Height) {
+				vscroll.Enabled = true;
+				vscroll.Minimum = 0;
+				vscroll.Maximum = document.Height - ClientSize.Height;
+			} else {
+				vscroll.Maximum = document.ViewPortHeight;
+				vscroll.Enabled = false;
+			}
+
+
+			if (!multiline) {
+				return;
+			}
+
+			if (!WordWrap) {
+				if ((scrollbars & RichTextBoxScrollBars.Horizontal) != 0) {
+					if (((scrollbars & RichTextBoxScrollBars.ForcedHorizontal) != 0) || hscroll.Enabled) {
+						hscroll.Visible = true;
+					}
+				}
+			}
+
+			if ((scrollbars & RichTextBoxScrollBars.Vertical) != 0) {
+				if (((scrollbars & RichTextBoxScrollBars.ForcedVertical) != 0) || vscroll.Enabled) {
+					vscroll.Visible = true;
+				}
+			}
+
+			if (hscroll.Visible) {
+				vscroll.Maximum += hscroll.Height;
+				canvas_height = ClientSize.Height - hscroll.Height;
+			}
+
+			if (vscroll.Visible) {
+				hscroll.Maximum += vscroll.Width * 2;
+				canvas_width = ClientSize.Width - vscroll.Width * 2;
+			}
+		}
+
+		private void document_WidthChanged(object sender, EventArgs e) {
+			CalculateScrollBars();
+		}
+
+		private void document_HeightChanged(object sender, EventArgs e) {
+			CalculateScrollBars();
+		}
+
 		private void hscroll_ValueChanged(object sender, EventArgs e) {
 			XplatUI.ScrollWindow(this.Handle, document.ViewPortX-this.hscroll.Value, 0, false);
 			document.ViewPortX = this.hscroll.Value;
 			document.UpdateCaret();
-			Console.WriteLine("Dude scrolled");
+
+			if (HScrolled != null) {
+				HScrolled(this, EventArgs.Empty);
+			}
+		}
+
+		private void vscroll_ValueChanged(object sender, EventArgs e) {
+			XplatUI.ScrollWindow(this.Handle, 0, document.ViewPortY-this.vscroll.Value, false);
+			document.ViewPortY = this.vscroll.Value;
+			document.UpdateCaret();
+
+			if (VScrolled != null) {
+				VScrolled(this, EventArgs.Empty);
+			}
 		}
 
 		private void TextBoxBase_MouseMove(object sender, MouseEventArgs e) {
 			// FIXME - handle auto-scrolling if mouse is to the right/left of the window
-			if (grabbed) {
-				document.PositionCaret(e.X + viewport_x, e.Y + viewport_y);
-				document.SetSelectionToCaret(false);
-				document.DisplayCaret();
+			if (Capture) {
+				document.PositionCaret(e.X + document.ViewPortX, e.Y + document.ViewPortY);
+				if (click_mode == CaretSelection.Position) {
+					document.SetSelectionToCaret(false);
+					document.DisplayCaret();
+				} else {
+					document.ExpandSelection(click_mode, true);
+				}
 			}
 		}
 									      
@@ -1158,5 +1508,116 @@ static int current;
 				document.AlignCaret();
 			}
 		}
+
+		/// <summary>Ensure the caret is always visible</summary>
+		internal void CaretMoved(object sender, EventArgs e) {
+			Point	pos;
+			int	height;
+			
+			pos = document.Caret;
+			//Console.WriteLine("Caret now at {0} (Thumb: {1}x{2}, Canvas: {3}x{4}, Document {5}x{6})", pos, hscroll.Value, vscroll.Value, canvas_width, canvas_height, document.Width, document.Height);
+
+			// Handle horizontal scrolling
+			if (document.CaretLine.alignment == HorizontalAlignment.Left) {
+				if (pos.X < (document.ViewPortX + track_width)) {
+					do {
+						if ((hscroll.Value - track_width) >= hscroll.Minimum) {
+							hscroll.Value -= track_width;
+						} else {
+							hscroll.Value = hscroll.Minimum;
+						}
+					} while (hscroll.Value > pos.X);
+				}
+
+				if ((pos.X > (this.canvas_width + document.ViewPortX - track_width)) && (hscroll.Value != hscroll.Maximum)) {
+					do {
+						if ((hscroll.Value + track_width) <= hscroll.Maximum) {
+							hscroll.Value += track_width;
+						} else {
+							hscroll.Value = hscroll.Maximum;
+						}
+					} while (pos.X > (hscroll.Value + this.canvas_width));
+				}
+			} else if (document.CaretLine.alignment == HorizontalAlignment.Right) {
+				if (pos.X < document.ViewPortX) {
+					if (pos.X > hscroll.Minimum) {
+						hscroll.Value = pos.X;
+					} else {
+						hscroll.Value = hscroll.Minimum;
+					}
+				}
+
+				if ((pos.X > (this.canvas_width + document.ViewPortX)) && (hscroll.Value != hscroll.Maximum)) {
+					hscroll.Value = hscroll.Maximum;
+				}
+			} else {
+			}
+
+			if (!multiline) {
+				return;
+			}
+
+			// Handle vertical scrolling
+			height = document.CaretLine.Height;
+
+			if (pos.Y < document.ViewPortY) {
+				vscroll.Value = pos.Y;
+			}
+
+			if ((pos.Y + height) > (document.ViewPortY + canvas_height)) {
+				vscroll.Value = pos.Y - canvas_height + height;
+			}
+		}
+
+		internal bool Paste(DataFormats.Format format, bool obey_length) {
+			IDataObject	clip;
+			string		s;
+
+			clip = Clipboard.GetDataObject();
+
+			if (format == null) {
+				if ((this is RichTextBox) && clip.GetDataPresent(DataFormats.Rtf)) {
+					format = DataFormats.GetFormat(DataFormats.Rtf);
+				} else if (clip.GetDataPresent(DataFormats.UnicodeText)) {
+					format = DataFormats.GetFormat(DataFormats.UnicodeText);
+				} else if (clip.GetDataPresent(DataFormats.Text)) {
+					format = DataFormats.GetFormat(DataFormats.Text);
+				} else {
+					return false;
+				}
+			} else {
+				if ((format.Name == DataFormats.Rtf) && !(this is RichTextBox)) {
+					return false;
+				}
+
+				if (!clip.GetDataPresent(format.Name)) {
+					return false;
+				}
+			}
+
+			if (format.Name == DataFormats.Rtf) {
+				((RichTextBox)this).SelectedRtf = (string)clip.GetData(DataFormats.Rtf);
+				return true;
+			} else if (format.Name == DataFormats.UnicodeText) {
+				s = (string)clip.GetData(DataFormats.UnicodeText);
+			} else if (format.Name == DataFormats.Text) {
+				s = (string)clip.GetData(DataFormats.Text);
+			} else {
+				return false;
+			}
+
+			if (!obey_length) {
+				this.SelectedText = s;
+			} else {
+				if ((s.Length + document.Length) < max_length) {
+					this.SelectedText = s;
+				} else if (document.Length < max_length) {
+					this.SelectedText = s.Substring(0, max_length - document.Length);
+				}
+			}
+
+			return true;
+		}
+		#endregion	// Private Methods
 	}
 }
