@@ -70,7 +70,7 @@ namespace System.Windows.Forms {
 				Visible = false;
 				Size = new Size(100, 20);
 				ForeColor = ThemeEngine.Current.ColorInfoText;
-				BackColor = ThemeEngine.Current.ColorInfoWindow;
+				BackColor = ThemeEngine.Current.ColorInfo;
 
 				VisibleChanged += new EventHandler(ToolTipWindow_VisibleChanged);
 
@@ -270,7 +270,10 @@ namespace System.Windows.Forms {
 		[Localizable (true)]
 		[DefaultValue ("")]
 		public string GetToolTip(Control control) {
-			return (string)tooltip_strings[control];
+			string tooltip = (string)tooltip_strings[control];
+			if (tooltip == null)
+				return "";
+			return tooltip;
 		}
 
 		public void RemoveAll() {
