@@ -119,8 +119,7 @@ namespace System.Windows.Forms
 			MouseMove += new MouseEventHandler (OnMouseMoveTB);
 			holdclick_timer.Elapsed += new ElapsedEventHandler (OnFirstClickTimer);
 
-			SetStyle (ControlStyles.UserPaint | ControlStyles.AllPaintingInWmPaint, true);
-			SetStyle (ControlStyles.ResizeRedraw | ControlStyles.Opaque, true);			
+			SetStyle (ControlStyles.UserPaint | ControlStyles.Opaque, false);
 		}
 
 		#region Private & Internal Properties
@@ -452,9 +451,9 @@ namespace System.Windows.Forms
 			case Msg.WM_PAINT: {				
 				PaintEventArgs	paint_event;
 
-				paint_event = XplatUI.PaintEventStart (Handle);
+				paint_event = XplatUI.PaintEventStart (Handle, true);
 				OnPaintTB (paint_event);
-				XplatUI.PaintEventEnd (Handle);
+				XplatUI.PaintEventEnd (Handle, true);
 				return;
 			}		
 
