@@ -36,7 +36,6 @@ namespace System.Windows.Forms {
 		#region Fields
 		private TreeView tree_view;
 		internal TreeNode parent;
-		private int index;
 
 		private string text;
 		private int image_index = -1;
@@ -59,6 +58,23 @@ namespace System.Windows.Forms {
 		internal TreeNode (TreeView tree_view) : this ()
 		{
 			this.tree_view = tree_view;
+			is_expanded = true;
+		}
+
+		private TreeNode (SerializationInfo info, StreamingContext context) : this ()
+		{
+			Text = (string) info.GetValue ("Text", typeof (string));
+			prop_bag = (OwnerDrawPropertyBag) info.GetValue ("prop_bag", typeof (OwnerDrawPropertyBag));
+			image_index = (int) info.GetValue ("ImageIndex", typeof (int));
+			selected_image_index = (int) info.GetValue ("SelectedImageIndex", typeof (int));
+			tag = info.GetValue ("Tag", typeof (object));
+			check = (bool) info.GetValue ("Checked", typeof (bool));
+
+			int count = (int) info.GetValue ("NumberOfChildren", typeof (int));
+			for (int i = 0; i < count; i++) {
+				TreeNode node = (TreeNode) info.GetValue ("Child-" + i, typeof (TreeNode));
+				Nodes.Add (node);
+			}
 		}
 
 		#endregion	// Internal Constructors
@@ -111,9 +127,18 @@ namespace System.Windows.Forms {
 		#endregion	// ICloneable Members
 
 		#region ISerializable Members
-		[MonoTODO]
-		void ISerializable.GetObjectData(SerializationInfo info, StreamingContext context) {
-			throw new NotImplementedException();
+		void ISerializable.GetObjectData (SerializationInfo info, StreamingContext context)
+		{
+			info.AddValue ("Text", Text);
+			info.AddValue ("prop_bag", prop_bag, typeof (OwnerDrawPropertyBag));
+			info.AddValue ("ImageIndex", ImageIndex);
+			info.AddValue ("SelectedImageIndex", SelectedImageIndex);
+			info.AddValue ("Tag", Tag);
+			info.AddValue ("Checked", Checked);
+			
+			info.AddValue ("NumberOfChildren", Nodes.Count);
+			for (int i = 0; i < Nodes.Count; i++)
+				info.AddValue ("Child-" + i, Nodes [i], typeof (TreeNode));
 		}
 		#endregion	// ISerializable Members
 
@@ -145,7 +170,7 @@ namespace System.Windows.Forms {
 				check = value;
 
 				if (TreeView != null)
-					tree_view.UpdateNode (this);
+					TreeView.UpdateNode (this);
 			}
 		}
 
@@ -174,7 +199,7 @@ namespace System.Windows.Forms {
 
 		public string FullPath {
 			get {
-				if (tree_view == null)
+				if (TreeView == null)
 					throw new Exception ("No TreeView associated");
 
 				StringBuilder builder = new StringBuilder ();
@@ -233,6 +258,7 @@ namespace System.Windows.Forms {
 			get {
 				if (parent == null)
 					return null;
+				int index = Index;
 				if (parent.Nodes.Count > index + 1)
 					return parent.Nodes [index + 1];
 				return null;
@@ -264,6 +290,7 @@ namespace System.Windows.Forms {
 				if (prop_bag == null)
 					prop_bag = new OwnerDrawPropertyBag (); 
 				prop_bag.Font = value;
+				InvalidateWidth ();
 			}
 		}
 
@@ -278,6 +305,7 @@ namespace System.Windows.Forms {
 
 		public TreeNode Parent {
 			get {
+				TreeView tree_view = TreeView;
 				if (tree_view != null && tree_view.root_node == parent)
 					return null;
 				return parent;
@@ -288,7 +316,8 @@ namespace System.Windows.Forms {
 			get {
 				if (parent == null)
 					return null;
-				if (index == 0 || index > parent.Nodes.Count)
+				int index = Index;
+				if (index <= 0 || index > parent.Nodes.Count)
 					return null;
 				return parent.Nodes [index - 1];
 			}
@@ -343,17 +372,19 @@ namespace System.Windows.Forms {
 				TreeNode walk = parent;
 				while (walk != null) {
 					if (walk.TreeView != null)
-						tree_view = walk.TreeView;
+						break;
 					walk = walk.parent;
 				}
-				return tree_view;
+				if (walk == null)
+					return null;
+				return walk.TreeView;
 			}
 		}
 
                 public IntPtr Handle {
 			get {
 				// MS throws a NullReferenceException if the TreeView isn't set...
-				if (handle == IntPtr.Zero)
+				if (handle == IntPtr.Zero && TreeView != null)
 					handle = TreeView.CreateNodeHandle ();
 				return handle;
 			}
@@ -382,7 +413,7 @@ namespace System.Windows.Forms {
 		public void EndEdit (bool cancel) {
 			is_editing = false;
 			if (!cancel && TreeView != null)
-				text = TreeView.LabelEditText;
+				Text = TreeView.LabelEditText;
 		}
 
 		public void Expand () {
@@ -392,7 +423,7 @@ namespace System.Windows.Forms {
 		public void ExpandAll () {
 			ExpandRecursive (this);
 			if(TreeView != null)
-				TreeView.Refresh();
+				TreeView.UpdateNode (TreeView.root_node);
 		}
 
 		public void EnsureVisible ()
@@ -400,18 +431,14 @@ namespace System.Windows.Forms {
 			if (TreeView == null)
 				return;
 
-			TreeView.BeginUpdate ();
-
 			if (this.Parent != null)
 				ExpandParentRecursive (this.Parent);
 
 			if (bounds.Y < 0) {
 				TreeView.SetTop (this);
-			} else if (bounds.Y > TreeView.ClientRectangle.Height) {
+			} else if (bounds.Bottom > TreeView.ViewportRectangle.Bottom) {
 				TreeView.SetBottom (this);
 			}
-				
-			TreeView.EndUpdate ();
 		}
 
 		public int GetNodeCount (bool include_subtrees) {
@@ -427,7 +454,8 @@ namespace System.Windows.Forms {
 		public void Remove () {
 			if (parent == null)
 				return;
-			parent.Nodes.RemoveAt (Index);
+			int index = Index;
+			parent.Nodes.RemoveAt (index);
 		}
 
 		public void Toggle () {
@@ -445,20 +473,35 @@ namespace System.Windows.Forms {
 
 		#region Internal & Private Methods and Properties
 
+		internal bool IsRoot {
+			get {
+				TreeView tree_view = TreeView;
+				if (tree_view == null)
+					return false;
+				if (tree_view.root_node == this)
+					return true;
+				return false;
+			}
+		}
+
 		bool BuildFullPath (StringBuilder path)
 		{
 			if (parent == null)
 				return false;
 
 			if (parent.BuildFullPath (path))
-				path.Append (tree_view.PathSeparator);
+				path.Append (TreeView.PathSeparator);
 
 			path.Append (text);
 			return true;
 		}
 
 		public int Index {
-			get { return index; }
+			get {
+				if (parent == null)
+					return 0;
+				return parent.Nodes.IndexOf (this);
+			}
 		}
 
 		private void Expand (bool byInternal)
@@ -468,14 +511,14 @@ namespace System.Windows.Forms {
 			bool cancel = false;
 			if (TreeView != null) {
 				TreeViewCancelEventArgs e = new TreeViewCancelEventArgs (this, false, TreeViewAction.Expand);
-				TreeView.OnBeforeCollapse (e);
+				TreeView.OnBeforeExpand (e);
 				cancel = e.Cancel;
 			}
 
 			if (!cancel) {
 				is_expanded = true;
 				if (TreeView != null)
-					TreeView.OnAfterCollapse (new TreeViewEventArgs (this));
+					TreeView.OnAfterExpand (new TreeViewEventArgs (this));
 				if (IsVisible && TreeView != null)
 					TreeView.UpdateBelow (this);
 			}
@@ -486,7 +529,7 @@ namespace System.Windows.Forms {
 			if (!is_expanded)
 				return;
 
-			if (tree_view != null && tree_view.root_node == this)
+			if (IsRoot)
 				return;
 
 			bool cancel = false;
@@ -573,24 +616,34 @@ namespace System.Windows.Forms {
 			}
 		}
 
-		internal void UpdateBounds (int x, int y, int width, int height)
+		internal bool NeedsWidth {
+			get { return bounds.Width == 0; }
+		}
+
+		internal void InvalidateWidth ()
+		{
+			bounds.Width = 0;
+		}
+
+		internal void SetWidth (int width)
 		{
-			bounds.X = x;
-			bounds.Y = y;
 			bounds.Width = width;
+		}
+
+		internal void SetHeight (int height)
+		{
 			bounds.Height = height;
 		}
 
-		internal void SetAddedData (TreeView tree_view, TreeNode parent, int index)
+		internal void SetPosition (int x, int y)
 		{
-			this.tree_view = tree_view;
-			this.parent = parent;
-			this.index = index;
+			bounds.X = x;
+			bounds.Y = y;
 		}
 
-		internal void SetIndex (int index)
+		internal void SetParent (TreeNode parent)
 		{
-			this.index = index;
+			this.parent = parent;
 		}
 
 		private bool IsInClippingRect
@@ -598,7 +651,7 @@ namespace System.Windows.Forms {
 			get {
 				if (TreeView == null)
 					return false;
-				if (bounds.Y < 0 && bounds.Y > tree_view.ClientRectangle.Height)
+				if (bounds.Y < 0 && bounds.Y > TreeView.ClientRectangle.Height)
 					return false;
 				return true;
 			}
