@@ -81,7 +81,7 @@ namespace System.Windows.Forms {
 				if (index < 0 || index >= Count)
 					throw new ArgumentOutOfRangeException ("index");
 				TreeNode node = (TreeNode) value;
-				SetData (node);
+				node.parent = owner;
 				nodes [index] = node;
 			}
 		}
@@ -95,7 +95,7 @@ namespace System.Windows.Forms {
 			set {
 				if (index < 0 || index >= Count)
 					throw new ArgumentOutOfRangeException ("index");
-				SetData (value);
+				value.parent = owner;
 				nodes [index] = value;
 			}
 		}
@@ -112,15 +112,27 @@ namespace System.Windows.Forms {
 			if (node == null)
 				throw new ArgumentNullException("node");
 
-			if (owner != null && owner.TreeView != null && owner.TreeView.Sorted)
+			// Remove it from any old parents
+			node.Remove ();
+
+			TreeView tree_view = null;
+			if (owner != null)
+				tree_view = owner.TreeView;
+			if (tree_view != null && tree_view.Sorted)
 				return AddSorted (node);
-			SetData (node);
+
+			node.parent = owner;
 			if (count >= nodes.Length)
 				Grow ();
 			nodes [count++] = node;
 
-			if (owner.TreeView != null)
-				owner.TreeView.Refresh ();
+			if (owner != null && tree_view != null && (owner.IsExpanded || owner.IsRoot)) {
+				// XXX: Need to ensure the boxes for the nodes have been created
+				tree_view.UpdateNode (owner);
+			} else if (owner != null && tree_view != null) {
+				tree_view.UpdateNodePlusMinus (owner);
+			}
+
 			return count;
 		}
 
@@ -137,8 +149,18 @@ namespace System.Windows.Forms {
 
 		public virtual void Clear ()
 		{
+			for (int i = 0; i < count; i++)
+				RemoveAt (i, false);
+			
 			Array.Clear (nodes, 0, count);
 			count = 0;
+
+			TreeView tree_view = null;
+			if (owner != null) {
+				tree_view = owner.TreeView;
+				if (tree_view != null)
+					tree_view.UpdateBelow (owner);
+			}
 		}
 
 		public bool Contains (TreeNode node)
@@ -163,12 +185,17 @@ namespace System.Windows.Forms {
 
 		public virtual void Insert (int index, TreeNode node)
 		{
-			SetData (node);
-			IList list = (IList) nodes;
-			list.Insert (index, node);
+			node.parent = owner;
+
+			if (count >= nodes.Length)
+				Grow ();
+
+			Array.Copy (nodes, index, nodes, index + 1, count - index);
+			nodes [index] = node;
+			count++;
 		}
 
-		public virtual void Remove (TreeNode node)
+		public void Remove (TreeNode node)
 		{
 			int index = IndexOf (node);
 			if (index > 0)
@@ -177,12 +204,34 @@ namespace System.Windows.Forms {
 
 		public virtual void RemoveAt (int index)
 		{
-			Array.Copy (nodes, index, nodes, index - 1, count - index);
+			RemoveAt (index, true);
+		}
+
+		private void RemoveAt (int index, bool update)
+		{
+			TreeNode removed = nodes [index];
+			
+			Array.Copy (nodes, index + 1, nodes, index, count - index);
 			count--;
 			if (nodes.Length > OrigSize && nodes.Length > (count * 2))
 				Shrink ();
-			if (owner.TreeView != null)
-				owner.TreeView.TotalNodeCount--;
+
+                        TreeView tree_view = null;
+			if (owner != null)
+				tree_view = owner.TreeView;
+			if (tree_view != null && removed == tree_view.top_node) {
+				OpenTreeNodeEnumerator oe = new OpenTreeNodeEnumerator (removed);
+				if (oe.MoveNext () && oe.MoveNext ())
+					tree_view.top_node = oe.CurrentNode;
+				else
+					tree_view.top_node = null;
+			}
+
+			TreeNode parent = removed.parent;
+			removed.parent = null;
+
+			if (tree_view != null)
+				tree_view.UpdateBelow (parent);
 		}
 
 		int IList.Add (object node)
@@ -235,12 +284,11 @@ namespace System.Windows.Forms {
 			// Move the nodes up and adjust their indices
 			for (int i = count - 1; i >= pos; i--) {
 				nodes [i + 1] = nodes [i];
-				nodes [i + 1].SetIndex (i + 1);
 			}
 			count++;
 			nodes [pos] = node;
 
-			SetData (node, pos);
+			node.parent = owner;
 			return count;
 		}
 
@@ -250,21 +298,10 @@ namespace System.Windows.Forms {
 			Array.Sort (nodes, 0, count, new TreeNodeComparer (Application.CurrentCulture.CompareInfo));
 
 			for (int i = 0; i < count; i++) {
-				nodes [i].SetIndex (i);
 				nodes [i].Nodes.Sort ();
 			}
 		}
 
-		private void SetData (TreeNode node)
-		{
-			SetData (node, count);
-		}
-
-		private void SetData (TreeNode node, int pos)
-		{
-			node.SetAddedData ((owner != null ? owner.TreeView : null), owner, pos);
-		}
-
 		private void Grow ()
 		{
 			TreeNode [] nn = new TreeNode [nodes.Length + 50];
