@@ -40,10 +40,10 @@ namespace System.Windows.Forms {
 		private string path_separator = "\\";
 		private int item_height = -1;
 		private bool sorted;
-		private TreeNode top_node;
+		internal TreeNode top_node;
 		internal TreeNode root_node;
+		internal bool nodes_added;
 		private TreeNodeCollection nodes;
-		private int total_node_count;
 
 		private TreeNode selected_node = null;
 		private TreeNode focused_node = null;
@@ -72,14 +72,15 @@ namespace System.Windows.Forms {
 		private bool add_vscroll;
 		private int max_node_width;
 		private VScrollBar vbar;
-		private bool vbar_added;
 		private int skipped_nodes;
 		private HScrollBar hbar;
-		private bool hbar_added;
 		private int hbar_offset;
+		private int used_height;
+		private bool update_node_bounds;
 		
 		private int update_stack;
-
+		private bool update_needed;
+		
 		private TreeViewEventHandler on_after_check;
 		private TreeViewEventHandler on_after_collapse;
 		private TreeViewEventHandler on_after_expand;
@@ -92,8 +93,11 @@ namespace System.Windows.Forms {
 		private TreeViewCancelEventHandler on_before_select;
 
 		private Pen dash;
+		private StringFormat string_format;
 		private int open_node_count = -1;
 
+		private int drag_begin_x = 0;
+		private int drag_begin_y = 0;
 		private long handle_count = 1;
 
 		#endregion	// Fields
@@ -113,12 +117,14 @@ namespace System.Windows.Forms {
 			MouseUp += new MouseEventHandler(MouseUpHandler);
 			MouseMove += new MouseEventHandler(MouseMoveHandler);
 			SizeChanged += new EventHandler (SizeChangedHandler);
+			FontChanged += new EventHandler (FontChangedHandler);
+			LostFocus += new EventHandler (LostFocusHandler);
 
-			SetStyle (ControlStyles.AllPaintingInWmPaint | ControlStyles.ResizeRedraw, true);
-			SetStyle (ControlStyles.UserPaint | ControlStyles.Selectable, true);
+			SetStyle (ControlStyles.UserPaint | ControlStyles.StandardClick, false);
 
-			dash = new Pen (SystemColors.ControlLight, 1);
-			dash.DashStyle = DashStyle.Dash;
+			string_format = new StringFormat ();
+			string_format.LineAlignment = StringAlignment.Center;
+			string_format.Alignment = StringAlignment.Center;
 		}
 
 		#endregion	// Public Constructors
@@ -313,21 +319,43 @@ namespace System.Windows.Forms {
 
 				Rectangle invalid = Rectangle.Empty;
 
-				if (selected_node != null)
-					invalid = selected_node.Bounds;
-				if (focused_node != null)
-					invalid = Rectangle.Union (focused_node.Bounds, invalid);
-				invalid = Rectangle.Union (invalid, value.Bounds);
+				if (selected_node != null) {
+					invalid = Bloat (selected_node.Bounds);
+				}
+				if (focused_node != null) {
+					invalid = Rectangle.Union (invalid,
+							Bloat (focused_node.Bounds));
+				}
+
+				invalid = Rectangle.Union (invalid, Bloat (value.Bounds));
 
 				selected_node = value;
 				focused_node = value;
 
+				if (full_row_select) {
+					invalid.X = 0;
+					invalid.Width = ViewportRectangle.Width;
+				}
+
 				Invalidate (invalid);
-				
+
+				// We ensure its visible after we update because
+				// scrolling is used for insure visible
+				selected_node.EnsureVisible ();
+
 				OnAfterSelect (new TreeViewEventArgs (value, TreeViewAction.Unknown));
 			}
 		}
 
+		private Rectangle Bloat (Rectangle rect)
+		{
+			rect.Y--;
+			rect.X--;
+			rect.Height += 2;
+			rect.Width += 2;
+			return rect;
+		}
+
 		[DefaultValue(true)]
 		public bool ShowLines {
 			get { return show_lines; }
@@ -369,6 +397,7 @@ namespace System.Windows.Forms {
 					sorted = value;
 				if (sorted) {
 					Nodes.Sort ();
+					top_node = null;
 					Refresh ();
 				}
 			}
@@ -414,24 +443,24 @@ namespace System.Windows.Forms {
 
 		#region Public Instance Methods
 		public void BeginUpdate () {
-			if (!IsHandleCreated)
-				return;
 			update_stack++;
 		}
 
-		public void CollapseAll () {
+		public void CollapseAll ()
+		{
 			root_node.CollapseAll ();
 		}
 
-		public void EndUpdate () {
-			if (!IsHandleCreated)
-				return;
-
+		public void EndUpdate ()
+		{
 			if (update_stack > 1) {
 				update_stack--;
 			} else {
 				update_stack = 0;
-				Refresh ();
+				if (update_needed) {
+					Invalidate (ViewportRectangle);
+					update_needed = false;
+				}
 			}
 		}
 
@@ -445,7 +474,7 @@ namespace System.Windows.Forms {
 
 		public TreeNode GetNodeAt (int x, int y) {
 			TreeNode node = GetNodeAt (y);
-			if (node == null || !IsTextArea (node, x))
+			if (node == null || !(IsTextArea (node, x) || full_row_select))
 				return null;
 			return node;
 					
@@ -457,7 +486,7 @@ namespace System.Windows.Forms {
 
 		public override string ToString () {
 			int count = Nodes.Count;
-			if (count < 0)
+			if (count <= 0)
 				return String.Concat (base.ToString (), "Node Count: 0");
 			return String.Concat (base.ToString (), "Node Count: ", count, " Nodes[0]: ", Nodes [0]);
 						
@@ -471,10 +500,9 @@ namespace System.Windows.Forms {
 		}
 
 		protected override void Dispose (bool disposing) {
-			if (disposing) {
-				if (image_list != null)
-					image_list.Dispose ();
-			}
+			if (disposing)
+				image_list = null;
+
 			base.Dispose (disposing);
 		}
 
@@ -483,24 +511,149 @@ namespace System.Windows.Forms {
 		}
 
 		protected override bool IsInputKey (Keys key_data) {
-			if (label_edit && (key_data & Keys.Alt) == 0) {
+
+			if ((key_data & Keys.Alt) == 0) {
 				switch (key_data & Keys.KeyCode) {
-					case Keys.Enter:
-					case Keys.Escape:
-					case Keys.Prior:
-					case Keys.Next:
-					case Keys.End:
-					case Keys.Home:
-					case Keys.Left:
-					case Keys.Up:
-					case Keys.Right:
-					case Keys.Down:
-						return true;
+				case Keys.Enter:
+				case Keys.Escape:
+				case Keys.Prior:
+				case Keys.Next:
+				case Keys.End:
+				case Keys.Home:
+				case Keys.Left:
+				case Keys.Up:
+				case Keys.Right:
+				case Keys.Down:
+					return true;
 				}
 			}
 			return base.IsInputKey (key_data);
 		}
 
+		protected override void OnKeyDown (KeyEventArgs e)
+		{
+			OpenTreeNodeEnumerator ne;
+
+			switch (e.KeyData & Keys.KeyCode) {
+			case Keys.Add:
+				if (selected_node != null && selected_node.IsExpanded)
+					selected_node.Expand ();
+				break;
+			case Keys.Subtract:
+				if (selected_node != null && selected_node.IsExpanded)
+					selected_node.Collapse ();
+				break;
+			case Keys.Left:
+				if (selected_node != null) {
+					if (selected_node.IsExpanded)
+						selected_node.Collapse ();
+					else {
+						ne = new OpenTreeNodeEnumerator (selected_node);
+						if (ne.MovePrevious () && ne.MovePrevious ())
+							SelectedNode = ne.CurrentNode;
+					}
+				}
+				break;
+			case Keys.Right:
+				if (selected_node != null) {
+					if (!selected_node.IsExpanded)
+						selected_node.Expand ();
+					else {
+						ne = new OpenTreeNodeEnumerator (selected_node);
+						if (ne.MoveNext () && ne.MoveNext ())
+							SelectedNode = ne.CurrentNode;
+					}
+				}
+				break;
+			case Keys.Up:
+				if (selected_node != null) {
+					ne = new OpenTreeNodeEnumerator (selected_node);
+					if (ne.MovePrevious () && ne.MovePrevious ())
+						SelectedNode = ne.CurrentNode;
+				}
+				break;
+			case Keys.Down:
+				if (selected_node != null) {
+					ne = new OpenTreeNodeEnumerator (selected_node);
+					if (ne.MoveNext () && ne.MoveNext ())
+						SelectedNode = ne.CurrentNode;
+				}
+				break;
+			case Keys.Home:
+				if (root_node.Nodes.Count > 0) {
+					ne = new OpenTreeNodeEnumerator (root_node.Nodes [0]);
+					if (ne.MoveNext ())
+						SelectedNode = ne.CurrentNode;
+				}
+				break;
+			case Keys.End:
+				if (root_node.Nodes.Count > 0) {
+					ne = new OpenTreeNodeEnumerator (root_node.Nodes [0]);
+					while (ne.MoveNext ())
+					{ }
+					SelectedNode = ne.CurrentNode;
+				}
+				break;
+			case Keys.PageDown:
+				if (selected_node != null) {
+					ne = new OpenTreeNodeEnumerator (selected_node);
+					int move = ViewportRectangle.Height / ItemHeight;
+					for (int i = 0; i < move && ne.MoveNext (); i++) {
+						
+					}
+					SelectedNode = ne.CurrentNode;
+				}
+				break;
+			case Keys.PageUp:
+				if (selected_node != null) {
+					ne = new OpenTreeNodeEnumerator (selected_node);
+					int move = ViewportRectangle.Height / ItemHeight;
+					for (int i = 0; i < move && ne.MovePrevious (); i++)
+					{ }
+					SelectedNode = ne.CurrentNode;
+				}
+				break;
+			case Keys.Multiply:
+				if (selected_node != null)
+					selected_node.ExpandAll ();
+				break;
+			}
+			base.OnKeyDown (e);
+
+			if (!e.Handled && checkboxes &&
+					selected_node != null &&
+					(e.KeyData & Keys.KeyCode) == Keys.Space) {
+				TreeViewCancelEventArgs args = new TreeViewCancelEventArgs (
+					selected_node, false, TreeViewAction.ByKeyboard);
+				OnBeforeCheck (args);
+				if (!args.Cancel) {
+					selected_node.Checked = !selected_node.Checked;
+					OnAfterCheck (new TreeViewEventArgs (selected_node, TreeViewAction.ByKeyboard));
+				}
+				e.Handled = true;
+			}
+		}
+
+		protected override void OnKeyPress (KeyPressEventArgs e)
+		{
+			base.OnKeyPress (e);
+			if (e.KeyChar == ' ')
+				e.Handled = true;
+		}
+
+		protected override void OnKeyUp (KeyEventArgs e)
+		{
+			base.OnKeyUp (e);
+			if ((e.KeyData & Keys.KeyCode) == Keys.Space)
+				e.Handled = true;
+		}
+
+		protected virtual void OnItemDrag (ItemDragEventArgs e)
+		{
+			if (ItemDrag != null)
+				ItemDrag (this, e);
+		}
+
 		protected virtual void OnAfterCheck (TreeViewEventArgs e) {
 			if (on_after_check != null)
 				on_after_check (this, e);
@@ -553,6 +706,19 @@ namespace System.Windows.Forms {
 
 		protected override void OnHandleCreated (EventArgs e) {
 			base.OnHandleCreated (e);
+
+			vbar = new ImplicitVScrollBar ();
+			hbar = new ImplicitHScrollBar ();
+
+			vbar.Visible = false;
+			hbar.Visible = false;
+			vbar.ValueChanged += new EventHandler (VScrollBarValueChanged);
+			hbar.ValueChanged += new EventHandler (HScrollBarValueChanged);
+
+			SuspendLayout ();
+			Controls.AddImplicit (vbar);
+			Controls.AddImplicit (hbar);
+			ResumeLayout ();
 		}
 
 		protected override void OnHandleDestroyed (EventArgs e) {
@@ -561,17 +727,20 @@ namespace System.Windows.Forms {
 
 		protected override void WndProc(ref Message m) {
 			switch ((Msg) m.Msg) {
-				case Msg.WM_PAINT: {				
-					PaintEventArgs	paint_event;
+			case Msg.WM_PAINT: {				
+				PaintEventArgs	paint_event;
 
-					paint_event = XplatUI.PaintEventStart (Handle);
-					DoPaint (paint_event);
-					XplatUI.PaintEventEnd (Handle);
-					return;
-				}
-				case Msg.WM_LBUTTONDBLCLK:
-					int val = m.LParam.ToInt32();
-					DoubleClickHandler (null, new MouseEventArgs (MouseButtons.Left, 2, val & 0xffff, (val>>16) & 0xffff, 0));
+				paint_event = XplatUI.PaintEventStart (Handle, true);
+				DoPaint (paint_event);
+				XplatUI.PaintEventEnd (Handle, true);
+				return;
+			}
+			case Msg.WM_LBUTTONDBLCLK:
+				int val = m.LParam.ToInt32();
+				DoubleClickHandler (null, new
+						MouseEventArgs (MouseButtons.Left,
+								2, val & 0xffff,
+								(val>>16) & 0xffff, 0));
 					break;
 			}
 			base.WndProc (ref m);
@@ -588,11 +757,6 @@ namespace System.Windows.Forms {
 			}
 		}
 
-		internal int TotalNodeCount {
-			get { return total_node_count; }
-			set { total_node_count = value; }
-		}
-
 		internal IntPtr CreateNodeHandle ()
 		{
 			return (IntPtr) handle_count++;
@@ -618,7 +782,7 @@ namespace System.Windows.Forms {
 		}
 
 		// TODO: we shouldn't have to compute this on the fly
-		private Rectangle ViewportRectangle {
+	        internal Rectangle ViewportRectangle {
 			get {
 				Rectangle res = ClientRectangle;
 
@@ -630,8 +794,10 @@ namespace System.Windows.Forms {
 			}
 		}
 
-		[MonoTODO ("Need to know if we are editing, not if editing is enabled")]
-		private TreeNode GetNodeAt (int y) {
+		private TreeNode GetNodeAt (int y)
+		{
+			if (nodes.Count <= 0)
+				return null;
 
 			if (top_node == null)
 				top_node = nodes [0];
@@ -646,12 +812,27 @@ namespace System.Windows.Forms {
 			return o.CurrentNode;
 		}
 
-		private bool IsTextArea (TreeNode node, int x) {
+		private bool IsTextArea (TreeNode node, int x)
+		{
 			return node != null && node.Bounds.Left <= x && node.Bounds.Right >= x;
 		}
 
+		private bool IsSelectableArea (TreeNode node, int x)
+		{
+			if (node == null)
+				return false;
+			int l = node.Bounds.Left;
+			if (ImageList != null)
+				l -= ImageList.ImageSize.Width;
+			return l <= x && node.Bounds.Right >= x;
+				
+		}
+
 		private bool IsPlusMinusArea (TreeNode node, int x)
 		{
+			if (node.Nodes.Count == 0 || (node.parent == root_node && !show_root_lines))
+				return false;
+
 			int l = node.Bounds.Left + 5;
 
 			if (show_root_lines || node.Parent != null)
@@ -687,32 +868,72 @@ namespace System.Windows.Forms {
 
 		internal void SetBottom (TreeNode node)
 		{
-			int visible = ClientRectangle.Height / ItemHeight;
-
 			OpenTreeNodeEnumerator walk = new OpenTreeNodeEnumerator (node);
-			TreeNode top = null;
 
-			visible--;
-			while (visible-- > 0 && walk.MovePrevious ())
-				top = walk.CurrentNode;
-
-			if (top != null)
-				SetTop (top);
+			int bottom = ViewportRectangle.Bottom;
+			int offset = 0;
+			while (walk.MovePrevious ()) {
+				if (walk.CurrentNode.Bounds.Bottom <= bottom)
+					break;
+				offset++;
+			}
+			vbar.Value += offset;
 		}
 
 		internal void UpdateBelow (TreeNode node)
 		{
+			if (update_stack > 0) {
+				update_needed = true;
+				return;
+			}
+
+			if (node == root_node) {
+				Refresh ();
+				return;
+			}
+				
 			// We need to update the current node so the plus/minus block gets update too
-			Rectangle invalid = new Rectangle (0, node.Bounds.Top, Width, Height - node.Bounds.Top);
+			Rectangle invalid = new Rectangle (0, node.Bounds.Top - 1,
+					Width, Height - node.Bounds.Top + 1);
 			Invalidate (invalid);
 		}
 
 		internal void UpdateNode (TreeNode node)
 		{
-			Rectangle invalid = new Rectangle (0, node.Bounds.Top, Width, node.Bounds.Height);
+			if (update_stack > 0) {
+				update_needed = true;
+				return;
+			}
+
+			if (node == root_node) {
+				Refresh ();
+				return;
+			}
+
+			Rectangle invalid = new Rectangle (0, node.Bounds.Top - 1, Width,
+					node.Bounds.Height + 1);
 			Invalidate (invalid);
 		}
 
+		internal void UpdateNodePlusMinus (TreeNode node)
+		{
+			if (update_stack > 0) {
+				update_needed = true;
+				return;
+			}
+
+			int l = node.Bounds.Left + 5;
+
+			if (show_root_lines || node.Parent != null)
+				l -= indent;
+			if (ImageList != null)
+				l -= ImageList.ImageSize.Width + 3;
+			if (checkboxes)
+				l -= 19;
+
+			Invalidate (new Rectangle (l, node.Bounds.Top, 8, node.Bounds.Height));
+		}
+
 		private void DoPaint (PaintEventArgs pe)
 		{
 			if (Width <= 0 || Height <=  0 || Visible == false)
@@ -725,6 +946,10 @@ namespace System.Windows.Forms {
 		{
 			if (top_node == null && Nodes.Count > 0)
 				top_node = nodes [0];
+
+			if (selected_node == null && Nodes.Count > 0)
+				SelectedNode = nodes [0];
+
 			// Decide if we need a scrollbar
 			int old_open_node_count = open_node_count;
 
@@ -734,14 +959,20 @@ namespace System.Windows.Forms {
 			
 			dc.FillRectangle (ThemeEngine.Current.ResPool.GetSolidBrush (BackColor), clip);
 
+			Color dash_color = ControlPaint.Dark (BackColor);
+			if (dash_color == BackColor)
+				dash_color = ControlPaint.Light (BackColor);
+			dash = new Pen (dash_color, 1);
+			dash.DashStyle = DashStyle.Dot;
+
 			int depth = 0;
 			int item_height = ItemHeight;
-			Font font = Font;
 			int height = ClientRectangle.Height;
 
 			open_node_count = 0;
+			used_height = 0;
 			foreach (TreeNode node in nodes) {
-				DrawNode (node, dc, clip, ref depth, item_height, font, height);
+				DrawNode (node, dc, clip, ref depth, item_height, height);
 				depth = 0;
 			}
 
@@ -754,7 +985,7 @@ namespace System.Windows.Forms {
 				add_hscroll = max_node_width > ClientRectangle.Width - ThemeEngine.Current.VScrollBarDefaultSize.Width;
 			if (add_hscroll)
 				add_vscroll = (open_node_count * ItemHeight) > ClientRectangle.Height - ThemeEngine.Current.HScrollBarDefaultSize.Height;
-                        
+			
 			if (add_hscroll) {
 				AddHorizontalScrollBar ();
 			} else if (hbar != null) {
@@ -772,15 +1003,12 @@ namespace System.Windows.Forms {
 			if (add_hscroll && add_vscroll) {
 				Rectangle corner = new Rectangle (hbar.Right, vbar.Bottom, vbar.Width, hbar.Height);
 				if (clip.IntersectsWith (corner))
-					dc.FillRectangle (ThemeEngine.Current.ResPool.GetSolidBrush (ThemeEngine.Current.ColorButtonFace), corner);
+					dc.FillRectangle (ThemeEngine.Current.ResPool.GetSolidBrush (ThemeEngine.Current.ColorControl), corner);
 			}
 		}
 
-		private void DrawNodePlusMinus (TreeNode node, Graphics dc, Rectangle clip, int x, int y, int middle)
+		private void DrawNodePlusMinus (TreeNode node, Graphics dc, int x, int middle)
 		{
-			if (!RectsIntersect (clip, x, middle - 4, 8, 8))
-				return;
-
 			dc.DrawRectangle (SystemPens.ControlDark, x, middle - 4, 8, 8);
 
 			if (node.IsExpanded) {
@@ -791,29 +1019,24 @@ namespace System.Windows.Forms {
 			}
 		}
 
-		private void DrawNodeCheckBox (TreeNode node, Graphics dc, Rectangle clip, int x, int y)
+		private void DrawNodeCheckBox (TreeNode node, Graphics dc, int x, int middle)
 		{
-			int offset = (ItemHeight - 13);
-
-			// new rectangle that factors in line width
-			if (!RectsIntersect (clip, x + 3, y + offset, 12, 12))
-				return;
-
-			dc.DrawRectangle (new Pen (Color.Black, 2), x + 0.5F + 3, y + 0.5F + offset, 11, 11);
+			using(Pen pen = new Pen (Color.Black, 2) ) 
+				dc.DrawRectangle (pen, x+ 3, middle - 4, 11, 11);
 
 			if (node.Checked) {
-				Pen check_pen = new Pen (Color.Black, 1);
-
-				dc.DrawLine (check_pen, x + 6, y + offset + 5, x + 8, y + offset + 8);
-				dc.DrawLine (check_pen, x + 6, y + offset + 6, x + 8, y + offset + 9);
-
-				dc.DrawLine (check_pen, x + 7, y + offset + 8, x + 13, y + offset + 3);
-				dc.DrawLine (check_pen, x + 7, y + offset + 9, x + 13, y + offset + 4);
+				using(Pen check_pen = new Pen (Color.Black, 1)) {
+					dc.DrawLine (check_pen, x + 6, middle + 0, x + 8, middle + 3);
+					dc.DrawLine (check_pen, x + 6, middle + 1, x + 8, middle + 4);
+	
+					dc.DrawLine (check_pen, x + 7, middle + 3, x + 13, middle - 2);
+					dc.DrawLine (check_pen, x + 7, middle + 4, x + 13, middle - 1);
+				}
 			}
 		}
 
 		private void DrawNodeLines (TreeNode node, Graphics dc, bool visible, Pen dash, int x, int y,
-                                int middle, int item_height, int node_count)
+				int middle, int item_height, int node_count)
 		{
 			int ladjust = 9; // left adjust
 			int radjust = 0; // right adjust
@@ -837,7 +1060,7 @@ namespace System.Windows.Forms {
 				dc.DrawLine (dash, x - indent + 9, middle - myadjust, x - indent + 9, ly);
 			} else if (node.Parent != null) {
 				int myadjust = (node.Nodes.Count > 0 && show_plus_minus ? 4 : 0);
-				ly = node.Parent.Bounds.Bottom - 1;
+				ly = node.Parent.Bounds.Bottom + 1;
 				dc.DrawLine (dash, x - indent + 9, middle - myadjust, x - indent + 9, ly);
 			}
 		}
@@ -867,7 +1090,7 @@ namespace System.Windows.Forms {
 				edit_text_box.BorderStyle = BorderStyle.FixedSingle;
 				edit_text_box.KeyUp += new KeyEventHandler (EditTextBoxKeyDown);
 				edit_text_box.Leave += new EventHandler (EditTextBoxLeave);
-				Controls.Add (edit_text_box);
+				Controls.AddImplicit (edit_text_box);
 			}
 
 			edit_text_box.Bounds = node.Bounds;
@@ -896,17 +1119,48 @@ namespace System.Windows.Forms {
 		{
 			edit_text_box.Visible = false;
 			edit_node.EndEdit (false);
-			Invalidate (edit_node.Bounds);
+			UpdateNode(edit_node);
 		}
 
-		private void UpdateNodeBounds (TreeNode node, int x, int y, int item_height)
+		private void UpdateNodeBounds (TreeNode node, int x, int y, int item_height, Graphics dc)
 		{
-			int width = (int) (node.Text.Length * Font.Size);
-			node.UpdateBounds (x, y, width, item_height);
+			Font font = node.NodeFont;
+			if (node.NodeFont == null)
+				font = Font;
+
+			if (node.NeedsWidth || update_node_bounds)
+				node.SetWidth ((int) dc.MeasureString (node.Text, font, 0,
+							       string_format).Width + 3);
+			node.SetHeight (item_height);
+			node.SetPosition (x, y);
+		}
+		
+		private void DrawSelectionAndFocus(TreeNode node, Graphics dc, Rectangle r)
+		{
+			if ((!HideSelection || Focused) && SelectedNode == node)
+				dc.FillRectangle (ThemeEngine.Current.ResPool.GetSolidBrush (ThemeEngine.Current.ColorHighlight), r);
+			else
+				dc.FillRectangle (ThemeEngine.Current.ResPool.GetSolidBrush (node.BackColor), r);
+			if (Focused && focused_node == node)
+				ControlPaint.DrawFocusRectangle (dc, r, ForeColor, BackColor);
+		}
+		 
+		private void DrawStaticNode (TreeNode node, Graphics dc)
+		{
+			if (!full_row_select)
+				DrawSelectionAndFocus(node, dc, node.Bounds);
+
+			Font font = node.NodeFont;
+			if (node.NodeFont == null)
+				font = Font;
+			Color text_color = ((Focused || !HideSelection) && SelectedNode == node ?
+					ThemeEngine.Current.ColorHighlightText : node.ForeColor);
+			dc.DrawString (node.Text, font,
+					ThemeEngine.Current.ResPool.GetSolidBrush (text_color),
+					node.Bounds, string_format);
 		}
 
-		private void DrawNode (TreeNode node, Graphics dc, Rectangle clip, ref int depth, int item_height,
-				Font font, int max_height)
+		private void DrawNode (TreeNode node, Graphics dc, Rectangle clip, ref int depth, int item_height, int max_height)
 		{
 			open_node_count++;
 			int x = (!show_root_lines && node.Parent != null ? depth  - 1 : depth) * indent - hbar_offset;
@@ -919,11 +1173,16 @@ namespace System.Windows.Forms {
 			if (clip.Top > y + ItemHeight || clip.Bottom < y)
 				visible = false;
 
+			if (visible && full_row_select) {
+				Rectangle r = new Rectangle(1, y+2, ViewportRectangle.Width-2, item_height);
+				DrawSelectionAndFocus(node, dc, r);
+			}
+
 			if (show_root_lines || node.Parent != null) {
 				x += 5;
 				if (_n_count > 0) {
 					if (show_plus_minus && visible) {
-						DrawNodePlusMinus (node, dc, clip, x, y, middle);
+						DrawNodePlusMinus (node, dc, x, middle);
 					}
 				}
 				x += indent - 5; 
@@ -931,48 +1190,33 @@ namespace System.Windows.Forms {
 
 			int ox = x;
 
-			if (visible && checkboxes) {
-				DrawNodeCheckBox (node, dc, clip, ox, y);
+			if (checkboxes) {
+				if (visible)
+					DrawNodeCheckBox (node, dc, ox, middle);
 				ox += 19;
 			}
 
 			if (show_lines)
 				DrawNodeLines (node, dc, visible, dash, x, y, middle, item_height, _n_count);
 
-			if (visible && ImageList != null) {
-				if (visible)
-					DrawNodeImage (node, dc, clip, ox, y);
+			if (ImageList != null) {
+                                if (visible)
+                                        DrawNodeImage (node, dc, clip, ox, y);
 				// MS leaves the space for the image if the ImageList is
 				// non null regardless of whether or not an image is drawn
 				ox += ImageList.ImageSize.Width + 3; // leave a little space so the text isn't against the image
 			}
 
-			UpdateNodeBounds (node, ox, y, item_height);
+			UpdateNodeBounds (node, ox, y, item_height, dc);
 
-			bool bounds_in_clip = clip.IntersectsWith (node.Bounds);
-			if (visible &&	bounds_in_clip && !node.IsEditing) {
-				Rectangle r = node.Bounds;
-				StringFormat format = new StringFormat ();
-				format.LineAlignment = StringAlignment.Center;
-
-				r.Y += 2; // we have to adjust this to get nice middle alignment
-				
-				Color text_color = (Focused && SelectedNode == node ? ThemeEngine.Current.ColorHilightText : node.ForeColor);
-				if (Focused) {
-					if (SelectedNode == node)
-						dc.FillRectangle (ThemeEngine.Current.ResPool.GetSolidBrush (ThemeEngine.Current.ColorHilight), r);
-					if (focused_node == node)
-                                                ControlPaint.DrawFocusRectangle (dc, r, ForeColor, BackColor);
-				} else {
-					if (!HideSelection && SelectedNode == node)
-						dc.FillRectangle (ThemeEngine.Current.ResPool.GetSolidBrush (ThemeEngine.Current.ColorButtonFace), r);
-				}
-				dc.DrawString (node.Text, font, ThemeEngine.Current.ResPool.GetSolidBrush (text_color), r, format);
-				y += item_height + 1;
-			} else if (visible && bounds_in_clip) {
-				DrawEditNode (node);
+			bool bounds_in_clip = clip.IntersectsWith (node.Bounds) || full_row_select;
+			if (visible &&	bounds_in_clip) {
+				if (node.IsEditing)
+					DrawEditNode (node);
+				else
+					DrawStaticNode (node, dc);
 			}
-
+			
 			if (node.Bounds.Right > max_node_width) {
 				max_node_width = node.Bounds.Right;
 				if (max_node_width > ClientRectangle.Width && !add_hscroll) {
@@ -981,11 +1225,14 @@ namespace System.Windows.Forms {
 				}
 			}
 
+			if (node.Bounds.Bottom > used_height)
+				used_height = node.Bounds.Bottom;
+
 			depth++;
 			if (node.IsExpanded) {
 				for (int i = 0; i < _n_count; i++) {
 					int tdepth = depth;
-					DrawNode (node.nodes [i], dc, clip, ref tdepth, item_height, font, max_height);
+					DrawNode (node.nodes [i], dc, clip, ref tdepth, item_height, max_height);
 				}
 			}
 
@@ -993,11 +1240,6 @@ namespace System.Windows.Forms {
 
 		private void AddVerticalScrollBar (int total_nodes, bool count_changed)
 		{
-			if (vbar == null) {
-				vbar = new VScrollBar ();
-				count_changed = true;
-			}
-
 			vbar.Bounds = new Rectangle (ClientRectangle.Width - vbar.Width,
 				0, vbar.Width, (add_hscroll ? Height - ThemeEngine.Current.HScrollBarDefaultSize.Height : Height));
 
@@ -1007,29 +1249,23 @@ namespace System.Windows.Forms {
 				vbar.LargeChange = height / ItemHeight;
 			}
 
-			if (!vbar_added) {
-				Controls.Add (vbar);
-				vbar.ValueChanged += new EventHandler (VScrollBarValueChanged);
-				vbar_added = true;
-			}
-
 			vbar.Visible = true;
 		}
 
 		private void AddHorizontalScrollBar ()
 		{
-			if (hbar == null)
-				hbar = new HScrollBar ();
-
-			hbar.Bounds = new Rectangle (ClientRectangle.Left, ClientRectangle.Bottom - hbar.Height,
+			hbar.Bounds = new Rectangle (ClientRectangle.Left,
+					ClientRectangle.Bottom - hbar.Height,
 					(add_vscroll ? Width - ThemeEngine.Current.VScrollBarDefaultSize.Width : Width), hbar.Height);
 
-			if (!hbar_added) {
-				Controls.Add (hbar);
-				hbar.ValueChanged += new EventHandler (HScrollBarValueChanged);
-				hbar_added = true;
-			}
+			int width = ClientRectangle.Width;
+			if (vbar.Visible)
+				width -= vbar.Width;
 
+			hbar.SmallChange = 15;
+			hbar.LargeChange = 50;
+			int num_pixels = max_node_width - width;
+			hbar.Maximum = num_pixels + 50;
 			hbar.Visible = true;
 		}
 
@@ -1037,17 +1273,24 @@ namespace System.Windows.Forms {
 		{
 			SuspendLayout ();
 
-			if (max_node_width > ClientRectangle.Width) {
+			if (max_node_width > ClientRectangle.Width && ClientRectangle.Width >= 0) {
 				add_hscroll = true;
 				AddHorizontalScrollBar ();
+				return;
 			}
 
-			if (vbar != null) {
+			if (used_height > ClientRectangle.Height && ClientRectangle.Height >= 0) {
+				add_vscroll = true;
+				AddVerticalScrollBar (open_node_count, true);
+				return;
+			}
+
+			if (vbar != null && vbar.Visible) {
 				int height = (hbar != null && hbar.Visible ? Height - hbar.Height : Height);
 				vbar.SetBounds (Right - vbar.Width, 0, 0, height, BoundsSpecified.X | BoundsSpecified.Height);
 			}
 
-			if (hbar != null) {
+			if (hbar != null && hbar.Visible) {
 				int width = (vbar != null && vbar.Visible ? Width - vbar.Width : Width);
 				hbar.SetBounds (0, Bottom - hbar.Height, width, 0, BoundsSpecified.Y | BoundsSpecified.Width);
 			}
@@ -1062,6 +1305,9 @@ namespace System.Windows.Forms {
 
 		private void SetVScrollPos (int pos, TreeNode new_top)
 		{
+			if (pos < 0)
+				pos = 0;
+
 			if (skipped_nodes == pos)
 				return;
 
@@ -1096,23 +1342,21 @@ namespace System.Windows.Forms {
 			int old_offset = hbar_offset;
 			hbar_offset = hbar.Value;
 
+			if (hbar_offset < 0)
+				hbar_offset = 0;
+
 			XplatUI.ScrollWindow (Handle, ViewportRectangle, old_offset - hbar_offset, 0, false);
 		}
 
-		private int GetOpenNodeCount ()
+		private void FontChangedHandler (object sender, EventArgs e)
 		{
+			update_node_bounds = true;
+		}
 
-			if (Nodes.Count < 1)
-				return 0;
-
-			OpenTreeNodeEnumerator e = new OpenTreeNodeEnumerator (root_node.Nodes [0]);
-
-			int count = 0;
-			while (e.MoveNext ()) {
-				count++;
-			}
-
-			return count;
+		private void LostFocusHandler (object sender, EventArgs e)
+		{
+			if (selected_node != null)
+				UpdateNode (selected_node);
 		}
 
 		private void MouseDownHandler (object sender, MouseEventArgs e)
@@ -1121,37 +1365,51 @@ namespace System.Windows.Forms {
 			if (node == null)
 				return;
 
-			if (IsTextArea (node, e.X)) {
-				TreeNode old_selected = selected_node;
-				selected_node = node;
-				if (label_edit && e.Clicks == 1 && selected_node == old_selected) {
-					Rectangle invalid = node.Bounds;
-					node.BeginEdit ();
-					if (edit_node != null) {
-						invalid = Rectangle.Union (invalid, edit_node.Bounds);
-						edit_node.EndEdit (false);
-					}
-					edit_node = node;
-					Invalidate (selected_node.Bounds);
-				} else if (selected_node != focused_node) {
-					select_mmove = true;
-					Rectangle invalid = (old_selected == null ? Rectangle.Empty : old_selected.Bounds);
-					invalid = Rectangle.Union (invalid, selected_node.Bounds);
-					Invalidate (invalid);
-				}
-			} else if (show_plus_minus && IsPlusMinusArea (node, e.X)) {
+			if (show_plus_minus && IsPlusMinusArea (node, e.X)) {
 				node.Toggle ();
 				return;
 			} else if (checkboxes && IsCheckboxArea (node, e.X)) {
 				node.Checked = !node.Checked;
+				UpdateNode(node);
 				return;
-			}
+			} else if (IsSelectableArea (node, e.X) || full_row_select) {
+				TreeNode old_selected = selected_node;
+				selected_node = node;
+				if (label_edit && e.Clicks == 1 && selected_node == old_selected) {
+					node.BeginEdit ();
+					if (edit_node != null) {
+						edit_node.EndEdit (false);
+						UpdateNode (edit_node);
+					}
+					edit_node = node;
+					UpdateNode (edit_node);
+				} else if (selected_node != focused_node) {
+					select_mmove = true;
+					
+					if (old_selected != null) {
+						Invalidate (Rectangle.Union (Bloat (old_selected.Bounds),
+									    Bloat (selected_node.Bounds)));
+					} else {
+						Invalidate (Bloat (selected_node.Bounds));
+					}
+				}
+
+				// We ensure its visible after we update because
+				// scrolling is used for insure visible
+				selected_node.EnsureVisible ();
+			} 
 		}
 
 		private void MouseUpHandler (object sender, MouseEventArgs e) {
+
+			drag_begin_x = -1;
+			drag_begin_y = -1;
+
+			OnClick (EventArgs.Empty);
+
 			if (!select_mmove)
 				return;
-				
+
 			select_mmove = false;
 
 			TreeViewCancelEventArgs ce = new TreeViewCancelEventArgs (selected_node, false, TreeViewAction.ByMouse);
@@ -1159,12 +1417,15 @@ namespace System.Windows.Forms {
 
 			Rectangle invalid;
 			if (!ce.Cancel) {
-				if (focused_node != null)
-					invalid = Rectangle.Union (focused_node.Bounds, selected_node.Bounds);
-				else
-					invalid = selected_node.Bounds;
+				if (focused_node != null) {
+					invalid = Rectangle.Union (Bloat (focused_node.Bounds),
+							Bloat (selected_node.Bounds));
+				} else {
+					invalid = Bloat (selected_node.Bounds);
+				}
 				focused_node = selected_node;
 				OnAfterSelect (new TreeViewEventArgs (selected_node, TreeViewAction.ByMouse));
+
 				Invalidate (invalid);
 			} else {
 				selected_node = focused_node;
@@ -1174,6 +1435,27 @@ namespace System.Windows.Forms {
 		}
 
 		private void MouseMoveHandler (object sender, MouseEventArgs e) {
+
+			if (e.Button == MouseButtons.Left || e.Button == MouseButtons.Right) {
+				if (drag_begin_x == -1 && drag_begin_y == -1) {
+					drag_begin_x = e.X;
+					drag_begin_y = e.Y;
+				} else {
+					double rise = Math.Pow (drag_begin_x - e.X, 2);
+					double run = Math.Pow (drag_begin_y - e.Y, 2);
+					double move = Math.Sqrt (rise + run);
+					if (move > 3) {
+						TreeNode drag = GetNodeAt (e.X, e.Y);
+						
+						if (drag != null) {
+							OnItemDrag (new ItemDragEventArgs (e.Button, drag));
+						}
+						drag_begin_x = -1;
+						drag_begin_y = -1;
+					}
+				}
+				
+			}
 			if(!select_mmove)
 				return;
 			TreeNode node = GetNodeAt(e.X,e.Y);
@@ -1202,6 +1484,8 @@ namespace System.Windows.Forms {
 		#endregion	// Internal & Private Methods and Properties
 
 		#region Events
+		public event ItemDragEventHandler ItemDrag;
+
 		public event TreeViewEventHandler AfterCheck {
 			add { on_after_check += value; }
 			remove { on_after_check -= value; }
@@ -1227,6 +1511,13 @@ namespace System.Windows.Forms {
 			remove { on_after_select -= value; }
 		}
 
+		[Browsable (false)]
+		[EditorBrowsable (EditorBrowsableState.Never)]	
+		public event EventHandler BackgroundImageChanged {
+			add { base.BackgroundImageChanged += value; }
+			remove { base.BackgroundImageChanged -= value; }
+		}
+
 		public event TreeViewCancelEventHandler BeforeCheck {
 			add { on_before_check += value; }
 			remove { on_before_check -= value; }
