@@ -67,7 +67,7 @@ namespace System.Windows.Forms {
 
 		#region Constructor & Destructor
 		static XplatUI() {
-			Console.WriteLine("Mono System.Windows.Forms Assembly [Revision: 44786; built: 2005/5/25 22:34:45]");
+			Console.WriteLine("Mono System.Windows.Forms Assembly [Revision: 54007; built: 2005/12/6 14:35:24]");
 
 			// Don't forget to throw the mac in here somewhere, too
 			default_class_name="SWFClass";
@@ -86,10 +86,8 @@ namespace System.Windows.Forms {
 
 			driver.InitializeDriver();
 
-		}
-
-		~XplatUI() {
-			Console.WriteLine("XplatUI Destructor called");
+			// Initialize things that need to be done after the driver is ready
+			DataFormats.GetFormat(0);
 		}
 		#endregion	// Constructor & Destructor
 
@@ -239,11 +237,18 @@ namespace System.Windows.Forms {
 			driver.Activate(handle);
 		}
 
-		internal static bool CalculateWindowRect(IntPtr handle, ref Rectangle ClientRect, int Style, int ExStyle, IntPtr MenuHandle, out Rectangle WindowRect) {
+		internal static void AudibleAlert() {
+			#if DriverDebug
+				Console.WriteLine("AudibleAlert(): Called");
+			#endif
+			driver.AudibleAlert();
+		}
+
+		internal static bool CalculateWindowRect(IntPtr handle, ref Rectangle ClientRect, int Style, int ExStyle, Menu menu, out Rectangle WindowRect) {
 			#if DriverDebug
 				Console.WriteLine("CalculateWindowRect({0:X}): Called", handle.ToInt32());
 			#endif
-			return driver.CalculateWindowRect(handle, ref ClientRect, Style, ExStyle, MenuHandle, out WindowRect);
+			return driver.CalculateWindowRect(handle, ref ClientRect, Style, ExStyle, menu, out WindowRect);
 		}
 
 		internal static void CaretVisible(IntPtr handle, bool visible) {
@@ -367,6 +372,13 @@ namespace System.Windows.Forms {
 			driver.DoEvents();
 		}
 
+		internal static void DrawReversibleRectangle(IntPtr handle, Rectangle rect, int line_width) {
+			#if DriverDebug
+				Console.WriteLine("DrawReversibleRectangle({0:X}, {1}, {2}): Called", handle.ToInt32(), rect, line_width);
+			#endif
+			driver.DrawReversibleRectangle(handle, rect, line_width);
+		}
+
 		internal static void EnableThemes() {
 			driver.EnableThemes();
 		}
@@ -378,14 +390,6 @@ namespace System.Windows.Forms {
 			driver.EnableWindow(handle, Enable);
 		}
 
-		internal static void Exit() {
-			driver.Exit();
-		}
-
-		internal static void EraseWindowBackground(IntPtr handle, IntPtr wParam) {
-			driver.EraseWindowBackground(handle, wParam);
-		}
-
 		internal static IntPtr GetActive() {
 			#if DriverDebug
 				Console.WriteLine("GetActive(): Called");
@@ -393,6 +397,13 @@ namespace System.Windows.Forms {
 			return driver.GetActive();
 		}
 
+		internal static SizeF GetAutoScaleSize(Font font) {
+			#if DriverDebug
+				Console.WriteLine("GetAutoScaleSize({0}): Called", font);
+			#endif
+			return driver.GetAutoScaleSize(font);
+		}
+
 		internal static void GetCursorInfo(IntPtr cursor, out int width, out int height, out int hotspot_x, out int hotspot_y) {
 			#if DriverDebug
 				Console.WriteLine("GetCursorInfo({0:X}): Called", cursor.ToInt32());
@@ -521,24 +532,28 @@ namespace System.Windows.Forms {
 			driver.OverrideCursor(cursor);
 		}
 
-		internal static void PaintEventEnd(IntPtr handle) {
+		internal static void PaintEventEnd(IntPtr handle, bool client) {
 			#if DriverDebug
-				Console.WriteLine("PaintEventEnd({0:X}): Called", handle.ToInt32());
+				Console.WriteLine("PaintEventEnd({0:X}, {1}): Called", handle.ToInt32(), client);
 			#endif
-			driver.PaintEventEnd(handle);
+			driver.PaintEventEnd(handle, client);
 		}
 
-		internal static PaintEventArgs PaintEventStart(IntPtr handle) {
+		internal static PaintEventArgs PaintEventStart(IntPtr handle, bool client) {
 			#if DriverDebug
-				Console.WriteLine("PaintEventStart({0:X}): Called", handle.ToInt32());
+				Console.WriteLine("PaintEventStart({0:X}, {1}): Called", handle.ToInt32(), client);
 			#endif
-			return driver.PaintEventStart(handle);
+			return driver.PaintEventStart(handle, client);
 		}
 
 		internal static bool PeekMessage(ref MSG msg, IntPtr hWnd, int wFilterMin, int wFilterMax, uint flags) {
 			return driver.PeekMessage(ref msg, hWnd, wFilterMin, wFilterMax, flags);
 		}
 
+		internal static void PostQuitMessage(int exitCode) {
+			driver.PostQuitMessage(exitCode);
+		}
+
 		internal static void ReleaseMenuDC(IntPtr handle, Graphics dc) {
 			#if DriverDebug
 				Console.WriteLine("ReleaseMenuDC({0:X}): Called", handle.ToInt32());
@@ -589,7 +604,7 @@ namespace System.Windows.Forms {
 			driver.SetAllowDrop (handle, value);
 		}
 
-		internal static void SetBorderStyle(IntPtr handle, BorderStyle border_style) {
+		internal static void SetBorderStyle(IntPtr handle, FormBorderStyle border_style) {
 			#if DriverDebug
 				Console.WriteLine("SetBorderStyle({0:X}, {1}): Called", handle.ToInt32(), border_style);
 			#endif
@@ -631,11 +646,11 @@ namespace System.Windows.Forms {
 			driver.SetIcon(handle, icon);
 		}
 
-		internal static void SetMenu(IntPtr handle, IntPtr menu_handle) {
+		internal static void SetMenu(IntPtr handle, Menu menu) {
 			#if DriverDebug
-				Console.WriteLine("SetMenu({0:X}, {1:X}): Called", handle.ToInt32(), menu_handle.ToInt32());
+				Console.WriteLine("SetMenu({0:X}, {1}): Called", handle.ToInt32(), menu);
 			#endif
-			driver.SetMenu(handle, menu_handle);
+			driver.SetMenu(handle, menu);
 		}
 
 		internal static void SetModal(IntPtr handle, bool Modal) {
@@ -695,13 +710,6 @@ namespace System.Windows.Forms {
 			driver.SetWindowStyle(handle, cp);
 		}
 
-		internal static void SetWindowBackground(IntPtr handle, Color color) {
-			#if DriverDebug
-				Console.WriteLine("SetWindowBackground({0:X}, {1}): Called", handle.ToInt32(), color);
-			#endif
-			driver.SetWindowBackground(handle, color);
-		}
-			
 		internal static bool SetZOrder(IntPtr handle, IntPtr AfterhWnd, bool Top, bool Bottom) {
 			#if DriverDebug
 				Console.WriteLine("SetZOrder({0:X}, {1:X}, {2}, {3}): Called", handle.ToInt32(), AfterhWnd.ToInt32(), Top, Bottom);
