@@ -684,31 +684,7 @@ namespace System.Windows.Forms {
 		}
 		
 		private void SetHwndStyles(Hwnd hwnd, CreateParams cp) {
-			if ((cp.Style & (int)WindowStyles.WS_CHILD) != 0) {
-				if ((cp.Style & (int)WindowStyles.WS_THICKFRAME) != 0) {
-					hwnd.BorderStyle = BorderStyle.Fixed3D;
-				} else if ((cp.Style & (int)WindowStyles.WS_BORDER) != 0) {
-					hwnd.BorderStyle = BorderStyle.FixedSingle;
-				}
-			}
-
-			if ((cp.ExStyle & (int)WindowStyles.WS_EX_CLIENTEDGE) != 0) {
-				hwnd.edge_style = Border3DStyle.Sunken;
-			} else if ((cp.ExStyle & (int)WindowStyles.WS_EX_STATICEDGE) != 0) {
-				hwnd.edge_style = Border3DStyle.Flat;
-			} else if ((cp.ExStyle & (int)WindowStyles.WS_EX_WINDOWEDGE) != 0) {
-				hwnd.edge_style = Border3DStyle.Raised;
-			} else if ((cp.ExStyle & (int)WindowStyles.WS_EX_WINDOWEDGE) != 0) {
-				hwnd.edge_style = Border3DStyle.Raised;
-			}
-
-			if ((cp.Style & (int)WindowStyles.WS_CAPTION) != 0) {
-				if ((cp.ExStyle & (int)WindowStyles.WS_EX_TOOLWINDOW) != 0) {
-					hwnd.title_style = TitleStyle.Tool;
-				} else {
-					hwnd.title_style = TitleStyle.Normal;
-				}
-			}
+			throw new NotImplementedException();
 		}
 		
 		internal void ShowCaret () {
@@ -789,7 +765,11 @@ namespace System.Windows.Forms {
 		internal override void Activate(IntPtr handle) {
 			ActivateWindow (GetControlOwner (handle), true);
 		}
-		
+
+		internal override void AudibleAlert() {
+			throw new NotImplementedException();
+		}
+
 		internal override void CaretVisible (IntPtr hwnd, bool visible) {
 			if (Caret.Hwnd == hwnd) {
 				if (visible) {
@@ -811,8 +791,8 @@ namespace System.Windows.Forms {
 			}
 		}
 		
-		internal override bool CalculateWindowRect(IntPtr hWnd, ref Rectangle ClientRect, int Style, int ExStyle, IntPtr MenuHandle, out Rectangle WindowRect) {
-			BorderStyle	border_style;
+		internal override bool CalculateWindowRect(IntPtr hWnd, ref Rectangle ClientRect, int Style, int ExStyle, Menu menu, out Rectangle WindowRect) {
+			FormBorderStyle	border_style;
 			TitleStyle	title_style;
 
 			title_style = TitleStyle.None;
@@ -824,16 +804,28 @@ namespace System.Windows.Forms {
 				}
 			}
 
-			border_style = BorderStyle.None;
-			if ((Style & (int)WindowStyles.WS_CHILD) != 0) {
-				if ((Style & (int)WindowStyles.WS_THICKFRAME) != 0) {
-					border_style = BorderStyle.Fixed3D;
-				} else if ((Style & (int)WindowStyles.WS_BORDER) != 0) {
-					border_style = BorderStyle.FixedSingle;
+			border_style = FormBorderStyle.None;
+			if ((ExStyle & (int)WindowStyles.WS_EX_WINDOWEDGE) != 0) {
+				if ((ExStyle & (int)WindowStyles.WS_EX_TOOLWINDOW) != 0) {
+					if ((Style & (int)WindowStyles.WS_THICKFRAME) != 0) {
+						border_style = FormBorderStyle.SizableToolWindow;
+					} else {
+						border_style = FormBorderStyle.FixedToolWindow;
+					}
+				} else if ((ExStyle & (int)WindowStyles.WS_EX_DLGMODALFRAME) != 0) {
+					border_style = FormBorderStyle.FixedDialog;
+				} else if ((ExStyle & (int)WindowStyles.WS_THICKFRAME) != 0) {
+					border_style = FormBorderStyle.Sizable;
+				} else {
+					border_style = FormBorderStyle.FixedSingle;
 				}
+			} else {
+				border_style = FormBorderStyle.Fixed3D;
 			}
 
-			WindowRect = Hwnd.GetWindowRectangle(border_style, MenuHandle, title_style, ClientRect);
+			WindowRect = Hwnd.GetWindowRectangle(border_style, menu, title_style,
+					SystemInformation.CaptionHeight,
+					SystemInformation.ToolWindowCaptionHeight, ClientRect);
 
 			return true;
 		}
@@ -1096,7 +1088,7 @@ namespace System.Windows.Forms {
 				case Msg.WM_DESTROY: {
 					if (WindowMapping [hwnd.Handle] != null)
 
-						XplatUI.Exit ();
+						Exit ();
 					break;
 				}
 			}
@@ -1146,12 +1138,7 @@ namespace System.Windows.Forms {
 			//Like X11 we need not do anything here
 		}
 
-		internal override void EraseWindowBackground(IntPtr handle, IntPtr wParam) {
-			throw new NotImplementedException();
-		}
-
-
-		internal override void Exit() {
+		internal void Exit() {
 			GetMessageResult = false;
 			ExitToShell ();
 		}
@@ -1341,7 +1328,7 @@ namespace System.Windows.Forms {
 			throw new NotImplementedException ();
 		}
 
-		internal override PaintEventArgs PaintEventStart(IntPtr handle) {
+		internal override PaintEventArgs PaintEventStart(IntPtr handle, bool client) {
 			PaintEventArgs	paint_event;
 			Hwnd		hwnd;
 
@@ -1351,13 +1338,14 @@ namespace System.Windows.Forms {
 				Caret.Paused = true;
 				HideCaret();
 			}
+
 			hwnd.client_dc  = Graphics.FromHwnd (hwnd.client_window);
 			paint_event = new PaintEventArgs(hwnd.client_dc, hwnd.invalid);
 			
 			return paint_event;
 		}
 		
-		internal override void PaintEventEnd(IntPtr handle) {
+		internal override void PaintEventEnd(IntPtr handle, bool client) {
 			Hwnd	hwnd;
 
 			hwnd = Hwnd.ObjectFromHandle(handle);
@@ -1388,6 +1376,11 @@ namespace System.Windows.Forms {
 			msg.lParam = lParam;
 			MessageQueue.Enqueue (msg);
 		}
+
+		[MonoTODO]
+		internal override void PostQuitMessage(int exitCode) {
+			throw new NotImplementedException();
+		}
 		
 		[MonoTODO]
 		internal override void ReleaseMenuDC(IntPtr hwnd, Graphics dc) {
@@ -1577,22 +1570,7 @@ namespace System.Windows.Forms {
 			return true;
 		}
 		
-		internal override void SetWindowBackground(IntPtr handle, Color color) {
-			Hwnd hwnd = Hwnd.ObjectFromHandle (handle);
-			
-			if (WindowMapping [hwnd.Handle] != null) {
-				RGBColor backColor = new RGBColor ();
-				backColor.red = (short)(color.R * 257); 
-				backColor.green = (short)(color.G * 257);
-				backColor.blue = (short)(color.B * 257);
-
-				CheckError (SetWindowContentColor ((IntPtr) WindowMapping [hwnd.Handle], ref backColor));
-			} else {
-				WindowBackgrounds [hwnd] = color;
-			}
-		}
-		
-		internal override void SetBorderStyle(IntPtr handle, BorderStyle border_style) {
+		internal override void SetBorderStyle(IntPtr handle, FormBorderStyle border_style) {
 			Hwnd	hwnd;
 
 			hwnd = Hwnd.ObjectFromHandle(handle);
@@ -1601,18 +1579,20 @@ namespace System.Windows.Forms {
 			// FIXME - do we need to trigger some resize?
 		}
 
-		internal override void SetMenu(IntPtr handle, IntPtr menu_handle) {
+		internal override void SetMenu(IntPtr handle, Menu menu) {
 			Hwnd	hwnd;
 
 			hwnd = Hwnd.ObjectFromHandle(handle);
-			hwnd.menu_handle = menu_handle;
+			hwnd.menu = menu;
 
 			// FIXME - do we need to trigger some resize?
 		}
 		
 		internal override void SetWindowPos(IntPtr handle, int x, int y, int width, int height) {
 			Hwnd hwnd = Hwnd.ObjectFromHandle (handle);
-			Rectangle client_rect = Hwnd.GetClientRectangle(hwnd.border_style, hwnd.menu_handle, hwnd.title_style, width, height);
+			Rectangle client_rect = Hwnd.GetClientRectangle(hwnd.border_style, hwnd.menu,
+					hwnd.title_style, SystemInformation.CaptionHeight,
+					SystemInformation.ToolWindowCaptionHeight, width, height);
 
 			// Save a server roundtrip (and prevent a feedback loop)
 			if ((hwnd.x == x) && (hwnd.y == y) && (hwnd.width == width) && (hwnd.height == height)) {
@@ -1789,6 +1769,14 @@ namespace System.Windows.Forms {
 			return true;
 		}
 		
+		internal override void DrawReversibleRectangle(IntPtr handle, Rectangle rect, int line_width) {
+			throw new NotImplementedException();
+		}
+
+		[MonoTODO]
+		internal override SizeF GetAutoScaleSize(Font font) {
+			throw new NotImplementedException();
+		}
 		#endregion
 		
 		#region System information
