@@ -210,38 +210,45 @@ namespace System.Windows.Forms {
 		}
 
 		internal enum GetSysColorIndex {
-			COLOR_SCROLLBAR			=0,
-			COLOR_BACKGROUND		=1,
-			COLOR_ACTIVECAPTION		=2,
-			COLOR_INACTIVECAPTION		=3,
-			COLOR_MENU			=4,
-			COLOR_WINDOW			=5,
-			COLOR_WINDOWFRAME		=6,
-			COLOR_MENUTEXT			=7,
-			COLOR_WINDOWTEXT		=8,
-			COLOR_CAPTIONTEXT		=9,
-			COLOR_ACTIVEBORDER		=10,
-			COLOR_INACTIVEBORDER		=11,
-			COLOR_APPWORKSPACE		=12,
-			COLOR_HIGHLIGHT			=13,
-			COLOR_HIGHLIGHTTEXT		=14,
-			COLOR_BTNFACE			=15,
-			COLOR_BTNSHADOW			=16,
-			COLOR_GRAYTEXT			=17,
-			COLOR_BTNTEXT			=18,
-			COLOR_INACTIVECAPTIONTEXT	=19,
-			COLOR_BTNHIGHLIGHT		=20,
-			COLOR_3DDKSHADOW		=21,
-			COLOR_3DLIGHT			=22,
-			COLOR_INFOTEXT			=23,
-			COLOR_INFOBK			=24,
-			COLOR_DESKTOP			=1,
-			COLOR_3DFACE			=16,
-			COLOR_3DSHADOW			=16,
-			COLOR_3DHIGHLIGHT		=20,
-			COLOR_3DHILIGHT			=20,
-			COLOR_BTNHILIGHT		=20,
-			COLOR_MAXVALUE			=24,/* Maximum value */
+			COLOR_SCROLLBAR			= 0,
+			COLOR_BACKGROUND		= 1,
+			COLOR_ACTIVECAPTION		= 2,
+			COLOR_INACTIVECAPTION		= 3,
+			COLOR_MENU			= 4,
+			COLOR_WINDOW			= 5,
+			COLOR_WINDOWFRAME		= 6,
+			COLOR_MENUTEXT			= 7,
+			COLOR_WINDOWTEXT		= 8,
+			COLOR_CAPTIONTEXT		= 9,
+			COLOR_ACTIVEBORDER		= 10,
+			COLOR_INACTIVEBORDER		= 11,
+			COLOR_APPWORKSPACE		= 12,
+			COLOR_HIGHLIGHT			= 13,
+			COLOR_HIGHLIGHTTEXT		= 14,
+			COLOR_BTNFACE			= 15,
+			COLOR_BTNSHADOW			= 16,
+			COLOR_GRAYTEXT			= 17,
+			COLOR_BTNTEXT			= 18,
+			COLOR_INACTIVECAPTIONTEXT	= 19,
+			COLOR_BTNHIGHLIGHT		= 20,
+			COLOR_3DDKSHADOW		= 21,
+			COLOR_3DLIGHT			= 22,
+			COLOR_INFOTEXT			= 23,
+			COLOR_INFOBK			= 24,
+			
+			COLOR_HOTLIGHT			= 26,
+			COLOR_GRADIENTACTIVECAPTION	= 27,
+			COLOR_GRADIENTINACTIVECAPTION	= 28,
+			COLOR_MENUHIGHLIGHT		= 29,
+			COLOR_MENUBAR			= 30,
+
+			COLOR_DESKTOP			= 1,
+			COLOR_3DFACE			= 16,
+			COLOR_3DSHADOW			= 16,
+			COLOR_3DHIGHLIGHT		= 20,
+			COLOR_3DHILIGHT			= 20,
+			COLOR_BTNHILIGHT		= 20,
+			COLOR_MAXVALUE			= 24,/* Maximum value */
 		}       
 
 		private enum LoadCursorType {
@@ -300,7 +307,7 @@ namespace System.Windows.Forms {
 			HS_DIAGCROSS        		= 5,       /* xxxxx */
 		}
 
-		private struct COLORREF {
+		internal struct COLORREF {
 			internal byte			B;
 			internal byte			G;
 			internal byte			R;
@@ -531,6 +538,83 @@ namespace System.Windows.Forms {
 			GPTR                		= (GMEM_FIXED | GMEM_ZEROINIT)
 		}
 
+		internal enum ROP2DrawMode : int {
+			R2_BLACK			= 1,
+			R2_NOTMERGEPEN      		= 2,
+			R2_MASKNOTPEN       		= 3,
+			R2_NOTCOPYPEN       		= 4,
+			R2_MASKPENNOT       		= 5,
+			R2_NOT              		= 6,
+			R2_XORPEN           		= 7,
+			R2_NOTMASKPEN       		= 8,
+			R2_MASKPEN          		= 9,
+			R2_NOTXORPEN        		= 10,
+			R2_NOP              		= 11,
+			R2_MERGENOTPEN      		= 12,
+			R2_COPYPEN          		= 13,
+			R2_MERGEPENNOT      		= 14,
+			R2_MERGEPEN         		= 15,
+			R2_WHITE            		= 16,
+			R2_LAST             		= 16
+		}
+
+		internal enum PenStyle : int {
+			PS_SOLID			= 0,
+			PS_DASH             		= 1,
+			PS_DOT              		= 2,
+			PS_DASHDOT          		= 3,
+			PS_DASHDOTDOT       		= 4,
+			PS_NULL             		= 5,
+			PS_INSIDEFRAME      		= 6,
+			PS_USERSTYLE        		= 7,
+			PS_ALTERNATE        		= 8
+		}
+
+		internal enum StockObject : int {
+			WHITE_BRUSH			= 0,
+			LTGRAY_BRUSH        		= 1,
+			GRAY_BRUSH          		= 2,
+			DKGRAY_BRUSH        		= 3,
+			BLACK_BRUSH         		= 4,
+			NULL_BRUSH          		= 5,
+			HOLLOW_BRUSH        		= NULL_BRUSH,
+			WHITE_PEN   			= 6,
+			BLACK_PEN           		= 7,
+			NULL_PEN            		= 8,
+			OEM_FIXED_FONT      		= 10,
+			ANSI_FIXED_FONT     		= 11,
+			ANSI_VAR_FONT       		= 12,
+			SYSTEM_FONT         		= 13,
+			DEVICE_DEFAULT_FONT 		= 14,
+			DEFAULT_PALETTE     		= 15,
+			SYSTEM_FIXED_FONT  		= 16
+		}
+
+		internal enum HatchStyle : int {
+			HS_HORIZONTAL			= 0,
+			HS_VERTICAL         		= 1,
+			HS_FDIAGONAL        		= 2,
+			HS_BDIAGONAL        		= 3,
+			HS_CROSS            		= 4,
+			HS_DIAGCROSS        		= 5
+		}
+
+		[Flags]
+		internal enum SndFlags : int {
+			SND_SYNC			= 0x0000,
+			SND_ASYNC			= 0x0001,
+			SND_NODEFAULT			= 0x0002,
+			SND_MEMORY			= 0x0004,
+			SND_LOOP			= 0x0008,
+			SND_NOSTOP			= 0x0010,
+			SND_NOWAIT     			= 0x00002000,
+			SND_ALIAS			= 0x00010000,
+			SND_ALIAS_ID			= 0x00110000,
+			SND_FILENAME			= 0x00020000,
+			SND_RESOURCE			= 0x00040004,
+			SND_PURGE			= 0x0040,
+			SND_APPLICATION			= 0x0080,
+		}
 		#endregion
 
 		#region Constructor & Destructor
@@ -910,8 +994,8 @@ namespace System.Windows.Forms {
 			Console.WriteLine("Xplat version $revision: $");
 		}
 
-		internal override void Exit() {
-			Win32PostQuitMessage(0);
+		internal override void AudibleAlert() {
+			Win32PlaySound("Default", IntPtr.Zero, SndFlags.SND_ALIAS | SndFlags.SND_ASYNC | SndFlags.SND_NOSTOP | SndFlags.SND_NOWAIT);
 		}
 
 		internal override void GetDisplaySize(out Size size) {
@@ -961,7 +1045,7 @@ namespace System.Windows.Forms {
 
 			hwnd.ClientWindow = WindowHandle;
 
-			Win32SetWindowLong(WindowHandle, WindowLong.GWL_USERDATA, (IntPtr)ThemeEngine.Current.DefaultControlBackColor.ToArgb());
+			Win32SetWindowLong(WindowHandle, WindowLong.GWL_USERDATA, (uint)ThemeEngine.Current.DefaultControlBackColor.ToArgb());
 
 			return WindowHandle;
 		}
@@ -1025,20 +1109,16 @@ namespace System.Windows.Forms {
 		}
 
 		internal override void SetWindowStyle(IntPtr handle, CreateParams cp) {
-			Win32SetWindowLong(handle, WindowLong.GWL_STYLE, (IntPtr)cp.Style);
-			Win32SetWindowLong(handle, WindowLong.GWL_EXSTYLE, (IntPtr)cp.ExStyle);
+			Win32SetWindowLong(handle, WindowLong.GWL_STYLE, (uint)cp.Style);
+			Win32SetWindowLong(handle, WindowLong.GWL_EXSTYLE, (uint)cp.ExStyle);
 		}
 
 		internal override void UpdateWindow(IntPtr handle) {
 			Win32UpdateWindow(handle);
 		}
 
-		internal override void SetWindowBackground(IntPtr handle, Color color) {
-			Win32SetWindowLong(handle, WindowLong.GWL_USERDATA, (IntPtr)color.ToArgb());
-		}
-
-		[MonoTODO("Add support for internal table of windows/DCs for cleanup")]
-		internal override PaintEventArgs PaintEventStart(IntPtr handle) {
+		[MonoTODO("FIXME - Add support for internal table of windows/DCs for cleanup; handle client=false to draw in NC area")]
+		internal override PaintEventArgs PaintEventStart(IntPtr handle, bool client) {
 			IntPtr		hdc;
 			PAINTSTRUCT	ps;
 			PaintEventArgs	paint_event;
@@ -1057,14 +1137,7 @@ namespace System.Windows.Forms {
 
 				hwnd.user_data = (object)ps;
 
-				// FIXME: Figure out why the rectangle is always 0 size
 				clip_rect = new Rectangle(ps.rcPaint.left, ps.rcPaint.top, ps.rcPaint.right-ps.rcPaint.left, ps.rcPaint.bottom-ps.rcPaint.top);
-//				clip_rect = new Rectangle(rect.left, rect.top, rect.right-rect.left, rect.bottom-rect.top);
-
-				if (ps.fErase != 0) {
-Console.WriteLine("Hit Clear background");
-					EraseWindowBackground(handle, hdc);
-				}
 			} else {
 				hdc = Win32GetDC(handle);
 				// FIXME: Add the DC to internal list
@@ -1077,7 +1150,7 @@ Console.WriteLine("Hit Clear background");
 			return paint_event;
 		}
 
-		internal override void PaintEventEnd(IntPtr handle) {
+		internal override void PaintEventEnd(IntPtr handle, bool client) {
 			Hwnd		hwnd;
 			PAINTSTRUCT	ps;
 
@@ -1160,6 +1233,10 @@ Console.WriteLine("Hit Clear background");
 			return Win32PeekMessage(ref msg, hWnd, wFilterMin, wFilterMax, flags);
 		}
 
+		internal override void PostQuitMessage(int exitCode) {
+			Win32PostQuitMessage(exitCode);
+		}
+
 		internal override bool GetMessage(ref MSG msg, IntPtr hWnd, int wFilterMin, int wFilterMax) {
 			bool		result;
 
@@ -1333,7 +1410,7 @@ Console.WriteLine("Hit Clear background");
 			grab_hwnd = IntPtr.Zero;
 		}
 
-		internal override bool CalculateWindowRect(IntPtr hWnd, ref Rectangle ClientRect, int Style, int ExStyle, IntPtr MenuHandle, out Rectangle WindowRect) {
+		internal override bool CalculateWindowRect(IntPtr hWnd, ref Rectangle ClientRect, int Style, int ExStyle, Menu menu, out Rectangle WindowRect) {
 			RECT	rect;
 
 			rect.left=ClientRect.Left;
@@ -1341,7 +1418,7 @@ Console.WriteLine("Hit Clear background");
 			rect.right=ClientRect.Right;
 			rect.bottom=ClientRect.Bottom;
 
-			if (!Win32AdjustWindowRectEx(ref rect, Style, MenuHandle != IntPtr.Zero, ExStyle)) {
+			if (!Win32AdjustWindowRectEx(ref rect, Style, menu != null, ExStyle)) {
 				WindowRect = new Rectangle(ClientRect.Left, ClientRect.Top, ClientRect.Width, ClientRect.Height);
 				return false;
 			}
@@ -1351,11 +1428,8 @@ Console.WriteLine("Hit Clear background");
 		}
 
 		internal override void SetCursor(IntPtr window, IntPtr cursor) {
-			if (override_cursor == IntPtr.Zero) {
-				Win32SetCursor(cursor);
-				return;
-			}
-			Win32SetCursor(override_cursor);
+			Win32SetCursor(cursor);
+			return;
 		}
 
 		internal override void ShowCursor(bool show) {
@@ -1363,7 +1437,7 @@ Console.WriteLine("Hit Clear background");
 		}
 
 		internal override void OverrideCursor(IntPtr cursor) {
-			override_cursor = cursor;
+			Win32SetCursor(cursor);
 		}
 
 		internal override IntPtr DefineCursor(Bitmap bitmap, Bitmap mask, Color cursor_pixel, Color mask_pixel, int xHotSpot, int yHotSpot) {
@@ -1507,14 +1581,24 @@ Console.WriteLine("Hit Clear background");
 			y = pnt.y;
 		}
 
-		internal override void ScreenToMenu(IntPtr handle, ref int x, ref int y) {
-			ScreenToClient(handle, ref x, ref y);
-		}
-
-		internal override void MenuToScreen(IntPtr handle, ref int x, ref int y) {			
-			ClientToScreen(handle, ref x, ref y);
-		}
-
+  		internal override void ScreenToMenu(IntPtr handle, ref int x, ref int y) {
+ 			RECT	rect;
+ 
+ 			Win32GetWindowRect(handle, out rect);
+ 			x -= rect.left + SystemInformation.FrameBorderSize.Width;
+ 			y -= rect.top + SystemInformation.FrameBorderSize.Height + ThemeEngine.Current.CaptionHeight;
+ 			return;
+  		}
+  
+  		internal override void MenuToScreen(IntPtr handle, ref int x, ref int y) {			
+ 			RECT	rect;
+ 
+ 			Win32GetWindowRect(handle, out rect);
+ 			x += rect.left + SystemInformation.FrameBorderSize.Width;
+ 			y += rect.top + SystemInformation.FrameBorderSize.Height + ThemeEngine.Current.CaptionHeight;
+ 			return;
+  		}
+  
 		internal override void SendAsyncMethod (AsyncMethodData method)
 		{
 			Win32PostMessage(FosterParent, Msg.WM_ASYNC_MESSAGE, IntPtr.Zero, (IntPtr)GCHandle.Alloc (method));
@@ -1624,7 +1708,6 @@ Console.WriteLine("Hit Clear background");
 			nid = new NOTIFYICONDATA();
 
 			nid.cbSize = (uint)Marshal.SizeOf(nid);
-			nid.hIcon = icon.Handle;
 			nid.hWnd = hwnd;
 			nid.uID = 1;
 			nid.uCallbackMessage = (uint)Msg.WM_USER;
@@ -1684,12 +1767,41 @@ Console.WriteLine("Hit Clear background");
 		}
 
 
-		internal override void SetBorderStyle(IntPtr handle, BorderStyle border_style) {
+		internal override void SetBorderStyle(IntPtr handle, FormBorderStyle border_style) {
+			uint	style;
+			uint	exstyle;
+
+			style = Win32GetWindowLong(handle, WindowLong.GWL_STYLE);
+			exstyle = Win32GetWindowLong(handle, WindowLong.GWL_EXSTYLE);
+
+			switch (border_style) {
+				case FormBorderStyle.None: {
+					style &= ~(uint)WindowStyles.WS_BORDER;
+					exstyle &= ~(uint)WindowStyles.WS_EX_CLIENTEDGE;
+					break;
+				}
+
+				case FormBorderStyle.FixedSingle: {
+					style |= (uint)WindowStyles.WS_BORDER;
+					exstyle &= ~(uint)WindowStyles.WS_EX_CLIENTEDGE;
+					break;
+				}
+
+				case FormBorderStyle.Fixed3D: {
+					style &= ~(uint)WindowStyles.WS_BORDER;
+					exstyle |= (uint)WindowStyles.WS_EX_CLIENTEDGE;
+					break;
+				}
+			}
+
+			Win32SetWindowLong(handle, WindowLong.GWL_STYLE, style);
+			Win32SetWindowLong(handle, WindowLong.GWL_EXSTYLE, exstyle);
+			
 			Win32SetWindowPos(handle, IntPtr.Zero, 0, 0, 0, 0, 
 				SetWindowPosFlags.SWP_FRAMECHANGED | SetWindowPosFlags.SWP_NOMOVE | SetWindowPosFlags.SWP_NOSIZE | SetWindowPosFlags.SWP_NOACTIVATE);
 		}
 
-		internal override void SetMenu(IntPtr handle, IntPtr menu_handle) {
+		internal override void SetMenu(IntPtr handle, Menu menu) {
 			// Trigger WM_NCCALC
 			Win32SetWindowPos(handle, IntPtr.Zero, 0, 0, 0, 0, SetWindowPosFlags.SWP_FRAMECHANGED | SetWindowPosFlags.SWP_NOMOVE | SetWindowPosFlags.SWP_NOSIZE);
 		}
@@ -1723,27 +1835,6 @@ Console.WriteLine("Hit Clear background");
 			Win32SendMessage(hwnd, Msg.WM_SETICON, (IntPtr)1, icon.Handle);	// 1 = large icon (0 would be small)
 		}
 
-		internal override void EraseWindowBackground(IntPtr hWnd, IntPtr hDc) {
-			IntPtr          hbr;
-			LOGBRUSH        lb;
-			uint            argb;
-			RECT            rect;
-
-			//msg.wParam
-			argb = Win32GetWindowLong(hWnd, WindowLong.GWL_USERDATA);
-			lb = new LOGBRUSH();
-
-			lb.lbColor.B = (byte)((argb & 0xff0000)>>16);
-			lb.lbColor.G = (byte)((argb & 0xff00)>>8);
-			lb.lbColor.R = (byte)(argb & 0xff);
-
-			lb.lbStyle = LogBrushStyle.BS_SOLID;
-			hbr = Win32CreateBrushIndirect(ref lb);
-			Win32GetClientRect(hWnd, out rect);
-			Win32FillRect(hDc, ref rect, hbr);
-			Win32DeleteObject(hbr);
-		}
-
 		internal override void ClipboardClose(IntPtr handle) {
 			if (handle != clip_magic) {
 				throw new ArgumentException("handle is not a valid clipboard handle");
@@ -1837,7 +1928,9 @@ Console.WriteLine("Hit Clear background");
 
 			obj = null;
 
-			switch ((ClipboardFormats)type) {
+			if (type == DataFormats.GetFormat(DataFormats.Rtf).Id) {
+				obj = AnsiToString(data);
+			} else switch ((ClipboardFormats)type) {
 				case ClipboardFormats.CF_TEXT: {
 					obj = AnsiToString(data);
 					break;
@@ -1889,10 +1982,19 @@ Console.WriteLine("Hit Clear background");
 				}
 			}
 
-			switch((ClipboardFormats)type) {
+			if (type == DataFormats.GetFormat(DataFormats.Rtf).Id) {
+				hmem = Marshal.StringToHGlobalAnsi((string)obj);
+				Win32SetClipboardData((uint)type, hmem);
+				return;
+			} else switch((ClipboardFormats)type) {
 				case ClipboardFormats.CF_UNICODETEXT: {
 					hmem = Marshal.StringToHGlobalUni((string)obj);
-					Win32EmptyClipboard();
+					Win32SetClipboardData((uint)type, hmem);
+					return;
+				}
+
+				case ClipboardFormats.CF_TEXT: {
+					hmem = Marshal.StringToHGlobalAnsi((string)obj);
 					Win32SetClipboardData((uint)type, hmem);
 					return;
 				}
@@ -1904,7 +2006,6 @@ Console.WriteLine("Hit Clear background");
 					hmem_ptr = Win32GlobalLock(hmem);
 					Marshal.Copy(data, 0, hmem_ptr, data.Length);
 					Win32GlobalUnlock(hmem);
-					Win32EmptyClipboard();
 					Win32SetClipboardData((uint)type, hmem);
 					return;
 				}
@@ -1915,7 +2016,6 @@ Console.WriteLine("Hit Clear background");
 						hmem_ptr = Win32GlobalLock(hmem);
 						Marshal.Copy(data, 0, hmem_ptr, data.Length);
 						Win32GlobalUnlock(hmem);
-						Win32EmptyClipboard();
 						Win32SetClipboardData((uint)type, hmem);
 					}
 					return;
@@ -1934,7 +2034,61 @@ Console.WriteLine("Hit Clear background");
 		internal override DragDropEffects StartDrag(IntPtr hwnd, object data, DragDropEffects allowedEffects) {
 			return Win32DnD.StartDrag(hwnd, data, allowedEffects);
 		}
-		
+
+		internal override void DrawReversibleRectangle(IntPtr handle, Rectangle rect, int line_width) {
+			IntPtr		hdc;
+			IntPtr		pen;
+			IntPtr		oldpen;
+			POINT		pt;
+
+			pt = new POINT();
+			pt.x = 0;
+			pt.y = 0;
+			Win32ClientToScreen(handle, ref pt);
+
+			// If we want the standard hatch pattern we would
+			// need to create a brush
+
+			// Grab a pen
+			pen = Win32CreatePen(PenStyle.PS_SOLID, line_width, IntPtr.Zero);
+
+			hdc = Win32GetDC(IntPtr.Zero);
+			Win32SetROP2(hdc, ROP2DrawMode.R2_NOT);
+			oldpen = Win32SelectObject(hdc, pen);
+
+			// We might need to add clipping to the WindowRect of 'handle' - right now we're drawing on the desktop
+
+			Win32MoveToEx(hdc, pt.x + rect.Left, pt.y + rect.Top, IntPtr.Zero);
+			if ((rect.Width > 0) && (rect.Height > 0)) {
+				Win32LineTo(hdc, pt.x + rect.Right, pt.y + rect.Top);
+				Win32LineTo(hdc, pt.x + rect.Right, pt.y + rect.Bottom);
+				Win32LineTo(hdc, pt.x + rect.Left, pt.y + rect.Bottom);
+				Win32LineTo(hdc, pt.x + rect.Left, pt.y + rect.Top);
+			} else {
+				if (rect.Width > 0) {
+					Win32LineTo(hdc, pt.x + rect.Right, pt.y + rect.Top);
+				} else {
+					Win32LineTo(hdc, pt.x + rect.Left, pt.y + rect.Bottom);
+				}
+			}
+
+			Win32SelectObject(hdc, oldpen);
+			Win32DeleteObject(pen);
+			Win32ReleaseDC(IntPtr.Zero, hdc);
+		}
+
+		internal override SizeF GetAutoScaleSize(Font font) {
+			Graphics	g;
+			float		width;
+			string		magic_string = "The quick brown fox jumped over the lazy dog.";
+			double		magic_number = 44.549996948242189;
+
+			g = Graphics.FromHwnd(FosterParent);
+
+			width = (float) (g.MeasureString (magic_string, font).Width / magic_number);
+			return new SizeF(width, font.Height);
+		}
+
 		internal override int KeyboardSpeed {
 			get {
 				Console.WriteLine ("KeyboardSpeed: need to query Windows");
@@ -2087,9 +2241,6 @@ Console.WriteLine("Hit Clear background");
 		[DllImport ("user32.dll", EntryPoint="SetActiveWindow", CallingConvention=CallingConvention.StdCall)]
 		private extern static IntPtr Win32SetActiveWindow(IntPtr hWnd);
 
-		[DllImport ("user32.dll", EntryPoint="PostQuitMessage", CallingConvention=CallingConvention.StdCall)]
-		private extern static IntPtr Win32PostQuitMessage(IntPtr hWnd);
-
 		[DllImport ("user32.dll", EntryPoint="AdjustWindowRectEx", CallingConvention=CallingConvention.StdCall)]
 		private extern static bool Win32AdjustWindowRectEx(ref RECT lpRect, int dwStyle, bool bMenu, int dwExStyle);
 
@@ -2112,7 +2263,7 @@ Console.WriteLine("Hit Clear background");
 		private extern static int Win32FillRect(IntPtr hdc, ref RECT rect, IntPtr hbr);
 
 		[DllImport ("user32.dll", EntryPoint="SetWindowLong", CallingConvention=CallingConvention.StdCall)]
-		private extern static IntPtr Win32SetWindowLong(IntPtr hwnd, WindowLong index, IntPtr value);
+		private extern static uint Win32SetWindowLong(IntPtr hwnd, WindowLong index, uint value);
 
 		[DllImport ("user32.dll", EntryPoint="GetWindowLong", CallingConvention=CallingConvention.StdCall)]
 		private extern static uint Win32GetWindowLong(IntPtr hwnd, WindowLong index);
@@ -2166,7 +2317,7 @@ Console.WriteLine("Hit Clear background");
 		internal extern static bool Win32GetTextMetrics(IntPtr hdc, ref TEXTMETRIC tm);
 
 		[DllImport ("gdi32.dll", EntryPoint="SelectObject", CallingConvention=CallingConvention.StdCall)]
-		internal extern static bool Win32SelectObject(IntPtr hdc, IntPtr hgdiobject);
+		internal extern static IntPtr Win32SelectObject(IntPtr hdc, IntPtr hgdiobject);
 
 		[DllImport ("user32.dll", EntryPoint="ScrollWindowEx", CallingConvention=CallingConvention.StdCall)]
 		private extern static bool Win32ScrollWindowEx(IntPtr hwnd, int dx, int dy, ref RECT prcScroll, ref RECT prcClip, IntPtr hrgnUpdate, out RECT prcUpdate, ScrollWindowExFlags flags);
@@ -2241,7 +2392,37 @@ Console.WriteLine("Hit Clear background");
 		internal extern static IntPtr Win32GlobalLock(IntPtr hMem);
 
 		[DllImport ("kernel32.dll", EntryPoint="GlobalUnlock", CallingConvention=CallingConvention.StdCall)]
-		internal extern static bool Win32GlobalUnlock(IntPtr hMem);
+		internal extern static IntPtr Win32GlobalUnlock(IntPtr hMem);
+
+		[DllImport ("gdi32.dll", EntryPoint="SetROP2", CallingConvention=CallingConvention.StdCall)]
+		internal extern static int Win32SetROP2(IntPtr hdc, ROP2DrawMode fnDrawMode);
+
+		[DllImport ("gdi32.dll", EntryPoint="MoveToEx", CallingConvention=CallingConvention.StdCall)]
+		internal extern static bool Win32MoveToEx(IntPtr hdc, int x, int y, ref POINT lpPoint);
+
+		[DllImport ("gdi32.dll", EntryPoint="MoveToEx", CallingConvention=CallingConvention.StdCall)]
+		internal extern static bool Win32MoveToEx(IntPtr hdc, int x, int y, IntPtr lpPoint);
+
+		[DllImport ("gdi32.dll", EntryPoint="LineTo", CallingConvention=CallingConvention.StdCall)]
+		internal extern static bool Win32LineTo(IntPtr hdc, int x, int y);
+
+		[DllImport ("gdi32.dll", EntryPoint="CreatePen", CallingConvention=CallingConvention.StdCall)]
+		internal extern static IntPtr Win32CreatePen(PenStyle fnPenStyle, int nWidth, ref COLORREF color);
+
+		[DllImport ("gdi32.dll", EntryPoint="CreatePen", CallingConvention=CallingConvention.StdCall)]
+		internal extern static IntPtr Win32CreatePen(PenStyle fnPenStyle, int nWidth, IntPtr color);
+
+		[DllImport ("gdi32.dll", EntryPoint="GetStockObject", CallingConvention=CallingConvention.StdCall)]
+		internal extern static IntPtr Win32GetStockObject(StockObject fnObject);
+
+		[DllImport ("gdi32.dll", EntryPoint="CreateHatchBrush", CallingConvention=CallingConvention.StdCall)]
+		internal extern static IntPtr Win32CreateHatchBrush(HatchStyle fnStyle, IntPtr color);
+
+		[DllImport ("gdi32.dll", EntryPoint="CreateHatchBrush", CallingConvention=CallingConvention.StdCall)]
+		internal extern static IntPtr Win32CreateHatchBrush(HatchStyle fnStyle, ref COLORREF color);
+
+		[DllImport ("winmm.dll", EntryPoint="PlaySoundW", CallingConvention=CallingConvention.StdCall, CharSet=CharSet.Unicode)]
+		internal extern static IntPtr Win32PlaySound(string pszSound, IntPtr hmod, SndFlags fdwSound);
 		#endregion
 	}
 }
