@@ -35,14 +35,19 @@ namespace Microsoft.JScript {
 
 		public object arguments;
 		public object caller;
+		internal FunctionObject func;
 
 		public Closure (FunctionObject func)
 		{
+			this._prototype = func._prototype;
+			this.method = func.method;
+			this.vsa_engine = func.vsa_engine;
+			this.func = func;
 		}
 
 		public override string ToString ()
 		{
-			throw new NotImplementedException ();
+			return func.ToString ();
 		}
 	}
 }
