@@ -31,6 +31,7 @@
 //
 
 using System.Collections;
+using Microsoft.Vsa;
 using System.IO;
 using System;
 
@@ -63,28 +64,83 @@ namespace Microsoft.JScript {
 		CaseAfterDefault
 	}
 
+	internal class Location {
+		private string source_name;
+		private int line_number;
+
+		internal string SourceName {
+			get { return source_name; }
+		}
+
+		internal int LineNumber {
+			get { return line_number; }
+		}
+
+		internal Location (string source_name, int line_number)
+		{
+			this.source_name = source_name;
+			this.line_number = line_number;
+		}
+	}
+
 	internal class Parser {
 
 		TokenStream ts;
 		bool ok; // did the parse encounter an error?
-		ScriptBlock current_script_or_fn;
 		int nesting_of_function;
 		int nesting_of_with;
 		bool allow_member_expr_as_function_name;
+		Decompiler decompiler; 
+		ArrayList code_items;
 
 		internal Parser ()
 		{
 		}
+		
+		internal Parser (ArrayList code_items)
+		{
+			this.code_items = code_items;
+		}
+		
+		internal ScriptBlock [] ParseAll ()
+		{
+			int i = 0, n = code_items.Count;
+			ScriptBlock [] blocks = new ScriptBlock [n];
+
+			foreach (VsaCodeItem item in code_items)
+				blocks [i++] = Parse (item.SourceText, item.Name, 0);
+
+			return blocks;
+		}
+
+		internal Decompiler CreateDecompiler ()
+		{
+			return new Decompiler ();
+		}
+
+		/// <summary>
+		/// Test if n is between the range stablished by min and max
+		/// </summary>
+		private bool InRangeOf (double n, double min, double max)
+		{
+			return min <= n && n <= max;
+		}
+
+		private bool HasNoDecimals (double v)
+		{
+			return Math.Round (v) == v;
+		}
 
 		/// <summary>
 		///   Build a parse tree from a given source_string
 		/// </summary>
 		///
 		/// <remarks>
-		///   return an AST representing the parsed program.
-		///    If the parse fails, null will be returned.
+		///   return an ScriptBlock representing the parsed program
+		///   that corresponds to a source file.
+		///   If the parse fails, null will be returned.
 		/// </remarks>
-		internal AST Parse (string source_string, string source_location, int line_number)
+		internal ScriptBlock Parse (string source_string, string source_location, int line_number)
 		{
 			ts = new TokenStream (null, source_string, source_location, line_number);
 			try {
@@ -124,11 +180,13 @@ namespace Microsoft.JScript {
 			throw new ParserException ();
 		}
 	
-		AST Parse ()
+		ScriptBlock Parse ()
 		{
-			current_script_or_fn = new ScriptBlock ();
+			decompiler = CreateDecompiler ();
+			ScriptBlock current_script_or_fn = new ScriptBlock (new Location (ts.SourceName, ts.LineNumber));
+			decompiler.GetCurrentOffset ();
+			decompiler.AddToken (Token.SCRIPT);
 			ok = true;
-			int base_line_number = ts.LineNumber; // line number where source starts
 
 			try {
 				for (;;) {
@@ -160,6 +218,7 @@ namespace Microsoft.JScript {
 			if (!ok)
 				return null;
 
+			this.decompiler = null; // It helps GC
 			return current_script_or_fn;
 		}
 
@@ -174,7 +233,8 @@ namespace Microsoft.JScript {
 		Block ParseFunctionBody (AST parent)
 		{
 			++nesting_of_function;
-			Block pn = new Block (ts.LineNumber);
+			Block pn = new Block (parent, new Location (ts.SourceName, ts.LineNumber));
+
 			try {
 				int tt;
 				while ((tt = ts.PeekToken ()) > Token.EOF && tt != Token.RC) {
@@ -197,7 +257,6 @@ namespace Microsoft.JScript {
 		AST Function (AST parent, FunctionType ft)
 		{
 			FunctionType synthetic_type = ft;
-			int base_line_number = ts.LineNumber; // line number where source starts
 			string name;
 			AST member_expr = null;
 
@@ -208,7 +267,9 @@ namespace Microsoft.JScript {
 						// Extension to ECMA: if 'function <name>' does not follow
 						// by '(', assume <name> starts memberExpr
 						// FIXME: is StringLiteral the correct AST to build?
-						AST member_expr_head = new StringLiteral (null, name);
+						decompiler.AddName (name);
+						AST member_expr_head = new StringLiteral (null, name, 
+									  new Location (ts.SourceName, ts.LineNumber));
 						name = "";
 						member_expr = MemberExprTail (parent, false, member_expr_head);
 					}
@@ -228,8 +289,10 @@ namespace Microsoft.JScript {
 				MustMatchToken (Token.LP, "msg.no.paren.parms");
 			}
 
-			if (member_expr != null)
+			if (member_expr != null) {
 				synthetic_type = FunctionType.Expression;
+				decompiler.AddToken (Token.ASSIGN);
+			}
 			
 			bool nested = InsideFunction;
 			Function fn = CreateFunction (parent, synthetic_type, name);
@@ -245,32 +308,54 @@ namespace Microsoft.JScript {
 				// of with object.
 				fn.IgnoreDynamicScope = true;
 			}
-			
-			ScriptBlock saved_script_or_fn = current_script_or_fn;
+
+			// FIXME: which is old version of Decompiler.MarkFunctionStart
+			int functionSourceStart = decompiler.MarkFunctionStart ((int) synthetic_type);
+
+			if (name != "")
+				decompiler.AddName (name);
+
 			int saved_nesting_of_with = nesting_of_with;
 			nesting_of_with = 0;
 
-			FormalParameterList _params = new FormalParameterList ();
+			FormalParameterList _params = new FormalParameterList (new Location (ts.SourceName, ts.LineNumber));
 			Block body;
-			string source;
 
 			try {
+				decompiler.AddToken (Token.LP);
 				if (!ts.MatchToken (Token.RP)) {
+					bool first = true;
 					do {
+						if (!first)
+							decompiler.AddToken (Token.COMMA);
+						first = false;
 						MustMatchToken (Token.NAME, "msg.no.parm");
 						string s = ts.GetString;
-						_params.Add (s, String.Empty);
+						_params.Add (s, String.Empty, new Location (ts.SourceName, ts.LineNumber));
+						decompiler.AddName (s);
 					} while (ts.MatchToken (Token.COMMA));
 					MustMatchToken (Token.RP, "msg.no.paren.after.parms");
 				}
+				decompiler.AddToken (Token.RP);
+
 				MustMatchToken (Token.LC, "msg.no.brace.body");
+				decompiler.AddEOL (Token.LC);
 				body = ParseFunctionBody (fn);
 				MustMatchToken (Token.RC, "msg.no.brace.after.body");
 
-				if (ft != FunctionType.Expression)
+				decompiler.AddToken (Token.RC);
+				decompiler.MarkFunctionEnd (functionSourceStart);
+
+				fn.func_obj.source = decompiler.SourceToString (functionSourceStart);
+
+				if (ft != FunctionType.Expression) {
 					CheckWellTerminatedFunction ();
+					if (member_expr == null)
+						decompiler.AddToken (Token.EOL);
+					else
+						decompiler.AddEOL (Token.SEMI);
+				}
 			} finally {
-				current_script_or_fn = saved_script_or_fn;
 				nesting_of_with = saved_nesting_of_with;
 			}
 
@@ -287,7 +372,8 @@ namespace Microsoft.JScript {
 			} else {
 				// FIXME
 				pn = fn;
-				pn = new Assign (null, member_expr, pn, JSToken.Assign, false);
+				pn = new Assign (null, member_expr, pn, JSToken.Assign, false, new Location (ts.SourceName, ts.LineNumber));
+
 				// FIXME, research about createExprStatement
 				if (ft != FunctionType.Expression)
 					;
@@ -298,10 +384,12 @@ namespace Microsoft.JScript {
 		Function CreateFunction (AST parent, FunctionType func_type, string name)
 		{
 			Function func;
+			Location location = new Location (ts.SourceName, ts.LineNumber);
+
 			if (func_type == FunctionType.Statement)
-				func = new FunctionDeclaration (parent, name);
+				func = new FunctionDeclaration (parent, name, location);
 			else if (func_type == FunctionType.Expression)
-				func = new FunctionExpression (parent, name);
+				func = new FunctionExpression (parent, name, location);
 			else if (func_type == FunctionType.ExpressionStatement)
 				throw new NotImplementedException ();
 			else
@@ -312,7 +400,7 @@ namespace Microsoft.JScript {
 		AST Statements (AST parent)
 		{
 			int tt;
-			Block pn = new Block (parent);
+			Block pn = new Block (parent, new Location (ts.SourceName, ts.LineNumber));
 			while ((tt = ts.PeekToken ()) > Token.EOF && tt != Token.RC)
 				pn.Add (Statement (pn));
 			return pn;
@@ -322,11 +410,28 @@ namespace Microsoft.JScript {
 		{
 			AST pn;
 			MustMatchToken (Token.LP, "msg.no.paren.cond");
+			decompiler.AddToken (Token.LP);
 			pn = Expr (parent, false);
 			MustMatchToken (Token.RP, "msg.no.paren.after.cond");
+			decompiler.AddToken (Token.RP);
 			return pn;
 		}
 
+		AST Import (AST parent)
+		{
+			System.Text.StringBuilder @namespace = new System.Text.StringBuilder ();
+
+			while (true) {
+				MustMatchToken (Token.NAME, "msg.bad.namespace.name");
+				@namespace.Append (ts.GetString);
+				if (ts.MatchToken (Token.DOT))
+					@namespace.Append (".");
+				else
+					break;
+			}
+			return new Import (parent, @namespace.ToString (), new Location (ts.SourceName, ts.LineNumber));
+		}
+
 		void CheckWellTerminated ()
 		{
 			int tt = ts.PeekTokenSameLine ();
@@ -364,7 +469,6 @@ namespace Microsoft.JScript {
 				return StatementHelper (parent);
 			} catch (ParserException e) {
 				// skip to end of statement
-				int line_number = ts.LineNumber;
 				int t;
 				do {
 					t = ts.GetToken ();
@@ -393,32 +497,57 @@ namespace Microsoft.JScript {
 
 			if (tt == Token.IF) {
 				skip_semi = true;
-				int line_number = ts.LineNumber;
+				decompiler.AddToken (Token.IF);
 				AST cond = Condition (parent);
+
+				decompiler.AddEOL (Token.LC);
+
 				AST if_true = Statement (parent);
 				AST if_false = null;
-				if (ts.MatchToken (Token.ELSE))
+
+				if (ts.MatchToken (Token.ELSE)) {
+					decompiler.AddToken (Token.RC);
+					decompiler.AddToken (Token.ELSE);
+					decompiler.AddEOL (Token.LC);
 					if_false = Statement (parent);
-				pn = new If (parent, cond, if_true, if_false, line_number);
+				}
+				decompiler.AddEOL (Token.RC);
+				pn = new If (parent, cond, if_true, if_false, new Location (ts.SourceName, ts.LineNumber));
 			} else if (tt == Token.SWITCH) {
 				skip_semi = true;
-				pn = new Switch (parent, ts.LineNumber);
+
+				decompiler.AddToken (Token.SWITCH);
+
+				pn = new Switch (parent, new Location (ts.SourceName, ts.LineNumber));
 				Clause cur_case;
 				MustMatchToken (Token.LP, "msg.no.paren.switch");
+
+				decompiler.AddToken (Token.LP);
+
 				((Switch) pn).exp = Expr (parent, false);
 				MustMatchToken (Token.RP, "msg.no.paren.after.switch");
+
+				decompiler.AddToken (Token.RP);
+
 				MustMatchToken (Token.LC, "msg.no.brace.switch");
+
+				decompiler.AddEOL (Token.LC);
+
 				ClauseType clause_type = ClauseType.Case;
 
 				while ((tt = ts.GetToken ()) != Token.RC && tt != Token.EOF) {
 					if (tt == Token.CASE) {
-						cur_case = new Clause (pn);
+						decompiler.AddToken (Token.CASE);
+						cur_case = new Clause (pn, new Location (ts.SourceName, ts.LineNumber));
 						cur_case.exp = Expr (pn, false);
+						decompiler.AddEOL (Token.COLON);
 						if (clause_type == ClauseType.Default)
 							clause_type = ClauseType.CaseAfterDefault;
 					} else if (tt == Token.DEFAULT) {
 						cur_case = null;
 						clause_type = ClauseType.Default;
+						decompiler.AddToken (Token.DEFAULT);
+						decompiler.AddEOL (Token.COLON);
 					} else {
 						cur_case = null;
 						ReportError ("msg.bad.switch");
@@ -433,28 +562,36 @@ namespace Microsoft.JScript {
 					}
 					((Switch) pn).AddClause (cur_case, clause_type);
 				}
+				decompiler.AddEOL (Token.RC);
 			} else if (tt == Token.WHILE) {
 				skip_semi = true;
-				int line_number = ts.LineNumber;
-				While w = new While ();
+				decompiler.AddToken (Token.WHILE);
+				While w = new While (new Location (ts.SourceName, ts.LineNumber));
 				AST cond = Condition (w);
+				decompiler.AddEOL (Token.LC);
 				AST body = Statement (w);
-				w.Init (parent, cond, body, line_number);
+				decompiler.AddEOL (Token.RC);
+				w.Init (parent, cond, body);
 				pn = w;
 			} else if (tt == Token.DO) {
+				decompiler.AddToken (Token.DO);
+				decompiler.AddEOL (Token.LC);
 				int line_number = ts.LineNumber;
-				DoWhile do_while = new DoWhile ();
+				DoWhile do_while = new DoWhile (new Location (ts.SourceName, line_number));
 				AST body = Statement (do_while);
+				decompiler.AddToken (Token.RC);
 				MustMatchToken (Token.WHILE, "msg.no.while.do");
+				decompiler.AddToken (Token.WHILE);
 				AST cond = Condition (do_while);
-				do_while.Init (parent, body, cond, line_number);
+				do_while.Init (parent, body, cond);
 				pn  = do_while;
 			} else if (tt == Token.FOR) {
 				skip_semi = true;
-				int line_number = ts.LineNumber;
+				decompiler.AddToken (Token.FOR);
 				AST init, cond, incr = null, body;
 
 				MustMatchToken (Token.LP, "msg.no.paren.for");
+				decompiler.AddToken (Token.LP);
 				tt = ts.PeekToken ();
 
 				if (tt == Token.SEMI)
@@ -468,11 +605,13 @@ namespace Microsoft.JScript {
 						init = Expr (parent, true);
 				}
 				
-				if (ts.MatchToken (Token.IN))
+				if (ts.MatchToken (Token.IN)) {
+					decompiler.AddToken (Token.IN);
 					cond = Expr (parent, false); // 'cond' is the object over which we're iterating
-				else { 
+				} else { 
 					// ordinary for loop
 					MustMatchToken (Token.SEMI, "msg.no.semi.for");
+					decompiler.AddToken (Token.SEMI);
 					
 					if (ts.PeekToken () == Token.SEMI)
 						cond = null; // no loop condition
@@ -480,7 +619,8 @@ namespace Microsoft.JScript {
 						cond = Expr (parent, false);
 
 					MustMatchToken (Token.SEMI, "msg.no.semi.for.cond");
-					
+					decompiler.AddToken (Token.SEMI);
+
 					if (ts.PeekToken () == Token.RP)
 						incr = null;
 					else
@@ -488,12 +628,15 @@ namespace Microsoft.JScript {
 				}
 
 				MustMatchToken (Token.RP, "msg.no.paren.for.ctrl");
+				decompiler.AddToken (Token.RP);
+				decompiler.AddEOL (Token.LC);
 				body = Statement (pn);
-								
+				decompiler.AddEOL (Token.RC);
+
 				if (incr == null) // cond could be null if 'in obj' got eaten by the init node. 
-					pn = new ForIn (parent, line_number, init, cond, body);
+					pn = new ForIn (parent, init, cond, body, new Location (ts.SourceName, ts.LineNumber));
 				else
-					pn = new For (parent, line_number, init, cond, incr, body);
+					pn = new For (parent, init, cond, incr, body, new Location (ts.SourceName, ts.LineNumber));
 				body.parent = pn;
 			} else if (tt == Token.TRY) {
 				int line_number = ts.LineNumber;
@@ -502,8 +645,11 @@ namespace Microsoft.JScript {
 				AST finally_block = null;
 
 				skip_semi = true;
+				decompiler.AddToken (Token.TRY);
+				decompiler.AddEOL (Token.LC);
 
 				try_block = Statement (parent);
+				decompiler.AddEOL (Token.RC);
 				catch_blocks = new ArrayList ();
 
 				bool saw_default_catch = false;
@@ -513,50 +659,76 @@ namespace Microsoft.JScript {
 					while (ts.MatchToken (Token.CATCH)) {
 						if (saw_default_catch)
 							ReportError ("msg.catch.unreachable");
+						decompiler.AddToken (Token.CATCH);
 						MustMatchToken (Token.LP, "msg.no.paren.catch");
+						decompiler.AddToken (Token.LP);
 						MustMatchToken (Token.NAME, "msg.bad.catchcond");
 						string var_name = ts.GetString;
+						decompiler.AddName (var_name);
 						AST catch_cond = null;
 						
-						if (ts.MatchToken (Token.IF))
+						if (ts.MatchToken (Token.IF)) {
+							decompiler.AddToken (Token.IF);
 							catch_cond = Expr (parent, false);
-						else
+						} else
 							saw_default_catch = true;
 						
 						MustMatchToken (Token.RP, "msg.bad.catchcond");
+						decompiler.AddToken (Token.RP);
 						MustMatchToken (Token.LC, "msg.no.brace.catchblock");
+						decompiler.AddEOL (Token.LC);
 
 						catch_blocks.Add (new Catch (var_name, catch_cond, 
-									     Statements (null), parent, ts.LineNumber));
+									     Statements (null), parent, new Location (ts.SourceName, line_number)));
 						MustMatchToken (Token.RC, "msg.no.brace.after.body");
+						decompiler.AddEOL (Token.RC);
 					}
 				} else if (peek != Token.FINALLY)
 					MustMatchToken (Token.FINALLY, "msg.try.no.catchfinally");
 				
-				if (ts.MatchToken (Token.FINALLY))
+				if (ts.MatchToken (Token.FINALLY)) {
+					decompiler.AddToken (Token.FINALLY);
+					decompiler.AddEOL (Token.LC);
 					finally_block = Statement (parent);
-				pn = new Try (try_block, catch_blocks, finally_block, parent, line_number);
+					decompiler.AddEOL (Token.RC);
+				}
+				pn = new Try (try_block, catch_blocks, finally_block, parent, new Location (ts.SourceName, ts.LineNumber));
 			} else if (tt == Token.THROW) {
 				int line_number = ts.LineNumber;
-				pn = new Throw (Expr (parent, false), line_number);
+				decompiler.AddToken (Token.THROW);
+				pn = new Throw (Expr (parent, false), new Location (ts.SourceName, ts.LineNumber));
 
 				if (line_number == ts.LineNumber)
 					CheckWellTerminated ();
 			} else if (tt == Token.BREAK) {
-				int line_number = ts.LineNumber;
+				decompiler.AddToken (Token.BREAK);
+
 				// MatchLabel only matches if there is one
 				string label = MatchLabel ();
-				pn = new Break (parent, label, line_number);
+
+				if (label != null)
+					decompiler.AddName (label);
+
+				pn = new Break (parent, label, new Location (ts.SourceName, ts.LineNumber));
 			} else if (tt == Token.CONTINUE) {
-				int line_number = ts.LineNumber;
+				decompiler.AddToken (Token.CONTINUE);
+
+				// MatchLabel only matches if there is one
 				string label = MatchLabel ();
-				pn = new Continue (parent, label, line_number);
+
+				if (label != null)
+					decompiler.AddName (label);
+
+				pn = new Continue (parent, label, new Location (ts.SourceName, ts.LineNumber));
 			} else if (tt == Token.WITH) {
 				skip_semi = true;
-				int line_number = ts.LineNumber;
+				decompiler.AddToken (Token.WITH);
 				MustMatchToken (Token.LP, "msg.no.paren.with");
+				decompiler.AddToken (Token.LP);
 				AST obj = Expr (parent, false);
 				MustMatchToken (Token.RP, "msg.no.paren.after.with");
+				decompiler.AddToken (Token.RP);
+				decompiler.AddToken (Token.LC);
 				++nesting_of_with;
 				AST body;
 				try {
@@ -564,7 +736,8 @@ namespace Microsoft.JScript {
 				} finally {
 					--nesting_of_with;
 				}
-				pn = new With (parent, obj, body, line_number);
+				decompiler.AddEOL (Token.RC);
+				pn = new With (parent, obj, body, new Location (ts.SourceName, ts.LineNumber));
 			} else if (tt == Token.VAR) {
 				int line_number = ts.LineNumber;
 				pn = Variables (parent, false);
@@ -572,7 +745,8 @@ namespace Microsoft.JScript {
 					CheckWellTerminated ();
 			} else if (tt == Token.RETURN) {
 				AST ret_expr = null;
-				pn = new Return ();
+				decompiler.AddToken (Token.RETURN);
+				pn = new Return (new Location (ts.SourceName, ts.LineNumber));
 
 				if (!InsideFunction)
 					ReportError ("msg.bad.return");
@@ -588,7 +762,7 @@ namespace Microsoft.JScript {
 					if (ts.LineNumber == line_number)
 						CheckWellTerminated ();
 				}
-				((Return) pn).Init (parent, ret_expr, line_number);
+				((Return) pn).Init (parent, ret_expr);
 			} else if (tt == Token.LC) { 
 				skip_semi = true;
 				pn = Statements (parent);
@@ -599,6 +773,9 @@ namespace Microsoft.JScript {
 				skip_semi = true;
 			} else if (tt == Token.FUNCTION) {
 				pn = Function (parent, FunctionType.ExpressionStatement);
+			} else if (tt == Token.IMPORT) {
+				decompiler.AddToken (Token.IMPORT);
+				pn = Import (parent);
 			} else {
 				int last_expr_type = tt;
 				int token_number = ts.TokenNumber;
@@ -619,9 +796,12 @@ namespace Microsoft.JScript {
 					string name = ts.GetString;
 
 					// bind 'Statement (pn)' to the label
-					Labelled labelled = new Labelled ();
-					labelled.Init (parent, name, Statement (labelled), line_number);
+					Labelled labelled = new Labelled (parent, new Location (ts.SourceName, ts.LineNumber));
+					labelled.Init (parent, name, Statement (labelled), new Location (ts.SourceName, ts.LineNumber));
 					pn = labelled;
+					// depend on decompiling lookahead to guess that that
+					// last name was a label.
+					decompiler.AddEOL (Token.COLON);
 					return pn;
 				}
 				// FIXME:
@@ -630,21 +810,35 @@ namespace Microsoft.JScript {
 					CheckWellTerminated ();
 			}
 			ts.MatchToken (Token.SEMI);
+
+			if (!skip_semi)
+				decompiler.AddEOL (Token.SEMI);
+
 			return pn;
 		}
 		
 		AST Variables (AST parent, bool in_for_init)
 		{
-			VariableStatement pn = new VariableStatement (parent, ts.LineNumber);
+			VariableStatement pn = new VariableStatement (parent, new Location (ts.SourceName, ts.LineNumber));
+			bool first = true;
+			decompiler.AddToken (Token.VAR);
+
 			for (;;) {
 				VariableDeclaration name;
 				AST init = null;
 				MustMatchToken (Token.NAME, "msg.bad.var");
-				string s = ts.GetString;				
-				name = new VariableDeclaration (parent, s, null, null);
+				string s = ts.GetString;
+
+				if (!first)
+					decompiler.AddToken (Token.COMMA);
+				first = false;
+				
+				decompiler.AddName (s);
+				name = new VariableDeclaration (parent, s, null, null, new Location (ts.SourceName, ts.LineNumber));
 
 				// ommited check for argument hiding				
 				if (ts.MatchToken (Token.ASSIGN)) {
+					decompiler.AddToken (Token.ASSIGN);
 					init = AssignExpr (parent, in_for_init);
 					name.val = init;
 				}
@@ -658,15 +852,17 @@ namespace Microsoft.JScript {
 
 		AST Expr (AST parent, bool in_for_init)
 		{
-			Expression pn = new Expression (parent);
+			Expression pn = new Expression (parent, new Location (ts.SourceName, ts.LineNumber));
 			AST init = AssignExpr (parent, in_for_init);
 			pn.Add (init);
 
 			if (init == null)
 				throw new Exception ("Expr, L680, AST is null");
 			
-			while (ts.MatchToken (Token.COMMA))
+			while (ts.MatchToken (Token.COMMA)) {
+				decompiler.AddToken (Token.COMMA);
 				pn.Add (AssignExpr (parent, in_for_init));
+			}
 			return pn;
 		}
 
@@ -678,12 +874,15 @@ namespace Microsoft.JScript {
 			// omitted: "invalid assignment left-hand side" check.
 			if (tt == Token.ASSIGN) {
 				ts.GetToken ();
-				pn = new Assign (parent, pn, AssignExpr (parent, in_for_init), JSToken.Assign, false);
+				decompiler.AddToken (Token.ASSIGN);
+				pn = new Assign (parent, pn, AssignExpr (parent, in_for_init), JSToken.Assign, false, new Location (ts.SourceName, ts.LineNumber));
 				return pn;
 			} else if (tt == Token.ASSIGNOP) {
 				ts.GetToken ();
 				int op = ts.GetOp ();
-				pn = new Assign (parent, pn, AssignExpr (parent, in_for_init), ToJSToken (op, tt), false);
+				decompiler.AddAssignOp (op);
+				pn = new Assign (parent, pn, AssignExpr (parent, in_for_init), ToJSToken (op, tt), false, 
+					 new Location (ts.SourceName, ts.LineNumber));
 			}
 			return pn;
 		}
@@ -695,10 +894,12 @@ namespace Microsoft.JScript {
 			AST pn = OrExpr (parent, in_for_init);
 
 			if (ts.MatchToken (Token.HOOK)) {
+				decompiler.AddToken (Token.HOOK);
 				if_true = AssignExpr (parent, false);
 				MustMatchToken (Token.COLON, "msg.no.colon.cond");
+				decompiler.AddToken (Token.COLON);
 				if_false = AssignExpr (parent, in_for_init);
-				return new Conditional (parent, pn, if_true, if_false);
+				return new Conditional (parent, pn, if_true, if_false, new Location (ts.SourceName, ts.LineNumber));
 			}
 			return pn;
 		}
@@ -706,8 +907,11 @@ namespace Microsoft.JScript {
 		AST OrExpr (AST parent, bool in_for_init)
 		{
 			AST pn = AndExpr (parent, in_for_init);
-			if (ts.MatchToken (Token.OR))
-				return new Binary (parent, pn, OrExpr (parent, in_for_init), JSToken.LogicalOr);
+			if (ts.MatchToken (Token.OR)) {
+				decompiler.AddToken (Token.OR);
+				return new Binary (parent, pn, OrExpr (parent, in_for_init), JSToken.LogicalOr, 
+					   new Location (ts.SourceName, ts.LineNumber));
+			}
 			return pn;
 		}
 
@@ -715,7 +919,9 @@ namespace Microsoft.JScript {
 		{
 			AST pn = BitOrExpr (parent, in_for_init);
 			if (ts.MatchToken (Token.AND)) {
-				return new Binary (parent, pn, AndExpr (parent, in_for_init), JSToken.LogicalAnd);
+				decompiler.AddToken (Token.AND);
+				return new Binary (parent, pn, AndExpr (parent, in_for_init), JSToken.LogicalAnd,
+					   new Location (ts.SourceName, ts.LineNumber));
 			}
 			return pn;
 		}
@@ -724,7 +930,9 @@ namespace Microsoft.JScript {
 		{
 			AST pn = BitXorExpr (parent, in_for_init);
 			while (ts.MatchToken (Token.BITOR)) {
-				pn = new Binary (parent, pn, BitXorExpr (parent, in_for_init), JSToken.BitwiseOr);
+				decompiler.AddToken (Token.BITOR);
+				pn = new Binary (parent, pn, BitXorExpr (parent, in_for_init), JSToken.BitwiseOr,
+					 new Location (ts.SourceName, ts.LineNumber));
 			}
 			return pn;
 		}
@@ -732,31 +940,48 @@ namespace Microsoft.JScript {
 		AST BitXorExpr (AST parent, bool in_for_init)
 		{
 			AST pn = BitAndExpr (parent, in_for_init);
-			while (ts.MatchToken (Token.BITXOR))
-				pn = new Binary (parent, pn, BitAndExpr (parent, in_for_init), JSToken.BitwiseXor);
+			while (ts.MatchToken (Token.BITXOR)) {
+				decompiler.AddToken (Token.BITXOR);
+				pn = new Binary (parent, pn, BitAndExpr (parent, in_for_init), JSToken.BitwiseXor,
+					 new Location (ts.SourceName, ts.LineNumber));
+			}
 			return pn;
 		}
 
 		AST BitAndExpr (AST parent, bool in_for_init)
 		{
 			AST pn = EqExpr (parent, in_for_init);
-			while (ts.MatchToken (Token.BITAND))
-				pn = new Binary (parent, pn, EqExpr (parent, in_for_init), JSToken.BitwiseAnd);
+			while (ts.MatchToken (Token.BITAND)) {
+				decompiler.AddToken (Token.BITAND);
+				pn = new Binary (parent, pn, EqExpr (parent, in_for_init), JSToken.BitwiseAnd,
+					 new Location (ts.SourceName, ts.LineNumber));
+			}
 			return pn;
 		}
 
 		AST EqExpr (AST parent, bool in_for_init)
 		{
 			AST pn = RelExpr (parent, in_for_init);
+			ArrayList tokens = new ArrayList ();
 			for (;;) {
 				int tt = ts.PeekToken ();
+				tokens.Add (tt);
 				if (tt == Token.EQ || tt == Token.NE) {
+					foreach (int token in tokens)
+						decompiler.AddToken (token);
+					tokens.Clear ();
+
 					ts.GetToken ();
-					pn = new Equality (parent, pn, RelExpr (parent, in_for_init), ToJSToken (tt));
+					pn = new Equality (parent, pn, RelExpr (parent, in_for_init), ToJSToken (tt), 
+						   new Location (ts.SourceName, ts.LineNumber));
 					continue;
 				} else if (tt == Token.SHEQ || tt == Token.SHNE) {
+					foreach (int token in tokens)
+						decompiler.AddToken (token);
+					tokens.Clear ();
+					
 					ts.GetToken ();
-					pn = new StrictEquality (parent, pn, RelExpr (parent, in_for_init), ToJSToken (tt));
+					pn = new StrictEquality (parent, pn, RelExpr (parent, in_for_init), ToJSToken (tt), new Location (ts.SourceName, ts.LineNumber));
 					continue;
 				}				
 				break;
@@ -772,9 +997,16 @@ namespace Microsoft.JScript {
 				if (tt == Token.IN) {
 					if (in_for_init)
 						break;
+					else {
+						ts.GetToken ();
+						decompiler.AddToken (tt);
+						pn = new Relational (parent, pn, ShiftExpr (parent), ToJSToken (tt), new Location (ts.SourceName, ts.LineNumber));
+						continue;
+					}
 				} else if (tt == Token.INSTANCEOF || tt == Token.LE || tt == Token.LT || tt == Token.GE || tt == Token.GT) {
 					ts.GetToken ();
-					pn = new Relational (parent, pn, ShiftExpr (parent), ToJSToken (tt));
+					decompiler.AddToken (tt);
+					pn = new Relational (parent, pn, ShiftExpr (parent), ToJSToken (tt), new Location (ts.SourceName, ts.LineNumber));
 					continue;
 				}
 				break;
@@ -789,7 +1021,16 @@ namespace Microsoft.JScript {
 				int tt = ts.PeekToken ();
 				if (tt == Token.LSH || tt == Token.URSH || tt == Token.RSH) {
 					ts.GetToken ();
-					pn = new Binary (parent, pn, AddExpr (parent), JSToken.LeftShift);
+					decompiler.AddToken (tt);
+					
+					JSToken op = JSToken.LeftShift;
+					if (tt == Token.RSH)
+						op = JSToken.RightShift;
+					else if (tt == Token.URSH)
+						op = JSToken.UnsignedRightShift;
+
+					pn = new Binary (parent, pn, AddExpr (parent), op,
+						 new Location (ts.SourceName, ts.LineNumber));
 					continue;
 				}
 				break;
@@ -804,7 +1045,9 @@ namespace Microsoft.JScript {
 				int tt = ts.PeekToken ();
 				if (tt == Token.ADD || tt == Token.SUB) {
 					ts.GetToken ();
-					pn = new Binary (parent, pn, MulExpr (parent), ToJSToken (tt));
+					decompiler.AddToken (tt);
+					pn = new Binary (parent, pn, MulExpr (parent), ToJSToken (tt),
+						 new Location (ts.SourceName, ts.LineNumber));
 					continue;
 				}
 				break;
@@ -818,8 +1061,10 @@ namespace Microsoft.JScript {
 			for (;;) {
 				int tt = ts.PeekToken ();
 				if (tt == Token.MUL || tt == Token.DIV || tt == Token.MOD) {
-					ts.GetToken ();					
-					pn = new Binary (parent, pn, UnaryExpr (parent), ToJSToken (tt));
+					ts.GetToken ();
+					decompiler.AddToken (tt);
+					pn = new Binary (parent, pn, UnaryExpr (parent), ToJSToken (tt),
+						 new Location (ts.SourceName, ts.LineNumber));
 					continue;
 				}
 				break;
@@ -837,11 +1082,22 @@ namespace Microsoft.JScript {
 
 			if (tt == Token.VOID || tt == Token.NOT || tt == Token.BITNOT || tt == Token.TYPEOF ||
 			    tt == Token.ADD || tt == Token.SUB || tt == Token.DELPROP) {
-				Unary u = new Unary (parent, ToJSToken (tt));
+				if (tt == Token.VOID || tt == Token.NOT || tt == Token.BITNOT || tt == Token.TYPEOF)
+					decompiler.AddToken (tt);
+				else if (tt == Token.ADD)
+					decompiler.AddToken (Token.POS);
+				else if (tt == Token.SUB)
+					decompiler.AddToken (Token.NEG);
+				else 
+					decompiler.AddToken (tt);
+
+				Unary u = new Unary (parent, ToJSToken (tt), new Location (ts.SourceName, ts.LineNumber));
 				u.operand = UnaryExpr (u);
 				return u;
 			} else if (tt == Token.INC || tt == Token.DEC) {
-				return new PostOrPrefixOperator (parent, MemberExpr (parent, true), ToJSToken (tt), true);
+				decompiler.AddToken (tt);
+				return new PostOrPrefixOperator (parent, MemberExpr (parent, true), ToJSToken (tt), true,
+							 new Location (ts.SourceName, ts.LineNumber));
 			} else if (tt == Token.ERROR) {
 				;
 			} else {
@@ -860,11 +1116,13 @@ namespace Microsoft.JScript {
 				int peeked;				
 				if (((peeked = ts.PeekToken ()) == Token.INC || peeked == Token.DEC) && ts.LineNumber == line_number) {
 					int pf = ts.GetToken ();
-					return new PostOrPrefixOperator (parent, pn, ToJSToken (peeked), false);
+					decompiler.AddToken (pf);
+					return new PostOrPrefixOperator (parent, pn, ToJSToken (peeked), false,
+								 new Location (ts.SourceName, ts.LineNumber));
 				}
 				return pn;
 			}
-			return new StringLiteral (null, "Error");  // Only reached on error.  Try to continue.
+			return new StringLiteral (null, "Error", new Location (ts.SourceName, ts.LineNumber));  // Only reached on error.  Try to continue.
 		}
 
 		JSToken ToJSToken (int tt)
@@ -901,6 +1159,8 @@ namespace Microsoft.JScript {
 				return JSToken.Divide;
 			else if (tt == Token.MOD)
 				return JSToken.Modulo;
+			else if (tt == Token.IN)
+				return JSToken.In;
 			else if (tt == Token.INSTANCEOF)
 				return JSToken.Instanceof;
 			else if (tt == Token.LE)
@@ -955,11 +1215,16 @@ namespace Microsoft.JScript {
 			ts.allow_reg_exp = false;
 
 			if (!matched) {
+				bool first = true;
 				do {
+					if (!first)
+						decompiler.AddToken (Token.COMMA);
+					first = false;
 					list.AddArg (AssignExpr (parent, false));
 				} while (ts.MatchToken (Token.COMMA));				
 				MustMatchToken (Token.RP, "msg.no.paren.arg");
 			}
+			decompiler.AddToken (Token.RP);
 		}
 
 		AST MemberExpr (AST parent, bool allow_call_syntax)
@@ -975,13 +1240,16 @@ namespace Microsoft.JScript {
 			if (tt == Token.NEW) {
 				/* Eat the NEW token. */
 				ts.GetToken ();
+				decompiler.AddToken (Token.NEW);
 
 				/* Make a NEW node to append to. */
-				pn = new New (parent, MemberExpr (parent, false));
+				pn = new New (parent, MemberExpr (parent, false), new Location (ts.SourceName, ts.LineNumber));
 
-				if (ts.MatchToken (Token.LP))
+				if (ts.MatchToken (Token.LP)) {
+					decompiler.AddToken (Token.LP);
 					ArgumentList (parent, (ICallable) pn);
-				
+				}
+
 				/* XXX there's a check in the C source against
 				 * "too many constructor arguments" - how many
 				 * do we claim to support?
@@ -1006,18 +1274,26 @@ namespace Microsoft.JScript {
 
 			while ((tt = ts.GetToken ()) > Token.EOF) {
 				if (tt == Token.DOT) {
+					decompiler.AddToken (Token.DOT);
 					MustMatchToken (Token.NAME, "msg.no.name.after.dot");
 					string s = ts.GetString;
+					decompiler.AddName (s);
 					// FIXME: is 'new Identifier' appropriate here?
-					pn = new Binary (parent, pn, new Identifier (parent, ts.GetString), JSToken.AccessField);
+					pn = new Binary (parent, pn, 
+						 new Identifier (parent, ts.GetString, new Location (ts.SourceName, ts.LineNumber)),
+						 JSToken.AccessField, new Location (ts.SourceName, ts.LineNumber));
 				} else if (tt == Token.LB) {
-					Binary b = new Binary (parent, pn, JSToken.LeftBracket);
+					decompiler.AddToken (Token.LB);
+					Binary b = new Binary (parent, pn, JSToken.LeftBracket, 
+						       new Location (ts.SourceName, ts.LineNumber));
 					b.right = Expr (b, false);
 					pn = b;
 					MustMatchToken (Token.RB, "msg.no.bracket.index");
+					decompiler.AddToken (Token.RB);
 				} else if (allow_call_syntax && tt == Token.LP) {
 					/* make a call node */
-					pn = new Call (parent, pn);
+					decompiler.AddToken (Token.LP);
+					pn = new Call (parent, pn, new Location (ts.SourceName, ts.LineNumber));
 					
 					/* Add the arguments to pn, if any are supplied. */
 					ArgumentList (parent, (ICallable) pn);
@@ -1041,8 +1317,9 @@ namespace Microsoft.JScript {
 			if (tt == Token.FUNCTION) {
 				return Function (parent, FunctionType.Expression);
 			} else if (tt == Token.LB) {
-				ASTList elems = new ASTList ();
+				ASTList elems = new ASTList (parent, new Location (ts.SourceName, ts.LineNumber));
 				int skip_count = 0;
+				decompiler.AddToken (Token.LB);
 				bool after_lb_or_comma = true;
 				for (;;) {
 					ts.allow_reg_exp = true;
@@ -1051,6 +1328,7 @@ namespace Microsoft.JScript {
 					
 					if (tt == Token.COMMA) {
 						ts.GetToken ();
+						decompiler.AddToken (Token.COMMA);
 						if (!after_lb_or_comma) 
 							after_lb_or_comma = true;
 						else {
@@ -1059,6 +1337,7 @@ namespace Microsoft.JScript {
 						}
 					} else if (tt == Token.RB) {
 						ts.GetToken ();
+						decompiler.AddToken (Token.RB);
 						break;
 					} else {
 						if (!after_lb_or_comma) 
@@ -1068,9 +1347,11 @@ namespace Microsoft.JScript {
 					}
 				}
 				// FIXME: pass a real Context
-				return new ArrayLiteral (null, elems, skip_count);
+				return new ArrayLiteral (null, elems, skip_count, new Location (ts.SourceName, ts.LineNumber));
 			} else if (tt == Token.LC) {
+				Location location = new Location (ts.SourceName, ts.LineNumber);
 				ArrayList elems = new ArrayList ();
+				decompiler.AddToken (Token.LC);
 
 				if (!ts.MatchToken (Token.RC)) {
 					bool first = true;
@@ -1080,7 +1361,7 @@ namespace Microsoft.JScript {
 						ObjectLiteralItem property;
 
 						if (!first)
-							;
+							decompiler.AddToken (Token.COMMA);
 						else
 							first = false;
 						
@@ -1088,9 +1369,14 @@ namespace Microsoft.JScript {
 						
 						if (tt == Token.NAME || tt == Token.STRING) {
 							string s = ts.GetString;
+							if (tt == Token.NAME)
+								decompiler.AddName (s);
+							else
+								decompiler.AddString (s);
 							property = new ObjectLiteralItem (s);
 						} else if (tt == Token.NUMBER) {
 							double n = ts.GetNumber;
+							decompiler.AddNumber (n);
 							property = new ObjectLiteralItem (n);
 						} else if (tt == Token.RC) {
 							// trailing comma is OK
@@ -1101,6 +1387,9 @@ namespace Microsoft.JScript {
 							goto leave_commaloop;
 						}
 						MustMatchToken (Token.COLON, "msg.no.colon.prop");
+						// OBJLIT is used as ':' in object literal for
+						// decompilation to solve spacing ambiguity.
+						decompiler.AddToken (Token.OBJECTLIT);
 						property.exp = AssignExpr (parent, false);
 						elems.Add (property);
 					} while (ts.MatchToken (Token.COMMA));
@@ -1109,37 +1398,68 @@ namespace Microsoft.JScript {
 					leave_commaloop:
 					;
 				}
-				return new ObjectLiteral (elems);
+				return new ObjectLiteral (elems, location);
 			} else if (tt == Token.LP) {
+				decompiler.AddToken (Token.LP);
 				pn = Expr (parent, false);
+				decompiler.AddToken (Token.RP);
 				MustMatchToken (Token.RP, "msg.no.paren");
+				decompiler.AddToken (Token.RP);
 				return pn;
 			} else if (tt == Token.NAME) {
 				string name = ts.GetString;
-				return new Identifier (parent, name);
+				decompiler.AddName (name);
+				return new Identifier (parent, name, new Location (ts.SourceName, ts.LineNumber));
 			} else if (tt == Token.NUMBER) {
 				double n = ts.GetNumber;
-				return new NumericLiteral (parent, n);
+				decompiler.AddNumber (n);
+
+				Location location = new Location (ts.SourceName, ts.LineNumber);
+
+				if (HasNoDecimals (n)) {
+					if (InRangeOf (n, Byte.MinValue, Byte.MaxValue))
+						return new ByteConstant (parent, (byte) n, location);
+					else if (InRangeOf (n, Int16.MinValue, Int16.MaxValue))
+						return new ShortConstant (parent, (short) n, location);
+					else if (InRangeOf (n, Int32.MinValue, Int32.MaxValue))
+						return new IntConstant (parent, (int) n, location);
+					else if (InRangeOf (n, Int64.MinValue, Int64.MaxValue))
+						return new LongConstant (parent, (long) n, location);
+					else
+						return new DoubleConstant (parent, n, location);
+				} else {
+					if (InRangeOf (n, Single.MinValue, Single.MaxValue))
+						return new FloatConstant (parent, (float) n, location);
+					else if (InRangeOf (n, Double.MinValue, Double.MaxValue))
+						return new DoubleConstant (parent, n, location);
+					else
+						return new DoubleConstant (parent, n, location);
+				}
 			} else if (tt == Token.STRING) {
 				string s = ts.GetString;
-				return new StringLiteral (null, s);
+				decompiler.AddString (s);
+				return new StringLiteral (null, s, new Location (ts.SourceName, ts.LineNumber));
 			} else if (tt == Token.REGEXP) {
 				string flags = ts.reg_exp_flags;
 				ts.reg_exp_flags = null;
 				string re = ts.GetString;
-				return new RegExpLiteral (parent, re, flags);
+				decompiler.AddRegexp (re, flags);
+				return new RegExpLiteral (parent, re, flags, new Location (ts.SourceName, ts.LineNumber));
 			} else if (tt == Token.NULL) {
+				decompiler.AddToken (tt);
 				// FIXME, build the null object;
 				return null;
 			} else if (tt ==  Token.THIS) {
-				return new This (parent); 
+				decompiler.AddToken (tt);
+				return new This (parent, new Location (ts.SourceName, ts.LineNumber));
 			} else if (tt == Token.FALSE || tt == Token.TRUE) {
-				bool v;	
+				decompiler.AddToken (tt);
+				bool v;
 				if (tt == Token.FALSE)
 					v = false;
 				else
 					v = true;
-				return new BooleanLiteral (null, v);
+				return new BooleanConstant (null, v, new Location (ts.SourceName, ts.LineNumber));
 			} else if (tt == Token.RESERVED) {
 				ReportError ("msg.reserved.id");
 			} else if (tt == Token.ERROR) {
@@ -1147,6 +1467,6 @@ namespace Microsoft.JScript {
 			} else
 				ReportError ("msg.syntax");
 			return null; // should never reach here
-		}		
+		}
 	}
 }
