@@ -5,6 +5,7 @@
 //	Cesar Octavio Lopez Nataren
 //
 // (C) 2003, 2004 Cesar Octavio Lopez Nataren, <cesar@ciencias.unam.mx>
+// Copyright (C) 2005 Novell Inc (http://novell.com)
 //
 
 //
@@ -36,16 +37,24 @@ using System.Collections;
 
 namespace Microsoft.JScript {
 
-	internal class If : AST {
+	internal interface ICanModifyContext {
+		//
+		// Populate the symbol table before resolving references
+		//
+		void PopulateContext (Environment env, string ns);
+		void EmitDecls (EmitContext ec);
+	}
+
+	internal class If : AST, ICanModifyContext {
 
 		internal AST cond, true_stm, false_stm;
 
-		internal If (AST parent, AST condition, AST true_stm, AST false_stm, int line_number)
+		internal If (AST parent, AST condition, AST true_stm, AST false_stm, Location location)
+			: base (parent, location)
 		{
 			this.cond = condition;
 			this.true_stm = true_stm;
 			this.false_stm = false_stm;
-			this.line_number = line_number;
 		}
 
 		public override string ToString ()
@@ -61,26 +70,44 @@ namespace Microsoft.JScript {
 			
 			return sb.ToString ();
 		}
+
+		void ICanModifyContext.PopulateContext (Environment env, string ns)
+		{
+			if (true_stm is ICanModifyContext)
+				((ICanModifyContext) true_stm).PopulateContext (env, ns);
+
+			if (false_stm is ICanModifyContext)
+				((ICanModifyContext) false_stm).PopulateContext (env, ns);
+		}
 		
-		internal override bool Resolve (IdentificationTable context)
+		void ICanModifyContext.EmitDecls (EmitContext ec)
+		{
+			if (true_stm is ICanModifyContext)
+				((ICanModifyContext) true_stm).EmitDecls (ec);
+
+			if (false_stm is ICanModifyContext)
+				((ICanModifyContext) false_stm).EmitDecls (ec);
+		}
+
+		internal override bool Resolve (Environment env)
 		{
 			bool r = true;
 
 			if (cond != null)
 				if (cond is Exp)
-					r &= ((Exp) cond).Resolve (context, false);
+					r &= ((Exp) cond).Resolve (env, false);
 
 			if (true_stm != null)
 				if (true_stm is Exp)
-					r &= ((Exp) true_stm).Resolve (context, true);
+					r &= ((Exp) true_stm).Resolve (env, true);
 				else
-					r &= true_stm.Resolve (context);			
+					r &= true_stm.Resolve (env);
 
 			if (false_stm != null)
 				if (false_stm is Exp)
-					r &= ((Exp) false_stm).Resolve (context, true);
+					r &= ((Exp) false_stm).Resolve (env, true);
 				else
-					r &= false_stm.Resolve (context);			
+					r &= false_stm.Resolve (env);
 			return r;
 		}
 
@@ -90,8 +117,9 @@ namespace Microsoft.JScript {
 			Label false_lbl = ig.DefineLabel ();
 			Label merge_lbl = ig.DefineLabel ();
 			CodeGenerator.fall_true (ec, cond, false_lbl);
+			CodeGenerator.EmitBox (ig, cond);
 			if (true_stm != null)
-				true_stm.Emit (ec);			
+				true_stm.Emit (ec);
 			ig.Emit (OpCodes.Br, merge_lbl);
 			ig.MarkLabel (false_lbl);
 			if (false_stm != null)
@@ -103,6 +131,11 @@ namespace Microsoft.JScript {
 	abstract class Jump : AST {
 		protected string label = String.Empty;
 		protected object binding;
+		
+		internal Jump (AST parent, Location location)
+			: base (parent, location)
+		{
+		}
 
 		bool IsLabel (object binding)
 		{
@@ -119,14 +152,13 @@ namespace Microsoft.JScript {
 	}
 
 	internal class Continue : Jump {
-		internal Continue (AST parent, string label, int line_number)
+		internal Continue (AST parent, string label, Location location)
+			: base (parent, location)
 		{
-			this.parent = parent;
 			this.label += label;
-			this.line_number = line_number;
 		}
 
-		internal override bool Resolve (IdentificationTable context)
+		internal override bool Resolve (Environment env)
                 {
                         if (!InLoop)
                                 throw new Exception ("A continue can't be outside a iteration stm");
@@ -147,14 +179,13 @@ namespace Microsoft.JScript {
 
 	internal class Break : Jump {
 
-		internal Break (AST parent, string label, int line_number)
+		internal Break (AST parent, string label, Location location)
+			: base (parent, location)
 		{
-			this.parent = parent;
 			this.label += label;
-			this.line_number = line_number;
 		}
 
-		 internal override bool Resolve (IdentificationTable context)
+		internal override bool Resolve (Environment env)
                 {
                         if (!InLoop && !InSwitch)
 				throw new Exception ("A break statement can't be outside a switch or iteration stm");
@@ -180,9 +211,14 @@ namespace Microsoft.JScript {
 	
 	internal class Return : AST {
 
-		internal AST expression;		
+		internal AST expression;
+		private bool exp_returns_void = false;
 		public event NotVoidReturnEventHandler not_void_return;
-		
+	
+		internal Return (Location location)
+			: base (null, location)
+		{
+		}
 		
 		internal void OnNotVoidReturn (NotVoidReturnEventArgs args)
 		{
@@ -190,11 +226,10 @@ namespace Microsoft.JScript {
 				not_void_return (this, args);
 		}
 
-		internal void Init (AST parent, AST exp, int line_number)
+		internal void Init (AST parent, AST exp)
 		{
 			this.parent = parent;
 			expression = exp;
-			this.line_number = line_number;
 			Function cont_func = GetContainerFunction;
 			this.not_void_return = new NotVoidReturnEventHandler (cont_func.NotVoidReturnHappened);
 		}
@@ -207,13 +242,24 @@ namespace Microsoft.JScript {
 				return String.Empty;
 		}
 
-		internal override bool Resolve (IdentificationTable context)
+		internal override bool Resolve (Environment env)
 		{
 			if (!InFunction)
 				throw new Exception ("error JS1018: 'return' statement outside of function");
 			if (expression != null) {
+				if (expression is Expression) {
+					AST ast = ((Expression) expression).Last;
+					if (ast is Call) {
+						Call call = (Call) ast;
+						if (call.member_exp is Identifier) {
+							object obj = env.Get (String.Empty, ((Identifier) call.member_exp).name);
+							if (obj is Function)
+								exp_returns_void = ((Function) obj).HandleReturnType == typeof (void);
+						}
+					}
+				}					
 				OnNotVoidReturn (null);
-				return expression.Resolve (context);
+				return expression.Resolve (env);
 			} else 
 				return true;
 		}
@@ -227,6 +273,10 @@ namespace Microsoft.JScript {
 			if (expression != null) {
 				expression.Emit (ec);
 				loc = ig.DeclareLocal (typeof (object));
+
+				if (exp_returns_void)
+					ig.Emit (OpCodes.Ldnull);
+
 				ig.Emit (OpCodes.Stloc, loc);
 			}
 					 
@@ -238,26 +288,42 @@ namespace Microsoft.JScript {
 		}
 	}
 
-	internal class DoWhile : AST {
+	internal class DoWhile : AST, ICanModifyContext {
 
 		AST stm, exp;
-		
-		internal void Init (AST parent, AST stm, AST exp, int line_number)
+
+		internal DoWhile (Location location)
+			: base (null, location)
+		{
+		}
+
+		internal void Init (AST parent, AST stm, AST exp)
 		{
 			this.parent = parent;
 			this.stm = stm;
 			this.exp = exp;
-			this.line_number = line_number;
 		}
 
-		internal override bool Resolve (IdentificationTable context)
+		void ICanModifyContext.PopulateContext (Environment env, string ns)
+		{
+			if (stm is ICanModifyContext)
+				((ICanModifyContext) stm).PopulateContext (env, ns);
+		}
+
+		void ICanModifyContext.EmitDecls (EmitContext ec)
+		{
+			if (stm is ICanModifyContext)
+				((ICanModifyContext) stm).EmitDecls (ec);
+		}
+
+		internal override bool Resolve (Environment env)
 		{
 			bool r = true;
 
 			if (stm != null)
-				r &= stm.Resolve (context);
+				r &= stm.Resolve (env);
 			if (exp != null)
-				r &= exp.Resolve (context);
+				r &= exp.Resolve (env);
 			return r;
 		}
 
@@ -290,27 +356,43 @@ namespace Microsoft.JScript {
 	}
 
 
-	internal class While : AST {		
+	internal class While : AST, ICanModifyContext {
 		AST exp, stm;
 
-		internal void Init (AST parent, AST exp, AST stm, int line_number)
+		internal While (Location location)
+			: base (null, location)
+		{
+		}
+
+		internal void Init (AST parent, AST exp, AST stm)
 		{
 			this.parent = parent;
 			this.exp = exp;
 			this.stm = stm;
-			this.line_number = line_number;
 		}
 
-		internal override bool Resolve (IdentificationTable context)
+		void ICanModifyContext.PopulateContext (Environment env, string ns)
+		{
+			if (stm is ICanModifyContext)
+				((ICanModifyContext) stm).PopulateContext (env, ns);
+		}
+
+		void ICanModifyContext.EmitDecls (EmitContext ec)
+		{
+			if (stm is ICanModifyContext)
+				((ICanModifyContext) stm).EmitDecls (ec);
+		}
+		
+		internal override bool Resolve (Environment env)
 		{
 			bool r = true;
 			if (exp != null)
 				if (exp is Exp)
-					r &= ((Exp) exp).Resolve (context, false);
+					r &= ((Exp) exp).Resolve (env, false);
 				else 
-					r &= exp.Resolve (context);
+					r &= exp.Resolve (env);
 			if (stm != null)
-				r &= stm.Resolve (context);
+				r &= stm.Resolve (env);
 			return r;
 		}
 
@@ -349,32 +431,48 @@ namespace Microsoft.JScript {
 		}
 	}
 
-	internal class For : AST {
+	internal class For : AST, ICanModifyContext {
 		
 		AST [] exprs = new AST [3];
 		AST stms;
 
-		internal For (AST parent, int line_number, AST init, AST test, AST incr, AST body)
+		internal For (AST parent, AST init, AST test, AST incr, AST body, Location location)
+			: base (parent, location)
 		{
-			this.parent = parent;
-			this.line_number = line_number;
 			exprs [0] = init;
 			exprs [1] = test;
 			exprs [2] = incr;
 			stms = body;
 		}
 
-		internal override bool Resolve (IdentificationTable context)
+		void ICanModifyContext.PopulateContext (Environment env, string ns)
+		{
+			foreach (AST ast in exprs)
+				if (ast is ICanModifyContext)
+					((ICanModifyContext) ast).PopulateContext (env, ns);
+
+			if (stms is ICanModifyContext)
+				((ICanModifyContext) stms).PopulateContext (env, ns);
+		}
+		
+		void ICanModifyContext.EmitDecls (EmitContext ec)
+		{
+			foreach (AST ast in exprs)
+				if (ast is ICanModifyContext)
+					((ICanModifyContext) ast).EmitDecls (ec);
+
+			if (stms is ICanModifyContext)
+				((ICanModifyContext) stms).EmitDecls (ec);
+		}
+
+		internal override bool Resolve (Environment env)
 		{
 			bool r = true;
 
 			foreach (AST ast in exprs)
-				if (ast is VariableStatement)
-					((VariableStatement) ast).PopulateContext (context);
-				else
-					r &= ast.Resolve (context);
+				r &= ast.Resolve (env);
 			if (stms != null)
-				r &= stms.Resolve (context);
+				r &= stms.Resolve (env);
 			return true;
 		}
 
@@ -390,13 +488,9 @@ namespace Microsoft.JScript {
 
 			/* emit init expr */
 			tmp = exprs [0];
-			if (tmp != null) {
-				if (tmp is VariableStatement) {
-					VariableStatement stm = (VariableStatement) tmp;
-					stm.EmitVariableDecls (ec);
-				}
+			if (tmp != null)
 				tmp.Emit (ec);
-			}
+
                         ec.LoopBegin = ig.DefineLabel ();
 			ec.LoopEnd = ig.DefineLabel ();
 			
@@ -440,17 +534,16 @@ namespace Microsoft.JScript {
 		}
 	}
 
-	internal class Switch : AST {	       
+	internal class Switch : AST, ICanModifyContext {
 
 		internal AST exp;
 		internal ArrayList case_clauses;
 		internal ArrayList default_clauses;
 		internal ArrayList sec_case_clauses;
 
-		internal Switch (AST parent, int line_number)
+		internal Switch (AST parent, Location location)
+			: base (parent, location)
 		{
-			this.parent = parent;
-			this.line_number = line_number;
 			case_clauses = new ArrayList ();
 			default_clauses = new ArrayList ();
 			sec_case_clauses = new ArrayList ();
@@ -464,20 +557,50 @@ namespace Microsoft.JScript {
 				sec_case_clauses.Add (clause);
 		}
 
-		internal override bool Resolve (IdentificationTable context)
+		void ICanModifyContext.PopulateContext (Environment env, string ns)
+		{
+			foreach (AST ast in case_clauses)
+				if (ast is ICanModifyContext)
+					((ICanModifyContext) ast).PopulateContext (env, ns);
+
+			foreach (AST ast in default_clauses)
+				if (ast is ICanModifyContext)
+					((ICanModifyContext) ast).PopulateContext (env, ns);
+
+			foreach (AST ast in sec_case_clauses)
+				if (ast is ICanModifyContext)
+					((ICanModifyContext) ast).PopulateContext (env, ns);
+		}
+		
+		void ICanModifyContext.EmitDecls (EmitContext ec)
+		{
+			foreach (AST ast in case_clauses)
+				if (ast is ICanModifyContext)
+					((ICanModifyContext) ast).EmitDecls (ec);
+
+			foreach (AST ast in default_clauses)
+				if (ast is ICanModifyContext)
+					((ICanModifyContext) ast).EmitDecls (ec);
+
+			foreach (AST ast in sec_case_clauses)
+				if (ast is ICanModifyContext)
+					((ICanModifyContext) ast).EmitDecls (ec);
+		}
+
+		internal override bool Resolve (Environment env)
 		{
 			bool r = true;
 			if (exp != null)
-				r &= exp.Resolve (context);
+				r &= exp.Resolve (env);
 			if (case_clauses != null)
 				foreach (Clause c in case_clauses)
-					r &= c.Resolve (context);
+					r &= c.Resolve (env);
 			if (default_clauses != null)
 				foreach (AST dc in default_clauses)
-					r &= dc.Resolve (context);
+					r &= dc.Resolve (env);
 			if (sec_case_clauses != null)
 				foreach (Clause sc in sec_case_clauses)
-					r &= sc.Resolve (context);
+					r &= sc.Resolve (env);
 			return r;
 		}
 
@@ -489,6 +612,7 @@ namespace Microsoft.JScript {
 			ILGenerator ig = ec.ig;
 			Label init_default = ig.DefineLabel ();
 			Label end_of_default = ig.DefineLabel ();
+			Label old_end = ec.LoopEnd;
 			ec.LoopEnd = ig.DefineLabel ();
 
 			LocalBuilder loc = ig.DeclareLocal (typeof (object));
@@ -525,18 +649,19 @@ namespace Microsoft.JScript {
 					c.EmitStms (ec);				
 				}
 			} 
-			ig.MarkLabel (ec.LoopEnd);			
+			ig.MarkLabel (ec.LoopEnd);
+			ec.LoopEnd = old_end;
 		}
 	}
 
-	internal class Clause : AST {
+	internal class Clause : AST, ICanModifyContext {
 		internal AST exp;
 		internal ArrayList stm_list;
 		internal Label matched_block;
 
-		internal Clause (AST parent)
+		internal Clause (AST parent, Location location)
+			: base (parent, location)
 		{
-			this.parent = parent;
 			stm_list = new ArrayList ();
 		}
 
@@ -545,13 +670,28 @@ namespace Microsoft.JScript {
 			stm_list.Add (stm);
 		}
 
-		internal override bool Resolve (IdentificationTable context)
+		void ICanModifyContext.PopulateContext (Environment env, string ns)
+		{
+			foreach (AST ast in stm_list)
+				if (ast is ICanModifyContext)
+					((ICanModifyContext) ast).PopulateContext (env, ns);
+		}
+		
+
+		void ICanModifyContext.EmitDecls (EmitContext ec)
+		{
+			foreach (AST ast in stm_list)
+				if (ast is ICanModifyContext)
+					((ICanModifyContext) ast).EmitDecls (ec);
+		}
+
+		internal override bool Resolve (Environment env)
 		{
 			bool r = true;
 			if (exp != null)
-				r &= exp.Resolve (context);
+				r &= exp.Resolve (env);
 			foreach (AST ast in stm_list)
-				r &= ast.Resolve (context);
+				r &= ast.Resolve (env);
 			return r;			
 		}
 
@@ -576,7 +716,7 @@ namespace Microsoft.JScript {
 		}	
 	}
 
-	internal class Catch : AST {
+	internal class Catch : AST, ICanModifyContext {
 		internal string id;
 		internal AST catch_cond;
 		internal AST stms;
@@ -584,19 +724,31 @@ namespace Microsoft.JScript {
 		internal FieldBuilder field_info;
 		internal LocalBuilder local_builder;
 
-		internal Catch (string id, AST catch_cond, AST stms, AST parent, int line_number)
+		internal Catch (string id, AST catch_cond, AST stms, AST parent, Location location)
+			: base (parent, location)
 		{
 			this.id = id;
 			this.catch_cond = catch_cond;
 			this.stms = stms;
-			this.line_number = line_number;
 		}
 
-		internal override bool Resolve (IdentificationTable context)
+		void ICanModifyContext.PopulateContext (Environment env, string ns)
+		{
+			if (stms is ICanModifyContext)
+				((ICanModifyContext) stms).PopulateContext (env, ns);
+		}
+		
+		void ICanModifyContext.EmitDecls (EmitContext ec)
+		{
+			if (stms is ICanModifyContext)
+				((ICanModifyContext) stms).EmitDecls (ec);
+		}
+
+		internal override bool Resolve (Environment env)
 		{
 			bool r = true;
 			if (stms != null)
-				r &= stms.Resolve (context);
+				r &= stms.Resolve (env);
 			return r;
 		}
 
@@ -604,24 +756,22 @@ namespace Microsoft.JScript {
 		{
 			ILGenerator ig = ec.ig;
 			Type t = typeof (object);
-			bool not_in_func = parent == null;
+			bool in_function = InFunction;
 
-			if (not_in_func)
-				field_info = ec.type_builder.DefineField (mangle_id (id), t, FieldAttributes.Public | FieldAttributes.Static);
-			else
+			if (in_function)
 				local_builder = ig.DeclareLocal (t);
+			else
+				field_info = ec.type_builder.DefineField (mangle_id (id), t, FieldAttributes.Public | FieldAttributes.Static);
 
 			ig.BeginCatchBlock (typeof (Exception));
-			if (not_in_func) {
-				ig.Emit (OpCodes.Ldarg_0);
-				ig.Emit (OpCodes.Ldfld, typeof (ScriptObject).GetField ("engine"));
-				ig.Emit (OpCodes.Call, typeof (Try).GetMethod ("JScriptExceptionValue"));
-				ig.Emit (OpCodes.Stsfld, field_info);
-			} else {
-				ig.Emit (OpCodes.Ldarg_1);
-				ig.Emit (OpCodes.Call, typeof (Try).GetMethod ("JScriptExceptionValue"));
+			CodeGenerator.load_engine (in_function, ig);
+			ig.Emit (OpCodes.Call, typeof (Try).GetMethod ("JScriptExceptionValue"));
+			
+			if (in_function)
 				ig.Emit (OpCodes.Stloc, local_builder);
-			}
+			else
+				ig.Emit (OpCodes.Stsfld, field_info);
+
 			stms.Emit (ec);			
 		}
 
@@ -631,12 +781,17 @@ namespace Microsoft.JScript {
 		}
 	}
 
-	internal class Labelled : AST {
+	internal class Labelled : AST, ICanModifyContext {
 		string name;
 		Label init_addrs; 
 		Label end_addrs;
 		AST stm;
 
+		internal Labelled (AST parent, Location location)
+			: base (parent, location)
+		{
+		}
+
 		internal Label InitAddrs {
 			set { init_addrs = value; }
 			get { return init_addrs; }
@@ -647,15 +802,27 @@ namespace Microsoft.JScript {
 			get { return end_addrs; }
 		}
 
-		internal void Init (AST parent, string name, AST stm, int line_number)
+		internal void Init (AST parent, string name, AST stm, Location location)
 		{
 			this.parent = parent;
 			this.name = name;
 			this.stm = stm;
-			this.line_number = line_number;
+			this.location = location;
 		}
 
-		internal override bool Resolve (IdentificationTable context)
+		void ICanModifyContext.PopulateContext (Environment env, string ns)
+		{
+			if (stm is ICanModifyContext)
+				((ICanModifyContext) stm).PopulateContext (env, ns);
+		}
+		
+		void ICanModifyContext.EmitDecls (EmitContext ec)
+		{
+			if (stm is ICanModifyContext)
+				((ICanModifyContext) stm).EmitDecls (ec);
+		}
+
+		internal override bool Resolve (Environment env)
 		{
 			try {
 				SemanticAnalyser.AddLabel (name, this);
@@ -663,7 +830,7 @@ namespace Microsoft.JScript {
 				throw new Exception ("error JS1025: Label redefined");
 			}
 			if (stm != null)
-				stm.Resolve (context);
+				stm.Resolve (env);
 			SemanticAnalyser.RemoveLabel (name);
 			return true;
 		}
