@@ -767,10 +767,12 @@ internal class TokenStream {
 					if (_base == 10 && !is_integer) {
 						try {
 							// Use C# conversion to number from string
-							dval = Double.Parse (num_string);
+							dval = Double.Parse (num_string, CultureInfo.InvariantCulture);
 						} catch (FormatException ex) {
 							ReportCurrentLineError ("msg.caught.nfe");
 							return Token.ERROR;
+						} catch (OverflowException) {
+							dval = Double.NaN;
 						}
 					} else
 						dval = StringToNumber (num_string, 0, _base);
@@ -1152,7 +1154,7 @@ internal class TokenStream {
 				 * answer.
 				 */
 				try {
-					return Double.Parse (s);
+					return Double.Parse (s, CultureInfo.InvariantCulture);
 				} catch (FormatException fe) {
 					return Double.NaN;
 				}
@@ -1437,7 +1439,7 @@ internal class TokenStream {
 
 	internal void ReportCurrentLineError (string message)
 	{
-		Console.WriteLine ("error: {0}, {1}, {2}", message, SourceName, LineNumber);
+		Console.WriteLine ("{0} ({1}, 0): error: {2}", SourceName, LineNumber, message);
 	}
 
 	// FIXME: we don't check for combining mark yet
