@@ -209,8 +209,12 @@ namespace Microsoft.VisualBasic {
 				case TypeCode.Single:
 					return Int (Convert.ToSingle (Number));
 				case TypeCode.String:
+					// Just to throw the right exception
+					return Int (DoubleType.FromString (Number.ToString ()));
+				/*
 					return Int (Double.Parse (
 					CastToString(Number)));
+				*/
 
 				// Int on integer types does nothing
 				case TypeCode.Byte:
@@ -227,8 +231,9 @@ namespace Microsoft.VisualBasic {
 				case TypeCode.Empty:
 					return 0;
 
-				// otherwise, it's we can't cast to a numeric
 				case TypeCode.Boolean:
+					return Int (Convert.ToDouble (Number));
+				// otherwise, it's we can't cast to a numeric
 				case TypeCode.Char:
 				case TypeCode.DateTime:
 				case TypeCode.DBNull:
@@ -699,7 +704,7 @@ namespace Microsoft.VisualBasic {
 			}
 		
 			try {
-				return Val (CastToString (Expression)); 
+				return Val (StringType.FromObject (Expression)); 
 			} 
 			catch {
 				
