@@ -67,10 +67,10 @@ namespace Microsoft.VisualBasic
 			close(iArr);
 		}
 
-		public static void FileClose(int[] fileNumbers)
+		public static void FileClose(params int[] fileNumbers)
 		{
 			int[] iArr = null;
-			if (fileNumbers.Length == 0)
+			if (fileNumbers == null || fileNumbers.Length == 0)
 			{
 				ICollection keySet = FileSystem._openFilesMap.Keys;
 				iArr = new int[keySet.Count];
@@ -154,7 +154,8 @@ namespace Microsoft.VisualBasic
 				VBFile vbFile = new BinaryVBFile(fileName,mode,access,recordLength);
 				_openFilesMap.Add(fileNumber, vbFile);
 			}
-			_fileNameIdMap.Add(Path.GetFullPath(string.Intern(fileName)),fileNumber);
+			if (share != OpenShare.Shared && !_fileNameIdMap.ContainsKey(Path.GetFullPath(string.Intern(fileName))))
+				_fileNameIdMap.Add(Path.GetFullPath(string.Intern(fileName)),fileNumber);
 		}
 
 		public static void FilePut(int fileNumber,
@@ -545,7 +546,7 @@ namespace Microsoft.VisualBasic
 			return vbFile.readLine();
 		}
 
-		public static void Print(int fileNumber, Object[] output)
+		public static void Print(int fileNumber, params Object[] output)
 		{
 			VBFile vbFile = getVBFile(fileNumber);
 			vbFile.print(output);
@@ -558,19 +559,19 @@ namespace Microsoft.VisualBasic
 //			vbFile.printLine(null);
 //		}
 
-		public static void PrintLine(int fileNumber, Object[] output)
+		public static void PrintLine(int fileNumber, params Object[] output)
 		{
 			VBFile vbFile = getVBFile(fileNumber);
 			vbFile.printLine(output);
 		}
 
-		public static void Write(int fileNumber, Object[] output)
+		public static void Write(int fileNumber, params Object[] output)
 		{
 			VBFile vbFile = getVBFile(fileNumber);
 			vbFile.write(output);
 		}
 
-		public static void WriteLine(int fileNumber, Object[] output)
+		public static void WriteLine(int fileNumber, params Object[] output)
 		{
 			VBFile vbFile = getVBFile(fileNumber);
 			vbFile.writeLine(output);
@@ -584,8 +585,8 @@ namespace Microsoft.VisualBasic
 										   VBErrors.FileAlreadyExists);
 			try
 			{
-				Directory.Move(oldPath, newPath);
-			}catch (DirectoryNotFoundException e){
+				File.Move(oldPath, newPath);
+			}catch (Exception e){
 				throw (ArgumentException)ExceptionUtils.VbMakeException(VBErrors.IllegalFuncCall);
 			}
 		}
@@ -601,6 +602,7 @@ namespace Microsoft.VisualBasic
 							    Utils.GetResourceString("Argument_PathNullOrEmpty"));
 			}
 
+
 			if ((destination == null) || (destination.Length == 0))
 			{
 				ExceptionUtils.VbMakeException(VBErrors.BadFileNameOrNumber);
@@ -635,11 +637,14 @@ namespace Microsoft.VisualBasic
 
 			}
 
+/*
 			f = new FileInfo(destination);
 			if (f.Exists)
 				throw (IOException) ExceptionUtils.VbMakeException(
 										   VBErrors.FileAlreadyExists);
-			File.Copy(source, destination);
+*/
+
+			File.Copy(source, destination, true);
 		}
 
 		public static void MkDir(String path)
@@ -945,6 +950,12 @@ namespace Microsoft.VisualBasic
 
 		public static void ChDrive(char Drive)
 		{
+#if NET_2_0
+			if (Environment.OSVersion.Platform == PlatformID.Unix)
+#else
+			if (Environment.OSVersion.Platform == (PlatformID) 128)
+#endif
+				throw new InvalidOperationException("ChDrive function is not supported on this platform !");
 			Drive = char.ToUpper(Drive, CultureInfo.InvariantCulture);
 			if ((Drive < 65) || (Drive > 90))
 			{
