@@ -183,54 +183,45 @@ namespace Microsoft.VisualBasic
 
 			for (int i = 0; i < count; i++)
 			{
-				if (InStr(1, Source[i], Match, Compare) != 0)
-				{
-					//found one more
+				int result = InStr(1, Source[i], Match, Compare);
+				if (Include) {
+					if (result != 0) {
+						matches[i] = true;
+						matchesCount ++;
+					} else
+						matches[i] = false;
+				} else if (result == 0) {
 					matches[i] = true;
 					matchesCount ++;
-				}
-				else
-				{
+				} else
 					matches[i] = false;
-				}
 			}
 
-			if (matchesCount == 0)
+			if (matchesCount == count)
 			{
 				if (Include)
-					return new string[0];
-				else
 					return Source;
-			}
-			else
-			{
-				if (matchesCount == count)
-				{
-					if (Include)
-						return Source;
-					else
-						return new string[0];
-				}
 				else
-				{
-					string[] ret;
-					int j = 0;
-					if (Include)
-						ret = new string [matchesCount];
-					else
-						ret = new string [count - matchesCount];
+					return new string[0];
+			}
 
-					for (int i=0; i < count; i++)
-					{
-						if ((matches[i] && Include) || !(matches[i] || Include))
-						{
-							ret[j] = Source[i];
-							j++;
-						}
-					}
-					return ret;
+			string[] ret;
+			int cnt = 0;
+			for (int i = 0; i < count; i++) {
+				if (matches [i] && Source [i] != null)
+					cnt ++;
+			}
+			ret = new string [cnt];
+			cnt = 0;
+			for (int i=0; i < count; i++)
+			{
+				if (matches [i] && Source [i] != null)
+				{
+					ret[cnt] = Source[i];
+					cnt++;
 				}
 			}
+			return ret;
 		}
 
 		public static string Format(object expression, 
@@ -447,7 +438,7 @@ namespace Microsoft.VisualBasic
 			case DateFormat.LongTime:
 				return Expression.ToString("T");
 			case DateFormat.ShortTime:
-				return Expression.ToString("t");
+				return String.Format ("{0}:{1}", Expression.Hour, Expression.Minute);
 			default:
 				throw new ArgumentException("Argument 'NamedFormat' must be a member of DateFormat", "NamedFormat");
 			}
@@ -470,15 +461,35 @@ namespace Microsoft.VisualBasic
 											      
 			if (Expression == null)
 				return "";
-															     
+
+			if (Expression is string)
+				Expression = DoubleType.FromString ( (string) Expression);
+			if (Expression is bool)
+				Expression = - (Convert.ToDouble ((bool) Expression));
+
 			if (!(Expression is IFormattable))
 				throw new InvalidCastException(
 							       VBUtils.GetResourceString("InvalidCast_FromStringTo",Expression.ToString(),"Double"));
 
-			String formatStr = "00";
+			CultureInfo currentCulture = Utils.GetCultureInfo ();
+			NumberFormatInfo ninfo = currentCulture.NumberFormat;
+
+			if (NumDigitsAfterDecimal == -1)
+				NumDigitsAfterDecimal = ninfo.NumberDecimalDigits;
+
+			String formatStr = "";
+			if (IncludeLeadingDigit == TriState.True)
+				formatStr = "0";
+
+			if (GroupDigits == TriState.UseDefault) {
+				if (ninfo.NumberGroupSizes == null || ninfo.NumberGroupSizes.Length == 0)
+					GroupDigits = TriState.False;
+				else
+					GroupDigits = TriState.True;
+			}
 
 			if (GroupDigits == TriState.True)
-				formatStr = formatStr + ",00";
+				formatStr = "0,0";
 
 			if (NumDigitsAfterDecimal > -1)	{
 				string decStr = ".";
@@ -504,7 +515,10 @@ namespace Microsoft.VisualBasic
 			case "System.Int16":	case "System.Int32":	case "System.Int64":
 			case "System.Double":	case "System.Single":	case "System.UInt16":
 			case "System.UInt32":	case "System.UInt64":
-				returnstr = Convert.ToDouble(Expression).ToString (formatStr);
+				if (formatStr != "")
+					returnstr = Convert.ToDouble(Expression).ToString (formatStr);
+				else
+					returnstr = Convert.ToDouble(Expression).ToString ();
 				break;
 			default:
 				throw new InvalidCastException(
@@ -531,15 +545,32 @@ namespace Microsoft.VisualBasic
 											      
 			if (Expression == null)
 				return "";
-															     
+
+			if (Expression is string)
+				Expression = DoubleType.FromString ((string) Expression);
+
 			if (!(Expression is IFormattable))
 				throw new InvalidCastException(
 							       VBUtils.GetResourceString("InvalidCast_FromStringTo",Expression.ToString(),"Double"));
 
-			String formatStr = "00";
+			CultureInfo currentCulture = Utils.GetCultureInfo ();
+			NumberFormatInfo ninfo = currentCulture.NumberFormat;
+
+			if (NumDigitsAfterDecimal == -1)
+				NumDigitsAfterDecimal = ninfo.NumberDecimalDigits;
+
+			String formatStr = "";
+			if (IncludeLeadingDigit == TriState.True)
+				formatStr = "0";
+			if (GroupDigits == TriState.UseDefault) {
+				if (ninfo.NumberGroupSizes == null || ninfo.NumberGroupSizes.Length == 0)
+					GroupDigits = TriState.False;
+				else
+					GroupDigits = TriState.True;
+			}
 
 			if (GroupDigits == TriState.True)
-				formatStr = formatStr + ",00";
+				formatStr = "0,0";
 
 			if (NumDigitsAfterDecimal > -1) {
 				string decStr = ".";
@@ -556,7 +587,8 @@ namespace Microsoft.VisualBasic
 				formatStr = formatStr + ")";
 			}
 
-			formatStr = formatStr + "%";
+			if (formatStr != "")
+				formatStr = formatStr + "%";
 
 			string returnstr=null;
 			string expstring= Expression.GetType().ToString();
@@ -565,7 +597,10 @@ namespace Microsoft.VisualBasic
 			case "System.Int16":	case "System.Int32":	case "System.Int64":
 			case "System.Double":	case "System.Single":	case "System.UInt16":
 			case "System.UInt32":	case "System.UInt64":
-				returnstr = Convert.ToDouble(Expression).ToString (formatStr);
+				if (formatStr != "")
+					returnstr = Convert.ToDouble(Expression).ToString (formatStr);
+				else
+					returnstr = Convert.ToDouble(Expression).ToString ();
 				break;
 			default:
 				throw new InvalidCastException(
@@ -638,7 +673,7 @@ namespace Microsoft.VisualBasic
 					   CompareMethod Compare)
 		{
 			if ((Start == 0) || (Start < -1))
-				throw new ArgumentException("Argument 'StringCheck' must be greater than 0 or equal to -1", "StringCheck");
+				throw new ArgumentException("Argument 'Start' must be greater than 0 or equal to -1", "Start");
 
 			if (StringCheck == null)
 				return 0;
@@ -646,41 +681,20 @@ namespace Microsoft.VisualBasic
 			if (Start == -1)
 				Start = StringCheck.Length;
 										
-			if (Start > StringCheck.Length || StringCheck.Length == 0)
-				return 0;
-																		  
 			if (StringMatch == null || StringMatch.Length == 0)
 				return Start;
 
-			int retindex = -1;
-			int index = -1;
-			while (index == 0){
-				switch (Compare)
-				{
-				case CompareMethod.Text:
-					index = System.Globalization.CultureInfo.CurrentCulture.CompareInfo.IndexOf(
-														    StringCheck.ToLower(System.Globalization.CultureInfo.CurrentCulture), 
-														    StringMatch.ToLower(System.Globalization.CultureInfo.CurrentCulture), 
-														    Start - 1) + 1;
-					break;
-				case CompareMethod.Binary:
-					index = StringCheck.IndexOf(StringMatch, Start - 1) + 1;
-					break;
-				default:
-					throw new System.ArgumentException("Argument 'Compare' must be CompareMethod.Binary or CompareMethod.Text.", "Compare");
-				}
-				if (index == 0){
-					if (retindex == -1)
-						return index;
-					else
-						return retindex;
-				}
-				else {
-					retindex = index;
-					Start = index;
-				}
+			if (Start > StringCheck.Length || StringCheck.Length == 0)
+				return 0;																		  
+
+			if (Compare == CompareMethod.Text) {
+				// FIXME: this wastes memory and time, remove when CompareInfo.LastIndexOf works correctly 
+				StringCheck = StringCheck.ToLower();
+				StringMatch = StringMatch.ToLower();
+				// FIXME: depends on Managed Collation being ready to be able to use with CompareOptions.IgnoreCase
+				// return CultureInfo.CurrentCulture.CompareInfo.LastIndexOf(StringCheck, StringMatch, Start - 1, CompareOptions.IgnoreCase) + 1; 
 			}
-			return retindex;
+			return StringCheck.LastIndexOf(StringMatch, Start - 1) + 1;
 		}
 
 		public static string Join(string[] SourceArray, 
@@ -709,7 +723,8 @@ namespace Microsoft.VisualBasic
 				string[] dest;
 				dest = new string[SourceArray.Length];
 
-				SourceArray.CopyTo(dest, 0);
+				for (int i = 0; i < SourceArray.Length; i++)
+					dest [i] = StringType.FromObject (SourceArray [i]);
 				return string.Join(Delimiter, dest);
 			}
 			catch (System.InvalidCastException ie){
@@ -992,37 +1007,52 @@ namespace Microsoft.VisualBasic
 					     CompareMethod Compare)
 		{
 			if (Expression == null)
-				return new string[1];
+				return new string[1] {""};
 
 			if ((Delimiter == null) || (Delimiter.Length == 0)){
 				string [] ret = new string[1];
 				ret[0] = Expression;
 				return ret;
 			}
-			if (Limit == 0)
-				Limit = 1; 
-			else if (Limit < -1)
+			if (Limit < -1)
 				throw new OverflowException("Arithmetic operation resulted in an overflow.");
 
-			if (Limit != -1) {
-				switch (Compare){
-				case CompareMethod.Binary:
-					return Expression.Split(Delimiter.ToCharArray(0, 1), Limit);
-				case CompareMethod.Text:
-					return Expression.Split(Delimiter.ToCharArray(0, 1), Limit);
-				default:
-					throw new System.ArgumentException("Argument 'Compare' must be CompareMethod.Binary or CompareMethod.Text.", "Compare");
-				}
-			} else {
-				switch (Compare) {
-				case CompareMethod.Binary:
-					return Expression.Split(Delimiter.ToCharArray(0, 1));
-				case CompareMethod.Text:
-					return Expression.Split(Delimiter.ToCharArray(0, 1));
-				default:
-					throw new System.ArgumentException("Argument 'Compare' must be CompareMethod.Binary or CompareMethod.Text.", "Compare");
-				}
+			string source = Expression;
+			string limitStr = Delimiter;
+			if (Compare == CompareMethod.Text) {
+				source = Expression.ToLower ();
+				limitStr = Delimiter.ToLower ();
 			}
+
+			if (Limit == 0)
+				return new string [0] {};
+
+			if (Limit == -1)
+				Limit = Expression.Length;
+			string [] retArray = new string [Limit];
+			int numMatches = 0;
+			int curIndex = 0;
+			int beg = 0;
+			while (beg + limitStr.Length <= source.Length) {
+				string tmpStr = source.Substring (beg);
+				int indexOf = tmpStr.IndexOf (limitStr);
+				if (indexOf == -1)
+					break;
+
+				retArray [numMatches ++] = Expression.Substring (beg, indexOf);
+				if (numMatches == Limit)
+					break;
+				beg += (indexOf + limitStr.Length); 
+			}
+
+			if (numMatches == 0)
+				return new string [1] {Expression};
+
+			string [] newArray = new string [numMatches];
+			for (int i = 0; i < numMatches; i++)
+				newArray [i] = retArray [i];
+
+			return newArray;
 		}
 
 		public static int StrComp(string String1, 
@@ -1038,7 +1068,7 @@ namespace Microsoft.VisualBasic
 			switch (Compare)
 			{
 			case CompareMethod.Binary:
-				return string.Compare(String2, String1, false);
+				return string.Compare(String1, String2, false);
 			case CompareMethod.Text:
 				CultureInfo curCulture = CultureInfo.CurrentCulture;
 				return curCulture.CompareInfo.Compare(String1.ToLower(curCulture), String2.ToLower(curCulture));
