@@ -31,6 +31,17 @@
 //    Methods that require tuning are bound as `private sys_NAME' methods
 //    and then a `NAME' method is exposed.
 //
+// Deprecated Warning:
+//
+//    This class is deprecated, and exists only for backward compatibility
+//    with development versions of Mono 1.1.x.  It will be removed with 
+//    Mono 1.2.  Migrate to the Mono.Unix.Native types, or use the Unix*
+//    wrapper classes instead.
+//
+//    The [Map] attributes have been removed.  However, since the type names
+//    are identical to those in Mono.Unix.Native, the MonoPosixHelper methods
+//    will continue to exist and function correctly.
+//
 
 //
 // Permission is hereby granted, free of charge, to any person obtaining
@@ -59,32 +70,15 @@ using System.Runtime.InteropServices;
 using System.Text;
 using Mono.Unix;
 
-[assembly:Mono.Unix.IncludeAttribute (
-	new string [] {
-		"sys/types.h", 
-		"sys/stat.h", 
-		"ah:sys/poll.h", 
-		"ah:sys/wait.h",
-		"ah:sys/statvfs.h",
-		"ah:sys/xattr.h",
-		"unistd.h", 
-		"fcntl.h", 
-		"signal.h", 
-		"ah:poll.h", 
-		"ah:grp.h", 
-		"errno.h", 
-		"ah:syslog.h",
-	}, 
-	new string [] {
-		"_GNU_SOURCE", 
-		"_XOPEN_SOURCE",
-	})]
+using FileNameMarshaler = Mono.Unix.Native.FileNameMarshaler;
 
 namespace Mono.Unix {
 
 	#region Enumerations
 
-	[Flags][Map]
+	[Flags]
+	[CLSCompliant (false)]
+	[Obsolete ("Use Mono.Unix.Native.SyslogOptions")]
 	public enum SyslogOptions {
 		LOG_PID    = 0x01,  // log the pid with each message
 		LOG_CONS   = 0x02,  // log on the console if errors in sending
@@ -94,9 +88,12 @@ namespace Mono.Unix {
 		LOG_PERROR = 0x20   // log to stderr as well
 	}
 
-	[Flags][Map]
+	[CLSCompliant (false)]
+	[Obsolete ("Use Mono.Unix.Native.SyslogFacility")]
 	public enum SyslogFacility {
 		LOG_KERN      = 0 << 3,
+		LOG_USER      = 1 << 3,
+		[Obsolete ("use SyslogFacility.LOG_USER")]
 		LOG_USRE      = 1 << 3,
 		LOG_MAIL      = 2 << 3,
 		LOG_DAEMON    = 3 << 3,
@@ -105,7 +102,7 @@ namespace Mono.Unix {
 		LOG_LPR       = 6 << 3,
 		LOG_NEWS      = 7 << 3,
 		LOG_UUCP      = 8 << 3,
-		LOG_CRON      = 8 << 3,
+		LOG_CRON      = 9 << 3,
 		LOG_AUTHPRIV  = 10 << 3,
 		LOG_FTP       = 11 << 3,
 		LOG_LOCAL0    = 16 << 3,
@@ -118,7 +115,8 @@ namespace Mono.Unix {
 		LOG_LOCAL7    = 23 << 3,
 	}
 
-	[Flags][Map]
+	[CLSCompliant (false)]
+	[Obsolete ("Use Mono.Unix.Native.SyslogLevel")]
 	public enum SyslogLevel {
 		LOG_EMERG   = 0,  // system is unusable
 		LOG_ALERT   = 1,  // action must be taken immediately
@@ -130,7 +128,9 @@ namespace Mono.Unix {
 		LOG_DEBUG   = 7   // debug-level messages
 	}
 
-	[Map][Flags]
+	[Flags]
+	[CLSCompliant (false)]
+	[Obsolete ("Use Mono.Unix.Native.OpenFlags")]
 	public enum OpenFlags : int {
 		//
 		// One of these
@@ -168,7 +168,9 @@ namespace Mono.Unix {
 	}
 	
 	// mode_t
-	[Flags][Map]
+	[Flags]
+	[CLSCompliant (false)]
+	[Obsolete ("Use Mono.Unix.Native.FilePermissions")]
 	public enum FilePermissions : uint {
 		S_ISUID     = 0x0800, // Set user ID on execution
 		S_ISGID     = 0x0400, // Set gorup ID on execution
@@ -202,7 +204,8 @@ namespace Mono.Unix {
 		S_IFSOCK    = 0xC000, // Socket
 	}
 
-	[Map]
+	[CLSCompliant (false)]
+	[Obsolete ("Use Mono.Unix.Native.FcntlCommand")]
 	public enum FcntlCommand : int {
 		// Form /usr/include/bits/fcntl.h
 		F_DUPFD    =    0, // Duplicate file descriptor.
@@ -222,14 +225,16 @@ namespace Mono.Unix {
 		F_NOTIFY   = 1026, // Required notifications on a directory
 	}
 
-	[Map]
+	[CLSCompliant (false)]
+	[Obsolete ("Use Mono.Unix.Native.LockType")]
 	public enum LockType : short {
 		F_RDLCK = 0, // Read lock.
 		F_WRLCK = 1, // Write lock.
 		F_UNLCK = 2, // Remove lock.
 	}
 
-	[Map]
+	[CLSCompliant (false)]
+	[Obsolete ("Use Mono.Unix.Native.SeekFlags")]
 	public enum SeekFlags : short {
 		// values liberally copied from /usr/include/unistd.h
 		SEEK_SET = 0, // Seek from beginning of file.
@@ -241,7 +246,9 @@ namespace Mono.Unix {
 		L_XTND   = SEEK_END, // BSD alias for SEEK_END
 	}
 	
-	[Map, Flags]
+	[Flags]
+	[CLSCompliant (false)]
+	[Obsolete ("Use Mono.Unix.Native.DirectoryNotifyFlags")]
 	public enum DirectoryNotifyFlags : int {
 		// from /usr/include/bits/fcntl.h
 		DN_ACCESS    = 0x00000001, // File accessed.
@@ -253,7 +260,8 @@ namespace Mono.Unix {
 		DN_MULTISHOT = unchecked ((int)0x80000000), // Don't remove notifier
 	}
 
-	[Map]
+	[CLSCompliant (false)]
+	[Obsolete ("Use Mono.Unix.Native.PosixFadviseAdvice")]
 	public enum PosixFadviseAdvice : int {
 		POSIX_FADV_NORMAL     = 0,  // No further special treatment.
 		POSIX_FADV_RANDOM     = 1,  // Expect random page references.
@@ -263,7 +271,8 @@ namespace Mono.Unix {
 		POSIX_FADV_NOREUSE    = 5,  // Data will be accessed once.
 	}
 
-	[Map]
+	[CLSCompliant (false)]
+	[Obsolete ("Use Mono.Unix.Native.PosixMadviseAdvice")]
 	public enum PosixMadviseAdvice : int {
 		POSIX_MADV_NORMAL     = 0,  // No further special treatment.
 		POSIX_MADV_RANDOM     = 1,  // Expect random page references.
@@ -272,7 +281,8 @@ namespace Mono.Unix {
 		POSIX_MADV_DONTNEED   = 4,  // Don't need these pages.
 	}
 
-	[Map]
+	[CLSCompliant (false)]
+	[Obsolete ("Use Mono.Unix.Native.Signum")]
 	public enum Signum : int {
 		SIGHUP    =  1, // Hangup (POSIX).
 		SIGINT    =  2, // Interrupt (ANSI).
@@ -311,13 +321,17 @@ namespace Mono.Unix {
 		SIGUNUSED = 31
 	}
 
-	[Flags][Map]
+	[Flags]
+	[CLSCompliant (false)]
+	[Obsolete ("Use Mono.Unix.Native.WaitOptions")]
 	public enum WaitOptions : int {
 		WNOHANG   = 1,  // Don't block waiting
 		WUNTRACED = 2,  // Report status of stopped children
 	}
 
-  [Flags][Map]
+	[Map][Flags]
+	[CLSCompliant (false)]
+	[Obsolete ("Use Mono.Unix.Native.AccessModes")]
 	public enum AccessMode : int {
 		R_OK = 1,
 		W_OK = 2,
@@ -326,6 +340,8 @@ namespace Mono.Unix {
 	}
 
 	[Map]
+	[CLSCompliant (false)]
+	[Obsolete ("Use Mono.Unix.Native.PathconfName")]
 	public enum PathConf : int {
 		_PC_LINK_MAX,
 		_PC_MAX_CANON,
@@ -351,6 +367,8 @@ namespace Mono.Unix {
 	}
 
 	[Map]
+	[CLSCompliant (false)]
+	[Obsolete ("Use Mono.Unix.Native.SysconfName")]
 	public enum SysConf : int {
 		_SC_ARG_MAX,
 		_SC_CHILD_MAX,
@@ -559,6 +577,8 @@ namespace Mono.Unix {
 	}
 
 	[Map]
+	[CLSCompliant (false)]
+	[Obsolete ("Use Mono.Unix.Native.ConfstrName")]
 	public enum ConfStr : int {
 		_CS_PATH,			/* The default search path.  */
 		_CS_V6_WIDTH_RESTRICTED_ENVS,
@@ -606,7 +626,8 @@ namespace Mono.Unix {
 		_CS_POSIX_V6_LPBIG_OFFBIG_LINTFLAGS
 	}
 
-	[Map]
+	[CLSCompliant (false)]
+	[Obsolete ("Use Mono.Unix.Native.LockfCommand")]
 	public enum LockfCommand : int {
 		F_ULOCK = 0, // Unlock a previously locked region.
 		F_LOCK  = 1, // Lock a region for exclusive use.
@@ -614,7 +635,9 @@ namespace Mono.Unix {
 		F_TEST  = 3, // Test a region for other process locks.
 	}
 
-	[Map][Flags]
+	[Flags]
+	[CLSCompliant (false)]
+	[Obsolete ("Use Mono.Unix.Native.PollEvents")]
 	public enum PollEvents : short {
 		POLLIN      = 0x0001, // There is data to read
 		POLLPRI     = 0x0002, // There is urgent data to read
@@ -629,14 +652,18 @@ namespace Mono.Unix {
 		POLLWRBAND  = 0x0200, // Priority data may be written
 	}
 
-	[Map][Flags]
+	[Flags]
+	[CLSCompliant (false)]
+	[Obsolete ("Use Mono.Unix.Native.XattrFlags")]
 	public enum XattrFlags : int {
 		XATTR_AUTO = 0,
 		XATTR_CREATE = 1,
 		XATTR_REPLACE = 2,
 	}
 
-	[Map][Flags]
+	[Flags]
+	[CLSCompliant (false)]
+	[Obsolete ("Use Mono.Unix.Native.MountFlags")]
 	public enum MountFlags : ulong {
 		ST_RDONLY      =    1,  // Mount read-only
 		ST_NOSUID      =    2,  // Ignore suid and sgid bits
@@ -650,7 +677,9 @@ namespace Mono.Unix {
 		ST_NODIRATIME  = 2048,  // Do not update directory access times
 	}
 
-	[Map][Flags]
+	[Flags]
+	[CLSCompliant (false)]
+	[Obsolete ("Use Mono.Unix.Native.MmapFlags")]
 	public enum MmapFlags : int {
 		MAP_SHARED      = 0x01,     // Share changes.
 		MAP_PRIVATE     = 0x02,     // Changes are private.
@@ -671,6 +700,8 @@ namespace Mono.Unix {
 	}
 
 	[Map][Flags]
+	[CLSCompliant (false)]
+	[Obsolete ("Use Mono.Unix.Native.MmapProts")]
 	public enum MmapProt : int {
 		PROT_READ       = 0x1,  // Page can be read.
 		PROT_WRITE      = 0x2,  // Page can be written.
@@ -682,20 +713,26 @@ namespace Mono.Unix {
 		                              //   growsup vma (mprotect only).
 	}
 
-	[Map][Flags]
+	[Flags]
+	[CLSCompliant (false)]
+	[Obsolete ("Use Mono.Unix.Native.MsyncFlags")]
 	public enum MsyncFlags : int {
 		MS_ASYNC      = 0x1,  // Sync memory asynchronously.
 		MS_SYNC       = 0x4,  // Synchronous memory sync.
 		MS_INVALIDATE = 0x2,  // Invalidate the caches.
 	}
 
-	[Map][Flags]
+	[Flags]
+	[CLSCompliant (false)]
+	[Obsolete ("Use Mono.Unix.Native.MlockallFlags")]
 	public enum MlockallFlags : int {
 		MCL_CURRENT	= 0x1,	// Lock all currently mapped pages.
 		MCL_FUTURE  = 0x2,	// Lock all additions to address
 	}
 
-	[Map][Flags]
+	[Flags]
+	[CLSCompliant (false)]
+	[Obsolete ("Use Mono.Unix.Native.MremapFlags")]
 	public enum MremapFlags : ulong {
 		MREMAP_MAYMOVE = 0x1,
 	}
@@ -704,6 +741,8 @@ namespace Mono.Unix {
 
 	#region Structures
 
+	[CLSCompliant (false)]
+	[Obsolete ("Use Mono.Unix.Native.Flock")]
 	public struct Flock {
 		public LockType         l_type;    // Type of lock: F_RDLCK, F_WRLCK, F_UNLCK
 		public SeekFlags        l_whence;  // How to interpret l_start
@@ -712,6 +751,8 @@ namespace Mono.Unix {
 		public /* pid_t */ int  l_pid;     // PID of process blocking our lock (F_GETLK only)
 	}
 
+	[CLSCompliant (false)]
+	[Obsolete ("Use Mono.Unix.Native.Pollfd")]
 	[StructLayout(LayoutKind.Sequential)]
 	public struct Pollfd {
 		public int fd;
@@ -719,14 +760,22 @@ namespace Mono.Unix {
 		public PollEvents revents;
 	}
 
+	[Obsolete ("Use Mono.Unix.Native.Stat")]
 	public struct Stat {
+		[CLSCompliant (false)]
 		public  /* dev_t */     ulong   st_dev;     // device
+		[CLSCompliant (false)]
 		public  /* ino_t */     ulong   st_ino;     // inode
+		[CLSCompliant (false)]
 		public  FilePermissions         st_mode;    // protection
 		private uint                    _padding_;  // padding for structure alignment
+		[CLSCompliant (false)]
 		public  /* nlink_t */   ulong   st_nlink;   // number of hard links
+		[CLSCompliant (false)]
 		public  /* uid_t */     uint    st_uid;     // user ID of owner
+		[CLSCompliant (false)]
 		public  /* gid_t */     uint    st_gid;     // group ID of owner
+		[CLSCompliant (false)]
 		public  /* dev_t */     ulong   st_rdev;    // device type (if inode device)
 		public  /* off_t */     long    st_size;    // total size, in bytes
 		public  /* blksize_t */ long    st_blksize; // blocksize for filesystem I/O
@@ -736,6 +785,8 @@ namespace Mono.Unix {
 		public  /* time_t */    long    st_ctime;   // time of last status change
 	}
 
+	[CLSCompliant (false)]
+	[Obsolete ("Use Mono.Unix.Native.Statvfs")]
 	public struct Statvfs {
 		public                  ulong f_bsize;	  // file system block size
 		public                  ulong f_frsize;   // fragment size
@@ -750,16 +801,19 @@ namespace Mono.Unix {
 		public                  ulong f_namemax;  // maximum filename length
 	}
 
+	[Obsolete ("Use Mono.Unix.Native.Timeval")]
 	public struct Timeval {
 		public  /* time_t */      long    tv_sec;   // seconds
 		public  /* suseconds_t */ long    tv_usec;  // microseconds
 	}
 
+	[Obsolete ("Use Mono.Unix.Native.Timezone")]
 	public struct Timezone {
 		public  int tz_minuteswest; // minutes W of Greenwich
 		private int tz_dsttime;     // type of dst correction (OBSOLETE)
 	}
 
+	[Obsolete ("Use Mono.Unix.Native.Utimbuf")]
 	public struct Utimbuf {
 		public  /* time_t */      long    actime;   // access time
 		public  /* time_t */      long    modtime;  // modification time
@@ -769,10 +823,13 @@ namespace Mono.Unix {
 
 	#region Classes
 
+	[Obsolete ("Use Mono.Unix.Native.Dirent")]
 	public sealed class Dirent
 	{
+		[CLSCompliant (false)]
 		public /* ino_t */ ulong  d_ino;
 		public /* off_t */ long   d_off;
+		[CLSCompliant (false)]
 		public ushort             d_reclen;
 		public byte               d_type;
 		public string             d_name;
@@ -810,6 +867,7 @@ namespace Mono.Unix {
 		}
 	}
 
+	[Obsolete ("Use Mono.Unix.Native.Fstab")]
 	public sealed class Fstab
 	{
 		public string fs_spec;
@@ -854,10 +912,12 @@ namespace Mono.Unix {
 		}
 	}
 
+	[Obsolete ("Use Mono.Unix.Native.Group")]
 	public sealed class Group
 	{
 		public string           gr_name;
 		public string           gr_passwd;
+		[CLSCompliant (false)]
 		public /* gid_t */ uint gr_gid;
 		public string[]         gr_mem;
 
@@ -924,11 +984,14 @@ namespace Mono.Unix {
 		}
 	}
 
+	[Obsolete ("Use Mono.Unix.Native.Passwd")]
 	public sealed class Passwd
 	{
 		public string           pw_name;
 		public string           pw_passwd;
+		[CLSCompliant (false)]
 		public /* uid_t */ uint pw_uid;
+		[CLSCompliant (false)]
 		public /* gid_t */ uint pw_gid;
 		public string           pw_gecos;
 		public string           pw_dir;
@@ -1065,6 +1128,8 @@ namespace Mono.Unix {
 	//      EntryPointNotFoundException.  This is also consistent with a missing 
 	//      P/Invoke into libc.so.
 	//
+	[CLSCompliant (false)]
+	[Obsolete ("Use Mono.Unix.Native.Syscall")]
 	public sealed class Syscall : Stdlib
 	{
 		new internal const string LIBC  = "libc";
@@ -1094,11 +1159,21 @@ namespace Mono.Unix {
 				EntryPoint="Mono_Posix_Syscall_setxattr")]
 		public static extern int setxattr (string path, string name, byte[] value, ulong size, XattrFlags flags);
 
+		public static int setxattr (string path, string name, byte [] value, ulong size)
+		{
+			return setxattr (path, name, value, size, XattrFlags.XATTR_AUTO);
+		}
+
 		public static int setxattr (string path, string name, byte [] value, XattrFlags flags)
 		{
 			return setxattr (path, name, value, (ulong) value.Length, flags);
 		}
 
+		public static int setxattr (string path, string name, byte [] value)
+		{
+			return setxattr (path, name, value, (ulong) value.Length);
+		}
+
 		// lsetxattr(2)
 		// 	  int lsetxattr (const char *path, const char *name,
 		//                   const void *value, size_t size, int flags);
@@ -1106,11 +1181,21 @@ namespace Mono.Unix {
 				EntryPoint="Mono_Posix_Syscall_lsetxattr")]
 		public static extern int lsetxattr (string path, string name, byte[] value, ulong size, XattrFlags flags);
 
+		public static int lsetxattr (string path, string name, byte [] value, ulong size)
+		{
+			return lsetxattr (path, name, value, size, XattrFlags.XATTR_AUTO);
+		}
+
 		public static int lsetxattr (string path, string name, byte [] value, XattrFlags flags)
 		{
 			return lsetxattr (path, name, value, (ulong) value.Length, flags);
 		}
 
+		public static int lsetxattr (string path, string name, byte [] value)
+		{
+			return lsetxattr (path, name, value, (ulong) value.Length);
+		}
+
 		// fsetxattr(2)
 		// 	  int fsetxattr (int fd, const char *name,
 		//                   const void *value, size_t size, int flags);
@@ -1118,11 +1203,21 @@ namespace Mono.Unix {
 				EntryPoint="Mono_Posix_Syscall_fsetxattr")]
 		public static extern int fsetxattr (int fd, string name, byte[] value, ulong size, XattrFlags flags);
 
+		public static int fsetxattr (int fd, string name, byte [] value, ulong size)
+		{
+			return fsetxattr (fd, name, value, size, XattrFlags.XATTR_AUTO);
+		}
+
 		public static int fsetxattr (int fd, string name, byte [] value, XattrFlags flags)
 		{
 			return fsetxattr (fd, name, value, (ulong) value.Length, flags);
 		}
 
+		public static int fsetxattr (int fd, string name, byte [] value)
+		{
+			return fsetxattr (fd, name, value, (ulong) value.Length);
+		}
+
 		// getxattr(2)
 		// 	  ssize_t getxattr (const char *path, const char *name,
 		//                      void *value, size_t size);
@@ -1272,13 +1367,16 @@ namespace Mono.Unix {
 			return 0;
 		}
 
-		[DllImport (LIBC, SetLastError=true)]
+		[DllImport (MPH, SetLastError=true,
+				EntryPoint="Mono_Posix_Syscall_removexattr")]
 		public static extern int removexattr (string path, string name);
 
-		[DllImport (LIBC, SetLastError=true)]
+		[DllImport (MPH, SetLastError=true,
+				EntryPoint="Mono_Posix_Syscall_lremovexattr")]
 		public static extern int lremovexattr (string path, string name);
 
-		[DllImport (LIBC, SetLastError=true)]
+		[DllImport (MPH, SetLastError=true,
+				EntryPoint="Mono_Posix_Syscall_fremovexattr")]
 		public static extern int fremovexattr (int fd, string name);
 		#endregion
 
@@ -1289,7 +1387,9 @@ namespace Mono.Unix {
 		// TODO: scandir(3), alphasort(3), versionsort(3), getdirentries(3)
 
 		[DllImport (LIBC, SetLastError=true)]
-		public static extern IntPtr opendir (string name);
+		public static extern IntPtr opendir (
+				[MarshalAs (UnmanagedType.CustomMarshaler, MarshalTypeRef=typeof(FileNameMarshaler))]
+				string name);
 
 		[DllImport (LIBC, SetLastError=true)]
 		public static extern int closedir (IntPtr dir);
@@ -1403,19 +1503,25 @@ namespace Mono.Unix {
 
 		[DllImport (MPH, SetLastError=true, 
 				EntryPoint="Mono_Posix_Syscall_open")]
-		public static extern int open (string pathname, OpenFlags flags);
+		public static extern int open (
+				[MarshalAs (UnmanagedType.CustomMarshaler, MarshalTypeRef=typeof(FileNameMarshaler))]
+				string pathname, OpenFlags flags);
 
 		// open(2)
 		//    int open(const char *pathname, int flags, mode_t mode);
 		[DllImport (MPH, SetLastError=true, 
 				EntryPoint="Mono_Posix_Syscall_open_mode")]
-		public static extern int open (string pathname, OpenFlags flags, FilePermissions mode);
+		public static extern int open (
+				[MarshalAs (UnmanagedType.CustomMarshaler, MarshalTypeRef=typeof(FileNameMarshaler))]
+				string pathname, OpenFlags flags, FilePermissions mode);
 
 		// creat(2)
 		//    int creat(const char *pathname, mode_t mode);
 		[DllImport (MPH, SetLastError=true, 
 				EntryPoint="Mono_Posix_Syscall_creat")]
-		public static extern int creat (string pathname, FilePermissions mode);
+		public static extern int creat (
+				[MarshalAs (UnmanagedType.CustomMarshaler, MarshalTypeRef=typeof(FileNameMarshaler))]
+				string pathname, FilePermissions mode);
 
 		// posix_fadvise(2)
 		//    int posix_fadvise(int fd, off_t offset, off_t len, int advice);
@@ -1496,7 +1602,9 @@ namespace Mono.Unix {
 
 		[DllImport (MPH, SetLastError=true,
 				EntryPoint="Mono_Posix_Syscall_getfsfile")]
-		private static extern int sys_getfsfile (string mount_point, out _Fstab fs);
+		private static extern int sys_getfsfile (
+				[MarshalAs (UnmanagedType.CustomMarshaler, MarshalTypeRef=typeof(FileNameMarshaler))]
+				string mount_point, out _Fstab fs);
 
 		public static Fstab getfsfile (string mount_point)
 		{
@@ -1514,7 +1622,9 @@ namespace Mono.Unix {
 
 		[DllImport (MPH, SetLastError=true,
 				EntryPoint="Mono_Posix_Syscall_getfsspec")]
-		private static extern int sys_getfsspec (string special_file, out _Fstab fs);
+		private static extern int sys_getfsspec (
+				[MarshalAs (UnmanagedType.CustomMarshaler, MarshalTypeRef=typeof(FileNameMarshaler))]
+				string special_file, out _Fstab fs);
 
 		public static Fstab getfsspec (string special_file)
 		{
@@ -1628,7 +1738,9 @@ namespace Mono.Unix {
 
 		[DllImport (MPH, SetLastError=true,
 				EntryPoint="Mono_Posix_Syscall_getgrnam_r")]
-		private static extern int sys_getgrnam_r (string name, out _Group grbuf, out IntPtr grbufp);
+		private static extern int sys_getgrnam_r (
+				[MarshalAs (UnmanagedType.CustomMarshaler, MarshalTypeRef=typeof(FileNameMarshaler))]
+				string name, out _Group grbuf, out IntPtr grbufp);
 
 		public static int getgrnam_r (string name, Group grbuf, out Group grbufp)
 		{
@@ -1799,7 +1911,9 @@ namespace Mono.Unix {
 
 		[DllImport (MPH, SetLastError=true,
 				EntryPoint="Mono_Posix_Syscall_getpwnam_r")]
-		private static extern int sys_getpwnam_r (string name, out _Passwd pwbuf, out IntPtr pwbufp);
+		private static extern int sys_getpwnam_r (
+				[MarshalAs (UnmanagedType.CustomMarshaler, MarshalTypeRef=typeof(FileNameMarshaler))]
+				string name, out _Passwd pwbuf, out IntPtr pwbufp);
 
 		public static int getpwnam_r (string name, Passwd pwbuf, out Passwd pwbufp)
 		{
@@ -1949,6 +2063,8 @@ namespace Mono.Unix {
 
 		public static readonly int L_cuserid = _L_cuserid ();
 
+		internal static object getlogin_lock = new object ();
+
 		[DllImport (LIBC, SetLastError=true, EntryPoint="cuserid")]
 		private static extern IntPtr sys_cuserid ([Out] StringBuilder @string);
 
@@ -1960,8 +2076,10 @@ namespace Mono.Unix {
 			if (@string.Capacity < L_cuserid) {
 				throw new ArgumentOutOfRangeException ("string", "string.Capacity < L_cuserid");
 			}
-			IntPtr r = sys_cuserid (@string);
-			return UnixMarshal.PtrToString (r);
+			lock (getlogin_lock) {
+				IntPtr r = sys_cuserid (@string);
+				return UnixMarshal.PtrToString (r);
+			}
 		}
 
 		#endregion
@@ -2147,7 +2265,9 @@ namespace Mono.Unix {
 		//
 		[DllImport (MPH, SetLastError=true, 
 				EntryPoint="Mono_Posix_Syscall_stat")]
-		public static extern int stat (string file_name, out Stat buf);
+		public static extern int stat (
+				[MarshalAs (UnmanagedType.CustomMarshaler, MarshalTypeRef=typeof(FileNameMarshaler))]
+				string file_name, out Stat buf);
 
 		[DllImport (MPH, SetLastError=true, 
 				EntryPoint="Mono_Posix_Syscall_fstat")]
@@ -2155,7 +2275,9 @@ namespace Mono.Unix {
 
 		[DllImport (MPH, SetLastError=true, 
 				EntryPoint="Mono_Posix_Syscall_lstat")]
-		public static extern int lstat (string file_name, out Stat buf);
+		public static extern int lstat (
+				[MarshalAs (UnmanagedType.CustomMarshaler, MarshalTypeRef=typeof(FileNameMarshaler))]
+				string file_name, out Stat buf);
 
 		// TODO:
 		// S_ISDIR, S_ISCHR, S_ISBLK, S_ISREG, S_ISFIFO, S_ISLNK, S_ISSOCK
@@ -2164,7 +2286,9 @@ namespace Mono.Unix {
 		// chmod(2)
 		//    int chmod(const char *path, mode_t mode);
 		[DllImport (LIBC, SetLastError=true, EntryPoint="chmod")]
-		private static extern int sys_chmod (string path, uint mode);
+		private static extern int sys_chmod (
+				[MarshalAs (UnmanagedType.CustomMarshaler, MarshalTypeRef=typeof(FileNameMarshaler))]
+				string path, uint mode);
 
 		public static int chmod (string path, FilePermissions mode)
 		{
@@ -2198,7 +2322,9 @@ namespace Mono.Unix {
 		// mkdir(2)
 		//    int mkdir(const char *pathname, mode_t mode);
 		[DllImport (LIBC, SetLastError=true, EntryPoint="mkdir")]
-		private static extern int sys_mkdir (string oldpath, uint mode);
+		private static extern int sys_mkdir (
+				[MarshalAs (UnmanagedType.CustomMarshaler, MarshalTypeRef=typeof(FileNameMarshaler))]
+				string oldpath, uint mode);
 
 		public static int mkdir (string oldpath, FilePermissions mode)
 		{
@@ -2210,12 +2336,16 @@ namespace Mono.Unix {
 		//    int mknod (const char *pathname, mode_t mode, dev_t dev);
 		[DllImport (MPH, SetLastError=true,
 				EntryPoint="Mono_Posix_Syscall_mknod")]
-		public static extern int mknod (string pathname, FilePermissions mode, ulong dev);
+		public static extern int mknod (
+				[MarshalAs (UnmanagedType.CustomMarshaler, MarshalTypeRef=typeof(FileNameMarshaler))]
+				string pathname, FilePermissions mode, ulong dev);
 
 		// mkfifo(3)
 		//    int mkfifo(const char *pathname, mode_t mode);
 		[DllImport (LIBC, SetLastError=true, EntryPoint="mkfifo")]
-		private static extern int sys_mkfifo (string pathname, uint mode);
+		private static extern int sys_mkfifo (
+				[MarshalAs (UnmanagedType.CustomMarshaler, MarshalTypeRef=typeof(FileNameMarshaler))]
+				string pathname, uint mode);
 
 		public static int mkfifo (string pathname, FilePermissions mode)
 		{
@@ -2232,7 +2362,9 @@ namespace Mono.Unix {
 
 		[DllImport (MPH, SetLastError=true,
 				EntryPoint="Mono_Posix_Syscall_statvfs")]
-		public static extern int statvfs (string path, out Statvfs buf);
+		public static extern int statvfs (
+				[MarshalAs (UnmanagedType.CustomMarshaler, MarshalTypeRef=typeof(FileNameMarshaler))]
+				string path, out Statvfs buf);
 
 		[DllImport (MPH, SetLastError=true,
 				EntryPoint="Mono_Posix_Syscall_fstatvfs")]
@@ -2283,7 +2415,9 @@ namespace Mono.Unix {
 
 		[DllImport (MPH, SetLastError=true, 
 				EntryPoint="Mono_Posix_Syscall_utimes")]
-		public static extern int utimes (string filename, ref Timeval tvp);
+		public static extern int utimes (
+				[MarshalAs (UnmanagedType.CustomMarshaler, MarshalTypeRef=typeof(FileNameMarshaler))]
+				string filename, ref Timeval tvp);
 
 		#endregion
 
@@ -2299,6 +2433,12 @@ namespace Mono.Unix {
 
 		// TODO: times(2)
 
+		//
+		// <sys/utsname.h>
+		//
+
+		// TODO: uname(2)
+
 		#region <sys/wait.h> Declarations
 		//
 		// <sys/wait.h>
@@ -2486,7 +2626,9 @@ namespace Mono.Unix {
 		//       other TODOs listed below.
 
 		[DllImport (LIBC, SetLastError=true, EntryPoint="access")]
-		private static extern int sys_access (string pathname, int mode);
+		private static extern int sys_access (
+				[MarshalAs (UnmanagedType.CustomMarshaler, MarshalTypeRef=typeof(FileNameMarshaler))]
+				string pathname, int mode);
 
 		public static int access (string pathname, AccessMode mode)
 		{
@@ -2585,7 +2727,9 @@ namespace Mono.Unix {
 		// chown(2)
 		//    int chown(const char *path, uid_t owner, gid_t group);
 		[DllImport (LIBC, SetLastError=true)]
-		public static extern int chown (string path, uint owner, uint group);
+		public static extern int chown (
+				[MarshalAs (UnmanagedType.CustomMarshaler, MarshalTypeRef=typeof(FileNameMarshaler))]
+				string path, uint owner, uint group);
 
 		// fchown(2)
 		//    int fchown(int fd, uid_t owner, gid_t group);
@@ -2595,10 +2739,14 @@ namespace Mono.Unix {
 		// lchown(2)
 		//    int lchown(const char *path, uid_t owner, gid_t group);
 		[DllImport (LIBC, SetLastError=true)]
-		public static extern int lchown (string path, uint owner, uint group);
+		public static extern int lchown (
+				[MarshalAs (UnmanagedType.CustomMarshaler, MarshalTypeRef=typeof(FileNameMarshaler))]
+				string path, uint owner, uint group);
 
 		[DllImport (LIBC, SetLastError=true)]
-		public static extern int chdir (string path);
+		public static extern int chdir (
+				[MarshalAs (UnmanagedType.CustomMarshaler, MarshalTypeRef=typeof(FileNameMarshaler))]
+				string path);
 
 		[DllImport (LIBC, SetLastError=true)]
 		public static extern int fchdir (int fd);
@@ -2625,17 +2773,23 @@ namespace Mono.Unix {
 
 		// TODO: does Mono marshal arrays properly?
 		[DllImport (LIBC, SetLastError=true)]
-		public static extern int execve (string path, string[] argv, string[] envp);
+		public static extern int execve (
+				[MarshalAs (UnmanagedType.CustomMarshaler, MarshalTypeRef=typeof(FileNameMarshaler))]
+				string path, string[] argv, string[] envp);
 
 		[DllImport (LIBC, SetLastError=true)]
 		public static extern int fexecve (int fd, string[] argv, string[] envp);
 
 		[DllImport (LIBC, SetLastError=true)]
-		public static extern int execv (string path, string[] argv);
+		public static extern int execv (
+				[MarshalAs (UnmanagedType.CustomMarshaler, MarshalTypeRef=typeof(FileNameMarshaler))]
+				string path, string[] argv);
 
 		// TODO: execle, execl, execlp
 		[DllImport (LIBC, SetLastError=true)]
-		public static extern int execvp (string path, string[] argv);
+		public static extern int execvp (
+				[MarshalAs (UnmanagedType.CustomMarshaler, MarshalTypeRef=typeof(FileNameMarshaler))]
+				string path, string[] argv);
 
 		[DllImport (LIBC, SetLastError=true)]
 		public static extern int nice (int inc);
@@ -2643,17 +2797,22 @@ namespace Mono.Unix {
 		[DllImport (LIBC, SetLastError=true)]
 		public static extern int _exit (int status);
 
-		[DllImport (MPH, SetLastError=true,
-				EntryPoint="Mono_Posix_Syscall_fpathconf")]
-		public static extern long fpathconf (int filedes, PathConf name);
+		public static long fpathconf (int filedes, PathConf name)
+		{
+			return Native.Syscall.fpathconf (filedes, (Native.PathconfName) (int) name);
+		}
 
-		[DllImport (MPH, SetLastError=true,
-				EntryPoint="Mono_Posix_Syscall_pathconf")]
-		public static extern long pathconf (string path, PathConf name);
+		public static long pathconf (
+				[MarshalAs (UnmanagedType.CustomMarshaler, MarshalTypeRef=typeof(FileNameMarshaler))]
+				string path, PathConf name)
+		{
+			return Native.Syscall.pathconf (path, (Native.PathconfName) (int) name);
+		}
 
-		[DllImport (MPH, SetLastError=true,
-				EntryPoint="Mono_Posix_Syscall_sysconf")]
-		public static extern long sysconf (SysConf name);
+		public static long sysconf (SysConf name)
+		{
+			return Native.Syscall.sysconf ((Native.SysconfName) (int) name);
+		}
 
 		// confstr(3)
 		//    size_t confstr(int name, char *buf, size_t len);
@@ -2793,11 +2952,11 @@ namespace Mono.Unix {
 				"shutdown code.\nUse System.Diagnostics.Process instead")]
 		private static extern int vfork ();
 
+		private static object tty_lock = new object ();
+
 		[DllImport (LIBC, SetLastError=true, EntryPoint="ttyname")]
 		private static extern IntPtr sys_ttyname (int fd);
 
-		private static object tty_lock = new object ();
-
 		public static string ttyname (int fd)
 		{
 			lock (tty_lock) {
@@ -2826,16 +2985,26 @@ namespace Mono.Unix {
 		}
 
 		[DllImport (LIBC, SetLastError=true)]
-		public static extern int link (string oldpath, string newpath);
+		public static extern int link (
+				[MarshalAs (UnmanagedType.CustomMarshaler, MarshalTypeRef=typeof(FileNameMarshaler))]
+				string oldpath, 
+				[MarshalAs (UnmanagedType.CustomMarshaler, MarshalTypeRef=typeof(FileNameMarshaler))]
+				string newpath);
 
 		[DllImport (LIBC, SetLastError=true)]
-		public static extern int symlink (string oldpath, string newpath);
+		public static extern int symlink (
+				[MarshalAs (UnmanagedType.CustomMarshaler, MarshalTypeRef=typeof(FileNameMarshaler))]
+				string oldpath, 
+				[MarshalAs (UnmanagedType.CustomMarshaler, MarshalTypeRef=typeof(FileNameMarshaler))]
+				string newpath);
 
 		// readlink(2)
 		//    int readlink(const char *path, char *buf, size_t bufsize);
 		[DllImport (MPH, SetLastError=true,
 				EntryPoint="Mono_Posix_Syscall_readlink")]
-		public static extern int readlink (string path, [Out] StringBuilder buf, ulong bufsiz);
+		public static extern int readlink (
+				[MarshalAs (UnmanagedType.CustomMarshaler, MarshalTypeRef=typeof(FileNameMarshaler))]
+				string path, [Out] StringBuilder buf, ulong bufsiz);
 
 		public static int readlink (string path, [Out] StringBuilder buf)
 		{
@@ -2843,10 +3012,14 @@ namespace Mono.Unix {
 		}
 
 		[DllImport (LIBC, SetLastError=true)]
-		public static extern int unlink (string pathname);
+		public static extern int unlink (
+				[MarshalAs (UnmanagedType.CustomMarshaler, MarshalTypeRef=typeof(FileNameMarshaler))]
+				string pathname);
 
 		[DllImport (LIBC, SetLastError=true)]
-		public static extern int rmdir (string pathname);
+		public static extern int rmdir (
+				[MarshalAs (UnmanagedType.CustomMarshaler, MarshalTypeRef=typeof(FileNameMarshaler))]
+				string pathname);
 
 		// tcgetpgrp(3)
 		//    pid_t tcgetpgrp(int fd);
@@ -2861,11 +3034,12 @@ namespace Mono.Unix {
 		[DllImport (LIBC, SetLastError=true, EntryPoint="getlogin")]
 		private static extern IntPtr sys_getlogin ();
 
-		[Obsolete ("Not re-entrant.  Use getlogin_r instead.")]
 		public static string getlogin ()
 		{
-			IntPtr r = sys_getlogin ();
-			return UnixMarshal.PtrToString (r);
+			lock (getlogin_lock) {
+				IntPtr r = sys_getlogin ();
+				return UnixMarshal.PtrToString (r);
+			}
 		}
 
 		// getlogin_r(3)
@@ -2939,12 +3113,16 @@ namespace Mono.Unix {
 
 		// Revoke doesn't appear to be POSIX.  Include it?
 		[DllImport (LIBC, SetLastError=true)]
-		public static extern int revoke (string file);
+		public static extern int revoke (
+				[MarshalAs (UnmanagedType.CustomMarshaler, MarshalTypeRef=typeof(FileNameMarshaler))]
+				string file);
 
 		// TODO: profil?  It's not POSIX.
 
 		[DllImport (LIBC, SetLastError=true)]
-		public static extern int acct (string filename);
+		public static extern int acct (
+				[MarshalAs (UnmanagedType.CustomMarshaler, MarshalTypeRef=typeof(FileNameMarshaler))]
+				string filename);
 
 		[DllImport (LIBC, SetLastError=true, EntryPoint="getusershell")]
 		private static extern IntPtr sys_getusershell ();
@@ -2988,7 +3166,9 @@ namespace Mono.Unix {
 		}
 
 		[DllImport (LIBC, SetLastError=true)]
-		public static extern int chroot (string path);
+		public static extern int chroot (
+				[MarshalAs (UnmanagedType.CustomMarshaler, MarshalTypeRef=typeof(FileNameMarshaler))]
+				string path);
 
 		// skipping getpass(3) as the man page states:
 		//   This function is obsolete.  Do not use it.
@@ -3011,7 +3191,9 @@ namespace Mono.Unix {
 		//    int truncate(const char *path, off_t length);
 		[DllImport (MPH, SetLastError=true, 
 				EntryPoint="Mono_Posix_Syscall_truncate")]
-		public static extern int truncate (string path, long length);
+		public static extern int truncate (
+				[MarshalAs (UnmanagedType.CustomMarshaler, MarshalTypeRef=typeof(FileNameMarshaler))]
+				string path, long length);
 
 		// ftruncate(2)
 		//    int ftruncate(int fd, off_t length);
@@ -3037,15 +3219,21 @@ namespace Mono.Unix {
 				EntryPoint="Mono_Posix_Syscall_lockf")]
 		public static extern int lockf (int fd, LockfCommand cmd, long len);
 
+		internal static object crypt_lock = new object ();
+
 		[DllImport (CRYPT, SetLastError=true, EntryPoint="crypt")]
 		private static extern IntPtr sys_crypt (string key, string salt);
 
 		public static string crypt (string key, string salt)
 		{
-			IntPtr r = sys_crypt (key, salt);
-			return UnixMarshal.PtrToString (r);
+			lock (crypt_lock) {
+				IntPtr r = sys_crypt (key, salt);
+				return UnixMarshal.PtrToString (r);
+			}
 		}
 
+		internal static object encrypt_lock = new object ();
+
 		[DllImport (CRYPT, SetLastError=true, EntryPoint="encrypt")]
 		private static extern void sys_encrypt ([In, Out] byte[] block, int edflag);
 
@@ -3053,7 +3241,9 @@ namespace Mono.Unix {
 		{
 			if (block.Length < 64)
 				throw new ArgumentOutOfRangeException ("block", "Must refer to at least 64 bytes");
-			sys_encrypt (block, decode ? 1 : 0);
+			lock (encrypt_lock) {
+				sys_encrypt (block, decode ? 1 : 0);
+			}
 		}
 
 		// swab(3)
@@ -3076,7 +3266,9 @@ namespace Mono.Unix {
 
 		[DllImport (MPH, SetLastError=true, 
 				EntryPoint="Mono_Posix_Syscall_utime")]
-		private static extern int sys_utime (string filename, ref Utimbuf buf, int use_buf);
+		private static extern int sys_utime (
+				[MarshalAs (UnmanagedType.CustomMarshaler, MarshalTypeRef=typeof(FileNameMarshaler))]
+				string filename, ref Utimbuf buf, int use_buf);
 
 		public static int utime (string filename, ref Utimbuf buf)
 		{
