@@ -54,7 +54,10 @@ namespace System.Data.Common {
 
 		#region Properties
 
-		public abstract DbProviderSupportedClasses SupportedClasses { get; }
+		[MonoTODO]
+		public virtual bool CanCreateDataSourceEnumerator { 
+			get {throw new NotImplementedException ();}
+		}
 		#endregion // Properties
 
 		#region Methods
@@ -112,8 +115,13 @@ namespace System.Data.Common {
 		{
 			throw new NotImplementedException ();
 		}
-
-
+#if NET_2_0 
+		[MonoTODO]
+		public virtual DbConnectionStringBuilder CreateConnectionStringBuilder ()
+		{
+			throw new NotImplementedException ();
+		}
+#endif
 		#endregion // Methods
 	}
 }
