@@ -90,7 +90,11 @@ namespace System.Data.SqlClient {
 		public new SqlDataAdapter DataAdapter {
 			get { return adapter; }
 			set { 
+				if (adapter != null)
+					adapter.RowUpdating -= new SqlRowUpdatingEventHandler (RowUpdatingHandler);
+
 				adapter = value; 
+
 				if (adapter != null)
 					adapter.RowUpdating += new SqlRowUpdatingEventHandler (RowUpdatingHandler);
 			}
@@ -117,7 +121,7 @@ namespace System.Data.SqlClient {
 		}
 
 		[Browsable (false)]
-		[DataSysDescription ("The character used in a text command as the closing quote for quoting identifiers that contain special characters.")]
+		[DataSysDescription ("The character used in a text command as the closing quote for quoting identifiers that contain special characters. ")]
 		[DesignerSerializationVisibility (DesignerSerializationVisibility.Hidden)]
 		public 
 #if NET_2_0
@@ -172,14 +176,17 @@ namespace System.Data.SqlClient {
 		{
 			tableName = String.Empty;
 			foreach (DataRow schemaRow in schemaTable.Rows) {
+				if (schemaRow.IsNull ("BaseTableName") ||
+				    schemaRow ["BaseTableName"] == String.Empty)
+					continue;
+
 				if (tableName == String.Empty) 
-					tableName = schemaRow.IsNull ("BaseTableName") ? null : (string) schemaRow ["BaseTableName"];
-				else if (schemaRow.IsNull ("BaseTableName")) {
-					if (tableName != null)
-						throw new InvalidOperationException ("Dynamic SQL generation is not supported against multiple base tables.");
-				} else if (tableName != (string) schemaRow["BaseTableName"])
+					tableName = (string) schemaRow ["BaseTableName"];
+				else if (tableName != (string) schemaRow["BaseTableName"])
 					throw new InvalidOperationException ("Dynamic SQL generation is not supported against multiple base tables.");
 			}
+			if (tableName == String.Empty)
+				throw new InvalidOperationException ("Dynamic SQL generation is not supported with no base table.");
 			dbSchemaTable = schemaTable;
 		}
 
@@ -211,10 +218,15 @@ namespace System.Data.SqlClient {
 
 				if (!isKey) {
 					parameter = deleteCommand.Parameters.Add (CreateParameter (parmIndex++, schemaRow));
+					parameter.SourceVersion = DataRowVersion.Original;
 
-					dsColumnName = tableMapping.ColumnMappings [parameter.SourceColumn].DataSetColumn;
+					dsColumnName = parameter.SourceColumn;
+					if (tableMapping != null 
+					    && tableMapping.ColumnMappings.Contains (parameter.SourceColumn))
+						dsColumnName = tableMapping.ColumnMappings [parameter.SourceColumn].DataSetColumn;
+				
 					if (row != null)
-						parameter.Value = row [dsColumnName, DataRowVersion.Current];
+						parameter.Value = row [dsColumnName, DataRowVersion.Original];
 					whereClause.Append ("(");
 					whereClause.Append (String.Format (clause1, GetQuotedString (parameter.SourceColumn), parameter.ParameterName));
 					whereClause.Append (" OR ");
@@ -223,10 +235,15 @@ namespace System.Data.SqlClient {
 					keyFound = true;
 					
 				parameter = deleteCommand.Parameters.Add (CreateParameter (parmIndex++, schemaRow));
+				parameter.SourceVersion = DataRowVersion.Original;
+
+				dsColumnName = parameter.SourceColumn;
+				if (tableMapping != null 
+				    && tableMapping.ColumnMappings.Contains (parameter.SourceColumn))
+					dsColumnName = tableMapping.ColumnMappings [parameter.SourceColumn].DataSetColumn;
 
-				dsColumnName = tableMapping.ColumnMappings [parameter.SourceColumn].DataSetColumn;
 				if (row != null)
-					parameter.Value = row [dsColumnName, DataRowVersion.Current];
+					parameter.Value = row [dsColumnName, DataRowVersion.Original];
 
 				whereClause.Append (String.Format (clause2, GetQuotedString (parameter.SourceColumn), parameter.ParameterName));
 
@@ -266,8 +283,13 @@ namespace System.Data.SqlClient {
 				}
 
 				SqlParameter parameter = insertCommand.Parameters.Add (CreateParameter (parmIndex++, schemaRow));
+				parameter.SourceVersion = DataRowVersion.Current;
+
+				dsColumnName = parameter.SourceColumn;
+				if (tableMapping != null 
+				    && tableMapping.ColumnMappings.Contains (parameter.SourceColumn))
+					dsColumnName = tableMapping.ColumnMappings [parameter.SourceColumn].DataSetColumn;
 
-				dsColumnName = tableMapping.ColumnMappings [parameter.SourceColumn].DataSetColumn;
 				if (row != null)
 					parameter.Value = row [dsColumnName];
 
@@ -313,10 +335,15 @@ namespace System.Data.SqlClient {
 					columns.Append (" , ");
 
 				SqlParameter parameter = updateCommand.Parameters.Add (CreateParameter (parmIndex++, schemaRow));
+				parameter.SourceVersion = DataRowVersion.Current;
+
+				dsColumnName = parameter.SourceColumn;
+				if (tableMapping != null 
+				    && tableMapping.ColumnMappings.Contains (parameter.SourceColumn))
+					dsColumnName = tableMapping.ColumnMappings [parameter.SourceColumn].DataSetColumn;
 
-				dsColumnName = tableMapping.ColumnMappings [parameter.SourceColumn].DataSetColumn;
 				if (row != null)
-					parameter.Value = row [dsColumnName, DataRowVersion.Proposed];
+					parameter.Value = row [dsColumnName];
 
 				columns.Append (String.Format ("{0} = {1}", GetQuotedString (parameter.SourceColumn), parameter.ParameterName));
 			}
@@ -336,10 +363,15 @@ namespace System.Data.SqlClient {
 
 				if (!isKey) {
 					parameter = updateCommand.Parameters.Add (CreateParameter (parmIndex++, schemaRow));
+					parameter.SourceVersion = DataRowVersion.Original;
+
+					dsColumnName = parameter.SourceColumn;
+					if (tableMapping != null 
+					    && tableMapping.ColumnMappings.Contains (parameter.SourceColumn))
+						dsColumnName = tableMapping.ColumnMappings [parameter.SourceColumn].DataSetColumn;
 
-					dsColumnName = tableMapping.ColumnMappings [parameter.SourceColumn].DataSetColumn;
 					if (row != null)
-						parameter.Value = row [dsColumnName];
+						parameter.Value = row [dsColumnName, DataRowVersion.Original];
 
 					whereClause.Append ("(");
 					whereClause.Append (String.Format (clause1, GetQuotedString (parameter.SourceColumn), parameter.ParameterName));
@@ -349,10 +381,15 @@ namespace System.Data.SqlClient {
 					keyFound = true;
 					
 				parameter = updateCommand.Parameters.Add (CreateParameter (parmIndex++, schemaRow));
+				parameter.SourceVersion = DataRowVersion.Original;
+
+				dsColumnName = parameter.SourceColumn;
+				if (tableMapping != null 
+				    && tableMapping.ColumnMappings.Contains (parameter.SourceColumn))
+					dsColumnName = tableMapping.ColumnMappings [parameter.SourceColumn].DataSetColumn;
 
-				dsColumnName = tableMapping.ColumnMappings [parameter.SourceColumn].DataSetColumn;
 				if (row != null)
-					parameter.Value = row [dsColumnName];
+					parameter.Value = row [dsColumnName, DataRowVersion.Original];
 
 				whereClause.Append (String.Format (clause2, GetQuotedString (parameter.SourceColumn), parameter.ParameterName));
 
@@ -491,7 +528,10 @@ namespace System.Data.SqlClient {
 
 #if NET_2_0
                 [MonoTODO]
-                protected override void ApplyParameterInfo (IDbDataParameter dbParameter, DataRow row)
+                protected override void ApplyParameterInfo (DbParameter dbParameter,
+							    DataRow row,
+							    StatementType statementType,
+							    bool whereClause)
                 {
                         throw new NotImplementedException ();
                 }
@@ -501,6 +541,12 @@ namespace System.Data.SqlClient {
                 {
                         throw new NotImplementedException ();                        
                 }
+
+                [MonoTODO]
+                protected override string GetParameterName (string parameterName)
+                {
+                        throw new NotImplementedException ();                        
+                }
                 
 
                 [MonoTODO]
@@ -509,62 +555,31 @@ namespace System.Data.SqlClient {
                         throw new NotImplementedException ();                        
                 }
                 
-                [MonoTODO]
-                protected override DbProviderFactory ProviderFactory
-                {
-                        get {throw new NotImplementedException ();}
-                }
 #endif // NET_2_0
 		#endregion // Methods
 
 		#region Event Handlers
 
-		private void RowUpdatingHandler (object sender, SqlRowUpdatingEventArgs e)
+		private void RowUpdatingHandler (object sender, SqlRowUpdatingEventArgs args)
 		{
-			if (e.Status != UpdateStatus.Continue)
+			if (args.Command != null)
 				return;
-
-			switch (e.StatementType) {
-			case StatementType.Delete:
-				deleteCommand = e.Command;
-				break;
-			case StatementType.Insert:
-				insertCommand = e.Command;
-				break;
-			case StatementType.Update:
-				updateCommand = e.Command;
-				break;
-			default:
-				return;
-			}
-
 			try {
-				BuildCache (false);
-
-				switch (e.StatementType) {
-				case StatementType.Delete:
-					e.Command = CreateDeleteCommand (e.Row, e.TableMapping);
-					e.Status = UpdateStatus.Continue;
-					break;
+				switch (args.StatementType) {
 				case StatementType.Insert:
-					e.Command = CreateInsertCommand (e.Row, e.TableMapping);
-					e.Status = UpdateStatus.Continue;
+					args.Command = GetInsertCommand ();
 					break;
 				case StatementType.Update:
-					e.Command = CreateUpdateCommand (e.Row, e.TableMapping);
-					e.Status = UpdateStatus.Continue;
+					args.Command = GetUpdateCommand ();
+					break;
+				case StatementType.Delete:
+					args.Command = GetDeleteCommand ();
 					break;
 				}
-
-				if (e.Command != null && e.Row != null) {
-					e.Row.AcceptChanges ();
-					e.Status = UpdateStatus.SkipCurrentRow;
-				}
-			}
-			catch (Exception exception) {
-				e.Errors = exception;
-				e.Status = UpdateStatus.ErrorsOccurred;
-			}
+			} catch (Exception e) {
+				args.Errors = e;
+				args.Status = UpdateStatus.ErrorsOccurred;
+			}		
 		}
 
 #if NET_2_0
