@@ -39,7 +39,8 @@ using System.Runtime.InteropServices;
 using System.Data.Common;
 
 namespace System.Data {
-	[Editor]
+	[Editor ("Microsoft.VSDesigner.Data.Design.ForeignKeyConstraintEditor, " + Consts.AssemblyMicrosoft_VSDesigner,
+		 "System.Drawing.Design.UITypeEditor, " + Consts.AssemblySystem_Drawing)]
 	[DefaultProperty ("ConstraintName")]
 	[Serializable]
 	public class ForeignKeyConstraint : Constraint 
@@ -460,8 +461,11 @@ namespace System.Data {
 			
 			//Make sure we can create this thing
 			//AssertConstraint(); 
-			if (IsConstraintViolated())
-				throw new ArgumentException("This constraint cannot be enabled as not all values have corresponding parent values.");
+			if ( (Table.DataSet != null && Table.DataSet.EnforceConstraints)
+			     || (Table.DataSet == null && Table.EnforceConstraints)) {
+				if (IsConstraintViolated())
+					throw new ArgumentException("This constraint cannot be enabled as not all values have corresponding parent values.");
+			}
 			//FIXME : if this fails and we created a unique constraint
 			//we should probably roll it back
 			// and remove index form Table			
@@ -473,14 +477,11 @@ namespace System.Data {
 			Index = null;
 		}
 		
-		protected override bool IsConstraintViolated()
+		internal override bool IsConstraintViolated()
 		{
 			if (Table.DataSet == null || RelatedTable.DataSet == null) 
 				return false;
 			
-			if (!Table.DataSet.EnforceConstraints)
-				return false;
-				
 			bool hasErrors = false;
 			foreach (DataRow row in Table.Rows) {
 				// first we check if all values in _childColumns place are nulls.
