@@ -34,6 +34,7 @@
 using NUnit.Framework;
 using System;
 using System.Data;
+using System.Data.SqlTypes;
 using System.Globalization;
 using System.IO;
 using System.Runtime.Serialization.Formatters.Binary;
@@ -117,6 +118,11 @@ namespace MonoTests.System.Data
                 	Row [0] = "Teresa";
                 	Row [1] = "Mack";
                 	Mom.Rows.Add (Row);
+
+			Row = Mom.NewRow ();
+                	Row [0] = "'Jhon O'' Collenal'";
+                	Row [1] = "Pack";
+                	Mom.Rows.Add (Row);
                 	
                 	Row = Child.NewRow ();
                 	Row [0] = "Nick";
@@ -147,9 +153,18 @@ namespace MonoTests.System.Data
                 	Row [0] = "Mack";
                 	Row [1] = 99;
                 	Child.Rows.Add (Row);
+
+			Row = Child.NewRow ();
+                	Row [0] = "Pack";
+                	Row [1] = 66;
+                	Child.Rows.Add (Row);
                 	
                 	DataRow [] Rows = Mom.Select ("Name = 'Teresa'");
                 	AssertEquals ("test#01", 2, Rows.Length);
+
+			// test with apos escaped
+			Rows = Mom.Select ("Name = '''Jhon O'''' Collenal'''");
+                	AssertEquals ("test#01.1", 1, Rows.Length);
                 	
                 	Rows = Mom.Select ("Name = 'Teresa' and ChildName = 'Nick'");
                 	AssertEquals ("test#02", 0, Rows.Length);
@@ -161,7 +176,7 @@ namespace MonoTests.System.Data
                 	AssertEquals ("test#04", "Mack", Rows [0] [1]);
                 	
                 	Rows = Mom.Select ("Name = 'Teresa' or ChildName <> 'Jack'");
-                	AssertEquals ("test#05", 5, Rows.Length);
+                	AssertEquals ("test#05", 6, Rows.Length);
 			
                 	Rows = Child.Select ("age = 20 - 1");
                 	AssertEquals ("test#06", 1, Rows.Length);
@@ -170,7 +185,7 @@ namespace MonoTests.System.Data
                 	AssertEquals ("test#07", 3, Rows.Length);
 			
                 	Rows = Child.Select ("age >= 20");
-                	AssertEquals ("test#08", 3, Rows.Length);
+                	AssertEquals ("test#08", 4, Rows.Length);
 			
                 	Rows = Child.Select ("age >= 20 and name = 'Mack' or name = 'Nick'");
                 	AssertEquals ("test#09", 2, Rows.Length);
@@ -180,7 +195,7 @@ namespace MonoTests.System.Data
                 	AssertEquals ("test#11", "Mack", Rows [0] [0]);
                 	
                 	Rows = Child.Select ("not (Name = 'Jack')");
-                	AssertEquals ("test#12", 5, Rows.Length);
+                	AssertEquals ("test#12", 6, Rows.Length);
                 }
                 
 		[Test]
@@ -1417,6 +1432,14 @@ namespace MonoTests.System.Data
                         }
                 }
 
+				[Test]
+				public void ColumnObjectTypeTest() {
+					DataTable dt = new DataTable();
+					dt.Columns.Add("Series Label", typeof(SqlInt32));
+					dt.Rows.Add(new object[] {"sss"});
+					AssertEquals(1, dt.Rows.Count);
+				}
+
                 public void OnRowChanging (object src, DataRowChangeEventArgs args)
                 {
                         rowActionChanging = args.Action;
@@ -1442,5 +1465,46 @@ namespace MonoTests.System.Data
                                                                                                     
          }
 
-	
+	[Serializable]
+	[TestFixture]
+	public class AppDomainsAndFormatInfo
+	{
+		public void Remote ()
+		{
+			int n = (int) Convert.ChangeType ("5", typeof (int));
+			Assertion.AssertEquals ("n", 5, n);
+		}
+#if !TARGET_JVM
+		[Test]
+		public void NFIFromBug55978 ()
+		{
+			AppDomain domain = AppDomain.CreateDomain ("testdomain");
+			AppDomainsAndFormatInfo test = new AppDomainsAndFormatInfo ();
+			test.Remote ();
+			domain.DoCallBack (new CrossAppDomainDelegate (test.Remote));
+			AppDomain.Unload (domain);
+		}
+#endif
+
+		[Test]
+		public void Bug55978 ()
+		{
+			DataTable dt = new DataTable ();
+			dt.Columns.Add ("StartDate", typeof (DateTime));
+	 
+			DataRow dr;
+			DateTime date = DateTime.Now;
+	 
+			for (int i = 0; i < 10; i++) {
+				dr = dt.NewRow ();
+				dr ["StartDate"] = date.AddDays (i);
+				dt.Rows.Add (dr);
+			}
+	 
+			DataView dv = dt.DefaultView;
+			dv.RowFilter = "StartDate >= '" + DateTime.Now.AddDays (2) + "' and StartDate <= '" + DateTime.Now.AddDays (4) + "'";
+			Assertion.AssertEquals ("Table", 10, dt.Rows.Count);
+			Assertion.AssertEquals ("View", 2, dv.Count);
+		}
+	}
 }
