@@ -13,7 +13,7 @@
 //
 
 //
-// Copyright (C) 2004 Novell, Inc (http://www.novell.com)
+// Copyright (C) 2004-2005 Novell, Inc (http://www.novell.com)
 //
 // Permission is hereby granted, free of charge, to any person obtaining
 // a copy of this software and associated documentation files (the
@@ -35,12 +35,12 @@
 // WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 //
 
-using System;
 using System.IO;
 using System.Drawing.Imaging;
 using System.Runtime.Serialization;
 using System.Runtime.InteropServices;
 using System.ComponentModel;
+using System.Security.Permissions;
 
 namespace System.Drawing
 {
@@ -97,14 +97,6 @@ namespace System.Drawing
 			
 		}
 
-		void InitFromFile (string filename)
-		{		
-			IntPtr imagePtr;
-			Status st = GDIPlus.GdipLoadImageFromFile (filename, out imagePtr);
-			GDIPlus.CheckStatus (st);
-			nativeObject = imagePtr;			
-		}
-
 		public Bitmap (Stream stream, bool useIcm)
 		{
 			if (stream == null)
@@ -115,7 +107,16 @@ namespace System.Drawing
 
 		public Bitmap (string filename, bool useIcm)
 		{
-			InitFromFile (filename);
+			IntPtr imagePtr;
+			Status st;
+
+			if (useIcm)
+				st = GDIPlus.GdipCreateBitmapFromFileICM (filename, out imagePtr);
+			else
+				st = GDIPlus.GdipCreateBitmapFromFile (filename, out imagePtr);
+
+			GDIPlus.CheckStatus (st);
+			nativeObject = imagePtr;
 		}
 
 		public Bitmap (Type type, string resource)
@@ -226,12 +227,14 @@ namespace System.Drawing
 		}
 
 		[EditorBrowsable (EditorBrowsableState.Advanced)]
+		[SecurityPermission (SecurityAction.LinkDemand, UnmanagedCode = true)]
 		public IntPtr GetHbitmap ()
 		{
 			return GetHbitmap(Color.Gray);
 		}
 
 		[EditorBrowsable (EditorBrowsableState.Advanced)]
+		[SecurityPermission (SecurityAction.LinkDemand, UnmanagedCode = true)]
 		public IntPtr GetHbitmap (Color background)
 		{
 			IntPtr HandleBmp;
@@ -243,6 +246,7 @@ namespace System.Drawing
 		}
 
 		[EditorBrowsable (EditorBrowsableState.Advanced)]
+		[SecurityPermission (SecurityAction.LinkDemand, UnmanagedCode = true)]
 		public IntPtr GetHicon ()
 		{
 			IntPtr HandleIcon;
@@ -260,19 +264,24 @@ namespace System.Drawing
 			if (nativeObject == (IntPtr) 0)
 				throw new Exception ("nativeObject is null");			
 			
-			IntPtr lfBuffer = Marshal.AllocHGlobal(Marshal.SizeOf(result));
-     			Marshal.StructureToPtr(result, lfBuffer, false);						
-     		
-			Status status = GDIPlus.GdipBitmapLockBits (nativeObject, ref rect, flags, format,  lfBuffer);
+			Status status = GDIPlus.GdipBitmapLockBits (nativeObject, ref rect, flags, format,  result);
 			
-			result = (BitmapData) Marshal.PtrToStructure(lfBuffer,  typeof(BitmapData));											
-			Marshal.FreeHGlobal (lfBuffer);			
 			//NOTE: scan0 points to piece of memory allocated in the unmanaged space
 			GDIPlus.CheckStatus (status);
 
 			return  result;
 		}
 
+#if NET_2_0
+		public BitmapData LockBits (Rectangle rect, ImageLockMode flags, PixelFormat format, BitmapData bitmapData)
+		{
+			Status status = GDIPlus.GdipBitmapLockBits (nativeObject, ref rect, flags, format,  bitmapData);
+			GDIPlus.CheckStatus (status);
+
+			return bitmapData;
+		}
+#endif
+
 		public void MakeTransparent ()
 		{
 			Color clr = GetPixel(0,0);			
