@@ -46,7 +46,9 @@ namespace System.Drawing
 {
 	[TypeConverter(typeof(ColorConverter))]
 	[ComVisible (true)]
+#if !TARGET_JVM
 	[Editor ("System.Drawing.Design.ColorEditor, " + Consts.AssemblySystem_Drawing_Design, typeof (System.Drawing.Design.UITypeEditor))]
+#endif
 	[Serializable]
 	public struct Color
 	{
@@ -64,13 +66,27 @@ namespace System.Drawing
 		// The specs also indicate that all three of these propities are true
 		// if created with FromKnownColor or FromNamedColor, false otherwise (FromARGB).
 		// Per Microsoft and ECMA specs these varibles are set by which constructor is used, not by their values.
-		private bool isknowncolor;
-		private bool isnamedcolor;
-		private bool issystemcolor;
+		[Flags]
+		enum ColorType {
+			Empty=0,
+			ARGB=1,
+			Known=2,
+			Named=4,
+			System=8
+		}
+		private ColorType colorType;
 		private KnownColor knownColor;
 
 		private string myname;
 
+#if TARGET_JVM
+		internal java.awt.Color NativeObject {
+			get {
+				return new java.awt.Color (R, G, B, A);
+			}
+		}
+#endif
+
 		public string Name {
 			get{
 				if (myname == null || myname == String.Empty)
@@ -81,21 +97,19 @@ namespace System.Drawing
 
 		public bool IsKnownColor {
 			get{
-				return isknowncolor;
+				return (colorType & ColorType.Known) != 0;
 			}
 		}
 
 		public bool IsSystemColor {
 			get{
-				return issystemcolor;
+				return (colorType & ColorType.System) != 0;
 			}
 		}
 
 		public bool IsNamedColor {
 			get{
-				if (!isnamedcolor)
-					return IsKnownColor;
-				return isnamedcolor;
+				return (colorType & (ColorType.Known|ColorType.Named)) != 0;
 			}
 		}
 
@@ -109,6 +123,7 @@ namespace System.Drawing
 		{
 			CheckARGBValues (alpha, red, green, blue);
 			Color color = new Color ();
+			color.colorType = ColorType.ARGB;
 			color.a = (byte) alpha;
 			color.r = (byte) red;
 			color.g = (byte) green;
@@ -120,8 +135,7 @@ namespace System.Drawing
 		private static Color FromArgbNamed (int alpha, int red, int green, int blue, string name, KnownColor knownColor)
 		{
 			Color color = FromArgb (alpha, red, green, blue);
-			color.isknowncolor = true;
-			color.isnamedcolor = true;
+			color.colorType = ColorType.Known|ColorType.Named;
 			//color.issystemcolor = false; //???
 			color.myname = name;
 			// FIXME: here happens SEGFAULT.
@@ -133,7 +147,7 @@ namespace System.Drawing
 		internal static Color FromArgbSystem (int alpha, int red, int green, int blue, string name, KnownColor knownColor)
 		{
 			Color color = FromArgbNamed (alpha, red, green, blue, name, knownColor);
-			color.issystemcolor = true;
+			color.colorType |= ColorType.System;
 			return color;
 		}
 
@@ -157,11 +171,15 @@ namespace System.Drawing
 			if (knownColors == null)
 				FillColorNames ();
 			
+#if NET_2_0
+			if (c < KnownColor.ActiveBorder || c > KnownColor.MenuHighlight) {
+#else
 			if (c < KnownColor.ActiveBorder || c > KnownColor.YellowGreen) {
+#endif			
 				// This is what it returns!
 				Color d = FromArgb (0, 0, 0, 0);
 				d.myname = c.ToString ();
-				d.isnamedcolor = true;
+				d.colorType |= ColorType.Named;
 				d.knownColor = c;
 				return d;
 			}
@@ -201,8 +219,11 @@ namespace System.Drawing
 				if (systemColors != null)
 					return;
 				
-				knownColors = new Color [(int)KnownColor.YellowGreen + 1];
-				
+#if NET_2_0
+				knownColors = new Color [(int)KnownColor.MenuHighlight + 1];			
+#else
+				knownColors = new Color [(int)KnownColor.YellowGreen + 1];			
+#endif				
 				Hashtable colorHash = GetColorHashtableFromType (typeof (Color));
 				namedColors = colorHash;
 
@@ -210,6 +231,11 @@ namespace System.Drawing
 				systemColors = colorHash;
 			}
 		}
+
+		internal static void UpdateKnownColor (int alpha, int red, int green, int blue, string name, KnownColor known_color) {
+			FillColorNames ();
+			knownColors[(int)known_color] = Color.FromArgbSystem(alpha, red, green, blue, name, known_color);
+		}
 		
 		public static Color FromName (string colorName)
 		{
@@ -220,7 +246,7 @@ namespace System.Drawing
 					// This is what it returns!
 					Color d = FromArgb (0, 0, 0, 0);
 					d.myname = colorName;
-					d.isnamedcolor = true;
+					d.colorType |= ColorType.Named;
 					c = d;
 				}
 			}
@@ -270,9 +296,23 @@ namespace System.Drawing
 
 		public static bool operator == (Color colorA, Color colorB)
 		{
-			return ((colorA.a == colorB.a) && (colorA.r == colorB.r)
-			&& (colorA.g == colorB.g) && (colorA.b == colorB.b) &&
-			(colorA.myname == colorB.myname));
+			if (colorA.a != colorB.a)
+				return false;
+			if (colorA.r != colorB.r)
+				return false;
+            if (colorA.g != colorB.g)
+				return false;
+			if (colorA.b != colorB.b)
+				return false;
+			if (colorA.IsNamedColor != colorB.IsNamedColor)
+				return false;
+			if (colorA.IsSystemColor != colorB.IsSystemColor)
+				return false;
+			if (colorA.Name != colorB.Name)
+				return false;
+			if (colorA.IsEmpty != colorB.IsEmpty)
+				return false;
+			return true;
 		}
 		
 		/// <summary>
@@ -287,11 +327,9 @@ namespace System.Drawing
 
 		public static bool operator != (Color colorA, Color colorB)
 		{
-			return ((colorA.a != colorB.a) || (colorA.r != colorB.r)
-			|| (colorA.g != colorB.g) || (colorA.b != colorB.b) ||
-			(colorA.myname != colorB.myname));
+			return ! (colorA == colorB);
 		}
-
+		
 		public float GetBrightness ()
 		{
 			byte minval = Math.Min (r, Math.Min (g, b));
@@ -368,7 +406,7 @@ namespace System.Drawing
 		public bool IsEmpty 
 		{
 			get {
-				return (a + r + g + b) == 0;
+				return colorType == ColorType.Empty;
 			}
 		}
 
@@ -442,16 +480,10 @@ namespace System.Drawing
 		
 		public override bool Equals (object o)
 		{
-			if (!(o is Color))
+			if (! (o is Color))
 				return false;
-
 			Color c = (Color) o;
-			if (c.r == r && c.g == g && c.b == b) {
-				if (myname != null || c.myname != null)
-					return (myname == c.myname);
-				return true;
-			}
-			return false;
+			return this == c;
 		}
 
 		/// <summary>
@@ -481,7 +513,7 @@ namespace System.Drawing
 		
 		public override int GetHashCode ()
 		{
-			return ToArgb().GetHashCode();
+			return ToArgb().GetHashCode () ^ Name.GetHashCode ();
 		}
 
 		/// <summary>
@@ -494,6 +526,9 @@ namespace System.Drawing
 		
 		public override string ToString ()
 		{
+			if (IsEmpty)
+				return "Color [Empty]";
+
 			if (myname != "")
 				return "Color [" + myname + "]";
 
@@ -503,21 +538,24 @@ namespace System.Drawing
 		private static void CheckRGBValues (int red,int green,int blue)
 		{
 			if( (red > 255) || (red < 0))
-				throw new System.ArgumentOutOfRangeException
-					("red",red,"Value must be in the range 0 - 255");
+				throw CreateColorArgumentException(red, "red");
 			if( (green > 255) || (green < 0))
-				throw new System.ArgumentOutOfRangeException
-					("green",green,"Value must be in the range 0 - 255");
+				throw CreateColorArgumentException (green, "green");
 			if( (blue > 255) || (blue < 0))
-				throw new System.ArgumentOutOfRangeException
-					("blue",blue,"Value must be in the range 0 - 255");
+				throw CreateColorArgumentException (blue, "blue");
+		}
+
+		private static ArgumentException CreateColorArgumentException (int value, string color)
+		{
+			return new ArgumentException (string.Format ("'{0}' is not a valid"
+				+ " value for '{1}'. '{1}' should be greater or equal to 0 and"
+				+ " less than or equal to 255.", value, color));
 		}
 
 		private static void CheckARGBValues (int alpha,int red,int green,int blue)
 		{
 			if( (alpha > 255) || (alpha < 0))
-				throw new System.ArgumentOutOfRangeException
-					("alpha",alpha,"Value must be in the range 0 - 255");
+				throw CreateColorArgumentException (alpha, "alpha");
 			CheckRGBValues(red,green,blue);
 		}
 
