@@ -46,13 +46,47 @@ namespace System.Drawing
 	public sealed class Font : MarshalByRefObject, ISerializable, ICloneable, IDisposable
 	{
 		private IntPtr	fontObject = IntPtr.Zero;
+		private string  systemFontName;
+
+		private void CreateFont(string familyName, float emSize, FontStyle style, GraphicsUnit unit, byte charSet, bool isVertical) {
+                        Status		status;                  
+                        FontFamily	family;                      
+
+			// NOTE: If family name is null, empty or invalid,
+			// MS creates Microsoft Sans Serif font.
+			try {
+				family = new FontFamily (familyName);
+			}
+			catch (Exception){
+				family = FontFamily.GenericSansSerif;
+			}
+
+			setProperties (family, emSize, style, unit, charSet, isVertical);           
+			status = GDIPlus.GdipCreateFont (family.NativeObject, emSize,  style, unit, out fontObject);
+			GDIPlus.CheckStatus (status);
+		}
 
        		private Font (SerializationInfo info, StreamingContext context)
 		{
+			string		name;
+			float		size;
+			FontStyle	style;
+			GraphicsUnit	unit;
+
+			name = (string)info.GetValue("Name", typeof(string));
+			size = (float)info.GetValue("Size", typeof(float));
+			style = (FontStyle)info.GetValue("Style", typeof(FontStyle));
+			unit = (GraphicsUnit)info.GetValue("Unit", typeof(GraphicsUnit));
+ 
+			CreateFont(name, size, style, unit, (byte)0, false);
 		}
 
 		void ISerializable.GetObjectData(SerializationInfo info, StreamingContext context)
 		{
+			info.AddValue("Name", Name);
+			info.AddValue("Size", Size);
+			info.AddValue("Style", Style);
+			info.AddValue("Unit", Unit);
 		}
 
 		~Font()
@@ -168,12 +202,10 @@ namespace System.Drawing
 			}
 
 			if ((int) osInfo.Platform == 128 || (int) osInfo.Platform == 4) {
-			// If we're on Unix we use our private gdiplus API to avoid Wine 
-			// dependencies in S.D
-
+				// If we're on Unix we use our private gdiplus API to avoid Wine 
+				// dependencies in S.D
 				Status s = GDIPlus.GdipCreateFontFromHfont (Hfont, out newObject, ref lf);
 				GDIPlus.CheckStatus (s);
-				
 			} else {
 
 				// This needs testing
@@ -187,7 +219,7 @@ namespace System.Drawing
 				GDIPlus.ReleaseDC (hdc);
 				return f;				
 			}
-
+			
 			if (lf.lfItalic != 0) {
 				newStyle |= FontStyle.Italic;
 			}
@@ -320,21 +352,7 @@ namespace System.Drawing
 		public Font (string familyName, float emSize, FontStyle style,
 				GraphicsUnit unit, byte charSet, bool isVertical)
 		{
-			// NOTE: If family name is null, empty or invalid,
-			// MS creates Microsoft Sans Serif font.
-			Status status;			
-			FontFamily family;			
-			try {
-				family = new FontFamily (familyName);
-			}
-			catch (Exception){
-				family = FontFamily.GenericSansSerif;
-			}
-			
-			setProperties (family, emSize, style, unit, charSet, isVertical);				
-
-			status = GDIPlus.GdipCreateFont (family.NativeObject, emSize,  style, unit, out fontObject);
-			GDIPlus.CheckStatus (status);			
+			CreateFont(familyName, emSize, style, unit, charSet, isVertical);
 		}
 
 		public object Clone ()
@@ -351,6 +369,14 @@ namespace System.Drawing
 			}
 		}
 
+#if NET_2_0
+		internal string SysFontName {
+			set {
+				systemFontName = value;
+			}
+		}
+#endif
+
 		private bool _bold;
 
 		[DesignerSerializationVisibility (DesignerSerializationVisibility.Hidden)]
@@ -394,6 +420,18 @@ namespace System.Drawing
 			}
 		}
 
+#if NET_2_0
+		[Browsable(false)]
+		public bool IsSystemFont {
+			get {
+				if (systemFontName == null)
+					return false;
+
+				return StringComparer.InvariantCulture.Compare (systemFontName, string.Empty) != 0;
+			}
+		}
+#endif
+
 		private bool _italic;
 
 		[DesignerSerializationVisibility (DesignerSerializationVisibility.Hidden)]
@@ -448,6 +486,14 @@ namespace System.Drawing
 			}
 		}
 
+#if NET_2_0
+		[Browsable(false)]
+		public string SystemFontName {
+			get {
+				return systemFontName;
+			}
+		}
+#endif
 		private bool _underline;
 
 		[DesignerSerializationVisibility (DesignerSerializationVisibility.Hidden)]
@@ -507,28 +553,96 @@ namespace System.Drawing
 			return GetHeight (Graphics.systemDpiY);
 		}
 
-		[MonoTODO]
 		public static Font FromLogFont (object lf)
 		{
-			throw new NotImplementedException ();
+			if ((int) Environment.OSVersion.Platform == 128 || (int) Environment.OSVersion.Platform == 4) {
+				return FromLogFont(lf, IntPtr.Zero);
+			} else {
+				IntPtr	hDC;
+
+				hDC = IntPtr.Zero;
+
+				try {
+					hDC = GDIPlus.GetDC(IntPtr.Zero);
+					return FromLogFont (lf, hDC);
+				}
+
+				finally {
+					GDIPlus.ReleaseDC(hDC);
+				}
+			}
+
 		}
 
 		public void ToLogFont (object logFont)
 		{
-			using (Graphics g = Graphics.FromHdc (GDIPlus.GetDC (IntPtr.Zero))) {
-				ToLogFont (logFont, g);
+			Graphics g;
+
+			g = null;
+
+			if ((int) Environment.OSVersion.Platform == 128 || (int) Environment.OSVersion.Platform == 4) {
+				// Unix
+				Bitmap	img;
+
+				img = null;
+
+				try {
+					// We don't have a window we could associate the DC with
+					// so we use an image instead
+					img = new Bitmap(1, 1, Imaging.PixelFormat.Format32bppArgb);
+					g = Graphics.FromImage(img);
+					ToLogFont(logFont, g);
+				}
+
+				finally {
+					if (g != null) {
+						g.Dispose();
+					}
+
+					if (img != null) {
+						img.Dispose();
+					}
+				}
+			} else {
+				// Windows
+				IntPtr	hDC;
+
+				hDC = IntPtr.Zero;
+
+				try {
+
+					hDC = GDIPlus.GetDC(IntPtr.Zero);
+					g = Graphics.FromHdc(hDC);
+
+					ToLogFont (logFont, g);
+				}
+
+				finally {
+					if (g != null) {
+						g.Dispose();
+					}
+
+					GDIPlus.ReleaseDC(hDC);
+				}
 			}
 		}
 
 		public void ToLogFont (object logFont, Graphics graphics)
 		{
+			IntPtr lf;
+
 			if (graphics == null) {
 				throw new ArgumentNullException ("graphics");
 			}
 
-			// TODO: Does it make a sense to deal with LOGFONTW ?
-			LOGFONTA o = (LOGFONTA)logFont;
-			GDIPlus.CheckStatus (GDIPlus.GdipGetLogFontA(NativeObject, graphics.NativeObject, ref o));
+			lf = Marshal.AllocHGlobal(Marshal.SizeOf(typeof(LOGFONTW)));
+			if (lf == IntPtr.Zero) {
+				throw new OutOfMemoryException("Could not allocate logfont structure memory");
+			}
+			GDIPlus.CheckStatus (GDIPlus.GdipGetLogFontW(NativeObject, graphics.NativeObject, lf));
+
+			Marshal.PtrToStructure(lf, logFont);
+			Marshal.FreeHGlobal(lf);
 		}
 
 		public float GetHeight (Graphics graphics)
