@@ -54,10 +54,12 @@ namespace System.Drawing {
 		{
 			if (nativeFontFamily != IntPtr.Zero) {
 				int language = 0;			
-				StringBuilder sBuilder = new StringBuilder (GDIPlus.FACESIZE * UnicodeEncoding.CharSize);	
-				Status status = GDIPlus.GdipGetFamilyName (nativeFontFamily, sBuilder, language);
+				IntPtr buffer;
+
+				buffer = Marshal.AllocHGlobal(GDIPlus.FACESIZE * UnicodeEncoding.CharSize);
+				Status status = GDIPlus.GdipGetFamilyName (nativeFontFamily, buffer, language);
 				GDIPlus.CheckStatus (status);
-				name = sBuilder.ToString();    		    		
+				name = Marshal.PtrToStringUni(buffer);
 			}
 		}
 		
@@ -263,6 +265,7 @@ namespace System.Drawing {
 			Status status = GDIPlus.GdipDeleteFontFamily (nativeFontFamily);
 			if ( status == Status.Ok ) 
 				nativeFontFamily = IntPtr.Zero;								
+			GC.SuppressFinalize(this);
 		}		
 		
 		public override bool Equals(object obj)
