@@ -53,6 +53,7 @@ namespace System.Drawing
 public abstract class Image : MarshalByRefObject, IDisposable , ICloneable, ISerializable 
 {
 	public delegate bool GetThumbnailImageAbort();
+	private object tag;
 	
 	internal IntPtr nativeObject = IntPtr.Zero;	
 	
@@ -93,24 +94,40 @@ public abstract class Image : MarshalByRefObject, IDisposable , ICloneable, ISer
 	// static
 	public static Image FromFile(string filename)
 	{
-		return new Bitmap (filename);
+		return FromFile (filename, false);
 	}
 	
 	public static Image FromFile(string filename, bool useEmbeddedColorManagement)
 	{
-		return new Bitmap (filename, useEmbeddedColorManagement);
+		IntPtr imagePtr;
+		Status st;
+
+		if (!File.Exists (filename))
+			throw new FileNotFoundException (filename);
+
+		if (useEmbeddedColorManagement)
+			st = GDIPlus.GdipLoadImageFromFileICM (filename, out imagePtr);
+		else
+			st = GDIPlus.GdipLoadImageFromFile (filename, out imagePtr);
+
+		GDIPlus.CheckStatus (st);
+		return new Bitmap (imagePtr);
 	}
 
-	[MonoTODO]	
 	public static Bitmap FromHbitmap(IntPtr hbitmap)
-	{		
-		throw new NotImplementedException ();
+	{
+		return FromHbitmap (hbitmap, IntPtr.Zero);
 	}
 
-	[MonoTODO]	
 	public static Bitmap FromHbitmap(IntPtr hbitmap, IntPtr hpalette)
 	{		
-		throw new NotImplementedException ();
+		IntPtr imagePtr;
+		Status st;
+
+		st = GDIPlus.GdipCreateBitmapFromHBITMAP (hbitmap, hpalette, out imagePtr);
+
+		GDIPlus.CheckStatus (st);
+		return new Bitmap (imagePtr);
 	}
 
 	public static Image FromStream (Stream stream)
@@ -123,6 +140,15 @@ public abstract class Image : MarshalByRefObject, IDisposable , ICloneable, ISer
 		return new Bitmap (stream, useECM);
 	}
 
+#if NET_2_0
+
+	// See http://support.microsoft.com/default.aspx?scid=kb;en-us;831419 for performance discussion	
+	public static Image FromStream (Stream stream, bool useECM, bool validateImageData)
+	{
+		return new Bitmap (stream, useECM);
+	}
+
+#endif
 	public static int GetPixelFormatSize(PixelFormat pixfmt)
 	{
 		int result = 0;
@@ -354,7 +380,7 @@ public abstract class Image : MarshalByRefObject, IDisposable , ICloneable, ISer
 		ImageCodecInfo encoder = null;
 		
 		if (format.Guid.Equals (ImageFormat.MemoryBmp.Guid))
-			format = ImageFormat.Bmp;
+			format = ImageFormat.Png;
 	
 		/* Look for the right encoder for our format*/
 		for (int i = 0; i < encoders.Length; i++) {
@@ -640,7 +666,8 @@ public abstract class Image : MarshalByRefObject, IDisposable , ICloneable, ISer
 				return items;			
 					
 			/* Get PropertyItem list*/
-			properties = Marshal.AllocHGlobal (propsSize);
+			properties = Marshal.AllocHGlobal (propsSize * propNums);
+			
 			status = GDIPlus.GdipGetAllPropertyItems (nativeObject, propsSize, 
 								propNums, properties);
 			GDIPlus.CheckStatus (status);
@@ -676,7 +703,17 @@ public abstract class Image : MarshalByRefObject, IDisposable , ICloneable, ISer
 			return new Size(Width, Height);
 		}
 	}
-	
+
+#if NET_2_0
+
+	[LocalizableAttribute(false)] 
+	[BindableAttribute(true)] 	
+	[TypeConverter (typeof (StringConverter))]
+	public object Tag { 
+		get { return tag; }
+		set { tag = value; }
+	}
+#endif	
 	public float VerticalResolution {
 		get {
 			float resolution;
