@@ -34,7 +34,9 @@ using System.Runtime.InteropServices;
 
 namespace System.Drawing
 {
+#if !NET_2_0
 	[ComVisible (false)]
+#endif
 	public sealed class Region : MarshalByRefObject, IDisposable
 	{
                 private IntPtr nativeRegion = IntPtr.Zero;
@@ -535,13 +537,13 @@ namespace System.Drawing
 
 		public void Dispose ()
 		{
-			Dispose (true);
+			DisposeHandle ();
 			System.GC.SuppressFinalize (this);
 		}
 
-		void Dispose (bool disposing)
+		private void DisposeHandle ()
 		{
-			if (disposing) {
+			if (nativeRegion != IntPtr.Zero) {
 				GDIPlus.GdipDeleteRegion (nativeRegion);
 				nativeRegion = IntPtr.Zero;
 			}
@@ -549,7 +551,7 @@ namespace System.Drawing
 
 		~Region ()
 		{
-			Dispose (false);
+			DisposeHandle ();
 		}
 
                 internal IntPtr NativeObject
@@ -561,7 +563,14 @@ namespace System.Drawing
 				nativeRegion = value;
 			}
 		}
+#if NET_2_0
+		public void ReleaseHrgn (IntPtr regionHandle)		
+		{
+			if (regionHandle == IntPtr.Zero) 
+				throw new ArgumentNullException ("regionHandle is a null reference ");
 
+		}
+#endif
         
 	}
 }
