@@ -36,6 +36,7 @@ using System.Text;
 using System.Drawing.Drawing2D;
 using System.Drawing.Imaging;
 using System.Drawing.Text;
+using System.Security;
 #if NET_2_0
 using System.Runtime.InteropServices.ComTypes;
 #else
@@ -47,6 +48,7 @@ namespace System.Drawing
 	/// <summary>
 	/// GDI+ API Functions
 	/// </summary>
+	[SuppressUnmanagedCodeSecurity]
 	internal class GDIPlus
 	{
 		public const int FACESIZE = 32;
@@ -54,6 +56,7 @@ namespace System.Drawing
 		public static IntPtr Display = IntPtr.Zero;
 		public static bool UseX11Drawable;
 		public static bool UseQuartzDrawable = (Environment.GetEnvironmentVariable ("MONO_MWF_USE_QUARTZ_BACKEND") != null);
+		public static bool UseCocoaDrawable = (Environment.GetEnvironmentVariable ("MONO_GDIP_USE_COCOA_BACKEND") != null);
 
 		#region gdiplus.dll functions
 
@@ -66,7 +69,11 @@ namespace System.Drawing
 		static ulong GdiPlusToken;
 
 		static void ProcessExit (object sender, EventArgs e)
-		{			
+		{		
+			// Called all pending objects and claim any pending handle before
+			// shutting down
+			GC.Collect ();	
+			GC.WaitForPendingFinalizers ();
 			GdiplusShutdown (ref GdiPlusToken);
 
 			if (UseX11Drawable && Display != IntPtr.Zero) {
@@ -555,7 +562,7 @@ namespace System.Drawing
 		[DllImport("gdiplus.dll", CharSet=CharSet.Unicode)]
 		static internal extern Status GdipDrawString (IntPtr graphics, string text, int len, IntPtr font, ref RectangleF rc, IntPtr format, IntPtr brush);
 		[DllImport("gdiplus.dll")]
-		static internal extern Status GdipGetDC (IntPtr graphics, out int hdc);
+		static internal extern Status GdipGetDC (IntPtr graphics, out IntPtr hdc);
 		[DllImport("gdiplus.dll")]
 		static internal extern Status GdipReleaseDC (IntPtr graphics, IntPtr hdc);
 		[DllImport("gdiplus.dll")]
@@ -891,16 +898,11 @@ namespace System.Drawing
 		internal static extern Status GdipCreateBitmapFromGraphics (int width, int height, IntPtr target, out IntPtr bitmap);
 
 		[DllImport("gdiplus.dll")]
-		internal static extern Status GdipBitmapLockBits (IntPtr bmp, ref Rectangle rc, ImageLockMode flags, PixelFormat format, [In, Out] IntPtr bmpData);
+		internal static extern Status GdipBitmapLockBits (IntPtr bmp, ref Rectangle rc, ImageLockMode flags, PixelFormat format, [In, Out] BitmapData bmpData);
 		
 		[DllImport("gdiplus.dll")]
 		internal static extern Status GdipBitmapSetResolution(IntPtr bmp, float xdpi, float ydpi);
-
-		
-		// This an internal GDIPlus Cairo function, not part GDIPlus interface
-		//[DllImport("gdiplus.dll")]
-		//(internal static extern Status ____BitmapLockBits (IntPtr bmp, ref GpRect  rc, ImageLockMode flags, PixelFormat format, ref int width, ref int height, ref int stride, ref int format2, ref int reserved, ref IntPtr scan0);
-		
+				
 		[DllImport("gdiplus.dll")]
 		internal static extern Status GdipBitmapUnlockBits (IntPtr bmp, [In,Out] BitmapData bmpData);
 		
@@ -1089,6 +1091,12 @@ namespace System.Drawing
 		
 		[DllImport("gdiplus.dll")]		
 		internal static extern Status GdipCreateHBITMAPFromBitmap (IntPtr bmp, out IntPtr HandleBmp, int clrbackground);
+
+		[DllImport("gdiplus.dll", CharSet=CharSet.Auto)]
+		internal static extern Status GdipCreateBitmapFromFile ([MarshalAs (UnmanagedType.LPWStr)] string filename, out IntPtr bitmap);
+
+		[DllImport("gdiplus.dll", CharSet=CharSet.Auto)]
+		internal static extern Status GdipCreateBitmapFromFileICM ([MarshalAs (UnmanagedType.LPWStr)] string filename, out IntPtr bitmap);
 		
 		[DllImport("gdiplus.dll")]
 		internal static extern Status GdipCreateHICONFromBitmap (IntPtr bmp, out IntPtr HandleIcon);
@@ -1357,8 +1365,11 @@ namespace System.Drawing
 		internal static extern Status GdipCreateFont (IntPtr fontFamily, float emSize, FontStyle style, GraphicsUnit  unit,  out IntPtr font);
 		[DllImport("gdiplus.dll")]                   
 		internal static extern Status GdipDeleteFont (IntPtr font);		
-		[DllImport("gdiplus.dll")]                   
-		internal static extern Status GdipGetLogFontA(IntPtr font, IntPtr graphics, ref LOGFONTA logfontA);
+		[DllImport("gdiplus.dll", CharSet=CharSet.Ansi)]
+		internal static extern Status GdipGetLogFontA(IntPtr font, IntPtr graphics, IntPtr logfontA);
+		[DllImport("gdiplus.dll", CharSet=CharSet.Unicode)]
+		internal static extern Status GdipGetLogFontW(IntPtr font, IntPtr graphics, IntPtr logfontW);
+
 		[DllImport("gdiplus.dll")]                   
 		internal static extern Status GdipCreateFontFromDC(IntPtr hdc, out IntPtr font);
 		[DllImport("gdiplus.dll", SetLastError=true)]
@@ -1380,6 +1391,16 @@ namespace System.Drawing
 		internal static extern int ReleaseDC(IntPtr hdc);
 		[DllImport("gdi32.dll", EntryPoint="SelectObject", CallingConvention = CallingConvention.StdCall, CharSet = CharSet.Ansi)]
 		internal static extern IntPtr SelectObject(IntPtr hdc, IntPtr obj);	
+		[DllImport("user32.dll", SetLastError=true)]
+		internal static extern bool GetIconInfo (IntPtr hIcon, out IconInfo iconinfo);
+ 
+		[DllImport("user32.dll")]
+		internal static extern IntPtr GetDesktopWindow ();
+
+		[DllImport("gdi32.dll", SetLastError=true)]
+		public static extern int BitBlt(IntPtr hdcDest, int nXDest, int nYDest, 
+			int nWidth, int nHeight, IntPtr hdcSrc, int nXSrc, int nYSrc, int dwRop);
+
 
 		// Some special X11 stuff
 		[DllImport("libX11", EntryPoint="XOpenDisplay")]
@@ -1388,6 +1409,31 @@ namespace System.Drawing
 		[DllImport("libX11", EntryPoint="XCloseDisplay")]
 		internal extern static int XCloseDisplay(IntPtr display);	
 
+		[DllImport ("libX11", EntryPoint="XRootWindow")]
+		internal extern static IntPtr XRootWindow(IntPtr display, int screen);
+			
+		[DllImport ("libX11", EntryPoint="XDefaultDepth")]
+		internal extern static uint XDefaultDepth(IntPtr display, int screen);
+
+		[DllImport ("libX11", EntryPoint="XGetImage")]
+		internal extern static IntPtr XGetImage(IntPtr display, IntPtr drawable, int src_x, int src_y, int width, int height, int pane, int format);
+
+		[DllImport ("libX11", EntryPoint="XGetPixel")]
+		internal extern static int XGetPixel(IntPtr image, int x, int y);
+
+		[DllImport ("libX11", EntryPoint="XDestroyImage")]
+		internal extern static int XDestroyImage(IntPtr image);
+
+		[DllImport ("libX11", EntryPoint="XDefaultVisual")]
+		internal extern static IntPtr XDefaultVisual(IntPtr display, int screen);
+
+		[DllImport ("libX11", EntryPoint="XGetVisualInfo")]
+		internal extern static IntPtr XGetVisualInfo (IntPtr display, int vinfo_mask, ref XVisualInfo vinfo_template, ref int nitems);
+
+		[DllImport ("libX11", EntryPoint="XVisualIDFromVisual")]
+		internal extern static int XVisualIDFromVisual(IntPtr visual);
+	
+		
 		// FontCollection
 		[DllImport ("gdiplus.dll")]
 		internal static extern Status GdipGetFontCollectionFamilyCount (IntPtr collection, out int found);
@@ -1417,8 +1463,8 @@ namespace System.Drawing
 		internal static extern Status GdipCreateFontFamilyFromName (
                         [MarshalAs(UnmanagedType.LPWStr)] string fName, IntPtr collection, out IntPtr fontFamily);
 
-		[DllImport ("gdiplus.dll", CharSet=CharSet.Unicode)]
-		internal static extern Status GdipGetFamilyName(IntPtr family, StringBuilder fName, int language);
+		[DllImport ("gdiplus.dll")]
+		internal static extern Status GdipGetFamilyName(IntPtr family, IntPtr fName, int language);
 
 		[DllImport ("gdiplus.dll")]
 		internal static extern Status GdipGetGenericFontFamilySansSerif (out IntPtr fontFamily);
