@@ -31,11 +31,13 @@
 
 using System;
 using System.Drawing;
+using System.Security.Permissions;
 using NUnit.Framework;
 
 namespace MonoTests.System.Drawing
 {
 	[TestFixture]
+	[SecurityPermission (SecurityAction.Deny, UnmanagedCode = true)]
 	public class ColorTest : Assertion
 	{
 		[SetUp]
@@ -898,6 +900,12 @@ namespace MonoTests.System.Drawing
 			AssertEquals ("#YellowGreen.B", 50, color.B);
 		}
 		
+		static bool FloatsAlmostEqual (float v1, float v2)
+		{
+			float error = Math.Abs(v1-v2)/(v1+v2+float.Epsilon);
+			return error < 0.0001;
+		}
+
 		[Test]
 		public void TestHBSValues ()
 		{
@@ -905,14 +913,227 @@ namespace MonoTests.System.Drawing
 			AssertEquals ("BrightnessWhite", 1.0f, Color.White.GetBrightness ());
 		
 			Color c1 = Color.FromArgb (0, 13, 45, 7); //just some random color
-			AssertEquals ("Hue1",        110.5263f, c1.GetHue ());
-			AssertEquals ("Brightness1", 0.1019608f, c1.GetBrightness ());
-			AssertEquals ("Saturation1", 0.7307692f, c1.GetSaturation ());
+			Assert ("Hue1",        FloatsAlmostEqual (110.5263f, c1.GetHue ()));
+			Assert ("Brightness1", FloatsAlmostEqual (0.1019608f, c1.GetBrightness ()));
+			Assert ("Saturation1", FloatsAlmostEqual (0.7307692f, c1.GetSaturation ()));
 	
 			Color c2 = Color.FromArgb (0, 112, 75, 29); //another random color
-			AssertEquals ("Hue2",        33.25302f, c2.GetHue ());
-			AssertEquals ("Brightness2", 0.2764706f, c2.GetBrightness ());
-			AssertEquals ("Saturation2", 0.5886525f, c2.GetSaturation ());
+			Assert ("Hue2",        FloatsAlmostEqual (33.25302f, c2.GetHue ()));
+			Assert ("Brightness2", FloatsAlmostEqual (0.2764706f, c2.GetBrightness ()));
+			Assert ("Saturation2", FloatsAlmostEqual (0.5886525f, c2.GetSaturation ()));
+		}
+		[Test]
+		public void TestEquals ()
+		{
+			Color color = Color.Red;
+			Color color1 = Color.FromArgb (color.R, color.G, color.B);
+			Assert ("Named color not == unnamed color", !(color==color1));
+			Assert ("Named color not equals unnamed color", !color.Equals (color1));
+			Assert ("Named color != unnamed color", color!=color1);
+			color = Color.FromArgb (0, color1);
+			Assert ("Alpha takes part in comparison", !color.Equals (color1));
+		}
+		[Test]
+		public void TestIsEmpty ()
+		{
+			Color color = Color.Empty;
+			Assert ("Empty color", color.IsEmpty);
+			Assert ("Not empty color", !Color.FromArgb (0, Color.Black).IsEmpty);
+		}
+		[Test]
+		public void IsKnownColor ()
+		{
+			Assert ("KnownColor", Color.FromKnownColor(KnownColor.AliceBlue).IsKnownColor);
+			Assert ("KnownColor", Color.FromName("AliceBlue").IsKnownColor);
+			AssertEquals ("Not KnownColor", false,
+				Color.FromArgb (50, 100, 150, 200).IsKnownColor);
+		}
+		[Test]
+		public void IsNamedColor ()
+		{
+			Assert ("NamedColor", Color.AliceBlue.IsNamedColor);
+			Assert ("NamedColor", Color.FromKnownColor(KnownColor.AliceBlue).IsNamedColor);
+			Assert ("NamedColor", Color.FromName("AliceBlue").IsNamedColor);
+			AssertEquals ("Not NamedColor", false,
+				Color.FromArgb (50, 100, 150, 200).IsNamedColor);
+		}
+		[Test]
+		public void IsSystemColor () {
+			Assert ("SystemColor#1", Color.FromKnownColor(KnownColor.ActiveBorder).IsSystemColor);
+			Assert ("SystemColor#2", Color.FromName("ActiveBorder").IsSystemColor);
+			AssertEquals ("Not SystemColor#1", false,
+				Color.FromArgb (50, 100, 150, 200).IsSystemColor);
+			AssertEquals ("Not SystemColor#2", false,
+				Color.FromKnownColor(KnownColor.AliceBlue).IsSystemColor);
+			AssertEquals ("Not SystemColor#3", false,
+				Color.FromName("AliceBlue").IsSystemColor);
+		}
+		[Test]
+		public void Name ()
+		{
+			AssertEquals ("Color.Name#1", "AliceBlue", Color.AliceBlue.Name);
+			AssertEquals ("Color.Name#2", "ActiveBorder",
+				Color.FromKnownColor (KnownColor.ActiveBorder).Name);
+			AssertEquals ("Color.Name#3", "1122ccff",
+				Color.FromArgb(0x11, 0x22, 0xcc, 0xff).Name);
+		}
+		[Test]
+		public void GetHashCodeTest ()
+		{
+			Color c = Color.AliceBlue;
+			AssertEquals ("GHC#1", false, Color.FromArgb (c.A, c.R, c.G, c.B).GetHashCode () ==
+				c.GetHashCode ());
+			AssertEquals ("GHC#2", c.GetHashCode (), Color.FromName ("AliceBlue").GetHashCode ());
+		}
+		[Test]
+		public void ToArgb ()
+		{
+			AssertEquals (0x11cc8833, Color.FromArgb (0x11, 0xcc, 0x88, 0x33).ToArgb());
+			AssertEquals (unchecked((int)0xf1cc8833), Color.FromArgb (0xf1, 0xcc, 0x88, 0x33).ToArgb());
+		}
+
+		[Test]
+		public void ToKnownColor ()
+		{
+			AssertEquals ("TKC#1", KnownColor.ActiveBorder, Color.FromName ("ActiveBorder").ToKnownColor ());
+			AssertEquals ("TKC#2", KnownColor.AliceBlue, Color.AliceBlue.ToKnownColor ());
+			KnownColor zero = Color.FromArgb (1, 2, 3, 4).ToKnownColor ();
+			AssertEquals ("TKC#3", (KnownColor)0, zero);
+		}
+
+		[Test]
+		public void ToStringTest ()
+		{
+			AssertEquals ("TS#1", "Color [AliceBlue]", Color.AliceBlue.ToString ());
+			AssertEquals ("TS#2", "Color [ActiveBorder]",
+				Color.FromKnownColor (KnownColor.ActiveBorder).ToString ());
+			AssertEquals ("TS#3", "Color [A=1, R=2, G=3, B=4]",
+				Color.FromArgb(1, 2, 3, 4).ToString ());
+			AssertEquals ("TS#4", "Color [Empty]", Color.Empty.ToString ());
+		}
+
+		[Test]
+		public void Equality ()
+		{
+			Color c = Color.AliceBlue;
+			Assert ("EQ#1", c == Color.FromName ("AliceBlue"));
+			AssertEquals ("EQ#2", false, c == Color.FromArgb (c.A, c.R, c.G, c.B));
+			Assert ("EQ#3", c.Equals (Color.FromName ("AliceBlue")));
+			AssertEquals ("EQ#4", false, c.Equals(Color.FromArgb (c.A, c.R, c.G, c.B)));
+			AssertEquals ("EQ#5", false, c != Color.FromName ("AliceBlue"));
+			Assert("EQ#6", c != Color.FromArgb (c.A, c.R, c.G, c.B));
+		}
+
+		[Test]
+		[ExpectedException (typeof (ArgumentException))]
+		public void FromArgb_InvalidAlpha1 ()
+		{
+			Color.FromArgb (-1, Color.Red);
+		}
+
+		[Test]
+		[ExpectedException (typeof (ArgumentException))]
+		public void FromArgb_InvalidAlpha2 ()
+		{
+			Color.FromArgb (256, Color.Red);
+		}
+
+		[Test]
+		[ExpectedException (typeof (ArgumentException))]
+		public void FromArgb_InvalidAlpha3 ()
+		{
+			Color.FromArgb (-1, 0, 0, 0);
+		}
+
+		[Test]
+		[ExpectedException (typeof (ArgumentException))]
+		public void FromArgb_InvalidAlpha4 ()
+		{
+			Color.FromArgb (256, 0, 0, 0);
+		}
+
+		[Test]
+		[ExpectedException (typeof (ArgumentException))]
+		public void FromArgb_InvalidRed1 ()
+		{
+			Color.FromArgb (-1, 0, 0);
+		}
+
+		[Test]
+		[ExpectedException (typeof (ArgumentException))]
+		public void FromArgb_InvalidRed2 ()
+		{
+			Color.FromArgb (256, 0, 0);
+		}
+
+		[Test]
+		[ExpectedException (typeof (ArgumentException))]
+		public void FromArgb_InvalidRed3 ()
+		{
+			Color.FromArgb (0, -1, 0, 0);
+		}
+
+		[Test]
+		[ExpectedException (typeof (ArgumentException))]
+		public void FromArgb_InvalidRed4 ()
+		{
+			Color.FromArgb (0, 256, 0, 0);
+		}
+
+		[Test]
+		[ExpectedException (typeof (ArgumentException))]
+		public void FromArgb_InvalidGreen1 ()
+		{
+			Color.FromArgb (0, -1, 0);
+		}
+
+		[Test]
+		[ExpectedException (typeof (ArgumentException))]
+		public void FromArgb_InvalidGreen2 ()
+		{
+			Color.FromArgb (0, 256, 0);
+		}
+
+		[Test]
+		[ExpectedException (typeof (ArgumentException))]
+		public void FromArgb_InvalidGreen3 ()
+		{
+			Color.FromArgb (0, 0, -1, 0);
+		}
+
+		[Test]
+		[ExpectedException (typeof (ArgumentException))]
+		public void FromArgb_InvalidGreen4 ()
+		{
+			Color.FromArgb (0, 0, 256, 0);
+		}
+
+		[Test]
+		[ExpectedException (typeof (ArgumentException))]
+		public void FromArgb_InvalidBlue1 ()
+		{
+			Color.FromArgb (0, 0, -1);
+		}
+
+		[Test]
+		[ExpectedException (typeof (ArgumentException))]
+		public void FromArgb_InvalidBlue2 ()
+		{
+			Color.FromArgb (0, 0, 256);
+		}
+
+		[Test]
+		[ExpectedException (typeof (ArgumentException))]
+		public void FromArgb_InvalidBlue3 ()
+		{
+			Color.FromArgb (0, 0, 0, -1);
+		}
+
+		[Test]
+		[ExpectedException (typeof (ArgumentException))]
+		public void FromArgb_InvalidBlue4 ()
+		{
+			Color.FromArgb (0, 0, 0, 256);
 		}
 	}
 }
