@@ -33,10 +33,12 @@ using System;
 using System.Drawing;
 using NUnit.Framework;
 using System.IO;
+using System.Security.Permissions;
 
 namespace MonoTests.System.Drawing{
 
 	[TestFixture]	
+	[SecurityPermission (SecurityAction.Deny, UnmanagedCode = true)]
 	public class TestIcon {
 		
 		Icon icon;
