@@ -47,9 +47,15 @@ namespace System.Web.Compilation
 		const string cachePrefix = "@@Assembly";
 		const string cacheTypePrefix = "@@@Type";
 
-		public static void InsertType (Type type, string filename)
+		public static void InsertTypeFileDep (Type type, string filename)
 		{
-			string [] cacheKeys = new string [] { cachePrefix + filename };
+			CacheDependency dep = new CacheDependency (filename);
+			HttpRuntime.Cache.InsertPrivate (cacheTypePrefix + filename, type, dep);
+		}
+
+ 		public static void InsertType (Type type, string filename)
+ 		{
+ 			string [] cacheKeys = new string [] { cachePrefix + filename };
 			CacheDependency dep = new CacheDependency (null, cacheKeys);
 			HttpRuntime.Cache.InsertPrivate (cacheTypePrefix + filename, type, dep);
 		}
@@ -64,11 +70,18 @@ namespace System.Web.Compilation
 			Cache cache = HttpRuntime.Cache;
 			string key = cachePrefix + compiler.Parser.InputFile;
 			CompilerResults results = (CompilerResults) cache [key];
+
+#if NET_2_0
+			if (!compiler.IsRebuildingPartial)
+#endif
 			if (results != null)
 				return results;
 
 			lock (compilationLock) {
 				results = (CompilerResults) cache [key];
+#if NET_2_0
+				if (!compiler.IsRebuildingPartial)
+#endif
 				if (results != null)
 					return results;
 
@@ -133,13 +146,18 @@ namespace System.Web.Compilation
 				if (results != null)
 					return results;
  
+#if CONFIGURATION_2_0
+				CompilationSection config = (CompilationSection) WebConfigurationManager.GetSection ("system.web/compilation");
+				Compiler c = config.Compilers[language];
+				CodeDomProvider provider = c.Provider;
+#else
 				CompilationConfiguration config;
 				config = CompilationConfiguration.GetInstance (HttpContext.Current);
 				CodeDomProvider provider = config.GetProvider (language);
+#endif
 				if (provider == null)
 					throw new HttpException ("Configuration error. Language not supported: " +
 								  language, 500);
-
 				ICodeCompiler compiler = provider.CreateCompiler ();
 				CompilerParameters options = GetOptions (assemblies);
 				TempFileCollection tempcoll = new TempFileCollection (config.TempDirectory, true);
