@@ -5,6 +5,7 @@
 //	Gonzalo Paniagua Javier (gonzalo@ximian.com)
 //
 // (C) 2002,2003 Ximian, Inc (http://www.ximian.com)
+// Copyright (c) 2005 Novell, Inc (http://www.novell.com)
 //
 
 //
@@ -28,10 +29,10 @@
 // WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 //
 
-using System;
 using System.Collections;
 using System.IO;
 using System.Security.Cryptography;
+using System.Security.Permissions;
 using System.Text;
 using System.Web;
 using System.Web.Configuration;
@@ -39,20 +40,33 @@ using System.Web.Util;
 
 namespace System.Web.Security
 {
+	// CAS - no InheritanceDemand here as the class is sealed
+	[AspNetHostingPermission (SecurityAction.LinkDemand, Level = AspNetHostingPermissionLevel.Minimal)]
 	public sealed class FormsAuthentication
 	{
+		const int MD5_hash_size = 16;
+		const int SHA1_hash_size = 20;
+
 		static string authConfigPath = "system.web/authentication";
 		static bool initialized;
 		static string cookieName;
 		static string cookiePath;
 		static int timeout;
-		//static FormsProtectionEnum protection;
+		static FormsProtectionEnum protection;
 		static object locker = new object ();
+		static byte [] init_vector; // initialization vector used for 3DES
 #if NET_1_1
 		static bool requireSSL;
 		static bool slidingExpiration;
 #endif
-
+#if NET_2_0
+		static string cookie_domain;
+		static HttpCookieMode cookie_mode;
+		static bool cookies_supported;
+		static string default_url;
+		static bool enable_crossapp_redirects;
+		static string login_url;
+#endif
 		// same names and order used in xsp
 		static string [] indexFiles = { "index.aspx",
 						"Default.aspx",
@@ -60,6 +74,13 @@ namespace System.Web.Security
 						"index.html",
 						"index.htm" };
 
+#if NET_2_0
+		[Obsolete]
+#endif
+		public FormsAuthentication ()
+		{
+		}
+
 		public static bool Authenticate (string name, string password)
 		{
 			if (name == null || password == null)
@@ -91,28 +112,71 @@ namespace System.Web.Security
 			return (password == stored);
 		}
 
+		static FormsAuthenticationTicket Decrypt2 (byte [] bytes)
+		{
+			if (protection == FormsProtectionEnum.None)
+				return FormsAuthenticationTicket.FromByteArray (bytes);
+
+			MachineKeyConfig config = HttpContext.GetAppConfig ("system.web/machineKey") as MachineKeyConfig;
+			bool all = (protection == FormsProtectionEnum.All);
+
+			byte [] result = bytes;
+			if (all || protection == FormsProtectionEnum.Encryption) {
+				ICryptoTransform decryptor;
+				decryptor = TripleDES.Create ().CreateDecryptor (config.DecryptionKey192Bits, init_vector);
+				result = decryptor.TransformFinalBlock (bytes, 0, bytes.Length);
+				bytes = null;
+			}
+
+			if (all || protection == FormsProtectionEnum.Validation) {
+				int count;
+
+				if (config.ValidationType == MachineKeyValidation.MD5)
+					count = MD5_hash_size;
+				else
+					count = SHA1_hash_size; // 3DES and SHA1
+
+				byte [] vk = config.ValidationKey;
+				byte [] mix = new byte [result.Length - count + vk.Length];
+				Buffer.BlockCopy (result, 0, mix, 0, result.Length - count);
+				Buffer.BlockCopy (vk, 0, mix, result.Length - count, vk.Length);
+
+				byte [] hash = null;
+				switch (config.ValidationType) {
+				case MachineKeyValidation.MD5:
+					hash = MD5.Create ().ComputeHash (mix);
+					break;
+				// From MS docs: "When 3DES is specified, forms authentication defaults to SHA1"
+				case MachineKeyValidation.TripleDES:
+				case MachineKeyValidation.SHA1:
+					hash = SHA1.Create ().ComputeHash (mix);
+					break;
+				}
+
+				if (result.Length < count)
+					throw new ArgumentException ("Error validating ticket (length).", "encryptedTicket");
+
+				int i, k;
+				for (i = result.Length - count, k = 0; k < count; i++, k++) {
+					if (result [i] != hash [k])
+						throw new ArgumentException ("Error validating ticket.", "encryptedTicket");
+				}
+			}
+
+			return FormsAuthenticationTicket.FromByteArray (result);
+		}
+
 		public static FormsAuthenticationTicket Decrypt (string encryptedTicket)
 		{
 			if (encryptedTicket == null || encryptedTicket == String.Empty)
 				throw new ArgumentException ("Invalid encrypted ticket", "encryptedTicket");
 
 			Initialize ();
-			byte [] bytes = MachineKeyConfigHandler.GetBytes (encryptedTicket, encryptedTicket.Length);
-			string decrypted = Encoding.ASCII.GetString (bytes);
-			FormsAuthenticationTicket ticket = null;
+
+			FormsAuthenticationTicket ticket;
+			byte [] bytes = MachineKeyConfig.GetBytes (encryptedTicket, encryptedTicket.Length);
 			try {
-				string [] values = decrypted.Split ((char) 1, (char) 2, (char) 3, (char) 4, (char) 5, (char) 6, (char) 7);
-				if (values.Length != 8)
-					throw new Exception (values.Length + " " + encryptedTicket);
-
-
-				ticket = new FormsAuthenticationTicket (Int32.Parse (values [0]),
-									values [1],
-									new DateTime (Int64.Parse (values [2])),
-									new DateTime (Int64.Parse (values [3])),
-									(values [4] == "1"),
-									values [5],
-									values [6]);
+				ticket = Decrypt2 (bytes);
 			} catch (Exception) {
 				ticket = null;
 			}
@@ -126,24 +190,45 @@ namespace System.Web.Security
 				throw new ArgumentNullException ("ticket");
 
 			Initialize ();
-			StringBuilder allTicket = new StringBuilder ();
-			allTicket.Append (ticket.Version);
-			allTicket.Append ('\u0001');
-			allTicket.Append (ticket.Name);
-			allTicket.Append ('\u0002');
-			allTicket.Append (ticket.IssueDate.Ticks);
-			allTicket.Append ('\u0003');
-			allTicket.Append (ticket.Expiration.Ticks);
-			allTicket.Append ('\u0004');
-			allTicket.Append (ticket.IsPersistent ? '1' : '0');
-			allTicket.Append ('\u0005');
-			allTicket.Append (ticket.UserData);
-			allTicket.Append ('\u0006');
-			allTicket.Append (ticket.CookiePath);
-			allTicket.Append ('\u0007');
-			//if (protection == FormsProtectionEnum.None)
-				return GetHexString (allTicket.ToString ());
-			//TODO: encrypt and validate
+			byte [] ticket_bytes = ticket.ToByteArray ();
+			if (protection == FormsProtectionEnum.None)
+				return GetHexString (ticket_bytes);
+
+			byte [] result = ticket_bytes;
+			MachineKeyConfig config = HttpContext.GetAppConfig ("system.web/machineKey") as MachineKeyConfig;
+			bool all = (protection == FormsProtectionEnum.All);
+			if (all || protection == FormsProtectionEnum.Validation) {
+				byte [] valid_bytes = null;
+				byte [] vk = config.ValidationKey;
+				byte [] mix = new byte [ticket_bytes.Length + vk.Length];
+				Buffer.BlockCopy (ticket_bytes, 0, mix, 0, ticket_bytes.Length);
+				Buffer.BlockCopy (vk, 0, mix, result.Length, vk.Length);
+
+				switch (config.ValidationType) {
+				case MachineKeyValidation.MD5:
+					valid_bytes = MD5.Create ().ComputeHash (mix);
+					break;
+				// From MS docs: "When 3DES is specified, forms authentication defaults to SHA1"
+				case MachineKeyValidation.TripleDES:
+				case MachineKeyValidation.SHA1:
+					valid_bytes = SHA1.Create ().ComputeHash (mix);
+					break;
+				}
+
+				int tlen = ticket_bytes.Length;
+				int vlen = valid_bytes.Length;
+				result = new byte [tlen + vlen];
+				Buffer.BlockCopy (ticket_bytes, 0, result, 0, tlen);
+				Buffer.BlockCopy (valid_bytes, 0, result, tlen, vlen);
+			}
+
+			if (all || protection == FormsProtectionEnum.Encryption) {
+				ICryptoTransform encryptor;
+				encryptor = TripleDES.Create ().CreateEncryptor (config.DecryptionKey192Bits, init_vector);
+				result = encryptor.TransformFinalBlock (result, 0, result.Length);
+			}
+
+			return GetHexString (result);
 		}
 
 		public static HttpCookie GetAuthCookie (string userName, bool createPersistentCookie)
@@ -187,7 +272,6 @@ namespace System.Web.Security
 			if (userName == null)
 				return null;
 
-			//TODO: what's createPersistentCookie used for?
 			Initialize ();
 			HttpRequest request = HttpContext.Current.Request;
 			string returnUrl = request ["RETURNURL"];
@@ -213,11 +297,6 @@ namespace System.Web.Security
 			return returnUrl;
 		}
 
-		static string GetHexString (string str)
-		{
-			return GetHexString (Encoding.ASCII.GetBytes (str));
-		}
-
 		static string GetHexString (byte [] bytes)
 		{
 			StringBuilder result = new StringBuilder (bytes.Length * 2);
@@ -257,27 +336,55 @@ namespace System.Web.Security
 					return;
 
 				HttpContext context = HttpContext.Current;
-				if (context == null)
-					throw new HttpException ("Context is null!");
-
+#if NET_2_0
+				AuthConfig authConfig = null;
+				if (context != null)
+					authConfig = context.GetConfig (authConfigPath) as AuthConfig;
+#else
 				AuthConfig authConfig = context.GetConfig (authConfigPath) as AuthConfig;
+#endif
 				if (authConfig != null) {
 					cookieName = authConfig.CookieName;
 					timeout = authConfig.Timeout;
 					cookiePath = authConfig.CookiePath;
-					//protection = authConfig.Protection;
+					protection = authConfig.Protection;
 #if NET_1_1
 					requireSSL = authConfig.RequireSSL;
 					slidingExpiration = authConfig.SlidingExpiration;
+#endif
+#if NET_2_0
+					cookie_domain = authConfig.CookieDomain;
+					cookie_mode = authConfig.CookieMode;
+					cookies_supported = authConfig.CookiesSupported;
+					default_url = authConfig.DefaultUrl;
+					enable_crossapp_redirects = authConfig.EnableCrossAppRedirects;
+					login_url = authConfig.LoginUrl;
 #endif
 				} else {
 					cookieName = ".MONOAUTH";
 					timeout = 30;
 					cookiePath = "/";
-					//protection = FormsProtectionEnum.All;
+					protection = FormsProtectionEnum.All;
 #if NET_1_1
 					slidingExpiration = true;
 #endif
+#if NET_2_0
+					cookie_domain = String.Empty;
+					cookie_mode = HttpCookieMode.UseDeviceProfile;
+					cookies_supported = true;
+					default_url = "/default.aspx";
+					login_url = "/login.aspx";
+#endif
+				}
+
+				// IV is 8 bytes long for 3DES
+				init_vector = new byte [8];
+				int len = cookieName.Length;
+				for (int i = 0; i < 8; i++) {
+					if (i >= len)
+						break;
+
+					init_vector [i] = (byte) cookieName [i];
 				}
 
 				initialized = true;
@@ -296,8 +403,7 @@ namespace System.Web.Security
 
 			Initialize ();
 			SetAuthCookie (userName, createPersistentCookie, strCookiePath);
-			HttpResponse resp = HttpContext.Current.Response;
-			resp.Redirect (GetRedirectUrl (userName, createPersistentCookie), false);
+			Redirect (GetRedirectUrl (userName, createPersistentCookie));
 		}
 
 		public static FormsAuthenticationTicket RenewTicketIfOld (FormsAuthenticationTicket tOld)
@@ -347,7 +453,12 @@ namespace System.Web.Security
 			if (response == null)
 				throw new HttpException ("Response is null!");
 
-			response.Cookies.MakeCookieExpire (cookieName, cookiePath);
+			HttpCookieCollection cc = response.Cookies;
+			cc.Remove (cookieName);
+			HttpCookie expiration_cookie = new HttpCookie (cookieName, "");
+			expiration_cookie.Expires = new DateTime (1999, 10, 12);
+			expiration_cookie.Path = cookiePath;
+			cc.Add (expiration_cookie);
 		}
 
 		public static string FormsCookieName
@@ -380,6 +491,47 @@ namespace System.Web.Security
 			}
 		}
 #endif
+
+#if NET_2_0
+		public static string CookieDomain {
+			get { return cookie_domain; }
+		}
+
+		public static HttpCookieMode CookieMode {
+			get { return cookie_mode; }
+		}
+
+		public static bool CookiesSupported {
+			get { return cookies_supported; }
+		}
+
+		public static string DefaultUrl {
+			get { return default_url; }
+		}
+
+		public static bool EnableCrossAppRedirects {
+			get { return enable_crossapp_redirects; }
+		}
+
+		public static string LoginUrl {
+			get { return login_url; }
+		}
+
+		public static void RedirectToLoginPage ()
+		{
+			Redirect (LoginUrl);
+		}
+
+		[MonoTODO ("needs more tests")]
+		public static void RedirectToLoginPage (string extraQueryString)
+		{
+			// TODO: if ? is in LoginUrl (legal?), ? in query (legal?) ...
+			Redirect (LoginUrl + "?" + extraQueryString);
+		}
+#endif
+		private static void Redirect (string url)
+		{
+			HttpContext.Current.Response.Redirect (url);
+		}
 	}
 }
-
