@@ -5,8 +5,7 @@
 //	Gonzalo Paniagua Javier (gonzalo@ximian.com)
 //
 // (C) 2002 Ximian, Inc (http://www.ximian.com)
-//
-
+// Copyright (C) 2005 Novell, Inc (http://www.novell.com)
 //
 // Permission is hereby granted, free of charge, to any person obtaining
 // a copy of this software and associated documentation files (the
@@ -28,17 +27,23 @@
 // WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 //
 
-using System;
+using System.Security.Permissions;
 using System.Security.Principal;
 using System.Text;
-using System.Web;
 using System.Web.Configuration;
 using System.Web.Util;
 
 namespace System.Web.Security
 {
+	// CAS - no InheritanceDemand here as the class is sealed
+	[AspNetHostingPermission (SecurityAction.LinkDemand, Level = AspNetHostingPermissionLevel.Minimal)]
 	public sealed class FormsAuthenticationModule : IHttpModule
 	{
+		[SecurityPermission (SecurityAction.Demand, UnmanagedCode = true)]
+		public FormsAuthenticationModule ()
+		{
+		}
+
 		public void Dispose ()
 		{
 		}
@@ -53,17 +58,41 @@ namespace System.Web.Security
 		{
 			HttpApplication app = (HttpApplication) sender;
 			HttpContext context = app.Context;
+
+			string cookieName;
+			string cookiePath;
+			string loginPage;
+			bool slidingExpiration;
+
+#if CONFIGURATION_2_0
+			AuthenticationSection config = (AuthenticationSection) WebConfigurationManager.GetWebApplicationSection ("system.web/authentication");
+#else
 			AuthConfig config = (AuthConfig) context.GetConfig ("system.web/authentication");
-			if (config.Mode != AuthenticationMode.Forms) {
+#endif
+
+			if (config == null || config.Mode != AuthenticationMode.Forms) {
 				return;
 			}
 
-			string cookieName = config.CookieName;
-			string cookiePath = config.CookiePath;
-			string loginPage = config.LoginUrl;
+#if CONFIGURATION_2_0
+			cookieName = config.Forms.Name;
+			cookiePath = config.Forms.Path;
+			loginPage = config.Forms.LoginUrl;
+			slidingExpiration = config.Forms.SlidingExpiration;
+#else
+			cookieName = config.CookieName;
+			cookiePath = config.CookiePath;
+			loginPage = config.LoginUrl;
+			slidingExpiration = config.SlidingExpiration;
+#endif
+
+			string reqPath = "";
+			string loginPath = null;
+			try {
+				reqPath = context.Request.PhysicalPath;
+				loginPath = context.Request.MapPath (loginPage);
+			} catch {} // ignore
 
-			string reqPath = context.Request.PhysicalPath;
-			string loginPath = context.Request.MapPath (loginPage);
 			context.SkipAuthorization = (reqPath == loginPath);
 			
 			FormsAuthenticationEventArgs formArgs = new FormsAuthenticationEventArgs (context);
@@ -85,7 +114,7 @@ namespace System.Web.Security
 			if (ticket == null || (ticket.IsPersistent && ticket.Expired))
 				return;
 
-			if (config.SlidingExpiration)
+			if (slidingExpiration)
 				ticket = FormsAuthentication.RenewTicketIfOld (ticket);
 
 			context.User = new GenericPrincipal (new FormsIdentity (ticket), new string [0]);
@@ -105,12 +134,19 @@ namespace System.Web.Security
 			if (context.Response.StatusCode != 401 || context.Request.QueryString ["ReturnUrl"] != null)
 				return;
 
+			string loginPage;
+#if CONFIGURATION_2_0
+			AuthenticationSection config = (AuthenticationSection) WebConfigurationManager.GetWebApplicationSection ("system.web/authentication");
+			loginPage = config.Forms.LoginUrl;
+#else
 			AuthConfig config = (AuthConfig) context.GetConfig ("system.web/authentication");
-			if (config.Mode != AuthenticationMode.Forms)
+			loginPage = config.LoginUrl;
+#endif
+			if (config == null || config.Mode != AuthenticationMode.Forms)
 				return;
 
 			StringBuilder login = new StringBuilder ();
-			login.Append (UrlUtils.Combine (context.Request.ApplicationPath, config.LoginUrl));
+			login.Append (UrlUtils.Combine (context.Request.ApplicationPath, loginPage));
 			login.AppendFormat ("?ReturnUrl={0}", HttpUtility.UrlEncode (context.Request.RawUrl));
 			context.Response.Redirect (login.ToString ());
 		}
