@@ -41,12 +41,12 @@ using System.Reflection;
 namespace System.Web.UI.WebControls
 {
 	[ToolboxDataAttribute ("<{0}:DetailsView runat=\"server\" Width=\"125px\" Height=\"50px\"></{0}:DetailsView>")]
-	[DesignerAttribute ("System.Web.UI.Design.WebControls.DetailsViewDesigner, System.Design, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b03f5f7f11d50a3a", "System.ComponentModel.Design.IDesigner")]
+	[DesignerAttribute ("System.Web.UI.Design.WebControls.DetailsViewDesigner, " + Consts.AssemblySystem_Design, "System.ComponentModel.Design.IDesigner")]
 	[ControlValuePropertyAttribute ("SelectedValue")]
 	[DefaultEventAttribute ("PageIndexChanging")]
 	[AspNetHostingPermissionAttribute (SecurityAction.LinkDemand, Level = AspNetHostingPermissionLevel.Minimal)]
 	[AspNetHostingPermissionAttribute (SecurityAction.InheritanceDemand, Level = AspNetHostingPermissionLevel.Minimal)]
-	public class DetailsView: CompositeDataBoundControl, ICallbackEventHandler, ICallbackContainer, IDataItemContainer
+	public class DetailsView: CompositeDataBoundControl, ICallbackEventHandler, ICallbackContainer, IDataItemContainer, INamingContainer
 	{
 		object dataItem;
 		
@@ -359,7 +359,7 @@ namespace System.Web.UI.WebControls
 		[UrlPropertyAttribute]
 		[WebCategoryAttribute ("Appearance")]
 		[DefaultValueAttribute ("")]
-		[EditorAttribute ("System.Web.UI.Design.ImageUrlEditor, System.Design, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b03f5f7f11d50a3a", "System.Drawing.Design.UITypeEditor, System.Drawing, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b03f5f7f11d50a3a")]
+		[EditorAttribute ("System.Web.UI.Design.ImageUrlEditor, " + Consts.AssemblySystem_Design, "System.Drawing.Design.UITypeEditor, " + Consts.AssemblySystem_Drawing)]
 		public virtual string BackImageUrl {
 			get {
 				object ob = ViewState ["BackImageUrl"];
@@ -479,7 +479,7 @@ namespace System.Web.UI.WebControls
 			}
 		}
 	
-		[EditorAttribute ("System.Web.UI.Design.WebControls.DataControlFieldTypeEditor, System.Design, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b03f5f7f11d50a3a", "System.Drawing.Design.UITypeEditor, System.Drawing, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b03f5f7f11d50a3a")]
+		[EditorAttribute ("System.Web.UI.Design.WebControls.DataControlFieldTypeEditor, " + Consts.AssemblySystem_Design, "System.Drawing.Design.UITypeEditor, " + Consts.AssemblySystem_Drawing)]
 		[MergablePropertyAttribute (false)]
 		[PersistenceModeAttribute (PersistenceMode.InnerProperty)]
 		[DefaultValueAttribute (null)]
@@ -499,7 +499,7 @@ namespace System.Web.UI.WebControls
 		[DefaultValueAttribute (null)]
 		[WebCategoryAttribute ("Data")]
 		[TypeConverter (typeof(StringArrayConverter))]
-		[EditorAttribute ("System.Web.UI.Design.WebControls.DataFieldEditor, System.Design, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b03f5f7f11d50a3a", "System.Drawing.Design.UITypeEditor, System.Drawing, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b03f5f7f11d50a3a")]
+		[EditorAttribute ("System.Web.UI.Design.WebControls.DataFieldEditor, " + Consts.AssemblySystem_Design, "System.Drawing.Design.UITypeEditor, " + Consts.AssemblySystem_Drawing)]
 		public virtual string[] DataKeyNames
 		{
 			get {
@@ -719,6 +719,7 @@ namespace System.Web.UI.WebControls
 			}
 		}
 		
+		[Category ("Layout")]
 		[DefaultValueAttribute (HorizontalAlign.NotSet)]
 		public virtual HorizontalAlign HorizontalAlign {
 			get {
@@ -803,7 +804,8 @@ namespace System.Web.UI.WebControls
 		
 		
 		[DefaultValue (null)]
-		[TemplateContainer (typeof(DetailsView), BindingDirection.OneWay)]
+		/* DataControlPagerCell isnt specified in the docs */
+		//[TemplateContainer (typeof(DataControlPagerCell), BindingDirection.OneWay)]
 		[PersistenceMode (PersistenceMode.InnerProperty)]
 	    [Browsable (false)]
 		public ITemplate PagerTemplate {
@@ -983,6 +985,12 @@ namespace System.Web.UI.WebControls
 			return table;
 		}
 	
+		[MonoTODO]
+		protected override Style CreateControlStyle ()
+		{
+			throw new NotImplementedException ();
+		}
+		
 		protected override int CreateChildControls (IEnumerable data, bool dataBinding)
 		{
 			PagedDataSource dataSource;
@@ -1118,6 +1126,12 @@ namespace System.Web.UI.WebControls
 			
 			return dataSource.DataSourceCount;
 		}
+
+		[MonoTODO]
+		protected override void EnsureDataBound ()
+		{
+			throw new NotImplementedException ();
+		}
 		
 		DataControlRowState GetRowState (int index)
 		{
@@ -1240,12 +1254,18 @@ namespace System.Web.UI.WebControls
 			}
 		}
 		
-		protected override void PerformDataBinding (IEnumerable data)
+		protected internal override void PerformDataBinding (IEnumerable data)
 		{
 			base.PerformDataBinding (data);
 		}
+
+		[MonoTODO]
+		protected internal virtual void PrepareControlHierarchy ()
+		{
+			throw new NotImplementedException ();
+		}
 		
-		protected override void OnInit (EventArgs e)
+		protected internal override void OnInit (EventArgs e)
 		{
 			Page.RegisterRequiresControlState (this);
 			base.OnInit (e);
@@ -1625,6 +1645,12 @@ namespace System.Web.UI.WebControls
 			RenderGrid (writer);
 			return sw.ToString ();
 		}
+
+		[MonoTODO]
+		protected virtual string GetCallbackScript (IButtonControl buttonControl, string argument)
+		{
+			throw new NotImplementedException ();
+		}
 		
 		string ICallbackContainer.GetCallbackScript (IButtonControl control, string argument)
 		{
@@ -1633,8 +1659,14 @@ namespace System.Web.UI.WebControls
 			else
 				return null;
 		}
+
+		[MonoTODO]
+		protected override void OnPagePreLoad (object sender, EventArgs e)
+		{
+			throw new NotImplementedException ();
+		}
 		
-		protected override void OnPreRender (EventArgs e)
+		protected internal override void OnPreRender (EventArgs e)
 		{
 			base.OnPreRender (e);
 			
@@ -1657,7 +1689,7 @@ namespace System.Web.UI.WebControls
 			}
 		}
 		
-		protected override void Render (HtmlTextWriter writer)
+		protected internal override void Render (HtmlTextWriter writer)
 		{
 			if (EnablePagingCallbacks)
 				base.RenderBeginTag (writer);
