@@ -1,4 +1,9 @@
-
+//
+// System.Web.UI.WebControls.ValidationSummaryDisplayMode.cs
+//
+// Author: Duncan Mak (duncan@novell.com)
+//
+// Copyright (C) 2005 Novell, Inc (http://www.novell.com)
 //
 // Permission is hereby granted, free of charge, to any person obtaining
 // a copy of this software and associated documentation files (the
@@ -19,21 +24,12 @@
 // OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 // WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 //
-/**
- * Namespace: System.Web.UI.WebControls
- * Enumeration:     ValidationDataType
- * 
- * Author:  Gaurav Vaish
- * Maintainer: gvaish@iitk.ac.in
- * Contact: <my_scripts2001@yahoo.com>, <gvaish@iitk.ac.in>
- * Implementation: yes
- * Status:  100%
- * 
- * (C) Gaurav Vaish (2001)
- */
 
-namespace System.Web.UI.WebControls
-{
+using System;
+
+namespace System.Web.UI.WebControls {
+
+	[Serializable]
 	public enum ValidationDataType
 	{
 		String,
