@@ -5,8 +5,7 @@
 //	Gonzalo Paniagua Javier (gonzalo@ximian.com)
 //
 // (C) 2002,2003 Ximian, Inc (http://www.ximian.com)
-//
-
+// Copyright (C) 2005 Novell, Inc (http://www.novell.com)
 //
 // Permission is hereby granted, free of charge, to any person obtaining
 // a copy of this software and associated documentation files (the
@@ -27,18 +26,20 @@
 // OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 // WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 //
-using System;
+
 using System.Collections;
 using System.Collections.Specialized;
 using System.Globalization;
+using System.Security.Permissions;
 using System.Text;
-using System.Web;
 using System.Web.Compilation;
 using System.Web.Configuration;
 using System.Web.Util;
 
 namespace System.Web.UI
 {
+	// CAS - no InheritanceDemand here as the class is sealed
+	[AspNetHostingPermission (SecurityAction.LinkDemand, Level = AspNetHostingPermissionLevel.Minimal)]
 	public sealed class PageParser : TemplateControlParser
 	{
 		bool enableSessionState = true;
@@ -87,7 +88,13 @@ namespace System.Web.UI
 
 		internal override void ProcessMainAttributes (Hashtable atts)
 		{
-			string enabless = GetString (atts, "EnableSessionState", PagesConfig.EnableSessionState);
+			string enabless = GetString (atts, "EnableSessionState",
+#if CONFIGURATION_2_0
+						     PagesConfig.EnableSessionState.ToString()
+#else
+						     PagesConfig.EnableSessionState
+#endif
+						     );
 			if (enabless != null) {
 				readonlySessionState = (String.Compare (enabless, "readonly", true) == 0);
 				if (readonlySessionState == true || String.Compare (enabless, "true", true) == 0) {
