@@ -40,6 +40,7 @@ using Mono.Xml;
 
 #if NET_2_0
 
+using System.Collections.Generic;
 using QName = System.Xml.XmlQualifiedName;
 using Form = System.Xml.Schema.XmlSchemaForm;
 using Use = System.Xml.Schema.XmlSchemaUse;
@@ -113,9 +114,9 @@ namespace Mono.Xml.Schema
 			nsResolver = reader as IXmlNamespaceResolver;
 			if (nsResolver == null)
 				throw new ArgumentException ("Argument XmlReader must implement IXmlNamespaceResolver.");
-			options = ValidationFlags.IgnoreValidationWarnings
-				| ValidationFlags.IgnoreSchemaLocation
-				| ValidationFlags.IgnoreInlineSchema;
+			options = ValidationFlags.ReportValidationWarnings
+				| ValidationFlags.ProcessSchemaLocation
+				| ValidationFlags.ProcessInlineSchema;
 
 			this.reader = reader;
 			if (schemas == null)
@@ -197,8 +198,7 @@ namespace Mono.Xml.Schema
 				navigator.NameTable,
 				schemas,
 				navigator,
-				ValidationFlags.IgnoreSchemaLocation |
-					ValidationFlags.IgnoreInlineSchema);
+				ValidationFlags.ProcessIdentityConstraints);
 
 			readerLineInfo = navigator as IXmlLineInfo;
 			getter = delegate () { return Value; };
@@ -280,7 +280,7 @@ namespace Mono.Xml.Schema
 			}
 		}
 
-		public IDictionary GetNamespacesInScope (XmlNamespaceScope scope)
+		public IDictionary<string, string> GetNamespacesInScope (XmlNamespaceScope scope)
 		{
 			IXmlNamespaceResolver resolver = reader as IXmlNamespaceResolver;
 			if (resolver == null)
@@ -794,7 +794,7 @@ namespace Mono.Xml.Schema
 				defaultAttributes = (XsAttr [])
 					defaultAttributesCache.ToArray (
 					typeof (XsAttr));
-				v.ValidateEndOfAttributes ();
+				v.ValidateEndOfAttributes (xsinfo);
 				defaultAttributesCache.Clear ();
 
 				if (reader.IsEmptyElement)
