@@ -29,6 +29,9 @@
 //
 using System;
 using System.Collections;
+#if NET_2_0
+using System.Collections.Generic;
+#endif
 using System.Collections.Specialized;
 using System.IO;
 using System.Text;
@@ -55,7 +58,7 @@ using XsDatatype = System.Xml.Schema.XmlSchemaDatatype;
 
 namespace Mono.Xml.Schema
 {
-	internal class XsdValidatingReader : XmlReader, IXmlLineInfo, IHasXmlSchemaInfo, IHasXmlParserContext, IXmlNamespaceResolver
+	internal class XsdValidatingReader : XmlReader, IXmlLineInfo, IHasXmlSchemaInfo, IHasXmlParserContext
 	{
 		static readonly XsAttribute [] emptyAttributeArray =
 			new XsAttribute [0];
@@ -67,6 +70,7 @@ namespace Mono.Xml.Schema
 		ValidationType validationType;
 		XmlSchemaSet schemas = new XmlSchemaSet ();
 		bool namespaces = true;
+		bool validationStarted;
 
 #region ID Constraints
 		bool checkIdentity = true;
@@ -144,7 +148,7 @@ namespace Mono.Xml.Schema
 		public XmlSchemaSet Schemas {
 			get { return schemas; }
 			set {
-				if (ReadState != ReadState.Initial)
+				if (validationStarted)
 					throw new InvalidOperationException ("Schemas must be set before the first call to Read().");
 				schemas = value;
 			}
@@ -186,38 +190,17 @@ namespace Mono.Xml.Schema
 		public ValidationType ValidationType {
 			get { return validationType; }
 			set {
-				if (ReadState != ReadState.Initial)
+				if (validationStarted)
 					throw new InvalidOperationException ("ValidationType must be set before reading.");
 				validationType = value;
 			}
 		}
 
-		IDictionary IXmlNamespaceResolver.GetNamespacesInScope (XmlNamespaceScope scope)
-		{
-			IXmlNamespaceResolver resolver = reader as IXmlNamespaceResolver;
-			if (resolver == null)
-				throw new NotSupportedException ("The input XmlReader does not implement IXmlNamespaceResolver and thus this validating reader cannot collect in-scope namespaces.");
-			return resolver.GetNamespacesInScope (scope);
-		}
-
-		string IXmlNamespaceResolver.LookupPrefix (string ns)
-		{
-			IXmlNamespaceResolver resolver = reader as IXmlNamespaceResolver;
-			if (resolver == null)
-				throw new NotSupportedException ("The input XmlReader does not implement IXmlNamespaceResolver and thus this validating reader cannot execute namespace prefix lookup.");
-			return resolver.LookupPrefix (ns);
-		}
-
 		// It is used only for independent XmlReader use, not for XmlValidatingReader.
-#if NET_2_0
-		[Obsolete]
-		public override object ReadTypedValue ()
-#else
 		public object ReadTypedValue ()
-#endif
 		{
 			object o = XmlSchemaUtil.ReadTypedValue (this,
-				SchemaType, ParserContext.NamespaceManager,
+				SchemaType, NamespaceManager,
 				storedCharacters);
 			storedCharacters.Length = 0;
 			return o;
@@ -354,6 +337,10 @@ namespace Mono.Xml.Schema
 			get { return XmlSchemaUtil.GetParserContext (reader); }
 		}
 
+		internal XmlNamespaceManager NamespaceManager {
+			get { return ParserContext != null ? ParserContext.NamespaceManager : null; }
+		}
+
 		public override string Prefix {
 			get {
 				if (currentDefaultAttribute < 0)
@@ -361,7 +348,7 @@ namespace Mono.Xml.Schema
 				if (defaultAttributeConsumed)
 					return String.Empty;
 				QName qname = defaultAttributes [currentDefaultAttribute].QualifiedName;
-				string prefix = this.ParserContext.NamespaceManager.LookupPrefix (qname.Namespace, false);
+				string prefix = NamespaceManager != null ? NamespaceManager.LookupPrefix (qname.Namespace, false) : null;
 				if (prefix == null)
 					return String.Empty;
 				else
@@ -584,7 +571,7 @@ namespace Mono.Xml.Schema
 						// validate against ValidatedItemType
 						if (itemDatatype != null) {
 							try {
-								itemDatatype.ParseValue (each, NameTable, ParserContext.NamespaceManager);
+								itemDatatype.ParseValue (each, NameTable, NamespaceManager);
 							} catch (Exception ex) { // FIXME: (wishlist) better exception handling ;-(
 								HandleError ("List type value contains one or more invalid values.", ex);
 								break;
@@ -605,7 +592,7 @@ namespace Mono.Xml.Schema
 							itemSimpleType = eachType as SimpleType;
 							if (itemDatatype != null) {
 								try {
-									itemDatatype.ParseValue (each, NameTable, ParserContext.NamespaceManager);
+									itemDatatype.ParseValue (each, NameTable, NamespaceManager);
 								} catch (Exception) { // FIXME: (wishlist) better exception handling ;-(
 									continue;
 								}
@@ -650,7 +637,7 @@ namespace Mono.Xml.Schema
 			}
 			if (validatedDatatype != null) {
 				try {
-					validatedDatatype.ParseValue (value, NameTable, ParserContext.NamespaceManager);
+					validatedDatatype.ParseValue (value, NameTable, NamespaceManager);
 				} catch (Exception ex) {	// FIXME: (wishlist) It is bad manner ;-(
 					HandleError ("Invalidly typed data was specified.", ex);
 				}
@@ -968,13 +955,13 @@ namespace Mono.Xml.Schema
 				string normalized = dt.Normalize (reader.Value);
 				object parsedValue = null;
 				try {
-					parsedValue = dt.ParseValue (normalized, reader.NameTable, this.ParserContext.NamespaceManager);
+					parsedValue = dt.ParseValue (normalized, reader.NameTable, NamespaceManager);
 				} catch (Exception ex) { // FIXME: (wishlist) It is bad manner ;-(
 					HandleError ("Attribute value is invalid against its data type " + dt.TokenizedType, ex);
 				}
 				if (attr.ValidatedFixedValue != null && attr.ValidatedFixedValue != normalized) {
 					HandleError ("The value of the attribute " + attr.QualifiedName + " does not match with its fixed value.");
-					parsedValue = dt.ParseValue (attr.ValidatedFixedValue, reader.NameTable, this.ParserContext.NamespaceManager);
+					parsedValue = dt.ParseValue (attr.ValidatedFixedValue, reader.NameTable, NamespaceManager);
 				}
 #region ID Constraints
 				if (this.checkIdentity) {
@@ -1103,7 +1090,7 @@ namespace Mono.Xml.Schema
 		private void ProcessKeyEntry (XsdKeyEntry entry)
 		{
 			bool isNil = XsiNilDepth == Depth;
-			entry.ProcessMatch (false, elementQNameStack, this, NameTable, BaseURI, SchemaType, ParserContext.NamespaceManager, readerLineInfo, Depth, null, null, null, isNil, CurrentKeyFieldConsumers);
+			entry.ProcessMatch (false, elementQNameStack, this, NameTable, BaseURI, SchemaType, NamespaceManager, readerLineInfo, Depth, null, null, null, isNil, CurrentKeyFieldConsumers);
 			if (MoveToFirstAttribute ()) {
 				try {
 					do {
@@ -1112,7 +1099,16 @@ namespace Mono.Xml.Schema
 						case XmlSchema.InstanceNamespace:
 							continue;
 						}
-						entry.ProcessMatch (true, elementQNameStack, this, NameTable, BaseURI, SchemaType, ParserContext.NamespaceManager, readerLineInfo, Depth, LocalName, NamespaceURI, Value, false, CurrentKeyFieldConsumers);
+						XmlSchemaDatatype dt = SchemaType as XmlSchemaDatatype;
+						XmlSchemaSimpleType st = SchemaType as XmlSchemaSimpleType;
+						if (dt == null && st != null)
+							dt = st.Datatype;
+						object identity = null;
+						if (dt != null)
+							identity = dt.ParseValue (Value, NameTable, NamespaceManager);
+						if (identity == null)
+							identity = Value;
+						entry.ProcessMatch (true, elementQNameStack, this, NameTable, BaseURI, SchemaType, NamespaceManager, readerLineInfo, Depth, LocalName, NamespaceURI, identity, false, CurrentKeyFieldConsumers);
 					} while (MoveToNextAttribute ());
 				} finally {
 					MoveToElement ();
@@ -1140,7 +1136,7 @@ namespace Mono.Xml.Schema
 					object identity = null; // This means empty value
 					if (dt != null) {
 						try {
-							identity = dt.ParseValue (value, NameTable, ParserContext.NamespaceManager);
+							identity = dt.ParseValue (value, NameTable, NamespaceManager);
 						} catch (Exception ex) { // FIXME: (wishlist) This is bad manner ;-(
 							HandleError ("Identity value is invalid against its data type " + dt.TokenizedType, ex);
 						}
@@ -1484,16 +1480,20 @@ namespace Mono.Xml.Schema
 					schemaLocation = XsDatatype.FromName ("token", XmlSchema.Namespace).Normalize (schemaLocation);
 					tmp = schemaLocation.Split (XmlChar.WhitespaceChars);
 				} catch (Exception ex) {
-					HandleError ("Invalid schemaLocation attribute format.", ex, true);
+					if (schemas.Count == 0)
+						HandleError ("Invalid schemaLocation attribute format.", ex, true);
 					tmp = new string [0];
 				}
 				if (tmp.Length % 2 != 0)
-					HandleError ("Invalid schemaLocation attribute format.");
+					if (schemas.Count == 0)
+						HandleError ("Invalid schemaLocation attribute format.");
 				for (int i = 0; i < tmp.Length; i += 2) {
 					try {
 						schema = ReadExternalSchema (tmp [i + 1]);
 					} catch (Exception) { // FIXME: (wishlist) It is bad manner ;-(
-						HandleError ("Could not resolve schema location URI: " + tmp [i + 1], null, true);
+						if (!schemas.Contains (tmp [i]))
+							HandleError (String.Format ("Could not resolve schema location URI: {0}",
+								i + 1 < tmp.Length ? tmp [i + 1] : String.Empty), null, true);
 						continue;
 					}
 					if (schema.TargetNamespace == null)
@@ -1514,7 +1514,8 @@ namespace Mono.Xml.Schema
 				try {
 					schema = ReadExternalSchema (noNsSchemaLocation);
 				} catch (Exception) { // FIXME: (wishlist) It is bad manner ;-(
-					HandleError ("Could not resolve schema location URI: " + noNsSchemaLocation, null, true);
+					if (schemas.Count != 0)
+						HandleError ("Could not resolve schema location URI: " + noNsSchemaLocation, null, true);
 				}
 				if (schema != null && schema.TargetNamespace != null)
 					HandleError ("Specified schema has different target namespace.");
@@ -1532,6 +1533,7 @@ namespace Mono.Xml.Schema
 
 		public override bool Read ()
 		{
+			validationStarted = true;
 			currentDefaultAttribute = -1;
 			defaultAttributeConsumed = false;
 			currentAttrType = null;
@@ -1771,11 +1773,6 @@ namespace Mono.Xml.Schema
 			return null;
 		}
 
-		public object FindID (string name)
-		{
-			return idList [name];
-		}
-
 		public bool HasMissingIDReferences ()
 		{
 			return missingIDReferences != null
