@@ -49,69 +49,13 @@ namespace Mono.Xml.XPath
 {
 	internal class XPathEditableDocument : IXPathNavigable
 	{
-		/*
-		public static void Main ()
-		{
-			try {
-#if true
-				XmlDocument doc = new XmlDocument ();
-				XPathEditableDocument pd = new XPathEditableDocument (doc);
-				XPathNavigator nav = pd.CreateNavigator ();
-				IChangeTracking xp = pd;
-#else
-				XPathDocument doc = new XPathDocument ();
-				XPathNavigator nav = doc.CreateNavigator ();
-				IChangeTracking xp = doc;
-#endif
-				doc.LoadXml ("<root/>");
-				nav.MoveToFirstChild (); // root
-				XmlWriter w = nav.AppendChild ();
-				Console.WriteLine (((IChangeTracking) xp).IsChanged);
-				w.WriteElementString ("foo", "foo_text");
-				w.WriteElementString ("bar", "bar_text");
-				w.WriteStartElement ("hoge");
-				w.WriteAttributeString ("fuga", "fugafuga");
-				w.WriteAttributeString ("unya", "unyaunya");
-				w.WriteFullEndElement ();
-				w.Close ();
-
-				w = nav.CreateAttributes ();
-				w.WriteStartAttribute ("namara");
-				w.WriteString ("mokera");
-				w.WriteEndAttribute ();
-				w.WriteAttributeString ("beccho", "giccho");
-				w.Close ();
-
-				nav.MoveToRoot ();
-				nav.MoveToFirstChild ();
-				nav.MoveToFirstChild ();
-				nav.DeleteSelf (); // delete foo
-				Console.WriteLine (nav.Name);
-				nav.MoveToNext ();
-				Console.WriteLine (nav.Name);
-				Console.WriteLine (nav.MoveToFirstAttribute ());
-				nav.DeleteSelf (); // delete fuga
-
-				doc.Save (Console.Out);
-			} catch (Exception ex) {
-				Console.WriteLine (ex);
-			}
-		}
-		*/
-
 		XmlNode node;
 
-		ArrayList changes = new ArrayList ();
-
 		public XPathEditableDocument (XmlNode node)
 		{
 			this.node = node;
 		}
 
-		public virtual bool CanEdit {
-			get { return true; }
-		}
-
 		public XmlNode Node {
 			get { return node; }
 		}
@@ -120,136 +64,34 @@ namespace Mono.Xml.XPath
 		{
 			return new XmlDocumentEditableNavigator (this);
 		}
-
-		public XmlWriter CreateWriter ()
-		{
-			return CreateNavigator ().AppendChild ();
-		}
-
-		public bool HasChanges ()
-		{
-			return IsChanged;
-		}
-
-		#region IRevertibleChangeTracking/IChangeTracking
-		public bool IsChanged {
-			get { return changes.Count != 0; }
-		}
-
-		public void AcceptChanges ()
-		{
-			changes.Clear ();
-		}
-
-		public void RejectChanges ()
-		{
-			for (int i = changes.Count - 1; i >= 0; i--) {
-				Insertion ins = changes [i] as Insertion;
-				if (ins != null) {
-					ins.ParentNode.RemoveChild (ins.InsertedNode);
-					continue;
-				}
-				
-				Removal rem = changes [i] as Removal;
-				if (rem != null) {
-					if (rem.RemovedNode.NodeType == XmlNodeType.Attribute) {
-						XmlElement el = (XmlElement) rem.OwnerNode;
-						el.SetAttributeNode ((XmlAttribute) rem.RemovedNode);
-					}
-					else
-						rem.OwnerNode.InsertBefore (rem.RemovedNode, rem.NextSibling);
-					continue;
-				}
-				AttributeUpdate au = changes [i] as AttributeUpdate;
-				if (au != null) {
-					if (au.OldAttribute != null)
-						au.Element.SetAttributeNode (au.OldAttribute);
-					else
-						au.Element.RemoveAttributeNode (au.NewAttribute);
-					continue;
-				}
-			}
-			changes.Clear ();
-		}
-		#endregion
-
-		#region IXmlSerializable
-		public void WriteXml (XmlWriter writer)
-		{
-			throw new NotImplementedException ();
-		}
-
-		public void ReadXml (XmlReader reader)
-		{
-			throw new NotImplementedException ();
-		}
-
-		public XmlSchema GetSchema ()
-		{
-			throw new NotImplementedException ();
-		}
-		#endregion
-
-		internal bool DeleteNode (XmlNode node)
-		{
-			Removal rem = new Removal ();
-			if (node.NodeType == XmlNodeType.Attribute) {
-				XmlAttribute attr = node as XmlAttribute;
-				rem.OwnerNode = attr.OwnerElement;
-				rem.RemovedNode = node;
-				attr.OwnerElement.RemoveAttributeNode (attr);
-				return false;
-			} else {
-				rem.OwnerNode = node.ParentNode;
-				rem.NextSibling = node.NextSibling;
-				rem.RemovedNode = node;
-				node.ParentNode.RemoveChild (node);
-				return rem.NextSibling != null;
-			}
-		}
-
-		internal XmlWriter CreateInsertionWriter (XmlNode owner, XmlNode previousSibling)
-		{
-			return new XmlDocumentInsertionWriter (owner, previousSibling, this);
-		}
-
-		internal XmlWriter CreateAttributesWriter (XmlNode owner)
-		{
-			return new XmlDocumentAttributeWriter (owner, this);
-		}
-
-		internal void AttributeUpdate (XmlElement element, XmlAttribute oldAttr, XmlAttribute newAttr)
-		{
-			AttributeUpdate au = new AttributeUpdate ();
-			au.Element = element;
-			au.OldAttribute = oldAttr;
-			au.NewAttribute = newAttr;
-			changes.Add (au);
-		}
-		
-		internal void AppendChild (XmlNode parent, XmlNode child)
-		{
-			Insertion ins = new Insertion ();
-			ins.ParentNode = parent;
-			ins.InsertedNode = child;
-			changes.Add (ins);
-		}
 	}
 
+	internal delegate void XmlWriterClosedEventHandler (
+		XmlWriter writer);
+
 	internal class XmlDocumentInsertionWriter : XmlWriter
 	{
+		XmlNode parent;
 		XmlNode current;
-		XmlNode previousSibling;
-		XPathEditableDocument document;
+		XmlNode nextSibling;
 		Stack nodeStack = new Stack ();
 
-		public XmlDocumentInsertionWriter (XmlNode owner, XmlNode previousSibling, XPathEditableDocument doc)
+		public XmlDocumentInsertionWriter (XmlNode owner, XmlNode nextSibling)
 		{
-			this.current = (XmlNode) owner;
-			if (current == null)
+			this.parent = (XmlNode) owner;
+			if (parent == null)
 				throw new InvalidOperationException ();
-			this.previousSibling = previousSibling;
-			this.document = doc;
+			switch (parent.NodeType) {
+			case XmlNodeType.Document:
+				current = ((XmlDocument) parent).CreateDocumentFragment ();
+				break;
+			case XmlNodeType.Element:
+				current = parent.OwnerDocument.CreateDocumentFragment ();
+				break;
+			default:
+				throw new InvalidOperationException (String.Format ("Insertion into {0} node is not allowed.", parent.NodeType));
+			}
+			this.nextSibling = nextSibling;
 			state = WriteState.Content;
 		}
 
@@ -262,8 +104,20 @@ namespace Mono.Xml.XPath
 
 		public override void Close ()
 		{
+			while (nodeStack.Count > 0) {
+				XmlNode n = nodeStack.Pop () as XmlNode;
+				n.AppendChild (current);
+				current = n;
+			}
+			parent.InsertBefore ((XmlDocumentFragment) current, nextSibling);
+			if (Closed != null)
+				Closed (this);
 		}
 
+		internal event XmlWriterClosedEventHandler Closed;
+
+		internal XmlNode AppendedFirstChild;
+
 		public override void Flush ()
 		{
 		}
@@ -285,33 +139,24 @@ namespace Mono.Xml.XPath
 		{
 			XmlProcessingInstruction pi = current.OwnerDocument.CreateProcessingInstruction (name, value);
 			current.AppendChild (pi);
-			document.AppendChild (current, pi);
 		}
 
 		public override void WriteComment (string text)
 		{
 			XmlComment comment = current.OwnerDocument.CreateComment (text);
 			current.AppendChild (comment);
-			document.AppendChild (current, comment);
 		}
 
 		public override void WriteCData (string text)
 		{
 			XmlCDataSection cdata = current.OwnerDocument.CreateCDataSection (text);
 			current.AppendChild (cdata);
-			document.AppendChild (current, cdata);
 		}
 
 		public override void WriteStartElement (string prefix, string name, string ns)
 		{
-			XmlDocument doc = current.OwnerDocument;
-			if (doc == null)
-				doc = current as XmlDocument;
-			if (doc == null)
-				throw new SystemException ("Should not happen.");
-			XmlElement el = doc.CreateElement (prefix, name, ns);
+			XmlElement el = current.OwnerDocument.CreateElement (prefix, name, ns);
 			current.AppendChild (el);
-			document.AppendChild (current, el);
 			nodeStack.Push (current);
 			current = el;
 		}
@@ -325,10 +170,10 @@ namespace Mono.Xml.XPath
 
 		public override void WriteFullEndElement ()
 		{
-			WriteEndElement ();
 			XmlElement el = current as XmlElement;
 			if (el != null)
 				el.IsEmpty = false;
+			WriteEndElement ();
 		}
 
 		public override void WriteDocType (string name, string pubid, string systemId, string intsubset)
@@ -395,7 +240,6 @@ namespace Mono.Xml.XPath
 			else {
 				XmlText t = current.OwnerDocument.CreateTextNode (text);
 				current.AppendChild (t);
-				document.AppendChild (current, t);
 			}
 		}
 
@@ -414,7 +258,7 @@ namespace Mono.Xml.XPath
 			XmlElement element = current as XmlElement;
 			if (state != WriteState.Attribute || element == null)
 				throw new InvalidOperationException ("Current state is not inside attribute. Cannot close attribute.");
-			document.AttributeUpdate (element, element.SetAttributeNode (attribute), attribute);
+			element.SetAttributeNode (attribute);
 			attribute = null;
 			state = WriteState.Content;
 		}
@@ -423,15 +267,13 @@ namespace Mono.Xml.XPath
 	internal class XmlDocumentAttributeWriter : XmlWriter
 	{
 		XmlElement element;
-		XPathEditableDocument document;
 
-		public XmlDocumentAttributeWriter (XmlNode owner, XPathEditableDocument doc)
+		public XmlDocumentAttributeWriter (XmlNode owner)
 		{
 			element = owner as XmlElement;
 			if (element == null)
 				throw new ArgumentException ("To write attributes, current node must be an element.");
 			state = WriteState.Content;
-			document = doc;
 		}
 
 		WriteState state;
@@ -570,37 +412,12 @@ namespace Mono.Xml.XPath
 		{
 			if (state != WriteState.Attribute)
 				throw new InvalidOperationException ("Current state is not inside attribute. Cannot close attribute.");
-			document.AttributeUpdate (element, element.SetAttributeNode (attribute), attribute);
+			element.SetAttributeNode (attribute);
 			attribute = null;
 			state = WriteState.Content;
 		}
 	}
 
-	internal class Insertion
-	{
-		// AppendChild : last child / true
-		// InsertBefore : current node / false
-		// InsertAfter : current node / true
-		// PrependChild : first child / false
-		public XmlNode ParentNode;
-		public XmlNode InsertedNode;
-		public bool Afterward;
-	}
-
-	internal class Removal
-	{
-		public XmlNode OwnerNode;
-		public XmlNode NextSibling;
-		public XmlNode RemovedNode;
-	}
-
-	internal class AttributeUpdate
-	{
-		public XmlElement Element;
-		public XmlAttribute NewAttribute;
-		public XmlAttribute OldAttribute;
-	}
-
 	internal class XmlDocumentEditableNavigator : XPathNavigator, IHasXmlNode
 	{
 		static readonly bool isXmlDocumentNavigatorImpl;
@@ -662,6 +479,14 @@ namespace Mono.Xml.XPath
 			get { return navigator.Prefix; }
 		}
 
+		public override IXmlSchemaInfo SchemaInfo {
+			get { return navigator.SchemaInfo; }
+		}
+
+		public override object UnderlyingObject {
+			get { return navigator.UnderlyingObject; }
+		}
+
 		public override string Value {
 			get { return navigator.Value; }
 		}
@@ -749,19 +574,88 @@ namespace Mono.Xml.XPath
 			XmlNode n = ((IHasXmlNode) navigator).GetNode ();
 			if (n == null)
 				throw new InvalidOperationException ("Should not happen.");
-			return document.CreateInsertionWriter (n, null);
+			return new XmlDocumentInsertionWriter (n, null);
+		}
+
+		public override void DeleteRange (XPathNavigator lastSiblingToDelete)
+		{
+			if (lastSiblingToDelete == null)
+				throw new ArgumentNullException ();
+
+			XmlNode start = ((IHasXmlNode) navigator).GetNode ();
+			XmlNode end = null;
+			if (lastSiblingToDelete is IHasXmlNode)
+				end = ((IHasXmlNode) lastSiblingToDelete).GetNode ();
+			// After removal, it moves to parent node.
+			if (!navigator.MoveToParent ())
+				throw new InvalidOperationException ("There is no parent to remove current node.");
+
+			if (end == null || start.ParentNode != end.ParentNode)
+				throw new InvalidOperationException ("Argument XPathNavigator has different parent node.");
+
+			XmlNode parent = start.ParentNode;
+			XmlNode next;
+			bool loop = true;
+			for (XmlNode n = start; loop; n = next) {
+				loop = n != end;
+				next = n.NextSibling;
+				parent.RemoveChild (n);
+			}
+		}
+
+		public override XmlWriter ReplaceRange (XPathNavigator nav)
+		{
+			if (nav == null)
+				throw new ArgumentNullException ();
+
+			XmlNode start = ((IHasXmlNode) navigator).GetNode ();
+			XmlNode end = null;
+			if (nav is IHasXmlNode)
+				end = ((IHasXmlNode) nav).GetNode ();
+			if (end == null || start.ParentNode != end.ParentNode)
+				throw new InvalidOperationException ("Argument XPathNavigator has different parent node.");
+
+			XmlDocumentInsertionWriter w =
+				(XmlDocumentInsertionWriter) InsertBefore ();
+
+			// local variables to anonymous delegate
+			XPathNavigator prev = Clone ();
+			if (!prev.MoveToPrevious ())
+				prev = null;
+			XPathNavigator parentNav = Clone ();
+			parentNav.MoveToParent ();
+
+			w.Closed += delegate (XmlWriter w) {
+				XmlNode parent = start.ParentNode;
+				XmlNode next;
+				bool loop = true;
+				for (XmlNode n = start; loop; n = next) {
+					loop = n != end;
+					next = n.NextSibling;
+					parent.RemoveChild (n);
+				}
+				if (prev != null) {
+					MoveTo (prev);
+					MoveToNext ();
+				} else {
+					MoveTo (parentNav);
+					MoveToFirstChild ();
+				}
+			};
+
+			return w;
 		}
 
 		public override XmlWriter InsertBefore ()
 		{
 			XmlNode n = ((IHasXmlNode) navigator).GetNode ();
-			return document.CreateInsertionWriter (n.ParentNode, n.PreviousSibling);
+			return new XmlDocumentInsertionWriter (n.ParentNode, n);
 		}
 
 		public override XmlWriter CreateAttributes ()
 		{
 			XmlNode n = ((IHasXmlNode) navigator).GetNode ();
-			return document.CreateInsertionWriter (n, null);
+			return new XmlDocumentAttributeWriter (n);
 		}
 
 		public override void DeleteSelf ()
@@ -769,17 +663,57 @@ namespace Mono.Xml.XPath
 			XmlNode n = ((IHasXmlNode) navigator).GetNode ();
 			if (!navigator.MoveToNext ())
 				navigator.MoveToParent ();
-			document.DeleteNode (n);
+			if (n.ParentNode == null)
+				throw new InvalidOperationException ("This node cannot be removed since it has no parent.");
+			n.ParentNode.RemoveChild (n);
+		}
+
+		public override void ReplaceSelf (XmlReader reader)
+		{
+			XmlNode n = ((IHasXmlNode) navigator).GetNode ();
+			XmlNode p = n.ParentNode;
+			if (p == null)
+				throw new InvalidOperationException ("This node cannot be removed since it has no parent.");
+
+			bool movenext = false;
+			if (!MoveToPrevious ())
+				MoveToParent ();
+			else
+				movenext = true;
+
+			XmlDocument doc = p.NodeType == XmlNodeType.Document ?
+				p as XmlDocument : p.OwnerDocument;
+			bool error = false;
+			if (reader.ReadState == ReadState.Initial) {
+				reader.Read ();
+				if (reader.EOF)
+					error = true;
+				else
+					while (!reader.EOF)
+						p.AppendChild (doc.ReadNode (reader));
+			} else {
+				if (reader.EOF)
+					error = true;
+				else
+					p.AppendChild (doc.ReadNode (reader));
+			}
+			if (error)
+				throw new InvalidOperationException ("Content is required in argument XmlReader to replace current node.");
+
+			p.RemoveChild (n);
+
+			if (movenext)
+				MoveToNext ();
+			else
+				MoveToFirstChild ();
 		}
 
 		public override void SetValue (string value)
 		{
 			XmlNode n = ((IHasXmlNode) navigator).GetNode ();
-			foreach (XmlNode c in n.ChildNodes)
-				document.DeleteNode (c);
-			XmlWriter w = document.CreateInsertionWriter (n, null);
-			w.WriteValue (value);
-			w.Close ();
+			while (n.FirstChild != null)
+				n.RemoveChild (n.FirstChild);
+			n.InnerText = value;
 		}
 	}
 }
