@@ -137,7 +137,8 @@ namespace Mono.Xml.Xsl
 				this.res = new XmlUrlResolver ();
 			this.evidence = evidence;
 
-			if (!nav.MoveToFirstChild ())
+			// reject empty document.
+			if (nav.NodeType == XPathNodeType.Root && !nav.MoveToFirstChild ())
 				throw new XsltCompileException ("Stylesheet root element must be either \"stylesheet\" or \"transform\" or any literal element.", null, nav);
 				
 			outputs [""] = new XslOutput ("");
@@ -841,8 +842,9 @@ namespace Mono.Xml.Xsl
 			if (colon > 0)
 				return new QName (name.Substring (colon + 1), nsDecls [name.Substring (0, colon)] as string);
 			else if (colon < 0)
-				// Default namespace is not used for unprefixed names.
-				return new QName (name, "");
+				return new QName (name,
+					nsDecls.ContainsKey (String.Empty) ?
+					(string) nsDecls [String.Empty] : String.Empty);
 			else
 				throw new ArgumentException ("Invalid name: " + name);
 		}
