@@ -119,8 +119,8 @@ namespace System.Xml.Schema
 		
 		#region Elements
 				
-		[XmlElement("simpleContent",typeof(XmlSchemaSimpleContent),Namespace="http://www.w3.org/2001/XMLSchema")]
-		[XmlElement("complexContent",typeof(XmlSchemaComplexContent),Namespace="http://www.w3.org/2001/XMLSchema")]
+		[XmlElement("simpleContent",typeof(XmlSchemaSimpleContent))]
+		[XmlElement("complexContent",typeof(XmlSchemaComplexContent))]
 		public XmlSchemaContentModel ContentModel 
 		{
 			get{ return  contentModel; } 
@@ -128,24 +128,24 @@ namespace System.Xml.Schema
 		}
 
 		//LAMESPEC: The default value for particle in Schema is of Type EmptyParticle (internal?)
-		[XmlElement("group",typeof(XmlSchemaGroupRef),Namespace="http://www.w3.org/2001/XMLSchema")]
-		[XmlElement("all",typeof(XmlSchemaAll),Namespace="http://www.w3.org/2001/XMLSchema")]
-		[XmlElement("choice",typeof(XmlSchemaChoice),Namespace="http://www.w3.org/2001/XMLSchema")]
-		[XmlElement("sequence",typeof(XmlSchemaSequence),Namespace="http://www.w3.org/2001/XMLSchema")]
+		[XmlElement("group",typeof(XmlSchemaGroupRef))]
+		[XmlElement("all",typeof(XmlSchemaAll))]
+		[XmlElement("choice",typeof(XmlSchemaChoice))]
+		[XmlElement("sequence",typeof(XmlSchemaSequence))]
 		public XmlSchemaParticle Particle 
 		{
 			get{ return  particle; } 
 			set{ particle = value; }
 		}
 
-		[XmlElement("attribute",typeof(XmlSchemaAttribute),Namespace="http://www.w3.org/2001/XMLSchema")]
-		[XmlElement("attributeGroup",typeof(XmlSchemaAttributeGroupRef),Namespace="http://www.w3.org/2001/XMLSchema")]
+		[XmlElement("attribute",typeof(XmlSchemaAttribute))]
+		[XmlElement("attributeGroup",typeof(XmlSchemaAttributeGroupRef))]
 		public XmlSchemaObjectCollection Attributes 
 		{
 			get{ return attributes; }
 		}
 
-		[XmlElement("anyAttribute",Namespace="http://www.w3.org/2001/XMLSchema")]
+		[XmlElement("anyAttribute")]
 		public XmlSchemaAnyAttribute AnyAttribute 
 		{
 			get{ return  anyAttribute; }
@@ -599,6 +599,11 @@ namespace System.Xml.Schema
 					error (h, "Particle whose term is -all- and consists of complex type content particle must have maxOccurs = 1.");
 			}
 
+#if NET_2_0
+			if (schema.Schemas.CompilationSettings != null &&
+				schema.Schemas.CompilationSettings.EnableUpaCheck)
+#endif
+			// This check is optional only after 2.0
 			contentTypeParticle.ValidateUniqueParticleAttribution (new XmlSchemaObjectTable (),
 				new ArrayList (), h, schema);
 			contentTypeParticle.ValidateUniqueTypeAttribution (
@@ -644,7 +649,7 @@ namespace System.Xml.Schema
 			// also checks 3.4.6 Properties Correct :: 4 (Distinct attributes)
 			attributeUses = new XmlSchemaObjectTable ();
 			XmlSchemaUtil.ValidateAttributesResolved (attributeUses,
-				h, schema, attributes, anyAttribute, ref attributeWildcard, null);
+				h, schema, attributes, anyAttribute, ref attributeWildcard, null, false);
 		}
 
 		private void ValidateContentModel (ValidationEventHandler h, XmlSchema schema)
@@ -754,7 +759,7 @@ namespace System.Xml.Schema
 				// attributes
 				errorCount += XmlSchemaUtil.ValidateAttributesResolved (
 					this.attributeUses, h, schema, cce.Attributes, 
-					cce.AnyAttribute , ref attributeWildcard, null);
+					cce.AnyAttribute , ref attributeWildcard, null, true);
 
 				// After adding them, test extension validity.
 				if (baseComplexType != null)
@@ -783,7 +788,7 @@ namespace System.Xml.Schema
 				// and its {attribute uses} and {attribute wildcard} are done here (descendantly)
 				errorCount += XmlSchemaUtil.ValidateAttributesResolved (
 					this.attributeUses, h, schema, ccr.Attributes, 
-					ccr.AnyAttribute, ref attributeWildcard, null);
+					ccr.AnyAttribute, ref attributeWildcard, null, false);
 				foreach (DictionaryEntry entry in baseComplexType.AttributeUses) {
 					XmlSchemaAttribute attr = (XmlSchemaAttribute) entry.Value;
 					if (attributeUses [attr.QualifiedName] == null)
@@ -799,7 +804,7 @@ namespace System.Xml.Schema
 			if (sce != null) {
 				errorCount += XmlSchemaUtil.ValidateAttributesResolved (
 					this.attributeUses, h, schema, sce.Attributes, 
-					sce.AnyAttribute, ref attributeWildcard, null);
+					sce.AnyAttribute, ref attributeWildcard, null, true);
 
 				// Attributes
 				// I don't think 3.4.6 Derivation Valid (Extension) :: 1.2
@@ -839,7 +844,7 @@ namespace System.Xml.Schema
 				// and its {attribute uses} and {attribute wildcard} are done here (descendantly)
 				errorCount += XmlSchemaUtil.ValidateAttributesResolved (
 					this.attributeUses, h, schema, scr.Attributes, 
-					scr.AnyAttribute, ref attributeWildcard, null);
+					scr.AnyAttribute, ref attributeWildcard, null, false);
 			}
 
 			// Common process of AttributeWildcard.
@@ -862,9 +867,12 @@ namespace System.Xml.Schema
 				return;
 			if (bst != null && (resolvedDerivedBy & bst.FinalResolved) != 0) // 1.
 				error (h, "Derivation type " + resolvedDerivedBy + " is prohibited by the base type.");
-			if (BaseXmlSchemaType == b) // 2.2
+			// FIXME: here BaseSchemaType should be 
+			// BaseXmlSchemaType, however for some case it 
+			// seems not working.
+			if (BaseSchemaType == b) // 2.2
 				return;
-			if (BaseXmlSchemaType == null || BaseXmlSchemaType == XmlSchemaComplexType.AnyType) { // 2.3.1
+			if (BaseSchemaType == null || BaseXmlSchemaType == XmlSchemaComplexType.AnyType) { // 2.3.1
 				error (h, "Derived type's base schema type is anyType.");
 				return;
 			}
