@@ -516,6 +516,7 @@ namespace System.Xml.Serialization
 			if (emap.IsFlags)
 			{
 				WriteLineInd ("default:");
+				WriteLine ("if (val.ToString () == \"0\") return string.Empty;");
 				WriteLine ("System.Text.StringBuilder sb = new System.Text.StringBuilder ();");
 				WriteLine ("string[] enumNames = val.ToString().Split (',');");
 				WriteLine ("foreach (string name in enumNames)");
@@ -565,7 +566,7 @@ namespace System.Xml.Serialization
 			
 			if (!typeMap.TypeData.IsValueType)
 			{
-				WriteLine ("if (ob == null)");
+				WriteLine ("if (((object)ob) == null)");
 				WriteLineInd ("{");
 				WriteLineInd ("if (isNullable)");
 				
@@ -743,7 +744,7 @@ namespace System.Xml.Serialization
 						else if (memType == typeof(XmlTypeMapMemberFlatList))
 						{
 							WriteLineInd ("if (" + memberValue + " != null) {"); 
-							GenerateWriteListContent (member.TypeData, ((XmlTypeMapMemberFlatList)member).ListMap, memberValue, false);
+							GenerateWriteListContent (ob, member.TypeData, ((XmlTypeMapMemberFlatList)member).ListMap, memberValue, false);
 							WriteLineUni ("}");
 						}
 						else if (memType == typeof(XmlTypeMapMemberAnyElement))
@@ -834,7 +835,7 @@ namespace System.Xml.Serialization
 					}
 					else {
 						WriteMetCall ("WriteStartElement", GetLiteral(elem.ElementName), GetLiteral(elem.Namespace), memberValue);
-						GenerateWriteListContent (elem.TypeData, (ListMap) elem.MappedType.ObjectMap, memberValue, false);
+						GenerateWriteListContent (null, elem.TypeData, (ListMap) elem.MappedType.ObjectMap, memberValue, false);
 						WriteMetCall ("WriteEndElement", memberValue);
 					}
 					WriteLineUni ("}");
@@ -887,7 +888,7 @@ namespace System.Xml.Serialization
 				
 				WriteMetCall ("WriteAttribute", GetLiteral("arrayType"), GetLiteral(XmlSerializer.EncodingNamespace), arrayType);
 			}
-			GenerateWriteListContent (typeMap.TypeData, (ListMap) typeMap.ObjectMap, ob, false);
+			GenerateWriteListContent (null, typeMap.TypeData, (ListMap) typeMap.ObjectMap, ob, false);
 		}
 		
 		void GenerateWriteAnyElementContent (XmlTypeMapMemberAnyElement member, string memberValue)
@@ -903,8 +904,8 @@ namespace System.Xml.Serialization
 			}
 			
 			string elem = GetObTempVar ();
-			WriteLine ("XmlElement " + elem + " = " + var + " as XmlElement;");
-			WriteLine ("if (" + elem + " == null) throw CreateUnknownTypeException (" + elem + ");");
+			WriteLine ("XmlNode " + elem + " = " + var + ";");
+			WriteLine ("if (" + elem + " is XmlElement) {");
 			
 			if (!member.IsDefaultAny) {
 				for (int n=0; n<member.ElementInfo.Count; n++) {
@@ -915,7 +916,9 @@ namespace System.Xml.Serialization
 					else WriteLine ("|| " + txt);
 				}				
 			}
-			
+			WriteLine ("}");
+			WriteLine ("else " + elem + ".WriteTo (Writer);");
+
 			if (_format == SerializationFormat.Literal) 
 				WriteLine ("WriteElementLiteral (" + elem + ", \"\", \"\", false, true);");
 			else 
@@ -950,7 +953,7 @@ namespace System.Xml.Serialization
 				string str = GetStrTempVar ();
 				WriteLine ("string " + str + " = null;");
 				WriteLineInd ("if (" + value + " != null) {");
-				string res = GenerateWriteListContent (typeMap.TypeData, (ListMap)typeMap.ObjectMap, value, true);
+				string res = GenerateWriteListContent (null, typeMap.TypeData, (ListMap)typeMap.ObjectMap, value, true);
 				WriteLine (str + " = " + res + ".ToString ().Trim ();");
 				WriteLineUni ("}");
 				return str;
@@ -1011,7 +1014,7 @@ namespace System.Xml.Serialization
 			}
 		}
 
-		string GenerateWriteListContent (TypeData listType, ListMap map, string ob, bool writeToString)
+		string GenerateWriteListContent (string container, TypeData listType, ListMap map, string ob, bool writeToString)
 		{
 			string targetString = null;
 			
@@ -1025,21 +1028,21 @@ namespace System.Xml.Serialization
 			{
 				string itemVar = GetNumTempVar ();
 				WriteLineInd ("for (int "+itemVar+" = 0; "+itemVar+" < " + ob + ".Length; "+itemVar+"++) {");
-				GenerateListLoop (map, ob + "["+itemVar+"]", listType.ListItemTypeData, targetString);
+				GenerateListLoop (container, map, ob + "["+itemVar+"]", itemVar, listType.ListItemTypeData, targetString);
 				WriteLineUni ("}");
 			}
 			else if (typeof(ICollection).IsAssignableFrom (listType.Type))
 			{
 				string itemVar = GetNumTempVar ();
 				WriteLineInd ("for (int "+itemVar+" = 0; "+itemVar+" < " + ob + ".Count; "+itemVar+"++) {");
-				GenerateListLoop (map, ob + "["+itemVar+"]", listType.ListItemTypeData, targetString);
+				GenerateListLoop (container, map, ob + "["+itemVar+"]", itemVar, listType.ListItemTypeData, targetString);
 				WriteLineUni ("}");
 			}
 			else if (typeof(IEnumerable).IsAssignableFrom (listType.Type))
 			{
 				string itemVar = GetObTempVar ();
 				WriteLineInd ("foreach (" + listType.ListItemTypeData.FullTypeName + " " + itemVar + " in " + ob + ") {");
-				GenerateListLoop (map, itemVar, listType.ListItemTypeData, targetString);
+				GenerateListLoop (container, map, itemVar, null, listType.ListItemTypeData, targetString);
 				WriteLineUni ("}");
 			}
 			else
@@ -1048,18 +1051,26 @@ namespace System.Xml.Serialization
 			return targetString;
 		}
 		
-		void GenerateListLoop (ListMap map, string item, TypeData itemTypeData, string targetString)
+		void GenerateListLoop (string container, ListMap map, string item, string index, TypeData itemTypeData, string targetString)
 		{
 			bool multichoice = (map.ItemInfo.Count > 1);
 
-			if (multichoice)
-				WriteLine ("if (" + item + " == null) { }");
+			if (map.ChoiceMember != null && container != null && index != null) {
+				WriteLineInd ("if ((" + container + ".@" + map.ChoiceMember + " == null) || (" + index + " >= " + container + ".@" + map.ChoiceMember + ".Length))");
+				WriteLine ("throw CreateInvalidChoiceIdentifierValueException (" + container + ".GetType().ToString(), \"" + map.ChoiceMember + "\");");
+				Unindent ();
+			}
 			
+			if (multichoice)
+				WriteLine ("if (((object)" + item + ") == null) { }");
+				
 			foreach (XmlTypeMapElementInfo info in map.ItemInfo)
 			{
-				if (multichoice)
+				if (map.ChoiceMember != null && multichoice)
+					WriteLineInd ("else if (" + container + ".@" + map.ChoiceMember + "[" + index + "] == " + GetLiteral (info.ChoiceValue) + ") {");
+				else if (multichoice)
 					WriteLineInd ("else if (" + item + ".GetType() == typeof(" + info.TypeData.FullTypeName + ")) {");
-
+				
 				if (targetString == null) 
 					GenerateWriteMemberElement (info, GetCast (info.TypeData, itemTypeData, item));
 				else
@@ -1268,7 +1279,7 @@ namespace System.Xml.Serialization
 					WriteLine ("throw CreateUnknownNodeException();");
 					Unindent ();
 
-					WriteLine ("return " + GetReadObjectCall (typeMap, "true", "true") + ";");
+					WriteLine ("return " + GetReadObjectCall (typeMap, GetLiteral(typeMap.IsNullable), "true") + ";");
 				}
 				else
 				{
@@ -1584,6 +1595,7 @@ namespace System.Xml.Serialization
 				WriteLine ("Reader.MoveToElement();");
 				WriteLineInd ("if (Reader.IsEmptyElement) {"); 
 				WriteLine ("Reader.Skip ();");
+				GenerateSetListMembersDefaults (typeMap, map, ob, isValueList);
 				WriteLine ("return " + ob + ";");
 				WriteLineUni ("}");
 				WriteLine ("");
@@ -1614,6 +1626,7 @@ namespace System.Xml.Serialization
 				
 				string[] indexes = null;
 				string[] flatLists = null;
+				string[] flatListsChoices = null;
 	
 				if (map.FlatLists != null) 
 				{
@@ -1623,15 +1636,35 @@ namespace System.Xml.Serialization
 					string code = "int ";
 					for (int n=0; n<map.FlatLists.Count; n++) 
 					{
-						XmlTypeMapMember mem = (XmlTypeMapMember)map.FlatLists[n];
+						XmlTypeMapMemberElement mem = (XmlTypeMapMemberElement)map.FlatLists[n];
 						indexes[n] = GetNumTempVar ();
 						if (n > 0) code += ", ";
 						code += indexes[n] + "=0";
 						if (!MemberHasReadReplaceHook (xmlMapType, mem)) {
 							flatLists[n] = GetObTempVar ();
-							string rval = "null";
-							if (IsReadOnly (typeMap, mem, mem.TypeData, isValueList)) rval = ob + ".@" + mem.Name;
-							WriteLine (mem.TypeData.FullTypeName + " " + flatLists[n] + " = " + rval + ";");
+							string rval;
+							WriteLine (mem.TypeData.FullTypeName + " " + flatLists[n] + ";");
+							if (IsReadOnly (typeMap, mem, mem.TypeData, isValueList)) {
+								rval = GenerateGetMemberValue (mem, ob, isValueList);
+								WriteLine (flatLists[n] + " = " + rval + ";");
+							} else if (mem.TypeData.Type.IsArray) {
+								rval = GenerateInitializeList (mem.TypeData);
+								WriteLine (flatLists[n] + " = " + rval + ";");
+							} else {
+								WriteLine (flatLists[n] + " = " + GenerateGetMemberValue (mem, ob, isValueList) + ";");
+								WriteLineInd ("if (((object)" + flatLists[n] + ") == null) {");
+								WriteLine (flatLists[n] + " = " + GenerateInitializeList (mem.TypeData) + ";");
+								GenerateSetMemberValue (mem, ob, flatLists[n], isValueList);
+								WriteLineUni ("}");
+							}
+						}
+						
+						if (mem.ChoiceMember != null) {
+							if (flatListsChoices == null)
+								flatListsChoices = new string [map.FlatLists.Count];
+							flatListsChoices[n] = GetObTempVar ();
+							string rval = GenerateInitializeList (mem.ChoiceTypeData);
+							WriteLine (mem.ChoiceTypeData.FullTypeName + " " + flatListsChoices[n] + " = " + rval + ";");
 						}
 					}
 					WriteLine (code + ";");
@@ -1716,8 +1749,27 @@ namespace System.Xml.Serialization
 						else
 						{
 							if (!GenerateReadMemberHook (xmlMapType, info.Member)) {
-								if (IsReadOnly (typeMap, info.Member, info.TypeData, isValueList)) GenerateReadListElement (info.MappedType, GenerateGetMemberValue (info.Member, ob, isValueList), GetLiteral(info.IsNullable), false);
-								else GenerateSetMemberValue (info.Member, ob, GenerateReadListElement (info.MappedType, null, GetLiteral(info.IsNullable), true), isValueList);
+								if (IsReadOnly (typeMap, info.Member, info.TypeData, isValueList)) {
+									GenerateReadListElement (info.MappedType, GenerateGetMemberValue (info.Member, ob, isValueList), GetLiteral(info.IsNullable), false);
+								} else if (info.MappedType.TypeData.Type.IsArray) {
+									if (info.IsNullable)
+										GenerateSetMemberValue (info.Member, ob, GenerateReadListElement (info.MappedType, null, GetLiteral(info.IsNullable), true), isValueList);
+									else {
+										string list = GetObTempVar ();
+										WriteLine (info.MappedType.TypeFullName + " " + list + " = " + GenerateReadListElement (info.MappedType, null, GetLiteral(info.IsNullable), true) + ";");
+										WriteLineInd ("if (((object)" + list + ") != null) {");
+										GenerateSetMemberValue (info.Member, ob, list, isValueList);
+										WriteLineUni ("}");
+									}
+								} else {
+									string list = GetObTempVar ();
+									WriteLine (info.MappedType.TypeFullName + " " + list + " = " + GenerateGetMemberValue (info.Member, ob, isValueList) + ";");
+									WriteLineInd ("if (((object)" + list + ") == null) {");
+									WriteLine (list + " = " + GenerateCreateList (info.MappedType.TypeData.Type) + ";");
+									GenerateSetMemberValue (info.Member, ob, list, isValueList);
+									WriteLineUni ("}");
+									GenerateReadListElement (info.MappedType, list, GetLiteral(info.IsNullable), true);
+								}
 								GenerateEndHook ();
 							}
 						}
@@ -1729,6 +1781,9 @@ namespace System.Xml.Serialization
 						XmlTypeMapMemberFlatList mem = (XmlTypeMapMemberFlatList)info.Member;
 						if (!GenerateReadArrayMemberHook (xmlMapType, info.Member, indexes[mem.FlatArrayIndex])) {
 							GenerateAddListValue (mem.TypeData, flatLists[mem.FlatArrayIndex], indexes[mem.FlatArrayIndex], GenerateReadObjectElement (info), !IsReadOnly (typeMap, info.Member, info.TypeData, isValueList));
+							if (mem.ChoiceMember != null) {
+								GenerateAddListValue (mem.ChoiceTypeData, flatListsChoices[mem.FlatArrayIndex], indexes[mem.FlatArrayIndex], GetLiteral (info.ChoiceValue), true);
+							}
 							GenerateEndHook ();
 						}
 						WriteLine (indexes[mem.FlatArrayIndex] + "++;");
@@ -1871,10 +1926,27 @@ namespace System.Xml.Serialization
 						string list = flatLists[mem.FlatArrayIndex];
 						if (mem.TypeData.Type.IsArray)
 							WriteLine (list + " = (" + mem.TypeData.FullTypeName + ") ShrinkArray (" + list + ", " + indexes[mem.FlatArrayIndex] + ", " + GetTypeOf(mem.TypeData.Type.GetElementType()) + ", true);");
-						if (!IsReadOnly (typeMap, mem, mem.TypeData, isValueList))
+						if (!IsReadOnly (typeMap, mem, mem.TypeData, isValueList) && mem.TypeData.Type.IsArray)
 							GenerateSetMemberValue (mem, ob, list, isValueList);
 					}
 				}
+				
+				if (flatListsChoices != null)
+				{
+					WriteLine ("");
+					foreach (XmlTypeMapMemberExpandable mem in map.FlatLists)
+					{
+						if (MemberHasReadReplaceHook (xmlMapType, mem)) continue;
+						if (mem.ChoiceMember == null) continue;
+						
+						string list = flatListsChoices[mem.FlatArrayIndex];
+						WriteLine (list + " = (" + mem.ChoiceTypeData.FullTypeName + ") ShrinkArray (" + list + ", " + indexes[mem.FlatArrayIndex] + ", " + GetTypeOf(mem.ChoiceTypeData.Type.GetElementType()) + ", true);");
+						WriteLine (ob + ".@" + mem.ChoiceMember + " = " + list + ";");
+					}
+				}
+				
+				GenerateSetListMembersDefaults (typeMap, map, ob, isValueList);
+				
 				GenerateEndHook ();
 			}			
 
@@ -1885,6 +1957,22 @@ namespace System.Xml.Serialization
 			}
 		}
 		
+		void GenerateSetListMembersDefaults (XmlTypeMapping typeMap, ClassMap map, string ob, bool isValueList)
+		{
+			if (map.ListMembers != null)
+			{
+				ArrayList members = map.ListMembers;
+				for (int n=0; n<members.Count; n++) {
+					XmlTypeMapMember mem = (XmlTypeMapMember) members[n];
+					if (IsReadOnly (typeMap, mem, mem.TypeData, isValueList))
+						continue;
+					WriteLineInd ("if (" + GenerateGetMemberValue (mem, ob, isValueList) + " == null) {");
+					GenerateSetMemberValue (mem, ob, GenerateInitializeList (mem.TypeData), isValueList);
+					WriteLineUni ("}");
+				}
+			}
+		}
+		
 		bool IsReadOnly (XmlTypeMapping map, XmlTypeMapMember member, TypeData memType, bool isValueList)
 		{
 			if (isValueList) return !memType.HasPublicConstructor;
@@ -1965,28 +2053,31 @@ namespace System.Xml.Serialization
 		{
 			Type listType = typeMap.TypeData.Type;
 			ListMap listMap = (ListMap)typeMap.ObjectMap;
+			bool doNullCheck = typeMap.TypeData.Type.IsArray;
 
 			if (canCreateInstance && typeMap.TypeData.HasPublicConstructor) 
 			{
-				list = GetObTempVar ();
-				WriteLine (typeMap.TypeFullName + " " + list + " = null;");
-				WriteLineInd ("if (!ReadNull()) {");
-				WriteLine (list + " = " + GenerateCreateList (listType) + ";");
+				if (list == null) {
+					list = GetObTempVar ();
+					WriteLine (typeMap.TypeFullName + " " + list + " = null;");
+					if (doNullCheck)
+						WriteLineInd ("if (!ReadNull()) {");
+					WriteLine (list + " = " + GenerateCreateList (listType) + ";");
+				} else {
+					if (doNullCheck)
+						WriteLineInd ("if (!ReadNull()) {");
+				}
 			}
 			else
 			{
 				if (list != null) {
-					WriteLineInd ("if (" + list + " == null)");
+					WriteLineInd ("if (((object)" + list + ") == null)");
 					WriteLine ("throw CreateReadOnlyCollectionException (" + GetLiteral (typeMap.TypeFullName) + ");");
 					Unindent ();
-					WriteLineInd ("if (!ReadNull()) {");
+					doNullCheck = false;
 				}
 				else {
-					list = GetObTempVar ();
-					WriteLine (typeMap.TypeFullName + " " + list + " = null;");
-					WriteLineInd ("if (" + list + " == null)");
 					WriteLine ("throw CreateReadOnlyCollectionException (" + GetLiteral (typeMap.TypeFullName) + ");");
-					Unindent ();
 					return list;
 				}
 			}
@@ -1994,7 +2085,7 @@ namespace System.Xml.Serialization
 			WriteLineInd ("if (Reader.IsEmptyElement) {");
 			WriteLine ("Reader.Skip();");
 			if (listType.IsArray)
-				WriteLine (list + " = (" + typeMap.TypeFullName + ") ShrinkArray (" + list + ", 0, " + GetTypeOf(listType.GetElementType()) + ", " + isNullable + ");");
+				WriteLine (list + " = (" + typeMap.TypeFullName + ") ShrinkArray (" + list + ", 0, " + GetTypeOf(listType.GetElementType()) + ", false);");
 
 			Unindent ();
 			WriteLineInd ("} else {");
@@ -2031,10 +2122,11 @@ namespace System.Xml.Serialization
 			WriteLine ("ReadEndElement();");
 
 			if (listType.IsArray)
-				WriteLine (list + " = (" + typeMap.TypeFullName + ") ShrinkArray (" + list + ", " + index + ", " + GetTypeOf(listType.GetElementType()) + ", " + isNullable + ");");
+				WriteLine (list + " = (" + typeMap.TypeFullName + ") ShrinkArray (" + list + ", " + index + ", " + GetTypeOf(listType.GetElementType()) + ", false);");
 
 			WriteLineUni ("}");
-			WriteLineUni ("}");
+			if (doNullCheck)
+				WriteLineUni ("}");
 
 			return list;
 		}
@@ -2080,7 +2172,7 @@ namespace System.Xml.Serialization
 			}
 			else	// Must be IEnumerable
 			{
-				WriteLine ("if (" + list + " == null)");
+				WriteLine ("if (((object)" + list + ") == null)");
 				if (canCreateInstance) 
 					WriteLine ("\t" + list + " = new " + listType.FullTypeName + "();");
 				else 
@@ -2098,6 +2190,14 @@ namespace System.Xml.Serialization
 				return "new " + listType.FullName + "()";
 		}
 		
+		string GenerateInitializeList (TypeData listType)
+		{
+			if (listType.Type.IsArray)
+				return "null";
+			else
+				return "new " + listType.Type.FullName + "()";
+		}
+		
 		void GenerateFillerCallbacks ()
 		{
 			foreach (TypeData td in _listsToFill)
@@ -2105,7 +2205,7 @@ namespace System.Xml.Serialization
 				string metName = GetFillListName (td);
 				WriteLine ("void " + metName + " (object list, object source)");
 				WriteLineInd ("{");
-				WriteLine ("if (list == null) throw CreateReadOnlyCollectionException (" + GetLiteral (td.FullTypeName) + ");");
+				WriteLine ("if (((object)list) == null) throw CreateReadOnlyCollectionException (" + GetLiteral (td.FullTypeName) + ");");
 				WriteLine ("");
 
 				WriteLine (td.FullTypeName + " dest = (" + td.FullTypeName + ") list;");
