@@ -31,8 +31,9 @@
 // WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 //
 
-using System.Reflection;
 using System.Collections;
+using System.Globalization;
+using System.Reflection;
 using System.Xml.Schema;
 
 namespace System.Xml.Serialization {
@@ -234,7 +235,7 @@ namespace System.Xml.Serialization {
 
 			if (root != null)
 			{
-				if (root.ElementName != null && root.ElementName != String.Empty)
+				if (root.ElementName.Length != 0)
 					elementName = XmlConvert.EncodeLocalName(root.ElementName);
 				if (root.Namespace != null)
 					rootNamespace = root.Namespace;
@@ -417,14 +418,19 @@ namespace System.Xml.Serialization {
 				elem.IsNullable = att.IsNullable && CanBeNull (elem.TypeData);
 				elem.NestingLevel = att.NestingLevel;
 
-				if (isMultiArray)
+				if (isMultiArray) {
 					elem.MappedType = ImportListMapping (elemType, null, elem.Namespace, atts, nestingLevel + 1);
-				else if (elem.TypeData.IsComplexType)
+				} else if (elem.TypeData.IsComplexType) {
 					elem.MappedType = ImportTypeMapping (elemType, null, elem.Namespace);
+				}
 
-				if (att.ElementName != null) elem.ElementName = XmlConvert.EncodeLocalName(att.ElementName);
-				else if (elem.MappedType != null) elem.ElementName = elem.MappedType.ElementName;
-				else elem.ElementName = TypeTranslator.GetTypeData(elemType).XmlType;
+				if (att.ElementName.Length != 0) {
+					elem.ElementName = XmlConvert.EncodeLocalName (att.ElementName);
+				} else if (elem.MappedType != null) {
+					elem.ElementName = elem.MappedType.ElementName;
+				} else {
+					elem.ElementName = TypeTranslator.GetTypeData (elemType).XmlType;
+				}
 
 				list.Add (elem);
 			}
@@ -437,8 +443,11 @@ namespace System.Xml.Serialization {
 				else if (elem.TypeData.IsComplexType)
 					elem.MappedType = ImportTypeMapping (itemType, null, defaultNamespace);
 
-				if (elem.MappedType != null) elem.ElementName = elem.MappedType.XmlType;
-				else elem.ElementName = TypeTranslator.GetTypeData(itemType).XmlType ;
+				if (elem.MappedType != null) {
+					elem.ElementName = elem.MappedType.XmlType;
+				} else {
+					elem.ElementName = TypeTranslator.GetTypeData (itemType).XmlType;
+				}
 
 				elem.Namespace = (defaultNamespace != null) ? defaultNamespace : "";
 				elem.IsNullable = CanBeNull (elem.TypeData);
@@ -451,13 +460,15 @@ namespace System.Xml.Serialization {
 			// be "ArrayOfXXX" it must be something like ArrayOfChoiceNNN
 
 			string baseName;
-			if (list.Count > 1)
+			if (list.Count > 1) {
 				baseName = "ArrayOfChoice" + (arrayChoiceCount++);
-			else
-			{
-				XmlTypeMapElementInfo elem = ((XmlTypeMapElementInfo)list[0]);
-				if (elem.MappedType != null) baseName = TypeTranslator.GetArrayName (elem.MappedType.XmlType);
-				else baseName = TypeTranslator.GetArrayName (elem.ElementName);
+			} else {
+				XmlTypeMapElementInfo elem = ((XmlTypeMapElementInfo) list[0]);
+				if (elem.MappedType != null) {
+					baseName = TypeTranslator.GetArrayName (elem.MappedType.XmlType);
+				} else {
+					baseName = TypeTranslator.GetArrayName (elem.ElementName);
+				}
 			}
 
 			// Avoid name colisions
@@ -595,6 +606,16 @@ namespace System.Xml.Serialization {
 			// Read all Fields via reflection.
 			ArrayList fieldList = new ArrayList();
 			FieldInfo[] tfields = type.GetFields (BindingFlags.Instance | BindingFlags.Public);
+#if TARGET_JVM
+			// This statement ensures fields are ordered starting from the base type.
+			for (int ti=0; ti<typeList.Count; ti++) {
+				for (int i=0; i<tfields.Length; i++) {
+					FieldInfo field = tfields[i];
+					if (field.DeclaringType == typeList[ti])
+						fieldList.Add (field);
+				}
+			}
+#else
 			currentType = null;
 			int currentIndex = 0;
 			foreach (FieldInfo field in tfields)
@@ -607,10 +628,22 @@ namespace System.Xml.Serialization {
 				}
 				fieldList.Insert(currentIndex++, field);
 			}
-
+#endif
 			// Read all Properties via reflection.
 			ArrayList propList = new ArrayList();
 			PropertyInfo[] tprops = type.GetProperties (BindingFlags.Instance | BindingFlags.Public);
+#if TARGET_JVM
+			// This statement ensures properties are ordered starting from the base type.
+			for (int ti=0; ti<typeList.Count; ti++) {
+				for (int i=0; i<tprops.Length; i++) {
+					PropertyInfo prop = tprops[i];
+					if (!prop.CanRead) continue;
+					if (prop.GetIndexParameters().Length > 0) continue;
+					if (prop.DeclaringType == typeList[ti])
+						propList.Add (prop);
+				}
+			}
+#else
 			currentType = null;
 			currentIndex = 0;
 			foreach (PropertyInfo prop in tprops)
@@ -625,7 +658,7 @@ namespace System.Xml.Serialization {
 				if (prop.GetIndexParameters().Length > 0) continue;
 				propList.Insert(currentIndex++, prop);
 			}
-
+#endif
 			ArrayList members = new ArrayList();
 			int fieldIndex=0;
 			int propIndex=0;
@@ -714,7 +747,7 @@ namespace System.Xml.Serialization {
 					throw new Exception ("XmlAttributeAttribute and XmlElementAttribute cannot be applied to the same member");
 
 				XmlTypeMapMemberAttribute mapAttribute = new XmlTypeMapMemberAttribute ();
-				if (atts.XmlAttribute.AttributeName == null) 
+				if (atts.XmlAttribute.AttributeName.Length == 0) 
 					mapAttribute.AttributeName = rmember.MemberName;
 				else 
 					mapAttribute.AttributeName = atts.XmlAttribute.AttributeName;
@@ -759,6 +792,7 @@ namespace System.Xml.Serialization {
 					member.ListMap = new ListMap ();
 					member.ListMap.ItemInfo = ImportElementInfo (declaringType, XmlConvert.EncodeLocalName (rmember.MemberName), defaultNamespace, typeData.ListItemType, member, atts);
 					member.ElementInfo = member.ListMap.ItemInfo;
+					member.ListMap.ChoiceMember = member.ChoiceMember;
 					mapMember = member;
 				}
 				else
@@ -770,7 +804,7 @@ namespace System.Xml.Serialization {
 					// Creates an ElementInfo that identifies the array instance. 
 					member.ElementInfo = new XmlTypeMapElementInfoList();
 					XmlTypeMapElementInfo elem = new XmlTypeMapElementInfo (member, typeData);
-					elem.ElementName = XmlConvert.EncodeLocalName((atts.XmlArray != null && atts.XmlArray.ElementName != null) ? atts.XmlArray.ElementName : rmember.MemberName);
+					elem.ElementName = XmlConvert.EncodeLocalName((atts.XmlArray != null && atts.XmlArray.ElementName.Length != 0) ? atts.XmlArray.ElementName : rmember.MemberName);
 					elem.Namespace = (atts.XmlArray != null && atts.XmlArray.Namespace != null) ? atts.XmlArray.Namespace : defaultNamespace;
 					elem.MappedType = ImportListMapping (rmember.MemberType, null, elem.Namespace, atts, 0);
 					elem.IsNullable = (atts.XmlArray != null) ? atts.XmlArray.IsNullable : false;
@@ -814,10 +848,22 @@ namespace System.Xml.Serialization {
 				if (mems.Length == 0)
 					throw new InvalidOperationException ("Choice member '" + member.ChoiceMember + "' not found in class '" + cls);
 					
-				if (mems[0] is PropertyInfo) choiceEnumType = ((PropertyInfo)mems[0]).PropertyType;
+				if (mems[0] is PropertyInfo) {
+					PropertyInfo pi = (PropertyInfo)mems[0];
+					if (!pi.CanWrite || !pi.CanRead)
+						throw new InvalidOperationException ("Choice property '" + member.ChoiceMember + "' must be read/write.");
+					choiceEnumType = pi.PropertyType;
+				}
 				else choiceEnumType = ((FieldInfo)mems[0]).FieldType;
 				
-				choiceEnumMap = (EnumMap) ImportTypeMapping (choiceEnumType).ObjectMap;
+				member.ChoiceTypeData = TypeTranslator.GetTypeData (choiceEnumType);
+				
+				if (choiceEnumType.IsArray)
+					choiceEnumType = choiceEnumType.GetElementType ();
+				
+				choiceEnumMap = ImportTypeMapping (choiceEnumType).ObjectMap as EnumMap;
+				if (choiceEnumMap == null)
+					throw new InvalidOperationException ("The member '" + mems[0].Name + "' is not a valid target for XmlChoiceIdentifierAttribute.");
 			}
 			
 			if (atts.XmlElements.Count == 0 && list.Count == 0)
@@ -844,18 +890,26 @@ namespace System.Xml.Serialization {
 					
 				if (elem.TypeData.IsComplexType)
 				{
-					if (att.DataType != null) throw new InvalidOperationException ("'" + att.DataType + "' is an invalid value for the XmlElementAttribute.DateTime property. The property may only be specified for primitive types.");
+					if (att.DataType.Length != 0) throw new InvalidOperationException (
+						string.Format(CultureInfo.InvariantCulture, "'{0}' is "
+							+ "an invalid value for '{1}.{2}' of type '{3}'. "
+							+ "The property may only be specified for primitive types.",
+							att.DataType, cls.FullName, defaultName, 
+							elem.TypeData.FullTypeName));
 					elem.MappedType = ImportTypeMapping (elemType, null, elem.Namespace);
 				}
 
-				if (att.ElementName != null) 
+				if (att.ElementName.Length != 0)  {
 					elem.ElementName = XmlConvert.EncodeLocalName(att.ElementName);
-				else if (multiType) {
-					if (elem.MappedType != null) elem.ElementName = elem.MappedType.ElementName;
-					else elem.ElementName = TypeTranslator.GetTypeData(elemType).XmlType;
-				}
-				else
+				} else if (multiType) {
+					if (elem.MappedType != null) {
+						elem.ElementName = elem.MappedType.ElementName;
+					} else {
+						elem.ElementName = TypeTranslator.GetTypeData (elemType).XmlType;
+					}
+				} else {
 					elem.ElementName = defaultName;
+				}
 
 				if (choiceEnumMap != null) {
 					string cname = choiceEnumMap.GetEnumName (elem.ElementName);
@@ -877,7 +931,7 @@ namespace System.Xml.Serialization {
 			foreach (XmlAnyElementAttribute att in atts.XmlAnyElements)
 			{
 				XmlTypeMapElementInfo elem = new XmlTypeMapElementInfo (member, TypeTranslator.GetTypeData(typeof(XmlElement)));
-				if (att.Name != null && att.Name != string.Empty) 
+				if (att.Name.Length != 0) 
 				{
 					elem.ElementName = XmlConvert.EncodeLocalName(att.Name);
 					elem.Namespace = (att.Namespace != null) ? att.Namespace : "";
@@ -930,11 +984,19 @@ namespace System.Xml.Serialization {
 			if (includedTypes == null) includedTypes = new ArrayList ();
 			if (!includedTypes.Contains (type))
 				includedTypes.Add (type);
+			
+			if (relatedMaps.Count > 0) {
+				foreach (XmlTypeMapping map in (ArrayList) relatedMaps.Clone ()) {
+					if (map.TypeData.Type == typeof(object))
+						map.DerivedTypes.Add (ImportTypeMapping (type));
+				}
+			}
 		}
 
 		public void IncludeTypes (ICustomAttributeProvider provider)
 		{ 
 			object[] ats = provider.GetCustomAttributes (typeof(XmlIncludeAttribute), true);
+			
 			foreach (XmlIncludeAttribute at in ats)
 				IncludeType (at.Type);
 		}
