@@ -207,6 +207,7 @@ namespace System.Xml.Serialization
 		ArrayList _flatLists;
 		ArrayList _allMembers = new ArrayList ();
 		ArrayList _membersWithDefault;
+		ArrayList _listMembers;
 		XmlTypeMapMemberAnyElement _defaultAnyElement;
 		XmlTypeMapMemberAnyAttribute _defaultAnyAttribute;
 		XmlTypeMapMemberNamespaces _namespaceDeclarations;
@@ -219,7 +220,7 @@ namespace System.Xml.Serialization
 		{
 			_allMembers.Add (member);
 			
-			if (!(member.DefaultValue is System.DBNull)) {
+			if (!(member.DefaultValue is System.DBNull) && member.DefaultValue != null) {
 				if (_membersWithDefault == null) _membersWithDefault = new ArrayList ();
 				_membersWithDefault.Add (member);
 			}
@@ -233,7 +234,7 @@ namespace System.Xml.Serialization
 				if (_attributeMembers == null) _attributeMembers = new Hashtable();
 				string key = BuildKey (atm.AttributeName, atm.Namespace);
 				if (_attributeMembers.ContainsKey (key))
-					throw new InvalidOperationException ("The XML attribute named '" + atm.AttributeName + "' from namespace '" + atm.Namespace + "' already present in the current scope. Use XML attributes to specify another XML name or namespace for the attribute.");
+					throw new InvalidOperationException ("The XML attribute named '" + atm.AttributeName + "' from namespace '" + atm.Namespace + "' is already present in the current scope. Use XML attributes to specify another XML name or namespace for the attribute.");
 				member.Index = _attributeMembers.Count;
 				_attributeMembers.Add (key, member);
 				return;
@@ -278,9 +279,14 @@ namespace System.Xml.Serialization
 			{
 				string key = BuildKey (elem.ElementName, elem.Namespace);
 				if (_elements.ContainsKey (key)) 
-					throw new InvalidOperationException ("The XML element named '" + elem.ElementName + "' from namespace '" + elem.Namespace + "' already present in the current scope. Use XML attributes to specify another XML name or namespace for the element.");
+					throw new InvalidOperationException ("The XML element named '" + elem.ElementName + "' from namespace '" + elem.Namespace + "' is already present in the current scope. Use XML attributes to specify another XML name or namespace for the element.");
 				_elements.Add (key, elem);
 			}
+			
+			if (member.TypeData.IsListType && member.TypeData.Type != null && !member.TypeData.Type.IsArray) {
+				if (_listMembers == null) _listMembers = new ArrayList ();
+				_listMembers.Add (member);
+			}
 		}
 
 		void RegisterFlatList (XmlTypeMapMemberExpandable member)
@@ -394,6 +400,11 @@ namespace System.Xml.Serialization
 		{
 			get { return _membersWithDefault; }
 		}
+		
+		public ArrayList ListMembers
+		{
+			get { return _listMembers; }
+		}
 
 		public XmlTypeMapMember XmlTextCollector
 		{
@@ -442,6 +453,7 @@ namespace System.Xml.Serialization
 		XmlTypeMapElementInfoList _itemInfo;
 		bool _gotNestedMapping;
 		XmlTypeMapping _nestedArrayMapping;
+		string _choiceMember;
 
 		public bool IsMultiArray
 		{
@@ -451,6 +463,12 @@ namespace System.Xml.Serialization
 			}
 		}
 
+		public string ChoiceMember
+		{
+			get { return _choiceMember; }
+			set { _choiceMember = value; }
+		}
+
 		public XmlTypeMapping NestedArrayMapping
 		{
 			get
@@ -483,10 +501,20 @@ namespace System.Xml.Serialization
 			set { _itemInfo = value; }
 		}
 
-		public XmlTypeMapElementInfo FindElement (object memberValue)
+		public XmlTypeMapElementInfo FindElement (object ob, int index, object memberValue)
 		{
 			if (_itemInfo.Count == 1) 
 				return (XmlTypeMapElementInfo) _itemInfo[0];
+			else if (_choiceMember != null && index != -1)
+			{
+				Array values = (Array) XmlTypeMapMember.GetValue (ob, _choiceMember);
+				if (values == null || index >= values.Length)
+					throw new InvalidOperationException ("Invalid or missing choice enum value in member '" + _choiceMember + "'.");
+				object val = values.GetValue (index);
+				foreach (XmlTypeMapElementInfo elem in _itemInfo)
+					if (elem.ChoiceValue != null && elem.ChoiceValue.Equals (val))
+						return elem;
+			}
 			else
 			{
 				if (memberValue == null) return null;
@@ -637,7 +665,12 @@ namespace System.Xml.Serialization
 			foreach (EnumMapMember mem in _members)
 				if (mem.EnumName == enumName) return mem.XmlName;
 			
-			return Convert.ToInt64(enumValue).ToString(CultureInfo.InvariantCulture);
+			// Enum default value will not be written
+			// unless it is one of the enum legal values
+			if (enumName == "0" && IsFlags)
+				return String.Empty;
+			else
+				return Convert.ToInt64(enumValue).ToString(CultureInfo.InvariantCulture);
 		}
 
 		public string GetEnumName (string xmlName)
