@@ -351,11 +351,7 @@ namespace System.Xml.XPath
 						// FIXME: We have to set this in
 						// reverse order since currently
 						// we don't support collation.
-#if TARGET_JVM
 						_nMulCase = (orderCase == XmlCaseOrder.UpperFirst) ? -1 : 1;
-#else
-						_nMulCase = (orderCase == XmlCaseOrder.UpperFirst) ? 1 : -1;
-#endif
 						_nMulSort = (orderSort == XmlSortOrder.Ascending) ? 1 : -1;
 
 						if (strLang == null || strLang == "")
@@ -1609,11 +1605,11 @@ namespace System.Xml.XPath
 
 			// get the prefix for the given name
 			String strURI1 = "";
-			if (nsm != null && _name.Namespace != "")
+			if (_name.Namespace != "")
 			{
 				if (resolvedName)
 					strURI1 = _name.Namespace;
-				else
+				else if (nsm != null)
 // We still need to have such tricky switch, because the behavior is 
 // inconsistent between .NET 1.x and 2.0 when the argument is not
 // atomic string.
