@@ -31,7 +31,11 @@
 //
 
 using System;
+#if NET_2_0
 using System.Collections;
+using System.Collections.Generic;
+using System.Diagnostics;
+#endif
 using System.IO;
 using System.Xml;
 using System.Xml.Schema;
@@ -52,6 +56,14 @@ namespace System.Xml.XPath
 	public abstract class XPathNavigator : ICloneable
 #endif
 	{
+		#region Static members
+#if NET_2_0
+		public static IEqualityComparer NavigatorComparer {
+			get { return XPathNavigatorComparer.Instance; }
+		}
+#endif
+		#endregion
+
 		#region Constructor
 
 		protected XPathNavigator ()
@@ -450,6 +462,7 @@ namespace System.Xml.XPath
 			return false;
 		}
 
+		/*
 		public virtual bool MoveToFirst ()
 		{
 			if (MoveToPrevious ()) {
@@ -460,6 +473,12 @@ namespace System.Xml.XPath
 			}
 			return false;
 		}
+		*/
+
+		public virtual bool MoveToFirst ()
+		{
+			return MoveToFirstImpl ();
+		}
 
 		public virtual void MoveToRoot ()
 		{
@@ -476,6 +495,22 @@ namespace System.Xml.XPath
 		public abstract void MoveToRoot ();
 #endif
 
+		internal bool MoveToFirstImpl ()
+		{
+			switch (NodeType) {
+			case XPathNodeType.Attribute:
+			case XPathNodeType.Namespace:
+				return false;
+			default:
+				if (!MoveToParent ())
+					return false;
+				// Follow these 2 steps so that we can skip 
+				// some types of nodes .
+				MoveToFirstChild ();
+				return true;
+			}
+		}
+
 		public abstract bool MoveToFirstAttribute ();
 
 		public abstract bool MoveToFirstChild ();
@@ -622,9 +657,9 @@ namespace System.Xml.XPath
 		}
 
 		[MonoTODO]
-		public virtual IDictionary GetNamespacesInScope (XmlNamespaceScope scope)
+		public virtual IDictionary<string, string> GetNamespacesInScope (XmlNamespaceScope scope)
 		{
-			Hashtable table = new Hashtable ();
+			IDictionary<string, string> table = new Dictionary<string, string> ();
 			XPathNamespaceScope xpscope =
 				scope == XmlNamespaceScope.Local ?
 					XPathNamespaceScope.Local :
@@ -686,16 +721,6 @@ namespace System.Xml.XPath
 			return MoveTo (SelectChildren (localName, namespaceURI));
 		}
 
-		bool MoveToDescendant (XPathNodeType type)
-		{
-			return MoveTo (SelectDescendants (type, false));
-		}
-
-		bool MoveToDescendant (string localName, string namespaceURI)
-		{
-			return MoveTo (SelectDescendants (localName, namespaceURI, false));
-		}
-
 		public virtual bool MoveToNext (string localName, string namespaceURI)
 		{
 			XPathNavigator nav = Clone ();
@@ -721,79 +746,91 @@ namespace System.Xml.XPath
 			return false;
 		}
 
-		[MonoTODO]
 		public virtual bool MoveToFollowing (string localName,
 			string namespaceURI)
 		{
 			return MoveToFollowing (localName, namespaceURI, null);
 		}
 
-		[MonoTODO]
 		public virtual bool MoveToFollowing (string localName,
 			string namespaceURI, XPathNavigator end)
 		{
+			if (localName == null)
+				throw new ArgumentNullException ("localName");
+			if (namespaceURI == null)
+				throw new ArgumentNullException ("namespaceURI");
+			localName = NameTable.Get (localName);
+			if (localName == null)
+				return false;
+			namespaceURI = NameTable.Get (namespaceURI);
+			if (namespaceURI == null)
+				return false;
+
 			XPathNavigator nav = Clone ();
-			bool skip = false;
+			switch (nav.NodeType) {
+			case XPathNodeType.Attribute:
+			case XPathNodeType.Namespace:
+				nav.MoveToParent ();
+				break;
+			}
 			do {
-				if (!skip && nav.MoveToDescendant (localName,
-					namespaceURI)) {
-					if (end != null) {
-						switch (nav.ComparePosition (end)) {
-						case XmlNodeOrder.After:
-						case XmlNodeOrder.Unknown:
-							return false;
+				if (!nav.MoveToFirstChild ()) {
+					do {
+						if (!nav.MoveToNext ()) {
+							if (!nav.MoveToParent ())
+								return false;
 						}
-					}
+						else
+							break;
+					} while (true);
+				}
+				if (end != null && end.IsSamePosition (nav))
+					return false;
+				if (object.ReferenceEquals (localName, nav.LocalName) &&
+					object.ReferenceEquals (namespaceURI, nav.NamespaceURI)) {
 					MoveTo (nav);
 					return true;
 				}
-				else
-					skip = false;
-				if (!nav.MoveToNext ()) {
-					if (!nav.MoveToParent ())
-						break;
-					skip = true;
-				}
 			} while (true);
-			return false;
 		}
 
-		[MonoTODO]
 		public virtual bool MoveToFollowing (XPathNodeType type)
 		{
 			return MoveToFollowing (type, null);
 		}
 
-		[MonoTODO]
 		public virtual bool MoveToFollowing (XPathNodeType type,
 			XPathNavigator end)
 		{
+			if (type == XPathNodeType.Root)
+				return false; // will never match
 			XPathNavigator nav = Clone ();
-			bool skip = false;
+			switch (nav.NodeType) {
+			case XPathNodeType.Attribute:
+			case XPathNodeType.Namespace:
+				nav.MoveToParent ();
+				break;
+			}
 			do {
-				if (!skip && nav.MoveToDescendant (type)) {
-					if (end != null) {
-						switch (nav.ComparePosition (end)) {
-						case XmlNodeOrder.After:
-						case XmlNodeOrder.Unknown:
-							return false;
+				if (!nav.MoveToFirstChild ()) {
+					do {
+						if (!nav.MoveToNext ()) {
+							if (!nav.MoveToParent ())
+								return false;
 						}
-					}
+						else
+							break;
+					} while (true);
+				}
+				if (end != null && end.IsSamePosition (nav))
+					return false;
+				if (nav.NodeType == type) {
 					MoveTo (nav);
 					return true;
 				}
-				else
-					skip = false;
-				if (!nav.MoveToNext ()) {
-					if (!nav.MoveToParent ())
-						break;
-					skip = true;
-				}
 			} while (true);
-			return false;
 		}
 
-		[MonoTODO]
 		public virtual XmlReader ReadSubtree ()
 		{
 			return new XPathNavigatorReader (this);
@@ -816,7 +853,7 @@ namespace System.Xml.XPath
 			return SelectSingleNode (expr);
 		}
 
-		public XPathNavigator SelectSingleNode (XPathExpression expression)
+		public virtual XPathNavigator SelectSingleNode (XPathExpression expression)
 		{
 			XPathNodeIterator iter = Select (expression);
 			if (iter.MoveNext ())
@@ -865,16 +902,10 @@ namespace System.Xml.XPath
 			}
 		}
 
-		[MonoTODO]
 		public override bool IsNode {
 			get { return true; }
 		}
 
-		[MonoTODO]
-		public virtual IKeyComparer NavigatorComparer {
-			get { throw new NotImplementedException (); }
-		}
-
 		[MonoTODO]
 		public virtual string OuterXml {
 			get {
@@ -922,37 +953,30 @@ namespace System.Xml.XPath
 			}
 		}
 
-		[MonoTODO]
 		public virtual object UnderlyingObject {
-			get { throw new NotImplementedException (); }
+			get { return null; }
 		}
 
-		[MonoTODO]
 		public override bool ValueAsBoolean {
 			get { return XQueryConvert.StringToBoolean (Value); }
 		}
 
-		[MonoTODO]
 		public override DateTime ValueAsDateTime {
 			get { return XmlConvert.ToDateTime (Value); }
 		}
 
-		[MonoTODO]
 		public override double ValueAsDouble {
 			get { return XQueryConvert.StringToDouble (Value); }
 		}
 
-		[MonoTODO]
 		public override int ValueAsInt {
 			get { return XQueryConvert.StringToInt (Value); }
 		}
 
-		[MonoTODO]
 		public override long ValueAsLong {
 			get { return XQueryConvert.StringToInteger (Value); }
 		}
 
-		[MonoTODO]
 		public override Type ValueType {
 			get {
 				return SchemaInfo != null &&
@@ -972,31 +996,31 @@ namespace System.Xml.XPath
 			}
 		}
 
-
-
-
-
-
-
 		private XmlReader CreateFragmentReader (string fragment)
 		{
-			return new XmlTextReader (fragment, XmlNodeType.Element, new XmlParserContext (NameTable, null, null, XmlSpace.None));
+			XmlReaderSettings settings = new XmlReaderSettings ();
+			settings.ConformanceLevel = ConformanceLevel.Fragment;
+			XmlNamespaceManager nsmgr = new XmlNamespaceManager (NameTable);
+			foreach (KeyValuePair<string,string> nss in GetNamespacesInScope (XmlNamespaceScope.All))
+				nsmgr.AddNamespace (nss.Key, nss.Value);
+			return XmlReader.Create (
+				new StringReader (fragment),
+				settings,
+				new XmlParserContext (NameTable, nsmgr, null, XmlSpace.None));
 		}
 
+		// must override it.
 		public virtual XmlWriter AppendChild ()
 		{
 			throw new NotSupportedException ();
 		}
 
-		[MonoTODO]
 		public virtual void AppendChild (
 			string xmlFragments)
 		{
-			// FIXME: should XmlParserContext be something?
 			AppendChild (CreateFragmentReader (xmlFragments));
 		}
 
-		[MonoTODO]
 		public virtual void AppendChild (
 			XmlReader reader)
 		{
@@ -1013,7 +1037,7 @@ namespace System.Xml.XPath
 			AppendChild (new XPathNavigatorReader (nav));
 		}
 
-		public void AppendChildElement (string prefix, string name, string ns, string value)
+		public virtual void AppendChildElement (string prefix, string name, string ns, string value)
 		{
 			XmlWriter xw = AppendChild ();
 			xw.WriteStartElement (prefix, name, ns);
@@ -1029,23 +1053,45 @@ namespace System.Xml.XPath
 			}
 		}
 
+		// must override it.
+		[MonoTODO ("needs tests")]
 		public virtual XmlWriter CreateAttributes ()
 		{
 			throw new NotSupportedException ();
 		}
 
+		// must override it.
 		public virtual void DeleteSelf ()
 		{
 			throw new NotSupportedException ();
 		}
 
+		// must override it.
+		public virtual void DeleteRange (XPathNavigator nav)
+		{
+			throw new NotSupportedException ();
+		}
+
+		public virtual XmlWriter ReplaceRange (XPathNavigator nav)
+		{
+			throw new NotSupportedException ();
+		}
+	
 		public virtual XmlWriter InsertAfter ()
 		{
+			switch (NodeType) {
+			case XPathNodeType.Root:
+			case XPathNodeType.Attribute:
+			case XPathNodeType.Namespace:
+				throw new InvalidOperationException (String.Format ("Insertion after {0} is not allowed.", NodeType));
+			}
 			XPathNavigator nav = Clone ();
 			if (nav.MoveToNext ())
 				return nav.InsertBefore ();
+			else if (nav.MoveToParent ())
+				return nav.AppendChild ();
 			else
-				return AppendChild ();
+				throw new InvalidOperationException ("Could not move to parent to insert sibling node");
 		}
 
 		public virtual void InsertAfter (string xmlFragments)
@@ -1053,7 +1099,6 @@ namespace System.Xml.XPath
 			InsertAfter (CreateFragmentReader (xmlFragments));
 		}
 
-		[MonoTODO]
 		public virtual void InsertAfter (XmlReader reader)
 		{
 			using (XmlWriter w = InsertAfter ()) {
@@ -1077,7 +1122,6 @@ namespace System.Xml.XPath
 			InsertBefore (CreateFragmentReader (xmlFragments));
 		}
 
-		[MonoTODO]
 		public virtual void InsertBefore (XmlReader reader)
 		{
 			using (XmlWriter w = InsertBefore ()) {
@@ -1113,7 +1157,7 @@ namespace System.Xml.XPath
 			if (nav.MoveToFirstChild ())
 				return nav.InsertBefore ();
 			else
-				return InsertBefore ();
+				return AppendChild ();
 		}
 
 		public virtual void PrependChild (string xmlFragments)
@@ -1121,7 +1165,6 @@ namespace System.Xml.XPath
 			PrependChild (CreateFragmentReader (xmlFragments));
 		}
 
-		[MonoTODO]
 		public virtual void PrependChild (XmlReader reader)
 		{
 			using (XmlWriter w = PrependChild ()) {
@@ -1143,17 +1186,15 @@ namespace System.Xml.XPath
 			}
 		}
 
-		[MonoTODO]
 		public virtual void ReplaceSelf (string xmlFragment)
 		{
-			ReplaceSelf (XmlReader.Create (new StringReader (xmlFragment)));
+			ReplaceSelf (CreateFragmentReader (xmlFragment));
 		}
 
-		[MonoTODO]
+		// must override it.
 		public virtual void ReplaceSelf (XmlReader reader)
 		{
-			InsertBefore (reader);
-			DeleteSelf ();
+			throw new NotSupportedException ();
 		}
 
 		[MonoTODO]
