@@ -30,6 +30,9 @@
 //
 using System;
 using System.Collections;
+#if NET_2_0
+using System.Collections.Generic;
+#endif
 using System.IO;
 using System.Text;
 using System.Xml;
@@ -69,7 +72,7 @@ namespace Mono.Xml
 			attributeLocalNames = new Hashtable ();
 			attributeNamespaces = new Hashtable ();
 			attributePrefixes = new Hashtable ();
-			nsdecls = new Hashtable ();
+			nsmgr = new XmlNamespaceManager (reader.NameTable);
 			this.validatingReader = validatingReader;
 			valueBuilder = new StringBuilder ();
 			idList = new ArrayList ();
@@ -105,7 +108,7 @@ namespace Mono.Xml
 		Hashtable attributeLocalNames;
 		Hashtable attributeNamespaces;
 		Hashtable attributePrefixes;
-		Hashtable nsdecls;
+		XmlNamespaceManager nsmgr;
 		StringBuilder valueBuilder;
 		ArrayList idList;
 		ArrayList missingIDReferences;
@@ -115,6 +118,7 @@ namespace Mono.Xml
 		bool isWhitespace;
 		bool isText;
 		bool dontResetTextType;
+		bool popScope;
 
 		// This field is used to get properties and to raise events.
 		XmlValidatingReader validatingReader;
@@ -172,10 +176,10 @@ namespace Mono.Xml
 		}
 
 #if NET_2_0
-		IDictionary IXmlNamespaceResolver.GetNamespacesInScope (XmlNamespaceScope scope)
+		IDictionary<string, string> IXmlNamespaceResolver.GetNamespacesInScope (XmlNamespaceScope scope)
 		{
 			IXmlNamespaceResolver res = reader as IXmlNamespaceResolver;
-			return res != null ? res.GetNamespacesInScope (scope) : new Hashtable ();
+			return res != null ? res.GetNamespacesInScope (scope) : new Dictionary<string, string> ();
 		}
 #endif
 
@@ -190,9 +194,8 @@ namespace Mono.Xml
 
 		public override string LookupNamespace (string prefix)
 		{
-			if (nsdecls.Count > 0 && nsdecls [prefix] != null)
-				return (string) nsdecls [prefix];
-			return reader.LookupNamespace (prefix);
+			string s = nsmgr.LookupNamespace (NameTable.Get (prefix));
+			return s == String.Empty ? null : s;
 		}
 
 #if NET_2_0
@@ -366,7 +369,6 @@ namespace Mono.Xml
 			attributeValues.Clear ();
 			attributeNamespaces.Clear ();
 			attributePrefixes.Clear ();
-			nsdecls.Clear ();
 			isWhitespace = false;
 			isSignificantWhitespace = false;
 			isText = false;
@@ -387,6 +389,10 @@ namespace Mono.Xml
 
 		private bool ReadContent ()
 		{
+			if (popScope) {
+				nsmgr.PopScope ();
+				popScope = false;
+			}
 			if (nextEntityReader != null) {
 				if (DTD == null || DTD.EntityDecls [reader.Name] == null)
 					throw NotWFError (String.Format ("Entity '{0}' was not declared.", reader.Name));
@@ -477,6 +483,8 @@ namespace Mono.Xml
 				break;
 
 			case XmlNodeType.Element:
+				nsmgr.PushScope ();
+				popScope = reader.IsEmptyElement;
 				if (constructingTextValue != null) {
 					currentTextValue = constructingTextValue;
 					constructingTextValue = null;
@@ -527,12 +535,6 @@ namespace Mono.Xml
 					// SetupValidityIgnorantAttributes ();
 					ValidateAttributes (null, false);
 				}
-				foreach (string attr in attributes)
-					if (attr == "xmlns" ||
-						String.CompareOrdinal (attr, 0, "xmlns", 0, 5) == 0)
-						nsdecls.Add (
-							attr == "xmlns" ? String.Empty : attributeLocalNames [attr],
-							attributeValues [attr]);
 				// If it is empty element then directly check end element.
 				if (reader.IsEmptyElement)
 					goto case XmlNodeType.EndElement;
@@ -544,6 +546,7 @@ namespace Mono.Xml
 					constructingTextValue = null;
 					return true;
 				}
+				popScope = true;
 				elementStack.Pop ();
 				// endElementDeriv
 				// If no schema specification, then skip validation.
@@ -569,6 +572,11 @@ namespace Mono.Xml
 				break;
 
 			case XmlNodeType.CDATA:
+				isSignificantWhitespace = isWhitespace = false;
+				isText = true;
+
+				ValidateText ();
+
 				if (currentTextValue != null) {
 					currentTextValue = constructingTextValue;
 					constructingTextValue = null;
@@ -578,26 +586,21 @@ namespace Mono.Xml
 			case XmlNodeType.SignificantWhitespace:
 				if (!isText)
 					isSignificantWhitespace = true;
+				isWhitespace = false;
 				dontResetTextType = true;
-				goto case XmlNodeType.Text;
+				goto case XmlNodeType.DocumentFragment;
 			case XmlNodeType.Text:
+				isWhitespace = isSignificantWhitespace = false;
 				isText = true;
-				if (!dontResetTextType) {
-					isWhitespace = isSignificantWhitespace = false;
-				}
-				// If no schema specification, then skip validation.
-				if (currentAutomata == null)
+				goto case XmlNodeType.DocumentFragment;
+			case XmlNodeType.DocumentFragment:
+				// it should not happen, but in case if
+				// XmlReader really returns it, just ignore.
+				if (reader.NodeType == XmlNodeType.DocumentFragment)
 					break;
 
-				if (elementStack.Count > 0)
-					elem = dtd.ElementDecls [elementStack.Peek () as string];
-				// Here element should have been already validated, so
-				// if no matching declaration is found, simply ignore.
-				if (elem != null && !elem.IsMixedContent && !elem.IsAny) {
-					HandleError (String.Format ("Current element {0} does not allow character data content.", elementStack.Peek () as string),
-						XmlSeverityType.Error);
-					currentAutomata = previousAutomata;
-				}
+				ValidateText ();
+
 				if (entityReaderStack.Count > 0 && validatingReader.EntityHandling == EntityHandling.ExpandEntities) {
 					constructingTextValue += reader.Value;
 					return ReadContent ();
@@ -606,7 +609,7 @@ namespace Mono.Xml
 			case XmlNodeType.Whitespace:
 				if (!isText && !isSignificantWhitespace)
 					isWhitespace = true;
-				goto case XmlNodeType.Text;
+				goto case XmlNodeType.DocumentFragment;
 			case XmlNodeType.EntityReference:
 				if (validatingReader.EntityHandling == EntityHandling.ExpandEntities) {
 					ResolveEntity ();
@@ -622,6 +625,23 @@ namespace Mono.Xml
 			return true;
 		}
 
+		private void ValidateText ()
+		{
+			if (currentAutomata == null)
+				return;
+
+			DTDElementDeclaration elem = null;
+			if (elementStack.Count > 0)
+				elem = dtd.ElementDecls [elementStack.Peek () as string];
+			// Here element should have been already validated, so
+			// if no matching declaration is found, simply ignore.
+			if (elem != null && !elem.IsMixedContent && !elem.IsAny && !isWhitespace) {
+				HandleError (String.Format ("Current element {0} does not allow character data content.", elementStack.Peek () as string),
+					XmlSeverityType.Error);
+				currentAutomata = previousAutomata;
+			}
+		}
+
 		private void ValidateWhitespaceNode ()
 		{
 			// VC Standalone Document Declaration (2.9)
@@ -671,13 +691,32 @@ namespace Mono.Xml
 		Stack attributeValueEntityStack = new Stack ();
 
 		private void ValidateAttributes (DTDAttListDeclaration decl, bool validate)
+		{
+			DtdValidateAttributes (decl, validate);
+
+			foreach (string attr in attributes)
+				if (attr == "xmlns" ||
+					String.CompareOrdinal (attr, 0, "xmlns:", 0, 6) == 0)
+					nsmgr.AddNamespace (
+						attr == "xmlns" ? String.Empty : (string) attributeLocalNames [attr],
+						(string) attributeValues [attr]);
+
+			foreach (string attr in attributes) {
+				string prefix = attr == "xmlns" ? "xmlns" : attributePrefixes [attr] as string;
+				if (prefix == String.Empty)
+					attributeNamespaces.Add (attr, String.Empty);
+				else
+					attributeNamespaces.Add (attr, LookupNamespace (prefix));
+			}
+		}
+
+		private void DtdValidateAttributes (DTDAttListDeclaration decl, bool validate)
 		{
 			while (reader.MoveToNextAttribute ()) {
 				string attrName = reader.Name;
 				this.currentAttribute = attrName;
 				attributes.Add (attrName);
 				attributeLocalNames.Add (attrName, reader.LocalName);
-				attributeNamespaces.Add (attrName, reader.NamespaceURI);
 				attributePrefixes.Add (attrName, reader.Prefix);
 				XmlReader targetReader = reader;
 				string attrValue = null;
@@ -887,7 +926,6 @@ namespace Mono.Xml
 						String.Empty :
 						def.Name.Substring (0, colonAt);
 					attributePrefixes.Add (def.Name, prefix);
-					attributeNamespaces.Add (def.Name, reader.LookupNamespace (prefix));
 					attributeValues.Add (def.Name, def.DefaultValue);
 					break;
 				}
@@ -1062,8 +1100,6 @@ namespace Mono.Xml
 					return ((XmlReader) entityReaderStack.Peek ()).LocalName; // name of EndEntity
 				else if (NodeType == XmlNodeType.Attribute)
 					return (string) attributeLocalNames [currentAttribute];
-				else if (IsDefault)
-					return String.Empty;
 				else
 					return reader.LocalName;
 			}
@@ -1077,8 +1113,6 @@ namespace Mono.Xml
 					return ((XmlReader) entityReaderStack.Peek ()).Name; // name of EndEntity
 				else if (NodeType == XmlNodeType.Attribute)
 					return currentAttribute;
-				else if (IsDefault)
-					return String.Empty;
 				else
 					return reader.Name;
 			}
@@ -1088,14 +1122,15 @@ namespace Mono.Xml
 			get {
 				if (currentTextValue != null || consumedAttribute)
 					return String.Empty;
-				else if (NodeType == XmlNodeType.Attribute)
+				switch (NodeType) {
+				case XmlNodeType.Attribute:
 					return (string) attributeNamespaces [currentAttribute];
-				else if (IsDefault)
+				case XmlNodeType.Element:
+				case XmlNodeType.EndElement:
+					return nsmgr.LookupNamespace (Prefix);
+				default:
 					return String.Empty;
-				else if (nsdecls.Count > 0 && nsdecls [Prefix] != null)
-					return (string) nsdecls [Prefix];
-				else
-					return reader.NamespaceURI;
+				}
 			}
 		}
 
@@ -1130,8 +1165,6 @@ namespace Mono.Xml
 					return String.Empty;
 				else if (NodeType == XmlNodeType.Attribute)
 					return (string) attributePrefixes [currentAttribute];
-				else if (IsDefault)
-					return String.Empty;
 				else
 					return reader.Prefix;
 			}
@@ -1199,18 +1232,11 @@ namespace Mono.Xml
 			get {
 				if (currentTextValue != null)
 					return currentTextValue;
-				// This check also covers value node of default attributes.
-				if (IsDefault) {
-					DTDAttributeDefinition def = 
-						dtd.AttListDecls [currentElement] [currentAttribute] as DTDAttributeDefinition;
-					return sourceTextReader != null && sourceTextReader.Normalization ?
-						def.NormalizedDefaultValue : def.DefaultValue;
-				}
 				// As to this property, MS.NET seems ignorant of EntityHandling...
-				else if (NodeType == XmlNodeType.Attribute)// &&
+				else if (NodeType == XmlNodeType.Attribute
+					// It also covers default attribute text.
+ 					|| consumedAttribute)
 					return FilterNormalization (Name, (string) attributeValues [currentAttribute]);
-				else if (consumedAttribute)
-					return FilterNormalization (Name, (string) attributeValues [this.currentAttribute]);
 				else
 					return FilterNormalization (Name, reader.Value);
 			}
