@@ -287,7 +287,7 @@ namespace System.Xml {
 			string modestr = null;
 			switch (mode) {
 			case XmlDateTimeSerializationMode.Local:
-			case XmlDateTimeSerializationMode.RoundTripKind:
+			case XmlDateTimeSerializationMode.RoundtripKind:
 			default:
 				return ToDateTime (value, "yyyy-MM-ddTHH:mm:ss.fffffffzzz");
 			case XmlDateTimeSerializationMode.Utc:
@@ -497,7 +497,7 @@ namespace System.Xml {
 			string modestr = null;
 			switch (mode) {
 			case XmlDateTimeSerializationMode.Local:
-			case XmlDateTimeSerializationMode.RoundTripKind:
+			case XmlDateTimeSerializationMode.RoundtripKind:
 			default:
 				return value.ToString (
 					"yyyy-MM-ddTHH:mm:ss.fffffffzzz",
@@ -542,6 +542,8 @@ namespace System.Xml {
 			int hours = 0;
 			int minutes = 0;
 			int seconds = 0;
+			int milliseconds = 0;
+			int parsedDigits = 0;
 
 			bool error = false;
 
@@ -554,11 +556,19 @@ namespace System.Xml {
 					start = i;
 					continue;
 				}
-				for (; i < s.Length; i++) {
-					if (!Char.IsDigit (s [i]))
+				for (; i < s.Length; i++)
+					if (s [i] < '0' || '9' < s [i])
 						break;
-				}
+				if (parseStep == 7)
+					parsedDigits = i - start;
 				int value = int.Parse (s.Substring (start, i - start), CultureInfo.InvariantCulture);
+				if (parseStep == 7) {
+					// adjust to 3 digits so that it makes sense as millisecond digits
+					for (; parsedDigits > 3; parsedDigits--)
+						value /= 10;
+					for (; parsedDigits < 3; parsedDigits++)
+						value *= 10;
+				}
 				switch (s [i]) {
 				case 'Y':
 					days += value * 365;
@@ -593,11 +603,20 @@ namespace System.Xml {
 						parseStep = 5;
 					break;
 				case 'S':
-					seconds = value;
-					if (!isTime || parseStep > 6)
+					if (parseStep == 7)
+						milliseconds = value;
+					else
+						seconds = value;
+					if (!isTime || parseStep > 7)
 						error = true;
 					else
-						parseStep = 7;
+						parseStep = 8;
+					break;
+				case '.':
+					if (parseStep > 7)
+						error = true;
+					seconds = value;
+					parseStep = 7;
 					break;
 				default:
 					error = true;
@@ -610,7 +629,7 @@ namespace System.Xml {
 			}
 			if (error)
 				throw new ArgumentException ("Invalid format string for duration schema datatype.");
-			TimeSpan ts = new TimeSpan (days, hours, minutes, seconds);
+			TimeSpan ts = new TimeSpan (days, hours, minutes, seconds, milliseconds);
 			return minusValue ? -ts : ts;
 		}
 
@@ -634,7 +653,7 @@ namespace System.Xml {
 
 		public static string VerifyName (string name)
 		{
-			if (name == null)
+			if (name == null || name.Length == 0)
 				throw new ArgumentNullException("name");
 
 			if (!XmlChar.IsName (name))
@@ -645,7 +664,7 @@ namespace System.Xml {
 
 		public static string VerifyNCName (string ncname)
 		{
-			if (ncname == null)
+			if (ncname == null || ncname.Length == 0)
 				throw new ArgumentNullException("ncname");
 
 			if (!XmlChar.IsNCName (ncname))
@@ -653,6 +672,29 @@ namespace System.Xml {
 			return ncname;
 		}
 
+#if NET_2_0
+		public static string VerifyTOKEN (string name)
+#else
+		internal static string VerifyTOKEN (string name)
+#endif
+		{
+			if (name == null)
+				throw new ArgumentNullException("name");
+
+			if (name.Length == 0)
+				return name;
+
+			if (XmlChar.IsWhitespace (name [0]) ||
+				XmlChar.IsWhitespace (name [name.Length - 1]))
+				throw new XmlException ("Whitespace characters (#xA, #xD, #x9, #x20) are not allowed as leading or trailing whitespaces of xs:token.");
+
+			for (int i = 0; i < name.Length; i++)
+				if (XmlChar.IsWhitespace (name [i]) && name [i] != ' ')
+				throw new XmlException ("Either #xA, #xD or #x9 are not allowed inside xs:token.");
+
+			return name;
+		}
+
 #if NET_2_0
 		public static string VerifyNMTOKEN (string name)
 #else
