@@ -12,8 +12,7 @@
 // (C) 2001 Daniel Weber
 // (C) 2002 Kral Ferch, Jason Diamond, Miguel de Icaza, Duncan Mak,
 //   Atsushi Enomoto
-//
-
+// Copyright (C) 2005 Novell, Inc (http://www.novell.com)
 //
 // Permission is hereby granted, free of charge, to any person obtaining
 // a copy of this software and associated documentation files (the
@@ -35,9 +34,9 @@
 // WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 //
 
-using System;
 using System.Globalization;
 using System.IO;
+using System.Security.Permissions;
 using System.Text;
 using System.Xml.XPath;
 using System.Diagnostics;
@@ -63,6 +62,7 @@ namespace System.Xml
 		XmlNameEntryCache nameCache = new XmlNameEntryCache ();
 #if NET_2_0
 		XmlSchemaSet schemas;
+		IXmlSchemaInfo schemaInfo;
 #endif
 
 		// MS.NET rejects undeclared entities _only_ during Load(),
@@ -140,11 +140,10 @@ namespace System.Xml
 
 		public virtual XmlDocumentType DocumentType {
 			get {
-				for (int i = 0; i < ChildNodes.Count; i++) {
-					XmlNode n = ChildNodes [i];
+				for (XmlNode n = FirstChild; n != null; n = n.NextSibling)
 					if(n.NodeType == XmlNodeType.DocumentType)
 						return (XmlDocumentType)n;
-				}
+
 				return null;
 			}
 		}
@@ -239,10 +238,19 @@ namespace System.Xml
 		}
 
 #if NET_2_0
+		public override XmlNode ParentNode {
+			get { return null; }
+		}
+
 		public XmlSchemaSet Schemas {
 			get { return schemas; }
 			set { schemas = value; }
 		}
+
+		public override IXmlSchemaInfo SchemaInfo {
+			get { return schemaInfo; }
+			internal set { schemaInfo = value; }
+		}
 #endif
 
 		#endregion
@@ -260,8 +268,8 @@ namespace System.Xml
 
 			if(deep)
 			{
-				for (int i = 0; i < ChildNodes.Count; i++)
-					doc.AppendChild (doc.ImportNode (ChildNodes [i], deep));
+				for (XmlNode n = FirstChild; n != null; n = n.NextSibling)
+					doc.AppendChild (doc.ImportNode (n, deep));
 			}
 			return doc;
 		}
@@ -327,6 +335,7 @@ namespace System.Xml
 			return new XmlDocumentFragment (this);
 		}
 
+		[PermissionSet (SecurityAction.InheritanceDemand, Unrestricted = true)]
 		public virtual XmlDocumentType CreateDocumentType (string name, string publicId,
 								   string systemId, string internalSubset)
 		{
@@ -373,6 +382,13 @@ namespace System.Xml
 			return new XmlEntityReference (name, this);
 		}
 
+#if NET_2_0
+		public override XPathNavigator CreateNavigator ()
+		{
+			return CreateNavigator (this);
+		}
+#endif
+
 		protected internal virtual XPathNavigator CreateNavigator (XmlNode node)
 		{
 #if NET_2_0
@@ -424,7 +440,13 @@ namespace System.Xml
 				case XmlNodeType.Text: return CreateTextNode (null);
 				case XmlNodeType.Whitespace: return CreateWhitespace (String.Empty);
 				case XmlNodeType.XmlDeclaration: return CreateXmlDeclaration ("1.0", null, null);
-				default: throw new ArgumentOutOfRangeException(String.Format("{0}\nParameter name: {1}",
+				default:
+#if NET_2_0
+					throw new ArgumentException (
+#else // makes less sense
+					throw new ArgumentOutOfRangeException (
+#endif
+						String.Format("{0}\nParameter name: {1}",
 							 "Specified argument was out of the range of valid values", type.ToString ()));
 			}
 		}
@@ -497,6 +519,10 @@ namespace System.Xml
 
 		private XmlNodeType GetNodeTypeFromString (string nodeTypeString)
 		{
+#if NET_2_0 // actually should be done in any version
+			if (nodeTypeString == null)
+				throw new ArgumentNullException ("nodeTypeString");
+#endif
 			switch (nodeTypeString) {
 				case "attribute": return XmlNodeType.Attribute;
 				case "cdatasection": return XmlNodeType.CDATA;
@@ -536,8 +562,8 @@ namespace System.Xml
 			case XmlNodeType.Attribute:
 				XmlAttribute srcAtt = node as XmlAttribute;
 				XmlAttribute dstAtt = this.CreateAttribute (srcAtt.Prefix, srcAtt.LocalName, srcAtt.NamespaceURI);
-				for (int i = 0; i < srcAtt.ChildNodes.Count; i++)
-					dstAtt.AppendChild (this.ImportNode (srcAtt.ChildNodes [i], deep));
+				for (XmlNode n = srcAtt.FirstChild; n != null; n = n.NextSibling)
+					dstAtt.AppendChild (this.ImportNode (n, deep));
 				return dstAtt;
 
 			case XmlNodeType.CDATA:
@@ -552,8 +578,8 @@ namespace System.Xml
 			case XmlNodeType.DocumentFragment:
 				XmlDocumentFragment df = this.CreateDocumentFragment ();
 				if(deep)
-					for (int i = 0; i < node.ChildNodes.Count; i++)
-						df.AppendChild (this.ImportNode (node.ChildNodes [i], deep));
+					for (XmlNode n = node.FirstChild; n != null; n = n.NextSibling)
+						df.AppendChild (this.ImportNode (n, deep));
 				return df;
 
 			case XmlNodeType.DocumentType:
@@ -568,8 +594,8 @@ namespace System.Xml
 						dst.SetAttributeNode ((XmlAttribute) this.ImportNode (attr, deep));
 				}
 				if(deep)
-					for (int i = 0; i < src.ChildNodes.Count; i++)
-						dst.AppendChild (this.ImportNode (src.ChildNodes [i], deep));
+					for (XmlNode n = src.FirstChild; n != null; n = n.NextSibling)
+						dst.AppendChild (this.ImportNode (n, deep));
 				return dst;
 
 			case XmlNodeType.EndElement:
@@ -678,8 +704,7 @@ namespace System.Xml
 		{
 			if (NodeChanged != null)
 				NodeChanged (node, new XmlNodeChangedEventArgs
-					(XmlNodeChangedAction.Change,
-					node, parent, oldValue, newValue));
+					(node, parent, parent, oldValue, newValue, XmlNodeChangedAction.Change));
 		}
 
 		internal void onNodeChanging(XmlNode node, XmlNode parent, string oldValue, string newValue)
@@ -688,40 +713,35 @@ namespace System.Xml
 				throw new ArgumentException ("Node is read-only.");
 			if (NodeChanging != null)
 				NodeChanging (node, new XmlNodeChangedEventArgs
-					(XmlNodeChangedAction.Change,
-					node, parent, oldValue, newValue));
+					(node, parent, parent, oldValue, newValue, XmlNodeChangedAction.Change));
 		}
 
 		internal void onNodeInserted (XmlNode node, XmlNode newParent)
 		{
 			if (NodeInserted != null)
 				NodeInserted (node, new XmlNodeChangedEventArgs
-					(XmlNodeChangedAction.Insert,
-					node, null, newParent));
+					(node, null, newParent, null, null, XmlNodeChangedAction.Insert));
 		}
 
 		internal void onNodeInserting (XmlNode node, XmlNode newParent)
 		{
 			if (NodeInserting != null)
 				NodeInserting (node, new XmlNodeChangedEventArgs
-					(XmlNodeChangedAction.Insert,
-					node, null, newParent));
+					(node, null, newParent, null, null, XmlNodeChangedAction.Insert));
 		}
 
 		internal void onNodeRemoved (XmlNode node, XmlNode oldParent)
 		{
 			if (NodeRemoved != null)
 				NodeRemoved (node, new XmlNodeChangedEventArgs
-					(XmlNodeChangedAction.Remove,
-					node, oldParent, null));
+					(node, oldParent, null, null, null, XmlNodeChangedAction.Remove));
 		}
 
 		internal void onNodeRemoving (XmlNode node, XmlNode oldParent)
 		{
 			if (NodeRemoving != null)
 				NodeRemoving (node, new XmlNodeChangedEventArgs
-					(XmlNodeChangedAction.Remove,
-					node, oldParent, null));
+					(node, oldParent, null, null, null, XmlNodeChangedAction.Remove));
 		}
 
 		private void ParseName (string name, out string prefix, out string localName)
@@ -773,12 +793,17 @@ namespace System.Xml
 			}
 		}
 
+		[PermissionSet (SecurityAction.InheritanceDemand, Unrestricted = true)]
 		public virtual XmlNode ReadNode (XmlReader reader)
 		{
 			switch (reader.ReadState) {
 			case ReadState.Interactive:
 				break;
 			case ReadState.Initial:
+#if NET_2_0
+				if (reader.SchemaInfo != null)
+					this.SchemaInfo = new XmlSchemaInfo (reader.SchemaInfo);
+#endif
 				reader.Read ();
 				break;
 			default:
@@ -963,8 +988,8 @@ namespace System.Xml
 
 		public override void WriteContentTo (XmlWriter w)
 		{
-			for (int i = 0; i < ChildNodes.Count; i++)
-				ChildNodes [i].WriteTo (w);
+			for (XmlNode n = FirstChild; n != null; n = n.NextSibling)
+				n.WriteTo (w);
 		}
 
 		public override void WriteTo (XmlWriter w)
@@ -991,19 +1016,26 @@ namespace System.Xml
 			nameTable.Add ("#significant-whitespace");
 		}
 
+		internal void CheckIdTableUpdate (XmlAttribute attr, string oldValue, string newValue)
+		{
+			if (idTable [oldValue] == attr) {
+				idTable.Remove (oldValue);
+				idTable [newValue] = attr;
+			}
+		}
+
 #if NET_2_0
 		public void Validate (ValidationEventHandler handler)
 		{
 			Validate (handler, this,
-				XmlSchemaValidationFlags.IgnoreValidationWarnings);
+				XmlSchemaValidationFlags.ProcessIdentityConstraints);
 		}
 
 		public void Validate (ValidationEventHandler handler,
 			XmlNode node)
 		{
 			Validate (handler, node,
-				XmlSchemaValidationFlags.IgnoreValidationWarnings |
-				XmlSchemaValidationFlags.IgnoreIdentityConstraints);
+				XmlSchemaValidationFlags.ProcessIdentityConstraints);
 		}
 
 		private void Validate (ValidationEventHandler handler,
