@@ -37,6 +37,7 @@ using System.IO;
 using System.Text;
 using System.Xml.XPath;
 #if NET_2_0
+using System.Diagnostics;
 using System.Xml.Schema;
 #endif
 
@@ -108,16 +109,16 @@ namespace System.Xml
 		public virtual string InnerText {
 			get {
 				StringBuilder builder = new StringBuilder ();
-				AppendChildValues (this, builder);
+				AppendChildValues (builder);
 				return builder.ToString ();
 			}
 
 			set { throw new InvalidOperationException ("This node is read only. Cannot be modified."); }
 		}
 
-		private void AppendChildValues (XmlNode parent, StringBuilder builder)
+		private void AppendChildValues (StringBuilder builder)
 		{
-			XmlNode node = parent.FirstChild;
+			XmlNode node = FirstChild;
 
 			while (node != null) {
 				switch (node.NodeType) {
@@ -128,7 +129,7 @@ namespace System.Xml
  					builder.Append (node.Value);
 					break;
 				}
-				AppendChildValues (node, builder);
+				node.AppendChildValues (builder);
 				node = node.NextSibling;
 			}
 		}
@@ -299,7 +300,6 @@ namespace System.Xml
 		}
 
 #if NET_2_0
-		[CLSCompliant(false)]
 		public virtual IXmlSchemaInfo SchemaInfo {
 			get { return null; }
 			internal set { }
@@ -324,12 +324,21 @@ namespace System.Xml
 
 		public abstract XmlNode CloneNode (bool deep);
 
+#if NET_2_0
+		public virtual XPathNavigator CreateNavigator ()
+		{
+			// XmlDocument has overriden definition, so it is safe
+			// to use OwnerDocument here.
+			return OwnerDocument.CreateNavigator (this);
+		}
+#else
 		public XPathNavigator CreateNavigator ()
 		{
 			XmlDocument document = this.NodeType == XmlNodeType.Document ?
 				this as XmlDocument : this.ownerDocument;
 			return document.CreateNavigator (this);
 		}
+#endif
 
 		public IEnumerator GetEnumerator ()
 		{
@@ -338,8 +347,16 @@ namespace System.Xml
 
 		public virtual string GetNamespaceOfPrefix (string prefix)
 		{
-			if (prefix == null)
+			switch (prefix) {
+			case null:
 				throw new ArgumentNullException ("prefix");
+#if NET_2_0
+			case "xml":
+				return XmlNamespaceManager.XmlnsXml;
+			case "xmlns":
+				return XmlNamespaceManager.XmlnsXmlns;
+#endif
+			}
 
 			XmlNode node;
 			switch (NodeType) {
@@ -375,6 +392,15 @@ namespace System.Xml
 
 		public virtual string GetPrefixOfNamespace (string namespaceURI)
 		{
+#if NET_2_0
+			switch (namespaceURI) {
+			case XmlNamespaceManager.XmlnsXml:
+				return XmlNamespaceManager.PrefixXml;
+			case XmlNamespaceManager.XmlnsXmlns:
+				return XmlNamespaceManager.PrefixXmlns;
+			}
+#endif
+
 			XmlNode node;
 			switch (NodeType) {
 			case XmlNodeType.Attribute:
@@ -422,7 +448,7 @@ namespace System.Xml
 			XmlNode argNode = null;
 			if (refChild != null)
 				argNode = refChild.NextSibling;
-			else if (ChildNodes.Count > 0)
+			else if (FirstChild != null)
 				argNode = FirstChild;
 			return InsertBefore (newChild, argNode);
 		}
@@ -459,11 +485,15 @@ namespace System.Xml
 				newChild.ParentNode.RemoveChild (newChild, checkNodeType);
 
 			if (newChild.NodeType == XmlNodeType.DocumentFragment) {
+				// This recursively invokes events. (It is compatible with MS implementation.)
+				while (newChild.FirstChild != null)
+					this.InsertBefore (newChild.FirstChild, refChild);
+				/*
 				int x = newChild.ChildNodes.Count;
 				for (int i = 0; i < x; i++) {
 					XmlNode n = newChild.ChildNodes [0];
-					this.InsertBefore (n, refChild);	// recursively invokes events. (It is compatible with MS implementation.)
-				}
+					this.InsertBefore (n, refChild);					}
+				*/
 			}
 			else {
 				XmlLinkedNode newLinkedChild = (XmlLinkedNode) newChild;
@@ -729,8 +759,7 @@ namespace System.Xml
 
 		internal void SearchDescendantElements (string name, bool matchAll, ArrayList list)
 		{
-			for (int i = 0; i < ChildNodes.Count; i++) {
-				XmlNode n = ChildNodes [i];
+			for (XmlNode n = FirstChild; n != null; n = n.NextSibling) {
 				if (n.NodeType != XmlNodeType.Element)
 					continue;
 				if (matchAll || n.Name == name)
@@ -741,8 +770,7 @@ namespace System.Xml
 
 		internal void SearchDescendantElements (string name, bool matchAllName, string ns, bool matchAllNS, ArrayList list)
 		{
-			for (int i = 0; i < ChildNodes.Count; i++) {
-				XmlNode n = ChildNodes [i];
+			for (XmlNode n = FirstChild; n != null; n = n.NextSibling) {
 				if (n.NodeType != XmlNodeType.Element)
 					continue;
 				if ((matchAllName || n.LocalName == name)
