@@ -31,7 +31,9 @@
 //
 
 using System;
-using System.Collections;
+#if NET_2_0
+using System.Collections.Generic;
+#endif
 using System.Xml;
 using System.Xml.Schema;
 using System.Text;
@@ -84,17 +86,21 @@ namespace System.Xml
 			get { return true; }
 		}
 
+/*
 		public override bool CanReadValueChunk {
 			get { return true; }
 		}
+*/
 #else
 		internal override bool CanReadBinaryContent {
 			get { return true; }
 		}
 
+/*
 		internal override bool CanReadValueChunk {
 			get { return true; }
 		}
+*/
 #endif
 
 		public override bool CanResolveEntity {
@@ -242,7 +248,7 @@ namespace System.Xml
 		}
 
 #if NET_2_0
-		public IDictionary GetNamespacesInScope (XmlNamespaceScope scope)
+		IDictionary<string, string> IXmlNamespaceResolver.GetNamespacesInScope (XmlNamespaceScope scope)
 		{
 			return ((IXmlNamespaceResolver) Current).GetNamespacesInScope (scope);
 		}
@@ -254,7 +260,7 @@ namespace System.Xml
 		}
 
 #if NET_2_0
-		public string LookupPrefix (string ns)
+		string IXmlNamespaceResolver.LookupPrefix (string ns)
 		{
 			return ((IXmlNamespaceResolver) Current).LookupPrefix (ns);
 		}
