@@ -32,6 +32,7 @@
 // WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 //
 using System.Collections;
+using System.Diagnostics;
 using System.IO;
 using System.Text;
 using System.Xml.Schema; // only required for NET_2_0 (SchemaInfo)
@@ -122,15 +123,13 @@ namespace System.Xml
 
 		public abstract bool HasValue { get; }
 
+		public abstract bool IsEmptyElement { get; }
+
 #if NET_2_0
 		public virtual bool IsDefault {
 			get { return false; }
 		}
 
-		public virtual bool IsEmptyElement {
-			get { return false; }
-		}
-
 		public virtual string this [int i] {
 			get { return GetAttribute (i); }
 		}
@@ -145,8 +144,6 @@ namespace System.Xml
 #else
 		public abstract bool IsDefault { get; }
 
-		public abstract bool IsEmptyElement { get; }
-
 		public abstract string this [int i] { get; }
 
 		public abstract string this [string name] { get; }
@@ -156,7 +153,17 @@ namespace System.Xml
 
 		public abstract string LocalName { get; }
 
+#if NET_2_0
+		public virtual string Name {
+			get {
+				return Prefix.Length > 0 ?
+					String.Concat (Prefix, ":", LocalName) :
+					LocalName;
+			}
+		}
+#else
 		public abstract string Name { get; }
+#endif
 
 		public abstract string NamespaceURI { get; }
 
@@ -218,6 +225,16 @@ namespace System.Xml
 			return nameTable;
 		}
 
+		private static XmlParserContext PopulateParserContext (
+			XmlReaderSettings settings, string baseUri)
+		{
+			XmlNameTable nt = PopulateNameTable (settings);
+			return new XmlParserContext (nt,
+				new XmlNamespaceManager (nt),
+				baseUri,
+				XmlSpace.None);
+		}
+
 		private static XmlNodeType GetNodeType (
 			XmlReaderSettings settings)
 		{
@@ -225,12 +242,12 @@ namespace System.Xml
 			return
 				level == ConformanceLevel.Fragment ?
 				XmlNodeType.Element :
-				XmlNodeType.Element;
+				XmlNodeType.Document;
 		}
 
 		public static XmlReader Create (Stream stream)
 		{
-			return Create (stream, null, null, new XmlUrlResolver (), null);
+			return Create (stream, null);
 		}
 
 		public static XmlReader Create (string url)
@@ -240,79 +257,83 @@ namespace System.Xml
 
 		public static XmlReader Create (TextReader reader)
 		{
-			return Create (reader, null, new XmlUrlResolver (), null);
+			return Create (reader, null);
 		}
 
 		public static XmlReader Create (string url, XmlReaderSettings settings)
 		{
-			return Create (url, null, new XmlUrlResolver (), settings);
+			return Create (url, settings, null);
+		}
+
+		public static XmlReader Create (Stream stream, XmlReaderSettings settings)
+		{
+			return Create (stream, settings, String.Empty);
+		}
+
+		public static XmlReader Create (TextReader reader, XmlReaderSettings settings)
+		{
+			return Create (reader, settings, String.Empty);
+		}
+
+		static XmlReaderSettings PopulateSettings (XmlReaderSettings src)
+		{
+			if (src == null)
+				return new XmlReaderSettings ();
+			else
+				return src.Clone ();
+		}
+
+		public static XmlReader Create (Stream stream, XmlReaderSettings settings, string baseUri)
+		{
+			settings = PopulateSettings (settings);
+			return Create (stream, settings,
+				PopulateParserContext (settings, baseUri));
+		}
+
+		public static XmlReader Create (TextReader reader, XmlReaderSettings settings, string baseUri)
+		{
+			settings = PopulateSettings (settings);
+			return Create (reader, settings,
+				PopulateParserContext (settings, baseUri));
 		}
 
 		public static XmlReader Create (XmlReader reader, XmlReaderSettings settings)
 		{
-			return Create (reader, new XmlUrlResolver (), settings);
-		}
-
-		[MonoTODO ("ConformanceLevel")]
-		public static XmlReader Create (XmlReader reader, XmlResolver resolver, XmlReaderSettings settings)
-		{
-			if (settings == null) {
-				settings = new XmlReaderSettings ();
-				settings.XmlResolver = resolver;
-			}
-			XmlReader r = CreateFilteredXmlReader (reader, resolver, settings);
+			settings = PopulateSettings (settings);
+			XmlReader r = CreateFilteredXmlReader (reader, settings);
 			r.settings = settings;
 			return r;
 		}
 
-		[MonoTODO ("ConformanceLevel; Encoding")]
-		public static XmlReader Create (string url, Encoding encoding, XmlResolver resolver, XmlReaderSettings settings)
+		public static XmlReader Create (string url, XmlReaderSettings settings, XmlParserContext context)
 		{
-			if (settings == null) {
-				settings = new XmlReaderSettings ();
-				settings.XmlResolver = resolver;
-			}
-			return CreateCustomizedTextReader (new XmlTextReader (url, PopulateNameTable (settings)), resolver, settings);
-		}
-
-		[MonoTODO ("ConformanceLevel")]
-		public static XmlReader Create (Stream stream, XmlReaderSettings settings, XmlParserContext context)
-		{
-			if (settings == null)
-				settings = new XmlReaderSettings ();
-			return CreateCustomizedTextReader (new XmlTextReader (stream, GetNodeType (settings), context), settings.XmlResolver, settings);
-		}
-
-		[MonoTODO ("ConformanceLevel")]
-		public static XmlReader Create (TextReader reader, string baseUri, XmlResolver resolver, XmlReaderSettings settings)
-		{
-			if (settings == null) {
-				settings = new XmlReaderSettings ();
-				settings.XmlResolver = resolver;
-			}
-			return CreateCustomizedTextReader (new XmlTextReader (baseUri, reader, PopulateNameTable (settings)), resolver, settings);
-		}
-
-		[MonoTODO ("ConformanceLevel")]
-		public static XmlReader Create (Stream stream, string baseUri, Encoding encoding, XmlResolver resolver, XmlReaderSettings settings)
-		{
-			if (settings == null) {
-				settings = new XmlReaderSettings ();
-				settings.XmlResolver = resolver;
-			}
-			XmlNameTable nameTable = PopulateNameTable (settings);
-
+			settings = PopulateSettings (settings);
+			if (context == null)
+				context = PopulateParserContext (settings, url);
 			return CreateCustomizedTextReader (
-				encoding == null ?
-					new XmlTextReader (baseUri, stream, nameTable) :
-					new XmlTextReader (baseUri, new StreamReader (stream, encoding), nameTable),
-				resolver,
+				new XmlTextReader (true, url, GetNodeType (settings), context),
 				settings);
 		}
 
-		private static XmlReader CreateCustomizedTextReader (XmlTextReader reader, XmlResolver resolver, XmlReaderSettings settings)
+		public static XmlReader Create (Stream stream, XmlReaderSettings settings, XmlParserContext context)
 		{
-			reader.XmlResolver = resolver;
+			settings = PopulateSettings (settings);
+			if (context == null)
+				context = PopulateParserContext (settings, String.Empty);
+			return CreateCustomizedTextReader (new XmlTextReader (stream, GetNodeType (settings), context), settings);
+		}
+
+		public static XmlReader Create (TextReader reader, XmlReaderSettings settings, XmlParserContext context)
+		{
+			settings = PopulateSettings (settings);
+			if (context == null)
+				context = PopulateParserContext (settings, String.Empty);
+			return CreateCustomizedTextReader (new XmlTextReader (context.BaseURI, reader, GetNodeType (settings), context), settings);
+		}
+
+		private static XmlReader CreateCustomizedTextReader (XmlTextReader reader, XmlReaderSettings settings)
+		{
+			reader.XmlResolver = settings.XmlResolver;
 			// Normalization is set true by default.
 			reader.Normalization = true;
 
@@ -343,14 +364,26 @@ namespace System.Xml
 			if (settings.NameTable != null)
 				reader.SetNameTable (settings.NameTable);
 
-			XmlReader r = CreateFilteredXmlReader (reader, resolver, settings);
+			XmlReader r = CreateFilteredXmlReader (reader, settings);
 			r.settings = settings;
 			return r;
 		}
 
-		private static XmlReader CreateFilteredXmlReader (XmlReader reader, XmlResolver resolver, XmlReaderSettings settings)
+		private static XmlReader CreateFilteredXmlReader (XmlReader reader, XmlReaderSettings settings)
 		{
-			reader = CreateValidatingXmlReader (reader, resolver, settings);
+			ConformanceLevel conf = ConformanceLevel.Auto;
+			if (reader is XmlTextReader)
+				conf = ((XmlTextReader) reader).Conformance;
+			else if (reader.Settings != null)
+				conf = reader.Settings.ConformanceLevel;
+			else
+				conf = settings.ConformanceLevel;
+			if (settings.ConformanceLevel != ConformanceLevel.Auto &&
+				conf != settings.ConformanceLevel)
+				throw new InvalidOperationException (String.Format ("ConformanceLevel cannot be overwritten by a wrapping XmlReader. The source reader has {0}, while {1} is specified.", conf, settings.ConformanceLevel));
+			settings.ConformanceLevel = conf;
+
+			reader = CreateValidatingXmlReader (reader, settings);
 
 			if (reader.Settings != null ||
 				settings.IgnoreComments ||
@@ -363,7 +396,7 @@ namespace System.Xml
 			}
 		}
 
-		private static XmlReader CreateValidatingXmlReader (XmlReader reader, XmlResolver resolver, XmlReaderSettings settings)
+		private static XmlReader CreateValidatingXmlReader (XmlReader reader, XmlReaderSettings settings)
 		{
 			XmlValidatingReader xvr = null;
 			switch (settings.ValidationType) {
@@ -371,7 +404,7 @@ namespace System.Xml
 				return reader;
 			case ValidationType.DTD:
 				xvr = new XmlValidatingReader (reader);
-				xvr.XmlResolver = resolver;
+				xvr.XmlResolver = settings.XmlResolver;
 				xvr.ValidationType = ValidationType.DTD;
 				break;
 			case ValidationType.Schema:
@@ -389,13 +422,14 @@ namespace System.Xml
 			if (xvr != null)
 				xvr.SetSchemas (settings.Schemas);
 
-			if ((settings.ValidationFlags & XmlSchemaValidationFlags.IgnoreIdentityConstraints) != 0)
+			// Actually I don't think they are treated in DTD validation though...
+			if ((settings.ValidationFlags & XmlSchemaValidationFlags.ProcessIdentityConstraints) == 0)
 				throw new NotImplementedException ();
-			if ((settings.ValidationFlags & XmlSchemaValidationFlags.IgnoreInlineSchema) != 0)
+			if ((settings.ValidationFlags & XmlSchemaValidationFlags.ProcessInlineSchema) != 0)
 				throw new NotImplementedException ();
-			if ((settings.ValidationFlags & XmlSchemaValidationFlags.IgnoreSchemaLocation) != 0)
+			if ((settings.ValidationFlags & XmlSchemaValidationFlags.ProcessSchemaLocation) != 0)
 				throw new NotImplementedException ();
-			if ((settings.ValidationFlags & XmlSchemaValidationFlags.IgnoreValidationWarnings) == 0)
+			if ((settings.ValidationFlags & XmlSchemaValidationFlags.ReportValidationWarnings) == 0)
 				throw new NotImplementedException ();
 
 			return xvr != null ? xvr : reader;
@@ -403,7 +437,12 @@ namespace System.Xml
 #endif
 
 #if NET_2_0
-		public virtual void Dispose ()
+		void IDisposable.Dispose ()
+		{
+			Dispose (false);
+		}
+
+		protected virtual void Dispose (bool disposing)
 		{
 			if (ReadState != ReadState.Closed)
 				Close ();
@@ -452,15 +491,17 @@ namespace System.Xml
 		public abstract string LookupNamespace (string prefix);
 
 #if NET_2_0
-		public virtual string LookupNamespace (string prefix, bool atomizedNames)
-#else
-		internal virtual string LookupNamespace (string prefix, bool atomizedNames)
-#endif
+		public virtual void MoveToAttribute (int i)
 		{
-			return LookupNamespace (prefix);
+			if (i >= AttributeCount)
+				throw new ArgumentOutOfRangeException ();
+			MoveToFirstAttribute ();
+			for (int a = 1; a < i; a++)
+				MoveToNextAttribute ();
 		}
-
+#else
 		public abstract void MoveToAttribute (int i);
+#endif
 
 		public abstract bool MoveToAttribute (string name);
 
@@ -772,9 +813,13 @@ namespace System.Xml
 			get { return typeof (string); }
 		}
 
-		[MonoTODO]
 		public virtual bool ReadToDescendant (string name)
 		{
+			if (ReadState == ReadState.Initial) {
+				MoveToContent ();
+				if (IsStartElement (name))
+					return true;
+			}
 			if (NodeType != XmlNodeType.Element || IsEmptyElement)
 				return false;
 			int depth = Depth;
@@ -784,9 +829,13 @@ namespace System.Xml
 			return false;
 		}
 
-		[MonoTODO]
 		public virtual bool ReadToDescendant (string localName, string namespaceURI)
 		{
+			if (ReadState == ReadState.Initial) {
+				MoveToContent ();
+				if (IsStartElement (localName, namespaceURI))
+					return true;
+			}
 			if (NodeType != XmlNodeType.Element || IsEmptyElement)
 				return false;
 			int depth = Depth;
@@ -796,7 +845,6 @@ namespace System.Xml
 			return false;
 		}
 
-		[MonoTODO]
 		public virtual bool ReadToFollowing (string name)
 		{
 			while (Read ())
@@ -805,7 +853,6 @@ namespace System.Xml
 			return false;
 		}
 
-		[MonoTODO]
 		public virtual bool ReadToFollowing (string localName, string namespaceURI)
 		{
 			while (Read ())
@@ -814,73 +861,87 @@ namespace System.Xml
 			return false;
 		}
 
-		[MonoTODO]
 		public virtual bool ReadToNextSibling (string name)
 		{
-			if (NodeType != XmlNodeType.Element || IsEmptyElement)
+			if (ReadState != ReadState.Interactive)
 				return false;
 			int depth = Depth;
-			for (Skip (); depth < Depth; Skip ())
+			for (Skip (); depth >= Depth; Skip ())
 				if (NodeType == XmlNodeType.Element && name == Name)
 					return true;
 			return false;
 		}
 
-		[MonoTODO]
 		public virtual bool ReadToNextSibling (string localName, string namespaceURI)
 		{
-			if (NodeType != XmlNodeType.Element || IsEmptyElement)
+			if (ReadState != ReadState.Interactive)
 				return false;
 			int depth = Depth;
-			for (Skip (); depth < Depth; Skip ())
+			for (Skip (); depth >= Depth; Skip ())
 				if (NodeType == XmlNodeType.Element && localName == LocalName && namespaceURI == NamespaceURI)
 					return true;
 			return false;
 		}
 
-		[MonoTODO]
 		public virtual XmlReader ReadSubtree ()
 		{
 			return new SubtreeXmlReader (this);
 		}
 
-		[MonoTODO]
-		[Obsolete]
-		public virtual object ReadTypedValue ()
-		{
-			if (NodeType == XmlNodeType.Element)
-				return ReadElementContentAs (ValueType, this as IXmlNamespaceResolver);
-			else
-				return ReadContentAs (ValueType, this as IXmlNamespaceResolver);
-		}
-
 		private string ReadContentString ()
 		{
-			switch (NodeType) {
-			case XmlNodeType.Text:
-			case XmlNodeType.CDATA:
-			case XmlNodeType.SignificantWhitespace:
-			case XmlNodeType.Whitespace:
-				break;
-			default:
-				throw new InvalidOperationException (String.Format ("This method does not support node type {0}.", NodeType));
-			}
-			return ReadString ();
+			string value = String.Empty;
+			do {
+				switch (NodeType) {
+				case XmlNodeType.Element:
+					throw XmlError ("Child element is not expected in this operation.");
+				case XmlNodeType.EndElement:
+					return value;
+				case XmlNodeType.Text:
+				case XmlNodeType.CDATA:
+				case XmlNodeType.SignificantWhitespace:
+				case XmlNodeType.Whitespace:
+					value += Value;
+					break;
+				}
+			} while (Read ());
+			throw XmlError ("Unexpected end of document.");
 		}
 
 		[MonoTODO]
+		public virtual object ReadElementContentAsObject ()
+		{
+			return ReadElementContentAs (ValueType, null);
+		}
+
+		[MonoTODO]
+		public virtual object ReadElementContentAsObject (string localName, string namespaceURI)
+		{
+			return ReadElementContentAs (ValueType, null, localName, namespaceURI);
+		}
+
+		[MonoTODO]
+		public virtual object ReadContentAsObject ()
+		{
+			return ReadContentAs (ValueType, null);
+		}
+
 		public virtual object ReadElementContentAs (Type type, IXmlNamespaceResolver resolver)
 		{
-			return ValueAs (ReadElementString (), type, resolver);
+			ReadStartElement ();
+			object obj = ValueAs (ReadContentAsString (), type, resolver);
+			ReadEndElement ();
+			return obj;
 		}
 
-		[MonoTODO]
 		public virtual object ReadElementContentAs (Type type, IXmlNamespaceResolver resolver, string localName, string namespaceURI)
 		{
-			return ValueAs (ReadElementString (localName, namespaceURI), type, resolver);
+			ReadStartElement (localName, namespaceURI);
+			object obj = ReadContentAs (type, resolver);
+			ReadEndElement ();
+			return obj;
 		}
 
-		[MonoTODO]
 		public virtual object ReadContentAs (Type type, IXmlNamespaceResolver resolver)
 		{
 			return ValueAs (ReadContentString (), type, resolver);
@@ -889,118 +950,246 @@ namespace System.Xml
 		private object ValueAs (string text, Type type, IXmlNamespaceResolver resolver)
 		{
 			try {
-				if (type == typeof (XmlQualifiedName))
-					return XmlQualifiedName.Parse (text, resolver);
+				if (type == typeof (XmlQualifiedName)) {
+					if (resolver != null)
+						return XmlQualifiedName.Parse (text, resolver);
+					else
+						return XmlQualifiedName.Parse (text, this);
+				}
 
 				switch (Type.GetTypeCode (type)) {
 				case TypeCode.Boolean:
-					return ReadContentAsBoolean ();
+					return XQueryConvert.StringToBoolean (text);
 				case TypeCode.DateTime:
-					return ReadContentAsDateTime ();
+					return XQueryConvert.StringToDateTime (text);
+				case TypeCode.Decimal:
+					return XQueryConvert.StringToDecimal (text);
 				case TypeCode.Double:
-					return ReadContentAsDouble ();
+					return XQueryConvert.StringToDouble (text);
 				case TypeCode.Int32:
-					return ReadContentAsInt ();
+					return XQueryConvert.StringToInt (text);
+				case TypeCode.Int64:
+					return XQueryConvert.StringToInteger (text);
+				case TypeCode.Single:
+					return XQueryConvert.StringToFloat (text);
 				case TypeCode.String:
-					return ReadContentAsString ();
+					return text;
 				}
 			} catch (Exception ex) {
-				return new FormatException (String.Format ("Current text value '{0}' is not acceptable for specified type '{1}'.", text, type));
+				throw XmlError (String.Format ("Current text value '{0}' is not acceptable for specified type '{1}'. {2}", text, type, ex != null ? ex.Message : String.Empty), ex);
 			}
 			throw new ArgumentException (String.Format ("Specified type '{0}' is not supported.", type));
 		}
 
-		[MonoTODO]
 		public virtual bool ReadElementContentAsBoolean ()
 		{
-			return XQueryConvert.StringToBoolean (ReadElementString ());
+			try {
+				return XQueryConvert.StringToBoolean (ReadElementContentAsString ());
+			} catch (FormatException ex) {
+				throw XmlError ("Typed value is invalid.", ex);
+			}
 		}
 
-		[MonoTODO]
 		public virtual DateTime ReadElementContentAsDateTime ()
 		{
-			return XQueryConvert.StringToDateTime (ReadElementString ());
+			try {
+				return XQueryConvert.StringToDateTime (ReadElementContentAsString ());
+			} catch (FormatException ex) {
+				throw XmlError ("Typed value is invalid.", ex);
+			}
+		}
+
+		public virtual decimal ReadElementContentAsDecimal ()
+		{
+			try {
+				return XQueryConvert.StringToDecimal (ReadElementContentAsString ());
+			} catch (FormatException ex) {
+				throw XmlError ("Typed value is invalid.", ex);
+			}
 		}
 
-		[MonoTODO]
 		public virtual double ReadElementContentAsDouble ()
 		{
-			return XQueryConvert.StringToDouble (ReadElementString ());
+			try {
+				return XQueryConvert.StringToDouble (ReadElementContentAsString ());
+			} catch (FormatException ex) {
+				throw XmlError ("Typed value is invalid.", ex);
+			}
+		}
+
+		public virtual float ReadElementContentAsFloat ()
+		{
+			try {
+				return XQueryConvert.StringToFloat (ReadElementContentAsString ());
+			} catch (FormatException ex) {
+				throw XmlError ("Typed value is invalid.", ex);
+			}
 		}
 
-		[MonoTODO]
 		public virtual int ReadElementContentAsInt ()
 		{
-			return XQueryConvert.StringToInt (ReadElementString ());
+			try {
+				return XQueryConvert.StringToInt (ReadElementContentAsString ());
+			} catch (FormatException ex) {
+				throw XmlError ("Typed value is invalid.", ex);
+			}
 		}
 
-		[MonoTODO]
 		public virtual long ReadElementContentAsLong ()
 		{
-			return XQueryConvert.StringToInteger (ReadElementString ());
+			try {
+				return XQueryConvert.StringToInteger (ReadElementContentAsString ());
+			} catch (FormatException ex) {
+				throw XmlError ("Typed value is invalid.", ex);
+			}
 		}
 
-		[MonoTODO]
 		public virtual string ReadElementContentAsString ()
 		{
-			return ReadElementString ();
+			ReadStartElement ();
+			if (IsEmptyElement)
+				return String.Empty;
+			string s = ReadContentString ();
+			ReadEndElement ();
+			return s;
 		}
 
-		[MonoTODO]
 		public virtual bool ReadElementContentAsBoolean (string localName, string namespaceURI)
 		{
-			return XQueryConvert.StringToBoolean (ReadElementString (localName, namespaceURI));
+			try {
+				return XQueryConvert.StringToBoolean (ReadElementContentAsString (localName, namespaceURI));
+			} catch (FormatException ex) {
+				throw XmlError ("Typed value is invalid.", ex);
+			}
 		}
 
-		[MonoTODO]
 		public virtual DateTime ReadElementContentAsDateTime (string localName, string namespaceURI)
 		{
-			return XQueryConvert.StringToDateTime (ReadElementString (localName, namespaceURI));
+			try {
+				return XQueryConvert.StringToDateTime (ReadElementContentAsString (localName, namespaceURI));
+			} catch (FormatException ex) {
+				throw XmlError ("Typed value is invalid.", ex);
+			}
+		}
+
+		public virtual decimal ReadElementContentAsDecimal (string localName, string namespaceURI)
+		{
+			try {
+				return XQueryConvert.StringToDecimal (ReadElementContentAsString (localName, namespaceURI));
+			} catch (FormatException ex) {
+				throw XmlError ("Typed value is invalid.", ex);
+			}
 		}
 
-		[MonoTODO]
 		public virtual double ReadElementContentAsDouble (string localName, string namespaceURI)
 		{
-			return XQueryConvert.StringToDouble (ReadElementString (localName, namespaceURI));
+			try {
+				return XQueryConvert.StringToDouble (ReadElementContentAsString (localName, namespaceURI));
+			} catch (FormatException ex) {
+				throw XmlError ("Typed value is invalid.", ex);
+			}
+		}
+
+		public virtual float ReadElementContentAsFloat (string localName, string namespaceURI)
+		{
+			try {
+				return XQueryConvert.StringToFloat (ReadElementContentAsString (localName, namespaceURI));
+			} catch (FormatException ex) {
+				throw XmlError ("Typed value is invalid.", ex);
+			}
 		}
 
-		[MonoTODO]
 		public virtual int ReadElementContentAsInt (string localName, string namespaceURI)
 		{
-			return XQueryConvert.StringToInt (ReadElementString (localName, namespaceURI));
+			try {
+				return XQueryConvert.StringToInt (ReadElementContentAsString (localName, namespaceURI));
+			} catch (FormatException ex) {
+				throw XmlError ("Typed value is invalid.", ex);
+			}
+		}
+
+		public virtual long ReadElementContentAsLong (string localName, string namespaceURI)
+		{
+			try {
+				return XQueryConvert.StringToInteger (ReadElementContentAsString (localName, namespaceURI));
+			} catch (FormatException ex) {
+				throw XmlError ("Typed value is invalid.", ex);
+			}
 		}
 
-		[MonoTODO]
 		public virtual string ReadElementContentAsString (string localName, string namespaceURI)
 		{
-			return ReadElementString (localName, namespaceURI);
+			ReadStartElement (localName, namespaceURI);
+			if (IsEmptyElement)
+				return String.Empty;
+			string s = ReadContentString ();
+			ReadEndElement ();
+			return s;
 		}
 
-		[MonoTODO]
 		public virtual bool ReadContentAsBoolean ()
 		{
-			return XQueryConvert.StringToBoolean (ReadContentString ());
+			try {
+				return XQueryConvert.StringToBoolean (ReadContentString ());
+			} catch (FormatException ex) {
+				throw XmlError ("Typed value is invalid.", ex);
+			}
 		}
 
-		[MonoTODO]
 		public virtual DateTime ReadContentAsDateTime ()
 		{
-			return XQueryConvert.StringToDateTime (ReadContentString ());
+			try {
+				return XQueryConvert.StringToDateTime (ReadContentString ());
+			} catch (FormatException ex) {
+				throw XmlError ("Typed value is invalid.", ex);
+			}
+		}
+
+		public virtual decimal ReadContentAsDecimal ()
+		{
+			try {
+				return XQueryConvert.StringToDecimal (ReadContentString ());
+			} catch (FormatException ex) {
+				throw XmlError ("Typed value is invalid.", ex);
+			}
 		}
 
-		[MonoTODO]
 		public virtual double ReadContentAsDouble ()
 		{
-			return XQueryConvert.StringToDouble (ReadContentString ());
+			try {
+				return XQueryConvert.StringToDouble (ReadContentString ());
+			} catch (FormatException ex) {
+				throw XmlError ("Typed value is invalid.", ex);
+			}
+		}
+
+		public virtual float ReadContentAsFloat ()
+		{
+			try {
+				return XQueryConvert.StringToFloat (ReadContentString ());
+			} catch (FormatException ex) {
+				throw XmlError ("Typed value is invalid.", ex);
+			}
 		}
 
-		[MonoTODO]
 		public virtual int ReadContentAsInt ()
 		{
-			return XQueryConvert.StringToInt (ReadContentString ());
+			try {
+				return XQueryConvert.StringToInt (ReadContentString ());
+			} catch (FormatException ex) {
+				throw XmlError ("Typed value is invalid.", ex);
+			}
+		}
+
+		public virtual long ReadContentAsLong ()
+		{
+			try {
+				return XQueryConvert.StringToInteger (ReadContentString ());
+			} catch (FormatException ex) {
+				throw XmlError ("Typed value is invalid.", ex);
+			}
 		}
 
-		[MonoTODO]
 		public virtual string ReadContentAsString ()
 		{
 			return ReadContentString ();
@@ -1088,6 +1277,11 @@ namespace System.Xml
 			return new XmlException (this as IXmlLineInfo, BaseURI, message);
 		}
 
+		private XmlException XmlError (string message, Exception innerException)
+		{
+			return new XmlException (this as IXmlLineInfo, BaseURI, message);
+		}
+
 		#endregion
 	}
 }
