@@ -7,8 +7,7 @@
 //   Atsushi Enomoto  (ginga@kit.hi-ho.ne.jp)
 //
 // (C) 2001, 2002 Jason Diamond  http://injektilo.org/
-//
-
+// Copyright (C) 2005 Novell, Inc (http://www.novell.com)
 //
 // Permission is hereby granted, free of charge, to any person obtaining
 // a copy of this software and associated documentation files (the
@@ -32,8 +31,12 @@
 
 using System;
 using System.Collections;
+#if NET_2_0
+using System.Collections.Generic;
+#endif
 using System.Globalization;
 using System.IO;
+using System.Security.Permissions;
 using System.Text;
 using System.Xml.Schema;
 using Mono.Xml;
@@ -51,6 +54,7 @@ namespace System.Xml
 	internal class XmlTextReader : XmlReader,
 		IXmlLineInfo, IXmlNamespaceResolver, IHasXmlParserContext
 #else
+	[PermissionSet (SecurityAction.InheritanceDemand, Unrestricted = true)]
 	public class XmlTextReader : XmlReader, IXmlLineInfo, IHasXmlParserContext
 #endif
 	{
@@ -97,14 +101,13 @@ namespace System.Xml
 
 		public XmlTextReader (string url, XmlNameTable nt)
 		{
-			Uri uri = resolver.ResolveUri (null, url);
-			string uriString = uri != null ? uri.ToString () : String.Empty;
-			Stream s = resolver.GetEntity (uri, null, typeof (Stream)) as Stream;
+			string uriString;
+			Stream stream = GetStreamFromUrl (url, out uriString);
 			XmlParserContext ctx = new XmlParserContext (nt,
 				new XmlNamespaceManager (nt),
 				String.Empty,
 				XmlSpace.None);
-			this.InitializeContext (uriString, ctx, new XmlStreamReader (s), XmlNodeType.Document);
+			this.InitializeContext (uriString, ctx, new XmlStreamReader (stream), XmlNodeType.Document);
 		}
 
 		public XmlTextReader (TextReader input, XmlNameTable nt)
@@ -112,6 +115,15 @@ namespace System.Xml
 		{
 		}
 
+		// This is used in XmlReader.Create() to indicate that string
+		// argument is uri, not an xml fragment.
+		internal XmlTextReader (bool dummy, string url, XmlNodeType fragType, XmlParserContext context)
+		{
+			string uriString;
+			Stream stream = GetStreamFromUrl (url, out uriString);
+			this.InitializeContext (uriString, context, new XmlStreamReader (stream), fragType);
+		}
+
 		public XmlTextReader (Stream xmlFragment, XmlNodeType fragType, XmlParserContext context)
 			: this (context != null ? context.BaseURI : String.Empty,
 				new XmlStreamReader (xmlFragment),
@@ -120,6 +132,11 @@ namespace System.Xml
 		{
 		}
 
+		internal XmlTextReader (string baseURI, TextReader xmlFragment, XmlNodeType fragType)
+			: this (baseURI, xmlFragment, fragType, null)
+		{
+		}
+
 		public XmlTextReader (string url, Stream input, XmlNameTable nt)
 			: this (url, new XmlStreamReader (input), nt)
 		{
@@ -138,11 +155,18 @@ namespace System.Xml
 		{
 		}
 
-		XmlTextReader (string url, TextReader fragment, XmlNodeType fragType, XmlParserContext context)
+		internal XmlTextReader (string url, TextReader fragment, XmlNodeType fragType, XmlParserContext context)
 		{
 			InitializeContext (url, context, fragment, fragType);
 		}
 
+		private Stream GetStreamFromUrl (string url, out string absoluteUriString)
+		{
+			Uri uri = resolver.ResolveUri (null, url);
+			absoluteUriString = uri != null ? uri.ToString () : String.Empty;
+			return resolver.GetEntity (uri, null, typeof (Stream)) as Stream;
+		}
+
 		#endregion
 
 		#region Properties
@@ -176,7 +200,7 @@ namespace System.Xml
 #endif
 
 		internal bool CharacterChecking {
-			get { return checkCharacters && normalization; }
+			get { return checkCharacters; }
 			set { checkCharacters = value; }
 		}
 
@@ -397,10 +421,15 @@ namespace System.Xml
 		}
 
 #if NET_2_0
-		public IDictionary GetNamespacesInScope (XmlNamespaceScope scope)
+		public IDictionary<string, string> GetNamespacesInScope (XmlNamespaceScope scope)
 		{
 			return parserContext.NamespaceManager.GetNamespacesInScope (scope);
 		}
+
+		IDictionary<string, string> IXmlNamespaceResolver.GetNamespacesInScope (XmlNamespaceScope scope)
+		{
+			return GetNamespacesInScope (scope);
+		}
 #endif
 
 		public TextReader GetRemainder ()
@@ -424,13 +453,11 @@ namespace System.Xml
 			return LookupNamespace (prefix, false);
 		}
 
-#if NET_2_0
-		public override string LookupNamespace (string prefix, bool atomizedName)
-#else
-		internal override string LookupNamespace (string prefix, bool atomizedName)
-#endif
+		private string LookupNamespace (string prefix, bool atomizedNames)
 		{
-			return parserContext.NamespaceManager.LookupNamespace (prefix, atomizedName);
+			string s = parserContext.NamespaceManager.LookupNamespace (
+				prefix, atomizedNames);
+			return s == String.Empty ? null : s;
 		}
 
 #if NET_2_0
@@ -521,6 +548,7 @@ namespace System.Xml
 			if (startNodeType == XmlNodeType.Attribute) {
 				if (currentAttribute == 0)
 					return false;	// already read.
+				SkipTextDeclaration ();
 				ClearAttributes ();
 				IncrementAttributeToken ();
 				ReadAttributeValueTokens ('"');
@@ -529,6 +557,8 @@ namespace System.Xml
 				readState = ReadState.Interactive;
 				return true;
 			}
+			if (readState == ReadState.Initial && currentState == XmlNodeType.Element)
+				SkipTextDeclaration ();
 
 			if (Binary != null)
 				Binary.Reset ();
@@ -706,7 +736,7 @@ namespace System.Xml
 						return valueCache;
 					if (ValueBufferStart >= 0) {
 //Console.WriteLine (NodeType + " / " + ValueBuffer.Length + " / " + ValueBufferStart + " / " + ValueBufferEnd);
-						valueCache = new string (Reader.valueBuffer, ValueBufferStart, ValueBufferEnd - ValueBufferStart);
+						valueCache = Reader.valueBuffer.ToString (ValueBufferStart, ValueBufferEnd - ValueBufferStart);
 						return valueCache;
 					}
 					switch (NodeType) {
@@ -776,7 +806,7 @@ namespace System.Xml
 						}
 					}
 
-					valueCache = tmpBuilder.ToString ();
+					valueCache = tmpBuilder.ToString (0, tmpBuilder.Length);
 					return valueCache;
 				}
 
@@ -846,10 +876,7 @@ namespace System.Xml
 		private int nameCapacity;
 		private const int initialNameCapacity = 32;
 
-		private char [] valueBuffer;
-		private int valueLength;
-		private int valueCapacity;
-		private const int initialValueCapacity = 256;
+		private StringBuilder valueBuffer;
 
 		private char [] currentTagBuffer;
 		private int currentTagLength;
@@ -924,9 +951,7 @@ namespace System.Xml
 			nameLength = 0;
 			nameCapacity = initialNameCapacity;
 
-			valueBuffer = new char [initialValueCapacity];
-			valueLength = 0;
-			valueCapacity = initialValueCapacity;
+			valueBuffer = new StringBuilder ();
 
 			currentTagBuffer = new char [initialCurrentTagCapacity];
 			currentTagLength = 0;
@@ -988,12 +1013,10 @@ namespace System.Xml
 			switch (fragType) {
 			case XmlNodeType.Attribute:
 				reader = new StringReader (fragment.ReadToEnd ().Replace ("\"", "&quot;"));
-				SkipTextDeclaration ();
 				break;
 			case XmlNodeType.Element:
 				currentState = XmlNodeType.Element;
 				allowMultipleRoot = true;
-				SkipTextDeclaration ();
 				break;
 			case XmlNodeType.Document:
 				break;
@@ -1003,8 +1026,8 @@ namespace System.Xml
 		}
 
 #if NET_2_0
-		[MonoTODO ("Test")]
 		internal ConformanceLevel Conformance {
+			get { return allowMultipleRoot ? ConformanceLevel.Fragment : ConformanceLevel.Document; }
 			set {
 				if (value == ConformanceLevel.Fragment) {
 					currentState = XmlNodeType.Element;
@@ -1080,23 +1103,22 @@ namespace System.Xml
 		private int PeekChar ()
 		{
 			if (peekCharsLength == peekCharsIndex) {
-				if (!ReadTextReader ())
+				if (!ReadTextReader (-1))
 					return -1;
-					return PeekChar ();
+				return PeekChar ();
 			}
 
 			char c = peekChars [peekCharsIndex];
 			if (c == 0)
 				return -1;
-			if (!char.IsSurrogate (c))
+			if (c < 0xD800 || 0xDFFF < c)
 				return c;
-			if (peekCharsLength == peekCharsIndex+1) {
-				if (!ReadTextReader ())
+			if (peekCharsLength == peekCharsIndex + 1) {
+				if (!ReadTextReader (c))
 					//FIXME: copy MS.NET behaviour when unpaired surrogate found
-			return peekChars [peekCharsIndex];
-				return PeekChar ();
+					return c;
 			}
-            
+
 			char highhalfChar = peekChars [peekCharsIndex];
 			char lowhalfChar = peekChars [peekCharsIndex+1];
 
@@ -1128,10 +1150,14 @@ namespace System.Xml
 			return ch;
 		}
 
-		private bool ReadTextReader ()
+		private bool ReadTextReader (int remained)
 		{
 			peekCharsIndex = 0;
-			peekCharsLength = reader.Read (peekChars, 0, peekCharCapacity);
+			if (remained >= 0)
+				peekChars [0] = (char) remained;
+			int offset = remained >= 0 ? 1 : 0;
+			peekCharsLength = reader.Read (peekChars, offset,
+				peekCharCapacity - offset) + offset;
 			return (peekCharsLength != 0);
 		}
 
@@ -1265,6 +1291,12 @@ namespace System.Xml
 				attributeTokens [i].FillNamespace ();
 
 			// quick name check
+			if (namespaces)
+				for (int i = 0; i < attributeCount; i++)
+					if (attributeTokens [i].Prefix == "xmlns" &&
+						attributeTokens [i].Value == String.Empty)
+						throw NotWFError ("Empty namespace URI cannot be mapped to non-empty prefix.");
+
 			for (int i = 0; i < attributeCount; i++) {
 				for (int j = i + 1; j < attributeCount; j++)
 					if (Object.ReferenceEquals (attributeTokens [i].Name, attributeTokens [j].Name) ||
@@ -1416,10 +1448,10 @@ namespace System.Xml
 
 		private void AppendSurrogatePairNameChar (int ch)
 		{
-			nameBuffer [nameLength++] = (char) (ch / 0x10000 + 0xD800 - 1);
+			nameBuffer [nameLength++] = (char) ((ch - 0x10000) / 0x400 + 0xD800);
 			if (nameLength == nameCapacity)
 				ExpandNameCapacity ();
-			nameBuffer [nameLength++] = (char) (ch % 0x10000 + 0xDC00);
+			nameBuffer [nameLength++] = (char) ((ch - 0x10000) % 0x400 + 0xDC00);
 		}
 
 		private void ExpandNameCapacity ()
@@ -1432,38 +1464,28 @@ namespace System.Xml
 
 		private void AppendValueChar (int ch)
 		{
-			if (valueLength == valueCapacity)
-				ExpandValueCapacity ();
 			if (ch < Char.MaxValue)
-				valueBuffer [valueLength++] = (char) ch;
+				valueBuffer.Append ((char) ch);
 			else
 				AppendSurrogatePairValueChar (ch);
 		}
 
 		private void AppendSurrogatePairValueChar (int ch)
 		{
-			valueBuffer [valueLength++] = (char) (ch / 0x10000 + 0xD800 - 1);
-			if (valueLength == valueCapacity)
-				ExpandValueCapacity ();
-			valueBuffer [valueLength++] = (char) (ch % 0x10000 + 0xDC00);
-		}
-
-		private void ExpandValueCapacity ()
-		{
-			valueCapacity = valueCapacity * 2;
-			char [] oldValueBuffer = valueBuffer;
-			valueBuffer = new char [valueCapacity];
-			Array.Copy (oldValueBuffer, valueBuffer, valueLength);
+			valueBuffer.Append ((char) ((ch - 0x10000) / 0x400 + 0xD800));
+			valueBuffer.Append ((char) ((ch - 0x10000) % 0x400 + 0xDC00));
 		}
 
 		private string CreateValueString ()
 		{
-			return new string (valueBuffer, 0, valueLength);
+			return (valueBuffer.Capacity < 100) ?
+				valueBuffer.ToString (0, valueBuffer.Length) :
+				valueBuffer.ToString ();
 		}
 
 		private void ClearValueBuffer ()
 		{
-			valueLength = 0;
+			valueBuffer.Length = 0;
 		}
 
 		private void AppendCurrentTagChar (int ch)
@@ -1473,10 +1495,10 @@ namespace System.Xml
 			if (ch < Char.MaxValue)
 				currentTagBuffer [currentTagLength++] = (char) ch;
 			else {
-				currentTagBuffer [currentTagLength++] = (char) (ch / 0x10000 + 0xD800 - 1);
+				currentTagBuffer [currentTagLength++] = (char) ((ch - 0x10000) / 0x400 + 0xD800);
 				if (currentTagLength == currentTagCapacity)
 					ExpandCurrentTagCapacity ();
-				currentTagBuffer [currentTagLength++] = (char) (ch % 0x10000 + 0xDC00);
+				currentTagBuffer [currentTagLength++] = (char) ((ch - 0x10000) % 0x400 + 0xDC00);
 			}
 		}
 
@@ -1523,10 +1545,8 @@ namespace System.Xml
 				// FIXME: it might be optimized by the JIT later,
 //				AppendValueChar (ch);
 				{
-					if (valueLength == valueCapacity)
-						ExpandValueCapacity ();
 					if (ch < Char.MaxValue)
-						valueBuffer [valueLength++] = (char) ch;
+						valueBuffer.Append ((char) ch);
 					else
 						AppendSurrogatePairValueChar (ch);
 				}
@@ -1544,7 +1564,7 @@ namespace System.Xml
 				notWhitespace = true;
 			}
 
-			if (returnEntityReference && valueLength == 0) {
+			if (returnEntityReference && valueBuffer.Length == 0) {
 				SetEntityReferenceProperties ();
 			} else {
 				XmlNodeType nodeType = notWhitespace ? XmlNodeType.Text :
@@ -1614,7 +1634,8 @@ namespace System.Xml
 			ReadChar (); // ';'
 
 			// There is no way to save surrogate pairs...
-			if (CharacterChecking && XmlChar.IsInvalid (value))
+			if (CharacterChecking && Normalization &&
+				XmlChar.IsInvalid (value))
 				throw NotWFError ("Referenced character was not allowed in XML. Normalization is " + normalization + ", checkCharacters = " + checkCharacters);
 			return value;
 		}
@@ -1758,7 +1779,7 @@ namespace System.Xml
 			bool isNewToken = true;
 			bool loop = true;
 			int ch = 0;
-			currentAttributeValueToken.ValueBufferStart = valueLength;
+			currentAttributeValueToken.ValueBufferStart = valueBuffer.Length;
 			while (loop) {
 				ch = ReadChar ();
 				if (ch == quoteChar)
@@ -1766,7 +1787,7 @@ namespace System.Xml
 
 				if (incrementToken) {
 					IncrementAttributeValueToken ();
-					currentAttributeValueToken.ValueBufferStart = valueLength;
+					currentAttributeValueToken.ValueBufferStart = valueBuffer.Length;
 					currentAttributeValueToken.LineNumber = line;
 					currentAttributeValueToken.LinePosition = column;
 					incrementToken = false;
@@ -1808,8 +1829,6 @@ namespace System.Xml
 					if (PeekChar () == '#') {
 						ReadChar ();
 						ch = ReadCharacterReference ();
-						if (CharacterChecking && XmlChar.IsInvalid (ch))
-							throw NotWFError ("Not allowed character was found.");
 						AppendValueChar (ch);
 						break;
 					}
@@ -1827,7 +1846,7 @@ namespace System.Xml
 						} else
 #endif
 						{
-							currentAttributeValueToken.ValueBufferEnd = valueLength;
+							currentAttributeValueToken.ValueBufferEnd = valueBuffer.Length;
 							currentAttributeValueToken.NodeType = XmlNodeType.Text;
 							if (!isNewToken)
 								IncrementAttributeValueToken ();
@@ -1846,10 +1865,8 @@ namespace System.Xml
 					// FIXME: it might be optimized by the JIT later,
 //					AppendValueChar (ch);
 					{
-						if (valueLength == valueCapacity)
-							ExpandValueCapacity ();
 						if (ch < Char.MaxValue)
-							valueBuffer [valueLength++] = (char) ch;
+							valueBuffer.Append ((char) ch);
 						else
 							AppendSurrogatePairValueChar (ch);
 					}
@@ -1859,7 +1876,7 @@ namespace System.Xml
 				isNewToken = false;
 			}
 			if (!incrementToken) {
-				currentAttributeValueToken.ValueBufferEnd = valueLength;
+				currentAttributeValueToken.ValueBufferEnd = valueBuffer.Length;
 				currentAttributeValueToken.NodeType = XmlNodeType.Text;
 			}
 			currentAttributeToken.ValueTokenEndIndex = currentAttributeValue;
@@ -1935,7 +1952,7 @@ namespace System.Xml
 		// The reader is positioned after "<?xml "
 		private void ReadXmlDeclaration ()
 		{
-			if (currentState != XmlNodeType.None) {
+			if (!allowMultipleRoot && currentState != XmlNodeType.None) {
 				throw NotWFError ("XML declaration cannot appear in this state.");
 			}
 			currentState = XmlNodeType.XmlDeclaration;
@@ -1964,6 +1981,16 @@ namespace System.Xml
 			if (message != null)
 				throw NotWFError (message);
 
+			string encoding = GetAttribute ("encoding");
+			if (encoding != null) {
+				if (!XmlChar.IsValidIANAEncoding (encoding))
+					throw new XmlException (String.Format ("Encoding name must be a valid IANA name: {0}", encoding));
+				if (reader is XmlStreamReader)
+					parserContext.Encoding = ((XmlStreamReader) reader).Encoding;
+				else
+					parserContext.Encoding = Encoding.Unicode;
+			}
+
 			SetProperties (
 				XmlNodeType.XmlDeclaration, // nodeType
 				"xml", // name
@@ -1979,8 +2006,6 @@ namespace System.Xml
 
 		private void SkipTextDeclaration ()
 		{
-			this.currentState = XmlNodeType.Element;
-
 			if (PeekChar () != '<')
 				return;
 
@@ -1999,7 +2024,7 @@ namespace System.Xml
 					ReadChar ();
 			}
 			if (new string (peekChars, 2, 4) != "xml ") {
-				if (new string (peekChars, 2, 3).ToLower (CultureInfo.InvariantCulture) == "xml") {
+				if (new string (peekChars, 2, 4).ToLower (CultureInfo.InvariantCulture) == "xml ") {
 					throw NotWFError ("Processing instruction name must not be character sequence 'X' 'M' 'L' with case insensitivity.");
 				}
 				peekCharsIndex = 0;
@@ -2058,8 +2083,13 @@ namespace System.Xml
 				}
 				// Encoding value should be checked inside XmlInputStream.
 			}
-			else
+#if NET_2_0
+			// this condition is to check if this instance is
+			// not created by XmlReader.Create() (which just
+			// omits strict text declaration check).
+			else if (Conformance == ConformanceLevel.Auto)
 				throw NotWFError ("Encoding declaration is mandatory in text declaration.");
+#endif
 
 			Expect ("?>");
 		}
@@ -2169,10 +2199,8 @@ namespace System.Xml
 				// FIXME: it might be optimized by the JIT later,
 //				AppendValueChar (ch);
 				{
-					if (valueLength == valueCapacity)
-						ExpandValueCapacity ();
 					if (ch < Char.MaxValue)
-						valueBuffer [valueLength++] = (char) ch;
+						valueBuffer.Append ((char) ch);
 					else
 						AppendSurrogatePairValueChar (ch);
 				}
@@ -2623,10 +2651,8 @@ namespace System.Xml
 //				AppendValueChar (ReadChar ());
 				{
 					ch = ReadChar ();
-					if (valueLength == valueCapacity)
-						ExpandValueCapacity ();
 					if (ch < Char.MaxValue)
-						valueBuffer [valueLength++] = (char) ch;
+						valueBuffer.Append ((char) ch);
 					else
 						AppendSurrogatePairValueChar (ch);
 				}
@@ -2684,8 +2710,8 @@ namespace System.Xml
 					if (c < Char.MaxValue)
 						buffer [bufIndex++] = (char) c;
 					else {
-						buffer [bufIndex++] = (char) (c / 0x10000 + 0xD800 - 1);
-						buffer [bufIndex++] = (char) (c % 0x10000 + 0xDC00);
+						buffer [bufIndex++] = (char) ((c - 0x10000) / 0x400 + 0xD800);
+						buffer [bufIndex++] = (char) ((c - 0x10000) % 0x400 + 0xDC00);
 					}
 					break;
 				}
