@@ -7,8 +7,7 @@
 //
 // Copyright (C) Tim Coleman, 2002
 // (C)2003 Atsushi Enomoto
-//
-
+// Copyright (C) 2005 Novell, Inc (http://www.novell.com)
 //
 // Permission is hereby granted, free of charge, to any person obtaining
 // a copy of this software and associated documentation files (the
@@ -30,8 +29,11 @@
 // WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 //
 
-using System.Collections;
+#if NET_2_0
+using System.Collections.Generic;
+#endif
 using System.IO;
+using System.Security.Permissions;
 using System.Text;
 using System.Xml.Schema;
 using Mono.Xml;
@@ -39,6 +41,7 @@ using Mono.Xml.Schema;
 
 namespace System.Xml
 {
+	[PermissionSet (SecurityAction.InheritanceDemand, Unrestricted = true)]
 #if NET_2_0
 	[Obsolete("Use XmlReader created by XmlReader.Create() method using"
 		+ " appropriate XmlReaderSettings instead.")]
@@ -100,6 +103,12 @@ namespace System.Xml
 			get { return validatingReader == null ? sourceReader.BaseURI : validatingReader.BaseURI; }
 		}
 
+#if NET_2_0
+		public override bool CanReadBinaryContent {
+			get { return true; }
+		}
+#endif
+
 		// This property for this class always return true.
 		public override bool CanResolveEntity {
 			get { return true; }
@@ -382,7 +391,7 @@ namespace System.Xml
 		}
 
 #if NET_2_0
-		IDictionary IXmlNamespaceResolver.GetNamespacesInScope (XmlNamespaceScope scope)
+		IDictionary<string, string> IXmlNamespaceResolver.GetNamespacesInScope (XmlNamespaceScope scope)
 		{
 			return ((IHasXmlParserContext) this).ParserContext.NamespaceManager.GetNamespacesInScope (scope);
 		}
@@ -401,9 +410,9 @@ namespace System.Xml
 		public override string LookupNamespace (string prefix)
 		{
 			if (validatingReader != null)
-				return sourceReader.LookupNamespace (prefix);
-			else
 				return validatingReader.LookupNamespace (prefix);
+			else
+				return sourceReader.LookupNamespace (prefix);
 		}
 
 #if NET_2_0
@@ -533,12 +542,7 @@ namespace System.Xml
 		}
 #endif
 
-#if NET_2_0
-		[Obsolete]
-		public override object ReadTypedValue ()
-#else
 		public object ReadTypedValue ()
-#endif
 		{
 			if (dtdReader == null)
 				return null;
@@ -589,6 +593,44 @@ namespace System.Xml
 			else if (ValidationType != ValidationType.None && e.Severity == XmlSeverityType.Error)
 				throw e.Exception;
 		}
+
+#if NET_2_0
+		[MonoTODO ("Check how expanded entity is handled here.")]
+		public override int ReadContentAsBase64 (byte [] buffer, int offset, int length)
+		{
+			if (validatingReader != null)
+				return validatingReader.ReadContentAsBase64 (buffer, offset, length);
+			else
+				return sourceReader.ReadContentAsBase64 (buffer, offset, length);
+		}
+
+		[MonoTODO ("Check how expanded entity is handled here.")]
+		public override int ReadContentAsBinHex (byte [] buffer, int offset, int length)
+		{
+			if (validatingReader != null)
+				return validatingReader.ReadContentAsBinHex (buffer, offset, length);
+			else
+				return sourceReader.ReadContentAsBinHex (buffer, offset, length);
+		}
+
+		[MonoTODO ("Check how expanded entity is handled here.")]
+		public override int ReadElementContentAsBase64 (byte [] buffer, int offset, int length)
+		{
+			if (validatingReader != null)
+				return validatingReader.ReadElementContentAsBase64 (buffer, offset, length);
+			else
+				return sourceReader.ReadElementContentAsBase64 (buffer, offset, length);
+		}
+
+		[MonoTODO ("Check how expanded entity is handled here.")]
+		public override int ReadElementContentAsBinHex (byte [] buffer, int offset, int length)
+		{
+			if (validatingReader != null)
+				return validatingReader.ReadElementContentAsBinHex (buffer, offset, length);
+			else
+				return sourceReader.ReadElementContentAsBinHex (buffer, offset, length);
+		}
+#endif
 		#endregion // Methods
 
 		#region Events and Delegates
