@@ -58,7 +58,7 @@ namespace System.Xml
 		#region Properties
 
 #if NET_2_0
-		public XmlWriterSettings Settings {
+		public virtual XmlWriterSettings Settings {
 			get {
 				if (settings == null)
 					settings = new XmlWriterSettings ();
@@ -106,11 +106,17 @@ namespace System.Xml
 			return Create (writer, null);
 		}
 
+		public static XmlWriter Create (XmlWriter writer)
+		{
+			return Create (writer, null);
+		}
+
 		public static XmlWriter Create (StringBuilder builder)
 		{
 			return Create (builder, null);
 		}
 
+		[MonoTODO ("NewLineHandling/OutputMethod")]
 		public static XmlWriter Create (Stream stream, XmlWriterSettings settings)
 		{
 			// FIXME: this might result in encoding null reference
@@ -118,6 +124,7 @@ namespace System.Xml
 			return Create (new StreamWriter (stream, enc), settings);
 		}
 
+		[MonoTODO ("NewLineHandling/OutputMethod")]
 		public static XmlWriter Create (string file, XmlWriterSettings settings)
 		{
 			// FIXME: this might result in encoding null reference
@@ -125,16 +132,19 @@ namespace System.Xml
 			return Create (new StreamWriter (file, false, enc), settings);
 		}
 
+		[MonoTODO ("NewLineHandling/OutputMethod")]
 		public static XmlWriter Create (StringBuilder builder, XmlWriterSettings settings)
 		{
 			return Create (new StringWriter (builder), null);
 		}
 
+		[MonoTODO ("NewLineHandling/OutputMethod")]
 		public static XmlWriter Create (TextWriter writer, XmlWriterSettings settings)
 		{
 			return CreateTextWriter (writer, settings);
 		}
 
+		[MonoTODO ("NewLineHandling/OutputMethod")]
 		public static XmlWriter Create (XmlWriter writer, XmlWriterSettings settings)
 		{
 			if (settings == null)
@@ -152,13 +162,12 @@ namespace System.Xml
 			if (settings.Indent) {
 				xtw.Formatting = Formatting.Indented;
 				xtw.IndentChars = settings.IndentChars;
+				xtw.NewLineOnAttributes = settings.NewLineOnAttributes;
 			}
 			// NewLineChars
 			xtw.NewLineChars = settings.NewLineChars;
 			// CloseOutput
 			xtw.CloseOutput = settings.CloseOutput;
-			// NewLineOnAttributes
-			xtw.NewLineOnAttributes = settings.NewLineOnAttributes;
 			// ConformanceLevel
 			xtw.ConformanceLevel = settings.ConformanceLevel;
 			// OmitXmlDeclaration
@@ -168,10 +177,15 @@ namespace System.Xml
 			return Create (xtw, settings);
 		}
 
-		public virtual void Dispose ()
+		public virtual void Dispose (bool disposing)
 		{
 			Close ();
 		}
+
+		void IDisposable.Dispose ()
+		{
+			Dispose (false);
+		}
 #endif
 
 		public abstract void Flush ();
@@ -550,6 +564,12 @@ namespace System.Xml
 			WriteString (XmlConvert.ToString (value));
 		}
 
+		[MonoTODO]
+		public virtual void WriteValue (decimal value)
+		{
+			WriteString (XQueryConvert.DecimalToString (value));
+		}
+
 		[MonoTODO]
 		public virtual void WriteValue (double value)
 		{
