@@ -152,9 +152,8 @@ namespace Mono.CSharp
 			// FIXME: these lines had better be platform independent.
 			if (Path.DirectorySeparatorChar == '\\') {
 				mcs.StartInfo.FileName = windowsMonoPath;
-				mcs.StartInfo.Arguments = windowsMcsPath + ' ' + BuildArgs (options, fileNames);
-			}
-			else {
+				mcs.StartInfo.Arguments = "\"" + windowsMcsPath + "\" " + BuildArgs (options, fileNames);
+			} else {
 #if NET_2_0
 				// FIXME: This is a temporary hack to make code genaration work in 2.0
 				mcs.StartInfo.FileName="gmcs";
@@ -173,8 +172,8 @@ namespace Mono.CSharp
 				mcs_output=mcs.StandardError.ReadToEnd();
 				mcs.StandardOutput.ReadToEnd ();
 				mcs.WaitForExit();
-			} finally {
 				results.NativeCompilerReturnValue = mcs.ExitCode;
+			} finally {
 				mcs.Close();
 			}
 			mcs_output_lines=mcs_output.Split(
@@ -257,9 +256,11 @@ namespace Mono.CSharp
 			if (error_string.StartsWith ("BETA"))
 				return null;
 #endif
+			if (error_string == null || error_string == "")
+				return null;
 
 			CompilerError error=new CompilerError();
-			Regex reg = new Regex (@"^(\s*(?<file>.*)\((?<line>\d*)(,(?<column>\d*))?\)\s+)*(?<level>\w+)\s*(?<number>.*):\s(?<message>.*)",
+			Regex reg = new Regex (@"^(\s*(?<file>.*)\((?<line>\d*)(,(?<column>\d*))?\)(:)?\s+)*(?<level>\w+)\s*(?<number>.*):\s(?<message>.*)",
 				RegexOptions.Compiled | RegexOptions.ExplicitCapture);
 			Match match=reg.Match(error_string);
 			if (!match.Success) return null;
@@ -308,7 +309,7 @@ namespace Mono.CSharp
 				CodeCompileUnit compileUnit = ea[i];
 				fileNames[i] = GetTempFileNameWithExtension (options.TempFiles, i + ".cs");
 				FileStream f = new FileStream (fileNames[i], FileMode.OpenOrCreate);
-				StreamWriter s = new StreamWriter (f);
+				StreamWriter s = new StreamWriter (f, Encoding.UTF8);
 				if (compileUnit.ReferencedAssemblies != null) {
 					foreach (string str in compileUnit.ReferencedAssemblies) {
 						if (!assemblies.Contains (str))
@@ -338,7 +339,7 @@ namespace Mono.CSharp
 			for (int i = 0; i < sources.Length; i++) {
 				fileNames[i] = GetTempFileNameWithExtension (options.TempFiles, i + ".cs");
 				FileStream f = new FileStream (fileNames[i], FileMode.OpenOrCreate);
-				using (StreamWriter s = new StreamWriter (f)) {
+				using (StreamWriter s = new StreamWriter (f, Encoding.UTF8)) {
 					s.Write (sources[i]);
 					s.Close ();
 				}
