@@ -57,6 +57,14 @@ namespace System.CodeDom.Compiler {
 		//
 		// Properties
 		//
+#if NET_2_0
+		protected CodeTypeDeclaration CurrentClass {
+			get {
+				return currentType;
+			}
+		}
+#endif
+
 		protected CodeTypeMember CurrentMember {
 			get {
 				return currentMember;
@@ -65,8 +73,8 @@ namespace System.CodeDom.Compiler {
 		
 		protected string CurrentMemberName {
 			get {
-				if (currentType == null)
-					return null;
+				if (currentMember == null)
+					return "<% unknown %>";
 				return currentMember.Name;
 			}
 		}
@@ -74,7 +82,7 @@ namespace System.CodeDom.Compiler {
 		protected string CurrentTypeName {
 			get {
 				if (currentType == null)
-					return null;
+					return "<% unknown %>";
 				return currentType.Name;
 			}
 		}
@@ -92,7 +100,7 @@ namespace System.CodeDom.Compiler {
 			get {
 				if (currentType == null)
 					return false;
-				return currentType.IsClass;
+				return currentType.IsClass && !(currentType is CodeTypeDelegate);
 			}
 		}
 
@@ -175,6 +183,13 @@ namespace System.CodeDom.Compiler {
 		}
 
 		protected abstract void GenerateCastExpression (CodeCastExpression e);
+#if NET_2_0
+		[MonoTODO]
+		public virtual void GenerateCodeFromMember (CodeTypeMember member, TextWriter writer, CodeGeneratorOptions options)
+		{
+			throw new NotImplementedException ();
+		}
+#endif
 		protected abstract void GenerateComment (CodeComment comment);
 
 		protected virtual void GenerateCommentStatement (CodeCommentStatement statement)
@@ -234,7 +249,13 @@ namespace System.CodeDom.Compiler {
 		{
 			Output.Write (d.ToString (CultureInfo.InvariantCulture));
 		}
-
+#if NET_2_0
+		[MonoTODO]
+		protected virtual void GenerateDefaultValueExpression (CodeDefaultValueExpression e)
+		{
+			throw new NotImplementedException ();
+		}
+#endif
 		protected abstract void GenerateDelegateCreateExpression (CodeDelegateCreateExpression e);
 		protected abstract void GenerateDelegateInvokeExpression (CodeDelegateInvokeExpression e);
 
@@ -472,20 +493,26 @@ namespace System.CodeDom.Compiler {
 				output.Write ("'" + e.Value.ToString () + "'");
 			} else if (type == typeof (string)) {
 				output.Write (QuoteSnippetString ((string) e.Value));
-			} else if (type == typeof (byte) || type == typeof (sbyte) || type == typeof (short) ||
-				   type == typeof (int) || type == typeof (long) || type == typeof (float) ||
-				   type == typeof (double) || type == typeof (decimal)) {
+			} else if (type == typeof (float)) {
+				GenerateSingleFloatValue((float) e.Value);
+			} else if (type == typeof (double)) {
+				GenerateDoubleValue((double) e.Value);
+			} else if (type == typeof (decimal)) {
+				this.GenerateDecimalValue((decimal) e.Value);
+			} else if (type == typeof (byte) || type == typeof (short) || 
+				type == typeof (int) || type == typeof (long)) {
 				// All of these should be IFormatable, I am just being safe/slow 
 				IFormattable formattable = e.Value as IFormattable;
-				if (formattable != null)
+				if (formattable != null) {
 					output.Write (formattable.ToString (null, CultureInfo.InvariantCulture));
-				else
+				} else {
 					output.Write (e.Value.ToString ());
-					
-				if (type == typeof (float))
-					output.Write ("f");
+				}
 			} else {
-				throw new ArgumentException ("Value type (" + type + ") is not a primitive type");
+				throw new ArgumentException (string.Format(CultureInfo.InvariantCulture,
+					"Invalid Primitive Type: {0}. Only CLS compliant primitive " +
+					"types can be used. Consider using CodeObjectCreateExpression.",
+					type.FullName));
 			}
 		}
 
@@ -581,7 +608,17 @@ namespace System.CodeDom.Compiler {
 			}
 			CodeSnippetStatement snippet = s as CodeSnippetStatement;
 			if (snippet != null) {
+#if NET_2_0
+				int indent = Indent;
+				try {
+					Indent = 0;
+					GenerateSnippetStatement (snippet);
+				} finally {
+					Indent = indent;
+				}
+#else
 				GenerateSnippetStatement (snippet);
+#endif
 				handled = true;
 			}
 			CodeThrowExceptionStatement exception = s as CodeThrowExceptionStatement;
@@ -656,7 +693,7 @@ namespace System.CodeDom.Compiler {
 		protected virtual void OutputAttributeArgument (CodeAttributeArgument argument)
 		{
 			string name = argument.Name;
-			if (name != null) {
+			if ((name != null) && (name.Length > 0)) {
 				output.Write (name);
 				output.Write ('=');
 			}
@@ -768,61 +805,69 @@ namespace System.CodeDom.Compiler {
 		protected virtual void OutputMemberAccessModifier (MemberAttributes attributes)
 		{
 			switch (attributes & MemberAttributes.AccessMask) {
-			case MemberAttributes.Assembly:
-				output.Write ("internal ");
-				break;
-			case MemberAttributes.FamilyAndAssembly:
-				output.Write ("/* FamAndAssem */ internal "); 
-				break;
-			case MemberAttributes.Family:
-				output.Write ("protected ");
-				break;
-			case MemberAttributes.FamilyOrAssembly:
-				output.Write ("protected internal ");
-				break;
-			case MemberAttributes.Private:
-				output.Write ("private ");
-				break;
-			case MemberAttributes.Public:
-				output.Write ("public ");
-				break;
+				case MemberAttributes.Assembly:
+					output.Write ("internal ");
+					break;
+				case MemberAttributes.FamilyAndAssembly:
+#if NET_2_0
+					output.Write ("internal "); 
+#else
+					output.Write ("/*FamANDAssem*/ internal "); 
+#endif
+					break;
+				case MemberAttributes.Family:
+					output.Write ("protected ");
+					break;
+				case MemberAttributes.FamilyOrAssembly:
+					output.Write ("protected internal ");
+					break;
+				case MemberAttributes.Private:
+					output.Write ("private ");
+					break;
+				case MemberAttributes.Public:
+					output.Write ("public ");
+					break;
 			}
 		}
 
 		protected virtual void OutputMemberScopeModifier (MemberAttributes attributes)
 		{
+#if NET_2_0
 			if ((attributes & MemberAttributes.VTableMask) == MemberAttributes.New)
 				output.Write( "new " );
+#endif
 
 			switch (attributes & MemberAttributes.ScopeMask) {
-			case MemberAttributes.Abstract:
-				output.Write ("abstract ");
-				break;
-			case MemberAttributes.Final:
-				// Do nothing
-				break;
-			case MemberAttributes.Static:
-				output.Write ("static ");
-				break;
-			case MemberAttributes.Override:
-				output.Write ("override ");
-				break;
-			default:
-				//
-				// FUNNY! if the scope value is
-				// rubbish (0 or >Const), and access
-				// is public or protected, make it
-				// "virtual".
-				//
-				// i'm not sure whether this is 100%
-				// correct, but it seems to be MS
-				// behavior. 
-				//
-				MemberAttributes access = attributes & MemberAttributes.AccessMask;
-				if (access == MemberAttributes.Public || 
-				     access == MemberAttributes.Family)
-					output.Write ("virtual ");
-				break;
+				case MemberAttributes.Abstract:
+					output.Write ("abstract ");
+					break;
+				case MemberAttributes.Final:
+					// Do nothing
+					break;
+				case MemberAttributes.Static:
+					output.Write ("static ");
+					break;
+				case MemberAttributes.Override:
+					output.Write ("override ");
+					break;
+				default:
+					//
+					// FUNNY! if the scope value is
+					// rubbish (0 or >Const), and access
+					// is public or protected, make it
+					// "virtual".
+					//
+					// i'm not sure whether this is 100%
+					// correct, but it seems to be MS
+					// behavior.
+					//
+					// On .NET 2.0, internal members
+					// are also marked "virtual".
+					//
+					MemberAttributes access = attributes & MemberAttributes.AccessMask;
+					if (access == MemberAttributes.Public || access == MemberAttributes.Family)
+						output.Write ("virtual ");
+					break;
 			}
 		}
 				
@@ -916,9 +961,6 @@ namespace System.CodeDom.Compiler {
 				break;
 			}
 
-			if (!IsCurrentClass)
-				OutputExtraTypeAttribute (currentType);
-
 			if (isStruct)
 				output.Write ("struct ");
 
@@ -935,16 +977,11 @@ namespace System.CodeDom.Compiler {
 					if ((attributes & TypeAttributes.Abstract) != 0)
 						output.Write ("abstract ");
 					
-					OutputExtraTypeAttribute (currentType);
 					output.Write ("class ");
 				}
 			}
 		}
 
-		internal virtual void OutputExtraTypeAttribute (CodeTypeDeclaration type)
-		{
-		}
-		
 		protected virtual void OutputTypeNamePair (CodeTypeReference type,
 							   string name)
 		{
@@ -984,7 +1021,12 @@ namespace System.CodeDom.Compiler {
 								 CodeGeneratorOptions options)
 		{
 			InitOutput (output, options);
-			GenerateCompileUnit (compileUnit);
+
+			if (compileUnit is CodeSnippetCompileUnit) {
+				GenerateSnippetCompileUnit ((CodeSnippetCompileUnit) compileUnit);
+			} else {
+				GenerateCompileUnit (compileUnit);
+			}
 		}
 
 		void ICodeGenerator.GenerateCodeFromExpression (CodeExpression expression,
@@ -1021,6 +1063,8 @@ namespace System.CodeDom.Compiler {
 
 		private void GenerateType (CodeTypeDeclaration type)
 		{
+			this.currentType = type;
+
 #if NET_2_0
 			if (type.StartDirectives.Count > 0)
 				GenerateDirectives (type.StartDirectives);
@@ -1031,61 +1075,27 @@ namespace System.CodeDom.Compiler {
 			if (type.LinePragma != null)
 				GenerateLinePragmaStart (type.LinePragma);
 
-			CodeTypeDelegate del = type as CodeTypeDelegate;
-			if (del != null)
-				GenerateDelegate (del);
-			else
-				GenerateNonDelegateType (type);
-
-			if (type.LinePragma != null)
-				GenerateLinePragmaEnd (type.LinePragma);
-
-#if NET_2_0
-			if (type.EndDirectives.Count > 0)
-				GenerateDirectives (type.EndDirectives);
-#endif
-		}
-
-		private void GenerateDelegate (CodeTypeDelegate type)
-		{
-			CodeTypeDeclaration prevType = this.currentType;
-			this.currentType = type;
-
 			GenerateTypeStart (type);
 
-			OutputParameters (type.Parameters);
-
-			GenerateTypeEnd (type);
-			this.currentType = prevType;
-		}
-		
-		private void GenerateNonDelegateType (CodeTypeDeclaration type)
-		{
-			CodeTypeDeclaration prevType = this.currentType;
-			this.currentType = type;
-
-			GenerateTypeStart (type);
-
-			CodeTypeMember [] members = new CodeTypeMember [type.Members.Count];
+			CodeTypeMember[] members = new CodeTypeMember[type.Members.Count];
 			type.Members.CopyTo (members, 0);
 
 #if NET_2_0
 			if (!Options.VerbatimOrder)
 #endif
-			{
+ {
 				int[] order = new int[members.Length];
-				for (int n=0; n<members.Length; n++)
-					order[n] = Array.IndexOf(memberTypes, members[n].GetType()) * members.Length + n;
+				for (int n = 0; n < members.Length; n++)
+					order[n] = Array.IndexOf (memberTypes, members[n].GetType ()) * members.Length + n;
 
 				Array.Sort (order, members);
 			}
-			
+
 			// WARNING: if anything is missing in the foreach loop and you add it, add the type in
 			// its corresponding place in CodeTypeMemberComparer class (below)
 
 			CodeTypeDeclaration subtype = null;
-			foreach (CodeTypeMember member in members) 
-			{
+			foreach (CodeTypeMember member in members) {
 				CodeTypeMember prevMember = this.currentMember;
 				this.currentMember = member;
 
@@ -1104,6 +1114,7 @@ namespace System.CodeDom.Compiler {
 				subtype = member as CodeTypeDeclaration;
 				if (subtype != null) {
 					GenerateType (subtype);
+					this.currentType = type;
 					continue;
 				}
 
@@ -1118,54 +1129,46 @@ namespace System.CodeDom.Compiler {
 					GenerateLinePragmaStart (member.LinePragma);
 
 				CodeMemberEvent eventm = member as CodeMemberEvent;
-				if (eventm != null) 
-				{
+				if (eventm != null) {
 					GenerateEvent (eventm, type);
 					continue;
 				}
 				CodeMemberField field = member as CodeMemberField;
-				if (field != null) 
-				{
+				if (field != null) {
 					GenerateField (field);
 					continue;
 				}
 				CodeEntryPointMethod epmethod = member as CodeEntryPointMethod;
-				if (epmethod != null) 
-				{
+				if (epmethod != null) {
 					GenerateEntryPointMethod (epmethod, type);
 					continue;
 				}
 				CodeTypeConstructor typeCtor = member as CodeTypeConstructor;
-				if (typeCtor != null) 
-				{
+				if (typeCtor != null) {
 					GenerateTypeConstructor (typeCtor);
 					continue;
 				}
 				CodeConstructor ctor = member as CodeConstructor;
-				if (ctor != null) 
-				{
+				if (ctor != null) {
 					GenerateConstructor (ctor, type);
 					continue;
 				}
 				CodeMemberMethod method = member as CodeMemberMethod;
-				if (method != null) 
-				{
+				if (method != null) {
 					GenerateMethod (method, type);
 					continue;
 				}
 				CodeMemberProperty property = member as CodeMemberProperty;
-				if (property != null) 
-				{
+				if (property != null) {
 					GenerateProperty (property, type);
 					continue;
 				}
 				CodeSnippetTypeMember snippet = member as CodeSnippetTypeMember;
-				if (snippet != null) 
-				{
+				if (snippet != null) {
 					GenerateSnippetMember (snippet);
 					continue;
 				}
-				
+
 				this.currentMember = prevMember;
 			}
 
@@ -1178,9 +1181,17 @@ namespace System.CodeDom.Compiler {
 					GenerateDirectives (currentMember.EndDirectives);
 #endif
 			}
-				
+
+			this.currentType = type;
 			GenerateTypeEnd (type);
-			this.currentType = prevType;
+
+			if (type.LinePragma != null)
+				GenerateLinePragmaEnd (type.LinePragma);
+
+#if NET_2_0
+			if (type.EndDirectives.Count > 0)
+				GenerateDirectives (type.EndDirectives);
+#endif
 		}
 
 		protected abstract string GetTypeOutput (CodeTypeReference type);
@@ -1278,12 +1289,10 @@ namespace System.CodeDom.Compiler {
 						typeof (CodeEntryPointMethod)
 					};
 
-
 #if NET_2_0
 		protected virtual void GenerateDirectives (CodeDirectiveCollection directives)
 		{
 		}
 #endif
-
 	}
 }
