@@ -57,6 +57,9 @@ namespace System.Configuration
 			return config.GetConfig (sectionName);
 		}
 
+#if NET_2_0
+		[Obsolete ("This property is obsolete.  Please use System.Configuration.ConfigurationManager.AppSettings")]
+#endif
 		public static NameValueCollection AppSettings
 		{
 			get {
@@ -68,25 +71,6 @@ namespace System.Configuration
 			}
 		}
 
-#if NET_2_0
-#if XML_DEP
-                public static ConnectionStringSettingsCollection ConnectionStrings
-		{
-			get {
-                                ConnectionStringsSection connSection = (ConnectionStringsSection) GetConfig ("connectionStrings");
-                                ConnectionStringSettingsCollection connectionStrings = null;
-                                
-                                if (connSection != null)
-                                        connectionStrings = connSection.ConnectionStrings;
-                                else 
-                                        connectionStrings = new ConnectionStringSettingsCollection ();
-                                
-				return connectionStrings;
-			}
-		}
-#endif // XML_DEP
-#endif // NET_2_0
-
 		// Invoked from System.Web
 		static IConfigurationSystem ChangeConfigurationSystem (IConfigurationSystem newSystem)
 		{
@@ -119,6 +103,9 @@ namespace System.Configuration
 			return instance;
 		}
 
+#if NET_2_0
+		[Obsolete ("This method is obsolete.  Please use System.Configuration.ConfigurationManager.GetConfig")]
+#endif
 		public object GetConfig (string sectionName)
 		{
 			Init ();
@@ -278,9 +265,11 @@ namespace System.Configuration
 			if (t == null)
 				throw new ConfigurationException ("Cannot get Type for " + section.TypeName);
 
+#if false
 			Type iconfig = typeof (IConfigurationSectionHandler);
 			if (!iconfig.IsAssignableFrom (t))
 				throw new ConfigurationException (sectionName + " does not implement " + iconfig);
+#endif
 			
 			object o = Activator.CreateInstance (t, true);
 			if (o == null)
@@ -516,13 +505,23 @@ namespace System.Configuration
 			if (!reader.MoveToNextAttribute ())
 				ThrowException ("sectionGroup must have a 'name' attribute.", reader);
 
-			if (reader.Name != "name")
-				ThrowException ("Unrecognized attribute.", reader);
+			string value = null;
+			do {
+				if (reader.Name == "name") {
+					if (value != null)
+						ThrowException ("Duplicate 'name' attribute.", reader);
+					value = reader.Value;
+				}
+				else 
+#if NET_2_0
+				if (reader.Name != "type")
+#endif
+					ThrowException ("Unrecognized attribute.", reader);
+			} while (reader.MoveToNextAttribute ());
 
-			if (reader.MoveToNextAttribute ())
-				ThrowException ("Unrecognized attribute.", reader);
+			if (value == null)
+				ThrowException ("No 'name' attribute.", reader);
 
-			string value = reader.Value;
 			if (value == "location")
 				ThrowException ("location is a reserved section name", reader);
 			
