@@ -43,7 +43,7 @@ using System.Threading;
 
 namespace System.Diagnostics {
 	[DefaultEvent ("Exited"), DefaultProperty ("StartInfo")]
-	[Designer ("System.Diagnostics.Design.ProcessDesigner, " + Consts.AssemblySystem_Design, typeof (IDesigner))]
+	[Designer ("System.Diagnostics.Design.ProcessDesigner, " + Consts.AssemblySystem_Design)]
 	public class Process : Component 
 	{
 		[StructLayout(LayoutKind.Sequential)]
@@ -60,13 +60,14 @@ namespace System.Diagnostics {
 			public int tid;
 			public string [] envKeys;
 			public string [] envValues;
-			public bool useShellExecute;
 		};
 		
 		IntPtr process_handle;
 		int pid;
 		bool enableRaisingEvents;
+		bool already_waiting;
 		ISynchronizeInvoke synchronizingObject;
+		EventHandler exited_event;
 		
 		/* Private constructor called from other methods */
 		private Process(IntPtr handle, int id) {
@@ -74,8 +75,8 @@ namespace System.Diagnostics {
 			pid=id;
 		}
 		
-		[MonoTODO]
-		public Process() {
+		public Process ()
+		{
 		}
 
 		[MonoTODO]
@@ -87,6 +88,17 @@ namespace System.Diagnostics {
 			}
 		}
 
+		void StartExitCallbackIfNeeded ()
+		{
+			bool start = (!already_waiting && enableRaisingEvents && exited_event != null);
+			if (start && process_handle != IntPtr.Zero && !HasExited) {
+				WaitOrTimerCallback cb = new WaitOrTimerCallback (CBOnExit);
+				ProcessWaitHandle h = new ProcessWaitHandle (process_handle);
+				ThreadPool.RegisterWaitForSingleObject (h, cb, this, -1, true);
+				already_waiting = true;
+			}
+		}
+
 		[DefaultValue (false), Browsable (false)]
 		[MonitoringDescription ("Check for exiting of the process to raise the apropriate event.")]
 		public bool EnableRaisingEvents {
@@ -94,10 +106,10 @@ namespace System.Diagnostics {
 				return enableRaisingEvents;
 			}
 			set { 
-				if (process_handle != IntPtr.Zero)
-					throw new InvalidOperationException ("The process is already started.");
-
+				bool prev = enableRaisingEvents;
 				enableRaisingEvents = value;
+				if (enableRaisingEvents && !prev)
+					StartExitCallbackIfNeeded ();
 			}
 
 		}
@@ -181,6 +193,10 @@ namespace System.Diagnostics {
 		[MonitoringDescription ("Process identifier.")]
 		public int Id {
 			get {
+				if (pid == 0) {
+					throw new InvalidOperationException ("Process ID has not been set.");
+				}
+
 				return(pid);
 			}
 		}
@@ -498,7 +514,7 @@ namespace System.Diagnostics {
 
 		private ProcessStartInfo start_info=null;
 		
-		[DesignerSerializationVisibility (DesignerSerializationVisibility.Hidden), Browsable (false)]
+		[DesignerSerializationVisibility (DesignerSerializationVisibility.Content), Browsable (false)]
 		[MonitoringDescription ("Information for the start of this process.")]
 		public ProcessStartInfo StartInfo {
 			get {
@@ -707,38 +723,56 @@ namespace System.Diagnostics {
 		}
 
 		[MethodImplAttribute(MethodImplOptions.InternalCall)]
-		private extern static bool Start_internal(string appname,
-							  string cmdline,
-							  string dir,
-							  IntPtr stdin,
-							  IntPtr stdout,
-							  IntPtr stderr,
-							  ref ProcInfo proc_info);
+		private extern static bool ShellExecuteEx_internal(ProcessStartInfo startInfo,
+								   ref ProcInfo proc_info);
 
-		private static bool Start_common(ProcessStartInfo startInfo,
-						 Process process) {
+		[MethodImplAttribute(MethodImplOptions.InternalCall)]
+		private extern static bool CreateProcess_internal(ProcessStartInfo startInfo,
+								  IntPtr stdin,
+								  IntPtr stdout,
+								  IntPtr stderr,
+								  ref ProcInfo proc_info);
+
+		private static bool Start_shell (ProcessStartInfo startInfo,
+						 Process process)
+		{
+			ProcInfo proc_info=new ProcInfo();
+			bool ret;
+
+			if (startInfo.RedirectStandardInput ||
+			    startInfo.RedirectStandardOutput ||
+			    startInfo.RedirectStandardError) {
+				throw new InvalidOperationException ("UseShellExecute must be false when redirecting I/O.");
+			}
+
+			if (startInfo.HaveEnvVars) {
+				throw new InvalidOperationException ("UseShellExecute must be false in order to use environment variables.");
+			}
+
+			ret = ShellExecuteEx_internal (startInfo,
+						       ref proc_info);
+			if (!ret) {
+				throw new Win32Exception (-proc_info.pid);
+			}
+
+			process.process_handle = proc_info.process_handle;
+			process.pid = proc_info.pid;
+
+			process.StartExitCallbackIfNeeded ();
+
+			return(ret);
+		}
+
+		private static bool Start_noshell (ProcessStartInfo startInfo,
+						   Process process)
+		{
 			ProcInfo proc_info=new ProcInfo();
 			IntPtr stdin_rd, stdin_wr;
 			IntPtr stdout_rd, stdout_wr;
 			IntPtr stderr_rd, stderr_wr;
 			bool ret;
-			
-			if(startInfo.FileName == null || startInfo.FileName == "") {
-				throw new InvalidOperationException("File name has not been set");
-			}
-			
-			proc_info.useShellExecute = startInfo.UseShellExecute;
-			if (proc_info.useShellExecute && (startInfo.RedirectStandardInput ||
-			    startInfo.RedirectStandardOutput || startInfo.RedirectStandardError)) {
-				throw new InvalidOperationException ("UseShellExecute must be false when " +
-								     "redirecting I/O.");
-			}
 
 			if (startInfo.HaveEnvVars) {
-				if (startInfo.UseShellExecute)
-					throw new InvalidOperationException ("UseShellExecute must be false in order " +
-									     "to use environment variables.");
-
 				string [] strs = new string [startInfo.EnvironmentVariables.Count];
 				startInfo.EnvironmentVariables.Keys.CopyTo (strs, 0);
 				proc_info.envKeys = strs;
@@ -748,62 +782,46 @@ namespace System.Diagnostics {
 				proc_info.envValues = strs;
 			}
 
-			if(startInfo.RedirectStandardInput==true) {
-				ret=MonoIO.CreatePipe(out stdin_rd,
-						      out stdin_wr);
+			if (startInfo.RedirectStandardInput == true) {
+				ret = MonoIO.CreatePipe (out stdin_rd,
+						         out stdin_wr);
 				if (ret == false) {
-					throw new IOException("Error creating standard input pipe");
+					throw new IOException ("Error creating standard input pipe");
 				}
 			} else {
-				stdin_rd=MonoIO.ConsoleInput;
+				stdin_rd = MonoIO.ConsoleInput;
 				/* This is required to stop the
 				 * &$*ing stupid compiler moaning
 				 * that stdin_wr is unassigned, below.
 				 */
-				stdin_wr=(IntPtr)0;
+				stdin_wr = (IntPtr)0;
 			}
 
-			if(startInfo.RedirectStandardOutput==true) {
-				ret=MonoIO.CreatePipe(out stdout_rd,
-						      out stdout_wr);
+			if (startInfo.RedirectStandardOutput == true) {
+				ret = MonoIO.CreatePipe (out stdout_rd,
+						         out stdout_wr);
 				if (ret == false) {
-					throw new IOException("Error creating standard output pipe");
+					throw new IOException ("Error creating standard output pipe");
 				}
 			} else {
-				stdout_rd=(IntPtr)0;
-				stdout_wr=MonoIO.ConsoleOutput;
+				stdout_rd = (IntPtr)0;
+				stdout_wr = MonoIO.ConsoleOutput;
 			}
 
-			if(startInfo.RedirectStandardError==true) {
-				ret=MonoIO.CreatePipe(out stderr_rd,
-						      out stderr_wr);
+			if (startInfo.RedirectStandardError == true) {
+				ret = MonoIO.CreatePipe (out stderr_rd,
+						         out stderr_wr);
 				if (ret == false) {
-					throw new IOException("Error creating standard error pipe");
+					throw new IOException ("Error creating standard error pipe");
 				}
 			} else {
-				stderr_rd=(IntPtr)0;
-				stderr_wr=MonoIO.ConsoleError;
+				stderr_rd = (IntPtr)0;
+				stderr_wr = MonoIO.ConsoleError;
 			}
 			
-			string cmdline;
-			string appname;
-			if (startInfo.UseShellExecute) {
-				appname = null;
-				string args = startInfo.Arguments;
-				if (args == null || args.Trim () == "")
-					cmdline = startInfo.FileName;
-				else
-					cmdline = startInfo.FileName + " " + startInfo.Arguments.Trim ();
-			} else {
-				appname = startInfo.FileName;
-				cmdline = startInfo.Arguments.Trim ();
-			}
-
-			ret=Start_internal(appname,
-					   cmdline,
-					   startInfo.WorkingDirectory,
-					   stdin_rd, stdout_wr, stderr_wr,
-					   ref proc_info);
+			ret = CreateProcess_internal (startInfo,
+						      stdin_rd, stdout_wr, stderr_wr,
+						      ref proc_info);
 
 			MonoIOError error;
 			
@@ -820,33 +838,44 @@ namespace System.Diagnostics {
 				throw new Win32Exception (-proc_info.pid);
 			}
 
-			if (process.enableRaisingEvents && process.Exited != null) {
-				WaitOrTimerCallback cb = new WaitOrTimerCallback (CBOnExit);
-				ProcessWaitHandle h = new ProcessWaitHandle (proc_info.process_handle);
-				ThreadPool.RegisterWaitForSingleObject (h, cb, process, -1, true);
-			}
+			process.process_handle = proc_info.process_handle;
+			process.pid = proc_info.pid;
 			
-			process.process_handle=proc_info.process_handle;
-			process.pid=proc_info.pid;
-			
-			if(startInfo.RedirectStandardInput==true) {
-				MonoIO.Close(stdin_rd, out error);
-				process.input_stream=new StreamWriter(new FileStream(stdin_wr, FileAccess.Write, true));
-				process.input_stream.AutoFlush=true;
+			if (startInfo.RedirectStandardInput == true) {
+				MonoIO.Close (stdin_rd, out error);
+				process.input_stream = new StreamWriter (new FileStream (stdin_wr, FileAccess.Write, true));
+				process.input_stream.AutoFlush = true;
 			}
 
-			if(startInfo.RedirectStandardOutput==true) {
-				MonoIO.Close(stdout_wr, out error);
-				process.output_stream=new StreamReader(new FileStream(stdout_rd, FileAccess.Read, true));
+			if (startInfo.RedirectStandardOutput == true) {
+				MonoIO.Close (stdout_wr, out error);
+				process.output_stream = new StreamReader (new FileStream (stdout_rd, FileAccess.Read, true));
 			}
 
-			if(startInfo.RedirectStandardError==true) {
-				MonoIO.Close(stderr_wr, out error);
-				process.error_stream=new StreamReader(new FileStream(stderr_rd, FileAccess.Read, true));
+			if (startInfo.RedirectStandardError == true) {
+				MonoIO.Close (stderr_wr, out error);
+				process.error_stream = new StreamReader (new FileStream (stderr_rd, FileAccess.Read, true));
 			}
 
+			process.StartExitCallbackIfNeeded ();
+
 			return(ret);
 		}
+
+		private static bool Start_common (ProcessStartInfo startInfo,
+						  Process process)
+		{
+			if(startInfo.FileName == null ||
+			   startInfo.FileName == "") {
+				throw new InvalidOperationException("File name has not been set");
+			}
+			
+			if (startInfo.UseShellExecute) {
+				return (Start_shell (startInfo, process));
+			} else {
+				return (Start_noshell (startInfo, process));
+			}
+		}
 		
 		public bool Start() {
 			bool ret;
@@ -911,9 +940,22 @@ namespace System.Diagnostics {
 			return(false);
 		}
 
-		[Category ()]
+		[Category ("Behavior")]
 		[MonitoringDescription ("Raised when this process exits.")]
-		public event EventHandler Exited;
+		public event EventHandler Exited {
+			add {
+				if (process_handle != IntPtr.Zero && HasExited) {
+					value.BeginInvoke (null, null, null, null);
+				} else {
+					exited_event = (EventHandler) Delegate.Combine (exited_event, value);
+					if (exited_event != null)
+						StartExitCallbackIfNeeded ();
+				}
+			}
+			remove {
+				exited_event = (EventHandler) Delegate.Remove (exited_event, value);
+			}
+		}
 
 		// Closes the system process handle
 		[MethodImplAttribute(MethodImplOptions.InternalCall)]
@@ -967,16 +1009,20 @@ namespace System.Diagnostics {
 
 		protected void OnExited() 
 		{
-			if (Exited == null)
+			if (exited_event == null)
 				return;
 
 			if (synchronizingObject == null) {
-				Exited (this, EventArgs.Empty);
+				foreach (EventHandler d in exited_event.GetInvocationList ()) {
+					try {
+						d (this, EventArgs.Empty);
+					} catch {}
+				}
 				return;
 			}
 			
 			object [] args = new object [] {this, EventArgs.Empty};
-			synchronizingObject.BeginInvoke (Exited, args);
+			synchronizingObject.BeginInvoke (exited_event, args);
 		}
 
 		class ProcessWaitHandle : WaitHandle
