@@ -162,14 +162,11 @@ namespace System.Globalization
 			return (formatType == GetType()) ? this : null;
 		}
 
-		[MonoTODO]
-		public string GetAbbreviatedEraName(int era)
+		public string GetAbbreviatedEraName (int era)
 		{
-			if (era < _Calendar.Eras.Length || era >= _Calendar.Eras.Length)
-				throw new ArgumentOutOfRangeException();
-			notImplemented();
-			//FIXME: implement me
-			return null;
+			if (era < 0 || era >= _Calendar.AbbreviatedEraNames.Length)
+				throw new ArgumentOutOfRangeException ("era", era.ToString ());
+			return _Calendar.AbbreviatedEraNames [era];
 		}
 
 		public string GetAbbreviatedMonthName(int month)
@@ -178,29 +175,24 @@ namespace System.Globalization
 			return _AbbreviatedMonthNames[month-1];
 		}
 
-		[MonoTODO]
-		public int GetEra(string eraName)
+		public int GetEra (string eraName)
 		{
-			if (eraName == null) throw new ArgumentNullException();
-			eraName = eraName.ToUpper();
-			notImplemented();
-			//FIXME: implement me
+			if (eraName == null)
+				throw new ArgumentNullException ();
+			string [] eras = _Calendar.EraNames;
+			for (int i = 0; i < eras.Length; i++)
+				if (CultureInfo.InvariantCulture.CompareInfo
+					.Compare (eraName, eras [i],
+					CompareOptions.IgnoreCase) == 0)
+					return i;
 			return -1;
 		}
 
-		[MonoTODO]
-		public string GetEraName(int era)
+		public string GetEraName (int era)
 		{
-//			if (era < _Calendar.Eras.Length || era >= _Calendar.Eras.Length)
-//				throw new ArgumentOutOfRangeException();
-			try {
-				return INVARIANT_ERA_NAMES[era - 1];
-			}
-			catch {
-				//FIXME: implement me
-				notImplemented();
-				return null;
-			}
+			if (era < 0 || era > _Calendar.EraNames.Length)
+				throw new ArgumentOutOfRangeException ("era", era.ToString ());
+			return _Calendar.EraNames [era - 1];
 		}
 
 		public string GetMonthName(int month)
@@ -520,8 +512,29 @@ namespace System.Globalization
 		
 		// LAMESPEC: this is not in ECMA specs
 		[MonoTODO ("Not complete depending on GetAllDateTimePatterns(char)")]
-		public string[] GetAllDateTimePatterns()
+		public string[] GetAllDateTimePatterns() 
 		{
+			FillAllDateTimePatterns ();
+			return (string []) all_date_time_patterns.Clone ();
+		}
+
+
+		// Same as above, but with no cloning, because we know that
+		// clients are friendly
+		internal string [] GetAllDateTimePatternsInternal ()
+		{
+			FillAllDateTimePatterns ();
+			return all_date_time_patterns;
+		}
+		
+		// Prevent write reordering
+		volatile string [] all_date_time_patterns;
+		
+		void FillAllDateTimePatterns (){
+
+			if (all_date_time_patterns != null)
+				return;
+			
 			ArrayList al = new ArrayList ();
 			foreach (string s in GetAllDateTimePatterns ('d'))
 				al.Add (s);
@@ -563,7 +576,9 @@ namespace System.Globalization
 			foreach (string s in GetAllDateTimePatterns ('Y'))
 				al.Add (s);
 
-			return al.ToArray (typeof (string)) as string [];
+			// all_date_time_patterns needs to be volatile to prevent
+			// reordering of writes here and still avoid any locking.
+			all_date_time_patterns = (string []) al.ToArray (typeof (string)) as string [];
 		}
 
 		// LAMESPEC: this is not in ECMA specs
@@ -597,12 +612,12 @@ namespace System.Globalization
 				list = PopulateCombinedList (_ShortDatePatterns, _LongTimePatterns);
 				if (list != null && list.Length > 0)
 					return list;
-				return new string [] {ShortDatePattern + ' ' + LongTimePattern};
+				return new string [] {ShortDatePattern + " " + LongTimePattern};
 			case 'g':
 				list = PopulateCombinedList (_ShortDatePatterns, _ShortTimePatterns);
 				if (list != null && list.Length > 0)
 					return list;
-				return new string [] {ShortDatePattern + ' ' + ShortTimePattern};
+				return new string [] {ShortDatePattern + " " + ShortTimePattern};
 			// The 'U' pattern strings are always the same as 'F'.
 			// (only differs in assuming UTC or not.)
 			case 'U':
@@ -610,12 +625,12 @@ namespace System.Globalization
 				list = PopulateCombinedList (_LongDatePatterns, _LongTimePatterns);
 				if (list != null && list.Length > 0)
 					return list;
-				return new string [] {LongDatePattern + ' ' + LongTimePattern};
+				return new string [] {LongDatePattern + " " + LongTimePattern};
 			case 'f':
 				list = PopulateCombinedList (_LongDatePatterns, _ShortTimePatterns);
 				if (list != null && list.Length > 0)
 					return list;
-				return new string [] {LongDatePattern + ' ' + ShortTimePattern};
+				return new string [] {LongDatePattern + " " + ShortTimePattern};
 			// MonthDay
 			case 'm':
 			case 'M':
@@ -678,15 +693,10 @@ namespace System.Globalization
 				int i = 0;
 				foreach (string d in dates)
 					foreach (string t in times)
-						list [i++] = d + ' ' + t;
+						list [i++] = d + " " + t;
 				return list;
 			}
 			return null;
 		}
-
-		private static void notImplemented()
-		{
-			throw new Exception("Not implemented");
-		}
 	}
 }
