@@ -67,6 +67,7 @@ public class GregorianCalendar : Calendar {
 			return twoDigitYearMax;
 		}
 		set {
+			CheckReadOnly ();
 			M_ArgumentInRange ("value", value, 100, M_MaxYear);
 
 			twoDigitYearMax = value;
@@ -86,7 +87,8 @@ public class GregorianCalendar : Calendar {
 	public virtual GregorianCalendarTypes CalendarType {
 		get { return M_CalendarType; }
 		set { 
-			// mscorlib 1:0:33000:0 doesn't check anything here
+			CheckReadOnly ();
+			// mscorlib 1:0:3300:0 doesn't check anything here
 			M_CalendarType = value;
 		}
 	}
@@ -190,12 +192,6 @@ public class GregorianCalendar : Calendar {
 		return CCGregorianCalendar.AddMonths(time, months);
 	}
 
-	[MonoTODO]
-	public override DateTime AddWeeks(DateTime time, int weeks) 
-	{
-		throw new NotImplementedException();
-	}
-
 	/// <summary>
 	/// Overridden. Adds years to a given date.
 	/// </summary>
@@ -487,6 +483,29 @@ public class GregorianCalendar : Calendar {
 			M_TwoDigitYearMax = 2029;
 	}
 	
+#if NET_2_0
+	public override CalendarAlgorithmType AlgorithmType {
+		get {
+			return CalendarAlgorithmType.SolarCalendar;
+		}
+	}
+
+	static DateTime Min = new DateTime (1, 1, 1, 0, 0, 0);
+	static DateTime Max = new DateTime (9999, 12, 31, 11, 59, 59);
+		
+	public override DateTime MinSupportedDateTime {
+		get {
+			return Min;
+		}
+	}
+
+	public override DateTime MaxSupportedDateTime {
+		get {
+			return Max;
+		}
+	}
+#endif
+	
 	/// <summary>
 	/// Default constructor. Sets the Gregorian calendar type to 
 	/// <see
