@@ -111,6 +111,7 @@ public class HijriCalendar : Calendar {
 			return twoDigitYearMax;
 		}
 		set {
+			CheckReadOnly ();
 			M_ArgumentInRange ("value", value, 100, M_MaxYear);
 
 			twoDigitYearMax = value;
@@ -132,6 +133,7 @@ public class HijriCalendar : Calendar {
 			return M_AddHijriDate;
 		}
 		set {
+			CheckReadOnly ();
 			if (value < -3 && value > 3)
 				throw new ArgumentOutOfRangeException(
 					"AddHijriDate",
@@ -929,6 +931,29 @@ public class HijriCalendar : Calendar {
 	{
 		throw new NotImplementedException();
 	}
+
+#if NET_2_0
+	public override CalendarAlgorithmType AlgorithmType  {
+		get {
+			return CalendarAlgorithmType.LunarCalendar;
+		}
+	}
+
+	static DateTime Min = new DateTime (622, 7, 18, 0, 0, 0);
+	static DateTime Max = new DateTime (9999, 12, 31, 11, 59, 59);
+		
+	public override DateTime MinSupportedDateTime {
+		get {
+			return Min;
+		}
+	}
+
+	public override DateTime MaxSupportedDateTime {
+		get {
+			return Max;
+		}
+	}
+#endif
 	
 } // class HijriCalendar
 	
