@@ -89,6 +89,10 @@ namespace System.IO
 				message = String.Format ("Invalid parameter");
 				return new IOException (message);
 
+			case MonoIOError.ERROR_WRITE_FAULT:
+				message = String.Format ("Write fault on path {0}", path);
+				return new IOException (message);
+
 			case MonoIOError.ERROR_SHARING_VIOLATION:
 				message = String.Format ("Sharing violation on path {0}", path);
 				return new IOException (message);
@@ -96,6 +100,10 @@ namespace System.IO
 			case MonoIOError.ERROR_LOCK_VIOLATION:
 				message = String.Format ("Lock violation on path {0}", path);
 				return new IOException (message);
+
+			case MonoIOError.ERROR_DIR_NOT_EMPTY:
+				message = String.Format ("Directory {0} is not empty", path);
+				return new IOException (message);
 				
 			default:
 				message = String.Format ("Win32 IO returned {0}. Path: {1}", error, path);
@@ -112,14 +120,7 @@ namespace System.IO
 		public extern static bool RemoveDirectory (string path, out MonoIOError error);
 
 		[MethodImplAttribute (MethodImplOptions.InternalCall)]
-		public extern static IntPtr FindFirstFile (string path, out MonoIOStat stat, out MonoIOError error);
-
-		[MethodImplAttribute (MethodImplOptions.InternalCall)]
-		public extern static bool FindNextFile (IntPtr find, out MonoIOStat stat, out MonoIOError error);
-
-		[MethodImplAttribute (MethodImplOptions.InternalCall)]
-		public extern static bool FindClose (IntPtr find,
-						     out MonoIOError error);
+		public extern static string [] GetFileSystemEntries (string path, string pattern, int attrs, int mask, out MonoIOError error);
 
 		[MethodImplAttribute (MethodImplOptions.InternalCall)]
 		public extern static string GetCurrentDirectory (out MonoIOError error);
