@@ -63,11 +63,10 @@ namespace System.IO
 		ERROR_SEEK = 25,
 		ERROR_NOT_DOS_DISK = 26,
 		ERROR_SECTOR_NOT_FOUND = 27,
-		ERROR_OUT_OF_PAPER = 28,
+		ERROR_OUT_OF_PAPER = 28,*/
 		ERROR_WRITE_FAULT = 29,
 		ERROR_READ_FAULT = 30,
 		ERROR_GEN_FAILURE = 31,
-*/
 		ERROR_SHARING_VIOLATION = 32,
 		ERROR_LOCK_VIOLATION = 33,
 	/*	ERROR_WRONG_DISK = 34,
@@ -149,7 +148,9 @@ namespace System.IO
 		ERROR_BUSY_DRIVE = 142,
 		ERROR_SAME_DRIVE = 143,
 		ERROR_DIR_NOT_ROOT = 144,
+	*/
 		ERROR_DIR_NOT_EMPTY = 145,
+	/*
 		ERROR_IS_SUBST_PATH = 146,
 		ERROR_IS_JOIN_PATH = 147,
 		ERROR_PATH_BUSY = 148,
