@@ -83,11 +83,11 @@ namespace System.Reflection.Emit {
 
 #if NET_2_0
 	[ComVisible (true)]
-	[ClassInterfaceAttribute (ClassInterfaceType.None)]
-	[ComDefaultInterfaceAttribute (typeof (_AssemblyBuilder))]
+	[ComDefaultInterface (typeof (_AssemblyBuilder))]
 #endif
-	public sealed class AssemblyBuilder : Assembly {
-		#region Sync with reflection.h
+	[ClassInterface (ClassInterfaceType.None)]
+	public sealed class AssemblyBuilder : Assembly, _AssemblyBuilder {
+		#region Sync with object-internals.h
 		private IntPtr dynamic_assembly;
 		private MethodInfo entry_point;
 		private ModuleBuilder[] modules;
@@ -525,7 +525,9 @@ namespace System.Reflection.Emit {
 
 		private void DefineVersionInfoResourceImpl (string fileName) {
 			// Add missing info
-			if (version_res.FileVersion == "0.0.0.0")
+			if (version_res.Version == "0.0.0.0")
+				version_res.Version = version;
+			if (version_res.FileVersion.Trim () == "" && version != null)
 				version_res.FileVersion = version;
 			version_res.InternalName = Path.GetFileNameWithoutExtension (fileName);
 			version_res.OriginalFilename = fileName;
@@ -850,5 +852,34 @@ namespace System.Reflection.Emit {
 		{
 			return (str == "neutral" ? String.Empty : str);
 		}
+
+		internal override AssemblyName UnprotectedGetName ()
+		{
+			AssemblyName an = base.UnprotectedGetName ();
+			if (sn != null) {
+				an.SetPublicKey (sn.PublicKey);
+			}
+			return an;
+		}
+
+		void _AssemblyBuilder.GetIDsOfNames([In] ref Guid riid, IntPtr rgszNames, uint cNames, uint lcid, IntPtr rgDispId)
+		{
+			throw new NotImplementedException ();
+		}
+
+		void _AssemblyBuilder.GetTypeInfo (uint iTInfo, uint lcid, IntPtr ppTInfo)
+		{
+			throw new NotImplementedException ();
+		}
+
+		void _AssemblyBuilder.GetTypeInfoCount (out uint pcTInfo)
+		{
+			throw new NotImplementedException ();
+		}
+
+		void _AssemblyBuilder.Invoke (uint dispIdMember, [In] ref Guid riid, uint lcid, short wFlags, IntPtr pDispParams, IntPtr pVarResult, IntPtr pExcepInfo, IntPtr puArgErr)
+		{
+			throw new NotImplementedException ();
+		}
 	}
 }
