@@ -50,6 +50,7 @@ namespace System.Reflection.Emit
 		private int index;
 		private Type base_type;
 		private Type[] iface_constraints;
+		private CustomAttributeBuilder[] cattrs;
 		private GenericParameterAttributes attrs;
 	#endregion
 
@@ -316,10 +317,6 @@ namespace System.Reflection.Emit
 			throw not_supported ();
 		}
 
-		public override bool HasGenericArguments {
-			get { return false; }
-		}
-
 		public override bool ContainsGenericParameters {
 			get { return true; }
 		}
@@ -356,8 +353,26 @@ namespace System.Reflection.Emit
 			return ret;
 		}
 
-		public override MethodInfo DeclaringMethod {
-			get { return mbuilder; }
+		public override MethodBase DeclaringMethod {
+			get { 
+				return mbuilder;
+			}
+		}
+
+		public void SetCustomAttribute (CustomAttributeBuilder customBuilder)
+		{
+			if (customBuilder == null)
+				throw new ArgumentNullException ("customBuilder");
+
+			if (cattrs != null) {
+				CustomAttributeBuilder[] new_array = new CustomAttributeBuilder [cattrs.Length + 1];
+				cattrs.CopyTo (new_array, 0);
+				new_array [cattrs.Length] = customBuilder;
+				cattrs = new_array;
+			} else {
+				cattrs = new CustomAttributeBuilder [1];
+				cattrs [0] = customBuilder;
+			}
 		}
 
 		private Exception not_supported ()
