@@ -104,6 +104,17 @@ namespace System.Reflection.Emit {
 			handlers [i].extype = null;
 		}
 
+		internal void AddFilter (int offset)
+		{
+			int i;
+			End (offset);
+			add_block (offset);
+			i = handlers.Length - 1;
+			handlers [i].type = ILExceptionBlock.FILTER;
+			handlers [i].extype = null;
+			handlers [i].filter_offset = offset;
+		}
+
 		internal void End (int offset)
 		{
 			if (handlers == null)
@@ -121,6 +132,12 @@ namespace System.Reflection.Emit {
 				return ILExceptionBlock.CATCH;
 		}
 
+		internal void PatchLastClauseStart (int start)
+		{
+			if (handlers != null && handlers.Length > 0)
+				handlers [handlers.Length - 1].start = start;
+		}
+
 		internal void Debug (int b)
 		{
 #if NO
@@ -162,10 +179,9 @@ namespace System.Reflection.Emit {
 
 #if NET_2_0
 	[ComVisible (true)]
-	[ClassInterfaceAttribute (ClassInterfaceType.None)]
-	[ComDefaultInterfaceAttribute (typeof (_ILGenerator))]
 #endif
-	public class ILGenerator: Object {
+	[ClassInterface (ClassInterfaceType.None)]
+	public class ILGenerator: _ILGenerator {
 		private struct LabelFixup {
 			public int offset;    // The number of bytes between pos and the
 							      // offset of the jump
@@ -329,6 +345,7 @@ namespace System.Reflection.Emit {
 		{
 			switch (ex_handlers [cur_block].LastClauseType ()) {
 			case ILExceptionBlock.CATCH:
+			case ILExceptionBlock.FILTER:
 				// how could we optimize code size here?
 				Emit (OpCodes.Leave, ex_handlers [cur_block].end);
 				break;
@@ -336,9 +353,6 @@ namespace System.Reflection.Emit {
 			case ILExceptionBlock.FINALLY:
 				Emit (OpCodes.Endfinally);
 				break;
-			case ILExceptionBlock.FILTER:
-				Emit (OpCodes.Endfilter);
-				break;
 			}
 		}
 
@@ -346,19 +360,31 @@ namespace System.Reflection.Emit {
 		{
 			if (open_blocks.Count <= 0)
 				throw new NotSupportedException ("Not in an exception block");
-			InternalEndClause ();
-			ex_handlers [cur_block].AddCatch (exceptionType, code_len);
+
+			if (ex_handlers [cur_block].LastClauseType () == ILExceptionBlock.FILTER) {
+				if (exceptionType != null)
+					throw new ArgumentException ("Do not supply an exception type for filter clause");
+				Emit (OpCodes.Endfilter);
+				ex_handlers [cur_block].PatchLastClauseStart (code_len);
+			} else {
+				InternalEndClause ();
+				ex_handlers [cur_block].AddCatch (exceptionType, code_len);
+			}
+			
 			cur_stack = 1; // the exception object is on the stack by default
 			if (max_stack < cur_stack)
 				max_stack = cur_stack;
+
 			//System.Console.WriteLine ("Begin catch Block: {0} {1}",exceptionType.ToString(), max_stack);
-			//throw new NotImplementedException ();
 		}
 
-		[MonoTODO]
 		public virtual void BeginExceptFilterBlock ()
 		{
-			throw new NotImplementedException ();
+			if (open_blocks.Count <= 0)
+				throw new NotSupportedException ("Not in an exception block");
+			InternalEndClause ();
+
+			ex_handlers [cur_block].AddFilter (code_len);
 		}
 
 		public virtual Label BeginExceptionBlock ()
@@ -383,6 +409,7 @@ namespace System.Reflection.Emit {
 		{
 			if (open_blocks.Count <= 0)
 				throw new NotSupportedException ("Not in an exception block");
+			InternalEndClause ();
 			//System.Console.WriteLine ("Begin fault Block");
 			ex_handlers [cur_block].AddFault (code_len);
 		}
@@ -459,7 +486,9 @@ namespace System.Reflection.Emit {
 			code [code_len++] = val;
 		}
 		
+#if NET_2_0
 		[ComVisible (true)]
+#endif
 		public virtual void Emit (OpCode opcode, ConstructorInfo constructor)
 		{
 			int token = token_gen.GetToken (constructor);
@@ -475,8 +504,6 @@ namespace System.Reflection.Emit {
 		
 		public virtual void Emit (OpCode opcode, double val)
 		{
-			Double.AssertEndianity (out val);
-
 			byte[] s = System.BitConverter.GetBytes (val);
 			make_room (10);
 			ll_emit (opcode);
@@ -929,6 +956,26 @@ namespace System.Reflection.Emit {
 		{
 			return ig.code_len;
 		}
+
+		void _ILGenerator.GetIDsOfNames ([In] ref Guid riid, IntPtr rgszNames, uint cNames, uint lcid, IntPtr rgDispId)
+		{
+			throw new NotImplementedException ();
+		}
+
+		void _ILGenerator.GetTypeInfo (uint iTInfo, uint lcid, IntPtr ppTInfo)
+		{
+			throw new NotImplementedException ();
+		}
+
+		void _ILGenerator.GetTypeInfoCount (out uint pcTInfo)
+		{
+			throw new NotImplementedException ();
+		}
+
+		void _ILGenerator.Invoke (uint dispIdMember, [In] ref Guid riid, uint lcid, short wFlags, IntPtr pDispParams, IntPtr pVarResult, IntPtr pExcepInfo, IntPtr puArgErr)
+		{
+			throw new NotImplementedException ();
+		}
 	}
 	
 	internal class SequencePointList
