@@ -43,10 +43,10 @@ using System.Diagnostics.SymbolStore;
 namespace System.Reflection.Emit {
 #if NET_2_0
 	[ComVisible (true)]
-	[ClassInterfaceAttribute (ClassInterfaceType.None)]
-	[ComDefaultInterfaceAttribute (typeof (_MethodBuilder))]
+	[ComDefaultInterface (typeof (_MethodBuilder))]
 #endif
-	public sealed class MethodBuilder : MethodInfo {
+	[ClassInterface (ClassInterfaceType.None)]
+	public sealed class MethodBuilder : MethodInfo, _MethodBuilder {
 		private RuntimeMethodHandle mhandle;
 		private Type rtype;
 		private Type[] parameters;
@@ -215,7 +215,7 @@ namespace System.Reflection.Emit {
 
 		public void CreateMethodBody( byte[] il, int count) {
 			if ((il != null) && ((count < 0) || (count > il.Length)))
-				throw new ArgumentException ("Index was out of range.  Must be non-negative and less than the size of the collection.");
+				throw new ArgumentOutOfRangeException ("Index was out of range.  Must be non-negative and less than the size of the collection.");
 
 			if ((code != null) || type.is_created)
 				throw new InvalidOperationException ("Type definition of the method is complete.");
@@ -235,11 +235,24 @@ namespace System.Reflection.Emit {
 			throw NotSupported ();
 		}
 		public override object[] GetCustomAttributes( bool inherit) {
-			throw NotSupported ();
+			/*
+			 * On MS.NET, this always returns not_supported, but we can't do this
+			 * since there would be no way to obtain custom attributes of 
+			 * dynamically created ctors.
+			 */
+			if (type.is_created)
+				return MonoCustomAttrs.GetCustomAttributes (this, inherit);
+			else
+				throw NotSupported ();
 		}
+
 		public override object[] GetCustomAttributes( Type attributeType, bool inherit) {
-			throw NotSupported ();
+			if (type.is_created)
+				return MonoCustomAttrs.GetCustomAttributes (this, attributeType, inherit);
+			else
+				throw NotSupported ();
 		}
+
 		public ILGenerator GetILGenerator () {
 			return GetILGenerator (64);
 		}
@@ -275,7 +288,12 @@ namespace System.Reflection.Emit {
 
 		internal void fixup () {
 			if (((attrs & (MethodAttributes.Abstract | MethodAttributes.PinvokeImpl)) == 0) && ((iattrs & (MethodImplAttributes.Runtime | MethodImplAttributes.InternalCall)) == 0)) {
+#if NET_2_0
+				// do not allow zero length method body on MS.NET 2.0 (and higher)
+				if (((ilgen == null) || (ILGenerator.Mono_GetCurrentOffset (ilgen) == 0)) && (code == null || code.Length == 0))
+#else
 				if (((ilgen == null) || (ILGenerator.Mono_GetCurrentOffset (ilgen) == 0)) && (code == null))
+#endif
 					throw new InvalidOperationException ("Method '" + Name + "' does not have a method body.");
 			}
 			if (ilgen != null)
@@ -346,6 +364,12 @@ namespace System.Reflection.Emit {
 						ThrowOnUnmappableChar = (bool)value;
 #endif
 				}
+#if NET_2_0
+			if (attrname == "System.Runtime.CompilerServices.SpecialNameAttribute") {
+				attrs |= MethodAttributes.SpecialName;
+				return;
+			}
+#endif
 
 				attrs |= MethodAttributes.PinvokeImpl;
 				if (preserveSig)
@@ -456,7 +480,7 @@ namespace System.Reflection.Emit {
 
 #if NET_2_0 || BOOTSTRAP_NET_2_0
 		[MethodImplAttribute(MethodImplOptions.InternalCall)]
-		public override extern MethodInfo BindGenericParameters (Type [] types);
+		public override extern MethodInfo MakeGenericMethod (Type [] types);
 
 		public override bool Mono_IsInflatedMethod {
 			get {
@@ -538,6 +562,27 @@ namespace System.Reflection.Emit {
 			}
 		}
 #endif
+
+                void _MethodBuilder.GetIDsOfNames([In] ref Guid riid, IntPtr rgszNames, uint cNames, uint lcid, IntPtr rgDispId)
+                {
+                        throw new NotImplementedException ();
+                }
+
+                void _MethodBuilder.GetTypeInfo (uint iTInfo, uint lcid, IntPtr ppTInfo)
+                {
+                        throw new NotImplementedException ();
+                }
+
+                void _MethodBuilder.GetTypeInfoCount (out uint pcTInfo)
+                {
+                        throw new NotImplementedException ();
+                }
+
+                void _MethodBuilder.Invoke (uint dispIdMember, [In] ref Guid riid, uint lcid, short wFlags, IntPtr pDispParams, IntPtr pVarResult, IntPtr pExcepInfo, IntPtr puArgErr)
+                {
+                        throw new NotImplementedException ();
+                }
+
 	}
 }
 
