@@ -44,10 +44,10 @@ using System.Globalization;
 namespace System.Reflection.Emit {
 #if NET_2_0
 	[ComVisible (true)]
-	[ClassInterfaceAttribute (ClassInterfaceType.None)]
-	[ComDefaultInterfaceAttribute (typeof (_ModuleBuilder))]
+	[ComDefaultInterface (typeof (_ModuleBuilder))]
 #endif
-	public class ModuleBuilder : Module {
+	[ClassInterface (ClassInterfaceType.None)]
+	public class ModuleBuilder : Module, _ModuleBuilder {
 		#region Sync with reflection.h
 		private IntPtr dynamic_image;
 		private int num_types;
@@ -704,6 +704,26 @@ namespace System.Reflection.Emit {
 		{
 			return new Guid (mb.guid);
 		}
+
+		void _ModuleBuilder.GetIDsOfNames ([In] ref Guid riid, IntPtr rgszNames, uint cNames, uint lcid, IntPtr rgDispId)
+		{
+			throw new NotImplementedException ();
+		}
+
+		void _ModuleBuilder.GetTypeInfo (uint iTInfo, uint lcid, IntPtr ppTInfo)
+		{
+			throw new NotImplementedException ();
+		}
+
+		void _ModuleBuilder.GetTypeInfoCount (out uint pcTInfo)
+		{
+			throw new NotImplementedException ();
+		}
+
+		void _ModuleBuilder.Invoke (uint dispIdMember, [In] ref Guid riid, uint lcid, short wFlags, IntPtr pDispParams, IntPtr pVarResult, IntPtr pExcepInfo, IntPtr puArgErr)
+		{
+			throw new NotImplementedException ();
+		}
 	}
 
 	internal class ModuleBuilderTokenGenerator : TokenGenerator {
