@@ -42,10 +42,10 @@ using System.Runtime.InteropServices;
 namespace System.Reflection.Emit {
 #if NET_2_0
 	[ComVisible (true)]
-	[ClassInterfaceAttribute (ClassInterfaceType.None)]
-	[ComDefaultInterfaceAttribute (typeof (_ParameterBuilder))]
+	[ComDefaultInterface (typeof (_ParameterBuilder))]
 #endif
-	public class ParameterBuilder {
+	[ClassInterface (ClassInterfaceType.None)]
+	public class ParameterBuilder : _ParameterBuilder {
 		private MethodBase methodb; /* MethodBuilder or ConstructorBuilder */
 		private string name;
 		private CustomAttributeBuilder[] cattrs;
@@ -104,10 +104,18 @@ namespace System.Reflection.Emit {
 				attrs |= ParameterAttributes.Optional;
 				return;
 			} else if (attrname == "System.Runtime.InteropServices.MarshalAsAttribute") {
+				attrs |= ParameterAttributes.HasFieldMarshal;
 				marshal_info = CustomAttributeBuilder.get_umarshal (customBuilder, true);
 				/* FIXME: check for errors */
 				return;
+			} else if (attrname == "System.Runtime.InteropServices.DefaultParameterValueAttribute") {
+				/* MS.NET doesn't handle this attribute but we handle it for consistency */
+				CustomAttributeBuilder.CustomAttributeInfo cinfo = CustomAttributeBuilder.decode_cattr (customBuilder);
+				/* FIXME: check for type compatibility */
+				SetConstant (cinfo.ctorArgs [0]);
+				return;
 			}
+
 			if (cattrs != null) {
 				CustomAttributeBuilder[] new_array = new CustomAttributeBuilder [cattrs.Length + 1];
 				cattrs.CopyTo (new_array, 0);
@@ -134,6 +142,25 @@ namespace System.Reflection.Emit {
 			attrs |= ParameterAttributes.HasFieldMarshal;
 		}
 
+                void _ParameterBuilder.GetIDsOfNames([In] ref Guid riid, IntPtr rgszNames, uint cNames, uint lcid, IntPtr rgDispId)
+                {
+                        throw new NotImplementedException ();
+                }
+
+                void _ParameterBuilder.GetTypeInfo (uint iTInfo, uint lcid, IntPtr ppTInfo)
+                {
+                        throw new NotImplementedException ();
+                }
+
+                void _ParameterBuilder.GetTypeInfoCount (out uint pcTInfo)
+                {
+                        throw new NotImplementedException ();
+                }
+
+                void _ParameterBuilder.Invoke (uint dispIdMember, [In] ref Guid riid, uint lcid, short wFlags, IntPtr pDispParams, IntPtr pVarResult, IntPtr pExcepInfo, IntPtr puArgErr)
+                {
+                        throw new NotImplementedException ();
+                }
 	}
 }
 
