@@ -45,10 +45,10 @@ using System.Diagnostics.SymbolStore;
 namespace System.Reflection.Emit {
 #if NET_2_0
 	[ComVisible (true)]
-	[ClassInterfaceAttribute (ClassInterfaceType.None)]
-	[ComDefaultInterfaceAttribute (typeof (_TypeBuilder))]
+	[ComDefaultInterface (typeof (_TypeBuilder))]
 #endif
-	public sealed class TypeBuilder : Type {
+	[ClassInterface (ClassInterfaceType.None)]
+	public sealed class TypeBuilder : Type, _TypeBuilder {
 	#region Sync with reflection.h
 	private string tname;
 	private string nspace;
@@ -680,6 +680,8 @@ namespace System.Reflection.Emit {
 			if (created != null)
 				return created;
 
+			create_generic_class ();
+
 			// Fire TypeResolve events for fields whose type is an unfinished
 			// value type.
 			if (fields != null) {
@@ -973,7 +975,7 @@ namespace System.Reflection.Emit {
 			}
 			else
 				candidates = methods;
-					
+
 			if (candidates == null)
 				return new MethodInfo [0];
 
@@ -1273,10 +1275,19 @@ namespace System.Reflection.Emit {
 					}
 				}
 				return;
+#if NET_2_0
+			} else if (attrname == "System.Runtime.CompilerServices.SpecialNameAttribute") {
+				attrs |= TypeAttributes.SpecialName;
+				return;
+#endif
 			} else if (attrname == "System.SerializableAttribute") {
 				attrs |= TypeAttributes.Serializable;
 				return;
+			} else if (attrname == "System.Runtime.InteropServices.ComImportAttribute") {
+				attrs |= TypeAttributes.Import;
+				return;
 			}
+
 			if (cattrs != null) {
 				CustomAttributeBuilder[] new_array = new CustomAttributeBuilder [cattrs.Length + 1];
 				cattrs.CopyTo (new_array, 0);
@@ -1399,10 +1410,10 @@ namespace System.Reflection.Emit {
 		{
 			if (name == null)
 				throw new ArgumentNullException (argName);
-			if (name == "")
-				throw new ArgumentException (argName, "Empty name is not legal.");
+			if (name.Length == 0)
+				throw new ArgumentException ("Empty name is not legal", argName);
 			if (name.IndexOf ((char)0) != -1)
-				throw new ArgumentException (argName, "Illegal name.");
+				throw new ArgumentException ("Illegal name", argName);
 		}
 
 		public override String ToString ()
@@ -1466,12 +1477,6 @@ namespace System.Reflection.Emit {
 			return base.GetGenericTypeDefinition ();
 		}
 
-		public override bool HasGenericArguments {
-			get {
-				return generic_params != null;
-			}
-		}
-
 		public override bool ContainsGenericParameters {
 			get {
 				return generic_params != null;
@@ -1483,13 +1488,19 @@ namespace System.Reflection.Emit {
 			get;
 		}
 
+		public override bool IsGenericTypeDefinition {
+			get {
+				return generic_params != null;
+			}
+		}
+
 		public override int GenericParameterPosition {
 			get {
 				throw new NotImplementedException ();
 			}
 		}
 
-		public override MethodInfo DeclaringMethod {
+		public override MethodBase DeclaringMethod {
 			get {
 				throw new NotImplementedException ();
 			}
@@ -1539,5 +1550,25 @@ namespace System.Reflection.Emit {
 				return res;
                 }
 #endif
+
+                void _TypeBuilder.GetIDsOfNames([In] ref Guid riid, IntPtr rgszNames, uint cNames, uint lcid, IntPtr rgDispId)
+                {
+                        throw new NotImplementedException ();
+                }
+
+                void _TypeBuilder.GetTypeInfo (uint iTInfo, uint lcid, IntPtr ppTInfo)
+                {
+                        throw new NotImplementedException ();
+                }
+
+                void _TypeBuilder.GetTypeInfoCount (out uint pcTInfo)
+                {
+                        throw new NotImplementedException ();
+                }
+
+                void _TypeBuilder.Invoke (uint dispIdMember, [In] ref Guid riid, uint lcid, short wFlags, IntPtr pDispParams, IntPtr pVarResult, IntPtr pExcepInfo, IntPtr puArgErr)
+                {
+                        throw new NotImplementedException ();
+                }
 	}
 }
