@@ -5,7 +5,7 @@
 //   Paolo Molaro (lupus@ximian.com)
 //
 // (C) 2001 Ximian, Inc.  http://www.ximian.com
-// Copyright (C) 2004 Novell, Inc (http://www.novell.com)
+// Copyright (C) 2004-2005 Novell, Inc (http://www.novell.com)
 //
 // Permission is hereby granted, free of charge, to any person obtaining
 // a copy of this software and associated documentation files (the
@@ -27,7 +27,6 @@
 // WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 //
 
-using System;
 using System.Security;
 using System.Security.Policy;
 using System.Security.Permissions;
@@ -50,8 +49,9 @@ namespace System.Reflection {
 #endif
 	[Serializable]
 	[ClassInterface(ClassInterfaceType.None)]
-	public class Assembly : System.Reflection.ICustomAttributeProvider,
+	public class Assembly : System.Reflection.ICustomAttributeProvider, _Assembly,
 		System.Security.IEvidenceFactory, System.Runtime.Serialization.ISerializable {
+
 		internal class ResolveEventHolder {
 			public event ModuleResolveEventHandler ModuleResolve;
 		}
@@ -197,6 +197,13 @@ namespace System.Reflection {
 			UnitySerializationHolder.GetAssemblyData (this, info, context);
 		}
 
+#if ONLY_1_1
+		public new Type GetType ()
+		{
+			return base.GetType ();
+		}
+#endif
+
 		public virtual bool IsDefined (Type attributeType, bool inherit)
 		{
 			return MonoCustomAttrs.IsDefined (this, attributeType, inherit);
@@ -247,7 +254,7 @@ namespace System.Reflection {
 		}
 
 		[MethodImplAttribute (MethodImplOptions.InternalCall)]
-		private extern IntPtr GetManifestResourceInternal (String name, out int size, out Module module);
+		internal extern IntPtr GetManifestResourceInternal (String name, out int size, out Module module);
 
 		public virtual Stream GetManifestResourceStream (String name)
 		{
@@ -354,7 +361,7 @@ namespace System.Reflection {
 		}
 
 		// the security runtime requires access to the assemblyname (e.g. to get the strongname)
-		internal AssemblyName UnprotectedGetName ()
+		internal virtual AssemblyName UnprotectedGetName ()
 		{
 			AssemblyName aname = new AssemblyName ();
 			FillName (this, aname);
@@ -690,28 +697,7 @@ namespace System.Reflection {
 		//
 
 		[MethodImplAttribute (MethodImplOptions.InternalCall)]
-		internal static extern MethodBase MonoDebugger_GetMethod (Assembly assembly, int token);
-
-		[MethodImplAttribute (MethodImplOptions.InternalCall)]
-		internal static extern int MonoDebugger_GetMethodToken (Assembly assembly, MethodBase method);
-
-		[MethodImplAttribute (MethodImplOptions.InternalCall)]
-		internal static extern Type MonoDebugger_GetLocalTypeFromSignature (Assembly assembly, byte[] signature);
-
-		[MethodImplAttribute (MethodImplOptions.InternalCall)]
-		internal static extern Type MonoDebugger_GetType (Assembly assembly, int token);
-
-		[MethodImplAttribute (MethodImplOptions.InternalCall)]
-		internal static extern string MonoDebugger_CheckRuntimeVersion (string filename);
-
-		[MethodImplAttribute (MethodImplOptions.InternalCall)]
-		internal static extern string MonoDebugger_GetMethodIndex (MethodBase method);
-
-		[MethodImplAttribute (MethodImplOptions.InternalCall)]
-		internal static extern Type MonoDebugger_MakeArrayType (Type type, int rank);
-
-		[MethodImplAttribute (MethodImplOptions.InternalCall)]
-		internal static extern int MonoDebugger_GetTypeToken (Type type);
+		internal static extern int MonoDebugger_GetMethodToken (MethodBase method);
 
 #if NET_2_0
 		[MonoTODO]
@@ -720,43 +706,12 @@ namespace System.Reflection {
 			get { return 0; }
 		}
 
-		[ComVisible (false)]
-		[Obsolete ("Please use Assembly.ManifestModule.GetPEKind() instead - this will be removed before Whidbey ships.")]
-		public ImageFileMachine ImageFileMachine {
-			get {
-				ImageFileMachine machine;
-				PortableExecutableKinds kind;
-				ModuleHandle handle = ManifestModule.ModuleHandle;
-				handle.GetPEKind (out kind, out machine);
-				return machine;
-			}
-		}
-
 		[ComVisible (false)]
 		public extern Module ManifestModule {
 			[MethodImplAttribute (MethodImplOptions.InternalCall)]
 			get;
 		}
 
-		[ComVisible (false)]
-		[Obsolete ("This method has been deprecated and will be removed before v2.0 RTM is released.")]
-		public extern int MetadataToken {
-			[MethodImplAttribute (MethodImplOptions.InternalCall)]
-			get;
-		}
-
-		[ComVisible (false)]
-		[Obsolete ("Please use Assembly.ManifestModule.GetPEKind() instead - this will be removed before Whidbey ships.")]
-		public PortableExecutableKinds PortableExecutableKinds {
-			get {
-				ImageFileMachine machine;
-				PortableExecutableKinds kind;
-				ModuleHandle handle = ManifestModule.ModuleHandle;
-				handle.GetPEKind (out kind, out machine);
-				return kind;
-			}
-		}
-
 		[ComVisible (false)]
 		public virtual extern bool ReflectionOnly {
 			[MethodImplAttribute (MethodImplOptions.InternalCall)]
@@ -773,28 +728,22 @@ namespace System.Reflection {
 				// Demand it's too late to evaluate the Minimum permission set as a 
 				// condition to load the assembly into the AppDomain
 				LoadAssemblyPermissions ();
-				_granted = SecurityManager.ResolvePolicy (UnprotectedGetEvidence (),
+				Evidence e = new Evidence (UnprotectedGetEvidence ()); // we need a copy to add PRE
+				e.AddHost (new PermissionRequestEvidence (_minimum, _optional, _refuse));
+				_granted = SecurityManager.ResolvePolicy (e,
 					_minimum, _optional, _refuse, out _denied);
 			}
-#if false
-			Console.WriteLine ("*** ASSEMBLY RESOLVE INPUT ***");
-			if (_minimum != null)
-				Console.WriteLine ("Minimum: {0}", _minimum);
-			if (_optional != null)
-				Console.WriteLine ("Optional: {0}", _optional);
-			if (_refuse != null)
-				Console.WriteLine ("Refuse: {0}", _refuse);
-			Console.WriteLine ("*** ASSEMBLY RESOLVE RESULTS ***");
-			Console.WriteLine ("Granted: {0}", _granted);
-			if (_denied != null)
-				Console.WriteLine ("Denied: {0}", _denied);
-			Console.WriteLine ("*** ASSEMBLY RESOLVE END ***");
-#endif
 		}
 
 		internal PermissionSet GrantedPermissionSet {
 			get {
 				if (_granted == null) {
+					if (SecurityManager.ResolvingPolicyLevel != null) {
+						if (SecurityManager.ResolvingPolicyLevel.IsFullTrustAssembly (this))
+							return DefaultPolicies.FullTrust;
+						else
+							return null; // we can't resolve during resolution
+					}
 					Resolve ();
 				}
 				return _granted;
@@ -805,6 +754,12 @@ namespace System.Reflection {
 			get {
 				// yes we look for granted, as denied may be null
 				if (_granted == null) {
+					if (SecurityManager.ResolvingPolicyLevel != null) {
+						if (SecurityManager.ResolvingPolicyLevel.IsFullTrustAssembly (this))
+							return null;
+						else
+							return DefaultPolicies.FullTrust; // deny unrestricted
+					}
 					Resolve ();
 				}
 				return _denied;
