@@ -5,12 +5,7 @@
 //   Miguel de Icaza (miguel@ximian.com)
 //
 // (C) Ximian, Inc.  http://www.ximian.com
-//
-// TODO: Mucho left to implement.
-//
-
-//
-// Copyright (C) 2004 Novell, Inc (http://www.novell.com)
+// Copyright (C) 2004-2005 Novell, Inc (http://www.novell.com)
 //
 // Permission is hereby granted, free of charge, to any person obtaining
 // a copy of this software and associated documentation files (the
@@ -30,11 +25,8 @@
 // LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 // OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 // WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
-//
 
-using System;
 using System.Diagnostics;
-using System.Reflection;
 using System.Reflection.Emit;
 using System.Globalization;
 using System.Runtime.CompilerServices;
@@ -48,7 +40,8 @@ namespace System.Reflection {
 #endif
 	[Serializable]
 	[ClassInterface(ClassInterfaceType.None)]
-	public abstract class FieldInfo : MemberInfo {
+	public abstract class FieldInfo : MemberInfo, _FieldInfo {
+
 		public abstract FieldAttributes Attributes {get;}
 		public abstract RuntimeFieldHandle FieldHandle {get;}
 
@@ -141,6 +134,13 @@ namespace System.Reflection {
 
 		public abstract void SetValue (object obj, object val, BindingFlags invokeAttr, Binder binder, CultureInfo culture);
 
+#if ONLY_1_1
+		public new Type GetType ()
+		{
+			return base.GetType ();
+		}
+#endif
+
 		[DebuggerHidden]
 		[DebuggerStepThrough]
 		public void SetValue (object obj, object value)
@@ -188,6 +188,12 @@ namespace System.Reflection {
 		[MethodImplAttribute(MethodImplOptions.InternalCall)]
 		private extern UnmanagedMarshal GetUnmanagedMarshal ();
 
+		internal virtual UnmanagedMarshal UMarshal {
+			get {
+				return GetUnmanagedMarshal ();
+			}
+		}
+
 		internal object[] GetPseudoCustomAttributes ()
 		{
 			int count = 0;
@@ -198,7 +204,7 @@ namespace System.Reflection {
 			if (DeclaringType.IsExplicitLayout)
 				count ++;
 
-			UnmanagedMarshal marshalAs = GetUnmanagedMarshal ();
+			UnmanagedMarshal marshalAs = UMarshal;
 			if (marshalAs != null)
 				count ++;
 
@@ -218,20 +224,6 @@ namespace System.Reflection {
 		}
 
 #if NET_2_0 || BOOTSTRAP_NET_2_0
-		[Obsolete ("Use FieldInfo.GetOptionalCustomModifiers().")]
-		public virtual Type[] OptionalCustomModifiers {
-			get {
-				return GetOptionalCustomModifiers ();
-			}
-		}
-
-		[Obsolete ("Use FieldInfo.GetRequiredCustomModifiers().")]
-		public virtual Type[] RequiredCustomModifiers {
-			get {
-				return GetRequiredCustomModifiers ();
-			}
-		}
-
 		[MonoTODO]
 		public virtual Type[] GetOptionalCustomModifiers () {
 			throw new NotImplementedException ();
@@ -246,5 +238,25 @@ namespace System.Reflection {
 #if NET_2_0 || BOOTSTRAP_NET_2_0
 		public abstract FieldInfo Mono_GetGenericFieldDefinition ();
 #endif
+
+		void _FieldInfo.GetIDsOfNames ([In] ref Guid riid, IntPtr rgszNames, uint cNames, uint lcid, IntPtr rgDispId)
+		{
+			throw new NotImplementedException ();
+		}
+
+		void _FieldInfo.GetTypeInfo (uint iTInfo, uint lcid, IntPtr ppTInfo)
+		{
+			throw new NotImplementedException ();
+		}
+
+		void _FieldInfo.GetTypeInfoCount (out uint pcTInfo)
+		{
+			throw new NotImplementedException ();
+		}
+
+		void _FieldInfo.Invoke (uint dispIdMember, [In] ref Guid riid, uint lcid, short wFlags, IntPtr pDispParams, IntPtr pVarResult, IntPtr pExcepInfo, IntPtr puArgErr)
+		{
+			throw new NotImplementedException ();
+		}
 	}
 }
