@@ -5,10 +5,7 @@
 //   Paolo Molaro (lupus@ximian.com)
 //
 // (C) 2001 Ximian, Inc.  http://www.ximian.com
-//
-
-//
-// Copyright (C) 2004 Novell, Inc (http://www.novell.com)
+// Copyright (C) 2004-2005 Novell, Inc (http://www.novell.com)
 //
 // Permission is hereby granted, free of charge, to any person obtaining
 // a copy of this software and associated documentation files (the
@@ -30,7 +27,6 @@
 // WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 //
 
-using System;
 using System.Diagnostics;
 using System.Globalization;
 using System.Reflection.Emit;
@@ -45,7 +41,7 @@ namespace System.Reflection {
 #endif
 	[Serializable]
 	[ClassInterface(ClassInterfaceType.None)]
-	public abstract class MethodBase: MemberInfo {
+	public abstract class MethodBase: MemberInfo, _MethodBase {
 
 		[MethodImplAttribute (MethodImplOptions.InternalCall)]
 		public extern static MethodBase GetCurrentMethod ();
@@ -74,6 +70,13 @@ namespace System.Reflection {
 			return pi.Length;
 		}
 
+#if ONLY_1_1
+		public new Type GetType ()
+		{
+			return base.GetType ();
+		}
+#endif
+
 		[DebuggerHidden]
 		[DebuggerStepThrough]		
 #if NET_2_0 || BOOTSTRAP_NET_2_0
@@ -174,11 +177,6 @@ namespace System.Reflection {
 		}
 
 #if NET_2_0 || BOOTSTRAP_NET_2_0
-		public virtual MethodInfo BindGenericParameters (Type [] types)
-		{
-			throw new NotSupportedException ();
-		}
-
 		public virtual Type [] GetGenericArguments ()
 		{
 			throw new NotSupportedException ();
@@ -195,6 +193,12 @@ namespace System.Reflection {
 			}
 		}
 
+		public virtual bool ContainsGenericParameters {
+			get {
+				return false;
+			}
+		}
+
 		public virtual bool HasGenericParameters {
 			get {
 				throw new NotSupportedException ();
@@ -224,5 +228,25 @@ namespace System.Reflection {
 			throw new NotSupportedException ();
 		}
 #endif
+
+		void _MethodBase.GetIDsOfNames ([In] ref Guid riid, IntPtr rgszNames, uint cNames, uint lcid, IntPtr rgDispId)
+		{
+			throw new NotImplementedException ();
+		}
+
+		void _MethodBase.GetTypeInfo (uint iTInfo, uint lcid, IntPtr ppTInfo)
+		{
+			throw new NotImplementedException ();
+		}
+
+		void _MethodBase.GetTypeInfoCount (out uint pcTInfo)
+		{
+			throw new NotImplementedException ();
+		}
+
+		void _MethodBase.Invoke (uint dispIdMember, [In] ref Guid riid, uint lcid, short wFlags, IntPtr pDispParams, IntPtr pVarResult, IntPtr pExcepInfo, IntPtr puArgErr)
+		{
+			throw new NotImplementedException ();
+		}
 	}
 }
