@@ -44,11 +44,11 @@ namespace System.Reflection {
 
 #if NET_2_0
 	[ComVisible (true)]
-	[ClassInterfaceAttribute (ClassInterfaceType.None)]
 	[ComDefaultInterfaceAttribute (typeof (_Module))]
 #endif
 	[Serializable]
-	public class Module : ISerializable, ICustomAttributeProvider {
+	[ClassInterfaceAttribute (ClassInterfaceType.None)]
+	public class Module : ISerializable, ICustomAttributeProvider, _Module {
 	
 		public static readonly TypeFilter FilterTypeName;
 		public static readonly TypeFilter FilterTypeNameIgnoreCase;
@@ -105,7 +105,12 @@ namespace System.Reflection {
 		public extern int MetadataToken {
 			[MethodImplAttribute (MethodImplOptions.InternalCall)]
 			get;
-		}		
+		}
+
+		public extern int MDStreamVersion {
+			[MethodImplAttribute (MethodImplOptions.InternalCall)]
+			get;
+		}
 #endif
 	
 		public virtual Type[] FindTypes(TypeFilter filter, object filterCriteria) 
@@ -277,13 +282,7 @@ namespace System.Reflection {
 			return name;
 		}
 
-#if NET_2_0
-		[Obsolete ("Please use ModuleVersionId instead - this will be removed before Whidbey ships.")]
-		public
-#else
-		internal 
-#endif
-		Guid MvId {
+		internal Guid MvId {
 			get {
 				return Mono_GetGuid (this);
 			}
@@ -310,7 +309,6 @@ namespace System.Reflection {
 				return new ArgumentException (String.Format ("Token 0x{0:x} is not a valid {1} token in the scope of module {2}", metadataToken, tokenType, name), "metadataToken");
 		}
 
-		[Obsolete ("Please use ResolveField(int metadataToken, Type[] genericTypeArguments, Type[] genericMethodArguments) - this will be removed before Whidbey ships.")]
 		public FieldInfo ResolveField (int metadataToken) {
 			ResolveTokenError error;
 
@@ -331,7 +329,6 @@ namespace System.Reflection {
 				return m;
 		}
 
-		[Obsolete ("Please use ResolveMethod(int metadataToken, Type[] genericTypeArguments, Type[] genericMethodArguments) - this will be removed before Whidbey ships.")]
 		public MethodBase ResolveMethod (int metadataToken) {
 			ResolveTokenError error;
 
@@ -352,7 +349,6 @@ namespace System.Reflection {
 				return s;
 		}
 
-		[Obsolete ("Please use ResolveType(int metadataToken, Type[] genericTypeArguments, Type[] genericMethodArguments) - this will be removed before Whidbey ships.")]
 		public Type ResolveType (int metadataToken) {
 			ResolveTokenError error;
 
@@ -362,6 +358,11 @@ namespace System.Reflection {
 			else
 				return Type.GetTypeFromHandle (new RuntimeTypeHandle (handle));
 		}
+
+		[MonoTODO]
+		public byte[] ResolveSignature (int metadataToken) {
+			throw new NotImplementedException ();
+		}
 #endif
 
 		internal static Type MonoDebugger_ResolveType (Module module, int token)
@@ -420,5 +421,28 @@ namespace System.Reflection {
 
 		[MethodImplAttribute (MethodImplOptions.InternalCall)]
 		internal static extern void GetPEKind (IntPtr module, out PortableExecutableKinds peKind, out ImageFileMachine machine);
+
+#if NET_1_1
+		void _Module.GetIDsOfNames ([In] ref Guid riid, IntPtr rgszNames, uint cNames, uint lcid, IntPtr rgDispId)
+		{
+			throw new NotImplementedException ();
+		}
+
+		void _Module.GetTypeInfo (uint iTInfo, uint lcid, IntPtr ppTInfo)
+		{
+			throw new NotImplementedException ();
+		}
+
+		void _Module.GetTypeInfoCount (out uint pcTInfo)
+		{
+			throw new NotImplementedException ();
+		}
+
+		void _Module.Invoke (uint dispIdMember, [In] ref Guid riid, uint lcid, short wFlags, IntPtr pDispParams,
+			IntPtr pVarResult, IntPtr pExcepInfo, IntPtr puArgErr)
+		{
+			throw new NotImplementedException ();
+		}
+#endif
 	}
 }
