@@ -109,7 +109,15 @@ namespace System.Security.Cryptography {
 		protected override byte[] HashFinal() {
 			CompressFinal(_Length);
 			byte[] hash = new byte[20];
-			System.Buffer.BlockCopy(_HashValue, 0, hash, 0, 20);
+			if (BitConverter.IsLittleEndian) {
+				for (int i = 0; i < 5; i++) {
+					for (int j = 0; j < 4; j++) {
+						hash [i*4+j] = (byte)(_HashValue [i] >> j*8);
+					}
+				}
+			} else {
+				Buffer.BlockCopy (_HashValue, 0, hash, 0, 20);
+			}
 			return hash;
 		}
 		/// <summary>
@@ -124,8 +132,19 @@ namespace System.Security.Cryptography {
 		/// </summary>
 		/// <param name="buffer">The buffer with the data.</param>
 		/// <param name="offset">The offset in the buffer.</param>
-		private void ProcessBlock(byte[] buffer, int offset) {
-			System.Buffer.BlockCopy(buffer, offset, _X, 0, 64);
+		private void ProcessBlock (byte[] buffer, int offset)
+		{
+			if (BitConverter.IsLittleEndian) {
+				for (int i=0; i < _X.Length; i++) {
+					_X [i] = (uint)(buffer [offset])
+						| (((uint)(buffer [offset+1])) <<  8)
+						| (((uint)(buffer [offset+2])) << 16)
+						| (((uint)(buffer [offset+3])) << 24);
+					offset += 4;
+				}
+			} else {
+				Buffer.BlockCopy (buffer, offset, _X, 0, 64);
+			}
 			Compress();
 		}
 
