@@ -62,9 +62,9 @@ namespace System.Security {
 #endif
 		private static object _lockObject;
 		private static ArrayList _hierarchy;
-		private static PermissionSet _fullTrust; // for [AllowPartiallyTrustedCallers]
 		private static IPermission _unmanagedCode;
 		private static Hashtable _declsecCache;
+		private static PolicyLevel _level;
 
 		static SecurityManager () 
 		{
@@ -80,16 +80,19 @@ namespace System.Security {
 			get;
 
 			[MethodImplAttribute (MethodImplOptions.InternalCall)]
-			[SecurityPermission (SecurityAction.Demand, Flags=SecurityPermissionFlag.ControlPolicy)]
+			[SecurityPermission (SecurityAction.Demand, ControlPolicy = true)]
 			set;
 		}
 
+#if NET_2_0
+		[Obsolete ("The security manager cannot be turned off on MS runtime")]
+#endif
 		extern public static bool SecurityEnabled {
 			[MethodImplAttribute (MethodImplOptions.InternalCall)]
 			get;
 
 			[MethodImplAttribute (MethodImplOptions.InternalCall)]
-			[SecurityPermission (SecurityAction.Demand, Flags=SecurityPermissionFlag.ControlPolicy)]
+			[SecurityPermission (SecurityAction.Demand, ControlPolicy = true)]
 			set;
 		}
 
@@ -99,7 +102,7 @@ namespace System.Security {
 		// NOTE: This method doesn't show in the class library status page because
 		// it cannot be "found" with the StrongNameIdentityPermission for ECMA key.
 		// But it's there!
-		[MonoTODO]
+		[MonoTODO ("works for fulltrust (empty), documentation doesn't really make sense, type wise")]
 		[StrongNameIdentityPermission (SecurityAction.LinkDemand, PublicKey = "0x00000000000000000400000000000000")]
 		public static void GetZoneAndOrigin (out ArrayList zone, out ArrayList origin) 
 		{
@@ -120,25 +123,53 @@ namespace System.Security {
 			// - Not affected by overrides (like Assert, Deny and PermitOnly)
 			// - calls IsSubsetOf even for non CAS permissions
 			//   (i.e. it does call Demand so any code there won't be executed)
+#if NET_2_0
+			// with 2.0 identity permission are unrestrictable
 			return IsGranted (Assembly.GetCallingAssembly (), perm);
+#else
+			if (perm is IUnrestrictedPermission)
+				return IsGranted (Assembly.GetCallingAssembly (), perm);
+			else
+				return IsGrantedRestricted (Assembly.GetCallingAssembly (), perm);
+#endif
 		}
 
-		internal static bool IsGranted (Assembly a, IPermission perm)
+#if !NET_2_0
+		// only for permissions that do not implement IUnrestrictedPermission
+		internal static bool IsGrantedRestricted (Assembly a, IPermission perm)
 		{
-			CodeAccessPermission grant = null;
-
-			if (a.GrantedPermissionSet != null) {
-				grant = (CodeAccessPermission) a.GrantedPermissionSet.GetPermission (perm.GetType ());
-				if (grant == null) {
-					if (!a.GrantedPermissionSet.IsUnrestricted () || !(perm is IUnrestrictedPermission)) {
-						return perm.IsSubsetOf (null);
-					}
-				} else if (!perm.IsSubsetOf (grant)) {
+			PermissionSet granted = a.GrantedPermissionSet;
+			if (granted != null) {
+				CodeAccessPermission grant = (CodeAccessPermission) granted.GetPermission (perm.GetType ());
+				if (!perm.IsSubsetOf (grant)) {
 					return false;
 				}
 			}
 
-			if (a.DeniedPermissionSet != null) {
+			PermissionSet denied = a.DeniedPermissionSet;
+			if (denied != null) {
+				CodeAccessPermission refuse = (CodeAccessPermission) a.DeniedPermissionSet.GetPermission (perm.GetType ());
+				if ((refuse != null) && perm.IsSubsetOf (refuse))
+					return false;
+			}
+			return true;
+		}
+#endif
+		// note: in 2.0 *all* permissions (including identity permissions) support unrestricted
+		internal static bool IsGranted (Assembly a, IPermission perm)
+		{
+			PermissionSet granted = a.GrantedPermissionSet;
+			if ((granted != null) && !granted.IsUnrestricted ()) {
+				CodeAccessPermission grant = (CodeAccessPermission) granted.GetPermission (perm.GetType ());
+				if (!perm.IsSubsetOf (grant)) {
+					return false;
+				}
+			}
+
+			PermissionSet denied = a.DeniedPermissionSet;
+			if ((denied != null) && !denied.IsEmpty ()) {
+				if (denied.IsUnrestricted ())
+					return false;
 				CodeAccessPermission refuse = (CodeAccessPermission) a.DeniedPermissionSet.GetPermission (perm.GetType ());
 				if ((refuse != null) && perm.IsSubsetOf (refuse))
 					return false;
@@ -154,8 +185,18 @@ namespace System.Security {
 			foreach (IPermission p in ps) {
 				// note: this may contains non CAS permissions
 				if ((!noncas) && (p is CodeAccessPermission)) {
-					if (!SecurityManager.IsGranted (a, p))
+#if NET_2_0
+					if (!IsGranted (a, p))
 						return p;
+#else
+					if (p is IUnrestrictedPermission) {
+						if (!IsGranted (a, p))
+							return p;
+					} else {
+						if (!IsGrantedRestricted (a, p))
+							return p;
+					}
+#endif
 				} else {
 					// but non-CAS will throw on failure...
 					try {
@@ -178,8 +219,15 @@ namespace System.Security {
 			PermissionSet granted = ad.GrantedPermissionSet;
 			if (granted == null)
 				return null;
-			if ((ad.GrantedPermissionSet.Count == 0) && ad.GrantedPermissionSet.IsUnrestricted ())
+#if NET_2_0
+			if (granted.IsUnrestricted ())
 				return null;
+#else
+			if ((granted.Count == 0) && granted.IsUnrestricted ())
+				return null;
+#endif
+			if (ps.IsUnrestricted ())
+				return new SecurityPermission (SecurityPermissionFlag.NoFlags);
 
 			foreach (IPermission p in ps) {
 				if (p is CodeAccessPermission) {
@@ -206,7 +254,7 @@ namespace System.Security {
 			return null;
 		}
 
-		[SecurityPermission (SecurityAction.Demand, Flags=SecurityPermissionFlag.ControlPolicy)]
+		[SecurityPermission (SecurityAction.Demand, ControlPolicy = true)]
 		public static PolicyLevel LoadPolicyLevelFromFile (string path, PolicyLevelType type)
 		{
 			if (path == null)
@@ -216,7 +264,6 @@ namespace System.Security {
 			try {
 				pl = new PolicyLevel (type.ToString (), type);
 				pl.LoadFromFile (path);
-				pl.Initialize ();
 			}
 			catch (Exception e) {
 				throw new ArgumentException (Locale.GetText ("Invalid policy XML"), e);
@@ -224,7 +271,7 @@ namespace System.Security {
 			return pl;
 		}
 
-		[SecurityPermission (SecurityAction.Demand, Flags=SecurityPermissionFlag.ControlPolicy)]
+		[SecurityPermission (SecurityAction.Demand, ControlPolicy = true)]
 		public static PolicyLevel LoadPolicyLevelFromString (string str, PolicyLevelType type)
 		{
 			if (null == str)
@@ -241,7 +288,7 @@ namespace System.Security {
 			return pl;
 		}
 
-		[SecurityPermission (SecurityAction.Demand, Flags=SecurityPermissionFlag.ControlPolicy)]
+		[SecurityPermission (SecurityAction.Demand, ControlPolicy = true)]
 		public static IEnumerator PolicyHierarchy ()
 		{
 			return Hierarchy;
@@ -264,11 +311,12 @@ namespace System.Security {
 			}
 
 			ResolveIdentityPermissions (ps, evidence);
+
 			return ps;
 		}
 
 #if NET_2_0
-		[MonoTODO ("more tests are needed")]
+		[MonoTODO ("(2.0) more tests are needed")]
 		public static PermissionSet ResolvePolicy (Evidence[] evidences)
 		{
 			if ((evidences == null) || (evidences.Length == 0) ||
@@ -308,7 +356,6 @@ namespace System.Security {
 
 		static private SecurityPermission _execution = new SecurityPermission (SecurityPermissionFlag.Execution);
 
-		[MonoTODO()]
 		public static PermissionSet ResolvePolicy (Evidence evidence, PermissionSet reqdPset, PermissionSet optPset, PermissionSet denyPset, out PermissionSet denied)
 		{
 			PermissionSet resolved = ResolvePolicy (evidence);
@@ -317,17 +364,26 @@ namespace System.Security {
 				throw new PolicyException (Locale.GetText (
 					"Policy doesn't grant the minimal permissions required to execute the assembly."));
 			}
-			// do we have the right to execute ?
+
+			// do we check for execution rights ?
 			if (CheckExecutionRights) {
-				// unless we have "Full Trust"...
-				if (!resolved.IsUnrestricted ()) {
-					// ... we need to find a SecurityPermission
-					IPermission security = resolved.GetPermission (typeof (SecurityPermission));
-					if (!_execution.IsSubsetOf (security)) {
-						throw new PolicyException (Locale.GetText (
-							"Policy doesn't grant the right to execute to the assembly."));
+				bool execute = false;
+				// an empty permissionset doesn't include Execution
+				if (resolved != null) {
+					// unless we have "Full Trust"...
+					if (resolved.IsUnrestricted ()) {
+						execute = true;
+					} else {
+						// ... we need to find a SecurityPermission
+						IPermission security = resolved.GetPermission (typeof (SecurityPermission));
+						execute = _execution.IsSubsetOf (security);
 					}
 				}
+
+				if (!execute) {
+					throw new PolicyException (Locale.GetText (
+						"Policy doesn't grant the right to execute the assembly."));
+				}
 			}
 
 			denied = denyPset;
@@ -350,7 +406,7 @@ namespace System.Security {
 			return al.GetEnumerator ();
 		}
 
-		[SecurityPermission (SecurityAction.Demand, Flags=SecurityPermissionFlag.ControlPolicy)]
+		[SecurityPermission (SecurityAction.Demand, ControlPolicy = true)]
 		public static void SavePolicy () 
 		{
 			IEnumerator e = Hierarchy;
@@ -360,7 +416,7 @@ namespace System.Security {
 			}
 		}
 
-		[SecurityPermission (SecurityAction.Demand, Flags=SecurityPermissionFlag.ControlPolicy)]
+		[SecurityPermission (SecurityAction.Demand, ControlPolicy = true)]
 		public static void SavePolicyLevel (PolicyLevel level) 
 		{
 			// Yes this will throw a NullReferenceException, just like MS (see FDBK13121)
@@ -371,12 +427,9 @@ namespace System.Security {
 
 		private static IEnumerator Hierarchy {
 			get {
-				// double-lock pattern
-				if (_hierarchy == null) {
-					lock (_lockObject) {
-						if (_hierarchy == null)
-							InitializePolicyHierarchy ();
-					}
+				lock (_lockObject) {
+					if (_hierarchy == null)
+						InitializePolicyHierarchy ();
 				}
 				return _hierarchy.GetEnumerator ();
 			}
@@ -389,25 +442,24 @@ namespace System.Security {
 			string userPolicyPath = Path.Combine (Environment.InternalGetFolderPath (Environment.SpecialFolder.ApplicationData), "mono");
 
 			PolicyLevel enterprise = new PolicyLevel ("Enterprise", PolicyLevelType.Enterprise);
-			PolicyLevel machine = new PolicyLevel ("Machine", PolicyLevelType.Machine);
-			PolicyLevel user = new PolicyLevel ("User", PolicyLevelType.User);
-
+			_level = enterprise;
 			enterprise.LoadFromFile (Path.Combine (machinePolicyPath, "enterprisesec.config"));
+
+			PolicyLevel machine = new PolicyLevel ("Machine", PolicyLevelType.Machine);
+			_level = machine;
 			machine.LoadFromFile (Path.Combine (machinePolicyPath, "security.config"));
+
+			PolicyLevel user = new PolicyLevel ("User", PolicyLevelType.User);
+			_level = user;
 			user.LoadFromFile (Path.Combine (userPolicyPath, "security.config"));
 
 			ArrayList al = new ArrayList ();
 			al.Add (enterprise);
 			al.Add (machine);
 			al.Add (user);
-			// setting _hierarchy here allows for loading assemblies containing permissions
-			// FIXME: we still need to enforce the FullTrust list
-			_hierarchy = ArrayList.Synchronized (al);
 
-			// part II - creating the permission sets
-			enterprise.Initialize ();
-			machine.Initialize ();
-			user.Initialize ();
+			_hierarchy = ArrayList.Synchronized (al);
+			_level = null;
 		}
 
 		internal static bool ResolvePolicyLevel (ref PermissionSet ps, PolicyLevel pl, Evidence evidence)
@@ -431,9 +483,13 @@ namespace System.Security {
 			return false;
 		}
 
-		// TODO: this changes in 2.0 as identity permissions can now be unrestricted
 		internal static void ResolveIdentityPermissions (PermissionSet ps, Evidence evidence)
 		{
+#if NET_2_0
+			// in 2.0 identity permissions can now be unrestricted
+			if (ps.IsUnrestricted ())
+				return;
+#endif
 			// Only host evidence are used for policy resolution
 			IEnumerator ee = evidence.GetHostEnumerator ();
 			while (ee.MoveNext ()) {
@@ -445,21 +501,22 @@ namespace System.Security {
 			}
 		}
 
+		internal static PolicyLevel ResolvingPolicyLevel {
+			get { return _level; }
+			set { _level = value; }
+		}
+
 		internal static PermissionSet Decode (IntPtr permissions, int length)
 		{
 			// Permission sets from the runtime (declarative security) can be cached
 			// for performance as they can never change (i.e. they are read-only).
-
-			if (_declsecCache == null) {
-				lock (_lockObject) {
-					if (_declsecCache == null) {
-						_declsecCache = new Hashtable ();
-					}
-				}
-			}
-
 			PermissionSet ps = null;
+
 			lock (_lockObject) {
+				if (_declsecCache == null) {
+					_declsecCache = new Hashtable ();
+				}
+
 				object key = (object) (int) permissions;
 				ps = (PermissionSet) _declsecCache [key];
 				if (ps == null) {
@@ -492,62 +549,12 @@ namespace System.Security {
 				throw new SecurityException (Locale.GetText ("Unknown metadata format."));
 			}
 		}
-#if NET_2_0
-		internal static PermissionSetCollection DecodeCollection (IntPtr permissions, int length)
-		{
-			// Permission sets from the runtime (declarative security) can be cached
-			// for performance as they can never change (i.e. they are read-only).
-
-			if (_declsecCache == null) {
-				lock (_lockObject) {
-					if (_declsecCache == null) {
-						_declsecCache = new Hashtable ();
-					}
-				}
-			}
-
-			PermissionSetCollection psc = null;
-			lock (_lockObject) {
-				object key = (object) (int) permissions;
-				psc = (PermissionSetCollection) _declsecCache [key];
-				if (psc == null) {
-					// create permissionset and add it to the cache
-					byte[] data = new byte [length];
-					Marshal.Copy (permissions, data, 0, length);
-					psc = DecodeCollection (data);
-					_declsecCache.Add (key, psc);
-				}
-			}
-			return psc;
-		}
-
-		internal static PermissionSetCollection DecodeCollection (byte[] encodedPermissions)
-		{
-			if ((encodedPermissions == null) || (encodedPermissions.Length < 1))
-				throw new SecurityException ("Invalid metadata format.");
-
-			switch (encodedPermissions [0]) {
-			case 60:
-				// Fx 1.0/1.1 declarative security permissions metadata is in Unicode-encoded XML
-				throw new SecurityException (Locale.GetText ("1.0 metadata format doesn't support collections."));
-			case 0x2E:
-				// Fx 2.0 are encoded "somewhat, but not enough, like" custom attributes
-				// note: we still support the older format!
-				return PermissionSetCollection.CreateFromBinaryFormat (encodedPermissions);
-			default:
-				throw new SecurityException (Locale.GetText ("Unknown metadata format."));
-			}
-		}
-#endif
 
 		private static IPermission UnmanagedCode {
 			get {
-				// double-lock pattern
-				if (_unmanagedCode == null) {
-					lock (_lockObject) {
-						if (_unmanagedCode == null)
-							_unmanagedCode = new SecurityPermission (SecurityPermissionFlag.UnmanagedCode);
-					}
+				lock (_lockObject) {
+					if (_unmanagedCode == null)
+						_unmanagedCode = new SecurityPermission (SecurityPermissionFlag.UnmanagedCode);
 				}
 				return _unmanagedCode;
 			}
@@ -590,17 +597,6 @@ namespace System.Security {
 			// a single stack walk (not up to 4).
 			if (ps != null)
 				ps.Demand ();
-#if NET_2_0
-			// Process LinkDemandChoice (2.0)
-			if (klass.choice.size > 0) {
-				PermissionSetCollection psc = DecodeCollection (klass.choice.blob, klass.choice.size);
-				psc.DemandChoice ();
-			}
-			if (method.choice.size > 0) {
-				PermissionSetCollection psc = DecodeCollection (method.choice.blob, method.choice.size);
-				psc.DemandChoice ();
-			}
-#endif
 		}
 
 		internal unsafe static bool ReflectedLinkDemandQuery (MethodBase mb)
@@ -643,33 +639,6 @@ namespace System.Security {
 					ps = Decode (method->noncas.blob, method->noncas.size);
 					result = (SecurityManager.CheckPermissionSet (a, ps, true) == null);
 				}
-#if NET_2_0
-				// success if one of the permission is granted
-				if (result && (klass->choice.size > 0)) {
-					PermissionSetCollection psc = DecodeCollection (klass->choice.blob, klass->choice.size);
-					if (psc.Count > 0) {
-						result = false;
-						foreach (PermissionSet pset in psc) {
-							if (SecurityManager.CheckPermissionSet (a, pset, false) == null) {
-								result = true;
-								break;
-							}
-						}
-					}
-				}
-				if (result && (method->choice.size > 0)) {
-					PermissionSetCollection psc = DecodeCollection (method->choice.blob, method->choice.size);
-					if (psc.Count > 0) {
-						result = false;
-						foreach (PermissionSet pset in psc) {
-							if (SecurityManager.CheckPermissionSet (a, pset, false) == null) {
-								result = true;
-								break;
-							}
-						}
-					}
-				}
-#endif
 				return result;
 			}
 			catch (SecurityException) {
@@ -679,24 +648,22 @@ namespace System.Security {
 
 		private static bool LinkDemandFullTrust (Assembly a)
 		{
-			// double-lock pattern
-			if (_fullTrust == null) {
-				lock (_lockObject) {
-					if (_fullTrust == null)
-						_fullTrust = new NamedPermissionSet ("FullTrust");
-				}
-			}
-
-			try {
-				return (SecurityManager.CheckPermissionSet (a, _fullTrust, false) == null);
-			}
-			catch (SecurityException) {
+			// FullTrust is immutable (and means Unrestricted) 
+			// so we can skip the subset operations and jump to IsUnrestricted.
+			PermissionSet granted = a.GrantedPermissionSet;
+			if ((granted != null) && !granted.IsUnrestricted ())
 				return false;
-			}
+
+			PermissionSet denied = a.DeniedPermissionSet;
+			if ((denied != null) && !denied.IsEmpty ())
+				return false;
+
+			return true;
 		}
 
 		private static bool LinkDemandUnmanaged (Assembly a)
 		{
+			// note: we know that UnmanagedCode (SecurityPermission) implements IUnrestrictedPermission
 			return IsGranted (a, UnmanagedCode);
 		}
 
@@ -722,7 +689,7 @@ namespace System.Security {
 				break;
 			case 2: // MONO_JIT_LINKDEMAND_APTC
 				message = Locale.GetText ("Partially trusted callers aren't allowed to call into this assembly.");
-				demanded = (object) _fullTrust;
+				demanded = (object) DefaultPolicies.FullTrust; // immutable
 				break;
 			case 4: // MONO_JIT_LINKDEMAND_ECMA
 				message = Locale.GetText ("Calling internal calls is restricted to ECMA signed assemblies.");
@@ -765,7 +732,7 @@ namespace System.Security {
 				break;
 			}
 
-			throw new SecurityException (message, an, granted, refused, method, SecurityAction.LinkDemand, null, null, null);
+			throw new SecurityException (message, an, granted, refused, method, SecurityAction.InheritanceDemand, null, null, null);
 		}
 
 		// internal - get called by the class loader
@@ -773,7 +740,7 @@ namespace System.Security {
 		// Called when
 		// - class inheritance
 		// - method overrides
-		private unsafe static bool InheritanceDemand (Assembly a, RuntimeDeclSecurityActions *actions)
+		private unsafe static bool InheritanceDemand (AppDomain ad, Assembly a, RuntimeDeclSecurityActions *actions)
 		{
 			try {
 				PermissionSet ps = null;
@@ -781,26 +748,19 @@ namespace System.Security {
 				if (actions->cas.size > 0) {
 					ps = Decode (actions->cas.blob, actions->cas.size);
 					result = (SecurityManager.CheckPermissionSet (a, ps, false) == null);
+					if (result) {
+						// also check appdomain
+						result = (SecurityManager.CheckPermissionSet (ad, ps) == null);
+					}
 				}
 				if (actions->noncas.size > 0) {
 					ps = Decode (actions->noncas.blob, actions->noncas.size);
 					result = (SecurityManager.CheckPermissionSet (a, ps, true) == null);
-				}
-#if NET_2_0
-				// success if one of the permission is granted
-				if (result && (actions->choice.size > 0)) {
-					PermissionSetCollection psc = DecodeCollection (actions->choice.blob, actions->choice.size);
-					if (psc.Count > 0) {
-						result = false;
-						foreach (PermissionSet pset in psc) {
-							if (SecurityManager.CheckPermissionSet (a, pset, false) == null) {
-								result = true;
-								break;
-							}
-						}
+					if (result) {
+						// also check appdomain
+						result = (SecurityManager.CheckPermissionSet (ad, ps) == null);
 					}
 				}
-#endif
 				return result;
 			}
 			catch (SecurityException) {
@@ -819,12 +779,7 @@ namespace System.Security {
 
 		private static void InternalDemandChoice (IntPtr permissions, int length)
 		{
-#if NET_2_0
-			PermissionSetCollection psc = DecodeCollection (permissions, length);
-			psc.DemandChoice ();
-#else
-			throw new SecurityException ("SecurityAction.DemandChoice is only possible in 2.0");
-#endif
+			throw new SecurityException ("SecurityAction.DemandChoice was removed from 2.0");
 		}
 	}
 }
