@@ -32,19 +32,24 @@ using System.Reflection;
 using System.Globalization;
 using System.Security;
 using System.Runtime.CompilerServices;
+using System.Runtime.InteropServices;
 
 [Serializable]
 public abstract class Encoding
+#if NET_2_0
+	: ICloneable
+#endif
 {
 	// Code page used by this encoding.
 	internal int codePage;
 	internal int windows_code_page;
+	bool is_readonly = true;
 
 	// Constructor.
 	protected Encoding ()
 	{
-		codePage = 0;
 	}
+
 #if ECMA_COMPAT
 	protected internal
 #else
@@ -53,6 +58,31 @@ public abstract class Encoding
 	Encoding (int codePage)
 	{
 		this.codePage = windows_code_page = codePage;
+
+#if NET_2_0
+		switch (codePage) {
+		default:
+			// MS has "InternalBestFit{Decoder|Encoder}Fallback
+			// here, but we dunno what they are for.
+			decoder_fallback = DecoderFallback.ReplacementFallback;
+			encoder_fallback = EncoderFallback.ReplacementFallback;
+			break;
+		case 20127: // ASCII
+		case 54936: // GB18030
+			decoder_fallback = DecoderFallback.ReplacementFallback;
+			encoder_fallback = EncoderFallback.ReplacementFallback;
+			break;
+		case 1200: // UTF16
+		case 1201: // UTF16
+		case 12000: // UTF32
+		case 12001: // UTF32
+		case 65000: // UTF7
+		case 65001: // UTF8
+			decoder_fallback = new DecoderReplacementFallback (String.Empty);
+			encoder_fallback = new EncoderReplacementFallback (String.Empty);
+			break;
+		}
+#endif
 	}
 
 	// until we change the callers:
@@ -60,6 +90,62 @@ public abstract class Encoding
 		return arg;
 	}
 
+#if NET_2_0
+	DecoderFallback decoder_fallback;
+	EncoderFallback encoder_fallback;
+
+	[ComVisible (false)]
+	public bool IsReadOnly {
+		get { return is_readonly; }
+	}
+
+	public virtual bool IsSingleByte {
+		get { return false; }
+	}
+
+	[MonoTODO ("not used yet")]
+	[ComVisible (false)]
+	public DecoderFallback DecoderFallback {
+		get {
+			if (decoder_fallback == null)
+				decoder_fallback = new DecoderReplacementFallback (String.Empty);
+			return decoder_fallback;
+		}
+		set {
+			if (IsReadOnly)
+				throw new InvalidOperationException ("This Encoding is readonly.");
+			if (value == null)
+				throw new ArgumentNullException ();
+			decoder_fallback = value;
+		}
+	}
+
+	[MonoTODO ("not used yet")]
+	[ComVisible (false)]
+	public EncoderFallback EncoderFallback {
+		get {
+			if (encoder_fallback == null)
+				encoder_fallback = new EncoderReplacementFallback (String.Empty);
+			return encoder_fallback;
+		}
+		set {
+			if (IsReadOnly)
+				throw new InvalidOperationException ("This Encoding is readonly.");
+			if (value == null)
+				throw new ArgumentNullException ();
+			encoder_fallback = value;
+		}
+	}
+
+	internal void SetFallbackInternal (EncoderFallback e, DecoderFallback d)
+	{
+		if (e != null)
+			encoder_fallback = e;
+		if (d != null)
+			decoder_fallback = d;
+	}
+#endif
+
 	// Convert between two encodings.
 	public static byte[] Convert (Encoding srcEncoding, Encoding dstEncoding,
 								 byte[] bytes)
@@ -139,21 +225,60 @@ public abstract class Encoding
 	public virtual int GetBytes (String s, int charIndex, int charCount,
 								byte[] bytes, int byteIndex)
 	{
-		if (s == null) {
+		if (s == null)
 			throw new ArgumentNullException ("s");
+#if NET_2_0
+		if (charIndex < 0 || charIndex > s.Length)
+			throw new ArgumentOutOfRangeException ("charIndex", _("ArgRange_Array"));
+		if (charCount < 0 || charIndex + charCount > s.Length)
+			throw new ArgumentOutOfRangeException ("charCount", _("ArgRange_Array"));
+		if (byteIndex < 0 || byteIndex > bytes.Length)
+			throw new ArgumentOutOfRangeException ("byteIndex", _("ArgRange_Array"));
+
+		if (charCount == 0 || bytes.Length == byteIndex)
+			return 0;
+		unsafe {
+			fixed (char* cptr = s) {
+				fixed (byte* bptr = bytes) {
+					return GetBytes (cptr + charIndex,
+						charCount,
+						bptr + byteIndex,
+						bytes.Length - byteIndex);
+				}
+			}
 		}
+#else
 		return GetBytes (s.ToCharArray(), charIndex, charCount, bytes, byteIndex);
+#endif
 	}
 	public virtual byte[] GetBytes (String s)
 	{
-		if (s == null) {
+		if (s == null)
 			throw new ArgumentNullException ("s");
+
+#if NET_2_0
+		if (s.Length == 0)
+			return new byte [0];
+		int byteCount = GetByteCount (s);
+		if (byteCount == 0)
+			return new byte [0];
+		unsafe {
+			fixed (char* cptr = s) {
+				byte [] bytes = new byte [byteCount];
+				fixed (byte* bptr = bytes) {
+					GetBytes (cptr, s.Length,
+						bptr, byteCount);
+					return bytes;
+				}
+			}
 		}
+#else
 		char[] chars = s.ToCharArray ();
 		int numBytes = GetByteCount (chars, 0, chars.Length);
 		byte[] bytes = new byte [numBytes];
 		GetBytes (chars, 0, chars.Length, bytes, 0);
 		return bytes;
+#endif
 	}
 	public virtual byte[] GetBytes (char[] chars, int index, int count)
 	{
@@ -323,6 +448,11 @@ public abstract class Encoding
 			case UTF8Encoding.UTF8_CODE_PAGE:
 				return UTF8;
 
+#if NET_2_0
+			case UTF32Encoding.UTF32_CODE_PAGE:
+				return UTF32;
+#endif
+
 			case UnicodeEncoding.UNICODE_CODE_PAGE:
 				return Unicode;
 
@@ -338,6 +468,7 @@ public abstract class Encoding
 		// Try to obtain a code page handler from the I18N handler.
 		Encoding enc = (Encoding)(InvokeI18N ("GetEncoding", codePage));
 		if (enc != null) {
+			enc.is_readonly = true;
 			return enc;
 		}
 
@@ -348,14 +479,18 @@ public abstract class Encoding
 		Assembly assembly = Assembly.GetExecutingAssembly ();
 		Type type = assembly.GetType (cpName);
 		if (type != null) {
-			return (Encoding)(Activator.CreateInstance (type));
+			enc = (Encoding)(Activator.CreateInstance (type));
+			enc.is_readonly = true;
+			return enc;
 		}
 
 		// Look in any assembly, in case the application
 		// has provided its own code page handler.
 		type = Type.GetType (cpName);
 		if (type != null) {
-			return (Encoding)(Activator.CreateInstance (type));
+			enc = (Encoding)(Activator.CreateInstance (type));
+			enc.is_readonly = true;
+			return enc;
 		}
 
 		// We have no idea how to handle this code page.
@@ -365,6 +500,44 @@ public abstract class Encoding
 
 #if !ECMA_COMPAT
 
+#if NET_2_0
+
+	public virtual object Clone ()
+	{
+		return MemberwiseClone ();
+	}
+
+	public static Encoding GetEncoding (int codePage,
+		EncoderFallback encoderFallback, DecoderFallback decoderFallback)
+	{
+		if (encoderFallback == null)
+			throw new ArgumentNullException ("encoderFallback");
+		if (decoderFallback == null)
+			throw new ArgumentNullException ("decoderFallback");
+
+		Encoding e = GetEncoding (codePage).Clone () as Encoding;
+		e.is_readonly = false;
+		e.encoder_fallback = encoderFallback;
+		e.decoder_fallback = decoderFallback;
+		return e;
+	}
+
+	public static Encoding GetEncoding (string name,
+		EncoderFallback encoderFallback, DecoderFallback decoderFallback)
+	{
+		if (encoderFallback == null)
+			throw new ArgumentNullException ("encoderFallback");
+		if (decoderFallback == null)
+			throw new ArgumentNullException ("decoderFallback");
+
+		Encoding e = GetEncoding (name).Clone () as Encoding;
+		e.is_readonly = false;
+		e.encoder_fallback = encoderFallback;
+		e.decoder_fallback = decoderFallback;
+		return e;
+	}
+#endif
+
 	// Table of builtin web encoding names and the corresponding code pages.
 	private static readonly object[] encodings =
 		{
@@ -389,6 +562,14 @@ public abstract class Encoding
 			UnicodeEncoding.BIG_UNICODE_CODE_PAGE,
 			"unicodefffe", "utf_16be",
 
+#if NET_2_0
+			UTF32Encoding.UTF32_CODE_PAGE,
+			"utf_32", "UTF_32LE", "ucs_4",
+
+			UTF32Encoding.BIG_UTF32_CODE_PAGE,
+			"UTF_32BE",
+#endif
+
 			Latin1Encoding.ISOLATIN_CODE_PAGE,
 			"iso_8859_1", "latin1"
 		};
@@ -582,7 +763,10 @@ public abstract class Encoding
 	static volatile Encoding unicodeEncoding;
 	static volatile Encoding isoLatin1Encoding;
 	static volatile Encoding unixConsoleEncoding;
-	
+#if NET_2_0
+	static volatile Encoding utf32Encoding;
+#endif
+
 	static readonly object lockobj = new object ();
 
 	// Get the standard ASCII encoding object.
@@ -593,6 +777,7 @@ public abstract class Encoding
 				lock (lockobj) {
 					if (asciiEncoding == null) {
 						asciiEncoding = new ASCIIEncoding ();
+						asciiEncoding.is_readonly = true;
 					}
 				}
 			}
@@ -609,6 +794,7 @@ public abstract class Encoding
 				lock (lockobj) {
 					if (bigEndianEncoding == null) {
 						bigEndianEncoding = new UnicodeEncoding (true, true);
+						bigEndianEncoding.is_readonly = true;
 					}
 				}
 			}
@@ -651,6 +837,7 @@ public abstract class Encoding
 						} catch (NotSupportedException) {
 							defaultEncoding = UTF8Unmarked;
 						}
+						defaultEncoding.is_readonly = true;
 					}
 				}
 			}
@@ -667,6 +854,7 @@ public abstract class Encoding
 				lock (lockobj) {
 					if (isoLatin1Encoding == null) {
 						isoLatin1Encoding = new Latin1Encoding ();
+						isoLatin1Encoding.is_readonly = true;
 					}
 				}
 			}
@@ -688,6 +876,7 @@ public abstract class Encoding
 				lock (lockobj) {
 					if (utf7Encoding == null) {
 						utf7Encoding = new UTF7Encoding ();
+						utf7Encoding.is_readonly = true;
 					}
 				}
 			}
@@ -704,6 +893,7 @@ public abstract class Encoding
 				lock (lockobj) {
 					if (utf8EncodingWithMarkers == null) {
 						utf8EncodingWithMarkers = new UTF8Encoding (true);
+						utf8EncodingWithMarkers.is_readonly = true;
 					}
 				}
 			}
@@ -721,6 +911,7 @@ public abstract class Encoding
 				lock (lockobj){
 					if (utf8EncodingWithoutMarkers == null){
 						utf8EncodingWithoutMarkers = new UTF8Encoding (false, false);
+						utf8EncodingWithoutMarkers.is_readonly = true;
 					}
 				}
 			}
@@ -737,6 +928,7 @@ public abstract class Encoding
 				lock (lockobj) {
 					if (unicodeEncoding == null) {
 						unicodeEncoding = new UnicodeEncoding (false, true);
+						unicodeEncoding.is_readonly = true;
 					}
 				}
 			}
@@ -745,6 +937,25 @@ public abstract class Encoding
 		}
 	}
 
+#if NET_2_0
+	// Get the standard little-endian UTF-32 encoding object.
+	public static Encoding UTF32
+	{
+		get {
+			if (utf32Encoding == null) {
+				lock (lockobj) {
+					if (utf32Encoding == null) {
+						utf32Encoding = new UTF32Encoding (false, true);
+						utf32Encoding.is_readonly = true;
+					}
+				}
+			}
+
+			return utf32Encoding;
+		}
+	}
+#endif
+
 	// Forwarding decoder implementation.
 	private sealed class ForwardingDecoder : Decoder
 	{
@@ -754,6 +965,9 @@ public abstract class Encoding
 		public ForwardingDecoder (Encoding enc)
 		{
 			encoding = enc;
+#if NET_2_0
+			Fallback = encoding.DecoderFallback;
+#endif
 		}
 
 		// Override inherited methods.
@@ -779,6 +993,9 @@ public abstract class Encoding
 		public ForwardingEncoder (Encoding enc)
 		{
 			encoding = enc;
+#if NET_2_0
+			Fallback = encoding.EncoderFallback;
+#endif
 		}
 
 		// Override inherited methods.
@@ -795,6 +1012,91 @@ public abstract class Encoding
 
 	} // class ForwardingEncoder
 
+#if NET_2_0
+	[CLSCompliantAttribute(false)]
+	public unsafe virtual int GetByteCount (char *chars, int count)
+	{
+		if (chars == null)
+			throw new ArgumentNullException ("chars");
+		if (count < 0)
+			throw new ArgumentOutOfRangeException ("count");
+		char [] c = new char [count];
+
+		for (int p = 0; p < count; p++)
+			c [p] = chars [p];
+
+		return GetByteCount (c);
+	}
+
+	[CLSCompliantAttribute(false)]
+	public unsafe virtual int GetCharCount (byte *bytes, int count)
+	{
+		if (bytes == null)
+			throw new ArgumentNullException ("bytes");
+		if (count < 0)
+			throw new ArgumentOutOfRangeException ("count");
+		
+		byte [] ba = new byte [count];
+		for (int i = 0; i < count; i++)
+			ba [i] = bytes [i];
+		return GetCharCount (ba, 0, count);
+	}
+
+	[CLSCompliantAttribute(false)]
+	public unsafe virtual int GetChars (byte *bytes, int byteCount, char *chars, int charCount)
+	{
+		if (bytes == null)
+			throw new ArgumentNullException ("bytes");
+		if (chars == null)
+			throw new ArgumentNullException ("chars");
+		if (charCount < 0)
+			throw new ArgumentOutOfRangeException ("charCount");
+		if (byteCount < 0)
+			throw new ArgumentOutOfRangeException ("byteCount");
+		
+		byte [] ba = new byte [byteCount];
+		for (int i = 0; i < byteCount; i++)
+			ba [i] = bytes [i];
+		char [] ret = GetChars (ba, 0, byteCount);
+		int top = ret.Length;
+
+		if (top > charCount)
+			throw new ArgumentException ("charCount is less than the number of characters produced", "charCount");
+		
+		for (int i = 0; i < top; i++)
+			chars [i] = ret [i];
+		return top;
+	}
+
+	[CLSCompliantAttribute(false)]
+	public unsafe virtual int GetBytes (char *chars, int charCount, byte *bytes, int byteCount)
+	{
+		if (bytes == null)
+			throw new ArgumentNullException ("bytes");
+		if (chars == null)
+			throw new ArgumentNullException ("chars");
+		if (charCount < 0)
+			throw new ArgumentOutOfRangeException ("charCount");
+		if (byteCount < 0)
+			throw new ArgumentOutOfRangeException ("byteCount");
+		
+		char [] c = new char [charCount];
+		
+		for (int i = 0; i < charCount; i++)
+			c [i] = chars [i];
+
+		byte [] b = GetBytes (c, 0, charCount);
+		int top = b.Length;
+		if (top > byteCount)
+			throw new ArgumentException ("byteCount is less that the number of bytes produced", "byteCount");
+
+		for (int i = 0; i < top; i++)
+			bytes [i] = b [i];
+		
+		return b.Length;
+	}
+#endif
+
 }; // class Encoding
 
 }; // namespace System.Text
