@@ -67,7 +67,7 @@ namespace System.Threading {
 		private object abort_exc;
 		internal object abort_state;
 		/* thread_id is only accessed from unmanaged code */
-		private int thread_id;
+		private Int64 thread_id;
 		
 		/* start_notify is used by the runtime to signal that Start()
 		 * is ok to return
@@ -155,7 +155,7 @@ namespace System.Threading {
 
 		internal static int CurrentThreadId {
 			get {
-				return CurrentThread.thread_id;
+				return (int)(CurrentThread.thread_id);
 			}
 		}
 
@@ -576,6 +576,7 @@ namespace System.Threading {
 		[SecurityPermission (SecurityAction.Demand, ControlThread=true)]
 		public void Interrupt ()
 		{
+			throw new NotImplementedException ();
 		}
 
 		// The current thread joins with 'this'. Set ms to 0 to block
@@ -608,11 +609,8 @@ namespace System.Threading {
 		}
 
 #if NET_1_1
-		[MonoTODO ("seems required for multi-processors systems like Itanium")]
-		public static void MemoryBarrier ()
-		{
-			// Will be implemented when we support Itanium
-		}
+		[MethodImplAttribute(MethodImplOptions.InternalCall)]
+		public extern static void MemoryBarrier ();
 #endif
 		[MethodImplAttribute(MethodImplOptions.InternalCall)]
 		private extern void Resume_internal();
@@ -646,6 +644,8 @@ namespace System.Threading {
 			if (SecurityManager.SecurityEnabled)
 				_ec = ExecutionContext.Capture ();
 #endif
+			if (CurrentThread._principal != null)
+				_principal = CurrentThread._principal;
 
 			// Thread_internal creates and starts the new thread, 
 			if (Thread_internal(threadstart) == (IntPtr) 0)
@@ -668,6 +668,9 @@ namespace System.Threading {
 		[MethodImplAttribute(MethodImplOptions.InternalCall)]
 		private extern void Thread_free_internal(IntPtr handle);
 
+#if NET_2_0
+		[ReliabilityContract (Consistency.WillNotCorruptState, Cer.Success)]
+#endif
 		~Thread() {
 			// Free up the handle
 			if (system_thread_handle != (IntPtr) 0)
@@ -818,7 +821,7 @@ namespace System.Threading {
 
 		public int ManagedThreadId {
 		[ReliabilityContractAttribute (Consistency.WillNotCorruptState, Cer.Success)]
-			get { return thread_id; }
+			get { return (int)thread_id; }
 		}
 
 		[MonoTODO]
@@ -890,7 +893,7 @@ namespace System.Threading {
 		public override int GetHashCode ()
 		{
 			// ??? overridden but not guaranteed to be unique ???
-			return thread_id;
+			return (int)thread_id;
 		}
 
 		public void Start (object parameter)
@@ -913,6 +916,9 @@ namespace System.Threading {
 		// But it's there!
 		[SecurityPermission (SecurityAction.LinkDemand, UnmanagedCode = true)]
 		[StrongNameIdentityPermission (SecurityAction.LinkDemand, PublicKey="00000000000000000400000000000000")]
+#if NET_2_0
+		[Obsolete ("see CompressedStack class")]
+#endif
 #if NET_1_1
 		public
 #else
@@ -932,6 +938,9 @@ namespace System.Threading {
 		// But it's there!
 		[SecurityPermission (SecurityAction.LinkDemand, UnmanagedCode = true)]
 		[StrongNameIdentityPermission (SecurityAction.LinkDemand, PublicKey="00000000000000000400000000000000")]
+#if NET_2_0
+		[Obsolete ("see CompressedStack class")]
+#endif
 #if NET_1_1
 		public
 #else
