@@ -188,10 +188,10 @@ namespace System
 		public extern int GetLowerBound (int dimension);
 
 		[MethodImplAttribute (MethodImplOptions.InternalCall)]
-		public extern object GetValue (int[] indices);
+		public extern object GetValue (params int[] indices);
 
 		[MethodImplAttribute (MethodImplOptions.InternalCall)]
-		public extern void SetValue (object value, int[] indices);
+		public extern void SetValue (object value, params int[] indices);
 
 		// CAUTION! No bounds checking!
 		[MethodImplAttribute (MethodImplOptions.InternalCall)]
@@ -486,7 +486,7 @@ namespace System
 		}
 
 		[ComVisible (false)]
-		public object GetValue (long [] indices)
+		public object GetValue (params long [] indices)
 		{
 			if (indices == null) {
 				// LAMESPEC: Docs say we should throw a ArgumentNull, but .NET
@@ -497,7 +497,7 @@ namespace System
 		}
 
 		[ComVisible (false)]
-		public void SetValue (object value, long [] indices)
+		public void SetValue (object value, params long [] indices)
 		{
 			if (indices == null) {
 				// LAMESPEC: Docs say we should throw a ArgumentNull, but .NET
@@ -894,7 +894,7 @@ namespace System
 			return array.GetLowerBound (0) - 1;
 		}
 
-#if !BOOTSTRAP_WITH_OLDLIB && !NET_2_0
+#if !BOOTSTRAP_WITH_OLDLIB
 		/* delegate used to swap array elements */
 		delegate void Swapper (int i, int j);
 #endif
@@ -1359,7 +1359,13 @@ namespace System
 		}
 
 #if NET_2_0
-		public static void Resize <T> (ref T [] array, int newSize) {
+		public static void Resize<T> (ref T [] array, int newSize)
+		{
+			Resize<T> (ref array, array == null ? 0 : array.Length, newSize);
+		}
+
+		internal static void Resize<T> (ref T[] array, int length, int newSize)
+		{
 			if (newSize < 0)
 				throw new ArgumentOutOfRangeException ();
 			
@@ -1372,7 +1378,7 @@ namespace System
 				return;
 			
 			T [] a = new T [newSize];
-			Array.Copy (array, a, Math.Min (newSize, array.Length));
+			Array.Copy (array, a, Math.Min (newSize, length));
 			array = a;
 		}
 		
@@ -1690,6 +1696,190 @@ namespace System
 			Copy (s, s_i, d, d_i, c);
 		}
 #endif 
+
+#if NET_2_0
+		//
+		// This is used internally by the runtime to represent arrays; see #74953.
+		//
+		// Note that you normally can't derive a class from System.Array (CS0644),
+		// but GMCS makes an exception here for all classes which are nested inside
+		// System.Array.
+		//
+		internal class InternalArray<T> : Array, IList<T>
+		{
+			new public IEnumerator<T> GetEnumerator ()
+			{
+				return new InternalEnumerator (this);
+			}
+
+			public int Count {
+				get {
+					return Length;
+				}
+			}
+
+			bool ICollection<T>.IsReadOnly {
+				get {
+					return true;
+				}
+			}
+
+			void ICollection<T>.Clear ()
+			{
+				throw new NotSupportedException ("Collection is read-only");
+			}
+
+			void ICollection<T>.Add (T item)
+			{
+				throw new NotSupportedException ("Collection is read-only");
+			}
+
+			bool ICollection<T>.Remove (T item)
+			{
+				throw new NotSupportedException ("Collection is read-only");
+			}
+
+			public bool Contains (T item)
+			{
+				if (this.Rank > 1)
+					throw new RankException (Locale.GetText ("Only single dimension arrays are supported."));
+
+				int length = this.Length;
+				for (int i = 0; i < length; i++) {
+					T value;
+					GetGenericValueImpl (i, out value);
+					if (item.Equals (value))
+						return true;
+				}
+
+				return false;
+			}
+
+			public void CopyTo (T[] array, int index)
+			{
+				if (array == null)
+					throw new ArgumentNullException ("array");
+
+				// The order of these exception checks may look strange,
+				// but that's how the microsoft runtime does it.
+				if (this.Rank > 1)
+					throw new RankException (Locale.GetText ("Only single dimension arrays are supported."));
+				if (index + this.GetLength (0) > array.GetLowerBound (0) + array.GetLength (0))
+					throw new ArgumentException ();
+				if (array.Rank > 1)
+					throw new RankException (Locale.GetText ("Only single dimension arrays are supported."));
+				if (index < 0)
+					throw new ArgumentOutOfRangeException (
+						"index", Locale.GetText ("Value has to be >= 0."));
+
+				Copy (this, this.GetLowerBound (0), array, index, this.GetLength (0));
+			}
+
+			new public T this [int index] {
+				get {
+					if (unchecked ((uint) index) >= unchecked ((uint) Length))
+						throw new ArgumentOutOfRangeException ("index");
+
+					T value;
+					GetGenericValueImpl (index, out value);
+					return value;
+				}
+
+				set {
+					throw new NotSupportedException ("Collection is read-only");
+				}
+			}
+
+			void IList<T>.Insert (int index, T item)
+			{
+				throw new NotSupportedException ("Collection is read-only");
+			}
+
+			void IList<T>.RemoveAt (int index)
+			{
+				throw new NotSupportedException ("Collection is read-only");
+			}
+
+			public int IndexOf (T item)
+			{
+				if (this.Rank > 1)
+					throw new RankException (Locale.GetText ("Only single dimension arrays are supported."));
+
+				int length = this.Length;
+				for (int i = 0; i < length; i++) {
+					T value;
+					GetGenericValueImpl (i, out value);
+					if (item.Equals (value))
+						// array index may not be zero-based.
+						// use lower bound
+						return i + this.GetLowerBound (0);
+				}
+
+				int retVal;
+				unchecked {
+					// lower bound may be MinValue
+					retVal = this.GetLowerBound (0) - 1;
+				}
+
+				return retVal;
+			}
+
+			// CAUTION! No bounds checking!
+			[MethodImplAttribute (MethodImplOptions.InternalCall)]
+			protected extern void GetGenericValueImpl (int pos, out T value);
+
+			internal struct InternalEnumerator : IEnumerator<T>
+			{
+				const int NOT_STARTED = -2;
+			
+				// this MUST be -1, because we depend on it in move next.
+				// we just decr the size, so, 0 - 1 == FINISHED
+				const int FINISHED = -1;
+			
+				InternalArray<T> array;
+				int idx;
+
+				internal InternalEnumerator (InternalArray<T> array)
+				{
+					this.array = array;
+					idx = NOT_STARTED;
+				}
+
+				public void Dispose ()
+				{
+					idx = NOT_STARTED;
+				}
+
+				public bool MoveNext ()
+				{
+					if (idx == NOT_STARTED)
+						idx = array.Length;
+
+					return idx != FINISHED && -- idx != FINISHED;
+				}
+
+				public T Current {
+					get {
+						if (idx < 0)
+							throw new InvalidOperationException ();
+
+						return array [array.Length - 1 - idx];
+					}
+				}
+
+				void IEnumerator.Reset ()
+				{
+					throw new NotImplementedException ();
+				}
+
+				object IEnumerator.Current {
+					get {
+						return Current;
+					}
+				}
+			}
+		}
+#endif
 	}
 
 #if NET_2_0
@@ -1820,10 +2010,9 @@ namespace System
 			return new ReadOnlyArrayEnumerator <T> (this);
 		}
 	}
-
 #endif
 
-#if BOOTSTRAP_WITH_OLDLIB || NET_2_0
+#if BOOTSTRAP_WITH_OLDLIB
 	/* delegate used to swap array elements, keep defined outside Array */
 	delegate void Swapper (int i, int j);
 #endif
