@@ -110,7 +110,7 @@ namespace System {
 		}
 
 		[MethodImplAttribute (MethodImplOptions.InternalCall)]
-		extern static byte [] InternalFromBase64String (string str);
+		extern static byte [] InternalFromBase64String (string str, bool allowWhitespaceOnly);
 
 		[MethodImplAttribute (MethodImplOptions.InternalCall)]
 		extern static byte [] InternalFromBase64CharArray (char [] arr, int offset, int length);
@@ -135,7 +135,15 @@ namespace System {
 			if (s == null)
 				throw new ArgumentNullException ("s");
 
-			return InternalFromBase64String (s);
+			if (s.Length == 0) {
+				return new byte[0];
+			}
+
+#if NET_2_0
+			return InternalFromBase64String (s, true);
+#else
+			return InternalFromBase64String (s, false);
+#endif
 		}
 
 		public static TypeCode GetTypeCode (object value)
@@ -230,18 +238,21 @@ namespace System {
 				throw new ArgumentOutOfRangeException ("offset + length > array.Length");
 
 			Encoding encoding = new ASCIIEncoding ();
-			StringBuilder sb = new StringBuilder ();
 			BinaryReader reader = new BinaryReader (new MemoryStream (inArray, offset, length));
 			byte[] b = null;
 
-			do {
-				// 54 bytes of input makes for 72 bytes of output.
-				b = reader.ReadBytes (54);
-				if (b.Length > 0)
-					sb.AppendLine (encoding.GetString (toBase64Transform.InternalTransformFinalBlock (b, 0, b.Length)));
-			} while (b.Length > 0);
-
-			return sb.ToString ();
+			if (options == Base64FormattingOptions.InsertLineBreaks) {
+				StringBuilder sb = new StringBuilder ();
+				do {
+					// 54 bytes of input makes for 72 bytes of output.
+					b = reader.ReadBytes (54);
+					if (b.Length > 0)
+						sb.AppendLine (encoding.GetString (toBase64Transform.InternalTransformFinalBlock (b, 0, b.Length)));
+				} while (b.Length > 0);
+				return sb.ToString ();
+			} else {
+				return encoding.GetString (toBase64Transform.InternalTransformFinalBlock (inArray, offset, length));
+			}
 		}
 
 		public static string ToBase64String (byte[] inArray, int offset, int length, bool insertLineBreaks)
@@ -1080,6 +1091,10 @@ namespace System {
 		{
 			int result = ConvertFromBase (value, fromBase, false);
 			if (fromBase != 10) {
+				if (result > ushort.MaxValue) {
+					throw new OverflowException ("Value was either too large or too small for an Int16.");
+				}
+
 				// note: no sign are available to detect negatives
 				if (result > Int16.MaxValue) {
 					// return negative 2's complement
@@ -2382,8 +2397,8 @@ namespace System {
 		[CLSCompliant (false)]
 		public static ulong ToUInt64 (string value, int fromBase)
 		{
-			return (ulong) ConvertFromBase (value, fromBase, true);
-		}					      
+			return (ulong) ConvertFromBase64 (value, fromBase, true);
+		}
 
 		[CLSCompliant (false)]
 		public static ulong ToUInt64 (uint value) 
@@ -2427,16 +2442,28 @@ namespace System {
 			if ((value != null) && (conversionType == null))
 				throw new ArgumentNullException ("conversionType");
 			CultureInfo ci = CultureInfo.CurrentCulture;
-			NumberFormatInfo number = ci.NumberFormat;
-			return ToType (value, conversionType, number);
+			IFormatProvider provider;
+			if (conversionType == typeof(DateTime)) {
+				provider = ci.DateTimeFormat;
+			}
+			else {
+				provider = ci.NumberFormat;
+			}
+			return ToType (value, conversionType, provider);
 		}
 		
 		public static object ChangeType (object value, TypeCode typeCode)
 		{
 			CultureInfo ci = CultureInfo.CurrentCulture;
 			Type conversionType = conversionTable [(int) typeCode];
-			NumberFormatInfo number = ci.NumberFormat;
-			return ToType (value, conversionType, number);
+			IFormatProvider provider;
+			if (conversionType == typeof(DateTime)) {
+				provider = ci.DateTimeFormat;
+			}
+			else {
+				provider = ci.NumberFormat;
+			}
+			return ToType (value, conversionType, provider);
 		}
 
 		public static object ChangeType (object value, Type conversionType, IFormatProvider provider)
@@ -2478,59 +2505,79 @@ namespace System {
 
 			// special processing for some bases
 			switch (fromBase) {
-			case 10:
-				if (value[i] == '-') {
-					if (unsigned) {
-						throw new OverflowException (
-							Locale.GetText ("Cannot convert negative to unsigned"));
+				case 10:
+					if (value.Substring (i, 1) == "-") {
+						if (unsigned) {
+							throw new OverflowException (
+								Locale.GetText ("The string was being parsed as"
+								+ " an unsigned number and could not have a"
+								+ " negative sign."));
+						}
+						negative = true;
+						i++;
 					}
-					negative = true;
-					i++;
-				}
-				break;
-			case 16:
-				if (len >= i+2) {
-					// 0x00 or 0X00
-					if ((value[i] == '0') && ((value [i+1] == 'x') || (value [i+1] == 'X'))) {
-						i+=2;
+					break;
+				case 16:
+					if (value.Substring (i, 1) == "-") {
+						throw new ArgumentException ("String cannot contain a "
+							+ "minus sign if the base is not 10.");
 					}
-					if (len == i) {
-						throw new FormatException (
-							Locale.GetText ("Missing number after prefix"));
+					if (len >= i + 2) {
+						// 0x00 or 0X00
+						if ((value[i] == '0') && ((value [i+1] == 'x') || (value [i+1] == 'X'))) {
+							i+=2;
+						}
 					}
-				}
-				break;
+					break;
+				default:
+					if (value.Substring (i, 1) == "-") {
+						throw new ArgumentException ("String cannot contain a "
+							+ "minus sign if the base is not 10.");
+					}
+					break;
+			}
+
+			if (len == i) {
+				throw new FormatException ("Could not find any parsable digits.");
 			}
 
 			if (value[i] == '+') {
 				i++;
 			}
 
-			while(i<len) {
+			while (i < len) {
 				char c = value[i++];
-				if (Char.IsNumber (c))
+				if (Char.IsNumber (c)) {
 					digitValue = c - '0';
-				else if (Char.IsLetter (c))
+				} else if (Char.IsLetter (c)) {
 					digitValue = Char.ToLowerInvariant (c) - 'a' + 10;
-				else if (c == '-')
-					throw new ArgumentException ("Negative are valid only for base 10");
-				else
-					throw new FormatException ("This is an invalid string: " + value);
+				} else {
+					if (chars > 0) {
+						throw new FormatException ("Additional unparsable "
+							+ "characters are at the end of the string.");
+					} else {
+						throw new FormatException ("Could not find any parsable"
+							+ " digits.");
+					}
+				}
 
-				if (digitValue >= fromBase)
-					throw new FormatException ("the digits are invalid.");
+				if (digitValue >= fromBase) {
+					if (chars > 0) {
+						throw new FormatException ("Additional unparsable "
+							+ "characters are at the end of the string.");
+					} else {
+						throw new FormatException ("Could not find any parsable"
+							+ " digits.");
+					}
+				}
 
 				result = (fromBase) * result + digitValue;
 				chars ++;
 			}
 
-
 			if (chars == 0)
-				throw new FormatException ("Could not find any digits.");
+				throw new FormatException ("Could not find any parsable digits.");
 
-			if (result > Int32.MaxValue || result < Int32.MinValue)
-				throw new OverflowException ("There is an overflow.");
-			
 			if (negative)
 				return -result;
 			else
@@ -2547,38 +2594,91 @@ namespace System {
 
 			int chars = 0;
 			int digitValue = -1;
-			uint Base = (uint) fromBase;
-			ulong result = 0;
+			long result = 0;
 			bool negative = false;
 
-			foreach (char c in value) {
-				if (Char.IsNumber (c))
+			int i = 0;
+			int len = value.Length;
+
+			// special processing for some bases
+			switch (fromBase) {
+				case 10:
+					if (value.Substring(i, 1) == "-") {
+						if (unsigned) {
+							throw new OverflowException (
+								Locale.GetText ("The string was being parsed as"
+								+ " an unsigned number and could not have a"
+								+ " negative sign."));
+						}
+						negative = true;
+						i++;
+					}
+					break;
+				case 16:
+					if (value.Substring (i, 1) == "-") {
+						throw new ArgumentException ("String cannot contain a "
+							+ "minus sign if the base is not 10.");
+					}
+					if (len >= i + 2) {
+						// 0x00 or 0X00
+						if ((value[i] == '0') && ((value[i + 1] == 'x') || (value[i + 1] == 'X'))) {
+							i += 2;
+						}
+					}
+					break;
+				default:
+					if (value.Substring (i, 1) == "-") {
+						throw new ArgumentException ("String cannot contain a "
+							+ "minus sign if the base is not 10.");
+					}
+					break;
+			}
+
+			if (len == i) {
+				throw new FormatException ("Could not find any parsable digits.");
+			}
+
+			if (value[i] == '+') {
+				i++;
+			}
+
+			while (i < len) {
+				char c = value[i++];
+				if (Char.IsNumber (c)) {
 					digitValue = c - '0';
-				else if (Char.IsLetter (c))
+				} else if (Char.IsLetter (c)) {
 					digitValue = Char.ToLowerInvariant (c) - 'a' + 10;
-				else if ((c == '-') && (!unsigned)) {
-					if (fromBase != 10)
-						throw new ArgumentException ("Negative are valid only for base 10");
-					negative = true;
-					continue;
+				} else {
+					if (chars > 0) {
+						throw new FormatException ("Additional unparsable "
+							+ "characters are at the end of the string.");
+					} else {
+						throw new FormatException ("Could not find any parsable"
+							+ " digits.");
+					}
 				}
-				else
-					throw new FormatException ("This is an invalid string: " + value);
 
-				if (digitValue >= fromBase)
-					throw new FormatException ("the digits are invalid.");
+				if (digitValue >= fromBase) {
+					if (chars > 0) {
+						throw new FormatException ("Additional unparsable "
+							+ "characters are at the end of the string.");
+					} else {
+						throw new FormatException ("Could not find any parsable"
+							+ " digits.");
+					}
+				}
 
-				result = (ulong) (Base * result + (uint)digitValue);
+				result = (fromBase * result + digitValue);
 				chars ++;
 			}
 
 			if (chars == 0)
-				throw new FormatException ("Could not find any digits.");
+				throw new FormatException ("Could not find any parsable digits.");
 
 			if (negative)
-				return -1 * (long) result;
+				return -1 * result;
 			else
-				return (long) result;
+				return result;
 		}
 
 		private static void EndianSwap (ref byte[] value)
@@ -2612,8 +2712,6 @@ namespace System {
 
 		private static string ConvertToBase8 (byte[] value)
 		{
-			if (!BitConverter.IsLittleEndian)
-				EndianSwap (ref value);
 			ulong l = 0;
 			switch (value.Length) {
 			case 1:
