@@ -7,9 +7,7 @@
 //   Atsushi Enomoto (atsushi@ximian.com)
 //
 //   (C) 2001 Marcel Narings
-
-//
-// Copyright (C) 2004 Novell, Inc (http://www.novell.com)
+// Copyright (C) 2004-2005 Novell, Inc (http://www.novell.com)
 //
 // Permission is hereby granted, free of charge, to any person obtaining
 // a copy of this software and associated documentation files (the
@@ -31,7 +29,6 @@
 // WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 //
 
-using System;
 using System.Collections;
 using System.Globalization;
 using System.Runtime.CompilerServices;
@@ -104,6 +101,8 @@ namespace System
 			"yyyy/M/d HH:mm:ss",
 			// bug #47720
 			"yyyy/MM/dd HH:mm:ss 'GMT'",
+			// bug #53023
+			"MM/dd/yyyy",
 			// Close to RFC1123, but without 'GMT'
 			"ddd, d MMM yyyy HH:mm:ss",
 			// use UTC ('Z'; not literal "'Z'")
@@ -112,10 +111,17 @@ namespace System
 
 			// bug #60912
 			"M/d/yyyy HH':'mm':'ss tt",
+			"H':'mm':'ss tt",
+			// another funky COM dependent one
+			"dd-MMM-yy",
 
 			// DayOfTheWeek, dd full_month_name yyyy
-			// FIXME: 1054(th-TH) rejects it
+			// FIXME: 1054(th-TH) rejects them
 			"dddd, dd MMMM yyyy",
+			"dddd, dd MMMM yyyy HH:mm",
+			"dddd, dd MMMM yyyy HH:mm:ss",
+
+			"yyyy MMMM",
 			// DayOfTheWeek, dd yyyy. This works for every locales.
 			"MMMM dd, yyyy",
 #if NET_1_1
@@ -124,6 +130,13 @@ namespace System
 #endif
 			// In Parse() the 'r' equivalent pattern is first parsed as universal time
 			"ddd, dd MMM yyyy HH':'mm':'ss 'GMT'",
+
+			// Additionally there seems language-specific format
+			// patterns that however works in all language
+			// environment.
+			// For example, the pattern below is for Japanese.
+			"yyyy'\u5E74'MM'\u6708'dd'\u65E5' HH'\u6642'mm'\u5206'ss'\u79D2'",
+
 /*
 			// Full date and time
 			"F", "G", "r", "s", "u", "U",
@@ -216,23 +229,14 @@ namespace System
 		/// Constructs a DateTime for specified ticks
 		/// </summary>
 		/// 
-		public DateTime (long newticks)
-			// `local' must default to false here to avoid
-			// a recursion loop.
-			: this (false, newticks) {}
-
-		internal DateTime (bool local, long newticks)
+		public DateTime (long ticks)
 		{
-			ticks = new TimeSpan (newticks);
-			if (local) {
-				TimeZone tz = TimeZone.CurrentTimeZone;
-
-				TimeSpan utcoffset = tz.GetUtcOffset (this);
-
-				ticks = ticks + utcoffset;
+			this.ticks = new TimeSpan (ticks);
+			if (ticks < MinValue.Ticks || ticks > MaxValue.Ticks) {
+				string msg = Locale.GetText ("Value {0} is outside the valid range [{1},{2}].", 
+					ticks, MinValue.Ticks, MaxValue.Ticks);
+				throw new ArgumentOutOfRangeException ("ticks", msg);
 			}
-			if (ticks.Ticks < MinValue.Ticks || ticks.Ticks > MaxValue.Ticks)
-			    throw new ArgumentOutOfRangeException ();
 		}
 
 		public DateTime (int year, int month, int day)
@@ -370,7 +374,7 @@ namespace System
 		{
 			get	
 			{
-				return new DateTime (true, GetNow ());
+				return new DateTime (GetNow ()).ToLocalTime ();
 			}
 		}
 
@@ -558,7 +562,7 @@ namespace System
 			if (fileTime < 0)
 				throw new ArgumentOutOfRangeException ("fileTime", "< 0");
 
-			return new DateTime (true, w32file_epoch + fileTime);
+			return new DateTime (w32file_epoch + fileTime).ToLocalTime ();
 		}
 
 #if NET_1_1
@@ -567,7 +571,7 @@ namespace System
 			if (fileTime < 0)
 				throw new ArgumentOutOfRangeException ("fileTime", "< 0");
 
-			return new DateTime (false, w32file_epoch + fileTime);
+			return new DateTime (w32file_epoch + fileTime);
 		}
 #endif
 
@@ -702,7 +706,7 @@ namespace System
 			bool longYear = false;
 
 			// Try all the patterns
-			if (ParseExact (s, dfi.GetAllDateTimePatterns (), dfi, styles, out result, false, ref longYear))
+			if (ParseExact (s, dfi.GetAllDateTimePatternsInternal (), dfi, styles, out result, false, ref longYear))
 				return result;
 
 			// Try common formats.
@@ -819,7 +823,7 @@ namespace System
 			int valuePos = 0;
 			if (format.Length == 1)
 				format = _GetStandardPattern (format [0], dfi, out useutc, out use_invariant);
-			else if (!exact && format.IndexOf ("GMT") >= 0)
+			else if (!exact && CultureInfo.InvariantCulture.CompareInfo.IndexOf (format, "GMT", CompareOptions.Ordinal) >= 0)
 				useutc = true;
 
 			if ((style & DateTimeStyles.AllowLeadingWhite) != 0) {
@@ -895,15 +899,14 @@ namespace System
 					num = 0;
 					continue;
 				} else if (chars[pos] == '\\') {
-					if (pos+1 >= len)
+					pos += num + 1;
+					num = 0;
+					if (pos >= len)
 						return false;
 
-					if (s [valuePos] != chars [pos + num])
+					if (s [valuePos] != chars [pos])
 						return false;
 					valuePos++;
-					if (valuePos == s.Length)
-						return false;
-
 					pos++;
 					continue;
 				} else if (chars[pos] == '%') {
@@ -1282,9 +1285,11 @@ namespace System
 			// If no timezone was specified, default to the local timezone.
 			TimeSpan utcoffset;
 
-			if (useutc)
+			if (useutc) {
+				if ((style & DateTimeStyles.AdjustToUniversal) != 0)
+					use_localtime = false;
 				utcoffset = new TimeSpan (0, 0, 0);
-			else if (tzsign == -1) {
+			} else if (tzsign == -1) {
 				TimeZone tz = TimeZone.CurrentTimeZone;
 				utcoffset = tz.GetUtcOffset (result);
 			} else {
@@ -1303,7 +1308,9 @@ namespace System
 
 			long newticks = (result.ticks - utcoffset).Ticks;
 
-			result = new DateTime (use_localtime, newticks);
+			result = new DateTime (false, new TimeSpan (newticks));
+			if (use_localtime)
+				result = result.ToLocalTime ();
 
 			return true;
 		}
@@ -1787,47 +1794,14 @@ namespace System
 			return this._ToString (format, dfi);
 		}
 
-		public DateTime ToLocalTime()
+		public DateTime ToLocalTime ()
 		{
-			TimeZone tz = TimeZone.CurrentTimeZone;
-
-			TimeSpan offset = tz.GetUtcOffset (this);
-
-			if (offset.Ticks > 0) {
-				if (DateTime.MaxValue - offset < this)
-					return DateTime.MaxValue;
-			} else if (offset.Ticks < 0) {
-				// MS.NET fails to check validity here 
-				// - it may throw ArgumentOutOfRangeException
-				/*
-				if (DateTime.MinValue - offset > this)
-					return DateTime.MinValue;
-				*/
-			}
-			
-			DateTime lt = new DateTime(true, ticks+offset);
-			TimeSpan ltoffset = tz.GetUtcOffset(lt);
-			if(ltoffset != offset)
-				lt = lt.Add(ltoffset.Subtract(offset));
-
-			return lt;
+			return TimeZone.CurrentTimeZone.ToLocalTime (this);
 		}
 
 		public DateTime ToUniversalTime()
 		{
-			TimeZone tz = TimeZone.CurrentTimeZone;
-
-			TimeSpan offset = tz.GetUtcOffset (this);
-
-			if (offset.Ticks < 0) {
-				if (DateTime.MaxValue + offset < this)
-					return DateTime.MaxValue;
-			} else if (offset.Ticks > 0) {
-				if (DateTime.MinValue + offset > this)
-					return DateTime.MinValue;
-			}
-
-			return new DateTime (false, ticks - offset);
+			return TimeZone.CurrentTimeZone.ToUniversalTime (this);
 		}
 
 		/*  OPERATORS */
@@ -1964,17 +1938,3 @@ namespace System
 		}
 	}
 }
-
-namespace System
-{
-	public enum DayOfWeek
-	{
-		Sunday,
-		Monday,
-		Tuesday,
-		Wednesday,
-		Thursday,
-		Friday,
-		Saturday
-	}
-}
