@@ -8,7 +8,6 @@
 //
 // (C) Ximian, Inc.  http://www.ximian.com
 //
-// TODO: Mucho left to implement.
 //
 
 //
@@ -268,6 +267,41 @@ namespace System
 			return Parse (enumType, value, false);
 		}
 
+		private static int FindName (string [] names, string name,  bool ignoreCase)
+		{
+			for (int i = 0; i < names.Length; ++i) {
+				if (String.Compare (name, names [i], ignoreCase, CultureInfo.InvariantCulture) == 0)
+					return i;
+			}
+			return -1;
+		}
+
+		// Helper function for dealing with [Flags]-style enums.
+		private static ulong GetValue (object value, TypeCode typeCode)
+		{
+			switch (typeCode) {
+			case TypeCode.Byte:
+				return (byte) value;
+			case TypeCode.SByte:
+				return (byte) ((sbyte) value);
+			case TypeCode.Int16:
+				return (ushort) ((short) value);
+			case TypeCode.Int32:
+				return (uint) ((int) value);
+			case TypeCode.Int64:
+				return (ulong) ((long) value);
+			case TypeCode.UInt16:
+				return (ushort) value;
+			case TypeCode.UInt32:
+				return (uint) value;
+			case TypeCode.UInt64:
+				return (ulong) value;
+			}
+			throw new ArgumentException ("typeCode is not a valid type code for an Enum");
+		}
+
+		private static char [] split_char = { ',' };
+
 		public static object Parse (Type enumType, string value, bool ignoreCase)
 		{
 			if (enumType == null)
@@ -277,73 +311,41 @@ namespace System
 				throw new ArgumentNullException ("value");
 
 			if (!enumType.IsEnum)
-				throw new ArgumentException ("enumType is not an Enum type.");
+				throw new ArgumentException ("not an Enum type", "enumType");
 
-			if (String.Empty == value.Trim())
-				throw new ArgumentException ("value cannot be empty string.");
+			value = value.Trim ();
+			if (String.Empty == value)
+				throw new ArgumentException ("cannot be an empty string", "value");
 
 			MonoEnumInfo info;
-			int i;
 			MonoEnumInfo.GetInfo (enumType, out info);
 
-			long retVal = 0;
+			// is 'value' a named constant?
+			int loc = FindName (info.names, value, ignoreCase);
+			if (loc >= 0)
+				return info.values.GetValue (loc);
 
 			TypeCode typeCode = ((Enum) info.values.GetValue (0)).GetTypeCode ();
 
-			try {
-				// Attempt to convert to numeric type
-				return ToObject (enumType, Convert.ChangeType (value, typeCode) );
-			} catch {}
-
-			string[] names = value.Split (new char[] {','});
-			for (i = 0; i < names.Length; ++i)
-				names [i] = names [i].Trim ();
-
-			foreach (string name in names) {
-				bool found = false;
-				for (i = 0; i < info.values.Length; ++i) {
-					if (String.Compare (name, info.names [i], ignoreCase, CultureInfo.InvariantCulture) == 0) {
-						switch (typeCode) {
-							case TypeCode.Byte:
-								retVal |= (long)((byte)info.values.GetValue (i));
-								break;
-							case TypeCode.SByte:
-								// use the unsigned version in the cast to avoid 
-								// compiler warning
-								retVal |= (long)((byte)(SByte)info.values.GetValue (i));
-								break;
-							case TypeCode.Int16:
-								// use the unsigned version in the cast to avoid 
-								// compiler warning
-								retVal |= (long)((ushort)(short)info.values.GetValue (i));
-								break;
-							case TypeCode.Int32:
-								// use the unsigned version in the cast to avoid 
-								// compiler warning
-								retVal |= (long)((uint)(int)info.values.GetValue (i));
-								break;
-							case TypeCode.Int64:
-								retVal |= (long)info.values.GetValue (i);
-								break;
-							case TypeCode.UInt16:
-								retVal |= (long)((UInt16)info.values.GetValue (i));
-								break;
-							case TypeCode.UInt32:
-								retVal |= (long)((UInt32)info.values.GetValue (i));
-								break;
-							case TypeCode.UInt64:
-								retVal |= (long)((UInt64)info.values.GetValue (i));
-								break;
-						}
-						found = true;
-						break;
-					}
+			// is 'value' a list of named constants?
+			if (value.IndexOf (',') != -1) {
+				string [] names = value.Split (split_char);
+				ulong retVal = 0;
+				for (int i = 0; i < names.Length; ++i) {
+					loc = FindName (info.names, names [i].Trim (), ignoreCase);
+					if (loc < 0)
+						throw new ArgumentException ("The requested value was not found.");
+					retVal |= GetValue (info.values.GetValue (loc), typeCode);
 				}
-				if (!found)
-					throw new ArgumentException ("The requested value was not found.");
-				
+				return ToObject (enumType, retVal);
+			}
+
+			// is 'value' a number?
+			try {
+				return ToObject (enumType, Convert.ChangeType (value, typeCode));
+			} catch (Exception e) {
+				throw new ArgumentException (String.Format ("The requested value `{0}' was not found", value), e);
 			}
-			return ToObject (enumType, retVal);
 		}
 
 		/// <summary>
@@ -521,7 +523,7 @@ namespace System
 				sbyte enumValue;
 				for (int i = info.values.Length - 1; i >= 0; i--) {
 					enumValue = (sbyte) info.values.GetValue (i);
-					if (i == 0 && enumValue == 0)
+					if (enumValue == 0)
 						continue;
 
 					if ((flags & enumValue) == enumValue) {
@@ -537,7 +539,7 @@ namespace System
 				byte enumValue;
 				for (int i = info.values.Length - 1; i >= 0; i--) {
 					enumValue = (byte) info.values.GetValue (i);
-					if (i == 0 && enumValue == 0)
+					if (enumValue == 0)
 						continue;
 
 					if ((flags & enumValue) == enumValue) {
@@ -553,7 +555,7 @@ namespace System
 				short enumValue;
 				for (int i = info.values.Length - 1; i >= 0; i--) {
 					enumValue = (short) info.values.GetValue (i);
-					if (i == 0 && enumValue == 0)
+					if (enumValue == 0)
 						continue;
 
 					if ((flags & enumValue) == enumValue) {
@@ -569,7 +571,7 @@ namespace System
 				int enumValue;
 				for (int i = info.values.Length - 1; i >= 0; i--) {
 					enumValue = (int) info.values.GetValue (i);
-					if (i == 0 && enumValue == 0)
+					if (enumValue == 0)
 						continue;
 
 					if ((flags & enumValue) == enumValue) {
@@ -585,7 +587,7 @@ namespace System
 				ushort enumValue;
 				for (int i = info.values.Length - 1; i >= 0; i--) {
 					enumValue = (ushort) info.values.GetValue (i);
-					if (i == 0 && enumValue == 0)
+					if (enumValue == 0)
 						continue;
 
 					if ((flags & enumValue) == enumValue) {
@@ -601,7 +603,7 @@ namespace System
 				uint enumValue;
 				for (int i = info.values.Length - 1; i >= 0; i--) {
 					enumValue = (uint) info.values.GetValue (i);
-					if (i == 0 && enumValue == 0)
+					if (enumValue == 0)
 						continue;
 
 					if ((flags & enumValue) == enumValue) {
@@ -617,7 +619,7 @@ namespace System
 				long enumValue;
 				for (int i = info.values.Length - 1; i >= 0; i--) {
 					enumValue = (long) info.values.GetValue (i);
-					if (i == 0 && enumValue == 0)
+					if (enumValue == 0)
 						continue;
 
 					if ((flags & enumValue) == enumValue) {
@@ -633,7 +635,7 @@ namespace System
 				ulong enumValue;
 				for (int i = info.values.Length - 1; i >= 0; i--) {
 					enumValue = (ulong) info.values.GetValue (i);
-					if (i == 0 && enumValue == 0)
+					if (enumValue == 0)
 						continue;
 
 					if ((flags & enumValue) == enumValue) {
