@@ -59,8 +59,8 @@ namespace System {
 		 * Changes which are already detected at runtime, like the addition
 		 * of icalls, do not require an increment.
 		 */
-		private const int mono_corlib_version = 37;
-               
+		private const int mono_corlib_version = 45;
+		
 		[MonoTODO]
 		public enum SpecialFolder
 		{	// TODO: Determine if these windoze style folder identifiers 
@@ -254,15 +254,7 @@ namespace System {
 		/// </summary>
 		public static Version Version {
 			get {
-#if NET_2_0
-				// FIXME: this is the version number for MS.NET 2.0 beta1. 
-				// It must be changed when the final version is released.
-				return new Version (2, 0, 50215, 16);
-#elif NET_1_1				    
-				return new Version (1, 1, 4322, 573);
-#else
-				return new Version (1, 0, 3705, 288);
-#endif
+				return new Version (Consts.RuntimeVersion);
 			}
 		}
 
