@@ -596,7 +596,6 @@ namespace System
 			if (ns.IsFloatingSource || ns.IsDecimalSource)
 				throw new FormatException ();
 
-			int intSize = ns.DefaultByteSize;
 			ulong value = 0;
 			for (int i = 0; i < ns.IntegerDigits; i++) {
 				value *= 10;
@@ -604,7 +603,13 @@ namespace System
 			}
 
 			if (!ns.Positive) {
-				value = (ulong)(Math.Pow (2, intSize * 8)) - value;
+				int intSize = ns.DefaultByteSize;
+				/* for large values the cast to ulong is going to return 0 anyway (at least on x86, possibly a MS/Mono runtime bug) */
+				if (intSize < 8) {
+					value = (ulong)(Math.Pow (2, intSize * 8)) - value;
+				} else {
+					value = 0 - value;
+				}
 			}
 
 			char[] digits = (upper ? digitUpperTable : digitLowerTable);
@@ -701,7 +706,7 @@ namespace System
 				cb.Append (digitLowerTable [exponent / 100 % 10]);
 				cb.Append (digitLowerTable [exponent / 10 % 10]);
 				cb.Append (digitLowerTable [exponent % 10]);
-			/*} else { // exponent range is 0`}324
+			/*} else { // exponent range is 0...+-324
 				int pos = cb.Length;
 				int count = 3;
 				while (exponent > 0 || --count > 0) {
@@ -1965,20 +1970,23 @@ namespace System
 				if (digits + 1 < _digits.Length && _digits [digits + 1] == 5 && _digits [digits] % 2 == (carryEven ? 0 : 1))
 					carryFive = false;
 
-				for (int i = digits; i >= 0; i--) {
-					RoundHelper (i, carryFive, ref carry);
-					if (!carry)
-						break;
-				}
-
-				if (carry) {
-					byte[] temp = new byte [_digits.Length + 1];
-					_digits.CopyTo (temp, 1);
-					temp [0] = 1;
-					_digits = temp;
-					_decPointPos ++;
-					digits ++;
+				/// are we cutting from the maximum precision ?
+				if (_digits.Length == _defMaxPrecision) {
+					// special case if we *aren't* cutting inside the extra precision (e.g. 16 on 17)
+					if (digits != _defMaxPrecision - 1) {
+						// ok, here we look at the *two* extra numbers we're keeping
+						// (we keep 17 digits while the true precision is 15 digits).
+						int extra = _digits[_defMaxPrecision - 2] * 10 + _digits[_defMaxPrecision - 1];
+						carry = (extra >= 50);
+						if (carry) {
+							_digits[_defMaxPrecision - 2] = 0;
+							_digits[_defMaxPrecision - 1] = 0;
+							int d = _digits.Length - 3;
+							CarryPropagation (ref d, carryFive, ref carry);
+						}
+					}
 				}
+				CarryPropagation (ref digits, carryFive, ref carry);
 
 				for (int i = digits; i < _digits.Length; i++)
 					_digits [i] = 0;
@@ -1986,6 +1994,25 @@ namespace System
 
 				return carry;
 			}
+
+			private void CarryPropagation (ref int digits, bool carryFive, ref bool carry)
+			{
+				for (int i = digits; i >= 0; i--) {
+					RoundHelper (i, carryFive, ref carry);
+					if (!carry)
+						break;
+				}
+
+				if (carry) {
+					byte[] temp = new byte[_digits.Length + 1];
+					_digits.CopyTo (temp, 1);
+					temp[0] = 1;
+					_digits = temp;
+					_decPointPos++;
+					digits++;
+				}
+			}
+
 			#endregion
 
 			#region Trim
