@@ -202,6 +202,112 @@ namespace System
 			return InternalSplit (separator, count);
 		}
 
+#if NET_2_0
+		[ComVisible (false)]
+		[MonoTODO]
+		public String[] Split (char[] separator, int count, StringSplitOptions options)
+		{
+			if (separator == null || separator.Length == 0)
+				return Split (WhiteChars, count, options);
+
+			if (count < 0)
+				throw new ArgumentOutOfRangeException ("count", "Count cannot be less than zero.");
+			if ((options != StringSplitOptions.None) && (options != StringSplitOptions.RemoveEmptyEntries))
+				throw new ArgumentException ("options must be one of the values in the StringSplitOptions enumeration", "options");
+
+			bool removeEmpty = (options & StringSplitOptions.RemoveEmptyEntries) == StringSplitOptions.RemoveEmptyEntries;
+
+			if (!removeEmpty)
+				return Split (separator, count);
+			else
+				throw new NotImplementedException ();
+		}
+
+		[ComVisible (false)]
+		public String[] Split (string[] separator, int count, StringSplitOptions options)
+		{
+			if (separator == null || separator.Length == 0)
+				return Split (WhiteChars, count, options);
+
+			if (count < 0)
+				throw new ArgumentOutOfRangeException ("count", "Count cannot be less than zero.");
+			if ((options != StringSplitOptions.None) && (options != StringSplitOptions.RemoveEmptyEntries))
+				throw new ArgumentException ("Illegal enum value: " + options + ".", "options");
+
+			bool removeEmpty = (options & StringSplitOptions.RemoveEmptyEntries) == StringSplitOptions.RemoveEmptyEntries;
+
+			if (count == 0 || (this == String.Empty && removeEmpty))
+				return new String [0];
+
+			ArrayList arr = new ArrayList ();
+
+			int pos = 0;
+			while (pos < this.Length) {
+				int matchIndex = -1;
+				int matchPos = Int32.MaxValue;
+
+				// Find the first position where any of the separators matches
+				for (int i = 0; i < separator.Length; ++i) {
+					string sep = separator [i];
+					if (sep == null || sep == String.Empty)
+						continue;
+
+					int match = IndexOf (sep, pos);
+					if (match > -1 && match < matchPos) {
+						matchIndex = i;
+						matchPos = match;
+					}
+				}
+
+				if (matchIndex == -1)
+					break;
+
+				if (matchPos == pos && removeEmpty) {
+					pos = matchPos + separator [matchIndex].Length;
+				}
+				else {
+					arr.Add (this.Substring (pos, matchPos - pos));
+
+					pos = matchPos + separator [matchIndex].Length;
+
+					if (arr.Count == count - 1) {
+						break;
+					}
+				}
+			}
+
+			if (arr.Count == 0)
+				return new String [] { this };
+			else {
+				if (removeEmpty && pos == this.Length) {
+					String[] res = new String [arr.Count];
+					arr.CopyTo (0, res, 0, arr.Count);
+
+					return res;
+				}
+				else {
+					String[] res = new String [arr.Count + 1];
+					arr.CopyTo (0, res, 0, arr.Count);
+					res [arr.Count] = this.Substring (pos);
+
+					return res;
+				}
+			}
+		}
+
+		[ComVisible (false)]
+		public String[] Split (char[] separator, StringSplitOptions options)
+		{
+			return Split (separator, Int32.MaxValue, options);
+		}
+
+		[ComVisible (false)]
+		public String[] Split (String[] separator, StringSplitOptions options)
+		{
+			return Split (separator, Int32.MaxValue, options);
+		}
+#endif
+
 		public unsafe String Substring (int startIndex)
 		{
 			if (startIndex < 0 || startIndex > this.length)
@@ -239,6 +345,9 @@ namespace System
 		}	
 
 		private static readonly char[] WhiteChars = { (char) 0x9, (char) 0xA, (char) 0xB, (char) 0xC, (char) 0xD,
+#if NET_2_0
+			(char) 0x85, (char) 0x1680, (char) 0x2028, (char) 0x2029,
+#endif
 			(char) 0x20, (char) 0xA0, (char) 0x2000, (char) 0x2001, (char) 0x2002, (char) 0x2003, (char) 0x2004,
 			(char) 0x2005, (char) 0x2006, (char) 0x2007, (char) 0x2008, (char) 0x2009, (char) 0x200A, (char) 0x200B,
 			(char) 0x3000, (char) 0xFEFF };
@@ -365,7 +474,59 @@ namespace System
 
 			return culture.CompareInfo.Compare (strA, indexA, len1, strB, indexB, len2, compopts);
 		}
+#if NET_2_0
+		public static int Compare (string strA, string strB, StringComparison comparisonType)
+		{
+			switch (comparisonType) {
+			case StringComparison.CurrentCulture:
+				return Compare (strA, strB, false, CultureInfo.CurrentCulture);
+			case StringComparison.CurrentCultureIgnoreCase:
+				return Compare (strA, strB, true, CultureInfo.CurrentCulture);
+			case StringComparison.InvariantCulture:
+				return Compare (strA, strB, false, CultureInfo.InvariantCulture);
+			case StringComparison.InvariantCultureIgnoreCase:
+				return Compare (strA, strB, true, CultureInfo.InvariantCulture);
+			case StringComparison.Ordinal:
+				return CompareOrdinal (strA, strB, CompareOptions.Ordinal);
+			case StringComparison.OrdinalIgnoreCase:
+				return CompareOrdinal (strA, strB, CompareOptions.Ordinal | CompareOptions.IgnoreCase);
+			default:
+				string msg = Locale.GetText ("Invalid value '{0}' for StringComparison", comparisonType);
+				throw new ArgumentException ("comparisonType", msg);
+			}
+		}
 
+		public static int Compare (string strA, int indexA, string strB, int indexB, int length, StringComparison comparisonType)
+		{
+			switch (comparisonType) {
+			case StringComparison.CurrentCulture:
+				return Compare (strA, indexA, strB, indexB, length, false, CultureInfo.CurrentCulture);
+			case StringComparison.CurrentCultureIgnoreCase:
+				return Compare (strA, indexA, strB, indexB, length, true, CultureInfo.CurrentCulture);
+			case StringComparison.InvariantCulture:
+				return Compare (strA, indexA, strB, indexB, length, false, CultureInfo.InvariantCulture);
+			case StringComparison.InvariantCultureIgnoreCase:
+				return Compare (strA, indexA, strB, indexB, length, true, CultureInfo.InvariantCulture);
+			case StringComparison.Ordinal:
+				return CompareOrdinal (strA, indexA, strB, indexB, length, CompareOptions.Ordinal);
+			case StringComparison.OrdinalIgnoreCase:
+				return CompareOrdinal (strA, indexA, strB, indexB, length, CompareOptions.Ordinal | CompareOptions.IgnoreCase);
+			default:
+				string msg = Locale.GetText ("Invalid value '{0}' for StringComparison", comparisonType);
+				throw new ArgumentException ("comparisonType", msg);
+			}
+		}
+
+		public static bool Equals (string a, string b, StringComparison comparisonType)
+		{
+			return String.Compare (a, b, comparisonType) == 0;
+		}
+
+		public bool Equals (string value, StringComparison comparisonType)
+		{
+			return String.Equals (this, value, comparisonType);
+		}
+#endif
 		public int CompareTo (Object value)
 		{
 			if (value == null)
@@ -386,6 +547,11 @@ namespace System
 		}
 
 		public static int CompareOrdinal (String strA, String strB)
+		{
+			return CompareOrdinal (strA, strB, CompareOptions.Ordinal);
+		}
+
+		internal static int CompareOrdinal (String strA, String strB, CompareOptions options)
 		{
 			if (strA == null) {
 				if (strB == null)
@@ -401,10 +567,15 @@ namespace System
 			 * instantiate (and chances are it already has
 			 * been.)
 			 */
-			return CultureInfo.InvariantCulture.CompareInfo.Compare (strA, strB, CompareOptions.Ordinal);
+			return CultureInfo.InvariantCulture.CompareInfo.Compare (strA, strB, options);
 		}
 
 		public static int CompareOrdinal (String strA, int indexA, String strB, int indexB, int length)
+		{
+			return CompareOrdinal (strA, indexA, strB, indexB, length, CompareOptions.Ordinal);
+		}
+
+		internal static int CompareOrdinal (String strA, int indexA, String strB, int indexB, int length, CompareOptions options)
 		{
 			if ((indexA > strA.Length) || (indexB > strB.Length) || (indexA < 0) || (indexB < 0) || (length < 0))
 				throw new ArgumentOutOfRangeException ();
@@ -435,21 +606,24 @@ namespace System
 				len2 = strB.Length - indexB;
 			}
 
-			return CultureInfo.InvariantCulture.CompareInfo.Compare (strA, indexA, len1, strB, indexB, len2, CompareOptions.Ordinal);
+			return CultureInfo.InvariantCulture.CompareInfo.Compare (strA, indexA, len1, strB, indexB, len2, options);
 		}
 
 		public bool EndsWith (String value)
 		{
-			if (value == null)
-				throw new ArgumentNullException ("value");
+			return EndsWith (value, false, CultureInfo.CurrentCulture);
+		}
 
-			if (value.Length == 0)
-				return true;
-
-			if (value.length > this.length)
-				return false;
-
-			return (0 == Compare (this, length - value.length, value, 0, value.length));
+#if NET_2_0
+		public
+#else
+		internal
+#endif
+		bool EndsWith (String value, bool ignoreCase, CultureInfo culture)
+		{
+			return (culture.CompareInfo.IsSuffix (this, value,
+				ignoreCase ? CompareOptions.IgnoreCase :
+				CompareOptions.None));
 		}
 
 		public int IndexOfAny (char [] anyOf)
@@ -564,7 +738,7 @@ namespace System
 			if (anyOf == null) 
 				throw new ArgumentNullException ("anyOf");
 
-			if (startIndex < 0 || startIndex > this.length)
+			if (startIndex < 0 || startIndex >= this.length)
 				throw new ArgumentOutOfRangeException ();
 
 			if (this.length == 0)
@@ -578,7 +752,7 @@ namespace System
 			if (anyOf == null) 
 				throw new ArgumentNullException ("anyOf");
 
-			if ((startIndex < 0) || (startIndex > this.Length))
+			if ((startIndex < 0) || (startIndex >= this.Length))
 				throw new ArgumentOutOfRangeException ("startIndex", "< 0 || > this.Length");
 			if ((count < 0) || (count > this.Length))
 				throw new ArgumentOutOfRangeException ("count", "< 0 || > this.Length");
@@ -734,16 +908,19 @@ namespace System
 
 		public bool StartsWith (String value)
 		{
-			if (value == null)
-				throw new ArgumentNullException ("value");
-			
-			if (value.Length == 0)
-				return true;
+			return StartsWith (value, false, CultureInfo.CurrentCulture);
+		}
 
-			if (this.length < value.length)
-				return false;
-
-			return (0 == Compare (this, 0, value, 0 , value.length));
+#if NET_2_0
+		public
+#else
+		internal
+#endif
+		bool StartsWith (String value, bool ignoreCase, CultureInfo culture)
+		{
+			return (culture.CompareInfo.IsPrefix (this, value,
+				ignoreCase ? CompareOptions.IgnoreCase :
+				CompareOptions.None));
 		}
 
 		/* This method is culture insensitive */
@@ -1432,9 +1609,10 @@ namespace System
 
 				if (str[ptr] == ',') {
 					// White space between ',' and number or sign.
-					int start = ++ptr;
+					++ptr;
 					while (Char.IsWhiteSpace (str [ptr]))
 						++ptr;
+					int start = ptr;
 
 					format = str.Substring (start, ptr - start);
 
