@@ -74,6 +74,7 @@ namespace System {
 		bool noGetPosition;
 		Hashtable keymap;
 		ByteMatcher rootmap;
+		bool home_1_1; // if true, we have to add 1 to x and y when using cursorAddress
 
 		static string SearchTerminfo (string term)
 		{
@@ -200,6 +201,7 @@ namespace System {
 			cursorAddress = reader.Get (TermInfoStrings.CursorAddress);
 			if (cursorAddress != null) {
 				string result = cursorAddress.Replace ("%i", "");
+				home_1_1 = (cursorAddress != result);
 				cursorAddress = MangleParameters (result);
 			}
 		}
@@ -250,6 +252,7 @@ namespace System {
 		public ConsoleColor BackgroundColor {
 			get { return bgcolor; }
 			set {
+				Init ();
 				bgcolor = value;
 				WriteConsole (String.Format (setabcolor, TranslateColor (value)));
 			}
@@ -258,6 +261,7 @@ namespace System {
 		public ConsoleColor ForegroundColor {
 			get { return fgcolor; }
 			set {
+				Init ();
 				fgcolor = value;
 				WriteConsole (String.Format (setafcolor, TranslateColor (value)));
 			}
@@ -543,6 +547,8 @@ namespace System {
 			if (n >= 1 && n <= 26 && !(n == 8 || n == 9 || n == 12 || n == 13 || n == 19)) {
 				ctrl = true;
 				key = ConsoleKey.A + n - 1;
+			} else if (n == 27) {
+				key = ConsoleKey.Escape;
 			} else if (n >= 'a' && n <= 'z') {
 				key = ConsoleKey.A - 'a' + n;
 			} else if (n >= 'A' && n <= 'Z') {
@@ -652,7 +658,8 @@ namespace System {
 			if (cursorAddress == null)
 				throw new NotSupportedException ("This terminal does not suport setting the cursor position.");
 
-			WriteConsole (String.Format (cursorAddress, top, left));
+			int one = (home_1_1 ? 1 : 0);
+			WriteConsole (String.Format (cursorAddress, top + one, left + one));
 			cursorLeft = left;
 			cursorTop = top;
 		}
@@ -671,7 +678,7 @@ namespace System {
 		void CreateKeyMap ()
 		{
 			keymap = new Hashtable ();
-			keymap [TermInfoStrings.KeyBackspace] = new ConsoleKeyInfo ('\0', ConsoleKey.BackSpace, false, false, false);
+			keymap [TermInfoStrings.KeyBackspace] = new ConsoleKeyInfo ('\0', ConsoleKey.Backspace, false, false, false);
 			keymap [TermInfoStrings.KeyClear] = new ConsoleKeyInfo ('\0', ConsoleKey.Clear, false, false, false);
 			// Delete character...
 			keymap [TermInfoStrings.KeyDown] = new ConsoleKeyInfo ('\0', ConsoleKey.DownArrow, false, false, false);
