@@ -59,6 +59,11 @@ namespace System
 			}
 		}
 
+		internal static void ClearCurrentTimeZone ()
+		{
+			currentTimeZone = null;
+		}
+
 		public abstract string DaylightName {
 			get;
 		}
@@ -103,12 +108,48 @@ namespace System
 
 		public virtual DateTime ToLocalTime (DateTime time)
 		{
-			return time + GetUtcOffset (time);
+			DaylightTime dlt = GetDaylightChanges (time.Year);
+			TimeSpan utcOffset = GetUtcOffset (time);
+			if (utcOffset.Ticks > 0) {
+				if (DateTime.MaxValue - utcOffset < time)
+					return DateTime.MaxValue;
+			//} else if (utcOffset.Ticks < 0) {
+			//	LAMESPEC: MS.NET fails to check validity here
+			//	it may throw ArgumentOutOfRangeException.
+			}
+
+			DateTime local = time.Add (utcOffset);
+			if (dlt.Delta.Ticks == 0)
+				return local;
+
+			// FIXME: check all of the combination of
+			//	- basis: local-based or UTC-based
+			//	- hemisphere: Northern or Southern
+			//	- offset: positive or negative
+
+			// PST should work fine here.
+			if (local < dlt.End && dlt.End.Subtract (dlt.Delta) <= local)
+				return local;
+			if (local >= dlt.Start && dlt.Start.Add (dlt.Delta) > local)
+				return local.Subtract (dlt.Delta);
+
+			TimeSpan localOffset = GetUtcOffset (local);
+			return time.Add (localOffset);
 		}
 
 		public virtual DateTime ToUniversalTime (DateTime time)
 		{
-			return time - GetUtcOffset (time);
+			TimeSpan offset = GetUtcOffset (time);
+
+			if (offset.Ticks < 0) {
+				if (DateTime.MaxValue + offset < time)
+					return DateTime.MaxValue;
+			} else if (offset.Ticks > 0) {
+				if (DateTime.MinValue + offset > time)
+					return DateTime.MinValue;
+			}
+
+			return new DateTime (time.Ticks - offset.Ticks);
 		}
 	}
 
@@ -187,24 +228,22 @@ namespace System
 				throw new ArgumentOutOfRangeException ("year", year +
 					Locale.GetText (" is not in a range between 1 and 9999."));
 
-			if (daylightCache [year] == null) {
-				lock (daylightCache) {
-					if (daylightCache [year] == null) {
-						Int64[] data;
-						string[] names;
+			lock (daylightCache) {
+				DaylightTime dlt = (DaylightTime) daylightCache [year];
+				if (dlt == null) {
+					Int64[] data;
+					string[] names;
 
-						if (!GetTimeZoneData (year, out data, out names))
-							throw new ArgumentException (Locale.GetText ("Can't get timezone data for " + year));
+					if (!GetTimeZoneData (year, out data, out names))
+						throw new ArgumentException (Locale.GetText ("Can't get timezone data for " + year));
 
-						DaylightTime dlt = new DaylightTime (new DateTime (data[(int)TimeZoneData.DaylightSavingStartIdx]),
-										     new DateTime (data[(int)TimeZoneData.DaylightSavingEndIdx]),
-										     new TimeSpan (data[(int)TimeZoneData.AdditionalDaylightOffsetIdx]));
-						daylightCache.Add (year, dlt);
-					};
-				};
+					dlt = new DaylightTime (new DateTime (data[(int)TimeZoneData.DaylightSavingStartIdx]),
+									     new DateTime (data[(int)TimeZoneData.DaylightSavingEndIdx]),
+									     new TimeSpan (data[(int)TimeZoneData.AdditionalDaylightOffsetIdx]));
+					daylightCache.Add (year, dlt);
+				}
+				return dlt;
 			}
-
-			return (DaylightTime) daylightCache [year];
 		}
 
 		public override TimeSpan GetUtcOffset (DateTime time)
