@@ -468,6 +468,10 @@ namespace System {
 		internal extern static TypeCode GetTypeCodeInternal (Type type);
 
 		public static TypeCode GetTypeCode (Type type) {
+			if (type == null)
+				/* MS.NET returns this */
+				return TypeCode.Empty;
+
 			type = type.UnderlyingSystemType;
 
 			if (!type.IsSystemType)
@@ -551,6 +555,11 @@ namespace System {
 				return type_is_subtype_of (a, b, check_interfaces);
 		}
 
+		public new Type GetType ()
+		{
+			return base.GetType ();
+		}
+
 #if NET_2_0
 		[ComVisible (true)]
 #endif
@@ -1074,17 +1083,16 @@ namespace System {
 		}
 
 #if NET_2_0 || BOOTSTRAP_NET_2_0
-		public abstract Type[] GetGenericArguments ();
-
-		public abstract bool HasGenericArguments {
-			get;
+		public virtual Type[] GetGenericArguments ()
+		{
+			throw new NotSupportedException ();
 		}
 
 		public abstract bool ContainsGenericParameters {
 			get;
 		}
 
-		public extern bool IsGenericTypeDefinition {
+		public virtual extern bool IsGenericTypeDefinition {
 			[MethodImplAttribute(MethodImplOptions.InternalCall)]
 			get;
 		}
@@ -1106,13 +1114,15 @@ namespace System {
 			get;
 		}
 
-		[MethodImplAttribute(MethodImplOptions.InternalCall)]
-		static extern Type BindGenericParameters (Type gt, Type [] types);
+		public extern bool IsGenericType {
+			[MethodImplAttribute(MethodImplOptions.InternalCall)]
+			get;
+		}
 
-#if NET_2_0
-		[ComVisible (true)]
-#endif		
-		public Type BindGenericParameters (Type [] types)
+		[MethodImplAttribute(MethodImplOptions.InternalCall)]
+		static extern Type MakeGenericType (Type gt, Type [] types);
+
+		public virtual Type MakeGenericType (params Type[] types)
 		{
 			if (types == null)
 				throw new ArgumentNullException ("types");
@@ -1120,19 +1130,31 @@ namespace System {
 				if (t == null)
 					throw new ArgumentNullException ("types");
 			}
-			Type res = BindGenericParameters (this, types);
+			Type res = MakeGenericType (this, types);
 			if (res == null)
 				throw new TypeLoadException ();
 			return res;
 		}
 
-                public Type MakeGenericType (Type[] types)
-                {
-			return BindGenericParameters (types);
-                }
+		public virtual bool IsGenericParameter {
+			get {
+				return false;
+			}
+		}
 
-		public abstract bool IsGenericParameter {
-			get;
+		public bool IsNested {
+			get {
+				return DeclaringType != null;
+			}
+		}
+
+		public bool IsVisible {
+			get {
+				if (IsNestedPublic)
+					return DeclaringType.IsVisible;
+
+				return IsPublic;
+			}
 		}
 
 		[MethodImplAttribute(MethodImplOptions.InternalCall)]
@@ -1170,8 +1192,10 @@ namespace System {
 			return GetGenericParameterConstraints_impl ();
 		}
 
-		public abstract MethodInfo DeclaringMethod {
-			get;
+		public virtual MethodBase DeclaringMethod {
+			get {
+				return null;
+			}
 		}
 
 		[MethodImplAttribute(MethodImplOptions.InternalCall)]
@@ -1238,10 +1262,14 @@ namespace System {
 			}
 		}
 
-		internal object[] GetPseudoCustomAttributes () {
+		internal object[] GetPseudoCustomAttributes ()
+		{
 			int count = 0;
 
-			if (IsSerializable)
+			/* IsSerializable returns true for delegates/enums as well */
+			if ((Attributes & TypeAttributes.Serializable) != 0)
+				count ++;
+			if ((Attributes & TypeAttributes.Import) != 0)
 				count ++;
 
 			if (count == 0)
@@ -1249,12 +1277,34 @@ namespace System {
 			object[] attrs = new object [count];
 			count = 0;
 
-			if (IsSerializable)
+			if ((Attributes & TypeAttributes.Serializable) != 0)
 				attrs [count ++] = new SerializableAttribute ();
+			if ((Attributes & TypeAttributes.Import) != 0)
+				attrs [count ++] = new ComImportAttribute ();
 
 			return attrs;
 		}			
 
 #endif
+
+		void _Type.GetIDsOfNames ([In] ref Guid riid, IntPtr rgszNames, uint cNames, uint lcid, IntPtr rgDispId)
+		{
+			throw new NotImplementedException ();
+		}
+
+		void _Type.GetTypeInfo (uint iTInfo, uint lcid, IntPtr ppTInfo)
+		{
+			throw new NotImplementedException ();
+		}
+
+		void _Type.GetTypeInfoCount (out uint pcTInfo)
+		{
+			throw new NotImplementedException ();
+		}
+
+		void _Type.Invoke (uint dispIdMember, [In] ref Guid riid, uint lcid, short wFlags, IntPtr pDispParams, IntPtr pVarResult, IntPtr pExcepInfo, IntPtr puArgErr)
+		{
+			throw new NotImplementedException ();
+		}
 	}
 }
