@@ -1,10 +1,14 @@
 //
 // System.__ComObject
 //
-// Author:
-//	Sebastien Pouliot  <sebastien@ximian.com>
+// Authors:
+//   Sebastien Pouliot <sebastien@ximian.com>
+//   Kornl Pl <http://www.kornelpal.hu/>
 //
 // Copyright (C) 2004 Novell (http://www.novell.com)
+// Copyright (C) 2005 Kornl Pl
+//
+
 //
 // Permission is hereby granted, free of charge, to any person obtaining
 // a copy of this software and associated documentation files (the
@@ -13,10 +17,10 @@
 // distribute, sublicense, and/or sell copies of the Software, and to
 // permit persons to whom the Software is furnished to do so, subject to
 // the following conditions:
-// 
+//
 // The above copyright notice and this permission notice shall be
 // included in all copies or substantial portions of the Software.
-// 
+//
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 // EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 // MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
@@ -26,12 +30,21 @@
 // WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 //
 
-namespace System {
+namespace System
+{
+	// This is a private class that is used as a generic wrapper class
+	// for COM objects that have no specific wrapper class.
+	//
+	// It has no public methods, it's functionality is exposed trough
+	// System.Runtime.InteropServices.Marshal class and can be casted to
+	// any interface that is implemented by the wrapped COM object.
+	//
+	// This class is referenced in .NET Framework SDK Documentation so
+	// many times that obj.GetType().FullName == "System.__ComObject" and
+	// Type.GetType("System.__ComObject") may be used.
 
-	[CLSCompliant (false)]
-	[MonoTODO ("Is there someone in there ?")]
-	public class __ComObject : MarshalByRefObject {
-	
+	internal class __ComObject : MarshalByRefObject
+	{
 		private __ComObject ()
 		{
 		}
