@@ -109,6 +109,18 @@ namespace MonoTests.System.Reflection
 				a1 = 2;
 		}
 
+		public void HeyHey (out string out1, ref string ref1)
+		{
+			out1 = null;
+		}
+
+		[Test] // bug #76541
+		public void ToStringByRef ()
+		{
+			AssertEquals ("Void HeyHey(System.String ByRef, System.String ByRef)",
+				this.GetType ().GetMethod ("HeyHey").ToString ());
+		}
+
 #if NET_2_0
 		[Test]
 		[ExpectedException (typeof (ArgumentException))]
@@ -177,6 +189,16 @@ namespace MonoTests.System.Reflection
 			m.Invoke (null, new object [0]);
 		}
 
+		public void MakeGenericMethodArgsMismatchFoo<T> () {}
+
+		[Test]
+		[ExpectedException (typeof (ArgumentException))]
+		public void MakeGenericMethodArgsMismatch ()
+		{
+			MethodInfo gmi = this.GetType ().GetMethod (
+				"MakeGenericMethodArgsMismatchFoo")
+				.MakeGenericMethod ();
+		}
 #endif
 	}
 	
