@@ -555,7 +555,7 @@ public class EnumTest : TestCase
 	}
 	public void TestParse2() {
 		{
-			bool errorThrown = true;
+			bool errorThrown = false;
 			try {
 				String name = "huh?";
 				Enum.Parse(null, name, true);
@@ -566,7 +566,7 @@ public class EnumTest : TestCase
 			       errorThrown);
 		}
 		{
-			bool errorThrown = true;
+			bool errorThrown = false;
 			try {
 				TestingEnum x = TestingEnum.Test;
 				Enum.Parse(x.GetType(), null, true);
@@ -577,7 +577,7 @@ public class EnumTest : TestCase
 			       errorThrown);
 		}
 		{
-			bool errorThrown = true;
+			bool errorThrown = false;
 			try {
 				String bad = "huh?";
 				Enum.Parse(bad.GetType(), bad, true);
@@ -588,7 +588,7 @@ public class EnumTest : TestCase
 			       errorThrown);
 		}
 		{
-			bool errorThrown = true;
+			bool errorThrown = false;
 			try {
 				TestingEnum x = TestingEnum.Test;
 				String bad = "";
@@ -600,7 +600,7 @@ public class EnumTest : TestCase
 			       errorThrown);
 		}
 		{
-			bool errorThrown = true;
+			bool errorThrown = false;
 			try {
 				TestingEnum x = TestingEnum.Test;
 				String bad = " ";
@@ -612,7 +612,7 @@ public class EnumTest : TestCase
 			       errorThrown);
 		}
 		{
-			bool errorThrown = true;
+			bool errorThrown = false;
 			try {
 				String bad = "huh?";
 				TestingEnum x = TestingEnum.Test;
@@ -624,7 +624,7 @@ public class EnumTest : TestCase
 			       errorThrown);
 		}
 		{
-			bool errorThrown = true;
+			bool errorThrown = false;
 			try {
 				String bad = "test";
 				TestingEnum x = TestingEnum.Test;
@@ -841,5 +841,23 @@ public class EnumTest : TestCase
 			Assertion.Assert (nval + " is not contained in format return.", Array.IndexOf (fmtSpl, nval) >= 0);
 	}
 	// TODO - ToString with IFormatProviders
+
+        [Flags]
+        enum FlagsNegativeTestEnum {
+                None = 0,
+                One = 1,
+                Two = 2,
+                Negative = unchecked((int)0xFFFF0000)
+        }
+
+	[Test]
+	public void FlagTest3 () {
+                FlagsNegativeTestEnum t;
+
+                t = FlagsNegativeTestEnum.None;
+		Assertion.AssertEquals("#01", "None", t.ToString());
+                t = FlagsNegativeTestEnum.One;
+		Assertion.AssertEquals("#02", "One", t.ToString());
+	}
 }
 }
