@@ -47,11 +47,11 @@ namespace Mono.CSharp {
 		
 		// The emit context for the anonymous method
 		public EmitContext aec;
-		public InternalParameters amp;
+		public Parameters amp;
 		public string[] TypeParameters;
 		public Type[] TypeArguments;
 		protected bool unreachable;
-		
+
 		//
 		// The modifiers applied to the method, we aggregate them
 		//
@@ -79,22 +79,18 @@ namespace Mono.CSharp {
 			//
 			// The order is important: this setups the CaptureContext tree hierarchy.
 			//
+			if (container == null) {
+				Report.Error (1706, l, "Anonymous methods are not allowed in attribute declaration");
+				return;
+			}
 			container.SetHaveAnonymousMethods (l, this);
 			block.SetHaveAnonymousMethods (l, this);
 		}
 
 		protected AnonymousContainer (Parameters parameters, ToplevelBlock container,
-					      Location l)
+					      Location l):
+			this (parameters, container, new ToplevelBlock (container, parameters, l), l)
 		{
-			Parameters = parameters;
-			Block = new ToplevelBlock (container, Parameters, l);
-			loc = l;
-
-			//
-			// The order is important: this setups the CaptureContext tree hierarchy.
-			//
-			container.SetHaveAnonymousMethods (loc, this);
-			Block.SetHaveAnonymousMethods (loc, this);
 		}
 
 		public override Expression DoResolve (EmitContext ec)
@@ -119,6 +115,10 @@ namespace Mono.CSharp {
 
 		public abstract void CreateScopeType (EmitContext ec, ScopeInfo scope);
 
+		public abstract Iterator Iterator {
+			get;
+		}
+
 		public abstract bool IsIterator {
 			get;
 		}
@@ -132,6 +132,10 @@ namespace Mono.CSharp {
 		{
 		}
 
+		public override Iterator Iterator {
+			get { return null; }
+		}
+
 		public override bool IsIterator {
 			get { return false; }
 		}
@@ -160,11 +164,12 @@ namespace Mono.CSharp {
 			
 			if (type_host == null)
 				throw new Exception (String.Format ("Type host is null, Scope is {0}", Scope == null ? "null" : "Not null"));
-			
+
+			if (current_type != type_host)
+				method_modifiers = Modifiers.INTERNAL;
+
 			if (current_type == type_host && ec.IsStatic){
-				if (ec.IsStatic){
-					method_modifiers |= Modifiers.STATIC;
-				}
+				method_modifiers |= Modifiers.STATIC;
 				current_type = null;
 			} 
 
@@ -177,22 +182,22 @@ namespace Mono.CSharp {
 				foreach (string t in TypeParameters)
 					args.Add (new SimpleName (t, loc));
 
-				member_name = new MemberName (name, args);
+				member_name = new MemberName (name, args, loc);
 
 				generic_method = new GenericMethod (
 					ec.DeclSpace.NamespaceEntry,
-					(TypeContainer) ec.TypeContainer,
-					member_name, loc);
+					(TypeContainer) ec.TypeContainer, member_name,
+					new TypeExpression (invoke_mb.ReturnType, loc),
+					Parameters);
 
 				generic_method.SetParameterInfo (null);
 			} else
-				member_name = new MemberName (name);
+				member_name = new MemberName (name, loc);
 
 			method = new Method (
 				(TypeContainer) ec.TypeContainer, generic_method,
 				new TypeExpression (invoke_mb.ReturnType, loc),
-				method_modifiers, false, member_name,
-				Parameters, null, loc);
+				method_modifiers, false, member_name, Parameters, null);
 			method.Block = Block;
 
 			//
@@ -210,7 +215,7 @@ namespace Mono.CSharp {
 		void Error_ParameterMismatch (Type t)
 		{
 			Report.Error (1661, loc, "Anonymous method could not be converted to delegate `" +
-				      "{0}' since there is a parameter mismatch", t);
+				      "{0}' since there is a parameter mismatch", TypeManager.CSharpName (t));
 	}
 
 		//
@@ -228,60 +233,45 @@ namespace Mono.CSharp {
 			invoke_mb = (MethodInfo) invoke_mg.Methods [0];
 			ParameterData invoke_pd = TypeManager.GetParameterData (invoke_mb);
 
-			if (delegate_type.IsGenericInstance) {
-				TypeArguments = TypeManager.GetTypeArguments (delegate_type);
-
+			if (delegate_type.IsGenericType) {
 				Type def = delegate_type.GetGenericTypeDefinition ();
-				Type[] tparam = TypeManager.GetTypeArguments (def);
-				TypeParameters = new string [tparam.Length];
-				for (int i = 0; i < tparam.Length; i++)
-					TypeParameters [i] = tparam [i].Name;
+
+				if (def != delegate_type) {
+					Type[] tparam = TypeManager.GetTypeArguments (def);
+
+					TypeArguments = TypeManager.GetTypeArguments (delegate_type);
+					TypeParameters = new string [tparam.Length];
+					for (int i = 0; i < tparam.Length; i++)
+						TypeParameters [i] = tparam [i].Name;
+				}
 			}
 
-			if (Parameters == null){
-				int i, j;
-				
+			if (Parameters == null){				
 				//
 				// We provide a set of inaccessible parameters
 				//
-				int params_idx = -1;
-				for (i = 0; i < invoke_pd.Count; i++){
-					if (invoke_pd.ParameterModifier (i) == Parameter.Modifier.PARAMS)
-						params_idx = i;
-				}
-				int n = invoke_pd.Count - (params_idx != -1 ? 1 : 0);
-				Parameter [] fixedpars = new Parameter [n];
+				Parameter [] fixedpars = new Parameter [invoke_pd.Count];
 				
-				for (i =  j = 0; i < invoke_pd.Count; i++){
-					if (invoke_pd.ParameterModifier (i) == Parameter.Modifier.PARAMS)
-						continue;
-					fixedpars [j] = new Parameter (
-						new TypeExpression (invoke_pd.ParameterType (i), loc),
-						"+" + j, invoke_pd.ParameterModifier (i), null);
-					j++;
+				for (int i = 0; i < invoke_pd.Count; i++){
+					fixedpars [i] = new Parameter (
+						invoke_pd.ParameterType (i),
+						"+" + i, invoke_pd.ParameterModifier (i), null, loc);
 				}
 				
-				Parameter variable = null;
-				if (params_idx != -1){
-					variable = new Parameter (
-						new TypeExpression (invoke_pd.ParameterType (params_idx), loc),
-						"+" + params_idx, invoke_pd.ParameterModifier (params_idx), null);
-				}
-
-				Parameters = new Parameters (fixedpars, variable, loc);
+				Parameters = new Parameters (fixedpars);
 			}
 			
 			//
 			// First, parameter types of `delegate_type' must be compatible
 			// with the anonymous method.
 			//
-			amp = new InternalParameters (Parameters.GetParameterInfo (ec), Parameters);
+			Parameters.Resolve (ec);
+			amp = Parameters;
 			
 			if (amp.Count != invoke_pd.Count){
 				if (!probe){
-					Report.Error (1593, loc, 
-					      "Anonymous method has {0} parameters, while delegate requires {1}",
-					      amp.Count, invoke_pd.Count);
+					Report.Error (1593, loc, "Delegate `{0}' does not take `{1}' arguments",
+						TypeManager.CSharpName (delegate_type), amp.Count.ToString ());
 					Error_ParameterMismatch (delegate_type);
 				}
 				return null;
@@ -290,33 +280,29 @@ namespace Mono.CSharp {
 			for (int i = 0; i < amp.Count; i++){
 				Parameter.Modifier amp_mod = amp.ParameterModifier (i);
 
-				if ((amp_mod & (Parameter.Modifier.OUT | Parameter.Modifier.REF)) != 0){
-					if (!probe){
+				if (!probe) {
+					if ((amp_mod & (Parameter.Modifier.OUT | Parameter.Modifier.REF)) != 0){
+						Report.Error (1677, loc, "Parameter `{0}' should not be declared with the `{1}' keyword", 
+							(i+1).ToString (), Parameter.GetModifierSignature (amp_mod));
 						Error_ParameterMismatch (delegate_type);
-						Report.Error (1677, loc, "Parameter '{0}' should not be declared with the '{1}' keyword", 
-							i+1, amp.ModifierDesc (i));
+						return null;
 					}
-					return null;
-				}
 
-				if (amp_mod != invoke_pd.ParameterModifier (i)){
-					if (!probe){
-						Report.Error (1676, loc, 
-						      "Signature mismatch in parameter modifier for parameter #0", i + 1);
+					if (amp_mod != invoke_pd.ParameterModifier (i)){
+						Report.Error (1676, loc, "Parameter `{0}' must be declared with the `{1}' keyword",
+							(i+1).ToString (), Parameter.GetModifierSignature (invoke_pd.ParameterModifier (i)));
 						Error_ParameterMismatch (delegate_type);
+						return null;
 					}
-					return null;
-				}
 				
-				if (amp.ParameterType (i) != invoke_pd.ParameterType (i)){
-					if (!probe){
-						Report.Error (1678, loc, 
-								      "Signature mismatch in parameter {0}: need `{1}' got `{2}'", i + 1,
-								      TypeManager.CSharpName (invoke_pd.ParameterType (i)),
-								      TypeManager.CSharpName (amp.ParameterType (i)));
+					if (amp.ParameterType (i) != invoke_pd.ParameterType (i)){
+						Report.Error (1678, loc, "Parameter `{0}' is declared as type `{1}' but should be `{2}'",
+							(i+1).ToString (),
+							TypeManager.CSharpName (amp.ParameterType (i)),
+							TypeManager.CSharpName (invoke_pd.ParameterType (i)));
 						Error_ParameterMismatch (delegate_type);
+						return null;
 					}
-					return null;
 				}
 			}
 
@@ -349,20 +335,32 @@ namespace Mono.CSharp {
 			ContainerAnonymousMethod = ec.CurrentAnonymousMethod;
 			ContainingBlock = ec.CurrentBlock;
 
-			if (aec.ResolveTopBlock (ec, Block, amp, loc, out unreachable))
+			if (aec.ResolveTopBlock (ec, Block, amp, null, out unreachable))
 				return new AnonymousDelegate (this, delegate_type, loc).Resolve (ec);
 
 			return null;
 		}
 
+		public override string ExprClassName {
+			get {
+				return "anonymous method";
+			}
+		}
+
 		public MethodInfo GetMethodBuilder ()
 		{
 			MethodInfo builder = method.MethodData.MethodBuilder;
 			if (TypeArguments != null)
-				return builder.BindGenericParameters (TypeArguments);
+				return builder.MakeGenericMethod (TypeArguments);
 			else
 				return builder;
 		}
+
+		public override string GetSignatureForError ()
+		{
+			string s = TypeManager.CSharpSignature (invoke_mb);
+			return s.Substring (0, s.IndexOf (".Invoke("));
+		}
 		
 		public bool EmitMethod (EmitContext ec)
 		{
@@ -373,7 +371,7 @@ namespace Mono.CSharp {
 			ILGenerator ig = builder.GetILGenerator ();
 			aec.ig = ig;
 
-			Parameters.LabelParameters (aec, builder, loc);
+			Parameters.ApplyAttributes (aec, builder);
 
 			//
 			// Adjust based on the computed state of the
@@ -381,7 +379,7 @@ namespace Mono.CSharp {
 			
 			aec.MethodIsStatic = (method_modifiers & Modifiers.STATIC) != 0;
 			
-			aec.EmitMeta (Block, amp);
+			aec.EmitMeta (Block);
 			aec.EmitResolvedTopBlock (Block, unreachable);
 			return true;
 		}
@@ -396,24 +394,47 @@ namespace Mono.CSharp {
 				TypeAttributes.NestedAssembly, TypeManager.object_type, null);
 
 			Type [] constructor_types = TypeManager.NoTypes;
-			Parameters constructor_parameters = Parameters.EmptyReadOnlyParameters;
-			scope.ScopeConstructor = scope.ScopeTypeBuilder.DefineConstructor (
+			ConstructorBuilder ctor = scope.ScopeTypeBuilder.DefineConstructor (
 				MethodAttributes.Public | MethodAttributes.HideBySig |
 				MethodAttributes.SpecialName | MethodAttributes.RTSpecialName,
 				CallingConventions.HasThis, constructor_types);
-			InternalParameters parameter_info = new InternalParameters (constructor_types, constructor_parameters);
-			TypeManager.RegisterMethod (scope.ScopeConstructor, parameter_info, constructor_types);
+			TypeManager.RegisterMethod (ctor, Parameters.EmptyReadOnlyParameters);
 
-			ILGenerator cig = scope.ScopeConstructor.GetILGenerator ();
+			ILGenerator cig = ctor.GetILGenerator ();
 			cig.Emit (OpCodes.Ldarg_0);
 			cig.Emit (OpCodes.Call, TypeManager.object_ctor);
 			cig.Emit (OpCodes.Ret);
+
+			if (ec.TypeContainer.IsGeneric) {
+				TypeParameter[] tparam = ec.TypeContainer.TypeParameters;
+				string[] names = new string [tparam.Length];
+				Type[] types = new Type [tparam.Length];
+
+				for (int i = 0; i < names.Length; i++) {
+					names [i] = tparam [i].Name;
+					types [i] = tparam [i].Type;
+				}
+
+				scope.ScopeTypeBuilder.DefineGenericParameters (names);
+				scope.ScopeTypeBuilder.GetGenericTypeDefinition ();
+
+				scope.ScopeType = scope.ScopeTypeBuilder.MakeGenericType (types);
+			} else
+				scope.ScopeType = scope.ScopeTypeBuilder;
+
+
+			if (ec.TypeContainer.IsGeneric)
+				scope.ScopeConstructor = TypeBuilder.GetConstructor (
+					scope.ScopeType, ctor);
+			else
+				scope.ScopeConstructor = ctor;
 		}
 
 		public static void Error_AddressOfCapturedVar (string name, Location loc)
 		{
 			Report.Error (1686, loc,
-				      "Variable {0} is captured in an anonymous method and its address is also being taken: they are exclusive", name);
+				"Local variable `{0}' or its members cannot have their address taken and be used inside an anonymous method block",
+				name);
 		}
 	}
 
@@ -510,7 +531,8 @@ namespace Mono.CSharp {
 		// The types and fields generated
 		//
 		public TypeBuilder ScopeTypeBuilder;
-		public ConstructorBuilder ScopeConstructor;
+		public Type ScopeType;
+		public ConstructorInfo ScopeConstructor;
 		public FieldBuilder THIS;
 		public FieldBuilder ParentLink;
 
@@ -588,7 +610,7 @@ namespace Mono.CSharp {
 			Console.WriteLine ("NeedThis=" + NeedThis);
 			foreach (LocalInfo li in locals){
 				Pad ();
-				Console.WriteLine ("var {0}", li.Name);
+				Console.WriteLine ("var {0}", MakeFieldName (li.Name));
 			}
 			
 			foreach (ScopeInfo si in children)
@@ -603,6 +625,11 @@ namespace Mono.CSharp {
 			return String.Format ("<>AnonHelp<{0}>", id);
 		}
 
+		private string MakeFieldName (string local_name)
+		{
+			return "<" + id + ":" + local_name + ">";
+		}
+
 		public void EmitScopeType (EmitContext ec)
 		{
 			// EmitDebug ();
@@ -623,8 +650,8 @@ namespace Mono.CSharp {
 				}
 
 				if (ParentScope.ScopeTypeBuilder != ScopeTypeBuilder)
-					ParentLink = ScopeTypeBuilder.DefineField ("<>parent", ParentScope.ScopeTypeBuilder,
-										   FieldAttributes.Assembly);
+					ParentLink = ScopeTypeBuilder.DefineField (
+						"<>parent", ParentScope.ScopeType, FieldAttributes.Assembly);
 			}
 			
 			if (NeedThis && ParentScope != null)
@@ -632,7 +659,7 @@ namespace Mono.CSharp {
 
 			foreach (LocalInfo info in locals)
 				info.FieldBuilder = ScopeTypeBuilder.DefineField (
-					info.Name, info.VariableType, FieldAttributes.Assembly);
+					MakeFieldName (info.Name), info.VariableType, FieldAttributes.Assembly);
 
 			if (HostsParameters){
 				Hashtable captured_parameters = CaptureContext.captured_parameters;
@@ -673,8 +700,8 @@ namespace Mono.CSharp {
 				throw new Exception ("ScopeConstructor is null for" + this.ToString ());
 			
 			if (!CaptureContext.Host.IsIterator) {
-				scope_instance = ig.DeclareLocal (ScopeTypeBuilder);
-				ig.Emit (OpCodes.Newobj, (ConstructorInfo) ScopeConstructor);
+				scope_instance = ig.DeclareLocal (ScopeType);
+				ig.Emit (OpCodes.Newobj, ScopeConstructor);
 				ig.Emit (OpCodes.Stloc, scope_instance);
 			}
 
@@ -998,9 +1025,8 @@ namespace Mono.CSharp {
 		{
 			if (captured_parameters == null)
 				captured_parameters = new Hashtable ();
-			if (captured_parameters [name] != null)
-				return;
-			captured_parameters [name] = new CapturedParameter (t, idx);
+			if (captured_parameters [name] == null)
+				captured_parameters [name] = new CapturedParameter (t, idx);
 
 			if (topmost == null){
 				//
@@ -1034,24 +1060,45 @@ namespace Mono.CSharp {
 		// Captured fields are only recorded on the topmost CaptureContext, because that
 		// one is the one linked to the owner of instance fields
 		//
-		public void AddField (FieldExpr fe)
+		public void AddField (EmitContext ec, AnonymousContainer am, FieldExpr fe)
 		{
 			if (fe.FieldInfo.IsStatic)
 				throw new Exception ("Attempt to register a static field as a captured field");
-			
 			CaptureContext parent = ParentCaptureContext;
-			if (parent != null)
-				parent.AddField (fe);
-			else
-				captured_fields [fe] = fe;
+			if (parent != null) {
+				parent.AddField (ec, am, fe);
+				return;
+			}
+
+			if (topmost == null){
+				//
+				// Create one ScopeInfo, if there are none.
+				//
+				topmost = new ScopeInfo (this, toplevel_owner);
+				scopes [toplevel_owner.ID] = topmost;
+			}
+			
+			AdjustMethodScope (am, topmost);
 		}
 
-		public void CaptureThis ()
+		public void CaptureThis (AnonymousContainer am)
 		{
 			CaptureContext parent = ParentCaptureContext;
-			if (parent != null)
-				parent.CaptureThis ();
+			if (parent != null) {
+				parent.CaptureThis (am);
+				return;
+			}
 			referenced_this = true;
+
+			if (topmost == null){
+				//
+				// Create one ScopeInfo, if there are none.
+				//
+				topmost = new ScopeInfo (this, toplevel_owner);
+				scopes [toplevel_owner.ID] = topmost;
+			
+				AdjustMethodScope (am, topmost);
+			}
 		}
 
 		public bool HaveCapturedVariables {
@@ -1177,7 +1224,7 @@ namespace Mono.CSharp {
 				cc.EmitParameterInstance (ec, name);
 				return;
 			}
-			
+
 			CapturedParameter par_info = (CapturedParameter) captured_parameters [name];
 			if (par_info != null){
 				// 
@@ -1188,14 +1235,14 @@ namespace Mono.CSharp {
 
 			ScopeInfo si;
 
-			if (ec.CurrentBlock.Toplevel == toplevel_owner){
+			if (ec.CurrentBlock.Toplevel == toplevel_owner) {
 				si = GetScopeFromBlock (ec, toplevel_owner);
 				si.EmitScopeInstance (ig);
-				return;
+			} else {
+				si = ec.CurrentAnonymousMethod.Scope;
+				ig.Emit (OpCodes.Ldarg_0);
 			}
 
-			si = ec.CurrentAnonymousMethod.Scope;
-			ig.Emit (OpCodes.Ldarg_0);
 			if (si != null){
 				while (si.ParentLink != null) {
 					ig.Emit (OpCodes.Ldfld, si.ParentLink);
@@ -1208,14 +1255,15 @@ namespace Mono.CSharp {
 		// Emits the code necessary to load the parameter named `name' within
 		// an anonymous method.
 		//
-		public void EmitParameter (EmitContext ec, string name)
+		public void EmitParameter (EmitContext ec, string name, bool leave_copy, bool prepared, ref LocalTemporary temp)
 		{
 			CaptureContext cc = ContextForParameter (ec.CurrentBlock.Toplevel, name);
 			if (cc != this){
-				cc.EmitParameter (ec, name);
+				cc.EmitParameter (ec, name, leave_copy, prepared, ref temp);
 				return;
 			}
-			EmitParameterInstance (ec, name);
+			if (!prepared)
+				EmitParameterInstance (ec, name);
 			CapturedParameter par_info = (CapturedParameter) captured_parameters [name];
 			if (par_info != null){
 				// 
@@ -1223,27 +1271,40 @@ namespace Mono.CSharp {
 				//
 			}
 			ec.ig.Emit (OpCodes.Ldfld, par_info.FieldBuilder);
+
+			if (leave_copy){
+				ec.ig.Emit (OpCodes.Dup);
+				temp = new LocalTemporary (ec, par_info.FieldBuilder.FieldType);
+				temp.Store (ec);
+			}
 		}
 
 		//
 		// Implements the assignment of `source' to the paramenter named `name' within
 		// an anonymous method.
 		//
-		public void EmitAssignParameter (EmitContext ec, string name, Expression source, bool leave_copy, bool prepare_for_load)
+		public void EmitAssignParameter (EmitContext ec, string name, Expression source, bool leave_copy, bool prepare_for_load, ref LocalTemporary temp)
 		{
 			CaptureContext cc = ContextForParameter (ec.CurrentBlock.Toplevel, name);
 			if (cc != this){
-				cc.EmitAssignParameter (ec, name, source, leave_copy, prepare_for_load);
+				cc.EmitAssignParameter (ec, name, source, leave_copy, prepare_for_load, ref temp);
 				return;
 			}
 			ILGenerator ig = ec.ig;
 			CapturedParameter par_info = (CapturedParameter) captured_parameters [name];
 
 			EmitParameterInstance (ec, name);
-			source.Emit (ec);
-			if (leave_copy)
+			if (prepare_for_load)
 				ig.Emit (OpCodes.Dup);
+			source.Emit (ec);
+			if (leave_copy){
+				ig.Emit (OpCodes.Dup);
+				temp = new LocalTemporary (ec, par_info.FieldBuilder.FieldType);
+				temp.Store (ec);
+			}
 			ig.Emit (OpCodes.Stfld, par_info.FieldBuilder);
+			if (temp != null)
+				temp.Emit (ec);
 		}
 
 		//
@@ -1271,7 +1332,9 @@ namespace Mono.CSharp {
 			ILGenerator ig = target.ig;
 			ScopeInfo si = am.Scope;
 
-			if (si == null){
+			AnonymousContainer container = am.ContainerAnonymousMethod;
+
+			if ((si == null) || ((container != null) && (si == container.Scope))) {
 				ig.Emit (OpCodes.Ldarg_0);
 				return;
 			}
