@@ -7,7 +7,11 @@
 // (C) 2001, 2002, 2003 Ximian, Inc.
 // (C) 2004 Novell, Inc.
 //
-//#define PRODUCTION
+
+#if !DEBUG
+	#define PRODUCTION
+#endif
+
 using System;
 using System.IO;
 using System.Collections;
@@ -86,7 +90,7 @@ namespace Mono.CSharp {
 			//
 			if (SymbolWriter == null) {
 				Report.Warning (
-					-18, "Could not find the symbol writer assembly (Mono.CompilerServices.SymbolWriter.dll). This is normally an installation problem. Please make sure to compile and install the mcs/class/Mono.CompilerServices.SymbolWriter directory.");
+					-18, 1, "Could not find the symbol writer assembly (Mono.CompilerServices.SymbolWriter.dll). This is normally an installation problem. Please make sure to compile and install the mcs/class/Mono.CompilerServices.SymbolWriter directory.");
 				return;
 			}
 		}
@@ -104,11 +108,11 @@ namespace Mono.CSharp {
 			if (an.KeyPair != null) {
 				// If we are going to strong name our assembly make
 				// sure all its refs are strong named
-				foreach (Assembly a in TypeManager.GetAssemblies ()) {
+				foreach (Assembly a in RootNamespace.Global.Assemblies) {
 					AssemblyName ref_name = a.GetName ();
 					byte [] b = ref_name.GetPublicKeyToken ();
 					if (b == null || b.Length == 0) {
-						Report.Warning (1577, "Assembly generation failed " +
+						Report.Error (1577, "Assembly generation failed " +
 								"-- Referenced assembly '" +
 								ref_name.Name +
 								"' does not have a strong name.");
@@ -140,6 +144,10 @@ namespace Mono.CSharp {
 				return false;
 			}
 
+			// Get the complete AssemblyName from the builder
+			// (We need to get the public key and token)
+			Assembly.Name = Assembly.Builder.GetName ();
+			
 			//
 			// Pass a path-less name to DefineDynamicModule.  Wonder how
 			// this copes with output in different directories then.
@@ -237,6 +245,11 @@ namespace Mono.CSharp {
 		/// </summary>
 		public bool ResolvingTypeTree;
 
+		/// <summary>
+		///   We are resolving a generic method's return type and parameters.
+		/// </summary>
+		public bool ResolvingGenericMethod;
+
 		/// <summary>
 		///   The value that is allowed to be returned or NULL if there is no
 		///   return type.
@@ -259,6 +272,11 @@ namespace Mono.CSharp {
 		/// </summary>
 		public bool DoFlowAnalysis;
 
+		/// <summary>
+		///   Whether we're control flow analysis disabled on struct
+		/// </summary>
+		public bool OmitStructFlowAnalysis;
+
 		/// <summary>
 		///   Keeps track of the Type to LocalBuilder temporary storage created
 		///   to store structures (used to compute the address of the structure
@@ -344,7 +362,14 @@ namespace Mono.CSharp {
 		/// <summary>
 		///    The current iterator
 		/// </summary>
-		public Iterator CurrentIterator;
+		public Iterator CurrentIterator {
+			get {
+				if (CurrentAnonymousMethod != null)
+					return CurrentAnonymousMethod.Iterator;
+				else
+					return null;
+			}
+		}
 
 		/// <summary>
 		///    Whether we are in the resolving stage or not
@@ -510,7 +535,7 @@ namespace Mono.CSharp {
 
 		public void CaptureThis ()
 		{
-			capture_context.CaptureThis ();
+			capture_context.CaptureThis (CurrentAnonymousMethod);
 		}
 		
 		
@@ -519,7 +544,7 @@ namespace Mono.CSharp {
 		//
 		public void CaptureField (FieldExpr fe)
 		{
-			capture_context.AddField (fe);
+			capture_context.AddField (this, CurrentAnonymousMethod, fe);
 		}
 
 		//
@@ -571,7 +596,7 @@ namespace Mono.CSharp {
 			return false;
 		}
 		
-		public void EmitMeta (ToplevelBlock b, InternalParameters ip)
+		public void EmitMeta (ToplevelBlock b)
 		{
 			if (capture_context != null)
 				capture_context.EmitAnonymousHelperClasses (this);
@@ -586,15 +611,15 @@ namespace Mono.CSharp {
 		// currently can not cope with ig == null during resolve (which must
 		// be fixed for switch statements to work on anonymous methods).
 		//
-		public void EmitTopBlock (ToplevelBlock block, InternalParameters ip, Location loc)
+		public void EmitTopBlock (IMethodData md, ToplevelBlock block)
 		{
 			if (block == null)
 				return;
 			
 			bool unreachable;
 			
-			if (ResolveTopBlock (null, block, ip, loc, out unreachable)){
-				EmitMeta (block, ip);
+			if (ResolveTopBlock (null, block, md.ParameterInfo, md, out unreachable)){
+				EmitMeta (block);
 
 				current_phase = Phase.Emitting;
 				EmitResolvedTopBlock (block, unreachable);
@@ -604,7 +629,7 @@ namespace Mono.CSharp {
 		bool resolved;
 
 		public bool ResolveTopBlock (EmitContext anonymous_method_host, ToplevelBlock block,
-					     InternalParameters ip, Location loc, out bool unreachable)
+					     Parameters ip, IMethodData md, out bool unreachable)
 		{
 			current_phase = Phase.Resolving;
 			
@@ -615,7 +640,7 @@ namespace Mono.CSharp {
 
 			capture_context = block.CaptureContext;
 
-			if (!Location.IsNull (loc))
+			if (!loc.IsNull)
 				CurrentFile = loc.File;
 
 #if PRODUCTION
@@ -665,13 +690,11 @@ namespace Mono.CSharp {
 
 			if (ReturnType != null && !unreachable) {
 				if (CurrentAnonymousMethod == null) {
-					Report.Error (161, loc, "Not all code paths return a value");
+					Report.Error (161, md.Location, "`{0}': not all code paths return a value", md.GetSignatureForError ());
 					return false;
 				} else if (!CurrentAnonymousMethod.IsIterator) {
-					Report.Error (
-						1643, loc, "Not all code paths return a " +
-						"value in anonymous method of type `{0}'",
-						CurrentAnonymousMethod.Type);
+					Report.Error (1643, CurrentAnonymousMethod.Location, "Not all code paths return a value in anonymous method of type `{0}'",
+						CurrentAnonymousMethod.GetSignatureForError ());
 					return false;
 				}
 			}
@@ -731,13 +754,13 @@ namespace Mono.CSharp {
 		/// </summary>
 		public void Mark (Location loc, bool check_file)
 		{
-			if ((CodeGen.SymbolWriter == null) || Location.IsNull (loc))
+			if ((CodeGen.SymbolWriter == null) || loc.IsNull)
 				return;
 
 			if (check_file && (CurrentFile != loc.File))
 				return;
 
-			CodeGen.SymbolWriter.MarkSequencePoint (ig, loc.Row, 0);
+			CodeGen.SymbolWriter.MarkSequencePoint (ig, loc.Row, loc.Column);
 		}
 
 		public void DefineLocalVariable (string name, LocalBuilder builder)
@@ -916,14 +939,14 @@ namespace Mono.CSharp {
 			capture_context.EmitCapturedVariableInstance (this, li, CurrentAnonymousMethod);
 		}
 
-		public void EmitParameter (string name)
+		public void EmitParameter (string name, bool leave_copy, bool prepared, ref LocalTemporary temp)
 		{
-			capture_context.EmitParameter (this, name);
+			capture_context.EmitParameter (this, name, leave_copy, prepared, ref temp);
 		}
 
-		public void EmitAssignParameter (string name, Expression source, bool leave_copy, bool prepare_for_load)
+		public void EmitAssignParameter (string name, Expression source, bool leave_copy, bool prepare_for_load, ref LocalTemporary  temp)
 		{
-			capture_context.EmitAssignParameter (this, name, source, leave_copy, prepare_for_load);
+			capture_context.EmitAssignParameter (this, name, source, leave_copy, prepare_for_load, ref temp);
 		}
 
 		public void EmitAddressOfParameter (string name)
@@ -947,7 +970,7 @@ namespace Mono.CSharp {
 	}
 
 
-	public abstract class CommonAssemblyModulClass: Attributable {
+	public abstract class CommonAssemblyModulClass : Attributable {
 		protected CommonAssemblyModulClass ():
 			base (null)
 		{
@@ -989,20 +1012,24 @@ namespace Mono.CSharp {
 		}
 	}
 
-	public class AssemblyClass: CommonAssemblyModulClass {
+	public class AssemblyClass : CommonAssemblyModulClass {
 		// TODO: make it private and move all builder based methods here
 		public AssemblyBuilder Builder;
+		public AssemblyName Name;
                     
 		bool is_cls_compliant;
+		bool wrap_non_exception_throws;
+
 		public Attribute ClsCompliantAttribute;
 
 		ListDictionary declarative_security;
 
-		static string[] attribute_targets = new string [] { "assembly" };
+		// Module is here just because of error messages
+		static string[] attribute_targets = new string [] { "assembly", "module" };
 
 		public AssemblyClass (): base ()
 		{
-			is_cls_compliant = false;
+			wrap_non_exception_throws = true;
 		}
 
 		public bool IsClsCompliant {
@@ -1011,6 +1038,12 @@ namespace Mono.CSharp {
 			}
 			}
 
+		public bool WrapNonExceptionThrows {
+			get {
+				return wrap_non_exception_throws;
+			}
+		}
+
 		public override AttributeTargets AttributeTargets {
 			get {
 				return AttributeTargets.Assembly;
@@ -1022,13 +1055,21 @@ namespace Mono.CSharp {
 			return is_cls_compliant;
 		}
 
-		public void ResolveClsCompliance ()
+		public void Resolve ()
 		{
 			ClsCompliantAttribute = ResolveAttribute (TypeManager.cls_compliant_attribute_type);
-			if (ClsCompliantAttribute == null)
-				return;
+			if (ClsCompliantAttribute != null) {
+				is_cls_compliant = ClsCompliantAttribute.GetClsCompliantAttributeValue (null);
+			}
 
-			is_cls_compliant = ClsCompliantAttribute.GetClsCompliantAttributeValue (null);
+#if NET_2_0
+			Attribute a = ResolveAttribute (TypeManager.runtime_compatibility_attr_type);
+			if (a != null) {
+				object val = a.GetPropertyValue ("WrapNonExceptionThrows");
+				if (val != null)
+					wrap_non_exception_throws = (bool)val;
+			}
+#endif
 		}
 
 		// fix bug #56621
@@ -1056,8 +1097,7 @@ namespace Mono.CSharp {
 				}
 			}
 			catch (Exception) {
-				Report.Error (1548, "Could not strongname the assembly. File `" +
-					RootContext.StrongNameKeyFile + "' incorrectly encoded.");
+				Error_AssemblySigning ("The specified file `" + RootContext.StrongNameKeyFile + "' is incorrectly encoded");
 				Environment.Exit (1);
 			}
 		}
@@ -1071,16 +1111,18 @@ namespace Mono.CSharp {
 				       if (a.ExplicitTarget == null || a.ExplicitTarget != "assembly")
 					       continue;
 
-                                       // TODO: This code is buggy: comparing Attribute name without resolving it is wrong.
-                                       //       However, this is invoked by CodeGen.Init, at which time none of the namespaces
-                                       //       are loaded yet.
+					// TODO: This code is buggy: comparing Attribute name without resolving is wrong.
+					//       However, this is invoked by CodeGen.Init, when none of the namespaces
+					//       are loaded yet.
+					// TODO: Does not handle quoted attributes properly
 					switch (a.Name) {
 						case "AssemblyKeyFile":
                                                case "AssemblyKeyFileAttribute":
                                                case "System.Reflection.AssemblyKeyFileAttribute":
 							if (RootContext.StrongNameKeyFile != null) {
 								Report.SymbolRelatedToPreviousError (a.Location, a.Name);
-								Report.Warning (1616, "Compiler option '{0}' overrides '{1}' given in source", "keyfile", "System.Reflection.AssemblyKeyFileAttribute");
+								Report.Warning (1616, 1, "Option `{0}' overrides attribute `{1}' given in a source file or added module",
+                                    "keyfile", "System.Reflection.AssemblyKeyFileAttribute");
 							}
 							else {
 								string value = a.GetString ();
@@ -1093,7 +1135,8 @@ namespace Mono.CSharp {
                                                case "System.Reflection.AssemblyKeyNameAttribute":
 							if (RootContext.StrongNameKeyContainer != null) {
 								Report.SymbolRelatedToPreviousError (a.Location, a.Name);
-								Report.Warning (1616, "keycontainer", "Compiler option '{0}' overrides '{1}' given in source", "System.Reflection.AssemblyKeyNameAttribute");
+								Report.Warning (1616, 1, "Option `{0}' overrides attribute `{1}' given in a source file or added module",
+									"keycontainer", "System.Reflection.AssemblyKeyNameAttribute");
 							}
 							else {
 								string value = a.GetString ();
@@ -1156,13 +1199,11 @@ namespace Mono.CSharp {
 						catch (CryptographicException) {
 							if (snkeypair.Length == 16) {
 								// error # is different for ECMA key
-								Report.Error (1606, "Could not strongname the assembly. " + 
+								Report.Error (1606, "Could not sign the assembly. " + 
 									"ECMA key can only be used to delay-sign assemblies");
 							}
 							else {
-								Report.Error (1548, "Could not strongname the assembly. File `" +
-									RootContext.StrongNameKeyFile +
-									"' doesn't have a private key.");
+								Error_AssemblySigning ("The specified file `" + RootContext.StrongNameKeyFile + "' does not have a private key");
 							}
 							return null;
 						}
@@ -1170,13 +1211,46 @@ namespace Mono.CSharp {
 				}
 			}
 			else {
-				Report.Error (1548, "Could not strongname the assembly. File `" +
-					RootContext.StrongNameKeyFile + "' not found.");
+				Error_AssemblySigning ("The specified file `" + RootContext.StrongNameKeyFile + "' does not exist");
 				return null;
 			}
 			return an;
 		}
 
+		void Error_AssemblySigning (string text)
+		{
+			Report.Error (1548, "Error during assembly signing. " + text);
+		}
+
+		bool CheckInternalsVisibleAttribute (Attribute a)
+		{
+			string assembly_name = a.GetString ();
+			if (assembly_name.Length == 0)
+				return false;
+				
+			AssemblyName aname = null;
+			try {
+				aname = new AssemblyName (assembly_name);
+			} catch (FileLoadException) {
+			} catch (ArgumentException) {
+			}
+				
+			// Bad assembly name format
+			if (aname == null)
+				Report.Warning (1700, 3, a.Location, "Assembly reference `" + assembly_name + "' is invalid and cannot be resolved");
+			// Report error if we have defined Version or Culture
+			else if (aname.Version != null || aname.CultureInfo != null)
+				throw new Exception ("Friend assembly `" + a.GetString () + 
+						"' is invalid. InternalsVisibleTo cannot have version or culture specified.");
+			else if (aname.GetPublicKeyToken () == null && Name.GetPublicKeyToken () != null) {
+				Report.Error (1726, a.Location, "Friend assembly reference `" + aname.FullName + "' is invalid." +
+						" Strong named assemblies must specify a public key token in their InternalsVisibleTo declarations");
+				return false;
+			}
+
+			return true;
+		}
+
 		public override void ApplyAttributeBuilder (Attribute a, CustomAttributeBuilder customBuilder)
 		{
 			if (a.Type.IsSubclassOf (TypeManager.security_attr_type) && a.CheckSecurityActionValidity (true)) {
@@ -1187,6 +1261,20 @@ namespace Mono.CSharp {
 				return;
 			}
 
+			if (a.Type == TypeManager.assembly_culture_attribute_type) {
+				string value = a.GetString ();
+				if (value == null || value.Length == 0)
+					return;
+
+				if (RootContext.Target == Target.Exe) {
+					a.Error_AttributeEmitError ("The executables cannot be satelite assemblies, remove the attribute or keep it empty");
+					return;
+				}
+			}
+
+			if (a.Type == TypeManager.internals_visible_attr_type && !CheckInternalsVisibleAttribute (a))
+					return;
+			
 			Builder.SetCustomAttribute (customBuilder);
 		}
 
@@ -1218,6 +1306,19 @@ namespace Mono.CSharp {
 					Report.RuntimeMissingSupport (Location.Null, "assembly permission setting");
 				}
 			}
+
+			// FIXME: Does this belong inside SRE.AssemblyBuilder instead?
+			if (OptAttributes == null || !OptAttributes.Contains (TypeManager.runtime_compatibility_attr_type, null)) {
+				if (RootContext.StdLib) {
+					ConstructorInfo ci = TypeManager.runtime_compatibility_attr_type.GetConstructor (TypeManager.NoTypes);
+					PropertyInfo pi = TypeManager.runtime_compatibility_attr_type.GetProperty ("WrapNonExceptionThrows");
+					Builder.SetCustomAttribute (new CustomAttributeBuilder (ci, new object [0], 
+						new PropertyInfo [] { pi }, new object[] { true } ));
+				} else {
+					// FIXME: Do something appropriate for 'mscorlib'
+					Report.Warning (-31, 1, "FIXME: Did not emit 'RuntimeComatibilityAttribute' for mscorlib");
+				}
+			}
 		}
 
 		public override string[] ValidAttributeTargets {
@@ -1227,7 +1328,7 @@ namespace Mono.CSharp {
 		}
 	}
 
-	public class ModuleClass: CommonAssemblyModulClass {
+	public class ModuleClass : CommonAssemblyModulClass {
 		// TODO: make it private and move all builder based methods here
 		public ModuleBuilder Builder;
 		bool m_module_is_unsafe;
@@ -1272,10 +1373,10 @@ namespace Mono.CSharp {
 		{
 			if (a.Type == TypeManager.cls_compliant_attribute_type) {
 				if (CodeGen.Assembly.ClsCompliantAttribute == null) {
-					Report.Warning (3012, a.Location, "You must specify the CLSCompliant attribute on the assembly, not the module, to enable CLS compliance checking");
+					Report.Warning (3012, 1, a.Location, "You must specify the CLSCompliant attribute on the assembly, not the module, to enable CLS compliance checking");
 				}
 				else if (CodeGen.Assembly.IsClsCompliant != a.GetBoolean ()) {
-					Report.SymbolRelatedToPreviousError (CodeGen.Assembly.ClsCompliantAttribute.Location, CodeGen.Assembly.ClsCompliantAttribute.Name);
+					Report.SymbolRelatedToPreviousError (CodeGen.Assembly.ClsCompliantAttribute.Location, CodeGen.Assembly.ClsCompliantAttribute.GetSignatureForError ());
 					Report.Error (3017, a.Location, "You cannot specify the CLSCompliant attribute on a module that differs from the CLSCompliant attribute on the assembly");
 					return;
 				}
