@@ -32,17 +32,21 @@ namespace Mono.CSharp
 	public class Tokenizer : yyParser.yyInput
 	{
 		SeekableStreamReader reader;
-		public SourceFile ref_name;
-		public SourceFile file_name;
-		public int ref_line = 1;
-		public int line = 1;
-		public int col = 1;
-		public int current_token;
+		SourceFile ref_name;
+		SourceFile file_name;
+		int ref_line = 1;
+		int line = 1;
+		int col = 0;
+		int previous_col;
+		int current_token;
 		bool handle_get_set = false;
 		bool handle_remove_add = false;
 		bool handle_assembly = false;
 		bool handle_constraints = false;
 		bool handle_typeof = false;
+		Location current_location;
+		Location current_comment_location = Location.Null;
+		ArrayList escapedIdentifiers = new ArrayList ();
 
 		//
 		// XML documentation buffer. The save point is used to divide
@@ -173,12 +177,20 @@ namespace Mono.CSharp
 			set {
 				if (value == XmlCommentState.Allowed) {
 					check_incorrect_doc_comment ();
-					consume_doc_comment ();
+					reset_doc_comment ();
 				}
 				xmlDocState = value;
 			}
 		}
 
+		public bool IsEscapedIdentifier (Location loc)
+		{
+			foreach (LocatedToken lt in escapedIdentifiers)
+				if (lt.Location.Equals (loc))
+					return true;
+			return false;
+		}
+
 		//
 		// Class variables
 		// 
@@ -384,9 +396,7 @@ namespace Mono.CSharp
 		}
 
 		public Location Location {
-			get {
-				return new Location (ref_line);
-			}
+			get { return current_location; }
 		}
 
 		void define (string def)
@@ -419,7 +429,7 @@ namespace Mono.CSharp
 			// FIXME: This could be `Location.Push' but we have to
 			// find out why the MS compiler allows this
 			//
-			Mono.CSharp.Location.Push (file);
+			Mono.CSharp.Location.Push (file, 0);
 		}
 
 		static bool is_identifier_start_character (char c)
@@ -472,6 +482,12 @@ namespace Mono.CSharp
 		{
 		start:
 			int the_token = token ();
+			if (the_token == Token.OPEN_BRACKET) {
+				do {
+					the_token = token ();
+				} while (the_token != Token.CLOSE_BRACKET);
+				the_token = token ();
+			}
 			switch (the_token) {
 			case Token.IDENTIFIER:
 			case Token.OBJECT:
@@ -532,8 +548,10 @@ namespace Mono.CSharp
 
 			switch (c){
 			case '{':
+				val = Location;
 				return Token.OPEN_BRACE;
 			case '}':
+				val = Location;
 				return Token.CLOSE_BRACE;
 			case '[':
 				// To block doccomment inside attribute declaration.
@@ -552,8 +570,13 @@ namespace Mono.CSharp
 
 				// Save current position and parse next token.
 				int old = reader.Position;
+				int old_ref_line = ref_line;
+				int old_col = col;
+
 				int new_token = token ();
 				reader.Position = old;
+				ref_line = old_ref_line;
+				col = old_col;
 				putback_char = -1;
 
 				if (new_token == Token.OPEN_PARENS)
@@ -568,11 +591,11 @@ namespace Mono.CSharp
 
 			case ',':
 				return Token.COMMA;
-			case ':':
-				return Token.COLON;
 			case ';':
+				val = Location;
 				return Token.SEMICOLON;
 			case '~':
+				val = Location;
 				return Token.TILDE;
 			case '?':
 				return Token.INTERR;
@@ -646,24 +669,32 @@ namespace Mono.CSharp
 			d = peekChar ();
 			if (c == '+'){
 				
-				if (d == '+')
+				if (d == '+') {
+					val = Location;
 					t = Token.OP_INC;
+				}
 				else if (d == '=')
 					t = Token.OP_ADD_ASSIGN;
-				else
+				else {
+					val = Location;
 					return Token.PLUS;
+				}
 				doread = true;
 				return t;
 			}
 			if (c == '-'){
-				if (d == '-')
+				if (d == '-') {
+					val = Location;
 					t = Token.OP_DEC;
+				}
 				else if (d == '=')
 					t = Token.OP_SUB_ASSIGN;
 				else if (d == '>')
 					t = Token.OP_PTR;
-				else
+				else {
+					val = Location;
 					return Token.MINUS;
+				}
 				doread = true;
 				return t;
 			}
@@ -673,6 +704,7 @@ namespace Mono.CSharp
 					doread = true;
 					return Token.OP_NE;
 				}
+				val = Location;
 				return Token.BANG;
 			}
 
@@ -692,6 +724,7 @@ namespace Mono.CSharp
 					doread = true;
 					return Token.OP_AND_ASSIGN;
 				}
+				val = Location;
 				return Token.BITWISE_AND;
 			}
 
@@ -711,6 +744,7 @@ namespace Mono.CSharp
 					doread = true;
 					return Token.OP_MULT_ASSIGN;
 				}
+				val = Location;
 				return Token.STAR;
 			}
 
@@ -738,6 +772,15 @@ namespace Mono.CSharp
 				return Token.CARRET;
 			}
 
+			if (c == ':'){
+				if (d == ':'){
+					doread = true;
+					return Token.DOUBLE_COLON;
+				}
+				val = Location;
+				return Token.COLON;
+			}
+
 			return Token.ERROR;
 		}
 
@@ -857,7 +900,7 @@ namespace Mono.CSharp
 							// if we have not seen anything in between
 							// report this error
 							//
-							Report.Warning (78, Location, "The 'l' suffix is easily confused with the digit '1' (use 'L' for clarity)");
+							Report.Warning (78, 4, Location, "The 'l' suffix is easily confused with the digit '1' (use 'L' for clarity)");
 						}
 						//
 						// This goto statement causes the MS CLR 2.0 beta 1 csc to report an error, so
@@ -959,6 +1002,7 @@ namespace Mono.CSharp
 		int adjust_real (int t)
 		{
 			string s = new String (number_builder, 0, number_pos);
+			const string error_details = "Floating-point constant is outside the range of type `{0}'";
 
 			switch (t){
 			case Token.LITERAL_DECIMAL:
@@ -966,8 +1010,7 @@ namespace Mono.CSharp
 					val = System.Decimal.Parse (s, styles, csharp_format_info);
 				} catch (OverflowException) {
 					val = 0m;     
-					error_details = "Floating-point constant is outside the range of the type 'decimal'";
-					Report.Error (594, Location, error_details);
+					Report.Error (594, Location, error_details, "decimal");
 				}
 				break;
 			case Token.LITERAL_FLOAT:
@@ -975,8 +1018,7 @@ namespace Mono.CSharp
 					val = (float) System.Double.Parse (s, styles, csharp_format_info);
 				} catch (OverflowException) {
 					val = 0.0f;     
-					error_details = "Floating-point constant is outside the range of the type 'float'";
-					Report.Error (594, Location, error_details);
+					Report.Error (594, Location, error_details, "float");
 				}
 				break;
 				
@@ -987,8 +1029,7 @@ namespace Mono.CSharp
 					val = System.Double.Parse (s, styles, csharp_format_info);
 				} catch (OverflowException) {
 					val = 0.0;     
-					error_details = "Floating-point constant is outside the range of the type 'double'";
-					Report.Error (594, Location, error_details);
+					Report.Error (594, Location, error_details, "double");
 				}
 				break;
 			}
@@ -1021,6 +1062,11 @@ namespace Mono.CSharp
 				val = 0ul;
 				return Token.LITERAL_INTEGER;
 			}
+			catch (FormatException) {
+				Report.Error (1013, Location, "Invalid number");
+				val = 0ul;
+				return Token.LITERAL_INTEGER;
+			}
 			
 			return integer_type_suffix (ul, peekChar ());
 		}
@@ -1193,7 +1239,7 @@ namespace Mono.CSharp
 					goto default;
 				return v;
 			default:
-				Report.Error (1009, Location, "Unrecognized escape sequence in " + (char)d);
+				Report.Error (1009, Location, "Unrecognized escape sequence `\\{0}'", ((char)d).ToString ());
 				return d;
 			}
 			getChar ();
@@ -1202,13 +1248,22 @@ namespace Mono.CSharp
 
 		int getChar ()
 		{
-			if (putback_char != -1){
-				int x = putback_char;
+			int x;
+			if (putback_char != -1) {
+				x = putback_char;
 				putback_char = -1;
-
-				return x;
 			}
-			return reader.Read ();
+			else
+				x = reader.Read ();
+			if (x == '\n') {
+				line++;
+				ref_line++;
+				previous_col = col;
+				col = 0;
+			}
+			else
+				col++;
+			return x;
 		}
 
 		int peekChar ()
@@ -1235,6 +1290,14 @@ namespace Mono.CSharp
 				Console.WriteLine ("Current [{0}] putting back [{1}]  ", putback_char, c);
 				throw new Exception ("This should not happen putback on putback");
 			}
+			if (c == '\n' || col == 0) {
+				// It won't happen though.
+				line--;
+				ref_line--;
+				col = previous_col;
+			}
+			else
+				col--;
 			putback_char = c;
 		}
 
@@ -1344,22 +1407,16 @@ namespace Mono.CSharp
 			cmd = static_cmd_arg.ToString ();
 
 			if (c == '\n'){
-				line++;
-				ref_line++;
 				return;
-			} else if (c == '\r')
-				col = 0;
+			}
 
 			// skip over white space
 			while ((c = getChar ()) != -1 && (c != '\n') && ((c == '\r') || (c == ' ') || (c == '\t')))
 				;
 
 			if (c == '\n'){
-				line++;
-				ref_line++;
 				return;
 			} else if (c == '\r'){
-				col = 0;
 				return;
 			} else if (c == -1){
 				arg = "";
@@ -1373,11 +1430,6 @@ namespace Mono.CSharp
 				static_cmd_arg.Append ((char) c);
 			}
 
-			if (c == '\n'){
-				line++;
-				ref_line++;
-			} else if (c == '\r')
-				col = 0;
 			arg = static_cmd_arg.ToString ().Trim ();
 		}
 
@@ -1392,7 +1444,7 @@ namespace Mono.CSharp
 			if (arg == "default"){
 				ref_line = line;
 				ref_name = file_name;
-				Location.Push (ref_name);
+				Location.Push (ref_name, line);
 				return true;
 			} else if (arg == "hidden"){
 				//
@@ -1414,7 +1466,7 @@ namespace Mono.CSharp
 					ref_name = Location.LookupFile (name);
 					file_name.HasLineDirective = true;
 					ref_name.HasLineDirective = true;
-					Location.Push (ref_name);
+					Location.Push (ref_name, ref_line);
 				} else {
 					ref_line = System.Int32.Parse (arg);
 				}
@@ -1496,7 +1548,7 @@ namespace Mono.CSharp
 				Hashtable w_table = Report.warning_ignore_table;
 				foreach (int code in codes) {
 					if (w_table != null && w_table.Contains (code))
-						Report.Warning (1635, 1, Location, "Cannot restore warning 'CS{0:0000}' because it was disabled globally", code);
+						Report.Warning (1635, 1, Location, String.Format ("Cannot restore warning `CS{0:0000}' because it was disabled globally", code));
 					Report.RegisterWarningRegion (Location).WarningEnable (Location, code);
 				}
 				return;
@@ -1520,7 +1572,7 @@ namespace Mono.CSharp
 					values[index++] = int.Parse (string_code, System.Globalization.CultureInfo.InvariantCulture);
 				}
 				catch (FormatException) {
-					Report.Warning (1692, Location, "Invalid number");
+					Report.Warning (1692, 1, Location, "Invalid number");
 				}
 			}
 			return values;
@@ -1690,7 +1742,7 @@ namespace Mono.CSharp
 		
 		void Error_InvalidDirective ()
 		{
-			Report.Error (1517, Location, "Invalid pre-processor directive");
+			Report.Error (1517, Location, "Invalid preprocessor directive");
 		}
 
 		void Error_UnexpectedDirective (string extra)
@@ -1702,9 +1754,8 @@ namespace Mono.CSharp
 
 		void Error_TokensSeen ()
 		{
-			Report.Error (
-				1032, Location,
-				"Cannot define or undefine pre-processor symbols after a token in the file");
+			Report.Error (1032, Location,
+				"Cannot define or undefine preprocessor symbols after first token in file");
 		}
 		
 		//
@@ -1717,6 +1768,8 @@ namespace Mono.CSharp
 			string cmd, arg;
 			bool region_directive = false;
 
+			current_location = new Location (ref_line, Col);
+
 			get_cmd_arg (out cmd, out arg);
 
 			// Eat any trailing whitespaces and single-line comments
@@ -1741,7 +1794,7 @@ namespace Mono.CSharp
 				if (!PreProcessLine (arg))
 					Report.Error (
 						1576, Location,
-						"Argument to #line directive is missing or invalid");
+						"The line number specified for #line directive is missing or invalid");
 				return caller_is_taking;
 
 			case "region":
@@ -1792,13 +1845,12 @@ namespace Mono.CSharp
 					int pop = (int) ifstack.Pop ();
 					
 					if (region_directive && ((pop & REGION) == 0))
-						Report.Error (1027, Location, "#endif directive expected");
+						Report.Error (1027, Location, "Expected `#endif' directive");
 					else if (!region_directive && ((pop & REGION) != 0))
 						Report.Error (1038, Location, "#endregion directive expected");
 					
 					if (!region_directive && arg.Length != 0) {
-						Report.Error (1025, Location, 
-							"Single line comment, or end-of-line expected");
+						Report.Error (1025, Location, "Single-line comment or end-of-line expected");
 					}
 					
 					if (ifstack.Count == 0)
@@ -1843,9 +1895,7 @@ namespace Mono.CSharp
 
 			case "else":
 				if (ifstack == null || ifstack.Count == 0){
-					Report.Error (
-						1028, Location,
-						"Unexpected processor directive (no #if for this #else)");
+					Error_UnexpectedDirective ("no #if for this #else");
 					return true;
 				} else {
 					int state = (int) ifstack.Peek ();
@@ -1907,11 +1957,11 @@ namespace Mono.CSharp
 				return true;
 
 			case "warning":
-				Report.Warning (1030, Location, "#warning: '{0}'", arg);
+				Report.Warning (1030, 1, Location, "#warning: `{0}'", arg);
 				return true;
 			}
 
-			Report.Error (1024, Location, "Preprocessor directive expected (got: " + cmd + ")");
+			Report.Error (1024, Location, "Wrong preprocessor directive");
 			return true;
 
 		}
@@ -1936,11 +1986,7 @@ namespace Mono.CSharp
 				if (c == '\n'){
 					if (!quoted)
 						Report.Error (1010, Location, "Newline in constant");
-					line++;
-					ref_line++;
-					col = 0;
-				} else
-					col++;
+				}
 
 				if (!quoted){
 					c = escape (c);
@@ -1971,6 +2017,8 @@ namespace Mono.CSharp
 				// Save current position and parse next token.
 				int old = reader.Position;
 				int old_putback = putback_char;
+				int old_ref_line = ref_line;
+				int old_col = col;
 
 				putback_char = -1;
 
@@ -1981,12 +2029,14 @@ namespace Mono.CSharp
 					(next_token == Token.ENUM); // "partial" is a keyword in 'partial enum', even though it's not valid
 
 				reader.Position = old;
+				ref_line = old_ref_line;
+				col = old_col;
 				putback_char = old_putback;
 
 				if (ok)
 					return res;
 				else {
-					val = "partial";
+					val = new LocatedToken (Location, "partial");
 					return Token.IDENTIFIER;
 				}
 			}
@@ -1997,11 +2047,13 @@ namespace Mono.CSharp
 		private int consume_identifier (int s, bool quoted) 
 		{
 			int pos = 1;
-			int c;
+			int c = -1;
 			
 			id_builder [0] = (char) s;
 					
-			while ((c = reader.Read ()) != -1) {
+			current_location = new Location (ref_line, Col);
+
+			while ((c = getChar ()) != -1) {
 				if (is_identifier_part_character ((char) c)){
 					if (pos == max_id_size){
 						Report.Error (645, Location, "Identifier too long (limit is 512 chars)");
@@ -2009,10 +2061,10 @@ namespace Mono.CSharp
 					}
 					
 					id_builder [pos++] = (char) c;
-					putback_char = -1;
-					col++;
+//					putback_char = -1;
 				} else {
-					putback_char = c;
+//					putback_char = c;
+					putback (c);
 					break;
 				}
 			}
@@ -2023,8 +2075,10 @@ namespace Mono.CSharp
 			//
 			if (!quoted && (s >= 'a' || s == '_')){
 				int keyword = GetKeyword (id_builder, pos);
-				if (keyword != -1)
+				if (keyword != -1) {
+					val = Location;
 				return keyword;
+				}
 			}
 
 			//
@@ -2035,6 +2089,9 @@ namespace Mono.CSharp
 			if (identifiers [pos] != null) {
 				val = identifiers [pos][id_builder];
 				if (val != null) {
+					val = new LocatedToken (Location, (string) val);
+					if (quoted)
+						escapedIdentifiers.Add (val);
 					return Token.IDENTIFIER;
 				}
 			}
@@ -2042,12 +2099,24 @@ namespace Mono.CSharp
 				identifiers [pos] = new CharArrayHashtable (pos);
 
 			val = new String (id_builder, 0, pos);
+			if (RootContext.Version == LanguageVersion.ISO_1) {
+				for (int i = 1; i < id_builder.Length; i += 3) {
+					if (id_builder [i] == '_' && (id_builder [i - 1] == '_' || id_builder [i + 1] == '_')) {
+						Report.Error (1638, Location, 
+							"`{0}': Any identifier with double underscores cannot be used when ISO language version mode is specified", val.ToString ());
+						break;
+					}
+				}
+			}
 
 			char [] chars = new char [pos];
 			Array.Copy (id_builder, chars, pos);
 
 			identifiers [pos] [chars] = val;
 
+			val = new LocatedToken (Location, (string) val);
+			if (quoted)
+				escapedIdentifiers.Add (val);
 			return Token.IDENTIFIER;
 		}
 
@@ -2060,12 +2129,11 @@ namespace Mono.CSharp
 			
 			val = null;
 			// optimization: eliminate col and implement #directive semantic correctly.
-			for (;(c = getChar ()) != -1; col++) {
+			for (;(c = getChar ()) != -1;) {
 				if (c == ' ')
 					continue;
 				
 				if (c == '\t') {
-					col = (((col + 8) / 8) * 8) - 1;
 					continue;
 				}
 				
@@ -2076,9 +2144,6 @@ namespace Mono.CSharp
 					if (peekChar () == '\n')
 						getChar ();
 
-					line++;
-					ref_line++;
-					col = 0;
 					any_token_seen |= tokens_seen;
 					tokens_seen = false;
 					comments_seen = false;
@@ -2095,6 +2160,7 @@ namespace Mono.CSharp
 							getChar ();
 							// Don't allow ////.
 							if ((d = peekChar ()) != '/') {
+								update_comment_location ();
 								if (doc_state == XmlCommentState.Allowed)
 									handle_one_line_xml_comment ();
 								else if (doc_state == XmlCommentState.NotAllowed)
@@ -2102,11 +2168,7 @@ namespace Mono.CSharp
 							}
 						}
 						while ((d = getChar ()) != -1 && (d != '\n') && d != '\r')
-							col++;
 						if (d == '\n'){
-							line++;
-							ref_line++;
-							col = 0;
 						}
 						any_token_seen |= tokens_seen;
 						tokens_seen = false;
@@ -2117,6 +2179,7 @@ namespace Mono.CSharp
 						bool docAppend = false;
 						if (RootContext.Documentation != null && peekChar () == '*') {
 							getChar ();
+							update_comment_location ();
 							// But when it is /**/, just do nothing.
 							if (peekChar () == '/') {
 								getChar ();
@@ -2139,7 +2202,6 @@ namespace Mono.CSharp
 						while ((d = getChar ()) != -1){
 							if (d == '*' && peekChar () == '/'){
 								getChar ();
-								col++;
 								comments_seen = true;
 								break;
 							}
@@ -2147,9 +2209,6 @@ namespace Mono.CSharp
 								xml_comment_buffer.Append ((char) d);
 							
 							if (d == '\n'){
-								line++;
-								ref_line++;
-								col = 0;
 								any_token_seen |= tokens_seen;
 								tokens_seen = false;
 								// 
@@ -2172,9 +2231,6 @@ namespace Mono.CSharp
 			is_punct_label:
 				// white space
 				if (c == '\n'){
-					line++;
-					ref_line++;
-					col = 0;
 					any_token_seen |= tokens_seen;
 					tokens_seen = false;
 					comments_seen = false;
@@ -2201,17 +2257,12 @@ namespace Mono.CSharp
 					cont = handle_preprocessing_directive (cont);
 
 					if (cont){
-						col = 0;
 						continue;
 					}
-					col = 1;
 
 					bool skipping = false;
-					for (;(c = getChar ()) != -1; col++){
+					for (;(c = getChar ()) != -1;){
 						if (c == '\n'){
-							col = 0;
-							line++;
-							ref_line++;
 							skipping = false;
 						} else if (c == ' ' || c == '\t' || c == '\v' || c == '\r' || c == 0xa0)
 							continue;
@@ -2223,7 +2274,7 @@ namespace Mono.CSharp
 					any_token_seen |= tokens_seen;
 					tokens_seen = false;
 					if (c == -1)
-						Report.Error (1027, Location, "#endif/#endregion expected");
+						Report.Error (1027, Location, "Expected `#endif' directive");
 					continue;
 				}
 
@@ -2251,6 +2302,7 @@ namespace Mono.CSharp
 					return consume_identifier (c);
 			}
 
+			current_location = new Location (ref_line, Col);
 			if ((t = is_punct ((char)c, ref doread)) != Token.ERROR){
 				tokens_seen = true;
 				if (doread){
@@ -2284,6 +2336,10 @@ namespace Mono.CSharp
 					Report.Error (1011, Location, error_details);
 					return Token.ERROR;
 				}
+				if (c == '\r' || c == '\n') {
+					Report.Error (1010, Location, "Newline in constant");
+					return Token.ERROR;
+				}
 				c = escape (c);
 				if (c == -1)
 					return Token.ERROR;
@@ -2297,13 +2353,11 @@ namespace Mono.CSharp
 
 					// Try to recover, read until newline or next "'"
 					while ((c = getChar ()) != -1){
-						if (c == '\n' || c == '\''){
-							line++;
-							ref_line++;
-							col = 0;
+						if (c == '\n'){
+							break;
+						}
+						else if (c == '\'')
 							break;
-						} else
-							col++;
 					}
 					return Token.ERROR;
 				}
@@ -2345,7 +2399,6 @@ namespace Mono.CSharp
 			while ((c = peekChar ()) == ' ')
 				getChar (); // skip heading whitespaces.
 			while ((c = peekChar ()) != -1 && c != '\n' && c != '\r') {
-				col++;
 				xml_comment_buffer.Append ((char) getChar ());
 			}
 			if (c == '\r' || c == '\n')
@@ -2383,6 +2436,18 @@ namespace Mono.CSharp
 			xml_comment_buffer.Insert (current_comment_start, String.Join (Environment.NewLine, lines));
 		}
 
+		//
+		// Updates current comment location.
+		//
+		private void update_comment_location ()
+		{
+			if (current_comment_location.IsNull) {
+				// "-2" is for heading "//" or "/*"
+				current_comment_location =
+					new Location (ref_line, col - 2);
+			}
+		}
+
 		//
 		// Checks if there was incorrect doc comments and raise
 		// warnings.
@@ -2399,10 +2464,13 @@ namespace Mono.CSharp
 		//
 		private void warn_incorrect_doc_comment ()
 		{
-			doc_state = XmlCommentState.Error;
-			// in csc, it is 'XML comment is not placed on a valid 
-			// language element'. But that does not make sense.
-			Report.Warning (1587, 2, Location, "XML comment is placed on an invalid language element which can not accept it.");
+			if (doc_state != XmlCommentState.Error) {
+				doc_state = XmlCommentState.Error;
+				// in csc, it is 'XML comment is not placed on 
+				// a valid language element'. But that does not
+				// make sense.
+				Report.Warning (1587, 2, Location, "XML comment is not placed on a valid language element");
+			}
 		}
 
 		//
@@ -2413,12 +2481,18 @@ namespace Mono.CSharp
 		{
 			if (xml_comment_buffer.Length > 0) {
 				string ret = xml_comment_buffer.ToString ();
-				xml_comment_buffer.Length = 0;
+				reset_doc_comment ();
 				return ret;
 			}
 			return null;
 		}
 
+		void reset_doc_comment ()
+		{
+			xml_comment_buffer.Length = 0;
+			current_comment_location = Location.Null;
+		}
+
 		public void cleanup ()
 		{
 			if (ifstack != null && ifstack.Count >= 1) {
@@ -2426,7 +2500,7 @@ namespace Mono.CSharp
 				if ((state & REGION) != 0)
 					Report.Error (1038, Location, "#endregion directive expected");
 				else 
-					Report.Error (1027, Location, "#endif directive expected");
+					Report.Error (1027, "Expected `#endif' directive");
 			}
 				
 		}
