@@ -27,56 +27,82 @@ namespace Mono.CSharp {
 		public readonly TypeArguments TypeArguments;
 
 		public readonly MemberName Left;
+		public readonly Location Location;
 
-		public static readonly MemberName Null = new MemberName ("");
+		public static readonly MemberName Null = new MemberName ("", Location.Null);
 
-		public MemberName (string name)
+		bool is_double_colon;
+
+		private MemberName (MemberName left, string name, bool is_double_colon,
+				    TypeArguments args, Location loc)
 		{
 			this.Name = name;
+			this.Location = loc;
+			this.is_double_colon = is_double_colon;
+			this.TypeArguments = args;
+			this.Left = left;
 		}
 
-		public MemberName (string name, TypeArguments args)
-			: this (name)
+		public MemberName (string name, TypeArguments args, Location loc)
+			: this (name, loc)
 		{
 			this.TypeArguments = args;
 		}
 
-		public MemberName (MemberName left, string name)
-			: this (left, name, null)
+		public MemberName (string name, Location loc)
 		{
+			this.Name = name;
+			this.Location = loc;
 		}
 
-		public MemberName (MemberName left, string name, TypeArguments args)
-			: this (name, args)
+		public MemberName (MemberName left, string name, Location loc)
+			: this (name, loc)
 		{
 			this.Left = left;
 		}
 
-		public MemberName (MemberName left, MemberName right)
-			: this (left, right.Name, right.TypeArguments)
+		public MemberName (MemberName left, string name, TypeArguments args, Location loc)
+			: this (name, args, loc)
 		{
-			Name = right.Name;
-			Left = (right.Left == null) ? left : new MemberName (left, right.Left);
-			TypeArguments = right.TypeArguments;
+			this.Left = left;
+		}
+
+		public MemberName (string alias, string name, Location loc)
+			: this (new MemberName (alias, loc), name, true, null, loc)
+		{
+		}
+
+		public MemberName (MemberName left, MemberName right)
+			: this (left, right, right.Location)
+		{
+		}
+
+		public MemberName (MemberName left, MemberName right, Location loc)
+			: this (null, right.Name, false, right.TypeArguments, loc)
+		{
+			if (right.is_double_colon)
+				throw new InternalErrorException ("Cannot append double_colon member name");
+			this.Left = (right.Left == null) ? left : new MemberName (left, right.Left);
 		}
 
 		static readonly char [] dot_array = { '.' };
 
-		public static MemberName FromDotted (string name)
+		public static MemberName FromDotted (string name, Location loc)
 		{
 			string [] elements = name.Split (dot_array);
 			int count = elements.Length;
 			int i = 0;
-			MemberName n = new MemberName (elements [i++]);
+			MemberName n = new MemberName (elements [i++], loc);
 			while (i < count)
-				n = new MemberName (n, elements [i++]);
+				n = new MemberName (n, elements [i++], loc);
 			return n;
 		}
 
 		public string GetName ()
 		{
+			string connect = is_double_colon ? "::" : ".";
 			if (Left != null)
-				return Left.GetName () + "." + Name;
+				return Left.GetName () + connect + Name;
 			else
 				return Name;
 		}
@@ -95,8 +121,9 @@ namespace Mono.CSharp {
 		public string GetName (bool is_generic)
 		{
 			string name = is_generic ? Basename : Name;
+			string connect = is_double_colon ? "::" : ".";
 			if (Left != null)
-				return Left.GetName (is_generic) + "." + name;
+				return Left.GetName (is_generic) + connect + name;
 			else
 				return name;
 		}
@@ -112,8 +139,9 @@ namespace Mono.CSharp {
 
 		public string MethodName {
 			get {
+				string connect = is_double_colon ? "::" : ".";
 				if (Left != null)
-					return Left.FullName + "." + Name;
+					return Left.FullName + connect + Name;
 				else
 					return Name;
 			}
@@ -134,8 +162,9 @@ namespace Mono.CSharp {
 
 		public string GetTypeName ()
 		{
+			string connect = is_double_colon ? "::" : ".";
 			if (Left != null)
-				return Left.GetTypeName () + "." +
+				return Left.GetTypeName () + connect +
 					MakeName (Name, TypeArguments);
 			else
 				return MakeName (Name, TypeArguments);
@@ -164,33 +193,36 @@ namespace Mono.CSharp {
 			return true;
 		}
 
-		public Expression GetTypeExpression (Location loc)
+		public Expression GetTypeExpression ()
 		{
 			if (IsUnbound) {
-				if (!CheckUnbound (loc))
+				if (!CheckUnbound (Location))
 					return null;
 
-				return new UnboundTypeExpression (GetTypeName ());
+				return new UnboundTypeExpression (this, Location);
 			}
 
-			if (Left != null) {
-				Expression lexpr = Left.GetTypeExpression (loc);
-
-				return new MemberAccess (lexpr, Name, TypeArguments, loc);
-			} else {
+			if (Left == null) {
 				if (TypeArguments != null)
-					return new SimpleName (Basename, TypeArguments, loc);
+					return new SimpleName (Basename, TypeArguments, Location);
 				else
-					return new SimpleName (Name, loc);
+					return new SimpleName (Name, Location);
 			}
+
+			if (is_double_colon) {
+				if (Left.Left != null)
+					throw new InternalErrorException ("The left side of a :: should be an identifier");
+				return new QualifiedAliasMember (Left.Name, Name, Location);
+			}
+
+			Expression lexpr = Left.GetTypeExpression ();
+			return new MemberAccess (lexpr, Name, TypeArguments, Location);
 		}
 
 		public MemberName Clone ()
 		{
-			if (Left != null)
-				return new MemberName (Left.Clone (), Name, TypeArguments);
-			else
-				return new MemberName (Name, TypeArguments);
+			MemberName left_clone = Left == null ? null : Left.Clone ();
+			return new MemberName (left_clone, Name, is_double_colon, TypeArguments, Location);
 		}
 
 		public string Basename {
@@ -213,8 +245,9 @@ namespace Mono.CSharp {
 
 		public override string ToString ()
 		{
+			string connect = is_double_colon ? "::" : ".";
 			if (Left != null)
-				return Left.FullName + "." + FullName;
+				return Left.FullName + connect + FullName;
 			else
 				return FullName;
 		}
@@ -230,6 +263,8 @@ namespace Mono.CSharp {
 				return true;
 			if (other == null || Name != other.Name)
 				return false;
+			if (is_double_colon != other.is_double_colon)
+				return false;
 
 			if ((TypeArguments != null) &&
 			    (other.TypeArguments == null || TypeArguments.Count != other.TypeArguments.Count))
@@ -249,6 +284,8 @@ namespace Mono.CSharp {
 			int hash = Name.GetHashCode ();
 			for (MemberName n = Left; n != null; n = n.Left)
 				hash ^= n.Name.GetHashCode ();
+			if (is_double_colon)
+				hash ^= 0xbadc01d;
 
 			if (TypeArguments != null)
 				hash ^= TypeArguments.Count << 5;
@@ -291,7 +328,9 @@ namespace Mono.CSharp {
 		/// <summary>
 		///   Location where this declaration happens
 		/// </summary>
-		public readonly Location Location;
+		public Location Location {
+			get { return member_name.Location; }
+		}
 
 		/// <summary>
 		///   XML documentation comment
@@ -317,7 +356,8 @@ namespace Mono.CSharp {
 			Excluded_Undetected = 1 << 8,		// Conditional attribute has not been detected yet
 			Excluded = 1 << 9,					// Method is conditional
 			TestMethodDuplication = 1 << 10,		// Test for duplication must be performed
-			IsUsed = 1 << 11
+			IsUsed = 1 << 11,
+			IsAssigned = 1 << 12				// Field is assigned
 		}
   
 		/// <summary>
@@ -325,8 +365,7 @@ namespace Mono.CSharp {
  		/// </summary>
 		internal Flags caching_flags;
 
-		public MemberCore (TypeContainer parent, MemberName name, Attributes attrs,
-				   Location loc)
+		public MemberCore (TypeContainer parent, MemberName name, Attributes attrs)
 			: base (attrs)
 		{
 			if (parent is PartialContainer && !(this is PartialContainer))
@@ -334,7 +373,6 @@ namespace Mono.CSharp {
 
 			Parent = parent;
 			member_name = name;
-			Location = loc;
 			caching_flags = Flags.Obsolete_Undetected | Flags.ClsCompliance_Undetected | Flags.HasCompliantAttribute_Undetected | Flags.Excluded_Undetected;
 		}
 
@@ -344,21 +382,6 @@ namespace Mono.CSharp {
 			cached_name = null;
 		}
 
-		/// <summary>
-		/// Tests presence of ObsoleteAttribute and report proper error
-		/// </summary>
-		protected void CheckUsageOfObsoleteAttribute (Type type)
-		{
-			if (type == null)
-				return;
-
-			ObsoleteAttribute obsolete_attr = AttributeTester.GetObsoleteAttribute (type);
-			if (obsolete_attr == null)
-				return;
-
-			AttributeTester.Report_ObsoleteMessage (obsolete_attr, type.FullName, Location);
-		}
-
 		public abstract bool Define ();
 
 		// 
@@ -366,15 +389,10 @@ namespace Mono.CSharp {
 		//
 		public virtual string GetSignatureForError ()
 		{
-			return Name;
-		}
+			if (Parent == null || Parent.Parent == null)
+				return Name;
 
-		/// <summary>
-		/// Use this method when MethodBuilder is null
-		/// </summary>
-		public virtual string GetSignatureForError (TypeContainer tc)
-		{
-			return Name;
+			return String.Concat (Parent.GetSignatureForError (), '.', Name);
 		}
 
 		/// <summary>
@@ -382,16 +400,19 @@ namespace Mono.CSharp {
 		/// </summary>
 		public virtual void Emit ()
 		{
-			// Hack with Parent == null is for EnumMember
-			if (Parent == null || (GetObsoleteAttribute (Parent) == null && Parent.GetObsoleteAttribute (Parent) == null))
-				VerifyObsoleteAttribute ();
-
 			if (!RootContext.VerifyClsCompliance)
 				return;
 
 			VerifyClsCompliance (Parent);
 		}
 
+		public virtual EmitContext EmitContext
+		{
+			get {
+				return Parent.EmitContext;
+			}
+		}
+
 		public bool InUnsafe {
 			get {
 				return ((ModFlags & Modifiers.UNSAFE) != 0) || Parent.UnsafeContext;
@@ -430,7 +451,7 @@ namespace Mono.CSharp {
 		/// <summary>
 		/// Returns instance of ObsoleteAttribute for this MemberCore
 		/// </summary>
-		public ObsoleteAttribute GetObsoleteAttribute (DeclSpace ds)
+		public virtual ObsoleteAttribute GetObsoleteAttribute ()
 		{
 			// ((flags & (Flags.Obsolete_Undetected | Flags.Obsolete)) == 0) is slower, but why ?
 			if ((caching_flags & Flags.Obsolete_Undetected) == 0 && (caching_flags & Flags.Obsolete) == 0) {
@@ -443,11 +464,11 @@ namespace Mono.CSharp {
 				return null;
 
 			Attribute obsolete_attr = OptAttributes.Search (
-				TypeManager.obsolete_attribute_type, ds.EmitContext);
+				TypeManager.obsolete_attribute_type, EmitContext);
 			if (obsolete_attr == null)
 				return null;
 
-			ObsoleteAttribute obsolete = obsolete_attr.GetObsoleteAttribute (ds.EmitContext);
+			ObsoleteAttribute obsolete = obsolete_attr.GetObsoleteAttribute (EmitContext);
 			if (obsolete == null)
 				return null;
 
@@ -455,6 +476,34 @@ namespace Mono.CSharp {
 			return obsolete;
 		}
 
+		/// <summary>
+		/// Checks for ObsoleteAttribute presence. It's used for testing of all non-types elements
+		/// </summary>
+		public virtual void CheckObsoleteness (Location loc)
+		{
+			if (Parent != null)
+				Parent.CheckObsoleteness (loc);
+
+			ObsoleteAttribute oa = GetObsoleteAttribute ();
+			if (oa == null) {
+				return;
+			}
+
+			AttributeTester.Report_ObsoleteMessage (oa, GetSignatureForError (), loc);
+		}
+
+		protected void CheckObsoleteType (Expression type)
+		{
+			ObsoleteAttribute obsolete_attr = AttributeTester.GetObsoleteAttribute (type.Type);
+			if (obsolete_attr == null)
+				return;
+
+			if (GetObsoleteAttribute () != null || Parent.GetObsoleteAttribute () != null)
+				return;
+
+			AttributeTester.Report_ObsoleteMessage (obsolete_attr, TypeManager.CSharpName (type.Type), type.Location);
+		}
+
 		/// <summary>
 		/// Analyze whether CLS-Compliant verification must be execute for this MemberCore.
 		/// </summary>
@@ -534,29 +583,28 @@ namespace Mono.CSharp {
 			if (!IsClsComplianceRequired (ds)) {
 				if (HasClsCompliantAttribute && RootContext.WarningLevel >= 2) {
 					if (!IsExposedFromAssembly (ds))
-						Report.Warning (3019, Location, "CLS compliance checking will not be performed on '{0}' because it is private or internal", GetSignatureForError ());
+						Report.Warning (3019, 2, Location, "CLS compliance checking will not be performed on `{0}' because it is not visible from outside this assembly", GetSignatureForError ());
 					if (!CodeGen.Assembly.IsClsCompliant)
-						Report.Warning (3021, Location, "'{0}' does not need a CLSCompliant attribute because the assembly does not have a CLSCompliant attribute", GetSignatureForError ());
+						Report.Warning (3021, 2, Location, "`{0}' does not need a CLSCompliant attribute because the assembly is not marked as CLS-compliant", GetSignatureForError ());
 				}
 				return false;
 			}
 
 			if (!CodeGen.Assembly.IsClsCompliant) {
 				if (HasClsCompliantAttribute) {
-					Report.Error (3014, Location, "'{0}' cannot be marked as CLS-compliant because the assembly does not have a CLSCompliant attribute", GetSignatureForError ());
+					Report.Error (3014, Location,
+						"`{0}' cannot be marked as CLS-compliant because the assembly is not marked as CLS-compliant",
+						GetSignatureForError ());
 				}
 				return false;
 			}
 
-			int index = Name.LastIndexOf ('.');
-			if (Name [index > 0 ? index + 1 : 0] == '_') {
-				Report.Error (3008, Location, "Identifier '{0}' is not CLS-compliant", GetSignatureForError () );
+			if (member_name.Name [0] == '_') {
+				Report.Error (3008, Location, "Identifier `{0}' is not CLS-compliant", GetSignatureForError () );
 			}
 			return true;
 		}
 
-		protected abstract void VerifyObsoleteAttribute ();
-
 		//
 		// Raised (and passed an XmlElement that contains the comment)
 		// when GenerateDocComment is writing documentation expectedly.
@@ -595,7 +643,7 @@ namespace Mono.CSharp {
 	///   provides the common foundation for managing those name
 	///   spaces.
 	/// </remarks>
-	public abstract class DeclSpace : MemberCore, IAlias {
+	public abstract class DeclSpace : MemberCore {
 		/// <summary>
 		///   This points to the actual definition that is being
 		///   created with System.Reflection.Emit
@@ -628,8 +676,10 @@ namespace Mono.CSharp {
 		// The emit context for toplevel objects.
 		protected EmitContext ec;
 		
-		public EmitContext EmitContext {
-			get { return ec; }
+		public override EmitContext EmitContext {
+			get {
+				return ec;
+			}
 		}
 
 		//
@@ -649,8 +699,8 @@ namespace Mono.CSharp {
 		static string[] attribute_targets = new string [] { "type" };
 
 		public DeclSpace (NamespaceEntry ns, TypeContainer parent, MemberName name,
-				  Attributes attrs, Location l)
-			: base (parent, name, attrs, l)
+				  Attributes attrs)
+			: base (parent, name, attrs)
 		{
 			NamespaceEntry = ns;
 			Basename = name.Basename;
@@ -668,16 +718,19 @@ namespace Mono.CSharp {
 		/// </summary>
 		protected bool AddToContainer (MemberCore symbol, string name)
 		{
-			if (name == Basename && !(this is Interface) && !(this is Enum)) {
+			if (name == MemberName.Name && !(this is Interface) && !(this is Enum)) {
 				if (symbol is TypeParameter)
-					Report.Error (694, "Type parameter `{0}' has same name as " +
-						      "containing type or method", name);
+					Report.Error (694, symbol.Location,
+						      "Type parameter `{0}' has same name as " +
+						      "containing type, or method", name);
 				else {
 					Report.SymbolRelatedToPreviousError (this);
-					Report.Error (542, "'{0}': member names cannot be the same as their " +
-						      "enclosing type", symbol.Location, symbol.GetSignatureForError ());
+					Report.Error (542,  symbol.Location,
+						      "`{0}': member names cannot be the same " +
+						      "as their enclosing type",
+						      symbol.GetSignatureForError ());
+					return false;
 				}
-				return false;
 			}
 
 			MemberCore mc = (MemberCore) defined_names [name];
@@ -690,21 +743,28 @@ namespace Mono.CSharp {
 			if (symbol.MarkForDuplicationCheck () && mc.MarkForDuplicationCheck ())
 				return true;
 
-			if (symbol is TypeParameter)
-				Report.Error (692, symbol.Location, "Duplicate type parameter `{0}'", name);
-			else {
-				Report.SymbolRelatedToPreviousError (mc);
-				Report.Error (102, symbol.Location,
-					      "The type '{0}' already contains a definition for '{1}'",
-					      GetSignatureForError (), name);
+			Report.SymbolRelatedToPreviousError (mc);
+			if (symbol is PartialContainer || mc is PartialContainer) {
+				Report.Error (260, symbol.Location,
+					"Missing partial modifier on declaration of type `{0}'. Another partial declaration of this type exists",
+					name);
+				return false;
 			}
-			return false;
-		}
 
-		public void RecordDecl ()
-		{
-			if ((NamespaceEntry != null) && (Parent == RootContext.Tree.Types))
-				NamespaceEntry.DefineName (MemberName.Basename, this);
+			if (this is RootTypes) {
+				Report.Error (101, symbol.Location, 
+					"The namespace `{0}' already contains a definition for `{1}'",
+					((DeclSpace)symbol).NamespaceEntry.GetSignatureForError (), symbol.MemberName.Name);
+			} else if (symbol is TypeParameter) {
+				Report.Error (692, symbol.Location,
+					      "Duplicate type parameter `{0}'", name);
+			} else {
+				Report.Error (102, symbol.Location,
+					      "The type `{0}' already contains a definition for `{1}'",
+					      GetSignatureForError (), symbol.MemberName.Name);
+			}
+
+			return false;
 		}
 
 		/// <summary>
@@ -723,13 +783,7 @@ namespace Mono.CSharp {
 		// why there is a non-obvious test down here.
 		//
 		public bool IsTopLevel {
-			get {
-				if (Parent != null){
-					if (Parent.Parent == null)
-						return true;
-				}
-				return false;
-			}
+			get { return (Parent != null && Parent.Parent == null); }
 		}
 
 		public virtual void CloseType ()
@@ -754,9 +808,7 @@ namespace Mono.CSharp {
 		}
 
 		protected virtual TypeAttributes TypeAttr {
-			get {
-				return CodeGen.Module.DefaultCharSetType;
-			}
+			get { return CodeGen.Module.DefaultCharSetType; }
 		}
 
 		/// <remarks>
@@ -768,7 +820,23 @@ namespace Mono.CSharp {
 		///   Define all members, but don't apply any attributes or do anything which may
 		///   access not-yet-defined classes.  This method also creates the MemberCache.
 		/// </summary>
-		public abstract bool DefineMembers (TypeContainer parent);
+		public virtual bool DefineMembers (TypeContainer parent)
+		{
+			if (((ModFlags & Modifiers.NEW) != 0) && IsTopLevel) {
+				Report.Error (1530, Location, "Keyword `new' is not allowed on namespace elements");
+				return false;
+			}
+			return true;
+		}
+
+		public override string GetSignatureForError ()
+		{
+			if (IsGeneric) {
+				return SimpleName.RemoveGenericArity (Name) + TypeParameter.GetSignatureForError (CurrentTypeParameters);
+			}
+			// Parent.GetSignatureForError
+			return Name;
+		}
 
 		//
 		// Whether this is an `unsafe context'
@@ -807,13 +875,9 @@ namespace Mono.CSharp {
 		protected TypeExpr ResolveBaseTypeExpr (Expression e, bool silent, Location loc)
 		{
 			TypeResolveEmitContext.loc = loc;
-			TypeResolveEmitContext.ContainerType = TypeBuilder;
 			TypeResolveEmitContext.ResolvingTypeTree = true;
 			if (this is GenericMethod)
 				TypeResolveEmitContext.ContainerType = Parent.TypeBuilder;
-			else
-				TypeResolveEmitContext.ContainerType = TypeBuilder;
-
 			return e.ResolveAsTypeTerminal (TypeResolveEmitContext);
 		}
 		
@@ -825,9 +889,7 @@ namespace Mono.CSharp {
 			else
 				tb = TypeBuilder;
 
-			if (check_type.IsGenericInstance)
-				check_type = check_type.GetGenericTypeDefinition ();
-
+			check_type = TypeManager.DropGenericTypeArguments (check_type);
 			if (check_type == tb)
 				return true;
 
@@ -864,7 +926,8 @@ namespace Mono.CSharp {
   				//
   				// This test should probably use the declaringtype.
   				//
-				return check_type.Assembly == TypeBuilder.Assembly;
+				return check_type.Assembly == TypeBuilder.Assembly ||
+					TypeManager.IsFriendAssembly (check_type.Assembly);
 
 			case TypeAttributes.NestedPublic:
 				return true;
@@ -879,15 +942,18 @@ namespace Mono.CSharp {
 				return FamilyAccessible (tb, check_type);
 
 			case TypeAttributes.NestedFamANDAssem:
-				return (check_type.Assembly == tb.Assembly) &&
+				return ((check_type.Assembly == tb.Assembly) || 
+						TypeManager.IsFriendAssembly (check_type.Assembly)) && 
 					FamilyAccessible (tb, check_type);
 
 			case TypeAttributes.NestedFamORAssem:
 				return (check_type.Assembly == tb.Assembly) ||
-					FamilyAccessible (tb, check_type);
+					FamilyAccessible (tb, check_type) ||
+					TypeManager.IsFriendAssembly (check_type.Assembly);
 
 			case TypeAttributes.NestedAssembly:
-				return check_type.Assembly == tb.Assembly;
+				return check_type.Assembly == tb.Assembly ||
+					TypeManager.IsFriendAssembly (check_type.Assembly);
 			}
 
 			Console.WriteLine ("HERE: " + check_attr);
@@ -1009,13 +1075,6 @@ namespace Mono.CSharp {
 			return ~ (~ mAccess | pAccess) == 0;
 		}
 
-		public static void Error_AmbiguousTypeReference (Location loc, string name, string t1, string t2)
-		{
-			Report.Error (104, loc,
-				      "`{0}' is an ambiguous reference ({1} or {2})",
-				      name, t1, t2);
-		}
-
 		//
 		// Return the nested type with name @name.  Ensures that the nested type
 		// is defined if necessary.  Do _not_ use this when you have a MemberCache handy.
@@ -1037,6 +1096,7 @@ namespace Mono.CSharp {
 			for (Type current_type = TypeBuilder;
 			     current_type != null && current_type != TypeManager.object_type;
 			     current_type = current_type.BaseType) {
+				current_type = TypeManager.DropGenericTypeArguments (current_type);
 				if (current_type is TypeBuilder) {
 					DeclSpace decl = this;
 					if (current_type != TypeBuilder)
@@ -1054,8 +1114,7 @@ namespace Mono.CSharp {
 		}
 
 		//
-		// Public function used to locate types, this can only
-		// be used after the ResolveTree function has been invoked.
+		// Public function used to locate types.
 		//
 		// Set 'ignore_cs0104' to true if you want to ignore cs0104 errors.
 		//
@@ -1172,12 +1231,9 @@ namespace Mono.CSharp {
 				if (param.Name != name)
 					continue;
 
-				if (RootContext.WarningLevel >= 3)
-					Report.Warning (
-						693, Location,
-						"Type parameter `{0}' has same name " +
-						"as type parameter from outer type `{1}'",
-						name, Parent.GetInstantiationName ());
+				Report.Warning (693, 3, Location,
+					"Type parameter `{0}' has same name as type parameter from outer type `{1}'",
+					name, Parent.GetInstantiationName ());
 
 				return false;
 			}
@@ -1222,35 +1278,39 @@ namespace Mono.CSharp {
 			if (!is_generic) {
 				if (constraints_list != null) {
 					Report.Error (
-						80, Location, "Contraints are not allowed " +
+						80, Location, "Constraints are not allowed " +
 						"on non-generic declarations");
 				}
 
 				return;
 			}
 
-			string[] names = MemberName.TypeArguments.GetDeclarations ();
+			TypeParameterName[] names = MemberName.TypeArguments.GetDeclarations ();
 			type_params = new TypeParameter [names.Length];
 
 			//
 			// Register all the names
 			//
 			for (int i = 0; i < type_params.Length; i++) {
-				string name = names [i];
+				TypeParameterName name = names [i];
 
 				Constraints constraints = null;
 				if (constraints_list != null) {
 					foreach (Constraints constraint in constraints_list) {
-						if (constraint.TypeParameter == name) {
+						if (constraint == null)
+							continue;
+						if (constraint.TypeParameter == name.Name) {
 							constraints = constraint;
 							break;
 						}
 					}
 				}
 
-				type_params [i] = new TypeParameter (Parent, name, constraints, Location);
+				type_params [i] = new TypeParameter (
+					Parent, this, name.Name, constraints, name.OptAttributes,
+					Location);
 
-				AddToContainer (type_params [i], name);
+				AddToContainer (type_params [i], name.Name);
 			}
 		}
 
@@ -1265,7 +1325,7 @@ namespace Mono.CSharp {
 			}
 		}
 
-		protected TypeParameter[] CurrentTypeParameters {
+		public TypeParameter[] CurrentTypeParameters {
 			get {
 				if (!IsGeneric)
 					throw new InvalidOperationException ();
@@ -1306,29 +1366,38 @@ namespace Mono.CSharp {
 			return null;
 		}
 
-		bool IAlias.IsType {
-			get { return true; }
-		}
-
-		string IAlias.Name {
-			get { return Name; }
-		}
-
-		TypeExpr IAlias.ResolveAsType (EmitContext ec)
-		{
-			if (TypeBuilder == null)
-				throw new InvalidOperationException ();
-
-			if (CurrentType != null)
-				return new TypeExpression (CurrentType, Location);
-			else
-				return new TypeExpression (TypeBuilder, Location);
-		}
-
 		public override string[] ValidAttributeTargets {
-			get {
-				return attribute_targets;
+			get { return attribute_targets; }
+		}
+
+		protected override bool VerifyClsCompliance (DeclSpace ds)
+		{
+			if (!base.VerifyClsCompliance (ds)) {
+				return false;
 			}
+
+			IDictionary cache = TypeManager.AllClsTopLevelTypes;
+			string lcase = Name.ToLower (System.Globalization.CultureInfo.InvariantCulture);
+			if (!cache.Contains (lcase)) {
+				cache.Add (lcase, this);
+				return true;
+			}
+
+			object val = cache [lcase];
+			if (val == null) {
+				Type t = AttributeTester.GetImportedIgnoreCaseClsType (lcase);
+				if (t == null)
+					return true;
+				Report.SymbolRelatedToPreviousError (t);
+			}
+			else {
+				if (val is PartialContainer)
+					return true;
+
+				Report.SymbolRelatedToPreviousError ((DeclSpace)val);
+			}
+			Report.Warning (3005, 1, Location, "Identifier `{0}' differing only in case is not CLS-compliant", GetSignatureForError ());
+			return true;
 		}
 	}
 
@@ -2043,7 +2112,7 @@ namespace Mono.CSharp {
 		// Because the MemberCache holds members from this class and all the base classes,
 		// we can avoid tons of reflection stuff.
 		//
-		public MemberInfo FindMemberToOverride (Type invocationType, string name, Type [] paramTypes, bool is_property)
+		public MemberInfo FindMemberToOverride (Type invocationType, string name, Type [] paramTypes, GenericMethod genericMethod, bool is_property)
 		{
 			ArrayList applicable;
 			if (method_hash != null && !is_property)
@@ -2080,7 +2149,7 @@ namespace Mono.CSharp {
 					}
 				} else {
 					mi = (MethodInfo) entry.Member;
-					cmpAttrs = TypeManager.GetArgumentTypes (mi);
+					cmpAttrs = TypeManager.GetParameterData (mi).Types;
 				}
 
 				if (fi != null) {
@@ -2119,7 +2188,20 @@ namespace Mono.CSharp {
 					if (!TypeManager.IsEqual (paramTypes [j], cmpAttrs [j]))
 						goto next;
 				}
-				
+
+				//
+				// check generic arguments for methods
+				//
+				if (mi != null) {
+					Type [] cmpGenArgs = mi.GetGenericArguments ();
+					if (genericMethod != null && cmpGenArgs.Length > 0) {
+						if (genericMethod.TypeParameters.Length != cmpGenArgs.Length)
+							goto next;
+					}
+					else if (! (genericMethod == null && cmpGenArgs.Length == 0))
+						goto next;
+				}
+
 				//
 				// get one of the methods because this has the visibility info.
 				//
@@ -2293,7 +2375,7 @@ namespace Mono.CSharp {
  		
 				MethodBase method_to_compare = (MethodBase)entry.Member;
 				AttributeTester.Result result = AttributeTester.AreOverloadedMethodParamsClsCompliant (
-					method.ParameterTypes, TypeManager.GetArgumentTypes (method_to_compare));
+					method.ParameterTypes, TypeManager.GetParameterData (method_to_compare).Types);
 
  				if (result == AttributeTester.Result.Ok)
  					continue;
@@ -2308,10 +2390,10 @@ namespace Mono.CSharp {
  				Report.SymbolRelatedToPreviousError (entry.Member);
 				switch (result) {
 					case AttributeTester.Result.RefOutArrayError:
-						Report.Error (3006, method.Location, "Overloaded method '{0}' differing only in ref or out, or in array rank, is not CLS-compliant", method.GetSignatureForError ());
+						Report.Error (3006, method.Location, "Overloaded method `{0}' differing only in ref or out, or in array rank, is not CLS-compliant", method.GetSignatureForError ());
 						continue;
 					case AttributeTester.Result.ArrayArrayError:
-						Report.Error (3007, method.Location, "Overloaded method '{0}' differing only by unnamed array types is not CLS-compliant", method.GetSignatureForError ());
+						Report.Error (3007, method.Location, "Overloaded method `{0}' differing only by unnamed array types is not CLS-compliant", method.GetSignatureForError ());
 						continue;
 				}
 
