@@ -31,7 +31,6 @@ namespace Mono.CSharp {
 		public MethodBuilder      BeginInvokeBuilder;
 		public MethodBuilder      EndInvokeBuilder;
 		
-		Type [] param_types;
 		Type ret_type;
 
 		static string[] attribute_targets = new string [] { "type", "return" };
@@ -50,14 +49,14 @@ namespace Mono.CSharp {
 
  		public Delegate (NamespaceEntry ns, TypeContainer parent, Expression type,
 				 int mod_flags, MemberName name, Parameters param_list,
-				 Attributes attrs, Location l)
-			: base (ns, parent, name, attrs, l)
+				 Attributes attrs)
+			: base (ns, parent, name, attrs)
 
 		{
 			this.ReturnType = type;
 			ModFlags        = Modifiers.Check (AllowedModifiers, mod_flags,
 							   IsTopLevel ? Modifiers.INTERNAL :
-							   Modifiers.PRIVATE, l);
+							   Modifiers.PRIVATE, name.Location);
 			Parameters      = param_list;
 		}
 
@@ -88,7 +87,7 @@ namespace Mono.CSharp {
 			}
 			
 			if (TypeManager.multicast_delegate_type == null && !RootContext.StdLib) {
-				Namespace system = Namespace.LookupNamespace ("System", true);
+				Namespace system = RootNamespace.Global.GetNamespace ("System", true);
 				TypeExpr expr = system.Lookup (this, "MulticastDelegate", Location) as TypeExpr;
 				TypeManager.multicast_delegate_type = expr.ResolveType (ec);
 			}
@@ -115,7 +114,7 @@ namespace Mono.CSharp {
 					name, TypeAttr, TypeManager.multicast_delegate_type);
 			}
 
-			TypeManager.AddDelegateType (Name, TypeBuilder, this);
+			TypeManager.AddUserType (this);
 
 			if (IsGeneric) {
 				string[] param_names = new string [TypeParameters.Length];
@@ -134,10 +133,8 @@ namespace Mono.CSharp {
 						return null;
 				}
 
-				for (int i = offset; i < gen_params.Length; i++)
-					CurrentTypeParameters [i - offset].DefineConstraints ();
-
-				Expression current = new SimpleName (Name, TypeParameters, Location);
+				Expression current = new SimpleName (
+					MemberName.Basename, TypeParameters, Location);
 				current = current.ResolveAsTypeTerminal (ec);
 				if (current == null)
 					return null;
@@ -148,11 +145,6 @@ namespace Mono.CSharp {
 			return TypeBuilder;
 		}
 
- 		public override bool DefineMembers (TypeContainer container)
-		{
-			return true;
-		}
-
  		public override bool Define ()
 		{
 			MethodAttributes mattr;
@@ -184,19 +176,15 @@ namespace Mono.CSharp {
 			//
 			// HACK because System.Reflection.Emit is lame
 			//
-			//
-			// FIXME: POSSIBLY make these static, as they are always the same
 			Parameter [] fixed_pars = new Parameter [2];
-			fixed_pars [0] = new Parameter (TypeManager.system_object_expr, "object",
-							Parameter.Modifier.NONE, null);
-			fixed_pars [1] = new Parameter (TypeManager.system_intptr_expr, "method", 
-							Parameter.Modifier.NONE, null);
-			Parameters const_parameters = new Parameters (fixed_pars, null, Location);
+			fixed_pars [0] = new Parameter (TypeManager.object_type, "object",
+							Parameter.Modifier.NONE, null, Location);
+			fixed_pars [1] = new Parameter (TypeManager.intptr_type, "method", 
+							Parameter.Modifier.NONE, null, Location);
+			Parameters const_parameters = new Parameters (fixed_pars);
+			const_parameters.Resolve (null);
 			
-			TypeManager.RegisterMethod (
-				ConstructorBuilder,
-				new InternalParameters (const_arg_types, const_parameters),
-				const_arg_types);
+			TypeManager.RegisterMethod (ConstructorBuilder, const_parameters);
 				
 			
 			ConstructorBuilder.SetImplementationFlags (MethodImplAttributes.Runtime);
@@ -207,8 +195,7 @@ namespace Mono.CSharp {
 			// First, call the `out of band' special method for
 			// defining recursively any types we need:
 			
- 			param_types = Parameters.GetParameterInfo (ec);
-			if (param_types == null)
+			if (!Parameters.Resolve (ec))
 				return false;
 
 			//
@@ -216,11 +203,11 @@ namespace Mono.CSharp {
 			//
 
 			// Check accessibility
-			foreach (Type partype in param_types){
+			foreach (Type partype in Parameters.Types){
 				if (!Parent.AsAccessible (partype, ModFlags)) {
 					Report.Error (59, Location,
 						      "Inconsistent accessibility: parameter type `" +
-						      TypeManager.CSharpName (partype) + "` is less " +
+						      TypeManager.CSharpName (partype) + "' is less " +
 						      "accessible than delegate `" + Name + "'");
 					return false;
 				}
@@ -228,18 +215,20 @@ namespace Mono.CSharp {
 					return false;
 			}
 			
- 			ReturnType = ReturnType.ResolveAsTypeTerminal (ec);
-                        if (ReturnType == null)
-                            return false;
+			ReturnType = ReturnType.ResolveAsTypeTerminal (ec, false);
+			if (ReturnType == null)
+				return false;
                         
-   			ret_type = ReturnType.Type;
+			ret_type = ReturnType.Type;
 			if (ret_type == null)
 				return false;
 
+			CheckObsoleteType (ReturnType);
+
 			if (!Parent.AsAccessible (ret_type, ModFlags)) {
 				Report.Error (58, Location,
 					      "Inconsistent accessibility: return type `" +
-					      TypeManager.CSharpName (ret_type) + "` is less " +
+					      TypeManager.CSharpName (ret_type) + "' is less " +
 					      "accessible than delegate `" + Name + "'");
 				return false;
 			}
@@ -257,7 +246,7 @@ namespace Mono.CSharp {
 			// guaranteed to be accessible - they are standard types.
 			//
 			
-  			CallingConventions cc = Parameters.GetCallingConvention ();
+  			CallingConventions cc = Parameters.CallingConvention;
 
  			mattr = MethodAttributes.Public | MethodAttributes.HideBySig | MethodAttributes.Virtual;
 
@@ -265,58 +254,28 @@ namespace Mono.CSharp {
  								  mattr,		     
  								  cc,
  								  ret_type,		     
- 								  param_types);
+ 								  Parameters.Types);
 
 			//
 			// Define parameters, and count out/ref parameters
 			//
 			int out_params = 0;
-			i = 0;
-			if (Parameters.FixedParameters != null){
-				int top = Parameters.FixedParameters.Length;
-				Parameter p;
-				
-				for (; i < top; i++) {
-					p = Parameters.FixedParameters [i];
-					p.DefineParameter (ec, InvokeBuilder, null, i + 1, Location);
-
-					if ((p.ModFlags & Parameter.Modifier.ISBYREF) != 0)
-						out_params++;
-				}
-			}
-			if (Parameters.ArrayParameter != null){
-				if (TypeManager.param_array_type == null && !RootContext.StdLib) {
-					Namespace system = Namespace.LookupNamespace ("System", true);
-					TypeExpr expr = system.Lookup (this, "ParamArrayAttribute", Location) as TypeExpr;
-					TypeManager.param_array_type = expr.ResolveType (ec);
-				}
-
-				if (TypeManager.cons_param_array_attribute == null) {
-					Type [] void_arg = { };
-					TypeManager.cons_param_array_attribute = TypeManager.GetConstructor (
-						TypeManager.param_array_type, void_arg);
-				}
-
-				ParameterBuilder pb = InvokeBuilder.DefineParameter (
-					i + 1, Parameters.ArrayParameter.Attributes,Parameters.ArrayParameter.Name);
-				
-				pb.SetCustomAttribute (
-					new CustomAttributeBuilder (TypeManager.cons_param_array_attribute, new object [0]));
+			foreach (Parameter p in Parameters.FixedParameters) {
+				if ((p.ModFlags & Parameter.Modifier.ISBYREF) != 0)
+					out_params++;
 			}
 			
 			InvokeBuilder.SetImplementationFlags (MethodImplAttributes.Runtime);
 
-			TypeManager.RegisterMethod (InvokeBuilder,
-						    new InternalParameters (param_types, Parameters),
-						    param_types);
+			TypeManager.RegisterMethod (InvokeBuilder, Parameters);
 
 			//
 			// BeginInvoke
 			//
-			int params_num = param_types.Length;
+			int params_num = Parameters.Count;
 			Type [] async_param_types = new Type [params_num + 2];
 
-			param_types.CopyTo (async_param_types, 0);
+			Parameters.Types.CopyTo (async_param_types, 0);
 
 			async_param_types [params_num] = TypeManager.asynccallback_type;
 			async_param_types [params_num + 1] = TypeManager.object_type;
@@ -330,50 +289,28 @@ namespace Mono.CSharp {
 								       TypeManager.iasyncresult_type,
 								       async_param_types);
 
-			i = 0;
-			if (Parameters.FixedParameters != null){
-				int top = Parameters.FixedParameters.Length;
-				Parameter p;
-				
-				for (i = 0 ; i < top; i++) {
-					p = Parameters.FixedParameters [i];
-
-					p.DefineParameter (ec, BeginInvokeBuilder, null, i + 1, Location);
-				}
-			}
-			if (Parameters.ArrayParameter != null){
-				Parameter p = Parameters.ArrayParameter;
-				p.DefineParameter (ec, BeginInvokeBuilder, null, i + 1, Location);
-
-				i++;
-			}
-
+			i = Parameters.Count;
+			Parameters.ApplyAttributes (ec, BeginInvokeBuilder);
 			BeginInvokeBuilder.DefineParameter (i + 1, ParameterAttributes.None, "callback");
 			BeginInvokeBuilder.DefineParameter (i + 2, ParameterAttributes.None, "object");
 			
 			BeginInvokeBuilder.SetImplementationFlags (MethodImplAttributes.Runtime);
 
 			Parameter [] async_params = new Parameter [params_num + 2];
-			int n = 0;
-			if (Parameters.FixedParameters != null){
-				Parameters.FixedParameters.CopyTo (async_params, 0);
-				n = Parameters.FixedParameters.Length;
-			}
-			if (Parameters.ArrayParameter != null)
-				async_params [n] = Parameters.ArrayParameter;
+			Parameters.FixedParameters.CopyTo (async_params, 0);
 			
 			async_params [params_num] = new Parameter (
-				TypeManager.system_asynccallback_expr, "callback",
-								   Parameter.Modifier.NONE, null);
+				TypeManager.asynccallback_type, "callback",
+								   Parameter.Modifier.NONE, null, Location);
 			async_params [params_num + 1] = new Parameter (
-				TypeManager.system_object_expr, "object",
-								   Parameter.Modifier.NONE, null);
+				TypeManager.object_type, "object",
+								   Parameter.Modifier.NONE, null, Location);
 
-			Parameters async_parameters = new Parameters (async_params, null, Location);
+			Parameters async_parameters = new Parameters (async_params);
+			async_parameters.Resolve (ec);
+			async_parameters.ApplyAttributes (ec, BeginInvokeBuilder);
 			
-			TypeManager.RegisterMethod (BeginInvokeBuilder,
-						    new InternalParameters (async_parameters.GetParameterInfo (ec), async_parameters),
-						    async_param_types);
+			TypeManager.RegisterMethod (BeginInvokeBuilder, async_parameters);
 
 			//
 			// EndInvoke is a bit more interesting, all the parameters labeled as
@@ -390,13 +327,13 @@ namespace Mono.CSharp {
 					if ((p.ModFlags & Parameter.Modifier.ISBYREF) == 0)
 						continue;
 
-					end_param_types [param] = param_types [i];
+					end_param_types [param] = Parameters.Types [i];
 					end_params [param] = p;
 					param++;
 				}
 			}
 			end_param_types [out_params] = TypeManager.iasyncresult_type;
-			end_params [out_params] = new Parameter (TypeManager.system_iasyncresult_expr, "result", Parameter.Modifier.NONE, null);
+			end_params [out_params] = new Parameter (TypeManager.system_iasyncresult_expr, "result", Parameter.Modifier.NONE, null, Location);
 
 			//
 			// Create method, define parameters, register parameters with type system
@@ -412,20 +349,19 @@ namespace Mono.CSharp {
 				EndInvokeBuilder.DefineParameter (i + 1, end_params [i].Attributes, end_params [i].Name);
 			}
 
-			Parameters end_parameters = new Parameters (end_params, null, Location);
+			Parameters end_parameters = new Parameters (end_params);
+			end_parameters.Resolve (ec);
 
-			TypeManager.RegisterMethod (
-				EndInvokeBuilder,
-				new InternalParameters (end_parameters.GetParameterInfo (ec), end_parameters),
-				end_param_types);
+			TypeManager.RegisterMethod (EndInvokeBuilder, end_parameters);
 
 			return true;
 		}
 
 		public override void Emit ()
 		{
+			Parameters.ApplyAttributes (ec, InvokeBuilder);
+
 			if (OptAttributes != null) {
-				Parameters.LabelParameters (ec, InvokeBuilder, Location);
 				OptAttributes.Emit (ec, this);
 			}
 
@@ -453,10 +389,10 @@ namespace Mono.CSharp {
 				return false;
 			}
 
-			AttributeTester.AreParametersCompliant (Parameters.FixedParameters, Location);
+			Parameters.VerifyClsCompliance ();
 
 			if (!AttributeTester.IsClsCompliant (ReturnType.Type)) {
-				Report.Error (3002, Location, "Return type of '{0}' is not CLS-compliant", GetSignatureForError ());
+				Report.Error (3002, Location, "Return type of `{0}' is not CLS-compliant", GetSignatureForError ());
 			}
 			return true;
 		}
@@ -574,9 +510,7 @@ namespace Mono.CSharp {
 
 			int pd_count = pd.Count;
 
-			bool params_method = (pd_count != 0) &&
-				(pd.ParameterModifier (pd_count - 1) == Parameter.Modifier.PARAMS);
-
+			bool params_method = pd.HasParams;
 			bool is_params_applicable = false;
 			bool is_applicable = Invocation.IsApplicable (ec, me, args, arg_count, ref mb);
 
@@ -585,9 +519,8 @@ namespace Mono.CSharp {
 				is_applicable = is_params_applicable = true;
 
 			if (!is_applicable && !params_method && arg_count != pd_count) {
-				Report.Error (1593, loc,
-					      "Delegate '{0}' does not take {1} arguments",
-					      delegate_type.ToString (), arg_count);
+				Report.Error (1593, loc, "Delegate `{0}' does not take `{1}' arguments",
+					TypeManager.CSharpName (delegate_type), arg_count.ToString ());
 				return false;
 			}
 
@@ -644,24 +577,12 @@ namespace Mono.CSharp {
 		
 		public static string FullDelegateDesc (Type del_type, MethodBase mb, ParameterData pd)
 		{
-			StringBuilder sb = new StringBuilder (TypeManager.CSharpName (((MethodInfo) mb).ReturnType));
-			
-			sb.Append (" " + del_type.ToString ());
-			sb.Append (" (");
-
-			int length = pd.Count;
-			
-			for (int i = length; i > 0; ) {
-				i--;
-
-				sb.Append (pd.ParameterDesc (length - i - 1));
-				if (i != 0)
-					sb.Append (", ");
-			}
-			
-			sb.Append (")");
-			return sb.ToString ();
-			
+			StringBuilder sb = new StringBuilder ();
+			sb.Append (TypeManager.CSharpName (((MethodInfo) mb).ReturnType));
+			sb.Append (" ");
+			sb.Append (TypeManager.CSharpName (del_type));
+			sb.Append (pd.GetSignatureForError ());
+			return sb.ToString ();			
 		}
 		
 		// Hack around System.Reflection as found everywhere else
@@ -721,12 +642,6 @@ namespace Mono.CSharp {
 			}
 		}
 
-		public Type [] ParameterTypes {
-			get {
-				return param_types;
-			}
-		}
-
 		public override AttributeTargets AttributeTargets {
 			get {
 				return AttributeTargets.Delegate;
@@ -740,14 +655,6 @@ namespace Mono.CSharp {
 			get { return "T:"; }
 		}
 
-		protected override void VerifyObsoleteAttribute()
-		{
-			CheckUsageOfObsoleteAttribute (ret_type);
-
-			foreach (Type type in param_types) {
-				CheckUsageOfObsoleteAttribute (type);
-			}
-		}
 	}
 
 	//
@@ -786,10 +693,9 @@ namespace Mono.CSharp {
 					      "the usage. Try specifying the type " +
 					      "arguments explicitly.", method_desc);
 			else if (method.ReturnType != ((MethodInfo) found_method).ReturnType) {
-				Report.Error (407, loc, "'{0}' has the wrong return type to match delegate '{1}'", method_desc, delegate_desc);
+				Report.Error (407, loc, "`{0}' has the wrong return type to match the delegate `{1}'", method_desc, delegate_desc);
 			} else {
-				Report.Error (123, loc, "Method '" + method_desc + "' does not " +
-					"match delegate '" + delegate_desc + "'");
+				Report.Error (123, loc, "Method `{0}' does not match delegate `{1}'", method_desc, delegate_desc);
 			}
 		}
 		
@@ -857,12 +763,12 @@ namespace Mono.CSharp {
 			IMethodData md = TypeManager.GetMethod (delegate_method);
 			if (md == null) {
 				if (System.Attribute.GetCustomAttribute (delegate_method, TypeManager.conditional_attribute_type) != null) {
-					Report.Error (1618, loc, "Cannot create delegate with '{0}' because it has a Conditional attribute", TypeManager.CSharpSignature (delegate_method));
+					Report.Error (1618, loc, "Cannot create delegate with `{0}' because it has a Conditional attribute", TypeManager.CSharpSignature (delegate_method));
 				}
 			} else {
 				md.SetMemberIsUsed ();
 				if (md.OptAttributes != null && md.OptAttributes.Search (TypeManager.conditional_attribute_type, ec) != null) {
-					Report.Error (1618, loc, "Cannot create delegate with '{0}' because it has a Conditional attribute", TypeManager.CSharpSignature (delegate_method));
+					Report.Error (1618, loc, "Cannot create delegate with `{0}' because it has a Conditional attribute", TypeManager.CSharpSignature (delegate_method));
 				}
 			}
 			
@@ -870,9 +776,8 @@ namespace Mono.CSharp {
 				delegate_instance_expression = mg.InstanceExpression.Resolve (ec);
 			else if (ec.IsStatic) {
 				if (!delegate_method.IsStatic) {
-					Report.Error (120, loc,
-						      "An object reference is required for the non-static method " +
-						      delegate_method.Name);
+					Report.Error (120, loc, "`{0}': An object reference is required for the nonstatic field, method or property",
+						      TypeManager.CSharpSignature (delegate_method));
 					return null;
 				}
 				delegate_instance_expression = null;
@@ -880,7 +785,8 @@ namespace Mono.CSharp {
 				delegate_instance_expression = ec.GetThis (loc);
 
 			if (delegate_instance_expression != null && delegate_instance_expression.Type.IsValueType)
-				delegate_instance_expression = new BoxedCast (delegate_instance_expression);
+				delegate_instance_expression = new BoxedCast (
+					delegate_instance_expression, TypeManager.object_type);
 
 			method_group = mg;
 			eclass = ExprClass.Value;
@@ -1072,7 +978,8 @@ namespace Mono.CSharp {
 			// Pop the return value if there is one
 			//
 			if (method is MethodInfo){
-				if (((MethodInfo) method).ReturnType != TypeManager.void_type)
+				Type ret = ((MethodInfo)method).ReturnType;
+				if (TypeManager.TypeToCoreType (ret) != TypeManager.void_type)
 					ec.ig.Emit (OpCodes.Pop);
 			}
 		}
