@@ -32,7 +32,6 @@ namespace Mono.CSharp {
 	public class Yield : Statement {
 		Expression expr;
 		ArrayList finally_blocks;
-		bool resolved;
 
 		public Yield (Expression expr, Location l)
 		{
@@ -53,8 +52,7 @@ namespace Mono.CSharp {
 				return false;
 			}
 			if (ec.InCatch){
-				Report.Error (1631, loc, "Cannot yield in the body of a " +
-					      "catch clause");
+				Report.Error (1631, loc, "Cannot yield a value in the body of a catch clause");
 				return false;
 			}
 
@@ -78,8 +76,6 @@ namespace Mono.CSharp {
 			if (expr == null)
 				return false;
 
-			resolved = true;
-
 			if (!CheckContext (ec, loc))
 				return false;
 
@@ -127,15 +123,12 @@ namespace Mono.CSharp {
 	public class Iterator : Class {
 		protected ToplevelBlock original_block;
 		protected ToplevelBlock block;
-		string original_name;
 
-		Type iterator_type;
+		Type original_iterator_type;
 		TypeExpr iterator_type_expr;
 		bool is_enumerable;
 		public readonly bool IsStatic;
 
-		Hashtable fields;
-
 		//
 		// The state as we generate the iterator
 		//
@@ -146,11 +139,13 @@ namespace Mono.CSharp {
 		//
 		// Context from the original method
 		//
+		GenericMethod generic_method;
 		TypeContainer container;
 		TypeExpr current_type;
 		Type this_type;
-		Type return_type;
-		InternalParameters parameters;
+		Parameters parameters;
+		Parameters original_parameters;
+		IMethodData orig_method;
 
 		MethodInfo dispose_method;
 		MoveNextMethod move_next_method;
@@ -197,7 +192,7 @@ namespace Mono.CSharp {
 			resume_points.Add (entry_point);
 			entry_point.Define (ig);
 
-			ec.EmitTopBlock (original_block, parameters, Location);
+			ec.EmitTopBlock (orig_method, original_block);
 
 			EmitYieldBreak (ig);
 
@@ -341,32 +336,47 @@ namespace Mono.CSharp {
 			point.Define (ig);
 		}
 
-		private static MemberName MakeProxyName (string name)
+		private static MemberName MakeProxyName (string name, GenericMethod generic, Location loc)
 		{
 			int pos = name.LastIndexOf ('.');
 			if (pos > 0)
 				name = name.Substring (pos + 1);
 
-			return new MemberName ("<" + name + ">__" + (proxy_count++));
+			string proxy_name = "<" + name + ">__" + (proxy_count++);
+
+			if (generic != null) {
+				TypeArguments args = new TypeArguments (loc);
+				foreach (TypeParameter tparam in generic.CurrentTypeParameters)
+					args.Add (new SimpleName (tparam.Name, loc));
+				return new MemberName (proxy_name, args, loc);
+			} else
+				return new MemberName (proxy_name, loc);
 		}
 
 		//
 		// Our constructor
 		//
-		public Iterator (TypeContainer container, string name, Type return_type,
-				 InternalParameters parameters,
-				 int modifiers, ToplevelBlock block, Location loc)
-			: base (container.NamespaceEntry, container, MakeProxyName (name),
-				(modifiers & Modifiers.UNSAFE) | Modifiers.PRIVATE, null, loc)
+		public Iterator (IMethodData m_container, TypeContainer container, GenericMethod generic,
+				 int modifiers)
+			: base (container.NamespaceEntry, container,
+				MakeProxyName (m_container.MethodName.Name, generic, m_container.Location),
+				(modifiers & Modifiers.UNSAFE) | Modifiers.PRIVATE, null)
 		{
-			this.container = container;
-			this.return_type = return_type;
-			this.parameters = parameters;
-			this.original_name = name;
-			this.original_block = block;
-			this.block = new ToplevelBlock (block, parameters.Parameters, loc);
+			this.orig_method = m_container;
 
-			fields = new Hashtable ();
+			this.generic_method = generic;
+			this.container = container;
+			this.original_parameters = m_container.ParameterInfo;
+			this.original_block = orig_method.Block;
+			this.block = new ToplevelBlock (orig_method.Block, parameters, orig_method.Location);
+
+			if (generic != null) {
+				ArrayList constraints = new ArrayList ();
+				foreach (TypeParameter tparam in generic.TypeParameters)
+					constraints.Add (tparam.Constraints);
+
+				SetParameterInfo (constraints);
+			}
 
 			IsStatic = (modifiers & Modifiers.STATIC) != 0;
 		}
@@ -377,22 +387,19 @@ namespace Mono.CSharp {
 
 		public bool DefineIterator ()
 		{
-			ec = new EmitContext (this, Mono.CSharp.Location.Null, null, null, ModFlags);
+			ec = new EmitContext (this, Location, null, null, ModFlags);
 			ec.CurrentAnonymousMethod = move_next_method;
-			ec.CurrentIterator = this;
 			ec.InIterator = true;
 
-			if (!CheckType (return_type)) {
-				Report.Error (
-					1624, Location,
-					"The body of `{0}' cannot be an iterator block " +
-					"because '{1}' is not an iterator interface type",
-					original_name, TypeManager.CSharpName (return_type));
+			if (!CheckType ()) {
+				Report.Error (1624, Location,
+					"The body of `{0}' cannot be an iterator block because `{1}' is not an iterator interface type",
+					orig_method.GetSignatureForError (), TypeManager.CSharpName (orig_method.ReturnType));
 				return false;
 			}
 
-			for (int i = 0; i < parameters.Count; i++){
-				Parameter.Modifier mod = parameters.ParameterModifier (i);
+			for (int i = 0; i < original_parameters.Count; i++){
+				Parameter.Modifier mod = original_parameters.ParameterModifier (i);
 				if ((mod & (Parameter.Modifier.REF | Parameter.Modifier.OUT)) != 0){
 					Report.Error (
 						1623, Location,
@@ -405,7 +412,7 @@ namespace Mono.CSharp {
 					return false;
 				}
 
-				if (parameters.ParameterType (i).IsPointer) {
+				if (original_parameters.ParameterType (i).IsPointer) {
 					Report.Error (1637, Location, "Iterators cannot have unsafe parameters or yield types");
 					return false;
 				}
@@ -416,8 +423,131 @@ namespace Mono.CSharp {
 			else
 				this_type = container.TypeBuilder;
 
+			container.AddIterator (this);
+
+			orig_method.Block = block;
+			return true;
+		}
+
+		MethodInfo FetchMethodDispose ()
+		{
+			MemberList dispose_list;
+
+			dispose_list = FindMembers (
+				current_type.Type,
+				MemberTypes.Method, BindingFlags.Public | BindingFlags.Instance,
+				Type.FilterName, "Dispose");
+
+			if (dispose_list.Count != 1)
+				throw new InternalErrorException ("Cannot find Dipose() method.");
+
+			return (MethodInfo) dispose_list [0];
+		}
+
+		protected override bool DoDefineMembers ()
+		{
+			ec.InIterator = true;
+			ec.CurrentAnonymousMethod = move_next_method;
+			ec.capture_context = cc;
+
+			if (!base.DoDefineMembers ())
+				return false;
+
+			dispose_method = FetchMethodDispose ();
+			if (dispose_method == null)
+				return false;
+
+			return true;
+		}
+
+		public override bool Define ()
+		{
+			if (!base.Define ())
+				return false;
+
+			ec.InIterator = true;
+			ec.CurrentAnonymousMethod = move_next_method;
+			ec.capture_context = cc;
+
+			ec.TypeContainer = ec.TypeContainer.Parent;
+
+			if (ec.TypeContainer.CurrentType != null)
+				ec.ContainerType = ec.TypeContainer.CurrentType;
+			else
+				ec.ContainerType = ec.TypeContainer.TypeBuilder;
+
+			ec.ig = move_next_method.method.MethodBuilder.GetILGenerator ();
+
+			if (!ctor.Define ())
+				return false;
+
+			bool unreachable;
+
+			if (!ec.ResolveTopBlock (null, original_block, parameters, orig_method, out unreachable))
+				return false;
+
+			if (!ec.ResolveTopBlock (null, block, parameters, orig_method, out unreachable))
+				return false;
+
+			original_block.CompleteContexts ();
+
+			cc.EmitAnonymousHelperClasses (ec);
+
+			return true;
+		}
+
+		TypeExpr InflateType (Type it)
+		{
+			if (generic_method == null)
+				return new TypeExpression (it, Location);
+
+			if (it.IsGenericParameter && (it.DeclaringMethod != null)) {
+				int pos = it.GenericParameterPosition;
+				it = CurrentTypeParameters [pos].Type;
+			} else if (it.IsGenericInstance) {
+				Type[] args = it.GetGenericArguments ();
+
+				TypeArguments inflated = new TypeArguments (Location);
+				foreach (Type t in args)
+					inflated.Add (InflateType (t));
+
+				return new ConstructedType (it, inflated, Location);
+			} else if (it.IsArray) {
+				TypeExpr et_expr = InflateType (it.GetElementType ());
+				int rank = it.GetArrayRank ();
+
+				Type et = et_expr.ResolveAsTypeTerminal (ec).Type;
+				it = et.MakeArrayType (rank);
+			}
+
+			return new TypeExpression (it, Location);
+		}
+
+		Parameter InflateParameter (Parameter param)
+		{
+			TypeExpr te = InflateType (param.ParameterType);
+			return new Parameter (
+				te, param.Name, param.ModFlags, param.OptAttributes, param.Location);
+		}
+
+		Parameters InflateParameters (Parameters parameters, EmitContext ec)
+		{
+			int count = parameters.FixedParameters.Length;
+			if (count == 0)
+				return Parameters.EmptyReadOnlyParameters;
+			Parameter[] fixed_params = new Parameter [count];
+			for (int i = 0; i < count; i++)
+				fixed_params [i] = InflateParameter (parameters.FixedParameters [i]);
+
+			return new Parameters (fixed_params, parameters.HasArglist);
+		}
+
+		protected override TypeExpr [] GetClassBases (out TypeExpr base_class)
+		{
+			iterator_type_expr = InflateType (original_iterator_type);
+
 			generic_args = new TypeArguments (Location);
-			generic_args.Add (new TypeExpression (iterator_type, Location));
+			generic_args.Add (iterator_type_expr);
 
 			ArrayList list = new ArrayList ();
 			if (is_enumerable) {
@@ -442,80 +572,9 @@ namespace Mono.CSharp {
 				generic_args, Location);
 			list.Add (generic_enumerator_type);
 
-			iterator_type_expr = new TypeExpression (iterator_type, Location);
-
-			container.AddIterator (this);
-
 			Bases = list;
-			return true;
-		}
 
-		MethodInfo FetchMethodDispose ()
-		{
-			MemberList dispose_list;
-
-			dispose_list = FindMembers (
-				current_type.Type,
-				MemberTypes.Method, BindingFlags.Public | BindingFlags.Instance,
-				Type.FilterName, "Dispose");
-
-			if (dispose_list.Count != 1)
-				throw new InternalErrorException ("Cannot find Dipose() method.");
-
-			return (MethodInfo) dispose_list [0];
-		}
-
-		protected override bool DoDefineMembers ()
-		{
-			ec.InIterator = true;
-			ec.CurrentIterator = this;
-			ec.CurrentAnonymousMethod = move_next_method;
-			ec.capture_context = cc;
-
-			if (!base.DoDefineMembers ())
-				return false;
-
-			dispose_method = FetchMethodDispose ();
-			if (dispose_method == null)
-				return false;
-
-			return true;
-		}
-
-		public override bool Define ()
-		{
-			if (!base.Define ())
-				return false;
-
-			ec.InIterator = true;
-			ec.CurrentIterator = this;
-			ec.CurrentAnonymousMethod = move_next_method;
-			ec.capture_context = cc;
-			ec.TypeContainer = ec.TypeContainer.Parent;
-
-			if (ec.TypeContainer.CurrentType != null)
-				ec.ContainerType = ec.TypeContainer.CurrentType;
-			else
-				ec.ContainerType = ec.TypeContainer.TypeBuilder;
-
-			ec.ig = move_next_method.method.MethodBuilder.GetILGenerator ();
-
-			if (!ctor.Define ())
-				return false;
-
-			bool unreachable;
-
-			if (!ec.ResolveTopBlock (null, original_block, parameters, Location, out unreachable))
-				return false;
-
-			if (!ec.ResolveTopBlock (null, block, parameters, Location, out unreachable))
-				return false;
-
-			original_block.CompleteContexts ();
-
-			cc.EmitAnonymousHelperClasses (ec);
-
-			return true;
+			return base.GetClassBases (out base_class);
 		}
 
 		//
@@ -528,6 +587,11 @@ namespace Mono.CSharp {
 			else
 				current_type = new TypeExpression (TypeBuilder, Location);
 
+			parameters = InflateParameters (original_parameters, ec);
+			if (!parameters.Resolve (ec)) {
+				// TODO:
+			}
+
 			Define_Fields ();
 			Define_Current (false);
 			Define_Current (true);
@@ -535,10 +599,10 @@ namespace Mono.CSharp {
 			Define_Reset ();
 			Define_Dispose ();
 
-			Create_Block ();
-
 			Define_Constructor ();
 
+			Create_Block ();
+
 			if (is_enumerable) {
 				Define_GetEnumerator (false);
 				Define_GetEnumerator (true);
@@ -547,11 +611,8 @@ namespace Mono.CSharp {
 			return base.DefineNestedTypes ();
 		}
 
-
 		Field pc_field;
 		Field current_field;
-		LocalInfo pc_local;
-		LocalInfo current_local;
 		Method dispose;
 
 		void Create_Block ()
@@ -568,23 +629,37 @@ namespace Mono.CSharp {
 				Type t = this_type;
 				args.Add (new Argument (
 					new ThisParameterReference (t, Location)));
-				cc.CaptureThis ();
+				cc.CaptureThis (move_next_method);
 			}
 
-			args.Add (new Argument (new BoolLiteral (false)));
+			args.Add (new Argument (new BoolLiteral (false, Location)));
 
 			for (int i = 0; i < parameters.Count; i++) {
-				Type t = parameters.ParameterType (i);
+				Type t = original_parameters.ParameterType (i);
+				Type inflated = parameters.ParameterType (i);
 				string name = parameters.ParameterName (i);
 
 				args.Add (new Argument (
 					new SimpleParameterReference (t, first + i, Location)));
 
-				cc.AddParameterToContext (move_next_method, name, t, first + i);
+				cc.AddParameterToContext (move_next_method, name, inflated, first + i);
 			}
 
-			Expression new_expr = new New (current_type, args, Location);
+			TypeExpr proxy_type;
+			if (generic_method != null) {
+				TypeArguments new_args = new TypeArguments (Location);
+				if (Parent.IsGeneric) {
+					foreach (TypeParameter tparam in Parent.TypeParameters)
+						new_args.Add (new TypeParameterExpr (tparam, Location));
+				}
+				foreach (TypeParameter tparam in generic_method.TypeParameters)
+					new_args.Add (new TypeParameterExpr (tparam, Location));
+				ConstructedType ct = new ConstructedType (CurrentType, new_args, Location);
+				proxy_type = ct.ResolveAsTypeTerminal (ec);
+			} else
+				proxy_type = current_type;
 
+			Expression new_expr = new New (proxy_type, args, Location);
 			block.AddStatement (new NoCheckReturn (new_expr, Location));
 		}
 
@@ -592,12 +667,12 @@ namespace Mono.CSharp {
 		{
 			pc_field = new Field (
 				this, TypeManager.system_int32_expr, Modifiers.PRIVATE, "$PC",
-				null, null, Location);
+				null, Location);
 			AddField (pc_field);
 
 			current_field = new Field (
 				this, iterator_type_expr, Modifiers.PRIVATE, "$current",
-				null, null, Location);
+				null, Location);
 			AddField (current_field);
 		}
 
@@ -610,29 +685,27 @@ namespace Mono.CSharp {
 			if (!IsStatic)
 				list.Add (new Parameter (
 					new TypeExpression (this_type, Location),
-					"this", Parameter.Modifier.NONE, null));
+					"this", Parameter.Modifier.NONE,
+					null, Location));
 			list.Add (new Parameter (
-				TypeManager.system_boolean_expr, "initialized",
-				Parameter.Modifier.NONE, null));
+				TypeManager.bool_type, "initialized",
+				Parameter.Modifier.NONE, null, Location));
 
-			Parameter[] old_fixed = parameters.Parameters.FixedParameters;
-			if (old_fixed != null)
-				list.AddRange (old_fixed);
+			Parameter[] old_fixed = parameters.FixedParameters;
+			list.AddRange (old_fixed);
 
 			Parameter[] fixed_params = new Parameter [list.Count];
 			list.CopyTo (fixed_params);
 
-			ctor_params = new Parameters (
-				fixed_params, parameters.Parameters.ArrayParameter,
-				Location);
+			ctor_params = new Parameters (fixed_params);
 
 			ctor = new Constructor (
-				this, Name, Modifiers.PUBLIC, ctor_params,
-				new ConstructorBaseInitializer (null, Location),
+				this, MemberName.Name, Modifiers.PUBLIC, ctor_params,
+				new GeneratedBaseInitializer (Location),
 				Location);
 			AddConstructor (ctor);
 
-			ctor.Block = new ToplevelBlock (block, parameters.Parameters, Location);
+			ctor.Block = new ToplevelBlock (block, parameters, Location);
 
 			int first = IsStatic ? 2 : 3;
 
@@ -645,7 +718,7 @@ namespace Mono.CSharp {
 				new SetState (this, State.Running, Location),
 				Location));
 
-			ctor.Block.AddStatement (new InitScope (this, cc, Location));
+			ctor.Block.AddStatement (new InitScope (this, Location));
 		}
 
 		Statement Create_ThrowInvalidOperation ()
@@ -671,23 +744,23 @@ namespace Mono.CSharp {
 			if (is_generic) {
 				left = new MemberName (
 					"System.Collections.Generic.IEnumerator",
-					generic_args);
+					generic_args, Location);
 				type = iterator_type_expr;
 			} else {
-				left = new MemberName ("System.Collections.IEnumerator");
+				left = new MemberName ("System.Collections.IEnumerator", Location);
 				type = TypeManager.system_object_expr;
 			}
 
-			MemberName name = new MemberName (left, "Current", null);
+			MemberName name = new MemberName (left, "Current", null, Location);
 
 			ToplevelBlock get_block = new ToplevelBlock (
-				block, parameters.Parameters, Location);
+				block, parameters, Location);
 
 			get_block.AddStatement (new If (
 				new Binary (
 					Binary.Operator.LessThanOrEqual,
 					new FieldExpression (this, pc_field),
-					new IntLiteral ((int) State.Running), Location),
+					new IntLiteral ((int) State.Running, pc_field.Location)),
 				Create_ThrowInvalidOperation (),
 				new Return (
 					new FieldExpression (this, current_field), Location),
@@ -696,7 +769,7 @@ namespace Mono.CSharp {
 			Accessor getter = new Accessor (get_block, 0, null, Location);
 
 			Property current = new Property (
-				this, type, 0, false, name, null, getter, null, Location);
+				this, type, 0, false, name, null, getter, null);
 			AddProperty (current);
 		}
 
@@ -718,23 +791,22 @@ namespace Mono.CSharp {
 			if (is_generic) {
 				left = new MemberName (
 					"System.Collections.Generic.IEnumerable",
-					generic_args);
+					generic_args, Location);
 				type = generic_enumerator_type;
 			} else {
-				left = new MemberName ("System.Collections.IEnumerable");
+				left = new MemberName ("System.Collections.IEnumerable", Location);
 				type = enumerator_type;
 			}
 
-			MemberName name = new MemberName (left, "GetEnumerator", null);
+			MemberName name = new MemberName (left, "GetEnumerator", Location);
 
 			Method get_enumerator = new Method (
 				this, null, type, 0, false, name,
-				Parameters.EmptyReadOnlyParameters, null,
-				Location.Null);
+				Parameters.EmptyReadOnlyParameters, null);
 			AddMethod (get_enumerator);
 
 			get_enumerator.Block = new ToplevelBlock (
-				block, parameters.Parameters, Location);
+				block, parameters, Location);
 
 			get_enumerator.Block.SetHaveAnonymousMethods (Location, move_next_method);
 
@@ -743,8 +815,8 @@ namespace Mono.CSharp {
 				"CompareExchange", Location);
 
 			Expression pc = new FieldExpression (this, pc_field);
-			Expression before = new IntLiteral ((int) State.Running);
-			Expression uninitialized = new IntLiteral ((int) State.Uninitialized);
+			Expression before = new IntLiteral ((int) State.Running, Location);
+			Expression uninitialized = new IntLiteral ((int) State.Uninitialized, Location);
 
 			ArrayList args = new ArrayList ();
 			args.Add (new Argument (pc, Argument.AType.Ref));
@@ -754,8 +826,8 @@ namespace Mono.CSharp {
 			get_enumerator.Block.AddStatement (new If (
 				new Binary (
 					Binary.Operator.Equality,
-					new Invocation (ce, args, Location),
-					uninitialized, Location),
+					new Invocation (ce, args),
+					uninitialized),
 				new Return (new ThisParameterReference (type.Type, Location),
 					    Location),
 				Location));
@@ -765,7 +837,7 @@ namespace Mono.CSharp {
 				args.Add (new Argument (new CapturedThisReference (this, Location)));
 			}
 
-			args.Add (new Argument (new BoolLiteral (true)));
+			args.Add (new Argument (new BoolLiteral (true, Location)));
 
 			for (int i = 0; i < parameters.Count; i++) {
 				Expression cp = new CapturedParameterReference (
@@ -852,17 +924,16 @@ namespace Mono.CSharp {
 			{
 				ec.CurrentAnonymousMethod = iterator.move_next_method;
 
-				iterator.cc.EmitParameter (ec, name);
+				LocalTemporary dummy = null;
+				
+				iterator.cc.EmitParameter (ec, name, false, false, ref dummy);
 			}
 		}
 
 		protected class CapturedThisReference : Expression
 		{
-			Iterator iterator;
-
 			public CapturedThisReference (Iterator iterator, Location loc)
 			{
-				this.iterator = iterator;
 				this.loc = loc;
 				this.type = iterator.this_type;
 				eclass = ExprClass.Variable;
@@ -875,8 +946,7 @@ namespace Mono.CSharp {
 
 			public override void Emit (EmitContext ec)
 			{
-				ec.ig.Emit (OpCodes.Ldarg_0);
-				ec.ig.Emit (OpCodes.Ldfld, iterator.move_next_method.Scope.THIS);
+				ec.EmitThis ();
 			}
 		}
 
@@ -897,7 +967,7 @@ namespace Mono.CSharp {
 				FieldExpr fexpr = new FieldExpr (field.FieldBuilder, loc);
 				fexpr.InstanceExpression = new ThisParameterReference (
 					iterator.this_type, loc);
-				return fexpr.ResolveLValue (ec, right_side);
+				return fexpr.ResolveLValue (ec, right_side, loc);
 			}
 
 			public override Expression DoResolve (EmitContext ec)
@@ -919,7 +989,7 @@ namespace Mono.CSharp {
 			Iterator iterator;
 
 			public MoveNextMethod (Iterator iterator, Location loc)
-				: base (iterator.parameters.Parameters, iterator.original_block, loc)
+				: base (iterator.parameters, iterator.original_block, loc)
 			{
 				this.iterator = iterator;
 			}
@@ -928,8 +998,8 @@ namespace Mono.CSharp {
 			{
 				method = new Method (
 					iterator, null, TypeManager.system_boolean_expr,
-					Modifiers.PUBLIC, false, new MemberName ("MoveNext"),
-					Parameters.EmptyReadOnlyParameters, null, loc);
+					Modifiers.PUBLIC, false, new MemberName ("MoveNext", loc),
+					Parameters.EmptyReadOnlyParameters, null);
 
 				method.Block = Block;
 
@@ -944,6 +1014,10 @@ namespace Mono.CSharp {
 				return CreateMethodHost (ec);
 			}
 
+			public override Iterator Iterator {
+				get { return iterator; }
+			}
+
 			public override bool IsIterator {
 				get { return true; }
 			}
@@ -976,7 +1050,6 @@ namespace Mono.CSharp {
 
 			protected override void DoEmit (EmitContext ec)
 			{
-				ec.CurrentIterator = iterator;
 				ec.CurrentAnonymousMethod = iterator.move_next_method;
 				ec.InIterator = true;
 
@@ -1063,12 +1136,10 @@ namespace Mono.CSharp {
 		protected class InitScope : Statement
 		{
 			Iterator iterator;
-			CaptureContext cc;
 
-			public InitScope (Iterator iterator, CaptureContext cc, Location loc)
+			public InitScope (Iterator iterator, Location loc)
 			{
 				this.iterator = iterator;
-				this.cc = cc;
 				this.loc = loc;
 			}
 
@@ -1087,12 +1158,12 @@ namespace Mono.CSharp {
 		{
 			Method reset = new Method (
 				this, null, TypeManager.system_void_expr, Modifiers.PUBLIC,
-				false, new MemberName ("Reset"),
-				Parameters.EmptyReadOnlyParameters, null, Location);
+				false, new MemberName ("Reset", Location),
+				Parameters.EmptyReadOnlyParameters, null);
 			AddMethod (reset);
 
 			reset.Block = new ToplevelBlock (Location);
-			reset.Block = new ToplevelBlock (block, parameters.Parameters, Location);
+			reset.Block = new ToplevelBlock (block, parameters, Location);
 			reset.Block.SetHaveAnonymousMethods (Location, move_next_method);
 
 			reset.Block.AddStatement (Create_ThrowNotSupported ());
@@ -1102,22 +1173,18 @@ namespace Mono.CSharp {
 		{
 			dispose = new Method (
 				this, null, TypeManager.system_void_expr, Modifiers.PUBLIC,
-				false, new MemberName ("Dispose"),
-				Parameters.EmptyReadOnlyParameters, null, Location);
+				false, new MemberName ("Dispose", Location),
+				Parameters.EmptyReadOnlyParameters, null);
 			AddMethod (dispose);
 
-			dispose.Block = new ToplevelBlock (block, parameters.Parameters, Location);
+			dispose.Block = new ToplevelBlock (block, parameters, Location);
 			dispose.Block.SetHaveAnonymousMethods (Location, move_next_method);
 
 			dispose.Block.AddStatement (new DisposeMethod (this, Location));
 		}
 
-		public ToplevelBlock Block {
-			get { return block; }
-		}
-
 		public Type IteratorType {
-			get { return iterator_type; }
+			get { return iterator_type_expr.Type; }
 		}
 
 		//
@@ -1139,7 +1206,6 @@ namespace Mono.CSharp {
 				if (Expr == null)
 					return false;
 
-				FlowBranching.UsageVector vector = ec.CurrentBranching.CurrentUsageVector;
 				ec.CurrentBranching.CurrentUsageVector.Return ();
 
 				return true;
@@ -1152,32 +1218,35 @@ namespace Mono.CSharp {
 			}
 		}
 
-		bool CheckType (Type t)
+		bool CheckType ()
 		{
-			if (t == TypeManager.ienumerable_type) {
-				iterator_type = TypeManager.object_type;
+			Type ret = orig_method.ReturnType;
+
+			if (ret == TypeManager.ienumerable_type) {
+				original_iterator_type = TypeManager.object_type;
 				is_enumerable = true;
 				return true;
-			} else if (t == TypeManager.ienumerator_type) {
-				iterator_type = TypeManager.object_type;
+			}
+			if (ret == TypeManager.ienumerator_type) {
+				original_iterator_type = TypeManager.object_type;
 				is_enumerable = false;
 				return true;
 			}
 
-			if (!t.IsGenericInstance)
+			if (!ret.IsGenericInstance)
 				return false;
 
-			Type[] args = TypeManager.GetTypeArguments (t);
+			Type[] args = TypeManager.GetTypeArguments (ret);
 			if (args.Length != 1)
 				return false;
 
-			Type gt = t.GetGenericTypeDefinition ();
+			Type gt = ret.GetGenericTypeDefinition ();
 			if (gt == TypeManager.generic_ienumerable_type) {
-				iterator_type = args [0];
+				original_iterator_type = args [0];
 				is_enumerable = true;
 				return true;
 			} else if (gt == TypeManager.generic_ienumerator_type) {
-				iterator_type = args [0];
+				original_iterator_type = args [0];
 				is_enumerable = false;
 				return true;
 			}
