@@ -85,6 +85,7 @@ namespace Mono.CSharp {
 		public static string StrongNameKeyContainer;
 		public static bool StrongNameDelaySign;
 
+		public static bool BrokenCircularDeps;
 		//
 		// If set, enable XML documentation generation
 		//
@@ -201,7 +202,7 @@ namespace Mono.CSharp {
 		{
 			object o = root.GetDefinition (name);
 			if (o == null){
-				Report.Error (518, "The predefined type `" + name + "' is not defined");
+				Report.Error (518, "The predefined type `" + name + "' is not defined or imported");
 				return null;
 			}
 
@@ -226,7 +227,7 @@ namespace Mono.CSharp {
 		{
 			object o = root.GetDefinition (name);
 			if (o == null){
-				Report.Error (518, "The predefined type `" + name + "' is not defined");
+				Report.Error (518, "The predefined type `" + name + "' is not defined or imported");
 				return;
 			}
 
@@ -251,7 +252,7 @@ namespace Mono.CSharp {
 		{
 			object o = root.GetDefinition (name);
 			if (o == null){
-				Report.Error (518, "The predefined type `" + name + "' is not defined");
+				Report.Error (518, "The predefined type `" + name + "' is not defined or imported");
 				return;
 			}
 
@@ -276,7 +277,7 @@ namespace Mono.CSharp {
 		{
 			object o = root.GetDefinition (name);
 			if (o == null){
-				Report.Error (518, "The predefined type `" + name + "' is not defined");
+				Report.Error (518, "The predefined type `" + name + "' is not defined or imported");
 				return;
 			}
 
@@ -387,7 +388,6 @@ namespace Mono.CSharp {
 				"System.Runtime.InteropServices.DllImportAttribute",
 				"System.Runtime.CompilerServices.MethodImplAttribute",
 				"System.Runtime.InteropServices.MarshalAsAttribute",
-				"System.Runtime.CompilerServices.NewConstraintAttribute",
 				"System.Runtime.CompilerServices.CompilerGeneratedAttribute",
 				"System.Runtime.CompilerServices.FixedBufferAttribute",
 				"System.Diagnostics.ConditionalAttribute",
@@ -407,7 +407,8 @@ namespace Mono.CSharp {
 				"System.MarshalByRefObject",
 				"System.Security.CodeAccessPermission",
 				"System.Runtime.CompilerServices.RequiredAttributeAttribute",
-				"System.Runtime.InteropServices.GuidAttribute"
+				"System.Runtime.InteropServices.GuidAttribute",
+				"System.Reflection.AssemblyCultureAttribute"
 			};
 
 			foreach (string cname in classes_second_stage)
@@ -492,11 +493,6 @@ namespace Mono.CSharp {
 			helper_classes.Add (helper_class);
 		}
 		
-		static void Report1530 (Location loc)
-		{
-			Report.Error (1530, loc, "Keyword new not allowed for namespace elements");
-		}
-		
 		static public void PopulateCoreType (TypeContainer root, string name)
 		{
 			DeclSpace ds = (DeclSpace) root.GetDefinition (name);
@@ -531,6 +527,7 @@ namespace Mono.CSharp {
 				if (RootContext.StdLib){
 					foreach (TypeContainer tc in type_container_resolve_order)
 						tc.DefineMembers (root);
+
 				} else {
 					foreach (TypeContainer tc in type_container_resolve_order) {
 						// When compiling corlib, these types have already been
@@ -549,19 +546,13 @@ namespace Mono.CSharp {
 			ArrayList delegates = root.Delegates;
 			if (delegates != null){
 				foreach (Delegate d in delegates)
-					if ((d.ModFlags & Modifiers.NEW) == 0)
-						d.DefineMembers (root);
-					else
-						Report1530 (d.Location);
+					d.DefineMembers (root);
 			}
 
 			ArrayList enums = root.Enums;
 			if (enums != null){
 				foreach (Enum en in enums)
-					if ((en.ModFlags & Modifiers.NEW) == 0)
-						en.DefineMembers (root);
-					else
-						Report1530 (en.Location);
+					en.DefineMembers (root);
 			}
 
 			//
@@ -594,8 +585,7 @@ namespace Mono.CSharp {
 			ArrayList delegates = root.Delegates;
 			if (delegates != null){
 				foreach (Delegate d in delegates)
-					if ((d.ModFlags & Modifiers.NEW) == 0)
-						d.Define ();
+					d.Define ();
 			}
 
 			if (type_container_resolve_order != null){
@@ -609,16 +599,14 @@ namespace Mono.CSharp {
 					     (tc.Name == "System.Runtime.CompilerServices.IndexerNameAttribute")))
 						continue;
 
-					if ((tc.ModFlags & Modifiers.NEW) == 0)
-						tc.Define ();
+					tc.Define ();
 				}
 			}
 
 			ArrayList enums = root.Enums;
 			if (enums != null){
 				foreach (Enum en in enums)
-					if ((en.ModFlags & Modifiers.NEW) == 0)
-						en.Define ();
+					en.Define ();
 			}
 		}
 
@@ -705,6 +693,14 @@ namespace Mono.CSharp {
 			
 			return fb;
 		}
+
+		public static void CheckUnsafeOption (Location loc)
+		{
+			if (!Unsafe) {
+				Report.Error (227, loc, 
+					"Unsafe code requires the `unsafe' command line option to be specified");
+			}
+		}
 	}
 }
 	      
