@@ -16,6 +16,8 @@ using System.Collections;
 using System.Reflection;
 using System.Reflection.Emit;
 using System.Text;
+using System.Security;
+
 
 namespace Mono.ILASM {
 
@@ -33,6 +35,8 @@ namespace Mono.ILASM {
 		private int typedef_stack_top;
 		private SymbolWriter symwriter;
                 private ICustomAttrTarget current_customattrtarget;
+                private IDeclSecurityTarget current_declsectarget;
+                private PEAPI.NativeType current_field_native_type;
 
                 private byte [] assembly_public_key;
                 private int assembly_major_version;
@@ -42,6 +46,7 @@ namespace Mono.ILASM {
                 private string assembly_locale;
                 private int assembly_hash_algorithm;
                 private ArrayList assembly_custom_attributes;
+                private DeclSecurity assembly_declsec;
                         
                 private TypeManager type_manager;
                 private ExternTable extern_table;
@@ -56,11 +61,13 @@ namespace Mono.ILASM {
                 private int sub_system;
                 private int cor_flags;
                 private long image_base;
+                private long stack_reserve;
 
                 private string output_file;
 		private string debug_file;
                 private bool is_dll;
                 private bool is_assembly;
+                private bool entry_point;
 
                 private string module_name;
 
@@ -89,6 +96,8 @@ namespace Mono.ILASM {
                         sub_system = -1;
                         cor_flags = -1;
                         image_base = -1;
+                        stack_reserve = -1;
+                        entry_point = false;
                 }
 
 		private string CreateDebugFile (string output_file)
@@ -140,6 +149,11 @@ namespace Mono.ILASM {
                         set { current_customattrtarget = value; }
                 }
 
+                public IDeclSecurityTarget CurrentDeclSecurityTarget {
+                        get { return current_declsectarget; }
+                        set { current_declsectarget = value; }
+                }
+
                 public ExternTable ExternTable {
                         get { return extern_table; }
                 }
@@ -148,6 +162,16 @@ namespace Mono.ILASM {
                         get { return type_manager; }
                 }
 
+                public bool HasEntryPoint {
+                        get { return entry_point; }
+                        set { 
+                                /* if (!value) error: unsetting entrypoint ? */
+                                if (entry_point)
+                                        throw new Exception ("Multiple .entrypoint declarations.");
+                                entry_point = value;
+                        }                
+                }
+
                 public void SetSubSystem (int sub_system)
                 {
                         this.sub_system = sub_system;
@@ -163,9 +187,16 @@ namespace Mono.ILASM {
                         this.image_base = image_base;
                 }
 
+                public void SetStackReserve (long stack_reserve)
+                {
+                        this.stack_reserve = stack_reserve;
+                }
+
                 public void SetAssemblyName (string name)
                 {
                         assembly_name = name;
+                        if (assembly_name != "mscorlib")
+                                ExternTable.AddCorlib ();
                 }
 
                 public void SetModuleName (string module_name)
@@ -223,6 +254,7 @@ namespace Mono.ILASM {
                         if (typedef != null) {
                                 // Class head is allready defined, we are just reopening the class
                                 current_customattrtarget = current_typedef = typedef;
+                                current_declsectarget = typedef;
                                 typedef_stack.Add (current_typedef);
 				typedef_stack_top++;
                                 return;
@@ -236,12 +268,23 @@ namespace Mono.ILASM {
 
                         type_manager[cache_name] = typedef;
                         current_customattrtarget = current_typedef = typedef;
+                        current_declsectarget = typedef;
 			typedef_stack.Add (typedef);
 			typedef_stack_top++;
                 }
 
+                public void AddFieldMarshalInfo (PEAPI.NativeType native_type)
+                {
+                        current_field_native_type = native_type;
+                }
+
                 public void AddFieldDef (FieldDef fielddef)
                 {
+                        if (current_field_native_type != null) {
+                                fielddef.AddMarshalInfo (current_field_native_type);
+                                current_field_native_type = null;
+                        }
+
                         if (current_typedef != null) {
                                 current_typedef.AddFieldDef (fielddef);
                         } else {
@@ -282,6 +325,7 @@ namespace Mono.ILASM {
                         }
 
                         current_customattrtarget = current_methoddef = methoddef;
+                        current_declsectarget = methoddef;
                 }
 
                 public void EndMethodDef (Location location)
@@ -307,11 +351,14 @@ namespace Mono.ILASM {
                 public void BeginAssemblyRef (string name, AssemblyName asmb_name)
                 {
                         current_customattrtarget = current_assemblyref = ExternTable.AddAssembly (name, asmb_name);
+                        current_declsectarget = current_assemblyref;
                 }
 
                 public void EndAssemblyRef ()
                 {
                         current_assemblyref = null;
+                        current_customattrtarget = null;
+                        current_declsectarget = null;
                 }
 
                 public void AddToDefineContentsList (TypeDef typedef)
@@ -349,13 +396,40 @@ namespace Mono.ILASM {
                         assembly_custom_attributes.Add (attribute);
                 }
 
+                public void AddAssemblyPermission (PEAPI.SecurityAction sec_action, object perm)
+                {
+                        if (assembly_declsec == null)
+                                assembly_declsec = new DeclSecurity ();
+
+                        PermissionSet ps = perm as PermissionSet;
+                        if (ps == null)
+                                assembly_declsec.AddPermission (sec_action, (IPermission) perm);
+                        else
+                                assembly_declsec.AddPermissionSet (sec_action, ps);
+                }
+
+                public void AddPermission (PEAPI.SecurityAction sec_action, object perm)
+                {
+                        if (CurrentDeclSecurityTarget == null)
+                                return;
+
+                        PermissionSet ps = perm as PermissionSet;
+                        if (ps == null)
+                                CurrentDeclSecurityTarget.AddPermission (sec_action, (IPermission) perm);
+                        else
+                                CurrentDeclSecurityTarget.AddPermissionSet (sec_action, ps);
+                }
+
                 public void Write ()
                 {
                         FileStream out_stream = null;
 
                         try {
+                                if (!is_dll && !HasEntryPoint)
+                                        throw new Exception ("No EntryPoint found.");
+
                                 out_stream = new FileStream (output_file, FileMode.Create, FileAccess.Write);
-                                pefile = new PEFile (assembly_name, module_name, is_dll, is_assembly, out_stream);
+                                pefile = new PEFile (assembly_name, module_name, is_dll, is_assembly, null, out_stream);
                                 PEAPI.Assembly asmb = pefile.GetThisAssembly ();
 
                                 if (file_ref != null)
@@ -385,11 +459,16 @@ namespace Mono.ILASM {
                                         foreach (CustomAttr cattr in assembly_custom_attributes)
                                                 cattr.AddTo (this, asmb);
                                 }
+                                
+                                if (assembly_declsec != null)
+                                        assembly_declsec.AddTo (this, asmb);        
 
                                 if (sub_system != -1)
                                         pefile.SetSubSystem ((PEAPI.SubSystem) sub_system);
                                 if (cor_flags != -1)
                                         pefile.SetCorFlags (cor_flags);
+                                if (stack_reserve != -1)
+                                        pefile.SetStackReserve (stack_reserve);
 
                                 asmb.AddAssemblyInfo(assembly_major_version,
                                                 assembly_minor_version, assembly_build_version,
