@@ -230,8 +230,8 @@ namespace Mono.ILASM {
                         inst_table["mkrefany"] = new ILToken (Token.INSTR_TYPE, TypeOp.mkrefany);
                         inst_table["initobj"] = new ILToken (Token.INSTR_TYPE, TypeOp.initobj);
                         inst_table["sizeof"] = new ILToken (Token.INSTR_TYPE, TypeOp.sizeOf);
-                        inst_table["stelem"] = new ILToken (Token.INSTR_TYPE, TypeOp.stelem);
-                        inst_table["ldelem"] = new ILToken (Token.INSTR_TYPE, TypeOp.ldelem);
+                        // inst_table["stelem"] = new ILToken (Token.INSTR_TYPE, TypeOp.stelem);
+                        // inst_table["ldelem"] = new ILToken (Token.INSTR_TYPE, TypeOp.ldelem);
 
                         //
                         // MethodRef operations
