@@ -11,14 +11,15 @@
 using System;
 using System.Text;
 using System.Collections;
-
+using System.Security;
 
 namespace Mono.ILASM {
 
-        public class MethodDef : ICustomAttrTarget {
+        public class MethodDef : ICustomAttrTarget, IDeclSecurityTarget {
 
                 protected class GenericInfo {
                         public string Id;
+                        public int num;
                         public ArrayList ConstraintList;
                 }
 
@@ -33,6 +34,7 @@ namespace Mono.ILASM {
                 private ArrayList param_list;
                 private ArrayList inst_list;
                 private ArrayList customattr_list;
+                private DeclSecurity decl_sec;
                 private Hashtable label_table;
                 private Hashtable labelref_table;
                 private ArrayList label_list;
@@ -50,11 +52,12 @@ namespace Mono.ILASM {
                 private string pinvoke_name;
                 private PEAPI.PInvokeAttr pinvoke_attr;
 		private SourceMethod source;
+                private PEAPI.NativeType ret_native_type;
 
                 public MethodDef (CodeGen codegen, PEAPI.MethAttr meth_attr,
 				  PEAPI.CallConv call_conv, PEAPI.ImplAttr impl_attr,
 				  string name, ITypeRef ret_type, ArrayList param_list,
-				  Location start)
+				  Location start, ArrayList typars_list)
                 {
                         this.meth_attr = meth_attr;
                         this.call_conv = call_conv;
@@ -78,6 +81,7 @@ namespace Mono.ILASM {
 
                         is_defined = false;
                         is_resolved = false;
+			AddGenericParams (typars_list);
                         CreateSignature ();
 
 			codegen.BeginMethodDef (this);
@@ -148,16 +152,36 @@ namespace Mono.ILASM {
                         pinvoke_info = true;
                 }
 
-                public void AddGenericParam (string id)
+                internal void AddGenericParams (ArrayList typars_list)
                 {
-                        if (typar_list == null)
-                                typar_list = new ArrayList ();
+                        if (typars_list == null)
+                                return;
 
-                        GenericInfo gi = new GenericInfo ();
-                        gi.Id = id;
+			typar_list = new ArrayList ();
+			foreach (string id in typars_list) {
+				GenericInfo gi = new GenericInfo ();
+				gi.Id = id;
+				gi.num = typar_list.Count;
 
-                        typar_list.Add (gi);
+				typar_list.Add (gi);
+			}
                 }
+		
+		public int GenParamCount {
+			get { return (typar_list != null ? typar_list.Count : 0); }
+		}
+
+		public int GetGenericParamNum (string id)
+		{
+			if (typar_list == null)
+				//FIXME: Report error
+				throw new Exception (String.Format ("Invalid method type parameter '{0}'", id));
+
+			foreach (GenericInfo gi in typar_list)
+				if (gi.Id == id)
+					return gi.num;
+			return -1;
+		}
 
                 public void AddGenericConstraint (int index, ITypeRef constraint)
                 {
@@ -183,6 +207,26 @@ namespace Mono.ILASM {
                         customattr_list.Add (customattr);
                 }
 
+                public void AddPermissionSet (PEAPI.SecurityAction sec_action, PermissionSet ps)
+                {
+                        if (decl_sec == null)
+                                decl_sec = new DeclSecurity ();
+                        
+                        decl_sec.AddPermissionSet (sec_action, ps);
+                }
+                
+                public void AddPermission (PEAPI.SecurityAction sec_action, IPermission iper)
+                {
+                        if (decl_sec == null)
+                                decl_sec = new DeclSecurity ();
+                        
+                        decl_sec.AddPermission (sec_action, iper);
+                }
+
+                public void AddRetTypeMarshalInfo (PEAPI.NativeType native_type)
+                {
+                        this.ret_native_type = native_type;
+                }
 
                 public void AddLocals (ArrayList local_list)
                 {
@@ -243,6 +287,8 @@ namespace Mono.ILASM {
 
                 public void EntryPoint ()
                 {
+                        if (!IsStatic)
+                                throw new Exception ("Non-static method as entrypoint.");
                         entry_point = true;
                 }
 
@@ -256,22 +302,42 @@ namespace Mono.ILASM {
                         this.max_stack = max_stack;
                 }
 
+		public void ResolveGenParam (PEAPI.GenParam gpar, TypeDef type_def)
+		{
+			if (gpar.Index != -1)
+				return;
+	
+			if (gpar.Type == PEAPI.GenParamType.MVar)
+				gpar.Index = GetGenericParamNum (gpar.Name); 
+			else
+				gpar.Index = type_def.GetGenericParamNum (gpar.Name);
+
+			if (gpar.Index < 0)
+				/* TODO: Report error */
+				throw new Exception (String.Format ("Invalid {0}type parameter '{1}'", 
+							(gpar.Type == PEAPI.GenParamType.MVar ? "method " : ""),
+							 gpar.Name));
+		}
+
+                public void ResolveGenParams (TypeDef type_def)
+                {
+			GenericTypeRef gtr = ret_type as GenericTypeRef;
+			if (gtr != null)
+				ResolveGenParam ((PEAPI.GenParam) gtr.PeapiType, type_def);
+
+			if (param_list == null)
+				return;
+
+			foreach (ParamDef param in param_list) {
+				gtr = param.Type as GenericTypeRef;
+				if (gtr != null)
+					ResolveGenParam ((PEAPI.GenParam) gtr.PeapiType, type_def);
+			}
+                }
+
                 public PEAPI.MethodDef Resolve (CodeGen code_gen)
                 {
-                        if (is_resolved)
-                                return methoddef;
-
-                        PEAPI.Param [] param_array = GenerateParams (code_gen);
-                        FixAttributes ();
-                        ret_type.Resolve (code_gen);
-
-                        methoddef = code_gen.PEFile.AddMethod (meth_attr, impl_attr,
-                                        name, ret_type.PeapiType, param_array);
-
-                        methoddef.AddCallConv (call_conv);
-                        is_resolved = true;
-
-                        return methoddef;
+                        return Resolve (code_gen, null);
                 }
 
                 public PEAPI.MethodDef Resolve (CodeGen code_gen, PEAPI.ClassDef classdef)
@@ -283,10 +349,18 @@ namespace Mono.ILASM {
                         FixAttributes ();
                         ret_type.Resolve (code_gen);
 
-                        methoddef = classdef.AddMethod (meth_attr, impl_attr,
-                                        name, ret_type.PeapiType, param_array);
+                        if (classdef == null)
+                                methoddef = code_gen.PEFile.AddMethod (meth_attr, impl_attr,
+                                                name, ret_type.PeapiType, param_array);
+                        else			
+                                methoddef = classdef.AddMethod (meth_attr, impl_attr,
+                                                name, ret_type.PeapiType, param_array);
 
                         methoddef.AddCallConv (call_conv);
+
+                        if (ret_native_type != null)
+                                methoddef.AddRetTypeMarshallInfo (ret_native_type);
+
                         is_resolved = true;
 
                         return methoddef;
@@ -382,6 +456,20 @@ namespace Mono.ILASM {
 
                 protected void WriteCode (CodeGen code_gen, PEAPI.MethodDef methoddef)
                 {
+                        /// Add the custrom attributes to this method
+                        if (customattr_list != null)
+                                foreach (CustomAttr customattr in customattr_list) {
+                                        customattr.AddTo (code_gen, methoddef);
+                                        if (customattr.IsSuppressUnmanaged (code_gen))
+                                                methoddef.AddMethAttribute (PEAPI.MethAttr.HasSecurity);
+				}
+
+                        /// Add declarative security to this method
+			if (decl_sec != null) {
+				decl_sec.AddTo (code_gen, methoddef);
+                                methoddef.AddMethAttribute (PEAPI.MethAttr.HasSecurity);
+                        }        
+
                         if (IsAbstract)
                                 return;
 
@@ -412,15 +500,9 @@ namespace Mono.ILASM {
                                 max_stack = 8;
                         methoddef.SetMaxStack (max_stack);
 
-                        /// Add the custrom attributes to this method
-                        if (customattr_list != null)
-                                foreach (CustomAttr customattr in customattr_list)
-                                        customattr.AddTo (code_gen, methoddef);
-
                         if (pinvoke_info) {
                                 methoddef.AddPInvokeInfo (pinvoke_mod.ModuleRef,
                                                 (pinvoke_name != null ? pinvoke_name : name), pinvoke_attr);
-
                         }
 
                         if (inst_list.Count < 1)
@@ -561,10 +643,10 @@ namespace Mono.ILASM {
                         if (IsVararg)
                                 signature = CreateVarargSignature (RetType, name, param_list);
                         else
-                                signature = CreateSignature (RetType, name, param_list);
+                                signature = CreateSignature (RetType, name, param_list, (typar_list != null ? typar_list.Count : 0));
                 }
 
-                public static string CreateSignature (ITypeRef RetType, string name, IList param_list)
+                public static string CreateSignature (ITypeRef RetType, string name, IList param_list, int gen_param_count)
                 {
                         StringBuilder builder = new StringBuilder ();
 
@@ -583,6 +665,7 @@ namespace Mono.ILASM {
                                 }
                         }
                         builder.Append (')');
+			builder.AppendFormat ("`{0}", gen_param_count);
 
                         return builder.ToString ();
                 }
@@ -655,7 +738,7 @@ namespace Mono.ILASM {
                         return builder.ToString ();
                 }
 
-                public static string CreateSignature (ITypeRef RetType, string name, ITypeRef[] param_list)
+                public static string CreateSignature (ITypeRef RetType, string name, ITypeRef[] param_list, int gen_param_count)
                 {
                         StringBuilder builder = new StringBuilder ();
 
@@ -676,6 +759,7 @@ namespace Mono.ILASM {
                                 }
                         }
                         builder.Append (')');
+			builder.AppendFormat ("`{0}", gen_param_count);
 
                         return builder.ToString ();
                 }
