@@ -10,13 +10,15 @@
 
 using System;
 using System.Collections;
+using System.Security;
 
 namespace Mono.ILASM {
 
-        public class TypeDef : ICustomAttrTarget {
+        public class TypeDef : ICustomAttrTarget, IDeclSecurityTarget {
 
                 protected class GenericInfo {
                         public string Id;
+                        public int num;
                         public ArrayList ConstraintList;
                 }
 
@@ -32,6 +34,7 @@ namespace Mono.ILASM {
                 private ArrayList field_list;
                 private Hashtable method_table;
                 private ArrayList customattr_list;
+                private DeclSecurity decl_sec;
                 private ArrayList event_list;
                 private ArrayList property_list;
                 private ArrayList typar_list;
@@ -52,8 +55,6 @@ namespace Mono.ILASM {
                                 IClassRef parent, ArrayList impl_list, Location location)
                 {
                         this.attr = attr;
-                        this.name_space = name_space;
-                        this.name = name;
                         this.parent = parent;
                         this.impl_list = impl_list;
 
@@ -70,6 +71,19 @@ namespace Mono.ILASM {
 
                         is_value_class = false;
                         is_enum_class = false;
+
+
+                        int lastdot = name.LastIndexOf ('.');
+                        if (lastdot >= 0) {
+                                if (name_space == null || name_space == "")
+                                        this.name_space = name.Substring (0, lastdot);
+                                else
+                                        this.name_space = name_space + "." + name.Substring (0, lastdot);
+                                this.name = name.Substring (lastdot + 1);
+                        } else {
+                                this.name_space = name_space;
+                                this.name = name;
+                        }
                 }
 
                 public string Name {
@@ -80,6 +94,10 @@ namespace Mono.ILASM {
                         get { return MakeFullName (); }
                 }
 
+		public string NestedFullName {
+			get { return (outer == null ? FullName : (outer.NestedFullName + "/" + FullName)); }
+		}
+
                 public TypeDef OuterType {
                         get { return outer; }
                         set { outer = value; }
@@ -212,6 +230,22 @@ namespace Mono.ILASM {
                         customattr_list.Add (customattr);
                 }
 
+                public void AddPermissionSet (PEAPI.SecurityAction sec_action, PermissionSet ps)
+                {
+                        if (decl_sec == null)
+                                decl_sec = new DeclSecurity ();
+
+                        decl_sec.AddPermissionSet (sec_action, ps);
+                }
+
+                public void AddPermission (PEAPI.SecurityAction sec_action, IPermission iper)
+                {
+                        if (decl_sec == null)
+                                decl_sec = new DeclSecurity ();
+
+                        decl_sec.AddPermission (sec_action, iper);
+                }
+
                 public void AddGenericParam (string id)
                 {
                         if (typar_list == null)
@@ -219,10 +253,23 @@ namespace Mono.ILASM {
 
                         GenericInfo gi = new GenericInfo ();
                         gi.Id = id;
+                        gi.num = typar_list.Count;
 
                         typar_list.Add (gi);
                 }
 
+		public int GetGenericParamNum (string id)
+		{
+			if (typar_list == null)
+				// FIXME: Report error
+				throw new Exception (String.Format ("Invalid type parameter '{0}'", id));
+
+			foreach (GenericInfo gi in typar_list)
+				if (gi.Id == id)
+					return gi.num;
+			return -1;
+		}
+
                 public void AddGenericConstraint (int index, ITypeRef constraint)
                 {
                         GenericInfo gi = (GenericInfo) typar_list[index];
@@ -242,6 +289,18 @@ namespace Mono.ILASM {
                                 throw new Exception ("Circular definition of class: " + FullName);
                         }
 
+                        if (outer != null) {
+				PEAPI.TypeAttr vis = attr & PEAPI.TypeAttr.VisibilityMask;
+
+				if (vis == PEAPI.TypeAttr.Private || vis == PEAPI.TypeAttr.Public) {
+					/* Nested class, but attr not set accordingly. */
+					//FIXME: 'report' warning here
+					Console.WriteLine ("Warning -- Nested class '{0}' has non-nested visibility, set to such.", NestedFullName);
+					attr = attr ^ vis;
+					attr |= (vis == PEAPI.TypeAttr.Public ? PEAPI.TypeAttr.NestedPublic : PEAPI.TypeAttr.NestedPrivate);
+				}		
+                        }
+                        
                         if (parent != null) {
                                 is_intransit = true;
                                 parent.Resolve (code_gen);
@@ -295,8 +354,8 @@ namespace Mono.ILASM {
                                         classdef.SpecialNoSuper ();
                         }
 
-                        if (size != -1)
-                                classdef.AddLayoutInfo (pack, size);
+                        if (size != -1 || pack != -1)
+                                classdef.AddLayoutInfo ( (pack == -1) ? 1 : pack, (size == -1) ? 0 : size);
 
                         if (impl_list != null) {
                                 foreach (IClassRef impl in impl_list) {
@@ -352,9 +411,18 @@ namespace Mono.ILASM {
                         }
 
                         if (customattr_list != null) {
-                                foreach (CustomAttr customattr in customattr_list)
+                                foreach (CustomAttr customattr in customattr_list) {
                                         customattr.AddTo (code_gen, classdef);
+                                        if (customattr.IsSuppressUnmanaged (code_gen))
+                                                classdef.AddAttribute (PEAPI.TypeAttr.HasSecurity);
+				}
                         }
+                        
+                        /// Add declarative security to this class
+                        if (decl_sec != null) {
+                                decl_sec.AddTo (code_gen, classdef);
+                                classdef.AddAttribute (PEAPI.TypeAttr.HasSecurity);
+			}	
 
                         if (override_list != null) {
                                 foreach (DictionaryEntry entry in override_list) {
@@ -364,7 +432,7 @@ namespace Mono.ILASM {
                                         parent_type.Resolve (code_gen);
                                         string over_name = (string) decl.Value;
                                         IMethodRef over_meth = parent_type.GetMethodRef (body.RetType,
-                                                        body.CallConv, over_name, body.ParamTypeList ());
+                                                        body.CallConv, over_name, body.ParamTypeList (), body.GenParamCount);
                                         over_meth.Resolve (code_gen);
                                         classdef.AddMethodOverride (over_meth.PeapiMethod,
                                                         body.PeapiMethodDef);
