@@ -230,11 +230,15 @@ namespace Mono.MonoBASIC {
 			if (embedded != null)
 				source = (embedded.temp != null) ? embedded.temp : embedded.source;
 			//To support 'Mid' Assignment Statement
-			if(target is Invocation) {
+			if (target is Invocation) {
 
                         	Invocation i = (Invocation) target;
                         	Expression mid_expr;
-                        	mid_expr = i.Expr;
+				if (i.Expr is DecoratedIdentifier)
+					mid_expr = ((DecoratedIdentifier) i.Expr).Id;
+				else
+                        		mid_expr = i.Expr;
+					
 
                         	if (mid_expr is SimpleName) {
                                 	SimpleName sn = mid_expr as SimpleName;
@@ -291,26 +295,27 @@ namespace Mono.MonoBASIC {
 
 				arguments.Add (new Argument (source, Argument.AType.Expression));
 
-				Expression etmp = lateBindingExpr;
-				Type exprType = lateBindingExpr.Type;
-				// Get the target of the invocation/memberAccess
-				if (exprType == null) {
-					if (etmp is Invocation)
-						etmp = ((Invocation) etmp).Expr;
-					if (etmp is MemberAccess)
-						etmp = ((MemberAccess) etmp).Expr;
-					exprType = etmp.Type;
-				}
+				if (lateBindingExpr != null) {
+					Expression etmp = lateBindingExpr;
+					Type exprType = lateBindingExpr.Type;
+					// Get the target of the invocation/memberAccess
+					if (exprType == null) {
+						if (etmp is Invocation)
+							etmp = ((Invocation) etmp).Expr;
+						if (etmp is MemberAccess)
+							etmp = ((MemberAccess) etmp).Expr;
+						exprType = etmp.Type;
+					}
 
-				if (exprType == TypeManager.object_type) {
-					StatementSequence tmp = new StatementSequence (ec.CurrentBlock, loc, lateBindingExpr, 
-										arguments, false, true);
-					if (!tmp.ResolveArguments (ec))
-						return null;
-					tmp.GenerateLateBindingStatements ();
-					return tmp.Resolve (ec);
+					if (exprType == TypeManager.object_type) {
+						StatementSequence tmp = new StatementSequence (ec.CurrentBlock, loc, lateBindingExpr, 
+											arguments, false, true);
+						if (!tmp.ResolveArguments (ec))
+							return null;
+						tmp.GenerateLateBindingStatements ();
+						return tmp.Resolve (ec);
+					}
 				}
-
 				return null;
 			}
 
@@ -331,23 +336,8 @@ namespace Mono.MonoBASIC {
 			// set the `value' field on the property, and Resolve
 			// it.
 			//
-			if (target is PropertyExpr){
-				PropertyExpr property_assign = (PropertyExpr) target;
-
-				if (source_type != target_type){
-					source = ConvertImplicitRequired (ec, source, target_type, loc);
-					if (source == null)
-						return null;
-				}
-
-				//
-				// FIXME: Maybe handle this in the LValueResolve
-				//
-				if (!property_assign.VerifyAssignable ())
-					return null;
-
-				return this;
-			}
+			if (target is PropertyGroupExpr)
+				return target;
 
 			if (target is IndexerAccess) {
 				return this;
@@ -398,6 +388,11 @@ namespace Mono.MonoBASIC {
 					return this;
 				}
 				if (target_type.IsValueType) {
+					if (source_type != target_type) {
+		 				source = ConvertImplicitRequired (ec, source, target_type, loc);
+                        			if (source == null)
+                                			return null;
+					}
 					New n = (New) source;
 					n.ValueTypeVariable = target;
 					return n;
