@@ -33,6 +33,7 @@ namespace Mono.MonoBASIC
 		int line = 0;
 		int col = 1;
 		public int current_token = Token.ERROR;
+		public int last_token = Token.ERROR;
 		bool handle_get_set = false;
 		bool cant_have_a_type_character = false;
 
@@ -76,6 +77,7 @@ namespace Mono.MonoBASIC
 		StringBuilder number;
 		int putback_char = -1;
 		Object val;
+		long lon = 0;
 		
 		//
 		// Details about the error encoutered by the tokenizer
@@ -540,7 +542,16 @@ namespace Mono.MonoBASIC
 				switch (c){
 				case 'S': case 's':
 					t =  Token.LITERAL_INTEGER; // SHORT ?
-					val = ((IConvertible)val).ToInt16(null);
+			
+				// hexadecimal literals - like &H8000S is "-32768" 
+				// and not an overflow exception 
+				// Check for other literals ???
+
+					if(lon == 32768) {
+                                                val = (short) lon;
+					}
+					else 
+						val = ((IConvertible)val).ToInt16(null);
 					break;
 				case 'I': case 'i':
 					t = Token.LITERAL_INTEGER;
@@ -614,7 +625,8 @@ namespace Mono.MonoBASIC
 				} else
 					break;
 			}
-			return System.Int64.Parse (hexNumber.ToString(), NumberStyles.HexNumber);
+			lon = System.Int64.Parse (hexNumber.ToString(), NumberStyles.HexNumber);
+			return lon;
 		}
 
 		long octal_digits ()
@@ -795,7 +807,8 @@ namespace Mono.MonoBASIC
 			
 		public int token ()
 		{
-			int lastToken = current_token;
+			int before_last_token = last_token;
+			last_token = current_token;
 			do
 			{
 				current_token = xtoken ();
@@ -807,11 +820,25 @@ namespace Mono.MonoBASIC
 					 else 
 						return Token.END;
 				}	
+				if (current_token == Token.COLON) {
+					next_token = xtoken();
+					putbacktoken = true;
+					if (next_token == Token.EOL) {
+						if (last_token != Token.LABELNAME && last_token != Token.LITERAL_INTEGER) {
+							current_token = Token.EOL;
+							putbacktoken = false;
+						}
+						else if (before_last_token == Token.GOTO) {
+							current_token = Token.EOL;
+							putbacktoken = false;
+						}
+					}
+				}
 				if (current_token == 0) 
 					return Token.EOF;
 				if (current_token == Token.REM)
 					current_token = DropComments();
-			} while (lastToken == Token.EOL && current_token == Token.EOL);
+			} while (last_token == Token.EOL && current_token == Token.EOL);
 
 			return current_token;
 		}
@@ -825,6 +852,13 @@ namespace Mono.MonoBASIC
 				return null;
 		}
 
+		private bool IsLabel ()
+		{
+			char c = (char) peekChar();
+			//putback (c);
+			return (c == ':');
+		}
+
 		private string GetIdentifier(int c)
 		{
 			StringBuilder id = new StringBuilder ();
@@ -890,8 +924,9 @@ namespace Mono.MonoBASIC
 					int d = peekChar();
 					if (!is_identifier_part_character((char)d)) {
 						while ((c = getChar ()) != -1 && !IsEOL(c)) {}
-						c = getChar ();			
-					}		
+						c = getChar ();
+						tokens_seen = true;
+					}
 				}
 					
 				
@@ -919,11 +954,17 @@ namespace Mono.MonoBASIC
 				// Handle escaped identifiers
 				if (c == '[')
 				{
+					bool is_first_token_in_line = !tokens_seen;
 					if ((val = GetIdentifier()) == null)
 						break;
 					if ((c = getChar()) != ']')
 						break;
 					tokens_seen = true;
+					if (IsLabel() && is_first_token_in_line)
+						return Token.LABELNAME;
+
+					if (last_token == Token.GOTO)
+						return Token.LABELNAME;
 					return Token.IDENTIFIER;
 				}
 
@@ -931,12 +972,19 @@ namespace Mono.MonoBASIC
 				if (is_identifier_start_character ((char) c))
 				{
 					string id;
+					bool is_first_token_in_line = !tokens_seen;
 					if ((id = GetIdentifier(c)) == null)
 						break;
 					val = id;
 					tokens_seen = true;
 					if (is_keyword(id) && (current_token != Token.DOT))
 						return getKeyword(id);
+
+					if (IsLabel() && is_first_token_in_line)
+						return Token.LABELNAME;
+
+					if (last_token == Token.GOTO)
+						return Token.LABELNAME;
 					return Token.IDENTIFIER;
 				}
 
@@ -950,7 +998,7 @@ namespace Mono.MonoBASIC
 
 				if (Char.IsDigit ((char) c))
                                 {
-                                        cant_have_a_type_character = true;
+                                        cant_have_a_type_character = false;
                                         tokens_seen = true;
                                         return is_number (c);
                                 }
