@@ -85,17 +85,17 @@ public class TypeManager {
 	// Expressions representing the internal types.  Used during declaration
 	// definition.
 	//
-	static public Expression system_object_expr, system_string_expr; 
-	static public Expression system_boolean_expr, system_decimal_expr;
-	static public Expression system_single_expr, system_double_expr;
-	static public Expression system_sbyte_expr, system_byte_expr;
-	static public Expression system_int16_expr, system_uint16_expr;
-	static public Expression system_int32_expr, system_uint32_expr;
-	static public Expression system_int64_expr, system_uint64_expr;
-	static public Expression system_char_expr, system_void_expr;
-	static public Expression system_date_expr;
-	static public Expression system_asynccallback_expr;
-	static public Expression system_iasyncresult_expr;
+	static public TypeExpr system_object_expr, system_string_expr; 
+	static public TypeExpr system_boolean_expr, system_decimal_expr;
+	static public TypeExpr system_single_expr, system_double_expr;
+	static public TypeExpr system_sbyte_expr, system_byte_expr;
+	static public TypeExpr system_int16_expr, system_uint16_expr;
+	static public TypeExpr system_int32_expr, system_uint32_expr;
+	static public TypeExpr system_int64_expr, system_uint64_expr;
+	static public TypeExpr system_char_expr, system_void_expr;
+	static public TypeExpr system_date_expr;
+	static public TypeExpr system_asynccallback_expr;
+	static public TypeExpr system_iasyncresult_expr;
 
 	//
 	// This is only used when compiling corlib
@@ -499,9 +499,9 @@ public class TypeManager {
 		standardModules = n;
 	}
 
-	/// 
-	///  Registers a existing 'standard module' to lookup short-qualified or unqualified members
-	/// 
+	// 
+	//  Registers an existing 'standard module' to lookup short-qualified or unqualified members
+	// 
 	private static void AddStandardModule(Type type)
 	{
 	    object[] attributes = type.GetCustomAttributes(false);
@@ -519,23 +519,25 @@ public class TypeManager {
         }
 	}
 
-	//
-	// 
-	//
+	/// <summary>
+	/// Find types from pertinent standard modules (from those namespaces in the scope)
+	/// <summary>
 	public static Type[] GetPertinentStandardModules(params string[] namespaces)
 	{
 		ArrayList list = new ArrayList();
-		foreach(string Namespace in namespaces)
-		{ 
-			for(int i = 0; i < standardModules.Length; i++) {
-				if (standardModules[i].Namespace == Namespace) {
-					string name = standardModules[i].ToString();
-					Type t = LookupType(name);
-					if (t == null) {
-						System.Console.WriteLine("Could not find standard module '" + name + "'"); 
-					}
-					else {
-						list.Add(t);
+		if (standardModules != null && namespaces != null) {
+			foreach(string Namespace in namespaces)
+			{ 
+				for(int i = 0; i < standardModules.Length; i++) {
+					if (standardModules[i].Namespace.ToLower () == Namespace.ToLower ()) {
+						string name = standardModules[i].ToString();
+						Type t = LookupType(name);
+						if (t == null) {
+							System.Console.WriteLine("Could not find standard module '" + name + "'"); 
+						}
+						else {
+							list.Add(t);
+						}
 					}
 				}
 			}
@@ -1931,6 +1933,53 @@ public class TypeManager {
 		}
 	}
 
+	public static TypeExpr TypeToCoreTypeExpr (Type t)
+	{
+		if (t == null)
+			return null;
+
+		TypeCode tc = Type.GetTypeCode (t);
+
+		switch (tc){
+		case TypeCode.Boolean:
+			return TypeManager.system_boolean_expr;
+		case TypeCode.Byte:
+			return TypeManager.system_byte_expr;
+		case TypeCode.SByte:
+			return TypeManager.system_sbyte_expr;
+		case TypeCode.Char:
+			return TypeManager.system_char_expr;
+		case TypeCode.Int16:
+			return TypeManager.system_int16_expr;
+		case TypeCode.UInt16:
+			return TypeManager.system_uint16_expr;
+		case TypeCode.Int32:
+			return TypeManager.system_int32_expr;
+		case TypeCode.UInt32:
+			return TypeManager.system_uint32_expr;
+		case TypeCode.Int64:
+			return TypeManager.system_int64_expr;
+		case TypeCode.UInt64:
+			return TypeManager.system_uint64_expr;
+		case TypeCode.String:
+			return TypeManager.system_string_expr;
+		case TypeCode.Decimal:
+			return TypeManager.system_decimal_expr;
+		case TypeCode.Single:
+			return TypeManager.system_single_expr;
+		case TypeCode.Double:
+			return TypeManager.system_double_expr;
+		case TypeCode.DateTime:
+			return TypeManager.system_date_expr;
+		default:
+			if (t == typeof (void))
+				return TypeManager.system_void_expr;
+			if (t == typeof (object))
+				return TypeManager.system_object_expr;
+			//FIXME : return t; What should be returned here?
+			return null;
+		}
+	}
 	/// <summary>
 	///   Utility function that can be used to probe whether a type
 	///   is managed or not.  
@@ -1967,8 +2016,10 @@ public class TypeManager {
 	///
 	///   For example, the String class indexer is named 'Chars' not 'Item' 
 	/// </remarks>
-	public static string IndexerPropertyName (Type t)
+	public static string IndexerPropertyName (Type t, out Type innerType)
 	{
+		innerType = t;
+		
 		if (t is TypeBuilder) {
 			if (t.IsInterface) {
 				Interface i = LookupInterface (t);
@@ -1980,8 +2031,29 @@ public class TypeManager {
 			} else {
 				TypeContainer tc = LookupTypeContainer (t);
 
-				if ((tc == null) || (tc.DefaultPropName == null))
-					return "Item";
+				if ( tc == null )
+					return null;
+
+				if ( tc.DefaultPropName == null )
+				{
+					while ( tc is Class )
+					{
+						innerType = innerType.BaseType;
+						
+						tc = LookupTypeContainer (innerType);
+						
+						if (tc == null)
+						{	
+							innerType = t;
+							return null;
+						}
+						else if ( tc.DefaultPropName != null )
+							return tc.DefaultPropName;
+					}
+				}
+				
+				//if ((tc == null) || (tc.DefaultPropName == null))
+				//	return "Item";
 
 				return tc.DefaultPropName;
 			}
@@ -2416,8 +2488,9 @@ public class TypeManager {
 			// name
 			//
 			if (list [0] is PropertyInfo) {
-				if ((list.Count == 2) && (list [1] is FieldInfo))
-					return new MemberInfo [] {list [1]};
+				foreach (MemberInfo mi in list)
+					if (mi is FieldInfo)
+						return new MemberInfo [] {mi};
 				return (MemberInfo []) list;
 			}
 
