@@ -20,9 +20,10 @@ using System.Reflection.Emit;
 
 namespace Mono.CSharp {
 
-	public class AnonymousMethod : Expression {
+	public abstract class AnonymousContainer : Expression
+	{
 		// Used to generate unique method names.
-		static int anonymous_method_count;
+		protected static int anonymous_method_count;
 		    
 		// An array list of AnonymousMethodParameter or null
 		public Parameters Parameters;
@@ -42,17 +43,17 @@ namespace Mono.CSharp {
 		//
 		public Method method;
 
-		MethodInfo invoke_mb;
+		protected MethodInfo invoke_mb;
 		
 		// The emit context for the anonymous method
 		public EmitContext aec;
-		public InternalParameters amp;
-		bool unreachable;
+		public Parameters amp;
+		protected bool unreachable;
 
 		//
 		// The modifiers applied to the method, we aggregate them
 		//
-		int method_modifiers = Modifiers.PRIVATE;
+		protected int method_modifiers = Modifiers.PRIVATE;
 		
 		//
 		// During the resolve stage of the anonymous method body,
@@ -64,9 +65,10 @@ namespace Mono.CSharp {
 		//
 		// Points to our container anonymous method if its present
 		//
-		public AnonymousMethod ContainerAnonymousMethod;
-		
-		public AnonymousMethod (Parameters parameters, ToplevelBlock container, ToplevelBlock block, Location l)
+		public AnonymousContainer ContainerAnonymousMethod;	
+
+		protected AnonymousContainer (Parameters parameters, ToplevelBlock container,
+					      ToplevelBlock block, Location l)
 		{
 			Parameters = parameters;
 			Block = block;
@@ -75,10 +77,20 @@ namespace Mono.CSharp {
 			//
 			// The order is important: this setups the CaptureContext tree hierarchy.
 			//
+			if (container == null) {
+				Report.Error (1706, l, "Anonymous methods are not allowed in attribute declaration");
+				return;
+			}
 			container.SetHaveAnonymousMethods (l, this);
 			block.SetHaveAnonymousMethods (l, this);
 		}
 
+		protected AnonymousContainer (Parameters parameters, ToplevelBlock container,
+					      Location l):
+			this (parameters, container, new ToplevelBlock (container, parameters, l), l)
+		{
+		}
+
 		public override Expression DoResolve (EmitContext ec)
 		{
 			//
@@ -97,6 +109,27 @@ namespace Mono.CSharp {
 			return this;
 		}
 
+		protected abstract bool CreateMethodHost (EmitContext ec);
+
+		public abstract void CreateScopeType (EmitContext ec, ScopeInfo scope);
+
+		public abstract bool IsIterator {
+			get;
+		}
+	}
+
+	public class AnonymousMethod : AnonymousContainer
+	{
+		public AnonymousMethod (Parameters parameters, ToplevelBlock container,
+					ToplevelBlock block, Location l)
+			: base (parameters, container, block, l)
+		{
+		}
+
+		public override bool IsIterator {
+			get { return false; }
+		}
+
 		public override void Emit (EmitContext ec)
 		{
 			// nothing, as we only exist to not do anything.
@@ -105,7 +138,7 @@ namespace Mono.CSharp {
 		//
 		// Creates the host for the anonymous method
 		//
-		bool CreateMethodHost (EmitContext ec, Type return_type)
+		protected override bool CreateMethodHost (EmitContext ec)
 		{
 			//
 			// Crude hack follows: we replace the TypeBuilder during the
@@ -121,19 +154,20 @@ namespace Mono.CSharp {
 			
 			if (type_host == null)
 				throw new Exception (String.Format ("Type host is null, Scope is {0}", Scope == null ? "null" : "Not null"));
-			
+
+			if (current_type != type_host)
+				method_modifiers = Modifiers.INTERNAL;
+
 			if (current_type == type_host && ec.IsStatic){
-				if (ec.IsStatic){
-					method_modifiers |= Modifiers.STATIC;
-				}
+				method_modifiers |= Modifiers.STATIC;
 				current_type = null;
 			} 
 
 			method = new Method (
 				(TypeContainer) ec.TypeContainer,
-				new TypeExpression (return_type, loc),
-				method_modifiers, false, new MemberName ("<#AnonymousMethod>" + anonymous_method_count++),
-				Parameters, null, loc);
+				new TypeExpression (invoke_mb.ReturnType, loc),
+				method_modifiers, false, new MemberName ("<#AnonymousMethod>" + anonymous_method_count++, loc),
+				Parameters, null);
 			method.Block = Block;
 			
 			//
@@ -150,7 +184,7 @@ namespace Mono.CSharp {
 		void Error_ParameterMismatch (Type t)
 		{
 			Report.Error (1661, loc, "Anonymous method could not be converted to delegate `" +
-				      "{0}' since there is a parameter mismatch", t);
+				      "{0}' since there is a parameter mismatch", TypeManager.CSharpName (t));
 		}
 
 		//
@@ -167,50 +201,32 @@ namespace Mono.CSharp {
 			invoke_mb = (MethodInfo) Delegate.GetInvokeMethod (ec, delegate_type, loc);
 			ParameterData invoke_pd = TypeManager.GetParameterData (invoke_mb);
 
-			if (Parameters == null){
-				int i, j;
-				
+			if (Parameters == null){				
 				//
 				// We provide a set of inaccessible parameters
 				//
-				int params_idx = -1;
-				for (i = 0; i < invoke_pd.Count; i++){
-					if (invoke_pd.ParameterModifier (i) == Parameter.Modifier.PARAMS)
-						params_idx = i;
-				}
-				int n = invoke_pd.Count - (params_idx != -1 ? 1 : 0);
-				Parameter [] fixedpars = new Parameter [n];
+				Parameter [] fixedpars = new Parameter [invoke_pd.Count];
 				
-				for (i =  j = 0; i < invoke_pd.Count; i++){
-					if (invoke_pd.ParameterModifier (i) == Parameter.Modifier.PARAMS)
-						continue;
-					fixedpars [j] = new Parameter (
-						new TypeExpression (invoke_pd.ParameterType (i), loc),
-						"+" + j, invoke_pd.ParameterModifier (i), null, loc);
-					j++;
+				for (int i = 0; i < invoke_pd.Count; i++){
+					fixedpars [i] = new Parameter (
+						invoke_pd.ParameterType (i),
+						"+" + i, invoke_pd.ParameterModifier (i), null, loc);
 				}
 				
-				Parameter variable = null;
-				if (params_idx != -1){
-					variable = new Parameter (
-						new TypeExpression (invoke_pd.ParameterType (params_idx), loc),
-						"+" + params_idx, invoke_pd.ParameterModifier (params_idx), null, loc);
-				}
-
-				Parameters = new Parameters (fixedpars, variable);
+				Parameters = new Parameters (fixedpars);
 			}
 			
 			//
 			// First, parameter types of `delegate_type' must be compatible
 			// with the anonymous method.
 			//
-			amp = new InternalParameters (Parameters.GetParameterInfo (ec), Parameters);
+			Parameters.Resolve (ec);
+			amp = Parameters;
 			
 			if (amp.Count != invoke_pd.Count){
 				if (!probe){
-					Report.Error (1593, loc, 
-					      "Anonymous method has {0} parameters, while delegate requires {1}",
-					      amp.Count, invoke_pd.Count);
+					Report.Error (1593, loc, "Delegate `{0}' does not take `{1}' arguments",
+						TypeManager.CSharpName (delegate_type), amp.Count.ToString ());
 					Error_ParameterMismatch (delegate_type);
 				}
 				return null;
@@ -219,33 +235,29 @@ namespace Mono.CSharp {
 			for (int i = 0; i < amp.Count; i++){
 				Parameter.Modifier amp_mod = amp.ParameterModifier (i);
 
-				if ((amp_mod & (Parameter.Modifier.OUT | Parameter.Modifier.REF)) != 0){
-					if (!probe){
+				if (!probe) {
+					if ((amp_mod & (Parameter.Modifier.OUT | Parameter.Modifier.REF)) != 0){
+						Report.Error (1677, loc, "Parameter `{0}' should not be declared with the `{1}' keyword", 
+							(i+1).ToString (), Parameter.GetModifierSignature (amp_mod));
 						Error_ParameterMismatch (delegate_type);
-						Report.Error (1677, loc, "Parameter '{0}' should not be declared with the '{1}' keyword", 
-							i+1, amp.ModifierDesc (i));
+						return null;
 					}
-					return null;
-				}
 
-				if (amp_mod != invoke_pd.ParameterModifier (i)){
-					if (!probe){
-						Report.Error (1676, loc, 
-						      "Signature mismatch in parameter modifier for parameter #0", i + 1);
+					if (amp_mod != invoke_pd.ParameterModifier (i)){
+						Report.Error (1676, loc, "Parameter `{0}' must be declared with the `{1}' keyword",
+							(i+1).ToString (), Parameter.GetModifierSignature (invoke_pd.ParameterModifier (i)));
 						Error_ParameterMismatch (delegate_type);
+						return null;
 					}
-					return null;
-				}
 				
-				if (amp.ParameterType (i) != invoke_pd.ParameterType (i)){
-					if (!probe){
-						Report.Error (1678, loc, 
-								      "Signature mismatch in parameter {0}: need `{1}' got `{2}'", i + 1,
-								      TypeManager.CSharpName (invoke_pd.ParameterType (i)),
-								      TypeManager.CSharpName (amp.ParameterType (i)));
+					if (amp.ParameterType (i) != invoke_pd.ParameterType (i)){
+						Report.Error (1678, loc, "Parameter `{0}' is declared as type `{1}' but should be `{2}'",
+							(i+1).ToString (),
+							TypeManager.CSharpName (amp.ParameterType (i)),
+							TypeManager.CSharpName (invoke_pd.ParameterType (i)));
 						Error_ParameterMismatch (delegate_type);
+						return null;
 					}
-					return null;
 				}
 			}
 
@@ -278,27 +290,39 @@ namespace Mono.CSharp {
 			ContainerAnonymousMethod = ec.CurrentAnonymousMethod;
 			ContainingBlock = ec.CurrentBlock;
 
-			if (aec.ResolveTopBlock (ec, Block, amp, loc, out unreachable))
+			if (aec.ResolveTopBlock (ec, Block, amp, null, out unreachable))
 				return new AnonymousDelegate (this, delegate_type, loc).Resolve (ec);
 
 			return null;
 		}
 
+		public override string ExprClassName {
+			get {
+				return "anonymous method";
+			}
+		}
+
 		public MethodBuilder GetMethodBuilder ()
 		{
 			return method.MethodData.MethodBuilder;
 		}
+
+		public override string GetSignatureForError ()
+		{
+			string s = TypeManager.CSharpSignature (invoke_mb);
+			return s.Substring (0, s.IndexOf (".Invoke("));
+		}
 		
 		public bool EmitMethod (EmitContext ec)
 		{
-			if (!CreateMethodHost (ec, invoke_mb.ReturnType))
+			if (!CreateMethodHost (ec))
 				return false;
 
 			MethodBuilder builder = GetMethodBuilder ();
 			ILGenerator ig = builder.GetILGenerator ();
 			aec.ig = ig;
 			
-			Parameters.LabelParameters (aec, builder);
+			Parameters.ApplyAttributes (aec, builder);
 
 			//
 			// Adjust based on the computed state of the
@@ -306,15 +330,39 @@ namespace Mono.CSharp {
 			
 			aec.MethodIsStatic = (method_modifiers & Modifiers.STATIC) != 0;
 			
-			aec.EmitMeta (Block, amp);
+			aec.EmitMeta (Block);
 			aec.EmitResolvedTopBlock (Block, unreachable);
 			return true;
 		}
 
+		public override void CreateScopeType (EmitContext ec, ScopeInfo scope)
+		{
+			TypeBuilder container = ec.TypeContainer.TypeBuilder;
+			string name = String.Format ("<>AnonHelp<{0}>", scope.id);
+
+			scope.ScopeTypeBuilder = container.DefineNestedType (
+				name, TypeAttributes.AutoLayout | TypeAttributes.Class |
+				TypeAttributes.NestedAssembly, TypeManager.object_type, null);
+
+			Type [] constructor_types = TypeManager.NoTypes;
+			scope.ScopeConstructor = scope.ScopeTypeBuilder.DefineConstructor (
+				MethodAttributes.Public | MethodAttributes.HideBySig |
+				MethodAttributes.SpecialName | MethodAttributes.RTSpecialName,
+				CallingConventions.HasThis, constructor_types);
+
+			TypeManager.RegisterMethod (scope.ScopeConstructor, Parameters.EmptyReadOnlyParameters);
+
+			ILGenerator cig = scope.ScopeConstructor.GetILGenerator ();
+			cig.Emit (OpCodes.Ldarg_0);
+			cig.Emit (OpCodes.Call, TypeManager.object_ctor);
+			cig.Emit (OpCodes.Ret);
+		}
+
 		public static void Error_AddressOfCapturedVar (string name, Location loc)
 		{
 			Report.Error (1686, loc,
-				      "Variable {0} is captured in an anonymous method and its address is also being taken: they are exclusive", name);
+				"Local variable `{0}' or its members cannot have their address taken and be used inside an anonymous method block",
+				name);
 		}
 	}
 
@@ -419,8 +467,7 @@ namespace Mono.CSharp {
 		// Points to the object of type `ScopeTypeBuilder' that
 		// holds the data for the scope
 		//
-		public LocalBuilder ScopeInstance;
-
+		LocalBuilder scope_instance;
 		
 		public ScopeInfo (CaptureContext cc, Block b)
 		{
@@ -490,7 +537,7 @@ namespace Mono.CSharp {
 			Console.WriteLine ("NeedThis=" + NeedThis);
 			foreach (LocalInfo li in locals){
 				Pad ();
-				Console.WriteLine ("var {0}", li.Name);
+				Console.WriteLine ("var {0}", MakeFieldName (li.Name));
 			}
 			
 			foreach (ScopeInfo si in children)
@@ -505,36 +552,22 @@ namespace Mono.CSharp {
 			return String.Format ("<>AnonHelp<{0}>", id);
 		}
 
-		public void EmitScopeConstructor ()
+		private string MakeFieldName (string local_name)
 		{
-			Type [] constructor_types = TypeManager.NoTypes;
-			Parameters constructor_parameters = Parameters.EmptyReadOnlyParameters;
-			ScopeConstructor = ScopeTypeBuilder.DefineConstructor (
-				MethodAttributes.Public | MethodAttributes.HideBySig |
-				MethodAttributes.SpecialName | MethodAttributes.RTSpecialName,
-				CallingConventions.HasThis, constructor_types);
-			InternalParameters parameter_info = new InternalParameters (constructor_types, constructor_parameters);
-			TypeManager.RegisterMethod (ScopeConstructor, parameter_info, constructor_types);
-
-			ILGenerator cig = ScopeConstructor.GetILGenerator ();
-			cig.Emit (OpCodes.Ldarg_0);
-			cig.Emit (OpCodes.Call, TypeManager.object_ctor);
-			cig.Emit (OpCodes.Ret);
+			return "<" + id + ":" + local_name + ">";
 		}
-		
+
 		public void EmitScopeType (EmitContext ec)
 		{
-			//EmitDebug ();
+			// EmitDebug ();
 
 			if (ScopeTypeBuilder != null)
 				return;
 			
 			TypeBuilder container = ec.TypeContainer.TypeBuilder;
 
-			ScopeTypeBuilder = container.DefineNestedType (
-				MakeHelperName (), TypeAttributes.AutoLayout | TypeAttributes.Class | TypeAttributes.NestedAssembly,
-				TypeManager.object_type, null);
-
+			CaptureContext.Host.CreateScopeType (ec, this);
+			
 			if (NeedThis)
 				THIS = ScopeTypeBuilder.DefineField ("<>THIS", container, FieldAttributes.Assembly);
 
@@ -542,18 +575,18 @@ namespace Mono.CSharp {
 				if (ParentScope.ScopeTypeBuilder == null){
 					throw new Exception (String.Format ("My parent has not been initialized {0} and {1}", ParentScope, this));
 				}
-				
-				ParentLink = ScopeTypeBuilder.DefineField ("<>parent", ParentScope.ScopeTypeBuilder,
-									   FieldAttributes.Assembly);
+
+				if (ParentScope.ScopeTypeBuilder != ScopeTypeBuilder)
+					ParentLink = ScopeTypeBuilder.DefineField ("<>parent", ParentScope.ScopeTypeBuilder,
+										   FieldAttributes.Assembly);
 			}
 			
 			if (NeedThis && ParentScope != null)
 				throw new Exception ("I was not expecting THIS && having a parent");
 
-			foreach (LocalInfo info in locals){
+			foreach (LocalInfo info in locals)
 				info.FieldBuilder = ScopeTypeBuilder.DefineField (
-					info.Name, info.VariableType, FieldAttributes.Assembly);
-			}
+					MakeFieldName (info.Name), info.VariableType, FieldAttributes.Assembly);
 
 			if (HostsParameters){
 				Hashtable captured_parameters = CaptureContext.captured_parameters;
@@ -568,7 +601,6 @@ namespace Mono.CSharp {
 				}
 			}
 
-			EmitScopeConstructor ();
 			foreach (ScopeInfo si in children){
 				si.EmitScopeType (ec);
 			}
@@ -594,13 +626,20 @@ namespace Mono.CSharp {
 			if (ScopeConstructor == null)
 				throw new Exception ("ScopeConstructor is null for" + this.ToString ());
 			
-			ig.Emit (OpCodes.Newobj, (ConstructorInfo) ScopeConstructor);
-			ScopeInstance = ig.DeclareLocal (ScopeTypeBuilder);
-			ig.Emit (OpCodes.Stloc, ScopeInstance);
+			if (!CaptureContext.Host.IsIterator) {
+				scope_instance = ig.DeclareLocal (ScopeTypeBuilder);
+				ig.Emit (OpCodes.Newobj, (ConstructorInfo) ScopeConstructor);
+				ig.Emit (OpCodes.Stloc, scope_instance);
+			}
 
 			if (THIS != null){
-				ig.Emit (OpCodes.Ldloc, ScopeInstance);
-				ig.Emit (OpCodes.Ldarg_0);
+				if (CaptureContext.Host.IsIterator) {
+					ig.Emit (OpCodes.Ldarg_0);
+					ig.Emit (OpCodes.Ldarg_1);
+				} else {
+					ig.Emit (OpCodes.Ldloc, scope_instance);
+					ig.Emit (OpCodes.Ldarg_0);
+				}
 				ig.Emit (OpCodes.Stfld, THIS);
 			}
 
@@ -608,38 +647,50 @@ namespace Mono.CSharp {
 			// Copy the parameter values, if any
 			//
 			int extra = ec.IsStatic ? 0 : 1;
+			if (CaptureContext.Host.IsIterator)
+				extra++;
 			if (HostsParameters){
 				Hashtable captured_parameters = CaptureContext.captured_parameters;
 				
 				foreach (DictionaryEntry de in captured_parameters){
 					CapturedParameter cp = (CapturedParameter) de.Value;
 
-					ig.Emit (OpCodes.Ldloc, ScopeInstance);
+					EmitScopeInstance (ig);
 					ParameterReference.EmitLdArg (ig, cp.Idx + extra);
 					ig.Emit (OpCodes.Stfld, cp.FieldBuilder);
 				}
 			}
-			
+
 			if (ParentScope != null){
 				if (!ParentScope.inited)
 					ParentScope.EmitInitScope (ec);
-				
-				//
-				// Only emit initialization in our capturecontext world
-				//
-				if (ParentScope.CaptureContext == CaptureContext){
-					ig.Emit (OpCodes.Ldloc, ScopeInstance);
-					ig.Emit (OpCodes.Ldloc, ParentScope.ScopeInstance);
-					ig.Emit (OpCodes.Stfld, ParentLink);
-				} else {
-					ig.Emit (OpCodes.Ldloc, ScopeInstance);
-					ig.Emit (OpCodes.Ldarg_0);
-					ig.Emit (OpCodes.Stfld, ParentLink);
+
+				if (ParentScope.ScopeTypeBuilder != ScopeTypeBuilder) {
+					//
+					// Only emit initialization in our capturecontext world
+					//
+					if (ParentScope.CaptureContext == CaptureContext){
+						EmitScopeInstance (ig);
+						ParentScope.EmitScopeInstance (ig);
+						ig.Emit (OpCodes.Stfld, ParentLink);
+					} else {
+						EmitScopeInstance (ig);
+						ig.Emit (OpCodes.Ldarg_0);
+						ig.Emit (OpCodes.Stfld, ParentLink);
+					}
 				}
 			}
 			inited = true;
 		}
 
+		public void EmitScopeInstance (ILGenerator ig)
+		{
+			if (CaptureContext.Host.IsIterator)
+				ig.Emit (OpCodes.Ldarg_0);
+			else
+				ig.Emit (OpCodes.Ldloc, scope_instance);
+		}
+
 		static void DoPath (StringBuilder sb, ScopeInfo start)
 		{
 			if (start.ParentScope != null){
@@ -693,9 +744,10 @@ namespace Mono.CSharp {
 		Hashtable captured_fields = new Hashtable ();
 		Hashtable captured_variables = new Hashtable ();
 		public Hashtable captured_parameters = new Hashtable ();
-		public AnonymousMethod Host;
+		public AnonymousContainer Host;
 		
-		public CaptureContext (ToplevelBlock toplevel_owner, Location loc, AnonymousMethod host)
+		public CaptureContext (ToplevelBlock toplevel_owner, Location loc,
+				       AnonymousContainer host)
 		{
 			cc_id = count++;
 			this.toplevel_owner = toplevel_owner;
@@ -711,7 +763,18 @@ namespace Mono.CSharp {
 				DoPath (sb, cc.ParentCaptureContext);
 				sb.Append (".");
 			}
-			sb.Append (cc_id.ToString ());
+			sb.Append (cc.cc_id.ToString ());
+		}
+
+		public void ReParent (ToplevelBlock new_toplevel, AnonymousContainer new_host)
+		{
+			toplevel_owner = new_toplevel;
+			Host = new_host;
+
+			for (CaptureContext cc = ParentCaptureContext; cc != null;
+			     cc = cc.ParentCaptureContext) {
+				cc.Host = new_host;
+			}
 		}
 		
 		public override string ToString ()
@@ -775,7 +838,7 @@ namespace Mono.CSharp {
 			throw new Exception ("Should never be reached");
 		}
 
-		void AdjustMethodScope (AnonymousMethod am, ScopeInfo scope)
+		void AdjustMethodScope (AnonymousContainer am, ScopeInfo scope)
 		{
 			am.Scope = Deepest (am.Scope, scope);
 		}
@@ -790,7 +853,7 @@ namespace Mono.CSharp {
 				topmost = parent;
 		}
 		
-		public void AddLocal (AnonymousMethod am, LocalInfo li)
+		public void AddLocal (AnonymousContainer am, LocalInfo li)
 		{
 			if (li.Block.Toplevel != toplevel_owner){
 				ParentCaptureContext.AddLocal (am, li);
@@ -808,7 +871,7 @@ namespace Mono.CSharp {
 				topmost = scope;
 			} else {
 				// Link to parent
-				
+
 				for (Block b = scope.ScopeBlock.Parent; b != null; b = b.Parent){
 					if (scopes [b.ID] != null){
 						LinkScope (scope, b.ID);
@@ -874,7 +937,8 @@ namespace Mono.CSharp {
 		//
 		// Records the captured parameter at the appropriate CaptureContext
 		//
-		public void AddParameter (EmitContext ec, AnonymousMethod am, string name, Type t, int idx)
+		public void AddParameter (EmitContext ec, AnonymousContainer am,
+					  string name, Type t, int idx)
 		{
 			CaptureContext cc = ContextForParameter (ec.CurrentBlock.Toplevel, name);
 
@@ -884,13 +948,12 @@ namespace Mono.CSharp {
 		//
 		// Records the parameters in the context
 		//
-		void AddParameterToContext (AnonymousMethod am, string name, Type t, int idx)
+		public void AddParameterToContext (AnonymousContainer am, string name, Type t, int idx)
 		{
 			if (captured_parameters == null)
 				captured_parameters = new Hashtable ();
-			if (captured_parameters [name] != null)
-				return;
-			captured_parameters [name] = new CapturedParameter (t, idx);
+			if (captured_parameters [name] == null)
+				captured_parameters [name] = new CapturedParameter (t, idx);
 
 			if (topmost == null){
 				//
@@ -924,24 +987,45 @@ namespace Mono.CSharp {
 		// Captured fields are only recorded on the topmost CaptureContext, because that
 		// one is the one linked to the owner of instance fields
 		//
-		public void AddField (FieldExpr fe)
+		public void AddField (EmitContext ec, AnonymousContainer am, FieldExpr fe)
 		{
 			if (fe.FieldInfo.IsStatic)
 				throw new Exception ("Attempt to register a static field as a captured field");
-			
 			CaptureContext parent = ParentCaptureContext;
-			if (parent != null)
-				parent.AddField (fe);
-			else
-				captured_fields [fe] = fe;
+			if (parent != null) {
+				parent.AddField (ec, am, fe);
+				return;
+			}
+
+			if (topmost == null){
+				//
+				// Create one ScopeInfo, if there are none.
+				//
+				topmost = new ScopeInfo (this, toplevel_owner);
+				scopes [toplevel_owner.ID] = topmost;
+			}
+			
+			AdjustMethodScope (am, topmost);
 		}
 
-		public void CaptureThis ()
+		public void CaptureThis (AnonymousContainer am)
 		{
 			CaptureContext parent = ParentCaptureContext;
-			if (parent != null)
-				parent.CaptureThis ();
+			if (parent != null) {
+				parent.CaptureThis (am);
+				return;
+			}
 			referenced_this = true;
+
+			if (topmost == null){
+				//
+				// Create one ScopeInfo, if there are none.
+				//
+				topmost = new ScopeInfo (this, toplevel_owner);
+				scopes [toplevel_owner.ID] = topmost;
+			
+				AdjustMethodScope (am, topmost);
+			}
 		}
 
 		public bool HaveCapturedVariables {
@@ -995,6 +1079,13 @@ namespace Mono.CSharp {
 				topmost.CloseTypes ();
 		}
 
+		public void EmitInitScope (EmitContext ec)
+		{
+			EmitAnonymousHelperClasses (ec);
+			if (topmost != null)
+				topmost.EmitInitScope (ec);
+		}
+
 		ScopeInfo GetScopeFromBlock (EmitContext ec, Block b)
 		{
 			ScopeInfo si;
@@ -1011,20 +1102,25 @@ namespace Mono.CSharp {
 		// Emits the opcodes necessary to load the instance of the captured
 		// variable in `li'
 		//
-		public void EmitCapturedVariableInstance (EmitContext ec, LocalInfo li, AnonymousMethod am)
+		public void EmitCapturedVariableInstance (EmitContext ec, LocalInfo li,
+							  AnonymousContainer am)
 		{
 			ILGenerator ig = ec.ig;
 			ScopeInfo si;
-			
+
 			if (li.Block.Toplevel == toplevel_owner){
 				si = GetScopeFromBlock (ec, li.Block);
-				ig.Emit (OpCodes.Ldloc, si.ScopeInstance);
+				si.EmitScopeInstance (ig);
 				return;
 			}
 
 			si = am.Scope;
 			ig.Emit (OpCodes.Ldarg_0);
 			if (si != null){
+				if (am.IsIterator && (si.ScopeBlock.Toplevel == li.Block.Toplevel)) {
+					return;
+				}
+
 				while (si.ScopeBlock.ID != li.Block.ID){
 					if (si.ParentLink != null)
 						ig.Emit (OpCodes.Ldfld, si.ParentLink);
@@ -1055,7 +1151,7 @@ namespace Mono.CSharp {
 				cc.EmitParameterInstance (ec, name);
 				return;
 			}
-			
+
 			CapturedParameter par_info = (CapturedParameter) captured_parameters [name];
 			if (par_info != null){
 				// 
@@ -1065,15 +1161,15 @@ namespace Mono.CSharp {
 			ILGenerator ig = ec.ig;
 
 			ScopeInfo si;
-			
-			if (ec.CurrentBlock.Toplevel == toplevel_owner){
+
+			if (ec.CurrentBlock.Toplevel == toplevel_owner) {
 				si = GetScopeFromBlock (ec, toplevel_owner);
-				ig.Emit (OpCodes.Ldloc, si.ScopeInstance);
-				return;
+				si.EmitScopeInstance (ig);
+			} else {
+				si = ec.CurrentAnonymousMethod.Scope;
+				ig.Emit (OpCodes.Ldarg_0);
 			}
 
-			si = ec.CurrentAnonymousMethod.Scope;
-			ig.Emit (OpCodes.Ldarg_0);
 			if (si != null){
 				while (si.ParentLink != null) {
 					ig.Emit (OpCodes.Ldfld, si.ParentLink);
@@ -1086,14 +1182,15 @@ namespace Mono.CSharp {
 		// Emits the code necessary to load the parameter named `name' within
 		// an anonymous method.
 		//
-		public void EmitParameter (EmitContext ec, string name)
+		public void EmitParameter (EmitContext ec, string name, bool leave_copy, bool prepared, ref LocalTemporary temp)
 		{
 			CaptureContext cc = ContextForParameter (ec.CurrentBlock.Toplevel, name);
 			if (cc != this){
-				cc.EmitParameter (ec, name);
+				cc.EmitParameter (ec, name, leave_copy, prepared, ref temp);
 				return;
 			}
-			EmitParameterInstance (ec, name);
+			if (!prepared)
+				EmitParameterInstance (ec, name);
 			CapturedParameter par_info = (CapturedParameter) captured_parameters [name];
 			if (par_info != null){
 				// 
@@ -1101,27 +1198,40 @@ namespace Mono.CSharp {
 				//
 			}
 			ec.ig.Emit (OpCodes.Ldfld, par_info.FieldBuilder);
+
+			if (leave_copy){
+				ec.ig.Emit (OpCodes.Dup);
+				temp = new LocalTemporary (ec, par_info.FieldBuilder.FieldType);
+				temp.Store (ec);
+			}
 		}
 
 		//
 		// Implements the assignment of `source' to the paramenter named `name' within
 		// an anonymous method.
 		//
-		public void EmitAssignParameter (EmitContext ec, string name, Expression source, bool leave_copy, bool prepare_for_load)
+		public void EmitAssignParameter (EmitContext ec, string name, Expression source, bool leave_copy, bool prepare_for_load, ref LocalTemporary temp)
 		{
 			CaptureContext cc = ContextForParameter (ec.CurrentBlock.Toplevel, name);
 			if (cc != this){
-				cc.EmitAssignParameter (ec, name, source, leave_copy, prepare_for_load);
+				cc.EmitAssignParameter (ec, name, source, leave_copy, prepare_for_load, ref temp);
 				return;
 			}
 			ILGenerator ig = ec.ig;
 			CapturedParameter par_info = (CapturedParameter) captured_parameters [name];
 
 			EmitParameterInstance (ec, name);
-			source.Emit (ec);
-			if (leave_copy)
+			if (prepare_for_load)
 				ig.Emit (OpCodes.Dup);
+			source.Emit (ec);
+			if (leave_copy){
+				ig.Emit (OpCodes.Dup);
+				temp = new LocalTemporary (ec, par_info.FieldBuilder.FieldType);
+				temp.Store (ec);
+			}
 			ig.Emit (OpCodes.Stfld, par_info.FieldBuilder);
+			if (temp != null)
+				temp.Emit (ec);
 		}
 
 		//
@@ -1144,18 +1254,20 @@ namespace Mono.CSharp {
 		// The following methods are only invoked on the host for the
 		// anonymous method.
 		//
-		public void EmitMethodHostInstance (EmitContext target, AnonymousMethod am)
+		public void EmitMethodHostInstance (EmitContext target, AnonymousContainer am)
 		{
 			ILGenerator ig = target.ig;
 			ScopeInfo si = am.Scope;
-			
-			if (si == null){
+
+			AnonymousContainer container = am.ContainerAnonymousMethod;
+
+			if ((si == null) || ((container != null) && (si == container.Scope))) {
 				ig.Emit (OpCodes.Ldarg_0);
 				return;
 			}
 
 			si.EmitInitScope (target);
-			ig.Emit (OpCodes.Ldloc, si.ScopeInstance);
+			si.EmitScopeInstance (ig);
 		}
 
 		ArrayList all_scopes = new ArrayList ();
