@@ -29,24 +29,56 @@ namespace Mono.CSharp {
 	public class MemberName {
 		public readonly string Name;
 		public readonly MemberName Left;
+		public readonly Location Location;
+
+		bool is_double_colon;
 
 		public static readonly MemberName Null = new MemberName ("");
 
-		public MemberName (string name)
+		private MemberName (MemberName left, string name, bool is_double_colon, Location loc)
 		{
 			this.Name = name;
-		}
-
-		public MemberName (MemberName left, string name)
-			: this (name)
-		{
+			this.Location = loc;
+			this.is_double_colon = is_double_colon;
 			this.Left = left;
 		}
 
-		public MemberName (MemberName left, MemberName right)
+		public MemberName (string name)
+			: this (null, name, false, Location.Null)
 		{
-			Name = right.Name;
-			Left = (right.Left == null) ? left : new MemberName (left, right.Left);
+		}
+
+		public MemberName (MemberName left, string name)
+			: this (left, name, false, left != null ? left.Location : Location.Null)
+		{
+		}
+
+		public MemberName (string name, Location loc)
+			: this (null, name, false, loc)
+		{
+		}
+
+		public MemberName (MemberName left, string name, Location loc)
+			: this (left, name, false, loc)
+		{
+		}
+
+		public MemberName (string alias, string name, Location loc)
+			: this (new MemberName (alias), name, true, loc)
+		{
+		}
+
+		public MemberName (MemberName left, MemberName right)
+			: this (left, right, right.Location)
+		{
+		}
+
+		public MemberName (MemberName left, MemberName right, Location loc)
+			: this (null, right.Name, false, loc)
+		{
+			if (right.is_double_colon)
+				throw new InternalErrorException ("Cannot append double_colon member name");
+			this.Left = (right.Left == null) ? left : new MemberName (left, right.Left);
 		}
 
 		static readonly char [] dot_array = { '.' };
@@ -70,8 +102,9 @@ namespace Mono.CSharp {
 		public string GetName (bool is_generic)
 		{
 			string name = is_generic ? Basename : Name;
+			string connect = is_double_colon ? "::" : ".";
 			if (Left != null)
-				return Left.GetName (is_generic) + "." + name;
+				return Left.GetName (is_generic) + connect + name;
 			else
 				return name;
 		}
@@ -82,49 +115,51 @@ namespace Mono.CSharp {
 		///
 		public string GetPartialName ()
 		{
+			string connect = is_double_colon ? "::" : ".";
 			if (Left != null)
-				return Left.GetPartialName () + "." + Name;
+				return Left.GetPartialName () + connect + Name;
 			else
 				return Name;
 		}
 
 		public string GetTypeName ()
 		{
+			string connect = is_double_colon ? "::" : ".";
 			if (Left != null)
-				return Left.GetTypeName () + "." + Name;
+				return Left.GetTypeName () + connect + Name;
 			else
 				return Name;
 		}
 
-		public Expression GetTypeExpression (Location loc)
+		public Expression GetTypeExpression ()
 		{
-			if (Left != null) {
-				Expression lexpr = Left.GetTypeExpression (loc);
-
-				return new MemberAccess (lexpr, Name, loc);
-			} else {
-				return new SimpleName (Name, loc);
+			if (Left == null)
+				return new SimpleName (Name, Location);
+			if (is_double_colon) {
+				if (Left.Left != null)
+					throw new InternalErrorException ("The left side of a :: should be an identifier");
+				return new QualifiedAliasMember (Left.Name, Name, Location);
 			}
+
+			Expression lexpr = Left.GetTypeExpression ();
+			return new MemberAccess (lexpr, Name, Location);
 		}
 
 		public MemberName Clone ()
 		{
-			if (Left != null)
-				return new MemberName (Left.Clone (), Name);
-			else
-				return new MemberName (Name);
+			MemberName left_clone = Left == null ? null : Left.Clone ();
+			return new MemberName (left_clone, Name, is_double_colon, Location);
 		}
 
 		public string Basename {
-			get {
-				return Name;
-			}
+			get { return Name; }
 		}
 
 		public override string ToString ()
 		{
+			string connect = is_double_colon ? "::" : ".";
 			if (Left != null)
-				return Left + "." + Name;
+				return Left + connect + Name;
 			else
 				return Name;
 		}
@@ -140,6 +175,8 @@ namespace Mono.CSharp {
 				return true;
 			if (other == null || Name != other.Name)
 				return false;
+			if (is_double_colon != other.is_double_colon)
+				return false;
 #if NET_2_0
 			if (TypeArguments == null)
 				return other.TypeArguments == null;
@@ -158,7 +195,8 @@ namespace Mono.CSharp {
 			int hash = Name.GetHashCode ();
 			for (MemberName n = Left; n != null; n = n.Left)
 				hash ^= n.Name.GetHashCode ();
-
+			if (is_double_colon)
+				hash ^= 0xbadc01d;
 #if NET_2_0
 			if (TypeArguments != null)
 				hash ^= TypeArguments.Count << 5;
@@ -202,7 +240,9 @@ namespace Mono.CSharp {
 		/// <summary>
 		///   Location where this declaration happens
 		/// </summary>
-		public readonly Location Location;
+		public Location Location {
+			get { return member_name.Location; }
+		}
 
 		/// <summary>
 		///   XML documentation comment
@@ -228,7 +268,8 @@ namespace Mono.CSharp {
 			Excluded_Undetected = 1 << 8,		// Conditional attribute has not been detected yet
 			Excluded = 1 << 9,					// Method is conditional
 			TestMethodDuplication = 1 << 10,		// Test for duplication must be performed
-			IsUsed = 1 << 11
+			IsUsed = 1 << 11,
+			IsAssigned = 1 << 12				// Field is assigned
 		}
 
 		/// <summary>
@@ -236,8 +277,7 @@ namespace Mono.CSharp {
 		/// </summary>
 		internal Flags caching_flags;
 
-		public MemberCore (TypeContainer parent, MemberName name, Attributes attrs,
-				   Location loc)
+		public MemberCore (TypeContainer parent, MemberName name, Attributes attrs)
 			: base (attrs)
 		{
 			if (parent is PartialContainer && !(this is PartialContainer))
@@ -245,7 +285,6 @@ namespace Mono.CSharp {
 
 			Parent = parent;
 			member_name = name;
-			Location = loc;
 			caching_flags = Flags.Obsolete_Undetected | Flags.ClsCompliance_Undetected | Flags.HasCompliantAttribute_Undetected | Flags.Excluded_Undetected;
 		}
 
@@ -255,21 +294,6 @@ namespace Mono.CSharp {
 			cached_name = null;
 		}
 
-		/// <summary>
-		/// Tests presence of ObsoleteAttribute and report proper error
-		/// </summary>
-		protected void CheckUsageOfObsoleteAttribute (Type type)
-		{
-			if (type == null)
-				return;
-
-			ObsoleteAttribute obsolete_attr = AttributeTester.GetObsoleteAttribute (type);
-			if (obsolete_attr == null)
-				return;
-
-			AttributeTester.Report_ObsoleteMessage (obsolete_attr, type.FullName, Location);
-		}
-
 		public abstract bool Define ();
 
 		// 
@@ -277,15 +301,10 @@ namespace Mono.CSharp {
 		//
 		public virtual string GetSignatureForError ()
 		{
-			return Name;
-		}
+			if (Parent == null || Parent.Parent == null)
+				return Name;
 
-		/// <summary>
-		/// Use this method when MethodBuilder is null
-		/// </summary>
-		public virtual string GetSignatureForError (TypeContainer tc)
-		{
-			return Name;
+			return String.Concat (Parent.GetSignatureForError (), '.', Name);
 		}
 
 		/// <summary>
@@ -293,16 +312,19 @@ namespace Mono.CSharp {
 		/// </summary>
 		public virtual void Emit ()
 		{
-			// Hack with Parent == null is for EnumMember
-			if (Parent == null || (GetObsoleteAttribute (Parent) == null && Parent.GetObsoleteAttribute (Parent) == null))
-				VerifyObsoleteAttribute ();
-
 			if (!RootContext.VerifyClsCompliance)
 				return;
 
 			VerifyClsCompliance (Parent);
 		}
 
+		public virtual EmitContext EmitContext
+		{
+			get {
+				return Parent.EmitContext;
+			}
+		}
+
 		public bool InUnsafe {
 			get {
 				return ((ModFlags & Modifiers.UNSAFE) != 0) || Parent.UnsafeContext;
@@ -341,7 +363,7 @@ namespace Mono.CSharp {
 		/// <summary>
 		/// Returns instance of ObsoleteAttribute for this MemberCore
 		/// </summary>
-		public ObsoleteAttribute GetObsoleteAttribute (DeclSpace ds)
+		public virtual ObsoleteAttribute GetObsoleteAttribute ()
 		{
 			// ((flags & (Flags.Obsolete_Undetected | Flags.Obsolete)) == 0) is slower, but why ?
 			if ((caching_flags & Flags.Obsolete_Undetected) == 0 && (caching_flags & Flags.Obsolete) == 0) {
@@ -354,11 +376,11 @@ namespace Mono.CSharp {
 				return null;
 
 			Attribute obsolete_attr = OptAttributes.Search (
-				TypeManager.obsolete_attribute_type, ds.EmitContext);
+				TypeManager.obsolete_attribute_type, EmitContext);
 			if (obsolete_attr == null)
 				return null;
 
-			ObsoleteAttribute obsolete = obsolete_attr.GetObsoleteAttribute (ds.EmitContext);
+			ObsoleteAttribute obsolete = obsolete_attr.GetObsoleteAttribute (EmitContext);
 			if (obsolete == null)
 				return null;
 
@@ -366,10 +388,38 @@ namespace Mono.CSharp {
 			return obsolete;
 		}
 
+		/// <summary>
+		/// Checks for ObsoleteAttribute presence. It's used for testing of all non-types elements
+		/// </summary>
+		public virtual void CheckObsoleteness (Location loc)
+		{
+			if (Parent != null)
+				Parent.CheckObsoleteness (loc);
+
+			ObsoleteAttribute oa = GetObsoleteAttribute ();
+			if (oa == null) {
+				return;
+			}
+
+			AttributeTester.Report_ObsoleteMessage (oa, GetSignatureForError (), loc);
+		}
+
+		protected void CheckObsoleteType (Expression type)
+		{
+			ObsoleteAttribute obsolete_attr = AttributeTester.GetObsoleteAttribute (type.Type);
+			if (obsolete_attr == null)
+				return;
+
+			if (GetObsoleteAttribute () != null || Parent.GetObsoleteAttribute () != null)
+				return;
+
+			AttributeTester.Report_ObsoleteMessage (obsolete_attr, TypeManager.CSharpName (type.Type), type.Location);
+		}
+
 		/// <summary>
 		/// Analyze whether CLS-Compliant verification must be execute for this MemberCore.
 		/// </summary>
-		public override bool IsClsCompliaceRequired (DeclSpace container)
+		public override bool IsClsComplianceRequired (DeclSpace container)
 		{
 			if ((caching_flags & Flags.ClsCompliance_Undetected) == 0)
 				return (caching_flags & Flags.ClsCompliant) != 0;
@@ -442,32 +492,31 @@ namespace Mono.CSharp {
 		/// </summary>
 		protected virtual bool VerifyClsCompliance (DeclSpace ds)
 		{
-			if (!IsClsCompliaceRequired (ds)) {
+			if (!IsClsComplianceRequired (ds)) {
 				if (HasClsCompliantAttribute && RootContext.WarningLevel >= 2) {
 					if (!IsExposedFromAssembly (ds))
-						Report.Warning (3019, Location, "CLS compliance checking will not be performed on '{0}' because it is private or internal", GetSignatureForError ());
+						Report.Warning (3019, 2, Location, "CLS compliance checking will not be performed on `{0}' because it is not visible from outside this assembly", GetSignatureForError ());
 					if (!CodeGen.Assembly.IsClsCompliant)
-						Report.Warning (3021, Location, "'{0}' does not need a CLSCompliant attribute because the assembly does not have a CLSCompliant attribute", GetSignatureForError ());
+						Report.Warning (3021, 2, Location, "`{0}' does not need a CLSCompliant attribute because the assembly is not marked as CLS-compliant", GetSignatureForError ());
 				}
 				return false;
 			}
 
 			if (!CodeGen.Assembly.IsClsCompliant) {
 				if (HasClsCompliantAttribute) {
-					Report.Error (3014, Location, "'{0}' cannot be marked as CLS-compliant because the assembly does not have a CLSCompliant attribute", GetSignatureForError ());
+					Report.Error (3014, Location,
+						"`{0}' cannot be marked as CLS-compliant because the assembly is not marked as CLS-compliant",
+						GetSignatureForError ());
 				}
 				return false;
 			}
 
-			int index = Name.LastIndexOf ('.');
-			if (Name [index > 0 ? index + 1 : 0] == '_') {
-				Report.Error (3008, Location, "Identifier '{0}' is not CLS-compliant", GetSignatureForError () );
+			if (member_name.Name [0] == '_') {
+				Report.Error (3008, Location, "Identifier `{0}' is not CLS-compliant", GetSignatureForError () );
 			}
 			return true;
 		}
 
-		protected abstract void VerifyObsoleteAttribute ();
-
 		//
 		// Raised (and passed an XmlElement that contains the comment)
 		// when GenerateDocComment is writing documentation expectedly.
@@ -506,7 +555,7 @@ namespace Mono.CSharp {
 	///   provides the common foundation for managing those name
 	///   spaces.
 	/// </remarks>
-	public abstract class DeclSpace : MemberCore, IAlias {
+	public abstract class DeclSpace : MemberCore {
 		/// <summary>
 		///   This points to the actual definition that is being
 		///   created with System.Reflection.Emit
@@ -528,15 +577,17 @@ namespace Mono.CSharp {
 		// The emit context for toplevel objects.
 		protected EmitContext ec;
 		
-		public EmitContext EmitContext {
-			get { return ec; }
+		public override EmitContext EmitContext {
+			get {
+				return ec;
+			}
 		}
 
 		static string[] attribute_targets = new string [] { "type" };
 
 		public DeclSpace (NamespaceEntry ns, TypeContainer parent, MemberName name,
-				  Attributes attrs, Location l)
-			: base (parent, name, attrs, l)
+				  Attributes attrs)
+			: base (parent, name, attrs)
 		{
 			NamespaceEntry = ns;
 			Basename = name.Name;
@@ -550,7 +601,7 @@ namespace Mono.CSharp {
 		{
 			if (name == Basename && !(this is Interface) && !(this is Enum)) {
 				Report.SymbolRelatedToPreviousError (this);
-				Report.Error (542,  symbol.Location, "'{0}': member names cannot be the same as their enclosing type", symbol.GetSignatureForError ());
+				Report.Error (542,  symbol.Location, "`{0}': member names cannot be the same as their enclosing type", symbol.GetSignatureForError ());
 				return false;
 			}
 
@@ -565,7 +616,21 @@ namespace Mono.CSharp {
 				return true;
 
 			Report.SymbolRelatedToPreviousError (mc);
-			Report.Error (102, symbol.Location, "The type '{0}' already contains a definition for '{1}'", GetSignatureForError (), name);
+			if (symbol is PartialContainer || mc is PartialContainer) {
+				Report.Error (260, symbol.Location,
+					"Missing partial modifier on declaration of type `{0}'. Another partial declaration of this type exists",
+					name);
+				return false;
+			}
+
+			if (this is RootTypes) {
+				Report.Error (101, symbol.Location, 
+					"The namespace `{0}' already contains a definition for `{1}'",
+					((DeclSpace)symbol).NamespaceEntry.GetSignatureForError (), symbol.MemberName.Name);
+			} else {
+				Report.Error (102, symbol.Location, "The type `{0}' already contains a definition for `{1}'",
+					GetSignatureForError (), symbol.MemberName.Name);
+			}
 			return false;
 		}
 
@@ -585,13 +650,7 @@ namespace Mono.CSharp {
 		// why there is a non-obvious test down here.
 		//
 		public bool IsTopLevel {
-			get {
-				if (Parent != null){
-					if (Parent.Parent == null)
-						return true;
-				}
-				return false;
-			}
+			get { return (Parent != null && Parent.Parent == null); }
 		}
 
 		public virtual void CloseType ()
@@ -616,9 +675,7 @@ namespace Mono.CSharp {
 		}
 
 		protected virtual TypeAttributes TypeAttr {
-			get {
-				return CodeGen.Module.DefaultCharSetType;
-			}
+			get { return CodeGen.Module.DefaultCharSetType; }
 		}
 
 		/// <remarks>
@@ -630,7 +687,20 @@ namespace Mono.CSharp {
 		///   Define all members, but don't apply any attributes or do anything which may
 		///   access not-yet-defined classes.  This method also creates the MemberCache.
 		/// </summary>
-		public abstract bool DefineMembers (TypeContainer parent);
+		public virtual bool DefineMembers (TypeContainer parent)
+		{
+			if (((ModFlags & Modifiers.NEW) != 0) && IsTopLevel) {
+				Report.Error (1530, Location, "Keyword `new' is not allowed on namespace elements");
+				return false;
+			}
+			return true;
+		}
+
+		public override string GetSignatureForError ()
+		{	
+			// Parent.GetSignatureForError
+			return Name;
+		}
 
 		//
 		// Whether this is an `unsafe context'
@@ -669,8 +739,6 @@ namespace Mono.CSharp {
 		public TypeExpr ResolveBaseTypeExpr (Expression e, bool silent, Location loc)
 		{
 			TypeResolveEmitContext.loc = loc;
-			TypeResolveEmitContext.ContainerType = TypeBuilder;
-
 			return e.ResolveAsTypeTerminal (TypeResolveEmitContext, silent);
 		}
 		
@@ -845,13 +913,6 @@ namespace Mono.CSharp {
 			return ~ (~ mAccess | pAccess) == 0;
 		}
 
-		public static void Error_AmbiguousTypeReference (Location loc, string name, string t1, string t2)
-		{
-			Report.Error (104, loc,
-				      "`{0}' is an ambiguous reference ({1} or {2})",
-				      name, t1, t2);
-		}
-
 		//
 		// Return the nested type with name @name.  Ensures that the nested type
 		// is defined if necessary.  Do _not_ use this when you have a MemberCache handy.
@@ -890,8 +951,7 @@ namespace Mono.CSharp {
 		}
 
 		//
-		// Public function used to locate types, this can only
-		// be used after the ResolveTree function has been invoked.
+		// Public function used to locate types.
 		//
 		// Set 'ignore_cs0104' to true if you want to ignore cs0104 errors.
 		//
@@ -982,25 +1042,37 @@ namespace Mono.CSharp {
 		}
 
 		public override string[] ValidAttributeTargets {
-			get {
-				return attribute_targets;
-			}
+			get { return attribute_targets; }
 		}
 
-		bool IAlias.IsType {
-			get { return true; }
-		}
-
-		string IAlias.Name {
-			get { return Name; }
-		}
-
-		TypeExpr IAlias.ResolveAsType (EmitContext ec)
+		protected override bool VerifyClsCompliance (DeclSpace ds)
 		{
-			if (TypeBuilder == null)
-				throw new InvalidOperationException ();
+			if (!base.VerifyClsCompliance (ds)) {
+				return false;
+			}
 
-			return new TypeExpression (TypeBuilder, Location);
+			IDictionary cache = TypeManager.AllClsTopLevelTypes;
+			string lcase = Name.ToLower (System.Globalization.CultureInfo.InvariantCulture);
+			if (!cache.Contains (lcase)) {
+				cache.Add (lcase, this);
+				return true;
+			}
+
+			object val = cache [lcase];
+			if (val == null) {
+				Type t = AttributeTester.GetImportedIgnoreCaseClsType (lcase);
+				if (t == null)
+					return true;
+				Report.SymbolRelatedToPreviousError (t);
+			}
+			else {
+				if (val is PartialContainer)
+					return true;
+
+				Report.SymbolRelatedToPreviousError ((DeclSpace)val);
+			}
+			Report.Error (3005, Location, "Identifier `{0}' differing only in case is not CLS-compliant", GetSignatureForError ());
+			return true;
 		}
 	}
 
@@ -1746,7 +1818,7 @@ namespace Mono.CSharp {
 					}
 				} else {
 					mi = (MethodInfo) entry.Member;
-					cmpAttrs = TypeManager.GetArgumentTypes (mi);
+					cmpAttrs = TypeManager.GetParameterData (mi).Types;
 				}
 
 				if (fi != null) {
@@ -1958,7 +2030,7 @@ namespace Mono.CSharp {
  		
 				MethodBase method_to_compare = (MethodBase)entry.Member;
 				AttributeTester.Result result = AttributeTester.AreOverloadedMethodParamsClsCompliant (
-					method.ParameterTypes, TypeManager.GetArgumentTypes (method_to_compare));
+					method.ParameterTypes, TypeManager.GetParameterData (method_to_compare).Types);
 
  				if (result == AttributeTester.Result.Ok)
  					continue;
@@ -1967,16 +2039,16 @@ namespace Mono.CSharp {
 
 				// TODO: now we are ignoring CLSCompliance(false) on method from other assembly which is buggy.
 				// However it is exactly what csc does.
-				if (md != null && !md.IsClsCompliaceRequired (method.Parent))
+				if (md != null && !md.IsClsComplianceRequired (method.Parent))
 					continue;
  		
  				Report.SymbolRelatedToPreviousError (entry.Member);
 				switch (result) {
 					case AttributeTester.Result.RefOutArrayError:
-						Report.Error (3006, method.Location, "Overloaded method '{0}' differing only in ref or out, or in array rank, is not CLS-compliant", method.GetSignatureForError ());
+						Report.Error (3006, method.Location, "Overloaded method `{0}' differing only in ref or out, or in array rank, is not CLS-compliant", method.GetSignatureForError ());
 						continue;
 					case AttributeTester.Result.ArrayArrayError:
-						Report.Error (3007, method.Location, "Overloaded method '{0}' differing only by unnamed array types is not CLS-compliant", method.GetSignatureForError ());
+						Report.Error (3007, method.Location, "Overloaded method `{0}' differing only by unnamed array types is not CLS-compliant", method.GetSignatureForError ());
 						continue;
 				}
 
