@@ -73,7 +73,7 @@ namespace Mono.CSharp {
 					c.GenerateDocComment (t);
 
 			if (t.Fields != null)
-				foreach (Field f in t.Fields)
+				foreach (FieldBase f in t.Fields)
 					f.GenerateDocComment (t);
 
 			if (t.Events != null)
@@ -131,7 +131,7 @@ namespace Mono.CSharp {
 					lineHead, split, 0, j);
 				return el;
 			} catch (XmlException ex) {
-				Report.Warning (1570, 1, mc.Location, "XML comment on '{0}' has non-well-formed XML ({1}).", name, ex.Message);
+				Report.Warning (1570, 1, mc.Location, "XML comment on `{0}' has non-well-formed XML ({1})", name, ex.Message);
 				XmlComment com = doc.CreateComment (String.Format ("FIXME: Invalid documentation markup was found for member {0}", name));
 				return com;
 			}
@@ -176,7 +176,7 @@ namespace Mono.CSharp {
 				Constructor c = mc as Constructor;
 				if (c == null || !c.IsDefault ())
 					Report.Warning (1591, 4, mc.Location,
-						"Missing XML comment for publicly visible type or member '{0}'", mc.GetSignatureForError ());
+						"Missing XML comment for publicly visible type or member `{0}'", mc.GetSignatureForError ());
 			}
 		}
 
@@ -189,11 +189,11 @@ namespace Mono.CSharp {
 			string file = el.GetAttribute ("file");
 			string path = el.GetAttribute ("path");
 			if (file == "") {
-				Report.Warning (1590, 1, mc.Location, "Invalid XML 'include' element; Missing 'file' attribute.");
+				Report.Warning (1590, 1, mc.Location, "Invalid XML `include' element. Missing `file' attribute");
 				el.ParentNode.InsertBefore (el.OwnerDocument.CreateComment (" Include tag is invalid "), el);
 			}
 			else if (path == "") {
-				Report.Warning (1590, 1, mc.Location, "Invalid XML 'include' element; Missing 'path' attribute.");
+				Report.Warning (1590, 1, mc.Location, "Invalid XML `include' element. Missing `path' attribute");
 				el.ParentNode.InsertBefore (el.OwnerDocument.CreateComment (" Include tag is invalid "), el);
 			}
 			else {
@@ -204,8 +204,8 @@ namespace Mono.CSharp {
 						doc.Load (file);
 						RootContext.Documentation.StoredDocuments.Add (file, doc);
 					} catch (Exception) {
-						el.ParentNode.InsertBefore (el.OwnerDocument.CreateComment (String.Format (" Badly formed XML in at comment file '{0}': cannot be included ", file)), el);
-						Report.Warning (1592, 1, mc.Location, "Badly formed XML in included comments file -- '{0}'", file);
+						el.ParentNode.InsertBefore (el.OwnerDocument.CreateComment (String.Format (" Badly formed XML in at comment file `{0}': cannot be included ", file)), el);
+						Report.Warning (1592, 1, mc.Location, "Badly formed XML in included comments file -- `{0}'", file);
 					}
 				}
 				bool keepIncludeNode = false;
@@ -221,7 +221,7 @@ namespace Mono.CSharp {
 							el.ParentNode.InsertBefore (el.OwnerDocument.ImportNode (n, true), el);
 					} catch (Exception ex) {
 						el.ParentNode.InsertBefore (el.OwnerDocument.CreateComment (" Failed to insert some or all of included XML "), el);
-						Report.Warning (1589, 1, mc.Location, "Unable to include XML fragment '{0}' of file {1} -- {2}.", path, file, ex.Message);
+						Report.Warning (1589, 1, mc.Location, "Unable to include XML fragment `{0}' of file `{1}' ({2})", path, file, ex.Message);
 					}
 				}
 				if (!keepIncludeNode)
@@ -321,7 +321,7 @@ namespace Mono.CSharp {
 			if (e != null) {
 				if (!(e is TypeExpr))
 					return null;
-				return ((TypeExpr) e).Type;
+				return e.Type;
 			}
 			int index = identifier.LastIndexOf ('.');
 			if (index < 0)
@@ -333,42 +333,163 @@ namespace Mono.CSharp {
 			// no need to detect warning 419 here
 			return FindDocumentedMember (mc, parent,
 				identifier.Substring (index + 1),
-				emptyParamList,
-				ds, out warn, cref) as Type;
+				null, ds, out warn, cref, false, null).Member as Type;
+		}
+
+		private static MemberInfo [] empty_member_infos =
+			new MemberInfo [0];
+
+		private static MemberInfo [] FindMethodBase (Type type,
+			BindingFlags bindingFlags, MethodSignature signature)
+		{
+			MemberList ml = TypeManager.FindMembers (
+				type,
+				MemberTypes.Constructor | MemberTypes.Method | MemberTypes.Property | MemberTypes.Custom,
+				bindingFlags,
+				MethodSignature.method_signature_filter,
+				signature);
+			if (ml == null)
+				return empty_member_infos;
+
+			return FilterOverridenMembersOut (type, (MemberInfo []) ml);
+		}
+
+		static bool IsOverride (PropertyInfo deriv_prop, PropertyInfo base_prop)
+		{
+			if (!Invocation.IsAncestralType (base_prop.DeclaringType, deriv_prop.DeclaringType))
+				return false;
+
+			Type [] deriv_pd = TypeManager.GetArgumentTypes (deriv_prop);
+			Type [] base_pd = TypeManager.GetArgumentTypes (base_prop);
+		
+			if (deriv_pd.Length != base_pd.Length)
+				return false;
+
+			for (int j = 0; j < deriv_pd.Length; ++j) {
+				if (deriv_pd [j] != base_pd [j])
+					return false;
+				Type ct = TypeManager.TypeToCoreType (deriv_pd [j]);
+				Type bt = TypeManager.TypeToCoreType (base_pd [j]);
+
+				if (ct != bt)
+					return false;
+			}
+
+			return true;
+		}
+
+		private static MemberInfo [] FilterOverridenMembersOut (
+			Type type, MemberInfo [] ml)
+		{
+			if (ml == null)
+				return empty_member_infos;
+
+			ArrayList al = new ArrayList (ml.Length);
+			for (int i = 0; i < ml.Length; i++) {
+				MethodBase mx = ml [i] as MethodBase;
+				PropertyInfo px = ml [i] as PropertyInfo;
+				if (mx != null || px != null) {
+					bool overriden = false;
+					for (int j = 0; j < ml.Length; j++) {
+						if (j == i)
+							continue;
+						MethodBase my = ml [j] as MethodBase;
+						if (mx != null && my != null &&
+							Invocation.IsOverride (my, mx)) {
+							overriden = true;
+							break;
+						}
+						else if (mx != null)
+							continue;
+						PropertyInfo py = ml [j] as PropertyInfo;
+						if (px != null && py != null &&
+							IsOverride (py, px)) {
+							overriden = true;
+							break;
+						}
+					}
+					if (overriden)
+						continue;
+				}
+				al.Add (ml [i]);
+			}
+			return al.ToArray (typeof (MemberInfo)) as MemberInfo [];
+		}
+
+		struct FoundMember
+		{
+			public static FoundMember Empty = new FoundMember (true);
+
+			public bool IsEmpty;
+			public readonly MemberInfo Member;
+			public readonly Type Type;
+
+			public FoundMember (bool regardlessOfThisValueItsEmpty)
+			{
+				IsEmpty = true;
+				Member = null;
+				Type = null;
+			}
+
+			public FoundMember (Type foundType, MemberInfo member)
+			{
+				IsEmpty = false;
+				Type = foundType;
+				Member = member;
+			}
 		}
 
 		//
 		// Returns a MemberInfo that is referenced in XML documentation
 		// (by "see" or "seealso" elements).
 		//
-		private static MemberInfo FindDocumentedMember (MemberCore mc,
+		private static FoundMember FindDocumentedMember (MemberCore mc,
 			Type type, string memberName, Type [] paramList, 
-			DeclSpace ds, out int warningType, string cref)
+			DeclSpace ds, out int warningType, string cref,
+			bool warn419, string nameForError)
+		{
+			for (; type != null; type = type.DeclaringType) {
+				MemberInfo mi = FindDocumentedMemberNoNest (
+					mc, type, memberName, paramList, ds,
+					out warningType, cref, warn419,
+					nameForError);
+				if (mi != null)
+					return new FoundMember (type, mi);
+			}
+			warningType = 0;
+			return FoundMember.Empty;
+		}
+
+		private static MemberInfo FindDocumentedMemberNoNest (
+			MemberCore mc, Type type, string memberName,
+			Type [] paramList, DeclSpace ds, out int warningType, 
+			string cref, bool warn419, string nameForError)
 		{
 			warningType = 0;
-			MethodSignature msig = new MethodSignature (memberName, null, paramList);
-			MemberInfo [] mis = type.FindMembers (
-				MemberTypes.All,
-				BindingFlags.Public | BindingFlags.NonPublic | BindingFlags.Static | BindingFlags.Instance,
-				MethodSignature.method_signature_filter,
-				msig);
-			if (mis.Length > 0) {
-				if (IsAmbiguous (mis))
-					warningType = 419;
+			MemberInfo [] mis;
+
+			if (paramList == null) {
+				// search for fields/events etc.
+				mis = TypeManager.MemberLookup (type, null,
+					type, MemberTypes.All,
+					BindingFlags.Public | BindingFlags.NonPublic | BindingFlags.Static | BindingFlags.Instance,
+					memberName, null);
+				mis = FilterOverridenMembersOut (type, mis);
+				if (mis == null || mis.Length == 0)
+					return null;
+				if (warn419 && IsAmbiguous (mis))
+					Report419 (mc, nameForError, mis);
 				return mis [0];
 			}
 
-			if (paramList.Length == 0) {
-				// search for fields/events etc.
-				mis = type.FindMembers (
-					MemberTypes.All,
-					BindingFlags.Public | BindingFlags.NonPublic | BindingFlags.Static | BindingFlags.Instance,
-					Type.FilterName,
-					memberName);
-				if (mis.Length == 0)
-					return null;
+			MethodSignature msig = new MethodSignature (memberName, null, paramList);
+			mis = FindMethodBase (type, 
+				BindingFlags.Public | BindingFlags.NonPublic | BindingFlags.Static | BindingFlags.Instance,
+				msig);
+
+			if (warn419 && mis.Length > 0) {
 				if (IsAmbiguous (mis))
-					warningType = 419;
+					Report419 (mc, nameForError, mis);
 				return mis [0];
 			}
 
@@ -444,17 +565,17 @@ namespace Mono.CSharp {
 				default:
 					warningType = 1584;
 					Report.Warning (1020, 1, mc.Location, "Overloadable {0} operator is expected", paramList.Length == 2 ? "binary" : "unary");
-					Report.Warning (1584, 1, mc.Location, "XML comment on '{0}' has syntactically incorrect attribute '{1}'", mc.GetSignatureForError (), cref);
+					Report.Warning (1584, 1, mc.Location, "XML comment on `{0}' has syntactically incorrect cref attribute `{1}'",
+						mc.GetSignatureForError (), cref);
 					return null;
 				}
 			}
 			// here we still don't consider return type (to
 			// detect CS1581 or CS1002+CS1584).
 			msig = new MethodSignature (oper, null, paramList);
-			mis = type.FindMembers (
-				MemberTypes.Method,
-				BindingFlags.Public | BindingFlags.NonPublic | BindingFlags.Static,
-				MethodSignature.method_signature_filter,
+
+			mis = FindMethodBase (type, 
+				BindingFlags.Public | BindingFlags.NonPublic | BindingFlags.Static | BindingFlags.Instance,
 				msig);
 			if (mis.Length == 0)
 				return null; // CS1574
@@ -468,7 +589,7 @@ namespace Mono.CSharp {
 				Type returnType = FindDocumentedType (mc, returnTypeName, ds, cref);
 				if (returnType == null || returnType != expected) {
 					warningType = 1581;
-					Report.Warning (1581, 1, mc.Location, "Invalid return type in XML comment cref attribute '{0}'", cref);
+					Report.Warning (1581, 1, mc.Location, "Invalid return type in XML comment cref attribute `{0}'", cref);
 					return null;
 				}
 			}
@@ -488,8 +609,6 @@ namespace Mono.CSharp {
 			return true;
 		}
 
-		private static Type [] emptyParamList = new Type [0];
-
 		//
 		// Processes "see" or "seealso" elements.
 		// Checks cref attribute.
@@ -517,46 +636,48 @@ namespace Mono.CSharp {
 				signature = cref;
 
 			int parensPos = signature.IndexOf ('(');
+			int bracePos = parensPos >= 0 ? -1 :
+				signature.IndexOf ('[');
 			if (parensPos > 0 && signature [signature.Length - 1] == ')') {
 				name = signature.Substring (0, parensPos).Trim (wsChars);
-				parameters = signature.Substring (parensPos + 1, signature.Length - parensPos - 2);
+				parameters = signature.Substring (parensPos + 1, signature.Length - parensPos - 2).Trim (wsChars);
+			}
+			else if (bracePos > 0 && signature [signature.Length - 1] == ']') {
+				name = signature.Substring (0, bracePos).Trim (wsChars);
+				parameters = signature.Substring (bracePos + 1, signature.Length - bracePos - 2).Trim (wsChars);
 			}
 			else {
 				name = signature;
-				parameters = String.Empty;
+				parameters = null;
 			}
 			Normalize (mc, ref name);
 
-			string identifier = name;
-
-			if (name.Length > 0 && name [name.Length - 1] == ']') {
-				string tmp = name.Substring (0, name.Length - 1).Trim (wsChars);
-				if (tmp [tmp.Length - 1] == '[')
-					identifier = tmp.Substring (0, tmp.Length - 1).Trim (wsChars);
-			}
+			string identifier = GetBodyIdentifierFromName (name);
 
 			// Check if identifier is valid.
 			// This check is not necessary to mark as error, but
 			// csc specially reports CS1584 for wrong identifiers.
 			string [] nameElems = identifier.Split ('.');
 			for (int i = 0; i < nameElems.Length; i++) {
-				string nameElem = nameElems [i];
-				if (nameElem.EndsWith ("[]"))
-					nameElem = nameElem.Substring (
-						nameElem.Length - 2);
+				string nameElem = GetBodyIdentifierFromName (nameElems [i]);
 				if (i > 0)
 					Normalize (mc, ref nameElem);
 				if (!Tokenizer.IsValidIdentifier (nameElem)
 					&& nameElem.IndexOf ("operator") < 0) {
-					Report.Warning (1584, 1, mc.Location, "XML comment on '{0}' has syntactically incorrect attribute '{1}'", mc.GetSignatureForError (), cref);
+					Report.Warning (1584, 1, mc.Location, "XML comment on `{0}' has syntactically incorrect cref attribute `{1}'",
+						mc.GetSignatureForError (), cref);
 					xref.SetAttribute ("cref", "!:" + signature);
 					return;
 				}
 			}
 
 			// check if parameters are valid
-			Type [] parameterTypes = emptyParamList;
-			if (parameters.Length > 0) {
+			Type [] parameterTypes;
+			if (parameters == null)
+				parameterTypes = null;
+			else if (parameters.Length == 0)
+				parameterTypes = Type.EmptyTypes;
+			else {
 				string [] paramList = parameters.Split (',');
 				ArrayList plist = new ArrayList ();
 				for (int i = 0; i < paramList.Length; i++) {
@@ -564,39 +685,26 @@ namespace Mono.CSharp {
 					Normalize (mc, ref paramTypeName);
 					Type paramType = FindDocumentedType (mc, paramTypeName, ds, cref);
 					if (paramType == null) {
-						Report.Warning (1580, 1, mc.Location, "Invalid type for parameter '{0}' in XML comment cref attribute '{1}'", i + 1, cref);
+						Report.Warning (1580, 1, mc.Location, "Invalid type for parameter `{0}' in XML comment cref attribute `{1}'",
+							(i + 1).ToString (), cref);
 						return;
 					}
 					plist.Add (paramType);
 				}
 				parameterTypes = plist.ToArray (typeof (Type)) as Type [];
-				StringBuilder sb = new StringBuilder ();
-				sb.Append ('(');
-				for (int i = 0; i < parameterTypes.Length; i++) {
-					Type t = parameterTypes [i];
-					if (sb.Length > 1)
-						sb.Append (',');
-					sb.Append (t.FullName.Replace ('+', '.'));
-				}
-				sb.Append (')');
-				parameters = sb.ToString ();
 			}
 
 			Type type = FindDocumentedType (mc, name, ds, cref);
 			if (type != null
 				// delegate must not be referenced with args
 				&& (!type.IsSubclassOf (typeof (System.Delegate))
-				|| parameterTypes.Length == 0)) {
-				xref.SetAttribute ("cref", "T:" + type.FullName.Replace ("+", "."));
+				|| parameterTypes == null)) {
+				string result = type.FullName.Replace ("+", ".")
+					+ (bracePos < 0 ? String.Empty : signature.Substring (bracePos));
+				xref.SetAttribute ("cref", "T:" + result);
 				return; // a type
 			}
 
-			// don't use identifier here. System[] is not alloed.
-			if (Namespace.IsNamespace (name)) {
-				xref.SetAttribute ("cref", "N:" + name);
-				return; // a namespace
-			}
-
 			int period = name.LastIndexOf ('.');
 			if (period > 0) {
 				string typeName = name.Substring (0, period);
@@ -605,38 +713,107 @@ namespace Mono.CSharp {
 				type = FindDocumentedType (mc, typeName, ds, cref);
 				int warnResult;
 				if (type != null) {
-					MemberInfo mi = FindDocumentedMember (mc, type, memberName, parameterTypes, ds, out warnResult, cref);
-					if (warnResult == 419)
-						Report419 (mc, memberName);
-					else if (warnResult > 0)
+					FoundMember fm = FindDocumentedMember (mc, type, memberName, parameterTypes, ds, out warnResult, cref, true, name);
+					if (warnResult > 0)
 						return;
-					if (mi != null) {
-						xref.SetAttribute ("cref", GetMemberDocHead (mi.MemberType) + type.FullName.Replace ("+", ".") + "." + memberName + parameters);
+					if (!fm.IsEmpty) {
+						MemberInfo mi = fm.Member;
+						// we cannot use 'type' directly
+						// to get its name, since mi
+						// could be from DeclaringType
+						// for nested types.
+						xref.SetAttribute ("cref", GetMemberDocHead (mi.MemberType) + fm.Type.FullName.Replace ("+", ".") + "." + memberName + GetParametersFormatted (mi));
 						return; // a member of a type
 					}
 				}
 			}
 			else {
 				int warnResult;
-				MemberInfo mi = FindDocumentedMember (mc, ds.TypeBuilder, name, parameterTypes, ds, out warnResult, cref);
-				if (warnResult == 419)
-					Report419 (mc, name);
-				else if (warnResult > 0)
+				FoundMember fm = FindDocumentedMember (mc, ds.TypeBuilder, name, parameterTypes, ds, out warnResult, cref, true, name);
+				if (warnResult > 0)
 					return;
-				if (mi != null) {
-					xref.SetAttribute ("cref", GetMemberDocHead (mi.MemberType) + ds.TypeBuilder.FullName.Replace ("+", ".") + "." + name);
+				if (!fm.IsEmpty) {
+					MemberInfo mi = fm.Member;
+					// we cannot use 'type' directly
+					// to get its name, since mi
+					// could be from DeclaringType
+					// for nested types.
+					xref.SetAttribute ("cref", GetMemberDocHead (mi.MemberType) + fm.Type.FullName.Replace ("+", ".") + "." + name + GetParametersFormatted (mi));
 					return; // local member name
 				}
 			}
 
-			Report.Warning (1574, 1, mc.Location, "XML comment on '{0}' has cref attribute '{1}' that could not be resolved in '{2}'.", mc.GetSignatureForError (), cref, ds.GetSignatureForError ());
+			// It still might be part of namespace name.
+			Namespace ns = ds.NamespaceEntry.NS.GetNamespace (name, false);
+			if (ns != null) {
+				xref.SetAttribute ("cref", "N:" + ns.FullName);
+				return; // a namespace
+			}
+			if (RootNamespace.Global.IsNamespace (name)) {
+				xref.SetAttribute ("cref", "N:" + name);
+				return; // a namespace
+			}
+
+			Report.Warning (1574, 1, mc.Location, "XML comment on `{0}' has cref attribute `{1}' that could not be resolved",
+				mc.GetSignatureForError (), cref);
 
 			xref.SetAttribute ("cref", "!:" + name);
 		}
 
-		static void Report419 (MemberCore mc, string memberName)
+		static string GetParametersFormatted (MemberInfo mi)
 		{
-			Report.Warning (419, 3, mc.Location, "Ambiguous member specification in cref attribute: '{0}'. Check overloaded members and supply exact parameters.", memberName);
+			MethodBase mb = mi as MethodBase;
+			bool isSetter = false;
+			PropertyInfo pi = mi as PropertyInfo;
+			if (pi != null) {
+				mb = pi.GetGetMethod ();
+				if (mb == null) {
+					isSetter = true;
+					mb = pi.GetSetMethod ();
+				}
+			}
+			if (mb == null)
+				return String.Empty;
+
+			ParameterData parameters = TypeManager.GetParameterData (mb);
+			if (parameters == null || parameters.Count == 0)
+				return String.Empty;
+
+			StringBuilder sb = new StringBuilder ();
+			sb.Append ('(');
+			for (int i = 0; i < parameters.Count; i++) {
+				if (isSetter && i + 1 == parameters.Count)
+					break; // skip "value".
+				if (i > 0)
+					sb.Append (',');
+				Type t = parameters.ParameterType (i);
+				sb.Append (t.FullName.Replace ('+', '.').Replace ('&', '@'));
+			}
+			sb.Append (')');
+			return sb.ToString ();
+		}
+
+		static string GetBodyIdentifierFromName (string name)
+		{
+			string identifier = name;
+
+			if (name.Length > 0 && name [name.Length - 1] == ']') {
+				string tmp = name.Substring (0, name.Length - 1).Trim (wsChars);
+				int last = tmp.LastIndexOf ('[');
+				if (last > 0)
+					identifier = tmp.Substring (0, last).Trim (wsChars);
+			}
+
+			return identifier;
+		}
+
+		static void Report419 (MemberCore mc, string memberName, MemberInfo [] mis)
+		{
+			Report.Warning (419, 3, mc.Location, 
+				"Ambiguous reference in cref attribute `{0}'. Assuming `{1}' but other overloads including `{2}' have also matched",
+				memberName,
+				TypeManager.GetFullNameSignature (mis [0]),
+				TypeManager.GetFullNameSignature (mis [1]));
 		}
 
 		//
@@ -671,20 +848,15 @@ namespace Mono.CSharp {
 		public static string GetMethodDocCommentName (MethodCore mc, DeclSpace ds)
 		{
 			Parameter [] plist = mc.Parameters.FixedParameters;
-			Parameter parr = mc.Parameters.ArrayParameter;
 			string paramSpec = String.Empty;
 			if (plist != null) {
 				StringBuilder psb = new StringBuilder ();
 				foreach (Parameter p in plist) {
 					psb.Append (psb.Length != 0 ? "," : "(");
-					psb.Append (p.ParameterType.FullName.Replace ("+", "."));
+					psb.Append (p.ExternalType ().FullName.Replace ("+", ".").Replace ('&', '@'));
 				}
 				paramSpec = psb.ToString ();
 			}
-			if (parr != null)
-				paramSpec += String.Concat (
-					paramSpec == String.Empty ? "(" : ",",
-					parr.ParameterType.FullName.Replace ("+", "."));
 
 			if (paramSpec.Length > 0)
 				paramSpec += ")";
@@ -720,27 +892,18 @@ namespace Mono.CSharp {
 				if (xname == "")
 					continue; // really? but MS looks doing so
 				if (xname != "" && mc.Parameters.GetParameterByName (xname, out i) == null)
-					Report.Warning (1572, 2, mc.Location, "XML comment on '{0}' has a 'param' tag for '{1}', but there is no such parameter.", mc.Name, xname);
+					Report.Warning (1572, 2, mc.Location, "XML comment on `{0}' has a param tag for `{1}', but there is no parameter by that name",
+						mc.GetSignatureForError (), xname);
 				else if (paramTags [xname] != null)
-					Report.Warning (1571, 2, mc.Location, "XML comment on '{0}' has a duplicate param tag for '{1}'", mc.Name, xname);
+					Report.Warning (1571, 2, mc.Location, "XML comment on `{0}' has a duplicate param tag for `{1}'",
+						mc.GetSignatureForError (), xname);
 				paramTags [xname] = xname;
 			}
 			Parameter [] plist = mc.Parameters.FixedParameters;
-			if (plist != null) {
-				foreach (Parameter p in plist) {
-					if (paramTags.Count > 0 && paramTags [p.Name] == null)
-						Report.Warning (1573, 4, mc.Location, "Parameter '{0}' has no matching param tag in the XML comment for '{1}' (but other parameters do)", mc.Name, p.Name);
-				}
-			}
-		}
-
-		// Enum
-		public static void GenerateEnumDocComment (Enum e, DeclSpace ds)
-		{
-			GenerateDocComment (e, ds);
-			foreach (string name in e.ordered_enums) {
-				MemberCore mc = e.GetDefinition (name);
-				GenerateDocComment (mc, e);
+			foreach (Parameter p in plist) {
+				if (paramTags.Count > 0 && paramTags [p.Name] == null)
+					Report.Warning (1573, 4, mc.Location, "Parameter `{0}' has no matching param tag in the XML comment for `{1}'",
+						p.Name, mc.GetSignatureForError ());
 			}
 		}
 
@@ -748,8 +911,10 @@ namespace Mono.CSharp {
 		{
 			if (name.Length > 0 && name [0] == '@')
 				name = name.Substring (1);
+			else if (name == "this")
+				name = "Item";
 			else if (Tokenizer.IsKeyword (name) && !IsTypeName (name))
-				Report.Warning (1041, 1, mc.Location, String.Format ("Identifier expected, '{0}' is a keyword", name));
+				Report.Warning (1041, 1, mc.Location, "Identifier expected. `{0}' is a keyword", name);
 		}
 
 		private static bool IsTypeName (string name)
@@ -840,7 +1005,7 @@ namespace Mono.CSharp {
 				w.WriteEndDocument ();
 				return true;
 			} catch (Exception ex) {
-				Report.Error (1569, "Error generating XML documentation file '{0}' ('{1}')", docfilename, ex.Message);
+				Report.Error (1569, "Error generating XML documentation file `{0}' (`{1}')", docfilename, ex.Message);
 				return false;
 			} finally {
 				if (w != null)
@@ -877,10 +1042,12 @@ namespace Mono.CSharp {
 
 			if (root.Enums != null)
 				foreach (Enum e in root.Enums)
-					DocUtil.GenerateEnumDocComment (e, null);
+					e.GenerateDocComment (null);
 
 			IDictionary table = new ListDictionary ();
 			foreach (ClassPart cp in PartialComments.Keys) {
+				// FIXME: IDictionary does not guarantee that the keys will be
+				//        accessed in the order they were added.
 				table [cp.PartialContainer] += cp.DocComment;
 			}
 			foreach (PartialContainer pc in table.Keys) {
