@@ -30,9 +30,9 @@ namespace Mono.CSharp {
 	}
 	
 	public class Yield : Statement {
-		public Expression expr;
+		Expression expr;
 		ArrayList finally_blocks;
-		
+
 		public Yield (Expression expr, Location l)
 		{
 			this.expr = expr;
@@ -52,11 +52,12 @@ namespace Mono.CSharp {
 				return false;
 			}
 			if (ec.InCatch){
-				Report.Error (1631, loc, "Cannot yield in the body of a " +
-					      "catch clause");
+				Report.Error (1631, loc, "Cannot yield a value in the body of a catch clause");
 				return false;
 			}
-			if (ec.CurrentAnonymousMethod != null){
+
+			AnonymousContainer am = ec.CurrentAnonymousMethod;
+			if ((am != null) && !am.IsIterator){
 				Report.Error (1621, loc, "The yield statement cannot be used inside anonymous method blocks");
 				return false;
 			}
@@ -74,10 +75,12 @@ namespace Mono.CSharp {
 			expr = expr.Resolve (ec);
 			if (expr == null)
 				return false;
+
 			if (!CheckContext (ec, loc))
 				return false;
 
 			Iterator iterator = ec.CurrentIterator;
+
 			if (expr.Type != iterator.IteratorType){
 				expr = Convert.ImplicitConversionRequired (
 					ec, expr, iterator.IteratorType, loc);
@@ -118,16 +121,13 @@ namespace Mono.CSharp {
 	}
 
 	public class Iterator : Class {
-		ToplevelBlock original_block;
-		ToplevelBlock block;
-		string original_name;
+		protected ToplevelBlock original_block;
+		protected ToplevelBlock block;
 
 		Type iterator_type;
 		TypeExpr iterator_type_expr;
 		bool is_enumerable;
-		bool is_static;
-
-		Hashtable fields;
+		public readonly bool IsStatic;
 
 		//
 		// The state as we generate the iterator
@@ -140,8 +140,13 @@ namespace Mono.CSharp {
 		// Context from the original method
 		//
 		TypeContainer container;
-		Type return_type;
-		InternalParameters parameters;
+		Type this_type;
+		Parameters parameters;
+		IMethodData orig_method;
+
+		MoveNextMethod move_next_method;
+		Constructor ctor;
+		CaptureContext cc;
 
 		protected enum State {
 			Uninitialized	= -2,
@@ -177,7 +182,8 @@ namespace Mono.CSharp {
 			resume_points.Add (entry_point);
 			entry_point.Define (ig);
 
-			ec.EmitTopBlock (original_block, parameters, Location);
+			ec.EmitTopBlock (orig_method, original_block);
+
 			EmitYieldBreak (ig);
 
 			ig.MarkLabel (dispatcher);
@@ -222,7 +228,6 @@ namespace Mono.CSharp {
 			Label dispatcher = ig.DefineLabel ();
 			ig.Emit (OpCodes.Br, dispatcher);
 
-			ec.RemapToProxy = true;
 			Label [] labels = new Label [resume_points.Count];
 			for (int i = 0; i < labels.Length; i++) {
 				ResumePoint point = (ResumePoint) resume_points [i];
@@ -246,7 +251,6 @@ namespace Mono.CSharp {
 				ig.EndExceptionBlock ();
 				ig.Emit (OpCodes.Br, end);
 			}
-			ec.RemapToProxy = false;
 			
 			ig.MarkLabel (dispatcher);
 			ig.Emit (OpCodes.Ldarg_0);
@@ -280,26 +284,6 @@ namespace Mono.CSharp {
 			}
 		}
 
-		// 
-		// Invoked when a local variable declaration needs to be mapped to
-		// a field in our proxy class
-		//
-		// Prefixes registered:
-		//   v_   for EmitContext.MapVariable
-		//   s_   for Storage
-		//
-		public FieldBuilder MapVariable (string pfx, string name, Type t)
-		{
-			string full_name = pfx + name;
-			FieldBuilder fb = (FieldBuilder) fields [full_name];
-			if (fb != null)
-				return fb;
-
-			fb = TypeBuilder.DefineField (full_name, t, FieldAttributes.Private);
-			fields.Add (full_name, fb);
-			return fb;
-		}
-
 		//
 		// Called back from Yield
 		//
@@ -342,46 +326,48 @@ namespace Mono.CSharp {
 			point.Define (ig);
 		}
 
-		private static MemberName MakeProxyName (string name)
+		private static MemberName MakeProxyName (string name, Location loc)
 		{
 			int pos = name.LastIndexOf ('.');
 			if (pos > 0)
 				name = name.Substring (pos + 1);
 
-			return new MemberName ("<" + name + ">__" + (proxy_count++));
+			return new MemberName ("<" + name + ">__" + (proxy_count++), loc);
 		}
 
 		//
 		// Our constructor
 		//
-		public Iterator (TypeContainer container, string name, Type return_type,
-				 InternalParameters parameters,
-				 int modifiers, ToplevelBlock block, Location loc)
-			: base (container.NamespaceEntry, container, MakeProxyName (name),
-				(modifiers & Modifiers.UNSAFE) | Modifiers.PRIVATE, null, loc)
+		public Iterator (IMethodData m_container, TypeContainer container,
+				 int modifiers)
+			: base (container.NamespaceEntry, container, MakeProxyName (m_container.MethodName.Name, m_container.Location),
+				(modifiers & Modifiers.UNSAFE) | Modifiers.PRIVATE, null)
 		{
+			this.orig_method = m_container;
+
 			this.container = container;
-			this.return_type = return_type;
-			this.parameters = parameters;
-			this.original_name = name;
-			this.original_block = block;
-			this.block = new ToplevelBlock (loc);
+			this.parameters = m_container.ParameterInfo;
+			this.original_block = orig_method.Block;
+			this.block = new ToplevelBlock (orig_method.Block, parameters, orig_method.Location);
 
-			fields = new Hashtable ();
+			IsStatic = (modifiers & Modifiers.STATIC) != 0;
+		}
 
-			is_static = (modifiers & Modifiers.STATIC) != 0;
+		public AnonymousContainer Host {
+			get { return move_next_method; }
 		}
 
 		public bool DefineIterator ()
 		{
 			ec = new EmitContext (this, Mono.CSharp.Location.Null, null, null, ModFlags);
+			ec.CurrentAnonymousMethod = move_next_method;
+			ec.CurrentIterator = this;
+			ec.InIterator = true;
 
-			if (!CheckType (return_type)) {
-				Report.Error (
-					1624, Location,
-					"The body of `{0}' cannot be an iterator block " +
-					"because '{1}' is not an iterator interface type",
-					original_name, TypeManager.CSharpName (return_type));
+			if (!CheckType ()) {
+				Report.Error (1624, Location,
+					"The body of `{0}' cannot be an iterator block because `{1}' is not an iterator interface type",
+					orig_method.GetSignatureForError (), TypeManager.CSharpName (orig_method.ReturnType));
 				return false;
 			}
 
@@ -405,6 +391,8 @@ namespace Mono.CSharp {
 				}
 			}
 
+			this_type = container.TypeBuilder;
+
 			ArrayList list = new ArrayList ();
 			if (is_enumerable)
 				list.Add (new TypeExpression (
@@ -417,6 +405,53 @@ namespace Mono.CSharp {
 			container.AddIterator (this);
 
 			Bases = list;
+			orig_method.Block = block;
+			return true;
+		}
+
+		protected override bool DoDefineMembers ()
+		{
+			ec.InIterator = true;
+			ec.CurrentIterator = this;
+			ec.CurrentAnonymousMethod = move_next_method;
+			ec.capture_context = cc;
+
+			if (!base.DoDefineMembers ())
+				return false;
+
+			return true;
+		}
+
+		public override bool Define ()
+		{
+			if (!base.Define ())
+				return false;
+
+			ec.InIterator = true;
+			ec.CurrentIterator = this;
+			ec.CurrentAnonymousMethod = move_next_method;
+			ec.capture_context = cc;
+			ec.TypeContainer = ec.TypeContainer.Parent;
+
+			ec.ContainerType = ec.TypeContainer.TypeBuilder;
+
+			ec.ig = move_next_method.method.MethodBuilder.GetILGenerator ();
+
+			if (!ctor.Define ())
+				return false;
+
+			bool unreachable;
+
+			if (!ec.ResolveTopBlock (null, original_block, parameters, orig_method, out unreachable))
+				return false;
+
+			if (!ec.ResolveTopBlock (null, block, parameters, orig_method, out unreachable))
+				return false;
+
+			original_block.CompleteContexts ();
+
+			cc.EmitAnonymousHelperClasses (ec);
+
 			return true;
 		}
 
@@ -426,45 +461,52 @@ namespace Mono.CSharp {
 		protected override bool DefineNestedTypes ()
 		{
 			Define_Fields ();
-			Define_Constructor ();
 			Define_Current ();
 			Define_MoveNext ();
 			Define_Reset ();
 			Define_Dispose ();
 
+			Create_Block ();
+
+			Define_Constructor ();
+
 			if (is_enumerable)
 				Define_GetEnumerator ();
 
-			Create_Block ();
-
 			return base.DefineNestedTypes ();
 		}
 
-
 		Field pc_field;
 		Field current_field;
 		Method dispose;
 
-		public Field this_field;
-		public Field[] parameter_fields;
-
 		void Create_Block ()
 		{
-			int first = is_static ? 0 : 1;
+			original_block.SetHaveAnonymousMethods (Location, move_next_method);
+			block.SetHaveAnonymousMethods (Location, move_next_method);
+
+			cc = original_block.CaptureContext;
+
+			int first = IsStatic ? 0 : 1;
 
 			ArrayList args = new ArrayList ();
-			if (!is_static) {
+			if (!IsStatic) {
 				Type t = container.TypeBuilder;
 				args.Add (new Argument (
-					new ThisParameterReference (t, 0, Location)));
+					new ThisParameterReference (t, Location)));
+				cc.CaptureThis (move_next_method);
 			}
 
-			args.Add (new Argument (new BoolLiteral (false)));
+			args.Add (new Argument (new BoolLiteral (false, Location)));
 
 			for (int i = 0; i < parameters.Count; i++) {
 				Type t = parameters.ParameterType (i);
+				string name = parameters.ParameterName (i);
+
 				args.Add (new Argument (
 					new SimpleParameterReference (t, first + i, Location)));
+
+				cc.AddParameterToContext (move_next_method, name, t, first + i);
 			}
 
 			Expression new_expr = new New (
@@ -475,37 +517,15 @@ namespace Mono.CSharp {
 
 		void Define_Fields ()
 		{
-			Location loc = Location.Null;
-
 			pc_field = new Field (
-				this, TypeManager.system_int32_expr, Modifiers.PRIVATE, "PC",
-				null, null, loc);
+				this, TypeManager.system_int32_expr, Modifiers.PRIVATE, "$PC",
+				null, Location);
 			AddField (pc_field);
 
 			current_field = new Field (
-				this, iterator_type_expr, Modifiers.PRIVATE, "current",
-				null, null, loc);
+				this, iterator_type_expr, Modifiers.PRIVATE, "$current",
+				null, Location);
 			AddField (current_field);
-
-			if (!is_static) {
-				this_field = new Field (
-					this,
-					new TypeExpression (container.TypeBuilder, Location),
-					Modifiers.PRIVATE, "this", null, null, loc);
-				AddField (this_field);
-			}
-
-			parameter_fields = new Field [parameters.Count];
-			for (int i = 0; i < parameters.Count; i++) {
-				string fname = String.Format (
-					"field{0}_{1}", i, parameters.ParameterName (i));
-
-				parameter_fields [i] = new Field (
-					this,
-					new TypeExpression (parameters.ParameterType (i), loc),
-					Modifiers.PRIVATE, fname, null, null, loc);
-				AddField (parameter_fields [i]);
-			}
 		}
 
 		void Define_Constructor ()
@@ -514,66 +534,43 @@ namespace Mono.CSharp {
 
 			ArrayList list = new ArrayList ();
 
-			if (!is_static)
+			if (!IsStatic)
 				list.Add (new Parameter (
 					new TypeExpression (container.TypeBuilder, Location),
 					"this", Parameter.Modifier.NONE,
 					null, Location));
 			list.Add (new Parameter (
-				TypeManager.system_boolean_expr, "initialized",
+				TypeManager.bool_type, "initialized",
 				Parameter.Modifier.NONE, null, Location));
 
-			Parameter[] old_fixed = parameters.Parameters.FixedParameters;
-			if (old_fixed != null)
-				list.AddRange (old_fixed);
+			Parameter[] old_fixed = parameters.FixedParameters;
+			list.AddRange (old_fixed);
 
 			Parameter[] fixed_params = new Parameter [list.Count];
 			list.CopyTo (fixed_params);
 
-			ctor_params = new Parameters (
-				fixed_params, parameters.Parameters.ArrayParameter);
+			ctor_params = new Parameters (fixed_params);
 
-			Constructor ctor = new Constructor (
+			ctor = new Constructor (
 				this, Name, Modifiers.PUBLIC, ctor_params,
-				new ConstructorBaseInitializer (
-					null, Parameters.EmptyReadOnlyParameters, Location),
+				new GeneratedBaseInitializer (Location),
 				Location);
 			AddConstructor (ctor);
 
-			ToplevelBlock block = ctor.Block = new ToplevelBlock (Location);
+			ctor.Block = new ToplevelBlock (block, parameters, Location);
 
-			if (!is_static) {
-				Type t = container.TypeBuilder;
-
-				Assign assign = new Assign (
-					new FieldExpression (this_field),
-					new SimpleParameterReference (t, 1, Location),
-					Location);
-
-				block.AddStatement (new StatementExpression (assign, Location));
-			}
-
-			int first = is_static ? 2 : 3;
-
-			for (int i = 0; i < parameters.Count; i++) {
-				Type t = parameters.ParameterType (i);
-
-				Assign assign = new Assign (
-					new FieldExpression (parameter_fields [i]),
-					new SimpleParameterReference (t, first + i, Location),
-					Location);
-
-				block.AddStatement (new StatementExpression (assign, Location));
-			}
+			int first = IsStatic ? 2 : 3;
 
 			State initial = is_enumerable ? State.Uninitialized : State.Running;
-			block.AddStatement (new SetState (this, initial, Location));
+			ctor.Block.AddStatement (new SetState (this, initial, Location));
 
-			block.AddStatement (new If (
+			ctor.Block.AddStatement (new If (
 				new SimpleParameterReference (
 					TypeManager.bool_type, first - 1, Location),
 				new SetState (this, State.Running, Location),
 				Location));
+
+			ctor.Block.AddStatement (new InitScope (this, Location));
 		}
 
 		Statement Create_ThrowInvalidOperation ()
@@ -594,65 +591,65 @@ namespace Mono.CSharp {
 
 		void Define_Current ()
 		{
-			ToplevelBlock get_block = new ToplevelBlock (Location);
+			ToplevelBlock get_block = new ToplevelBlock (
+				block, parameters, Location);
 			MemberName left = new MemberName ("System.Collections.IEnumerator");
-			MemberName name = new MemberName (left, "Current");
+			MemberName name = new MemberName (left, "Current", Location);
 
 			get_block.AddStatement (new If (
 				new Binary (
 					Binary.Operator.LessThanOrEqual,
-					new FieldExpression (pc_field),
-					new IntLiteral ((int) State.Running), Location),
+					new FieldExpression (this, pc_field),
+					new IntLiteral ((int) State.Running, pc_field.Location)),
 				Create_ThrowInvalidOperation (),
 				new Return (
-					new FieldExpression (current_field), Location),
+					new FieldExpression (this, current_field), Location),
 				Location));
 
 			Accessor getter = new Accessor (get_block, 0, null, Location);
 
 			Property current = new Property (
 				this, iterator_type_expr, 0,
-				false, name, null, getter, null, Location);
+				false, name, null, getter, null);
 			AddProperty (current);
 		}
 
 		void Define_MoveNext ()
 		{
-			Method move_next = new Method (
-				this, TypeManager.system_boolean_expr,
-				Modifiers.PUBLIC, false, new MemberName ("MoveNext"),
-				Parameters.EmptyReadOnlyParameters, null,
-				Location.Null);
-			AddMethod (move_next);
+			move_next_method = new MoveNextMethod (this, Location);
 
-			ToplevelBlock block = move_next.Block = new ToplevelBlock (Location);
+			original_block.ReParent (block, move_next_method);
 
-			MoveNextMethod inline = new MoveNextMethod (this, Location);
-			block.AddStatement (inline);
+			move_next_method.CreateMethod (ec);
+
+			AddMethod (move_next_method.method);
 		}
 
 		void Define_GetEnumerator ()
 		{
 			MemberName left = new MemberName ("System.Collections.IEnumerable");
-			MemberName name = new MemberName (left, "GetEnumerator");
+
+			MemberName name = new MemberName (left, "GetEnumerator", Location);
 
 			Method get_enumerator = new Method (
 				this,
 				new TypeExpression (TypeManager.ienumerator_type, Location),
 				0, false, name,
-				Parameters.EmptyReadOnlyParameters, null,
-				Location.Null);
+				Parameters.EmptyReadOnlyParameters, null);
 			AddMethod (get_enumerator);
 
-			get_enumerator.Block = new ToplevelBlock (Location);
+			get_enumerator.Block = new ToplevelBlock (
+				block, parameters, Location);
+
+			get_enumerator.Block.SetHaveAnonymousMethods (Location, move_next_method);
 
 			Expression ce = new MemberAccess (
 				new SimpleName ("System.Threading.Interlocked", Location),
 				"CompareExchange", Location);
 
-			Expression pc = new FieldExpression (pc_field);
-			Expression before = new IntLiteral ((int) State.Running);
-			Expression uninitialized = new IntLiteral ((int) State.Uninitialized);
+			Expression pc = new FieldExpression (this, pc_field);
+			Expression before = new IntLiteral ((int) State.Running, Location);
+			Expression uninitialized = new IntLiteral ((int) State.Uninitialized, Location);
 
 			ArrayList args = new ArrayList ();
 			args.Add (new Argument (pc, Argument.AType.Ref));
@@ -662,20 +659,26 @@ namespace Mono.CSharp {
 			get_enumerator.Block.AddStatement (new If (
 				new Binary (
 					Binary.Operator.Equality,
-					new Invocation (ce, args, Location),
-					uninitialized, Location),
-				new Return (new This (block, Location), Location),
+					new Invocation (ce, args),
+					uninitialized),
+				new Return (new ThisParameterReference (
+						    TypeManager.ienumerator_type, Location),
+					    Location),
 				Location));
 
 			args = new ArrayList ();
-			if (!is_static)
-				args.Add (new Argument (new FieldExpression (this_field)));
+			if (!IsStatic) {
+				args.Add (new Argument (new CapturedThisReference (this, Location)));
+			}
 
-			args.Add (new Argument (new BoolLiteral (true)));
+			args.Add (new Argument (new BoolLiteral (true, Location)));
 
-			for (int i = 0; i < parameters.Count; i++)
-				args.Add (new Argument (
-						  new FieldExpression (parameter_fields [i])));
+			for (int i = 0; i < parameters.Count; i++) {
+				Expression cp = new CapturedParameterReference (
+					this, parameters.ParameterType (i),
+					parameters.ParameterName (i), Location);
+				args.Add (new Argument (cp));
+			}
 
 			Expression new_expr = new New (
 				new TypeExpression (TypeBuilder, Location), args, Location);
@@ -710,10 +713,10 @@ namespace Mono.CSharp {
 			}
 		}
 
-		protected class ThisParameterReference : SimpleParameterReference
+		protected class ThisParameterReference : SimpleParameterReference, IMemoryLocation
 		{
-			public ThisParameterReference (Type type, int idx, Location loc)
-				: base (type, idx, loc)
+			public ThisParameterReference (Type type, Location loc)
+				: base (type, 0, loc)
 			{ }
 
 			protected override void DoEmit (EmitContext ec)
@@ -722,26 +725,91 @@ namespace Mono.CSharp {
 				if (ec.TypeContainer is Struct)
 					ec.ig.Emit (OpCodes.Ldobj, type);
 			}
+
+			public void AddressOf (EmitContext ec, AddressOp mode)
+			{
+				if (ec.TypeContainer is Struct)
+					ec.ig.Emit (OpCodes.Ldarga, 0);
+				else
+					ec.ig.Emit (OpCodes.Ldarg, 0);
+			}
+		}
+
+		protected class CapturedParameterReference : Expression
+		{
+			Iterator iterator;
+			string name;
+
+			public CapturedParameterReference (Iterator iterator, Type type,
+							   string name, Location loc)
+			{
+				this.iterator = iterator;
+				this.loc = loc;
+				this.type = type;
+				this.name = name;
+				eclass = ExprClass.Variable;
+			}
+
+			public override Expression DoResolve (EmitContext ec)
+			{
+				return this;
+			}
+
+			public override void Emit (EmitContext ec)
+			{
+				ec.CurrentAnonymousMethod = iterator.move_next_method;
+
+				LocalTemporary dummy = null;
+				
+				iterator.cc.EmitParameter (ec, name, false, false, ref dummy);
+			}
+		}
+
+		protected class CapturedThisReference : Expression
+		{
+			public CapturedThisReference (Iterator iterator, Location loc)
+			{
+				this.loc = loc;
+				this.type = iterator.this_type;
+				eclass = ExprClass.Variable;
+			}
+
+			public override Expression DoResolve (EmitContext ec)
+			{
+				return this;
+			}
+
+			public override void Emit (EmitContext ec)
+			{
+				ec.EmitThis ();
+			}
 		}
 
 		protected class FieldExpression : Expression
 		{
+			Iterator iterator;
 			Field field;
 
-			public FieldExpression (Field field)
+			public FieldExpression (Iterator iterator, Field field)
 			{
+				this.iterator = iterator;
 				this.field = field;
+				this.loc = iterator.Location;
 			}
 
 			public override Expression DoResolveLValue (EmitContext ec, Expression right_side)
 			{
-				return DoResolve (ec);
+				FieldExpr fexpr = new FieldExpr (field.FieldBuilder, loc);
+				fexpr.InstanceExpression = new ThisParameterReference (
+					iterator.this_type, loc);
+				return fexpr.ResolveLValue (ec, right_side, loc);
 			}
 
 			public override Expression DoResolve (EmitContext ec)
 			{
 				FieldExpr fexpr = new FieldExpr (field.FieldBuilder, loc);
-				fexpr.InstanceExpression = ec.GetThis (loc);
+				fexpr.InstanceExpression = new ThisParameterReference (
+					iterator.this_type, loc);
 				return fexpr.Resolve (ec);
 			}
 
@@ -751,10 +819,56 @@ namespace Mono.CSharp {
 			}
 		}
 
-		protected class MoveNextMethod : Statement {
+		protected class MoveNextMethod : AnonymousContainer
+		{
 			Iterator iterator;
 
 			public MoveNextMethod (Iterator iterator, Location loc)
+				: base (iterator.parameters, iterator.original_block, loc)
+			{
+				this.iterator = iterator;
+			}
+
+			protected override bool CreateMethodHost (EmitContext ec)
+			{
+				method = new Method (
+					iterator, TypeManager.system_boolean_expr,
+					Modifiers.PUBLIC, false, new MemberName ("MoveNext", loc),
+					Parameters.EmptyReadOnlyParameters, null);
+
+				method.Block = Block;
+
+				MoveNextStatement inline = new MoveNextStatement (iterator, loc);
+				Block.AddStatement (inline);
+
+				return true;
+			}
+
+			public bool CreateMethod (EmitContext ec)
+			{
+				return CreateMethodHost (ec);
+			}
+
+			public override bool IsIterator {
+				get { return true; }
+			}
+
+			public override void CreateScopeType (EmitContext ec, ScopeInfo scope)
+			{
+				scope.ScopeTypeBuilder = iterator.TypeBuilder;
+				scope.ScopeConstructor = iterator.ctor.ConstructorBuilder;
+			}
+
+			public override void Emit (EmitContext ec)
+			{
+				throw new InternalErrorException ();
+			}
+		}
+
+		protected class MoveNextStatement : Statement {
+			Iterator iterator;
+
+			public MoveNextStatement (Iterator iterator, Location loc)
 			{
 				this.loc = loc;
 				this.iterator = iterator;
@@ -762,25 +876,16 @@ namespace Mono.CSharp {
 
 			public override bool Resolve (EmitContext ec)
 			{
-				ec.CurrentBranching.CurrentUsageVector.Return ();
 				return true;
 			}
 
 			protected override void DoEmit (EmitContext ec)
 			{
-				int code_flags = Modifiers.METHOD_YIELDS;
-				if (iterator.is_static)
-					code_flags |= Modifiers.STATIC;
+				ec.CurrentIterator = iterator;
+				ec.CurrentAnonymousMethod = iterator.move_next_method;
+				ec.InIterator = true;
 
-				code_flags |= iterator.ModFlags & Modifiers.UNSAFE;
-
-				EmitContext new_ec = new EmitContext (
-					iterator.container, loc, ec.ig,
-					TypeManager.int32_type, code_flags);
-
-				new_ec.CurrentIterator = iterator;
-
-				iterator.EmitMoveNext (new_ec);
+				iterator.EmitMoveNext (ec);
 			}
 		}
 
@@ -860,15 +965,39 @@ namespace Mono.CSharp {
 			}
 		}
 
+		protected class InitScope : Statement
+		{
+			Iterator iterator;
+
+			public InitScope (Iterator iterator, Location loc)
+			{
+				this.iterator = iterator;
+				this.loc = loc;
+			}
+
+			public override bool Resolve (EmitContext ec)
+			{
+				return true;
+			}
+
+			protected override void DoEmit (EmitContext ec)
+			{
+				iterator.cc.EmitInitScope (ec);
+			}
+		}
+
 		void Define_Reset ()
 		{
 			Method reset = new Method (
 				this, TypeManager.system_void_expr, Modifiers.PUBLIC,
-				false, new MemberName ("Reset"),
-				Parameters.EmptyReadOnlyParameters, null, Location);
+				false, new MemberName ("Reset", Location),
+				Parameters.EmptyReadOnlyParameters, null);
 			AddMethod (reset);
 
 			reset.Block = new ToplevelBlock (Location);
+			reset.Block = new ToplevelBlock (block, parameters, Location);
+			reset.Block.SetHaveAnonymousMethods (Location, move_next_method);
+
 			reset.Block.AddStatement (Create_ThrowNotSupported ());
 		}
 
@@ -876,16 +1005,14 @@ namespace Mono.CSharp {
 		{
 			dispose = new Method (
 				this, TypeManager.system_void_expr, Modifiers.PUBLIC,
-				false, new MemberName ("Dispose"),
-				Parameters.EmptyReadOnlyParameters, null, Location);
+				false, new MemberName ("Dispose", Location),
+				Parameters.EmptyReadOnlyParameters, null);
 			AddMethod (dispose);
 
-			dispose.Block = new ToplevelBlock (Location);
-			dispose.Block.AddStatement (new DisposeMethod (this, Location));
-		}
+			dispose.Block = new ToplevelBlock (block, parameters, Location);
+			dispose.Block.SetHaveAnonymousMethods (Location, move_next_method);
 
-		public ToplevelBlock Block {
-			get { return block; }
+			dispose.Block.AddStatement (new DisposeMethod (this, Location));
 		}
 
 		public Type IteratorType {
@@ -896,28 +1023,43 @@ namespace Mono.CSharp {
 		// This return statement tricks return into not flagging an error for being
 		// used in a Yields method
 		//
-		class NoCheckReturn : Return {
-			public NoCheckReturn (Expression expr, Location loc) : base (expr, loc)
+		class NoCheckReturn : Statement {
+			public Expression Expr;
+		
+			public NoCheckReturn (Expression expr, Location l)
 			{
+				Expr = expr;
+				loc = l;
 			}
 
 			public override bool Resolve (EmitContext ec)
 			{
-				ec.InIterator = false;
-				bool ret_val = base.Resolve (ec);
-				ec.InIterator = true;
+				Expr = Expr.Resolve (ec);
+				if (Expr == null)
+					return false;
 
-				return ret_val;
+				ec.CurrentBranching.CurrentUsageVector.Return ();
+
+				return true;
+			}
+
+			protected override void DoEmit (EmitContext ec)
+			{
+				Expr.Emit (ec);
+				ec.ig.Emit (OpCodes.Ret);
 			}
 		}
 
-		bool CheckType (Type t)
+		bool CheckType ()
 		{
-			if (t == TypeManager.ienumerable_type) {
+			Type ret = orig_method.ReturnType;
+
+			if (ret == TypeManager.ienumerable_type) {
 				iterator_type = TypeManager.object_type;
 				is_enumerable = true;
 				return true;
-			} else if (t == TypeManager.ienumerator_type) {
+			}
+			if (ret == TypeManager.ienumerator_type) {
 				iterator_type = TypeManager.object_type;
 				is_enumerable = false;
 				return true;
