@@ -2,13 +2,11 @@
 // report.cs: report errors and warnings.
 //
 // Author: Miguel de Icaza (miguel@ximian.com)
+//         Marek Safar (marek.safar@seznam.cz)         
 //
 // (C) 2001 Ximian, Inc. (http://www.ximian.com)
 //
 
-//
-// FIXME: currently our class library does not support custom number format strings
-//
 using System;
 using System.IO;
 using System.Text;
@@ -71,6 +69,30 @@ namespace Mono.CSharp {
 		/// </summary>
 		static StringCollection extra_information = new StringCollection ();
 
+		// 
+		// IF YOU ADD A NEW WARNING YOU HAVE TO ADD ITS ID HERE
+		//
+		public static readonly int[] AllWarnings = new int[] {
+																 28, 67, 78,
+																 105, 108, 109, 114, 162, 164, 168, 169, 183, 184, 197,
+																 219, 251, 252, 253, 282,
+																 419, 420, 429, 436, 440, 465,
+																 612, 618, 626, 628, 642, 649, 652, 658, 659, 660, 661, 665, 672,
+																 1030, 1058,
+																 1522, 1570, 1571, 1572, 1573, 1574, 1580, 1581, 1584, 1587, 1589, 1590, 1591, 1592,
+																 1616, 1633, 1634, 1635, 1691, 1692,
+																 1717, 1718,
+																 1901,
+																 2002, 2023,
+																 3005, 3012, 3019, 3021, 3022, 3023, 3026, 3027
+															 };
+
+		static Report ()
+		{
+			// Just to be sure that binary search is working
+			Array.Sort (AllWarnings);
+		}
+
 		public static void Reset ()
 		{
 			Errors = Warnings = 0;
@@ -123,21 +145,18 @@ namespace Mono.CSharp {
 
 			public virtual void Print (int code, Location location, string text)
 			{
-				if (location.Equals (Location.Null)) {
+				if (location.IsNull) {
 					Print (code, "", text);
 					return;
 				}
-				Print (code, String.Format ("{0}({1})", location.Name, location.Row), text);
+				Print (code, location.ToString (), text);
 			}
 		}
 
-		sealed class WarningMessage: AbstractMessage {
+		sealed class WarningMessage : AbstractMessage {
 			Location loc = Location.Null;
 			readonly int Level;
 
-			public WarningMessage ():
-				this (-1) {}
-
 			public WarningMessage (int level)
 			{
 				Level = level;
@@ -197,7 +216,7 @@ namespace Mono.CSharp {
 			}
 		}
 
-		sealed class ErrorMessage: AbstractMessage {
+		sealed class ErrorMessage : AbstractMessage {
 
 			public override void Print(int code, string location, string text)
 			{
@@ -219,7 +238,7 @@ namespace Mono.CSharp {
 
 		public static void FeatureIsNotStandardized (Location loc, string feature)
 		{
-			Report.Error (1644, loc, "Feature '{0}' cannot be used because it is not part of the standardized ISO C# language specification", feature);
+			Report.Error (1644, loc, "Feature `{0}' cannot be used because it is not part of the standardized ISO C# language specification", feature);
 		}
 		
 		public static string FriendlyStackTrace (Exception e)
@@ -263,31 +282,19 @@ namespace Mono.CSharp {
 			return sb.ToString ();
 		}
 
-		// !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
-		// IF YOU ADD A NEW WARNING YOU HAVE TO DUPLICATE ITS ID HERE
-		//
-		public static bool IsValidWarning (int code)
+		public static void StackTrace ()
 		{
-			int[] all_warnings = new int[] {
-				28, 67, 78, 105, 108, 109, 114, 192, 168, 169, 183, 184, 219, 251, 612, 618, 626, 628, 642, 649,
-				659, 660, 661, 672, 1030, 1522, 1616, 1691, 1692, 1901, 2002, 2023, 3012, 3019, 8024, 8028
-			};
-			
-			foreach (int i in all_warnings) {
-				if (i == code)
-					return true;
-			}
-			return false;
+			Console.WriteLine (FriendlyStackTrace (new StackTrace (true)));
 		}
-		
-		static public void LocationOfPreviousError (Location loc)
-		{
-			Stderr.WriteLine (String.Format ("{0}({1}) (Location of symbol related to previous error)", loc.Name, loc.Row));
-		}    
-        
+
+		public static bool IsValidWarning (int code)
+		{	
+			return Array.BinarySearch (AllWarnings, code) >= 0;
+		}
+		        
 		static public void RuntimeMissingSupport (Location loc, string feature) 
 		{
-			Report.Error (-88, loc, "Your .NET Runtime does not support '{0}'. Please use the latest Mono runtime instead.", feature);
+			Report.Error (-88, loc, "Your .NET Runtime does not support `{0}'. Please use the latest Mono runtime instead.", feature);
 		}
 
 		/// <summary>
@@ -296,7 +303,7 @@ namespace Mono.CSharp {
 		/// </summary>
 		static public void SymbolRelatedToPreviousError (Location loc, string symbol)
 		{
-			SymbolRelatedToPreviousError (String.Format ("{0}({1})", loc.Name, loc.Row), symbol);
+			SymbolRelatedToPreviousError (loc.ToString (), symbol);
 		}
 
 		static public void SymbolRelatedToPreviousError (MemberInfo mi)
@@ -307,7 +314,7 @@ namespace Mono.CSharp {
 			} else {
 				if (mi is MethodBase) {
 					IMethodData md = TypeManager.GetMethod ((MethodBase)mi);
-					SymbolRelatedToPreviousError (md.Location, md.GetSignatureForError (temp_ds));
+					SymbolRelatedToPreviousError (md.Location, md.GetSignatureForError ());
 					return;
 				}
 
@@ -335,12 +342,12 @@ namespace Mono.CSharp {
 
 		static void SymbolRelatedToPreviousError (string loc, string symbol)
 		{
-			extra_information.Add (String.Format ("{0}: '{1}' (name of symbol related to previous ", loc, symbol));
+			extra_information.Add (String.Format ("{0}: `{1}', name of symbol related to previous ", loc, symbol));
 		}
 
 		public static void ExtraInformation (Location loc, string msg)
 		{
-			extra_information.Add (String.Format ("{0}({1}) {2}", loc.Name, loc.Row, msg));
+			extra_information.Add (String.Format ("{0} {1}", loc, msg));
 		}
 
 		public static WarningRegions RegisterWarningRegion (Location location)
@@ -356,42 +363,28 @@ namespace Mono.CSharp {
 			return regions;
 		}
 
-		static public void Warning (int code, int level, Location loc, string format, params object[] args)
+		static public void Warning (int code, int level, Location loc, string format, params string[] args)
 		{
 			WarningMessage w = new WarningMessage (level);
 			w.Print (code, loc, String.Format (format, args));
 		}
 
-		static public void Warning (int code, Location loc, string format, params object[] args)
+		static public void Warning (int code, int level, string format, params string[] args)
 		{
-			WarningMessage w = new WarningMessage ();
-			w.Print (code, loc, String.Format (format, args));
+			Warning (code, level, Location.Null, String.Format (format, args));
 		}
 
-		static public void Warning (int code, string format, params object[] args)
-		{
-			Warning (code, Location.Null, String.Format (format, args));
-		}
-
-		/// <summary>
-		/// Did you test your WarningLevel, that you use this method
-		/// </summary>
-		static public void Warning (int code, string text)
-		{
-			Warning (code, Location.Null, text);
-		}
-
-		static public void Error (int code, string format, params object[] args)
+		static public void Error (int code, string format, params string[] args)
 		{
 			Error (code, Location.Null, String.Format (format, args));
 		}
 
-		static public void Error (int code, Location loc, string format, params object[] args)
+		static public void Error (int code, Location loc, string format, params string[] args)
 		{
 			Error (code, loc, String.Format (format, args));
 		}
 
-		static public void Error (int code, Location loc, string error)
+		static void Error (int code, Location loc, string error)
 		{
 			new ErrorMessage ().Print (code, loc, error);
 		}
@@ -586,7 +579,7 @@ namespace Mono.CSharp {
 			public abstract bool IsEnabled (int code, bool previous);
 		}
 		
-		class Disable: PragmaCmd
+		class Disable : PragmaCmd
 		{
 			int code;
 			public Disable (int line, int code)
@@ -601,7 +594,7 @@ namespace Mono.CSharp {
 			}
 		}
 
-		class DisableAll: PragmaCmd
+		class DisableAll : PragmaCmd
 		{
 			public DisableAll (int line)
 				: base (line) {}
@@ -612,7 +605,7 @@ namespace Mono.CSharp {
 			}
 		}
 
-		class Enable: PragmaCmd
+		class Enable : PragmaCmd
 		{
 			int code;
 			public Enable (int line, int code)
@@ -627,7 +620,7 @@ namespace Mono.CSharp {
 			}
 		}
 
-		class EnableAll: PragmaCmd
+		class EnableAll : PragmaCmd
 		{
 			public EnableAll (int line)
 				: base (line) {}
@@ -680,7 +673,7 @@ namespace Mono.CSharp {
 			if (Report.IsValidWarning (code))
 				return true;
 
-			Report.Warning (1691, 1, loc, "'{0}' is not a valid warning number", code);
+			Report.Warning (1691, 1, loc, "`{0}' is not a valid warning number", code.ToString ());
 			return false;
 		}
 	}
