@@ -200,7 +200,7 @@ namespace Mono.CSharp {
 		{
 			object o = root.GetDefinition (name);
 			if (o == null){
-				Report.Error (518, "The predefined type `" + name + "' is not defined");
+				Report.Error (518, "The predefined type `" + name + "' is not defined or imported");
 				return null;
 			}
 
@@ -225,7 +225,7 @@ namespace Mono.CSharp {
 		{
 			object o = root.GetDefinition (name);
 			if (o == null){
-				Report.Error (518, "The predefined type `" + name + "' is not defined");
+				Report.Error (518, "The predefined type `" + name + "' is not defined or imported");
 				return;
 			}
 
@@ -250,7 +250,7 @@ namespace Mono.CSharp {
 		{
 			object o = root.GetDefinition (name);
 			if (o == null){
-				Report.Error (518, "The predefined type `" + name + "' is not defined");
+				Report.Error (518, "The predefined type `" + name + "' is not defined or imported");
 				return;
 			}
 
@@ -275,7 +275,7 @@ namespace Mono.CSharp {
 		{
 			object o = root.GetDefinition (name);
 			if (o == null){
-				Report.Error (518, "The predefined type `" + name + "' is not defined");
+				Report.Error (518, "The predefined type `" + name + "' is not defined or imported");
 				return;
 			}
 
@@ -393,7 +393,8 @@ namespace Mono.CSharp {
 				"System.MarshalByRefObject",
 				"System.Security.CodeAccessPermission",
 				"System.Runtime.CompilerServices.RequiredAttributeAttribute",
-				"System.Runtime.InteropServices.GuidAttribute"
+				"System.Runtime.InteropServices.GuidAttribute",
+				"System.Reflection.AssemblyCultureAttribute"
 			};
 
 			foreach (string cname in classes_second_stage)
@@ -478,11 +479,6 @@ namespace Mono.CSharp {
 			helper_classes.Add (helper_class);
 		}
 		
-		static void Report1530 (Location loc)
-		{
-			Report.Error (1530, loc, "Keyword new not allowed for namespace elements");
-		}
-		
 		static public void PopulateCoreType (TypeContainer root, string name)
 		{
 			DeclSpace ds = (DeclSpace) root.GetDefinition (name);
@@ -515,6 +511,7 @@ namespace Mono.CSharp {
 				if (RootContext.StdLib){
 					foreach (TypeContainer tc in type_container_resolve_order)
 						tc.DefineMembers (root);
+
 				} else {
 					foreach (TypeContainer tc in type_container_resolve_order) {
 						// When compiling corlib, these types have already been
@@ -533,19 +530,13 @@ namespace Mono.CSharp {
 			ArrayList delegates = root.Delegates;
 			if (delegates != null){
 				foreach (Delegate d in delegates)
-					if ((d.ModFlags & Modifiers.NEW) == 0)
-						d.DefineMembers (root);
-					else
-						Report1530 (d.Location);
+					d.DefineMembers (root);
 			}
 
 			ArrayList enums = root.Enums;
 			if (enums != null){
 				foreach (Enum en in enums)
-					if ((en.ModFlags & Modifiers.NEW) == 0)
-						en.DefineMembers (root);
-					else
-						Report1530 (en.Location);
+					en.DefineMembers (root);
 			}
 
 			//
@@ -575,6 +566,12 @@ namespace Mono.CSharp {
 		{
 			TypeContainer root = Tree.Types;
 
+			ArrayList delegates = root.Delegates;
+			if (delegates != null){
+				foreach (Delegate d in delegates)
+					d.Define ();
+			}
+
 			if (type_container_resolve_order != null){
 				foreach (TypeContainer tc in type_container_resolve_order) {
 					// When compiling corlib, these types have already been
@@ -586,23 +583,14 @@ namespace Mono.CSharp {
 					     (tc.Name == "System.Runtime.CompilerServices.IndexerNameAttribute")))
 						continue;
 
-					if ((tc.ModFlags & Modifiers.NEW) == 0)
-						tc.Define ();
+					tc.Define ();
 				}
 			}
 
-			ArrayList delegates = root.Delegates;
-			if (delegates != null){
-				foreach (Delegate d in delegates)
-					if ((d.ModFlags & Modifiers.NEW) == 0)
-						d.Define ();
-			}
-
 			ArrayList enums = root.Enums;
 			if (enums != null){
 				foreach (Enum en in enums)
-					if ((en.ModFlags & Modifiers.NEW) == 0)
-						en.Define ();
+					en.Define ();
 			}
 		}
 
@@ -689,6 +677,14 @@ namespace Mono.CSharp {
 			
 			return fb;
 		}
+
+		public static void CheckUnsafeOption (Location loc)
+		{
+			if (!Unsafe) {
+				Report.Error (227, loc, 
+					"Unsafe code requires the `unsafe' command line option to be specified");
+			}
+		}
 	}
 }
 	      
