@@ -20,15 +20,18 @@ namespace Mono.CSharp {
 
 	public interface ParameterData {
 		Type ParameterType (int pos);
+		Type [] Types { get; }
 		int  Count { get; }
 		bool HasParams { get; }
 		string ParameterName (int pos);
 		string ParameterDesc (int pos);
 		Parameter.Modifier ParameterModifier (int pos);
+		string GetSignatureForError ();
 	}
 
 	public class ReflectionParameters : ParameterData {
 		ParameterInfo [] pi;
+		Type [] types;
 		bool last_arg_is_params = false;
 		bool is_varargs = false;
 		
@@ -42,6 +45,14 @@ namespace Mono.CSharp {
 			this.pi = pi;
 			int count = pi.Length-1;
 
+			if (pi.Length == 0) {
+				types = TypeManager.NoTypes;
+			} else {
+				types = new Type [pi.Length];
+				for (int i = 0; i < pi.Length; i++)
+					types [i] = pi [i].ParameterType;
+			}
+
 			if (count >= 0) {
 				attrs = pi [count].GetCustomAttributes (TypeManager.param_array_type, true);
 
@@ -54,7 +65,24 @@ namespace Mono.CSharp {
 				last_arg_is_params = true;
 			}
 		}
-		       
+		
+		public string GetSignatureForError ()
+		{
+			StringBuilder sb = new StringBuilder ("(");
+			for (int i = 0; i < pi.Length; ++i) {
+				if (i != 0)
+					sb.Append (", ");
+				sb.Append (ParameterDesc (i));
+			}
+			if (is_varargs) {
+				if (pi.Length > 0)
+					sb.Append (", ");
+				sb.Append ("__arglist");
+			}
+			sb.Append (')');
+			return sb.ToString ();
+		}
+
 		public Type ParameterType (int pos)
 		{
 			if (last_arg_is_params && pos >= pi.Length - 1)
@@ -99,8 +127,8 @@ namespace Mono.CSharp {
 
 			if (pos >= pi.Length - 1 && last_arg_is_params)
 				sb.Append ("params ");
-			
-			sb.Append (TypeManager.CSharpName (partype));
+
+			sb.Append (TypeManager.CSharpName (partype).Replace ("&", ""));
 
 			return sb.ToString ();
 			
@@ -116,9 +144,9 @@ namespace Mono.CSharp {
 			Type t = pi [pos].ParameterType;
 			if (t.IsByRef){
 				if ((pi [pos].Attributes & (ParameterAttributes.Out|ParameterAttributes.In)) == ParameterAttributes.Out)
-					return Parameter.Modifier.ISBYREF | Parameter.Modifier.OUT;
+					return Parameter.Modifier.OUT;
 				else
-					return Parameter.Modifier.ISBYREF | Parameter.Modifier.REF;
+					return Parameter.Modifier.REF;
 			}
 			
 			return Parameter.Modifier.NONE;
@@ -135,112 +163,12 @@ namespace Mono.CSharp {
 				return this.last_arg_is_params;
 			}
 		}
-		
-	}
 
-	public class InternalParameters : ParameterData {
-		Type [] param_types;
-		bool has_varargs;
-		int count;
-
-		public readonly Parameters Parameters;
-		
-		public InternalParameters (Type [] param_types, Parameters parameters)
-		{
-			this.param_types = param_types;
-			this.Parameters = parameters;
-
-			has_varargs = parameters.HasArglist;
-
-			if (param_types == null)
-				count = 0;
-			else
-				count = param_types.Length;
-		}
-
-		public int Count {
+		public Type[] Types {
 			get {
-				return has_varargs ? count + 1 : count;
+				return types;
 			}
 		}
-
-		public bool HasParams {
-			get {
-				return Parameters.ArrayParameter != null;
-			}
-		}
-
-		Parameter GetParameter (int pos)
-		{
-			Parameter [] fixed_pars = Parameters.FixedParameters;
-			if (fixed_pars != null){
-				int len = fixed_pars.Length;
-				if (pos < len)
-					return Parameters.FixedParameters [pos];
-			}
-
-			return Parameters.ArrayParameter;
-		}
-
-		public Type ParameterType (int pos)
-		{
-			if (has_varargs && pos >= count)
-				return TypeManager.runtime_argument_handle_type;
-
-			if (param_types == null)
-				return null;
-
-			return GetParameter (pos).ExternalType ();
-		}
-
-
-		public string ParameterName (int pos)
-		{
-			if (has_varargs && pos >= count)
-				return "__arglist";
-
-			return GetParameter (pos).Name;
-		}
-
-		public string ParameterDesc (int pos)
-		{
-			if (has_varargs && pos >= count)
-				return "__arglist";
-
-			Type t = ParameterType (pos);
-			return ModifierDesc (pos) + " " + TypeManager.CSharpName (t);
-		}
-
-		public string ModifierDesc (int pos)
-		{
-			Parameter p = GetParameter (pos);
-
-			//
-			// We need to and for REF/OUT, because if either is set the
-			// extra flag ISBYREF will be set as well
-			//
-			if ((p.ModFlags & Parameter.Modifier.REF) != 0)
-				return "ref";
-			if ((p.ModFlags & Parameter.Modifier.OUT) != 0)
-				return "out";
-			if (p.ModFlags == Parameter.Modifier.PARAMS)
-				return "params";
-
-			return "";
-		}
-
-		public Parameter.Modifier ParameterModifier (int pos)
-		{
-			if (has_varargs && pos >= count)
-				return Parameter.Modifier.ARGLIST;
-
-			Parameter.Modifier mod = GetParameter (pos).ModFlags;
-
-			if ((mod & (Parameter.Modifier.REF | Parameter.Modifier.OUT)) != 0)
-				mod |= Parameter.Modifier.ISBYREF;
-
-			return mod;
-		}
 		
 	}
 
@@ -320,116 +248,67 @@ namespace Mono.CSharp {
 	}
 
 	/// <summary>
-	///   This is a wrapper around StreamReader which is seekable.
+	///   This is a wrapper around StreamReader which is seekable backwards
+	///   within a window of around 2048 chars.
 	/// </summary>
 	public class SeekableStreamReader
 	{
 		public SeekableStreamReader (StreamReader reader)
 		{
 			this.reader = reader;
-			this.buffer = new char [DefaultCacheSize];
-			
-			// Compute the preamble size
-			
+			this.buffer = new char [AverageReadLength * 3];
+
 			// Let the StreamWriter autodetect the encoder
 			reader.Peek ();
-			
-			reader.BaseStream.Position = 0;
-			Encoding enc = reader.CurrentEncoding;
-			// First of all, get at least a char
-			
-			byte[] auxb = new byte [50];
-			int num_bytes = 0;
-			int num_chars = 0;
-			int br = 0;
-			do {
-				br = reader.BaseStream.Read (auxb, num_bytes, auxb.Length - num_bytes);
-				num_bytes += br;
-				num_chars = enc.GetCharCount (auxb, 0, num_bytes);
-			}
-			while (num_chars == 0 && br > 0);
-			
-			if (num_chars != 0)
-			{
-				// Now, check which bytes at the beginning have no effect in the
-				// char count
-				
-				int p = 0;
-				while (enc.GetCharCount (auxb, p, num_bytes-p) >= num_chars)
-					p++;
-				
-				preamble_size = p - 1;
-				reader.BaseStream.Position = 0;
-				reader.DiscardBufferedData ();
-				
-				buffer_start = preamble_size;
-			}
 		}
 
-		public SeekableStreamReader (Stream stream, Encoding encoding, bool detect_encoding_from_bytemarks)
-			: this (new StreamReader (stream, encoding, detect_encoding_from_bytemarks))
+		public SeekableStreamReader (Stream stream, Encoding encoding)
+			: this (new StreamReader (stream, encoding, true))
 		{ }
 
 		StreamReader reader;
 
-		private const int DefaultCacheSize = 1024;
+		private const int AverageReadLength = 1024;
 
 		char[] buffer;
-		int buffer_start;       // in bytes
-		int buffer_size;        // in bytes
+		int buffer_start;       // in chars
 		int char_count;         // count buffer[] valid characters
 		int pos;                // index into buffer[]
-		int preamble_size;
 
 		/// <remarks>
-		///   The difference to the StreamReader's BaseStream.Position is that this one is reliable; ie. it
-		//    always reports the correct position and if it's modified, it also takes care of the buffered data.
+		///   This value corresponds to the current position in a stream of characters.
+		///   The StreamReader hides its manipulation of the underlying byte stream and all
+		///   character set/decoding issues.  Thus, we cannot use this position to guess at
+		///   the corresponding position in the underlying byte stream even though there is
+		///   a correlation between them.
 		/// </remarks>
 		public int Position {
-			get {
-				return buffer_start + reader.CurrentEncoding.GetByteCount (buffer, 0, pos);
-			}
+			get { return buffer_start + pos; }
 
 			set {
-				// This one is easy: we're modifying the position within our current
-				// buffer.
-				if ((value >= buffer_start) && (value < buffer_start + buffer_size)) {
-					int byte_offset = value - buffer_start;
-
-					// pos is an index into a char
-					// buffer so it might be
-					// greater than the buffer
-					// length now, if the buffer
-					// contains multibyte chars
-					pos = byte_offset;
-					
-					// encoded characters can take
-					// more than 1 byte length.
-					while ((pos > buffer.Length) ||
-					       reader.CurrentEncoding.GetByteCount (buffer, 0, pos) > byte_offset) {
-						pos--;
-					}
-					
-					return;
-				}
-				
-				if (value == 0)	// Skip preamble
-					value = preamble_size;
-
-				// Ok, now we need to seek.
-				reader.DiscardBufferedData ();
-				reader.BaseStream.Position = buffer_start = value;
-				char_count = buffer_size = pos = 0;
+				if (value < buffer_start || value > buffer_start + char_count)
+					throw new InternalErrorException ("can't seek that far back: " + (pos - value));
+				pos = value - buffer_start;
 			}
 		}
 
 		private bool ReadBuffer ()
 		{
-			pos = 0;
-			buffer_start += buffer_size;
-			char_count = reader.Read (buffer, 0, buffer.Length);
-			buffer_size = reader.CurrentEncoding.GetByteCount (buffer, 0, char_count);
-			return buffer_size > 0;
+			int slack = buffer.Length - char_count;
+			if (slack <= AverageReadLength / 2) {
+				// shift the buffer to make room for AverageReadLength number of characters
+				int shift = AverageReadLength - slack;
+				Array.Copy (buffer, shift, buffer, 0, char_count - shift);
+				pos -= shift;
+				char_count -= shift;
+				buffer_start += shift;
+				slack += shift;		// slack == AverageReadLength
+			}
+
+			int chars_read = reader.Read (buffer, char_count, slack);
+			char_count += chars_read;
+
+			return pos < char_count;
 		}
 
 		public int Peek ()
