@@ -11,20 +11,27 @@ namespace TestRunner {
 	{
 		string Output { get; }
 		bool Invoke (string[] args);
+		bool IsWarning (int warningNumber);
 	}
 
 	class ReflectionTester: ITester {
 		MethodInfo ep;
 		object[] method_arg;
 		StringWriter output;
+		int[] all_warnings;
 
 		public ReflectionTester (Assembly a)
 		{
-			ep = a.GetType ("Mono.CSharp.CompilerCallableEntryPoint").GetMethod ("InvokeCompiler", 
+			Type t = a.GetType ("Mono.CSharp.CompilerCallableEntryPoint");
+			ep = t.GetMethod ("InvokeCompiler", 
 				BindingFlags.Static | BindingFlags.Public);
 			if (ep == null)
 				throw new MissingMethodException ("static InvokeCompiler");
 			method_arg = new object [2];
+
+			PropertyInfo pi = t.GetProperty ("AllWarningNumbers");
+			all_warnings = (int[])pi.GetValue (null, null);
+			Array.Sort (all_warnings);
 		}
 
 		public string Output {
@@ -40,6 +47,11 @@ namespace TestRunner {
 			method_arg [1] = output;
 			return (bool)ep.Invoke (null, method_arg);
 		}
+
+		public bool IsWarning (int warningNumber)
+		{
+			return Array.BinarySearch (all_warnings, warningNumber) >= 0;
+		}
 	}
 
 	class ProcessTester: ITester
@@ -79,6 +91,11 @@ namespace TestRunner {
 			p.WaitForExit ();
 			return p.ExitCode == 0;
 		}
+
+		public bool IsWarning (int warningNumber)
+		{
+			throw new NotImplementedException ();
+		}
 	}
 
 	class Checker: IDisposable
@@ -195,7 +212,7 @@ namespace TestRunner {
 			}
 		}
 
-		public void PrintSummary ()
+		public virtual void PrintSummary ()
 		{
 			LogLine ("Done" + Environment.NewLine);
 			LogLine ("{0} test cases passed ({1:.##%})", success, (float) (success) / (float)total);
@@ -349,7 +366,7 @@ namespace TestRunner {
 				if (!ExecuteFile (file, filename))
 					return false;
 			} else {
-				if (!ExecuteFile (mi, filename))
+				if (!ExecuteFile (mi, file, filename))
 					return false;
 			}
 
@@ -368,7 +385,7 @@ namespace TestRunner {
 			return true;
 		}
 
-		bool ExecuteFile (string exe_name, string filename)
+		int ExecFile (string exe_name, string filename)
 		{
 			if (mono == null)
 				pi.FileName = exe_name;
@@ -377,19 +394,23 @@ namespace TestRunner {
 
 			Process p = Process.Start (pi);
 			p.WaitForExit ();
+			return p.ExitCode;
+		}
 
-			// TODO: How can I recognize return type void ?
-			if (p.ExitCode == 0)
+		bool ExecuteFile (string exe_name, string filename)
+		{
+			int exit_code = ExecFile (exe_name, filename);
+			if (exit_code == 0)
 				return true;
 
-			HandleFailure (filename, TestResult.ExecError, "Wrong return code: " + p.ExitCode.ToString ());
+			HandleFailure (filename, TestResult.ExecError, "Wrong return code: " + exit_code);
 			return false;
 		}
 
-		bool ExecuteFile (MethodInfo entry_point, string filename)
+		bool ExecuteFile (MethodInfo entry_point, string exe_name, string filename)
 		{
-			TextWriter standart_ouput = Console.Out;
-			TextWriter standart_error = Console.Error;
+			TextWriter stdout = Console.Out;
+			TextWriter stderr = Console.Error;
 			Console.SetOut (TextWriter.Null);
 			Console.SetError (TextWriter.Null);
 			ParameterInfo[] pi = entry_point.GetParameters ();
@@ -397,13 +418,20 @@ namespace TestRunner {
 
 			object result = null;
 			try {
-				result = entry_point.Invoke (null, args);
-				Console.SetOut (standart_ouput);
-				Console.SetError (standart_error);
+				try {
+					result = entry_point.Invoke (null, args);
+				} finally {
+					Console.SetOut (stdout);
+					Console.SetError (stderr);
+				}
 			}
 			catch (Exception e) {
-				Console.SetOut (standart_ouput);
-				Console.SetError (standart_error);
+				int exit_code = ExecFile (exe_name, filename);
+				if (exit_code == 0) {
+					LogLine ("(appdomain method failed, external executable succeeded)");
+					LogLine (e.ToString ());
+					return true;
+				}
 				HandleFailure (filename, TestResult.ExecError, e.ToString ());
 				return false;
 			}
@@ -470,17 +498,24 @@ namespace TestRunner {
 	{
 		string expected_message;
 		string error_message;
+		bool check_msg;
+		bool check_error_line;
+		bool is_warning;
+		IDictionary wrong_warning;
 
 		protected enum CompilerError {
 			Expected,
 			Wrong,
 			Missing,
-			WrongMessage
+			WrongMessage,
+			MissingLocation
 		}
 
-		public NegativeChecker (ITester tester, string log_file, string issue_file):
+		public NegativeChecker (ITester tester, string log_file, string issue_file, bool check_msg):
 			base (tester, log_file, issue_file)
 		{
+			this.check_msg = check_msg;
+			wrong_warning = new Hashtable ();
 		}
 
 		protected override bool AnalyzeTestFile(int row, string line)
@@ -499,7 +534,28 @@ namespace TestRunner {
 				expected_message = line.Substring (index + 1).Trim ();
 			}
 
-			return base.AnalyzeTestFile (row, line);
+			if (row == 2) {
+				string filtered = line.Replace (" ", "");
+				check_error_line = !filtered.StartsWith ("//Line:0");
+
+				if (!filtered.StartsWith ("//Line:")) {
+					LogLine ("IGNORING: Wrong test syntax (second line must have `// Line: xx' syntax");
+					++ignored;
+					return false;
+				}
+			}
+
+			if (!base.AnalyzeTestFile (row, line))
+				return false;
+
+			is_warning = false;
+			if (compiler_options != null) {
+				foreach (string s in compiler_options) {
+					if (s.EndsWith ("warnaserror"))
+						is_warning = true;
+				}
+			}
+			return true;
 		}
 
 
@@ -524,6 +580,15 @@ namespace TestRunner {
 				return false;
 			}
 
+			int err_id = int.Parse (expected, System.Globalization.CultureInfo.InvariantCulture);
+			if (tester.IsWarning (err_id)) {
+				if (!is_warning)
+					wrong_warning [err_id] = true;
+			} else {
+				if (is_warning)
+					wrong_warning [err_id] = false;
+			}
+
 			CompilerError result_code = GetCompilerError (expected, tester.Output);
 			if (HandleFailure (filename, result_code)) {
 				success++;
@@ -547,11 +612,21 @@ namespace TestRunner {
 			while (line != null) {
 
 				if (line.IndexOf (tested_text) != -1) {
-//					string msg = line.Substring (line.IndexOf (':', 22) + 1).TrimEnd ('.').Trim ();
-//					if (msg != expected_message && msg != expected_message.Replace ('`', '\'')) {
-//						error_message = msg;
-//						return CompilerError.WrongMessage;
-//					}
+					if (check_msg) {
+						int first = line.IndexOf (':');
+						int second = line.IndexOf (':', first + 1);
+						if (second == -1 || !check_error_line)
+							second = first;
+
+						string msg = line.Substring (second + 1).TrimEnd ('.').Trim ();
+						if (msg != expected_message && msg != expected_message.Replace ('`', '\'')) {
+							error_message = msg;
+							return CompilerError.WrongMessage;
+						}
+
+						if (check_error_line && line.IndexOf (".cs(") == -1)
+							return CompilerError.MissingLocation;
+					}
 					return CompilerError.Expected;
 				}
 
@@ -603,8 +678,8 @@ namespace TestRunner {
 					}
 					else {
 						LogLine ("REGRESSION (CORRECT ERROR -> WRONG ERROR MESSAGE)");
-						Console.WriteLine ("E: {0}", expected_message);
-						Console.WriteLine ("W: {0}", error_message);
+						LogLine ("Exp: {0}", expected_message);
+						LogLine ("Was: {0}", error_message);
 					}
 					break;
 
@@ -624,11 +699,41 @@ namespace TestRunner {
 					}
 
 					break;
+
+				case CompilerError.MissingLocation:
+					if (know_issues.Contains (file)) {
+						LogLine ("KNOWN ISSUE (Missing error location)");
+						know_issues.Remove (file);
+						return false;
+					}
+					if (no_error_list.Contains (file)) {
+						LogLine ("REGRESSION (NO ERROR -> MISSING ERROR LOCATION)");
+						no_error_list.Remove (file);
+					}
+					else {
+						LogLine ("REGRESSION (CORRECT ERROR -> MISSING ERROR LOCATION)");
+					}
+					break;
+
 			}
 
 			regression.Add (file);
 			return false;
 		}
+
+		public override void PrintSummary()
+		{
+			base.PrintSummary ();
+
+			if (wrong_warning.Count > 0) {
+				LogLine ("");
+				LogLine ("List of not corectly defined warnings which should be either defined in compiler as a warning or their test case has redundant `warnaserror' option");
+				LogLine ("");
+				foreach (DictionaryEntry de in wrong_warning)
+					LogLine ("CS{0:0000} : {1}", de.Key, (bool)de.Value ? "incorrect warning definition" : "missing warning definition");
+			}
+		}
+
 	}
 
 	class Tester {
@@ -664,7 +769,7 @@ namespace TestRunner {
 			Checker checker;
 			switch (mode) {
 				case "negative":
-					checker = new NegativeChecker (tester, log_fname, issue_file);
+					checker = new NegativeChecker (tester, log_fname, issue_file, true);
 					break;
 				case "positive":
 					checker = new PositiveChecker (tester, log_fname, issue_file);
