@@ -193,28 +193,45 @@ namespace Mono.Tools {
 				string gacdir, string link_gacdir, string libdir, string link_libdir)
 		{
 			string failure_msg = "Failure adding assembly to the cache: ";
+			ArrayList resources;
 
 			if (!File.Exists (name)) {
 				WriteLine (failure_msg + "The system cannot find the file specified.");
 				Environment.Exit (1);
 			}
 
+			Assembly assembly = null;
 			AssemblyName an = null;
 			byte [] pub_tok;
 
 			try {
-				an = AssemblyName.GetAssemblyName (name);
+				assembly = Assembly.LoadFrom (name);
 			} catch {
 				WriteLine (failure_msg + "The file specified is not a valid assembly.");
 				return false;
 			}
 
+			an = assembly.GetName ();
 			pub_tok = an.GetPublicKeyToken ();
 			if (pub_tok == null || pub_tok.Length == 0) {
 				WriteLine (failure_msg + "Attempt to install an assembly without a strong name.");
 				return false;
 			}
 
+			resources = new ArrayList ();
+			foreach (string res_name in assembly.GetManifestResourceNames ()) {
+				ManifestResourceInfo res_info = assembly.GetManifestResourceInfo (res_name);
+				
+				if ((res_info.ResourceLocation & ResourceLocation.Embedded) == 0) {
+					if (!File.Exists (res_info.FileName)) {
+						WriteLine (failure_msg + "The system cannot find resource " + res_info.FileName);
+						return false;
+					}
+
+					resources.Add (res_info);
+				}
+			}
+
 			if (check_refs && !CheckReferencedAssemblies (an)) {
 				WriteLine (failure_msg + "Attempt to install an assembly that references non " +
 						"strong named assemblies with -check_refs enabled.");
@@ -250,6 +267,15 @@ namespace Mono.Tools {
 					File.Copy (sibling, String.Concat (asmb_path, ext), true);
 			}
 
+			foreach (ManifestResourceInfo resource_info in resources) {
+				try {
+					File.Copy (resource_info.FileName, Path.Combine (full_path, Path.GetFileName (resource_info.FileName)), true);
+				} catch {
+					WriteLine ("ERROR: Could not install resource file " + resource_info.FileName);
+					Environment.Exit (1);
+				}
+			}
+
 			if (package != null) {
 				string ref_dir = Path.Combine (libdir, package);
 				string ref_path = Path.Combine (ref_dir, asmb_file);
@@ -265,6 +291,20 @@ namespace Mono.Tools {
  				if (Path.DirectorySeparatorChar == '/') {
 					string pkg_path = "../gac/" + an.Name + "/" + version_token + "/" + asmb_file;
  					symlink (pkg_path, ref_path);
+
+					foreach (string ext in siblings) {
+						string sibling = String.Concat (pkg_path, ext);
+						string sref = String.Concat (ref_path, ext);
+						if (File.Exists (sibling))
+							symlink (sibling, sref);
+						else {
+							try {
+								File.Delete (sref);
+							} catch {
+								// Ignore error, just delete files that should not be there.
+							}
+						}
+					}
 					WriteLine ("Package exported to: {0} -> {1}", ref_path, pkg_path);
  				} else {
 					// string link_path = Path.Combine (Path.Combine (link_gacdir, an.Name), version_token);
@@ -286,6 +326,7 @@ namespace Mono.Tools {
 			Hashtable asm_info = new Hashtable ();
 
 			foreach (string item in assembly_pieces) {
+				if (item == String.Empty) continue;
 				string[] pieces = item.Trim ().Split (new char[] { '=' }, 2);
 				if(pieces.Length == 1)
 					asm_info ["assembly"] = pieces [0];
@@ -310,15 +351,23 @@ namespace Mono.Tools {
 					File.Delete (link);
 					if (Directory.GetFiles (link_dir).Length == 0) {
 						WriteLine ("Cleaning package directory, it is empty.");
-						Directory.Delete (link_dir);
+						try {
+							Directory.Delete (link_dir);
+						} catch {
+							// Workaround: GetFiles does not list Symlinks
+						}
 					}
 				}
 				WriteLine ("Assembly removed from the gac.");
 			}
 
 			if(Directory.GetDirectories (asmdir).Length == 0) {
-				WriteLine ("Cleaning assembly dir, its empty");
-				Directory.Delete (asmdir);
+				WriteLine ("Cleaning assembly dir, it is empty");
+				try {
+					Directory.Delete (asmdir);
+				} catch {
+					// Workaround: GetFiles does not list Symlinks
+				}
 			}
 
 			return true;
