@@ -14,7 +14,8 @@ using System.Collections;
 using System.Reflection;
 using System.IO;
 using System.Runtime.InteropServices;
-using Mono.Posix;
+using Mono.Unix;
+using ICSharpCode.SharpZipLib.Zip.Compression.Streams;
 
 class MakeBundle {
 	static string output = "a.out";
@@ -26,6 +27,8 @@ class MakeBundle {
 	static bool static_link = false;
 	static string config_file = null;
 	static string config_dir = null;
+	static string style = "linux";
+	static bool compress;
 	
 	static int Main (string [] args)
 	{
@@ -80,6 +83,8 @@ class MakeBundle {
 				break;
 			case "--static":
 				static_link = true;
+				Console.WriteLine ("Note that statically linking the LGPL Mono runtime has more licensing restrictions than dynamically linking.");
+				Console.WriteLine ("See http://www.mono-project.com/Licensing for details on licensing.");
 				break;
 			case "--config":
 				if (i+1 == top) {
@@ -97,12 +102,17 @@ class MakeBundle {
 
 				config_dir = args [++i];
 				break;
+			case "-z":
+				compress = true;
+				break;
 			default:
 				sources.Add (args [i]);
 				break;
 			}
 		}
 
+		DetectOS ();
+
 		Console.WriteLine ("Sources: {0} Auto-dependencies: {1}", sources.Count, autodeps);
 		if (sources.Count == 0 || output == null) {
 			Help ();
@@ -122,15 +132,28 @@ class MakeBundle {
 
 	static void WriteSymbol (StreamWriter sw, string name, long size)
 	{
-		sw.WriteLine (
-	        	".globl {0}\n" +
-			"\t.section .rodata\n" +
-			"\t.align 32\n" +
-			"\t.type {0}, @object\n" +
-			"\t.size {0}, {1}\n" +
-			"{0}:\n",
-			name, size);
-
+		switch (style){
+		case "linux":
+			sw.WriteLine (
+				".globl {0}\n" +
+				"\t.section .rodata\n" +
+				"\t.align 32\n" +
+				"\t.type {0}, @object\n" +
+				"\t.size {0}, {1}\n" +
+				"{0}:\n",
+				name, size);
+			break;
+		case "osx":
+			sw.WriteLine (
+				"\t.section __TEXT,__text,regular,pure_instructions\n" + 
+				"\t.section __TEXT,__picsymbolstub1,symbol_stubs,pure_instructions,32\n" + 
+				"\t.globl _{0}\n" +
+				"\t.data\n" +
+				"\t.align 4\n" +
+				"_{0}:\n",
+				name, size);
+			break;
+		}
 	}
 	
 	static void GenerateBundles (ArrayList files)
@@ -155,6 +178,14 @@ class MakeBundle {
 
 			tc.WriteLine ("/* This source code was produced by mkbundle, do not edit */");
 			tc.WriteLine ("#include <mono/metadata/assembly.h>\n");
+
+			if (compress) {
+				tc.WriteLine ("typedef struct _compressed_data {");
+				tc.WriteLine ("\tMonoBundledAssembly assembly;");
+				tc.WriteLine ("\tint compressed_size;");
+				tc.WriteLine ("} CompressedAssembly;\n");
+			}
+
 			foreach (string url in files){
 				string fname = url.Substring (7);
 				string aname = fname.Substring (fname.LastIndexOf ("/") + 1);
@@ -165,22 +196,44 @@ class MakeBundle {
 				
 				Console.WriteLine ("   embedding: " + fname);
 				
-				FileInfo fi = new FileInfo (fname);
-				FileStream fs = File.OpenRead (fname);
-
-				WriteSymbol (ts, "assembly_data_" + encoded, fi.Length);
+				Stream stream = File.OpenRead (fname);
 
+				long real_size = stream.Length;
 				int n;
-				while ((n = fs.Read (buffer, 0, 8192)) != 0){
+				if (compress) {
+					MemoryStream ms = new MemoryStream ();
+					DeflaterOutputStream deflate = new DeflaterOutputStream (ms);
+					while ((n = stream.Read (buffer, 0, 8192)) != 0){
+						deflate.Write (buffer, 0, n);
+					}
+					stream.Close ();
+					deflate.Finish ();
+					byte [] bytes = ms.GetBuffer ();
+					stream = new MemoryStream (bytes, 0, (int) ms.Length, false, false);
+				}
+
+				WriteSymbol (ts, "assembly_data_" + encoded, stream.Length);
+
+				while ((n = stream.Read (buffer, 0, 8192)) != 0){
 					for (int i = 0; i < n; i++){
 						ts.Write ("\t.byte {0}\n", buffer [i]);
 					}
 				}
 				ts.WriteLine ();
 
-				tc.WriteLine ("extern const unsigned char assembly_data_{0} [];", encoded);
-				tc.WriteLine ("static const MonoBundledAssembly assembly_bundle_{0} = {{\"{1}\", assembly_data_{0}, {2}}};",
-					      encoded, aname, fi.Length);
+				if (compress) {
+					tc.WriteLine ("extern const unsigned char assembly_data_{0} [];", encoded);
+					tc.WriteLine ("static CompressedAssembly assembly_bundle_{0} = {{{{\"{1}\"," +
+							" assembly_data_{0}, {2}}}, {3}}};",
+						      encoded, aname, real_size, stream.Length);
+					double ratio = ((double) stream.Length * 100) / real_size;
+					Console.WriteLine ("   compression ratio: {0:.00}%", ratio);
+				} else {
+					tc.WriteLine ("extern const unsigned char assembly_data_{0} [];", encoded);
+					tc.WriteLine ("static const MonoBundledAssembly assembly_bundle_{0} = {{\"{1}\", assembly_data_{0}, {2}}};",
+						      encoded, aname, real_size);
+				}
+				stream.Close ();
 
 				c_bundle_names.Add ("assembly_bundle_" + encoded);
 
@@ -234,7 +287,11 @@ class MakeBundle {
 				return;
 			}
 
-			tc.WriteLine ("\nstatic const MonoBundledAssembly *bundled [] = {");
+			if (compress)
+				tc.WriteLine ("\nstatic const CompressedAssembly *compressed [] = {");
+			else
+				tc.WriteLine ("\nstatic const MonoBundledAssembly *bundled [] = {");
+
 			foreach (string c in c_bundle_names){
 				tc.WriteLine ("\t&{0},", c);
 			}
@@ -253,25 +310,42 @@ class MakeBundle {
 				tc.WriteLine ("static const char *config_dir = \"{0}\";", config_dir);
 			else
 				tc.WriteLine ("static const char *config_dir = NULL;");
-				      
-			StreamReader s = new StreamReader (Assembly.GetAssembly (typeof(MakeBundle)).GetManifestResourceStream ("template.c"));
-			tc.Write (s.ReadToEnd ());
-			tc.Close ();
 
+			Stream template_stream;
+			if (compress) {
+				template_stream = Assembly.GetAssembly (typeof(MakeBundle)).GetManifestResourceStream ("template_z.c");
+			} else {
+				template_stream = Assembly.GetAssembly (typeof(MakeBundle)).GetManifestResourceStream ("template.c");
+			}
+
+			StreamReader s = new StreamReader (template_stream);
+			string template = s.ReadToEnd ();
+			tc.Write (template);
+			tc.Close ();
+			
 			if (compile_only)
 				return;
 
-			if (static_link)
-				cmd = String.Format ("cc -o {2} -Wall `pkg-config --cflags mono` {0} " +
-						     "`pkg-config --libs-only-L mono` -Wl,-Bstatic " +
-						     "`pkg-config --libs-only-l mono | sed -e \"s/\\-lm //\" | " +
-						     "sed -e \"s/\\-ldl //\" | sed -e \"s/\\-lpthread //\"` " +
-						     "-Wl,-Bdynamic -ldl -lm -lrt {1}",
-						     temp_c, temp_o, output);
-			else
-				cmd = String.Format ("cc -o {2} -Wall {0} `pkg-config --cflags --libs mono` {1}",
-						     temp_c, temp_o, output);
+			string zlib = (compress ? "-lz" : "");
+			string debugging = "-g";
 
+			if (style == "linux")
+				debugging = "-ggdb";
+			if (static_link) {
+				string smonolib;
+				if (style == "osx")
+					smonolib = "`pkg-config --variable=libdir mono`/libmono.a ";
+				else
+					smonolib = "-Wl,-Bstatic -lmono -Wl,-Bdynamic ";
+				cmd = String.Format ("cc -o {2} -Wall `pkg-config --cflags mono` {0} {3}" +
+						     "`pkg-config --libs-only-L mono` " + smonolib +
+						     "`pkg-config --libs-only-l mono | sed -e \"s/\\-lmono //\"` {1}",
+						     temp_c, temp_o, output, zlib);
+			} else {
+				
+				cmd = String.Format ("cc " + debugging + " -o {2} -Wall {0} `pkg-config --cflags --libs mono` {3} {1}",
+						     temp_c, temp_o, output, zlib);
+			}
                             
 			Console.WriteLine (cmd);
 			ret = system (cmd);
@@ -395,10 +469,29 @@ class MakeBundle {
 				   "    --keeptemp      Keeps the temporary files\n" +
 				   "    --config F      Bundle system config file `F'\n" +
 				   "    --config-dir D  Set MONO_CFG_DIR to `D'\n" +
-				   "    --static        Statically link to mono libs\n");
+				   "    --static        Statically link to mono libs\n" +
+				   "    -z		Compress the assemblies before embedding.\n");
 	}
 
 	[DllImport ("libc")]
 	static extern int system (string s);
+	[DllImport ("libc")]
+	static extern int uname (IntPtr buf);
+		
+	static void DetectOS ()
+	{
+		IntPtr buf = UnixMarshal.Alloc (8192);
+		if (uname (buf) != 0){
+			Console.WriteLine ("Warning: Unable to detect OS");
+			return;
+		}
+		string os = Marshal.PtrToStringAnsi (buf);
+		Console.WriteLine ("OS is: " + os);
+		if (os == "Darwin")
+			style = "osx";
+		
+		UnixMarshal.Free (buf);
+	}
+	
 }
 
