@@ -12,6 +12,7 @@
 #include <string.h>
 #include <ctype.h>
 #include <glib.h>
+#include <math.h>
 #include "meta.h"
 #include "util.h"
 #include "get.h"
@@ -445,7 +446,7 @@ static dis_map_t element_type_map [] = {
 	{ MONO_TYPE_R4         , "float32" },
 	{ MONO_TYPE_R8         , "float64" },
 	{ MONO_TYPE_STRING     , "string" },
-	{ MONO_TYPE_TYPEDBYREF , "TypedByRef" },
+	{ MONO_TYPE_TYPEDBYREF , "typedref" },
 	{ MONO_TYPE_I          , "native int" },
 	{ MONO_TYPE_U          , "native unsigned int" },
 	{ MONO_TYPE_OBJECT     , "object" },
@@ -539,6 +540,106 @@ dis_stringify_param (MonoImage *m, MonoType *param)
 	return result;
 }
 
+static char*
+dis_stringify_variant_type (MonoMarshalVariant variant)
+{
+	switch (variant) {
+	case MONO_VARIANT_EMPTY:
+		return g_strdup ("");
+	case MONO_VARIANT_NULL:
+		return g_strdup ("null");
+	case MONO_VARIANT_I2:
+		return g_strdup ("int16");
+	case MONO_VARIANT_I4:
+		return g_strdup ("int32");
+	case MONO_VARIANT_R4:
+		return g_strdup ("float32");
+	case MONO_VARIANT_R8:
+		return g_strdup ("float64");
+	case MONO_VARIANT_CY:
+		return g_strdup ("currency");
+	case MONO_VARIANT_DATE:
+		return g_strdup ("date");
+	case MONO_VARIANT_BSTR:
+		return g_strdup ("bstr");
+	case MONO_VARIANT_DISPATCH:
+		return g_strdup ("idispatch");
+	case MONO_VARIANT_ERROR:
+		return g_strdup ("error");
+	case MONO_VARIANT_BOOL:
+		return g_strdup ("bool");
+	case MONO_VARIANT_VARIANT:
+		return g_strdup ("variant");
+	case MONO_VARIANT_UNKNOWN:
+		return g_strdup ("iunknown");
+	case MONO_VARIANT_DECIMAL:
+		return g_strdup ("decimal");
+	case MONO_VARIANT_I1:
+		return g_strdup ("int8");
+	case MONO_VARIANT_UI1:
+		return g_strdup ("unsigned int8");
+	case MONO_VARIANT_UI2:
+		return g_strdup ("unsigned int16");
+	case MONO_VARIANT_UI4:
+		return g_strdup ("unsigned int32");
+	case MONO_VARIANT_I8:
+		return g_strdup ("int64");
+	case MONO_VARIANT_UI8:
+		return g_strdup ("unsigned int64");
+	case MONO_VARIANT_INT:
+		return g_strdup ("int");
+	case MONO_VARIANT_UINT:
+		return g_strdup ("unsigned int");
+	case MONO_VARIANT_VOID:
+		return g_strdup ("void");
+	case MONO_VARIANT_HRESULT:
+		return g_strdup ("hresult");
+	case MONO_VARIANT_PTR:
+		return g_strdup ("*");
+	case MONO_VARIANT_SAFEARRAY:
+		return g_strdup ("safearray");
+	case MONO_VARIANT_CARRAY:
+		return g_strdup ("carray");
+	case MONO_VARIANT_USERDEFINED:
+		return g_strdup ("userdefined");
+	case MONO_VARIANT_LPSTR:
+		return g_strdup ("lpstr");
+	case MONO_VARIANT_LPWSTR:
+		return g_strdup ("lpwstr");
+	case MONO_VARIANT_RECORD:
+		return g_strdup ("record");
+	case MONO_VARIANT_FILETIME:
+		return g_strdup ("filetime");
+	case MONO_VARIANT_BLOB:
+		return g_strdup ("blob");
+	case MONO_VARIANT_STREAM:
+		return g_strdup ("stream");
+	case MONO_VARIANT_STORAGE:
+		return g_strdup ("storage");
+	case MONO_VARIANT_STREAMED_OBJECT:
+		return g_strdup ("streamed_object");
+	case MONO_VARIANT_STORED_OBJECT:
+		return g_strdup ("stored_object");
+	case MONO_VARIANT_BLOB_OBJECT:
+		return g_strdup ("blob_object");
+	case MONO_VARIANT_CF:
+		return g_strdup ("cf");
+	case MONO_VARIANT_CLSID:
+		return g_strdup ("clsid");
+	case MONO_VARIANT_VECTOR:
+		/* FIXME: output: <v_type> vector */
+		return g_strdup ("vector");
+	case MONO_VARIANT_ARRAY:
+		/* FIXME: output: <v_type> [ ] */
+		return g_strdup ("[]");
+	case MONO_VARIANT_BYREF:
+		/* FIXME: output: <v_type> & */
+		return g_strdup ("&");
+	default:
+		return g_strdup ("unknown");
+	}
+}
+
 static char*
 dis_stringify_native_type (MonoMarshalNative native)
 {
@@ -614,16 +715,16 @@ dis_stringify_native_type (MonoMarshalNative native)
 	}
 }
 
-static char*
+char*
 dis_stringify_marshal_spec (MonoMarshalSpec *spec)
 {
 	switch (spec->native) {
 	case MONO_NATIVE_BYVALTSTR:
-		return g_strdup_printf ("fixed sysstring [%d]", spec->data.array_data.num_elem);
+		return g_strdup_printf (" marshal (fixed sysstring [%d])", spec->data.array_data.num_elem);
 	case MONO_NATIVE_BYVALARRAY:
-		return g_strdup_printf ("fixed array [%d]", spec->data.array_data.num_elem);
+		return g_strdup_printf (" marshal (fixed array [%d])", spec->data.array_data.num_elem);
 	case MONO_NATIVE_LPARRAY: {
-		char *elem_type, *elems;
+		char *elem_type, *elems, *ret;
 		guint32 num_elem = spec->data.array_data.num_elem;
 		guint32 param_num = spec->data.array_data.param_num;
 
@@ -636,11 +737,33 @@ dis_stringify_marshal_spec (MonoMarshalSpec *spec)
 			elems = g_strdup_printf ("+ %d", param_num);
 		else
 			elems = g_strdup_printf ("%d + %d", num_elem, param_num);
-
-		return g_strdup_printf ("%s[%s]", elem_type, elems);
+			
+		ret = g_strdup_printf (" marshal (%s[%s])", elem_type, elems);
+		g_free (elem_type);
+		g_free (elems);
+		return ret;
 	}
-	default:
-		return dis_stringify_native_type (spec->native);
+	case MONO_NATIVE_SAFEARRAY: {
+		char *elem_type = NULL, *ret;
+		
+		if (spec->data.safearray_data.elem_type != 0)
+			elem_type = dis_stringify_variant_type (spec->data.safearray_data.elem_type);
+		ret = g_strdup_printf (" marshal (safearray %s)", elem_type ? elem_type : "");
+		
+		g_free (elem_type);
+		return ret;
+	}
+	case MONO_NATIVE_CUSTOM:
+		return g_strdup_printf (" marshal (custom (\"%s\", \"%s\"))", 
+			spec->data.custom_data.custom_name ? spec->data.custom_data.custom_name : "", 
+			spec->data.custom_data.cookie ? spec->data.custom_data.cookie : "");
+	default: {
+		char *native_type, *ret;
+		native_type = dis_stringify_native_type (spec->native);
+		ret = g_strdup_printf (" marshal (%s)", native_type);
+		g_free (native_type);
+		return ret;
+	}	
 	}
 }
 
@@ -711,22 +834,34 @@ get_generic_param (MonoImage *m, MonoGenericContainer *container)
         return retval;
 }
 
+/* 
+ * @m: metadata context
+ * @method: MonoMethodSignature to dis-stringify
+ * @methoddef_row: row index in the Method table
+ * @context: generic context, generic method's context in case of a Generic method
+ *	     or a generic type's context. if !@context, treats it as a non-generic method
+ * @fully_qualified: TRUE to print type name also.
+ *
+ * Returns: Allocated stringified method signature
+ */
 char*
 dis_stringify_method_signature (MonoImage *m, MonoMethodSignature *method, int methoddef_row,
 				MonoGenericContext *context, gboolean fully_qualified)
 {
 	guint32 cols [MONO_METHOD_SIZE];
 	guint32 pcols [MONO_PARAM_SIZE];
-	guint32 param_index = 0;
-	const char *name = "";
+	guint32 param_index = 0, next_param_index = 0;
+	gboolean has_param_row;
+	const char *name = "", *method_name = "";
 	int free_method = 0;
 	char *retval, *esname;
 	char *type = NULL;
-	char *marshal_info = NULL;
+	char *marshal_info = NULL, *ret_marshal_info = NULL;
 	char *gen_param = NULL;
 	GString *result = g_string_new ("");
+	GString *result_ret = g_string_new ("");
 	MonoGenericContainer *container = NULL;
-	int i;
+	int i, start;
         
 	g_assert (method || methoddef_row);
 
@@ -734,15 +869,18 @@ dis_stringify_method_signature (MonoImage *m, MonoMethodSignature *method, int m
 		mono_metadata_decode_row (&m->tables [MONO_TABLE_METHOD], methoddef_row -1, cols, MONO_METHOD_SIZE);
 		if (fully_qualified)
 			type = get_typedef (m, mono_metadata_typedef_from_method (m, methoddef_row));
-		name = mono_metadata_string_heap (m, cols [MONO_METHOD_NAME]);
+		method_name = mono_metadata_string_heap (m, cols [MONO_METHOD_NAME]);
 		param_index = cols [MONO_METHOD_PARAMLIST];
 		if (!method) {
 			const char *sig = mono_metadata_blob_heap (m, cols [MONO_METHOD_SIGNATURE]);
 
 			container = mono_metadata_load_generic_params (
 				m, MONO_TOKEN_METHOD_DEF | methoddef_row, context ? context->container : NULL);
-			if (container)
+			if (container) {
+				mono_metadata_load_generic_param_constraints (
+					m, MONO_TOKEN_METHOD_DEF | methoddef_row, container);
 				context = (MonoGenericContext *) container;
+			}
 
 			mono_metadata_decode_blob_size (sig, &sig);
 			method = mono_metadata_parse_method_signature_full (m, context, methoddef_row, sig, &sig);
@@ -752,32 +890,31 @@ dis_stringify_method_signature (MonoImage *m, MonoMethodSignature *method, int m
 
 		if (container && container->is_method)
 			gen_param = get_generic_param (m, container);
-	}
-	
-	retval = dis_stringify_param (m, method->ret);
-	if (method->hasthis)
-		g_string_append (result, "instance ");
-	g_string_append (result, map (method->call_convention, call_conv_type_map));
-	g_string_sprintfa (result, " %s ", retval);
-	if (type) {
-		char *estype = get_escaped_name (type);
-		g_string_sprintfa (result, "%s::", estype);
-		g_free (estype);
-	}
-	esname = get_escaped_name (name);
-	g_string_append (result, esname);
-	g_free (esname);
-        if (gen_param) {
-                g_string_append (result, gen_param);
-                g_free (gen_param);
-        }
-	g_string_append (result, " (");
-	g_free (retval);
-	for (i = 0; i < method->param_count; ++i) {
-		if (param_index && param_index <= m->tables [MONO_TABLE_PARAM].rows) {
+		
+		if (methoddef_row < m->tables [MONO_TABLE_METHOD].rows) {
+			mono_metadata_decode_row (&m->tables [MONO_TABLE_METHOD], methoddef_row, cols, MONO_METHOD_SIZE);
+			next_param_index = cols [MONO_METHOD_PARAMLIST];
+		} else {
+			next_param_index = m->tables [MONO_TABLE_PARAM].rows + 1;
+		}
+	} 
+
+	start = method->hasthis ? 0 : 1;
+	for (i = 0; i < method->param_count + 1; ++i) {
+		marshal_info = NULL;
+		name = "";
+		has_param_row = param_index && param_index < next_param_index;
+
+		if (method->param_count == 0 && !has_param_row)
+			/* method has zero parameters, and no row for return val in the PARAM table */
+			continue;
+		
+		if (has_param_row)
 			mono_metadata_decode_row (&m->tables [MONO_TABLE_PARAM], param_index - 1, pcols, MONO_PARAM_SIZE);
-			name = mono_metadata_string_heap (m, pcols [MONO_PARAM_NAME]);
-			method->params [i]->attrs = pcols [MONO_PARAM_FLAGS];
+		
+		if (has_param_row && i == pcols [MONO_PARAM_SEQUENCE]) {
+			if (i)
+				name = mono_metadata_string_heap (m, pcols [MONO_PARAM_NAME]);
 
 			if (pcols [MONO_PARAM_FLAGS] & PARAM_ATTRIBUTE_HAS_FIELD_MARSHAL) {
 				const char *tp;
@@ -786,24 +923,58 @@ dis_stringify_method_signature (MonoImage *m, MonoMethodSignature *method, int m
 				g_assert (tp);
 				spec = mono_metadata_parse_marshal_spec (m, tp);
 
-				marshal_info = g_strdup_printf (" marshal (%s)", dis_stringify_marshal_spec (spec));
+				if (i)
+					marshal_info = dis_stringify_marshal_spec (spec);
+				else
+					ret_marshal_info = dis_stringify_marshal_spec (spec);
 			}
-
-			param_index++;
+			param_index ++;
 		} else {
-			name = "";
+			if (i)
+				name = g_strdup_printf ("A_%i", i - start);
 		}
-		if (i)
+
+		if (!i)
+			continue;
+
+		if (i > 1)
 			g_string_append (result, ", ");
-		retval = dis_stringify_param (m, method->params [i]);
+
+		retval = dis_stringify_param (m, method->params [i - 1]);
 
 		esname = get_escaped_name (name);
-		g_string_append_printf (result, "%s %s%s", retval, esname, marshal_info ? marshal_info : "");
+		g_string_append_printf (result, "%s%s %s", retval, marshal_info ? marshal_info : "", esname);
 		g_free (retval);
 		g_free (esname);
+		g_free (marshal_info);
 	}
 	g_string_append (result, ") ");
 
+	retval = dis_stringify_param (m, method->ret);
+
+	if (method->hasthis)
+		g_string_append (result_ret, "instance ");
+	g_string_append (result_ret, map (method->call_convention, call_conv_type_map));
+	g_string_sprintfa (result_ret, " %s%s ", retval, ret_marshal_info ? ret_marshal_info :"");
+	g_free (ret_marshal_info);
+	if (type) {
+		char *estype = get_escaped_name (type);
+		g_string_sprintfa (result_ret, "%s::", estype);
+		g_free (estype);
+	}
+	esname = get_escaped_name (method_name);
+	g_string_append (result_ret, esname);
+	g_free (esname);
+	if (gen_param) {
+		g_string_append (result_ret, gen_param);
+		g_free (gen_param);
+	}
+	g_string_append (result_ret, " (");
+	g_free (retval);
+
+	g_string_prepend (result, result_ret->str);
+	g_string_free (result_ret, FALSE);
+
 	if (show_method_tokens && methoddef_row)
 		g_string_append_printf (result, " /* 0x%X */ ",
 				(methoddef_row >> MONO_TYPEORMETHOD_BITS) | MONO_TOKEN_METHOD_DEF);
@@ -999,14 +1170,14 @@ dis_stringify_type (MonoImage *m, MonoType *type, gboolean is_def)
 	case MONO_TYPE_MVAR:
 		if (is_def) {
 			g_assert (type->data.generic_param->name);
-			bare = g_strdup_printf ("!!%s", type->data.generic_param->name);
+			bare = g_strdup_printf ("!!%s", get_escaped_name (type->data.generic_param->name));
 		} else
 			bare = g_strdup_printf ("!!%d", type->data.generic_param->num);
 		break;
 	case MONO_TYPE_VAR:
 		if (is_def) {
 			g_assert (type->data.generic_param->name);
-			bare = g_strdup_printf ("!%s", type->data.generic_param->name);
+			bare = g_strdup_printf ("!%s", get_escaped_name (type->data.generic_param->name));
 		} else
 			bare = g_strdup_printf ("!%d", type->data.generic_param->num);
 		break;
@@ -1205,6 +1376,7 @@ get_ret_type (MonoImage *m, const char *ptr, char **ret_type, MonoGenericContext
 	GString *str = g_string_new ("");
 	char *mod = NULL;
 	char *allocated_type_string;
+	int has_byref = 0;
 	
 	ptr = get_custom_mod (m, ptr, &mod);
 	if (mod){
@@ -1214,19 +1386,21 @@ get_ret_type (MonoImage *m, const char *ptr, char **ret_type, MonoGenericContext
 	}
 
 	if (*ptr == MONO_TYPE_TYPEDBYREF){
-		g_string_append (str, "typedbyref");
+		g_string_append (str, "typedref");
 		ptr++;
 	} else if (*ptr == MONO_TYPE_VOID){
 		 g_string_append (str, "void");
 		 ptr++;
 	} else {
                 if (*ptr == MONO_TYPE_BYREF){
-			g_string_append (str, "[out] ");
+			has_byref = 1;
 			ptr++;
 		}
 
 		ptr = get_type (m, ptr, &allocated_type_string, context);
 		g_string_append (str, allocated_type_string);
+		if (has_byref)
+			g_string_append (str, "& ");
 		g_free (allocated_type_string);
 	}
 
@@ -1260,7 +1434,7 @@ get_param (MonoImage *m, const char *ptr, char **retval, MonoGenericContext *con
 	}
 	
 	if (*ptr == MONO_TYPE_TYPEDBYREF){
-		g_string_append (str, " typedbyref ");
+		g_string_append (str, " typedref ");
 		ptr++;
 	} else {
 		gboolean by_ref = 0;
@@ -1281,6 +1455,36 @@ get_param (MonoImage *m, const char *ptr, char **retval, MonoGenericContext *con
 	return ptr;
 }
 
+/**
+ * str_escape 
+ * 
+ * @str: string to process
+ * @list: list of chars to escape
+ * 
+ * Returns: an allocated escaped string.
+ */
+static char*
+str_escape (const char *str, const char *list)
+{
+	const char *p = str;
+	GString *res;
+
+	res = g_string_sized_new (strlen (str));
+
+	for (;;) {
+		while (*p && !strchr (list, *p))
+			++p;
+		g_string_append_len (res, str, p - str);
+		if (!*p)
+			break;
+		g_string_append_c (res, '\\');
+		str = p;
+		++p;
+	}
+			
+	return g_string_free (res, FALSE);
+}
+
 /**
  * get_escaped_name
  *
@@ -1291,7 +1495,7 @@ char*
 get_escaped_name (const char *name)
 {
 	const char *s;
-	char *tmp, *tmp2;
+	char *ret, *esc;
 
 	g_assert (key_table);
 
@@ -1313,16 +1517,19 @@ get_escaped_name (const char *name)
 
 	for (s = name; *s; s++) {
 		if (isalnum (*s) || *s == '_' || *s == '$' || *s == '@' ||
-		    *s == '?' || *s == '.' || *s == 0 || *s == '!')
+		    *s == '?' || *s == '.' || *s == 0 || *s == '!' || *s == '`')
 			continue;
 
-		return g_strdup_printf ("'%s'", name);
+		esc = str_escape (name, "'\\");
+		ret = g_strdup_printf ("'%s'", esc);
+		g_free (esc);
+		return ret;
 	}
 	
 	if (g_hash_table_lookup (key_table, name))
 		return g_strdup_printf ("'%s'", name);
 			
-	return g_strdup (name);
+	return str_escape (name, "'\\");
 }
 
 static dis_map_t param_map [] = {
@@ -1359,7 +1566,7 @@ static dis_map_t field_flags_map [] = {
 	{ FIELD_ATTRIBUTE_SPECIAL_NAME,        "specialname " },
 	{ FIELD_ATTRIBUTE_PINVOKE_IMPL,        "FIXME:pinvokeimpl " },
 	{ FIELD_ATTRIBUTE_RT_SPECIAL_NAME,        "rtspecialname " },
-	{ FIELD_ATTRIBUTE_HAS_FIELD_MARSHAL,        "hasfieldmarshal " },
+	/*{ FIELD_ATTRIBUTE_HAS_FIELD_MARSHAL,        "hasfieldmarshal " },*/
 	{ 0, NULL }
 };
 
@@ -1620,6 +1827,7 @@ get_method_core (MonoImage *m, guint32 token, gboolean fullsig, MonoGenericConte
 	char *name;
 
 	MonoMethod *mh;
+	MonoGenericContext *gc = context;
 
 	mh = mono_get_method_full (m, token, NULL, context);
 	if (mh) {
@@ -1659,7 +1867,7 @@ get_method_core (MonoImage *m, guint32 token, gboolean fullsig, MonoGenericConte
                 mono_metadata_decode_row (&m->tables [MONO_TABLE_METHODSPEC],
                                 idx - 1, member_cols, MONO_METHODSPEC_SIZE);
 		token = member_cols [MONO_METHODSPEC_METHOD];
-                sig = get_methodspec (m, idx, token, name, context);
+                sig = get_methodspec (m, idx, token, name, gc);
 		break;
 	}
 
@@ -1764,6 +1972,9 @@ get_methodspec (MonoImage *m, int idx, guint32 token, const char *fancy_name, Mo
         const char *ptr;
 	guint32 sig = 0;
 	int param_count, cconv, i, gen_count = 0;
+	MonoGenericContainer *container = NULL;
+	MonoGenericContext *parent_context = context;
+	MonoMethod *mh = NULL;
 
 	switch (token & MONO_METHODDEFORREF_MASK) {
 	case MONO_METHODDEFORREF_METHODDEF:
@@ -1784,6 +1995,11 @@ get_methodspec (MonoImage *m, int idx, guint32 token, const char *fancy_name, Mo
 
 	ptr = mono_metadata_blob_heap (m, sig);
 	mono_metadata_decode_value (ptr, &ptr);
+	
+	mh = mono_get_method_full (m, method_dor_to_token (token), NULL, parent_context);
+	g_assert (mh);
+	if ((container = mh->generic_container))
+		context = (MonoGenericContext*) container;
 
 	if (*ptr & 0x20){
 		if (*ptr & 0x40)
@@ -1814,7 +2030,7 @@ get_methodspec (MonoImage *m, int idx, guint32 token, const char *fancy_name, Mo
         mono_metadata_decode_row (&m->tables [MONO_TABLE_METHODSPEC],
                         idx - 1, member_cols, MONO_METHODSPEC_SIZE);
         token = member_cols [MONO_METHODSPEC_SIGNATURE];
-        type_param = get_method_type_param (m, token, context);
+        type_param = get_method_type_param (m, token, parent_context);
         g_string_append (res, type_param);
 	g_string_append (res, " (");
 
@@ -1841,6 +2057,94 @@ get_methodspec (MonoImage *m, int idx, guint32 token, const char *fancy_name, Mo
 	return s;
 }
 
+/*
+ * get_encoded_user_string_bytearray:
+ * @ptr: pointer into the US heap
+ * @len: length of string in the heap.
+ *
+ * Strings on the US heap are encoded using UTF-16. Print a bytearray.
+ */
+static char*
+get_encoded_user_string_bytearray (const unsigned char* ptr, int len)
+{
+	gchar *str;
+	int i, j, tspaces = (len % 16);
+	GString *res;
+
+	if (len == 0)
+		return g_strdup_printf ("\"\"");
+
+	res = g_string_new ("bytearray (\n\t");
+
+	for (i = 1; i <= len; ++i) {
+		g_string_append_printf (res, "%02x ", ptr [i-1]);
+
+		if (i % 16 == 0) {
+			if (i == len)
+				g_string_append (res, ")// ");
+			else
+				g_string_append (res, " // ");
+
+			for(j = i - 16; j < i; ++j) 
+				g_string_append_printf (res, "%c", isprint (ptr [j]) ? ptr [j] : '.');
+			g_string_append (res, "\n\t");
+		}
+	}
+
+	if (tspaces) {
+		g_string_append (res, ")  ");
+		for (i = tspaces + 1; i < 16; ++i)
+			g_string_append_printf (res, "   ");
+
+		g_string_append (res, " // ");
+		for(i = len - tspaces; i < len; ++i)
+			g_string_append_printf (res, "%c", isprint (ptr [i]) ? ptr [i] : '.');
+		g_string_append (res, "\n\t");
+	} 
+
+	str = res->str;
+	g_string_free (res, FALSE);
+	return str;
+}
+
+/*
+ * get_encoded_user_string_or_bytearray:
+ * @ptr: pointer into the US heap
+ *
+ * Strings on the US heap are encoded using UTF-16. Print as string
+ * if possible, else emit a bytearray.
+ */
+char*
+get_encoded_user_string_or_bytearray (const unsigned char *ptr, int len)
+{
+	unsigned char *res, *eres, *result;
+	int i;
+
+	res = g_malloc ((len >> 1) + 1);
+
+	/*
+	 * I should really use some kind of libunicode here
+	 */
+	for (i = 0; i + 1 < len; i += 2) {
+		if (ptr [i + 1] || 
+		    (!isprint (ptr [i]) && ptr [i] != '\\' && ptr [i] != '"' && 
+		     ptr [i] != '\r' && ptr [i] != '\n' && ptr [i] != '\t')) {
+			g_free (res);
+			return get_encoded_user_string_bytearray (ptr, len);
+		}
+
+		res [i >> 1] = ptr [i];
+	}	
+
+	res [len >> 1] = 0;
+
+	eres = g_strescape (res, NULL);
+	result = g_strdup_printf ("\"%s\"", eres);
+	g_free (res);
+	g_free (eres);
+	
+	return result;
+}
 
 /**
  * get_constant:
@@ -1876,70 +2180,53 @@ get_constant (MonoImage *m, MonoTypeEnum t, guint32 blob_index)
 		
 	case MONO_TYPE_U4:
 	case MONO_TYPE_I4:
-		return g_strdup_printf ("int32(%d)", read32 (ptr));
+		return g_strdup_printf ("int32(0x%08x)", read32 (ptr));
 		
+	case MONO_TYPE_U8:
 	case MONO_TYPE_I8: {
 		guint32 low, high;
 		low = read32 (ptr);
 		high = read32 (ptr + 4);
 		return g_strdup_printf ("int64(0x%08x%08x)", high, low);
 	}
-	case MONO_TYPE_U8: {
-		guint32 low, high;
-		low = read32 (ptr);
-		high = read32 (ptr + 4);
-		return g_strdup_printf ("0x%08x%08x", high, low);
-	}
 	case MONO_TYPE_R4: {
+		gboolean normal;
 		float r;
 		readr4 (ptr, &r);
-		return g_strdup_printf ("float32(%g)", (double) r);
-	}
+
+		/* Crazy solaris systems doesn't have isnormal */
+#ifdef HAVE_FINITE
+		normal = finite (r);
+#else
+		normal = isnormal (r);
+#endif
+		if (!normal)
+			return g_strdup_printf ("float32(0x%08x)", read32 (ptr));
+		else
+			return g_strdup_printf ("float32(%.20g)", r);
+	}	
 	case MONO_TYPE_R8: {
+		gboolean normal;
 		double r;
 		readr8 (ptr, &r);
-		return g_strdup_printf ("float64(%g)", r);
-	}
-	case MONO_TYPE_STRING: {
-		gchar *str;
-		int i, j, tspaces = (len%16);
-		GString *res;
 
-		if (len == 0)
-			return g_strdup_printf ("\"\"");
-
-		res = g_string_new ("bytearray (\n\t");
-
-		for(i = 1; i <= len; ++i) {
-			g_string_append_printf(res, "%02x ", ptr[i-1]);
-
-			if(i%16 == 0) {
-				if(i == len)
-					g_string_append(res, ")// ");
-				else
-					g_string_append(res, " // ");
-
-				for(j = i-16; j < i; ++j) 
-					g_string_append_printf(res, "%c", isprint(ptr[j]) ? ptr[j] : '.');
-				g_string_append(res, "\n\t");
-			}
+		/* Crazy solaris systems doesn't have isnormal */
+#ifdef HAVE_FINITE
+		normal = finite (r);
+#else
+		normal = isnormal (r);
+#endif
+		if (!normal) {
+			guint32 low, high;
+			low = read32 (ptr);
+			high = read32 (ptr + 4);
+			return g_strdup_printf ("float64(0x%08x%08x)", high, low);
+		} else {
+			return g_strdup_printf ("float64(%.20g)", r);
 		}
-
-		if(tspaces) {
-			g_string_append(res, ")  ");
-			for(i = tspaces+1; i < 16; ++i)
-				g_string_append_printf(res, "   ");
-
-			g_string_append(res, " // ");
-			for(i = len-tspaces; i < len; ++i)
-				g_string_append_printf(res, "%c", isprint(ptr[i]) ? ptr[i] : '.');
-			g_string_append(res, "\n\t");
-		} 
-
-		str = res->str;
-		g_string_free(res, FALSE);
-		return str;
 	}
+	case MONO_TYPE_STRING:
+		return get_encoded_user_string_or_bytearray (ptr, len);
 		
 	case MONO_TYPE_CLASS:
 		return g_strdup ("nullref");
@@ -1992,9 +2279,11 @@ get_token (MonoImage *m, guint32 token, MonoGenericContext *context)
 			result = g_strdup_printf ("field %s", temp);
 			g_free (temp);
 			return result;
-		}
-		else {
-			g_error ("Do not know how to decode tokens of type 0x%08x", token);
+		} else {
+			temp = get_method (m, token, context);
+			result = g_strdup_printf ("method %s", temp);
+			g_free (temp);
+			return result;
 		}
 		break;
 	}
@@ -2084,6 +2373,9 @@ dis_get_custom_attrs (MonoImage *m, guint32 token)
 	case MONO_TABLE_ASSEMBLY:
 		idx |= MONO_CUSTOM_ATTR_ASSEMBLY;
 		break;
+	case MONO_TABLE_ASSEMBLYREF:
+		idx |= MONO_CUSTOM_ATTR_ASSEMBLYREF;
+		break;
 	case MONO_TABLE_MODULE:
 		idx |= MONO_CUSTOM_ATTR_MODULE;
 		break;
@@ -2102,6 +2394,9 @@ dis_get_custom_attrs (MonoImage *m, guint32 token)
 	case MONO_TABLE_PARAM:
 		idx |= MONO_CUSTOM_ATTR_PARAMDEF;
 		break;
+	case MONO_TABLE_GENERICPARAM:
+		idx |= MONO_CUSTOM_ATTR_GENERICPAR;
+		break;
 	default:
 		g_print ("Missing custom attr get support for token 0x%08x\n", token);
 		return NULL;
@@ -2289,6 +2584,7 @@ init_key_table (void)
 	g_hash_table_insert (key_table, (char *) "bytearray", GINT_TO_POINTER (TRUE));
 	g_hash_table_insert (key_table, (char *) "byvalstr", GINT_TO_POINTER (TRUE));
 	g_hash_table_insert (key_table, (char *) "call", GINT_TO_POINTER (TRUE));
+	g_hash_table_insert (key_table, (char *) "callconv", GINT_TO_POINTER (TRUE));
 	g_hash_table_insert (key_table, (char *) "calli", GINT_TO_POINTER (TRUE));
 	g_hash_table_insert (key_table, (char *) "callmostderived", GINT_TO_POINTER (TRUE));
 	g_hash_table_insert (key_table, (char *) "callvirt", GINT_TO_POINTER (TRUE));
@@ -2375,6 +2671,7 @@ init_key_table (void)
 	g_hash_table_insert (key_table, (char *) "final", GINT_TO_POINTER (TRUE));
 	g_hash_table_insert (key_table, (char *) "finally", GINT_TO_POINTER (TRUE));
 	g_hash_table_insert (key_table, (char *) "fixed", GINT_TO_POINTER (TRUE));
+	g_hash_table_insert (key_table, (char *) "flags", GINT_TO_POINTER (TRUE));
 	g_hash_table_insert (key_table, (char *) "float32", GINT_TO_POINTER (TRUE));
 	g_hash_table_insert (key_table, (char *) "float64", GINT_TO_POINTER (TRUE));
 	g_hash_table_insert (key_table, (char *) "float", GINT_TO_POINTER (TRUE));
@@ -2433,6 +2730,7 @@ init_key_table (void)
 	g_hash_table_insert (key_table, (char *) "ldc.i8", GINT_TO_POINTER (TRUE));
 	g_hash_table_insert (key_table, (char *) "ldc.r4", GINT_TO_POINTER (TRUE));
 	g_hash_table_insert (key_table, (char *) "ldc.r8", GINT_TO_POINTER (TRUE));
+	g_hash_table_insert (key_table, (char *) "ldelem", GINT_TO_POINTER (TRUE));
 	g_hash_table_insert (key_table, (char *) "ldelema", GINT_TO_POINTER (TRUE));
 	g_hash_table_insert (key_table, (char *) "ldelem.i1", GINT_TO_POINTER (TRUE));
 	g_hash_table_insert (key_table, (char *) "ldelem.i2", GINT_TO_POINTER (TRUE));
@@ -2574,6 +2872,7 @@ init_key_table (void)
 	g_hash_table_insert (key_table, (char *) "starg.s", GINT_TO_POINTER (TRUE));
 	g_hash_table_insert (key_table, (char *) "static", GINT_TO_POINTER (TRUE));
 	g_hash_table_insert (key_table, (char *) "stdcall", GINT_TO_POINTER (TRUE));
+	g_hash_table_insert (key_table, (char *) "stelem", GINT_TO_POINTER (TRUE));
 	g_hash_table_insert (key_table, (char *) "stelem.i1", GINT_TO_POINTER (TRUE));
 	g_hash_table_insert (key_table, (char *) "stelem.i2", GINT_TO_POINTER (TRUE));
 	g_hash_table_insert (key_table, (char *) "stelem.i4", GINT_TO_POINTER (TRUE));
@@ -2590,6 +2889,8 @@ init_key_table (void)
 	g_hash_table_insert (key_table, (char *) "stind.i", GINT_TO_POINTER (TRUE));
 	g_hash_table_insert (key_table, (char *) "stind.r4", GINT_TO_POINTER (TRUE));
 	g_hash_table_insert (key_table, (char *) "stind.r8", GINT_TO_POINTER (TRUE));
+	g_hash_table_insert (key_table, (char *) "stloc", GINT_TO_POINTER (TRUE));
+	g_hash_table_insert (key_table, (char *) "stobj", GINT_TO_POINTER (TRUE));
 	g_hash_table_insert (key_table, (char *) "storage", GINT_TO_POINTER (TRUE));
 	g_hash_table_insert (key_table, (char *) "stored_object", GINT_TO_POINTER (TRUE));
 	g_hash_table_insert (key_table, (char *) "streamed_object", GINT_TO_POINTER (TRUE));
@@ -2597,6 +2898,10 @@ init_key_table (void)
 	g_hash_table_insert (key_table, (char *) "strict", GINT_TO_POINTER (TRUE));
 	g_hash_table_insert (key_table, (char *) "string", GINT_TO_POINTER (TRUE));
 	g_hash_table_insert (key_table, (char *) "struct", GINT_TO_POINTER (TRUE));
+	g_hash_table_insert (key_table, (char *) "stsfld", GINT_TO_POINTER (TRUE));
+	g_hash_table_insert (key_table, (char *) "sub", GINT_TO_POINTER (TRUE));
+	g_hash_table_insert (key_table, (char *) "sub.ovf", GINT_TO_POINTER (TRUE));
+	g_hash_table_insert (key_table, (char *) "sub.ovf.un", GINT_TO_POINTER (TRUE));
 	g_hash_table_insert (key_table, (char *) "synchronized", GINT_TO_POINTER (TRUE));
 	g_hash_table_insert (key_table, (char *) "syschar", GINT_TO_POINTER (TRUE));
 	g_hash_table_insert (key_table, (char *) "sysstring", GINT_TO_POINTER (TRUE));
@@ -2607,6 +2912,8 @@ init_key_table (void)
 	g_hash_table_insert (key_table, (char *) "true", GINT_TO_POINTER (TRUE));
 	g_hash_table_insert (key_table, (char *) "type", GINT_TO_POINTER (TRUE));
 	g_hash_table_insert (key_table, (char *) "typedref", GINT_TO_POINTER (TRUE));
+	g_hash_table_insert (key_table, (char *) "uint", GINT_TO_POINTER (TRUE));
+	g_hash_table_insert (key_table, (char *) "unbox", GINT_TO_POINTER (TRUE));
 	g_hash_table_insert (key_table, (char *) "unicode", GINT_TO_POINTER (TRUE));
 	g_hash_table_insert (key_table, (char *) "unmanagedexp", GINT_TO_POINTER (TRUE));
 	g_hash_table_insert (key_table, (char *) "unmanaged", GINT_TO_POINTER (TRUE));
@@ -2622,6 +2929,7 @@ init_key_table (void)
 	g_hash_table_insert (key_table, (char *) "wchar", GINT_TO_POINTER (TRUE));
 	g_hash_table_insert (key_table, (char *) "winapi", GINT_TO_POINTER (TRUE));
 	g_hash_table_insert (key_table, (char *) "with", GINT_TO_POINTER (TRUE));
+	g_hash_table_insert (key_table, (char *) "xor", GINT_TO_POINTER (TRUE));
 }
 
 guint32
