@@ -21,6 +21,7 @@ char *get_methodref_signature (MonoImage *m, guint32 blob_signature, const char
 char *get_methodspec          (MonoImage *m, int idx, guint32 token, const char *fancy,
 			       MonoGenericContext *context);
 char *get_constant            (MonoImage *m, MonoTypeEnum t, guint32 blob_index);
+char *get_encoded_user_string_or_bytearray (const unsigned char *ptr, int len);
 char *get_token               (MonoImage *m, guint32 token, MonoGenericContext *context);
 char *get_token_type          (MonoImage *m, guint32 token, MonoGenericContext *context);
 char *get_typespec            (MonoImage *m, guint32 blob_idx, gboolean is_def, MonoGenericContext *context);
@@ -43,6 +44,7 @@ char *dis_stringify_param     (MonoImage *m, MonoType *param);
 char *dis_stringify_method_signature (MonoImage *m, MonoMethodSignature *method, int methoddef_row,
 				      MonoGenericContext *context, gboolean fully_qualified);
 char *dis_stringify_function_ptr (MonoImage *m, MonoMethodSignature *method);
+char *dis_stringify_marshal_spec (MonoMarshalSpec *spec);
 
 guint32 method_dor_to_token (guint32 idx);
 
