@@ -211,6 +211,7 @@ dis_directive_assemblyref (MonoImage *m)
 			 cols [MONO_ASSEMBLYREF_MAJOR_VERSION], cols [MONO_ASSEMBLYREF_MINOR_VERSION], 
 			 cols [MONO_ASSEMBLYREF_BUILD_NUMBER], cols [MONO_ASSEMBLYREF_REV_NUMBER]
 			);
+		dump_cattrs (m, MONO_TOKEN_ASSEMBLY_REF | (i + 1), "  ");
 		if (cols [MONO_ASSEMBLYREF_CULTURE]){
 			fprintf (output, "  .locale %s\n", mono_metadata_string_heap (m, cols [MONO_ASSEMBLYREF_CULTURE]));
 		}
@@ -257,16 +258,22 @@ dis_directive_moduleref (MonoImage *m)
 
 	for (i = 0; i < t->rows; i++){
 		guint32 cols [MONO_MODULEREF_SIZE];
-		const char *name;
 		
 		mono_metadata_decode_row (t, i, cols, MONO_MODULEREF_SIZE);
 
-		name = mono_metadata_string_heap (m, cols [MONO_MODULEREF_NAME]);
-		fprintf (output, ".module extern %s\n", name);
+		fprintf (output, ".module extern '%s'\n", mono_metadata_string_heap (m, cols [MONO_MODULEREF_NAME]));
 	}
 	
 }
 
+static void
+dis_nt_header (MonoImage *m)
+{
+	MonoCLIImageInfo *image_info = m->image_info;
+	if (image_info && image_info->cli_header.nt.pe_stack_reserve != 0x100000)
+		fprintf (output, ".stackreserve 0x%x\n", image_info->cli_header.nt.pe_stack_reserve);
+}
+
 static void
 dis_directive_file (MonoImage *m)
 {
@@ -416,6 +423,7 @@ dis_field_list (MonoImage *m, guint32 start, guint32 end, MonoGenericContext *co
 			
 	for (i = start; i < end; i++){
 		char *sig, *flags, *attrs = NULL;
+		char *marshal_str = NULL;
 		guint32 field_offset = -1;
 
 		if (!should_include_field (i + 1))
@@ -423,6 +431,15 @@ dis_field_list (MonoImage *m, guint32 start, guint32 end, MonoGenericContext *co
 		mono_metadata_decode_row (t, i, cols, MONO_FIELD_SIZE);
 		sig = get_field_signature (m, cols [MONO_FIELD_SIGNATURE], context);
 		flags = field_flags (cols [MONO_FIELD_FLAGS]);
+		
+		if (cols [MONO_FIELD_FLAGS] & FIELD_ATTRIBUTE_HAS_FIELD_MARSHAL) {
+			const char *tp;
+			MonoMarshalSpec *spec;
+			
+			tp = mono_metadata_get_marshal_info (m, i, TRUE);
+			spec = mono_metadata_parse_marshal_spec (m, tp);
+			marshal_str = dis_stringify_marshal_spec (spec);
+		}
 
 		if (cols [MONO_FIELD_FLAGS] & FIELD_ATTRIBUTE_HAS_FIELD_RVA) {
 			mono_metadata_field_info (m, i, NULL, &rva, NULL);
@@ -447,16 +464,16 @@ dis_field_list (MonoImage *m, guint32 start, guint32 end, MonoGenericContext *co
 				lit = g_strdup ("not found");
 			}
 			
-			
-			fprintf (output, "    .field %s %s %s = ",
-				 flags, sig, esname);
+			fprintf (output, "    .field %s%s%s %s = ",
+				 flags, marshal_str ? marshal_str : " ", sig, esname);
 			fprintf (output, "%s\n", lit);
 			g_free (lit);
 		} else
-			fprintf (output, "    .field %s %s %s %s%s\n",
-				 attrs? attrs: "", flags, sig, esname, rva_desc);
+			fprintf (output, "    .field %s %s%s%s %s%s\n",
+				 attrs? attrs: "", flags, marshal_str ? marshal_str : " ", sig, esname, rva_desc);
 		g_free (attrs);
 		g_free (flags);
+		g_free (marshal_str);
 		g_free (sig);
 		g_free (esname);
 		dump_cattrs (m, MONO_TOKEN_FIELD_DEF | (i + 1), "    ");
@@ -528,7 +545,8 @@ static dis_map_t pinvoke_call_conv_map [] = {
 	{ PINVOKE_ATTRIBUTE_CALL_CONV_STDCALL,     "stdcall " },
 	{ PINVOKE_ATTRIBUTE_CALL_CONV_THISCALL,    "thiscall " },
 	{ PINVOKE_ATTRIBUTE_CALL_CONV_FASTCALL,    "fastcall " },
-	{ 0, NULL }
+	{ 0, "" },
+	{ -1, NULL }
 };
 
 static dis_map_t pinvoke_char_set_map [] = {
@@ -551,7 +569,7 @@ pinvoke_flags (guint32 f)
 	int cset = f & PINVOKE_ATTRIBUTE_CHAR_SET_MASK;
 	int cconv = f & PINVOKE_ATTRIBUTE_CALL_CONV_MASK;
 	char *s;
-	
+
 	g_string_append (str, map (cset, pinvoke_char_set_map));
 	g_string_append (str, map (cconv, pinvoke_call_conv_map));
 	g_string_append (str, flags (f, pinvoke_flags_map));
@@ -581,7 +599,7 @@ static dis_map_t managed_impl_flags [] = {
 	{ METHOD_IMPL_ATTRIBUTE_PRESERVE_SIG,    "preservesig " },
 	{ METHOD_IMPL_ATTRIBUTE_INTERNAL_CALL,   "internalcall " },
 	{ METHOD_IMPL_ATTRIBUTE_SYNCHRONIZED,    "synchronized " },
-	{ METHOD_IMPL_ATTRIBUTE_NOINLINING,      "noinline " },
+	{ METHOD_IMPL_ATTRIBUTE_NOINLINING,      "noinlining " },
 	{ 0, NULL }
 };
 
@@ -721,6 +739,39 @@ pinvoke_info (MonoImage *m, guint32 mindex)
 	return NULL;
 }
 
+/*
+ * dump_cattrs_for_type_params
+ *
+ * @m: 
+ * @token: TypeOrMethodDef token, owner for GenericParam
+ *
+ * Dumps the custom attributes for @token's type parameters
+ */
+static void
+dump_cattrs_for_type_params (MonoImage *m, guint32 token, const char *indent)
+{
+	MonoTableInfo *tdef  = &m->tables [MONO_TABLE_GENERICPARAM];
+	guint32 cols [MONO_GENERICPARAM_SIZE];
+	guint32 owner = 0, i;
+	GList *list = NULL;
+
+	if (! (i = mono_metadata_get_generic_param_row (m, token, &owner)))
+		return;
+
+	mono_metadata_decode_row (tdef, i - 1, cols, MONO_GENERICPARAM_SIZE);
+	do {
+		list = dis_get_custom_attrs (m, mono_metadata_make_token (MONO_TABLE_GENERICPARAM, i));
+		if (list) {
+			fprintf (output, "%s.param type %s\n", indent, mono_metadata_string_heap (m, cols [MONO_GENERICPARAM_NAME]));
+			dump_cattrs_list (list, indent);
+		}
+
+		if (++i > tdef->rows)
+			break;
+		mono_metadata_decode_row (tdef, i - 1, cols, MONO_GENERICPARAM_SIZE);
+	} while (cols [MONO_GENERICPARAM_OWNER] == owner);
+}
+
 static void
 dump_cattrs_for_method_params (MonoImage *m, guint32 midx, MonoMethodSignature *sig) {
 	MonoTableInfo *methodt;
@@ -806,10 +857,13 @@ dis_method_list (const char *klass_name, MonoImage *m, guint32 start, guint32 en
 
 		container = mono_metadata_load_generic_params (
 			m, MONO_TOKEN_METHOD_DEF | (i + 1), context ? context->container : NULL);
-		if (container)
+		if (container) {
+			mono_metadata_load_generic_param_constraints (
+					m, MONO_TOKEN_METHOD_DEF | (i + 1), container);
 			method_context = (MonoGenericContext *) container;
+		}
 
-		ms = mono_metadata_parse_method_signature_full (m, method_context, i + 1, sig, &sig);
+		ms = mono_metadata_parse_method_signature_full (m, method_context ? method_context->container : NULL, i + 1, sig, &sig);
 		sig_str = dis_stringify_method_signature (m, ms, i + 1, method_context, FALSE);
 
 		fprintf (output, "    // method line %d\n", i + 1);
@@ -832,8 +886,9 @@ dis_method_list (const char *klass_name, MonoImage *m, guint32 start, guint32 en
 		
 		fprintf (output, "    {\n");
 		dump_cattrs (m, token, "        ");
+		dump_cattrs_for_type_params (m, MONO_TOKEN_METHOD_DEF | (i + 1), "        ");
 		dump_cattrs_for_method_params (m, i, ms);
-		/* FIXME: need to sump also param custom attributes */
+
 		fprintf (output, "        // Method begins at RVA 0x%x\n", cols [MONO_METHOD_RVA]);
 		dump_declarative_security (m, OBJECT_TYPE_METHODDEF, i + 1, "        ");
 		if (cols [MONO_METHOD_IMPLFLAGS] & METHOD_IMPL_ATTRIBUTE_NATIVE)
@@ -1118,6 +1173,8 @@ dis_type (MonoImage *m, int n, int is_nested, int forward)
 		fprintf (output, ".namespace %s\n{\n", nspace);
 
 	container = mono_metadata_load_generic_params (m, MONO_TOKEN_TYPE_DEF | (n + 1), NULL);
+	if (container)
+		mono_metadata_load_generic_param_constraints (m, MONO_TOKEN_TYPE_DEF | (n + 1), container);
 
 	esname = get_escaped_name (name);
 	if ((cols [MONO_TYPEDEF_FLAGS] & TYPE_ATTRIBUTE_CLASS_SEMANTIC_MASK) == TYPE_ATTRIBUTE_CLASS){
@@ -1150,6 +1207,7 @@ dis_type (MonoImage *m, int n, int is_nested, int forward)
 	dis_interfaces (m, n + 1, (MonoGenericContext *) container);
 	fprintf (output, "  {\n");
         if (!forward) {
+        	dump_cattrs_for_type_params (m, MONO_TOKEN_TYPE_DEF | (n + 1), "    ");
         	dump_cattrs (m, MONO_TOKEN_TYPE_DEF | (n + 1), "    ");
 	        dump_declarative_security (m, OBJECT_TYPE_TYPEDEF, (n + 1), "    ");
 
@@ -1362,6 +1420,7 @@ struct {
 	{ "--methodspec",  MONO_TABLE_METHODSPEC,       dump_table_methodspec },
 	{ "--moduleref",   MONO_TABLE_MODULEREF,   	dump_table_moduleref },
 	{ "--module",      MONO_TABLE_MODULE,      	dump_table_module },
+	{ "--mresources",  0,	dis_mresource },
 	{ "--nested",      MONO_TABLE_NESTEDCLASS, 	dump_table_nestedclass },
 	{ "--param",       MONO_TABLE_PARAM,       	dump_table_param },
 	{ "--parconst",    MONO_TABLE_GENERICPARAMCONSTRAINT, dump_table_parconstraint },
@@ -1410,6 +1469,7 @@ disassemble_file (const char *file)
 		dis_directive_mresource (img);
 		dis_directive_module (img);
 		dis_directive_moduleref (img);
+		dis_nt_header (img);
                 if (dump_managed_resources)
         		dis_mresource (img);
 		if (dump_forward_decls) {
@@ -1698,7 +1758,7 @@ usage (void)
 		if (((i-2) % 5) == 0)
 			g_string_append_c (args, '\n');
 	}
-	g_string_append (args, "[--forward-decls]\n[--mresources]");
+	g_string_append (args, "[--forward-decls]");
 	fprintf (stderr,
 		 "monodis -- Mono Common Intermediate Language Dissassembler\n" 
 		 "Usage is: monodis %s file ..\n", args->str);
@@ -1735,6 +1795,7 @@ main (int argc, char *argv [])
 						 argv [i]+9, strerror (errno));
 					exit (1);
 				}
+				dump_managed_resources = TRUE;
 				continue;
 			} else if (strncmp (argv [i], "--filter=", 9) == 0) {
 				load_filter (argv [i]+9);
@@ -1742,9 +1803,6 @@ main (int argc, char *argv [])
 			} else if (strcmp (argv [i], "--forward-decls") == 0) {
 				dump_forward_decls = TRUE;
 				continue;
-			} else if (strcmp (argv [i], "--mresources") == 0) {
-				dump_managed_resources = TRUE;
-				continue;
 			} else if (strcmp (argv [i], "--help") == 0)
 				usage ();
 			for (j = 0; table_list [j].name != NULL; j++) {
