@@ -11,10 +11,9 @@
 
 static const guchar *unused_details (struct _WapiHandleShared *handle);
 static const guchar *unshared_details (struct _WapiHandleShared *handle);
-#if 0
 static const guchar *thread_details (struct _WapiHandleShared *handle);
-#endif
 static const guchar *namedmutex_details (struct _WapiHandleShared *handle);
+static const guchar *namedsem_details (struct _WapiHandleShared *handle);
 static const guchar *process_details (struct _WapiHandleShared *handle);
 
 /* This depends on the ordering of the enum WapiHandleType in
@@ -25,11 +24,7 @@ static const guchar * (*details[])(struct _WapiHandleShared *)=
 	unused_details,
 	unshared_details,		/* file */
 	unshared_details,		/* console */
-#if 0
 	thread_details,
-#else
-	unshared_details,		/* thread */
-#endif
 	unshared_details,		/* sem */
 	unshared_details,		/* mutex */
 	unshared_details,		/* event */
@@ -38,21 +33,23 @@ static const guchar * (*details[])(struct _WapiHandleShared *)=
 	process_details,
 	unshared_details,		/* pipe */
 	namedmutex_details,
+	namedsem_details,
 	unused_details,
 };
 
 int main (int argc, char **argv)
 {
 	guint32 i;
-	
+	guint32 now;
+
 	_wapi_shared_layout = _wapi_shm_attach(WAPI_SHM_DATA);
-	if (_wapi_shared_layout == FALSE) {
+	if (_wapi_shared_layout == NULL) {
 		g_error ("Failed to attach shared memory!");
 		exit (-1);
 	}
 
 	_wapi_fileshare_layout = _wapi_shm_attach(WAPI_SHM_FILESHARE);
-	if (_wapi_fileshare_layout == FALSE) {
+	if (_wapi_fileshare_layout == NULL) {
 		g_error ("Failed to attach fileshare shared memory!");
 		exit (-1);
 	}
@@ -63,32 +60,21 @@ int main (int argc, char **argv)
 		_wapi_handle_collect ();
 	}
 	
-	g_print ("collection: %d signals: %d sem: 0x%x\n",
+	g_print ("collection: %d sem: 0x%x\n",
 		 _wapi_shared_layout->collection_count,
-		 _wapi_shared_layout->signal_count,
 		 _wapi_shared_layout->sem_key);
 	
+	now = (guint32)(time(NULL) & 0xFFFFFFFF);
 	for (i = 0; i < _WAPI_HANDLE_INITIAL_COUNT; i++) {
 		struct _WapiHandleShared *shared;
-		struct _WapiHandleSharedMetadata *meta;
-		guint32 now = (guint32)(time(NULL) & 0xFFFFFFFF);
-		
-		meta = &_wapi_shared_layout->metadata[i];
 		
 		shared = &_wapi_shared_layout->handles[i];
-		if (shared->stale == TRUE) {
-			g_print ("    (%3x) [%7s]  *STALE*  (%s)\n", i,
-				 _wapi_handle_typename[shared->type],
-				 details[shared->type](shared));
-		}
-		
-		shared = &_wapi_shared_layout->handles[meta->offset];
 		if (shared->type != WAPI_HANDLE_UNUSED) {
-			g_print ("%3x (%3x) [%7s] %4u %s (%s)\n",
-				 i, meta->offset,
+			g_print ("%3x (%3d) [%7s] %4u %s (%s)\n",
+				 i, shared->handle_refs,
 				 _wapi_handle_typename[shared->type],
-				 now - meta->timestamp,
-				 meta->signalled?"Sg":"Un",
+				 now - shared->timestamp,
+				 shared->signalled?"Sg":"Un",
 				 details[shared->type](shared));
 		}
 	}
@@ -117,7 +103,6 @@ static const guchar *unshared_details (struct _WapiHandleShared *handle)
 	return("unshared details");
 }
 
-#if 0
 static const guchar *thread_details (struct _WapiHandleShared *handle)
 {
 	static guchar buf[80];
@@ -130,7 +115,6 @@ static const guchar *thread_details (struct _WapiHandleShared *handle)
 	
 	return(buf);
 }
-#endif
 
 static const guchar *namedmutex_details (struct _WapiHandleShared *handle)
 {
@@ -147,6 +131,20 @@ static const guchar *namedmutex_details (struct _WapiHandleShared *handle)
 	return(buf);
 }
 
+static const guchar *namedsem_details (struct _WapiHandleShared *handle)
+{
+	static guchar buf[80];
+	gchar *name;
+	struct _WapiHandle_namedsem *sem = &handle->u.namedsem;
+	
+	name = sem->sharedns.name;
+	
+	g_snprintf (buf, sizeof(buf), "[%15s] val: %5u, max: %5d",
+		    name == NULL?(gchar *)"":name, sem->val, sem->max);
+
+	return(buf);
+}
+
 static const guchar *process_details (struct _WapiHandleShared *handle)
 {
 	static guchar buf[80];
@@ -155,7 +153,7 @@ static const guchar *process_details (struct _WapiHandleShared *handle)
 	
 	name = proc->proc_name;
 	
-	g_snprintf (buf, sizeof(buf), "[%15s] pid: %5u exit: %u",
+	g_snprintf (buf, sizeof(buf), "[%25.25s] pid: %5u exit: %u",
 		    name==NULL?(gchar *)"":name, proc->id, proc->exitstatus);
 	
 	return(buf);
