@@ -12,10 +12,13 @@
 
 #include <glib.h>
 
+G_BEGIN_DECLS
+
 extern gpointer CreateSemaphore(WapiSecurityAttributes *security,
 				gint32 initial, gint32 max,
 				const gunichar2 *name);
 extern gboolean ReleaseSemaphore(gpointer handle, gint32 count,
 				 gint32 *prevcount);
 
+G_END_DECLS
 #endif /* _WAPI_SEMAPHORES_H_ */
