@@ -44,10 +44,11 @@ static mono_once_t thread_hash_once = MONO_ONCE_INIT;
 static mono_mutex_t thread_hash_mutex = MONO_MUTEX_INITIALIZER;
 static GHashTable *thread_hash=NULL;
 
+static void thread_close (gpointer handle, gpointer data);
 static gboolean thread_own (gpointer handle);
 
 struct _WapiHandleOps _wapi_thread_ops = {
-	NULL,				/* close_shared */
+	thread_close,			/* close */
 	NULL,				/* signal */
 	thread_own,			/* own */
 	NULL,				/* is_owned */
@@ -69,6 +70,20 @@ static void thread_ops_init (void)
 #endif
 }
 
+static void thread_close (gpointer handle, gpointer data)
+{
+	struct _WapiHandle_thread *thread_handle = (struct _WapiHandle_thread *)data;
+
+#ifdef DEBUG
+	g_message ("%s: closing thread handle %p", __func__, handle);
+#endif
+
+	if (thread_handle->owner_pid == getpid ()) {
+		g_ptr_array_free (thread_handle->owned_mutexes, TRUE);
+	}
+}
+
+/* NB, always called with the shared handle lock held */
 static gboolean thread_own (gpointer handle)
 {
 	struct _WapiHandle_thread *thread_handle;
@@ -86,6 +101,17 @@ static gboolean thread_own (gpointer handle)
 		return(FALSE);
 	}
 	
+	if (thread_handle->owner_pid != getpid ()) {
+#ifdef DEBUG
+		g_message ("%s: can't join thread, %d not owner process %d",
+			   __func__, getpid (), thread_handle->owner_pid);
+#endif
+		/* FIXME: might need to return TRUE here so that other
+		 * processes can WaitFor thread handles
+		 */
+		return (FALSE);
+	}
+	
 	if (thread_handle->joined == FALSE) {
 		_wapi_timed_thread_join (thread_handle->thread, NULL, NULL);
 		thread_handle->joined = TRUE;
@@ -94,16 +120,12 @@ static gboolean thread_own (gpointer handle)
 	return(TRUE);
 }
 
-static void thread_exit(guint32 exitstatus, gpointer handle)
+/* Called by thread_exit(), but maybe by mono_thread_manage() too */
+void _wapi_thread_abandon_mutexes (gpointer handle)
 {
 	struct _WapiHandle_thread *thread_handle;
 	gboolean ok;
-	int thr_ret;
-	
-	pthread_cleanup_push ((void(*)(void *))_wapi_handle_unlock_handle,
-			      handle);
-	thr_ret = _wapi_handle_lock_handle (handle);
-	g_assert (thr_ret == 0);
+	int i;
 	
 	ok = _wapi_lookup_handle (handle, WAPI_HANDLE_THREAD,
 				  (gpointer *)&thread_handle);
@@ -113,7 +135,34 @@ static void thread_exit(guint32 exitstatus, gpointer handle)
 		return;
 	}
 	
-	_wapi_mutex_check_abandoned (getpid (), thread_handle->thread->id);
+	for (i = 0; i < thread_handle->owned_mutexes->len; i++) {
+		gpointer mutex = g_ptr_array_index (thread_handle->owned_mutexes, i);
+		
+		_wapi_mutex_abandon (mutex, getpid (),
+				     thread_handle->thread->id);
+		_wapi_thread_disown_mutex (thread_handle->thread->id, mutex);
+	}
+}
+
+/* Called by the timed_thread code as a thread is finishing up */
+static void thread_exit(guint32 exitstatus, gpointer handle)
+{
+	struct _WapiHandle_thread *thread_handle;
+	gboolean ok;
+	int thr_ret;
+	
+	_wapi_thread_abandon_mutexes (handle);
+	
+	thr_ret = _wapi_handle_lock_shared_handles ();
+	g_assert (thr_ret == 0);
+	
+	ok = _wapi_lookup_handle (handle, WAPI_HANDLE_THREAD,
+				  (gpointer *)&thread_handle);
+	if (ok == FALSE) {
+		g_warning ("%s: error looking up thread handle %p", __func__,
+			   handle);
+		return;
+	}
 
 #ifdef DEBUG
 	g_message ("%s: Recording thread handle %p exit status", __func__,
@@ -123,11 +172,9 @@ static void thread_exit(guint32 exitstatus, gpointer handle)
 	thread_handle->exitstatus = exitstatus;
 	thread_handle->state = THREAD_STATE_EXITED;
 
-	_wapi_handle_set_signal_state (handle, TRUE, TRUE);
+	_wapi_shared_handle_set_signal_state (handle, TRUE);
 
-	thr_ret = _wapi_handle_unlock_handle (handle);
-	g_assert (thr_ret == 0);
-	pthread_cleanup_pop (0);
+	_wapi_handle_unlock_shared_handles ();
 	
 #ifdef DEBUG
 	g_message("%s: Recording thread handle %p id %ld status as %d",
@@ -140,7 +187,7 @@ static void thread_exit(guint32 exitstatus, gpointer handle)
 	thr_ret = mono_mutex_lock(&thread_hash_mutex);
 	g_assert (thr_ret == 0);
 	
-	g_hash_table_remove (thread_hash, GUINT_TO_POINTER (thread_handle->thread->id));
+	g_hash_table_remove (thread_hash, (gpointer)(thread_handle->thread->id));
 
 	thr_ret = mono_mutex_unlock(&thread_hash_mutex);
 	g_assert (thr_ret == 0);
@@ -165,7 +212,9 @@ static void thread_hash_init(void)
  * @create: If 0, the new thread is ready to run immediately.  If
  * %CREATE_SUSPENDED, the new thread will be in the suspended state,
  * requiring a ResumeThread() call to continue running.
- * @tid: If non-NULL, the ID of the new thread is stored here.
+ * @tid: If non-NULL, the ID of the new thread is stored here.  NB
+ * this is defined as a DWORD (ie 32bit) in the MS API, but we need to
+ * cope with 64 bit IDs for s390x and amd64.
  *
  * Creates a new threading handle.
  *
@@ -173,7 +222,7 @@ static void thread_hash_init(void)
  */
 gpointer CreateThread(WapiSecurityAttributes *security G_GNUC_UNUSED, guint32 stacksize,
 		      WapiThreadStart start, gpointer param, guint32 create,
-		      guint32 *tid) 
+		      gsize *tid) 
 {
 	struct _WapiHandle_thread thread_handle = {0}, *thread_handle_p;
 	pthread_attr_t attr;
@@ -192,7 +241,8 @@ gpointer CreateThread(WapiSecurityAttributes *security G_GNUC_UNUSED, guint32 st
 	}
 
 	thread_handle.state = THREAD_STATE_START;
-	thread_handle.owner_pid = getpid();
+	thread_handle.owner_pid = getpid ();
+	thread_handle.owned_mutexes = g_ptr_array_new ();
 	
 	handle = _wapi_handle_new (WAPI_HANDLE_THREAD, &thread_handle);
 	if (handle == _WAPI_HANDLE_INVALID) {
@@ -202,9 +252,7 @@ gpointer CreateThread(WapiSecurityAttributes *security G_GNUC_UNUSED, guint32 st
 		return (NULL);
 	}
 
-	pthread_cleanup_push ((void(*)(void *))_wapi_handle_unlock_handle,
-			      handle);
-	thr_ret = _wapi_handle_lock_handle (handle);
+	thr_ret = _wapi_handle_lock_shared_handles ();
 	g_assert (thr_ret == 0);
 	
 	ok = _wapi_lookup_handle (handle, WAPI_HANDLE_THREAD,
@@ -277,7 +325,7 @@ gpointer CreateThread(WapiSecurityAttributes *security G_GNUC_UNUSED, guint32 st
 	ct_ret = handle;
 	
 	g_hash_table_insert (thread_hash,
-			     GUINT_TO_POINTER (thread_handle_p->thread->id),
+			     (gpointer)(thread_handle_p->thread->id),
 			     handle);
 	
 #ifdef DEBUG
@@ -288,7 +336,11 @@ gpointer CreateThread(WapiSecurityAttributes *security G_GNUC_UNUSED, guint32 st
 	
 	if (tid != NULL) {
 #ifdef PTHREAD_POINTER_ID
-		*tid = GPOINTER_TO_UINT (thread_handle_p->thread->id);
+		/* Don't use GPOINTER_TO_UINT here, it can't cope with
+		 * sizeof(void *) > sizeof(uint) when a cast to uint
+		 * would overflow
+		 */
+		*tid = (gsize)(thread_handle_p->thread->id);
 #else
 		*tid = thread_handle_p->thread->id;
 #endif
@@ -300,12 +352,10 @@ thread_hash_cleanup:
 	pthread_cleanup_pop (0);
 
 cleanup:
-	thr_ret = _wapi_handle_unlock_handle (handle);
-	g_assert (thr_ret == 0);
-	pthread_cleanup_pop (0);
+	_wapi_handle_unlock_shared_handles ();
 	
-	/* Must not call _wapi_handle_unref() with the handle already
-	 * locked
+	/* Must not call _wapi_handle_unref() with the shared handles
+	 * already locked
 	 */
 	for (i = 0; i < unrefs; i++) {
 		_wapi_handle_unref (handle);
@@ -314,29 +364,40 @@ cleanup:
 	return(ct_ret);
 }
 
-gpointer OpenThread (guint32 access G_GNUC_UNUSED, gboolean inherit G_GNUC_UNUSED, guint32 tid)
+gpointer _wapi_thread_handle_from_id (pthread_t tid)
 {
 	gpointer ret=NULL;
 	int thr_ret;
-	
-	mono_once (&thread_hash_once, thread_hash_init);
-	mono_once (&thread_ops_once, thread_ops_init);
-	
-#ifdef DEBUG
-	g_message ("%s: looking up thread %d", __func__, tid);
-#endif
 
 	pthread_cleanup_push ((void(*)(void *))mono_mutex_unlock_in_cleanup,
 			      (void *)&thread_hash_mutex);
 	thr_ret = mono_mutex_lock(&thread_hash_mutex);
 	g_assert (thr_ret == 0);
 	
-	ret = g_hash_table_lookup (thread_hash, GUINT_TO_POINTER (tid));
+	ret = g_hash_table_lookup (thread_hash, (gpointer)(tid));
 
 	thr_ret = mono_mutex_unlock(&thread_hash_mutex);
 	g_assert (thr_ret == 0);
 	pthread_cleanup_pop (0);
+
+	return(ret);
+}
+
+/* NB tid is 32bit in MS API, but we need 64bit on amd64 and s390x
+ * (and probably others)
+ */
+gpointer OpenThread (guint32 access G_GNUC_UNUSED, gboolean inherit G_GNUC_UNUSED, gsize tid)
+{
+	gpointer ret=NULL;
 	
+	mono_once (&thread_hash_once, thread_hash_init);
+	mono_once (&thread_ops_once, thread_ops_init);
+	
+#ifdef DEBUG
+	g_message ("%s: looking up thread %"G_GSIZE_FORMAT, __func__, tid);
+#endif
+
+	ret = _wapi_thread_handle_from_id ((pthread_t)tid);
 	if(ret!=NULL) {
 		_wapi_handle_ref (ret);
 	}
@@ -418,20 +479,27 @@ gboolean GetExitCodeThread(gpointer handle, guint32 *exitcode)
  * Looks up the thread ID of the current thread.  This ID can be
  * passed to OpenThread() to create a new handle on this thread.
  *
- * Return value: the thread ID.
+ * Return value: the thread ID.  NB this is defined as DWORD (ie 32
+ * bit) in the MS API, but we need to cope with 64 bit IDs for s390x
+ * and amd64.  This doesn't really break the API, it just embraces and
+ * extends it on 64bit platforms :)
  */
-guint32 GetCurrentThreadId(void)
+gsize GetCurrentThreadId(void)
 {
 	pthread_t tid = pthread_self();
 	
 #ifdef PTHREAD_POINTER_ID
-	return(GPOINTER_TO_UINT(tid));
+	/* Don't use GPOINTER_TO_UINT here, it can't cope with
+	 * sizeof(void *) > sizeof(uint) when a cast to uint would
+	 * overflow
+	 */
+	return((gsize)tid);
 #else
 	return(tid);
 #endif
 }
 
-static gpointer thread_attach(guint32 *tid)
+static gpointer thread_attach(gsize *tid)
 {
 	struct _WapiHandle_thread thread_handle = {0}, *thread_handle_p;
 	gpointer handle;
@@ -445,7 +513,8 @@ static gpointer thread_attach(guint32 *tid)
 	mono_once (&thread_ops_once, thread_ops_init);
 
 	thread_handle.state = THREAD_STATE_START;
-	thread_handle.owner_pid = getpid();
+	thread_handle.owner_pid = getpid ();
+	thread_handle.owned_mutexes = g_ptr_array_new ();
 
 	handle = _wapi_handle_new (WAPI_HANDLE_THREAD, &thread_handle);
 	if (handle == _WAPI_HANDLE_INVALID) {
@@ -455,9 +524,7 @@ static gpointer thread_attach(guint32 *tid)
 		return (NULL);
 	}
 
-	pthread_cleanup_push ((void(*)(void *))_wapi_handle_unlock_handle,
-			      handle);
-	thr_ret = _wapi_handle_lock_handle (handle);
+	thr_ret = _wapi_handle_lock_shared_handles ();
 	g_assert (thr_ret == 0);
 	
 	ok = _wapi_lookup_handle (handle, WAPI_HANDLE_THREAD,
@@ -499,7 +566,7 @@ static gpointer thread_attach(guint32 *tid)
 	ta_ret = handle;
 	
 	g_hash_table_insert (thread_hash,
-			     GUINT_TO_POINTER (thread_handle_p->thread->id),
+			     (gpointer)(thread_handle_p->thread->id),
 			     handle);
 
 #ifdef DEBUG
@@ -510,7 +577,11 @@ static gpointer thread_attach(guint32 *tid)
 
 	if (tid != NULL) {
 #ifdef PTHREAD_POINTER_ID
-		*tid = GPOINTER_TO_UINT(thread_handle_p->thread->id);
+		/* Don't use GPOINTER_TO_UINT here, it can't cope with
+		 * sizeof(void *) > sizeof(uint) when a cast to uint
+		 * would overflow
+		 */
+		*tid = (gsize)(thread_handle_p->thread->id);
 #else
 		*tid = thread_handle_p->thread->id;
 #endif
@@ -522,12 +593,10 @@ thread_hash_cleanup:
 	pthread_cleanup_pop (0);
 
 cleanup:
-	thr_ret = _wapi_handle_unlock_handle (handle);
-	g_assert (thr_ret == 0);
-	pthread_cleanup_pop (0);
+	_wapi_handle_unlock_shared_handles ();
 	
-	/* Must not call _wapi_handle_unref() with the handle already
-	 * locked
+	/* Must not call _wapi_handle_unref() with the shared handles
+	 * already locked
 	 */
 	for (i = 0; i < unrefs; i++) {
 		_wapi_handle_unref (handle);
@@ -550,25 +619,14 @@ cleanup:
 gpointer GetCurrentThread(void)
 {
 	gpointer ret=NULL;
-	guint32 tid;
-	int thr_ret;
+	gsize tid;
 	
 	mono_once(&thread_hash_once, thread_hash_init);
 	mono_once (&thread_ops_once, thread_ops_init);
 	
-	tid=GetCurrentThreadId();
-	
-	pthread_cleanup_push ((void(*)(void *))mono_mutex_unlock_in_cleanup,
-			      (void *)&thread_hash_mutex);
-	thr_ret = mono_mutex_lock(&thread_hash_mutex);
-	g_assert (thr_ret == 0);
-
-	ret = g_hash_table_lookup (thread_hash, GUINT_TO_POINTER (tid));
-
-	thr_ret = mono_mutex_unlock(&thread_hash_mutex);
-	g_assert (thr_ret == 0);
-	pthread_cleanup_pop (0);
+	tid = GetCurrentThreadId();
 	
+	ret = _wapi_thread_handle_from_id ((pthread_t)tid);
 	if (!ret) {
 		ret = thread_attach (NULL);
 	}
@@ -962,6 +1020,56 @@ gboolean _wapi_thread_dispatch_apc_queue (gpointer handle)
 	return(1);
 }
 
+void _wapi_thread_own_mutex (pthread_t tid, gpointer mutex)
+{
+	struct _WapiHandle_thread *thread_handle;
+	gboolean ok;
+	gpointer thread;
+
+	thread = _wapi_thread_handle_from_id (tid);
+	if (thread == NULL) {
+		g_warning ("%s: error looking up thread by ID", __func__);
+		return;
+	}
+
+	ok = _wapi_lookup_handle (thread, WAPI_HANDLE_THREAD,
+				  (gpointer *)&thread_handle);
+	if (ok == FALSE) {
+		g_warning ("%s: error looking up thread handle %p", __func__,
+			   thread);
+		return;
+	}
+
+	_wapi_handle_ref (mutex);
+	
+	g_ptr_array_add (thread_handle->owned_mutexes, mutex);
+}
+
+void _wapi_thread_disown_mutex (pthread_t tid, gpointer mutex)
+{
+	struct _WapiHandle_thread *thread_handle;
+	gboolean ok;
+	gpointer thread;
+
+	thread = _wapi_thread_handle_from_id (tid);
+	if (thread == NULL) {
+		g_warning ("%s: error looking up thread by ID", __func__);
+		return;
+	}
+
+	ok = _wapi_lookup_handle (thread, WAPI_HANDLE_THREAD,
+				  (gpointer *)&thread_handle);
+	if (ok == FALSE) {
+		g_warning ("%s: error looking up thread handle %p", __func__,
+			   thread);
+		return;
+	}
+
+	_wapi_handle_unref (mutex);
+	
+	g_ptr_array_remove (thread_handle->owned_mutexes, mutex);
+}
+
 
 
 #ifdef WITH_INCLUDED_LIBGC
