@@ -30,7 +30,7 @@
 #include <mono/metadata/threadpool.h>
 #include <mono/utils/mono-uri.h>
 
-#define MONO_CORLIB_VERSION 37
+#define MONO_CORLIB_VERSION 45
 
 CRITICAL_SECTION mono_delegate_section;
 
@@ -48,6 +48,10 @@ static MonoAssembly *
 mono_domain_assembly_search (MonoAssemblyName *aname,
 							 gpointer user_data);
 
+static MonoAssembly *
+mono_domain_assembly_postload_search (MonoAssemblyName *aname,
+									  gpointer user_data);
+
 static void
 mono_domain_fire_assembly_load (MonoAssembly *assembly, gpointer user_data);
 
@@ -87,6 +91,8 @@ mono_runtime_init (MonoDomain *domain, MonoThreadStartCB start_cb,
 	mono_install_assembly_refonly_preload_hook (mono_domain_assembly_preload, GUINT_TO_POINTER (TRUE));
 	mono_install_assembly_search_hook (mono_domain_assembly_search, GUINT_TO_POINTER (FALSE));
 	mono_install_assembly_refonly_search_hook (mono_domain_assembly_search, GUINT_TO_POINTER (TRUE));
+	mono_install_assembly_postload_search_hook (mono_domain_assembly_postload_search, GUINT_TO_POINTER (FALSE));
+	mono_install_assembly_postload_refonly_search_hook (mono_domain_assembly_postload_search, GUINT_TO_POINTER (TRUE));
 	mono_install_assembly_load_hook (mono_domain_fire_assembly_load, NULL);
 	mono_install_lookup_dynamic_token (mono_reflection_lookup_dynamic_token);
 
@@ -177,7 +183,13 @@ mono_context_init (MonoDomain *domain)
 	domain->default_context = context;
 }
 
-/* This must not be called while there are still running threads executing
+/**
+ * mono_runtime_cleanup:
+ * @domain: unused.
+ *
+ * Internal routine.
+ *
+ * This must not be called while there are still running threads executing
  * managed code.
  */
 void
@@ -206,18 +218,40 @@ mono_runtime_quit ()
 		quit_function (mono_get_root_domain (), NULL);
 }
 
+/** 
+ * mono_runtime_set_shutting_down:
+ *
+ * Invoked by System.Environment.Exit to flag that the runtime
+ * is shutting down.
+ */
 void
 mono_runtime_set_shutting_down (void)
 {
 	shutting_down = TRUE;
 }
 
+/**
+ * mono_runtime_is_shutting_down:
+ *
+ * Returns whether the runtime has been flagged for shutdown.
+ *
+ * This is consumed by the P:System.Environment.HasShutdownStarted
+ * property.
+ *
+ */
 gboolean
 mono_runtime_is_shutting_down (void)
 {
 	return shutting_down;
 }
 
+/**
+ * mono_domain_has_type_resolve:
+ * @domain: application domains being looked up
+ *
+ * Returns true if the AppDomain.TypeResolve field has been
+ * set.
+ */
 gboolean
 mono_domain_has_type_resolve (MonoDomain *domain)
 {
@@ -233,6 +267,19 @@ mono_domain_has_type_resolve (MonoDomain *domain)
 	return o != NULL;
 }
 
+/**
+ * mono_domain_try_type_resolve:
+ * @domain: application domainwhere the name where the type is going to be resolved
+ * @name: the name of the type to resolve or NULL.
+ * @tb: A System.Reflection.Emit.TypeBuilder, used if name is NULL.
+ *
+ * This routine invokes the internal System.AppDomain.DoTypeResolve and returns
+ * the assembly that matches name.
+ *
+ * If @name is null, the value of ((TypeBuilder)tb).FullName is used instead
+ *
+ * Returns: A MonoReflectionAssembly or NULL if not found
+ */
 MonoReflectionAssembly *
 mono_domain_try_type_resolve (MonoDomain *domain, char *name, MonoObject *tb)
 {
@@ -263,7 +310,7 @@ mono_domain_try_type_resolve (MonoDomain *domain, char *name, MonoObject *tb)
 /**
  * mono_domain_owns_vtable_slot:
  *
- *   Returns whenever VTABLE_SLOT is inside a vtable which belongs to DOMAIN.
+ *  Returns whenever VTABLE_SLOT is inside a vtable which belongs to DOMAIN.
  */
 gboolean
 mono_domain_owns_vtable_slot (MonoDomain *domain, gpointer vtable_slot)
@@ -281,7 +328,7 @@ mono_domain_owns_vtable_slot (MonoDomain *domain, gpointer vtable_slot)
  * @domain: domain
  * @force: force setting.
  *
- *   Set the current appdomain to @domain. If @force is set, set it even
+ * Set the current appdomain to @domain. If @force is set, set it even
  * if it is being unloaded.
  *
  * Returns:
@@ -516,6 +563,28 @@ try_assembly_resolve (MonoDomain *domain, MonoString *fname, gboolean refonly)
 	return (MonoReflectionAssembly *) mono_runtime_invoke (method, domain->domain, params, NULL);
 }
 
+static MonoAssembly *
+mono_domain_assembly_postload_search (MonoAssemblyName *aname,
+									  gpointer user_data)
+{
+	gboolean refonly = GPOINTER_TO_UINT (user_data);
+	MonoReflectionAssembly *assembly;
+	MonoDomain *domain = mono_domain_get ();
+	char *aname_str;
+
+	aname_str = mono_stringify_assembly_name (aname);
+
+	/* FIXME: We invoke managed code here, so there is a potential for deadlocks */
+	assembly = try_assembly_resolve (domain, mono_string_new (domain, aname_str), refonly);
+
+	g_free (aname_str);
+
+	if (assembly)
+		return assembly->assembly;
+	else
+		return NULL;
+}
+	
 /*
  * LOCKING: assumes assemblies_lock in the domain is already locked.
  */
@@ -581,7 +650,7 @@ mono_domain_fire_assembly_load (MonoAssembly *assembly, gpointer user_data)
 		g_assert (assembly_load_field);
 	}
 
-	mono_field_get_value (domain->domain, assembly_load_field, &load_value);
+	mono_field_get_value ((MonoObject*) domain->domain, assembly_load_field, &load_value);
 	if (load_value == NULL) {
 		/* No events waiting to be triggered */
 		return;
@@ -919,7 +988,7 @@ ves_icall_System_AppDomain_LoadAssemblyRaw (MonoAppDomain *ad,
 	MonoDomain *domain = ad->data;
 	MonoImageOpenStatus status;
 	guint32 raw_assembly_len = mono_array_length (raw_assembly);
-	MonoImage *image = mono_image_open_from_data (mono_array_addr (raw_assembly, gchar, 0), raw_assembly_len, TRUE, NULL);
+	MonoImage *image = mono_image_open_from_data_full (mono_array_addr (raw_assembly, gchar, 0), raw_assembly_len, TRUE, NULL, refonly);
 
 	if (raw_symbol_store)
 		mono_raise_exception (mono_get_exception_not_implemented ("LoadAssemblyRaw: Raw Symbol Store not Implemented"));
@@ -1198,7 +1267,7 @@ typedef struct unload_data {
 	char *failure_reason;
 } unload_data;
 
-static guint32
+static guint32 WINAPI
 unload_thread_main (void *arg)
 {
 	unload_data *data = (unload_data*)arg;
@@ -1244,9 +1313,11 @@ unload_thread_main (void *arg)
 
 /*
  * mono_domain_unload:
+ * @domain: The domain to unload
  *
  *  Unloads an appdomain. Follows the process outlined in:
  *  http://blogs.gotdotnet.com/cbrumme
+ *
  *  If doing things the 'right' way is too hard or complex, we do it the 
  *  'simple' way, which means do everything needed to avoid crashes and
  *  memory leaks, but not much else.
@@ -1255,7 +1326,8 @@ static void
 mono_domain_unload (MonoDomain *domain)
 {
 	HANDLE thread_handle;
-	guint32 tid, res;
+	gsize tid;
+	guint32 res;
 	MonoAppDomainState prev_state;
 	MonoMethod *method;
 	MonoObject *exc;
@@ -1329,11 +1401,11 @@ mono_domain_unload (MonoDomain *domain)
 
 		g_warning (thread_data.failure_reason);
 
+		ex = mono_get_exception_cannot_unload_appdomain (thread_data.failure_reason);
+
 		g_free (thread_data.failure_reason);
 		thread_data.failure_reason = NULL;
 
-		ex = mono_get_exception_cannot_unload_appdomain (thread_data.failure_reason);
-
 		mono_raise_exception (ex);
 	}
 }
