@@ -20,19 +20,15 @@
 #include <mono/metadata/loader.h>
 #include <mono/metadata/tabledefs.h>
 #include <mono/metadata/metadata-internals.h>
+#include <mono/metadata/class-internals.h>
 #include <mono/metadata/domain-internals.h>
+#include <mono/metadata/mono-endian.h>
 #include <mono/io-layer/io-layer.h>
 #include <mono/utils/mono-uri.h>
 #include <mono/metadata/mono-config.h>
 #include <mono/utils/mono-digest.h>
 #include <mono/utils/mono-logger.h>
-#ifdef PLATFORM_WIN32
 #include <mono/os/util.h>
-#ifdef _MSC_VER
-	/* not used on Windows - see mono_set_rootdir () */
-	#define MONO_ASSEMBLIES		NULL
-#endif
-#endif
 
 /* AssemblyVersionMap: an assembly name and the assembly version set on which it is based */
 typedef struct  {
@@ -40,10 +36,10 @@ typedef struct  {
 	guint8 version_set_index;
 } AssemblyVersionMap;
 
-/* the default search path is just MONO_ASSEMBLIES */
+/* the default search path is empty, the first slot is replaced with the computed value */
 static const char*
 default_path [] = {
-	MONO_ASSEMBLIES,
+	NULL,
 	NULL
 };
 
@@ -114,6 +110,8 @@ static GList *loaded_assemblies = NULL;
 static MonoAssembly *corlib;
 
 /* This protects loaded_assemblies and image->references */
+#define mono_assemblies_lock() EnterCriticalSection (&assemblies_mutex)
+#define mono_assemblies_unlock() LeaveCriticalSection (&assemblies_mutex)
 static CRITICAL_SECTION assemblies_mutex;
 
 /* A hastable of thread->assembly list mappings */
@@ -125,8 +123,11 @@ static GHashTable *assemblies_refonly_loading;
 /* If defined, points to the bundled assembly information */
 const MonoBundledAssembly **bundles;
 
-/* Reflection only private hook functions */
-static MonoAssembly* mono_assembly_refonly_invoke_search_hook (MonoAssemblyName *aname);
+/* Loaded assembly binding info */
+static GSList *loaded_assembly_bindings = NULL;
+
+static MonoAssembly*
+mono_assembly_invoke_search_hook_internal (MonoAssemblyName *aname, gboolean refonly, gboolean postload);
 
 static gchar*
 encode_public_tok (const guchar *token, gint32 len)
@@ -193,6 +194,142 @@ check_extra_gac_path_env (void) {
 	}
 }
 
+static gboolean
+assembly_binding_maps_name (MonoAssemblyBindingInfo *info, MonoAssemblyName *aname)
+{
+	if (strcmp (info->name, aname->name))
+		return FALSE;
+
+	if (info->major != aname->major || info->minor != aname->minor)
+		return FALSE;
+
+	if ((info->culture != NULL) != (aname->culture != NULL))
+		return FALSE;
+	
+	if (info->culture && strcmp (info->culture, aname->culture))
+		return FALSE;
+	
+	if (strcmp ((const char *)info->public_key_token, (const char *)aname->public_key_token))
+		return FALSE;
+
+	return TRUE;
+}
+
+static void
+mono_assembly_binding_info_free (MonoAssemblyBindingInfo *info)
+{
+	g_free (info->name);
+	g_free (info->culture);
+}
+
+static void
+get_publisher_policy_info (MonoImage *image, MonoAssemblyName *aname, MonoAssemblyBindingInfo *binding_info)
+{
+	MonoTableInfo *t;
+	guint32 cols [MONO_MANIFEST_SIZE];
+	const gchar *filename;
+	gchar *subpath, *fullpath;
+
+	t = &image->tables [MONO_TABLE_MANIFESTRESOURCE];
+	/* MS Impl. accepts policy assemblies with more than
+	 * one manifest resource, and only takes the first one */
+	if (t->rows < 1) {
+		binding_info->is_valid = FALSE;
+		return;
+	}
+	
+	mono_metadata_decode_row (t, 0, cols, MONO_MANIFEST_SIZE);
+	if ((cols [MONO_MANIFEST_IMPLEMENTATION] & MONO_IMPLEMENTATION_MASK) != MONO_IMPLEMENTATION_FILE) {
+		binding_info->is_valid = FALSE;
+		return;
+	}
+	
+	filename = mono_metadata_string_heap (image, cols [MONO_MANIFEST_NAME]);
+	g_assert (filename != NULL);
+	
+	subpath = g_path_get_dirname (image->name);
+	fullpath = g_build_path (G_DIR_SEPARATOR_S, subpath, filename, NULL);
+	mono_config_parse_publisher_policy (fullpath, binding_info);
+	g_free (subpath);
+	g_free (fullpath);
+	
+	/* Define the optional elements/attributes before checking */
+	if (!binding_info->culture)
+		binding_info->culture = g_strdup ("");
+	
+	/* Check that the most important elements/attributes exist */
+	if (!binding_info->name || !binding_info->public_key_token [0] || !binding_info->has_old_version_bottom ||
+			!binding_info->has_new_version || !assembly_binding_maps_name (binding_info, aname)) {
+		mono_assembly_binding_info_free (binding_info);
+		binding_info->is_valid = FALSE;
+		return;
+	}
+
+	binding_info->is_valid = TRUE;
+}
+
+static int
+compare_versions (AssemblyVersionSet *v, MonoAssemblyName *aname)
+{
+	if (v->major > aname->major)
+		return 1;
+	else if (v->major < aname->major)
+		return -1;
+
+	if (v->minor > aname->minor)
+		return 1;
+	else if (v->minor < aname->minor)
+		return -1;
+
+	if (v->build > aname->build)
+		return 1;
+	else if (v->build < aname->build)
+		return -1;
+
+	if (v->revision > aname->revision)
+		return 1;
+	else if (v->revision < aname->revision)
+		return -1;
+
+	return 0;
+}
+
+static gboolean
+check_policy_versions (MonoAssemblyBindingInfo *info, MonoAssemblyName *name)
+{
+	if (!info->is_valid)
+		return FALSE;
+	
+	/* If has_old_version_top doesn't exist, we don't have an interval */
+	if (!info->has_old_version_top) {
+		if (compare_versions (&info->old_version_bottom, name) == 0)
+			return TRUE;
+
+		return FALSE;
+	}
+
+	/* Check that the version defined by name is valid for the interval */
+	if (compare_versions (&info->old_version_top, name) < 0)
+		return FALSE;
+
+	/* We should be greater or equal than the small version */
+	if (compare_versions (&info->old_version_bottom, name) > 0)
+		return FALSE;
+
+	return TRUE;
+}
+
+/**
+ * mono_assembly_names_equal:
+ * @l: first assembly
+ * @r: second assembly.
+ *
+ * Compares two MonoAssemblyNames and returns whether they are equal.
+ * This compares the names, the cultures, the release version and their
+ * public tokens.
+ *
+ * Returns: TRUE if both assembly names are equal.
+ */
 gboolean
 mono_assembly_names_equal (MonoAssemblyName *l, MonoAssemblyName *r)
 {
@@ -226,7 +363,7 @@ search_loaded (MonoAssemblyName* aname, gboolean refonly)
 	MonoAssembly *ass;
 	GList *loading;
 
-	ass = refonly ? mono_assembly_refonly_invoke_search_hook (aname) : mono_assembly_invoke_search_hook (aname);
+	ass = mono_assembly_invoke_search_hook_internal (aname, refonly, FALSE);
 	if (ass)
 		return ass;
 	
@@ -282,12 +419,52 @@ mono_assembly_setrootdir (const char *root_dir)
 	default_path [0] = g_strdup (root_dir);
 }
 
+/**
+ * mono_assembly_getrootdir:
+ *
+ * Returns: The internal root directory used for looking up assemblies
+ */
 G_CONST_RETURN gchar *
 mono_assembly_getrootdir (void)
 {
 	return default_path [0];
 }
 
+/**
+ * mono_set_dirs:
+ * @assembly_dir: the base directory for assemblies
+ * @config_dir: the base directory for configuration files
+ *
+ * This routine is used internally and by developers embedding
+ * the runtime into their own applications.  There are a number
+ * of cases to consider: Mono as a system-installed package that
+ * is available on the location preconfigured or Mono in a relocated
+ * location.
+ *
+ * If you are using a system-installed Mono, you can pass NULL
+ * to both parameters.  If you are not, you should compute both
+ * directory values and call this routine.
+ *
+ * The values for a given PREFIX are:
+ *
+ *    assembly_dir: PREFIX/lib
+ *    config_dir:   PREFIX/etc
+ *
+ * Notice that embedders that use Mono in a relocated way must
+ * compute the location at runtime, as they will be in control
+ * of where Mono is installed.
+ */
+void
+mono_set_dirs (const char *assembly_dir, const char *config_dir)
+{
+	if (assembly_dir == NULL)
+		assembly_dir = MONO_ASSEMBLIES;
+	if (config_dir == NULL)
+		config_dir = MONO_CFG_DIR;
+	mono_assembly_setrootdir (assembly_dir);
+	mono_set_config_dir (config_dir);
+}
+
 /**
  * mono_assemblies_init:
  *
@@ -296,9 +473,12 @@ mono_assembly_getrootdir (void)
 void
 mono_assemblies_init (void)
 {
-#ifdef PLATFORM_WIN32
-	mono_set_rootdir ();
-#endif
+	/*
+	 * Initialize our internal paths if we have not been initialized yet.
+	 * This happens when embedders use Mono.
+	 */
+	if (mono_assembly_getrootdir () == NULL)
+		mono_set_rootdir ();
 
 	check_path_env ();
 	check_extra_gac_path_env ();
@@ -359,6 +539,24 @@ mono_assembly_fill_assembly_name (MonoImage *image, MonoAssemblyName *aname)
 	return TRUE;
 }
 
+/*
+ * mono_stringify_assembly_name:
+ *
+ *   Convert @aname into its string format. The returned string is dynamically
+ * allocated and should be freed by the caller.
+ */
+char*
+mono_stringify_assembly_name (MonoAssemblyName *aname)
+{
+	return g_strdup_printf (
+		"%s, Version=%d.%d.%d.%d, Culture=%s%s%s",
+		aname->name,
+		aname->major, aname->minor, aname->build, aname->revision,
+		aname->culture && *aname->culture? aname->culture: "neutral",
+		aname->public_key_token [0] ? ", PublicKeyToken=" : "",
+		aname->public_key_token [0] ? (char *)aname->public_key_token : "");
+}
+
 static gchar*
 assemblyref_public_tok (MonoImage *image, guint32 key_index, guint32 flags)
 {
@@ -377,13 +575,18 @@ assemblyref_public_tok (MonoImage *image, guint32 key_index, guint32 flags)
 	return encode_public_tok (public_tok, len);
 }
 
+/**
+ * mono_assembly_addref:
+ * @assemnly: the assembly to reference
+ *
+ * This routine increments the reference count on a MonoAssembly.
+ * The reference count is reduced every time the method mono_assembly_close() is
+ * invoked.
+ */
 void
 mono_assembly_addref (MonoAssembly *assembly)
 {
-	EnterCriticalSection (&assemblies_mutex);
-	/*g_print ("adding ref from %d to %s (%p)\n", assembly->ref_count, assembly->aname.name, assembly);*/
-	assembly->ref_count++;
-	LeaveCriticalSection (&assemblies_mutex);
+	InterlockedIncrement (&assembly->ref_count);
 }
 
 static MonoAssemblyName *
@@ -435,12 +638,45 @@ mono_assembly_remap_version (MonoAssemblyName *aname, MonoAssemblyName *dest_ana
 	return aname;
 }
 
+/*
+ * mono_assembly_get_assemblyref:
+ *
+ *   Fill out ANAME with the assembly name of the INDEXth assembly reference in IMAGE.
+ */
 void
-mono_assembly_load_reference (MonoImage *image, int index)
+mono_assembly_get_assemblyref (MonoImage *image, int index, MonoAssemblyName *aname)
 {
 	MonoTableInfo *t;
 	guint32 cols [MONO_ASSEMBLYREF_SIZE];
 	const char *hash;
+
+	t = &image->tables [MONO_TABLE_ASSEMBLYREF];
+
+	mono_metadata_decode_row (t, index, cols, MONO_ASSEMBLYREF_SIZE);
+		
+	hash = mono_metadata_blob_heap (image, cols [MONO_ASSEMBLYREF_HASH_VALUE]);
+	aname->hash_len = mono_metadata_decode_blob_size (hash, &hash);
+	aname->hash_value = hash;
+	aname->name = mono_metadata_string_heap (image, cols [MONO_ASSEMBLYREF_NAME]);
+	aname->culture = mono_metadata_string_heap (image, cols [MONO_ASSEMBLYREF_CULTURE]);
+	aname->flags = cols [MONO_ASSEMBLYREF_FLAGS];
+	aname->major = cols [MONO_ASSEMBLYREF_MAJOR_VERSION];
+	aname->minor = cols [MONO_ASSEMBLYREF_MINOR_VERSION];
+	aname->build = cols [MONO_ASSEMBLYREF_BUILD_NUMBER];
+	aname->revision = cols [MONO_ASSEMBLYREF_REV_NUMBER];
+
+	if (cols [MONO_ASSEMBLYREF_PUBLIC_KEY]) {
+		gchar *token = assemblyref_public_tok (image, cols [MONO_ASSEMBLYREF_PUBLIC_KEY], aname->flags);
+		g_strlcpy (aname->public_key_token, token, MONO_PUBLIC_KEY_TOKEN_LENGTH);
+		g_free (token);
+	} else {
+		memset (aname->public_key_token, 0, MONO_PUBLIC_KEY_TOKEN_LENGTH);
+	}
+}
+
+void
+mono_assembly_load_reference (MonoImage *image, int index)
+{
 	MonoAssembly *reference;
 	MonoAssemblyName aname;
 	MonoImageOpenStatus status;
@@ -449,34 +685,13 @@ mono_assembly_load_reference (MonoImage *image, int index)
 	 * image->references is shared between threads, so we need to access
 	 * it inside a critical section.
 	 */
-	EnterCriticalSection (&assemblies_mutex);
+	mono_assemblies_lock ();
 	reference = image->references [index];
-	LeaveCriticalSection (&assemblies_mutex);
+	mono_assemblies_unlock ();
 	if (reference)
 		return;
 
-	t = &image->tables [MONO_TABLE_ASSEMBLYREF];
-
-	mono_metadata_decode_row (t, index, cols, MONO_ASSEMBLYREF_SIZE);
-		
-	hash = mono_metadata_blob_heap (image, cols [MONO_ASSEMBLYREF_HASH_VALUE]);
-	aname.hash_len = mono_metadata_decode_blob_size (hash, &hash);
-	aname.hash_value = hash;
-	aname.name = mono_metadata_string_heap (image, cols [MONO_ASSEMBLYREF_NAME]);
-	aname.culture = mono_metadata_string_heap (image, cols [MONO_ASSEMBLYREF_CULTURE]);
-	aname.flags = cols [MONO_ASSEMBLYREF_FLAGS];
-	aname.major = cols [MONO_ASSEMBLYREF_MAJOR_VERSION];
-	aname.minor = cols [MONO_ASSEMBLYREF_MINOR_VERSION];
-	aname.build = cols [MONO_ASSEMBLYREF_BUILD_NUMBER];
-	aname.revision = cols [MONO_ASSEMBLYREF_REV_NUMBER];
-
-	if (cols [MONO_ASSEMBLYREF_PUBLIC_KEY]) {
-		gchar *token = assemblyref_public_tok (image, cols [MONO_ASSEMBLYREF_PUBLIC_KEY], aname.flags);
-		g_strlcpy (aname.public_key_token, token, MONO_PUBLIC_KEY_TOKEN_LENGTH);
-		g_free (token);
-	} else {
-		memset (aname.public_key_token, 0, MONO_PUBLIC_KEY_TOKEN_LENGTH);
-	}
+	mono_assembly_get_assemblyref (image, index, &aname);
 
 	if (image->assembly->ref_only) {
 		/* We use the loaded corlib */
@@ -516,17 +731,17 @@ mono_assembly_load_reference (MonoImage *image, int index)
 		g_free (extra_msg);
 	}
 
-	EnterCriticalSection (&assemblies_mutex);
+	mono_assemblies_lock ();
 	if (reference == NULL) {
 		/* Flag as not found */
 		reference = (gpointer)-1;
 	} else {
-		reference->ref_count++;
+		mono_assembly_addref (reference);
 	}	
 
 	if (!image->references [index])
 		image->references [index] = reference;
-	LeaveCriticalSection (&assemblies_mutex);
+	mono_assemblies_unlock ();
 
 	if (image->references [index] != reference) {
 		/* Somebody loaded it before us */
@@ -592,42 +807,37 @@ typedef struct AssemblySearchHook AssemblySearchHook;
 struct AssemblySearchHook {
 	AssemblySearchHook *next;
 	MonoAssemblySearchFunc func;
+	gboolean refonly;
+	gboolean postload;
 	gpointer user_data;
 };
 
 AssemblySearchHook *assembly_search_hook = NULL;
-static AssemblySearchHook *assembly_refonly_search_hook = NULL;
 
-MonoAssembly*
-mono_assembly_invoke_search_hook (MonoAssemblyName *aname)
+static MonoAssembly*
+mono_assembly_invoke_search_hook_internal (MonoAssemblyName *aname, gboolean refonly, gboolean postload)
 {
 	AssemblySearchHook *hook;
 
 	for (hook = assembly_search_hook; hook; hook = hook->next) {
-		MonoAssembly *ass = hook->func (aname, hook->user_data);
-		if (ass)
-			return ass;
+		if ((hook->refonly == refonly) && (hook->postload == postload)) {
+			MonoAssembly *ass = hook->func (aname, hook->user_data);
+			if (ass)
+				return ass;
+		}
 	}
 
 	return NULL;
 }
 
-static MonoAssembly*
-mono_assembly_refonly_invoke_search_hook (MonoAssemblyName *aname)
+MonoAssembly*
+mono_assembly_invoke_search_hook (MonoAssemblyName *aname)
 {
-	AssemblySearchHook *hook;
-
-	for (hook = assembly_refonly_search_hook; hook; hook = hook->next) {
-		MonoAssembly *ass = hook->func (aname, hook->user_data);
-		if (ass)
-			return ass;
-	}
-
-	return NULL;
+	return mono_assembly_invoke_search_hook_internal (aname, FALSE, FALSE);
 }
 
-void          
-mono_install_assembly_search_hook (MonoAssemblySearchFunc func, gpointer user_data)
+static void
+mono_install_assembly_search_hook_internal (MonoAssemblySearchFunc func, gpointer user_data, gboolean refonly, gboolean postload)
 {
 	AssemblySearchHook *hook;
 	
@@ -636,22 +846,34 @@ mono_install_assembly_search_hook (MonoAssemblySearchFunc func, gpointer user_da
 	hook = g_new0 (AssemblySearchHook, 1);
 	hook->func = func;
 	hook->user_data = user_data;
+	hook->refonly = refonly;
+	hook->postload = postload;
 	hook->next = assembly_search_hook;
 	assembly_search_hook = hook;
+}
+
+void          
+mono_install_assembly_search_hook (MonoAssemblySearchFunc func, gpointer user_data)
+{
+	mono_install_assembly_search_hook_internal (func, user_data, FALSE, FALSE);
 }	
 
 void
 mono_install_assembly_refonly_search_hook (MonoAssemblySearchFunc func, gpointer user_data)
 {
-	AssemblySearchHook *hook;
+	mono_install_assembly_search_hook_internal (func, user_data, TRUE, FALSE);
+}
 
-	g_return_if_fail (func != NULL);
+void          
+mono_install_assembly_postload_search_hook (MonoAssemblySearchFunc func, gpointer user_data)
+{
+	mono_install_assembly_search_hook_internal (func, user_data, FALSE, TRUE);
+}	
 
-	hook = g_new0 (AssemblySearchHook, 1);
-	hook->func = func;
-	hook->user_data = user_data;
-	hook->next = assembly_refonly_search_hook;
-	assembly_refonly_search_hook = hook;
+void
+mono_install_assembly_postload_refonly_search_hook (MonoAssemblySearchFunc func, gpointer user_data)
+{
+	mono_install_assembly_search_hook_internal (func, user_data, TRUE, TRUE);
 }
 
 typedef struct AssemblyPreLoadHook AssemblyPreLoadHook;
@@ -787,7 +1009,7 @@ absolute_dir (const gchar *filename)
  * returns NULL
  */
 static MonoImage *
-mono_assembly_open_from_bundle (const char *filename, MonoImageOpenStatus *status)
+mono_assembly_open_from_bundle (const char *filename, MonoImageOpenStatus *status, gboolean refonly)
 {
 	int i;
 	char *name = g_path_get_basename (filename);
@@ -797,14 +1019,14 @@ mono_assembly_open_from_bundle (const char *filename, MonoImageOpenStatus *statu
 	 * we do a very simple search for bundled assemblies: it's not a general 
 	 * purpose assembly loading mechanism.
 	 */
-	EnterCriticalSection (&assemblies_mutex);
+	mono_assemblies_lock ();
 	for (i = 0; !image && bundles [i]; ++i) {
 		if (strcmp (bundles [i]->name, name) == 0) {
-			image = mono_image_open_from_data ((char*)bundles [i]->data, bundles [i]->size, FALSE, status);
+			image = mono_image_open_from_data_full ((char*)bundles [i]->data, bundles [i]->size, FALSE, status, refonly);
 			break;
 		}
 	}
-	LeaveCriticalSection (&assemblies_mutex);
+	mono_assemblies_unlock ();
 	g_free (name);
 	if (image) {
 		mono_image_addref (image);
@@ -814,19 +1036,19 @@ mono_assembly_open_from_bundle (const char *filename, MonoImageOpenStatus *statu
 }
 
 static MonoImage*
-do_mono_assembly_open (const char *filename, MonoImageOpenStatus *status)
+do_mono_assembly_open (const char *filename, MonoImageOpenStatus *status, gboolean refonly)
 {
 	MonoImage *image = NULL;
 
 	if (bundles != NULL){
-		image = mono_assembly_open_from_bundle (filename, status);
+		image = mono_assembly_open_from_bundle (filename, status, refonly);
 
 		if (image != NULL)
 			return image;
 	}
-	EnterCriticalSection (&assemblies_mutex);
-	image = mono_image_open (filename, status);
-	LeaveCriticalSection (&assemblies_mutex);
+	mono_assemblies_lock ();
+	image = mono_image_open_full (filename, status, refonly);
+	mono_assemblies_unlock ();
 
 	return image;
 }
@@ -876,7 +1098,7 @@ mono_assembly_open_full (const char *filename, MonoImageOpenStatus *status, gboo
 
 	mono_trace (G_LOG_LEVEL_INFO, MONO_TRACE_ASSEMBLY,
 			"Assembly Loader probing location: '%s'.", filename);
-	image = do_mono_assembly_open (fname, status);
+	image = do_mono_assembly_open (fname, status, refonly);
 
 	if (!image){
 		*status = MONO_IMAGE_ERROR_ERRNO;
@@ -964,7 +1186,7 @@ mono_assembly_load_from_full (MonoImage *image, const char*fname,
 	/* 
 	 * Atomically search the loaded list and add ourselves to it if necessary.
 	 */
-	EnterCriticalSection (&assemblies_mutex);
+	mono_assemblies_lock ();
 	if (ass->aname.name) {
 		/* avoid loading the same assembly twice for now... */
 		ass2 = search_loaded (&ass->aname, refonly);
@@ -973,7 +1195,7 @@ mono_assembly_load_from_full (MonoImage *image, const char*fname,
 			g_free (base_dir);
 			mono_image_close (image);
 			*status = MONO_IMAGE_OK;
-			LeaveCriticalSection (&assemblies_mutex);
+			mono_assemblies_unlock ();
 			return ass2;
 		}
 	}
@@ -981,13 +1203,13 @@ mono_assembly_load_from_full (MonoImage *image, const char*fname,
 	loading = g_hash_table_lookup (ass_loading, GetCurrentThread ());
 	loading = g_list_prepend (loading, ass);
 	g_hash_table_insert (ass_loading, GetCurrentThread (), loading);
-	LeaveCriticalSection (&assemblies_mutex);
+	mono_assemblies_unlock ();
 
 	image->assembly = ass;
 
 	mono_assembly_load_references (image, status);
 
-	EnterCriticalSection (&assemblies_mutex);
+	mono_assemblies_lock ();
 
 	loading = g_hash_table_lookup (ass_loading, GetCurrentThread ());
 	loading = g_list_remove (loading, ass);
@@ -997,7 +1219,7 @@ mono_assembly_load_from_full (MonoImage *image, const char*fname,
 	else
 		g_hash_table_insert (ass_loading, GetCurrentThread (), loading);
 	if (*status != MONO_IMAGE_OK) {
-		LeaveCriticalSection (&assemblies_mutex);
+		mono_assemblies_unlock ();
 		mono_assembly_close (ass);
 		return NULL;
 	}
@@ -1006,14 +1228,14 @@ mono_assembly_load_from_full (MonoImage *image, const char*fname,
 		ass2 = search_loaded (&ass->aname, refonly);
 		if (ass2) {
 			/* Somebody else has loaded the assembly before us */
-			LeaveCriticalSection (&assemblies_mutex);
+			mono_assemblies_unlock ();
 			mono_assembly_close (ass);
 			return ass2;
 		}
 	}
 
 	loaded_assemblies = g_list_prepend (loaded_assemblies, ass);
-	LeaveCriticalSection (&assemblies_mutex);
+	mono_assemblies_unlock ();
 
 	mono_assembly_invoke_load_hook (ass);
 
@@ -1046,9 +1268,85 @@ mono_assembly_name_free (MonoAssemblyName *aname)
 }
 
 static gboolean
-build_assembly_name (const char *name, const char *version, const char *culture, const char *token, MonoAssemblyName *aname)
+parse_public_key (const gchar *key, gchar** pubkey)
+{
+	const gchar *pkey;
+	gchar header [16], val, *arr;
+	gint i, j, offset, bitlen, keylen, pkeylen;
+	
+	keylen = strlen (key) >> 1;
+	if (keylen < 1)
+		return FALSE;
+	
+	val = g_ascii_xdigit_value (key [0]) << 4;
+	val |= g_ascii_xdigit_value (key [1]);
+	switch (val) {
+		case 0x00:
+			if (keylen < 13)
+				return FALSE;
+			val = g_ascii_xdigit_value (key [24]);
+			val |= g_ascii_xdigit_value (key [25]);
+			if (val != 0x06)
+				return FALSE;
+			pkey = key + 24;
+			break;
+		case 0x06:
+			pkey = key;
+			break;
+		default:
+			return FALSE;
+	}
+		
+	/* We need the first 16 bytes
+	* to check whether this key is valid or not */
+	pkeylen = strlen (pkey) >> 1;
+	if (pkeylen < 16)
+		return FALSE;
+		
+	for (i = 0, j = 0; i < 16; i++) {
+		header [i] = g_ascii_xdigit_value (pkey [j++]) << 4;
+		header [i] |= g_ascii_xdigit_value (pkey [j++]);
+	}
+
+	if (header [0] != 0x06 || /* PUBLICKEYBLOB (0x06) */
+			header [1] != 0x02 || /* Version (0x02) */
+			header [2] != 0x00 || /* Reserved (word) */
+			header [3] != 0x00 ||
+			(guint)(read32 (header + 8)) != 0x31415352) /* DWORD magic = RSA1 */
+		return FALSE;
+
+	/* Based on this length, we _should_ be able to know if the length is right */
+	bitlen = read32 (header + 12) >> 3;
+	if ((bitlen + 16 + 4) != pkeylen)
+		return FALSE;
+		
+	/* Encode the size of the blob */
+	offset = 0;
+	if (keylen <= 127) {
+		arr = g_malloc (keylen + 1);
+		arr [offset++] = keylen;
+	} else {
+		arr = g_malloc (keylen + 2);
+		arr [offset++] = 0x80; /* 10bs */
+		arr [offset++] = keylen;
+	}
+		
+	for (i = offset, j = 0; i < keylen + offset; i++) {
+		arr [i] = g_ascii_xdigit_value (key [j++]) << 4;
+		arr [i] |= g_ascii_xdigit_value (key [j++]);
+	}
+	if (pubkey)
+		*pubkey = arr;
+
+	return TRUE;
+}
+
+static gboolean
+build_assembly_name (const char *name, const char *version, const char *culture, const char *token, const char *key, MonoAssemblyName *aname, gboolean save_public_key)
 {
 	gint major, minor, build, revision;
+	gint len;
+	gchar *pkey, *pkeyptr, *encoded, tok [8];
 
 	memset (aname, 0, sizeof (MonoAssemblyName));
 
@@ -1073,6 +1371,25 @@ build_assembly_name (const char *name, const char *version, const char *culture,
 	
 	if (token && strncmp (token, "null", 4) != 0)
 		g_strlcpy ((char*)aname->public_key_token, token, MONO_PUBLIC_KEY_TOKEN_LENGTH);
+
+	if (key && strncmp (key, "null", 4) != 0) {
+		if (!parse_public_key (key, &pkey)) {
+			mono_assembly_name_free (aname);
+			return FALSE;
+		}
+		
+		len = mono_metadata_decode_blob_size ((const gchar *) pkey, (const gchar **) &pkeyptr);
+		// We also need to generate the key token
+		mono_digest_get_public_token ((guchar*) tok, (guint8*) pkeyptr, len);
+		encoded = encode_public_tok ((guchar*) tok, 8);
+		g_strlcpy ((gchar*)aname->public_key_token, encoded, MONO_PUBLIC_KEY_TOKEN_LENGTH);
+		g_free (encoded);
+
+		if (save_public_key)
+			aname->public_key = (guint8*) pkey;
+		else
+			g_free (pkey);
+	}
 	
 	return TRUE;
 }
@@ -1089,32 +1406,29 @@ parse_assembly_directory_name (const char *name, const char *dirname, MonoAssemb
 		return FALSE;
 	}
 	
-	res = build_assembly_name (name, parts[0], parts[1], parts[2], aname);
+	res = build_assembly_name (name, parts[0], parts[1], parts[2], NULL, aname, FALSE);
 	g_strfreev (parts);
 	return res;
 }
 
-/**
-* mono_assembly_name_parse:
-* @name: name to parse
-* @aname: the destination assembly name
-* Returns: true if the name could be parsed.
-* 
-* Parses an assembly qualified type name and assigns the name,
-* version, culture and token to the provided assembly name object.
-*/
 gboolean
-mono_assembly_name_parse (const char *name, MonoAssemblyName *aname)
+mono_assembly_name_parse_full (const char *name, MonoAssemblyName *aname, gboolean save_public_key, gboolean *is_version_defined)
 {
 	gchar *dllname;
 	gchar *version = NULL;
 	gchar *culture = NULL;
 	gchar *token = NULL;
+	gchar *key = NULL;
 	gboolean res;
 	gchar *value;
 	gchar **parts;
 	gchar **tmp;
+	gboolean version_defined;
 
+	if (!is_version_defined)
+		is_version_defined = &version_defined;
+	*is_version_defined = FALSE;
+	
 	parts = tmp = g_strsplit (name, ",", 4);
 	if (!tmp || !*tmp) {
 		g_strfreev (tmp);
@@ -1128,6 +1442,7 @@ mono_assembly_name_parse (const char *name, MonoAssemblyName *aname)
 	while (*tmp) {
 		value = g_strstrip (*tmp);
 		if (!g_ascii_strncasecmp (value, "Version=", 8)) {
+			*is_version_defined = TRUE;
 			version = g_strstrip (value + 8);
 			tmp++;
 			continue;
@@ -1144,16 +1459,37 @@ mono_assembly_name_parse (const char *name, MonoAssemblyName *aname)
 			tmp++;
 			continue;
 		}
+
+		if (!g_ascii_strncasecmp (value, "PublicKey=", 10)) {
+			key = g_strstrip (value + 10);
+			tmp++;
+			continue;
+		}
 		
 		g_strfreev (parts);
 		return FALSE;
 	}
 
-	res = build_assembly_name (dllname, version, culture, token, aname);
+	res = build_assembly_name (dllname, version, culture, token, key, aname, save_public_key);
 	g_strfreev (parts);
 	return res;
 }
 
+/**
+* mono_assembly_name_parse:
+* @name: name to parse
+* @aname: the destination assembly name
+* Returns: true if the name could be parsed.
+* 
+* Parses an assembly qualified type name and assigns the name,
+* version, culture and token to the provided assembly name object.
+*/
+gboolean
+mono_assembly_name_parse (const char *name, MonoAssemblyName *aname)
+{
+	return mono_assembly_name_parse_full (name, aname, FALSE, NULL);
+}
+
 static MonoAssembly*
 probe_for_partial_name (const char *basepath, const char *fullname, MonoAssemblyName *aname, MonoImageOpenStatus *status)
 {
@@ -1290,6 +1626,149 @@ mono_assembly_load_with_partial_name (const char *name, MonoImageOpenStatus *sta
 	return res;
 }
 
+static MonoImage*
+mono_assembly_load_publisher_policy (MonoAssemblyName *aname)
+{
+	MonoImage *image;
+	gchar *filename, *pname, *name, *culture, *version, *fullpath, *subpath;
+	gchar **paths;
+	gint32 len;
+
+	if (strstr (aname->name, ".dll")) {
+		len = strlen (aname->name) - 4;
+		name = g_malloc (len);
+		strncpy (name, aname->name, len);
+	} else
+		name = g_strdup (aname->name);
+	
+	if (aname->culture) {
+		culture = g_strdup (aname->culture);
+		g_strdown (culture);
+	} else
+		culture = g_strdup ("");
+	
+	pname = g_strdup_printf ("policy.%d.%d.%s", aname->major, aname->minor, name);
+	version = g_strdup_printf ("0.0.0.0_%s_%s", culture, aname->public_key_token);
+	g_free (name);
+	g_free (culture);
+	
+	filename = g_strconcat (pname, ".dll", NULL);
+	subpath = g_build_path (G_DIR_SEPARATOR_S, pname, version, filename, NULL);
+	g_free (pname);
+	g_free (version);
+	g_free (filename);
+
+	image = NULL;
+	if (extra_gac_paths) {
+		paths = extra_gac_paths;
+		while (!image && *paths) {
+			fullpath = g_build_path (G_DIR_SEPARATOR_S, *paths,
+					"lib", "mono", "gac", subpath, NULL);
+			image = mono_image_open (fullpath, NULL);
+			g_free (fullpath);
+			paths++;
+		}
+	}
+
+	if (image) {
+		g_free (subpath);
+		return image;
+	}
+
+	fullpath = g_build_path (G_DIR_SEPARATOR_S, mono_assembly_getrootdir (), 
+			"mono", "gac", subpath, NULL);
+	image = mono_image_open (fullpath, NULL);
+	g_free (subpath);
+	g_free (fullpath);
+	
+	return image;
+}
+
+static MonoAssemblyName*
+mono_assembly_bind_version (MonoAssemblyBindingInfo *info, MonoAssemblyName *aname, MonoAssemblyName *dest_name)
+{
+	memcpy (dest_name, aname, sizeof (MonoAssemblyName));
+	dest_name->major = info->new_version.major;
+	dest_name->minor = info->new_version.minor;
+	dest_name->build = info->new_version.build;
+	dest_name->revision = info->new_version.revision;
+	
+	return dest_name;
+}
+
+/* LOCKING: Assumes that we are already locked */
+static MonoAssemblyBindingInfo*
+search_binding_loaded (MonoAssemblyName *aname)
+{
+	GSList *tmp;
+
+	for (tmp = loaded_assembly_bindings; tmp; tmp = tmp->next) {
+		MonoAssemblyBindingInfo *info = tmp->data;
+		if (assembly_binding_maps_name (info, aname))
+			return info;
+	}
+
+	return NULL;
+}
+
+static MonoAssemblyName*
+mono_assembly_apply_binding (MonoAssemblyName *aname, MonoAssemblyName *dest_name)
+{
+	MonoAssemblyBindingInfo *info, *info2;
+	MonoImage *ppimage;
+
+	if (aname->public_key_token [0] == 0)
+		return aname;
+
+	mono_loader_lock ();
+	info = search_binding_loaded (aname);
+	mono_loader_unlock ();
+	if (info) {
+		if (!check_policy_versions (info, aname))
+			return aname;
+		
+		mono_assembly_bind_version (info, aname, dest_name);
+		return dest_name;
+	}
+
+	info = g_new0 (MonoAssemblyBindingInfo, 1);
+	info->major = aname->major;
+	info->minor = aname->minor;
+	
+	ppimage = mono_assembly_load_publisher_policy (aname);
+	if (ppimage) {
+		get_publisher_policy_info (ppimage, aname, info);
+		mono_image_close (ppimage);
+	}
+
+	/* Define default error value if needed */
+	if (!info->is_valid) {
+		info->name = g_strdup (aname->name);
+		info->culture = g_strdup (aname->culture);
+		g_strlcpy ((char *)info->public_key_token, (const char *)aname->public_key_token, MONO_PUBLIC_KEY_TOKEN_LENGTH);
+	}
+	
+	mono_loader_lock ();
+	info2 = search_binding_loaded (aname);
+	if (info2) {
+		/* This binding was added by another thread 
+		 * before us */
+		mono_assembly_binding_info_free (info);
+		g_free (info);
+		
+		info = info2;
+	} else
+		loaded_assembly_bindings = g_slist_prepend (loaded_assembly_bindings, info);
+		
+	mono_loader_unlock ();
+	
+	if (!info->is_valid || !check_policy_versions (info, aname))
+		return aname;
+
+	mono_assembly_bind_version (info, aname, dest_name);
+	return dest_name;
+}
+
 /**
  * mono_assembly_load_from_gac
  *
@@ -1399,10 +1878,16 @@ mono_assembly_load_full (MonoAssemblyName *aname, const char *basedir, MonoImage
 {
 	MonoAssembly *result;
 	char *fullpath, *filename;
-	MonoAssemblyName maped_aname;
+	MonoAssemblyName maped_aname, maped_name_pp;
+	int ext_index;
+	const char *ext;
 
 	aname = mono_assembly_remap_version (aname, &maped_aname);
 	
+	/* Reflection only assemblies don't get assembly binding */
+	if (!refonly)
+		aname = mono_assembly_apply_binding (aname, &maped_name_pp);
+	
 	result = mono_assembly_loaded_full (aname, refonly);
 	if (result)
 		return result;
@@ -1420,32 +1905,41 @@ mono_assembly_load_full (MonoAssemblyName *aname, const char *basedir, MonoImage
 		return mono_assembly_load_corlib (mono_get_runtime_info (), status);
 	}
 
-	if (strstr (aname->name, ".dll"))
-		filename = g_strdup (aname->name);
-	else
-		filename = g_strconcat (aname->name, ".dll", NULL);
+	for (ext_index = 0; ext_index < 2; ext_index ++) {
+		ext = ext_index == 0 ? ".dll" : ".exe";
+		if (strstr (aname->name, ".dll"))
+			filename = g_strdup (aname->name);
+		else
+			filename = g_strconcat (aname->name, ext, NULL);
 
-	result = mono_assembly_load_from_gac (aname, filename, status, refonly);
-	if (result) {
-		g_free (filename);
-		return result;
-	}
-
-	if (basedir) {
-		fullpath = g_build_filename (basedir, filename, NULL);
-		result = mono_assembly_open_full (fullpath, status, refonly);
-		g_free (fullpath);
+		result = mono_assembly_load_from_gac (aname, filename, status, refonly);
 		if (result) {
-			result->in_gac = FALSE;
 			g_free (filename);
 			return result;
 		}
+
+		if (basedir) {
+			fullpath = g_build_filename (basedir, filename, NULL);
+			result = mono_assembly_open_full (fullpath, status, refonly);
+			g_free (fullpath);
+			if (result) {
+				result->in_gac = FALSE;
+				g_free (filename);
+				return result;
+			}
+		}
+
+		result = load_in_path (filename, default_path, status, refonly);
+		if (result)
+			result->in_gac = FALSE;
+		g_free (filename);
+		if (result)
+			return result;
 	}
 
-	result = load_in_path (filename, default_path, status, refonly);
-	if (result)
-		result->in_gac = FALSE;
-	g_free (filename);
+	/* Try a postload search hook */
+	result = mono_assembly_invoke_search_hook_internal (aname, refonly, TRUE);
+
 	return result;
 }
 
@@ -1463,9 +1957,9 @@ mono_assembly_loaded_full (MonoAssemblyName *aname, gboolean refonly)
 
 	aname = mono_assembly_remap_version (aname, &maped_aname);
 
-	EnterCriticalSection (&assemblies_mutex);
+	mono_assemblies_lock ();
 	res = search_loaded (aname, refonly);
-	LeaveCriticalSection (&assemblies_mutex);
+	mono_assemblies_unlock ();
 
 	return res;
 }
@@ -1476,23 +1970,24 @@ mono_assembly_loaded (MonoAssemblyName *aname)
 	return mono_assembly_loaded_full (aname, FALSE);
 }
 
+/**
+ * mono_assembly_close:
+ * @assembly: the assembly to release.
+ *
+ * This method releases a reference to the @assembly.  The assembly is
+ * only released when all the outstanding references to it are released.
+ */
 void
 mono_assembly_close (MonoAssembly *assembly)
 {
-	MonoImage *image;
-	
 	g_return_if_fail (assembly != NULL);
 
-	EnterCriticalSection (&assemblies_mutex);
-	/*g_print ("destroy assembly %p %d (%s)\n", assembly, assembly->ref_count, assembly->image->name);*/
-	g_assert (assembly->ref_count > 0);
-	if (--assembly->ref_count != 0) {
-		LeaveCriticalSection (&assemblies_mutex);
+	if (InterlockedDecrement (&assembly->ref_count))
 		return;
-	}
+	
+	mono_assemblies_lock ();
 	loaded_assemblies = g_list_remove (loaded_assemblies, assembly);
-	LeaveCriticalSection (&assemblies_mutex);
-	image = assembly->image;
+	mono_assemblies_unlock ();
 	/* assemblies belong to domains, so the domain code takes care of unloading the
 	 * referenced assemblies
 	 */
@@ -1526,9 +2021,9 @@ mono_assembly_foreach (GFunc func, gpointer user_data)
 	 * We make a copy of the list to avoid calling the callback inside the 
 	 * lock, which could lead to deadlocks.
 	 */
-	EnterCriticalSection (&assemblies_mutex);
+	mono_assemblies_lock ();
 	copy = g_list_copy (loaded_assemblies);
-	LeaveCriticalSection (&assemblies_mutex);
+	mono_assemblies_unlock ();
 
 	g_list_foreach (loaded_assemblies, func, user_data);
 
@@ -1553,6 +2048,8 @@ mono_assembly_get_main (void)
 	return(main_assembly);
 }
 
+/*
+ */
 MonoImage*
 mono_assembly_get_image (MonoAssembly *assembly)
 {
