@@ -30,6 +30,7 @@ MonoAssembly* mono_assembly_load_with_partial_name (const char *name, MonoImageO
 
 MonoAssembly* mono_assembly_loaded     (MonoAssemblyName *aname);
 MonoAssembly* mono_assembly_loaded_full (MonoAssemblyName *aname, gboolean refonly);
+void          mono_assembly_get_assemblyref (MonoImage *image, int index, MonoAssemblyName *aname);
 void          mono_assembly_load_reference (MonoImage *image, int index);
 void          mono_assembly_load_references (MonoImage *image, MonoImageOpenStatus *status);
 MonoImage*    mono_assembly_load_module (MonoAssembly *assembly, guint32 idx);
@@ -42,6 +43,7 @@ MonoAssembly *mono_assembly_get_main   (void);
 MonoImage    *mono_assembly_get_image  (MonoAssembly *assembly);
 gboolean      mono_assembly_fill_assembly_name (MonoImage *image, MonoAssemblyName *aname);
 gboolean      mono_assembly_names_equal (MonoAssemblyName *l, MonoAssemblyName *r);
+char*         mono_stringify_assembly_name (MonoAssemblyName *aname);
 
 /* Installs a function which is called each time a new assembly is loaded. */
 typedef void  (*MonoAssemblyLoadFunc)         (MonoAssembly *assembly, gpointer user_data);
@@ -57,6 +59,17 @@ void 	      mono_install_assembly_refonly_search_hook (MonoAssemblySearchFunc fu
 
 MonoAssembly* mono_assembly_invoke_search_hook (MonoAssemblyName *aname);
 
+/*
+ * Installs a new search function which is used as a last resort when loading 
+ * an assembly fails. This could invoke AssemblyResolve events.
+ */
+void          
+mono_install_assembly_postload_search_hook (MonoAssemblySearchFunc func, gpointer user_data);
+
+void          
+mono_install_assembly_postload_refonly_search_hook (MonoAssemblySearchFunc func, gpointer user_data);
+
+
 /* Installs a function which is called before a new assembly is loaded
  * The hook are invoked from last hooked to first. If any of them returns
  * a non-null value, that will be the value returned in mono_assembly_load */
@@ -80,6 +93,7 @@ typedef struct {
 void          mono_register_bundled_assemblies (const MonoBundledAssembly **assemblies);
 void          mono_register_config_for_assembly (const char* assembly_name, const char* config_xml);
 
+void          mono_set_dirs (const char *assembly_dir, const char *config_dir);
 G_END_DECLS
 #endif
 
