@@ -53,7 +53,7 @@ MonoType*
 mono_class_inflate_generic_type (MonoType *type, MonoGenericContext *context);
 
 MonoMethod*
-mono_class_inflate_generic_method (MonoMethod *method, MonoGenericContext *context, MonoClass *klass);
+mono_class_inflate_generic_method (MonoMethod *method, MonoGenericContext *context);
 
 MonoMethod *
 mono_get_inflated_method (MonoMethod *method);
@@ -122,9 +122,6 @@ mono_ldtoken               (MonoImage *image, guint32 token, MonoClass **retclas
 char*         
 mono_type_get_name         (MonoType *type);
 
-char*         
-mono_type_get_full_name    (MonoType *type);
-
 MonoType*
 mono_type_get_underlying_type (MonoType *type);
 
@@ -253,7 +250,7 @@ MonoMethod *
 mono_class_get_method_from_name (MonoClass *klass, const char *name, int param_count);
 
 char *
-mono_class_name_from_token (MonoImage *image, guint32 type_token, MonoGenericContext *context);
+mono_class_name_from_token (MonoImage *image, guint32 type_token);
 
 G_END_DECLS
 
