@@ -38,6 +38,7 @@ static const char *wrapper_type_names [] = {
 	"proxy_isinst",
 	"stelemref",
 	"unbox",
+	"ldflda",
 	"unknown"
 };
 
@@ -115,6 +116,10 @@ mono_type_get_desc (GString *res, MonoType *type, gboolean include_namespace) {
 	case MONO_TYPE_GENERICINST:
 		mono_type_get_desc (res, &type->data.generic_class->container_class->byval_arg, include_namespace);
 		break;
+	case MONO_TYPE_VAR:
+	case MONO_TYPE_MVAR:
+		g_string_append (res, type->data.generic_param->name);
+		break;
 	default:
 		break;
 	}
