@@ -62,6 +62,8 @@ static guint16 appdomain_list_size = 0;
 static guint16 appdomain_next = 0;
 static MonoDomain **appdomains_list = NULL;
 
+#define mono_appdomains_lock() EnterCriticalSection (&appdomains_mutex)
+#define mono_appdomains_unlock() LeaveCriticalSection (&appdomains_mutex)
 static CRITICAL_SECTION appdomains_mutex;
 
 static MonoDomain *mono_root_domain = NULL;
@@ -76,14 +78,30 @@ typedef struct {
 	int startup_count;
 } AppConfigInfo;
 
+/*
+ * AotModuleInfo: Contains information about AOT modules.
+ */
+typedef struct {
+	MonoImage *image;
+	gpointer start, end;
+} AotModuleInfo;
+
 static const MonoRuntimeInfo *current_runtime = NULL;
 
+static MonoJitInfoFindInAot jit_info_find_in_aot_func = NULL;
+
+/*
+ * Contains information about AOT loaded code.
+ */
+static MonoJitInfoTable *aot_modules = NULL;
+
 /* This is the list of runtime versions supported by this JIT.
  */
 static const MonoRuntimeInfo supported_runtimes[] = {
 	{"v1.0.3705", "1.0", { {1,0,5000,0}, {7,0,5000,0} }	},
 	{"v1.1.4322", "1.0", { {1,0,5000,0}, {7,0,5000,0} }	},
-	{"v2.0.50215","2.0", { {2,0,3600,0}, {8,0,3600,0} }	}
+	{"v2.0.50215","2.0", { {2,0,0,0},    {8,0,0,0} }	},
+	{"v2.0.50727","2.0", { {2,0,0,0},    {8,0,0,0} }	}
 };
 
 
@@ -96,6 +114,9 @@ get_runtimes_from_exe (const char *exe_file, const MonoRuntimeInfo** runtimes);
 static const MonoRuntimeInfo*
 get_runtime_by_version (const char *version);
 
+static MonoImage*
+mono_jit_info_find_aot_module (guint8* addr);
+
 guint32
 mono_domain_get_tls_key (void)
 {
@@ -150,6 +171,7 @@ MonoJitInfo *
 mono_jit_info_table_find (MonoDomain *domain, char *addr)
 {
 	MonoJitInfoTable *table = domain->jit_info_table;
+	MonoJitInfo *ji;
 	guint left = 0, right;
 
 	mono_domain_lock (domain);
@@ -157,7 +179,7 @@ mono_jit_info_table_find (MonoDomain *domain, char *addr)
 	right = table->len;
 	while (left < right) {
 		guint pos = (left + right) / 2;
-		MonoJitInfo *ji = g_array_index (table, gpointer, pos);
+		ji = g_array_index (table, gpointer, pos);
 
 		if (addr < (char*)ji->code_start)
 			right = pos;
@@ -171,10 +193,18 @@ mono_jit_info_table_find (MonoDomain *domain, char *addr)
 	mono_domain_unlock (domain);
 
 	/* maybe it is shared code, so we also search in the root domain */
+	ji = NULL;
 	if (domain != mono_root_domain)
-		return mono_jit_info_table_find (mono_root_domain, addr);
+		ji = mono_jit_info_table_find (mono_root_domain, addr);
 
-	return NULL;
+	if (ji == NULL) {
+		/* Maybe its an AOT module */
+		MonoImage *image = mono_jit_info_find_aot_module (addr);
+		if (image)
+			ji = jit_info_find_in_aot_func (domain, image, addr);
+	}
+	
+	return ji;
 }
 
 void
@@ -200,12 +230,96 @@ mono_jit_info_table_remove (MonoDomain *domain, MonoJitInfo *ji)
 
 	mono_domain_lock (domain);
 	pos = mono_jit_info_table_index (table, start);
+	if (g_array_index (table, gpointer, pos) != ji) {
+		MonoJitInfo *ji2 = g_array_index (table, gpointer, pos);
+		g_assert (ji == ji2);
+	}
 	g_assert (g_array_index (table, gpointer, pos) == ji);
 
 	g_array_remove_index (table, pos);
 	mono_domain_unlock (domain);
 }	
 
+static int
+aot_info_table_index (MonoJitInfoTable *table, char *addr)
+{
+	int left = 0, right = table->len;
+
+	while (left < right) {
+		int pos = (left + right) / 2;
+		AotModuleInfo *ainfo = g_array_index (table, gpointer, pos);
+		char *start = ainfo->start;
+		char *end = ainfo->end;
+
+		if (addr < start)
+			right = pos;
+		else if (addr >= end) 
+			left = pos + 1;
+		else
+			return pos;
+	}
+
+	return left;
+}
+
+void
+mono_jit_info_add_aot_module (MonoImage *image, gpointer start, gpointer end)
+{
+	AotModuleInfo *ainfo = g_new0 (AotModuleInfo, 1);
+	int pos;
+
+	ainfo->image = image;
+	ainfo->start = start;
+	ainfo->end = end;
+
+	mono_appdomains_lock ();
+
+	if (!aot_modules)
+		aot_modules = mono_jit_info_table_new ();
+
+	pos = aot_info_table_index (aot_modules, start);
+
+	g_array_insert_val (aot_modules, pos, ainfo);
+
+	mono_appdomains_unlock ();
+}
+
+static MonoImage*
+mono_jit_info_find_aot_module (guint8* addr)
+{
+	guint left = 0, right;
+
+	if (!aot_modules)
+		return NULL;
+
+	mono_appdomains_lock ();
+
+	right = aot_modules->len;
+	while (left < right) {
+		guint pos = (left + right) / 2;
+		AotModuleInfo *ai = g_array_index (aot_modules, gpointer, pos);
+
+		if (addr < (guint8*)ai->start)
+			right = pos;
+		else if (addr >= (guint8*)ai->end)
+			left = pos + 1;
+		else {
+			mono_appdomains_unlock ();
+			return ai->image;
+		}
+	}
+
+	mono_appdomains_unlock ();
+
+	return NULL;
+}
+
+void
+mono_install_jit_info_find_in_aot (MonoJitInfoFindInAot func)
+{
+	jit_info_find_in_aot_func = func;
+}
+
 gboolean
 mono_string_equal (MonoString *s1, MonoString *s2)
 {
@@ -235,7 +349,7 @@ mono_string_hash (MonoString *s)
 	return h;	
 }
 
-gboolean
+static gboolean
 mono_ptrarray_equal (gpointer *s1, gpointer *s2)
 {
 	int len = GPOINTER_TO_INT (s1 [0]);
@@ -245,7 +359,7 @@ mono_ptrarray_equal (gpointer *s1, gpointer *s2)
 	return memcmp (s1 + 1, s2 + 1, len * sizeof(gpointer)) == 0; 
 }
 
-guint
+static guint
 mono_ptrarray_hash (gpointer *s)
 {
 	int i;
@@ -333,13 +447,14 @@ mono_domain_create (void)
 	domain->jump_trampoline_hash = g_hash_table_new (mono_aligned_addr_hash, NULL);
 	domain->finalizable_objects_hash = g_hash_table_new (mono_aligned_addr_hash, NULL);
 	domain->jit_trampoline_hash = g_hash_table_new (mono_aligned_addr_hash, NULL);
+	domain->delegate_trampoline_hash = g_hash_table_new (mono_aligned_addr_hash, NULL);
 
 	InitializeCriticalSection (&domain->lock);
 	InitializeCriticalSection (&domain->assemblies_lock);
 
-	EnterCriticalSection (&appdomains_mutex);
+	mono_appdomains_lock ();
 	domain_id_alloc (domain);
-	LeaveCriticalSection (&appdomains_mutex);
+	mono_appdomains_unlock ();
 
 	return domain;
 }
@@ -642,6 +757,16 @@ mono_init_internal (const char *filename, const char *exe_filename, const char *
 	mono_defaults.executioncontext_class = mono_class_from_name (
 	        mono_defaults.corlib, "System.Threading", "ExecutionContext");
 
+	/*
+	 * Note that mono_defaults.generic_*_class is only non-NULL if we're
+	 * using the 2.0 corlib.
+	 */
+	mono_class_init (mono_defaults.array_class);
+	mono_defaults.generic_array_class = mono_class_from_name (
+	        mono_defaults.corlib, "System", "Array/InternalArray`1");
+	mono_defaults.generic_nullable_class = mono_class_from_name (
+		mono_defaults.corlib, "System", "Nullable`1");
+
 	domain->friendly_name = g_path_get_basename (filename);
 
 	return domain;
@@ -738,11 +863,11 @@ mono_domain_foreach (MonoDomainFunc func, gpointer user_data)
 	 * inside the lock because that could lead to deadlocks.
 	 * We can do this because this function is not perf. critical.
 	 */
-	EnterCriticalSection (&appdomains_mutex);
+	mono_appdomains_lock ();
 	size = appdomain_list_size;
 	copy = mono_gc_alloc_fixed (appdomain_list_size * sizeof (void*), NULL);
 	memcpy (copy, appdomains_list, appdomain_list_size * sizeof (void*));
-	LeaveCriticalSection (&appdomains_mutex);
+	mono_appdomains_unlock ();
 
 	for (i = 0; i < size; ++i) {
 		if (copy [i])
@@ -804,9 +929,9 @@ mono_domain_free (MonoDomain *domain, gboolean force)
 		return;
 	}
 
-	EnterCriticalSection (&appdomains_mutex);
+	mono_appdomains_lock ();
 	appdomains_list [domain->domain_id] = NULL;
-	LeaveCriticalSection (&appdomains_mutex);
+	mono_appdomains_unlock ();
 
 	/* FIXME: free delegate_hash_table when it's used */
 	if (domain->search_path) {
@@ -879,6 +1004,8 @@ mono_domain_free (MonoDomain *domain, gboolean force)
 	domain->finalizable_objects_hash = NULL;
 	g_hash_table_destroy (domain->jit_trampoline_hash);
 	domain->jit_trampoline_hash = NULL;
+	g_hash_table_destroy (domain->delegate_trampoline_hash);
+	domain->delegate_trampoline_hash = NULL;
 	if (domain->special_static_fields) {
 		g_hash_table_destroy (domain->special_static_fields);
 		domain->special_static_fields = NULL;
@@ -905,12 +1032,12 @@ mono_domain_get_by_id (gint32 domainid)
 {
 	MonoDomain * domain;
 
-	EnterCriticalSection (&appdomains_mutex);
+	mono_appdomains_lock ();
 	if (domainid < appdomain_list_size)
 		domain = appdomains_list [domainid];
 	else
 		domain = NULL;
-	LeaveCriticalSection (&appdomains_mutex);
+	mono_appdomains_unlock ();
 
 	return domain;
 }
