@@ -43,10 +43,142 @@ MonoDefaults mono_defaults;
  */
 static CRITICAL_SECTION loader_mutex;
 
+
+/*
+ * This TLS variable contains the last type load error encountered by the loader.
+ */
+guint32 loader_error_thread_id;
+
 void
 mono_loader_init ()
 {
 	InitializeCriticalSection (&loader_mutex);
+
+	loader_error_thread_id = TlsAlloc ();
+}
+
+/*
+ * Handling of type load errors should be done as follows:
+ *
+ *   If something could not be loaded, the loader should call one of the
+ * mono_loader_set_error_XXX functions ()
+ * with the appropriate arguments, then return NULL to report the failure. The error 
+ * should be propagated until it reaches code which can throw managed exceptions. At that
+ * point, an exception should be thrown based on the information returned by
+ * mono_loader_get_error (). Then the error should be cleared by calling 
+ * mono_loader_clear_error ().
+ */
+
+static void
+set_loader_error (MonoLoaderError *error)
+{
+	TlsSetValue (loader_error_thread_id, error);
+}	
+
+/*
+ * mono_loader_set_error_type_load:
+ *
+ *   Set the loader error for this thread. CLASS_NAME and ASSEMBLY_NAME should be
+ * dynamically allocated strings whose ownership is passed to this function.
+ */
+void
+mono_loader_set_error_type_load (char *class_name, char *assembly_name)
+{
+	MonoLoaderError *error;
+
+	if (mono_loader_get_last_error ()) {
+		g_free (class_name);
+		g_free (assembly_name);
+		return;
+	}
+
+	error = g_new0 (MonoLoaderError, 1);
+	error->kind = MONO_LOADER_ERROR_TYPE;
+	error->class_name = class_name;
+	error->assembly_name = assembly_name;
+
+	/* 
+	 * This is not strictly needed, but some (most) of the loader code still
+	 * can't deal with load errors, and this message is more helpful than an
+	 * assert.
+	 */
+	g_warning ("The class %s could not be loaded, used in %s", class_name, assembly_name);
+	
+	set_loader_error (error);
+}
+
+/*
+ * mono_loader_set_error_method_load:
+ *
+ *   Set the loader error for this thread. MEMBER_NAME should point to a string
+ * inside metadata.
+ */
+void
+mono_loader_set_error_method_load (MonoClass *klass, const char *member_name)
+{
+	MonoLoaderError *error;
+
+	/* FIXME: Store the signature as well */
+	if (mono_loader_get_last_error ())
+		return;
+
+	error = g_new0 (MonoLoaderError, 1);
+	error->kind = MONO_LOADER_ERROR_METHOD;
+	error->klass = klass;
+	error->member_name = member_name;
+
+	g_warning ("Missing member %s in type %s, assembly %s", member_name, mono_class_get_name (klass), klass->image->name);
+	
+	set_loader_error (error);
+}
+
+/*
+ * mono_loader_set_error_field_load:
+ *
+ *   Set the loader error for this thread. MEMBER_NAME should point to a string
+ * inside metadata.
+ */
+void
+mono_loader_set_error_field_load (MonoClass *klass, const char *member_name)
+{
+	MonoLoaderError *error;
+
+	/* FIXME: Store the signature as well */
+	if (mono_loader_get_last_error ())
+		return;
+
+	error = g_new0 (MonoLoaderError, 1);
+	error->kind = MONO_LOADER_ERROR_FIELD;
+	error->klass = klass;
+	error->member_name = member_name;
+	
+	set_loader_error (error);
+}
+
+/*
+ * mono_loader_get_last_error:
+ *
+ *   Returns information about the last type load exception encountered by the loader, or
+ * NULL. After use, the exception should be cleared by calling mono_loader_clear_error.
+ */
+MonoLoaderError*
+mono_loader_get_last_error (void)
+{
+	return (MonoLoaderError*)TlsGetValue (loader_error_thread_id);
+}
+
+void
+mono_loader_clear_error (void)
+{
+	MonoLoaderError *ex = (MonoLoaderError*)TlsGetValue (loader_error_thread_id);
+
+	g_assert (ex);	
+
+	g_free (ex->class_name);
+	g_free (ex->assembly_name);
+	g_free (ex);
+
+	TlsSetValue (loader_error_thread_id, NULL);
 }
 
 static MonoClassField*
@@ -54,6 +186,7 @@ field_from_memberref (MonoImage *image, guint32 token, MonoClass **retklass,
 		      MonoGenericContext *context)
 {
 	MonoClass *klass;
+	MonoClassField *field;
 	MonoTableInfo *tables = image->tables;
 	guint32 cols[6];
 	guint32 nindex, class;
@@ -81,7 +214,7 @@ field_from_memberref (MonoImage *image, guint32 token, MonoClass **retklass,
 	case MONO_MEMBERREF_PARENT_TYPEREF:
 		klass = mono_class_from_typeref (image, MONO_TOKEN_TYPE_REF | nindex);
 		if (!klass) {
-			char *name = mono_class_name_from_token (image, MONO_TOKEN_TYPE_REF | nindex, context);
+			char *name = mono_class_name_from_token (image, MONO_TOKEN_TYPE_REF | nindex);
 			g_warning ("Missing field %s in class %s (typeref index %d)", fname, name, nindex);
 			g_free (name);
 			return NULL;
@@ -89,7 +222,8 @@ field_from_memberref (MonoImage *image, guint32 token, MonoClass **retklass,
 		mono_class_init (klass);
 		if (retklass)
 			*retklass = klass;
-		return mono_class_get_field_from_name (klass, fname);
+		field = mono_class_get_field_from_name (klass, fname);
+		break;
 	case MONO_MEMBERREF_PARENT_TYPESPEC: {
 		/*guint32 bcols [MONO_TYPESPEC_SIZE];
 		guint32 len;
@@ -108,12 +242,18 @@ field_from_memberref (MonoImage *image, guint32 token, MonoClass **retklass,
 		mono_class_init (klass);
 		if (retklass)
 			*retklass = klass;
-		return mono_class_get_field_from_name (klass, fname);
+		field = mono_class_get_field_from_name (klass, fname);
+		break;
 	}
 	default:
 		g_warning ("field load from %x", class);
 		return NULL;
 	}
+
+	if (!field)
+		mono_loader_set_error_field_load (klass, fname);
+
+	return field;
 }
 
 MonoClassField*
@@ -154,7 +294,7 @@ mono_field_from_token (MonoImage *image, guint32 token, MonoClass **retklass,
 	}
 
 	mono_loader_lock ();
-	if (!field->parent->generic_class)
+	if (field && !field->parent->generic_class)
 		g_hash_table_insert (image->field_cache, GUINT_TO_POINTER (token), field);
 	mono_loader_unlock ();
 	return field;
@@ -185,15 +325,43 @@ mono_metadata_signature_vararg_match (MonoMethodSignature *sig1, MonoMethodSigna
 	return TRUE;
 }
 
+static MonoMethod *
+find_method_in_class (MonoClass *klass, const char *name, const char *qname, const char *fqname, MonoMethodSignature *sig)
+{
+	int i;
+
+	mono_class_setup_methods (klass);
+	for (i = 0; i < klass->method.count; ++i) {
+		MonoMethod *m = klass->methods [i];
+
+		if (!((fqname && !strcmp (m->name, fqname)) ||
+		      (qname && !strcmp (m->name, qname)) || !strcmp (m->name, name)))
+			continue;
+
+		if (sig->call_convention == MONO_CALL_VARARG) {
+			if (mono_metadata_signature_vararg_match (sig, mono_method_signature (m)))
+				return m;
+		} else {
+			MonoMethodSignature *msig = mono_method_signature (m);
+			if (mono_metadata_signature_equal (sig, msig))
+				return m;
+		}
+	}
+	return NULL;
+}
+
 static MonoMethod *
 find_method (MonoClass *klass, MonoClass *ic, const char* name, MonoMethodSignature *sig)
 {
 	int i;
 	char *qname, *fqname, *class_name;
+	gboolean is_interface;
 	MonoMethod *result = NULL;
 
+	is_interface = MONO_CLASS_IS_INTERFACE (klass);
+
 	if (ic) {
-		class_name = mono_class_get_name_full (ic, FALSE, FALSE, FALSE);
+		class_name = mono_type_get_name_full (&ic->byval_arg, MONO_TYPE_NAME_FORMAT_IL);
 
 		qname = g_strconcat (class_name, ".", name, NULL); 
 		if (ic->name_space && ic->name_space [0])
@@ -204,41 +372,27 @@ find_method (MonoClass *klass, MonoClass *ic, const char* name, MonoMethodSignat
 		class_name = qname = fqname = NULL;
 
 	while (klass) {
-		MonoGenericContext *context = NULL;
+		result = find_method_in_class (klass, name, qname, fqname, sig);
+		if (result)
+			goto out;
 
-		if (klass->generic_container)
-			context = &klass->generic_container->context;
-		else if (klass->generic_class)
-			context = klass->generic_class->context;
-
-		mono_class_setup_methods (klass);
-		for (i = 0; i < klass->method.count; ++i) {
-			MonoMethod *m = klass->methods [i];
-
-			if (!((fqname && !strcmp (m->name, fqname)) ||
-			      (qname && !strcmp (m->name, qname)) || !strcmp (m->name, name)))
-				continue;
-
-			if (sig->call_convention == MONO_CALL_VARARG) {
-				if (mono_metadata_signature_vararg_match (sig, mono_method_signature (m))) {
-					result = m;
-					goto out;
-				}
-			} else {
-				MonoMethodSignature *msig = mono_method_signature_full (m, context);
-				if (mono_metadata_signature_equal (sig, msig)) {
-					result = m;
-					goto out;
-				}
-			}
-		}
-
-		if (name [0] == '.' && (strcmp (name, ".ctor") == 0 || strcmp (name, ".cctor") == 0))
+		if (name [0] == '.' && (!strcmp (name, ".ctor") || !strcmp (name, ".cctor")))
 			break;
 
+		for (i = 0; i < klass->interface_count; i++) {
+			MonoClass *ic = klass->interfaces [i];
+
+			result = find_method_in_class (ic, name, qname, fqname, sig);
+			if (result)
+				goto out;
+		}
+
 		klass = klass->parent;
 	}
 
+	if (is_interface)
+		result = find_method_in_class (mono_defaults.object_class, name, qname, fqname, sig);
+
  out:
 	g_free (class_name);
 	g_free (fqname);
@@ -286,7 +440,8 @@ mono_method_get_signature_full (MonoMethod *method, MonoImage *image, guint32 to
 	
 		ptr = mono_metadata_blob_heap (image, cols [MONO_MEMBERREF_SIGNATURE]);
 		mono_metadata_decode_blob_size (ptr, &ptr);
-		sig = mono_metadata_parse_method_signature_full (image, context, 0, ptr, NULL);
+		sig = mono_metadata_parse_method_signature_full (
+			image, context ? context->container : NULL, 0, ptr, NULL);
 
 		mono_loader_lock ();
 		prev_sig = g_hash_table_lookup (image->memberref_signatures, GUINT_TO_POINTER (token));
@@ -312,15 +467,14 @@ mono_method_get_signature (MonoMethod *method, MonoImage *image, guint32 token)
 }
 
 static MonoMethod *
-method_from_memberref (MonoImage *image, guint32 idx, MonoGenericContext *context)
+method_from_memberref (MonoImage *image, guint32 idx, MonoGenericContext *typespec_context)
 {
+	MonoGenericContext *context = NULL;
 	MonoClass *klass = NULL;
 	MonoMethod *method = NULL;
 	MonoTableInfo *tables = image->tables;
 	guint32 cols[6];
 	guint32 nindex, class;
-	MonoGenericClass *gclass = NULL;
-	MonoGenericContainer *container = NULL;
 	const char *mname;
 	MonoMethodSignature *sig;
 	const char *ptr;
@@ -337,16 +491,19 @@ method_from_memberref (MonoImage *image, guint32 idx, MonoGenericContext *contex
 	case MONO_MEMBERREF_PARENT_TYPEREF:
 		klass = mono_class_from_typeref (image, MONO_TOKEN_TYPE_REF | nindex);
 		if (!klass) {
-			char *name = mono_class_name_from_token (image, MONO_TOKEN_TYPE_REF | nindex, context);
+			char *name = mono_class_name_from_token (image, MONO_TOKEN_TYPE_REF | nindex);
 			g_warning ("Missing method %s in assembly %s, type %s", mname, image->name, name);
 			g_free (name);
 			return NULL;
 		}
 		break;
 	case MONO_MEMBERREF_PARENT_TYPESPEC:
-		klass = mono_class_get_full (image, MONO_TOKEN_TYPE_SPEC | nindex, context);
+		/*
+		 * Parse the TYPESPEC in the parent's context.
+		 */
+		klass = mono_class_get_full (image, MONO_TOKEN_TYPE_SPEC | nindex, typespec_context);
 		if (!klass) {
-			char *name = mono_class_name_from_token (image, MONO_TOKEN_TYPE_SPEC | nindex, context);
+			char *name = mono_class_name_from_token (image, MONO_TOKEN_TYPE_SPEC | nindex);
 			g_warning ("Missing method %s in assembly %s, type %s", mname, image->name, name);
 			g_free (name);
 			return NULL;
@@ -355,7 +512,7 @@ method_from_memberref (MonoImage *image, guint32 idx, MonoGenericContext *contex
 	case MONO_MEMBERREF_PARENT_TYPEDEF:
 		klass = mono_class_get (image, MONO_TOKEN_TYPE_DEF | nindex);
 		if (!klass) {
-			char *name = mono_class_name_from_token (image, MONO_TOKEN_TYPE_DEF | nindex, context);
+			char *name = mono_class_name_from_token (image, MONO_TOKEN_TYPE_DEF | nindex);
 			g_warning ("Missing method %s in assembly %s, type %s", mname, image->name, name);
 			g_free (name);
 			return NULL;
@@ -368,24 +525,27 @@ method_from_memberref (MonoImage *image, guint32 idx, MonoGenericContext *contex
 		g_assert_not_reached ();
 	}
 	g_assert (klass);
-
-	if (klass->generic_class) {
-		gclass = klass->generic_class;
-		klass = gclass->container_class;
-	}
-	if (klass->generic_container)
-		container = klass->generic_container;
 	mono_class_init (klass);
 
 	ptr = mono_metadata_blob_heap (image, cols [MONO_MEMBERREF_SIGNATURE]);
 	mono_metadata_decode_blob_size (ptr, &ptr);
-	sig = mono_metadata_parse_method_signature_full (image, (MonoGenericContext *) container, 0, ptr, NULL);
+
+	sig = mono_metadata_parse_method_signature (image, 0, ptr, NULL);
+
+	/* If the class is a generic instantiation, ensure that all VAR references in the signature
+	   are replaced by the corresponding type argument.  Note that MVARs are left alone. */
+	if (klass->generic_class) {
+		MonoMethodSignature *old_sig = sig;
+		sig = mono_class_inflate_generic_signature (image, sig, klass->generic_class->context);
+		g_assert (sig != old_sig);
+		mono_metadata_free_method_signature (old_sig);
+	}
 
 	switch (class) {
 	case MONO_MEMBERREF_PARENT_TYPEREF:
 		method = find_method (klass, NULL, mname, sig);
 		if (!method)
-			g_warning ("Missing method %s in assembly %s, type %s", mname, image->name, mono_class_get_name (klass));
+			mono_loader_set_error_method_load (klass, mname);
 		mono_metadata_free_method_signature (sig);
 		break;
 	case MONO_MEMBERREF_PARENT_TYPESPEC: {
@@ -400,13 +560,13 @@ method_from_memberref (MonoImage *image, guint32 idx, MonoGenericContext *contex
 				g_warning ("Missing method %s in assembly %s, type %s", mname, image->name, mono_class_get_name (klass));
 			else if (klass->generic_class && (klass != method->klass))
 				method = mono_class_inflate_generic_method (
-					method, klass->generic_class->context, klass);
+					method, klass->generic_class->context);
 			mono_metadata_free_method_signature (sig);
 			break;
 		}
 
 		result = (MonoMethod *)g_new0 (MonoMethodPInvoke, 1);
-		result->klass = mono_class_get (image, MONO_TOKEN_TYPE_SPEC | nindex);
+		result->klass = klass;
 		result->iflags = METHOD_IMPL_ATTRIBUTE_INTERNAL_CALL;
 		result->signature = sig;
 		result->name = mname;
@@ -443,7 +603,9 @@ method_from_memberref (MonoImage *image, guint32 idx, MonoGenericContext *contex
 	case MONO_MEMBERREF_PARENT_TYPEDEF:
 		method = find_method (klass, NULL, mname, sig);
 		if (!method)
-			g_warning ("Missing method %s in assembly %s, type %s", mname, image->name, mono_class_get_name (klass));
+			g_warning (
+				"Missing method %s::%s(%s) in assembly %s", 
+				mono_type_get_name (&klass->byval_arg), mname, mono_signature_get_desc (sig, FALSE), image->name);
 		mono_metadata_free_method_signature (sig);
 		break;
 	default:
@@ -451,9 +613,6 @@ method_from_memberref (MonoImage *image, guint32 idx, MonoGenericContext *contex
 		g_assert_not_reached ();
 	}
 
-	if (gclass)
-		method = mono_class_inflate_generic_method (method, gclass->context, gclass->klass);
-
 	return method;
 }
 
@@ -467,42 +626,87 @@ method_from_methodspec (MonoImage *image, MonoGenericContext *context, guint32 i
 	MonoGenericContainer *container = NULL;
 	const char *ptr;
 	guint32 cols [MONO_METHODSPEC_SIZE];
-	guint32 token, param_count;
+	guint32 token, nindex, param_count;
 
 	mono_metadata_decode_row (&tables [MONO_TABLE_METHODSPEC], idx - 1, cols, MONO_METHODSPEC_SIZE);
 	token = cols [MONO_METHODSPEC_METHOD];
-	if ((token & MONO_METHODDEFORREF_MASK) == MONO_METHODDEFORREF_METHODDEF)
-		token = MONO_TOKEN_METHOD_DEF | (token >> MONO_METHODDEFORREF_BITS);
-	else
-		token = MONO_TOKEN_MEMBER_REF | (token >> MONO_METHODDEFORREF_BITS);
-
-	method = mono_get_method (image, token, NULL);
+	nindex = token >> MONO_METHODDEFORREF_BITS;
 
 	ptr = mono_metadata_blob_heap (image, cols [MONO_METHODSPEC_SIGNATURE]);
 	
 	mono_metadata_decode_value (ptr, &ptr);
 	ptr++;
 	param_count = mono_metadata_decode_value (ptr, &ptr);
-
 	g_assert (param_count);
-	if (method->is_inflated)
-		container = ((MonoMethodNormal *) ((MonoMethodInflated *) method)->declaring)->generic_container;
-	else
-		container = ((MonoMethodNormal *) method)->generic_container;
-	g_assert (container && container->is_method);
 
-	if (context) {
-		g_assert (context->container);
-		container->parent = context->container;
-		if (container->parent->is_method)
-			container->parent = container->parent->parent;
-	}
+	/*
+	 * Be careful with the two contexts here:
+	 *
+	 * ----------------------------------------
+	 * class Foo<S> {
+	 *   static void Hello<T> (S s, T t) { }
+	 *
+	 *   static void Test<U> (U u) {
+	 *     Foo<U>.Hello<string> (u, "World");
+	 *   }
+	 * }
+	 * ----------------------------------------
+	 *
+	 * Let's assume we're currently JITing Foo<int>.Test<long>
+	 * (ie. `S' is instantiated as `int' and `U' is instantiated as `long').
+	 *
+	 * The call to Hello() is encoded with a MethodSpec with a TypeSpec as parent
+	 * (MONO_MEMBERREF_PARENT_TYPESPEC).
+	 *
+	 * The TypeSpec is encoded as `Foo<!!0>', so we need to parse it in the current
+	 * context (S=int, U=long) to get the correct `Foo<long>'.
+	 * 
+	 * After that, we parse the memberref signature in the new context
+	 * (S=int, T=uninstantiated) and get the open generic method `Foo<long>.Hello<T>'.
+	 *
+	 */
+	if ((token & MONO_METHODDEFORREF_MASK) == MONO_METHODDEFORREF_METHODDEF)
+		method = mono_get_method_full (image, MONO_TOKEN_METHOD_DEF | nindex, NULL, context);
+	else
+		method = method_from_memberref (image, nindex, context);
+
+	method = mono_get_inflated_method (method);
+
+	container = method->generic_container;
+	g_assert (container);
 
 	gmethod = g_new0 (MonoGenericMethod, 1);
+	gmethod->generic_class = method->klass->generic_class;
 	gmethod->container = container;
 
-	gmethod->inst = mono_metadata_parse_generic_inst (
-		image, (MonoGenericContext *) container, param_count, ptr, &ptr);
+	new_context = g_new0 (MonoGenericContext, 1);
+	new_context->container = container;
+	new_context->gmethod = gmethod;
+	if (container->parent)
+		new_context->gclass = container->parent->context.gclass;
+
+	/*
+	 * When parsing the methodspec signature, we're in the old context again:
+	 *
+	 * ----------------------------------------
+	 * class Foo {
+	 *   static void Hello<T> (T t) { }
+	 *
+	 *   static void Test<U> (U u) {
+	 *     Foo.Hello<U> (u);
+	 *   }
+	 * }
+	 * ----------------------------------------
+	 *
+	 * Let's assume we're currently JITing "Foo.Test<float>".
+	 *
+	 * In this case, we already parsed the memberref as "Foo.Hello<T>" and the methodspec
+	 * signature is "<!!0>".  This means that we must instantiate the method type parameter
+	 * `T' from the new method with the method type parameter `U' from the current context;
+	 * ie. instantiate the method as `Foo.Hello<float>.
+	 */
+
+	gmethod->inst = mono_metadata_parse_generic_inst (image, context, param_count, ptr, &ptr);
 
 	if (context)
 		gmethod->inst = mono_metadata_inflate_generic_inst (gmethod->inst, context);
@@ -514,32 +718,19 @@ method_from_methodspec (MonoImage *image, MonoGenericContext *context, guint32 i
 	inflated = g_hash_table_lookup (container->method_hash, gmethod);
 	if (inflated) {
 		g_free (gmethod);
+		g_free (new_context);
 		return inflated;
 	}
 
-	if (!context) {
-		new_context = g_new0 (MonoGenericContext, 1);
-		new_context->container = container;
-		new_context->gmethod = gmethod;
-
-		context = new_context;
-	} else {
-		new_context = g_new0 (MonoGenericContext, 1);
-		new_context->container = container;
-		new_context->gmethod = gmethod;
-		new_context->gclass = context->gclass;
-
-		context = new_context;
-	}
+	context = new_context;
 
 	mono_stats.generics_metadata_size += sizeof (MonoGenericMethod) +
 		sizeof (MonoGenericContext) + param_count * sizeof (MonoType);
 
-	inflated = mono_class_inflate_generic_method (method, context, NULL);
+	inflated = mono_class_inflate_generic_method (method, new_context);
 	g_hash_table_insert (container->method_hash, gmethod, inflated);
 
-	if (new_context)
-		context->gclass = inflated->klass->generic_class;
+	new_context->gclass = inflated->klass->generic_class;
 	return inflated;
 }
 
@@ -698,7 +889,7 @@ mono_lookup_pinvoke_call (MonoMethod *method, const char **exc_class, const char
 	/*
 	 * Try loading the module using a variety of names
 	 */
-	for (i = 0; i < 3; ++i) {
+	for (i = 0; i < 4; ++i) {
 		switch (i) {
 		case 0:
 			/* Try the original name */
@@ -713,13 +904,26 @@ mono_lookup_pinvoke_call (MonoMethod *method, const char **exc_class, const char
 			else
 				continue;
 			break;
-		default:
+		case 2:
 			if (strstr (new_scope, "lib") != new_scope) {
 				file_name = g_strdup_printf ("lib%s", new_scope);
 			}
 			else
 				continue;
 			break;
+		default:
+#ifndef PLATFORM_WIN32
+			if (!g_ascii_strcasecmp ("user32.dll", new_scope) ||
+			    !g_ascii_strcasecmp ("kernel32.dll", new_scope) ||
+			    !g_ascii_strcasecmp ("user32", new_scope) ||
+			    !g_ascii_strcasecmp ("kernel", new_scope)) {
+				file_name = g_strdup ("libMonoSupportW.so");
+			} else
+#endif
+				    continue;
+#ifndef PLATFORM_WIN32
+			break;
+#endif
 		}
 
 		if (!gmodule) {
@@ -892,11 +1096,6 @@ mono_get_method_from_token (MonoImage *image, guint32 token, MonoClass *klass,
 		return mono_lookup_dynamic_token (image, token);
 
 	if (table != MONO_TABLE_METHOD) {
-		MonoGenericContainer *generic_container = NULL;
-		if (context) {
-			g_assert (context->container);
-			generic_container = context->container;
-		}
 		if (table == MONO_TABLE_METHODSPEC)
 			return method_from_methodspec (image, context, idx);
 		if (table != MONO_TABLE_MEMBERREF)
@@ -914,6 +1113,11 @@ mono_get_method_from_token (MonoImage *image, guint32 token, MonoClass *klass,
 		result = (MonoMethod *)g_new0 (MonoMethodPInvoke, 1);
 	else 
 		result = (MonoMethod *)g_new0 (MonoMethodNormal, 1);
+
+	if (!klass) {
+		guint32 type = mono_metadata_typedef_from_method (image, token);
+		klass = mono_class_get (image, MONO_TOKEN_TYPE_DEF | type);
+	}
 	
 	result->slot = -1;
 	result->klass = klass;
@@ -922,14 +1126,19 @@ mono_get_method_from_token (MonoImage *image, guint32 token, MonoClass *klass,
 	result->token = token;
 	result->name = mono_metadata_string_heap (image, cols [3]);
 
-	if (klass)
-		container = klass->generic_container;
+	container = klass->generic_container;
+	generic_container = mono_metadata_load_generic_params (image, token, container);
+	if (generic_container) {
+		mono_metadata_load_generic_param_constraints (image, token, generic_container);
 
-	if (!(cols [1] & METHOD_IMPL_ATTRIBUTE_INTERNAL_CALL) &&
-	    (!(cols [2] & METHOD_ATTRIBUTE_PINVOKE_IMPL) || cols [1] & METHOD_IMPL_ATTRIBUTE_NATIVE)) {
-		generic_container = mono_metadata_load_generic_params (image, token, container);
-		if (generic_container)
-			container = generic_container;
+		for (i = 0; i < generic_container->type_argc; i++) {
+			generic_container->type_params [i].method = result;
+
+			mono_class_from_generic_parameter (
+				&generic_container->type_params [i], image, TRUE);
+		}
+
+		container = generic_container;
 	}
 
 	if (!sig) /* already taken from the methodref */
@@ -937,14 +1146,22 @@ mono_get_method_from_token (MonoImage *image, guint32 token, MonoClass *klass,
 	size = mono_metadata_decode_blob_size (sig, &sig);
 	
 	/* there are generic params, or a container. FIXME: be lazy here for generics*/
-	if (* sig & 0x10 || container)
+	if (* sig & 0x10 || container) {
 		result->signature = mono_metadata_parse_method_signature_full (
-		    image, (MonoGenericContext *) container, idx, sig, NULL);
+			image, container, idx, sig, NULL);
 
+		/* Verify metadata consistency */
+		if (result->signature->generic_param_count) {
+			if (!container || !container->is_method)
+				g_error ("Signature claims method has generic parameters, but generic_params table says it doesn't");
+			if (container->type_argc != result->signature->generic_param_count)
+				g_error ("Inconsistent generic parameter count.  Signature says %d, generic_params table says %d",
+					 result->signature->generic_param_count, container->type_argc);
+		} else if (container && container->is_method && container->type_argc)
+			g_error ("generic_params table claims method has generic parameters, but signature says it doesn't");
 
-	if (!result->klass) {
-		guint32 type = mono_metadata_typedef_from_method (image, token);
-		result->klass = mono_class_get (image, MONO_TOKEN_TYPE_DEF | type);
+		if (cols [1] & METHOD_IMPL_ATTRIBUTE_INTERNAL_CALL)
+			result->signature->pinvoke = 1;
 	}
 
 	if (cols [1] & METHOD_IMPL_ATTRIBUTE_INTERNAL_CALL) {
@@ -956,42 +1173,19 @@ mono_get_method_from_token (MonoImage *image, guint32 token, MonoClass *klass,
 		
 		piinfo->implmap_idx = mono_metadata_implmap_from_method (image, idx - 1);
 		piinfo->piflags = mono_metadata_decode_row_col (im, piinfo->implmap_idx - 1, MONO_IMPLMAP_FLAGS);
-	} else {
-		if (result->signature && result->signature->generic_param_count) {
-			MonoMethodSignature *sig = result->signature;
-
-			for (i = 0; i < sig->generic_param_count; i++) {
-				generic_container->type_params [i].method = result;
-
-				mono_class_from_generic_parameter (
-					&generic_container->type_params [i], image, TRUE);
-			}
-
-			if (sig->ret->type == MONO_TYPE_MVAR) {
-				int num = sig->ret->data.generic_param->num;
-				sig->ret->data.generic_param = &generic_container->type_params [num];
-			}
-
-			for (i = 0; i < sig->param_count; i++) {
-				MonoType *t = sig->params [i];
-				if (t->type == MONO_TYPE_MVAR) {
-					int num = t->data.generic_param->num;
-					sig->params [i]->data.generic_param = &generic_container->type_params [num];
-				}
-			}
-		}
-		
-		/* FIXME: lazyness for generics too, but how? */
-		if (!result->klass->dummy && !(result->flags & METHOD_ATTRIBUTE_ABSTRACT) &&
-		    !(result->iflags & METHOD_IMPL_ATTRIBUTE_RUNTIME) && container) {
-			gpointer loc = mono_image_rva_map (image, cols [0]);
-			g_assert (loc);
-			((MonoMethodNormal *) result)->header = mono_metadata_parse_mh_full (
-				image, (MonoGenericContext *) container, loc);
-		}
-		
-		((MonoMethodNormal *) result)->generic_container = generic_container;
 	}
+		
+	/* FIXME: lazyness for generics too, but how? */
+	if (!result->klass->dummy && !(result->flags & METHOD_ATTRIBUTE_ABSTRACT) &&
+	    !(cols [1] & METHOD_IMPL_ATTRIBUTE_INTERNAL_CALL) &&
+	    !(result->iflags & METHOD_IMPL_ATTRIBUTE_RUNTIME) && container) {
+		gpointer loc = mono_image_rva_map (image, cols [0]);
+		g_assert (loc);
+		((MonoMethodNormal *) result)->header = mono_metadata_parse_mh_full (
+			image, (MonoGenericContext *) container, loc);
+	}
+		
+	result->generic_container = generic_container;
 
 	return result;
 }
@@ -1029,6 +1223,11 @@ mono_get_method_full (MonoImage *image, guint32 token, MonoClass *klass,
 	return result;
 }
 
+/**
+ * mono_get_method_constrained:
+ *
+ * This is used when JITing the `constrained.' opcode.
+ */
 MonoMethod *
 mono_get_method_constrained (MonoImage *image, guint32 token, MonoClass *constrained_class,
 			     MonoGenericContext *context)
@@ -1060,7 +1259,7 @@ mono_get_method_constrained (MonoImage *image, guint32 token, MonoClass *constra
 			   image->name, token);
 
 	if (gclass)
-		result = mono_class_inflate_generic_method (result, gclass->context, gclass->klass);
+		result = mono_class_inflate_generic_method (result, gclass->context);
 
 	mono_loader_unlock ();
 	return result;
@@ -1348,11 +1547,12 @@ mono_loader_unlock (void)
 MonoMethodSignature* 
 mono_method_signature (MonoMethod *m)
 {
+	m = mono_get_inflated_method (m);
 	return mono_method_signature_full (m, NULL);
 }
 
 MonoMethodSignature* 
-mono_method_signature_full (MonoMethod *m, MonoGenericContext *context)
+mono_method_signature_full (MonoMethod *m, MonoGenericContainer *container)
 {
 	int idx;
 	int size;
@@ -1376,7 +1576,7 @@ mono_method_signature_full (MonoMethod *m, MonoGenericContext *context)
 	sig = mono_metadata_blob_heap (img, mono_metadata_decode_row_col (&img->tables [MONO_TABLE_METHOD], idx - 1, MONO_METHOD_SIGNATURE));
 	size = mono_metadata_decode_blob_size (sig, &sig);
 	
-	m->signature = mono_metadata_parse_method_signature_full (img, context, idx, sig, NULL);
+	m->signature = mono_metadata_parse_method_signature_full (img, container, idx, sig, NULL);
 	
 	if (m->iflags & METHOD_IMPL_ATTRIBUTE_INTERNAL_CALL)
 		m->signature->pinvoke = 1;
@@ -1386,6 +1586,7 @@ mono_method_signature_full (MonoMethod *m, MonoGenericContext *context)
 		m->signature->pinvoke = 1;
 		
 		switch (piinfo->piflags & PINVOKE_ATTRIBUTE_CALL_CONV_MASK) {
+		case 0: /* no call conv, so using default */
 		case PINVOKE_ATTRIBUTE_CALL_CONV_WINAPI:
 			conv = MONO_CALL_DEFAULT;
 			break;
@@ -1404,7 +1605,7 @@ mono_method_signature_full (MonoMethod *m, MonoGenericContext *context)
 		case PINVOKE_ATTRIBUTE_CALL_CONV_GENERIC:
 		case PINVOKE_ATTRIBUTE_CALL_CONV_GENERICINST:
 		default:
-			g_warning ("unsupported calling convention");
+			g_warning ("unsupported calling convention : 0x%04x", piinfo->piflags);
 			g_assert_not_reached ();
 		}	
 		m->signature->call_convention = conv;
@@ -1441,6 +1642,9 @@ mono_method_get_header (MonoMethod *method)
 	gpointer loc;
 	MonoMethodNormal* mn = (MonoMethodNormal*) method;
 	
+	if (method->klass->dummy || (method->flags & METHOD_ATTRIBUTE_ABSTRACT) || (method->iflags & METHOD_IMPL_ATTRIBUTE_RUNTIME) || (method->iflags & METHOD_IMPL_ATTRIBUTE_INTERNAL_CALL) || (method->flags & METHOD_ATTRIBUTE_PINVOKE_IMPL))
+		return NULL;
+	
 #ifdef G_LIKELY
 	if (G_LIKELY (mn->header))
 #else
@@ -1448,9 +1652,6 @@ mono_method_get_header (MonoMethod *method)
 #endif
 		return mn->header;
 	
-	if (method->klass->dummy || (method->flags & METHOD_ATTRIBUTE_ABSTRACT) || (method->iflags & METHOD_IMPL_ATTRIBUTE_RUNTIME) || (method->iflags & METHOD_IMPL_ATTRIBUTE_INTERNAL_CALL) || (method->flags & METHOD_ATTRIBUTE_PINVOKE_IMPL))
-		return NULL;
-	
 	mono_loader_lock ();
 	
 	if (mn->header) {
@@ -1466,7 +1667,7 @@ mono_method_get_header (MonoMethod *method)
 	
 	g_assert (loc);
 	
-	mn->header = mono_metadata_parse_mh_full (img, (MonoGenericContext *) mn->generic_container, loc);
+	mn->header = mono_metadata_parse_mh_full (img, (MonoGenericContext *) method->generic_container, loc);
 	
 	mono_loader_unlock ();
 	return mn->header;
@@ -1488,6 +1689,9 @@ mono_method_get_index (MonoMethod *method) {
 	MonoClass *klass = method->klass;
 	int i;
 
+	if (method->token)
+		return mono_metadata_token_index (method->token);
+
 	mono_class_setup_methods (klass);
 	for (i = 0; i < klass->method.count; ++i) {
 		if (method == klass->methods [i])
