@@ -53,7 +53,7 @@
 
 struct _MonoThreadsSync
 {
-	guint32 owner;			/* thread ID */
+	gsize owner;			/* thread ID */
 	guint32 nest;
 	volatile guint32 entry_count;
 	HANDLE entry_sem;
@@ -69,6 +69,8 @@ struct _MonitorArray {
 	MonoThreadsSync monitors [MONO_ZERO_LEN_ARRAY];
 };
 
+#define mono_monitor_allocator_lock() EnterCriticalSection (&monitor_mutex)
+#define mono_monitor_allocator_unlock() LeaveCriticalSection (&monitor_mutex)
 static CRITICAL_SECTION monitor_mutex;
 static MonoThreadsSync *monitor_freelist;
 static MonitorArray *monitor_allocated;
@@ -105,7 +107,7 @@ mon_finalize (MonoThreadsSync *mon)
 
 /* LOCKING: this is called with monitor_mutex held */
 static MonoThreadsSync *
-mon_new (guint32 id)
+mon_new (gsize id)
 {
 	MonoThreadsSync *new;
 
@@ -169,7 +171,7 @@ static gint32
 mono_monitor_try_enter_internal (MonoObject *obj, guint32 ms, gboolean allow_interruption)
 {
 	MonoThreadsSync *mon;
-	guint32 id = GetCurrentThreadId ();
+	gsize id = GetCurrentThreadId ();
 	HANDLE sem;
 	guint32 then = 0, now, delta;
 	guint32 waitms;
@@ -183,16 +185,16 @@ retry:
 
 	/* If the object has never been locked... */
 	if (mon == NULL) {
-		EnterCriticalSection (&monitor_mutex);
+		mono_monitor_allocator_lock ();
 		mon = mon_new (id);
 		if (InterlockedCompareExchangePointer ((gpointer*)&obj->synchronisation, mon, NULL) == NULL) {
 			mono_gc_weak_link_add (&mon->data, obj);
-			LeaveCriticalSection (&monitor_mutex);
+			mono_monitor_allocator_unlock ();
 			/* Successfully locked */
 			return 1;
 		} else {
 			mon_finalize (mon);
-			LeaveCriticalSection (&monitor_mutex);
+			mono_monitor_allocator_unlock ();
 			goto retry;
 		}
 	}
@@ -213,7 +215,7 @@ retry:
 		 * should have been left at 1 by the previous unlock
 		 * operation
 		 */
-		if (InterlockedCompareExchange (&mon->owner, id, 0) == 0) {
+		if (InterlockedCompareExchangePointer ((gpointer *)&mon->owner, (gpointer)id, 0) == 0) {
 			/* Success */
 			g_assert (mon->nest == 1);
 			return 1;
