@@ -32,7 +32,7 @@ struct _MonoDebuggerBreakpointInfo {
  * Address of the x86 trampoline code.  This is used by the debugger to check
  * whether a method is a trampoline.
  */
-extern guint8 *mono_generic_trampoline_code;
+extern guint8 *mono_trampoline_code [];
 
 #ifndef PLATFORM_WIN32
 
@@ -62,7 +62,7 @@ struct _MonoDebuggerIOLayer
 	gpointer (*CreateThread) (WapiSecurityAttributes *security,
 				  guint32 stacksize, WapiThreadStart start,
 				  gpointer param, guint32 create, guint32 *tid);
-	guint32 (*GetCurrentThreadId) (void);
+	gsize (*GetCurrentThreadId) (void);
 };
 
 extern MonoDebuggerIOLayer mono_debugger_io_layer;
@@ -80,16 +80,8 @@ void            mono_debugger_event                       (MonoDebuggerEvent eve
 
 void            mono_debugger_add_symbol_file             (MonoDebugHandle *handle);
 void            mono_debugger_start_add_type              (MonoDebugHandle *symfile, MonoClass *klass);
-void            mono_debugger_add_type                    (MonoDebugHandle *symfile, MonoClass *klass);
 void            mono_debugger_add_builtin_types           (MonoDebugHandle *symfile);
 
-void            mono_debugger_add_method                  (MonoDebugMethodJitInfo *jit);
-
-void            mono_debugger_add_wrapper                 (MonoMethod *wrapper,
-							   MonoDebugMethodJitInfo *jit,
-							   gpointer addr);
-
-
 int             mono_debugger_insert_breakpoint_full      (MonoMethodDesc *desc);
 int             mono_debugger_remove_breakpoint           (int breakpoint_id);
 int             mono_debugger_insert_breakpoint           (const gchar *method_name, gboolean include_namespace);
@@ -112,19 +104,6 @@ gboolean        mono_debugger_throw_exception             (gpointer addr, gpoint
 void *
 mono_vtable_get_static_field_data (MonoVTable *vt);
 
-
-MonoReflectionMethod *
-ves_icall_MonoDebugger_GetMethod (MonoReflectionAssembly *assembly, guint32 token);
-
-int
-ves_icall_MonoDebugger_GetMethodToken (MonoReflectionAssembly *assembly, MonoReflectionMethod *method);
-
-MonoReflectionType *
-ves_icall_MonoDebugger_GetLocalTypeFromSignature (MonoReflectionAssembly *assembly, MonoArray *signature);
-
-MonoReflectionType *
-ves_icall_MonoDebugger_GetType (MonoReflectionAssembly *assembly, guint32 token);
-
 gchar *
 mono_debugger_check_runtime_version (const char *filename);
 
