@@ -229,7 +229,7 @@ struct _MonoThread {
 	guint32	    state;
 	MonoException *abort_exc;
 	MonoObject *abort_state;
-	guint32 tid;
+	guint64 tid;	/* This is accessed as a gsize in the code (so it can hold a 64bit pointer on systems that need it), but needs to reserve 64 bits of space on all machines as it corresponds to a field in managed code */
 	HANDLE	    start_notify;
 	gpointer stack_ptr;
 	gpointer *static_data;
@@ -371,6 +371,18 @@ typedef struct {
 	const void* text_info_data;
 } MonoCultureInfo;
 
+typedef struct {
+	MonoObject obj;
+	gint32 region_id;
+	MonoString *iso2name;
+	MonoString *iso3name;
+	MonoString *win3name;
+	MonoString *english_name;
+	MonoString *currency_symbol;
+	MonoString *iso_currency_symbol;
+	MonoString *currency_english_name;
+} MonoRegionInfo;
+
 typedef struct {
 	MonoObject obj;
 	MonoString *str;
@@ -875,6 +887,7 @@ typedef struct {
 	guint32 index;
 	MonoReflectionType *base_type;
 	MonoArray *iface_constraints;
+	MonoArray *cattrs;
 	guint32 attrs;
 } MonoReflectionGenericParam;
 
@@ -1013,10 +1026,14 @@ void        mono_reflection_create_generic_class  (MonoReflectionTypeBuilder *tb
 
 MonoReflectionType* mono_reflection_create_runtime_class  (MonoReflectionTypeBuilder *tb);
 
+void        mono_reflection_get_dynamic_overrides (MonoClass *klass, MonoMethod ***overrides, int *num_overrides);
+
 void mono_reflection_create_dynamic_method (MonoReflectionDynamicMethod *m);
 
 void        mono_reflection_initialize_generic_parameter (MonoReflectionGenericParam *gparam);
 
+MonoType*
+mono_class_bind_generic_parameters (MonoType *type, int type_argc, MonoType **types);
 MonoType*
 mono_reflection_bind_generic_parameters (MonoReflectionType *type, int type_argc, MonoType **types);
 MonoReflectionMethod*
@@ -1062,5 +1079,8 @@ mono_method_get_signature_full (MonoMethod *method, MonoImage *image, guint32 to
 void
 mono_upgrade_remote_class (MonoDomain *domain, MonoObject *tproxy, MonoClass *klass);
 
+gpointer
+mono_get_addr_from_ftnptr (gpointer descr);
+
 #endif /* __MONO_OBJECT_INTERNALS_H__ */
 
