@@ -31,8 +31,8 @@ void mono_profiler_method_end_jit  (MonoMethod *method, int result);
 void mono_profiler_code_transition (MonoMethod *method, int result);
 void mono_profiler_allocation      (MonoObject *obj, MonoClass *klass);
 void mono_profiler_stat_hit        (guchar *ip, void *context);
-void mono_profiler_thread_start    (guint32 tid);
-void mono_profiler_thread_end      (guint32 tid);
+void mono_profiler_thread_start    (gsize tid);
+void mono_profiler_thread_end      (gsize tid);
 
 void mono_profiler_assembly_event  (MonoAssembly *assembly, int code);
 void mono_profiler_assembly_loaded (MonoAssembly *assembly, int result);
