@@ -12,7 +12,10 @@
 #endif
 
 #include <mono/metadata/appdomain.h>
+#include <mono/metadata/image.h>
 #include <mono/metadata/exception.h>
+#include <mono/metadata/object-internals.h>
+#include <mono/metadata/metadata-internals.h>
 #include <mono/metadata/security.h>
 #include <mono/io-layer/io-layer.h>
 #include <mono/utils/strenc.h>
@@ -111,6 +114,19 @@ GetSidName (gunichar2 *server, PSID sid, gint32 *size)
 
 #else /* not PLATFORM_WIN32 */
 
+#define MONO_SYSCONF_DEFAULT_SIZE	((size_t) 1024)
+
+/*
+ * Ensure we always get a valid (usable) value from sysconf.
+ * In case of error, we return the default value.
+ */
+static size_t mono_sysconf (int name)
+{
+	size_t size = (size_t) sysconf (name);
+	/* default value */
+	return (size == -1) ? MONO_SYSCONF_DEFAULT_SIZE : size;
+}
+
 
 static gchar*
 GetTokenName (uid_t uid)
@@ -128,9 +144,9 @@ GetTokenName (uid_t uid)
 
 #ifdef HAVE_GETPWUID_R
 #ifdef _SC_GETPW_R_SIZE_MAX
- 	fbufsize = (size_t) sysconf (_SC_GETPW_R_SIZE_MAX);
+ 	fbufsize = mono_sysconf (_SC_GETPW_R_SIZE_MAX);
 #else
-	fbufsize = (size_t) 1024;
+	fbufsize = MONO_SYSCONF_DEFAULT_SIZE;
 #endif
 	fbuf = g_malloc0 (fbufsize);
 	retval = getpwuid_r (uid, &pwd, fbuf, fbufsize, &p);
@@ -194,9 +210,9 @@ IsDefaultGroup (uid_t user, gid_t group)
 
 #ifdef HAVE_GETPWUID_R
 #ifdef _SC_GETPW_R_SIZE_MAX
- 	fbufsize = (size_t) sysconf (_SC_GETPW_R_SIZE_MAX);
+ 	fbufsize = mono_sysconf (_SC_GETPW_R_SIZE_MAX);
 #else
-	fbufsize = (size_t) 1024;
+	fbufsize = MONO_SYSCONF_DEFAULT_SIZE;
 #endif
 
 	fbuf = g_malloc0 (fbufsize);
@@ -274,7 +290,7 @@ ves_icall_System_Security_Principal_WindowsIdentity_GetCurrentToken (void)
 		OpenProcessToken (GetCurrentProcess (), TOKEN_QUERY, &token);
 	}
 #else
-	token = (gpointer) geteuid ();
+	token = GINT_TO_POINTER (geteuid ());
 #endif
 	return token;
 }
@@ -299,7 +315,7 @@ ves_icall_System_Security_Principal_WindowsIdentity_GetTokenName (gpointer token
 		g_free (tu);
 	}
 #else 
-	gchar *uname = GetTokenName ((uid_t) token);
+	gchar *uname = GetTokenName ((uid_t) GPOINTER_TO_INT (token));
 
 	MONO_ARCH_SAVE_REGS;
 
@@ -356,9 +372,9 @@ ves_icall_System_Security_Principal_WindowsIdentity_GetUserToken (MonoString *us
 
 #ifdef HAVE_GETPWNAM_R
 #ifdef _SC_GETPW_R_SIZE_MAX
- 	fbufsize = (size_t) sysconf (_SC_GETPW_R_SIZE_MAX);
+ 	fbufsize = mono_sysconf (_SC_GETPW_R_SIZE_MAX);
 #else
-	fbufsize = (size_t) 1024;
+	fbufsize = MONO_SYSCONF_DEFAULT_SIZE;
 #endif
 
 	fbuf = g_malloc0 (fbufsize);
@@ -371,7 +387,7 @@ ves_icall_System_Security_Principal_WindowsIdentity_GetUserToken (MonoString *us
 #endif
 
 	if (result) {
-		token = (gpointer) p->pw_uid;
+		token = GINT_TO_POINTER (p->pw_uid);
 	}
 
 #ifdef HAVE_GETPWNAM_R
@@ -512,21 +528,21 @@ ves_icall_System_Security_Principal_WindowsPrincipal_IsMemberOfGroupId (gpointer
 
 #ifdef HAVE_GETGRGID_R
 #ifdef _SC_GETGR_R_SIZE_MAX
- 	fbufsize = (size_t) sysconf (_SC_GETGR_R_SIZE_MAX);
+ 	fbufsize = mono_sysconf (_SC_GETGR_R_SIZE_MAX);
 #else
-	fbufsize = (size_t) 1024;
+	fbufsize = MONO_SYSCONF_DEFAULT_SIZE;
 #endif
 	fbuf = g_malloc0 (fbufsize);
-	retval = getgrgid_r ((gid_t) group, &grp, fbuf, fbufsize, &g);
+	retval = getgrgid_r ((gid_t) GPOINTER_TO_INT (group), &grp, fbuf, fbufsize, &g);
 	result = ((retval == 0) && (g == &grp));
 #else
 	/* default to non thread-safe but posix compliant function */
-	g = getgrgid ((gid_t) group);
+	g = getgrgid ((gid_t) GPOINTER_TO_INT (group));
 	result = (g != NULL);
 #endif
 
 	if (result) {
-		result = IsMemberOf ((uid_t) user, g);
+		result = IsMemberOf ((uid_t) GPOINTER_TO_INT (user), g);
 	}
 
 #ifdef HAVE_GETGRGID_R
@@ -564,9 +580,9 @@ ves_icall_System_Security_Principal_WindowsPrincipal_IsMemberOfGroupName (gpoint
 		gchar *fbuf;
 		gint32 retval;
 #ifdef _SC_GETGR_R_SIZE_MAX
-	 	size_t fbufsize = (size_t) sysconf (_SC_GETGR_R_SIZE_MAX);
+	 	size_t fbufsize = mono_sysconf (_SC_GETGR_R_SIZE_MAX);
 #else
-		size_t fbufsize = (size_t) 1024;
+		size_t fbufsize = MONO_SYSCONF_DEFAULT_SIZE;
 #endif
 		fbuf = g_malloc0 (fbufsize);
 		retval = getgrnam_r (utf8_groupname, &grp, fbuf, fbufsize, &g);
@@ -578,7 +594,7 @@ ves_icall_System_Security_Principal_WindowsPrincipal_IsMemberOfGroupName (gpoint
 #endif
 
 		if (result) {
-			result = IsMemberOf ((uid_t) user, g);
+			result = IsMemberOf ((uid_t) GPOINTER_TO_INT (user), g);
 		}
 
 #ifdef HAVE_GETGRNAM_R
@@ -932,3 +948,19 @@ ves_icall_Mono_Security_Cryptography_KeyPairPersistence_ProtectUser (MonoString
 #endif
 	return ret;
 }
+
+
+/*
+ * Returns TRUE if there is "something" where the Authenticode signature is 
+ * normally located. Returns FALSE is data directory is empty.
+ *
+ * Note: Neither the structure nor the signature is verified by this function.
+ */
+MonoBoolean
+ves_icall_System_Security_Policy_Evidence_IsAuthenticodePresent (MonoReflectionAssembly *refass)
+{
+	if (refass && refass->assembly && refass->assembly->image) {
+		return mono_image_has_authenticode_entry (refass->assembly->image);
+	}
+	return FALSE;
+}
