@@ -461,7 +461,9 @@ static gint32 convert_sockopt_level_and_name(MonoSocketOptionLevel mono_level,
 			*system_name = IPV6_LEAVE_GROUP;
 			break;
 		case SocketOptionName_PacketInformation:
+#ifdef HAVE_IPV6_PKTINFO
 			*system_name = IPV6_PKTINFO;
+#endif
 			break;
 		case SocketOptionName_HeaderIncluded:
 		case SocketOptionName_IPOptions:
@@ -1580,7 +1582,7 @@ static struct in_addr ipaddress_to_struct_in_addr(MonoObject *ipaddr)
 	struct in_addr inaddr;
 	MonoClassField *field;
 	
-	field=mono_class_get_field_from_name(ipaddr->vtable->klass, "address");
+	field=mono_class_get_field_from_name(ipaddr->vtable->klass, "m_Address");
 
 	/* No idea why .net uses a 64bit type to hold a 32bit value...
 	 *
@@ -1601,7 +1603,7 @@ static struct in6_addr ipaddress_to_struct_in6_addr(MonoObject *ipaddr)
 	MonoArray *data;
 	int i;
 
-	field=mono_class_get_field_from_name(ipaddr->vtable->klass, "_numbers");
+	field=mono_class_get_field_from_name(ipaddr->vtable->klass, "m_Numbers");
 	data=*(MonoArray **)(((char *)ipaddr) + field->offset);
 
 /* Solaris has only the 8 bit version. */
@@ -1854,7 +1856,7 @@ is_loopback (int family, void *ad)
 	}
 #ifdef AF_INET6
 	else {
-		return (IN6_IS_ADDR_LOOPBACK (ptr));
+		return (IN6_IS_ADDR_LOOPBACK ((struct in6_addr *) ptr));
 	}
 #endif
 	return FALSE;
