@@ -240,16 +240,19 @@ async_invoke_io_thread (gpointer data)
 			}
 
 			domain = ((MonoObject *)ar)->vtable->domain;
+			mono_thread_push_appdomain_ref (domain);
 			if (mono_domain_set (domain, FALSE)) {
 				ASyncCall *ac;
 
-				mono_thread_push_appdomain_ref (domain);
 				mono_async_invoke (ar);
 				ac = (ASyncCall *) ar->data;
+				/*
 				if (ac->msg->exc != NULL)
 					mono_unhandled_exception (ac->msg->exc);
-				mono_thread_pop_appdomain_ref ();
+				*/
+				mono_domain_set (mono_get_root_domain (), TRUE);
 			}
+			mono_thread_pop_appdomain_ref ();
 			InterlockedDecrement (&busy_io_worker_threads);
 		}
 
@@ -303,7 +306,7 @@ start_io_thread_or_queue (MonoSocketAsyncResult *ares)
 		InterlockedIncrement (&busy_io_worker_threads);
 		InterlockedIncrement (&io_worker_threads);
 		domain = ((ares) ? ((MonoObject *) ares)->vtable->domain : mono_domain_get ());
-		mono_thread_create (domain, async_invoke_io_thread, ares);
+		mono_thread_create (mono_get_root_domain (), async_invoke_io_thread, ares);
 	} else {
 		append_job (&io_queue_lock, &async_io_queue, ares);
 		ReleaseSemaphore (io_job_added, 1, NULL);
@@ -351,7 +354,7 @@ mark_bad_fds (mono_pollfd *pfds, int nfds)
 		if (pfd->fd == -1)
 			continue;
 
-		ret = mono_poll (pfds, 1, 0);
+		ret = mono_poll (pfd, 1, 0);
 		if (ret == -1 && errno == EBADF) {
 			pfd->revents |= MONO_POLLNVAL;
 			count++;
@@ -775,6 +778,13 @@ socket_io_add_poll (MonoSocketAsyncResult *state)
 	}
 
 	events = get_events_from_list (list);
+#ifdef PLATFORM_MACOSX
+	/* select() for connect() does not work well on the Mac. Bug #75436. */
+	if (state->operation == AIO_OP_CONNECT && state->blocking == TRUE) {
+		start_io_thread_or_queue (state);
+		return;
+	}
+#endif
 	INIT_POLLFD (data->newpfd, GPOINTER_TO_INT (state->handle), events);
 	g_hash_table_replace (data->sock_to_state, GINT_TO_POINTER (state->handle), list);
 	LeaveCriticalSection (&data->io_lock);
@@ -909,8 +919,10 @@ mono_async_invoke (MonoAsyncResult *ares)
 		MonoObject *exc = NULL;
 		void *pa = &ares;
 		mono_runtime_invoke (ac->cb_method, ac->cb_target, pa, &exc);
-		if (!ac->msg->exc)
-			ac->msg->exc = exc;
+		/* 'exc' will be the previous ac->msg->exc if not NULL and not
+		 * catched. If catched, this will be set to NULL and the
+		 * exception will not be printed. */
+		ac->msg->exc = exc;
 	}
 
 	/* restore original thread execution context if flow isn't suppressed, i.e. non null */
@@ -1008,7 +1020,7 @@ start_thread_or_queue (MonoAsyncResult *ares)
 		InterlockedIncrement (&mono_worker_threads);
 		InterlockedIncrement (&busy_worker_threads);
 		domain = ((MonoObject *) ares)->vtable->domain;
-		mono_thread_create (domain, async_invoke_thread, ares);
+		mono_thread_create (mono_get_root_domain (), async_invoke_thread, ares);
 	} else {
 		append_job (&mono_delegate_section, &async_call_queue, ares);
 		ReleaseSemaphore (job_added, 1, NULL);
@@ -1139,21 +1151,24 @@ async_invoke_thread (gpointer data)
 			/* worker threads invokes methods in different domains,
 			 * so we need to set the right domain here */
 			domain = ((MonoObject *)ar)->vtable->domain;
+			mono_thread_push_appdomain_ref (domain);
 			if (mono_domain_set (domain, FALSE)) {
 				ASyncCall *ac;
 
-				mono_thread_push_appdomain_ref (domain);
 				mono_async_invoke (ar);
 				ac = (ASyncCall *) ar->data;
+				/*
 				if (ac->msg->exc != NULL)
 					mono_unhandled_exception (ac->msg->exc);
-				mono_thread_pop_appdomain_ref ();
+				*/
+				mono_domain_set (mono_get_root_domain (), TRUE);
 			}
+			mono_thread_pop_appdomain_ref ();
 			InterlockedDecrement (&busy_worker_threads);
 		}
 
 		data = dequeue_job (&mono_delegate_section, &async_call_queue);
-	
+
 		if (!data) {
 			guint32 wr;
 			int timeout = 10000;
