@@ -686,7 +686,7 @@ enum {
 	TYPE_MAX = 8
 };
 
-static const char* 
+static const char*  const
 arg_name [TYPE_MAX] = {
 	"Invalid",
 	"Int32",
@@ -1145,9 +1145,9 @@ mono_method_verify (MonoMethod *method, int level)
 	ILStackDesc *stack;
 	register const unsigned char *ip;
 	register const unsigned char *end;
-	const unsigned char *target; /* branch target */
+	const unsigned char *target = NULL; /* branch target */
 	int max_args, max_stack, cur_stack, i, n, need_merge, start;
-	guint32 token, ip_offset;
+	guint32 token, ip_offset = 0;
 	char *local_state = NULL;
 	GSList *list = NULL;
 	guint prefix = 0;
@@ -1157,8 +1157,8 @@ mono_method_verify (MonoMethod *method, int level)
 			(method->flags & (METHOD_ATTRIBUTE_PINVOKE_IMPL | METHOD_ATTRIBUTE_ABSTRACT))) {
 		return NULL;
 	}
-	signature = method->signature;
-	header = ((MonoMethodNormal *)method)->header;
+	signature = mono_method_signature (method);
+	header = mono_method_get_header (method);
 	ip = header->code;
 	end = ip + header->code_size;
 	max_args = signature->param_count + signature->hasthis;
@@ -1176,7 +1176,7 @@ mono_method_verify (MonoMethod *method, int level)
 		params = signature->params;
 	}
 
-	if (method->signature->is_inflated)
+	if (signature->is_inflated)
 		generic_context = ((MonoMethodInflated *) method)->context;
 
 	if (header->num_locals) {
@@ -1396,8 +1396,8 @@ mono_method_verify (MonoMethod *method, int level)
 			cmethod = mono_get_method_full (image, token, NULL, generic_context);
 			if (!cmethod)
 				ADD_INVALID (list, g_strdup_printf ("Method 0x%08x not found at 0x%04x", token, ip_offset));
-			if (cmethod->signature->pinvoke) {
-				csig = cmethod->signature;
+			if (mono_method_signature (cmethod)) {
+				csig = mono_method_signature (cmethod);
 			} else {
 				csig = mono_method_get_signature (cmethod, image, token);
 			}
@@ -1638,7 +1638,7 @@ mono_method_verify (MonoMethod *method, int level)
 			cmethod = mono_get_method_full (image, token, NULL, generic_context);
 			if (!cmethod)
 				ADD_INVALID (list, g_strdup_printf ("Constructor 0x%08x not found at 0x%04x", token, ip_offset));
-			csig = cmethod->signature;
+			csig = mono_method_signature (cmethod);
 			CHECK_STACK_UNDERFLOW (csig->param_count);
 			cur_stack -= csig->param_count;
 			CHECK_STACK_OVERFLOW ();
@@ -1826,8 +1826,8 @@ mono_method_verify (MonoMethod *method, int level)
 			cur_stack -= 3;
 			++ip;
 			break;
-		case CEE_LDELEM:
-		case CEE_STELEM:
+		case CEE_LDELEM_ANY:
+		case CEE_STELEM_ANY:
 		case CEE_UNBOX_ANY:
 		case CEE_UNUSED5:
 		case CEE_UNUSED6:
@@ -2225,7 +2225,8 @@ methodbuilder_fields[] = {
 	{"pinfo", G_STRUCT_OFFSET (MonoReflectionMethodBuilder, pinfo)},
 	{"pi_dll", G_STRUCT_OFFSET (MonoReflectionMethodBuilder, dll)},
 	{"pi_entry", G_STRUCT_OFFSET (MonoReflectionMethodBuilder, dllentry)},
-	{"ncharset", G_STRUCT_OFFSET (MonoReflectionMethodBuilder, charset)},
+	{"charset", G_STRUCT_OFFSET (MonoReflectionMethodBuilder, charset)},
+	{"extra_flags", G_STRUCT_OFFSET (MonoReflectionMethodBuilder, extra_flags)},
 	{"native_cc", G_STRUCT_OFFSET (MonoReflectionMethodBuilder, native_cc)},
 	{"call_conv", G_STRUCT_OFFSET (MonoReflectionMethodBuilder, call_conv)},
 	{NULL, 0}
