@@ -67,6 +67,9 @@ class Tests {
 		byte b = (byte)a;
 		if (b != 2)
 			return 5;
+		sbyte sb = (sbyte)a;
+		if (sb != 2)
+			return 6;
 		return 0;
 	}
 
@@ -531,5 +534,13 @@ class Tests {
 		return result;
 	}
 
+	static int test_0_float_precision () {
+		float f1 = 3.40282346638528859E+38f;
+		float f2 = 3.40282346638528859E+38f;		
+		float PositiveInfinity =  1.0f / 0.0f;
+		float f = f1 + f2;
+
+		return f == PositiveInfinity ? 0 : 1;
+	}
 }
 
