@@ -73,7 +73,7 @@ compute_dominators (MonoCompile *m) {
 
 #ifdef DEBUG_DOMINATORS
 	printf ("DTREE %s %d\n", mono_method_full_name (m->method, TRUE), 
-		((MonoMethodNormal *)m->method)->header->num_clauses);
+		mono_method_get_header (m->method)->num_clauses);
 	for (i = 0; i < m->num_bblocks; ++i) {
 		bb = m->bblocks [i];
 		printf ("BB%d: ", bb->block_num);
@@ -425,6 +425,8 @@ mono_compute_natural_loops (MonoCompile *cfg)
 			if (n != h && mono_bitset_test (n->dominators, h->dfn)) {
 				GList *todo;
 
+				n->loop_body_start = 1;
+
 				/* already in loop_blocks? */
 				if (h->loop_blocks && g_list_find (h->loop_blocks, n))
 					continue;
