@@ -22,6 +22,7 @@ MINI_OP(OP_OUTARG_IMM,	"outarg_imm")
 MINI_OP(OP_OUTARG_R4,	"outarg_r4")
 MINI_OP(OP_OUTARG_R8,	"outarg_r8")
 MINI_OP(OP_OUTARG_VT,	"outarg_vt")
+MINI_OP(OP_OUTARG_MEMBASE,	"outarg_membase")
 MINI_OP(OP_INARG_VT,	"inarg_vt")
 MINI_OP(OP_RETARG,	"retarg")
 MINI_OP(OP_SETRET,	"setret")
@@ -87,6 +88,26 @@ MINI_OP(OP_LOADU4_MEMBASE,"loadu4_membase")
 MINI_OP(OP_LOADI8_MEMBASE,"loadi8_membase")
 MINI_OP(OP_LOADR4_MEMBASE,"loadr4_membase")
 MINI_OP(OP_LOADR8_MEMBASE,"loadr8_membase")
+/* indexed loads: dreg = load at (sreg1 + sreg2)*/
+MINI_OP(OP_LOAD_MEMINDEX,  "load_memindex")
+MINI_OP(OP_LOADI1_MEMINDEX,"loadi1_memindex")
+MINI_OP(OP_LOADU1_MEMINDEX,"loadu1_memindex")
+MINI_OP(OP_LOADI2_MEMINDEX,"loadi2_memindex")
+MINI_OP(OP_LOADU2_MEMINDEX,"loadu2_memindex")
+MINI_OP(OP_LOADI4_MEMINDEX,"loadi4_memindex")
+MINI_OP(OP_LOADU4_MEMINDEX,"loadu4_memindex")
+MINI_OP(OP_LOADI8_MEMINDEX,"loadi8_memindex")
+MINI_OP(OP_LOADR4_MEMINDEX,"loadr4_memindex")
+MINI_OP(OP_LOADR8_MEMINDEX,"loadr8_memindex")
+/* indexed stores: store sreg1 at (destbasereg + sreg2) */
+MINI_OP(OP_STORE_MEMINDEX,"store_memindex")
+MINI_OP(OP_STOREI1_MEMINDEX,"storei1_memindex")
+MINI_OP(OP_STOREI2_MEMINDEX,"storei2_memindex")
+MINI_OP(OP_STOREI4_MEMINDEX,"storei4_memindex")
+MINI_OP(OP_STOREI8_MEMINDEX,"storei8_memindex")
+MINI_OP(OP_STORER4_MEMINDEX,"storer4_memindex")
+MINI_OP(OP_STORER8_MEMINDEX,"storer8_memindex")
+
 MINI_OP(OP_LOADR8_SPILL_MEMBASE,"loadr8_spill_membase")
 MINI_OP(OP_LOADU4_MEM,"loadu4_mem")
 MINI_OP(OP_MOVE,	"move")
@@ -123,6 +144,22 @@ MINI_OP(OP_COND_EXC_NO, "cond_exc_no")
 MINI_OP(OP_COND_EXC_C, "cond_exc_c")
 MINI_OP(OP_COND_EXC_NC, "cond_exc_nc")
 
+MINI_OP(OP_COND_EXC_IEQ, "cond_exc_ieq")
+MINI_OP(OP_COND_EXC_IGE, "cond_exc_ige")
+MINI_OP(OP_COND_EXC_IGT, "cond_exc_igt")
+MINI_OP(OP_COND_EXC_ILE, "cond_exc_ile")
+MINI_OP(OP_COND_EXC_ILT, "cond_exc_ilt")
+MINI_OP(OP_COND_EXC_INE_UN, "cond_exc_ine_un")
+MINI_OP(OP_COND_EXC_IGE_UN, "cond_exc_ige_un")
+MINI_OP(OP_COND_EXC_IGT_UN, "cond_exc_igt_un")
+MINI_OP(OP_COND_EXC_ILE_UN, "cond_exc_ile_un")
+MINI_OP(OP_COND_EXC_ILT_UN, "cond_exc_ilt_un")
+
+MINI_OP(OP_COND_EXC_IOV, "cond_exc_iov")
+MINI_OP(OP_COND_EXC_INO, "cond_exc_ino")
+MINI_OP(OP_COND_EXC_IC, "cond_exc_ic")
+MINI_OP(OP_COND_EXC_INC, "cond_exc_inc")
+
 /* 64 bit opcodes: must be in the same order as the matching CEE_ opcodes: binops_op_map */
 MINI_OP(OP_LADD,    "long_add")
 MINI_OP(OP_LSUB,    "long_sub")
@@ -208,12 +245,12 @@ MINI_OP(OP_LBGE_UN, "long_bge_un")
 MINI_OP(OP_LBLE,    "long_ble")
 MINI_OP(OP_LBLE_UN, "long_ble_un")
 
-/* 32 bit opcodes on 64 bit machines */
+MINI_OP(OP_LONG_SHRUN_32, "long_shr_un_32")
+
+/* 32 bit opcodes: must be in the same order as the matching CEE_ opcodes: binops_op_map */
 MINI_OP(OP_IADD,    "int_add")
 MINI_OP(OP_ISUB,    "int_sub")
 MINI_OP(OP_IMUL,    "int_mul")
-MINI_OP(OP_IMUL_OVF,    "int_mul_ovf")
-MINI_OP(OP_IMUL_OVF_UN, "int_mul_ovf_un")
 MINI_OP(OP_IDIV,    "int_div")
 MINI_OP(OP_IDIV_UN, "int_div_un")
 MINI_OP(OP_IREM,    "int_rem")
@@ -224,6 +261,54 @@ MINI_OP(OP_IXOR,    "int_xor")
 MINI_OP(OP_ISHL,    "int_shl")
 MINI_OP(OP_ISHR,    "int_shr")
 MINI_OP(OP_ISHR_UN, "int_shr_un")
+
+/* 32 bit opcodes: must be in the same order as the matching CEE_ opcodes: unops_op_map */
+MINI_OP(OP_INEG,       "int_neg")
+MINI_OP(OP_INOT,       "int_not")
+MINI_OP(OP_ICONV_TO_I1,"int_conv_to_i1")
+MINI_OP(OP_ICONV_TO_I2,"int_conv_to_i2")
+MINI_OP(OP_ICONV_TO_I4,"int_conv_to_i4")
+MINI_OP(OP_ICONV_TO_I8,"int_conv_to_i8")
+MINI_OP(OP_ICONV_TO_R4,"int_conv_to_r4")
+MINI_OP(OP_ICONV_TO_R8,"int_conv_to_r8")
+MINI_OP(OP_ICONV_TO_U4,"int_conv_to_u4")
+MINI_OP(OP_ICONV_TO_U8,"int_conv_to_u8")
+
+/* 32 bit opcodes: must be in the same order as the matching CEE_ opcodes: ovfops_op_map */
+MINI_OP(OP_ICONV_TO_U2,   "int_conv_to_u2")
+MINI_OP(OP_ICONV_TO_U1,   "int_conv_to_u1")
+MINI_OP(OP_ICONV_TO_I,    "int_conv_to_i")
+MINI_OP(OP_ICONV_TO_OVF_I,"int_conv_to_ovf_i")
+MINI_OP(OP_ICONV_TO_OVF_U,"int_conv_to_ovf_u")
+MINI_OP(OP_IADD_OVF,      "int_add_ovf")
+MINI_OP(OP_IADD_OVF_UN,   "int_add_ovf_un")
+MINI_OP(OP_IMUL_OVF,      "int_mul_ovf")
+MINI_OP(OP_IMUL_OVF_UN,   "int_mul_ovf_un")
+MINI_OP(OP_ISUB_OVF,      "int_sub_ovf")
+MINI_OP(OP_ISUB_OVF_UN,   "int_sub_ovf_un")
+
+/* 32 bit opcodes: must be in the same order as the matching CEE_ opcodes: ovf2ops_op_map */
+MINI_OP(OP_ICONV_TO_OVF_I1_UN,"int_conv_to_ovf_i1_un")
+MINI_OP(OP_ICONV_TO_OVF_I2_UN,"int_conv_to_ovf_i2_un")
+MINI_OP(OP_ICONV_TO_OVF_I4_UN,"int_conv_to_ovf_i4_un")
+MINI_OP(OP_ICONV_TO_OVF_I8_UN,"int_conv_to_ovf_i8_un")
+MINI_OP(OP_ICONV_TO_OVF_U1_UN,"int_conv_to_ovf_u1_un")
+MINI_OP(OP_ICONV_TO_OVF_U2_UN,"int_conv_to_ovf_u2_un")
+MINI_OP(OP_ICONV_TO_OVF_U4_UN,"int_conv_to_ovf_u4_un")
+MINI_OP(OP_ICONV_TO_OVF_U8_UN,"int_conv_to_ovf_u8_un")
+MINI_OP(OP_ICONV_TO_OVF_I_UN, "int_conv_to_ovf_i_un")
+MINI_OP(OP_ICONV_TO_OVF_U_UN, "int_conv_to_ovf_u_un")
+
+/* 32 bit opcodes: must be in the same order as the matching CEE_ opcodes: ovf3ops_op_map */
+MINI_OP(OP_ICONV_TO_OVF_I1,"int_conv_to_ovf_i1")
+MINI_OP(OP_ICONV_TO_OVF_U1,"int_conv_to_ovf_u1")
+MINI_OP(OP_ICONV_TO_OVF_I2,"int_conv_to_ovf_i2")
+MINI_OP(OP_ICONV_TO_OVF_U2,"int_conv_to_ovf_u2")
+MINI_OP(OP_ICONV_TO_OVF_I4,"int_conv_to_ovf_i4")
+MINI_OP(OP_ICONV_TO_OVF_U4,"int_conv_to_ovf_u4")
+MINI_OP(OP_ICONV_TO_OVF_I8,"int_conv_to_ovf_i8")
+MINI_OP(OP_ICONV_TO_OVF_U8,"int_conv_to_ovf_u8")
+
 MINI_OP(OP_IADC,     "int_adc")
 MINI_OP(OP_IADC_IMM, "int_adc_imm")
 MINI_OP(OP_ISBB,     "int_sbb")
@@ -245,9 +330,6 @@ MINI_OP(OP_ISHL_IMM,    "int_shl_imm")
 MINI_OP(OP_ISHR_IMM,    "int_shr_imm")
 MINI_OP(OP_ISHR_UN_IMM, "int_shr_un_imm")
 
-MINI_OP(OP_INEG,       "int_neg")
-MINI_OP(OP_INOT,       "int_not")
-
 MINI_OP(OP_ICEQ,   "int_ceq")
 MINI_OP(OP_ICGT,   "int_cgt")
 MINI_OP(OP_ICGT_UN,"int_cgt_un")
@@ -265,9 +347,6 @@ MINI_OP(OP_IBGE_UN, "int_bge_un")
 MINI_OP(OP_IBLE,    "int_ble")
 MINI_OP(OP_IBLE_UN, "int_ble_un")
 
-MINI_OP(OP_COND_EXC_IOV, "cond_exc_iov")
-MINI_OP(OP_COND_EXC_IC, "cond_exc_ic")
-
 MINI_OP(OP_FBEQ,   "float_beq")
 MINI_OP(OP_FBNE_UN,"float_bne_un")
 MINI_OP(OP_FBLT,   "float_blt")
@@ -279,8 +358,6 @@ MINI_OP(OP_FBGE_UN,"float_bge_un")
 MINI_OP(OP_FBLE,   "float_ble")
 MINI_OP(OP_FBLE_UN,"float_ble_un")
 
-MINI_OP(OP_LONG_SHRUN_32, "long_shr_un_32")
-
 /* float opcodes: must be in the same order as the matching CEE_ opcodes: binops_op_map */
 MINI_OP(OP_FADD,   "float_add")
 MINI_OP(OP_FSUB,   "float_sub")
@@ -444,6 +521,7 @@ MINI_OP(OP_ATOMIC_ADD_NEW_I8, "atomic_add_new_i8")
 MINI_OP(OP_ATOMIC_ADD_IMM_I8, "atomic_add_imm_i8")
 MINI_OP(OP_ATOMIC_ADD_IMM_NEW_I8, "atomic_add_imm_new_i8")
 MINI_OP(OP_ATOMIC_EXCHANGE_I8, "atomic_exchange_i8")
+MINI_OP(OP_MEMORY_BARRIER, "memory_barrier")
 
 /* x86 specific */
 MINI_OP(OP_X86_TEST_NULL,          "x86_test_null")
@@ -485,16 +563,22 @@ MINI_OP(OP_AMD64_ICOMPARE_MEMBASE_REG,   "amd64_icompare_membase_reg")
 MINI_OP(OP_AMD64_ICOMPARE_MEMBASE_IMM,   "amd64_icompare_membase_imm")
 MINI_OP(OP_AMD64_ICOMPARE_REG_MEMBASE,   "amd64_icompare_reg_membase")
 MINI_OP(OP_AMD64_OUTARG_ALIGN_STACK,     "amd64_outarg_align_stack")
+MINI_OP(OP_AMD64_LOADI8_MEMINDEX,        "amd64_loadi8_memindex")
 	
 MINI_OP(OP_PPC_SUBFIC,             "ppc_subfic")
 MINI_OP(OP_PPC_SUBFZE,             "ppc_subfze")
 
+MINI_OP(OP_ARM_RSBS_IMM,            "arm_rsbs_imm")
+MINI_OP(OP_ARM_RSC_IMM,             "arm_rsc_imm")
+
 MINI_OP(OP_SPARC_OUTARG_REGPAIR,   "sparc_outarg_regpair")
 MINI_OP(OP_SPARC_OUTARG_MEM,       "sparc_outarg_mem")
 MINI_OP(OP_SPARC_OUTARG_MEMPAIR,   "sparc_outarg_mempair")
 MINI_OP(OP_SPARC_OUTARG_SPLIT_REG_STACK, "sparc_outarg_split_reg_stack")
 MINI_OP(OP_SPARC_OUTARG_FLOAT_REG, "sparc_outarg_float_reg")
 MINI_OP(OP_SPARC_OUTARG_DOUBLE_REG, "sparc_outarg_double_reg")
+MINI_OP(OP_SPARC_OUTARG_FLOAT,      "sparc_outarg_float")
+MINI_OP(OP_SPARC_OUTARG_REGPAIR_FLOAT, "sparc_outarg_float")
 MINI_OP(OP_SPARC_INARG_VT,         "sparc_inarg_vt")
 MINI_OP(OP_SPARC_LOCALLOC_IMM,     "sparc_localloc_imm")
 MINI_OP(OP_SPARC_SETFREG_FLOAT,   "sparc_setfreg_float")
@@ -513,10 +597,12 @@ MINI_OP(OP_SPARC_COND_EXC_NEZ,     "sparc_cond_exc_nez")
 
 
 MINI_OP(OP_S390_LOADARG,	   "s390_loadarg")
+MINI_OP(OP_S390_ARGREG, 	   "s390_argreg")
 MINI_OP(OP_S390_ARGPTR, 	   "s390_argptr")
 MINI_OP(OP_S390_STKARG, 	   "s390_stkarg")
 MINI_OP(OP_S390_MOVE,	 	   "s390_move")
 MINI_OP(OP_S390_SETF4RET,	   "s390_setf4ret")
+MINI_OP(OP_S390_BKCHAIN, 	   "s390_bkchain")
 
 MINI_OP(OP_IA64_LOAD,          "ia64_load")
 MINI_OP(OP_IA64_LOADI1,        "ia64_loadi1")
@@ -599,3 +685,20 @@ MINI_OP(OP_IA64_COND_EXC,       "ia64_cond_exc")
 MINI_OP(OP_IA64_CSET,           "ia64_cset")
 
 MINI_OP(OP_IA64_OUTARG_R4,      "ia64_outarg_r4")
+MINI_OP(OP_IA64_STOREI1_MEMBASE_INC_REG, "ia64_storei1_membase_inc_reg")
+MINI_OP(OP_IA64_STOREI2_MEMBASE_INC_REG, "ia64_storei2_membase_inc_reg")
+MINI_OP(OP_IA64_STOREI4_MEMBASE_INC_REG, "ia64_storei4_membase_inc_reg")
+MINI_OP(OP_IA64_STOREI8_MEMBASE_INC_REG, "ia64_storei8_membase_inc_reg")
+MINI_OP(OP_IA64_STORER4_MEMBASE_INC_REG, "ia64_storer4_membase_inc_reg")
+MINI_OP(OP_IA64_STORER8_MEMBASE_INC_REG, "ia64_storer8_membase_inc_reg")
+MINI_OP(OP_IA64_LOADI1_MEMBASE_INC,"ia64_loadi1_membase_inc")
+MINI_OP(OP_IA64_LOADU1_MEMBASE_INC,"ia64_loadu1_membase_inc")
+MINI_OP(OP_IA64_LOADI2_MEMBASE_INC,"ia64_loadi2_membase_inc")
+MINI_OP(OP_IA64_LOADU2_MEMBASE_INC,"ia64_loadu2_membase_inc")
+MINI_OP(OP_IA64_LOADI4_MEMBASE_INC,"ia64_loadi4_membase_inc")
+MINI_OP(OP_IA64_LOADU4_MEMBASE_INC,"ia64_loadu4_membase_inc")
+MINI_OP(OP_IA64_LOADI8_MEMBASE_INC,"ia64_loadi8_membase_inc")
+MINI_OP(OP_IA64_LOADR4_MEMBASE_INC,"ia64_loadr4_membase_inc")
+MINI_OP(OP_IA64_LOADR8_MEMBASE_INC,"ia64_loadr8_membase_inc")
+MINI_OP(OP_IA64_FETCHADD4_IMM,   "ia64_fetchadd4_imm")
+MINI_OP(OP_IA64_FETCHADD8_IMM,   "ia64_fetchadd8_imm")
