@@ -93,12 +93,38 @@ struct sigcontext {
 #endif /* HAVE_WORKING_SIGALTSTACK */
 #endif /* !PLATFORM_WIN32 */
 
+#define MONO_ARCH_SIGNAL_STACK_SIZE (64 * 1024)
+
 /* Enables OP_LSHL, OP_LSHL_IMM, OP_LSHR, OP_LSHR_IMM, OP_LSHR_UN, OP_LSHR_UN_IMM */
 #define MONO_ARCH_NO_EMULATE_LONG_SHIFT_OPS
 
 #define MONO_MAX_IREGS 8
 #define MONO_MAX_FREGS 6
 
+/* Parameters used by the register allocator */
+#define MONO_ARCH_HAS_XP_LOCAL_REGALLOC
+
+#define MONO_ARCH_CALLEE_REGS X86_CALLEE_REGS
+#define MONO_ARCH_CALLEE_SAVED_REGS X86_CALLER_REGS
+
+#define MONO_ARCH_CALLEE_FREGS 0
+#define MONO_ARCH_CALLEE_SAVED_FREGS 0
+
+#define MONO_ARCH_USE_FPSTACK TRUE
+#define MONO_ARCH_FPSTACK_SIZE 6
+
+/* FIXME: Don't constrain byte regs to eax */
+#define MONO_ARCH_INST_FIXED_REG(desc) ((desc == 's') ? X86_ECX : ((desc == 'a') ? X86_EAX : ((desc == 'd') ? X86_EDX : ((desc == 'y') ? X86_EAX : ((desc == 'l') ? X86_EAX : -1)))))
+
+/* RDX is clobbered by the opcode implementation before accessing sreg2 */
+#define MONO_ARCH_INST_SREG2_MASK(ins) (((ins [MONO_INST_CLOB] == 'a') || (ins [MONO_INST_CLOB] == 'd')) ? (1 << X86_EDX) : 0)
+
+/*
+ * L is a generic register pair, while l means eax:rdx
+ */
+#define MONO_ARCH_INST_IS_REGPAIR(desc) (desc == 'l' || desc == 'L')
+#define MONO_ARCH_INST_REGPAIR_REG2(desc,hreg1) (desc == 'l' ? X86_EDX : -1)
+
 #define MONO_ARCH_FRAME_ALIGNMENT 4
 
 /* fixme: align to 16byte instead of 32byte (we align to 32byte to get 
@@ -131,7 +157,7 @@ struct MonoLMF {
 
 typedef void* MonoCompileArch;
 
-#if defined(__FreeBSD__) || defined(__NetBSD__) || defined(__OpenBSD__)
+#if defined(__FreeBSD__) || defined(__NetBSD__) || defined(__OpenBSD__) || defined(__APPLE__)
 # define SC_EAX sc_eax
 # define SC_EBX sc_ebx
 # define SC_ECX sc_ecx
@@ -173,12 +199,40 @@ typedef struct {
 #define MONO_CONTEXT_GET_BP(ctx) ((gpointer)((ctx)->ebp))
 #define MONO_CONTEXT_GET_SP(ctx) ((gpointer)((ctx)->esp))
 
+#ifdef _MSC_VER
+
+#define MONO_INIT_CONTEXT_FROM_FUNC(ctx, start_func) do { \
+    unsigned int stackptr; \
+	mono_arch_flush_register_windows (); \
+    { \
+	   __asm mov stackptr, ebp \
+    } \
+	MONO_CONTEXT_SET_IP ((ctx), (start_func)); \
+	MONO_CONTEXT_SET_BP ((ctx), stackptr); \
+} while (0)
+
+#define MONO_INIT_CONTEXT_FROM_CALLER(ctx) do { \
+	unsigned int stackptr, retaddr; \
+    { \
+	    __asm mov stackptr, ebp \
+        __asm mov eax, DWORD PTR [ebp + 4] \
+	    __asm mov retaddr, eax \
+    } \
+	MONO_CONTEXT_SET_IP ((ctx), retaddr); \
+	/* FIXME: NOT WORKING -- THIS IS __builtin_frame_address (0) NOT (1) */ \
+	MONO_CONTEXT_SET_BP ((ctx), stackptr); \
+} while (0)
+
+#else
+
 #define MONO_INIT_CONTEXT_FROM_FUNC(ctx,start_func) do {	\
 		mono_arch_flush_register_windows ();	\
 		MONO_CONTEXT_SET_IP ((ctx), (start_func));	\
 		MONO_CONTEXT_SET_BP ((ctx), __builtin_frame_address (0));	\
 	} while (0)
 
+#endif
+
 #define MONO_ARCH_BIGMUL_INTRINS 1
 #define MONO_ARCH_NEED_DIV_CHECK 1
 #define MONO_ARCH_HAVE_IS_INT_OVERFLOW 1
@@ -188,6 +242,8 @@ typedef struct {
 #define MONO_ARCH_HAVE_THROW_CORLIB_EXCEPTION 1
 #define MONO_ARCH_ENABLE_EMIT_STATE_OPT 1
 #define MONO_ARCH_HAVE_CREATE_TRAMPOLINE_FROM_TOKEN 1
+#define MONO_ARCH_HAVE_CREATE_SPECIFIC_TRAMPOLINE 1
+#define MONO_ARCH_HAVE_CREATE_DELEGATE_TRAMPOLINE 1
 #define MONO_ARCH_HAVE_CREATE_VARS 1
 
 #endif /* __MONO_MINI_X86_H__ */  
