@@ -33,7 +33,7 @@ mono_trace_eval (MonoMethod *method)
 		case MONO_TRACEOP_ALL:
 			inc = 1; break;
 		case MONO_TRACEOP_PROGRAM:
-			if (method->klass->image == mono_assembly_get_image (trace_spec.assembly))
+			if (trace_spec.assembly && (method->klass->image == mono_assembly_get_image (trace_spec.assembly)))
 				inc = 1; break;
 		case MONO_TRACEOP_METHOD:
 			if (mono_method_desc_full_match ((MonoMethodDesc *) op->data, method))
@@ -65,6 +65,8 @@ static int is_filenamechar (char p)
 		return TRUE;
 	if (p >= 'a' && p <= 'z')
 		return TRUE;
+	if (p >= '0' && p <= '9')
+		return TRUE;
 	if (p == '.' || p == ':' || p == '_')
 		return TRUE;
 	return FALSE;
@@ -94,6 +96,7 @@ enum Token {
 	TOKEN_NAMESPACE,
 	TOKEN_STRING,
 	TOKEN_EXCLUDE,
+	TOKEN_DISABLED,
 	TOKEN_SEPARATOR,
 	TOKEN_END,
 	TOKEN_ERROR
@@ -102,42 +105,45 @@ enum Token {
 static int
 get_token (void)
 {
-	while (*input != 0){
-		if (input [0] == 'M' && input [1] == ':'){
-			input += 2;
-			get_string ();
-			return TOKEN_METHOD;
-		}
-		if (input [0] == 'N' && input [1] == ':'){
-			input += 2;
-			get_string ();
-			return TOKEN_NAMESPACE;
-		}
-		if (input [0] == 'T' && input [1] == ':'){
-			input += 2;
-			get_string ();
-			return TOKEN_CLASS;
-		}
-		if (is_filenamechar (*input)){
-			get_string ();
-			if (strcmp (value, "all") == 0)
-				return TOKEN_ALL;
-			if (strcmp (value, "program") == 0)
-				return TOKEN_PROGRAM;
-			return TOKEN_STRING;
-		}
-		if (*input == '-'){
-			input++;
-			return TOKEN_EXCLUDE;
-		}
-		if (*input == ','){
-			input++;
-			return TOKEN_SEPARATOR;
-		}
-		input++;
-			
+	if (input [0] == '\0') {
+		return TOKEN_END;
 	}
-	return TOKEN_END;
+	if (input [0] == 'M' && input [1] == ':'){
+		input += 2;
+		get_string ();
+		return TOKEN_METHOD;
+	}
+	if (input [0] == 'N' && input [1] == ':'){
+		input += 2;
+		get_string ();
+		return TOKEN_NAMESPACE;
+	}
+	if (input [0] == 'T' && input [1] == ':'){
+		input += 2;
+		get_string ();
+		return TOKEN_CLASS;
+	}
+	if (is_filenamechar (*input)){
+		get_string ();
+		if (strcmp (value, "all") == 0)
+			return TOKEN_ALL;
+		if (strcmp (value, "program") == 0)
+			return TOKEN_PROGRAM;
+		if (strcmp (value, "disabled") == 0)
+			return TOKEN_DISABLED;
+		return TOKEN_STRING;
+	}
+	if (*input == '-'){
+		input++;
+		return TOKEN_EXCLUDE;
+	}
+	if (*input == ','){
+		input++;
+		return TOKEN_SEPARATOR;
+	}
+
+	fprintf (stderr, "Syntax error at or around '%s'\n", input);	
+	return TOKEN_ERROR;
 }
 
 static void
@@ -195,8 +201,9 @@ get_spec (int *last)
 	} else if (token == TOKEN_STRING){
 		trace_spec.ops [*last].op = MONO_TRACEOP_ASSEMBLY;
 		trace_spec.ops [*last].data = g_strdup (value);
-	}
-	else {
+	} else if (token == TOKEN_DISABLED) {
+		trace_spec.enabled = FALSE;
+	} else {
 		fprintf (stderr, "Syntax error in trace option specification\n");
 		return TOKEN_ERROR;
 	}
@@ -205,15 +212,14 @@ get_spec (int *last)
 }
 
 MonoTraceSpec *
-mono_trace_parse_options (MonoAssembly *assembly, char *options)
+mono_trace_parse_options (char *options)
 {
 	char *p = options;
 	int size = 1;
 	int last_used;
 	int token;
-	
-	trace_spec.assembly = assembly;
-	
+
+	trace_spec.enabled = TRUE;
 	if (*p == 0){
 		trace_spec.len = 1;
 		trace_spec.ops = g_new0 (MonoTraceOperation, 1);
@@ -241,6 +247,12 @@ mono_trace_parse_options (MonoAssembly *assembly, char *options)
 	return &trace_spec;
 }
 
+void
+mono_trace_set_assembly (MonoAssembly *assembly)
+{
+	trace_spec.assembly = assembly;
+}
+
 static int indent_level = 0;
 
 static void indent (int diff) {
@@ -255,8 +267,6 @@ static void indent (int diff) {
 		indent_level += diff;
 }
 
-static gboolean enable_trace = TRUE;
-
 void
 mono_trace_enter_method (MonoMethod *method, char *ebp)
 {
@@ -267,7 +277,7 @@ mono_trace_enter_method (MonoMethod *method, char *ebp)
 	MonoMethodSignature *sig;
 	char *fname;
 
-	if (!enable_trace)
+	if (!trace_spec.enabled)
 		return;
 
 	fname = mono_method_full_name (method, TRUE);
@@ -279,23 +289,23 @@ mono_trace_enter_method (MonoMethod *method, char *ebp)
 		printf (") ip: %p\n", __builtin_return_address (1));
 		return;
 	}	
-	if (((int)ebp & (MONO_ARCH_FRAME_ALIGNMENT - 1)) != 0) {
+	if ((GPOINTER_TO_INT (ebp) & (MONO_ARCH_FRAME_ALIGNMENT - 1)) != 0) {
 		g_error ("unaligned stack detected (%p)", ebp);
 	}
 
-	sig = method->signature;
+	sig = mono_method_signature (method);
 
 	arg_info = alloca (sizeof (MonoJitArgumentInfo) * (sig->param_count + 1));
 
 	mono_arch_get_argument_info (sig, sig->param_count, arg_info);
 
-	if (MONO_TYPE_ISSTRUCT (method->signature->ret)) {
-		g_assert (!method->signature->ret->byref);
+	if (MONO_TYPE_ISSTRUCT (mono_method_signature (method)->ret)) {
+		g_assert (!mono_method_signature (method)->ret->byref);
 
 		printf ("VALUERET:%p, ", *((gpointer *)(ebp + 8)));
 	}
 
-	if (method->signature->hasthis) {
+	if (mono_method_signature (method)->hasthis) {
 		gpointer *this = (gpointer *)(ebp + arg_info [0].offset);
 		if (method->klass->valuetype) {
 			printf ("value:%p, ", *this);
@@ -315,19 +325,19 @@ mono_trace_enter_method (MonoMethod *method, char *ebp)
 		}
 	}
 
-	for (i = 0; i < method->signature->param_count; ++i) {
+	for (i = 0; i < mono_method_signature (method)->param_count; ++i) {
 		gpointer *cpos = (gpointer *)(ebp + arg_info [i + 1].offset);
 		int size = arg_info [i + 1].size;
 
-		MonoType *type = method->signature->params [i];
+		MonoType *type = mono_method_signature (method)->params [i];
 		
 		if (type->byref) {
 			printf ("[BYREF:%p], ", *cpos); 
-		} else switch (type->type) {
+		} else switch (mono_type_get_underlying_type (type)->type) {
 			
 		case MONO_TYPE_I:
 		case MONO_TYPE_U:
-			printf ("%p, ", (gpointer)*((int *)(cpos)));
+			printf ("%p, ", *((gpointer **)(cpos)));
 			break;
 		case MONO_TYPE_BOOLEAN:
 		case MONO_TYPE_CHAR:
@@ -377,7 +387,7 @@ mono_trace_enter_method (MonoMethod *method, char *ebp)
 			break;
 		case MONO_TYPE_I8:
 		case MONO_TYPE_U8:
-			printf ("0x%016llx, ", *((gint64 *)(cpos)));
+			printf ("0x%016llx, ", (long long)*((gint64 *)(cpos)));
 			break;
 		case MONO_TYPE_R4:
 			printf ("%f, ", *((float *)(cpos)));
@@ -406,7 +416,7 @@ mono_trace_leave_method (MonoMethod *method, ...)
 	char *fname;
 	va_list ap;
 
-	if (!enable_trace)
+	if (!trace_spec.enabled)
 		return;
 
 	va_start(ap, method);
@@ -416,7 +426,7 @@ mono_trace_leave_method (MonoMethod *method, ...)
 	printf ("LEAVE: %s", fname);
 	g_free (fname);
 
-	type = method->signature->ret;
+	type = mono_method_signature (method)->ret;
 
 handle_enum:
 	switch (type->type) {
@@ -464,7 +474,7 @@ handle_enum:
 			} else if  (o->vtable->klass == mono_defaults.int32_class) {
 				printf ("[INT32:%p:%d]", o, *((gint32 *)((char *)o + sizeof (MonoObject))));	
 			} else if  (o->vtable->klass == mono_defaults.int64_class) {
-				printf ("[INT64:%p:%lld]", o, *((gint64 *)((char *)o + sizeof (MonoObject))));	
+				printf ("[INT64:%p:%lld]", o, (long long)*((gint64 *)((char *)o + sizeof (MonoObject))));	
 			} else
 				printf ("[%s.%s:%p]", o->vtable->klass->name_space, o->vtable->klass->name, o);
 		} else
@@ -482,12 +492,12 @@ handle_enum:
 	}
 	case MONO_TYPE_I8: {
 		gint64 l =  va_arg (ap, gint64);
-		printf ("lresult=0x%16llx", l);
+		printf ("lresult=0x%16llx", (long long)l);
 		break;
 	}
 	case MONO_TYPE_U8: {
 		gint64 l =  va_arg (ap, gint64);
-		printf ("lresult=0x%16llx", l);
+		printf ("lresult=0x%16llx", (long long)l);
 		break;
 	}
 	case MONO_TYPE_R4:
@@ -511,9 +521,22 @@ handle_enum:
 		}
 		break;
 	default:
-		printf ("(unknown return type %x)", method->signature->ret->type);
+		printf ("(unknown return type %x)", mono_method_signature (method)->ret->type);
 	}
 
 	//printf (" ip: %p\n", __builtin_return_address (1));
 	printf ("\n");
 }
+
+void
+mono_trace_enable (gboolean enable)
+{
+	trace_spec.enabled = enable;
+}
+
+gboolean
+mono_trace_is_enabled ()
+{
+	return trace_spec.enabled;
+}
+
