@@ -23,8 +23,12 @@
 #include "mini.h"
 #include "mini-amd64.h"
 
+#define IS_REX(inst) (((inst) >= 0x40) && ((inst) <= 0x4f))
+
+static guint8* nullified_class_init_trampoline;
+
 /*
- * get_unbox_trampoline:
+ * mono_arch_get_unbox_trampoline:
  * @m: method pointer
  * @addr: pointer to native code for @m
  *
@@ -32,8 +36,8 @@
  * this argument. This method returns a pointer to a trampoline which does
  * unboxing before calling the method
  */
-static gpointer
-get_unbox_trampoline (MonoMethod *m, gpointer addr)
+gpointer
+mono_arch_get_unbox_trampoline (MonoMethod *m, gpointer addr)
 {
 	guint8 *code, *start;
 	int this_reg = AMD64_RDI;
@@ -57,150 +61,45 @@ get_unbox_trampoline (MonoMethod *m, gpointer addr)
 	return start;
 }
 
-/**
- * amd64_magic_trampoline:
- */
-static gpointer
-amd64_magic_trampoline (long *regs, guint8 *code, MonoMethod *m, guint8* tramp)
+void
+mono_arch_patch_callsite (guint8 *code, guint8 *addr)
 {
-	gpointer addr;
-	gpointer *vtable_slot;
-
-	addr = mono_compile_method (m);
-	g_assert (addr);
-
-	//printf ("ENTER: %s\n", mono_method_full_name (m, TRUE));
-
-	/* the method was jumped to */
-	if (!code)
-		/* FIXME: Optimize the case when the call is from a delegate wrapper */
-		return addr;
-
-	vtable_slot = mono_arch_get_vcall_slot_addr (code, (gpointer*)regs);
-
-	if (vtable_slot) {
-		if (m->klass->valuetype)
-			addr = get_unbox_trampoline (m, addr);
-
-		g_assert (*vtable_slot);
-
-		if (mono_aot_is_got_entry (code, (guint8*)vtable_slot) || mono_domain_owns_vtable_slot (mono_domain_get (), vtable_slot))
-			*vtable_slot = addr;
-	}
-	else {
-		/* Patch calling code */
-
-		if (((code [-13] == 0x49) && (code [-12] == 0xbb)) ||
-			(code [-5] == 0xe8)) {
-			MonoJitInfo *ji = 
-				mono_jit_info_table_find (mono_domain_get (), code);
-			MonoJitInfo *target_ji = 
-				mono_jit_info_table_find (mono_domain_get (), addr);
-
-			if (mono_method_same_domain (ji, target_ji)) {
-				if (code [-5] != 0xe8)
-					InterlockedExchangePointer ((gpointer*)(code - 11), addr);
-				else {
-					g_assert ((((guint64)(addr)) >> 32) == 0);
-					g_assert ((((guint64)(code)) >> 32) == 0);
-					InterlockedExchange ((guint32*)(code - 4), ((gint64)addr - (gint64)code));
-				}
-			}
-		}
-		else if ((code [-7] == 0x41) && (code [-6] == 0xff) && (code [-5] == 0x15)) {
-			/* call *<OFFSET>(%rip) */
-			MonoJitInfo *ji = 
-				mono_jit_info_table_find (mono_domain_get (), code);
-			MonoJitInfo *target_ji = 
-				mono_jit_info_table_find (mono_domain_get (), addr);
-
-			if (mono_method_same_domain (ji, target_ji)) {
-				gpointer *got_entry = (gpointer*)((guint8*)code + (*(guint32*)(code - 4)));
-				InterlockedExchangePointer (got_entry, addr);
-			}
+	if (((code [-13] == 0x49) && (code [-12] == 0xbb)) || (code [-5] == 0xe8)) {
+		if (code [-5] != 0xe8)
+			InterlockedExchangePointer ((gpointer*)(code - 11), addr);
+		else {
+			g_assert ((((guint64)(addr)) >> 32) == 0);
+			g_assert ((((guint64)(code)) >> 32) == 0);
+			InterlockedExchange ((guint32*)(code - 4), ((gint64)addr - (gint64)code));
 		}
 	}
-
-	return addr;
+	else if ((code [-7] == 0x41) && (code [-6] == 0xff) && (code [-5] == 0x15)) {
+		/* call *<OFFSET>(%rip) */
+		gpointer *got_entry = (gpointer*)((guint8*)code + (*(guint32*)(code - 4)));
+		InterlockedExchangePointer (got_entry, addr);
+	}
 }
 
-/*
- * amd64_aot_trampoline:
- *
- *   This trampoline handles calls made from AOT code. We try to bypass the 
- * normal JIT compilation logic to avoid loading the metadata for the method.
- */
-static gpointer
-amd64_aot_trampoline (long *regs, guint8 *code, guint8 *token_info, 
-					  guint8* tramp)
+void
+mono_arch_nullify_class_init_trampoline (guint8 *code, gssize *regs)
 {
-	MonoImage *image;
-	guint32 token;
-	MonoMethod *method;
-	gpointer addr;
-	gpointer *vtable_slot;
-
-	image = *(gpointer*)token_info;
-	token_info += sizeof (gpointer);
-	token = *(guint32*)token_info;
-
-	/* Later we could avoid allocating the MonoMethod */
-	method = mono_get_method (image, token, NULL);
-	g_assert (method);
-
-	if (method->iflags & METHOD_IMPL_ATTRIBUTE_SYNCHRONIZED)
-		method = mono_marshal_get_synchronized_wrapper (method);
-
-	addr = mono_compile_method (method);
-	g_assert (addr);
-
-	vtable_slot = mono_arch_get_vcall_slot_addr (code, (gpointer*)regs);
-	g_assert (vtable_slot);
-
-	if (method->klass->valuetype)
-		addr = get_unbox_trampoline (method, addr);
-
-	if (mono_domain_owns_vtable_slot (mono_domain_get (), vtable_slot))
-		*vtable_slot = addr;
-
-	return addr;
-}
-
-/**
- * amd64_class_init_trampoline:
- *
- * This method calls mono_runtime_class_init () to run the static constructor
- * for the type, then patches the caller code so it is not called again.
- */
-static void
-amd64_class_init_trampoline (long *regs, guint8 *code, MonoVTable *vtable, guint8 *tramp)
-{
-	mono_runtime_class_init (vtable);
-
 	code -= 3;
 
-	if ((code [0] == 0x49) && (code [1] == 0xff)) {
-		if (!mono_running_on_valgrind ()) {
-			/* amd64_set_reg_template is 10 bytes long */
-			guint8* buf = code - 10;
+	/* 
+	 * A given byte sequence can match more than case here, so we have to be
+	 * really careful about the ordering of the cases. Longer sequences
+	 * come first.
+	 */
+	if ((code [-4] == 0x41) && (code [-3] == 0xff) && (code [-2] == 0x15)) {
+		gpointer *vtable_slot;
 
-			/* FIXME: Make this thread safe */
-			/* Padding code suggested by the AMD64 Opt Manual */
-			buf [0] = 0x66;
-			buf [1] = 0x66;
-			buf [2] = 0x66;
-			buf [3] = 0x90;
-			buf [4] = 0x66;
-			buf [5] = 0x66;
-			buf [6] = 0x66;
-			buf [7] = 0x90;
-			buf [8] = 0x66;
-			buf [9] = 0x66;
-			buf [10] = 0x90;
-			buf [11] = 0x66;
-			buf [12] = 0x90;
-		}
+		/* call *<OFFSET>(%rip) */
+		vtable_slot = mono_arch_get_vcall_slot_addr (code + 3, (gpointer*)regs);
+		g_assert (vtable_slot);
+
+		*vtable_slot = nullified_class_init_trampoline;
 	} else if (code [-2] == 0xe8) {
+		/* call <TARGET> */
 		guint8 *buf = code - 2;
 
 		buf [0] = 0x66;
@@ -208,19 +107,68 @@ amd64_class_init_trampoline (long *regs, guint8 *code, MonoVTable *vtable, guint
 		buf [2] = 0x90;
 		buf [3] = 0x66;
 		buf [4] = 0x90;
-	} else if (code [0] == 0x90 || code [0] == 0xeb || code [0] == 0x66)
+	} else if ((code [0] == 0x49) && (code [1] == 0xff)) {
+		/* call <REG> */
+		/* amd64_set_reg_template is 10 bytes long */
+		guint8* buf = code - 10;
+
+		/* FIXME: Make this thread safe */
+		/* Padding code suggested by the AMD64 Opt Manual */
+		buf [0] = 0x66;
+		buf [1] = 0x66;
+		buf [2] = 0x66;
+		buf [3] = 0x90;
+		buf [4] = 0x66;
+		buf [5] = 0x66;
+		buf [6] = 0x66;
+		buf [7] = 0x90;
+		buf [8] = 0x66;
+		buf [9] = 0x66;
+		buf [10] = 0x90;
+		buf [11] = 0x66;
+		buf [12] = 0x90;
+	} else if (code [0] == 0x90 || code [0] == 0xeb || code [0] == 0x66) {
 		/* Already changed by another thread */
 		;
-	else if ((code [-4] == 0x41) && (code [-3] == 0xff) && (code [-2] == 0x15))
-		/* call *<OFFSET>(%rip) */
-		;
-	else {
+	} else {
 		printf ("Invalid trampoline sequence: %x %x %x %x %x %x %x\n", code [0], code [1], code [2], code [3],
 				code [4], code [5], code [6]);
 		g_assert_not_reached ();
 	}
 }
 
+void
+mono_arch_patch_delegate_trampoline (guint8 *code, guint8 *tramp, gssize *regs, guint8 *addr)
+{
+	guint8 rex = 0;
+	guint32 reg;
+	guint32 disp;
+
+	if ((code [-3] == 0xff) && (amd64_modrm_reg (code [-2]) == 0x2) && (amd64_modrm_mod (code [-2]) == 0x1)) {
+		/* call *[reg+disp8] */
+		if (IS_REX (code [-4]))
+			rex = code [-4];
+		reg = amd64_modrm_rm (code [-2]);
+		disp = *(guint8*)(code - 1);
+		//printf ("B: [%%r%d+0x%x]\n", reg, disp);
+	}
+	else {
+		int i;
+
+		for (i = -16; i < 0; ++i)
+			printf ("%d ", code [i]);
+		printf ("\n");
+		g_assert_not_reached ();
+	}
+
+	reg += amd64_rex_b (rex);
+
+	/* R11 is clobbered by the trampoline code */
+	g_assert (reg != AMD64_R11);
+
+	*(gpointer*)(((guint64)(regs [reg])) + disp) = addr;
+}
+
 guchar*
 mono_arch_create_trampoline_code (MonoTrampolineType tramp_type)
 {
@@ -285,12 +233,14 @@ mono_arch_create_trampoline_code (MonoTrampolineType tramp_type)
 		amd64_mov_reg_imm (code, AMD64_R11, 0);
 	amd64_mov_membase_reg (code, AMD64_RBP, lmf_offset + G_STRUCT_OFFSET (MonoLMF, rip), AMD64_R11, 8);
 	/* Save fp */
-	amd64_mov_membase_reg (code, AMD64_RBP, lmf_offset + G_STRUCT_OFFSET (MonoLMF, ebp), AMD64_RBP, 8);
+	amd64_mov_reg_membase (code, AMD64_R11, AMD64_RSP, framesize, 8);
+	amd64_mov_membase_reg (code, AMD64_RBP, lmf_offset + G_STRUCT_OFFSET (MonoLMF, ebp), AMD64_R11, 8);
+	/* Save sp */
+	amd64_mov_reg_reg (code, AMD64_R11, AMD64_RSP, 8);
+	amd64_alu_reg_imm (code, X86_ADD, AMD64_R11, framesize + 16);
+	amd64_mov_membase_reg (code, AMD64_RBP, lmf_offset + G_STRUCT_OFFSET (MonoLMF, rsp), AMD64_R11, 8);
 	/* Save method */
-	if (tramp_type == MONO_TRAMPOLINE_GENERIC)
-		amd64_mov_reg_membase (code, AMD64_R11, AMD64_RBP, method_offset, 8);
-	else
-		amd64_mov_reg_imm (code, AMD64_R11, 0);
+	amd64_mov_reg_membase (code, AMD64_R11, AMD64_RBP, method_offset, 8);
 	amd64_mov_membase_reg (code, AMD64_RBP, lmf_offset + G_STRUCT_OFFSET (MonoLMF, method), AMD64_R11, 8);
 	/* Save callee saved regs */
 	amd64_mov_membase_reg (code, AMD64_RBP, lmf_offset + G_STRUCT_OFFSET (MonoLMF, rbx), AMD64_RBX, 8);
@@ -329,11 +279,13 @@ mono_arch_create_trampoline_code (MonoTrampolineType tramp_type)
 	amd64_mov_reg_membase (code, AMD64_RCX, AMD64_RBP, tramp_offset, 8);
 
 	if (tramp_type == MONO_TRAMPOLINE_CLASS_INIT)
-		tramp = (guint8*)amd64_class_init_trampoline;
+		tramp = (guint8*)mono_class_init_trampoline;
 	else if (tramp_type == MONO_TRAMPOLINE_AOT)
-		tramp = (guint8*)amd64_aot_trampoline;
+		tramp = (guint8*)mono_aot_trampoline;
+	else if (tramp_type == MONO_TRAMPOLINE_DELEGATE)
+		tramp = (guint8*)mono_delegate_trampoline;
 	else
-		tramp = (guint8*)amd64_magic_trampoline;
+		tramp = (guint8*)mono_magic_trampoline;
 
 	amd64_mov_reg_imm (code, AMD64_RAX, tramp);
 	amd64_call_reg (code, AMD64_RAX);
@@ -365,17 +317,22 @@ mono_arch_create_trampoline_code (MonoTrampolineType tramp_type)
 
 	mono_arch_flush_icache (buf, code - buf);
 
+	if (tramp_type == MONO_TRAMPOLINE_CLASS_INIT) {
+		/* Initialize the nullified class init trampoline used in the AOT case */
+		nullified_class_init_trampoline = code = mono_global_codeman_reserve (16);
+		x86_ret (code);
+	}
+
 	return buf;
 }
 
 #define TRAMPOLINE_SIZE 34
 
-static MonoJitInfo*
-create_specific_trampoline (gpointer arg1, MonoTrampolineType tramp_type, MonoDomain *domain)
+gpointer
+mono_arch_create_specific_trampoline (gpointer arg1, MonoTrampolineType tramp_type, MonoDomain *domain, guint32 *code_len)
 {
-	MonoJitInfo *ji;
-	guint8 *code, *buf, *tramp;
-	int jump_offset;
+	guint8 *code, *buf, *tramp, *real_code;
+	int size, jump_offset;
 
 	tramp = mono_get_trampoline_code (tramp_type);
 
@@ -398,111 +355,32 @@ create_specific_trampoline (gpointer arg1, MonoTrampolineType tramp_type, MonoDo
 
 	g_assert ((code - buf) <= TRAMPOLINE_SIZE);
 
-	ji = g_new0 (MonoJitInfo, 1);
-
 	mono_domain_lock (domain);
 	/* 
 	 * FIXME: Changing the size to code - buf causes strange crashes during
 	 * mcs bootstrap.
 	 */
-	ji->code_start = mono_code_manager_reserve (domain->code_mp, TRAMPOLINE_SIZE);
-	ji->code_size = code - buf;
+	real_code = mono_code_manager_reserve (domain->code_mp, TRAMPOLINE_SIZE);
+	size = code - buf;
 	mono_domain_unlock (domain);
 
-	memcpy (ji->code_start, buf, ji->code_size);
+	memcpy (real_code, buf, size);
 
 	/* Fix up jump */
 	g_assert ((((gint64)tramp) >> 32) == 0);
-	code = (guint8*)ji->code_start + jump_offset;
+	code = (guint8*)real_code + jump_offset;
 	amd64_jump_disp (code, tramp - code);
 
 	mono_jit_stats.method_trampolines++;
 
-	mono_arch_flush_icache (ji->code_start, ji->code_size);
+	if (code_len)
+		*code_len = size;
 
-	return ji;
+	mono_arch_flush_icache (real_code, size);
+
+	return real_code;
 }	
 
-MonoJitInfo*
-mono_arch_create_jump_trampoline (MonoMethod *method)
-{
-	MonoJitInfo *ji = create_specific_trampoline (method, MONO_TRAMPOLINE_JUMP, mono_domain_get ());
-
-	ji->method = method;
-	return ji;
-}
-
-/**
- * mono_arch_create_jit_trampoline:
- * @method: pointer to the method info
- *
- * Creates a trampoline function for virtual methods. If the created
- * code is called it first starts JIT compilation of method,
- * and then calls the newly created method. I also replaces the
- * corresponding vtable entry (see amd64_magic_trampoline).
- * 
- * Returns: a pointer to the newly created code 
- */
-gpointer
-mono_arch_create_jit_trampoline (MonoMethod *method)
-{
-	MonoJitInfo *ji;
-	gpointer code_start;
-
-	ji = create_specific_trampoline (method, MONO_TRAMPOLINE_GENERIC, mono_domain_get ());
-	code_start = ji->code_start;
-	g_free (ji);
-
-	return code_start;
-}
-
-gpointer
-mono_arch_create_jit_trampoline_from_token (MonoImage *image, guint32 token)
-{
-	MonoDomain *domain = mono_domain_get ();
-	MonoJitInfo *ji;
-	gpointer code_start;
-	guint8 *buf, *start;
-
-	mono_domain_lock (domain);
-	buf = start = mono_code_manager_reserve (domain->code_mp, 2 * sizeof (gpointer));
-	mono_domain_unlock (domain);
-
-	*(gpointer*)buf = image;
-	buf += sizeof (gpointer);
-	*(guint32*)buf = token;
-
-	ji = create_specific_trampoline (start, MONO_TRAMPOLINE_AOT, domain);
-	code_start = ji->code_start;
-	g_free (ji);
-
-	return code_start;
-}
-
-/**
- * mono_arch_create_class_init_trampoline:
- *  @vtable: the type to initialize
- *
- * Creates a trampoline function to run a type initializer. 
- * If the trampoline is called, it calls mono_runtime_class_init with the
- * given vtable, then patches the caller code so it does not get called any
- * more.
- * 
- * Returns: a pointer to the newly created code 
- */
-gpointer
-mono_arch_create_class_init_trampoline (MonoVTable *vtable)
-{
-	MonoJitInfo *ji;
-	gpointer code;
-
-	ji = create_specific_trampoline (vtable, MONO_TRAMPOLINE_CLASS_INIT, vtable->domain);
-	code = ji->code_start;
-	g_free (ji);
-
-	return code;
-}
-
 void
 mono_arch_invalidate_method (MonoJitInfo *ji, void *func, gpointer func_arg)
 {
