@@ -401,6 +401,19 @@ mono_test_marshal_struct (simplestruct ss)
 	return 1;
 }
 
+STDCALL int
+mono_test_marshal_byref_struct (simplestruct *ss, int a, int b, int c, char *d)
+{
+	gboolean res = (ss->a == a && ss->b == b && ss->c == c && strcmp (ss->d, d) == 0);
+
+	ss->a = !ss->a;
+	ss->b = !ss->b;
+	ss->c = !ss->c;
+	ss->d = g_strdup ("DEF");
+
+	return res ? 0 : 1;
+}
+
 typedef struct {
 	int a;
 	int b;
@@ -869,9 +882,14 @@ mono_test_empty_struct (int a, EmptyStruct es, int b)
 {
 	// printf ("mono_test_empty_struct %d %d\n", a, b);
 
+	// Intel icc on ia64 passes 'es' in 2 registers
+#if defined(__ia64) && defined(__INTEL_COMPILER)
+	return 0;
+#else
 	if (a == 1 && b == 2)
 		return 0;
 	return 1;
+#endif
 }
 
 typedef struct {
@@ -1316,6 +1334,35 @@ mono_test_marshal_amd64_pass_return_struct4 (amd64_struct4 s)
 	return s;
 }
 
+/*
+ * IA64 marshalling tests.
+ */
+typedef struct test_struct5 {
+	float d1, d2;
+} test_struct5;
+
+STDCALL test_struct5
+mono_test_marshal_ia64_pass_return_struct5 (double d1, double d2, test_struct5 s, double d3, double d4)
+{
+	s.d1 += d1 + d2;
+	s.d2 += d3 + d4;
+
+	return s;
+}
+
+typedef struct test_struct6 {
+	double d1, d2;
+} test_struct6;
+
+STDCALL test_struct6
+mono_test_marshal_ia64_pass_return_struct6 (double d1, double d2, test_struct6 s, double d3, double d4)
+{
+	s.d1 += d1 + d2;
+	s.d2 += d3 + d4;
+
+	return s;
+}
+
 static guint32 custom_res [2];
 
 STDCALL void*
@@ -1328,6 +1375,25 @@ mono_test_marshal_pass_return_custom (int i, guint32 *ptr, int j)
 	return &custom_res;
 }
 
+STDCALL int
+mono_test_marshal_pass_out_custom (int i, guint32 **ptr, int j)
+{
+	custom_res [0] = 0;
+	custom_res [1] = i + j + 10;
+
+	*ptr = custom_res;
+
+	return 0;
+}
+
+STDCALL int
+mono_test_marshal_pass_byref_custom (int i, guint32 **ptr, int j)
+{
+	(*ptr)[1] += i + j;
+
+	return 0;
+}
+
 STDCALL void*
 mono_test_marshal_pass_return_custom2 (int i, guint32 *ptr, int j)
 {
@@ -1690,3 +1756,24 @@ mono_test_marshal_cdecl_delegate (CdeclDelegate del)
 
 	return 0;
 }
+
+typedef char** (*ReturnStringArrayDelegate) (int i);
+
+STDCALL int
+mono_test_marshal_return_string_array_delegate (ReturnStringArrayDelegate d)
+{
+	char **arr = d (2);
+	int res;
+
+	if (arr == NULL)
+		return 3;
+
+	if (strcmp (arr [0], "ABC") || strcmp (arr [1], "DEF"))
+		res = 1;
+	else
+		res = 0;
+
+	marshal_free (arr);
+
+	return res;
+}
