@@ -1,6 +1,7 @@
 using System;
 using System.Text;
 using System.Runtime.InteropServices;
+using System.Runtime.CompilerServices;
 
 public class Tests {
 
@@ -137,7 +138,7 @@ public class Tests {
 	public static extern int mono_test_marshal_inout_array ([In, Out] int [] a1);
 
 	[DllImport ("libtest", EntryPoint="mono_test_marshal_out_array")]
-	public static extern int mono_test_marshal_out_array ([Out] int [] a1);
+	public static extern int mono_test_marshal_out_array ([Out] [MarshalAs(UnmanagedType.LPArray, SizeParamIndex = 1)] int [] a1, int n);
 
 	[DllImport ("libtest", EntryPoint="mono_test_marshal_inout_nonblittable_array", CharSet = CharSet.Unicode)]
 	public static extern int mono_test_marshal_inout_nonblittable_array ([In, Out] char [] a1);
@@ -151,6 +152,15 @@ public class Tests {
 	[DllImport ("libtest", EntryPoint="mono_test_marshal_struct2_2")]
 	public static extern int mono_test_marshal_struct2_2 (int i, int j, int k, SimpleStruct2 ss);
 
+	[DllImport ("libtest", EntryPoint="mono_test_marshal_byref_struct")]
+	public static extern int mono_test_marshal_byref_struct (ref SimpleStruct ss, bool a, bool b, bool c, String d);
+
+	[DllImport ("libtest", EntryPoint="mono_test_marshal_byref_struct")]
+	public static extern int mono_test_marshal_byref_struct_in ([In] ref SimpleStruct ss, bool a, bool b, bool c, String d);
+
+	[DllImport ("libtest", EntryPoint="mono_test_marshal_byref_struct")]
+	public static extern int mono_test_marshal_byref_struct_inout ([In, Out] ref SimpleStruct ss, bool a, bool b, bool c, String d);
+
 	[DllImport ("libtest", EntryPoint="mono_test_marshal_point")]
 	public static extern int mono_test_marshal_point (Point p);
 
@@ -265,10 +275,10 @@ public class Tests {
 		return res;
 	}
 
-	public static int test_0_marshal_inout_array () {
+	public static int test_0_marshal_out_array () {
 		int [] a1 = new int [50];
 
-		int res = mono_test_marshal_out_array (a1);
+		int res = mono_test_marshal_out_array (a1, 0);
 
 		for (int i = 0; i < 50; i++)
 			if (a1 [i] != i) {
@@ -453,6 +463,54 @@ public class Tests {
 		return 0;
 	}
 
+	public static int test_0_marshal_byref_struct () {
+		SimpleStruct s = new SimpleStruct ();
+		s.a = true;
+		s.b = false;
+		s.c = true;
+		s.d = "ABC";
+		s.d2 = "DEF";
+
+		int res = mono_test_marshal_byref_struct (ref s, true, false, true, "ABC");
+		if (res != 0)
+			return 1;
+		if (s.a != false || s.b != true || s.c != false || s.d != "DEF")
+			return 2;
+		return 0;
+	}
+
+	public static int test_0_marshal_byref_struct_in () {
+		SimpleStruct s = new SimpleStruct ();
+		s.a = true;
+		s.b = false;
+		s.c = true;
+		s.d = "ABC";
+		s.d2 = "DEF";
+
+		int res = mono_test_marshal_byref_struct_in (ref s, true, false, true, "ABC");
+		if (res != 0)
+			return 1;
+		if (s.a != true || s.b != false || s.c != true || s.d != "ABC")
+			return 2;
+		return 0;
+	}
+
+	public static int test_0_marshal_byref_struct_inout () {
+		SimpleStruct s = new SimpleStruct ();
+		s.a = true;
+		s.b = false;
+		s.c = true;
+		s.d = "ABC";
+		s.d2 = "DEF";
+
+		int res = mono_test_marshal_byref_struct_inout (ref s, true, false, true, "ABC");
+		if (res != 0)
+			return 1;
+		if (s.a != false || s.b != true || s.c != false || s.d != "DEF")
+			return 2;
+		return 0;
+	}
+
 	public static int test_0_marshal_point () {
 		Point pt = new Point();
 		pt.x = 1.25;
@@ -743,6 +801,50 @@ public class Tests {
 		return (s2.d1 == 6.0 && s2.d2 == -4.0) ? 0 : 1;
 	}
 
+	/*
+	 * IA64 struct tests
+	 */
+
+	/* Test 5: Float HFA */
+
+	[StructLayout(LayoutKind.Sequential)]
+	public struct TestStruct5 {
+		public float d1, d2;
+	}
+	
+	[DllImport ("libtest", EntryPoint="mono_test_marshal_ia64_pass_return_struct5")]
+	public static extern TestStruct5 mono_test_marshal_ia64_pass_return_struct5 (double d1, double d2, TestStruct5 s, double f3, double f4);
+
+	public static int test_0_ia64_struct5 () {
+		TestStruct5 s = new TestStruct5 ();
+		s.d1 = 5.0f;
+		s.d2 = -5.0f;
+
+		TestStruct5 s2 = mono_test_marshal_ia64_pass_return_struct5 (1.0, 2.0, s, 3.0, 4.0);
+
+		return (s2.d1 == 8.0 && s2.d2 == 2.0) ? 0 : 1;
+	}
+
+	/* Test 6: Double HFA */
+
+	[StructLayout(LayoutKind.Sequential)]
+	public struct TestStruct6 {
+		public double d1, d2;
+	}
+	
+	[DllImport ("libtest", EntryPoint="mono_test_marshal_ia64_pass_return_struct6")]
+	public static extern TestStruct6 mono_test_marshal_ia64_pass_return_struct6 (double d1, double d2, TestStruct6 s, double f3, double f4);
+
+	public static int test_0_ia64_struct6 () {
+		TestStruct6 s = new TestStruct6 ();
+		s.d1 = 6.0;
+		s.d2 = -6.0;
+
+		TestStruct6 s2 = mono_test_marshal_ia64_pass_return_struct6 (1.0, 2.0, s, 3.0, 4.0);
+
+		return (s2.d1 == 9.0 && s2.d2 == 1.0) ? 0 : 1;
+	}
+	
 	/* Blittable class */
 	[DllImport("libtest")]
 	private static extern VectorList TestVectorList (VectorList vl);
@@ -862,6 +964,11 @@ public class Tests {
 		[MarshalAs(UnmanagedType.LPWStr)] string s,
 		int length );
 
+	[DllImport("libtest", EntryPoint="test_lpwstr_marshal", CharSet=CharSet.Unicode)]
+	private static extern string mono_test_marshal_lpwstr_marshal2(
+		string s,
+		int length );
+
 	public static int test_0_pass_return_lwpstr () {
 		string s = "ABC";
 		
@@ -869,8 +976,133 @@ public class Tests {
 
 		if (res != "ABC")
 			return 1;
+
+		string res2 = mono_test_marshal_lpwstr_marshal2 (s, s.Length);
+
+		if (res2 != "ABC")
+			return 2;
 		
 		return 0;		
 	}
+
+	/*
+	 * Byref bool marshalling
+	 */
+
+	[DllImport("libtest")]
+	extern static int marshal_test_ref_bool
+	(
+		int i, 
+		[MarshalAs(UnmanagedType.I1)] ref bool b1, 
+		[MarshalAs(UnmanagedType.VariantBool)] ref bool b2, 
+		ref bool b3
+	);
+
+	public static int test_0_pass_byref_bool () {
+		for (int i = 0; i < 8; i++)
+		{
+			bool b1 = (i & 4) != 0;
+			bool b2 = (i & 2) != 0;
+			bool b3 = (i & 1) != 0;
+			bool orig_b1 = b1, orig_b2 = b2, orig_b3 = b3;
+			if (marshal_test_ref_bool(i, ref b1, ref b2, ref b3) != 0)
+				return 4 * i + 1;
+			if (b1 != !orig_b1)
+				return 4 * i + 2;
+			if (b2 != !orig_b2)
+				return 4 * i + 3;
+			if (b3 != !orig_b3)
+				return 4 * i + 4;
+		}
+
+		return 0;
+	}
+
+	/*
+	 * Bool struct field marshalling
+	 */
+
+	struct BoolStruct
+	{
+		public int i;
+		[MarshalAs(UnmanagedType.I1)] public bool b1;
+		[MarshalAs(UnmanagedType.VariantBool)] public bool b2;
+		public bool b3;
+	}
+
+	[DllImport("libtest")]
+	extern static int marshal_test_bool_struct(ref BoolStruct s);
+
+	public static int test_0_pass_bool_in_struct () {
+		for (int i = 0; i < 8; i++)
+		{
+			BoolStruct s = new BoolStruct();
+			s.i = i;
+			s.b1 = (i & 4) != 0;
+			s.b2 = (i & 2) != 0;
+			s.b3 = (i & 1) != 0;
+			BoolStruct orig = s;
+			if (marshal_test_bool_struct(ref s) != 0)
+				return 4 * i + 33;
+			if (s.b1 != !orig.b1)
+				return 4 * i + 34;
+			if (s.b2 != !orig.b2)
+				return 4 * i + 35;
+			if (s.b3 != !orig.b3)
+				return 4 * i + 36;
+		}
+
+		return 0;
+	}
+
+	/*
+	 * Invoking pinvoke methods through delegates
+	 */
+
+	delegate int MyDelegate (string name);
+
+	[DllImport ("libtest", EntryPoint="mono_test_puts_static")]
+	public static extern int puts_static (string name);
+
+	public static int test_0_invoke_pinvoke_through_delegate () {
+		puts_static ("A simple Test for PInvoke 1");
+
+		MyDelegate d = new MyDelegate (puts_static);
+		d ("A simple Test for PInvoke 2");
+
+		object [] args = {"A simple Test for PInvoke 3"};
+		d.DynamicInvoke (args);
+
+		return 0;
+	}
+
+	/*
+	 * Missing virtual pinvoke methods
+	 */
+
+	public class T {
+
+		public virtual object MyClone ()
+		{
+			return null;
+		}
+	}
+
+	public class T2 : T {
+		[MethodImplAttribute(MethodImplOptions.InternalCall)]
+		public override extern object MyClone ();
+	}
+
+	public static int test_0_missing_virtual_pinvoke_method () {
+		T2 t = new T2 ();
+
+		try {
+			t.MyClone ();
+		} catch (Exception ex) {
+			return 0;
+		}
+		
+		return 1;
+	}
 }
 
