@@ -168,6 +168,9 @@ query_pagesize (void)
 #if defined(__ppc__) || defined(__powerpc__)
 #define BIND_ROOM 4
 #endif
+#if defined(__arm__)
+#define BIND_ROOM 8
+#endif
 
 static CodeChunk*
 new_codechunk (int dynamic, int size)
@@ -206,10 +209,9 @@ new_codechunk (int dynamic, int size)
 	bsize += MIN_ALIGN -1;
 	bsize &= ~ (MIN_ALIGN - 1);
 	if (chunk_size - size < bsize) {
-		if (dynamic)
-			chunk_size = size + bsize;
-		else
-			chunk_size += pagesize;
+		chunk_size = size + bsize;
+		chunk_size += pagesize - 1;
+		chunk_size &= ~ (pagesize - 1);
 	}
 #endif
 
@@ -330,8 +332,9 @@ mono_code_manager_reserve (MonoCodeManager *cman, int size)
 		return NULL;
 	chunk->next = cman->current;
 	cman->current = chunk;
+	ptr = chunk->data + chunk->pos;
 	chunk->pos += size;
-	return chunk->data;
+	return ptr;
 }
 
 /* 
