@@ -23,6 +23,14 @@
 
 #if defined(__GNUC__) && defined(__i386__)
 #define MONO_THREAD_VAR_OFFSET(var,offset) __asm ("jmp 1f; .section writetext, \"awx\"; 1: movl $" #var "@ntpoff, %0; jmp 2f; .previous; 2:" : "=r" (offset));
+#elif defined(__x86_64__)
+#if defined(PIC)
+#define MONO_THREAD_VAR_OFFSET(var,offset) (offset) = -1
+#else
+#define MONO_THREAD_VAR_OFFSET(var,offset) do { guint64 foo;  __asm ("jmp 1f; .section writetext, \"awx\"; 1: movq $" #var "@TPOFF, %0; jmp 2f; .previous; 2:" : "=a" (foo)); offset = foo; } while (0);
+#endif
+#elif defined(__ia64__) && !defined(__INTEL_COMPILER)
+#define MONO_THREAD_VAR_OFFSET(var,offset) __asm ("addl %0 = @tprel(" #var "#), r0 ;;\n" : "=r" (offset));
 #else
 #define MONO_THREAD_VAR_OFFSET(var,offset) (offset) = -1
 #endif
@@ -38,7 +46,7 @@
 
 #include <float.h>
 #define isnan(x)	_isnan(x)
-#define trunc(x)	((int)(x))
+#define trunc(x)	floor((x))
 #define isinf(x)	(_isnan(x) ? 0 : (_fpclass(x) == _FPCLASS_NINF) ? -1 : (_fpclass(x) == _FPCLASS_PINF) ? 1 : 0)
 
 #define popen		_popen
