@@ -43,6 +43,14 @@ main(int argc, char* argv[]) {
 		return 1;
 	}
 	file = argv [1];
+
+	/*
+	 * Load the default Mono configuration file, this is needed
+	 * if you are planning on using the dllmaps defined on the
+	 * system configuration
+	 */
+	mono_set_dirs (NULL, NULL);
+	mono_config_parse (NULL);
 	/*
 	 * mono_jit_init() creates a domain: each assembly is
 	 * loaded and run in a MonoDomain.
