@@ -3,8 +3,9 @@
 //
 // Author(s):
 //  Jackson Harper (jackson@ximian.com)
+//  Atsushi Enomoto (atsushi@ximian.com)
 //
-// (C) 2004 Novell, Inc (http://www.novell.com)
+// (C) 2004-2005 Novell, Inc (http://www.novell.com)
 //
 
 
@@ -46,7 +47,8 @@ namespace Mono.Tools.LocaleBuilder {
                 private ArrayList cultures;
                 private Hashtable langs;
                 private Hashtable currency_types;
-                
+                private Hashtable regions;
+
 		// The lang is the language that display names will be displayed in
 		public string Lang {
 			get {
@@ -79,11 +81,16 @@ namespace Mono.Tools.LocaleBuilder {
 
                         langs = new Hashtable ();
                         cultures = new ArrayList ();
+                        regions = new Hashtable ();
+
+			LookupRegions ();
 
                         LookupCurrencyTypes ();
 
 			foreach (string file in Directory.GetFiles ("locales", "*.xml")) {
 				string fn = Path.GetFileNameWithoutExtension (file);
+				if (fn == "hy_AM")
+					continue; // see bug #75499
 				if (locales_regex == null || locales_regex.IsMatch (fn)) {
 					ParseLocale (fn);
                                 }
@@ -107,6 +114,39 @@ namespace Mono.Tools.LocaleBuilder {
 				}
 			 }
 
+			ArrayList regionList = new ArrayList (regions.Values);
+			regionList.Sort (RegionComparer.Instance);
+			int number = 0;
+			foreach (RegionInfoEntry r in regionList)
+				r.RegionId = number++;
+
+			foreach (CultureInfoEntry e in cultures) {
+				int lcid = int.Parse (e.Lcid.Substring (2),
+					NumberStyles.HexNumber);
+				int idx;
+				int start = e.Name.IndexOf ('-') + 1;
+				if (start == 0)
+					continue;
+				for (idx = start; idx < e.Name.Length; idx++)
+					if (!Char.IsLetter (e.Name [idx]))
+						break;
+				if (start == idx) {
+					Console.Error.WriteLine ("Culture {0} {1} is not mappable to Region.", e.Lcid, e.Name);
+					continue;
+				}
+				string name = e.Name.Substring (start, idx - start);
+				RegionInfoEntry rm = null;
+				foreach (RegionInfoEntry r in regions.Values)
+					if (r.ISO2Name == name) {
+						rm = r;
+						break;
+					}
+				if (rm == null) {
+					Console.Error.WriteLine ("No definition for region {0}", name);
+					continue;
+				}
+				e.RegionId = rm.RegionId;
+			}
 
                         /**
                          * Dump each table individually. Using StringBuilders
@@ -122,6 +162,7 @@ namespace Mono.Tools.LocaleBuilder {
                                 writer.WriteLine ("\n");
 
                                 writer.WriteLine ("#define NUM_CULTURE_ENTRIES " + cultures.Count);
+                                writer.WriteLine ("#define NUM_REGION_ENTRIES " + regionList.Count);
                                 writer.WriteLine ("\n");
 
                                 // Sort the cultures by lcid
@@ -192,6 +233,32 @@ namespace Mono.Tools.LocaleBuilder {
                                 writer.Write (builder);
                                 writer.WriteLine ("};\n\n");
 
+				builder = new StringBuilder ();
+				int rcount = 0;
+				foreach (RegionInfoEntry r in regionList) {
+					r.AppendTableRow (builder);
+					if (++rcount != regionList.Count)
+						builder.Append (',');
+					builder.Append ('\n');
+				}
+				writer.WriteLine ("static const RegionInfoEntry region_entries [] = {");
+				writer.Write (builder);
+				writer.WriteLine ("};\n\n");
+
+                                builder = new StringBuilder ();
+				rcount = 0;
+				foreach (RegionInfoEntry ri in regionList) {
+                                        builder.Append ("\t{" + Entry.EncodeStringIdx (ri.ISO2Name) + ", ");
+                                        builder.Append (ri.RegionId + "}");
+                                        if (++rcount < regionList.Count)
+                                                builder.Append (',');
+                                        builder.Append ('\n');
+                                }
+
+                                writer.WriteLine ("static const RegionInfoNameEntry region_name_entries [] = {");
+                                writer.Write (builder);
+                                writer.WriteLine ("};\n\n");
+
                                 writer.WriteLine ("static const char locale_strings [] = {");
                                 writer.Write (Entry.GetStrings ());
                                 writer.WriteLine ("};\n\n");
@@ -366,6 +433,12 @@ namespace Mono.Tools.LocaleBuilder {
 			}
 		}
 
+		private void AddPattern (ArrayList al, string pattern)
+		{
+			if (!al.Contains (pattern))
+				al.Add (pattern);
+		}
+
 		private void LookupDateTimeInfo (XPathNavigator nav, CultureInfoEntry ci)
 		{
 			/**
@@ -461,10 +534,13 @@ namespace Mono.Tools.LocaleBuilder {
 						ext = df_nav.Select ("extraPatterns/pattern");
 						if (ext.MoveNext ()) {
 							df.LongDatePatterns.Clear ();
+							AddPattern (df.LongDatePatterns, df.LongDatePattern);
 							do {
 								df.LongDatePatterns.Add (ext.Current.Value);
 							} while (ext.MoveNext ());
 						}
+						else
+							AddPattern (df.LongDatePatterns, df.LongDatePattern);
 						break;
 					case "short":
 						if (value != null)
@@ -472,10 +548,13 @@ namespace Mono.Tools.LocaleBuilder {
 						ext = df_nav.Select ("extraPatterns/pattern");
 						if (ext.MoveNext ()) {
 							df.ShortDatePatterns.Clear ();
+							AddPattern (df.ShortDatePatterns, df.ShortDatePattern);
 							do {
 								df.ShortDatePatterns.Add (ext.Current.Value);
 							} while (ext.MoveNext ());
 						}
+						else
+							AddPattern (df.ShortDatePatterns, df.ShortDatePattern);
 						break;
 					case "year_month":
 						if (value != null)
@@ -503,10 +582,13 @@ namespace Mono.Tools.LocaleBuilder {
 						ext = df_nav.Select ("extraPatterns/pattern");
 						if (ext.MoveNext ()) {
 							df.LongTimePatterns.Clear ();
+							AddPattern (df.LongTimePatterns, df.LongTimePattern);
 							do {
 								df.LongTimePatterns.Add (ext.Current.Value);
 							} while (ext.MoveNext ());
 						}
+						else
+							AddPattern (df.LongTimePatterns, df.LongTimePattern);
 						break;
 					case "short":
 						if (value != null)
@@ -514,10 +596,13 @@ namespace Mono.Tools.LocaleBuilder {
 						ext = df_nav.Select ("extraPatterns/pattern");
 						if (ext.MoveNext ()) {
 							df.ShortTimePatterns.Clear ();
+							AddPattern (df.ShortTimePatterns, df.ShortTimePattern);
 							do {
 								df.ShortTimePatterns.Add (ext.Current.Value);
 							} while (ext.MoveNext ());
 						}
+						else
+							AddPattern (df.ShortTimePatterns, df.ShortTimePattern);
 						break;
 					}
 				}
@@ -903,6 +988,42 @@ namespace Mono.Tools.LocaleBuilder {
 			return ret;
 		}
 
+		private void LookupRegions ()
+		{
+                        XPathDocument doc = GetXPathDocument ("supplementalData.xml");
+			XPathNavigator nav = doc.CreateNavigator ();
+			XPathNodeIterator ni = nav.Select ("supplementalData/currencyData/region");
+			while (ni.MoveNext ()) {
+				string territory = (string) ni.Current.GetAttribute ("iso3166", String.Empty);
+                                string currency = (string) ni.Current.Evaluate ("string(currency/@iso4217)");
+				RegionInfoEntry region = new RegionInfoEntry ();
+				region.ISO2Name = territory.ToUpper ();
+				region.ISOCurrencySymbol = currency;
+				regions [territory] = region;
+			}
+
+                        doc = GetXPathDocument ("langs/en.xml");
+			nav = doc.CreateNavigator ();
+			ni = nav.Select ("/ldml/localeDisplayNames/territories/territory");
+			while (ni.MoveNext ()) {
+				RegionInfoEntry r = (RegionInfoEntry)
+					regions [ni.Current.GetAttribute ("type", "")];
+				if (r == null)
+					continue;
+				r.EnglishName = ni.Current.Value;
+			}
+
+			Hashtable curNames = new Hashtable ();
+			ni = nav.Select ("/ldml/numbers/currencies/currency");
+			while (ni.MoveNext ())
+				curNames [ni.Current.GetAttribute ("type", "")] =
+					ni.Current.Evaluate ("string (displayName)");
+
+			foreach (RegionInfoEntry r in regions.Values)
+				r.CurrencyEnglishName =
+					(string) curNames [r.ISOCurrencySymbol];
+		}
+
                 private void LookupCurrencyTypes ()
                 {
                         XPathDocument doc = GetXPathDocument ("supplementalData.xml");
@@ -997,6 +1118,31 @@ namespace Mono.Tools.LocaleBuilder {
                                 return aa.Name.ToLower ().CompareTo (bb.Name.ToLower ());
                         }
                 }
+
+		class RegionComparer : IComparer
+		{
+			public static RegionComparer Instance = new RegionComparer ();
+			
+			public int Compare (object o1, object o2)
+			{
+				RegionInfoEntry r1 = (RegionInfoEntry) o1;
+				RegionInfoEntry r2 = (RegionInfoEntry) o2;
+				return String.CompareOrdinal (
+					r1.ISO2Name, r2.ISO2Name);
+			}
+		}
+
+		class RegionLCIDMap
+		{
+			public RegionLCIDMap (int lcid, int regionId)
+			{
+				LCID = lcid;
+				RegionId = regionId;
+			}
+
+			public int LCID;
+			public int RegionId;
+		}
         }
 }
 
